package com.bocloud.cmp.provider;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.aliyun.ecs20140526.Client;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.teaopenapi.models.Config;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.http.HttpClientConfig;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.profile.IClientProfile;
import com.aliyuncs.rds.Endpoint;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ResourceGroupModel;
import com.bocloud.cmp.provider.common.AliyunLocationProvider;
import com.bocloud.cmp.provider.common.LocationProvider;
import com.bocloud.cmp.provider.groups.AliyunResourceGroupsProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;

/**
 * 阿里云provider
 *
 * @ClassName: AliyunProvider.java
 * @Description:
 * @author: ymh
 * @date: 2021年4月9日 下午5:20:40
 */
public class AliyunProvider extends GenericProvider implements SuperProvider {

    private final static Logger logger = LoggerFactory.getLogger(AliyunProvider.class);
    private String accessKey;//
    private String secretKey;//
    private String regionId = "cn-hangzhou";//
    private Config config;
    private Butler butler;
    private String url;
    public static final String DATE_FORMAT = "yyyy-MM-dd'T'HH:mmZ";
    private IAcsClient client;
    private IClientProfile profile;
    private static HttpClientConfig httpClientConfig = null;

    // 超时设置
    static {
        if (null == httpClientConfig) {
            httpClientConfig = HttpClientConfig.getDefault();
            httpClientConfig.setConnectionTimeoutMillis(100000);
            httpClientConfig.setReadTimeoutMillis(100000);
            httpClientConfig.setWriteTimeoutMillis(100000);
        }
    }

    public AliyunProvider(Butler butler) {
        super(butler);
        this.butler = butler;
        this.accessKey = butler.getAccessKey().trim();
        this.secretKey = butler.getSecretKey().trim();
        this.regionId = butler.getRegionId() == null ? this.regionId : butler.getRegionId().trim();
        this.url = butler.getUrl();

        this.config = new Config().setAccessKeyId(accessKey).setAccessKeySecret(secretKey).setRegionId(regionId);

        if (!StringUtils.isEmpty(butler.getProxyUri())) {
            config.setHttpProxy(butler.getProxyUri());
            httpClientConfig.setHttpProxy(butler.getProxyUri());
        }
        // 旧版sdk暂时用于rds
        this.profile = DefaultProfile.getProfile(regionId, accessKey, secretKey);
        profile.setHttpClientConfig(httpClientConfig);
        this.client = new DefaultAcsClient(profile);
    }

    public Client getECSClient() throws Exception {
        if (StringUtils.isNotEmpty(butler.getUrl())) {
            this.config.setEndpoint("ecs." + butler.getUrl());
            this.config.setProtocol("http");
            this.config.connectTimeout = 50000;
            this.config.readTimeout = 50000;
        }
        return new com.aliyun.ecs20140526.Client(config);
    }

    public com.aliyun.slb20140515.Client getSLBClient() throws Exception {
        if (StringUtils.isNotEmpty(butler.getUrl())) {
            this.config.setEndpoint("slb." + butler.getUrl());
            this.config.setProtocol("http");
            this.config.connectTimeout = 50000;
            this.config.readTimeout = 50000;
        }
        return new com.aliyun.slb20140515.Client(config);
    }

    public com.aliyun.rds20140815.Client getRDSClient() throws Exception {
        if (StringUtils.isNotEmpty(butler.getUrl())) {
            this.config.setEndpoint("rds." + butler.getUrl());
            this.config.setProtocol("http");
            this.config.connectTimeout = 200000;
            this.config.readTimeout = 200000;
        }
        return new com.aliyun.rds20140815.Client(config);
    }

    public com.aliyun.vpc20160428.Client getVPCClient() throws Exception {
        if (StringUtils.isNotEmpty(butler.getUrl())) {
            this.config.setEndpoint("vpc." + butler.getUrl());
            this.config.setProtocol("http");
            this.config.connectTimeout = 50000;
            this.config.readTimeout = 50000;
        }
        return new com.aliyun.vpc20160428.Client(config);
    }

    public com.aliyun.bssopenapi20171214.Client getBSSClient() throws Exception {
        if (StringUtils.isNotEmpty(butler.getUrl())) {
            this.config.setEndpoint("business." + butler.getUrl());
            this.config.setProtocol("http");
            this.config.connectTimeout = 50000;
            this.config.readTimeout = 50000;
        }
        return new com.aliyun.bssopenapi20171214.Client(config);
    }

    public com.aliyun.resourcemanager20200331.Client getResourceClient() throws Exception {
        if (StringUtils.isNotEmpty(butler.getUrl())) {
            this.config.setEndpoint("resourcemanager." + butler.getUrl());
            this.config.setProtocol("https");
            this.config.connectTimeout = 5000;
            this.config.readTimeout = 5000;
        }
        return new com.aliyun.resourcemanager20200331.Client(config);
    }
    
	/**
	 * 对象存储目前只支持通过SDK方式对接，也就是说 不支持 专有云
	 * 
	 * @return
	 */
	public OSS getOSSClient() {
		String endpoint = "http://oss-" + this.getRegionId() + ".aliyuncs.com";
		OSS ossClient = new OSSClientBuilder().build(endpoint, this.accessKey, this.secretKey);
		return ossClient;
	}

    public String getAccessKey() {
        return accessKey;
    }

    public void setAccessKey(String accessKey) {
        this.accessKey = accessKey;
    }

    public String getSecretKey() {
        return secretKey;
    }

    public void setSecretKey(String secretKey) {
        this.secretKey = secretKey;
    }

    public String getRegionId() {
        return regionId;
    }

    public void setRegionId(String regionId) {
        this.regionId = regionId;
    }

    public String getUrl() {
        return url;
    }

    public void setUrl(String url) {
        this.url = url;
    }

    @Override
    public boolean accessable() {
        try {
            LocationProvider provider = new AliyunLocationProvider(this.butler);
            BocloudResult result = provider.listRegions();
            return result.isSuccess();
        } catch (Exception e) {
            logger.error("认证失败：" + e.getMessage());
            return false;
        }
    }

    public IAcsClient getClient() {
        // 阿里专有云处理rds获取实例规格无新版SDK使用的问题，使用旧版的SDK,在这里在这里传入域名做拼接，可以满足专有云的场景
    	if(!StringUtils.isEmpty(this.getUrl())) {
    		Endpoint.endpointMap.put(this.getRegionId(), "rds." + this.getUrl());
    	}
        return client;
    }

    public static HttpClientConfig getHttpClientConfig() {
        return httpClientConfig;
    }

    public static void setHttpClientConfig(HttpClientConfig httpClientConfig) {
        AliyunProvider.httpClientConfig = httpClientConfig;
    }

    public void setClient(IAcsClient client) {
        this.client = client;
    }

    /**
     * 查询默认资源组
     *
     * @return
     */
    public ResourceGroupModel getDefaultResourceGroup() {
        ResourceGroupModel defaultGroupModel = new ResourceGroupModel();
        try {
            AliyunResourceGroupsProvider resourceGroupsProvider = new AliyunResourceGroupsProvider(butler);
            BocloudResult resourceGroupsList = resourceGroupsProvider.list();
            if (resourceGroupsList.isSuccess()) {
                // 获取默认资源组
                @SuppressWarnings("unchecked")
                List<ResourceGroupModel> resourceGroupModels = (List<ResourceGroupModel>)resourceGroupsList.getData();
                if (!ListTool.isEmpty(resourceGroupModels)) {
                    for (ResourceGroupModel groupModel : resourceGroupModels) {
                        if ("default".equalsIgnoreCase(groupModel.getName())) {
                            defaultGroupModel = groupModel;
                            break;
                        }
                    }
                }
            }
        } catch (Exception e) {
            logger.error("查询默认资源组异常：" + e.getMessage());
        }
        return defaultGroupModel;
    }
}