package com.bocloud.cmp.provider.common;

import com.aliyun.bssopenapi20171214.models.*;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.PublicCloudBillModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.BillingConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @Package: com.alicloud.cmp.provider.common
 * @ClassName: BillingTest
 * @Author: 胡文涛
 * @CreateTime: 2020/7/29 16:21
 * @Description: 阿里云账单
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunBillingProvider extends AliyunProvider implements BillingProvider {

    private static Logger logger = LoggerFactory.getLogger(AliyunBillingProvider.class);
    private static final int PAGE_SIZE = 100;
    private BillingConvertor convertor = new BillingConvertor();

    public AliyunBillingProvider(Butler butler) {
        super(butler);
    }


    /**
     * @return com.bocloud.common.model.BocloudResult
     * @Author Hu Wentao
     * @Description 查询指定订单号的订单详情
     * @Date 18:02 2020/7/29
     * @Param [orderId]
     **/
    public BocloudResult getOrderDetail(String orderId) {
        try {
            GetOrderDetailRequest request = new GetOrderDetailRequest();
            request.setOrderId(orderId);
            GetOrderDetailResponse response = getBSSClient().getOrderDetail(request);
            if (null == response) {
                return new BocloudResult(false, "查询指定订单号的订单详情失败");
            }
            return new BocloudResult(true, response.getBody().getData(), "查询指定订单号的订单详情成功");
        } catch (Exception e) {
            logger.error("查询指定订单号的订单详情失败 ： " + e);
            return new BocloudResult(false, "查询指定订单号的订单详情失败", e.getMessage());
        }
    }

    /**
     * @return com.bocloud.common.model.BocloudResult
     * @Author Hu Wentao
     * @Description 调用QueryBill查询用户某个账期内结算账单。
     * @Date 16:26 2020/7/29
     * @Param [month]
     **/
    @Override
    public BocloudResult billDetailList(String month) {
        try {
            List<PublicCloudBillModel> billModels = new ArrayList<>();
            QueryBillRequest request = new QueryBillRequest();
            request.setBillingCycle(month);
            request.setPageNum(1);
            request.setPageSize(PAGE_SIZE);
            QueryBillResponse response = this.getBSSClient().queryBill(request);
            QueryBillResponseBody.QueryBillResponseBodyData data = response.getBody().getData();
            Integer totalCount = data.getTotalCount();
            List<QueryBillResponseBody.QueryBillResponseBodyDataItemsItem> items = data.getItems().getItem();
            if (ListTool.isEmpty(items)) {
                return new BocloudResult(true, "暂无该月账单信息！");
            }
            billModels.addAll(convertor.convertList(items));
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1) {
                return new BocloudResult(true, billModels, "查询用户某个账期内结算账单成功！");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNum(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getBSSClient().queryBill(request);
                items = response.getBody().getData().getItems().getItem();
                billModels.addAll(convertor.convertList(items));
            }
            return new BocloudResult(true, billModels, "查询用户某个账期内结算账单成功！");
        } catch (Exception e) {
            logger.error("查询用户某个账期内结算账单异常：" + e.getMessage());
            return new BocloudResult(false, "查询用户某个账期内结算账单异常：" + e.getMessage());
        }
    }

    /**
     * @return com.bocloud.common.model.BocloudResult
     * @Author Hu Wentao
     * @Description 调用QueryBillOverview查询用户某个账期内账单总览信息。
     * @Date 16:21 2020/7/29
     * @Param [yearmoth, productType]
     **/
    public BocloudResult queryBillOverview(String yearmoth, String productType) {
        try {
            //List<PublicCloudBillModel> billModels = new ArrayList<>();
            QueryBillOverviewRequest request = new QueryBillOverviewRequest();
            request.setBillingCycle(yearmoth);
            request.setProductType(productType);
            QueryBillOverviewResponse response = this.getBSSClient().queryBillOverview(request);

            if (null == response) {
                return new BocloudResult(false, "查询用户某个账期内账单总览信息失败");
            }
            QueryBillOverviewResponseBody.QueryBillOverviewResponseBodyData data = response.getBody().getData();
            List<QueryBillOverviewResponseBody.QueryBillOverviewResponseBodyDataItemsItem> items = data.getItems().getItem();
            if (ListTool.isEmpty(items)) {
                return new BocloudResult(true, "暂无该月账单信息！");
            }
            return new BocloudResult(true, data, "查询用户某个账期内账单总览信息成功");
        } catch (Exception e) {
            logger.error("查询用户某个账期内账单总览信息失败 ： " + e);
            return new BocloudResult(false, "查询用户某个账期内账单总览信息失败", e.getMessage());
        }
    }

    /**
     * @return com.bocloud.common.model.BocloudResult
     * @Author Hu Wentao
     * @Description 调用QuerySettleBill查询用户某个账期内结算账单。支持账单条目超过50000条的国内账号。
     * @Date 16:36 2020/7/29
     * @Param [yearmoth, productType]
     **/
    public BocloudResult querySettleBill(String yearmoth, String productType) {
        try {
            //List<PublicCloudBillModel> billModels = new ArrayList<>();
            QuerySettleBillRequest request = new QuerySettleBillRequest();
            request.setBillingCycle(yearmoth);
            request.setProductType(productType);
            QuerySettleBillResponse response = this.getBSSClient().querySettleBill(request);

            if (null == response) {
                return new BocloudResult(false, "查询用户某个账期内结算账单失败");
            }
            QuerySettleBillResponseBody.QuerySettleBillResponseBodyData data = response.getBody().getData();
            List<QuerySettleBillResponseBody.QuerySettleBillResponseBodyDataItemsItem> items = data.getItems().getItem();
            if (ListTool.isEmpty(items)) {
                return new BocloudResult(true, "暂无该月账单信息！");
            }
            return new BocloudResult(true, data, "查查询用户某个账期内结算账单成功");
        } catch (Exception e) {
            logger.error("查询用户某个账期内结算账单失败 ： " + e);
            return new BocloudResult(false, "查询用户某个账期内结算账单失败", e.getMessage());
        }
    }

    /**
     * @return com.bocloud.common.model.BocloudResult
     * @Author Hu Wentao
     * @Description 调用QueryInstanceBill查询用户某个账期内所有商品实例或计费项的消费汇总。
     * @Date 16:49 2020/7/29
     * @Param [yearmoth, productType]
     **/
    public BocloudResult queryInstanceBill(String yearmoth, String productType) {
        try {
            //List<PublicCloudBillModel> billModels = new ArrayList<>();
            QueryInstanceBillRequest request = new QueryInstanceBillRequest();
            request.setBillingCycle(yearmoth);
            //request.setProductType(productType);
            QueryInstanceBillResponse response = this.getBSSClient().queryInstanceBill(request);

            if (null == response) {
                return new BocloudResult(false, "查询用户某个账期内所有商品实例或计费项的消费汇总失败");
            }
            QueryInstanceBillResponseBody.QueryInstanceBillResponseBodyData data = response.getBody().getData();
            List<QueryInstanceBillResponseBody.QueryInstanceBillResponseBodyDataItemsItem> items = data.getItems().getItem();
            if (ListTool.isEmpty(items)) {
                return new BocloudResult(true, "暂无该月账单信息！");
            }
            return new BocloudResult(true, data, "查查询用户某个账期内所有商品实例或计费项的消费汇总成功");
        } catch (Exception e) {
            logger.error("查询用户某个账期内所有商品实例或计费项的消费汇总失败 ： " + e);
            return new BocloudResult(false, "查询用户某个账期内所有商品实例或计费项的消费汇总失败", e.getMessage());
        }
    }

    /**
     * @return com.bocloud.common.model.BocloudResult
     * @Author Hu Wentao
     * @Description 调用QuerySplitItemBill查询分账账单
     * @Date 16:57 2020/7/29
     * @Param [yearmoth, productType]
     **/
    public BocloudResult querySplitItemBill(String yearmoth, String productType) {
        try {
            //List<PublicCloudBillModel> billModels = new ArrayList<>();
            QuerySplitItemBillRequest request = new QuerySplitItemBillRequest();
            request.setBillingCycle(yearmoth);
            request.setProductType(productType);
            QuerySplitItemBillResponse response = this.getBSSClient().querySplitItemBill(request);

            if (null == response) {
                return new BocloudResult(false, "查询分账账单失败");
            }
            QuerySplitItemBillResponseBody.QuerySplitItemBillResponseBodyData data = response.getBody().getData();
            List<QuerySplitItemBillResponseBody.QuerySplitItemBillResponseBodyDataItemsItem> items = data.getItems().getItem();
            if (ListTool.isEmpty(items)) {
                return new BocloudResult(true, "暂无该月账单信息！");
            }
            return new BocloudResult(true, data, "查询分账账单成功");
        } catch (Exception e) {
            logger.error("查询分账账单失败 ： " + e);
            return new BocloudResult(false, "查询分账账单失败", e.getMessage());
        }
    }

    /**
     * @return com.bocloud.common.model.BocloudResult
     * @Author Hu Wentao
     * @Description 调用QueryAccountBill查询用户某个账期内的消费，并以资源所有者的维度进行汇总。
     * @Date 17:00 2020/7/29
     * @Param [yearmoth]
     **/
    public BocloudResult queryAccountBill(String yearmoth) {
        try {
            //List<PublicCloudBillModel> billModels = new ArrayList<>();
            QueryAccountBillRequest request = new QueryAccountBillRequest();
            request.setBillingCycle(yearmoth);
            QueryAccountBillResponse response = this.getBSSClient().queryAccountBill(request);

            if (null == response) {
                return new BocloudResult(false, "查询用户某个账期内的消费失败");
            }
            QueryAccountBillResponseBody.QueryAccountBillResponseBodyData data = response.getBody().getData();
            List<QueryAccountBillResponseBody.QueryAccountBillResponseBodyDataItemsItem> items = data.getItems().getItem();
            if (ListTool.isEmpty(items)) {
                return new BocloudResult(true, "暂无该月账单信息！");
            }
            return new BocloudResult(true, data, "查询用户某个账期内的消费成功");
        } catch (Exception e) {
            logger.error("查询用户某个账期内的消费失败 ： " + e);
            return new BocloudResult(false, "查询用户某个账期内的消费失败", e.getMessage());
        }
    }


}
