package com.bocloud.cmp.provider.common;

import java.math.BigDecimal;
import java.util.*;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.ecs20140526.models.*;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.database.DbFlavorModel;
import com.bocloud.cmp.model.database.DbInstanceModel;
import com.bocloud.cmp.model.database.DbInstanceParameterGroupModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.RegionConvertor;
import com.bocloud.cmp.provider.compute.AliyunImageProvider;
import com.bocloud.cmp.provider.compute.AliyunServerProvider;
import com.bocloud.cmp.provider.compute.AliyunSnapshotProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.database.AliyunDatabaseProvider;
import com.bocloud.cmp.provider.groups.AliyunResourceGroupsProvider;
import com.bocloud.cmp.provider.loadbalance.AliyunAccessControlProvider;
import com.bocloud.cmp.provider.loadbalance.AliyunLoadBalancerProvider;
import com.bocloud.cmp.provider.network.*;
import com.bocloud.cmp.provider.security.AliyunKeypairProvider;
import com.bocloud.cmp.provider.storage.AliyunBucketObjectProvider;
import com.bocloud.cmp.provider.storage.AliyunBucketProvider;
import com.bocloud.cmp.provider.storage.AliyunVolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;

/**
 * aliyun同步数据组装类
 *
 * @Author 郭鹏
 * @Date 2018/11/14
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunLocationProvider extends AliyunProvider implements LocationProvider {

	private static Logger logger = LoggerFactory.getLogger(AliyunLocationProvider.class);
	private Butler butler;

	private BeanConvertor<RegionModel, DescribeRegionsResponseBody.DescribeRegionsResponseBodyRegionsRegion> regionConvertor = new RegionConvertor();

	public AliyunLocationProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult syncAliCost(String billingCycle, String granularity, String billingDate) {
		try {
			logger.info("{} - ALIYUN Cost sync -->Start", this.butler.getName());
			AliyunSyncModel syncModel = new AliyunSyncModel();
			AliyunCostProvider costProvider = new AliyunCostProvider(this.butler);
			BocloudResult costResult = costProvider.cost(billingCycle, granularity, billingDate);
			if (costResult.isFailed()) {
				return new BocloudResult(false, "sync aliyun cost failure :" + costResult.getMessage());
			} else {
				List<AliCostModel> costModels = JSONObject.parseArray(JSONObject.toJSONString(costResult.getData()),
						AliCostModel.class);
				syncModel.setCostModels(costModels);
				logger.info("{} - ALIYUN Cost sync -->End", this.butler.getName());
				return new BocloudResult(true, syncModel, "success!");
			}
		} catch (Exception e) {
			logger.error("sync aliyun cost failure :", e);
			return new BocloudResult(false, "sync aliyun cost failure :" + e.getMessage());
		}
	}

	@Override
	public BocloudResult listRegions() {
		BocloudResult result = new BocloudResult(false, "查询失败");
		DescribeRegionsRequest request = new DescribeRegionsRequest();
		try {
			DescribeRegionsResponse response = this.getECSClient().describeRegions(request);
			List<DescribeRegionsResponseBody.DescribeRegionsResponseBodyRegionsRegion> regions = response.getBody()
					.getRegions().getRegion();
			result = new BocloudResult(true, regionConvertor.convertList(regions), "查询区域成功！");
		} catch (Exception e) {
			logger.error("listRegions error:", e);
			result = new BocloudResult(false, e.getMessage());
		}
		return result;
	}

	@SuppressWarnings("unchecked")
	@Override
	public BocloudResult sync() {
		BocloudResult result = null;
		AliyunSyncModel syncModel = new AliyunSyncModel();
		try {
			logger.info("开始同步阿里云数据当前区域 ：" + butler.getRegionId());
			// 同步域
			List<RegionModel> regionModels = Lists.newArrayList();
			RegionModel model = new RegionModel();
			model.setRegionId(this.getRegionId());
			model.setCode(this.getRegionId());
			DescribeZonesRequest describeZonesRequest = new DescribeZonesRequest();
			describeZonesRequest.setRegionId(this.getRegionId());
			DescribeZonesResponse describeZonesResponse = this.getECSClient().describeZones(describeZonesRequest);
			List<DescribeZonesResponseBody.DescribeZonesResponseBodyZonesZone> zones = describeZonesResponse.getBody()
					.getZones().getZone();
			model.setZones(this.convertZones(zones, this.getRegionId()));
			regionModels.add(model);
			syncModel.setRegionModels(regionModels);

			ResourceGroupModel defaultGroupModel = new ResourceGroupModel();
			logger.info("同步资源组 start ：" + butler.getRegionId());
			// 同步资源组
			AliyunResourceGroupsProvider resourceGroupsProvider = new AliyunResourceGroupsProvider(butler);
			BocloudResult resourceGroupsList = resourceGroupsProvider.list();
			if (resourceGroupsList.isSuccess()) {
				// 获取默认资源组
				List<ResourceGroupModel> resourceGroupModels = (List<ResourceGroupModel>) resourceGroupsList.getData();
				if (!ListTool.isEmpty(resourceGroupModels)) {
					for (ResourceGroupModel groupModel : resourceGroupModels) {
						if ("default".equalsIgnoreCase(groupModel.getName())) {
							defaultGroupModel = groupModel;
							break;
						}
					}
				}
				syncModel.setResourceGroupModels(resourceGroupModels);
			}
			logger.info("同步资源组 end ：" + butler.getRegionId());
			// 同步云主机规格
			logger.info("同步规格 start ：" + butler.getRegionId());
			List<FlavorModel> flavorModels = Lists.newArrayList();
			Map<String, FlavorModel> flavorModelMap = new HashMap<>();
			DescribeInstanceTypesRequest describeInstanceTypesRequest = new DescribeInstanceTypesRequest();
			DescribeInstanceTypesResponse describeInstanceTypesResponse = this.getECSClient()
					.describeInstanceTypes(describeInstanceTypesRequest);
			List<DescribeInstanceTypesResponseBody.DescribeInstanceTypesResponseBodyInstanceTypesInstanceType> instancesTypes = describeInstanceTypesResponse
					.getBody().getInstanceTypes().getInstanceType();
			for (DescribeInstanceTypesResponseBody.DescribeInstanceTypesResponseBodyInstanceTypesInstanceType instanceType : instancesTypes) {
				FlavorModel flavorModel = new FlavorModel();
				flavorModel.setCpu(instanceType.getCpuCoreCount());
				flavorModel.setMemory(new BigDecimal(instanceType.getMemorySize()));
				flavorModel.setFamily(instanceType.getInstanceTypeFamily());
				flavorModel.setPerformanceType(instancesType(instanceType.getInstanceTypeFamily()));
				flavorModel.setName(instanceType.getInstanceTypeId());
				flavorModel.setFlavorUuid(instanceType.getInstanceTypeId());
				flavorModel.setRegionId(this.getRegionId());
				flavorModelMap.put(instanceType.getInstanceTypeId(), flavorModel);
				flavorModels.add(flavorModel);
			}
			// 查询当前区域可用规格
			List<FlavorModel> availableFlavorModels = Lists.newArrayList();
			DescribeAvailableResourceRequest request = new DescribeAvailableResourceRequest();
			request.setIoOptimized("optimized");
			request.setDestinationResource("InstanceType");
			request.setRegionId(this.getRegionId());
			DescribeAvailableResourceResponse acsResponse = this.getECSClient().describeAvailableResource(request);
			List<DescribeAvailableResourceResponseBody.DescribeAvailableResourceResponseBodyAvailableZonesAvailableZone> availableZones = acsResponse
					.getBody().getAvailableZones().getAvailableZone();
			FlavorModel availableModel = null;
			for (DescribeAvailableResourceResponseBody.DescribeAvailableResourceResponseBodyAvailableZonesAvailableZone availableZone : availableZones) {
				String zoneId = availableZone.getZoneId();
				String availableRegion = availableZone.getRegionId();
				List<DescribeAvailableResourceResponseBody.DescribeAvailableResourceResponseBodyAvailableZonesAvailableZoneAvailableResourcesAvailableResource> availableResources = availableZone
						.getAvailableResources().getAvailableResource();
				for (DescribeAvailableResourceResponseBody.DescribeAvailableResourceResponseBodyAvailableZonesAvailableZoneAvailableResourcesAvailableResource availableResource : availableResources) {
					List<DescribeAvailableResourceResponseBody.DescribeAvailableResourceResponseBodyAvailableZonesAvailableZoneAvailableResourcesAvailableResourceSupportedResourcesSupportedResource> supportedResources = availableResource
							.getSupportedResources().getSupportedResource();
					for (DescribeAvailableResourceResponseBody.DescribeAvailableResourceResponseBodyAvailableZonesAvailableZoneAvailableResourcesAvailableResourceSupportedResourcesSupportedResource supportedResource : supportedResources) {
						availableModel = new FlavorModel();
						availableModel.setFlavorUuid(supportedResource.getValue());
						availableModel.setZone(zoneId);
						availableModel.setRegionId(availableRegion);
						availableFlavorModels.add(availableModel);
					}
				}
			}
			for (FlavorModel availableFlavorModel : availableFlavorModels) {
				String flavorUuid = availableFlavorModel.getFlavorUuid();
				if (flavorModelMap.containsKey(flavorUuid)) {
					FlavorModel falvorModel = flavorModelMap.get(flavorUuid);
					availableFlavorModel.setCpu(falvorModel.getCpu());
					availableFlavorModel.setMemory(falvorModel.getMemory());
					availableFlavorModel.setFamily(falvorModel.getFamily());
					availableFlavorModel.setPerformanceType(instancesType(falvorModel.getFamily()));
					availableFlavorModel.setName(falvorModel.getName());
				}
			}
			syncModel.setFlavorModels(availableFlavorModels);
			logger.info("同步规格 end ：" + butler.getRegionId());
			logger.info("同步镜像 start：" + butler.getRegionId());
			// 同步镜像
			AliyunImageProvider imageProvider = new AliyunImageProvider(this.butler);
			BocloudResult imageResult = imageProvider.list();
			if(imageResult.isFailed()) {
				return imageResult;
			}
			List<ImageModel> imageModels = JSONArray.parseArray(JSONArray.toJSONString(imageResult.getData()),
					ImageModel.class);
			syncModel.setImageModels(imageModels);
			logger.info("同步镜像 end：" + butler.getRegionId());
			logger.info("同步云硬盘 start ：" + butler.getRegionId());
			// 同步卷
			AliyunVolumeProvider volumeProvider = new AliyunVolumeProvider(this.butler);
			BocloudResult volumeResult = volumeProvider.list();
			if(volumeResult.isFailed()) {
				return volumeResult;
			}
			Map<String, List<VolumeModel>> volMap = new HashMap<String, List<VolumeModel>>();
			List<VolumeModel> volumeModels = JSONArray.parseArray(JSONArray.toJSONString(volumeResult.getData()),
					VolumeModel.class);
			for (VolumeModel volumeModel : volumeModels) {
				if (volMap.containsKey(volumeModel.getInstanceId())) {
					List<VolumeModel> list = volMap.get(volumeModel.getInstanceId());
					list.add(volumeModel);
					volMap.put(volumeModel.getInstanceId(), list);
				} else {
					List<VolumeModel> list = new ArrayList<>();
					list.add(volumeModel);
					volMap.put(volumeModel.getInstanceId(), list);
				}
				String resourceGroupUuid = volumeModel.getResourceGroupUuid();
				if (StringUtils.isEmpty(resourceGroupUuid)) {
					volumeModel.setResourceGroupUuid(defaultGroupModel.getResourceGroupUuid());
				}
			}
			syncModel.setVolumeModels(volumeModels);
			logger.info("同步云硬盘 end ：" + butler.getRegionId());
			logger.info("同步快照 start ：" + butler.getRegionId());
			// 同步快照
			AliyunSnapshotProvider snapshotProvider = new AliyunSnapshotProvider(this.butler);
			BocloudResult snapshotResult = snapshotProvider.list();
			if(snapshotResult.isFailed()) {
				return snapshotResult;
			}
			List<SnapshotModel> snapshotModels = JSONArray
					.parseArray(JSONArray.toJSONString(snapshotResult.getData()), SnapshotModel.class);
			for (SnapshotModel snapshotModel : snapshotModels) {
				String resourceGroupUuid = snapshotModel.getResourceGroupUuid();
				if (StringUtils.isEmpty(resourceGroupUuid)) {
					snapshotModel.setResourceGroupUuid(defaultGroupModel.getResourceGroupUuid());
				}
			}
			syncModel.setSnapshotModels(snapshotModels);
			logger.info("同步快照 end ：" + butler.getRegionId());
			logger.info("同步安全组 start ：" + butler.getRegionId());
			// 同步安全组
			AliyunSecurityGroupProvider groupProvider = new AliyunSecurityGroupProvider(this.butler);
			BocloudResult groupResult = groupProvider.list();
			if(groupResult.isFailed()) {
				return groupResult;
			}
			List<SecurityGroupModel> securityGroupModels = JSONArray
					.parseArray(JSONArray.toJSONString(groupResult.getData()), SecurityGroupModel.class);
			for (SecurityGroupModel securityGroupModel : securityGroupModels) {
				String resourceGroupUuid = securityGroupModel.getResourceGroupUuid();
				if (StringUtils.isEmpty(resourceGroupUuid)) {
					securityGroupModel.setResourceGroupUuid(defaultGroupModel.getResourceGroupUuid());
				}
			}
			syncModel.setSecurityGroupModels(securityGroupModels);
			logger.info("同步安全组 start ：" + butler.getRegionId());
			logger.info("同步网卡数据 start ：" + butler.getRegionId());
			// 同步弹性网卡(网络接口)
			AliyunNetworkInterfaceProvider networkInterfaceProvider = new AliyunNetworkInterfaceProvider(butler);
			BocloudResult networkInterfaceResult = networkInterfaceProvider.list();
			Map<String, NetworkInterfaceModel> netMap = new HashMap<String, NetworkInterfaceModel>();
			if (networkInterfaceResult.isSuccess()) {
				List<NetworkInterfaceModel> networkInterfaceModels = (List<NetworkInterfaceModel>) networkInterfaceResult
						.getData();
				for (NetworkInterfaceModel networkInterfaceModel : networkInterfaceModels) {
					String resourceGroupUuid = networkInterfaceModel.getResourceGroupUuid();
					if (StringUtils.isEmpty(resourceGroupUuid)) {
						networkInterfaceModel.setResourceGroupUuid(defaultGroupModel.getResourceGroupUuid());
					}
				}
				syncModel.setNetworkInterfaceModels(networkInterfaceModels);
				for (NetworkInterfaceModel networkInterfaceModel : networkInterfaceModels) {
					if ("Primary".equalsIgnoreCase(networkInterfaceModel.getInterfaceType())
							&& !StringUtils.isEmpty(networkInterfaceModel.getInstanceId())) {
						netMap.put(networkInterfaceModel.getInstanceId(), networkInterfaceModel);
					}
				}
			}
			logger.info("同步网卡数据 end ：" + butler.getRegionId());
			logger.info("同步云主机数据 start ：" + butler.getRegionId());
			// 同步云主机
			AliyunServerProvider serverProvider = new AliyunServerProvider(this.butler);
			BocloudResult serverResult = serverProvider.list();
			if(serverResult.isFailed()) {
				return serverResult;
			}
			List<ServerModel> serverModels = JSONArray.parseArray(JSONArray.toJSONString(serverResult.getData()),
					ServerModel.class);
			for (ServerModel serverModel : serverModels) {
				if (volMap.containsKey(serverModel.getInstanceId())) {
					Integer diskSize = 0;
					List<VolumeModel> list = volMap.get(serverModel.getInstanceId());
					for (VolumeModel volumeModel : list) {
						if (null != volumeModel.getBootVolume() && volumeModel.getBootVolume()) {
							diskSize += volumeModel.getSize();
							break;
						}
					}
					serverModel.setDisk(diskSize);
				}
				String resourceGroupUuid = serverModel.getResourceGroupUuid();
				if (StringUtils.isEmpty(resourceGroupUuid)) {
					serverModel.setResourceGroupUuid(defaultGroupModel.getResourceGroupUuid());
				}
				List<String> ips = JSONObject.parseArray(serverModel.getPrivateIps(), String.class);
				if (null == ips) {
					ips = new ArrayList<String>();
				}
				if (netMap.containsKey(serverModel.getInstanceId())) {
					String ipv6 = netMap.get(serverModel.getInstanceId()).getIpv6Address();
					if (!StringUtils.isEmpty(ipv6)) {
						ips.add(ipv6);
					}
				}
				serverModel.setPrivateIps(JSONObject.toJSONString(ips));
			}
			syncModel.setServerModels(serverModels);
			logger.info("同步云主机数据 start ：" + butler.getRegionId());
			logger.info("同步VPC数据 start ：" + butler.getRegionId());
			// 同步vpc
			AliyunVpcProvider vpcProvider = new AliyunVpcProvider(this.butler);
			BocloudResult vpcResult = vpcProvider.list();
			if(vpcResult.isFailed()) {
				return vpcResult;
			}
			List<VpcModel> vpcModels = JSONArray.parseArray(JSONArray.toJSONString(vpcResult.getData()),
					VpcModel.class);
			for (VpcModel vpcModel : vpcModels) {
				String resourceGroupUuid = vpcModel.getResourceGroupUuid();
				if (StringUtils.isEmpty(resourceGroupUuid)) {
					vpcModel.setResourceGroupUuid(defaultGroupModel.getResourceGroupUuid());
				}
			}
			syncModel.setVpcModels(vpcModels);
			logger.info("同步VPC数据 end ：" + butler.getRegionId());
			logger.info("同步子网数据 start ：" + butler.getRegionId());
			// 同步交换机
			AliyunSubnetProvider subnetProvider = new AliyunSubnetProvider(butler);
			BocloudResult switchResult = subnetProvider.list();
			if(switchResult.isFailed()) {
				return switchResult;
			}
			List<SubnetModel> subnetModels = (List<SubnetModel>) switchResult.getData();
			for (SubnetModel subnetModel : subnetModels) {
				String resourceGroupUuid = subnetModel.getResourceGroupUuid();
				if (StringUtils.isEmpty(resourceGroupUuid)) {
					subnetModel.setResourceGroupUuid(defaultGroupModel.getResourceGroupUuid());
				}
			}
			syncModel.setSubnetModels(subnetModels);
			logger.info("同步子网数据 end ：" + butler.getRegionId());
			logger.info("同步密钥数据 start ：" + butler.getRegionId());
			// 同步秘钥
			AliyunKeypairProvider keyPairProvider = new AliyunKeypairProvider(butler);
			BocloudResult keyPairResult = keyPairProvider.list();
			if (keyPairResult.isSuccess()) {
				List<KeypairModel> keypairModels = (List<KeypairModel>) keyPairResult.getData();
				for (KeypairModel keypairModel : keypairModels) {
					String resourceGroupUuid = keypairModel.getResourceGroupUuid();
					if (StringUtils.isEmpty(resourceGroupUuid)) {
						keypairModel.setResourceGroupUuid(defaultGroupModel.getResourceGroupUuid());
					}
				}
				syncModel.setKeypairModels(keypairModels);
			}
			logger.info("同步密钥数据 end ：" + butler.getRegionId());
			logger.info("同步路由数据 start ：" + butler.getRegionId());
			// 同步路由器
			AliyunRouterProvider routerProvider = new AliyunRouterProvider(butler);
			BocloudResult routerResult = routerProvider.list();
			if (routerResult.isSuccess()) {
				List<RouterModel> routerModels = (List<RouterModel>) routerResult.getData();
				syncModel.setRouterModels(routerModels);
			}
			logger.info("同步路由数据 end ：" + butler.getRegionId());
			// 同步路由表
			AliyunRouteTableProvider routeTableProvider = new AliyunRouteTableProvider(butler);
			BocloudResult routeTableResult = routeTableProvider.list();
			if (routeTableResult.isSuccess()) {
				List<RouteTableModel> routeTableModels = (List<RouteTableModel>) routeTableResult.getData();
				syncModel.setRouteTableModels(routeTableModels);
			}
			logger.info("同步浮动IP数据 start ：" + butler.getRegionId());
			// 同步弹性公网IP
			AliyunElasticIpProvider elasticIpProvider = new AliyunElasticIpProvider(butler);
			BocloudResult elasticIpResult = elasticIpProvider.list();
			if(elasticIpResult.isFailed()) {
				return elasticIpResult;
			}
			List<ElasticIpModel> elasticIpModels = (List<ElasticIpModel>) elasticIpResult.getData();
			syncModel.setElasticIpModels(elasticIpModels);
			logger.info("同步浮动IP数据 end ：" + butler.getRegionId());
			logger.info("同步NAT网关数据 start ：" + butler.getRegionId());
			// 同步NAT网关
			AliyunNatGatewayProvider natGatewayProvider = new AliyunNatGatewayProvider(butler);
			BocloudResult natGatewayResult = natGatewayProvider.list();
			if (natGatewayResult.isSuccess()) {
				List<NatGatewayModel> gatewayModels = (List<NatGatewayModel>) natGatewayResult.getData();
				syncModel.setNatGatewayModels(gatewayModels);
			}
			logger.info("同步NAT网关数据 end ：" + butler.getRegionId());
			logger.info("同步负载均衡数据 start ：" + butler.getRegionId());
			// 同步负载均衡实例
			AliyunLoadBalancerProvider loadBalancerProvider = new AliyunLoadBalancerProvider(butler);
			BocloudResult loadBalancerResult = loadBalancerProvider.list();
			if(loadBalancerResult.isFailed()) {
				return loadBalancerResult;
			}
			List<LbModel> lbModels = (List<LbModel>) loadBalancerResult.getData();
			syncModel.setLbModels(lbModels);
			// 同步负载监听器,pool虚拟服务器组
			List<LbListenerModel> listeners = Lists.newArrayList();
			List<LbPoolModel> pools = Lists.newArrayList();
			logger.info("同步负载均衡监听器 start ：" + butler.getRegionId());
			for (LbModel lbModel : lbModels) {
				// 监听器
				BocloudResult listenerResult = loadBalancerProvider.listListener(lbModel.getLbUuid());
				if (listenerResult.isSuccess()) {
					listeners.addAll((List<LbListenerModel>) listenerResult.getData());
				}
				// 虚拟服务器组
				BocloudResult vServerGroupResult = loadBalancerProvider.listVServerGroup(lbModel.getLbUuid());
				if (vServerGroupResult.isSuccess()) {
					pools.addAll((List<LbPoolModel>) vServerGroupResult.getData());
				}
			}
			syncModel.setListenerModels(listeners);
			syncModel.setPoolModels(pools);
			// 同步member后端服务器
			List<LbMemberModel> members = Lists.newArrayList();
			logger.info("同步负载均衡服务器组 start  ：" + butler.getRegionId());
			for (LbPoolModel pool : pools) {
				BocloudResult poolResult = loadBalancerProvider.listBackendServer(pool.getPoolUuid());
				if (poolResult.isSuccess()) {
					members.addAll((List<LbMemberModel>) poolResult.getData());
				}
			}
			syncModel.setMemberModels(members);
			logger.info("同步负载均衡数据 end ：" + butler.getRegionId());
			logger.info("同步访问控制 start ：" + butler.getRegionId());
			// 同步访问控制
			List<AccessControlModel> acls = Lists.newArrayList();
			AliyunAccessControlProvider accessControlProvider = new AliyunAccessControlProvider(butler);
			BocloudResult aclsResult = accessControlProvider.list();
			if (aclsResult.isSuccess()) {
				acls.addAll((List<AccessControlModel>) aclsResult.getData());
				syncModel.setAcls(acls);
			}
			// 同步rds
			logger.info("同步云数据库 start ：" + butler.getRegionId());
			AliyunDatabaseProvider databaseProvider = new AliyunDatabaseProvider(butler);
			BocloudResult listInstances = databaseProvider.listInstances();
			if (listInstances.isSuccess()) {
				List<DbInstanceModel> dbInstanceModels = (List<DbInstanceModel>) listInstances.getData();
				syncModel.setDbInstanceModels(dbInstanceModels);
			}
			logger.info("同步云数据库 end ：" + butler.getRegionId());
			// 同步rds实例的规格（目前只做按量付费）
			logger.info("同步云数据库规则信息 start ：" + butler.getRegionId());
			BocloudResult flavors = databaseProvider.getFlavor("Postpaid");
			if (flavors.isSuccess()) {
				List<DbFlavorModel> dbFlavorModels = (List<DbFlavorModel>) flavors.getData();
				syncModel.setDbFlavorModel(dbFlavorModels);
			}
			logger.info("同步云数据库规则信息 end ：" + butler.getRegionId());
			// 同步参数模板
			logger.info("同步云数据库参数模板数据 start ：" + butler.getRegionId());
			BocloudResult parameterGroups = databaseProvider.getParameterGroups(butler.getRegionId());
			if (parameterGroups.isSuccess()) {
				List<DbInstanceParameterGroupModel> parameterGroupsData = (List<DbInstanceParameterGroupModel>) parameterGroups
						.getData();
				syncModel.setDbInstanceParameterGroupModel(parameterGroupsData);
			}
			logger.info("同步云数据库参数模板数据 end ：" + butler.getRegionId());
			// 同步对象存储
			logger.info("同步对象存储数据 start ：" + butler.getRegionId());
			if (StringUtils.isEmpty(butler.getUrl())) {
				AliyunBucketProvider bucketProvider = new AliyunBucketProvider(butler);
				BocloudResult bucketResult = bucketProvider.list();
				if (bucketResult.isSuccess()) {
					AliyunBucketObjectProvider bucketObjectProvider = new AliyunBucketObjectProvider(butler);
					List<BucketModel> bucketModels = (List<BucketModel>) bucketResult.getData();
					for (BucketModel bucketModel : bucketModels) {
						BocloudResult bucketObjectResult = bucketObjectProvider.list(bucketModel.getName());
						if (bucketObjectResult.isSuccess()) {
							List<BucketObjectModel> bucketObjectModels = (List<BucketObjectModel>) bucketObjectResult
									.getData();
							bucketModel.setBucketObjectModels(bucketObjectModels);
						}
					}
					syncModel.setBucketModels(bucketModels);
				}
			}
			logger.info("同步对象存储数据 end ：" + butler.getRegionId());
			result = new BocloudResult(true, syncModel, "数据同步成功！");
			logger.info("阿里云平台数据同步完完成，数据保存中...... " + butler.getRegionId());
		} catch (Exception e) {
			logger.error("Get error message：", e);
			result = new BocloudResult(false, e.getMessage());
		} finally {
		}
		return result;
	}

	@Override
	public BocloudResult listZones(String region) {
		BocloudResult result = null;
		DescribeZonesRequest request = new DescribeZonesRequest();
		request.setRegionId(region);
		try {
			DescribeZonesResponse response = this.getECSClient().describeZones(request);
			List<DescribeZonesResponseBody.DescribeZonesResponseBodyZonesZone> zones = response.getBody().getZones()
					.getZone();
			result = new BocloudResult(true, zones, "success");
		} catch (Exception e) {
			logger.error("listRegions error:", e);
			result = new BocloudResult(false, e.getMessage());
		}
		return result;
	}

	@Override
	public BocloudResult syncPortByNetworkUuid(String networkUuid) {
		return null;
	}

	@Override
	public BocloudResult getHostModel(HostModel host) {
		return null;
	}

	private List<ZoneModel> convertZones(List<DescribeZonesResponseBody.DescribeZonesResponseBodyZonesZone> zones,
			String region) {
		List<ZoneModel> models = Lists.newArrayList();
		if (ListTool.isEmpty(zones)) {
			return models;
		}
		for (DescribeZonesResponseBody.DescribeZonesResponseBodyZonesZone zone : zones) {
			ZoneModel model = new ZoneModel();
			model.setName(zone.getLocalName());
			model.setRegionId(region);
			model.setZoneId(zone.getZoneId());
			model.setResources(JSONObject.toJSONString(zone.getAvailableResources()));
			model.setDiskCategory(convertDiskCategory(zone.getAvailableDiskCategories().getDiskCategories()));
			models.add(model);
		}
		return models;
	}

	// 本文所列实例规格在中国站已全部停售，其中，sn2、sn1、n1、n2和e3在国际站（International）仍然在售。
	private static final String[] SOLD_OUT_TYPE = { "ecs.sn2", "ecs.sn1", "ecs.c4", "ecs.ce4", "ecs.cm4", "ecs.n1",
			"ecs.n2", "ecs.e3" };
	// 通用型
	private static final String[] COMMON_TYPE = { "ecs.g6", "ecs.g5", "ecs.sn2ne", "ecs.sn2" };
	// 计算型
	private static final String[] COMPUTATIONAL_TYPE = { "ecs.ic5", "ecs.c6", "ecs.c5", "ecs.sn1ne", "ecs.sn1" };
	// 内存型
	private static final String[] MEMORY_TYPE = { "ecs.r6", "ecs.r5", "ecs.re4", "ecs.re4e", "ecs.se1ne", "ecs.se1" };
	// 大数据型
	private static final String[] BIG_DATA_TYPE = { "ecs.d1ne", "ecs.d1" };
	// 本地SSD型
	private static final String[] LOCAL_SSD_TYPE = { "ecs.i2", "ecs.i2g", "ecs.i1" };
	// 高主频型
	private static final String[] HIGH_FREQUENCY_TYPE = { "ecs.hfc6", "ecs.hfg6", "ecs.hfr6", "ecs.hfc5", "ecs.hfg5" };
	// 入门级(共享)
	private static final String[] SHARED_TYPE = { "ecs.xn4", "ecs.n4", "ecs.mn4", "ecs.e4", "ecs.n1", "ecs.n2",
			"ecs.e3" };

	private String instancesType(String instanceFamily) {
		if (StringUtils.isEmpty(instanceFamily)) {
			return "其他";
		}
		if (Arrays.asList(COMMON_TYPE).contains(instanceFamily)) {
			return "通用型";
		} else if (Arrays.asList(COMPUTATIONAL_TYPE).contains(instanceFamily)) {
			return "计算型";
		} else if (Arrays.asList(MEMORY_TYPE).contains(instanceFamily)) {
			return "内存型";
		} else if (Arrays.asList(BIG_DATA_TYPE).contains(instanceFamily)) {
			return "大数据型";
		} else if (Arrays.asList(LOCAL_SSD_TYPE).contains(instanceFamily)) {
			return "本地SSD型";
		} else if (Arrays.asList(HIGH_FREQUENCY_TYPE).contains(instanceFamily)) {
			return "高主频型";
		} else if (Arrays.asList(SHARED_TYPE).contains(instanceFamily)) {
			return "入门级(共享)";
		} else {
			return "其他";
		}
	}

	/**
	 * 硬盘类型元数据转换
	 *
	 * @param categorys
	 * @return
	 */
	private String convertDiskCategory(List<String> categorys) {
		StringBuffer stringBuffer = new StringBuffer();
		if (ListTool.isEmpty(categorys)) {
			return stringBuffer.toString();
		}
		for (String category : categorys) {
			if (category.equalsIgnoreCase("cloud")) {
				stringBuffer.append(category.toUpperCase()).append(",");
			} else {
				stringBuffer.append(category.split("_")[1].toUpperCase()).append(",");
			}
		}
		stringBuffer.deleteCharAt(stringBuffer.length() - 1);
		return stringBuffer.toString();
	}

}
