package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.ecs20140526.Client;
import com.aliyun.ecs20140526.models.*;
import com.aliyun.tea.TeaException;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.entity.enums.TagType;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.ServerConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.network.AliyunNetworkInterfaceProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.KeyGenerator;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * 阿里云 云主机
 *
 * @author ymh
 * @时间 2019年3月6日
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunServerProvider extends AliyunProvider implements ServerProvider {

	private static Logger logger = LoggerFactory.getLogger(AliyunServerProvider.class);

	private BeanConvertor<ServerModel, DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance> convertor = new ServerConvertor();

	private static final int PAGE_SIZE = 100;

	private Butler butler;

	public AliyunServerProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult list() {
		try {
			DescribeInstancesRequest describe = new DescribeInstancesRequest();
			describe.setPageNumber(1);
			describe.setPageSize(PAGE_SIZE);
			describe.setRegionId(this.getRegionId());
			List<ServerModel> models = Lists.newArrayList();

			Client client = this.getECSClient();
			DescribeInstancesResponse response = client.describeInstances(describe);
			List<DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance> instances = response
					.getBody().getInstances().getInstance();
			if(!ListTool.isEmpty(instances)) {
			    logger.info("云主机 info : " + JSON.toJSONString(instances.get(0)));
			}
			Integer totalCount = response.getBody().getTotalCount();
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, models, "empty servers");
			}
			for (DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance instance : instances) {
				ServerModel convertModel = convertor.convertModel(instance);
				convertModel.setRegionId(this.getRegionId());
				models.add(convertModel);
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, models, "云服务器列表查询成功");
			}
			for (int i = 2; i <= totalPage; i++) {
				describe.setPageNumber(i);
				describe.setPageSize(PAGE_SIZE);
				response = this.getECSClient().describeInstances(describe);
				instances = response.getBody().getInstances().getInstance();
				for (DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance instance : instances) {
					ServerModel convertModel = convertor.convertModel(instance);
					convertModel.setRegionId(this.getRegionId());
					models.add(convertModel);
				}
			}
			return new BocloudResult(true, models, "云服务器列表查询成功");
		} catch (Exception e) {
			logger.error("云服务器列表查询失败:" + e.getMessage());
			return new BocloudResult(false, "云服务器列表查询失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult list(String hostId) {
		return null;
	}

	@Override
	public BocloudResult create(ServerModel model) {
		logger.info("serverModel : " + JSON.toJSONString(model));
		CreateInstanceRequest request = new CreateInstanceRequest();
		request.setZoneId(model.getZone());
		request.setImageId(model.getImageUuid());
		request.setInstanceName(model.getName());
		request.setDescription(model.getRemark());
		request.setInstanceType(model.getInstanceType());
		request.setRegionId(this.getRegionId());
		String keypairName = model.getKeypairName();
		if (null != keypairName && !"".equals(keypairName)) {
			request.setKeyPairName(keypairName);
		} else if (!StringUtils.isEmpty(model.getPassword())) {
			logger.info("password : " + model.getPassword());
			request.setPassword(model.getPassword());
		}
		if (!StringUtils.isEmpty(model.getHostName())) {
			request.setHostName(model.getHostName());
		}
		request.setSecurityGroupId(model.getGroups());
		request.setVSwitchId(model.getSubnetUuid());
		CreateInstanceRequest.CreateInstanceRequestSystemDisk systemDisk = new CreateInstanceRequest.CreateInstanceRequestSystemDisk();
		systemDisk.setSize(Integer.valueOf(model.getSystemDiskSize()));
		if (!StringUtils.isEmpty(model.getSystemDiskCategory())) {
			systemDisk.setCategory(model.getSystemDiskCategory());
		}
		if (!StringUtils.isEmpty(model.getSystemDiskPerformanceLevel())) {
			systemDisk.setPerformanceLevel(model.getSystemDiskPerformanceLevel());
		}
		request.setSystemDisk(systemDisk);
		Integer diskSize = Integer.valueOf(model.getSystemDiskSize());
		if (!StringUtils.isEmpty(model.getDataDisks())) {
			JSONArray dataDisks = JSONArray.parseArray(model.getDataDisks());
			if (dataDisks.size() > 0) {
				request.setDataDisk(buildDataDisks(dataDisks));
				for (int i = 0; i < dataDisks.size(); i++) {
					diskSize += dataDisks.getJSONObject(i).getInteger("dataDiskSize");
				}
			}
		}
		if (!StringUtils.isEmpty(model.getManagerIp())) {
			request.setPrivateIpAddress(model.getManagerIp());
		}
		if (!StringUtils.isEmpty(model.getResourceGroupUuid())) {
			request.setResourceGroupId(model.getResourceGroupUuid());
		}
		request.setClientToken(KeyGenerator.uuid());
		request.setDeletionProtection(model.getTerminationProtected());
		/***
		 * PrePaid：包年包月 PostPaid（默认）：按量付费
		 */
		if (!StringUtils.isEmpty(model.getInstanceChargeType())) {
			request.setInstanceChargeType(model.getInstanceChargeType());
		}
		/***
		 * PeriodUnit=Week时，Period取值：{“1”, “2”, “3”, “4”}。
		 * PeriodUnit=Month时，Period取值：{“1”, “2”, “3”, “4”, “5”, “6”, “7”, “8”, “9”,
		 * “12”, “24”, “36”, ”48”, ”60”}。
		 */
		if (null != model.getPeriod()) {
			request.setPeriod(model.getPeriod());
		}
		if (!StringUtils.isEmpty(model.getPeriodUnit())) {
			request.setPeriodUnit(model.getPeriodUnit());
		}
		if (null != model.getAutoRenew()) {
			request.setAutoRenew(model.getAutoRenew());
		}
		if (null != model.getAutoRenewPeriod()) {
			request.setAutoRenewPeriod(model.getAutoRenewPeriod());
		}
		try {
			CreateInstanceResponse response = this.getECSClient().createInstance(request);
			String instanceId = response.getBody().getInstanceId();
			if (StringUtils.isEmpty(instanceId)) {
				return new BocloudResult(false, "云服务器创建失败");
			}
			ServerModel instance = getInstance(instanceId);
			if (null != instance) {
				instance.setRegionId(this.getRegionId());
			}
			BocloudResult serverAction = this.start(instanceId);
			if (serverAction.isSuccess()) {
				instance = getInstance(instanceId);
				instance.setStatus(BocloudStatus.Server.RUNNING.name());
			}
			// 设置默认安全组
			if (!StringUtils.isEmpty(model.getResourceGroupUuid())) {
				instance.setResourceGroupUuid(model.getResourceGroupUuid());
			} else {
				instance.setResourceGroupUuid(this.getDefaultResourceGroup().getResourceGroupUuid());
			}
			return new BocloudResult(true, instance, "success");
		} catch (Exception e) {
			logger.error("云服务器创建失败:" + e);
			return new BocloudResult(false, "云服务器创建失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult disableApiTermination(String instanceId, Boolean enabled) {
		ModifyInstanceAttributeRequest request = new ModifyInstanceAttributeRequest();
		request.setInstanceId(instanceId);
		request.setDeletionProtection(enabled);
		try {
			ModifyInstanceAttributeResponse response = this.getECSClient().modifyInstanceAttribute(request);
			return new BocloudResult(true, this.detail(instanceId).getData(), "云服务器终止保护状态修改成功");
		} catch (Exception e) {
			logger.error("云服务器终止保护状态修改失败:" + e);
			return new BocloudResult(false, "云服务器终止保护状态修改失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(ServerModel model) {
		ModifyInstanceAttributeRequest attributeRequest = new ModifyInstanceAttributeRequest();
		attributeRequest.setInstanceId(model.getInstanceId());
		if (!StringUtils.isEmpty(model.getRemark())) {
			attributeRequest.setDescription(model.getRemark());
		}
		if (!StringUtils.isEmpty(model.getName())) {
			attributeRequest.setInstanceName(model.getName());
		}
		if (!ListTool.isEmpty(model.getSecurityGroups())) {
			attributeRequest.setSecurityGroupIds(model.getSecurityGroups());
		}
		try {
			this.getECSClient().modifyInstanceAttribute(attributeRequest);
			return new BocloudResult(true, this.detail(model.getInstanceId()).getData(), "云服务器修改成功");
		} catch (Exception e) {
			logger.error("云服务器修改失败:" + e);
			return new BocloudResult(false, "云服务器修改失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult createTag(ServerModel model) {
		try {
			List<TagModel> tagModels = model.getTagModels();
			// 创建标签
			List<TagResourcesRequest.TagResourcesRequestTag> tags = Lists.newArrayList();
			TagResourcesRequest tagRequest = new TagResourcesRequest();
			tagRequest.setResourceType(TagType.instance.name());
			tagRequest.setResourceId(Collections.singletonList(model.getInstanceId()));
			tagRequest.setRegionId(this.getRegionId());
			for (TagModel tagModel : tagModels) {
				TagResourcesRequest.TagResourcesRequestTag tag = new TagResourcesRequest.TagResourcesRequestTag();
				tag.setKey(tagModel.getKey());
				tag.setValue(tagModel.getValue());
				tags.add(tag);
			}
			tagRequest.setTag(tags);
			tagRequest.setRegionId(this.getRegionId());
			TagResourcesResponse response = this.getECSClient().tagResources(tagRequest);
			return new BocloudResult(true, "云服务器标签创建成功");
		} catch (Exception e) {
			logger.error("云服务器标签创建失败:" + e);
			return new BocloudResult(false, "云服务器标签创建失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult removeTag(ServerModel model) {
		try {
			List<TagModel> tagModels = model.getTagModels();
			// 创建标签
			List<String> tagKeys = Lists.newArrayList();
			UntagResourcesRequest tagRequest = new UntagResourcesRequest();
			tagRequest.setResourceType(TagType.instance.name());
			tagRequest.setRegionId(this.getRegionId());
			tagRequest.setResourceId(Collections.singletonList(model.getInstanceId()));

			for (TagModel tagModel : tagModels) {
				tagKeys.add(tagModel.getKey());
			}
			tagRequest.setTagKey(tagKeys);
			UntagResourcesResponse response = this.getECSClient().untagResources(tagRequest);
			return new BocloudResult(true, "云服务器标签移除成功");
		} catch (Exception e) {
			logger.error("云服务器标签移除失败:" + e);
			return new BocloudResult(false, "云服务器标签移除失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult migrate(ServerModel server) {
		return null;
	}

	@Override
	public BocloudResult migrate(OSMigrateModel model) {
		return null;
	}

	@Override
	public BocloudResult remove(String instanceId, String name) {
		DeleteInstanceRequest deleteInstanceRequest = new DeleteInstanceRequest();
		deleteInstanceRequest.setInstanceId(instanceId);
		try {
			DeleteInstanceResponse response = this.getECSClient().deleteInstance(deleteInstanceRequest);
			return new BocloudResult(true, response.getBody().getRequestId(), "云服务器删除中");
		} catch (Exception e) {
			logger.error("云服务器删除失败:" + e);
			return new BocloudResult(false, "云服务器删除失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String instanceId) {
		DescribeInstancesRequest request = new DescribeInstancesRequest();
		request.setRegionId(this.getRegionId());
		JSONArray instanceIds = new JSONArray(1);
		instanceIds.add(instanceId);
		request.setInstanceIds(instanceIds.toJSONString());
		try {
			DescribeInstancesResponse response = this.getECSClient().describeInstances(request);
			List<DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance> instances = response
					.getBody().getInstances().getInstance();
			int totalCount = response.getBody().getTotalCount();
			if (totalCount == 0) {
				return new BocloudResult(false, "实例不存在");
			}
			DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance instance = instances.get(0);
			ServerModel model = convertor.convertModel(instance);
			List<String> ips = JSONObject.parseArray(model.getPrivateIps(), String.class);
			if (null == ips) {
				ips = new ArrayList<String>();
			}
			List<DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstanceNetworkInterfacesNetworkInterface> interfaces = instance
					.getNetworkInterfaces().getNetworkInterface();
			if (!ListTool.isEmpty(interfaces)) {
				for (DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstanceNetworkInterfacesNetworkInterface networkInterface : interfaces) {
					if (!"Primary".equalsIgnoreCase(networkInterface.getType())) {
						continue;
					}
					BocloudResult result = new AliyunNetworkInterfaceProvider(butler)
							.detail(networkInterface.getNetworkInterfaceId());
					if (!result.isFailed()) {
						continue;
					}
					NetworkInterfaceModel interfaceModel = (NetworkInterfaceModel) result.getData();
					if (!StringUtils.isEmpty(interfaceModel.getIpv6Address())) {
						ips.add(interfaceModel.getIpv6Address());
					}
				}
			}
			model.setPrivateIps(JSONObject.toJSONString(ips));
			return new BocloudResult(true, model, "success");
		} catch (Exception e) {
			logger.error("云服务器详情查询失败:" + e);
			return new BocloudResult(false, "云服务器详情查询失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult start(String instanceId) {
		StartInstanceRequest startInstanceRequest = new StartInstanceRequest();
		startInstanceRequest.setInstanceId(instanceId);
		StartInstanceResponse response;
		try {
			response = this.getECSClient().startInstance(startInstanceRequest);
			return new BocloudResult(true, response.getBody().getRequestId(), "云服务器启动中");
		} catch (Exception e) {
			logger.error("云服务器启动失败：" + e);
			return new BocloudResult(false, "云服务器启动失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult stop(String instanceId) {
		StopInstanceRequest stopInstanceRequest = new StopInstanceRequest();
		stopInstanceRequest.setInstanceId(instanceId);
		StopInstanceResponse response;
		try {
			response = this.getECSClient().stopInstance(stopInstanceRequest);
			return new BocloudResult(true, getInstance(instanceId), "云服务器关闭中");
		} catch (Exception e) {
			logger.error("云服务器停止失败：" + e);
			return new BocloudResult(false, "云服务器停止失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult reboot(String instanceId) {
		RebootInstanceRequest rebootInstanceRequest = new RebootInstanceRequest();
		rebootInstanceRequest.setInstanceId(instanceId);
		try {
			RebootInstanceResponse response = this.getECSClient().rebootInstance(rebootInstanceRequest);
			return new BocloudResult(true, response.getBody().getRequestId(), "云服务器重启中");
		} catch (Exception e) {
			logger.error("云服务器重启失败：" + e);
			return new BocloudResult(false, "云服务器重启失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult softReboot(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult pause(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult unPause(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult suspend(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult resume(String instanceId) {
		return null;
	}

	/**
	 * 加入安全组之前，实例必须处于 已停止（Stopped）或者 运行中（Running）状态。 一台实例最多可以加入 5 个安全组。 您可以
	 * 提交工单申请将实例加入更多安全组，最多不能超过 16 个安全组。 每个安全组最多能管理 1000 台实例。 您的安全组和实例必须属于同一个阿里云地域。
	 * 您的安全组和实例的网络类型必须相同。如果网络类型为 专有网络 VPC，则安全组和实例必须属于同一个 VPC。
	 */
	@Override
	public BocloudResult joinSGroup(String instanceId, String group) {
		JoinSecurityGroupRequest joinSecurityGroupRequest = new JoinSecurityGroupRequest();
		joinSecurityGroupRequest.setInstanceId(instanceId);
		joinSecurityGroupRequest.setRegionId(this.getRegionId());
		joinSecurityGroupRequest.setSecurityGroupId(group);
		try {
			JoinSecurityGroupResponse response = this.getECSClient().joinSecurityGroup(joinSecurityGroupRequest);
			return new BocloudResult(true, response.getBody().getRequestId(), "操作成功！");
		} catch (Exception e) {
			TeaException teaException = (TeaException)e;
			String code = teaException.getCode();
			if(!StringUtils.isEmpty(code) && code.contains("ResourceManagedByCloudProduct")) {
				return new BocloudResult<>(false, "当前安全组不支持该操作");
			}
			logger.error("instance [{}] join securitygroup [{}] error:", instanceId, group, e);
			return new BocloudResult(false,
					"instance [" + instanceId + "] join securitygroup [" + group + "] error:" + e.getMessage());
		}
	}

	/**
	 * 调用该接口时，您需要注意： 移出安全组之前，实例必须处于 已停止（Stopped）或者 运行中（Running）状态。
	 * 一台实例必须至少加入一个安全组，当该实例只加入了一个安全组时，则 LeaveSecurityGroup 请求会失败。
	 */
	@Override
	public BocloudResult leaveSGroup(String instanceId, String group) {
		LeaveSecurityGroupRequest leaveSecurityGroupRequest = new LeaveSecurityGroupRequest();
		leaveSecurityGroupRequest.setInstanceId(instanceId);
		leaveSecurityGroupRequest.setRegionId(this.getRegionId());
		leaveSecurityGroupRequest.setSecurityGroupId(group);
		try {
			LeaveSecurityGroupResponse response = this.getECSClient().leaveSecurityGroup(leaveSecurityGroupRequest);
			return new BocloudResult(true, response.getBody().getRequestId(), "操作成功！");
		} catch (Exception e) {
			logger.error("instance [{}] leave securitygroup [{}] error:", instanceId, group, e);
			return new BocloudResult(true, e.getMessage(), "操作失败！");
		}
	}

	@Override
	public BocloudResult snapshot(SnapshotModel snapshotModel) {
		CreateSnapshotRequest request = new CreateSnapshotRequest();
		request.setDescription(snapshotModel.getRemark());
		request.setDiskId(snapshotModel.getVolumeUuid());
		request.setSnapshotName(snapshotModel.getName());
		try {
			CreateSnapshotResponse response = this.getECSClient().createSnapshot(request);
			return new BocloudResult(true, response.getBody().getRequestId());
		} catch (Exception e) {
			logger.error("create snapshot [{}] error:", snapshotModel.getName(), e);
			return new BocloudResult(false, "");
		}
	}

	@Override
	public BocloudResult vncConsole(String vncType, String serverId) {
		DescribeInstanceVncUrlRequest request = new DescribeInstanceVncUrlRequest();
		request.setInstanceId(serverId);
		request.setRegionId(this.getRegionId());
		try {
			DescribeInstanceVncUrlResponse response = this.getECSClient().describeInstanceVncUrl(request);
			return new BocloudResult(true, response.getBody().getVncUrl(), "获取vnc控制台成功");
		} catch (Exception e) {
			logger.error("Get vnc console error:", e);
			return new BocloudResult(false, "获取vnc控制台失败");
		}
	}

	@Override
	public BocloudResult changePassword(String instanceId, String password) {
		ModifyInstanceAttributeRequest attributeRequest = new ModifyInstanceAttributeRequest();
		attributeRequest.setInstanceId(instanceId);
		attributeRequest.setPassword(password);
		try {
			this.getECSClient().modifyInstanceAttribute(attributeRequest);
			return new BocloudResult(true, this.detail(instanceId).getData(), "云服务器修改密码成功");
		} catch (Exception e) {
			logger.error("modify instance [{}] error:", instanceId, e);
			return new BocloudResult(false, "云服务器修改密码失败！", e.getMessage());
		}
	}

	@Override
	public BocloudResult attachNetwork(ServerModel server) {
		return null;
	}

	@Override
	public BocloudResult detachNetwork(String instanceId, String portUuid) {
		return null;
	}

	/**
	 * 实例必须处于无欠费状态。 实例状态必须为 运行中（Running）或者 已停止（Stopped）时才能调节公网带宽大小。
	 * 升级或者降低按量付费实例规格前，您可以通过 DescribeResourcesModification 查询当前实例支持变配的实例规格。 实例状态必须为
	 * 已停止（Stopped）时才能变更实例规格。 单次只能升级单项配置，即单次只能修改实例规格，或者只能调整公网带宽大小。
	 * 单台实例每成功操作一次，5分钟内不能继续操作。
	 */
	@Override
	public BocloudResult resize(String instanceId, String flavorId) {
		try {
			ModifyInstanceSpecRequest request = new ModifyInstanceSpecRequest();
			request.setInstanceId(instanceId);
			request.setInstanceType(flavorId);
			this.getECSClient().modifyInstanceSpec(request);
			ServerModel instance = this.getInstance(instanceId);
			return new BocloudResult(true, instance, "获取vnc控制台成功");
		} catch (Exception e) {
			logger.error("Get vnc console error:", e);
			return new BocloudResult(false, "获取vnc控制台失败");
		}
	}

	@Override
	public BocloudResult confirmResize(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult revertResize(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean MemHotEnable) {
		return null;
	}

	@Override
	public BocloudResult getTicket(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult createInResPool(ServerModel server) {
		return null;
	}

	@Override
	public BocloudResult mountToolsInstaller(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult unmountToolsInstaller(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult upgradeToolsTask(String instanceId, String options) {
		return null;
	}

	@Override
	public BocloudResult getNetCard(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult getVmIps(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult markAsTemplate(String instanceId) {
		return null;
	}

	/**
	 * 修改一台实例的部分信息，包括实例密码、名称、描述、云主机名和自定义数据等。如果是突发性能 t5 实例，可以切换这台实例的性能突发模式。 描述
	 * 调用该接口时，您需要注意： 实例状态为启动中（Starting）时，无法重置实例密码。 被 安全控制的 ECS 实例的 OperationLocks
	 * 不能标记为 "LockReason" : "security"。 重置密码后，您需要在控制台 重启实例 或者调用 API RebootInstance
	 * 使更改生效，在实例内部重启将不会生效。 实例状态为 已停止（Stopped）且满足 实例自定义数据 使用限制时，支持修改自定义数据。
	 */
	@Override
	public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
		try {
			ModifyInstanceAttributeRequest request = new ModifyInstanceAttributeRequest();
			request.setInstanceId(instanceId);
			request.setPassword(newPasswd);
			this.getECSClient().modifyInstanceAttribute(request);
			return new BocloudResult(true, "密码修改成功");
		} catch (Exception e) {
			logger.error("密码修改 error:", e);
			return new BocloudResult(false, "密码修改失败");
		}
	}

	@Override
	public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
		return null;
	}

	@Override
	public BocloudResult getMetric(String type) {
		return null;
	}

	@Override
	public BocloudResult vmrcConsole(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult getJnlpXml(Long vmId, String address) {
		return null;
	}

	private ServerModel getInstance(String instanceId) {
		logger.info("instanceId : " + instanceId);
		DescribeInstancesRequest request = new DescribeInstancesRequest();
		request.setRegionId(this.getRegionId());
		String[] instanceIds = new String[] { instanceId };
		request.setInstanceIds(JSON.toJSONString(instanceIds));
		ServerModel model = new ServerModel();
		try {
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				Thread.sleep(5000);
				DescribeInstancesResponse response = this.getECSClient().describeInstances(request);
				List<DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance> instances = response
						.getBody().getInstances().getInstance();
				if (ListTool.isEmpty(instances)) {
					flag = false;
					break;
				}
				if (("Running".equalsIgnoreCase(instances.get(0).getStatus())
						|| "Stopped".equalsIgnoreCase(instances.get(0).getStatus()))) {
					model = convertor.convertModel(instances.get(0));
					flag = false;
					break;
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 20 * 60 * 1000) {
					model = convertor.convertModel(instances.get(0));
					flag = false;
					break;
				}
			}
			if (null != model) {
				this.syncDisks(model);
			}
		} catch (Exception e) {
			logger.error("get instances error:", e);
		}
		return model;
	}

	private void syncDisks(ServerModel model) {
		try {
			List<VolumeModel> volumeModels = Lists.newArrayList();
			DescribeDisksRequest disksRequest = new DescribeDisksRequest();
			disksRequest.setRegionId(this.getRegionId());
			DescribeDisksResponse disksResponse = this.getECSClient().describeDisks(disksRequest);
			List<DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDisk> disks = disksResponse.getBody()
					.getDisks().getDisk();
			Integer systemDisk = 0;
			for (DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDisk disk : disks) {
				if (disk.getInstanceId().equals(model.getInstanceId())) {
					if (disk.getType().equalsIgnoreCase("SYSTEM")) {
						systemDisk += disk.getSize();
					}
					volumeModels.add(convert(disk));
				}
			}
			model.setDisk(systemDisk);
			model.setDiskDevices(volumeModels);
		} catch (Exception e) {
			logger.error("同步Disk异常：" + e.getMessage());
		}
	}

	/**
	 * 卷元数据转换
	 *
	 * @param disk
	 * @return
	 */
	private VolumeModel convert(DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDisk disk) {
		if (null == disk) {
			return null;
		}
		VolumeModel model = new VolumeModel();
		model.setVolumeUuid(disk.getDiskId());
		model.setZone(disk.getZoneId());
		if (!StringUtils.isEmpty(disk.getDiskName())) {
			model.setName(disk.getDiskName());
		} else {
			model.setName(disk.getDiskId());
		}
		model.setRemark(disk.getDescription());
		model.setDevice(disk.getDevice());
		model.setDeviceName(disk.getDevice());
		model.setType(disk.getType());
		model.setCategory(disk.getCategory());
		model.setSize(disk.getSize());
		model.setImageUuid(disk.getImageId());
		model.setSourceSnapshotId(disk.getSourceSnapshotId());
		model.setStatus(convertStatus(disk.getStatus()));
		model.setInstanceId(disk.getInstanceId());
		model.setDeleteWithInstance(disk.getDeleteWithInstance());
		model.setDeleteAutoSnapshot(disk.getDeleteAutoSnapshot());
		model.setEnableAutoSnapshot(disk.getEnableAutoSnapshot());
		model.setCreateTime(DateTools.transfer(AliyunProvider.DATE_FORMAT, disk.getCreationTime(), "Z"));
		model.setAttachedTime(DateTools.transfer(AliyunProvider.DATE_FORMAT, disk.getAttachedTime(), "Z"));
		model.setDetachedTime(DateTools.transfer(AliyunProvider.DATE_FORMAT, disk.getDetachedTime(), "Z"));
		List<DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDiskTagsTag> tags = disk.getTags().getTag();
		List<TagModel> tagModels = Lists.newArrayList();
		if (!ListTool.isEmpty(tags)) {
			for (DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDiskTagsTag tag : tags) {
				TagModel tagModel = new TagModel();
				tagModel.setKey(tag.getTagKey());
				tagModel.setValue(tag.getTagValue());
				tagModel.setResourceId(disk.getDiskId());
				tagModel.setResourceType(TagType.disk.name());
				tagModels.add(tagModel);
			}
			model.setTagModels(tagModels);
		}
		return model;
	}

	private String convertStatus(String status) {
		if (status.equalsIgnoreCase("In_use")) {
			return "IN_USE";
		}
		if (status.equalsIgnoreCase("Available")) {
			return "AVAILABLE";
		}
		return "EXCEPTION";
	}

	/**
	 * 构建云主机ECS的磁盘列表
	 *
	 * @param diskParam
	 * @return
	 */
	private List<CreateInstanceRequest.CreateInstanceRequestDataDisk> buildDataDisks(JSONArray diskParam) {
		List<CreateInstanceRequest.CreateInstanceRequestDataDisk> dataDisks = new ArrayList<>();
		if (null == diskParam || diskParam.isEmpty()) {
			return dataDisks;
		}
		for (int i = 0; i < diskParam.size(); i++) {
			CreateInstanceRequest.CreateInstanceRequestDataDisk dataDisk = new CreateInstanceRequest.CreateInstanceRequestDataDisk();
			dataDisk.setCategory(diskParam.getJSONObject(i).getString("dataDiskCategory"));
			dataDisk.setDeleteWithInstance(false);
			dataDisk.setSize(diskParam.getJSONObject(i).getInteger("dataDiskSize"));
			dataDisk.setPerformanceLevel(diskParam.getJSONObject(i).getString("performanceLevel"));
			dataDisks.add(dataDisk);
		}
		return dataDisks;
	}

	@Override
	public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult resetInstance(ServerModel serverModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult bindKeypair(ServerModel serverModel) {
		/*
		 * try { AttachKeyPairRequest request = new AttachKeyPairRequest();
		 * request.setInstanceIds(JSON.toJSONString(serverModel.getInstanceIds()));
		 * request.setKeyPairName(serverModel.getKeypairName()); AttachKeyPairResponse
		 * acsResponse = this.getECSClient().getAcsResponse(request); List<Result>
		 * results = acsResponse.getResults(); StringBuilder builder = new
		 * StringBuilder(); for (Result result : results) { String instanceId =
		 * result.getInstanceId(); String message = result.getMessage(); String code =
		 * result.getCode(); if (!"200".equals(code)) {
		 * builder.append("实例 【").append(instanceId).append("】绑定失败 ： ").append(message).
		 * append("\n"); } } return new BocloudResult(true, builder.toString()); } catch
		 * (Exception e) { logger.error("密码修改 error:", e); return new
		 * BocloudResult(false, "密码修改失败"); }
		 */
		return null;
	}

	@Override
	public BocloudResult unbindKeypair(ServerModel serverModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
		return null;
	}

	@Override
	public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * @param instanceIds
	 * @return
	 * @author hwt 批量启动云主机
	 */
	public BocloudResult startList(List<String> instanceIds) {
		try {
			StartInstancesRequest request = new StartInstancesRequest();
			request.setRegionId(this.getRegionId());
			request.setInstanceId(instanceIds);
			StartInstancesResponse response = this.getECSClient().startInstances(request);
			if (null == response) {
				return new BocloudResult(false, "批量启动云主机失败");
			}
			return new BocloudResult(true, response.getBody().getInstanceResponses(), "批量启动云主机成功");
		} catch (Exception e) {
			logger.error("批量启动云主机失败 ： " + e);
			return new BocloudResult(false, "批量启动云主机失败", e.getMessage());
		}
	}

	/**
	 * @param instanceIds
	 * @return
	 * @author hwt 批量停止云主机
	 */
	public BocloudResult stopList(List<String> instanceIds) {
		try {
			StopInstancesRequest request = new StopInstancesRequest();
			request.setRegionId(this.getRegionId());
			request.setInstanceId(instanceIds);
			StopInstancesResponse response = this.getECSClient().stopInstances(request);
			if (null == response) {
				return new BocloudResult(false, "批量停止云主机失败");
			}
			return new BocloudResult(true, response.getBody().getInstanceResponses(), "批量停止云主机成功");
		} catch (Exception e) {
			logger.error("批量停止云主机失败 ： " + e);
			return new BocloudResult(false, "批量停止云主机失败", e.getMessage());
		}
	}

	/**
	 * @param instanceIds
	 * @return
	 * @author hwt 批量重启云主机
	 */
	public BocloudResult rebootList(List<String> instanceIds) {
		try {
			RebootInstancesRequest request = new RebootInstancesRequest();
			request.setRegionId(this.getRegionId());
			request.setInstanceId(instanceIds);
			RebootInstancesResponse response = this.getECSClient().rebootInstances(request);
			if (null == response) {
				return new BocloudResult(false, "批量重启云主机失败");
			}
			return new BocloudResult(true, response.getBody().getInstanceResponses(), "批量重启云主机成功");
		} catch (Exception e) {
			logger.error("批量重启云主机失败 ： " + e);
			return new BocloudResult(false, "批量重启云主机失败", e.getMessage());
		}
	}

	/**
	 * @param instanceIds
	 * @return
	 * @author hwt 批量释放云主机
	 */
	public BocloudResult removeList(List<String> instanceIds) {
		try {
			DeleteInstancesRequest request = new DeleteInstancesRequest();
			request.setInstanceId(instanceIds);
			request.setRegionId(this.getRegionId());
			DeleteInstancesResponse response = this.getECSClient().deleteInstances(request);
			if (null == response) {
				return new BocloudResult(false, "批量释放云主机失败");
			}
			return new BocloudResult(true, response.getBody().getRequestId(), "批量释放云主机成功");
		} catch (Exception e) {
			logger.error("批量释放云主机失败 ： " + e);
			return new BocloudResult(false, "批量释放云主机失败", e.getMessage());
		}
	}

	/**
	 * @param regionId
	 * @param instanceIds
	 * @return
	 * @author: hwt 获取多台ECS实例的状态信息
	 */
	public BocloudResult describeInstanceStatus(String regionId, List<String> instanceIds) {
		try {
			DescribeInstancesFullStatusRequest request = new DescribeInstancesFullStatusRequest();
			request.setRegionId(regionId);
			request.setInstanceId(instanceIds);
			DescribeInstancesFullStatusResponse response = this.getECSClient().describeInstancesFullStatus(request);
			if (null == response) {
				return new BocloudResult(false, "获取多台ECS实例的状态信息失败");
			}
			return new BocloudResult(true, response, "获取多台ECS实例的状态信息成功");
		} catch (Exception e) {
			logger.error("获取多台ECS实例的状态信息失败 ： " + e);
			return new BocloudResult(false, "获取多台ECS实例的状态信息失败", e.getMessage());
		}
	}

	/**
	 * @param regionId
	 * @param instanceId
	 * @return
	 * @author: hwt 查询一台ECS实例的自定义数据
	 */
	public BocloudResult describeUserData(String regionId, String instanceId) {
		try {
			DescribeUserDataRequest request = new DescribeUserDataRequest();
			request.setRegionId(regionId);
			request.setInstanceId(instanceId);
			DescribeUserDataResponse response = this.getECSClient().describeUserData(request);
			if (null == response) {
				return new BocloudResult(false, "查询一台ECS实例的自定义数据失败");
			}
			return new BocloudResult(true, response.getBody().getUserData(), "查询一台ECS实例的自定义数据成功");
		} catch (Exception e) {
			logger.error("查询一台ECS实例的自定义数据失败 ： " + e);
			return new BocloudResult(false, "查询一台ECS实例的自定义数据失败", e.getMessage());
		}
	}

	/**
	 * @return
	 * @author: hwt 查询云服务器ECS提供的实例规格族列表
	 */
	public BocloudResult describeInstanceTypeFamilies() {
		try {
			DescribeInstanceTypeFamiliesRequest request = new DescribeInstanceTypeFamiliesRequest();
			request.setRegionId(this.getRegionId());
			DescribeInstanceTypeFamiliesResponse response = this.getECSClient().describeInstanceTypeFamilies(request);
			if (null == response) {
				return new BocloudResult(false, "查询云服务器ECS提供的实例规格族列表失败");
			}
			return new BocloudResult(true, response.getBody().getInstanceTypeFamilies(), "查询云服务器ECS提供的实例规格族列表成功");
		} catch (Exception e) {
			logger.error("查询云服务器ECS提供的实例规格族列表失败 ： " + e);
			return new BocloudResult(false, "查询云服务器ECS提供的实例规格族列表失败", e.getMessage());
		}
	}

	/**
	 * @return
	 * @author: hwt 查询云服务器ECS提供的实例规格资源
	 */
	public BocloudResult describeInstanceTypes() {
		try {
			DescribeInstanceTypesRequest request = new DescribeInstanceTypesRequest();
			DescribeInstanceTypesResponse response = this.getECSClient().describeInstanceTypes(request);
			if (null == response) {
				return new BocloudResult(false, "查询云服务器ECS提供的实例规格资源失败");
			}
			return new BocloudResult(true, response.getBody().getInstanceTypes(), "查询云服务器ECS提供的实例规格资源成功");
		} catch (Exception e) {
			logger.error("查询云服务器ECS提供的实例规格资源失败 ： " + e);
			return new BocloudResult(false, "查询云服务器ECS提供的实例规格资源失败", e.getMessage());
		}
	}

	/**
	 * @param regionId
	 * @param instanceId
	 * @param vncPassword
	 * @return
	 * @author: hwt 修改一台ECS实例的Web管理终端密码
	 */
	public BocloudResult modifyInstanceVncPasswd(String regionId, String instanceId, String vncPassword) {
		try {
			ModifyInstanceVncPasswdRequest request = new ModifyInstanceVncPasswdRequest();
			request.setRegionId(regionId);
			request.setInstanceId(instanceId);
			request.setVncPassword(vncPassword);
			ModifyInstanceVncPasswdResponse response = this.getECSClient().modifyInstanceVncPasswd(request);
			if (null == response) {
				return new BocloudResult(false, "修改一台ECS实例的Web管理终端密码失败");
			}
			return new BocloudResult(true, response.getBody().getRequestId(), "修改一台ECS实例的Web管理终端密码成功");
		} catch (Exception e) {
			logger.error("修改一台ECS实例的Web管理终端密码失败 ： " + e);
			return new BocloudResult(false, "修改一台ECS实例的Web管理终端密码失败", e.getMessage());
		}
	}

	/**
	 * @param regionId
	 * @param instanceId
	 * @param time
	 * @return
	 * @author: hwt 调用ModifyInstanceAutoReleaseTime为一台按量付费ECS实例设定或者取消自动释放时间。
	 *          设置自动释放时请谨慎操作，配置的时间到期后将自动释放ECS实例。
	 *          <p>
	 *          自动释放时间。按照ISO8601标准表示，并使用UTC +0时间，格式为yyyy-MM-ddTHH:mm:ssZ。
	 *          <p>
	 *          如果秒（ss）取值不是00，则自动取为当前分钟（mm）开始时。 最短释放时间为当前时间半小时之后。 最长释放时间不能超过当前时间三年。
	 *          <p>
	 *          如果不传入参数AutoReleaseTime，表示自动释放功能已取消，ECS实例不再自动释放。
	 *          <p>
	 *          两个条件：1、这个时间是从早上8:00开始算，比如12:00，则传4:00:00 2、必须是当前时间后的半小时之外
	 */
	public BocloudResult modifyInstanceAutoReleaseTime(String regionId, String instanceId, String time) {
		try {
			ModifyInstanceAutoReleaseTimeRequest request = new ModifyInstanceAutoReleaseTimeRequest();
			request.setRegionId(regionId);
			request.setInstanceId(instanceId);
			request.setAutoReleaseTime(time);
			ModifyInstanceAutoReleaseTimeResponse response = this.getECSClient().modifyInstanceAutoReleaseTime(request);
			if (null == response) {
				return new BocloudResult(false, "设定或者取消自动释放时间失败");
			}
			return new BocloudResult(true, "设定或者取消自动释放时间成功");
		} catch (Exception e) {
			logger.error("设定或者取消自动释放时间失败 ： " + e);
			return new BocloudResult(false, "设定或者取消自动释放时间失败", e.getMessage());
		}
	}

	/**
	 * @param model
	 * @return
	 * @author: hwt 调用ModifyInstanceSpec调整一台按量付费ECS实例的实例规格。
	 */
	public BocloudResult modifyInstanceType(ServerModel model) {
		try {
			ModifyInstanceSpecRequest request = new ModifyInstanceSpecRequest();
			request.setInstanceId(model.getInstanceId());
			request.setInstanceType(model.getInstanceType());
			// request.setInternetMaxBandwidthOut(model.getInternetMaxBandwidthOut());
			// request.setSystemDiskCategory(model.getSystemDiskCategory());
			ModifyInstanceSpecResponse response = this.getECSClient().modifyInstanceSpec(request);
			if (null == response) {
				return new BocloudResult(false, "调整一台按量付费ECS实例的实例规格失败");
			}
			return new BocloudResult(true, response.getBody().getRequestId(), "调整一台按量付费ECS实例的实例规格成功");
		} catch (Exception e) {
			logger.error("调整一台按量付费ECS实例的实例规格失败 ： " + e);
			return new BocloudResult(false, "调整一台按量付费ECS实例的实例规格失败", e.getMessage());
		}
	}

	/**
	 * @return com.bocloud.common.model.BocloudResult
	 * @Author Hu Wentao
	 * @Description 调用ModifyInstanceSpec调整一台按量付费ECS实例的宽带大小。
	 * @Date 11:29 2020/7/28
	 * @Param [model]
	 **/
	public BocloudResult modifyInternetMaxBandwidthOut(ServerModel model) {
		try {
			ModifyInstanceSpecRequest request = new ModifyInstanceSpecRequest();
			request.setResourceOwnerAccount(model.getRegionId());
			request.setInstanceId(model.getInstanceId());
			// request.setInstanceType(model.getInstanceType());
			request.setInternetMaxBandwidthOut(model.getInternetMaxBandwidthOut());
			// request.setSystemDiskCategory(model.getSystemDiskCategory());
			ModifyInstanceSpecResponse response = this.getECSClient().modifyInstanceSpec(request);
			if (null == response) {
				return new BocloudResult(false, "调整一台按量付费ECS实例的宽带大小失败");
			}
			return new BocloudResult(true, response.getBody().getRequestId(), "调整一台按量付费ECS实例的宽带大小成功");
		} catch (Exception e) {
			logger.error("调整一台按量付费ECS实例的宽带大小失败 ： " + e);
			return new BocloudResult(false, "调整一台按量付费ECS实例的宽带大小失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult modifyVmName(String instanceId, String newName) {
		ModifyInstanceAttributeRequest attributeRequest = new ModifyInstanceAttributeRequest();
		attributeRequest.setInstanceName(newName);
		attributeRequest.setInstanceId(instanceId);
		try {
			ModifyInstanceAttributeResponse response = this.getECSClient().modifyInstanceAttribute(attributeRequest);
			if (null == response) {
				return new BocloudResult(false, "云服务器名称修改失败");
			}
			return new BocloudResult(true, "云服务器名称修改成功");
		} catch (Exception e) {
			logger.error("云服务器名称修改失败:" + e);
			return new BocloudResult(false, "云服务器名称修改失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult migrate(H3cMigrateModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult create(H3cServerModel server) {
		// TODO Auto-generated method stub
		return null;
	}
}