package com.bocloud.cmp.provider.compute;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.aliyun.ecs20140526.models.CreateSnapshotRequest;
import com.aliyun.ecs20140526.models.CreateSnapshotResponse;
import com.aliyun.ecs20140526.models.DeleteSnapshotRequest;
import com.aliyun.ecs20140526.models.DeleteSnapshotResponse;
import com.aliyun.ecs20140526.models.DescribeSnapshotsRequest;
import com.aliyun.ecs20140526.models.DescribeSnapshotsResponse;
import com.aliyun.ecs20140526.models.DescribeSnapshotsResponseBody;
import com.aliyun.ecs20140526.models.DescribeSnapshotsUsageRequest;
import com.aliyun.ecs20140526.models.DescribeSnapshotsUsageResponse;
import com.aliyun.ecs20140526.models.ModifySnapshotAttributeRequest;
import com.aliyun.ecs20140526.models.ModifySnapshotAttributeResponse;
import com.aliyun.ecs20140526.models.ResetDiskRequest;
import com.aliyun.ecs20140526.models.ResetDiskResponse;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.SnapshotConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;

/**
 * @Package: com.bocloud.cmp.provider.storage
 * @ClassName: AliyunSnapshotProvider
 * @Author: 胡文涛
 * @CreateTime: 2020/7/30 14:17
 * @Description: 快照
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunSnapshotProvider extends AliyunProvider implements SnapshotProvider {

    private Logger logger = LoggerFactory.getLogger(AliyunSnapshotProvider.class);
    private static final int PAGE_SIZE = 50;
    private BeanConvertor<SnapshotModel,
        DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot> convertor =
            new SnapshotConvertor();

    public AliyunSnapshotProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeSnapshotsRequest request = new DescribeSnapshotsRequest();
            request.setPageNumber(1);
            request.setPageSize(PAGE_SIZE);
            request.setRegionId(this.getRegionId());
            List<SnapshotModel> snapshotModels = Lists.newArrayList();
            DescribeSnapshotsResponse response = this.getECSClient().describeSnapshots(request);
            if (null == response) {
                return new BocloudResult(false, "快照列表查询失败");
            } else if (null == response.getBody().getSnapshots()) {
                return new BocloudResult(false, "快照列表查询失败");
            }
            List<DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot> snapshots =
                response.getBody().getSnapshots().getSnapshot();
            if(!ListTool.isEmpty(snapshots)) {
                logger.info("快照 info : " + JSON.toJSONString(snapshots.get(0)));
            }
            Integer totalCount = response.getBody().getTotalCount();
            if (null == totalCount || totalCount == 0) {
                return new BocloudResult(true, snapshotModels, "快照列表为空");
            }
            for (DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot snapshot : snapshots) {
                SnapshotModel convertModel = convertor.convertModel(snapshot);
                convertModel.setRegionId(this.getRegionId());
                snapshotModels.add(convertModel);
            }
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1) {
                return new BocloudResult(true, snapshotModels, "快照列表查询成功");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getECSClient().describeSnapshots(request);
                snapshots = response.getBody().getSnapshots().getSnapshot();
                for (DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot snapshot : snapshots) {
                    SnapshotModel convertModel = convertor.convertModel(snapshot);
                    convertModel.setRegionId(this.getRegionId());
                    snapshotModels.add(convertModel);
                }
            }
            return new BocloudResult(true, snapshotModels, "快照列表查询成功");
        } catch (Exception e) {
            logger.error("快照列表查询失败：" + e);
            return new BocloudResult(false, "快照列表查询失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        try {
            DeleteSnapshotRequest request = new DeleteSnapshotRequest();
            request.setSnapshotId(model.getSnapshotUuid());
            DeleteSnapshotResponse response = this.getECSClient().deleteSnapshot(request);
            if (null == response) {
                return new BocloudResult(false, "快照删除失败");
            }
            return new BocloudResult(true, response.getBody().getRequestId(), "快照删除成功");
        } catch (Exception e) {
            logger.error("快照删除失败：" + e);
            return new BocloudResult(false, "快照删除失败：" + e.getMessage());
        }
    }

    public BocloudResult remove(String snapshotId) {
        try {
            DeleteSnapshotRequest request = new DeleteSnapshotRequest();
            request.setSnapshotId(snapshotId);
            DeleteSnapshotResponse response = this.getECSClient().deleteSnapshot(request);
            if (null == response) {
                return new BocloudResult(false, "快照删除失败");
            }
            return new BocloudResult(true, response.getBody().getRequestId(), "快照删除成功");
        } catch (Exception e) {
            logger.error("快照删除失败：" + e);
            return new BocloudResult(false, "快照删除失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult removeServerSnapshot(SnapshotModel snapshotModel) {
        return null;
    }

    @Override
    public BocloudResult create(SnapshotModel model) {
        try {
            CreateSnapshotRequest request = new CreateSnapshotRequest();
            request.setDiskId(model.getVolumeUuid());
            request.setDescription(model.getRemark());
            request.setSnapshotName(model.getName());
            CreateSnapshotResponse response = this.getECSClient().createSnapshot(request);
            if (null == response || null == response.getBody().getSnapshotId()) {
                return new BocloudResult(false, "快照创建失败");
            }
            String snapshotId = response.getBody().getSnapshotId();
            return this.snapshotProgress(snapshotId);
        } catch (Exception e) {
            if (!StringUtils.isEmpty(e.getMessage())
                && e.getMessage().contains("The specified disk has never been attached to any instance")) {
                return new BocloudResult(false, "快照创建失败：处于未挂载状态得磁盘，不允许创建快照");
            }
            logger.info("快照创建失败：" + e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    private BocloudResult snapshotProgress(String snapshotUuid) {
        BocloudResult result = new BocloudResult<>(false, "镜像创建失败");
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (true) {
                // 查询可用状态的快照
                Thread.sleep(5000);
                SnapshotModel snapshotModel = new SnapshotModel();
                snapshotModel.setSnapshotUuid(snapshotUuid);
                result = this.detail(snapshotModel);
                if (result.isFailed()) {
                    return result;
                }
                snapshotModel = (SnapshotModel)result.getData();
                if (!"progressing".equalsIgnoreCase(snapshotModel.getStatus())) {
                    return result;
                }
                Long endTime = System.currentTimeMillis();
                // 等待600s
                if (endTime - startTime > 20 * 60 * 1000) {
                    logger.error("获取快照状态状态异常：" + JSON.toJSONString(result));
                    return result;
                }
            }
        } catch (Exception e) {
            logger.error("镜像查询失败： " + e);
            return new BocloudResult(false, "镜像查询失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult createNoData(SnapshotModel model) {
        try {
            CreateSnapshotRequest request = new CreateSnapshotRequest();
            request.setDiskId(model.getVolumeUuid());
            request.setDescription(model.getRemark());
            request.setSnapshotName(model.getName());
            CreateSnapshotResponse response = this.getECSClient().createSnapshot(request);
            if (null == response || null == response.getBody().getSnapshotId()) {
                return new BocloudResult(false, "快照创建失败");
            }
            String snapshotId = response.getBody().getSnapshotId();
            model.setSnapshotUuid(snapshotId);
            return new BocloudResult(true, "快照创建成功");
        } catch (Exception e) {
            logger.info("快照创建失败：" + e);
            return new BocloudResult(false, "快照创建失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(SnapshotModel model) {
        try {
            DescribeSnapshotsRequest request = new DescribeSnapshotsRequest();
            request.setRegionId(this.getRegionId());
            JSONArray snapshotIds = new JSONArray(1);
            snapshotIds.add(model.getSnapshotUuid());
            request.setSnapshotIds(snapshotIds.toJSONString());
            request.setRegionId(this.getRegionId());
            DescribeSnapshotsResponse response = this.getECSClient().describeSnapshots(request);
            if (null == response || null == response.getBody().getSnapshots()
                || response.getBody().getSnapshots().getSnapshot().size() == 0) {
                return new BocloudResult(false, "快照详情查询失败");
            }
            DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot snapshot =
                response.getBody().getSnapshots().getSnapshot().get(0);
            return new BocloudResult(true, convertor.convertModel(snapshot), "快照详情查询成功");
        } catch (Exception e) {
            logger.error("快照详情查询失败" + e);
            return new BocloudResult(false, "快照详情查询失败" + e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(SnapshotModel model) {
        try {
            ModifySnapshotAttributeRequest request = new ModifySnapshotAttributeRequest();
            request.setSnapshotId(model.getSnapshotUuid());
            request.setSnapshotName(model.getName());
            request.setDescription(model.getRemark());
            ModifySnapshotAttributeResponse response = this.getECSClient().modifySnapshotAttribute(request);
            if (null == response) {
                return new BocloudResult(false, "快照修改失败");
            }
            BocloudResult detailResult = this.detail(model);
            if (detailResult.isFailed()) {
                return new BocloudResult(false, "快照修改失败");
            }
            return new BocloudResult(true, detailResult.getData(), "快照修改成功");
        } catch (Exception e) {
            logger.error("快照修改失败：" + e);
            return new BocloudResult(false, "快照修改失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        try {
            ResetDiskRequest resetDiskRequest = new ResetDiskRequest();
            resetDiskRequest.setSnapshotId(model.getSnapshotUuid());
            resetDiskRequest.setDiskId(model.getVolumeUuid());
            ResetDiskResponse acsResponse = this.getECSClient().resetDisk(resetDiskRequest);
            if (null == acsResponse) {
                return new BocloudResult(false, "快照回滚失败");
            }
            Thread.sleep(3000);
            return new BocloudResult(true, acsResponse.getBody().getRequestId(), "快照修改成功");
        } catch (Exception e) {
            logger.error("快照修改失败：" + e);
            return new BocloudResult(false, "快照修改失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult removeByVolumeId(String volumeId) {
        return null;
    }

    @Override
    public BocloudResult getSnapshot(String snapshotId) {
        BocloudResult result = new BocloudResult(false, "快照状态查询失败");
        DescribeSnapshotsRequest request = new DescribeSnapshotsRequest();
        request.setRegionId(this.getRegionId());
        JSONArray snapshotIds = new JSONArray(1);
        snapshotIds.add(snapshotId);
        request.setSnapshotIds(snapshotIds.toJSONString());
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                DescribeSnapshotsResponse response = this.getECSClient().describeSnapshots(request);
                List<DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot> snapshots =
                    response.getBody().getSnapshots().getSnapshot();
                if (null != snapshots && snapshots.size() > 0) {
                    DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot snapshot =
                        snapshots.get(0);
                    if ("100%".equals(snapshot.getProgress())) {
                        flag = false;
                        result = new BocloudResult(true, convertor.convertModel(snapshot), "快照状态查询成功");
                        break;
                    }
                }
                Thread.sleep(2000);
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 5 * 60 * 1000) {
                    flag = false;
                    break;
                }
            }
            return result;
        } catch (Exception e) {
            logger.error("快照状态查询失败： " + e);
            return new BocloudResult(false, "快照状态查询失败", e.getMessage());
        }
    }

    /**
     * @return com.bocloud.common.model.BocloudResult
     * @Author Hu Wentao
     * @Description //TODO 查询指定实例id的实例快照列表
     * @Date 15:08 2020/7/30
     * @Param [instanceId]
     **/
    @Override
    public BocloudResult list(Long instanceId) {
        try {
            DescribeSnapshotsRequest request = new DescribeSnapshotsRequest();
            request.setPageNumber(1);
            request.setPageSize(PAGE_SIZE);
            request.setRegionId(this.getRegionId());
            request.setInstanceId(String.valueOf(instanceId));
            List<SnapshotModel> snapshotModels = Lists.newArrayList();
            DescribeSnapshotsResponse response = this.getECSClient().describeSnapshots(request);
            if (null == response) {
                return new BocloudResult(false, "实例快照列表查询失败");
            } else if (null == response.getBody().getSnapshots().getSnapshot()) {
                return new BocloudResult(false, "实例快照列表查询失败");
            }
            List<DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot> snapshots =
                response.getBody().getSnapshots().getSnapshot();
            int totalCount = response.getBody().getTotalCount();
            if (totalCount == 0) {
                return new BocloudResult(true, snapshotModels, "实例快照列表为空");
            }
            for (DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot snapshot : snapshots) {
                SnapshotModel convertModel = convertor.convertModel(snapshot);
                convertModel.setRegionId(this.getRegionId());
                snapshotModels.add(convertModel);
            }
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1) {
                return new BocloudResult(true, snapshotModels, "实例快照列表查询成功");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getECSClient().describeSnapshots(request);
                snapshots = response.getBody().getSnapshots().getSnapshot();
                for (DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot snapshot : snapshots) {
                    SnapshotModel convertModel = convertor.convertModel(snapshot);
                    convertModel.setRegionId(this.getRegionId());
                    snapshotModels.add(convertModel);
                }
            }
            return new BocloudResult(true, snapshotModels, "实例快照列表查询成功");
        } catch (Exception e) {
            logger.error("实例快照列表查询失败： " + e);
            return new BocloudResult(false, "实例快照列表查询失败", e.getMessage());
        }
    }

    /**
     * @return com.bocloud.common.model.BocloudResult
     * @Author Hu Wentao
     * @Description //TODO 查询指定云盘id的云盘快照列表
     * @Date 15:09 2020/7/30
     * @Param [diskId]
     **/
    @Override
    public BocloudResult list(String diskId) {
        try {
            DescribeSnapshotsRequest request = new DescribeSnapshotsRequest();
            request.setPageNumber(1);
            request.setPageSize(PAGE_SIZE);
            request.setDiskId(diskId);
            request.setRegionId(this.getRegionId());
            List<SnapshotModel> snapshotModels = Lists.newArrayList();
            DescribeSnapshotsResponse response = this.getECSClient().describeSnapshots(request);
            if (null == response || null == response.getBody().getSnapshots()) {
                return new BocloudResult(false, "云盘快照列表查询失败");
            }
            List<DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot> snapshots =
                response.getBody().getSnapshots().getSnapshot();
            int totalCount = response.getBody().getTotalCount();
            if (totalCount == 0) {
                return new BocloudResult(true, snapshotModels, "云盘快照列表为空");
            }
            for (DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot snapshot : snapshots) {
                SnapshotModel convertModel = convertor.convertModel(snapshot);
                convertModel.setRegionId(this.getRegionId());
                snapshotModels.add(convertModel);
            }
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1) {
                return new BocloudResult(true, snapshotModels, "云盘快照列表查询成功");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getECSClient().describeSnapshots(request);
                snapshots = response.getBody().getSnapshots().getSnapshot();
                for (DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot snapshot : snapshots) {
                    SnapshotModel convertModel = convertor.convertModel(snapshot);
                    convertModel.setRegionId(this.getRegionId());
                    snapshotModels.add(convertModel);
                }
            }
            return new BocloudResult(true, snapshotModels, "云盘快照列表查询成功");
        } catch (Exception e) {
            logger.error("云盘快照列表查询失败： " + e);
            return new BocloudResult(false, "云盘快照列表查询失败", e.getMessage());
        }
    }

    /**
     * @return com.bocloud.common.model.BocloudResult
     * @Author Hu Wentao
     * @Description //TODO 查询您在一个地域下的快照数量以及快照容量。
     * @Date 15:03 2020/7/30
     * @Param [regionId]
     **/
    public BocloudResult describeSnapshotsUsage(String regionId) {
        try {
            DescribeSnapshotsUsageRequest request = new DescribeSnapshotsUsageRequest();
            request.setRegionId(regionId);
            DescribeSnapshotsUsageResponse response = this.getECSClient().describeSnapshotsUsage(request);
            if (null == response) {
                return new BocloudResult(false, "指定地域下的快照数量以及容量查询失败");
            }
            return new BocloudResult(true, response, "指定地域下的快照数量以及容量查询成功");
        } catch (Exception e) {
            logger.error("指定地域下的快照数量以及容量查询失败： " + e);
            return new BocloudResult(false, "指定地域下的快照数量以及容量查询失败", e.getMessage());
        }
    }
}
