package com.bocloud.cmp.provider.groups;

import com.aliyun.resourcemanager20200331.models.*;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ResourceGroupModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.ResourceGroupsConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.google.api.client.util.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.SimpleDateFormat;
import java.util.List;

/**
 * @Package: com.bocloud.cmp.provider.groups
 * @ClassName: ResourceGroupTest
 * @Author: 胡文涛
 * @CreateTime: 18:17 2020/7/28
 * @Description: 资源组
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunResourceGroupsProvider extends AliyunProvider implements ResourceGroupsProvider {

    private static Logger logger = LoggerFactory.getLogger(AliyunResourceGroupsProvider.class);
    private static final int PAGE_SIZE = 100;
    private ResourceGroupsConvertor convertor = new ResourceGroupsConvertor();

    public AliyunResourceGroupsProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            ListResourceGroupsRequest request = new ListResourceGroupsRequest();
            request.setPageNumber(1);
            request.setPageSize(PAGE_SIZE);
            List<ResourceGroupModel> resourceGroupModels = Lists.newArrayList();
            ListResourceGroupsResponse response = this.getResourceClient().listResourceGroups(request);
            if (null == response) {
                return new BocloudResult(false, "查询资源组列表失败");
            } else if (null == response.getBody().getResourceGroups()) {
                return new BocloudResult(false, "查询资源组列表失败");
            }
            List<ListResourceGroupsResponseBody.ListResourceGroupsResponseBodyResourceGroupsResourceGroup> resourceGroups = response.getBody().getResourceGroups().getResourceGroup();
            Integer totalCount = response.getBody().totalCount;
			if (null == totalCount || totalCount == 0) {
                return new BocloudResult(true, resourceGroupModels, "empty resource group");
            }
            for (ListResourceGroupsResponseBody.ListResourceGroupsResponseBodyResourceGroupsResourceGroup resourceGroup : resourceGroups) {
                ResourceGroupModel convertModel = convertor.convertModel(resourceGroup);
                convertModel.setRegionId(this.getRegionId());
                resourceGroupModels.add(convertModel);
            }
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1) {
                return new BocloudResult(true, resourceGroupModels, "success");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getResourceClient().listResourceGroups(request);
                resourceGroups = response.getBody().getResourceGroups().getResourceGroup();
                for (ListResourceGroupsResponseBody.ListResourceGroupsResponseBodyResourceGroupsResourceGroup resourceGroup : resourceGroups) {
                    ResourceGroupModel convertModel = convertor.convertModel(resourceGroup);
                    convertModel.setRegionId(this.getRegionId());
                    resourceGroupModels.add(convertModel);
                }
            }
            return new BocloudResult(true, resourceGroupModels, "查询资源组列表成功");
        } catch (Exception e) {
            logger.error("查询资源组列表失败：" + e);
            return new BocloudResult(false, "查询资源组列表失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult create(ResourceGroupModel model) {
        try {
            CreateResourceGroupRequest request = new CreateResourceGroupRequest()
                    .setName(model.getName())
                    .setDisplayName(model.getDisplayName());
            CreateResourceGroupResponse response = this.getResourceClient().createResourceGroup(request);
            if (null == response || null == response.getBody().getResourceGroup()) {
                return new BocloudResult(false, "创建资源组失败");
            }
            CreateResourceGroupResponseBody.CreateResourceGroupResponseBodyResourceGroup resourceGroup = response.getBody().getResourceGroup();
            return waitJob(resourceGroup.getId(), "创建资源组");
        } catch (Exception e) {
            logger.error("创建资源组失败:" + e);
            return new BocloudResult(false, "创建资源组失败:" + e.getMessage());
        }
    }

    private BocloudResult waitJob(String resourceGroupId, String operate) {
        try {
            Long startTime = System.currentTimeMillis();
            while (true) {
                Thread.sleep(500);
                BocloudResult detail = this.detail(resourceGroupId);
                if (detail.isFailed()) {
                    return detail;
                }
                GetResourceGroupResponseBody.GetResourceGroupResponseBodyResourceGroup resourceGroup = (GetResourceGroupResponseBody.GetResourceGroupResponseBodyResourceGroup) detail
                        .getData();
                if (("ok".equalsIgnoreCase(resourceGroup.getStatus())
                        || "error".equalsIgnoreCase(resourceGroup.getStatus()))) {
                    return new BocloudResult(true, convertModel(resourceGroup), operate + "成功！");
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 3 * 60 * 1000) {
                    return new BocloudResult(false, convertModel(resourceGroup), operate + "失败！");
                }
            }
        } catch (Exception e) {
            logger.error("get instances error:", e);
            return new BocloudResult(false, "资源创建失败:" + e.getMessage());
        }
    }

    public ResourceGroupModel convertModel(GetResourceGroupResponseBody.GetResourceGroupResponseBodyResourceGroup resourceGroup) {
        ResourceGroupModel resourceGroupModel = null;
        if (null != resourceGroup) {
            resourceGroupModel = new ResourceGroupModel();
            resourceGroupModel.setResourceGroupUuid(resourceGroup.getId());
            resourceGroupModel.setName(resourceGroup.getName());
            resourceGroupModel.setStatus(resourceGroup.getStatus());
            resourceGroupModel.setDisplayName(resourceGroup.getDisplayName());
            try {
                String createDate = resourceGroup.getCreateDate();
                if (!StringUtils.isEmpty(createDate)) {
                    SimpleDateFormat format1 = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
                    resourceGroupModel.setCreateTime(format1.parse(createDate));
                }
            } catch (Exception e) {
                logger.error("资源组转换失败:" + e);
            }
        }
        return resourceGroupModel;
    }

    @Override
    public BocloudResult remove(String resourceGroupId) {
        try {
            DeleteResourceGroupRequest request = new DeleteResourceGroupRequest()
                    .setResourceGroupId(resourceGroupId);
            DeleteResourceGroupResponse response = this.getResourceClient().deleteResourceGroup(request);
            if (null == response || null == response.getBody().getResourceGroup()) {
                return new BocloudResult(false, "删除资源组失败");
            }
            BocloudResult detail = this.detail(response.getBody().getResourceGroup().getId());
            if (detail.isFailed()) {
                return detail;
            }
            GetResourceGroupResponseBody.GetResourceGroupResponseBodyResourceGroup resourceGroup = (GetResourceGroupResponseBody.GetResourceGroupResponseBodyResourceGroup) detail
                    .getData();
            return new BocloudResult(true, convertModel(resourceGroup), "删除资源组成功");
        } catch (Exception e) {
			if (e.getMessage().contains("one or more cloud resources exist")) {
				return new BocloudResult<>(false, "当前资源组下存在资源，不允许删除");
			}
            logger.error("删除资源组失败:" + e);
            return new BocloudResult(false, "删除资源组失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String resourceGroupId) {
        try {
            GetResourceGroupRequest request = new GetResourceGroupRequest()
                    .setResourceGroupId(resourceGroupId);
            GetResourceGroupResponse response = this.getResourceClient().getResourceGroup(request);
            if (null == response || null == response.getBody().getResourceGroup()) {
                return new BocloudResult(false, "查询资源组详情失败");
            }
            GetResourceGroupResponseBody.GetResourceGroupResponseBodyResourceGroup resourceGroup = response.getBody().getResourceGroup();
            return new BocloudResult(true, resourceGroup, "查询资源组详情成功");
        } catch (Exception e) {
            logger.error("查询资源组详情失败 ： " + e);
            return new BocloudResult(false, "查询资源组详情失败", e.getMessage());
        }
    }

    /**
     * @return com.bocloud.common.model.BocloudResult
     * @Author Hu Wentao
     * @Description 更新资源组
     * @Date 15:33 2020/7/28
     * @Param [azureResourceGroupModel]
     **/
    public BocloudResult update(ResourceGroupModel model) {
        try {
            UpdateResourceGroupRequest request = new UpdateResourceGroupRequest()
                    .setResourceGroupId(model.getResourceGroupUuid())
                    .setNewDisplayName(model.getDisplayName());
            UpdateResourceGroupResponse response = this.getResourceClient().updateResourceGroup(request);
            if (null == response || null == response.getBody().getResourceGroup()) {
                return new BocloudResult(false, "更新资源组失败");
            }
            return waitJob(response.getBody().getResourceGroup().getId(), "更新资源组");
        } catch (Exception e) {
            logger.error("更新资源组失败 ： " + e);
            return new BocloudResult(false, "更新资源组失败", e.getMessage());
        }
    }

}
