package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSON;
import com.aliyun.vpc20160428.models.*;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ElasticIpModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.ElasticIpConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @author: yangzhenrong
 * @Date: 2020/12/14 15:05
 * @Description: 弹性公网IP
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunElasticIpProvider extends AliyunProvider implements ElasticIpProvider {

	public AliyunElasticIpProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(AliyunElasticIpProvider.class);

	private static final int PAGE_SIZE = 50;

	private BeanConvertor<ElasticIpModel, DescribeEipAddressesResponseBody.DescribeEipAddressesResponseBodyEipAddressesEipAddress> convertor = new ElasticIpConvertor();

	@Override
	public BocloudResult list() {
		try {
			DescribeEipAddressesRequest request = new DescribeEipAddressesRequest();
			request.setPageNumber(1);
			request.setPageSize(PAGE_SIZE);
			request.setRegionId(this.getRegionId());
			List<ElasticIpModel> models = Lists.newArrayList();
			DescribeEipAddressesResponse response = this.getVPCClient().describeEipAddresses(request);
			if (null == response) {
				return new BocloudResult(false, "弹性公网IP列表查询失败");
			} else if (null == response.getBody().getEipAddresses()) {
				return new BocloudResult(false, "弹性公网IP列表查询失败");
			}
			List<DescribeEipAddressesResponseBody.DescribeEipAddressesResponseBodyEipAddressesEipAddress> eipAddresses = response
					.getBody().getEipAddresses().getEipAddress();
			if(!ListTool.isEmpty(eipAddresses)) {
			    logger.info("浮动IP info : " + JSON.toJSONString(eipAddresses.get(0)));
			}
			Integer totalCount = response.getBody().getTotalCount();
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, models, "empty eipAddress");
			}
			for (DescribeEipAddressesResponseBody.DescribeEipAddressesResponseBodyEipAddressesEipAddress eipAddress : eipAddresses) {
				ElasticIpModel convertModel = convertor.convertModel(eipAddress);
				models.add(convertModel);
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, models, "success");
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setPageNumber(i);
				request.setPageSize(PAGE_SIZE);
				response = this.getVPCClient().describeEipAddresses(request);
				eipAddresses = response.getBody().getEipAddresses().getEipAddress();
				for (DescribeEipAddressesResponseBody.DescribeEipAddressesResponseBodyEipAddressesEipAddress eipAddress : eipAddresses) {
					ElasticIpModel convertModel = convertor.convertModel(eipAddress);
					models.add(convertModel);
				}
			}
			return new BocloudResult(true, models, "弹性公网IP列表查询成功");
		} catch (Exception e) {
			logger.error("弹性公网IP列表查询失败：" + e);
			return new BocloudResult(false, "弹性公网IP列表查询失败");
		}

	}

	@Override
	public BocloudResult allocate(ElasticIpModel model) {
		try {
			logger.info("创建浮动IP ：" + JSON.toJSONString(model));
			AllocateEipAddressRequest request = new AllocateEipAddressRequest();
			request.setName(model.getName());
			request.setRegionId(this.getRegionId());
			request.setBandwidth(model.getBandwidth());
			if (!StringUtils.isEmpty(model.getResourceGroupUuid())) {
				request.setResourceGroupId(model.getResourceGroupUuid());
			}
			AllocateEipAddressResponse response = this.getVPCClient().allocateEipAddress(request);
			if (null == response) {
				return new BocloudResult(false, "弹性公网IP分配失败");
			} else if (null == response.getBody().getAllocationId()) {
				return new BocloudResult(false, "弹性公网IP分配失败");
			}
			return new BocloudResult(true, this.getAvailable(response.getBody().getAllocationId()), "弹性公网IP分配成功");
		} catch (Exception e) {
			logger.error("弹性公网IP分配失败:" + e);
			return new BocloudResult(false, "弹性公网IP分配失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult allocatePro(ElasticIpModel model) {
		try {
			AllocateEipAddressProRequest request = new AllocateEipAddressProRequest();
			request.setRegionId(this.getRegionId());
			/***
			 * 要指定申请的EIP的IP地址。 IpAddress和InstanceId参数仅需传入一个，如果都不传，系统会随机申请EIP
			 */
			if (!StringUtils.isEmpty(model.getPublicIp())) {
				request.setIpAddress(model.getPublicIp());
			}
			if (!StringUtils.isEmpty(model.getInstanceId())) {
				request.setInstanceId(model.getInstanceId());
			}
			if (!StringUtils.isEmpty(model.getBandwidth())) {
				request.setBandwidth(model.getBandwidth());
			}
			if (!StringUtils.isEmpty(model.getResourceGroupUuid())) {
				request.setResourceGroupId(model.getResourceGroupUuid());
			}
			AllocateEipAddressProResponse response = this.getVPCClient().allocateEipAddressPro(request);
			if (null == response) {
				return new BocloudResult(false, "指定的弹性公网IP分配失败");
			} else if (null == response.getBody().getAllocationId()) {
				return new BocloudResult(false, "指定的弹性公网IP分配失败");
			}
			return new BocloudResult(true, this.getAvailable(response.getBody().getAllocationId()), "弹性公网IP分配成功");
		} catch (Exception e) {
			logger.error("指定的弹性公网IP分配失败:" + e);
			return new BocloudResult(false, "指定的弹性公网IP分配失败：" + e.getMessage());
		}
	}

	private ElasticIpModel getAvailable(String allocationId) {
		DescribeEipAddressesRequest request = new DescribeEipAddressesRequest();
		request.setAllocationId(allocationId);
		request.setRegionId(this.getRegionId());
		ElasticIpModel model = new ElasticIpModel();
		try {
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
				DescribeEipAddressesResponse response = this.getVPCClient().describeEipAddresses(request);
				List<DescribeEipAddressesResponseBody.DescribeEipAddressesResponseBodyEipAddressesEipAddress> eips = response
						.getBody().getEipAddresses().getEipAddress();
				if (ListTool.isEmpty(eips)) {
					flag = false;
					break;
				}
				if (("Available".equalsIgnoreCase(eips.get(0).getStatus()))) {
					model = convertor.convertModel(eips.get(0));
					flag = false;
					break;
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 5 * 60 * 1000) {
					model = convertor.convertModel(eips.get(0));
					flag = false;
					break;
				}
			}
		} catch (Exception e) {
			logger.error("get eip error:", e);
		}
		return model;
	}

	@Override
	public BocloudResult detail(String allocationId) {
		try {
			DescribeEipAddressesRequest request = new DescribeEipAddressesRequest();
			request.setAllocationId(allocationId);
			request.setRegionId(this.getRegionId());
			DescribeEipAddressesResponse response = this.getVPCClient().describeEipAddresses(request);
			List<DescribeEipAddressesResponseBody.DescribeEipAddressesResponseBodyEipAddressesEipAddress> eipAddresses = response
					.getBody().getEipAddresses().getEipAddress();
			if (ListTool.isEmpty(eipAddresses)) {
				return new BocloudResult(false, "弹性公网IP不存在");
			}
			return new BocloudResult(true,
					convertor.convertModel(response.getBody().getEipAddresses().getEipAddress().get(0)),
					"弹性公网IP详情查询成功");
		} catch (Exception e) {
			logger.error("弹性公网IP详情查询失败:" + e);
			return new BocloudResult(false, "弹性公网IP详情查询失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult release(String allocationId) {
		try {
			ReleaseEipAddressRequest request = new ReleaseEipAddressRequest();
			request.setAllocationId(allocationId);
			request.setRegionId(this.getRegionId());
			ReleaseEipAddressResponse response = this.getVPCClient().releaseEipAddress(request);
			return new BocloudResult(true, response.getBody().getRequestId(), "弹性公网IP释放成功！");
		} catch (Exception e) {
			logger.error("弹性公网IP释放失败:" + e);
			return new BocloudResult(false, "弹性公网IP释放失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult update(ElasticIpModel model) {
		try {
			ModifyEipAddressAttributeRequest request = new ModifyEipAddressAttributeRequest();
			request.setAllocationId(model.getElasticIpUuid());
			request.setBandwidth(model.getBandwidth());
			request.setName(model.getName());
			request.setRegionId(this.getRegionId());
			ModifyEipAddressAttributeResponse response = this.getVPCClient().modifyEipAddressAttribute(request);
			return new BocloudResult(true, response.getBody().getRequestId(), "弹性公网IP修改成功！");
		} catch (Exception e) {
			logger.error("弹性公网IP修改失败:" + e);
			return new BocloudResult(false, "弹性公网IP修改失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult associate(ElasticIpModel model) {
		try {
			AssociateEipAddressRequest request = new AssociateEipAddressRequest();
			request.setAllocationId(model.getElasticIpUuid());
			request.setInstanceId(model.getInstanceId());
			request.setRegionId(this.getRegionId());
			/***
			 * 要绑定EIP的实例的类型，取值： Nat：NAT网关。 SlbInstance：负载均衡SLB。 EcsInstance：云服务器ECS。
			 * NetworkInterface：辅助弹性网卡。 HaVip：高可用虚拟IP。
			 */
			request.setInstanceType(model.getInstanceType());
			AssociateEipAddressResponse response = this.getVPCClient().associateEipAddress(request);
			return new BocloudResult(true, response.getBody().getRequestId(), "绑定成功！");
		} catch (Exception e) {
			logger.error("绑定IP失败:" + e);
			return new BocloudResult(false, "绑定IP失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult disassociate(ElasticIpModel model) {
		try {
			UnassociateEipAddressRequest request = new UnassociateEipAddressRequest();
			request.setAllocationId(model.getElasticIpUuid());
			request.setInstanceId(model.getInstanceId());
			request.setInstanceType(model.getInstanceType());
			request.setRegionId(this.getRegionId());
			UnassociateEipAddressResponse response = this.getVPCClient().unassociateEipAddress(request);
			return new BocloudResult(true, response.getBody().getRequestId(), "解绑成功！");
		} catch (Exception e) {
			logger.error("解绑IP失败:" + e);
			return new BocloudResult(false, "解绑IP失败:" + e.getMessage());
		}
	}

}
