package com.bocloud.cmp.provider.network;

import com.aliyun.vpc20160428.models.*;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NatGatewayDnatRuleModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @author: yangzhenrong
 * @Date: 2020/12/14 15:05
 * @Description: NAT网关
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunNatGatewayDnatRuleProvider extends AliyunProvider implements NatGatewayDnatRuleProvider {

	public AliyunNatGatewayDnatRuleProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(AliyunNatGatewayDnatRuleProvider.class);

	private static final int PAGE_SIZE = 50;

	@Override
	public BocloudResult create(NatGatewayDnatRuleModel model) {
		try {
			CreateForwardEntryRequest request = new CreateForwardEntryRequest();
			// 必填参数
			request.setExternalIp(model.getFloatingIpAddress());
			request.setForwardTableId(model.getForwardTableId());
			request.setInternalIp(model.getPrivateIp());
			request.setIpProtocol(model.getProtocol());
			request.setRegionId(model.getRegionId());
			if (NatGatewayDnatRuleModel.Protocol.Any.name().equalsIgnoreCase(model.getProtocol())) {
				request.setExternalPort(NatGatewayDnatRuleModel.Protocol.Any.name());
				request.setInternalPort(NatGatewayDnatRuleModel.Protocol.Any.name());
			} else {
				request.setExternalPort(String.valueOf(model.getExternalServicePort()));
				request.setInternalPort(String.valueOf(model.getInternalServicePort()));
			}

			// 选填参数
			if (null != model.getName()) {
				request.setForwardEntryName(model.getName());
			}
			if (null != model.getClientToken()) {
				request.setClientToken(model.getClientToken());
			}
			if (null != model.getPortBreak()) {
				request.setPortBreak(model.getPortBreak());
			}

			CreateForwardEntryResponse response = this.getVPCClient().createForwardEntry(request);
			if (null == response) {
				return new BocloudResult(false, "DNAT创建失败");
			} else if (null == response.getBody().getForwardEntryId()) {
				return new BocloudResult(false, "DNAT创建失败");
			}
			return new BocloudResult(true, this.getAvailable(model.getForwardTableId(), response.getBody().getForwardEntryId()),
					"NAT网关创建成功");
		} catch (Exception e) {
			logger.error("DNAT创建失败:" + e);
			return new BocloudResult(false, "DNAT创建失败：" + e.getMessage());
		}
	}

	private NatGatewayDnatRuleModel getAvailable(String forwardTableId, String uuid) {
		DescribeForwardTableEntriesRequest request = new DescribeForwardTableEntriesRequest();
		// 必填
		request.setForwardTableId(forwardTableId);
		request.setRegionId(this.getRegionId());
		// 选填
		request.setForwardEntryId(uuid);

		NatGatewayDnatRuleModel model = new NatGatewayDnatRuleModel();
		try {
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
				DescribeForwardTableEntriesResponse response = this.getVPCClient().describeForwardTableEntries(request);
                List<DescribeForwardTableEntriesResponseBody.DescribeForwardTableEntriesResponseBodyForwardTableEntriesForwardTableEntry> forwardTableEntries = response
                        .getBody().getForwardTableEntries().getForwardTableEntry();
                if (ListTool.isEmpty(forwardTableEntries)) {
					flag = false;
					break;
				}
				if (("Available".equalsIgnoreCase(forwardTableEntries.get(0).getStatus()))) {
					model = convertModel(forwardTableEntries.get(0));
					flag = false;
					break;
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 5 * 60 * 1000) {
					model = convertModel(forwardTableEntries.get(0));
					flag = false;
					break;
				}
			}
		} catch (Exception e) {
			logger.error("get gateway error:", e);
		}
		return model;
	}

	@Override
	public BocloudResult detail(String forwardTableId, String uuid) {
		try {
			DescribeForwardTableEntriesRequest request = new DescribeForwardTableEntriesRequest();
			// 必填
			request.setForwardTableId(forwardTableId);
			request.setRegionId(this.getRegionId());
			// 选填
			request.setForwardEntryId(uuid);

			DescribeForwardTableEntriesResponse response = this.getVPCClient().describeForwardTableEntries(request);
			if (null == response) {
				return new BocloudResult(false, "DNAT详情查询失败");
			} else if (null == response.getBody().getForwardTableEntries()) {
				return new BocloudResult(false, "DNAT详情查询失败");
			}
			return new BocloudResult(true, convertModel(response.getBody().getForwardTableEntries().getForwardTableEntry().get(0)), "DNAT详情查询成功");
		} catch (Exception e) {
			logger.error("DNAT详情查询失败:" + e);
			return new BocloudResult(false, "DNAT详情查询失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(NatGatewayDnatRuleModel model) {
		try {
			ModifyForwardEntryRequest request = new ModifyForwardEntryRequest();
			// 必填
			request.setForwardEntryId(model.getUuid());
			request.setForwardTableId(model.getForwardTableId());
			request.setRegionId(model.getRegionId());

			// 选填
			if (StringUtils.isNotBlank(model.getFloatingIpAddress())) {
				request.setExternalIp(model.getFloatingIpAddress());
			}

			if (StringUtils.isNotBlank(model.getPrivateIp())) {
				request.setInternalIp(model.getPrivateIp());
			}
			// 选填参数
			if (null != model.getName()) {
				request.setForwardEntryName(model.getName());
			}
			if (StringUtils.isNotBlank(model.getProtocol())) {
				request.setIpProtocol(model.getProtocol());
				if (NatGatewayDnatRuleModel.Protocol.Any.name().equalsIgnoreCase(model.getProtocol())) {
					request.setInternalPort(NatGatewayDnatRuleModel.Protocol.Any.name());
					request.setExternalPort(NatGatewayDnatRuleModel.Protocol.Any.name());
				} else {
					if (null != model.getInternalServicePort()) {
						request.setInternalPort(String.valueOf(model.getInternalServicePort()));
					}
					if (null != model.getExternalServicePort()) {
						request.setExternalPort(String.valueOf(model.getExternalServicePort()));
					}
				}
			}

			ModifyForwardEntryResponse response = this.getVPCClient().modifyForwardEntry(request);
			return new BocloudResult(true, response.getBody().getRequestId(), "DNAT修改成功！");
		} catch (Exception e) {
			logger.error("DNAT修改失败：" + e);
			return new BocloudResult(false, "DNAT修改失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String natGatewayId, String dnanRuleId) {
		try {
			DeleteForwardEntryRequest request = new DeleteForwardEntryRequest();
			// 必填
			request.setForwardEntryId(dnanRuleId);
			request.setForwardTableId(natGatewayId);
			request.setRegionId(this.getRegionId());

			DeleteForwardEntryResponse response = this.getVPCClient().deleteForwardEntry(request);
			return new BocloudResult(true, response.getBody().getRequestId(), "DNAT删除成功！");
		} catch (Exception e) {
			logger.error("DNAT删除失败：" + e);
			return new BocloudResult(false, "DNAT删除失败:" + e.getMessage());
		}
	}

	private NatGatewayDnatRuleModel convertModel(DescribeForwardTableEntriesResponseBody.DescribeForwardTableEntriesResponseBodyForwardTableEntriesForwardTableEntry forwardEntry) {
		NatGatewayDnatRuleModel model = null;
		if (null != forwardEntry) {
			model = new NatGatewayDnatRuleModel();
			model.setFloatingIpAddress(forwardEntry.getExternalIp());
			model.setUuid(forwardEntry.getForwardEntryId());
			model.setName(forwardEntry.getForwardEntryName());
			model.setForwardTableId(forwardEntry.getForwardTableId());
			model.setPrivateIp(forwardEntry.getInternalIp());
			if (!NatGatewayDnatRuleModel.Protocol.Any.name().equalsIgnoreCase(forwardEntry.getIpProtocol())) {
				model.setInternalServicePort(
						null != forwardEntry.getInternalPort() ? Integer.valueOf(forwardEntry.getInternalPort())
								: null);
				model.setExternalServicePort(
						null != forwardEntry.getExternalPort() ? Integer.valueOf(forwardEntry.getExternalPort())
								: null);
			} else {
				model.setProtocol(NatGatewayDnatRuleModel.Protocol.Any.name());
			}
			if (NatGatewayDnatRuleModel.Protocol.TCP.name().equalsIgnoreCase(forwardEntry.getIpProtocol())) {
				model.setProtocol(NatGatewayDnatRuleModel.Protocol.TCP.name());
			} else if (NatGatewayDnatRuleModel.Protocol.UDP.name().equalsIgnoreCase(forwardEntry.getIpProtocol())) {
				model.setProtocol(NatGatewayDnatRuleModel.Protocol.UDP.name());
			}
			model.setStatus(forwardEntry.getStatus());
		}
		return model;
	}
}
