package com.bocloud.cmp.provider.network;

import com.aliyun.vpc20160428.models.*;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NatGatewaySnatRuleModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @author: yangzhenrong
 * @Date: 2020/12/14 15:05
 * @Description: NAT网关
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunNatGatewaySnatRuleProvider extends AliyunProvider implements NatGatewaySnatRuleProvider {

    public AliyunNatGatewaySnatRuleProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AliyunNatGatewaySnatRuleProvider.class);

    private static final int PAGE_SIZE = 50;


    @Override
    public BocloudResult create(NatGatewaySnatRuleModel model) {
        try {
            CreateSnatEntryRequest request = new CreateSnatEntryRequest();
            //必填
            request.setRegionId(this.getRegionId());
            request.setSnatIp(model.getFloatingIpAddress());
            request.setSnatTableId(model.getSnatTableId());

            //选填
            if (StringUtils.isNotBlank(model.getSubnetUuid())) {
                request.setSourceVSwitchId(model.getSubnetUuid());
            }
            if (StringUtils.isNotBlank(model.getCidr())) {
                request.setSourceCIDR(model.getCidr());
            }
            if (StringUtils.isNotBlank(model.getName())) {
                request.setSnatEntryName(model.getName());
            }
            if (StringUtils.isNotBlank(model.getClientToken())) {
                request.setClientToken(model.getClientToken());
            }

            CreateSnatEntryResponse response = this.getVPCClient().createSnatEntry(request);
            if (null == response) {
                return new BocloudResult(false, "SNAT创建失败");
            } else if (null == response.getBody().getSnatEntryId()) {
                return new BocloudResult(false, "SNAT创建失败");
            }
            return new BocloudResult(true, this.getAvailable(model.getSnatTableId(), response.getBody().getSnatEntryId()), "SNAT网关创建成功");
        } catch (Exception e) {
            logger.error("SNAT创建失败:" + e);
            return new BocloudResult(false, "SNAT创建失败：" + e.getMessage());
        }
    }

    private NatGatewaySnatRuleModel getAvailable(String snatTableId, String uuid) {
        DescribeSnatTableEntriesRequest request = new DescribeSnatTableEntriesRequest();
        //必填
        request.setSnatTableId(snatTableId);
        request.setRegionId(this.getRegionId());
        //选填
        request.setSnatEntryId(uuid);

        NatGatewaySnatRuleModel model = new NatGatewaySnatRuleModel();
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    logger.error("Get error message:", e);
                    Thread.currentThread().interrupt();
                }
                DescribeSnatTableEntriesResponse response = this.getVPCClient().describeSnatTableEntries(request);
                List<DescribeSnatTableEntriesResponseBody.DescribeSnatTableEntriesResponseBodySnatTableEntriesSnatTableEntry> snatTableEntries = response.getBody().getSnatTableEntries().getSnatTableEntry();
                if (ListTool.isEmpty(snatTableEntries)) {
                    flag = false;
                    break;
                }
                if (("Available".equalsIgnoreCase(snatTableEntries.get(0).getStatus()))) {
                    model = convertModel(snatTableEntries.get(0));
                    flag = false;
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 5 * 60 * 1000) {
                    model = convertModel(snatTableEntries.get(0));
                    flag = false;
                    break;
                }
            }
        } catch (Exception e) {
            logger.error("get gateway error:", e);
        }
        return model;
    }

    @Override
    public BocloudResult detail(String snatTableId, String uuid) {
        try {
            DescribeSnatTableEntriesRequest request = new DescribeSnatTableEntriesRequest();
            //必填
            request.setSnatTableId(snatTableId);
            request.setRegionId(this.getRegionId());
            request.setRegionId(this.getRegionId());
            //选填
            request.setSnatEntryId(uuid);

            DescribeSnatTableEntriesResponse response = this.getVPCClient().describeSnatTableEntries(request);
            if (null == response) {
                return new BocloudResult(false, "SNAT详情查询失败");
            } else if (null == response.getBody().getSnatTableEntries()) {
                return new BocloudResult(false, "SNAT详情查询失败");
            }
            return new BocloudResult(true, convertModel(response.getBody().getSnatTableEntries().getSnatTableEntry().get(0)), "SNAT详情查询成功");
        } catch (Exception e) {
            logger.error("SNAT详情查询失败:" + e);
            return new BocloudResult(false, "SNAT详情查询失败：" + e.getMessage());
        }
    }


    @Override
    public BocloudResult modify(NatGatewaySnatRuleModel model) {
        try {
            ModifySnatEntryRequest request = new ModifySnatEntryRequest();
            //必填
            request.setSnatEntryId(model.getUuid());
            request.setSnatTableId(model.getSnatTableId());
            request.setRegionId(model.getRegionId());

            //选填
            if (StringUtils.isNotBlank(model.getFloatingIpAddress())) {
                request.setSnatIp(model.getFloatingIpAddress());
            }
            if (StringUtils.isNotBlank(model.getName())) {
                request.setSnatEntryName(model.getName());
            }
            if (StringUtils.isNotBlank(model.getClientToken())) {
                request.setClientToken(model.getClientToken());
            }

            ModifySnatEntryResponse response = this.getVPCClient().modifySnatEntry(request);
            return new BocloudResult(true, response.getBody().getRequestId(), "SNAT修改成功！");
        } catch (Exception e) {
            logger.error("SNAT修改失败：" + e);
            return new BocloudResult(false, "SNAT修改失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String snatTableId, String snatRuleId) {
        try {
            DeleteSnatEntryRequest request = new DeleteSnatEntryRequest();
            //必填
            request.setSnatEntryId(snatRuleId);
            request.setSnatTableId(snatTableId);
            request.setRegionId(this.getRegionId());
            DeleteSnatEntryResponse response = this.getVPCClient().deleteSnatEntry(request);
            return new BocloudResult(true, response.getBody().getRequestId(), "SNAT删除成功！");
        } catch (Exception e) {
            logger.error("SNAT删除失败：" + e);
            return new BocloudResult(false, "SNAT删除失败:" + e.getMessage());
        }
    }


    private NatGatewaySnatRuleModel convertModel(DescribeSnatTableEntriesResponseBody.DescribeSnatTableEntriesResponseBodySnatTableEntriesSnatTableEntry snatTableEntry) {
        NatGatewaySnatRuleModel model = null;
        if (null != snatTableEntry) {
            model = new NatGatewaySnatRuleModel();

            model.setUuid(snatTableEntry.getSnatEntryId());
            model.setName(snatTableEntry.getSnatEntryName());
            model.setFloatingIpAddress(snatTableEntry.getSnatIp());
            model.setSnatTableId(snatTableEntry.getSnatTableId());
            model.setCidr(snatTableEntry.getSourceCIDR());
            model.setSubnetUuid(snatTableEntry.getSourceVSwitchId());
            model.setStatus(snatTableEntry.getStatus());
        }
        return model;
    }
}
