package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.ecs20140526.models.*;
import com.bocloud.cmp.entity.enums.TagType;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkInterfaceModel;
import com.bocloud.cmp.model.TagModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.NetworkInterfaceConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @Package: com.bocloud.cmp.provider.network
 * @ClassName: AliyunNetworkInterfaceProvider
 * @Author: 胡文涛
 * @CreateTime: 2020/8/3 10:05
 * @Description: 弹性网卡
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunNetworkInterfaceProvider extends AliyunProvider implements NetworkInterfaceProvider {

	private Logger logger = LoggerFactory.getLogger(AliyunNetworkInterfaceProvider.class);
	private static final int PAGE_SIZE = 50;
	private BeanConvertor<NetworkInterfaceModel, DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSet> convertor = new NetworkInterfaceConvertor();

	/**
	 * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
	 *
	 * @param butler 云平台认证信息
	 */
	public AliyunNetworkInterfaceProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			DescribeNetworkInterfacesRequest request = new DescribeNetworkInterfacesRequest();
			request.setPageNumber(1);
			request.setPageSize(PAGE_SIZE);
			request.setRegionId(this.getRegionId());
			List<NetworkInterfaceModel> networkInterfaceModels = Lists.newArrayList();
			DescribeNetworkInterfacesResponse response = this.getECSClient().describeNetworkInterfaces(request);
			if (null == response || null == response.getBody()
					|| null == response.getBody().getNetworkInterfaceSets()) {
				return new BocloudResult(false, "弹性网卡列表查询失败");
			}
			List<DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSet> networkInterfaceSets = response
					.getBody().getNetworkInterfaceSets().getNetworkInterfaceSet();
			if(!ListTool.isEmpty(networkInterfaceSets)) {
			    logger.info("网卡 info : " + JSON.toJSONString(networkInterfaceSets.get(0)));
			}
			Integer totalCount = response.getBody().getTotalCount();
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, networkInterfaceModels, "empty networkInterface");
			}
			for (DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSet networkInterfaceSet : networkInterfaceSets) {
				NetworkInterfaceModel convertModel = convertor.convertModel(networkInterfaceSet);
				convertModel.setRegionId(this.getRegionId());
				networkInterfaceModels.add(convertModel);
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, networkInterfaceModels, "success");
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setPageNumber(i);
				request.setPageSize(PAGE_SIZE);
				response = this.getECSClient().describeNetworkInterfaces(request);
				networkInterfaceSets = response.getBody().getNetworkInterfaceSets().getNetworkInterfaceSet();
				for (DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSet networkInterfaceSet : networkInterfaceSets) {
					NetworkInterfaceModel convertModel = convertor.convertModel(networkInterfaceSet);
					convertModel.setRegionId(this.getRegionId());
					networkInterfaceModels.add(convertModel);
				}
			}
			return new BocloudResult(true, networkInterfaceModels, "弹性网卡列表查询成功");
		} catch (Exception e) {
			logger.error("弹性网卡列表查询失败：" + e);
			return new BocloudResult(false, "弹性网卡列表查询失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String networkInterfaceId) {
		try {
			DescribeNetworkInterfacesRequest request = new DescribeNetworkInterfacesRequest();
			List<String> networkInterfaceIds = new ArrayList<>();
			networkInterfaceIds.add(networkInterfaceId);
			request.setNetworkInterfaceId(networkInterfaceIds);
			request.setRegionId(this.getRegionId());
			DescribeNetworkInterfacesResponse response = this.getECSClient().describeNetworkInterfaces(request);
			if (null == response) {
				return new BocloudResult(false, "弹性网卡详情查询失败");
			} else if (null == response.getBody().getNetworkInterfaceSets()) {
				return new BocloudResult(false, "弹性网卡详情查询失败");
			}
			DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSet networkInterface = response
					.getBody().getNetworkInterfaceSets().getNetworkInterfaceSet().get(0);
			NetworkInterfaceModel networkInterfaceModel = convertor.convertModel(networkInterface);
			return new BocloudResult(true, networkInterfaceModel, "弹性网卡详情查询成功");
		} catch (Exception e) {
			logger.error("弹性网卡详情查询失败:" + e);
			return new BocloudResult(false, "弹性网卡详情查询失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String networkInterfaceId) {
		try {
			DeleteNetworkInterfaceRequest request = new DeleteNetworkInterfaceRequest();
			request.setRegionId(this.getRegionId());
			request.setNetworkInterfaceId(networkInterfaceId);
			DeleteNetworkInterfaceResponse response = this.getECSClient().deleteNetworkInterface(request);
			if (null == response) {
				return new BocloudResult(false, "弹性网卡删除失败");
			}
			return new BocloudResult(true, response.getBody().getRequestId(), "弹性网卡删除成功");
		} catch (Exception e) {
			logger.error("弹性网卡删除失败:" + e);
			return new BocloudResult(false, "弹性网卡删除失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(NetworkInterfaceModel model) {
		try {
			ModifyNetworkInterfaceAttributeRequest request = new ModifyNetworkInterfaceAttributeRequest();
			request.setRegionId(this.getRegionId());
			request.setNetworkInterfaceId(model.getNetworkInterfaceUuid());
			request.setSecurityGroupId(model.getSecurityGroups());
			request.setNetworkInterfaceName(model.getName());
			request.setDescription(model.getRemark());
			ModifyNetworkInterfaceAttributeResponse response = this.getECSClient()
					.modifyNetworkInterfaceAttribute(request);
			if (null == response) {
				return new BocloudResult(false, "弹性网卡修改失败");
			}
			return new BocloudResult(true, response.getBody().getRequestId(), "弹性网卡修改成功");
		} catch (Exception e) {
			logger.error("弹性网卡修改失败:" + e);
			return new BocloudResult(false, "弹性网卡修改失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult create(NetworkInterfaceModel model) {
		try {
			CreateNetworkInterfaceRequest request = new CreateNetworkInterfaceRequest();
			request.setRegionId(model.getRegionId());
			request.setNetworkInterfaceName(model.getName());
			request.setVSwitchId(model.getSubnetUuid());
			List<String> securityGroups = model.getSecurityGroups();
			if (!ListTool.isEmpty(securityGroups)) {
				request.setSecurityGroupId(securityGroups.get(0));
			}
			request.setDescription(model.getRemark());
			if (!StringUtils.isEmpty(model.getResourceGroupUuid())) {
				request.setResourceGroupId(model.getResourceGroupUuid());
			}
			if (!StringUtils.isEmpty(model.getPrimaryPrivateIPAddress())) {
				request.setPrimaryIpAddress(model.getPrimaryPrivateIPAddress());// 主私网IP
			}
			if (!ListTool.isEmpty(model.getPrivateIpAddress())) {
				request.setPrivateIpAddress(model.getPrivateIpAddress());// 辅助私网IP
			}
			CreateNetworkInterfaceResponse response = this.getECSClient().createNetworkInterface(request);
			if (null == response) {
				return new BocloudResult(false, "弹性网卡创建失败");
			}
			return new BocloudResult(true, this.getAvailable(response.getBody().getNetworkInterfaceId()), "弹性网卡创建成功");
		} catch (Exception e) {
			logger.error("弹性网卡创建失败:" + e);
			return new BocloudResult(false, "弹性网卡创建失败：" + e.getMessage());
		}
	}

	private NetworkInterfaceModel getAvailable(String networkInterfaceId) {
		DescribeNetworkInterfacesRequest request = new DescribeNetworkInterfacesRequest();
		request.setRegionId(this.getRegionId());
		List<String> networkInterfaceIds = new ArrayList<>();
		networkInterfaceIds.add(networkInterfaceId);
		request.setNetworkInterfaceId(networkInterfaceIds);
		NetworkInterfaceModel model = new NetworkInterfaceModel();
		try {
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
				DescribeNetworkInterfacesResponse response = this.getECSClient().describeNetworkInterfaces(request);
				List<DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSet> nics = response
						.getBody().getNetworkInterfaceSets().getNetworkInterfaceSet();
				if (ListTool.isEmpty(nics)) {
					flag = false;
					break;
				}
				if ("Available".equalsIgnoreCase(nics.get(0).getStatus())) {
					model = convertor.convertModel(nics.get(0));
					model.setRegionId(this.getRegionId());
					flag = false;
					break;
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 5 * 60 * 1000) {
					model = convertor.convertModel(nics.get(0));
					flag = false;
					break;
				}
			}
		} catch (Exception e) {
			logger.error("get networkInterface error:", e);
		}
		return model;
	}

	@Override
	public BocloudResult attach(NetworkInterfaceModel model) {
		try {
			AttachNetworkInterfaceRequest request = new AttachNetworkInterfaceRequest();
			request.setRegionId(model.getRegionId());
			request.setInstanceId(model.getInstanceId());
			request.setNetworkInterfaceId(model.getNetworkInterfaceUuid());
			logger.info("instanceId : " + model.getInstanceId());
			logger.info("NetworkInterfaceUuid : " + model.getNetworkInterfaceUuid());
			AttachNetworkInterfaceResponse response = this.getECSClient().attachNetworkInterface(request);
			if (null == response) {
				return new BocloudResult(false, "附加网卡到ECS失败");
			}
			Thread.sleep(4000);
			return new BocloudResult(true, detail(model.getNetworkInterfaceUuid()).getData(), "附加网卡到ECS成功");
		} catch (Exception e) {
			logger.error("附加网卡到ECS失败：" + e);
			return new BocloudResult(false, "附加网卡到ECS失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detach(NetworkInterfaceModel model) {
		try {
			DetachNetworkInterfaceRequest request = new DetachNetworkInterfaceRequest();
			request.setRegionId(this.getRegionId());
			request.setInstanceId(model.getInstanceId());
			request.setNetworkInterfaceId(model.getNetworkInterfaceUuid());
			DetachNetworkInterfaceResponse response = this.getECSClient().detachNetworkInterface(request);
			if (null == response) {
				return new BocloudResult(false, "从ECS分离弹性网卡失败");
			}
			Thread.sleep(4000);
			return new BocloudResult(true, response.getBody().getRequestId(), "从ECS分离弹性网卡成功");
		} catch (Exception e) {
			logger.error("从ECS分离弹性网卡失败：" + e);
			return new BocloudResult(false, "从ECS分离弹性网卡失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult changeSubnet(NetworkInterfaceModel model) {
		return null;
	}

	@Override
	public BocloudResult vmChangeInterface(String netFaceId, String instanceId) {
		return null;
	}

	@Override
	public BocloudResult attachSecurityGroup(NetworkInterfaceModel model) {
		return null;
	}

	@Override
	public BocloudResult detachSecurityGroup(NetworkInterfaceModel model) {
		return null;
	}

	@Override
	public BocloudResult assignPrivateIpAddresses(NetworkInterfaceModel model) {
		try {
			logger.info("绑定IP参数：" + JSON.toJSONString(model));
			BocloudResult detail = this.detail(model.getNetworkInterfaceUuid());
			if (detail.isFailed()) {
				return detail;
			}
			logger.info("detail : " + JSON.toJSONString(detail));
			NetworkInterfaceModel networkInterfaceModel = (NetworkInterfaceModel) detail.getData();
			// 如果存在直接解绑
			List<String> oldPrivateIps = networkInterfaceModel.getPrivateIpAddress();
			if (!ListTool.isEmpty(oldPrivateIps)) {
				BocloudResult disassignPrivateIpAddresses = this.disassignPrivateIpAddresses(networkInterfaceModel);
				if (disassignPrivateIpAddresses.isFailed()) {
					return disassignPrivateIpAddresses;
				}
			}
			Thread.sleep(2000);
			// 绑定新得IP
			List<String> privateIpAddress = model.getPrivateIpAddress();
			if (!ListTool.isEmpty(privateIpAddress)) {
				AssignPrivateIpAddressesRequest request = new AssignPrivateIpAddressesRequest();
				logger.info("ipAddress : " + privateIpAddress);
				request.setNetworkInterfaceId(model.getNetworkInterfaceUuid());
				// 测试   直接传递list 接口报错
				String[] strings = new String[privateIpAddress.size()];
				privateIpAddress.toArray(strings);
				request.setPrivateIpAddress(Arrays.asList(strings));
				request.setRegionId(this.getRegionId());
				AssignPrivateIpAddressesResponse response = this.getECSClient().assignPrivateIpAddresses(request);
				if (null == response) {
					return new BocloudResult(false, "分配辅助私网IP失败");
				}
			}
			detail = this.detail(model.getNetworkInterfaceUuid());
			return new BocloudResult(true, detail.getData(), "分配辅助私网IP成功");
		} catch (Exception e) {
			logger.error("分配辅助私网IP失败：" + e);
			String message = e.getMessage();
			if (!StringUtils.isEmpty(message) && message.contains("has been managed")) {
				return new BocloudResult<>(false, "当前网卡不支持该操作！");
			}
			return new BocloudResult(false, "分配辅助私网IP失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult disassignPrivateIpAddresses(NetworkInterfaceModel model) {
		try {
			UnassignPrivateIpAddressesRequest request = new UnassignPrivateIpAddressesRequest();
			request.setNetworkInterfaceId(model.getNetworkInterfaceUuid());
			request.setPrivateIpAddress(model.getPrivateIpAddress());
			request.setRegionId(this.getRegionId());
			UnassignPrivateIpAddressesResponse response = this.getECSClient().unassignPrivateIpAddresses(request);
			if (null == response) {
				return new BocloudResult(false, "取消分配辅助私网IP失败");
			}
			return new BocloudResult(true, response.getBody().getRequestId(), "取消分配辅助私网IP成功");
		} catch (Exception e) {
			logger.error("取消分配辅助私网IP失败:" + e);
			return new BocloudResult(false, "取消分配辅助私网IP失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult assignIpv6Addresses(String networkInterfaceUuid, List<String> ipv6Addresss,
			Integer ipv6AddressCount) {
		try {
			AssignIpv6AddressesRequest request = new AssignIpv6AddressesRequest();
			request.setNetworkInterfaceId(networkInterfaceUuid);
			request.setRegionId(this.getRegionId());
			if (ListTool.isEmpty(ipv6Addresss)) {
				request.setIpv6AddressCount(ipv6AddressCount == null ? 1 : ipv6AddressCount);
			} else {
				request.setIpv6Address(ipv6Addresss);
			}
			AssignIpv6AddressesResponse response = this.getECSClient().assignIpv6Addresses(request);
			if (null == response) {
				return new BocloudResult(false, "分配IPv6地址失败");
			}
			return new BocloudResult(true, "分配IPv6地址成功");
		} catch (Exception e) {
			logger.error("分配IPv6地址失败：" + e);
			return new BocloudResult(false, "分配IPv6地址失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult disassignIpv6Addresses(String networkInterfaceUuid, List<String> ipv6Addresss) {
		try {
			UnassignIpv6AddressesRequest request = new UnassignIpv6AddressesRequest();
			request.setNetworkInterfaceId(networkInterfaceUuid);
			request.setIpv6Address(ipv6Addresss);
			request.setRegionId(this.getRegionId());
			UnassignIpv6AddressesResponse response = this.getECSClient().unassignIpv6Addresses(request);
			if (null == response) {
				return new BocloudResult(false, "回收IPv6地址失败");
			}
			return new BocloudResult(true, response.getBody().getRequestId(), "回收IPv6地址成功");
		} catch (Exception e) {
			logger.error("回收IPv6地址失败:" + e);
			return new BocloudResult(false, "回收IPv6地址失败：" + e.getMessage());
		}
	}

	private NetworkInterfaceModel detailResponseToModel(CreateNetworkInterfaceResponse response) {
		NetworkInterfaceModel model = new NetworkInterfaceModel();
		CreateNetworkInterfaceResponseBody networkInterface = response.getBody();
		if (networkInterface != null) {
			if (StringUtils.isEmpty(networkInterface.getNetworkInterfaceName())) {
				model.setName(networkInterface.getNetworkInterfaceId());
			} else {
				model.setName(networkInterface.getNetworkInterfaceName());
			}
			model.setNetworkInterfaceUuid(networkInterface.getNetworkInterfaceId());
			model.setStatus(networkInterface.getStatus());
			model.setInterfaceType(networkInterface.getType());
			model.setNetworkUuid(networkInterface.getVpcId());
			model.setSubnetUuid(networkInterface.getVSwitchId());
			model.setZone(networkInterface.getZoneId());
			model.setPrimaryPrivateIPAddress(networkInterface.getPrivateIpAddress());// 主私网IP
			model.setMacAddress(networkInterface.getMacAddress());
			List<CreateNetworkInterfaceResponseBody.CreateNetworkInterfaceResponseBodyPrivateIpSetsPrivateIpSet> privateIpSets = networkInterface
					.getPrivateIpSets().getPrivateIpSet();
			List<String> privateIpAddresssList = new ArrayList<>();
			for (CreateNetworkInterfaceResponseBody.CreateNetworkInterfaceResponseBodyPrivateIpSetsPrivateIpSet privateIpSet : privateIpSets) {
				if (privateIpSet.getPrimary()) {
					continue;
				}
				privateIpAddresssList.add(privateIpSet.getPrivateIpAddress());
			}
			model.setPrivateIpAddress(privateIpAddresssList);// 辅助私网IP
			model.setSecurityGroups(networkInterface.getSecurityGroupIds().getSecurityGroupId());
			model.setRemark(networkInterface.getDescription());
			model.setResourceGroupUuid(networkInterface.getResourceGroupId());
			model.setSecurityGroupUuid(JSONObject.toJSONString(networkInterface.getSecurityGroupIds()));
			List<CreateNetworkInterfaceResponseBody.CreateNetworkInterfaceResponseBodyTagsTag> tags = networkInterface
					.getTags().getTag();
			List<TagModel> tagModels = Lists.newArrayList();
			if (!ListTool.isEmpty(tags)) {
				for (CreateNetworkInterfaceResponseBody.CreateNetworkInterfaceResponseBodyTagsTag tag : tags) {
					TagModel tagModel = new TagModel();
					tagModel.setKey(tag.getTagKey());
					tagModel.setValue(tag.getTagValue());
					tagModel.setResourceId(networkInterface.getNetworkInterfaceId());
					tagModel.setResourceType(TagType.eni.name());
					tagModels.add(tagModel);
				}
				model.setTagModels(tagModels);
			}
		}
		return model;
	}
}
