package com.bocloud.cmp.provider.network;

import com.aliyun.vpc20160428.models.*;
import com.aliyun.vpc20160428.models.DescribeRouteEntryListResponseBody.DescribeRouteEntryListResponseBodyRouteEntrysRouteEntry;
import com.aliyun.vpc20160428.models.DescribeRouteEntryListResponseBody.DescribeRouteEntryListResponseBodyRouteEntrysRouteEntryNextHopsNextHop;
import com.aliyun.vpc20160428.models.DescribeRouteTableListResponseBody.DescribeRouteTableListResponseBodyRouterTableList;
import com.aliyun.vpc20160428.models.DescribeRouteTableListResponseBody.DescribeRouteTableListResponseBodyRouterTableListRouterTableListType;
import com.aliyun.vpc20160428.models.DescribeRouteTableListResponseBody.DescribeRouteTableListResponseBodyRouterTableListRouterTableListTypeVSwitchIds;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.RouteModel;
import com.bocloud.cmp.model.RouteTableModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.util.ArrayList;
import java.util.List;

/**
 * 路由表provider
 * 
 * @date 2021/09/28
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunRouteTableProvider extends AliyunProvider implements RouteTableProvider {

	private static Logger logger = LoggerFactory.getLogger(AliyunRouteTableProvider.class);

	private static final int PAGE_SIZE = 50;

	public AliyunRouteTableProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			DescribeRouteTableListRequest request = new DescribeRouteTableListRequest();
			request.setPageNumber(1);
			request.setPageSize(PAGE_SIZE);
			request.setRegionId(this.getRegionId());
			List<RouteTableModel> routeTableModels = Lists.newArrayList();
			DescribeRouteTableListResponse describeRouteTableListResponse = this.getVPCClient()
					.describeRouteTableList(request);
			if (null == describeRouteTableListResponse || null == describeRouteTableListResponse.getBody()) {
				return new BocloudResult(false, "路由表列表查询失败");
			} else if (null == describeRouteTableListResponse.getBody().getRouterTableList().getRouterTableListType()) {
				return new BocloudResult(false, "路由表列表查询失败");
			}
			Integer totalCount = describeRouteTableListResponse.getBody().getTotalCount();
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, routeTableModels, "路由表列表为空");
			}
			List<DescribeRouteTableListResponseBodyRouterTableListRouterTableListType> routerTableListType = describeRouteTableListResponse
					.getBody().getRouterTableList().getRouterTableListType();
			for (DescribeRouteTableListResponseBodyRouterTableListRouterTableListType routeTable : routerTableListType) {
				RouteTableModel convertModel = this.convertRouteTable(routeTable);
				routeTableModels.add(convertModel);
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, routeTableModels, "路由表列表查询成功");
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setPageNumber(i);
				request.setPageSize(PAGE_SIZE);
				describeRouteTableListResponse = this.getVPCClient().describeRouteTableList(request);
				if (null == describeRouteTableListResponse) {
					continue;
				}
				routerTableListType = describeRouteTableListResponse.getBody().getRouterTableList()
						.getRouterTableListType();
				if (!ListTool.isEmpty(routerTableListType)) {
					for (DescribeRouteTableListResponseBodyRouterTableListRouterTableListType routeTable : routerTableListType) {
						RouteTableModel convertModel = this.convertRouteTable(routeTable);
						routeTableModels.add(convertModel);
					}
				}
			}
			return new BocloudResult(true, routeTableModels, "路由表列表查询成功");
		} catch (Exception e) {
			logger.error("路由表列表查询失败：" + e);
			return new BocloudResult(false, "路由表列表查询失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult create(RouteTableModel model) {
		try {
			CreateRouteTableRequest request = new CreateRouteTableRequest();
			request.setVpcId(model.getVpcUuid());
			request.setRegionId(this.getRegionId());
			request.setRouteTableName(model.getName());
			request.setDescription(model.getRemark());
			CreateRouteTableResponse createRouteTable = this.getVPCClient().createRouteTable(request);
			if (null == createRouteTable || null == createRouteTable.getBody()) {
				return new BocloudResult(false, "路由表创建失败");
			}
			String routeTableId = createRouteTable.getBody().getRouteTableId();
			return new BocloudResult(true, this.detail(routeTableId).getData(), "路由表创建成功");
		} catch (Exception e) {
			logger.error("路由表创建失败：" + e);
			if(e.getMessage().contains("This vpc feature is not supported in this region")){
				return new BocloudResult(false, e.getMessage(), "您所选择的区域阿里公有云不支持创建自定义路由表");
			}
			return new BocloudResult(false, e.getMessage(), "路由表创建失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(RouteTableModel model) {
		try {
			ModifyRouteTableAttributesRequest request = new ModifyRouteTableAttributesRequest();
			request.setRouteTableId(model.getUuid());
			request.setRegionId(this.getRegionId());
			request.setRouteTableName(model.getName());
			request.setDescription(model.getRemark());
			ModifyRouteTableAttributesResponse modifyRouteTableAttributes = this.getVPCClient()
					.modifyRouteTableAttributes(request);
			return new BocloudResult(true, "路由表修改成功");
		} catch (Exception e) {
			logger.error("路由表修改失败：" + e);
			return new BocloudResult(false, "路由表修改失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult delete(String routeTableId) {
		try {
			DeleteRouteTableRequest request = new DeleteRouteTableRequest();
			request.setRouteTableId(routeTableId);
			request.setRegionId(this.getRegionId());
			DeleteRouteTableResponse deleteRouteTable = this.getVPCClient().deleteRouteTable(request);
			return new BocloudResult(true, "路由表删除成功");
		} catch (Exception e) {
			logger.error("路由表删除失败：" + e);
			return new BocloudResult(false, "路由表删除失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String routeTableId) {
		try {
			DescribeRouteTableListRequest request = new DescribeRouteTableListRequest();
			request.setRouteTableId(routeTableId);
			request.setRegionId(this.getRegionId());
			DescribeRouteTableListResponse describeRouteTableList = this.getVPCClient().describeRouteTableList(request);
			if (null == describeRouteTableList || null == describeRouteTableList.getBody()) {
				return new BocloudResult<>(false, "资源不存在");
			}
			DescribeRouteTableListResponseBodyRouterTableList routerTableList = describeRouteTableList.getBody()
					.getRouterTableList();
			List<DescribeRouteTableListResponseBodyRouterTableListRouterTableListType> routerTableListType = routerTableList
					.getRouterTableListType();
			if (ListTool.isEmpty(routerTableListType)) {
				return new BocloudResult<>(false, "资源不存在");
			}
			RouteTableModel convert = this.convertRouteTable(routerTableListType.get(0));
			return new BocloudResult(true, convert, "路由表详情查询成功");
		} catch (Exception e) {
			logger.error("路由表详情查询失败：" + e);
			return new BocloudResult(false, e.getMessage(), "路由表详情查询失败");
		}
	}

	private List<RouteTableModel> convertRouteTableList(
			List<? extends DescribeRouteTableListResponseBodyRouterTableListRouterTableListType> objects) {
		List<RouteTableModel> routeTableModels = new ArrayList<>();
		for (DescribeRouteTableListResponseBodyRouterTableListRouterTableListType object : objects) {
			routeTableModels.add(convertRouteTable(object));
		}
		return routeTableModels;
	}

	private RouteTableModel convertRouteTable(
			DescribeRouteTableListResponseBodyRouterTableListRouterTableListType routeTable) {
		RouteTableModel model = null;
		if (null != routeTable) {
			model = new RouteTableModel();
			model.setName(routeTable.getRouteTableName());
			model.setVpcUuid(routeTable.getVpcId());
			model.setUuid(routeTable.getRouteTableId());
			model.setType(routeTable.getRouteTableType());
			model.setRemark(routeTable.getDescription());
			model.setResourceGroupUuid(routeTable.getResourceGroupId());
			model.setStatus(routeTable.getStatus());
			model.setRouterId(routeTable.getRouterId());// 路由器
			DescribeRouteTableListResponseBodyRouterTableListRouterTableListTypeVSwitchIds vSwitchIds = routeTable
					.getVSwitchIds();
			if (!ListTool.isEmpty(vSwitchIds.getVSwitchId())) {
				model.setSubnetlds(vSwitchIds.getVSwitchId());
			}
			BocloudResult routeResult = listRoute(routeTable.getRouteTableId());
			if (routeResult.isSuccess() && null != routeResult.getData()) {
				model.setRouteModels((List<RouteModel>) routeResult.getData());
			}
		}

		return model;
	}

	private BocloudResult listRoute(String routeTableId) {
		try {
			DescribeRouteEntryListRequest request = new DescribeRouteEntryListRequest();
			request.setRouteTableId(routeTableId);
			request.setRegionId(this.getRegionId());
			DescribeRouteEntryListResponse describeRouteEntryList = this.getVPCClient().describeRouteEntryList(request);
			if (null == describeRouteEntryList || null == describeRouteEntryList.getBody()) {
				return new BocloudResult(false, "路由条目列表查询失败");
			} else if (null == describeRouteEntryList.getBody().getRouteEntrys()) {
				return new BocloudResult(false, "路由条目列表查询失败");
			}
			List<DescribeRouteEntryListResponseBodyRouteEntrysRouteEntry> routeEntry = describeRouteEntryList.getBody()
					.getRouteEntrys().getRouteEntry();
			List<RouteModel> routeModels = this.convertRoute(routeEntry);
			return new BocloudResult(true, routeModels, "路由条目列表查询成功");
		} catch (Exception e) {
			logger.error("路由条目列表查询失败： " + e);
			return new BocloudResult(false, "路由条目列表查询失败：" + e.getMessage());
		}
	}

	private List<RouteModel> convertRoute(
			List<? extends DescribeRouteEntryListResponseBodyRouteEntrysRouteEntry> objects) {
		List<RouteModel> routeModels = new ArrayList<>();
		for (DescribeRouteEntryListResponseBodyRouteEntrysRouteEntry routeEntry : objects) {
			if (null == routeEntry) {
				continue;
			}
			RouteModel model = new RouteModel();
			if (StringUtils.isEmpty(routeEntry.getRouteEntryName())) {
				model.setName(routeEntry.getRouteEntryId());
			} else {
				model.setName(routeEntry.getRouteEntryName());
			}
			model.setRouteTableUuid(routeEntry.getRouteTableId());
			model.setDestination(routeEntry.getDestinationCidrBlock());
			model.setType(routeEntry.getType());
			model.setUuid(routeEntry.getRouteEntryId());
			model.setState(routeEntry.getStatus());
			model.setDestinationType(routeEntry.getIpVersion());
			model.setOrigin(routeEntry.getDescription());
			if (null != routeEntry.getNextHops()) {
				List<DescribeRouteEntryListResponseBodyRouteEntrysRouteEntryNextHopsNextHop> nextHop = routeEntry
						.getNextHops().getNextHop();
				if (!ListTool.isEmpty(nextHop)) {
					int i = 0;
					for (DescribeRouteEntryListResponseBodyRouteEntrysRouteEntryNextHopsNextHop routeEntryNextHop : nextHop) {
						if (i > 0) {
							break;
						}
						model.setTargetType(routeEntryNextHop.getNextHopType());
						model.setTargetUuid(routeEntryNextHop.getNextHopId());
						i++;
					}
				}
			}
			routeModels.add(model);
		}
		return routeModels;
	}

	@Override
	public BocloudResult associate(String routeTableId, String vSwitchId) {
		try {
			AssociateRouteTableRequest request = new AssociateRouteTableRequest();
			request.setRouteTableId(routeTableId);
			request.setVSwitchId(vSwitchId);
			request.setRegionId(this.getRegionId());
			AssociateRouteTableResponse associateRouteTable = this.getVPCClient().associateRouteTable(request);
			if (null == associateRouteTable || null == associateRouteTable.getBody()) {
				return new BocloudResult(false, "子网绑定失败");
			}
			return new BocloudResult(true, "子网绑定成功");
		} catch (Exception e) {
			logger.error("子网绑定失败：" + e);
			return new BocloudResult(false, "子网绑定失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult associateSubnet(List<String> subnetUuids, String routeTableId) {
		try {
			AssociateRouteTableRequest request = new AssociateRouteTableRequest();
			request.setRouteTableId(routeTableId);
			request.setVSwitchId(subnetUuids.get(0));
			request.setRegionId(this.getRegionId());
			AssociateRouteTableResponse associateRouteTable = this.getVPCClient().associateRouteTable(request);
			if (null == associateRouteTable || null == associateRouteTable.getBody()) {
				return new BocloudResult(false, "子网绑定失败");
			}
			return new BocloudResult(true, "子网绑定成功");
		} catch (Exception e) {
			logger.error("子网绑定失败：" + e);
			return new BocloudResult(false, "子网绑定失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult unAssociate(String routeTableId, String vSwitchId) {
		try {
			UnassociateRouteTableRequest request = new UnassociateRouteTableRequest();
			request.setRouteTableId(routeTableId);
			request.setVSwitchId(vSwitchId);
			request.setRegionId(this.getRegionId());
			UnassociateRouteTableResponse unassociateRouteTable = this.getVPCClient().unassociateRouteTable(request);
			if (null == unassociateRouteTable || null == unassociateRouteTable.getBody()) {
				return new BocloudResult(false, "子网解绑失败");
			}
			return new BocloudResult(true, "子网解绑成功");
		} catch (Exception e) {
			logger.error("子网解绑失败：" + e);
			return new BocloudResult(false, "解绑子网失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult unAssociateSubnet(String subnetUuid, String routeTableId) {
		try {
			UnassociateRouteTableRequest request = new UnassociateRouteTableRequest();
			request.setRouteTableId(routeTableId);
			request.setVSwitchId(subnetUuid);
			request.setRegionId(this.getRegionId());
			UnassociateRouteTableResponse unassociateRouteTable = this.getVPCClient().unassociateRouteTable(request);
			if (null == unassociateRouteTable || null == unassociateRouteTable.getBody()) {
				return new BocloudResult(false, "子网解绑失败");
			}
			return new BocloudResult(true, "子网解绑成功");
		} catch (Exception e) {
			logger.error("子网解绑失败：" + e);
			return new BocloudResult(false, "解绑子网失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult getByVpc(String vpcUuid) {
		try {
			DescribeRouteTableListRequest request = new DescribeRouteTableListRequest();
			request.setVpcId(vpcUuid);
			request.setPageNumber(1);
			request.setPageSize(PAGE_SIZE);
			request.setRegionId(this.getRegionId());
			List<RouteTableModel> routeTableModels = Lists.newArrayList();
			DescribeRouteTableListResponse describeRouteTableListResponse = this.getVPCClient()
					.describeRouteTableList(request);
			if (null == describeRouteTableListResponse || null == describeRouteTableListResponse.getBody()) {
				return new BocloudResult(false, "路由表列表查询失败");
			} else if (null == describeRouteTableListResponse.getBody().getRouterTableList().getRouterTableListType()) {
				return new BocloudResult(false, "路由表列表查询失败");
			}
			List<DescribeRouteTableListResponseBodyRouterTableListRouterTableListType> routerTableListType = describeRouteTableListResponse
					.getBody().getRouterTableList().getRouterTableListType();
			int totalCount = describeRouteTableListResponse.getBody().getTotalCount();
			if (totalCount == 0) {
				return new BocloudResult(true, routeTableModels, "路由表列表为空");
			}
			for (DescribeRouteTableListResponseBodyRouterTableListRouterTableListType routeTable : routerTableListType) {
				RouteTableModel convertModel = this.convertRouteTable(routeTable);
				routeTableModels.add(convertModel);
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, routeTableModels, "路由表列表查询成功");
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setPageNumber(i);
				request.setPageSize(PAGE_SIZE);
				describeRouteTableListResponse = this.getVPCClient().describeRouteTableList(request);
				routerTableListType = describeRouteTableListResponse.getBody().getRouterTableList()
						.getRouterTableListType();
				for (DescribeRouteTableListResponseBodyRouterTableListRouterTableListType routeTable : routerTableListType) {
					RouteTableModel convertModel = this.convertRouteTable(routeTable);
					routeTableModels.add(convertModel);
				}
			}
			return new BocloudResult(true, routeTableModels, "路由表列表查询成功");
		} catch (Exception e) {
			logger.error("路由表列表查询失败：" + e);
			return new BocloudResult(false, "路由表列表查询失败：" + e.getMessage());
		}
	}

}
