package com.bocloud.cmp.provider.network;

import com.aliyun.ecs20140526.models.AuthorizeSecurityGroupEgressRequest;
import com.aliyun.ecs20140526.models.AuthorizeSecurityGroupEgressResponse;
import com.aliyun.ecs20140526.models.AuthorizeSecurityGroupRequest;
import com.aliyun.ecs20140526.models.AuthorizeSecurityGroupResponse;
import com.aliyun.ecs20140526.models.RevokeSecurityGroupEgressRequest;
import com.aliyun.ecs20140526.models.RevokeSecurityGroupEgressResponse;
import com.aliyun.ecs20140526.models.RevokeSecurityGroupRequest;
import com.aliyun.ecs20140526.models.RevokeSecurityGroupResponse;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 安全组规则操作类
 *
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunSecuGroupRuleProvider extends AliyunProvider implements SecurityGroupRuleProvider {

	private static Logger logger = LoggerFactory.getLogger(AliyunSecuGroupRuleProvider.class);

	public AliyunSecuGroupRuleProvider(Butler butler) {
		super(butler);
	}

	/**
	 * 一个安全组的安全组规则最多为 100 条。 安全组规则分为接受访问（accept）和拒绝访问（drop）两类。
	 * 安全组规则优先级（Priority）可选范围为 1~100。数字越小，代表优先级越高。 优先级相同的安全组规则，以拒绝访问（drop）的规则优先。
	 * 源端设备可以是指定 IP 地址范围（SourceCidrIp），也可以是其他安全组（SourceGroupId）中的实例。
	 * 以下任意一组参数可以确定一条安全组规则，只指定一个参数无法确定一条安全组规则。如果匹配的安全组规则已存在，此次
	 * AuthorizeSecurityGroup 调用失败。
	 */
	@Override
	public BocloudResult create(SecurityGroupRuleModel model) {
		try {
			String direction = model.getDirection();
			if ("egress".equalsIgnoreCase(direction)) {
				return this.authorizeSecurityGroupEgress(model);
			} else if ("ingress".equalsIgnoreCase(direction)) {
				return this.authorizeSecurityGroup(model);
			} else {
				return new BocloudResult(false, "添加失败");
			}
		} catch (Exception e) {
			logger.error("添加安全组规则失败:" + e);
			return new BocloudResult(false, "添加安全组规则失败:" + e.getMessage());
		}
	}

	private BocloudResult authorizeSecurityGroupEgress(SecurityGroupRuleModel model) {
		try {
			AuthorizeSecurityGroupEgressRequest request = new AuthorizeSecurityGroupEgressRequest();
			/**
			 * 传输层协议。取值大小写敏感。取值范围： tcp udp icmp gre all：支持所有协议
			 */
			request.setIpProtocol(model.getProtocol().toLowerCase());
			/**
			 * SecurityGroupId 方开放的传输层协议相关的端口范围。取值范围： TCP/UDP 协议：取值范围为 1~65535
			 * 使用斜线（/）隔开起始端口和终止端口。 正确示范：1/200；错误示范：200/1。 ICMP 协议：-1/-1 GRE 协议：-1/-1
			 * IpProtocol 取值为 all：-1/-1
			 */
			request.setPortRange(model.getPortMin() + "/" + model.getPortMax());
			request.setSecurityGroupId(model.getGroupUuid());
			/**
			 * 目的端 IPv4 CIDR 地址段。支持 CIDR 格式和 IPv4 格式的 IP 地址范围。 默认值：无。
			 */
			request.setDestCidrIp(model.getRemoteIpPrefix());
			/**
			 * 设置访问权限。取值范围： accept（默认）：接受访问。 drop：拒绝访问，不发回拒绝信息。
			 */
			request.setPolicy(model.getAction().toLowerCase());
			request.setPriority(String.valueOf(model.getPriority()));
			request.setRegionId(this.getRegionId());
			AuthorizeSecurityGroupEgressResponse authorizeSecurityGroupEgress = this.getECSClient()
					.authorizeSecurityGroupEgress(request);
			return new BocloudResult(true, model, "添加成功！");
		} catch (Exception e) {
			logger.error("添加安全组规则失败:" + e);
			return new BocloudResult(false, "添加安全组规则失败:" + e.getMessage());
		}
	}

	private BocloudResult authorizeSecurityGroup(SecurityGroupRuleModel model) {
		try {
			AuthorizeSecurityGroupRequest request = new AuthorizeSecurityGroupRequest();
			/**
			 * 传输层协议。取值大小写敏感。取值范围： tcp udp icmp gre all：支持所有协议
			 */
			request.setIpProtocol(model.getProtocol().toLowerCase());
			/**
			 * SecurityGroupId 方开放的传输层协议相关的端口范围。取值范围： TCP/UDP 协议：取值范围为 1~65535
			 * 使用斜线（/）隔开起始端口和终止端口。 正确示范：1/200；错误示范：200/1。 ICMP 协议：-1/-1 GRE 协议：-1/-1
			 * IpProtocol 取值为 all：-1/-1
			 */
			request.setPortRange(model.getPortMin() + "/" + model.getPortMax());
			request.setSecurityGroupId(model.getGroupUuid());
			/**
			 * 目的端 IPv4 CIDR 地址段。支持 CIDR 格式和 IPv4 格式的 IP 地址范围。 默认值：无。
			 */
			request.setSourceCidrIp(model.getRemoteIpPrefix());
			/**
			 * 设置访问权限。取值范围： accept（默认）：接受访问。 drop：拒绝访问，不发回拒绝信息。
			 */
			request.setPolicy(model.getAction().toLowerCase());
			request.setPriority(String.valueOf(model.getPriority()));
			request.setRegionId(this.getRegionId());
			request.setDescription(model.getRemark());
			AuthorizeSecurityGroupResponse authorizeSecurityGroup = this.getECSClient().authorizeSecurityGroup(request);
			return new BocloudResult(true, model, "添加成功！");
		} catch (Exception e) {
			logger.error("添加安全组规则失败:" + e);
			return new BocloudResult(false, "添加安全组规则失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String ruleId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult remove(SecurityGroupRuleModel ruleModel) {
		try {
			String direction = ruleModel.getDirection();
			if ("egress".equalsIgnoreCase(direction)) {
				return this.revokeSecurityGroupEgress(ruleModel);
			} else if ("ingress".equalsIgnoreCase(direction)) {
				return this.removekeSecurityGroup(ruleModel);
			} else {
				return new BocloudResult(false, "删除失败");
			}
		} catch (Exception e) {
			logger.error("删除安全组规则失败:" + e);
			return new BocloudResult(false, "删除安全组规则失败:" + e.getMessage());
		}
	}

	/**
	 * 删除一条安全组入方向规则,撤销安全组出方向的权限设置。
	 *
	 * @param model
	 * @return
	 */
	private BocloudResult removekeSecurityGroup(SecurityGroupRuleModel model) {
		try {
			RevokeSecurityGroupRequest request = new RevokeSecurityGroupRequest();
			/**
			 * 传输层协议。取值大小写敏感。取值范围： tcp udp icmp gre all：支持所有协议
			 */
			request.setIpProtocol(model.getProtocol().toLowerCase());
			/**
			 * SecurityGroupId 方开放的传输层协议相关的端口范围。取值范围： TCP/UDP 协议：取值范围为 1~65535
			 * 使用斜线（/）隔开起始端口和终止端口。 正确示范：1/200；错误示范：200/1。 ICMP 协议：-1/-1 GRE 协议：-1/-1
			 * IpProtocol 取值为 all：-1/-1
			 */
			request.setPortRange(model.getPortMin() + "/" + model.getPortMax());
			request.setSecurityGroupId(model.getGroupUuid());
			/**
			 * 目的端 IPv4 CIDR 地址段。支持 CIDR 格式和 IPv4 格式的 IP 地址范围。 默认值：无。
			 */
			request.setSourceCidrIp(model.getRemoteIpPrefix());
			/**
			 * 设置访问权限。取值范围： accept（默认）：接受访问。 drop：拒绝访问，不发回拒绝信息。
			 */
			request.setPolicy(model.getAction().toLowerCase());
			request.setRegionId(this.getRegionId());
			RevokeSecurityGroupResponse revokeSecurityGroup = this.getECSClient().revokeSecurityGroup(request);
			return new BocloudResult(true, revokeSecurityGroup.getBody(), "删除成功！");
		} catch (Exception e) {
			logger.error("删除安全组规则失败:" + e);
			return new BocloudResult(false, "删除安全组规则失败:" + e.getMessage());
		}
	}

	/**
	 * 删除一条安全组入方向规则,撤销安全组出方向的权限设置。
	 *
	 * @param model
	 * @return
	 */
	private BocloudResult revokeSecurityGroupEgress(SecurityGroupRuleModel model) {
		try {
			RevokeSecurityGroupEgressRequest request = new RevokeSecurityGroupEgressRequest();
			/**
			 * 传输层协议。取值大小写敏感。取值范围： tcp udp icmp gre all：支持所有协议
			 */
			request.setIpProtocol(model.getProtocol().toLowerCase());
			/**
			 * SecurityGroupId 方开放的传输层协议相关的端口范围。取值范围： TCP/UDP 协议：取值范围为 1~65535
			 * 使用斜线（/）隔开起始端口和终止端口。 正确示范：1/200；错误示范：200/1。 ICMP 协议：-1/-1 GRE 协议：-1/-1
			 * IpProtocol 取值为 all：-1/-1
			 */
			request.setPortRange(model.getPortMin() + "/" + model.getPortMax());
			request.setSecurityGroupId(model.getGroupUuid());
			/**
			 * 目的端 IPv4 CIDR 地址段。支持 CIDR 格式和 IPv4 格式的 IP 地址范围。 默认值：无。
			 */
			request.setDestCidrIp(model.getRemoteIpPrefix());
			/**
			 * 设置访问权限。取值范围： accept（默认）：接受访问。 drop：拒绝访问，不发回拒绝信息。
			 */
			request.setPolicy(model.getAction().toLowerCase());
			request.setRegionId(this.getRegionId());
			RevokeSecurityGroupEgressResponse revokeSecurityGroupEgress = this.getECSClient()
					.revokeSecurityGroupEgress(request);
			return new BocloudResult(true, "删除成功！");
		} catch (Exception e) {
			logger.error("删除安全组规则失败:" + e);
			return new BocloudResult(false, "删除安全组规则失败:" + e.getMessage());
		}
	}

}
