package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSON;
import com.aliyun.ecs20140526.models.*;
import com.aliyun.ecs20140526.models.DescribeSecurityGroupAttributeResponseBody.DescribeSecurityGroupAttributeResponseBodyPermissionsPermission;
import com.aliyun.ecs20140526.models.DescribeSecurityGroupsResponseBody.DescribeSecurityGroupsResponseBodySecurityGroupsSecurityGroup;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.Map.Entry;

/**
 * 阿里云 安全组
 *
 * @author ymh
 * @时间 2019年3月27日
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunSecurityGroupProvider extends AliyunProvider implements SecurityGroupProvider {

	private static Logger logger = LoggerFactory.getLogger(AliyunSecurityGroupProvider.class);

	private static final int PAGE_SIZE = 50;

	private Map<Integer, String> groupRuleMap = new HashMap<>();

	private Butler butler;

	public AliyunSecurityGroupProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	/**
	 * 新创建的安全组默认生成规则
	 */
	{
		groupRuleMap.put(80, "TCP");
		groupRuleMap.put(443, "TCP");
		groupRuleMap.put(22, "TCP");
		groupRuleMap.put(-1, "ICMP");
		groupRuleMap.put(3389, "TCP");
	}

	@Override
	public BocloudResult list() {
		DescribeSecurityGroupsRequest request = new DescribeSecurityGroupsRequest();
		request.setPageNumber(1);
		request.setPageSize(PAGE_SIZE);
		request.setRegionId(this.getRegionId());
		List<SecurityGroupModel> models = Lists.newArrayList();
		try {
			DescribeSecurityGroupsResponse describeSecurityGroups = this.getECSClient().describeSecurityGroups(request);
			if (null == describeSecurityGroups || null == describeSecurityGroups.getBody()) {
				return new BocloudResult<>(true, "暂无数据");
			}
			List<DescribeSecurityGroupsResponseBodySecurityGroupsSecurityGroup> securityGroup = describeSecurityGroups
					.getBody().getSecurityGroups().getSecurityGroup();
			if(!ListTool.isEmpty(securityGroup)) {
			    logger.info("安全组 info : " + JSON.toJSONString(securityGroup.get(0)));
			}
			for (DescribeSecurityGroupsResponseBodySecurityGroupsSecurityGroup group : securityGroup) {
				models.add(this.convert(group));
			}
			Integer totalCount = describeSecurityGroups.getBody().getTotalCount();
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, models, "success");
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				for (SecurityGroupModel securityGroupModel : models) {
					securityGroupModel.setRegionId(this.getRegionId());
				}
				return new BocloudResult(true, models, "success");
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setPageNumber(i);
				request.setPageSize(PAGE_SIZE);
				describeSecurityGroups = this.getECSClient().describeSecurityGroups(request);
				securityGroup = describeSecurityGroups.getBody().getSecurityGroups().getSecurityGroup();
				for (DescribeSecurityGroupsResponseBodySecurityGroupsSecurityGroup group : securityGroup) {
					models.add(this.convert(group));
				}
			}
			for (SecurityGroupModel securityGroupModel : models) {
				securityGroupModel.setRegionId(this.getRegionId());
			}
			return new BocloudResult(true, models, "success");
		} catch (Exception e) {
			logger.error("安全组列表查询失败:" + e.getMessage());
			return new BocloudResult(false, "安全组列表查询失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult create(SecurityGroupModel group) {
		CreateSecurityGroupRequest request = new CreateSecurityGroupRequest();
		request.setDescription(group.getRemark());
		request.setRegionId(this.getRegionId());
		request.setSecurityGroupName(group.getName());
		request.setVpcId(group.getVpcUuid());
		String resourceGroupUuid = group.getResourceGroupUuid();
		if (!StringUtils.isEmpty(resourceGroupUuid)) {
			request.setResourceGroupId(resourceGroupUuid);
		}
		if (!StringUtils.isEmpty(group.getSecurityGroupType())) {
			request.setSecurityGroupType(group.getSecurityGroupType());
		}
		try {
			CreateSecurityGroupResponse createSecurityGroup = this.getECSClient().createSecurityGroup(request);
			if (null == createSecurityGroup || null == createSecurityGroup.getBody()) {
				return new BocloudResult<>(false, "安全组添加失败");
			}
			String securityGroupId = createSecurityGroup.getBody().getSecurityGroupId();
			addSecurityRule(securityGroupId, group.getCategory());
			// 企业安全组出方向默认禁止所有访问，即从安全组内ECS访问外部都是禁止的
			if ("enterprise".equalsIgnoreCase(group.getSecurityGroupType())) {
				addEgressRule(securityGroupId);
			}
			return detail(securityGroupId);
		} catch (Exception e) {
			logger.error("安全组创建失败：" + e);
			return new BocloudResult(false, "安全组创建失败：" + e.getMessage());
		}
	}

	private void addEgressRule(String securityGroupId) {
		AliyunSecuGroupRuleProvider groupRule = new AliyunSecuGroupRuleProvider(butler);
		SecurityGroupRuleModel model = null;
		model = new SecurityGroupRuleModel();
		model.setDirection("egress");
		model.setProtocol("ALL");
		model.setPortMax(-1);
		model.setPortMin(-1);
		model.setRemoteIpPrefix("0.0.0.0/0");
		model.setGroupUuid(securityGroupId);
		model.setAction("accept");
		model.setPriority(1);
		groupRule.create(model);
	}

	private void addSecurityRule(String securityGroupId, String category) {
		AliyunSecuGroupRuleProvider groupRule = new AliyunSecuGroupRuleProvider(butler);
		SecurityGroupRuleModel model = null;
		Set<Entry<Integer, String>> entrySet = groupRuleMap.entrySet();
		for (Entry<Integer, String> entry : entrySet) {
			model = new SecurityGroupRuleModel();
			model.setDirection("ingress");
			model.setProtocol(entry.getValue());
			model.setPortMax(entry.getKey());
			model.setPortMin(entry.getKey());
			model.setRemoteIpPrefix("0.0.0.0/0");
			model.setGroupUuid(securityGroupId);
			model.setAction("accept");
			model.setPriority(1);
			groupRule.create(model);
		}
	}

	@Override
	public BocloudResult remove(String sgId) {
		DeleteSecurityGroupRequest request = new DeleteSecurityGroupRequest();
		request.setRegionId(this.getRegionId());
		request.setSecurityGroupId(sgId);
		try {
			DeleteSecurityGroupResponse deleteSecurityGroup = this.getECSClient().deleteSecurityGroup(request);
			return new BocloudResult(true, "安全组删除成功");
		} catch (Exception e) {
			logger.error("安全组删除失败：" + e);
			return new BocloudResult(false, "安全组删除失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String sgId) {
		DescribeSecurityGroupsRequest request = new DescribeSecurityGroupsRequest();
		request.setRegionId(this.getRegionId());
		request.setSecurityGroupId(sgId);
		try {
			DescribeSecurityGroupsResponse describeSecurityGroups = this.getECSClient().describeSecurityGroups(request);
			if (null == describeSecurityGroups || null == describeSecurityGroups.getBody()) {
				return new BocloudResult<>(false, "资源不存在");
			}
			List<DescribeSecurityGroupsResponseBodySecurityGroupsSecurityGroup> securityGroup = describeSecurityGroups
					.getBody().getSecurityGroups().getSecurityGroup();
			SecurityGroupModel convert = convert(securityGroup.get(0));
			convert.setRegionId(this.getRegionId());
			return new BocloudResult(true, convert, "安全组详情查询成功");
		} catch (Exception e) {
			logger.error("安全组详情查询失败：" + e);
			return new BocloudResult(false, "安全组详情失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(SecurityGroupModel model) {
		ModifySecurityGroupAttributeRequest request = new ModifySecurityGroupAttributeRequest();
		request.setSecurityGroupId(model.getGroupUuid());
		request.setSecurityGroupName(model.getName());
		request.setDescription(model.getRemark());
		request.setRegionId(this.getRegionId());
		try {
			ModifySecurityGroupAttributeResponse modifySecurityGroupAttribute = this.getECSClient()
					.modifySecurityGroupAttribute(request);
			return new BocloudResult(true, "安全组修改成功");
		} catch (Exception e) {
			logger.error("安全组修改失败：" + e);
			return new BocloudResult(false, "安全组修改失败：" + e.getMessage());
		}
	}

	private SecurityGroupModel convert(DescribeSecurityGroupsResponseBodySecurityGroupsSecurityGroup group) {
		if (null == group) {
			return null;
		}
		SecurityGroupModel model = new SecurityGroupModel();
		model.setName(group.getSecurityGroupName());
		model.setGroupUuid(group.getSecurityGroupId());
		model.setRemark(group.getDescription());
		model.setVpcUuid(group.getVpcId());
		model.setCreateTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", group.getCreationTime(), "Z"));
		model.setResourceGroupUuid(group.getResourceGroupId());
		model.setSecurityGroupType(group.getSecurityGroupType());
		BocloudResult ruleList = listGroups(group.getSecurityGroupId());
		@SuppressWarnings("unchecked")
		List<SecurityGroupRuleModel> ruleModels = (List<SecurityGroupRuleModel>) ruleList.getData();
		if (null != ruleModels) {
			model.setModels(ruleModels);
		}
		return model;
	}

	private BocloudResult listGroups(String secuId) {
		try {
			DescribeSecurityGroupAttributeRequest request = new DescribeSecurityGroupAttributeRequest();
			request.setSecurityGroupId(secuId);
			request.setRegionId(this.getRegionId());
			DescribeSecurityGroupAttributeResponse describeSecurityGroupAttribute = this.getECSClient()
					.describeSecurityGroupAttribute(request);
			if (null == describeSecurityGroupAttribute || null == describeSecurityGroupAttribute.getBody()) {
				return new BocloudResult<>(true, "暂无安全组规则");
			}
			List<DescribeSecurityGroupAttributeResponseBodyPermissionsPermission> permission = describeSecurityGroupAttribute
					.getBody().getPermissions().getPermission();
			return new BocloudResult(true, convertModel(permission), "安全组规则查询成功");
		} catch (Exception e) {
			logger.error("安全组规则查询失败 ： " + e);
			return new BocloudResult(false, "安全组规则查询失败：" + e.getMessage());
		}
	}

	public List<SecurityGroupRuleModel> convertModel(
			List<DescribeSecurityGroupAttributeResponseBodyPermissionsPermission> permissions) {
		List<SecurityGroupRuleModel> ruleModels = new ArrayList<>();
		SecurityGroupRuleModel ruleModel = null;
		if (null != permissions && permissions.size() > 0) {
			for (DescribeSecurityGroupAttributeResponseBodyPermissionsPermission permission : permissions) {
				ruleModel = new SecurityGroupRuleModel();
				ruleModel.setProtocol(permission.getIpProtocol());
				ruleModel.setDirection(permission.getDirection());
				String portRange = permission.getPortRange();
				if (null != portRange) {
					String[] split = portRange.split("/");
					ruleModel.setPortMin(Integer.parseInt(split[0]));
					ruleModel.setPortMax(Integer.parseInt(split[1]));
				}
				String remoteIpPrefix = permission.getDestCidrIp();
				if (null == remoteIpPrefix || "".equals(remoteIpPrefix)) {
					remoteIpPrefix = permission.getSourceCidrIp();
				}
				ruleModel.setRemoteIpPrefix(remoteIpPrefix);
				ruleModel.setAction(permission.getPolicy());
				ruleModel.setPriority(Integer.valueOf(permission.getPriority()));// 优先级
				ruleModel.setRemark(permission.getDescription());
				ruleModels.add(ruleModel);
			}
		}
		return ruleModels;
	}

}
