package com.bocloud.cmp.provider.storage;

import com.alibaba.fastjson.JSON;
import com.aliyun.credentials.utils.StringUtils;
import com.aliyun.ecs20140526.models.*;
import com.bocloud.cmp.entity.enums.TagType;
import com.bocloud.cmp.entity.enums.VolumeCategory;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.TagModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.VolumeConvertor;
import com.bocloud.cmp.provider.compute.AliyunSnapshotProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collections;
import java.util.List;

/**
 * 阿里云硬盘
 *
 * @author ymh
 * @时间 2019年3月5日
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunVolumeProvider extends AliyunProvider implements VolumeProvider {

	private static Logger logger = LoggerFactory.getLogger(AliyunVolumeProvider.class);
	private static final int PAGE_SIZE = 50;

	private BeanConvertor<VolumeModel, DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDisk> convertor = new VolumeConvertor();
	private Butler butler;

	public AliyunVolumeProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult list() {
		try {
			DescribeDisksRequest request = new DescribeDisksRequest();
			request.setPageNumber(1);
			request.setPageSize(PAGE_SIZE);
			request.setRegionId(this.getRegionId());
			List<VolumeModel> models = Lists.newArrayList();

			DescribeDisksResponse response = this.getECSClient().describeDisks(request);
			List<DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDisk> disks = response.getBody().getDisks()
					.getDisk();
			if(!ListTool.isEmpty(disks)) {
			    logger.info("硬盘 info : " + JSON.toJSONString(disks.get(0)));
			}
			for (DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDisk disk : disks) {
				VolumeModel convert = convertor.convertModel(disk);
				convert.setRegionId(this.getRegionId());
				models.add(convert);
			}
			Integer totalCount = response.getBody().getTotalCount();
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, models, "云硬盘列表查询成功");
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, models, "云硬盘列表查询成功");
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setPageNumber(i);
				request.setPageSize(PAGE_SIZE);
				response = this.getECSClient().describeDisks(request);
				disks = response.getBody().getDisks().getDisk();
				for (DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDisk disk : disks) {
					VolumeModel convert = convertor.convertModel(disk);
					convert.setRegionId(this.getRegionId());
					models.add(convert);
				}
			}
			return new BocloudResult(true, models, "云硬盘列表查询成功");
		} catch (Exception e) {
			logger.error("云硬盘列表查询失败:" + e);
			return new BocloudResult(false, "云硬盘列表查询失败:" + e);
		}
	}

	@Override
	public BocloudResult create(VolumeModel model) {
		try {
			logger.info("provider model : " + JSON.toJSONString(model));
			CreateDiskRequest request = new CreateDiskRequest();
			request.setRegionId(this.getRegionId());
			if (!StringUtils.isEmpty(model.getZone())) {
				request.setZoneId(model.getZone());
			}
			if (!StringUtils.isEmpty(model.getSourceSnapUuid())) {
				request.setSnapshotId(model.getSourceSnapUuid());
			}
			request.setDiskName(model.getName());
			/***
			 * 容量大小，以GiB为单位。指定该参数后，其取值必须≥指定快照ID的容量大小。取值范围： cloud：5~2000
			 * cloud_efficiency：20~32768 cloud_ssd：20~32768 cloud_essd：20~32768
			 */
			request.setSize(model.getSize());
			/***
			 * 数据盘的云盘种类。取值范围：
			 *
			 * cloud：普通云盘 cloud_efficiency：高效云盘 cloud_ssd：SSD云盘 cloud_essd：ESSD云盘 默认值：cloud
			 */
			logger.info("disk category : " + convertDiskCategory(model.getCategory()));
			request.setDiskCategory(convertDiskCategory(model.getCategory()));
			if (null != model.getEncrypted()) {
				request.setEncrypted(model.getEncrypted());
			}
			request.setDescription(model.getRemark());
			request.setInstanceId(model.getInstanceId());
			if (!StringUtils.isEmpty(model.getResourceGroupUuid())) {
				request.setResourceGroupId(model.getResourceGroupUuid());
			}
			if ("essd".equalsIgnoreCase(model.getCategory())) {
				request.setPerformanceLevel(model.getPerformanceLevel());
			}
			CreateDiskResponse response = this.getECSClient().createDisk(request);
			Thread.sleep(10000);
			return new BocloudResult(true, getAvailable(response.getBody().getDiskId(), "Available"), "云硬盘创建成功");
		} catch (Exception e) {
			logger.error("云硬盘创建失败:" + e);
			return new BocloudResult(false, "云硬盘创建失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult createTag(VolumeModel model) {
		try {
			List<TagModel> tagModels = model.getTagModels();
			// 创建标签
			List<TagResourcesRequest.TagResourcesRequestTag> tags = Lists.newArrayList();
			TagResourcesRequest tagRequest = new TagResourcesRequest();
			tagRequest.setRegionId(this.getRegionId());
			tagRequest.setResourceType(TagType.disk.name());
			tagRequest.setResourceId(Collections.singletonList(model.getVolumeUuid()));
			for (TagModel tagModel : tagModels) {
				TagResourcesRequest.TagResourcesRequestTag tag = new TagResourcesRequest.TagResourcesRequestTag();
				tag.setKey(tagModel.getKey());
				tag.setValue(tagModel.getValue());
				tags.add(tag);
			}
			tagRequest.setTag(tags);
			TagResourcesResponse response = this.getECSClient().tagResources(tagRequest);
			return new BocloudResult(true, "云硬盘标签创建成功");
		} catch (Exception e) {
			logger.error("云硬盘标签创建失败:" + e);
			return new BocloudResult(false, "云硬盘标签创建失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult removeTag(VolumeModel model) {
		try {
			List<TagModel> tagModels = model.getTagModels();
			// 创建标签
			List<String> tagKeys = Lists.newArrayList();
			UntagResourcesRequest tagRequest = new UntagResourcesRequest();
			tagRequest.setResourceType(TagType.instance.name());
			tagRequest.setRegionId(this.getRegionId());
			tagRequest.setResourceId(Collections.singletonList(model.getInstanceId()));

			for (TagModel tagModel : tagModels) {
				tagKeys.add(tagModel.getKey());
			}
			tagRequest.setTagKey(tagKeys);
			UntagResourcesResponse response = this.getECSClient().untagResources(tagRequest);
			return new BocloudResult(true, "云硬盘标签移除成功");
		} catch (Exception e) {
			logger.error("云硬盘标签移除失败:" + e);
			return new BocloudResult(false, "云硬盘标签移除失败:" + e.getMessage());
		}
	}

	private VolumeModel getAvailable(String volumeId, String status) {
		DescribeDisksRequest request = new DescribeDisksRequest();
		request.setRegionId(this.getRegionId());
		request.setDiskIds(JSON.toJSONString(new String[] { volumeId }));
		VolumeModel model = new VolumeModel();
		try {
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				try {
					Thread.sleep(10000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
				DescribeDisksResponse response = this.getECSClient().describeDisks(request);
				List<DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDisk> disks = response.getBody().getDisks()
						.getDisk();
				if (ListTool.isEmpty(disks)) {
					flag = false;
					break;
				}
				if (status.equalsIgnoreCase(disks.get(0).getStatus())) {
					model = convertor.convertModel(disks.get(0));
					model.setRegionId(this.getRegionId());
					flag = false;
					break;
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 5 * 60 * 1000) {
					model = convertor.convertModel(disks.get(0));
					flag = false;
					break;
				}
			}
		} catch (Exception e) {
			logger.error("get disk error:", e);
		}
		return model;
	}

	@Override
	public BocloudResult modify(VolumeModel model) {
		try {
			ModifyDiskAttributeRequest request = new ModifyDiskAttributeRequest();
			request.setDiskId(model.getVolumeUuid());
			request.setDiskName(model.getName());
			request.setDescription(model.getRemark());
			ModifyDiskAttributeResponse response = this.getECSClient().modifyDiskAttribute(request);
			if (null == response) {
				return new BocloudResult(false, "云盘修改失败");
			}
			BocloudResult bocloudResult = this.detail(model.getVolumeUuid());
			if (bocloudResult.isFailed()) {
				return new BocloudResult(false, "云盘修改失败");
			}
			return new BocloudResult(true, bocloudResult.getData(), "云盘修改成功");
		} catch (Exception e) {
			logger.error("云盘修改失败 ：" + e);
			return new BocloudResult(false, "云盘修改失败 ：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String volumeId) {
		DeleteDiskRequest request = new DeleteDiskRequest();
		request.setDiskId(volumeId);
		try {
			DeleteDiskResponse response = this.getECSClient().deleteDisk(request);
			return new BocloudResult(true, "云硬盘删除成功");
		} catch (Exception e) {
			logger.error("云硬盘删除失败:" + e);
			return new BocloudResult(false, "云硬盘删除失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String volumeId) {
		DescribeDisksRequest request = new DescribeDisksRequest();
		request.setRegionId(this.getRegionId());
		request.setDiskIds(JSON.toJSONString(new String[] { volumeId }));
		try {
			DescribeDisksResponse response = this.getECSClient().describeDisks(request);
			if (null == response || ((null == response.getBody().getDisks()
					&& response.getBody().getDisks().getDisk().size() == 0))) {
				return new BocloudResult(false, "数据不存在");
			}
			VolumeModel convert = convertor.convertModel(response.getBody().getDisks().getDisk().get(0));
			return new BocloudResult(true, convert, "云硬盘详情查询成功");
		} catch (Exception e) {
			logger.error("云硬盘详情查询失败:" + e);
			return new BocloudResult(false, "云硬盘详情查询失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult attach(VolumeModel model) {
		AttachDiskRequest request = new AttachDiskRequest();
		request.setDiskId(model.getVolumeUuid());
		request.setInstanceId(model.getInstanceId());
		request.setDeleteWithInstance(model.getDeleteWithInstance());
		if (!StringUtils.isEmpty(model.getDevice())) {
			request.setDevice(model.getDevice());
		}
		try {
			this.getECSClient().attachDisk(request);
			Thread.sleep(3000);
			BocloudResult bocloudResult = this.detail(model.getVolumeUuid());
			if (bocloudResult.isFailed()) {
				return new BocloudResult(false, "云硬盘挂载失败");
			}
			VolumeModel volumeModel = (VolumeModel) bocloudResult.getData();
			return new BocloudResult(true, volumeModel.getDeviceName(), "云硬盘挂载成功");
		} catch (Exception e) {
			logger.error("云硬盘挂载失败:" + e);
			return new BocloudResult(false, "云硬盘挂载失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detach(String serverId, String volumeId) {
		DetachDiskRequest request = new DetachDiskRequest();
		request.setDiskId(volumeId);
		request.setInstanceId(serverId);
		try {
			DetachDiskResponse response = this.getECSClient().detachDisk(request);
			if (!StringUtils.isEmpty(response.getBody().getRequestId())) {
				return new BocloudResult(true, "卸载云硬盘成功");
			}
			return new BocloudResult(false, "卸载云硬盘失败");
		} catch (Exception e) {
			logger.error("卸载云硬盘失败:" + e);
			return new BocloudResult(false, "卸载云硬盘失败:" + e.getMessage());
		}
	}

	/**
	 * @return com.bocloud.common.model.BocloudResult
	 * @Author Hu Wentao
	 * @Description //TODO 使用磁盘的历史快照回滚至某一阶段的磁盘状态。 页面无法回滚，未测通 调用该接口时，您需要注意：
	 *              磁盘的状态必须为使用中（In_Use）的状态。 磁盘挂载的实例的状态必须为已停止（Stopped）。
	 *              指定的参数SnapshotId必须是由DiskId创建的历史快照。
	 *              通过DescribeInstances查询ECS实例信息时，如果返回数据中包含{"OperationLocks":
	 *              {"LockReason" : "security"}}，则禁止一切操作。
	 * @Date 14:05 2020/7/30
	 * @Param [volumeId, status]
	 **/
	@Override
	public BocloudResult resetStatus(String volumeId, String snapshotId) {
		try {
			ResetDiskRequest request = new ResetDiskRequest();
			request.setDiskId(volumeId);
			request.setSnapshotId(snapshotId);
			ResetDiskResponse response = this.getECSClient().resetDisk(request);
			if (null == response) {
				return new BocloudResult(false, "回滚至某一阶段的磁盘状态失败");
			}
			return new BocloudResult(true, response.getBody().getRequestId(), "回滚至某一阶段的磁盘状态成功");
		} catch (Exception e) {
			logger.error("回滚至某一阶段的磁盘状态失败:" + e);
			return new BocloudResult(false, "回滚至某一阶段的磁盘状态失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult extend(String volumeId, Integer newSize, String type) {
		try {
			ResizeDiskRequest request = new ResizeDiskRequest();
			request.setDiskId(volumeId);
			request.setNewSize(newSize);
			if (Boolean.parseBoolean(type)) {
				type = "online";
			} else {
				type = "offline";
			}
			request.setType(type);
			ResizeDiskResponse response = this.getECSClient().resizeDisk(request);
			if (null == response) {
				return new BocloudResult(false, "硬盘扩容失败");
			}
			BocloudResult bocloudResult = this.detail(volumeId);
			if (bocloudResult.isFailed()) {
				return new BocloudResult(false, "硬盘扩容失败");
			}
			return new BocloudResult(true, bocloudResult.getData(), "云硬盘扩容成功");
		} catch (Exception e) {
			logger.info("云硬盘扩容失败：" + e);
			return new BocloudResult(false, "云硬盘扩容失败：" + e.getMessage());
		}
	}
	
	@Override
	public BocloudResult extend(String volumeId, Integer newSize) {
		try {
			ResizeDiskRequest request = new ResizeDiskRequest();
			request.setDiskId(volumeId);
			request.setNewSize(newSize);
			ResizeDiskResponse response = this.getECSClient().resizeDisk(request);
			if (null == response) {
				return new BocloudResult(false, "硬盘扩容失败");
			}
			BocloudResult bocloudResult = this.detail(volumeId);
			if (bocloudResult.isFailed()) {
				return new BocloudResult(false, "硬盘扩容失败");
			}
			return new BocloudResult(true, bocloudResult.getData(), "云硬盘扩容成功");
		} catch (Exception e) {
			logger.info("云硬盘扩容失败：" + e);
			return new BocloudResult(false, "云硬盘扩容失败：" + e.getMessage());
		}
	}

	private String convertDiskCategory(String raw) {
		if (StringUtils.isEmpty(raw) || raw.equalsIgnoreCase("CLOUD")) {
			return "cloud";
		}
		if (raw.equalsIgnoreCase(VolumeCategory.EFFICIENCY.name())) {
			return "cloud_" + VolumeCategory.EFFICIENCY.name().toLowerCase();
		}
		if (raw.equalsIgnoreCase(VolumeCategory.SSD.name())) {
			return "cloud_" + VolumeCategory.SSD.name().toLowerCase();
		}
		if (raw.equalsIgnoreCase(VolumeCategory.ESSD.name())) {
			return "cloud_" + VolumeCategory.ESSD.name().toLowerCase();
		}
		return "cloud";
	}

	/**
	 * @return com.bocloud.common.model.BocloudResult
	 * @Author Hu Wentao
	 * @Description 调用ReplaceSystemDisk更换一台ECS实例的系统盘, 系统盘的云盘ID会发生变化，原云盘会被释放。 接口说明：
	 *              更换换系统盘时，您需要注意： 不支持更换系统盘的云盘类型。 不支持变更系统盘计费方式。
	 *              实例的状态必须为已停止（Stopped）状态。
	 * @Date 11:51 2020/7/30
	 * @Param [volumeModel]
	 **/
	public BocloudResult replaceSystemDisk(VolumeModel volumeModel) {
		try {
			ReplaceSystemDiskRequest request = new ReplaceSystemDiskRequest();
			request.setInstanceId(volumeModel.getInstanceId());
			request.setImageId(volumeModel.getImageUuid());
			ReplaceSystemDiskRequest.ReplaceSystemDiskRequestSystemDisk disk = new ReplaceSystemDiskRequest.ReplaceSystemDiskRequestSystemDisk();
			disk.setSize(volumeModel.getSize());
			request.setSystemDisk(disk);
			ReplaceSystemDiskResponse response = this.getECSClient().replaceSystemDisk(request);
			if (null == response) {
				return new BocloudResult(false, "更换一台ECS实例的系统盘失败");
			} else if (null == response.getBody().getDiskId()) {
				return new BocloudResult(false, "更换一台ECS实例的系统盘失败");
			}
			return new BocloudResult(true, response.getBody().getDiskId(), "更换一台ECS实例的系统盘成功");
		} catch (Exception e) {
			logger.error("更换一台ECS实例的系统盘失败:" + e);
			return new BocloudResult(false, "更换一台ECS实例的系统盘失败:" + e.getMessage());
		}
	}

	/**
	 * @return com.bocloud.common.model.BocloudResult
	 * @Author Hu Wentao
	 * @Description 重新初始化一块云盘到创建时的初始状态。 调用该接口时，您需要注意：
	 *              云盘的状态必须为使用中（In_use），云盘挂载的ECS实例的状态必须为已停止（Stopped）。
	 *              实例首次启动前，不能重新初始化挂载在其上的云盘。 对于系统盘，初始化到镜像的最初状态。若创建云盘的源镜像被删除，则无法初始化。
	 *              对于直接创建的数据盘，初始化到空盘状态。 对于通过快照创建的数据盘，初始化到快照状态。若源快照已被删除，则无法初始化并返回错误。
	 * @Date 14:01 2020/7/30
	 * @Param [diskId]
	 **/
	public BocloudResult reInitDisk(String diskId) {
		try {
			ReInitDiskRequest request = new ReInitDiskRequest();
			request.setDiskId(diskId);
			ReInitDiskResponse response = this.getECSClient().reInitDisk(request);
			if (null == response) {
				return new BocloudResult(false, "重新初始化云盘失败");
			}
			return new BocloudResult(true, response.getBody().getRequestId(), "重新初始化云盘成功");
		} catch (Exception e) {
			logger.error("重新初始化云盘失败:" + e);
			return new BocloudResult(false, "重新初始化云盘失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult snapshot(SnapshotModel snapshotModel) {
		try {
			CreateSnapshotRequest request = new CreateSnapshotRequest();
			request.setDiskId(snapshotModel.getVolumeUuid());
			request.setDescription(snapshotModel.getRemark());
			request.setSnapshotName(snapshotModel.getName());
			CreateSnapshotResponse response = this.getECSClient().createSnapshot(request);
			if (null == response || null == response.getBody().getSnapshotId()) {
				return new BocloudResult(false, "快照创建失败");
			}
			String snapshotId = response.getBody().getSnapshotId();
			snapshotModel.setSnapshotUuid(snapshotId);
			AliyunSnapshotProvider snapshotProvider = new AliyunSnapshotProvider(butler);
			BocloudResult detailResult = snapshotProvider.detail(snapshotModel);
			if (detailResult.isFailed()) {
				return new BocloudResult(false, "快照创建失败");
			}
			return new BocloudResult(true, detailResult.getData(), "快照创建成功");
		} catch (Exception e) {
			logger.info("快照创建失败：" + e);
			return new BocloudResult(false, "快照创建失败：" + e.getMessage());
		}
	}
}
