package com.bocloud.cmp.awsconvertor;

import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.ListTool;
import org.springframework.util.CollectionUtils;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: SecurityGroupConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 15:51
 */
public class SecurityGroupConvertor implements BeanConvertor<SecurityGroupModel, SecurityGroup> {

    private static final String IPV4 = "IPV4";
    private static final String IPV6 = "IPV6";

    @Override
    public List<SecurityGroupModel> convertList(List<? extends SecurityGroup> groups) {
        List<SecurityGroupModel> models = new ArrayList<>();
        for (SecurityGroup group : groups) {
            models.add(convertModel(group));
        }
        return models;
    }

    @Override
    public SecurityGroupModel convertModel(SecurityGroup group) {
        SecurityGroupModel model = new SecurityGroupModel();
        model.setRemark(group.description());
        model.setName(group.groupId());
        if (!ListTool.isEmpty(group.tags())) {
            for (Tag tag :
                    group.tags()) {
                if ("Name".equals(tag.key())) {
                    model.setName(tag.value());
                    break;
                }
            }
        }
        model.setAwsGroupName(group.groupName());
        model.setGroupUuid(group.groupId());
        model.setVpcUuid(group.vpcId());
        List<SecurityGroupRuleModel> ruleModels = new ArrayList<>();
        Optional.ofNullable(group.ipPermissions()).ifPresent(ipPermissions -> {
            ipPermissions.stream().forEach(ipPermission -> {
                ruleModels.add(convertRule(ipPermission, group.groupId(), "ingress", IPV4));
                ruleModels.add(convertRule(ipPermission, group.groupId(), "ingress", IPV6));
            });
        });
        Optional.ofNullable(group.ipPermissionsEgress()).ifPresent(ipPermissionsEgress -> {
            ipPermissionsEgress.stream().forEach(ipPermission -> {
                ruleModels.add(convertRule(ipPermission, group.groupId(), "egress", IPV4));
                ruleModels.add(convertRule(ipPermission, group.groupId(), "egress", IPV6));
            });
        });
        model.setModels(ruleModels);
        return model;
    }


    /**
     * 安全组规则转换
     *
     * @param rule
     * @param groupUuid
     * @return
     */
    private SecurityGroupRuleModel convertRule(IpPermission rule, String groupUuid, String direction, String type) {
        SecurityGroupRuleModel model = new SecurityGroupRuleModel();
        model.setProtocol("-1".equalsIgnoreCase(rule.ipProtocol()) ? "*" : rule.ipProtocol().toUpperCase());//协议
        model.setGroupUuid(groupUuid);//安全组uuid
        model.setDirection(direction);//方向
        model.setPortMin(Optional.ofNullable(rule.fromPort()).orElse(0));
        model.setPortMax(Optional.ofNullable(rule.toPort()).orElse(65535));
        switch (type) {
            case IPV4:
                List<IpRange> ipRanges = Optional.ofNullable(rule.ipRanges()).orElse(new ArrayList<>());
                if (!CollectionUtils.isEmpty(ipRanges)) {
                    //来源
                    model.setSourcePortRange(Optional.ofNullable(ipRanges.get(0)).map(ip -> ip.cidrIp()).orElse(null));
                    //描述
                    model.setRemark(Optional.ofNullable(ipRanges.get(0)).map(ip -> ip.description()).orElse(null));
                }
                model.setProps(IPV4);
                break;
            case IPV6:
                List<Ipv6Range> ipv6Ranges = Optional.ofNullable(rule.ipv6Ranges()).orElse(new ArrayList<>());
                if (!CollectionUtils.isEmpty(ipv6Ranges)) {
                    //来源
                    model.setSourcePortRange(Optional.ofNullable(ipv6Ranges.get(0)).map(ipv6 -> ipv6.cidrIpv6()).orElse(null));
                    //描述
                    model.setRemark(Optional.ofNullable(ipv6Ranges.get(0)).map(ipv6 -> ipv6.description()).orElse(null));
                }
                model.setProps(IPV6);
                break;
            default:
                break;
        }
        return model;
    }


}
