package com.bocloud.cmp.awsconvertor;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.NetworkInterfaceModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import software.amazon.awssdk.services.ec2.model.Instance;
import software.amazon.awssdk.services.ec2.model.Tag;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: ServerConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 14:59
 */
public class ServerConvertor implements BeanConvertor<ServerModel, Instance> {

    private final String dateFormat = "yyyy-MM-dd'T'HH:mm:ssZ";

    @Override
    public List<ServerModel> convertList(List<? extends Instance> instances) {
        List<ServerModel> serverModels = new ArrayList<>();
        for (Instance instance : instances) {
            if (!"TERMINATED".equalsIgnoreCase(instance.state().name().name())) {
                serverModels.add(convertModel(instance));
            }
        }
        return serverModels;
    }

    @Override
    public ServerModel convertModel(Instance instance) {
        ServerModel model = new ServerModel();
        model.setName(instance.instanceId());
        if (!ListTool.isEmpty(instance.tags())) {
            for (Tag tag :
                    instance.tags()) {
                if ("Name".equals(tag.key())) {
                    model.setName(tag.value());
                    break;
                }
            }
        }
        model.setImageUuid(instance.imageId());
        model.setInstanceId(instance.instanceId());
        model.setFlavorUuid(instance.instanceType().toString());
        model.setKeypairName(Optional.ofNullable(instance.keyName()).map(keypairName -> JSON.toJSONString(Arrays.asList(keypairName))).orElse(null));
        model.setGmtVmCreate(DateTools.transfer(dateFormat, instance.launchTime().toString(), "Z"));
        model.setZone(instance.placement().availabilityZone());
        model.setPrivateIps(Optional.ofNullable(instance.privateIpAddress()).map(privateIp -> JSON.toJSONString(Arrays.asList(privateIp))).orElse(null));
        model.setPublicIps(Optional.ofNullable(instance.publicIpAddress()).map(publicIp -> JSON.toJSONString(Arrays.asList(publicIp))).orElse(null));
        model.setStatus(instance.state().name().name());
        model.setSubnetUuid(instance.subnetId());
        model.setVpcUuid(instance.vpcId());
        Optional.ofNullable(instance.securityGroups()).ifPresent(groups -> model.setSecurityGroups(groups.stream().map(securityGroup -> securityGroup.groupId()).collect(Collectors.toList())));
        model.setDns(instance.privateDnsName());
        //挂载磁盘设备
        List<VolumeModel> diskDevices = new ArrayList<>();
        Optional.ofNullable(instance.blockDeviceMappings()).ifPresent(
                instanceDevices ->
                        instanceDevices.stream().forEach(instanceBlockDeviceMapping -> {
                            VolumeModel volumeModel = new VolumeModel();
                            volumeModel.setDevice(instanceBlockDeviceMapping.deviceName());
                            volumeModel.setAttachedTime(DateTools.transfer(dateFormat, instanceBlockDeviceMapping.ebs().attachTime().toString(), "Z"));
                            volumeModel.setVolumeUuid(instanceBlockDeviceMapping.ebs().volumeId());
                            volumeModel.setStatus(instanceBlockDeviceMapping.ebs().statusAsString());
                            volumeModel.setDeleteWithInstance(instanceBlockDeviceMapping.ebs().deleteOnTermination());
                            diskDevices.add(volumeModel);
                        }));
        model.setDiskDevices(diskDevices);
        //主机挂载的网络接口
        List<NetworkInterfaceModel> nicsModels = new ArrayList<>();
        Optional.ofNullable(instance.networkInterfaces()).ifPresent(
                networkInterfaces ->
                        networkInterfaces.stream().forEach(networkInterface -> {
                            NetworkInterfaceModel nicsModel = new NetworkInterfaceModel();
                            nicsModel.setNetworkInterfaceUuid(networkInterface.networkInterfaceId());
                            nicsModel.setPrimaryPrivateIPAddress(networkInterface.privateIpAddress());
                            nicsModel.setIpv6Address(networkInterface.ipv6Addresses().size() > 0 ? networkInterface.ipv6Addresses().get(0).ipv6Address() : null);
                            nicsModel.setMacAddress(networkInterface.macAddress());
                            nicsModel.setPublicIPAddress(Optional.ofNullable(networkInterface.association()).map(association -> association.publicIp()).orElse(null));
                            nicsModel.setAttachmentUuid(Optional.ofNullable(networkInterface.attachment()).map(attachment -> attachment.attachmentId()).orElse(null));
                            nicsModel.setAttachedTime(Optional.ofNullable(networkInterface.attachment()).map(attachment -> DateTools.transfer(dateFormat, attachment.attachTime().toString(), "Z")).orElse(null));
                            nicsModel.setDeviceIndex(Optional.ofNullable(networkInterface.attachment()).map(attachment -> attachment.deviceIndex()).orElse(-1));
                            nicsModel.setStatus(networkInterface.statusAsString());
                            nicsModels.add(nicsModel);
                        }));
        model.setNetworkInterfaces(nicsModels);
        Object tags = JSONObject.parseObject(JSONObject.toJSONString(instance.toBuilder())).get("tags");
        if (tags != null) {
            model.setTags(JSONObject.parseArray(
                    tags.toString(),
                    JSONObject.class));
        }
        return model;
    }
}
