package com.bocloud.cmp.provider;

import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;
import software.amazon.awssdk.auth.credentials.AwsBasicCredentials;
import software.amazon.awssdk.auth.credentials.AwsCredentials;
import software.amazon.awssdk.auth.credentials.StaticCredentialsProvider;
import software.amazon.awssdk.core.SdkClient;
import software.amazon.awssdk.http.SdkHttpClient;
import software.amazon.awssdk.http.apache.ApacheHttpClient;
import software.amazon.awssdk.http.apache.ProxyConfiguration;
import software.amazon.awssdk.regions.Region;
import software.amazon.awssdk.services.costexplorer.CostExplorerClient;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.DescribeAvailabilityZonesResponse;
import software.amazon.awssdk.services.elasticloadbalancingv2.ElasticLoadBalancingV2Client;
import software.amazon.awssdk.services.opsworks.OpsWorksClient;
import software.amazon.awssdk.services.sts.StsClient;

import java.net.URI;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AwsProvider
 * @Description:aws驱动客户端
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 14:55
 */
public class AwsProvider extends GenericProvider implements SuperProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsProvider.class);

    private final Butler butler;
    private String regionId = "cn-north-1";// 可用域
    private final String accessKey;// access key
    private final String secretKey;// secret access key
    private final AwsCredentials credentials;// 访问凭证

    public AwsProvider(Butler butler) {
        super(butler);
        this.regionId = butler.getRegionId();
        this.accessKey = butler.getAccessKey();
        this.secretKey = butler.getSecretKey();
        this.butler = butler;
        this.credentials = AwsBasicCredentials.create(this.accessKey, this.secretKey);
    }

    public SdkClient getClient(AwsService awsService) {
        SdkHttpClient httpClient = null;
        if (!StringUtils.isEmpty(butler.getProxyUri())) {
            httpClient = ApacheHttpClient.builder()
                    .proxyConfiguration(ProxyConfiguration.builder().endpoint(URI.create(butler.getProxyUri())).build())
                    .build();
        }
        SdkClient sdkClient = null;
        switch (awsService) {
            case EC2:
                sdkClient = Ec2Client.builder().region(Region.of(regionId))
                        .credentialsProvider(StaticCredentialsProvider.create(this.credentials)).httpClient(httpClient)
                        .build();
                break;
            case STS:
                sdkClient = StsClient.builder().region(Region.of(regionId))
                        .credentialsProvider(StaticCredentialsProvider.create(this.credentials)).httpClient(httpClient)
                        .build();
                break;
            case ELB:
                sdkClient = ElasticLoadBalancingV2Client.builder().region(Region.of(regionId))
                        .credentialsProvider(StaticCredentialsProvider.create(this.credentials)).httpClient(httpClient)
                        .build();
                break;
            case CE:
                sdkClient = CostExplorerClient.builder().region(Region.CN_NORTHWEST_1)
                        .credentialsProvider(StaticCredentialsProvider.create(this.credentials)).httpClient(httpClient)
                        .build();
                break;
            case OPSWORKS:
                sdkClient = OpsWorksClient.builder().region(Region.of(regionId))
                        .credentialsProvider(StaticCredentialsProvider.create(this.credentials)).httpClient(httpClient)
                        .build();
                break;
            default:
                break;
        }
        return sdkClient;
    }

    @Override
    public boolean accessable() {
        boolean flag;
        try{
            Ec2Client client = (Ec2Client) this.getClient(AwsService.EC2);
            client.describeAvailabilityZones();
            flag = true;
        }catch (Exception e){
            logger.error("aws accessable error :", e);
            flag = false;
        }
        return flag;
    }

    public String getRegionId() {
        return regionId;
    }

    public String getAccessKey() {
        return accessKey;
    }

    public String getSecretKey() {
        return secretKey;
    }

    public AwsCredentials getCredentials() {
        return credentials;
    }

    public Butler getButler() {
        return butler;
    }
}