package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.awsconvertor.RegionConvertor;
import com.bocloud.cmp.awsconvertor.ZoneConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.compute.*;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.loadbalance.AwsLoadBalancerProvider;
import com.bocloud.cmp.provider.loadbalance.LoadBalanceProvider;
import com.bocloud.cmp.provider.network.*;
import com.bocloud.cmp.provider.security.AwsKeypairProvider;
import com.bocloud.cmp.provider.security.KeypairProvider;
import com.bocloud.cmp.provider.storage.AwsVolumeProvider;
import com.bocloud.cmp.provider.storage.VolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.AvailabilityZone;
import software.amazon.awssdk.services.ec2.model.DescribeAvailabilityZonesResponse;
import software.amazon.awssdk.services.ec2.model.DescribeRegionsResponse;
import software.amazon.awssdk.services.ec2.model.Region;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AwsLocationProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 14:44
 */
@Provider("bocloud.cmp.aws")
public class AwsLocationProvider extends AwsProvider implements LocationProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsLocationProvider.class);
    private final BeanConvertor<RegionModel, Region> regionConvertor = new RegionConvertor();
    private final BeanConvertor<ZoneModel, AvailabilityZone> zoneConvertor = new ZoneConvertor();

    private final Butler butler;
    private ImageProvider imageProvider = null;
    private ServerProvider serverProvider = null;
    private VpcProvider vpcProvider = null;
    private SubnetProvider subnetProvider = null;
    private SecurityGroupProvider securityGroupProvider = null;
    private KeypairProvider keypairProvider = null;
    private SnapshotProvider snapshotProvider = null;
    private VolumeProvider volumeProvider = null;
    private FlavorProvider flavorProvider = null;
    private NetworkInterfaceProvider networkInterfaceProvider = null;
    private ElasticIpProvider elasticIpProvider = null;
    private InternetGatewayProvider internetGatewayProvider = null;
    private TagProvider tagProvider = null;
    private CostProvider costProvider = null;
    private LoadBalanceProvider loadBalancerProvider = null;
    private TargetGroupProvider targetGroupProvider = null;
    private TargetProvider targetProvider = null;
    private ListenerProvider listenerProvider = null;
    private RouteTableProvider routeTableProvider = null;
    private NatGatewayProvider natGatewayProvider = null;

    public AwsLocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
        imageProvider = new AwsImageProvider(butler);
        serverProvider = new AwsServerProvider(butler);
        vpcProvider = new AwsVpcProvider(butler);
        subnetProvider = new AwsSubnetProvider(butler);
        securityGroupProvider = new AwsSecurityGroupProvider(butler);
        keypairProvider = new AwsKeypairProvider(butler);
        snapshotProvider = new AwsSnapshotsProvider(butler);
        volumeProvider = new AwsVolumeProvider(butler);
        flavorProvider = new AwsFlavorProvider(butler);
        networkInterfaceProvider = new AwsNetworkInterfaceProvider(butler);
        elasticIpProvider = new AwsElasticIpProvider(butler);
        internetGatewayProvider = new AwsInternetGatewayProvider(butler);
        tagProvider = new AwsTagProvider(butler);
        costProvider = new AwsCostProvider(butler);
        loadBalancerProvider = new AwsLoadBalancerProvider(butler);
        targetGroupProvider = new AwsTargetGroupProvider(butler);
        targetProvider = new AwsTargetProvider(butler);
        listenerProvider = new AwsListenerProvider(butler);
        routeTableProvider = new AwsRouteTableProvider(butler);
        natGatewayProvider = new AwsNatGatewayProvider(butler);
    }

    @Override
    public BocloudResult sync() {
        try {
            AwsSyncModel syncModel = new AwsSyncModel();

            List<RegionModel> regions = new ArrayList<>();
            List<FlavorModel> flavorModels = new ArrayList<>();
            List<KeypairModel> keypairModels = new ArrayList<>();
            List<VpcModel> vpcModels = new ArrayList<>();
            List<SecurityGroupModel> securityGroupModels = new ArrayList<>();
            List<VolumeModel> volumeModels = new ArrayList<>();
            List<ServerModel> serverModels = new ArrayList<>();
            List<SnapshotModel> snapshotModels = new ArrayList<>();
            List<SubnetModel> subnetModels = new ArrayList<>();
            List<NetworkInterfaceModel> networkInterfaceModels = new ArrayList<>();
            List<ElasticIpModel> elasticIpModels = new ArrayList<>();
            List<InternetGatewayModel> internetGatewayModels = new ArrayList<>();
            List<TagModel> tagModels = new ArrayList<>();
            List<CloudCostModel> cloudCostModels = new ArrayList<>();
            List<LbModel> lbModels = new ArrayList<>();
            List<LbTargetGroupModel> lbTargetGroupModels = new ArrayList<>();
            List<LbTargetModel> lbTargetModels = new ArrayList<>();
            List<LbListenerModel> lbListenerModels = new ArrayList<>();
            List<RouteTableModel> routeTableModels = new ArrayList<>();
            List<NatGatewayModel> natGatewayModels = new ArrayList<>();

            logger.info("{} - region sync -->Start", this.butler.getName());
            RegionModel regionModel = new RegionModel();
            regionModel.setCode(this.getRegionId());
            if (null != this.getRegionId() && !"".equals(this.getRegionId())) {
                BocloudResult listZones = this.listZones(this.getRegionId());
                if (listZones.isSuccess()) {
                    regionModel.setZones(JSONObject.parseArray(JSONObject.toJSONString(listZones.getData()), ZoneModel.class));
                } else {
                    regionModel.setZones(null);
                }
            }
            regions.add(regionModel);
            logger.info("{} - region sync -->End", this.butler.getName());
            logger.info("{} - flavor sync -->Start", this.butler.getName());
            BocloudResult flavorResult = flavorProvider.sync();
            if (flavorResult.isSuccess()) {
                flavorModels = JSONObject.parseArray(JSONObject.toJSONString(flavorResult.getData()),
                        FlavorModel.class);
            } else {
                flavorModels = null;
            }
            logger.info("{} - flavor sync -->End", this.butler.getName());
            logger.info("{} - networkInterface sync -->Start", this.butler.getName());
            BocloudResult networkInterfaceResult = networkInterfaceProvider.list();
            if (networkInterfaceResult.isSuccess()) {
                networkInterfaceModels = JSONObject.parseArray(
                        JSONObject.toJSONString(networkInterfaceResult.getData()), NetworkInterfaceModel.class);
            } else {
                networkInterfaceModels = null;
            }
            logger.info("{} - networkInterface sync -->End", this.butler.getName());
            logger.info("{} - nat gateway sync -->Start", this.butler.getName());
            BocloudResult natGatewayResult = natGatewayProvider.list();
            if (natGatewayResult.isSuccess()) {
                natGatewayModels = JSONObject.parseArray(
                        JSONObject.toJSONString(natGatewayResult.getData()), NatGatewayModel.class);
            } else {
                natGatewayModels = null;
            }
            logger.info("{} - nat gateway sync -->End", this.butler.getName());
            logger.info("{} - cloudServer sync -->Start", this.butler.getName());
            BocloudResult serverResult = serverProvider.list();
            Map<String, String> serverZoneMap = new HashMap(100);
            if (serverResult.isSuccess()) {
                serverModels = JSONObject.parseArray(JSONObject.toJSONString(serverResult.getData()),
                        ServerModel.class);
                if (!ListTool.isEmpty(serverModels)) {
                    for (ServerModel serverModel :
                            serverModels) {
                        serverZoneMap.put(serverModel.getInstanceId(), serverModel.getZone());
                    }
                }
            } else {
                serverModels = null;
            }
            logger.info("{} - cloudServer sync -->End", this.butler.getName());
            logger.info("{} - volume sync -->Start", this.butler.getName());
            BocloudResult volumeResult = volumeProvider.list();
            if (volumeResult.isSuccess()) {
                volumeModels = JSONArray.parseArray(JSONArray.toJSONString(volumeResult.getData()), VolumeModel.class);
            } else {
                volumeModels = null;
            }
            logger.info("{} - volume sync -->End", this.butler.getName());
            logger.info("{} - snapshot sync -->Start", this.butler.getName());
            BocloudResult snapshotsResult = snapshotProvider.list();
            if (snapshotsResult.isSuccess()) {
                snapshotModels = JSONObject.parseArray(JSONObject.toJSONString(snapshotsResult.getData()),
                        SnapshotModel.class);
            } else {
                snapshotModels = null;
            }
            logger.info("{} - snapshot sync -->End", this.butler.getName());
            logger.info("{} - vpc sync -->Start", this.butler.getName());
            BocloudResult vpcResult = vpcProvider.list();
            if (vpcResult.isSuccess()) {
                vpcModels = JSONArray.parseArray(JSONArray.toJSONString(vpcResult.getData()), VpcModel.class);
            } else {
                vpcModels = null;
            }
            logger.info("{} - vpc sync -->End", this.butler.getName());
            logger.info("{} - subnet sync -->Start", this.butler.getName());
            BocloudResult subnetResult = subnetProvider.list();
            if (subnetResult.isSuccess()) {
                subnetModels = JSONArray.parseArray(JSONArray.toJSONString(subnetResult.getData()), SubnetModel.class);
            } else {
                subnetModels = null;
            }
            logger.info("{} - subnet sync -->End", this.butler.getName());
            logger.info("{} - securityGroup sync -->Start", this.butler.getName());
            BocloudResult securityGroupResult = securityGroupProvider.list();
            if (securityGroupResult.isSuccess()) {
                securityGroupModels = JSONArray.parseArray(JSONArray.toJSONString(securityGroupResult.getData()),
                        SecurityGroupModel.class);
            } else {
                securityGroupModels = null;
            }
            logger.info("{} - securityGroup sync -->End", this.butler.getName());
            logger.info("{} - keypair sync -->Start", this.butler.getName());
            BocloudResult keypairResult = keypairProvider.list();
            if (keypairResult.isSuccess()) {
                keypairModels = JSONObject.parseArray(JSONObject.toJSONString(keypairResult.getData()),
                        KeypairModel.class);
            } else {
                keypairModels = null;
            }
            logger.info("{} - keypair sync -->End", this.butler.getName());
            logger.info("{} - lb sync -->Start", this.butler.getName());
            BocloudResult lbResult = loadBalancerProvider.list();
            if (lbResult.isSuccess()) {
                lbModels = JSONObject.parseArray(JSONObject.toJSONString(lbResult.getData()),
                        LbModel.class);
            } else {
                lbModels = null;
            }
            logger.info("{} - lb sync -->End", this.butler.getName());
            if (!ListTool.isEmpty(lbModels)) {
                logger.info("{} - Listener sync -->Start", this.butler.getName());
                for (LbModel lbModel :
                        lbModels) {
                    BocloudResult listenrResult = listenerProvider.list(lbModel.getLbUuid());
                    if (listenrResult.isSuccess()) {
                        lbListenerModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(listenrResult.getData()), LbListenerModel.class));
                    } else {
                        lbListenerModels = null;
                    }
                }
                logger.info("{} - Listener sync -->End", this.butler.getName());
            }
            logger.info("{} - target group sync -->Start", this.butler.getName());
            BocloudResult tgResult = targetGroupProvider.list();
            if (tgResult.isSuccess()) {
                lbTargetGroupModels = JSONObject.parseArray(JSONObject.toJSONString(tgResult.getData()),
                        LbTargetGroupModel.class);
            } else {
                lbTargetGroupModels = null;
            }
            logger.info("{} - target group sync -->End", this.butler.getName());

            if (!ListTool.isEmpty(lbTargetGroupModels)) {
                logger.info("{} - target sync -->Start", this.butler.getName());
                for (LbTargetGroupModel groupModel :
                        lbTargetGroupModels) {
                    BocloudResult tResult = targetProvider.list(groupModel, serverZoneMap);
                    if (tResult.isSuccess()) {
                        lbTargetModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(tResult.getData()),
                                LbTargetModel.class));
                    } else {
                        lbTargetModels = null;
                    }
                }
                logger.info("{} - target sync -->End", this.butler.getName());
            }
            logger.info("{} - Route tables sync -->Start", this.butler.getName());
            BocloudResult awsRouteTableResult = routeTableProvider.list();
            if (awsRouteTableResult.isSuccess()) {
                routeTableModels = JSONObject.parseArray(JSONObject.toJSONString(awsRouteTableResult.getData()),
                        RouteTableModel.class);
            } else {
                routeTableModels = null;
            }
            logger.info("{} - Route tables sync -->End", this.butler.getName());
            logger.info("{} - elasticIp sync -->Start", this.butler.getName());
            BocloudResult elasticIpResult = elasticIpProvider.list();
            if (elasticIpResult.isSuccess()) {
                elasticIpModels = JSONObject.parseArray(JSONObject.toJSONString(elasticIpResult.getData()),
                        ElasticIpModel.class);
            } else {
                elasticIpModels = null;
            }
            logger.info("{} - elasticIp sync -->End", this.butler.getName());
            logger.info("{} - InternetGateway sync -->Start", this.butler.getName());
            BocloudResult internetGatewayResult = internetGatewayProvider.list();
            if (internetGatewayResult.isSuccess()) {
                internetGatewayModels = JSONObject.parseArray(JSONObject.toJSONString(internetGatewayResult.getData()),
                        InternetGatewayModel.class);
            } else {
                internetGatewayModels = null;
            }
            logger.info("{} - InternetGateway sync -->End", this.butler.getName());
            logger.info("{} - Tag sync -->Start", this.butler.getName());
            BocloudResult tagResult = tagProvider.list();
            if (tagResult.isSuccess()) {
                tagModels = JSONObject.parseArray(JSONObject.toJSONString(tagResult.getData()), TagModel.class);
            } else {
                tagModels = null;
            }
            logger.info("{} - Tag sync -->End", this.butler.getName());

            syncModel.setNetworkInterfaceModels(networkInterfaceModels);
            syncModel.setFlavorModels(flavorModels);
            syncModel.setRegionModels(regions);
            syncModel.setServerModels(serverModels);
            syncModel.setVolumeModels(volumeModels);
            syncModel.setSnapshotModels(snapshotModels);
            syncModel.setVpcModels(vpcModels);
            syncModel.setSubnetModels(subnetModels);
            syncModel.setSecurityGroupModels(securityGroupModels);
            syncModel.setKeypairModels(keypairModels);
            syncModel.setElasticIpModels(elasticIpModels);
            syncModel.setInternetGatewayModels(internetGatewayModels);
            syncModel.setTagModels(tagModels);
            syncModel.setCloudCostModels(cloudCostModels);
            syncModel.setLbModels(lbModels);
            syncModel.setLbTargetGroupModels(lbTargetGroupModels);
            syncModel.setLbTargetModels(lbTargetModels);
            syncModel.setLbListenerModels(lbListenerModels);
            syncModel.setRouteTableModels(routeTableModels);
            syncModel.setNatGatewayModels(natGatewayModels);
            return new BocloudResult(true, syncModel, "success");
        } catch (Exception e) {
            logger.error("Aws sync failure :", e);
            return new BocloudResult(false, "Aws sync failure :" + e.getMessage());
        }
    }

    @Override
    public BocloudResult listRegions() {
        try {
            DescribeRegionsResponse describeRegionsResponse = ((Ec2Client) this.getClient(AwsService.EC2))
                    .describeRegions();
            List<RegionModel> regionModels = regionConvertor.convertList(describeRegionsResponse.regions());
            return new BocloudResult(true, regionModels, "list region success!");
        } catch (Exception e) {
            logger.error("list region failure :", e);
            return new BocloudResult(false, "list region failure :" + e.getMessage());
        }
    }

    @Override
    public BocloudResult listZones(String region) {
        try {
            DescribeAvailabilityZonesResponse describeAvailabilityZonesResponse = ((Ec2Client) this
                    .getClient(AwsService.EC2)).describeAvailabilityZones();
            List<ZoneModel> zoneModels = zoneConvertor
                    .convertList(describeAvailabilityZonesResponse.availabilityZones());
            return new BocloudResult(true, zoneModels, "list zone success!");
        } catch (Exception e) {
            logger.error("list zone failure :", e);
            return new BocloudResult(false, "list zone failure :" + e.getMessage());
        }
    }

    @Override
    public BocloudResult syncAwsCost() {
        try {
            logger.info("{} - AWS Cloud Cost sync -->Start", this.butler.getName());

            AwsSyncModel syncModel = new AwsSyncModel();
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            Date endTime = new Date();
            Date startTime = new Date(endTime.getTime() - 86400000);
            String start = format.format(startTime);
            String end = format.format(endTime);
            BocloudResult costResult = costProvider.listByResource("DAILY", "UNBLENDED_COST", start, end);
            if (costResult.isFailed()) {
                return new BocloudResult(false, "sync awscost failure :" + costResult.getMessage());
            }
            List<CloudCostModel> cloudCostModels = JSONObject.parseArray(JSONObject.toJSONString(costResult.getData()),
                    CloudCostModel.class);
            syncModel.setCloudCostModels(cloudCostModels);

            logger.info("{} - AWS Cloud Cost sync -->End", this.butler.getName());
            return new BocloudResult(true, syncModel, "success!");
        } catch (Exception e) {
            logger.error("sync awscost failure :", e);
            return new BocloudResult(false, "sync awscost failure :" + e.getMessage());
        }
    }

    @Override
    public BocloudResult syncAwsLoadBalancer() {
        try {
            AwsSyncModel syncModel = new AwsSyncModel();

            List<VpcModel> vpcModels = new ArrayList<>();
            List<LbModel> lbModels = new ArrayList<>();
            List<LbTargetGroupModel> lbTargetGroupModels = new ArrayList<>();
            List<LbTargetModel> lbTargetModels = new ArrayList<>();
            List<LbListenerModel> lbListenerModels = new ArrayList<>();

            logger.info("{} - cloudServer sync -->Start", this.butler.getName());
            BocloudResult serverResult = serverProvider.list();
            Map<String, String> serverZoneMap = new HashMap(100);
            if (serverResult.isSuccess()) {
                List<ServerModel> serverModels = JSONObject.parseArray(JSONObject.toJSONString(serverResult.getData()),
                        ServerModel.class);
                if (!ListTool.isEmpty(serverModels)) {
                    for (ServerModel serverModel :
                            serverModels) {
                        serverZoneMap.put(serverModel.getInstanceId(), serverModel.getZone());
                    }
                }
            }
            logger.info("{} - cloudServer sync -->End", this.butler.getName());

            logger.info("{} - vpc sync -->Start", this.butler.getName());
            BocloudResult vpcResult = vpcProvider.list();
            if (vpcResult.isSuccess()) {
                vpcModels = JSONArray.parseArray(JSONArray.toJSONString(vpcResult.getData()), VpcModel.class);
            } else {
                vpcModels = null;
            }
            logger.info("{} - vpc sync -->End", this.butler.getName());
            logger.info("{} - lb sync -->Start", this.butler.getName());
            BocloudResult lbResult = loadBalancerProvider.list();
            if (lbResult.isSuccess()) {
                lbModels = JSONObject.parseArray(JSONObject.toJSONString(lbResult.getData()),
                        LbModel.class);
            } else {
                lbModels = null;
            }
            logger.info("{} - lb sync -->End", this.butler.getName());
            if (!ListTool.isEmpty(lbModels)) {
                logger.info("{} - Listener sync -->Start", this.butler.getName());
                for (LbModel lbModel :
                        lbModels) {
                    BocloudResult listenrResult = listenerProvider.list(lbModel.getLbUuid());
                    if (listenrResult.isSuccess()) {
                        lbListenerModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(listenrResult.getData()), LbListenerModel.class));
                    } else {
                        lbListenerModels = null;
                    }
                }
                logger.info("{} - Listener sync -->End", this.butler.getName());
            }
            logger.info("{} - target group sync -->Start", this.butler.getName());
            BocloudResult tgResult = targetGroupProvider.list();
            if (tgResult.isSuccess()) {
                lbTargetGroupModels = JSONObject.parseArray(JSONObject.toJSONString(tgResult.getData()),
                        LbTargetGroupModel.class);
            } else {
                lbTargetGroupModels = null;
            }
            logger.info("{} - target group sync -->End", this.butler.getName());

            if (!ListTool.isEmpty(lbTargetGroupModels)) {
                logger.info("{} - target sync -->Start", this.butler.getName());
                for (LbTargetGroupModel groupModel :
                        lbTargetGroupModels) {
                    BocloudResult tResult = targetProvider.list(groupModel, serverZoneMap);
                    if (tResult.isSuccess()) {
                        lbTargetModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(tResult.getData()),
                                LbTargetModel.class));
                    } else {
                        lbTargetModels = null;
                    }
                }
                logger.info("{} - target sync -->End", this.butler.getName());
            }
            syncModel.setVpcModels(vpcModels);
            syncModel.setLbModels(lbModels);
            syncModel.setLbTargetGroupModels(lbTargetGroupModels);
            syncModel.setLbTargetModels(lbTargetModels);
            syncModel.setLbListenerModels(lbListenerModels);
            return new BocloudResult(true, syncModel, "success");
        } catch (Exception e) {
            logger.error("Aws sync failure :", e);
            return new BocloudResult(false, "Aws sync failure :" + e.getMessage());
        }
    }

    @Override
    public BocloudResult syncAwsImages() {
        try {
            AwsSyncModel syncModel = new AwsSyncModel();
            List<ImageModel> imageModels = new ArrayList<>();
            logger.info("{} - image sync-->Start", this.butler.getName());
            BocloudResult imageResult = imageProvider.list();
            if (imageResult.isSuccess()) {
                imageModels = JSONArray.parseArray(JSONArray.toJSONString(imageResult.getData()), ImageModel.class);
            }
            logger.info("{} - image sync-->End", this.butler.getName());
            syncModel.setImageModels(imageModels);
            return new BocloudResult(true, syncModel, "success!");
        } catch (Exception e) {
            logger.error("list image failure :", e);
            return new BocloudResult(false, "list image failure :" + e.getMessage());
        }
    }

    @Override
    public BocloudResult syncPortByNetworkUuid(String networkUuid) {
        return null;
    }

    @Override
    public BocloudResult getHostModel(HostModel host) {
        return null;
    }
}
