package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.awsconvertor.ImageConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.ObjectUtils;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AwsImageProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 14:53
 */
@Provider("bocloud.cmp.aws")
public class AwsImageProvider extends AwsProvider implements ImageProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsImageProvider.class);

    private final BeanConvertor<ImageModel, Image> convertor = new ImageConvertor();

    public AwsImageProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeImagesResponse describeImagesResponse = ((Ec2Client) this.getClient(AwsService.EC2)).describeImages();
            List<ImageModel> imageModels = convertor.convertList(describeImagesResponse.images());
            return new BocloudResult(true, imageModels, "list image success!");
        } catch (Exception e) {
            logger.error("list image error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String imageId) {
        try {
            DescribeImagesRequest request = DescribeImagesRequest.builder().imageIds(imageId).build();
            DescribeImagesResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeImages(request);
            List<Image> images = response.images();
			if (ListTool.isEmpty(images)) {
				return new BocloudResult(false, new ImageModel(), "镜像不存在");
			}
            ImageModel imageModel = convertor.convertModel(response.images().get(0));
            imageModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, imageModel, "detail image success!");
        } catch (Exception e) {
            logger.error("detail image error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String imageId) {
        try {
            DeregisterImageRequest request = DeregisterImageRequest.builder()
                    .imageId(imageId)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).deregisterImage(request);
            return new BocloudResult(true, "remove image success!");
        } catch (Exception e) {
            logger.error("remove image error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(ImageModel image) {
        try {
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(image.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(image.getImageUuid())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            ModifyImageAttributeRequest request = ModifyImageAttributeRequest.builder()
                    .imageId(image.getImageUuid())
                    .description(AttributeValue.builder().value(image.getRemark()).build())
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).modifyImageAttribute(request);
            return this.detail(image.getImageUuid());
        } catch (Exception e) {
            logger.error("update image error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(ImageModel imageModel) {
        try {
            CreateImageRequest request = CreateImageRequest.builder()
                    .description(imageModel.getRemark())
                    .instanceId(imageModel.getInstanceId())
                    .name(imageModel.getProps())
                    .noReboot(imageModel.getIsNoRestart())
                    .build();
            CreateImageResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).createImage(request);
            String imageId = response.imageId();
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(imageModel.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(imageId)
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return getImages(response.imageId());
        } catch (Exception e) {
            logger.error("create image error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    private BocloudResult getImages(String imageId) {
        BocloudResult result = new BocloudResult(false, "data does not exist");
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            while (endTime - startTime < 10 * 60 * 1000) {
                //创建时间不确定，增加休眠减少轮询次数
                Thread.sleep(6000);
                BocloudResult detail = this.detail(imageId);
                ImageModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), ImageModel.class);
                if (ObjectUtils.isEmpty(model)) {
                    logger.error("data does not exist :", imageId);
                    break;
                }
                if (("active".equalsIgnoreCase(model.getStatus()))) {
                    result = this.detail(imageId);
                    break;
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error("create image get status error", e);
            result = new BocloudResult(false, "create image get status error： " + e.getMessage());
        }
        return result;
    }

    @Override
    public BocloudResult copy(ImageModel imageModel) {
        try {
            CopyImageRequest request = CopyImageRequest.builder().description(imageModel.getRemark()).name(imageModel.getProps()).sourceImageId(imageModel.getSourceImageId()).sourceRegion(imageModel.getSourceRegion()).build();
            CopyImageResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).copyImage(request);
            String imageId = response.imageId();
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(imageModel.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(imageId)
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return getImages(imageId);
        } catch (Exception e) {
            logger.error("create image error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
