package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.awsconvertor.ServerConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AwsServerProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 14:41
 */
@Provider("bocloud.cmp.aws")
public class AwsServerProvider extends AwsProvider implements ServerProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsServerProvider.class);

    private final BeanConvertor<ServerModel, Instance> convertor = new ServerConvertor();

    public AwsServerProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            Ec2Client ec2Client = (Ec2Client) this.getClient(AwsService.EC2);
            DescribeInstancesResponse describeInstancesResponse = ec2Client.describeInstances();
            List<Reservation> reservations = describeInstancesResponse.reservations();
            List<ServerModel> serverModels = new ArrayList<>();
            for (Reservation reservation : reservations) {
                serverModels.addAll(convertor.convertList(reservation.instances()));
            }
            for (ServerModel model : serverModels) {
                Boolean disableApiTermination = ec2Client
                        .describeInstanceAttribute(
                                DescribeInstanceAttributeRequest.builder().instanceId(model.getInstanceId())
                                        .attribute(InstanceAttributeName.DISABLE_API_TERMINATION).build())
                        .disableApiTermination().value();
                model.setTerminationProtected(disableApiTermination);

                model.setOsName(((ImageModel) new AwsImageProvider(getButler()).detail(model.getImageUuid()).getData()).getName());
            }
            return new BocloudResult(true, serverModels, "list cloudServer success!");
        } catch (Exception e) {
            logger.error("list cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult list(String hostId) {
        return null;
    }

    @Override
    public BocloudResult create(ServerModel server) {
        try {
            // 获取磁盘
            List<BlockDeviceMapping> blockDeviceMappings = new ArrayList<>();
            List<VolumeModel> diskDevices = server.getDiskDevices();
            for (VolumeModel volumeModel : diskDevices) {
                EbsBlockDevice ebsBlockDevice = EbsBlockDevice.builder()
                        .deleteOnTermination(volumeModel.getDeleteWithInstance()).encrypted(false)
                        .volumeSize(volumeModel.getSize()).volumeType(VolumeType.fromValue(volumeModel.getCategory()))
                        .build();
                BlockDeviceMapping blockDeviceMapping = BlockDeviceMapping.builder().deviceName(volumeModel.getDevice())
                        .ebs(ebsBlockDevice).build();
                blockDeviceMappings.add(blockDeviceMapping);
            }
            // 获取网络接口
            List<InstanceNetworkInterfaceSpecification> networkInterfaces = new ArrayList<>();
            List<NetworkInterfaceModel> models = server.getNetworkInterfaces();
            if (!CollectionUtils.isEmpty(models)) {
                for (NetworkInterfaceModel model : models) {
                    networkInterfaces.add(InstanceNetworkInterfaceSpecification.builder()
                            .networkInterfaceId(model.getNetworkInterfaceUuid()).deviceIndex(model.getDeviceIndex())
                            .build());
                }
            }

            // 采用网络接口创建还是采用子网方式创建
            RunInstancesRequest.Builder builder = RunInstancesRequest.builder().imageId(server.getImageUuid())// 镜像
                    .instanceType(InstanceType.fromValue(server.getFlavorUuid()))// 规格
                    .blockDeviceMappings(blockDeviceMappings)// 系统盘
                    .maxCount(1).minCount(1);
            if (!StringUtils.isEmpty(server.getKeypairName())) {
                builder.keyName(server.getKeypairName()); // 密钥
            }
            if (!StringUtils.isEmpty(server.getManagerIp())) {
                builder.privateIpAddress(server.getManagerIp()); // 子网IP
            }
            if (null != server.getTerminationProtected() || server.getTerminationProtected()) {
                builder.disableApiTermination(server.getTerminationProtected()); // 终止保护
            }

            if (!CollectionUtils.isEmpty(networkInterfaces)) {
                // 网络接口 -不指定强类型的子网和实例安全组
                builder.networkInterfaces(networkInterfaces);
            } else {
                // 子网
                builder.securityGroupIds(server.getSecurityGroups())// 安全组
                        .subnetId(server.getSubnetUuid());
            }
            RunInstancesRequest request = builder.build();

            RunInstancesResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).runInstances(request);
            String instanceId = response.instances().get(0).instanceId();
            Tag tag = Tag.builder().key("Name").value(server.getName()).build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder().resources(instanceId).tags(tag).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return getInstances(instanceId);
        } catch (Exception e) {
            logger.error("create cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    public BocloudResult getInstances(String instanceId) {
        BocloudResult result = new BocloudResult(false, "data does not exist");
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            // 轮询10m
            while (endTime - startTime < 10 * 60 * 1000) {
                // 主机创建时间不确定，增加休眠减少轮询次数
                Thread.sleep(6000);
                BocloudResult detail = this.detail(instanceId);
                ServerModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()),
                        ServerModel.class);
                if (ObjectUtils.isEmpty(model)) {
                    logger.error("data does not exist :", instanceId);
                    break;
                }
                if (("RUNNING".equalsIgnoreCase(model.getStatus()))) {
                    result = this.detail(instanceId);
                    break;
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error("create server get status error：", e);
            result = new BocloudResult(false, "create server get status error： " + e.getMessage());
        }
        return result;
    }

    @Override
    public BocloudResult modify(ServerModel server) {
        try {
            Tag tag = Tag.builder().key("Name").value(server.getName()).build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder().resources(server.getInstanceId()).tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return new BocloudResult(true, this.detail(server.getInstanceId()).getData(), "编辑成功！");
        } catch (Exception e) {
            logger.error("modify instance error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modifyVmName(String instanceId, String newName) {
        try {
            Tag tag = Tag.builder().key("Name").value(newName).build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder().resources(instanceId).tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return new BocloudResult(true, this.detail(instanceId).getData(), "修改名称成功！");
        } catch (Exception e) {
            logger.error("modify vm name error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult assignIpv6(String networkInterfaceId, String ipv6) {
        try {
            AssignIpv6AddressesRequest request = AssignIpv6AddressesRequest.builder().ipv6Addresses(ipv6)
                    .networkInterfaceId(networkInterfaceId).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).assignIpv6Addresses(request);
            return new BocloudResult(true, "分配地址成功！");
        } catch (Exception e) {
            logger.error("assignIpv6 error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult migrate(ServerModel server) {
        return null;
    }

    @Override
    public BocloudResult migrate(OSMigrateModel model) {
        return null;
    }

    @Override
    public BocloudResult remove(String instanceId, String name) {
        try {
            // 终止实例 短时间在aws控制台能看见实例处于终止状态，约一小时实例会自动删除
            TerminateInstancesRequest request = TerminateInstancesRequest.builder().instanceIds(instanceId).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).terminateInstances(request);
            return new BocloudResult(true, "remove cloudServer success!");
        } catch (Exception e) {
            logger.error("remove cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String instanceId) {
        try {
            DescribeInstancesRequest request = DescribeInstancesRequest.builder().instanceIds(instanceId).build();
            DescribeInstancesResponse response = ((Ec2Client) this.getClient(AwsService.EC2))
                    .describeInstances(request);
            ServerModel model = convertor.convertModel(response.reservations().get(0).instances().get(0));
            model.setOsName(((ImageModel) new AwsImageProvider(getButler()).detail(model.getImageUuid()).getData()).getName());
            model.setRegionId(this.getRegionId());
            return new BocloudResult(true, model, "detail cloudServer success!");
        } catch (Exception e) {
            logger.error("detail cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult disableApiTermination(String instanceId, Boolean enabled) {
        try {
            ((Ec2Client) this.getClient(AwsService.EC2))
                    .modifyInstanceAttribute(ModifyInstanceAttributeRequest.builder().instanceId(instanceId)
                            .disableApiTermination(AttributeBooleanValue.builder().value(enabled).build()).build());
            return new BocloudResult(true, "disable ApiTermination success!");
        } catch (Exception e) {
            logger.error("disable ApiTermination error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult start(String instanceId) {
        try {
            StartInstancesRequest request = StartInstancesRequest.builder().instanceIds(instanceId).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).startInstances(request);
            return new BocloudResult(true, "start cloudServer success!");
        } catch (Exception e) {
            logger.error("start cloudServer error : ", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult stop(String instanceId) {
        try {
            StopInstancesRequest request = StopInstancesRequest.builder().instanceIds(instanceId).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).stopInstances(request);
            return new BocloudResult(true, "stop cloudServer success!");
        } catch (Exception e) {
            logger.error("stop cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }

    @Override
    public BocloudResult reboot(String instanceId) {
        try {
            RebootInstancesRequest request = RebootInstancesRequest.builder().instanceIds(instanceId).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).rebootInstances(request);
            return new BocloudResult(true, "reboot cloudServer success!");
        } catch (Exception e) {
            logger.error("reboot cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult softReboot(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult pause(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult unPause(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult suspend(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult resume(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult joinSGroup(String instanceId, String group) {
        try {
            Ec2Client ec2Client = (Ec2Client) this.getClient(AwsService.EC2);
            DescribeNetworkInterfacesRequest describeNetworkInterfacesRequest = DescribeNetworkInterfacesRequest.builder()
                    .filters(Filter.builder().name("attachment.instance-id").values(instanceId).build())
                    .build();
            List<NetworkInterface> networkInterfaces = ec2Client.describeNetworkInterfaces(describeNetworkInterfacesRequest).networkInterfaces();
            for (NetworkInterface networkInterface :
                    networkInterfaces) {
                List<String> groups = new ArrayList<>();
                groups.add(group);
                for (GroupIdentifier groupIdentifier :
                        networkInterface.groups()) {
                    groups.add(groupIdentifier.groupId());
                }

                ModifyNetworkInterfaceAttributeRequest request = ModifyNetworkInterfaceAttributeRequest.builder()
                        .groups(groups)
                        .networkInterfaceId(networkInterface.networkInterfaceId())
                        .build();
                ec2Client.modifyNetworkInterfaceAttribute(request);
            }
            return new BocloudResult(true, "join security group success!");
        } catch (Exception e) {
            logger.error("join security group error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult allocationSGroups(String instanceId, List<String> groups) {
        try {
            // 此功能暂时不用 因为当aws实例绑定多个网络接口时
            // 官网默认把安全组分配给主网络接口（挂载点为0）
            // sdk未提供指定的网络接口字段，当默认一个网络接口时请求成功 ，两个网络接口报错
            // 需调研！
            ModifyInstanceAttributeRequest request = ModifyInstanceAttributeRequest.builder().instanceId(instanceId)
                    .groups(groups).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).modifyInstanceAttribute(request);
            return new BocloudResult(true, "allocation securityGroup success!");
        } catch (Exception e) {
            logger.error("allocation securityGroup error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult leaveSGroup(String instanceId, String group) {
        try {
            Ec2Client ec2Client = (Ec2Client) this.getClient(AwsService.EC2);
            DescribeNetworkInterfacesRequest describeNetworkInterfacesRequest = DescribeNetworkInterfacesRequest.builder()
                    .filters(Filter.builder().name("attachment.instance-id").values(instanceId).build())
                    .build();
            List<NetworkInterface> networkInterfaces = ec2Client.describeNetworkInterfaces(describeNetworkInterfacesRequest).networkInterfaces();
            for (NetworkInterface networkInterface :
                    networkInterfaces) {
                List<String> groups = new ArrayList<>();
                for (GroupIdentifier groupIdentifier :
                        networkInterface.groups()) {
                    groups.add(groupIdentifier.groupId());
                }

                if (groups.contains(group)) {
                    groups.remove(group);
                    ModifyNetworkInterfaceAttributeRequest request = ModifyNetworkInterfaceAttributeRequest.builder()
                            .groups(groups)
                            .networkInterfaceId(networkInterface.networkInterfaceId())
                            .build();
                    ec2Client.modifyNetworkInterfaceAttribute(request);
                }
            }

            return new BocloudResult(true, "leave security group success!");
        } catch (Exception e) {
            logger.error("leave security group error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        return null;
    }

    @Override
    public BocloudResult vncConsole(String vncType, String serverId) {
        return null;
    }

    @Override
    public BocloudResult changePassword(String instanceId, String password) {
        return null;
    }

    @Override
    public BocloudResult attachNetwork(ServerModel server) {
        return null;
    }

    @Override
    public BocloudResult detachNetwork(String instanceId, String portUuid) {
        return null;
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId) {
        try {
            AttributeValue attributeValue = AttributeValue.builder().value(flavorId).build();
            ModifyInstanceAttributeRequest request = ModifyInstanceAttributeRequest.builder().instanceId(instanceId)
                    .instanceType(attributeValue).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).modifyInstanceAttribute(request);
            return new BocloudResult(true, this.detail(instanceId).getData(), "resize cloudServer success!");
        } catch (Exception e) {
            logger.error("resize cloudServer error ", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult confirmResize(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult revertResize(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean MemHotEnable) {
        return null;
    }

    @Override
    public BocloudResult getTicket(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult createInResPool(ServerModel server) {
        return null;
    }

    @Override
    public BocloudResult mountToolsInstaller(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult unmountToolsInstaller(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult upgradeToolsTask(String instanceId, String options) {
        return null;
    }

    @Override
    public BocloudResult getNetCard(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult getVmIps(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult markAsTemplate(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
        return null;
    }

    @Override
    public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
        return null;
    }

    @Override
    public BocloudResult getMetric(String type) {
        return null;
    }

    @Override
    public BocloudResult vmrcConsole(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult getJnlpXml(Long vmId, String address) {
        return null;
    }

    @Override
    public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
        return null;
    }

    @Override
    public BocloudResult resetInstance(ServerModel serverModel) {
        return null;
    }

    @Override
    public BocloudResult bindKeypair(ServerModel serverModel) {
        return null;
    }

    @Override
    public BocloudResult unbindKeypair(ServerModel serverModel) {
        return null;
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
        return null;
    }

    @Override
    public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
        return null;
    }
}
