package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.awsconvertor.SnapshotConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.ObjectUtils;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;
import software.amazon.awssdk.services.sts.StsClient;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AwsSnapshotsProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 15:22
 */
@Provider("bocloud.cmp.aws")
public class AwsSnapshotsProvider extends AwsProvider implements SnapshotProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsSnapshotsProvider.class);

    private final BeanConvertor<SnapshotModel, Snapshot> convertor = new SnapshotConvertor();

    public AwsSnapshotsProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            String account = ((StsClient) this.getClient(AwsService.STS)).getCallerIdentity().account();
            DescribeSnapshotsRequest request = DescribeSnapshotsRequest.builder().ownerIds(account).build();
            DescribeSnapshotsResponse describeSnapshotsResponse = ((Ec2Client) this.getClient(AwsService.EC2))
                    .describeSnapshots(request);
            List<SnapshotModel> snapshotModels = convertor.convertList(describeSnapshotsResponse.snapshots());
            return new BocloudResult(true, snapshotModels, "list snapshot success!");
        } catch (Exception e) {
            logger.error("list snapshot error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        try {
            DeleteSnapshotRequest request = DeleteSnapshotRequest.builder().snapshotId(model.getSnapshotUuid()).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).deleteSnapshot(request);
            return new BocloudResult(true, "remove snapshot success!");
        } catch (Exception e) {
            logger.error("remove snapshot error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(SnapshotModel model) {
        try {
            DescribeSnapshotsRequest request = DescribeSnapshotsRequest.builder().snapshotIds(model.getSnapshotUuid())
                    .build();
            DescribeSnapshotsResponse response = ((Ec2Client) this.getClient(AwsService.EC2))
                    .describeSnapshots(request);
            SnapshotModel snapshotModel = convertor.convertModel(response.snapshots().get(0));
            snapshotModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, snapshotModel, "detail snapshot success!");
        } catch (Exception e) {
            logger.error("detail snapshot error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(SnapshotModel model) {
        try {
            Tag tag = Tag.builder().key("Name").value(model.getName()).build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(model.getSnapshotUuid())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return new BocloudResult(true, "modify snapshot success!");
        } catch (Exception e) {
            logger.error("modify snapshot error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        return null;
    }

    @Override
    public BocloudResult removeByVolumeId(String volumeId) {
        return null;
    }

    @Override
    public BocloudResult getSnapshot(String snapshotId) {
        BocloudResult result = new BocloudResult(false, "data does not exist");
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            SnapshotModel snapshotModel = new SnapshotModel();
            snapshotModel.setSnapshotUuid(snapshotId);
            while (endTime - startTime < 5 * 60 * 1000) {
                //创建时间不确定，增加休眠减少轮询次数
                Thread.sleep(5000);
                BocloudResult detail = this.detail(snapshotModel);
                SnapshotModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), SnapshotModel.class);
                if (ObjectUtils.isEmpty(model)) {
                    logger.error("data does not exist :", snapshotId);
                    break;
                }
                if ("AVAILABLE".equalsIgnoreCase(model.getStatus())) {
                    result = this.detail(snapshotModel);
                    break;
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error("create snapshot get status error：", e);
            result = new BocloudResult(false, "create snapshot get status error： " + e.getMessage());
        }
        return result;
    }

    @Override
    public BocloudResult getAvaliableSnap(String snapshotId) {
        return null;
    }

    @Override
    public BocloudResult list(Long cloudServerId) {
        return null;
    }
}
