package com.bocloud.cmp.provider.loadbalance;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.awsconvertor.LoadBalancerConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.LbModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.elasticloadbalancingv2.ElasticLoadBalancingV2Client;
import software.amazon.awssdk.services.elasticloadbalancingv2.model.*;

import java.util.ArrayList;
import java.util.List;

@Provider("bocloud.cmp.aws")
public class AwsLoadBalancerProvider extends AwsProvider implements LoadBalanceProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsLoadBalancerProvider.class);
    private final BeanConvertor<LbModel, LoadBalancer> convertor = new LoadBalancerConvertor();

    public AwsLoadBalancerProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeLoadBalancersResponse response = ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).describeLoadBalancers();
            List<LbModel> models = convertor.convertList(response.loadBalancers());
            return new BocloudResult(true, models, "list load balancers success!");
        } catch (Exception e) {
            logger.error("list load balancers error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(LbModel model) {
        try {
            CreateLoadBalancerRequest.Builder builder = CreateLoadBalancerRequest.builder()
                    .type(model.getLoadBalancerType())
                    .name(model.getName())
                    .scheme(model.getScheme())
                    .ipAddressType(model.getIpAddressType());
            if (model.getAvailabilityZones() != null && !model.getAvailabilityZones().isEmpty()) {
                List subnetList = new ArrayList();
                for (Object object :
                        model.getAvailabilityZones()) {
                    AvailabilityZone zone = JSONObject.parseObject(object.toString(), AvailabilityZone.serializableBuilderClass()).build();
                    SubnetMapping.Builder subnetBuilder = SubnetMapping.builder().subnetId(zone.subnetId());
                    if (!ListTool.isEmpty(zone.loadBalancerAddresses())) {
                        for (LoadBalancerAddress address :
                                zone.loadBalancerAddresses()) {
                            if (StringUtils.isNotEmpty(address.allocationId())) {
                                subnetBuilder.allocationId(address.allocationId());
                            }
                            if (StringUtils.isNotEmpty(address.privateIPv4Address())) {
                                subnetBuilder.privateIPv4Address(address.privateIPv4Address());
                            }
                            if (StringUtils.isNotEmpty(address.iPv6Address())) {
                                subnetBuilder.iPv6Address(address.iPv6Address());
                            }
                        }
                    }
                    subnetList.add(subnetBuilder.build());
                }
                builder.subnetMappings(subnetList);
            }
            if (LoadBalancerTypeEnum.APPLICATION.name().equals(model.getLoadBalancerType())) {
                builder.securityGroups(model.getSecurityGroups());
            }

            CreateLoadBalancerRequest request = builder.build();
            CreateLoadBalancerResponse response = ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).createLoadBalancer(request);
            return new BocloudResult(true, convertor.convertModel(response.loadBalancers().get(0)), "create load balancer success!");
        } catch (Exception e) {
            logger.error("create balancer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String loadBalancerArn) {
        try {
            DeleteLoadBalancerRequest request = DeleteLoadBalancerRequest.builder().loadBalancerArn(loadBalancerArn).build();
            ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).deleteLoadBalancer(request);
            return new BocloudResult(true, "delete load balancer success!");
        } catch (Exception e) {
            logger.error("delete load balancer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String lbId) {
        try {
            DescribeLoadBalancersRequest request = DescribeLoadBalancersRequest.builder().loadBalancerArns(lbId).build();
            DescribeLoadBalancersResponse response = ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).describeLoadBalancers(request);
            LbModel model = convertor.convertModel(response.loadBalancers().get(0));
            return new BocloudResult(true, model, "detail load balancers success!");
        } catch (Exception e) {
            logger.error("detail load balancers error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
