package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.awsconvertor.ElasticIpConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ElasticIpModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.List;

@Provider("bocloud.cmp.aws")
public class AwsElasticIpProvider extends AwsProvider implements ElasticIpProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsElasticIpProvider.class);
    private final BeanConvertor<ElasticIpModel, Address> convertor = new ElasticIpConvertor();

    public AwsElasticIpProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult allocate(String name) {
        try {
            AllocateAddressResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).allocateAddress();
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(name)
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(response.allocationId())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            ElasticIpModel model = (ElasticIpModel) detail(response.allocationId()).getData();
            return new BocloudResult(true, model, "allocate addresse success!");
        } catch (Exception e) {
            logger.error("allocate addresse error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult release(String allocationId) {
        try {
            Ec2Client client = (Ec2Client) this.getClient(AwsService.EC2);
            ReleaseAddressRequest request = ReleaseAddressRequest.builder().allocationId(allocationId).build();
            ReleaseAddressResponse response = client.releaseAddress(request);
            return new BocloudResult(true, response, "release address success!");
        } catch (Exception e) {
            logger.error("release address error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult update(ElasticIpModel model) {
        try {
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(model.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(model.getElasticIpUuid())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return new BocloudResult(true, "update addresse success!");
        } catch (Exception e) {
            logger.error("update addresse error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult associateNetworkInterface(String allocationId, String networkInterfaceId, String privateIpAddress, boolean allowReassociation) {
        try {
            AssociateAddressResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).associateAddress(AssociateAddressRequest.builder().allocationId(allocationId).networkInterfaceId(networkInterfaceId).privateIpAddress(privateIpAddress).allowReassociation(allowReassociation).build());
            return new BocloudResult(true, response.associationId(), "associate address success!");
        } catch (Exception e) {
            logger.error("associate address error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult associateInstance(String allocationId, String instanceId, String privateIpAddress, boolean allowReassociation) {
        try {
            AssociateAddressResponse response = null;
            if (StringUtils.isEmpty(privateIpAddress)) {
                response = ((Ec2Client) this.getClient(AwsService.EC2)).associateAddress(AssociateAddressRequest.builder().allocationId(allocationId).instanceId(instanceId).allowReassociation(allowReassociation).build());
            } else {
                response = ((Ec2Client) this.getClient(AwsService.EC2)).associateAddress(AssociateAddressRequest.builder().allocationId(allocationId).instanceId(instanceId).privateIpAddress(privateIpAddress).allowReassociation(allowReassociation).build());
            }
            return new BocloudResult(true, response.associationId(), "associate address success!");
        } catch (Exception e) {
            logger.error("associate address error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult disassociate(String associationId) {
        try {
            DisassociateAddressResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).disassociateAddress(DisassociateAddressRequest.builder().associationId(associationId).build());
            return new BocloudResult(true, "disassociate address success!");
        } catch (Exception e) {
            logger.error("disassociate address error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeAddressesResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeAddresses();
            List<ElasticIpModel> models = convertor.convertList(response.addresses());
            return new BocloudResult(true, models, "list addresses success!");
        } catch (Exception e) {
            logger.error("list addresses error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String id) {
        try {
            DescribeAddressesRequest request = DescribeAddressesRequest.builder().allocationIds(id).build();
            DescribeAddressesResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeAddresses(request);
            ElasticIpModel model = convertor.convertModel(response.addresses().get(0));
            return new BocloudResult(true, model, "detail elastic ip success!");
        } catch (Exception e) {
            logger.error("detail elastic ip error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
