package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.awsconvertor.NetworkInterfaceConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkInterfaceModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.ObjectUtils;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AwsNetworkInterfaceProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/11/21 15:03
 */
@Provider("bocloud.cmp.aws")
public class AwsNetworkInterfaceProvider extends AwsProvider implements NetworkInterfaceProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsNetworkInterfaceProvider.class);
    private final BeanConvertor<NetworkInterfaceModel, NetworkInterface> convertor = new NetworkInterfaceConvertor();

    public AwsNetworkInterfaceProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeNetworkInterfacesResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeNetworkInterfaces();
            List<NetworkInterfaceModel> interfaceModels = convertor.convertList(response.networkInterfaces());
            return new BocloudResult(true, interfaceModels, "list networkInterface success!");
        } catch (Exception e) {
            logger.error("list networkInterface error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String netFaceId) {
        try {
            DescribeNetworkInterfacesRequest request = DescribeNetworkInterfacesRequest.builder().networkInterfaceIds(netFaceId).build();
            DescribeNetworkInterfacesResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeNetworkInterfaces(request);
            List<NetworkInterface> networkInterfaces = response.networkInterfaces();
            NetworkInterfaceModel interfaceModel = convertor.convertModel(networkInterfaces.get(0));
            interfaceModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, interfaceModel, "detail networkInterface success!");
        } catch (Exception e) {
            logger.error("detail networkInterface error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String netFaceId) {
        try {
            DeleteNetworkInterfaceRequest request = DeleteNetworkInterfaceRequest.builder().networkInterfaceId(netFaceId).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).deleteNetworkInterface(request);
            return new BocloudResult(true, "remove networkInterface success!");
        } catch (Exception e) {
            logger.error("remove networkInterface error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(NetworkInterfaceModel model) {
        try {
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(model.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(model.getNetworkInterfaceUuid())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            ModifyNetworkInterfaceAttributeRequest request = ModifyNetworkInterfaceAttributeRequest.builder()
                    .description(AttributeValue.builder().value(model.getRemark()).build())
                    .networkInterfaceId(model.getNetworkInterfaceUuid())
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).modifyNetworkInterfaceAttribute(request);
            return new BocloudResult(true, "update networkInterface success!");
        } catch (Exception e) {
            logger.error("update networkInterface error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(NetworkInterfaceModel model) {
        try {
            CreateNetworkInterfaceRequest build = CreateNetworkInterfaceRequest.builder()
                    .description(model.getRemark())
                    .groups(model.getSecurityGroups())
                    .interfaceType(NetworkInterfaceCreationType.fromValue(model.getInterfaceType()))
                    .privateIpAddress(model.getPrimaryPrivateIPAddress())
                    .subnetId(model.getSubnetUuid())
                    .build();
            CreateNetworkInterfaceResponse networkInterfaceResponse = ((Ec2Client) this.getClient(AwsService.EC2)).createNetworkInterface(build);
            String networkInterfaceId = networkInterfaceResponse.networkInterface().networkInterfaceId();
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(model.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(networkInterfaceId)
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return getStatus(networkInterfaceId, "available");
        } catch (Exception e) {
            logger.error("create networkInterface error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult attach(NetworkInterfaceModel model) {
        try {
            AttachNetworkInterfaceRequest request = AttachNetworkInterfaceRequest.builder()
                    .networkInterfaceId(model.getNetworkInterfaceUuid())
                    .instanceId(model.getVirtualMachineUuid())
                    .deviceIndex(model.getDeviceIndex())
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).attachNetworkInterface(request);
            return getNetworkInterfaceAttchment(model.getNetworkInterfaceUuid());
        } catch (Exception e) {
            logger.error("attach networkInterface error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }

    @Override
    public BocloudResult detach(NetworkInterfaceModel model) {
        try {
            DetachNetworkInterfaceRequest request = DetachNetworkInterfaceRequest.builder()
                    .attachmentId(model.getAttachmentUuid())
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).detachNetworkInterface(request);
            return new BocloudResult(true, "detach networkInterface success!");
        } catch (Exception e) {
            logger.error("detach networkInterface error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult sgroups(NetworkInterfaceModel model) {
        try {
            ModifyNetworkInterfaceAttributeRequest request = ModifyNetworkInterfaceAttributeRequest.builder()
                    .groups(model.getSecurityGroups())
                    .networkInterfaceId(model.getNetworkInterfaceUuid())
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).modifyNetworkInterfaceAttribute(request);
            return new BocloudResult(true, "sgroups networkInterface success!");
        } catch (Exception e) {
            logger.error("sgroups networkInterface error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult getStatus(String id, String status) {
        BocloudResult result = new BocloudResult(false, "data does not exist");
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            while (endTime - startTime < 5 * 60 * 1000) {
                //创建时间不确定，增加休眠减少轮询次数
                Thread.sleep(6000);
                BocloudResult detail = this.detail(id);
                NetworkInterfaceModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), NetworkInterfaceModel.class);
                if (ObjectUtils.isEmpty(model)) {
                    logger.error("data does not exist :", id);
                    break;
                }
                if ((status.equalsIgnoreCase(model.getStatus()))) {
                    result = this.detail(id);
                    break;
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error("query status error : ", e);
            result = new BocloudResult(false, "query status error： " + e.getMessage());
        }
        return result;
    }

    private BocloudResult getNetworkInterfaceAttchment(String networkInterfaceId) {
        BocloudResult result = new BocloudResult(false, "data does not exist");
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            while (endTime - startTime < 5 * 60 * 1000) {
                //创建时间不确定，增加休眠减少轮询次数
                Thread.sleep(6000);
                BocloudResult detail = this.detail(networkInterfaceId);
                NetworkInterfaceModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), NetworkInterfaceModel.class);
                if (ObjectUtils.isEmpty(model)) {
                    logger.error("data does not exist :", networkInterfaceId);
                    break;
                }
                if (("attached".equalsIgnoreCase(model.getAttchmentStatus()))) {
                    result = this.detail(networkInterfaceId);
                    break;
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error("attach networkinterface get status error : ", e);
            result = new BocloudResult(false, "attach networkinterface get status error： " + e.getMessage());
        }
        return result;
    }
}
