package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.entity.enums.RouteTableType;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.RouteModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.CreateRouteRequest;
import software.amazon.awssdk.services.ec2.model.CreateRouteResponse;
import software.amazon.awssdk.services.ec2.model.DeleteRouteRequest;

@Provider("bocloud.cmp.aws")
public class AwsRouteProvider extends AwsProvider implements RouteProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsRouteProvider.class);

    public AwsRouteProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(RouteModel model) {
        try {
            CreateRouteRequest.Builder builder = CreateRouteRequest.builder().routeTableId(model.getRouteTableUuid());
            switch (RouteTableType.RouteTargetType.valueOf(model.getTargetType())) {
                case EGRESS_ONLY_INTERNET_GATEWAY:
                    builder.egressOnlyInternetGatewayId(model.getTargetUuid());
                    break;
                case INSTANCE:
                    builder.instanceId(model.getTargetUuid());
                    break;
                case INTERNET_GATEWAY:
                    builder.gatewayId(model.getTargetUuid());
                    break;
                case LOCAL:
                    builder.localGatewayId(model.getTargetUuid());
                    break;
                case NAT_GATEWAY:
                    builder.natGatewayId(model.getTargetUuid());
                    break;
                case NETWORK_INTERFACE:
                    builder.networkInterfaceId(model.getTargetUuid());
                    break;
                case PEERING_CONNECTION:
                    builder.vpcPeeringConnectionId(model.getTargetUuid());
                    break;
                case TRANSIT_GATEWAY:
                    builder.transitGatewayId(model.getTargetUuid());
                    break;
                case VIRTUAL_PRIVATE_GATEWAY:
                    builder.carrierGatewayId(model.getTargetUuid());
                    break;
                default:
                    break;

            }

            String destinationType = StringUtils.isNotEmpty(model.getNewDestinationType()) ? model.getNewDestinationType() : model.getDestinationType();
            switch (RouteTableType.RouteDestinationType.valueOf(destinationType)) {
                case IPV4:
                    builder.destinationCidrBlock(model.getDestination());
                    break;
                case IPV6:
                    builder.destinationIpv6CidrBlock(model.getDestination());
                    break;
                case PREFIX_LIST:
                    builder.destinationPrefixListId(model.getDestination());
                    break;
                default:
                    break;
            }

            CreateRouteRequest request = builder.build();
            CreateRouteResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).createRoute(request);
            return new BocloudResult(true, response.returnValue(), "create route success!");
        } catch (Exception e) {
            logger.error("create route error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult delete(RouteModel model) {
        try {
            DeleteRouteRequest.Builder builder = DeleteRouteRequest.builder().routeTableId(model.getRouteTableUuid());
            switch (RouteTableType.RouteDestinationType.valueOf(model.getDestinationType())) {
                case IPV4:
                    builder.destinationCidrBlock(model.getDestination());
                    break;
                case IPV6:
                    builder.destinationIpv6CidrBlock(model.getDestination());
                    break;
                case PREFIX_LIST:
                    builder.destinationPrefixListId(model.getDestination());
                    break;
                default:
                    break;
            }
            DeleteRouteRequest request = builder.build();
            ((Ec2Client) this.getClient(AwsService.EC2)).deleteRoute(request);
            return new BocloudResult(true, "delete route table success!");
        } catch (Exception e) {
            logger.error("delete route table error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
