package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.awsconvertor.RouteTableConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.RouteTableModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.List;

@Provider("bocloud.cmp.aws")
public class AwsRouteTableProvider extends AwsProvider implements RouteTableProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsRouteTableProvider.class);
    private final BeanConvertor<RouteTableModel, RouteTable> convertor = new RouteTableConvertor();

    public AwsRouteTableProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeRouteTablesResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeRouteTables();
            List<RouteTableModel> models = convertor.convertList(response.routeTables());
            return new BocloudResult(true, models, "list route tables success!");
        } catch (Exception e) {
            logger.error("list route tables error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(RouteTableModel model) {
        try {
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(model.getName())
                    .build();
            TagSpecification specification = TagSpecification.builder().tags(tag).resourceType(ResourceType.ROUTE_TABLE).build();
            CreateRouteTableRequest request = CreateRouteTableRequest.builder().vpcId(model.getVpcUuid()).tagSpecifications(specification).build();
            CreateRouteTableResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).createRouteTable(request);
            model = convertor.convertModel(response.routeTable());
            return new BocloudResult(true, model, "create route table success!");
        } catch (Exception e) {
            logger.error("create route table error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult delete(String routeTableId) {
        try {
            DeleteRouteTableRequest request = DeleteRouteTableRequest.builder().routeTableId(routeTableId).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).deleteRouteTable(request);
            return new BocloudResult(true, "delete route table success!");
        } catch (Exception e) {
            logger.error("delete route table error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(RouteTableModel model) {
        try {
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(model.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(model.getUuid())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return new BocloudResult(true, "update success!");
        } catch (Exception e) {
            logger.error("update routetable error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult getByVpc(String vpcId) {
        try {
            DescribeRouteTablesRequest request = DescribeRouteTablesRequest.builder()
                    .filters(Filter.builder().name("vpc-id").values(vpcId).build())
                    .build();
            DescribeRouteTablesResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeRouteTables(request);
            RouteTableModel model = convertor.convertModel(response.routeTables().get(0));
            return new BocloudResult(true, model, "get route table success!");
        } catch (Exception e) {
            logger.error("get route table error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
