package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.awsconvertor.SecurityGroupConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AwsSecurityGroupProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 15:47
 */
@Provider("bocloud.cmp.aws")
public class AwsSecurityGroupProvider extends AwsProvider implements SecurityGroupProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsSecurityGroupProvider.class);

    private final BeanConvertor<SecurityGroupModel, SecurityGroup> convertor = new SecurityGroupConvertor();

    public AwsSecurityGroupProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeSecurityGroupsResponse describeSecurityGroupsResponse = ((Ec2Client) this.getClient(AwsService.EC2)).describeSecurityGroups();
            List<SecurityGroupModel> securityGroupModels = convertor.convertList(describeSecurityGroupsResponse.securityGroups());
            return new BocloudResult(true, securityGroupModels, "list securityGroup success!");
        } catch (Exception e) {
            logger.error("list securityGroup error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(SecurityGroupModel group) {
        try {
            String awsGroupName = group.getAwsGroupName();
            if (StringUtils.isEmpty(awsGroupName)) {
                awsGroupName = group.getName();
            }
            CreateSecurityGroupRequest request = CreateSecurityGroupRequest.builder()
                    .description(group.getRemark())
                    .groupName(awsGroupName)
                    .vpcId(group.getVpcUuid())
                    .build();
            CreateSecurityGroupResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).createSecurityGroup(request);
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(group.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(response.groupId())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return new BocloudResult(true, this.detail(response.groupId()).getData(), "create securityGroup success!");
        } catch (Exception e) {
            logger.error("create securityGroup error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult remove(String sgId) {
        try {
            DeleteSecurityGroupRequest request = DeleteSecurityGroupRequest.builder()
                    .groupId(sgId)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).deleteSecurityGroup(request);
            return new BocloudResult(true, "remove securityGroup success!");
        } catch (Exception e) {
            logger.error("remove securityGroup error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult getByVpc(String vpcId) {
        try {
            DescribeSecurityGroupsRequest request = DescribeSecurityGroupsRequest.builder()
                    .filters(Filter.builder().name("vpc-id").values(vpcId).build())
                    .build();
            DescribeSecurityGroupsResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeSecurityGroups(request);
            SecurityGroupModel securityGroupModel = convertor.convertModel(response.securityGroups().get(0));
            securityGroupModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, securityGroupModel, "get securityGroup success!");
        } catch (Exception e) {
            System.out.println(e);
            logger.error("get securityGroup error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String sgId) {
        try {
            DescribeSecurityGroupsRequest request = DescribeSecurityGroupsRequest.builder()
                    .groupIds(sgId)
                    .build();
            DescribeSecurityGroupsResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeSecurityGroups(request);
            SecurityGroupModel securityGroupModel = convertor.convertModel(response.securityGroups().get(0));
            securityGroupModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, securityGroupModel, "detail securityGroup success!");
        } catch (Exception e) {
            logger.error("detail securityGroup error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(SecurityGroupModel model) {
        try {
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(model.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(model.getGroupUuid())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return new BocloudResult(true, "update securityGroup success!");
        } catch (Exception e) {
            logger.error("update securityGroup error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
