package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.awsconvertor.SubnetConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AwsSubnetProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 15:40
 */
@Provider("bocloud.cmp.aws")
public class AwsSubnetProvider extends AwsProvider implements SubnetProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsSubnetProvider.class);

    private final BeanConvertor<SubnetModel, Subnet> convertor = new SubnetConvertor();

    public AwsSubnetProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeSubnetsResponse describeSubnetsResponse = ((Ec2Client) this.getClient(AwsService.EC2)).describeSubnets();
            List<SubnetModel> subnetModels = convertor.convertList(describeSubnetsResponse.subnets());
            return new BocloudResult(true, subnetModels, "list subnet success!");
        } catch (Exception e) {
            logger.error("list subnet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(SubnetModel subnet) {
        try {
            CreateSubnetRequest.Builder builder = CreateSubnetRequest.builder()
                    .vpcId(subnet.getVpcUuid())
                    .availabilityZone(subnet.getZone())
                    .cidrBlock(subnet.getCidr());

            if (!StringUtils.isEmpty(subnet.getIpv6Cidr())) {
                builder.ipv6CidrBlock(subnet.getIpv6Cidr());
            }
            CreateSubnetRequest request = builder.build();
            CreateSubnetResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).createSubnet(request);
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(subnet.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(response.subnet().subnetId())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return getStatus(response.subnet().subnetId());
        } catch (Exception e) {
            logger.error("create subnet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    public BocloudResult getStatus(String id) {
        BocloudResult result = new BocloudResult(false, "data does not exist");
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            while (endTime - startTime < 5 * 60 * 1000) {
                //创建时间不确定，增加休眠减少轮询次数
                Thread.sleep(6000);
                BocloudResult detail = this.detail(id);
                SubnetModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), SubnetModel.class);
                if (ObjectUtils.isEmpty(model)) {
                    logger.error("data does not exist :", id);
                    break;
                }
                if (("available".equalsIgnoreCase(model.getStatus()))) {
                    result = this.detail(id);
                    break;
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error("query status error : ", e);
            result = new BocloudResult(false, "query status error： " + e.getMessage());
        }
        return result;
    }


    @Override
    public BocloudResult modify(SubnetModel subnet) {
        try {
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(subnet.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(subnet.getSubnetUuid())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return new BocloudResult(true, "modify subnet success!");
        } catch (Exception e) {
            logger.error("modify subnet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String subnetId) {
        try {
            DeleteSubnetRequest request = DeleteSubnetRequest.builder()
                    .subnetId(subnetId)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).deleteSubnet(request);
            return new BocloudResult(true, "remove subnet success!");
        } catch (Exception e) {
            logger.error("remove subnet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String subnetId) {
        try {
            DescribeSubnetsRequest request = DescribeSubnetsRequest.builder().subnetIds(subnetId).build();
            DescribeSubnetsResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeSubnets(request);
            SubnetModel subnetModel = convertor.convertModel(response.subnets().get(0));
            subnetModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, subnetModel, "detail subnet success!");
        } catch (Exception e) {
            logger.error("detail subnet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SubnetModel subnet) {
        try {
            DeleteSubnetRequest request = DeleteSubnetRequest.builder()
                    .subnetId(subnet.getSubnetUuid())
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).deleteSubnet(request);
            return new BocloudResult(true, "remove subnet success!");
        } catch (Exception e) {
            logger.error("remove subnet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult associateIpv6CidrBlock(String subnetId, String ipv6CidrBlock) {
        try {
            AssociateSubnetCidrBlockRequest request = AssociateSubnetCidrBlockRequest.builder().ipv6CidrBlock(ipv6CidrBlock).subnetId(subnetId).build();
            AssociateSubnetCidrBlockResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).associateSubnetCidrBlock(request);
            return new BocloudResult(true, response.ipv6CidrBlockAssociation().associationId(), "Associate SubnetCidrBlock success!");
        } catch (Exception e) {
            logger.error("Associate SubnetCidrBlock error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult disassociateCidrBlock(String associationId) {
        try {
            DisassociateSubnetCidrBlockRequest request = DisassociateSubnetCidrBlockRequest.builder().associationId(associationId).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).disassociateSubnetCidrBlock(request);
            return new BocloudResult(true, "Disassociate SubnetCidrBlock success!");
        } catch (Exception e) {
            logger.error("Disassociate SubnetCidrBlock error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
