package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.awsconvertor.TargetGroupConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.LbTargetGroupModel;
import com.bocloud.cmp.model.LbTargetModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.elasticloadbalancingv2.ElasticLoadBalancingV2Client;
import software.amazon.awssdk.services.elasticloadbalancingv2.model.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Provider("bocloud.cmp.aws")
public class AwsTargetGroupProvider extends AwsProvider implements TargetGroupProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsTargetGroupProvider.class);
    private final BeanConvertor<LbTargetGroupModel, TargetGroup> convertor = new TargetGroupConvertor();
    private final Butler butler;
    private TargetProvider targetProvider = null;

    public AwsTargetGroupProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeTargetGroupsResponse response = ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).describeTargetGroups();
            List<LbTargetGroupModel> models = convertor.convertList(response.targetGroups());
            return new BocloudResult(true, models, "list load target groups success!");
        } catch (Exception e) {
            logger.error("list load target groups error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(LbTargetGroupModel model) {
        try {
            CreateTargetGroupRequest.Builder builder = CreateTargetGroupRequest.builder()
                    .targetType(model.getType())
                    .name(model.getName())
                    .protocol(model.getProtocol())
                    .port(model.getPort())
                    .vpcId(model.getVpcUuid())
                    .healthCheckProtocol(model.getHealthCheckProtocol())
                    .healthCheckPath(model.getHealthCheckPath())
                    .healthCheckPort(model.getHealthCheckPort())
                    .healthyThresholdCount(model.getHealthyThresholdCount())
                    .unhealthyThresholdCount(model.getUnhealthyThresholdCount())
                    .healthCheckTimeoutSeconds(model.getHealthCheckTimeoutSeconds())
                    .healthCheckIntervalSeconds(model.getHealthCheckIntervalSeconds());
            if (model.getProtocolVersion() != null) {
                builder.protocolVersion(model.getProtocolVersion());
            }
            if (model.getGrpcCode() != null) {
                builder.matcher(Matcher.builder().grpcCode(model.getGrpcCode()).build());
            } else {
                builder.matcher(Matcher.builder().httpCode(model.getHttpCode()).build());
            }

            CreateTargetGroupRequest request = builder.build();
            CreateTargetGroupResponse response = ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).createTargetGroup(request);
            return new BocloudResult(true, convertor.convertModel(response.targetGroups().get(0)), "create target group success!");
        } catch (Exception e) {
            logger.error("create target group error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult delete(String targetGroupArn) {
        try {
            DeleteTargetGroupRequest request = DeleteTargetGroupRequest.builder().targetGroupArn(targetGroupArn).build();
            ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).deleteTargetGroup(request);
            return new BocloudResult(true, "delete target group success!");
        } catch (Exception e) {
            logger.error("delete target group error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String targetGroupArn) {
        try {
            DescribeTargetGroupsRequest request = DescribeTargetGroupsRequest.builder().targetGroupArns(targetGroupArn).build();
            DescribeTargetGroupsResponse response = ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).describeTargetGroups(request);
            List<LbTargetGroupModel> models = convertor.convertList(response.targetGroups());
            return new BocloudResult(true, models.get(0), "list load target group success!");
        } catch (Exception e) {
            logger.error("list load target group error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult registerTargets(String targetGroupArn, Map<String, Integer> targetIdPorts) {
        if (targetProvider == null) {
            targetProvider = new AwsTargetProvider(butler);
        }
        try {
            List<TargetDescription> targets = new ArrayList<>();
            if (!targetIdPorts.isEmpty()) {
                for (String targetId :
                        targetIdPorts.keySet()) {
                    targets.add(TargetDescription.builder().id(targetId).port(targetIdPorts.get(targetId)).build());
                }
            }
            RegisterTargetsRequest request = RegisterTargetsRequest.builder().targetGroupArn(targetGroupArn).targets(targets).build();
            ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).registerTargets(request);
            List<LbTargetModel> targetModels = new ArrayList<>();
            for (String key :
                    targetIdPorts.keySet()) {
                BocloudResult result = targetProvider.detail(targetGroupArn, key, targetIdPorts.get(key));
                if (result.isFailed()) {
                    return result;
                }
                targetModels.add((LbTargetModel) result.getData());
            }
            return new BocloudResult(true, targetModels, "register targets success!");
        } catch (Exception e) {
            logger.error("register targets error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult deregisterTargets(String targetGroupArn, Map<String, Integer> targetIdPorts) {
        try {
            List<TargetDescription> targets = new ArrayList<>();
            if (!targetIdPorts.isEmpty()) {
                for (String targetId :
                        targetIdPorts.keySet()) {
                    targets.add(TargetDescription.builder().id(targetId).port(targetIdPorts.get(targetId)).build());
                }
            }
            DeregisterTargetsRequest request = DeregisterTargetsRequest.builder().targetGroupArn(targetGroupArn).targets(targets).build();
            ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).deregisterTargets(request);
            return new BocloudResult(true, "deregister targets success!");
        } catch (Exception e) {
            logger.error("deregister targets error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
