package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.awsconvertor.VpcConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.ObjectUtils;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AwsVpcProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 15:34
 */
@Provider("bocloud.cmp.aws")
public class AwsVpcProvider extends AwsProvider implements VpcProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsVpcProvider.class);

    private final BeanConvertor<VpcModel, Vpc> convertor = new VpcConvertor();

    public AwsVpcProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeVpcsResponse describeVpcsResponse = ((Ec2Client) this.getClient(AwsService.EC2)).describeVpcs();
            List<VpcModel> vpcModels = convertor.convertList(describeVpcsResponse.vpcs());
            return new BocloudResult(true, vpcModels, "list vpc success!");
        } catch (Exception e) {
            logger.error("list vpc error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult create(VpcModel model) {
        try {
            CreateVpcRequest request = CreateVpcRequest.builder()
                    .cidrBlock(model.getCidr())
                    .amazonProvidedIpv6CidrBlock(model.getProvidedIpv6Cidr())
                    .build();
            CreateVpcResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).createVpc(request);
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(model.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(response.vpc().vpcId())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return getStatus(response.vpc().vpcId());
        } catch (Exception e) {
            logger.error("create vpc error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    public BocloudResult getStatus(String id) {
        BocloudResult result = new BocloudResult(false, "data does not exist");
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            while (endTime - startTime < 5 * 60 * 1000) {
                //创建时间不确定，增加休眠减少轮询次数
                Thread.sleep(6000);
                BocloudResult detail = this.detail(id, null);
                VpcModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), VpcModel.class);
                if (ObjectUtils.isEmpty(model)) {
                    logger.error("data does not exist :", id);
                    break;
                }
                if (("ACTIVE".equalsIgnoreCase(model.getStatus()))) {
                    result = this.detail(id, null);
                    break;
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error("query status error : ", e);
            result = new BocloudResult(false, "query status error： " + e.getMessage());
        }
        return result;
    }


    @Override
    public BocloudResult remove(String vpcId) {
        try {
            DeleteVpcRequest request = DeleteVpcRequest.builder()
                    .vpcId(vpcId)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).deleteVpc(request);
            return new BocloudResult(true, "remove network success!");
        } catch (Exception e) {
            logger.error("remove network error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }


    @Override
    public BocloudResult detail(String vpcId, String regionId) {
        try {
            DescribeVpcsRequest request = DescribeVpcsRequest.builder()
                    .vpcIds(vpcId)
                    .build();
            DescribeVpcsResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeVpcs(request);
            VpcModel vpcModel = convertor.convertModel(response.vpcs().get(0));
            vpcModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, vpcModel, "detail network success!");
        } catch (Exception e) {
            logger.error("detail network error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult update(VpcModel model) {
        try {
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(model.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(model.getVpcId())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return this.detail(model.getVpcId(), model.getRegionId());
        } catch (Exception e) {
            logger.error("update network error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult associateCidrBlock(String vpcId, String cidrBlock) {
        try {
            AssociateVpcCidrBlockRequest request = AssociateVpcCidrBlockRequest.builder().cidrBlock(cidrBlock).vpcId(vpcId).build();
            AssociateVpcCidrBlockResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).associateVpcCidrBlock(request);
            return new BocloudResult(true, response.cidrBlockAssociation().associationId(), "Associate VpcCidrBlock success!");
        } catch (Exception e) {
            logger.error("Associate VpcCidrBlock error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult associateIpv6CidrBlock(String vpcId) {
        try {
            AssociateVpcCidrBlockRequest request = AssociateVpcCidrBlockRequest.builder().amazonProvidedIpv6CidrBlock(true).vpcId(vpcId).build();
            AssociateVpcCidrBlockResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).associateVpcCidrBlock(request);
            return new BocloudResult(true, response.ipv6CidrBlockAssociation().associationId(), "Associate VpcCidrBlock success!");
        } catch (Exception e) {
            logger.error("Associate VpcCidrBlock error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult disassociateCidrBlock(String associationId) {
        try {
            DisassociateVpcCidrBlockRequest request = DisassociateVpcCidrBlockRequest.builder().associationId(associationId).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).disassociateVpcCidrBlock(request);
            return new BocloudResult(true, "Disassociate VpcCidrBlock success!");
        } catch (Exception e) {
            logger.error("Disassociate VpcCidrBlock error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
