package com.bocloud.cmp.provider.storage;

import com.bocloud.cmp.awsconvertor.VolumeConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.compute.AwsSnapshotsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AwsVolumeProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 14:55
 */
@Provider("bocloud.cmp.aws")
public class AwsVolumeProvider extends AwsProvider implements VolumeProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsVolumeProvider.class);

    private final BeanConvertor<VolumeModel, Volume> convertor = new VolumeConvertor();

    public AwsVolumeProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeVolumesResponse describeVolumesResponse = ((Ec2Client) this.getClient(AwsService.EC2))
                    .describeVolumes();
            List<VolumeModel> volumeModels = convertor.convertList(describeVolumesResponse.volumes());
            return new BocloudResult(true, volumeModels, "list disk success!");
        } catch (Exception e) {
            logger.error("list disk error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(VolumeModel volume) {
        try {
            CreateVolumeRequest request;
            if (!StringUtils.isEmpty(volume.getSourceSnapshotId())) {
                request = CreateVolumeRequest.builder().availabilityZone(volume.getZone())
                        .snapshotId(volume.getSourceSnapshotId()).build();
            } else {
                request = CreateVolumeRequest.builder().availabilityZone(volume.getZone()).size(volume.getSize())
                        .volumeType(VolumeType.fromValue(volume.getCategory())).build();
                if (volume.getIops() != null) {
                    request = request.toBuilder().iops(volume.getIops()).build();
                }
            }
            CreateVolumeResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).createVolume(request);
            Tag tag = Tag.builder().key("Name").value(volume.getName()).build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder().resources(response.volumeId()).tags(tag).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            volume.setVolumeUuid(response.volumeId());
            return new BocloudResult(true, volume, "create disk success!");
        } catch (Exception e) {
            logger.error("create disk error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(VolumeModel volume) {
        try {
            ModifyVolumeRequest request = ModifyVolumeRequest.builder().size(volume.getSize())
                    .volumeId(volume.getVolumeUuid()).build();
            if (volume.getIops() != null) {
                request.toBuilder().iops(volume.getIops());
            }
            ((Ec2Client) this.getClient(AwsService.EC2)).modifyVolume(request);
            if (!StringUtils.isEmpty(volume.getName())) {
                Tag tag = Tag.builder().key("Name").value(volume.getName()).build();
                CreateTagsRequest tagRequest = CreateTagsRequest.builder().resources(volume.getVolumeUuid()).tags(tag)
                        .build();
                ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            }
            return new BocloudResult(true, "modify disk success!");
        } catch (Exception e) {
            logger.error("modify disk error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String volumeId) {
        try {
            DeleteVolumeRequest request = DeleteVolumeRequest.builder().volumeId(volumeId).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).deleteVolume(request);
            return new BocloudResult(true, "remove disk success!");
        } catch (Exception e) {
            logger.error("remove disk error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String volumeId) {
        try {
            DescribeVolumesRequest request = DescribeVolumesRequest.builder().volumeIds().volumeIds(volumeId).build();
            DescribeVolumesResponse describeVolumesResponse = ((Ec2Client) this.getClient(AwsService.EC2))
                    .describeVolumes(request);
            VolumeModel volumeModel = convertor.convertModel(describeVolumesResponse.volumes().get(0));
            volumeModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, volumeModel, "detail disk success!");
        } catch (Exception e) {
            logger.error("detail disk error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        try {
            CreateSnapshotRequest request = CreateSnapshotRequest.builder().description(snapshotModel.getRemark())
                    .volumeId(snapshotModel.getVolumeUuid()).build();
            CreateSnapshotResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).createSnapshot(request);
            Tag tag = Tag.builder().key("Name").value(snapshotModel.getName()).build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder().resources(response.snapshotId()).tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return new BocloudResult(true, this.snapshotDetail(response.snapshotId()), "create snapshot success!");
        } catch (Exception e) {
            logger.error("create snapshot error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    private SnapshotModel snapshotDetail(String snapshotId) {
        AwsSnapshotsProvider provider = new AwsSnapshotsProvider(this.getButler());
        SnapshotModel model = new SnapshotModel();
        model.setSnapshotUuid(snapshotId);
        BocloudResult detail = provider.detail(model);
        // 因为aws的快照创建很快，造成返回状态与服务层相同造成逻辑循环，所以重新赋值
        model = (SnapshotModel) detail.getData();
        model.setStatus("BUILDING");
        return model;
    }

    @Override
    public BocloudResult attach(String serverId, String volumeId) {
        return null;
    }

    @Override
    public BocloudResult attach(String serverId, String mountpoint, String volumeId) {
        try {
            AttachVolumeRequest request = AttachVolumeRequest.builder().instanceId(serverId).volumeId(volumeId)
                    .device(mountpoint).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).attachVolume(request);
            return new BocloudResult(true, mountpoint, "attach disk success!");
        } catch (Exception e) {
            logger.error("attach disk error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detach(String serverId, String volumeId) {
        try {
            DetachVolumeRequest request = DetachVolumeRequest.builder().instanceId(serverId).volumeId(volumeId).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).detachVolume(request);
            return new BocloudResult(true, "detach disk success!");
        } catch (Exception e) {
            logger.error("detach disk error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult resetStatus(String volumeId, String status) {
        return null;
    }

    @Override
    public BocloudResult uploadToImage(String volumeId, ImageModel imageModel) {
        return null;
    }

    @Override
    public BocloudResult extend(String volumeId, Integer newSize) {
        return null;
    }

    @Override
    public BocloudResult list(String hostId, String storagePoolName) {
        return null;
    }

}
