package com.bocloud.cmp.azureconvertor.compute;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.NetworkInterfaceModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.microsoft.azure.management.compute.*;
import com.microsoft.azure.management.network.NetworkInterface;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: ServerConvertor
 * @Description:云主机数据转换器
 * @author: zhenpingLi/李振平
 * @date: 2019/9/17 17:55
 */
public class ServerConvertor implements BeanConvertor<ServerModel, VirtualMachine> {


    @Override
    public ServerModel convertModel(VirtualMachine machine) {
        ServerModel model = new ServerModel();
        model.setName(machine.name());
        model.setRegionId(machine.regionName());
        model.setZone(machine.regionName());
        model.setInstanceId(machine.id());
        model.setResourceGroupName(machine.resourceGroupName());
        Optional.ofNullable(machine.instanceView()).ifPresent(instanceView -> {
            List<InstanceViewStatus> statuses = instanceView.statuses();
            if (!CollectionUtils.isEmpty(statuses)) {
                if (statuses.size() >= 2) {
                    String status = statusConvertor(statuses.get(0).displayStatus(), statuses.get(1).displayStatus());
                    model.setStatus(status);
                } else {
                    model.setStatus("ERROR");
                }
            } else {
                model.setStatus("ERROR");
            }
        });
        //公共IP
        model.setPublicIps(Optional.ofNullable(machine.getPrimaryPublicIPAddress()).map(publicIp -> JSON.toJSONString(Arrays.asList(publicIp.ipAddress()))).orElse(null));
        //获取主网络接口
        NetworkInterface networkInterface = machine.getPrimaryNetworkInterface();
        //todo--2021/4/20  添加dns数据
        Optional.ofNullable(machine.getPrimaryPublicIPAddress()).ifPresent(publicIpAddress -> {
            model.setDnsServers(publicIpAddress.fqdn());
        });
        //获取vpc uuid
        Optional.ofNullable(networkInterface.primaryIPConfiguration()).map(ipConfiguration -> ipConfiguration.networkId())
                .ifPresent(model::setNetworkUuid);
        Optional.ofNullable(networkInterface.primaryIPConfiguration()).map(ipConfiguration -> ipConfiguration.networkId())
                .ifPresent(model::setVpcUuid);
        //专有IP
        model.setPrivateIps(Optional.ofNullable(networkInterface.primaryPrivateIP()).map(privateIp -> JSON.toJSONString(Arrays.asList(privateIp))).orElse(null));
        //网络接口UUID
        model.setNetworkInterfaceUuid(networkInterface.id());
        VirtualMachineInstanceView virtualMachineInstanceView = machine.instanceView();
        //OsName  OsVersion 系统盘大小 系统盘类 操作系统类型
        model.setSystemDiskUuid(machine.osDiskId());
        model.setOsName(virtualMachineInstanceView.osName());
        model.setOsVersion(virtualMachineInstanceView.osVersion());
        model.setSystemDiskSize(machine.osDiskSize());
        model.setSystemDiskCategory(Optional.ofNullable(machine.osDiskStorageAccountType()).map(accountType -> accountType.toString()).orElse(null));
        model.setOsCategory(machine.osType().toString());
        //安全组 UUID
        model.setSecurityGroups(Optional.ofNullable(networkInterface.getNetworkSecurityGroup()).map(securityGroup -> Arrays.asList(securityGroup.id())).orElse(null));
        //设置规格
        model.setFlavorUuid(Optional.ofNullable(machine.size()).map(flavor -> flavor.toString()).orElse(null));
        //包含该云主机下的所有数据盘
        List<VolumeModel> diskDevices = new ArrayList<>();
        int diskSize = 0;
        List<VirtualMachineDataDisk> diskList = new ArrayList<>(machine.dataDisks().values());
        for (VirtualMachineDataDisk virtualMachineDataDisk : diskList) {
            diskSize += virtualMachineDataDisk.size();
            VolumeModel volumeModel = new VolumeModel();
            volumeModel.setVolumeUuid(virtualMachineDataDisk.id());
            diskDevices.add(volumeModel);
        }
        model.setDiskDevices(diskDevices);
        model.setDisk(diskSize + machine.osDiskSize());
        //主机挂载的网络接口
        List<NetworkInterfaceModel> nicsModels = new ArrayList<>();
        Optional.ofNullable(machine.networkInterfaceIds()).ifPresent(
                networkInterfaceUuids ->
                        networkInterfaceUuids.stream().forEach(networkInterfaceUuid -> {
                            NetworkInterfaceModel nicsModel = new NetworkInterfaceModel();
                            nicsModel.setNetworkInterfaceUuid(networkInterfaceUuid);
                            nicsModels.add(nicsModel);
                        }));
        model.setNetworkInterfaces(nicsModels);
        model.setAccount(machine.osProfile().adminUsername());
        model.setPassword(machine.osProfile().adminPassword());
        Optional.ofNullable(machine.storageProfile().imageReference()).map(imageReference -> imageReference.id()).ifPresent(model::setImageUuid);
        //云主机是否启动了更新管理 --包含了和工作区绑定关系，后续入库开发,先放在props里供前端判断
        Collection<VirtualMachineExtension> values = machine.listExtensions().values();
        if (CollectionUtils.isEmpty(values)) {
            model.setProps("unbind");
        } else {
            model.setProps("bind");
        }
        //云主机可用性集uuid
        model.setAvailabilitySetUuid(machine.availabilitySetId());
        //密钥 ,只能获取到公钥的密钥
        Optional.ofNullable(machine.osProfile().linuxConfiguration())
                .ifPresent(linuxConfiguration -> Optional.ofNullable(linuxConfiguration.ssh())
                        .ifPresent(ssh -> {
                            List<String> collect = ssh.publicKeys()
                                    .stream()
                                    .map(publicKey -> publicKey.keyData())
                                    .collect(Collectors.toList());
                            model.setSshKey(JSONObject.toJSONString(collect));
                        })
                );
        //获取公共ip地址
        model.setPublicIpAddressUuid(machine.getPrimaryPublicIPAddressId());
        model.setLicenseType(machine.licenseType());
        //插入可用性区域
        if (!CollectionUtils.isEmpty(machine.availabilityZones())) {
            model.setAvailabilityZoneIds(JSON.toJSONString(machine.availabilityZones().stream().map(availabilityZoneId -> availabilityZoneId.toString()).collect(Collectors.toList())));
        }
//        model.setTags(null == machine.tags() ? null : JSONObject.toJSONString(machine.tags()));
        return model;
    }


    /**
     * 根据sdk获取中间状态不准，采用api模式获取中间状态，但是存在一个偶发问题，概率非常小
     * 由于云主机创建成功后azure立马返回，有可能造成provisionStatus ||powerStateStatus是null
     *
     * @param provisionStatus
     * @param powerStateStatus
     * @return
     */
    private String statusConvertor(String provisionStatus, String powerStateStatus) {
        String cmpStatus = null;
        if (provisionStatus.equals("Updating") && powerStateStatus.equals("VM running")) {
            cmpStatus = "RESTARTING"; //重启
        } else if (provisionStatus.equals("Provisioning succeeded") && powerStateStatus.equals("VM running")) {
            cmpStatus = "RUNNING"; //运行
        } else if (provisionStatus.equals("Updating") && powerStateStatus.equals("VM deallocating")) {
            cmpStatus = "DEALLOCATEING"; //取消分配中
        } else if (provisionStatus.equals("Provisioning succeeded") && powerStateStatus.equals("VM deallocated")) {
            cmpStatus = "DEALLOCATE"; //已取消分配
        } else if (provisionStatus.equals("Updating") && powerStateStatus.equals("VM starting")) {
            cmpStatus = "STARTING"; //开机中
        } else if (provisionStatus.equals("Deleting") && powerStateStatus.equals("VM deallocated")) {
            cmpStatus = ""; //删除
        } else if (provisionStatus.equals("Creating") && powerStateStatus.equals("VM starting")) {
            cmpStatus = "BUILDING"; //创建
        } else if (provisionStatus.equals("Provisioning succeeded") && powerStateStatus.equals("VM stopped")) {
            cmpStatus = "STOPPED"; //已关机
        } else if (provisionStatus.equals("Updating") && powerStateStatus.equals("VM stopping")) {
            cmpStatus = "STOPPING"; //关机中
        } else if (provisionStatus.equals("VM generalized") && powerStateStatus.equals("Provisioning succeeded")) {
            cmpStatus = "GENERAL"; //一般化
        }
        return cmpStatus;
    }


}
