package com.bocloud.cmp.azureconvertor.slb;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.azure.slb.*;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.microsoft.azure.management.network.LoadBalancer;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @ClassName: LoadBalanceConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 */
public class LoadBalanceConvertor implements BeanConvertor<SlbModel, LoadBalancer> {

    @Override
    public SlbModel convertModel(LoadBalancer loadBalancer) {
        SlbModel model = new SlbModel();
        model.setUuid(loadBalancer.id());
        model.setName(loadBalancer.name());
        model.setRegionId(loadBalancer.regionName());
        model.setResourceGroupName(loadBalancer.resourceGroupName());
        model.setStatus("AVAILABLE");

        //前端网络池
        List<InboundNatPoolModel> inboundNatPoolModes = new ArrayList<>();
        loadBalancer.inboundNatPools().values().forEach(inboundNatPool -> {
            InboundNatPoolModel poolModel = new InboundNatPoolModel();
            poolModel.setSlbUuid(loadBalancer.id());
            poolModel.setUuid(inboundNatPool.inner().id());
            poolModel.setName(inboundNatPool.name());
            poolModel.setProtocol(inboundNatPool.protocol().toString());
            poolModel.setFrontendPortRangeStart(inboundNatPool.frontendPortRangeStart());
            poolModel.setFrontendPortRangeEnd(inboundNatPool.frontendPortRangeEnd());
            poolModel.setBackendPort(inboundNatPool.backendPort());
            inboundNatPoolModes.add(poolModel);
        });
        model.setInboundNatPoolModels(inboundNatPoolModes);

        //前端ip配置
        List<FrontEndModel> frontEndModels = new ArrayList<>();
        loadBalancer.publicFrontends().values().forEach(frontEnd -> {
            FrontEndModel frontEndModel = new FrontEndModel();
            frontEndModel.setSlbUuid(loadBalancer.id());
            frontEndModel.setUuid(frontEnd.inner().id());
            frontEndModel.setName(frontEnd.name());
            model.setSlbType("EXTERNAL");
            frontEndModel.setFrontEndType("EXTERNAL");
            frontEndModel.setPublicIpUuid(frontEnd.publicIPAddressId());
            frontEndModels.add(frontEndModel);
        });
        model.setFrontEndModels(frontEndModels);

        loadBalancer.privateFrontends().values().forEach(frontEnd -> {
            FrontEndModel frontEndModel = new FrontEndModel();
            frontEndModel.setSlbUuid(loadBalancer.id());
            frontEndModel.setUuid(frontEnd.inner().id());
            frontEndModel.setName(frontEnd.name());
            model.setSlbType("INTERNAL");
            frontEndModel.setFrontEndType("INTERNAL");
            frontEndModel.setPrivateIpAddress(frontEnd.inner().privateIPAddress());
            frontEndModel.setSubnetUuid(frontEnd.inner().subnet().id());
            frontEndModels.add(frontEndModel);
        });

        //后端池
        List<BackEndModel> backEndModels = new ArrayList<>();
        loadBalancer.backends().values().forEach(backEnd -> {
            BackEndModel backEndModel = new BackEndModel();
            backEndModel.setSlbUuid(loadBalancer.id());
            backEndModel.setUuid(backEnd.inner().id());
            backEndModel.setName(backEnd.name());
            backEndModel.setVirtualMachineUuids(backEnd.getVirtualMachineIds().stream().collect(Collectors.toList()));
            backEndModels.add(backEndModel);
        });
        model.setBackEndModels(backEndModels);

        //运行状况探测
        List<ProbeModel> probeModels = new ArrayList<>();
        loadBalancer.tcpProbes().values().forEach(tcpProbe -> {
            ProbeModel probeModel = new ProbeModel();
            probeModel.setSlbUuid(loadBalancer.id());
            probeModel.setUuid(tcpProbe.inner().id());
            probeModel.setName(tcpProbe.name());
            probeModel.setIntervalInSeconds(tcpProbe.intervalInSeconds());
            probeModel.setNumberOfProbes(tcpProbe.numberOfProbes());
            probeModel.setProtocol(tcpProbe.protocol().toString());
            probeModel.setPort(tcpProbe.port());
            probeModels.add(probeModel);
        });
        loadBalancer.httpProbes().values().forEach(httpProbe -> {
            ProbeModel probeModel = new ProbeModel();
            probeModel.setSlbUuid(loadBalancer.id());
            probeModel.setUuid(httpProbe.inner().id());
            probeModel.setName(httpProbe.name());
            probeModel.setIntervalInSeconds(httpProbe.intervalInSeconds());
            probeModel.setNumberOfProbes(httpProbe.numberOfProbes());
            probeModel.setProtocol(httpProbe.protocol().toString());
            probeModel.setRequestPath(httpProbe.requestPath());
            probeModel.setPort(httpProbe.port());
            probeModels.add(probeModel);
        });
        loadBalancer.httpsProbes().values().forEach(httpsProbe -> {
            ProbeModel probeModel = new ProbeModel();
            probeModel.setSlbUuid(loadBalancer.id());
            probeModel.setUuid(httpsProbe.inner().id());
            probeModel.setName(httpsProbe.name());
            probeModel.setIntervalInSeconds(httpsProbe.intervalInSeconds());
            probeModel.setNumberOfProbes(httpsProbe.numberOfProbes());
            probeModel.setProtocol(httpsProbe.protocol().toString());
            probeModel.setRequestPath(httpsProbe.requestPath());
            probeModel.setPort(httpsProbe.port());
            probeModels.add(probeModel);
        });
        model.setProbeModels(probeModels);

        //负载均衡规则
        List<LoadBalanceRuleModel> loadBalanceRuleModels = new ArrayList<>();
        loadBalancer.loadBalancingRules().values().forEach(lbRule -> {
            LoadBalanceRuleModel lbmodel = new LoadBalanceRuleModel();
            lbmodel.setSlbUuid(loadBalancer.id());
            lbmodel.setUuid(lbRule.inner().id());
            lbmodel.setName(lbRule.name());
            lbmodel.setBackEndPort(lbRule.backendPort());
            Optional.ofNullable(lbRule.backend()).ifPresent(backend ->
                    lbmodel.setBackEndUuid(backend.inner().id())
            );
            lbmodel.setFloatingIpEnabled(lbRule.floatingIPEnabled());
            lbmodel.setFrontEndPort(lbRule.frontendPort());
            lbmodel.setFrontEndUuid(lbRule.frontend().inner().id());
            lbmodel.setIdleTimeoutInMinutes(lbRule.idleTimeoutInMinutes());
            lbmodel.setLoadDistribution(lbRule.loadDistribution().toString());
            lbmodel.setProbeUuid(lbRule.probe().inner().id());
            lbmodel.setProtocol(lbRule.protocol().toString());
            loadBalanceRuleModels.add(lbmodel);
        });
        model.setLoadBalanceRuleModels(loadBalanceRuleModels);

        //入站Nat规则
        List<InboundNatRuleModel> inboundNatRuleModels = new ArrayList<>();
        loadBalancer.inboundNatRules().values().forEach(inboundNatRule -> {
            InboundNatRuleModel natRuleModel = new InboundNatRuleModel();
            natRuleModel.setSlbUuid(loadBalancer.id());
            natRuleModel.setUuid(inboundNatRule.inner().id());
            natRuleModel.setName(inboundNatRule.name());
            natRuleModel.setBackEndPort(inboundNatRule.backendPort());
            natRuleModel.setFloatingIPEnabled(inboundNatRule.floatingIPEnabled());
            natRuleModel.setFrontEndPort(inboundNatRule.frontendPort());
            natRuleModel.setFrontEndUuid(inboundNatRule.frontend().inner().id());
            natRuleModel.setIdleTimeoutInMinutes(inboundNatRule.idleTimeoutInMinutes());
            natRuleModel.setProtocol(inboundNatRule.protocol().toString());
            inboundNatRuleModels.add(natRuleModel);
        });
        model.setInboundNatRuleModels(inboundNatRuleModels);
        model.setPublicIPAddressUuids(loadBalancer.publicIPAddressIds());
        model.setTags(null == loadBalancer.tags() ? null : JSONObject.toJSONString(loadBalancer.tags()));
        return model;
    }
}