package com.bocloud.cmp.provider;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.Result;
import com.microsoft.azure.AzureEnvironment;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.io.IOUtils;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.*;
import org.apache.http.entity.StringEntity;
import org.apache.http.message.BasicNameValuePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureProvider
 * @Description:中国版
 * @author: zhenpingLi/李振平
 * @date: 2019/7/22 16:58
 */
@Provider("bocloud.cmp.azure")
public class AzureRestApiProvider extends GenericProvider implements SuperProvider {

    private static final Logger LOGGER = LoggerFactory.getLogger(AzureRestApiProvider.class);
    private static final Integer TIMEOUT = 20 * 60 * 1000;//连接超时时间
    private String subscriptionId;//订阅ID
    private String clientId;//应用程序(客户端) ID
    private String clientSecret;//密钥
    private String tenantId; //租户ID
    private String version; //版本
    private String regionId;//地域
    private Butler butler;
    private AzureToken token;
    private HttpClient client;
    private AzureEnvironment azureEnvironment;
    private String enrollmentNumber;//合约编号

    public AzureRestApiProvider(Butler butler) {
        super(butler);
        this.tenantId = butler.getTenant();
        this.subscriptionId = butler.getSubscriptionId();
        this.clientId = butler.getClientId();
        this.clientSecret = butler.getSecretKey();
        this.regionId = butler.getRegionId();
        this.butler = butler;
        this.version = butler.getVersion();
        this.enrollmentNumber = butler.getEnrollmentNumber();
        switch (version) {
            case "AZURE":
                azureEnvironment = AzureEnvironment.AZURE;
                break;
            case "AZURE_CHINA":
                azureEnvironment = AzureEnvironment.AZURE_CHINA;
                break;
            case "AZURE_US_GOVERNMENT":
                azureEnvironment = AzureEnvironment.AZURE_US_GOVERNMENT;
                break;
            case "AZURE_GERMANY":
                azureEnvironment = AzureEnvironment.AZURE_GERMANY;
                break;
            default:
                azureEnvironment = AzureEnvironment.AZURE_CHINA;
                break;
        }
        this.token = this.getToken(tenantId, clientId, clientSecret);
    }

    @Override
    public boolean accessable() {
        boolean flag = this.getToken() != null;
//        LOGGER.info("AZURE API 认证 :" + flag);
        return flag;
    }

    public String request(RequestStatus method, String url, String params) {
        Result result = this.connection(method, url, null, JSON.parseObject(params, Map.class), true);
        if (result.isFailed()) {
            throw new IllegalStateException("[" + unicodeToString(result.getErrorMsg()) + "]" + unicodeToString(result.getMessage()));
        }
        return unicodeToString(result.getMessage());
    }

    public String requestNoEndpoint(RequestStatus method, String url, String params) {
        Result result = this.connection(method, url, null, JSON.parseObject(params, Map.class), false);
        if (result.isFailed()) {
            throw new IllegalStateException("[" + unicodeToString(result.getErrorMsg()) + "]" + unicodeToString(result.getMessage()));
        }
        return unicodeToString(result.getMessage());
    }

    public Result connection(RequestStatus method, String url, Map<String, Object> header, Map<String, Object> params, Boolean isEndpoint) {
        if (MapUtils.isEmpty(header)) {
            header = this.buildHeader();
        }
        if (isEndpoint) {
            url = this.azureEnvironment.endpoints().get("resourceManagerEndpointUrl") + url;
        }
//        System.out.println("最终请求URL：" + url);
        LOGGER.debug(url);
        Result result;
        switch (method) {
            case GET:
                result = this.get(url, header, params);
                break;
            case PUT:
                result = this.put(url, header, params);
                break;
            case POST:
                result = this.post(url, header, params);
                break;
            case DELETE:
                result = this.delete(url, header, params);
                break;
            case PATCH:
                result = this.patch(url, header, params);
                break;
            default:
                result = new Result(false, "请求方式不存在！", "请求方式不存在！");
                break;
        }
        return result;
    }

    public Result get(String url, Map<String, Object> header, Map<String, Object> params) {
        client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
        try {
            Map<String, Object> auth = new HashMap<>();
            auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
            if (null != header) {
                auth.putAll(header);
            }
            client.setDataFormat(null);
            Result result = client.get(auth, params, url);
            client.close();
            return result;
        } finally {
            client.close();
        }
    }

    public Result put(String url, Map<String, Object> header, Map<String, Object> params) {
        client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
        try {
            Map<String, Object> auth = new HashMap<>();
            auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
            if (null != header) {
                auth.putAll(header);
            }
            client.setDataFormat(PostDataFormat.RAW);
            Result result = this.client.put(auth, params, url);
            client.close();
            return result;
        } finally {
            this.client.close();
        }
    }

    public Result post(String url, Map<String, Object> header, Map<String, Object> params) {
        client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
        try {
            Map<String, Object> auth = new HashMap<>();
            auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
            if (null != header) {
                auth.putAll(header);
            }
            client.setDataFormat(PostDataFormat.RAW);
            Result result = this.client.post(auth, params, url);
            client.close();
            return result;
        } finally {
            this.client.close();
        }
    }

    public Result patch(String url, Map<String, Object> header, Map<String, Object> params) {
        client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
        try {
            Map<String, Object> auth = new HashMap<>();
            auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
            if (null != header) {
                auth.putAll(header);
            }
            client.setDataFormat(PostDataFormat.RAW);
            Result result = this.client.patch(auth, params, url);
            client.close();
            return result;
        } finally {
            this.client.close();
        }
    }

    public Result delete(String url, Map<String, Object> header, Map<String, Object> params) {
        client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
        try {
            Map<String, Object> auth = new HashMap<>();
            auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
            if (null != header) {
                auth.putAll(header);
            }
            client.setDataFormat(PostDataFormat.RAW);
            Result result = this.client.delete(auth, params, url);
            client.close();
            return result;
        } finally {
            this.client.close();
        }
    }

    private String unicodeToString(String string) {
        if (StringUtils.isEmpty(string)) {
            return string;
        }
        StringBuffer stringBuffer = new StringBuffer();
        String pattern = "\\\\u[0-9,a-f,A-F]{4}";
        Matcher matcher = Pattern.compile(pattern).matcher(string);
        while (matcher.find()) {
            char c = (char) Integer.parseInt(matcher.group().replaceAll("\\\\u", ""), 16);
            matcher.appendReplacement(stringBuffer, new String(new char[]{c}));
        }
        matcher.appendTail(stringBuffer);
        return stringBuffer.toString();
    }

    /**
     * OAuth 2.0 客户端凭据授予 全球版
     * 第一种情况：使用共享机密访问令牌请求
     * https://docs.microsoft.com/zh-cn/azure/active-directory/develop/v2-oauth2-client-creds-grant-flow
     *
     * @param clientSecret
     * @return
     */
    public AzureToken getToken(String tenantId, String clientId, String clientSecret) {
        AzureToken token = null;
        try {
            String url = azureEnvironment.endpoints().get("activeDirectoryEndpointUrl") + tenantId + "/oauth2/v2.0/token";
            client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
            HttpPost httpPost = new HttpPost(url);
            Map<String, Object> header = new HashMap<>();
            header.put("content-type", "application/x-www-form-urlencoded");
            client.buildHeader(httpPost, header);
            Map<String, Object> queryString = new HashMap<>();
            queryString.put("grant_type", "client_credentials");
            queryString.put("scope", azureEnvironment.endpoints().get("resourceManagerEndpointUrl") + ".default");
            queryString.put("client_id", clientId);
            queryString.put("client_secret", clientSecret);
            queryString.put("tenant", tenantId);
            httpPost.setEntity(getEntityConvertor(queryString));
            CloseableHttpResponse response = client.getHttpClient().execute(httpPost);
            token = JSONObject.parseObject(Respone2Content.getBody(response).toJSONString(), AzureToken.class);
        } catch (Exception e) {
            LOGGER.error("AZURE令牌请求失败:" + e.getMessage());
        } finally {
            client.close();
        }
        return token;
    }

    //当content-type为application/x-www-form-urlencoded，传入queryString为map时请求报404，所以做下数据转换
    private UrlEncodedFormEntity getEntityConvertor(Map<String, Object> queryString) throws Exception {
        List<NameValuePair> params = new ArrayList<>();
        for (Map.Entry<String, Object> entry : queryString.entrySet()) {
            NameValuePair nameValuePair = new BasicNameValuePair(entry.getKey(), String.valueOf(entry.getValue()));
            params.add(nameValuePair);
        }
        UrlEncodedFormEntity urlEncodedFormEntity = new UrlEncodedFormEntity(params, "UTF-8");
        return urlEncodedFormEntity;
    }

    private Map<String, Object> buildHeader() {
        Map<String, Object> header = new HashMap<>();
        header.put("Content-Type", "application/json");
        return header;
    }

    //升级token获取,建议应用

    /**
     * OAuth 2.0 客户端凭据授予 国内版本
     * 第一种情况：使用共享机密访问令牌请求
     * https://docs.azure.cn/zh-cn/active-directory/azuread-dev/v1-oauth2-client-creds-grant-flow
     *
     * @param clientSecret
     * @return
     */
    public AzureToken getStrongToken(String tenantId, String clientId, String clientSecret) {
        AzureToken token = null;
        try {
            String url = "https://login.partner.microsoftonline.cn/" + tenantId + "/oauth2/token";
            client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
            HttpPost httpPost = new HttpPost(url);
            Map<String, Object> header = new HashMap<>();
            header.put("content-type", "application/x-www-form-urlencoded");
            client.buildHeader(httpPost, header);
            Map<String, Object> queryString = new HashMap<>();
            queryString.put("grant_type", "client_credentials");
            queryString.put("client_id", clientId);
            queryString.put("client_secret", clientSecret);
            queryString.put("resource", "https://api.loganalytics.azure.cn");//获取的token应用于resource代表地址请求
            httpPost.setEntity(getEntityConvertor(queryString));
            CloseableHttpResponse response = client.getHttpClient().execute(httpPost);
            token = JSONObject.parseObject(Respone2Content.getBody(response).toJSONString(), AzureToken.class);
        } catch (Exception e) {
            LOGGER.error("AZURE令牌请求失败:" + e.getMessage());
        } finally {
            client.close();
        }
        return token;
    }


    //----------------------------------------------------------我是分界线--------------------------------------

    public String getRegionId() {
        return regionId;
    }

    public void setRegionId(String regionId) {
        this.regionId = regionId;
    }

    public String getVersion() {
        return version;
    }

    public String getEnrollmentNumber() {
        return enrollmentNumber;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public String getSubscriptionId() {
        return subscriptionId;
    }

    public Butler getButler() {
        return butler;
    }

    public void setButler(Butler butler) {
        this.butler = butler;
    }

    public AzureToken getToken() {
        return token;
    }

    public static class AzureToken {
        private String token_type;
        private String expires_in;
        private String ext_expires_in;
        private String expires_on;
        private String not_before;
        private String resource;
        private String access_token;

        public String getToken_type() {
            return token_type;
        }

        public void setToken_type(String token_type) {
            this.token_type = token_type;
        }

        public String getExpires_in() {
            return expires_in;
        }

        public void setExpires_in(String expires_in) {
            this.expires_in = expires_in;
        }

        public String getExt_expires_in() {
            return ext_expires_in;
        }

        public void setExt_expires_in(String ext_expires_in) {
            this.ext_expires_in = ext_expires_in;
        }

        public String getExpires_on() {
            return expires_on;
        }

        public void setExpires_on(String expires_on) {
            this.expires_on = expires_on;
        }

        public String getNot_before() {
            return not_before;
        }

        public void setNot_before(String not_before) {
            this.not_before = not_before;
        }

        public String getResource() {
            return resource;
        }

        public void setResource(String resource) {
            this.resource = resource;
        }

        public String getAccess_token() {
            return access_token;
        }

        public void setAccess_token(String access_token) {
            this.access_token = access_token;
        }
    }

    public static class Respone2Content {
        public static JSONObject getBody(CloseableHttpResponse httpResponse) throws Exception {
            JSONObject content = JSONObject.parseObject(IOUtils.toString(httpResponse.getEntity().getContent(), "UTF-8"));
            return content;
        }
    }

    public static enum RequestStatus {
        GET,
        POST,
        DELETE,
        PUT,
        PATCH
    }

    //暂时不用了------------------------------------------------------------------------------------
    //api1.0使用共享机密访问令牌请求  已废弃
    public AzureToken getOneToken(String tenantId, String clientId, String clientSecret) {
        client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
        HttpPost httpPost = new HttpPost("https://login.chinacloudapi.cn/" + tenantId + "/oauth2/token?api-version=1.0");
        Map<String, Object> header = new HashMap<>();
        header.put("content-type", "application/x-www-form-urlencoded");
        client.buildHeader(httpPost, header);
        Map<String, Object> queryString = new HashMap<>();
        queryString.put("grant_type", "client_credentials");
        queryString.put("resource", "https://management.chinacloudapi.cn/");
        queryString.put("client_id", clientId);
        queryString.put("client_secret", clientSecret);
        AzureToken token = null;
        try {
            httpPost.setEntity(getEntityConvertor(queryString));
            CloseableHttpResponse response = client.getHttpClient().execute(httpPost);
            token = JSONObject.parseObject(Respone2Content.getBody(response).toJSONString(), AzureToken.class);
        } catch (Exception e) {
            LOGGER.error("令牌请求失败： " + e.getMessage());
        } finally {
            client.close();
        }
        return token;
    }

    public CloseableHttpResponse doConnection(RequestStatus method, String url, Map<String, Object> header, String params) throws Exception {
        if (MapUtils.isEmpty(header)) {
            header = this.buildHeader();
        }
        CloseableHttpResponse response = null;
        url = this.azureEnvironment.endpoints().get("resourceManagerEndpointUrl") + url;
        switch (method) {
            case GET:
                response = this.doGet(url, header, params);
                break;
            case PUT:
                response = this.doPut(url, header, params);
                break;
            case POST:
                response = this.doPost(url, header, params);
                break;
            case DELETE:
                response = this.doDelete(url, header, params);
                break;
            case PATCH:
                response = this.doPatch(url, header, params);
                break;
            default:
                break;
        }
        return response;
    }

    public CloseableHttpResponse doGet(String url, Map<String, Object> header, String params) throws Exception {
        client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
        //添加请求头
        Map<String, Object> auth = new HashMap<>();
        auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
        if (null != header) {
            auth.putAll(header);
        }

        url = client.buildURL(url, JSON.parseObject(params, Map.class));
        HttpGet httpGet = new HttpGet(url);
        client.buildHeader(httpGet, auth);
        CloseableHttpResponse httpResponse = client.getHttpClient().execute(httpGet);
        this.client.close();
        return httpResponse;
    }

    public CloseableHttpResponse doPost(String url, Map<String, Object> header, String params) throws Exception {
        client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
        //添加请求头
        Map<String, Object> auth = new HashMap<>();
        auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
        if (null != header) {
            auth.putAll(header);
        }

        HttpPost httpPost = new HttpPost(url);
        client.buildHeader(httpPost, auth);
        if (!StringUtils.isEmpty(params) && !"{}".equals(params)) {
            StringEntity httpEntity = new StringEntity(params, "UTF-8");
            httpPost.setEntity(httpEntity);
        }
        CloseableHttpResponse httpResponse = client.getHttpClient().execute(httpPost);
        this.client.close();
        return httpResponse;
    }

    public CloseableHttpResponse doPut(String url, Map<String, Object> header, String params) throws Exception {
        client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
        //添加请求头
        Map<String, Object> auth = new HashMap<>();
        auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
        if (null != header) {
            auth.putAll(header);
        }

        HttpPut httpPut = new HttpPut(url);
        client.buildHeader(httpPut, auth);
        if (!StringUtils.isEmpty(params)) {
            StringEntity httpEntity = new StringEntity(params, "UTF-8");
            httpPut.setEntity(httpEntity);
        }
        CloseableHttpResponse httpResponse = client.getHttpClient().execute(httpPut);
        this.client.close();
        return httpResponse;
    }

    public CloseableHttpResponse doPatch(String url, Map<String, Object> header, String params) throws Exception {
        client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
        //添加请求头
        Map<String, Object> auth = new HashMap<>();
        auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
        if (null != header) {
            auth.putAll(header);
        }

        HttpPatch httpPatch = new HttpPatch(url);
        client.buildHeader(httpPatch, auth);
        if (!StringUtils.isEmpty(params) && !"{}".equals(params)) {
            StringEntity httpEntity = new StringEntity(params, "UTF-8");
            httpPatch.setEntity(httpEntity);
        }
        CloseableHttpResponse httpResponse = client.getHttpClient().execute(httpPatch);
        this.client.close();
        return httpResponse;
    }

    public CloseableHttpResponse doDelete(String url, Map<String, Object> header, String params) throws Exception {
        client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
        //添加请求头
        Map<String, Object> auth = new HashMap<>();
        auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
        if (null != header) {
            auth.putAll(header);
        }

        url = client.buildURL(url, JSON.parseObject(params, Map.class));
        HttpDelete httpDelete = new HttpDelete(url);
        client.buildHeader(httpDelete, auth);
        CloseableHttpResponse httpResponse = client.getHttpClient().execute(httpDelete);
        this.client.close();
        return httpResponse;
    }
}
