package com.bocloud.cmp.provider.azure.account;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.account.StorageAccountConvertor;
import com.bocloud.cmp.entity.enums.azure.StorageAccountFirewallType;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.account.StorageAccountFirewallModel;
import com.bocloud.cmp.model.azure.account.StorageAccountModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.ServiceEndpointType;
import com.microsoft.azure.management.storage.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureStorageAccountProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/7/24 13:53
 */
@Provider("bocloud.cmp.azure")
public class AzureStorageAccountProvider extends AzureProvider implements StorageAccountProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureStorageAccountProvider.class);

    public AzureStorageAccountProvider(Butler butler) {
        super(butler);
    }

    private BeanConvertor<StorageAccountModel, StorageAccount> convertor = new StorageAccountConvertor();

    @Override
    public BocloudResult list() {
        try {
            List<StorageAccountModel> regionModels = convertor.convertList(this.getAzure().storageAccounts().list()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
//            for (StorageAccountModel model : regionModels) {
//                JSONObject object = this.detailApi(model);
//                Optional.ofNullable(object.getJSONObject("properties")).ifPresent(properties -> {
//                    model.setMinimumTlsVersion(properties.getString("minimumTlsVersion"));
//                });
//            }
            return new BocloudResult(true, regionModels, "list storageAccount success!");
        } catch (Exception e) {
            logger.error("list storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String uuid) {
        try {
            StorageAccount storageAccount = this.getAzure().storageAccounts().getById(uuid);
            return new BocloudResult(true, convertor.convertModel(storageAccount), "detail storageAccount success!");
        } catch (Exception e) {
            logger.error("detail storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String uuid) {
        try {
            this.getAzure().storageAccounts().deleteById(uuid);
            return new BocloudResult(true, "remove storageAccount success!");
        } catch (Exception e) {
            logger.error("remove storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(StorageAccountModel model) {
        try {
            StorageAccount storageAccount = this.getAzure()
                    .storageAccounts()
                    .getById(model.getUuid())
                    .update()
                    .withSku(StorageAccountSkuType.fromSkuName(SkuName.fromString(model.getSku())))
                    .apply();
            return new BocloudResult(true, convertor.convertModel(storageAccount), "modify storageAccount success!");
        } catch (Exception e) {
            logger.error("modify storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(StorageAccountModel model) {
        try {
            //SDK方式创建
//            StorageAccount storageAccount = this.getCreateWaySDK(model);
//            StorageAccountModel accountModel = convertor.convertModel(storageAccount);
//            return new BocloudResult(true, accountModel, "create storageAccount success!");
            //API方式创建
            StorageAccountModel accountModel = this.getCreateWayAPI(model);
            return new BocloudResult(true, accountModel, "create storageAccount success!");
        } catch (Exception e) {
            logger.error("create storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult checkName(String name) {
        try {
            CheckNameAvailabilityResult checkNameAvailabilityResult = this.getAzure().storageAccounts().checkNameAvailability(name);
            return new BocloudResult(true, checkNameAvailabilityResult, " checkName storageAccount success!");
        } catch (Exception e) {
            logger.error("checkName storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult customDomain(StorageAccountModel model) {
        try {
            StorageAccount storageAccount = this.getAzure().storageAccounts().getById(model.getUuid())
                    .update()
                    .withCustomDomain(model.getCustomDomain())
                    .apply();
            return new BocloudResult(true, convertor.convertModel(storageAccount), "customDomain storageAccount success!");
        } catch (Exception e) {
            logger.error("customDomain storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult minimumTlsVersion(StorageAccountModel model) {
        try {
            AzureRestApiProvider provider = new AzureRestApiProvider(this.getButler());
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Storage/storageAccounts/%s?api-version=2021-04-01";
            String url = String.format(api, provider.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            JSONObject params = new JSONObject();
            JSONObject properties = new JSONObject();
            properties.put("minimumTlsVersion", model.getMinimumTlsVersion());
            params.put("properties", properties);
            provider.request(AzureRestApiProvider.RequestStatus.PATCH, url, params.toJSONString());
            return new BocloudResult(true, "minimumTlsVersion storageAccount success!");
        } catch (Exception e) {
            logger.error("minimumTlsVersion storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }

    @Override
    public BocloudResult withAccessFromNetworkSubnet(StorageAccountModel model) {
        try {
            this.getAzure().networks().getById(model.getVpcUuid())
                    .update()
                    .updateSubnet(model.getSubnetName())
                    .withAccessFromService(ServiceEndpointType.MICROSOFT_STORAGE)
                    .parent()
                    .apply();
            StorageAccount storageAccount = this.getAzure().storageAccounts().getById(model.getUuid())
                    .update()
                    .withAccessFromNetworkSubnet(model.getSubnetUuid())
                    .withAccessFromSelectedNetworks()
                    .apply();
            return new BocloudResult(true, convertor.convertModel(storageAccount), "withAccessFromNetworkSubnet storageAccount success!");
        } catch (Exception e) {
            logger.error("withAccessFromNetworkSubnet storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult withoutNetworkSubnetAccess(StorageAccountModel model) {
        try {
            StorageAccount storageAccount = this.getAzure().storageAccounts().getById(model.getUuid())
                    .update()
                    .withoutNetworkSubnetAccess(model.getSubnetUuid())
                    .apply();
            return new BocloudResult(true, convertor.convertModel(storageAccount), "withoutNetworkSubnetAccess storageAccount success!");
        } catch (Exception e) {
            logger.error("withoutNetworkSubnetAccess storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult withAccessFromAllNetworks(StorageAccountModel model) {
        try {
            StorageAccount storageAccount = this.getAzure().storageAccounts().getById(model.getUuid())
                    .update()
                    .withAccessFromAllNetworks()
                    .apply();
            return new BocloudResult(true, convertor.convertModel(storageAccount), "withoutNetworkSubnetAccess storageAccount success!");
        } catch (Exception e) {
            logger.error("withoutNetworkSubnetAccess storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult withAccessFromIpAddress(StorageAccountFirewallModel model) {
        try {
            StorageAccountFirewallType firewallType = StorageAccountFirewallType.valueOf(model.getIpType());
            switch (firewallType) {
                case IP:
                    this.getAzure()
                            .storageAccounts()
                            .getById(model.getStorageAccountUuid())
                            .update()
                            .withAccessFromIpAddress(model.getIpAddresses())
                            .apply();
                    break;
                case IPRANGE:
                    this.getAzure()
                            .storageAccounts()
                            .getById(model.getStorageAccountUuid())
                            .update()
                            .withAccessFromIpAddressRange(model.getIpAddresses())
                            .apply();
                    break;
                default:
                    break;
            }
            return new BocloudResult(true, "withAccessFromIpAddress storageAccount success!");
        } catch (Exception e) {
            logger.error("withAccessFromIpAddress storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult withoutIpAddressAccess(StorageAccountFirewallModel model) {
        try {
            StorageAccountFirewallType firewallType = StorageAccountFirewallType.valueOf(model.getIpType());
            switch (firewallType) {
                case IP:
                    this.getAzure()
                            .storageAccounts()
                            .getById(model.getStorageAccountUuid())
                            .update()
                            .withoutIpAddressAccess(model.getIpAddresses())
                            .apply();
                    break;
                case IPRANGE:
                    this.getAzure()
                            .storageAccounts()
                            .getById(model.getStorageAccountUuid())
                            .update()
                            .withoutIpAddressRangeAccess(model.getIpAddresses())
                            .apply();
                    break;
                default:
                    break;
            }
            return new BocloudResult(true, "withAccessFromIpAddress storageAccount success!");
        } catch (Exception e) {
            logger.error("withoutIpAddressAccess storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }

    @Override
    public BocloudResult regenerateKey(StorageAccountModel model) {
        try {
            List<StorageAccountKey> keys = this.getAzure().storageAccounts().getById(model.getUuid())
                    .regenerateKey(model.getRegenerateKey());
            if (!CollectionUtils.isEmpty(keys)) {
                for (StorageAccountKey key : keys) {
                    String keyName = key.keyName();
                    if ("key1".equalsIgnoreCase(keyName)) {
                        model.setPrimaryKey(key.value());
                    } else {
                        model.setSecondaryKey(key.value());
                    }
                }
            }
            return new BocloudResult(true, model, "regenerateKey storageAccount success!");
        } catch (Exception e) {
            logger.error("regenerateKey storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    private StorageAccount getCreateWaySDK(StorageAccountModel model) {
        StorageAccount.DefinitionStages.WithCreate network = this.withCreateNetwork(model);
        StorageAccount.DefinitionStages.WithCreate kind = this.withCreateKind(model, network);
        StorageAccount.DefinitionStages.WithCreate sku = this.withCreateSku(model, kind);
        return sku.create();
    }

    private StorageAccount.DefinitionStages.WithCreate withCreateNetwork(StorageAccountModel model) {
        StorageAccount.DefinitionStages.WithCreate withCreate = this.getAzure().storageAccounts().define(model.getName())
                .withRegion(model.getRegionId())
                .withExistingResourceGroup(model.getResourceGroupName())
                .withAccessFromAllNetworks();
        return withCreate;
    }

    private StorageAccount.DefinitionStages.WithCreate withCreateKind(StorageAccountModel model, StorageAccount.DefinitionStages.WithCreate withCreate) {

        StorageAccount.DefinitionStages.WithCreate withkind;
        switch (model.getKind()) {
            case "Storage":
                withkind = withCreate.withGeneralPurposeAccountKind();
                break;
            case "StorageV2":
                withkind = withCreate.withGeneralPurposeAccountKindV2();
                break;
            case "BlobStorage":
                withkind = withCreate.withBlobStorageAccountKind().withAccessTier(AccessTier.fromString(model.getAccessTier()));
                break;
            case "FileStorage":
                withkind = withCreate.withFileStorageAccountKind();
                break;
            case "BlockBlobStorage":
                withkind = withCreate.withBlockBlobStorageAccountKind();
                break;
            default:
                withkind = withCreate.withGeneralPurposeAccountKind();
                break;

        }
        return withkind;
    }

    private StorageAccount.DefinitionStages.WithCreate withCreateSku(StorageAccountModel model, StorageAccount.DefinitionStages.WithCreate withCreate) {
        return withCreate.withSku(StorageAccountSkuType.fromSkuName(SkuName.fromString(model.getSku())));
    }

    private StorageAccountModel getCreateWayAPI(StorageAccountModel model) throws Exception {
        AzureRestApiProvider provider = new AzureRestApiProvider(this.getButler());
        String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Storage/storageAccounts/%s?api-version=2021-04-01";
        String url = String.format(api, provider.getSubscriptionId(), model.getResourceGroupName(), model.getName());

        JSONObject params = new JSONObject();
        JSONObject sku = new JSONObject();
        sku.put("name", model.getSku());
        sku.put("tier", null == model.getSkuTier() ? "Standard" : model.getSkuTier());
        params.put("sku", sku);
        params.put("kind", model.getKind());
        params.put("location", model.getRegionId());
        JSONObject properties = new JSONObject();
        properties.put("accessTier", null == model.getAccessTier() ? "Hot" : model.getAccessTier());
        if (!StringUtils.isEmpty(model.getMinimumTlsVersion())) {
            properties.put("minimumTlsVersion", model.getMinimumTlsVersion());
        }
        //不传默认是true
        //properties.put("allowBlobPublicAccess", true);
        //properties.put("allowCrossTenantReplication", true);
        //properties.put("allowSharedKeyAccess", true);
        params.put("properties", properties);
        //没有返回值
        provider.request(AzureRestApiProvider.RequestStatus.PUT, url, params.toJSONString());
        StorageAccountModel newModel = this.getResource(model);
        Assert.notNull(newModel, "资源创建失败");
        return newModel;
    }

    public StorageAccountModel getResource(StorageAccountModel oldModel) {
        StorageAccountModel newModel = null;
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            while (endTime - startTime < 15 * 60 * 1000) {
                logger.info("执行轮询存储对象：{}", oldModel.getName());
                Thread.sleep(2 * 60 * 1000);
                BocloudResult detail = this.detailApi(oldModel);
                if (detail.isSuccess()) {
                    JSONObject newObject = JSONObject.parseObject((String) detail.getData());
                    JSONObject properties = newObject.getJSONObject("properties");
                    String status = properties.getString("provisioningState");
                    logger.info("存储对象状态：{} - {}", oldModel.getName(), status);
                    if (status.equalsIgnoreCase("Succeeded")) {
                        String uuid = newObject.getString("id");
                        StorageAccount storageAccount = this.getAzure().storageAccounts().getById(uuid);
                        newModel = convertor.convertModel(storageAccount);
                        logger.info("存储对象：{},创建成功退出轮询状态", oldModel.getName());
                        break;
                    }
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
        }
        return newModel;
    }

    public BocloudResult detailApi(StorageAccountModel model) {
        try {
            AzureRestApiProvider provider = new AzureRestApiProvider(this.getButler());
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Storage/storageAccounts/%s?api-version=2021-04-01";
            String url = String.format(api, provider.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            String response = provider.request(AzureRestApiProvider.RequestStatus.GET, url, null);
            return new BocloudResult(true, response, "查询成功");
        } catch (Exception e) {
            logger.error("detail storageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

}