package com.bocloud.cmp.provider.azure.db;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.db.CloudDatabaseServerConvertor;
import com.bocloud.cmp.entity.enums.azure.PerformanceTierType;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.db.AzureDatabaseServerModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.DateTimeTool;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureDatabaseForMysqlProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/12/18 13:44
 */
@Provider("bocloud.cmp.azure")
public class AzureCloudDatabaseServerProvider extends AzureRestApiProvider implements CloudDatabaseServerProvider {

    public AzureCloudDatabaseServerProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureCloudDatabaseServerProvider.class);

    private BeanJsonConvertor<AzureDatabaseServerModel> convertor = new CloudDatabaseServerConvertor();

    @Override
    public BocloudResult list() {
        try {
            //url -> 订阅id、资源组名称、自动化账户名称
            String api = "subscriptions/%s/providers/Microsoft.DBForMySQL/servers?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<AzureDatabaseServerModel> models = convertor.convertListParams(jsonObject.getJSONArray("value"), PerformanceTierType.MYSQL.getValue()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            for (AzureDatabaseServerModel databaseServerModel : models) {
                List<String> replicaUuids = new ArrayList<>();
                List<AzureDatabaseServerModel> replicaModels = listReplicas(databaseServerModel);
                if (!CollectionUtils.isEmpty(replicaModels)) {
                    for (AzureDatabaseServerModel replicaModel : replicaModels) {
                        replicaUuids.add(replicaModel.getDatabaseServerUuid());
                    }
                }
                databaseServerModel.setReplicaUuids(replicaUuids);
            }
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    /**
     * properties 目前采用第一种方案
     * ServerPropertiesForDefaultCreate 用于创建新服务器的属性。
     * ServerPropertiesForRestore 用于通过从备份还原来创建新服务器的属性
     * ServerPropertiesForGeoRestore 用于通过从异地复制备份还原到不同区域来创建新服务器的属性。
     * ServerPropertiesForReplica 用于创建新副本的属性。
     */
    @Override
    public BocloudResult create(AzureDatabaseServerModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            JSONObject params = this.createMode(model);
            logger.info("mysql创建参数:{}", params.toJSONString());
            String response = this.request(RequestStatus.PUT, url, params.toJSONString());
            logger.info("mysql创建response:{}", response);
            JSONObject jsonObject = JSONObject.parseObject(response);
            AzureDatabaseServerModel databaseModel;
            if (null != jsonObject.getString("operation")) {
                //数据库处于创建状态，需要根据名称查询
                databaseModel = this.getResource(model);
            } else {
                databaseModel = convertor.convertModelParams(jsonObject, PerformanceTierType.MYSQL.getValue());
            }
            if (ObjectUtils.isEmpty(databaseModel)) {
                return new BocloudResult(false, "创建失败");
            } else {
                return new BocloudResult(true, databaseModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    /**
     * 区分创建方式
     *
     * @param model
     * @return
     */
    private JSONObject createMode(AzureDatabaseServerModel model) {
        String createMode = model.getCreateMode() == null ? "Default" : model.getCreateMode();
        JSONObject jsonObject;
        switch (createMode) {
            case "Default":
                jsonObject = this.createDefault(model);
                break;
            case "Replica":
                jsonObject = this.createReplica(model);
                break;
            case "GeoRestore":
                jsonObject = this.createGeoRestore(model);
                break;
            case "PointInTimeRestore":
                jsonObject = this.createPointInTimeRestore(model);
                break;
            default:
                jsonObject = this.createDefault(model);
                break;
        }
        return jsonObject;
    }

    /**
     * ServerPropertiesForDefaultCreate 用于创建新服务器的属性。
     *
     * @param model
     * @return
     */
    private JSONObject createDefault(AzureDatabaseServerModel model) {
        JSONObject params = new JSONObject();
        JSONObject properties = new JSONObject();
        //服务器的管理员登录名。 只能在 (创建服务器时指定，并且是创建) 所必需的。
        properties.put("administratorLogin", model.getAdministratorName());
        //管理员登录名的密码。
        properties.put("administratorLoginPassword", model.getAdministratorPassword());
        //是否允许访问此服务器的公共网络。 值是可选的，但是如果传入了，则必须为 "Enabled" 或 "Disabled"
        properties.put("publicNetworkAccess", model.getPublicNetworkAccess());
        //在连接到服务器时启用 ssl 强制。但是如果传入了，则必须为 "Enabled" 或 "Disabled"
        properties.put("sslEnforcement", model.getSslEnforcement());
        if (model.getSslEnforcement().equals("Enabled")) {
            properties.put("minimalTlsVersion", model.getMinimalTlsVersion());
        }
        //用于创建新服务器的模式
        properties.put("createMode", "Default");
        //服务器版本。5.6/ 5.7 /8.0
        properties.put("version", model.getVersion());
        //显示服务器是否启用了基础结构加密的状态。"Enabled" 或 "Disabled"
        properties.put("infrastructureEncryption", model.getInfrastructureEncryption());
        //服务器的存储配置文件。
        JSONObject storageProfile = new JSONObject();
        //服务器允许的最大存储空间。
        storageProfile.put("storageMB", model.getStorageMb());
        //服务器的备份保留天数。
        storageProfile.put("backupRetentionDays", model.getBackupRetentionDays());
        //为服务器备份启用异地冗余。 "Enabled" 或 "Disabled"
        storageProfile.put("geoRedundantBackup", model.getGeoRedundantBackup());
        //启用存储自动增长 "Enabled" 或 "Disabled"
        storageProfile.put("storageAutogrow", model.getStorageAutogrow());
        properties.put("storageProfile", storageProfile);

        JSONObject sku = new JSONObject();
        sku.put("name", model.getSloModel().getUuid());//Sku 的名称，通常为层 + 系列 + 核心，如 B_Gen4_1，GP_Gen5_8。
        sku.put("tier", model.getSloModel().getEdition());//特定 SKU 的层，如 "基本"。Basic/GeneralPurpose/MemoryOptimized
        sku.put("capacity", model.getSloModel().getvCore());//用于表示服务器计算单元的向上扩展/向外扩展容量。
        sku.put("family", model.getSloModel().getHardwareGeneration());//硬件系列。
        JSONObject tags = new JSONObject();
        tags.put("ElasticServer", "1");

        params.put("location", model.getRegionId());
        params.put("properties", properties);
        params.put("sku", sku);
        params.put("tags", tags);
        return params;
    }

    /**
     * ServerPropertiesForReplica 用于创建新副本的属性。
     *
     * @param model
     * @return
     */
    private JSONObject createReplica(AzureDatabaseServerModel model) {
        JSONObject params = new JSONObject();
        JSONObject properties = new JSONObject();
        properties.put("sourceServerId", model.getMasterServerUuid());
        properties.put("createMode", "Replica");
        params.put("location", model.getRegionId());
        params.put("properties", properties);
        return params;
    }

    /**
     * ServerPropertiesForGeoRestore 用于通过从异地复制备份还原到不同区域来创建新服务器的属性。
     *
     * @param model
     * @return
     */
    private JSONObject createGeoRestore(AzureDatabaseServerModel model) {
        JSONObject params = new JSONObject();
        JSONObject properties = new JSONObject();
        properties.put("sourceServerId", model.getMasterServerUuid());
        properties.put("createMode", "GeoRestore");
        params.put("location", model.getRegionId());
        params.put("properties", properties);
        JSONObject sku = new JSONObject();
        sku.put("name", model.getSloModel().getUuid());//Sku 的名称，通常为层 + 系列 + 核心，如 B_Gen4_1，GP_Gen5_8。
        sku.put("tier", model.getSloModel().getEdition());//特定 SKU 的层，如 "基本"。Basic/GeneralPurpose/MemoryOptimized
        sku.put("capacity", model.getSloModel().getvCore());//用于表示服务器计算单元的向上扩展/向外扩展容量。
        sku.put("family", model.getSloModel().getHardwareGeneration());//硬件系列。
        params.put("sku", sku);
        return params;
    }

    /**
     * ServerPropertiesForRestore。 用于通过从备份还原来创建新服务器的属性。。
     *
     * @param model
     * @return
     */
    private JSONObject createPointInTimeRestore(AzureDatabaseServerModel model) {
        JSONObject params = new JSONObject();
        params.put("location", model.getRegionId());
        JSONObject properties = new JSONObject();
        properties.put("createMode", "PointInTimeRestore");
        properties.put("restorePointInTime", DateTimeTool.getUtcTime(model.getRestorePointInTime()));
        properties.put("sourceServerId", model.getMasterServerUuid());
        //显示服务器是否启用了基础结构加密的状态。"Enabled" 或 "Disabled"
        properties.put("infrastructureEncryption", model.getInfrastructureEncryption());
        params.put("properties", properties);
        return params;
    }


    public AzureDatabaseServerModel getResource(AzureDatabaseServerModel oldModel) {
        AzureDatabaseServerModel newModel = null;
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            while (endTime - startTime < 3 * 60 * 60 * 1000) {
                logger.info("执行轮询数据库服务器：{}", oldModel.getName());
                Thread.sleep(2 * 60 * 1000);
                BocloudResult detail = this.detail(oldModel);
                if (detail.isSuccess()) {
                    newModel = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), AzureDatabaseServerModel.class);
                    logger.info("数据库服务器状态：{} - {}", newModel.getName(), newModel.getStatus());
                    if (newModel.getStatus().equalsIgnoreCase("Ready")) {
                        logger.info("数据库服务器创建成功，即将退出轮询状态！：{} - {}", newModel.getName(), newModel.getStatus());
                        break;
                    }
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
        }
        return newModel;
    }

    @Override
    public BocloudResult modify(AzureDatabaseServerModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            JSONObject params = new JSONObject();
            JSONObject properties = new JSONObject();
            if (!StringUtils.isEmpty(model.getAdministratorPassword())) {
                //管理员登录名的密码。
                properties.put("administratorLoginPassword", model.getAdministratorPassword());
            }
            if (!StringUtils.isEmpty(model.getPublicNetworkAccess())) {
                //是否允许访问此服务器的公共网络。 值是可选的，但是如果传入了，则必须为 "Enabled" 或 "Disabled"
                properties.put("publicNetworkAccess", model.getPublicNetworkAccess());
            }
            if (!StringUtils.isEmpty(model.getSslEnforcement())) {
                //在连接到服务器时启用 ssl 强制。但是如果传入了，则必须为 "Enabled" 或 "Disabled"
                properties.put("sslEnforcement", model.getSslEnforcement());
                if (model.getSslEnforcement().equals("Enabled")) {
                    properties.put("minimalTlsVersion", model.getMinimalTlsVersion());
                }
            }
            params.put("properties", properties);
            String response = this.request(RequestStatus.PATCH, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            AzureDatabaseServerModel databaseModel;
            if (null != jsonObject.getString("operation")) {
                //数据库处于创建状态，需要根据名称查询
                databaseModel = this.getResource(model);
            } else {
                databaseModel = convertor.convertModelParams(jsonObject, PerformanceTierType.MYSQL.getValue());
            }
            if (ObjectUtils.isEmpty(databaseModel)) {
                return new BocloudResult(false, "修改失败");
            } else {
                return new BocloudResult(true, databaseModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(AzureDatabaseServerModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            AzureDatabaseServerModel databaseModel = convertor.convertModelParams(jsonObject, PerformanceTierType.MYSQL.getValue());
            return new BocloudResult(true, databaseModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(AzureDatabaseServerModel model) {
        try {
            String api;
            switch (model.getType()) {
                case "Microsoft.DBforMySQL/servers":
                    api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s?api-version=2017-12-01";
                    break;
                default:
                    api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s?api-version=2017-12-01";
                    break;
            }
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            this.request(RequestStatus.DELETE, url, null);
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult restart(AzureDatabaseServerModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/restart?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            String response = this.request(RequestStatus.POST, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            AzureDatabaseServerModel databaseModel;
            if (null != jsonObject.getString("operation")) {
                //数据库处于创建状态，需要根据名称查询
                databaseModel = this.getResource(model);
            } else {
                databaseModel = convertor.convertModelParams(jsonObject, PerformanceTierType.MYSQL.getValue());
            }
            if (ObjectUtils.isEmpty(databaseModel)) {
                return new BocloudResult(false, "重启失败");
            } else {
                return new BocloudResult(true, databaseModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult pricingLayer(AzureDatabaseServerModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            JSONObject params = new JSONObject();
            JSONObject properties = new JSONObject();
            //服务器的存储配置文件。
            JSONObject storageProfile = new JSONObject();
            //服务器允许的最大存储空间。
            storageProfile.put("storageMB", model.getStorageMb());
            //服务器的备份保留天数。
            storageProfile.put("backupRetentionDays", model.getBackupRetentionDays());
//            //为服务器备份启用异地冗余。 "Enabled" 或 "Disabled"
//            storageProfile.put("geoRedundantBackup", model.getGeoRedundantBackup());
            //启用存储自动增长 "Enabled" 或 "Disabled"
            storageProfile.put("storageAutogrow", model.getStorageAutogrow());
            properties.put("storageProfile", storageProfile);

            JSONObject sku = new JSONObject();
            sku.put("name", model.getSloModel().getUuid());//Sku 的名称，通常为层 + 系列 + 核心，如 B_Gen4_1，GP_Gen5_8。
            sku.put("tier", model.getSloModel().getEdition());//特定 SKU 的层，如 "基本"。Basic/GeneralPurpose/MemoryOptimized
            sku.put("capacity", model.getSloModel().getvCore());//用于表示服务器计算单元的向上扩展/向外扩展容量。
            sku.put("family", model.getSloModel().getHardwareGeneration());//硬件系列。
            params.put("location", model.getRegionId());
            params.put("properties", properties);
            params.put("sku", sku);
            logger.info("mysql调整配置参数:{}", params.toJSONString());
            this.request(RequestStatus.PUT, url, params.toJSONString());
            Thread.sleep(2 * 60 * 1000);
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    public List<AzureDatabaseServerModel> listReplicas(AzureDatabaseServerModel model) {
        String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/replicas?api-version=2017-12-01";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
        String response = this.request(RequestStatus.GET, url, null);
        JSONObject jsonObject = JSONObject.parseObject(response);
        List<AzureDatabaseServerModel> models = convertor.convertListParams(jsonObject.getJSONArray("value"), PerformanceTierType.MYSQL.getValue());
        return models;
    }

}