package com.bocloud.cmp.provider.azure.db.configuration;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.db.DbConfigurationConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.db.AzureDatabaseServerModel;
import com.bocloud.cmp.model.azure.db.DbConfigurationModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.ObjectUtils;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureCloudDatabaseServerConfigurationProvider
 * @Description:服务器参数
 * @author: zhenpingLi/李振平
 * @date: 2021/2/4 17:44
 */
@Provider("bocloud.cmp.azure")
public class AzureDbConfigurationProvider extends AzureRestApiProvider implements DbConfigurationProvider {

    public AzureDbConfigurationProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureDbConfigurationProvider.class);

    private BeanJsonConvertor<DbConfigurationModel> convertor = new DbConfigurationConvertor();

    @Override
    public BocloudResult list(AzureDatabaseServerModel databaseModel) {
        try {
            //url -> 订阅id、资源组名称、自动化账户名称
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/configurations?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), databaseModel.getResourceGroupName(), databaseModel.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<DbConfigurationModel> models = convertor.convertListParams(jsonObject.getJSONArray("value"), databaseModel.getDatabaseServerUuid());
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(DbConfigurationModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/configurations/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getDatabaseServerName(), model.getName());
            JSONObject params = new JSONObject();
            JSONObject properties = new JSONObject();
            properties.put("value", model.getConfigurationValue());
            properties.put("source", model.getSource());
            params.put("properties", properties);
            String response = this.request(RequestStatus.PUT, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            DbConfigurationModel configurationModel;
            if (null != jsonObject.getString("operation")) {
                //数据库处于创建状态，需要根据名称查询
                configurationModel = this.getResource(model);
            } else {
                configurationModel = convertor.convertModelParams(jsonObject, model.getDatabaseServerUuid());
            }
            if (ObjectUtils.isEmpty(configurationModel)) {
                return new BocloudResult(false, "创建失败");
            } else {
                return new BocloudResult(true, configurationModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    public DbConfigurationModel getResource(DbConfigurationModel dbConfigurationModel) {
        DbConfigurationModel model = null;
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            while (endTime - startTime < 30 * 60 * 1000) {
                logger.info("执行轮询数据库服务器配置：{}", dbConfigurationModel.getName());
                Thread.sleep(60 * 1000);
                BocloudResult detail = this.detail(dbConfigurationModel);
                if (detail.isSuccess()) {
                    model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), DbConfigurationModel.class);
                    if (dbConfigurationModel.getConfigurationValue().equalsIgnoreCase(model.getConfigurationValue())) {
                        break;
                    }
                    break;
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
        }
        return model;
    }

    @Override
    public BocloudResult detail(DbConfigurationModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/configurations/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getDatabaseServerName(), model.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            DbConfigurationModel ruleModel = convertor.convertModelParams(jsonObject, model.getDatabaseServerUuid());
            return new BocloudResult(true, ruleModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


}