package com.bocloud.cmp.provider.azure.db.firewall;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.db.DbFirewallRuleConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.db.AzureDatabaseServerModel;
import com.bocloud.cmp.model.azure.db.DbFirewallRuleModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.ObjectUtils;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureDbFirewallRuleProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/2/1 14:00
 */
@Provider("bocloud.cmp.azure")
public class AzureDbFirewallRuleProvider extends AzureRestApiProvider implements DbFirewallRuleProvider {

    public AzureDbFirewallRuleProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureDbFirewallRuleProvider.class);

    private BeanJsonConvertor<DbFirewallRuleModel> convertor = new DbFirewallRuleConvertor();

    @Override
    public BocloudResult list(AzureDatabaseServerModel databaseModel) {
        try {
            //url -> 订阅id、资源组名称、自动化账户名称
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/firewallRules?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), databaseModel.getResourceGroupName(), databaseModel.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<DbFirewallRuleModel> ruleModels = convertor.convertListParams(jsonObject.getJSONArray("value"), databaseModel.getDatabaseServerUuid());
            return new BocloudResult(true, ruleModels, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(DbFirewallRuleModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/firewallRules/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getDatabaseServerName(), model.getName());
            this.request(RequestStatus.DELETE, url, null);
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(DbFirewallRuleModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/firewallRules/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getDatabaseServerName(), model.getName());
            JSONObject params = new JSONObject();
            JSONObject properties = new JSONObject();
            properties.put("startIpAddress", model.getStartIpAddress());
            properties.put("endIpAddress", model.getEndIpAddress());
            params.put("properties", properties);
            String response = this.request(RequestStatus.PUT, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            DbFirewallRuleModel ruleModel;
            if (null != jsonObject.getString("operation")) {
                //数据库处于创建状态，需要根据名称查询
                ruleModel = this.getResource(model);
            } else {
                ruleModel = convertor.convertModelParams(jsonObject, model.getDatabaseServerUuid());
            }
            if (ObjectUtils.isEmpty(ruleModel)) {
                return new BocloudResult(false, "创建失败");
            } else {
                return new BocloudResult(true, ruleModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(DbFirewallRuleModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/firewallRules/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getDatabaseServerName(), model.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            DbFirewallRuleModel ruleModel = convertor.convertModelParams(jsonObject, model.getDatabaseServerUuid());
            return new BocloudResult(true, ruleModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    public DbFirewallRuleModel getResource(DbFirewallRuleModel dbFirewallRuleModel) {
        DbFirewallRuleModel model = null;
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            while (endTime - startTime < 30 * 60 * 1000) {
                logger.info("执行轮询数据库服务器防火墙：{}", dbFirewallRuleModel.getName());
                Thread.sleep(60 * 1000);
                BocloudResult detail = this.detail(dbFirewallRuleModel);
                if (detail.isSuccess()) {
                    model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), DbFirewallRuleModel.class);
                    break;
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
        }
        return model;
    }

}