package com.bocloud.cmp.provider.azure.db.virtualnetwork;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.db.DbVnetRuleConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.db.AzureDatabaseServerModel;
import com.bocloud.cmp.model.azure.db.DbVnetRuleModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.ServiceEndpointType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.ObjectUtils;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureCloudDatabaseServerVirtualNetworkRuleProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/2/4 17:06
 */
@Provider("bocloud.cmp.azure")
public class AzureDbVnetRuleProvider extends AzureRestApiProvider implements DbVnetRuleProvider {

    public AzureDbVnetRuleProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureDbVnetRuleProvider.class);

    private BeanJsonConvertor<DbVnetRuleModel> convertor = new DbVnetRuleConvertor();

    @Override
    public BocloudResult list(AzureDatabaseServerModel databaseModel) {
        try {
            //url -> 订阅id、资源组名称、自动化账户名称
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/virtualNetworkRules?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), databaseModel.getResourceGroupName(), databaseModel.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<DbVnetRuleModel> ruleModels = convertor.convertListParams(jsonObject.getJSONArray("value"), databaseModel.getDatabaseServerUuid());
            return new BocloudResult(true, ruleModels, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(DbVnetRuleModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/virtualNetworkRules/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getDatabaseServerName(), model.getName());
            this.request(RequestStatus.DELETE, url, null);
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(DbVnetRuleModel model) {
        try {
            AzureProvider provider = new AzureProvider(this.getButler());
            provider.getAzure().networks().getById(model.getVpcUuid())
                    .update()
                    .updateSubnet(model.getSubnetName())
                    .withAccessFromService(ServiceEndpointType.MICROSOFT_SQL)
                    .parent()
                    .apply();
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/virtualNetworkRules/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getDatabaseServerName(), model.getName());
            JSONObject params = new JSONObject();
            JSONObject properties = new JSONObject();
            properties.put("ignoreMissingVnetServiceEndpoint", model.getIgnoreMissingVnetServiceEndpoint());
            properties.put("virtualNetworkSubnetId", model.getSubnetUuid());
            params.put("properties", properties);
            String response = this.request(RequestStatus.PUT, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            DbVnetRuleModel ruleModel;
            if (null != jsonObject.getString("operation")) {
                //数据库处于创建状态，需要根据名称查询
                ruleModel = this.getResource(model);
            } else {
                ruleModel = convertor.convertModelParams(jsonObject, model.getDatabaseServerUuid());
            }
            if (ObjectUtils.isEmpty(ruleModel)) {
                return new BocloudResult(false, "创建失败");
            } else {
                return new BocloudResult(true, ruleModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(DbVnetRuleModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/virtualNetworkRules/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getDatabaseServerName(), model.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            DbVnetRuleModel ruleModel = convertor.convertModelParams(jsonObject, model.getDatabaseServerUuid());
            return new BocloudResult(true, ruleModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    public DbVnetRuleModel getResource(DbVnetRuleModel dbVnetRuleModel) {
        DbVnetRuleModel model = null;
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            //轮询20m
            while (endTime - startTime < 20 * 60 * 1000) {
                //主机创建时间不确定，增加休眠减少轮询次数
                logger.info("执行轮询数据库服务器Vnet规则：{}", dbVnetRuleModel.getName());
                Thread.sleep(60 * 1000);
                BocloudResult detail = this.detail(dbVnetRuleModel);
                if (detail.isSuccess()) {
                    model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), DbVnetRuleModel.class);
                    break;
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
        }
        return model;
    }


}