package com.bocloud.cmp.provider.azure.network;

import com.bocloud.cmp.azureconvertor.network.RouterTableConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.RouterModel;
import com.bocloud.cmp.model.azure.network.RouterTableModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.Route;
import com.microsoft.azure.management.network.RouteNextHopType;
import com.microsoft.azure.management.network.RouteTable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureRouterTableProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/12/31 14:34
 */
@Provider("bocloud.cmp.azure")
public class AzureRouterTableProvider extends AzureProvider implements RouterTableProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureRouterTableProvider.class);

    private BeanConvertor<RouterTableModel, RouteTable> convertor = new RouterTableConvertor();

    public AzureRouterTableProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            List<RouterTableModel> regionModels = convertor.convertList(this.getAzure().routeTables().list()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            return new BocloudResult(true, regionModels, "list routeTable success!");
        } catch (Exception e) {
            logger.error("list routerTable error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(RouterTableModel model) {
        try {
            RouteTable routeTable;
            if (model.getRoutePropagation()) {
                routeTable = this.getAzure().routeTables()
                        .define(model.getName())
                        .withRegion(model.getRegionId())
                        .withExistingResourceGroup(model.getResourceGroupName())
                        .create();
            } else {
                routeTable = this.getAzure().routeTables()
                        .define(model.getName())
                        .withRegion(model.getRegionId())
                        .withExistingResourceGroup(model.getResourceGroupName())
                        .withDisableBgpRoutePropagation()
                        .create();
            }
            return new BocloudResult(true, convertor.convertModel(routeTable), "create routerTable success!");
        } catch (Exception e) {
            logger.error("create routerTable error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(RouterTableModel model) {
        try {
            if (model.getRoutePropagation()) {
                this.getAzure().routeTables().getById(model.getUuid())
                        .update()
                        .withEnableBgpRoutePropagation()
                        .apply();
            } else {
                this.getAzure().routeTables().getById(model.getUuid())
                        .update()
                        .withDisableBgpRoutePropagation()
                        .apply();
            }
            return new BocloudResult(true, "modify routerTable success!");
        } catch (Exception e) {
            logger.error("modify routerTable error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String uuid) {
        try {
            this.getAzure().routeTables().deleteById(uuid);
            return new BocloudResult(true, "remove routerTable success!");
        } catch (Exception e) {
            logger.error("remove routerTable error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String uuid) {
        try {
            RouteTable routeTable = this.getAzure().routeTables().getById(uuid);
            return new BocloudResult(true, convertor.convertModel(routeTable), "detail routerTable success!");
        } catch (Exception e) {
            logger.error("detail routerTable error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override //弃用
    public BocloudResult addRouter(RouterModel model) {
        try {
            RouteTable routeTable = this.getAzure()
                    .routeTables()
                    .getById(model.getRouterTableUuid())
                    .update()
                    .defineRoute(model.getName())
                    .withDestinationAddressPrefix(model.getAddress())
                    .withNextHop(RouteNextHopType.fromString(model.getNextHopType()))
                    .attach()
                    .apply();
            Route route = routeTable.routes().get(model.getName());
            model.setRouterUuid(route.inner().id());
            model.setAddress(route.destinationAddressPrefix());
            model.setRouterType(route.nextHopType().toString());
            model.setRegionId(route.parent().regionName());
            model.setResourceGroupName(route.parent().resourceGroupName());
            model.setRouterTableUuid(route.parent().id());
            model.setNextHopType(route.nextHopType().toString());
            model.setNextHopIpAddress(route.nextHopIPAddress());
            return new BocloudResult(true, model, "addRouter routerTable success!");
        } catch (Exception e) {
            logger.error("addRouter routerTable error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }

    @Override//弃用
    public BocloudResult removeRouter(RouterModel model) {
        try {
            this.getAzure()
                    .routeTables()
                    .getById(model.getRouterTableUuid())
                    .update()
                    .withoutRoute(model.getName())
                    .apply();
            return new BocloudResult(true, "removeRouter routerTable success!");
        } catch (Exception e) {
            logger.error("removeRouter routerTable error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }


}