package com.bocloud.cmp.provider.azure.redis;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.redis.RedisConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.redis.RedisModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureRedisProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/2/25 14:15
 */
@Provider("bocloud.cmp.azure")
public class AzureRedisProvider extends AzureRestApiProvider implements RedisProvider {

    public AzureRedisProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureRedisProvider.class);

    private BeanJsonConvertor<RedisModel> convertor = new RedisConvertor();

    @Override
    public BocloudResult list() {
        try {
            //url -> 订阅id、资源组名称、自动化账户名称
            String api = "subscriptions/%s/providers/Microsoft.Cache/redis?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<RedisModel> models = convertor.convertList(jsonObject.getJSONArray("value")).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            for (RedisModel redisModel : models) {
                Map<String, String> keys = this.getKeys(redisModel);
                redisModel.setPrimaryKey(keys.get("primaryKey"));
                redisModel.setSecondaryKey(keys.get("secondaryKey"));
                Map<String, String> csMap = this.getConnectionString(redisModel);
                redisModel.setPrimaryConnectionString(csMap.get("pcs"));
                redisModel.setSecondaryConnectionString(csMap.get("scs"));
            }
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(RedisModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Cache/redis/%s?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            RedisModel redisModel = convertor.convertModel(jsonObject);
            Map<String, String> keys = this.getKeys(redisModel);
            redisModel.setPrimaryKey(keys.get("primaryKey"));
            redisModel.setSecondaryKey(keys.get("secondaryKey"));
            Map<String, String> csMap = this.getConnectionString(redisModel);
            redisModel.setPrimaryConnectionString(csMap.get("pcs"));
            redisModel.setSecondaryConnectionString(csMap.get("scs"));
            return new BocloudResult(true, redisModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(RedisModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Cache/redis/%s?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            this.request(RequestStatus.DELETE, url, null);
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult regenerateKey(RedisModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Cache/redis/%s/regenerateKey?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            JSONObject object = new JSONObject();
            object.put("keyType", model.getKeyType());
            String response = this.request(RequestStatus.POST, url, object.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            model.setPrimaryKey(jsonObject.getString("primaryKey"));
            model.setSecondaryKey(jsonObject.getString("secondaryKey"));
            Map<String, String> csMap = this.getConnectionString(model);
            model.setPrimaryConnectionString(csMap.get("pcs"));
            model.setSecondaryConnectionString(csMap.get("scs"));
            return new BocloudResult(true, model, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(RedisModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Cache/redis/%s?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            JSONObject params = new JSONObject();

            JSONObject properties = new JSONObject();

            JSONObject sku = new JSONObject();
            sku.put("name", model.getSkuName());
            sku.put("family", model.getSkuFamily());
            sku.put("capacity", model.getSkuCapacity());
            properties.put("sku", sku);
            properties.put("enableNonSslPort", model.getEnableNonSslPort());
            properties.put("minimumTlsVersion", model.getMinimumTlsVersion());
            properties.put("publicNetworkAccess", model.getPublicNetworkAccess());
            //使用副本就不能使用集群 ，他俩互斥
            properties.put("replicasPerMaster", model.getReplicasPerMaster());
            properties.put("shardCount", model.getShardCount());
            //指定subnetId怎么redis部署在虚拟网络中
            properties.put("subnetId", model.getSubnetUuid());
            //不指定静态ip则随机在子网分配IP地址
            properties.put("staticIP", model.getStaticIp());
            //redisConfiguration
            JSONObject redisConfiguration = new JSONObject();
            redisConfiguration.put("maxmemory-policy", model.getMaxmemoryPolicy());
            redisConfiguration.put("maxmemory-reserved", model.getMaxmemoryReserved());
            redisConfiguration.put("maxfragmentationmemory-reserved", model.getMaxfragmentationmemoryReserved());
            properties.put("redisConfiguration", redisConfiguration);

            params.put("location", model.getRegionId());
            params.put("properties", properties);
            String response = this.request(RequestStatus.PUT, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            RedisModel redisModel = this.getResource(convertor.convertModel(jsonObject));
            Assert.notNull(redisModel, "创建失败");
            return new BocloudResult(true, redisModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(RedisModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Cache/redis/%s?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            JSONObject params = new JSONObject();
            JSONObject properties = new JSONObject();
            properties.put("enableNonSslPort", model.getEnableNonSslPort());
            properties.put("minimumTlsVersion", model.getMinimumTlsVersion());
            properties.put("publicNetworkAccess", model.getPublicNetworkAccess());
            JSONObject redisConfiguration = new JSONObject();
            redisConfiguration.put("maxmemory-policy", model.getMaxmemoryPolicy());
            redisConfiguration.put("maxmemory-reserved", model.getMaxmemoryReserved());
            redisConfiguration.put("maxfragmentationmemory-reserved", model.getMaxfragmentationmemoryReserved());
            properties.put("redisConfiguration", redisConfiguration);
            params.put("properties", properties);
            String response = this.request(RequestStatus.PATCH, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            RedisModel redisModel = convertor.convertModel(jsonObject);
            return new BocloudResult(true, redisModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }

    @Override //暂时没调通
    public BocloudResult restart(RedisModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Cache/redis/%s/forceReboot?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            JSONObject params = new JSONObject();
            //重启节点 AllNodes  PrimaryNode SecondaryNode
            params.put("rebootType", "AllNodes");
            params.put("shardId", "0");
            Integer[] aaa = new Integer[]{13000, 15001};
            params.put("ports", aaa);
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            System.out.println("model = [" + model + "]");
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult pricingLayer(RedisModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Cache/redis/%s?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            JSONObject params = new JSONObject();
            JSONObject properties = new JSONObject();
            JSONObject sku = new JSONObject();
            sku.put("name", model.getSkuName());
            sku.put("family", model.getSkuFamily());
            sku.put("capacity", model.getSkuCapacity());
            properties.put("sku", sku);
            if (null != model.getShardCount()) {
                properties.put("shardCount", model.getShardCount());
            }
            params.put("location", model.getRegionId());
            params.put("properties", properties);
            String response = this.request(RequestStatus.PUT, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            RedisModel redisModel = this.getResource(convertor.convertModel(jsonObject));
            return new BocloudResult(true, redisModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    private RedisModel getResource(RedisModel oldModel) {
        RedisModel newModel = null;
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            //轮询30m
            while (endTime - startTime < 3 * 60 * 60 * 1000) {
                logger.info("执行轮询Redis服务器：{}", oldModel.getName());
                Thread.sleep(2 * 60 * 1000);
                BocloudResult detail = this.detail(oldModel);
                if (detail.isSuccess()) {
                    newModel = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), RedisModel.class);
                    logger.info("Redis服务器状态：{} - {}", newModel.getName(), newModel.getStatus());
                    if (newModel.getStatus().equalsIgnoreCase("Succeeded")) {
                        logger.info("Redis服务器获取状态成功，即将退出轮询状态！：{} - {}", newModel.getName(), newModel.getStatus());
                        break;
                    }
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
        }
        return newModel;
    }

    private Map<String, String> getKeys(RedisModel model) {
        Map<String, String> map = new HashMap<>();
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Cache/redis/%s/listKeys?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            String response = this.request(RequestStatus.POST, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            map.put("primaryKey", jsonObject.getString("primaryKey"));
            map.put("secondaryKey", jsonObject.getString("secondaryKey"));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
        }
        return map;
    }

    private Map<String, String> getConnectionString(RedisModel model) {
        Map<String, String> map = new HashMap<>();

        //----------------------------主连接字符串
        StringBuffer pcsBuffer = new StringBuffer()
                .append(model.getHostName());
//        if (model.getEnableNonSslPort()) {
        pcsBuffer.append(":")
                .append(model.getSslPort());
//        }
        pcsBuffer.append(",")
                .append("password=")
                .append(model.getPrimaryKey())
                .append(",ssl=")
                .append(true)
                .append(",abortConnect=False")
                .toString();

        //----------------------------辅助连接字符串
        StringBuffer scsBuffer = new StringBuffer()
                .append(model.getHostName());
//        if (model.getEnableNonSslPort()) {
        scsBuffer.append(":")
                .append(model.getSslPort());
//        }
        scsBuffer.append(",")
                .append("password=")
                .append(model.getSecondaryKey())
                .append(",ssl=")
                .append(true)
                .append(",abortConnect=False")
                .toString();
        String pcs = pcsBuffer.toString();
        String scs = scsBuffer.toString();
        map.put("pcs", pcs);
        map.put("scs", scs);
        return map;
    }
}