package com.bocloud.cmp.provider.azure.slb;

import com.bocloud.cmp.azureconvertor.slb.LoadBalanceConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.slb.InboundNatPoolModel;
import com.bocloud.cmp.model.azure.slb.SlbModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.LoadBalancer;
import com.microsoft.azure.management.network.LoadBalancerSkuType;
import com.microsoft.azure.management.network.TransportProtocol;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @ClassName: AzureLoadBalanceProvider
 * @Description:负载均衡器
 * @author: zhenpingLi/李振平
 */
@Provider("bocloud.cmp.azure")
public class AzureLoadBalanceProvider extends AzureProvider implements LoadBalanceProvider {

    public AzureLoadBalanceProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureLoadBalanceProvider.class);

    private BeanConvertor<SlbModel, LoadBalancer> convertor = new LoadBalanceConvertor();

    @Override
    public BocloudResult list() {
        try {
            List<SlbModel> regionModels = convertor.convertList(this.getAzure().loadBalancers().list()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            return new BocloudResult(true, regionModels, "list loadBalancer success!");
        } catch (Exception e) {
            logger.error("list loadBalancer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult create(SlbModel model) {
        try {
            //前端网络池
            InboundNatPoolModel inboundNatPoolModel = model.getInboundNatPoolModels().get(0);
            LoadBalancer loadBalancer = null;
            switch (model.getSlbType()) {
                case "INTERNAL":
                    //获取网络对象
                    loadBalancer = this.getAzure().loadBalancers()
                            .define(model.getName())
                            .withRegion(model.getRegionId())
                            .withExistingResourceGroup(model.getResourceGroupName())
                            .defineInboundNatPool(inboundNatPoolModel.getName())
                            .withProtocol(TransportProtocol.fromString(inboundNatPoolModel.getProtocol()))
                            .fromExistingSubnet(model.getVpcUuid(), model.getSubnetName())
                            .fromFrontendPortRange(inboundNatPoolModel.getFrontendPortRangeStart(), inboundNatPoolModel.getFrontendPortRangeEnd())
                            .toBackendPort(inboundNatPoolModel.getBackendPort())
                            .attach()
                            .withSku(LoadBalancerSkuType.STANDARD)
                            .create();
                    break;
                case "EXTERNAL":
                    String publicIPAddressUuid = model.getPublicIPAddressUuids().get(0);
                    loadBalancer = this.getAzure().loadBalancers()
                            .define(model.getName())
                            .withRegion(model.getRegionId())
                            .withExistingResourceGroup(model.getResourceGroupName())
                            .defineInboundNatPool(inboundNatPoolModel.getName())
                            .withProtocol(TransportProtocol.fromString(inboundNatPoolModel.getProtocol()))
                            .fromExistingPublicIPAddress(publicIPAddressUuid)
                            .fromFrontendPortRange(inboundNatPoolModel.getFrontendPortRangeStart(), inboundNatPoolModel.getFrontendPortRangeEnd())
                            .toBackendPort(inboundNatPoolModel.getBackendPort())
                            .attach()
                            .withSku(LoadBalancerSkuType.STANDARD)
                            .create();
                    break;
                default:
                    break;
            }
            return new BocloudResult(true, convertor.convertModel(loadBalancer), "create loadBalancer success!");
        } catch (Exception e) {
            logger.error("create loadBalancer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String uuid) {
        try {
            this.getAzure().loadBalancers().deleteById(uuid);
            return new BocloudResult(true, "remove loadBalancer success!");
        } catch (Exception e) {
            logger.error("remove loadBalancer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


}