package com.bocloud.cmp.provider.blob;

import com.bocloud.cmp.azureconvertor.blob.BlobConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.account.StorageAccountModel;
import com.bocloud.cmp.model.azure.blob.BlobModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.storage.StorageCredentials;
import com.microsoft.azure.storage.StorageCredentialsAccountAndKey;
import com.microsoft.azure.storage.StorageUri;
import com.microsoft.azure.storage.blob.*;
import io.jsonwebtoken.lang.Assert;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;

import java.io.File;
import java.io.OutputStream;
import java.net.URI;
import java.util.Date;

/**
 * 对象存储blob访问
 *
 * @author wangyu
 * @version 5.0.0-SNAPSHOT
 * @since 2021/2/1
 */
@Slf4j
@Provider("bocloud.cmp.azure")
public class AzureBlobProvider extends AzureProvider implements BlobProvider {

    private static final BeanConvertor<BlobModel, CloudBlob> BLOB_CONVERTER = new BlobConvertor();


    public AzureBlobProvider(Butler butler) {
        super(butler);
    }

    /**
     * 上传文件到容器
     *
     * @param file          要上传的文件
     * @param containerName 容器名
     * @param account       存储账号
     * @return
     */
    @Override
    public BocloudResult upload(File file, String blobType, String standardBlobTier, String containerName, StorageAccountModel account) {
        try {
            Assert.notNull(account.getName(), "存储账号为空");
            Assert.notNull(account.getPrimaryKey(), "存储账号primary key");
            Assert.notNull(account.getPrimaryEndPoint(), "存储账号primary endpoint");
            StorageCredentials credentials = new StorageCredentialsAccountAndKey(account.getName(), account.getPrimaryKey());
            URI primaryUri = new URI(account.getPrimaryEndPoint());
            URI secondaryUri = null;
            if (org.springframework.util.StringUtils.hasText(account.getSecondaryEndPoint())) {
                secondaryUri = new URI(account.getSecondaryEndPoint());
            }
            StorageUri storageUri = new StorageUri(primaryUri, secondaryUri);
            CloudBlobClient client = new CloudBlobClient(storageUri, credentials);
            CloudBlobContainer container = client.getContainerReference(containerName);
            Assert.notNull(container, "容器不存在");
            Assert.isTrue(file.exists(), "上传文件不存在");
            CloudBlob blob = container.getBlockBlobReference(file.getName());
            BlobType type = BlobType.valueOf(blobType);
            if (type == BlobType.APPEND_BLOB) {
                blob = container.getAppendBlobReference(file.getName());
            } else if (type == BlobType.PAGE_BLOB) {
                blob = container.getPageBlobReference(file.getName());
            } else if (type == BlobType.BLOCK_BLOB) {
                blob = container.getBlockBlobReference(file.getName());
            }
            log.info("start upload blob {} to container:{}", file.getName(), DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
            blob.uploadFromFile(file.getAbsolutePath(), null, null, null);
            log.info("end upload blob {} to container:{}", file.getName(), DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
            //删除服务器文件
            file.deleteOnExit();
            blob = container.getBlobReferenceFromServer(file.getName());
            if (StringUtils.isNotEmpty(standardBlobTier) && blob instanceof CloudBlockBlob) {
                CloudBlockBlob blockBlob = (CloudBlockBlob) blob;
                blockBlob.uploadStandardBlobTier(StandardBlobTier.valueOf(standardBlobTier));
                log.info("{}更改层成功", blockBlob.getName());
            }
            return new BocloudResult(true, getBlob(container, blob.getName(), null), "上传成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e);
            return new BocloudResult(false, "上传失败：" + e.getMessage());
        }
    }

    public BlobModel getBlob(CloudBlobContainer container, String name, String snapshotId) throws Exception {
        BlobModel model = null;
        Long startTime = System.currentTimeMillis();
        while (true) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                log.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            CloudBlob blob = container.getBlobReferenceFromServer(name, snapshotId, null, null, null);
            if (null == blob) {
                break;
            }
            if (StringUtils.isEmpty(snapshotId) && StringUtils.isEmpty(blob.getSnapshotID())) {
                if (null != blob.getProperties() && null != blob.getProperties().getLeaseState()) {
                    model = BLOB_CONVERTER.convertModel(blob);
                    break;
                }
            } else {
                model = BLOB_CONVERTER.convertModel(blob);
                break;
            }
            Long endTime = System.currentTimeMillis();
            if (endTime - startTime > 60 * 1000) {
                break;
            }
        }
        return model;
    }

    /**
     * 从容器下载文件
     *
     * @param outputStream  输出流
     * @param blobName      blob名称
     * @param containerName 容器名
     * @param account       存储账号
     * @return
     */
    @Override
    public BocloudResult download(OutputStream outputStream, String blobName, String containerName, StorageAccountModel account) {
        try {
            Assert.notNull(account.getName(), "存储账号为空");
            Assert.notNull(account.getPrimaryKey(), "存储账号primary key");
            Assert.notNull(account.getPrimaryEndPoint(), "存储账号primary endpoint");
            StorageCredentials credentials = new StorageCredentialsAccountAndKey(account.getName(), account.getPrimaryKey());
            URI primaryUri = new URI(account.getPrimaryEndPoint());
            URI secondaryUri = null;
            if (org.springframework.util.StringUtils.hasText(account.getSecondaryEndPoint())) {
                secondaryUri = new URI(account.getSecondaryEndPoint());
            }
            StorageUri storageUri = new StorageUri(primaryUri, secondaryUri);
            CloudBlobClient client = new CloudBlobClient(storageUri, credentials);
            CloudBlobContainer container = client.getContainerReference(containerName);
            Assert.notNull(container, "容器不存在");
            CloudBlob blob = container.getBlobReferenceFromServer(blobName);
            log.info("start download blob {} from container:{}", blobName, DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
            blob.download(outputStream);
            log.info("end download blob {} from container:{}", blobName, DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
            return new BocloudResult(true, "下载成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e);
            return new BocloudResult(false, "下载失败：" + e.getMessage());
        }
    }

    /**
     * 从容器下载文件
     *
     * @param filePath
     * @param blobName
     * @param containerName
     * @param account
     * @return
     */
    @Override
    public BocloudResult download(String filePath, String blobName, String containerName, StorageAccountModel account) {
        try {
            Assert.notNull(account.getName(), "存储账号为空");
            Assert.notNull(account.getPrimaryKey(), "存储账号primary key");
            Assert.notNull(account.getPrimaryEndPoint(), "存储账号primary endpoint");
            StorageCredentials credentials = new StorageCredentialsAccountAndKey(account.getName(), account.getPrimaryKey());
            URI primaryUri = new URI(account.getPrimaryEndPoint());
            URI secondaryUri = null;
            if (org.springframework.util.StringUtils.hasText(account.getSecondaryEndPoint())) {
                secondaryUri = new URI(account.getSecondaryEndPoint());
            }
            StorageUri storageUri = new StorageUri(primaryUri, secondaryUri);
            CloudBlobClient client = new CloudBlobClient(storageUri, credentials);
            CloudBlobContainer container = client.getContainerReference(containerName);
            Assert.notNull(container, "容器不存在");
            CloudBlob blob = container.getBlobReferenceFromServer(blobName);
            log.info("start download file to container:{}", DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
            blob.downloadToFile(filePath);
            log.info("end download file to container:{}", DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
            return new BocloudResult(true, "下载成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e);
            return new BocloudResult(false, "下载失败：" + e.getMessage());
        }
    }

    /**
     * 删除blob
     *
     * @param blobName      blob名称
     * @param containerName 容器名称
     * @param account       存储账号
     * @return
     */
    @Override
    public BocloudResult delete(String blobName, String containerName, StorageAccountModel account) {
        try {
            log.info("删除blob：{}", blobName);
            Assert.notNull(account.getName(), "存储账号为空");
            Assert.notNull(account.getPrimaryKey(), "存储账号primary key");
            Assert.notNull(account.getPrimaryEndPoint(), "存储账号primary endpoint");
            StorageCredentials credentials = new StorageCredentialsAccountAndKey(account.getName(), account.getPrimaryKey());
            URI primaryUri = new URI(account.getPrimaryEndPoint());
            URI secondaryUri = null;
            if (org.springframework.util.StringUtils.hasText(account.getSecondaryEndPoint())) {
                secondaryUri = new URI(account.getSecondaryEndPoint());
            }
            StorageUri storageUri = new StorageUri(primaryUri, secondaryUri);
            CloudBlobClient client = new CloudBlobClient(storageUri, credentials);
            CloudBlobContainer container = client.getContainerReference(containerName);
            Assert.notNull(container, "容器不存在");
            CloudBlob blob = container.getBlobReferenceFromServer(blobName);
            if (null == blob) {
                return new BocloudResult(true, "删除成功");
            }
            // 删除blob以及快照
            blob.deleteIfExists(DeleteSnapshotsOption.INCLUDE_SNAPSHOTS, null, null, null);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            log.error("删除blob失败：{}", e);
            return new BocloudResult(false, "删除失败：" + e.getMessage());
        }
    }

    /**
     * 创建快照
     *
     * @param blobName      blob名称
     * @param containerName 容器名称
     * @param account       存储账号
     * @return
     */
    @Override
    public BocloudResult createSnapshot(String blobName, String containerName, StorageAccountModel account) {
        try {
            Assert.notNull(account.getName(), "存储账号为空");
            Assert.notNull(account.getPrimaryKey(), "存储账号primary key");
            Assert.notNull(account.getPrimaryEndPoint(), "存储账号primary endpoint");
            StorageCredentials credentials = new StorageCredentialsAccountAndKey(account.getName(), account.getPrimaryKey());
            URI primaryUri = new URI(account.getPrimaryEndPoint());
            URI secondaryUri = null;
            if (org.springframework.util.StringUtils.hasText(account.getSecondaryEndPoint())) {
                secondaryUri = new URI(account.getSecondaryEndPoint());
            }
            StorageUri storageUri = new StorageUri(primaryUri, secondaryUri);
            CloudBlobClient client = new CloudBlobClient(storageUri, credentials);
            CloudBlobContainer container = client.getContainerReference(containerName);
            Assert.notNull(container, "容器不存在");
            CloudBlob blob = container.getBlobReferenceFromServer(blobName);
            CloudBlob snapshot = blob.createSnapshot();
            Assert.notNull(snapshot, "快照创建失败");
            snapshot = container.getBlobReferenceFromServer(snapshot.getName(), snapshot.getSnapshotID(), null, null, null);
            return new BocloudResult(true, getBlob(container, snapshot.getName(), snapshot.getSnapshotID()), "创建成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e);
            return new BocloudResult(false, "创建失败：" + e.getMessage());
        }
    }

    /**
     * 删除快照
     *
     * @param snapshotName  快照名称
     * @param snapshotId    快照ID
     * @param containerName 容器名称
     * @param account       存储账号
     * @return
     */
    @Override
    public BocloudResult deleteSnapshot(String snapshotName, String snapshotId, String containerName, StorageAccountModel account) {
        try {
            Assert.notNull(account.getName(), "存储账号为空");
            Assert.notNull(account.getPrimaryKey(), "存储账号primary key");
            Assert.notNull(account.getPrimaryEndPoint(), "存储账号primary endpoint");
            StorageCredentials credentials = new StorageCredentialsAccountAndKey(account.getName(), account.getPrimaryKey());
            URI primaryUri = new URI(account.getPrimaryEndPoint());
            URI secondaryUri = null;
            if (org.springframework.util.StringUtils.hasText(account.getSecondaryEndPoint())) {
                secondaryUri = new URI(account.getSecondaryEndPoint());
            }
            StorageUri storageUri = new StorageUri(primaryUri, secondaryUri);
            CloudBlobClient client = new CloudBlobClient(storageUri, credentials);
            CloudBlobContainer container = client.getContainerReference(containerName);
            Assert.notNull(container, "容器不存在");
            CloudBlob snapshot = container.getBlobReferenceFromServer(snapshotName, snapshotId, null, null, null);
            if (null == snapshot) {
                return new BocloudResult(true, "删除成功");
            }
            boolean flag = snapshot.deleteIfExists();
            Assert.isTrue(flag, "快照删除失败");
            return new BocloudResult(true, "快照删除成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e);
            return new BocloudResult(false, "快照删除失败：" + e.getMessage());
        }
    }

    /**
     * 下载快照
     *
     * @param filePath      文件下载路径
     * @param snapshotName  快照名称
     * @param snapshotId    快照ID
     * @param containerName 容器名称
     * @param account       存储账号
     * @return
     */
    @Override
    public BocloudResult downloadSnapshot(String filePath, String snapshotName, String snapshotId, String containerName, StorageAccountModel account) {
        try {
            Assert.notNull(account.getName(), "存储账号为空");
            Assert.notNull(account.getPrimaryKey(), "存储账号primary key");
            Assert.notNull(account.getPrimaryEndPoint(), "存储账号primary endpoint");
            StorageCredentials credentials = new StorageCredentialsAccountAndKey(account.getName(), account.getPrimaryKey());
            URI primaryUri = new URI(account.getPrimaryEndPoint());
            URI secondaryUri = null;
            if (org.springframework.util.StringUtils.hasText(account.getSecondaryEndPoint())) {
                secondaryUri = new URI(account.getSecondaryEndPoint());
            }
            StorageUri storageUri = new StorageUri(primaryUri, secondaryUri);
            CloudBlobClient client = new CloudBlobClient(storageUri, credentials);
            CloudBlobContainer container = client.getContainerReference(containerName);
            Assert.notNull(container, "容器不存在");
            CloudBlob blob = container.getBlobReferenceFromServer(snapshotName, snapshotId, null, null, null);
            log.info("start download blob snapshot {} from container:{}", snapshotName, DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
            blob.downloadToFile(filePath);
            log.info("end download blob snapshot {} from container:{}", snapshotName, DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
            return new BocloudResult(true, "下载成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e);
            return new BocloudResult(false, "下载失败：" + e.getMessage());
        }
    }


    /**
     * 更改blob或快照访问层
     *
     * @param blobOrSnapshotName blob或快照名称
     * @param snapshotId         快照id
     * @param standardBlobTier   访问层
     * @param containerName      容器名称
     * @param account            存储账号
     * @return
     */
    @Override
    public BocloudResult uploadStandardBlobTier(String blobOrSnapshotName, String snapshotId, String standardBlobTier, String containerName, StorageAccountModel account) {
        try {
            Assert.notNull(account.getName(), "存储账号为空");
            Assert.notNull(account.getPrimaryKey(), "存储账号primary key");
            Assert.notNull(account.getPrimaryEndPoint(), "存储账号primary endpoint");
            StorageCredentials credentials = new StorageCredentialsAccountAndKey(account.getName(), account.getPrimaryKey());
            URI primaryUri = new URI(account.getPrimaryEndPoint());
            URI secondaryUri = null;
            if (org.springframework.util.StringUtils.hasText(account.getSecondaryEndPoint())) {
                secondaryUri = new URI(account.getSecondaryEndPoint());
            }
            StorageUri storageUri = new StorageUri(primaryUri, secondaryUri);
            CloudBlobClient client = new CloudBlobClient(storageUri, credentials);
            CloudBlobContainer container = client.getContainerReference(containerName);
            Assert.notNull(container, "容器不存在");
            CloudBlob blob = container.getBlobReferenceFromServer(blobOrSnapshotName, snapshotId, null, null, null);
            Assert.notNull(blob, "数据不存在");
            if (blob instanceof CloudBlockBlob) {
                CloudBlockBlob cloudBlockBlob = (CloudBlockBlob) blob;
                cloudBlockBlob.uploadStandardBlobTier(StandardBlobTier.valueOf(standardBlobTier));
                return new BocloudResult(true, "更新层成功");
            } else {
                return new BocloudResult(false, "仅允许在块blob上设置访问层");
            }
        } catch (Exception e) {
            log.error("Get error message：{}", e);
            return new BocloudResult(false, "更新层失败：" + e.getMessage());
        }
    }


}
