package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.azureconvertor.compute.ImageConvertor;
import com.bocloud.cmp.azureconvertor.compute.PublisherImageConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.compute.ImageUpdate;
import com.microsoft.azure.management.compute.VirtualMachineCustomImage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;


/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureImageProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/9/17 17:55
 */
@Provider("bocloud.cmp.azure")
public class AzureImageProvider extends AzureProvider implements ImageProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureImageProvider.class);

    private BeanConvertor<ImageModel, VirtualMachineCustomImage> privateConvertor = new ImageConvertor();

    private PublisherImageConvertor convertorPublisher = new PublisherImageConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public AzureImageProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult list() {
        try {
            List<ImageModel> regionModels = new ArrayList<>();
            //自定义镜像
            List<ImageModel> privateModels = privateConvertor.convertList(this.getAzure().virtualMachineCustomImages().list()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            regionModels.addAll(privateModels);
            //市场镜像
            List<ImageModel> publicModels = convertorPublisher.convertList(this.getAzure().virtualMachineImages().publishers().listByRegion(this.getRegionId()));
            publicModels = publicModels.stream().filter(model -> !StringUtils.isEmpty(model.getImageUuid()) && !StringUtils.isEmpty(model.getSku()) && !StringUtils.isEmpty(model.getOffer())).collect(Collectors.toList());
            regionModels.addAll(publicModels);
            //单一镜像
//            VirtualMachineImage image = this.getAzure().virtualMachineImages().getImage(this.getRegionId(), "OpenLogic", "CentOS", "7.6", "7.6.201909120");
            logger.info("自定义镜像数量:{}", privateModels.size());
            logger.info("公共镜像数量:{}", publicModels.size());
            return new BocloudResult(true, regionModels, "list virtualMachineCustomImage success!");
        } catch (Exception e) {
            logger.error("list virtualMachineCustomImage error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String imageId) {
        try {
            VirtualMachineCustomImage virtualMachineCustomImage = this.getAzure().virtualMachineCustomImages().getById(imageId);
            return new BocloudResult(true, privateConvertor.convertModel(virtualMachineCustomImage), "detail virtualMachineCustomImage success!");
        } catch (Exception e) {
            logger.error("detail virtualMachineCustomImage error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult remove(String imageId) {
        try {
            this.getAzure().virtualMachineCustomImages().deleteById(imageId);
            return new BocloudResult(true, "remove virtualMachineCustomImage success!");
        } catch (Exception e) {
            logger.error("remove virtualMachineCustomImage error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult modify(ImageModel image) {
        try {
            ImageUpdate parameters = new ImageUpdate();
            this.getAzure().virtualMachineCustomImages().inner().update(image.getResourceGroupName(), image.getName(), parameters);
            return new BocloudResult(true, "update virtualMachineCustomImage success!");
        } catch (Exception e) {
            logger.error("update virtualMachineCustomImage error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }

    @Override
    public BocloudResult create(ImageModel imageModel) {
        try {
//            this.getAzure().virtualMachines().getById(imageModel.getInstanceId()).generalize();
            VirtualMachineCustomImage virtualMachineCustomImage = this.getAzure().virtualMachineCustomImages()
                    .define(imageModel.getName())
                    .withRegion(imageModel.getRegionId())
                    .withExistingResourceGroup(imageModel.getResourceGroupName())
//                    .withHyperVGeneration(HyperVGenerationTypes.fromString(""))
                    .fromVirtualMachine(imageModel.getInstanceId())
                    .create();
            return new BocloudResult(true, privateConvertor.convertModel(virtualMachineCustomImage), "create virtualMachineCustomImage success!");
        } catch (Exception e) {
            logger.error("create virtualMachineCustomImage error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
