package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.compute.ServerConvertor;
import com.bocloud.cmp.azureconvertor.compute.VirtualMachineSizeConvertor;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.azure.automation.LogAnalyticModel;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.microsoft.azure.management.compute.*;
import com.microsoft.azure.management.compute.implementation.VirtualMachineInner;
import com.microsoft.azure.management.network.Network;
import com.microsoft.azure.management.network.NetworkInterface;
import com.microsoft.azure.management.network.PublicIPAddress;
import com.microsoft.azure.management.resources.fluentcore.arm.AvailabilityZoneId;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureServerProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/8/5 17:33
 */
@Provider("bocloud.cmp.azure")
public class AzureServerProvider extends AzureProvider implements ServerProvider {

    private static final Logger logger = LoggerFactory.getLogger(AzureServerProvider.class);

    private final BeanConvertor<ServerModel, VirtualMachine> convertor = new ServerConvertor();

    private final BeanConvertor<FlavorModel, VirtualMachineSize> vmSizeConvertor = new VirtualMachineSizeConvertor();

    public AzureServerProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult list() {
        try {
            List<ServerModel> regionModels = convertor.convertList(this.getAzure().virtualMachines().list()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            return new BocloudResult(true, regionModels, "list cloudServer success!");
        } catch (Exception e) {
            logger.error("list cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult list(String hostId) {
        return null;
    }

    @Override
    public BocloudResult changePassword(String instanceId, String password) {
        try {
            VirtualMachine virtualMachine = this.getAzure().virtualMachines().getById(instanceId);
            OSProfile osProfile = virtualMachine.osProfile().withAdminPassword(password);

            VirtualMachineInner virtualMachineInner = virtualMachine.inner().withOsProfile(osProfile);
            String resourceGroupName = virtualMachine.resourceGroupName();
            String name = virtualMachine.name();

            VirtualMachineInner orUpdate = this.getAzure().virtualMachines().inner().createOrUpdate(resourceGroupName, name, virtualMachineInner);


            //云主机扩展
//            VirtualMachine apply = virtualMachine.update()
//                    .defineNewExtension("ddd")
//                    .withPublisher("Microsoft.Compute")
//                    .withType("VMAccessAgent")
//                    .withVersion("2.0")
//                    .withProtectedSetting("Password", password).attach().apply();
            return new BocloudResult(true, orUpdate, "list cloudServer success!");
        } catch (Exception e) {
            logger.error("changePassword cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(ServerModel server) {
        try {
            VirtualMachine virtualMachine = this.getCreateWay(server);
            ServerModel serverModel = convertor.convertModel(virtualMachine);
            if (server.getImageType().equalsIgnoreCase("PUBLIC")) {
                logger.info("云主机第一次获取系统版本号......");
                serverModel = this.getInstances(serverModel);

                if (StringUtils.isEmpty(serverModel.getOsName()) || StringUtils.isEmpty(serverModel.getOsVersion())) {
                    logger.info("云主机第二次获取系统版本号......");
                    if (server.getImageType().equalsIgnoreCase("PUBLIC") && server.getOsCategory().equalsIgnoreCase("LINUX")) {
                        KnownLinuxVirtualMachineImage linuxImage = KnownLinuxVirtualMachineImage.valueOf(server.getPopularLinuxImage());
                        serverModel.setOsName(linuxImage.offer());
                        serverModel.setOsVersion(linuxImage.sku());
                    } else if (server.getImageType().equalsIgnoreCase("PUBLIC") && server.getOsCategory().equalsIgnoreCase("WINDOWS")) {
                        KnownWindowsVirtualMachineImage windowsImage = KnownWindowsVirtualMachineImage.valueOf(server.getPopularWindowsImage());
                        serverModel.setOsName(windowsImage.offer());
                        serverModel.setOsVersion(windowsImage.sku());

                    }
                }

            }
            return new BocloudResult(true, serverModel, "create cloudServer success!");
        } catch (Exception e) {
            logger.error("create cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    public ServerModel getInstances(ServerModel serverModel) {
        ServerModel newModel = serverModel;
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            // 轮询30m
            while (endTime - startTime < 30 * 60 * 1000) {
                // 主机创建时间不确定，增加休眠减少轮询次数
                Thread.sleep(60 * 1000);
                BocloudResult detail = this.detail(serverModel.getInstanceId());
                ServerModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), ServerModel.class);
                if (ObjectUtils.isEmpty(model)) {
                    break;
                }
                if (StringUtils.isEmpty(model.getStatus())) {
                    logger.info("云主机获取不到状态，继续轮询！");
                    continue;
                }
                if (!StringUtils.isEmpty(model.getOsName()) || !StringUtils.isEmpty(model.getOsVersion())) {
                    newModel = model;
                    break;
                }
                logger.info("云主机获取不到版本号，继续轮询！");
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            return newModel;
        }
        return newModel;
    }

    @Override
    public BocloudResult detail(String instanceId) {
        try {
            VirtualMachine byResourceGroup = this.getAzure().virtualMachines().getById(instanceId);
            ServerModel model = convertor.convertModel(byResourceGroup);
            return new BocloudResult(true, model, "detail cloudServer success!");
        } catch (Exception e) {
            logger.error("detail cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult availableSizes(String instanceId) {
        try {
            VirtualMachine virtualMachine = this.getAzure().virtualMachines().getById(instanceId);
            List<FlavorModel> flavorModels = vmSizeConvertor.convertList(virtualMachine.availableSizes());
            return new BocloudResult(true, flavorModels, "availableSizes cloudServer success!");
        } catch (Exception e) {
            logger.error("availableSizes cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    private VirtualMachine getCreateWay(ServerModel serverModel) {
        //第一步 指定VPC 子网
        VirtualMachine.DefinitionStages.WithPrivateIP withPrivateIP = this.withPrivateIP(serverModel);
        //第二步 指定EIP
        VirtualMachine.DefinitionStages.WithProximityPlacementGroup withProximityPlacementGroup = this.withProximityPlacementGroup(serverModel, withPrivateIP);
        //第三步 指定镜像 规格 系统盘
        VirtualMachine.DefinitionStages.WithCreate with3Create = this.withCreate(serverModel, withProximityPlacementGroup);
        //第四步 指定可用性集
        VirtualMachine.DefinitionStages.WithCreate with4Create = this.withCreate(serverModel, with3Create);
        return with4Create.create();
    }

    //第一步 指定VPC 子网
    private VirtualMachine.DefinitionStages.WithPrivateIP withPrivateIP(ServerModel serverModel) {
        //Network
        Network network = this.getAzure().networks().getById(serverModel.getVpcUuid());
        return this.getAzure().virtualMachines()
                .define(serverModel.getName())
                .withRegion(serverModel.getRegionId())
                .withExistingResourceGroup(serverModel.getResourceGroupName())
                .withExistingPrimaryNetwork(network)//指定vpc
                .withSubnet(serverModel.getSubnetName());
    }

    //第二步 指定EIP
    private VirtualMachine.DefinitionStages.WithProximityPlacementGroup withProximityPlacementGroup(ServerModel serverModel, VirtualMachine.DefinitionStages.WithPrivateIP withPrivateIP) {
        //是否指定EIP 批量创建
        Boolean isUsePublicIp = serverModel.getIsUsePublicIp();
        //指定单独EIP
        PublicIPAddress publicIPAddress = null;
        if (!StringUtils.isEmpty(serverModel.getPublicIpAddressUuid())) {
            publicIPAddress = this.getAzure().publicIPAddresses().getById(serverModel.getPublicIpAddressUuid());
        }

        VirtualMachine.DefinitionStages.WithProximityPlacementGroup withProximityPlacementGroup = null;

        if (isUsePublicIp && null == publicIPAddress) { //使用自动生成的EIP(动态)
            withProximityPlacementGroup = withPrivateIP.withPrimaryPrivateIPAddressDynamic().withNewPrimaryPublicIPAddress(serverModel.getName());
        } else if (isUsePublicIp) {//使用选择的EIP（动态/静态）
            withProximityPlacementGroup = withPrivateIP.withPrimaryPrivateIPAddressDynamic().withExistingPrimaryPublicIPAddress(publicIPAddress);
        } else {//不使用EIP
            withProximityPlacementGroup = withPrivateIP.withPrimaryPrivateIPAddressDynamic().withoutPrimaryPublicIPAddress();
        }
        return withProximityPlacementGroup;
    }

    //第三步 指定镜像 规格 系统盘
    private VirtualMachine.DefinitionStages.WithCreate withCreate(ServerModel serverModel, VirtualMachine.DefinitionStages.WithProximityPlacementGroup withProximityPlacementGroup) {

        VirtualMachine.DefinitionStages.WithCreate withCreate = null;
        //镜像类型：通用镜像，公共镜像，自定义镜像
        String imageType = serverModel.getImageType();
        //windows /linux
        String osCategory = serverModel.getOsCategory();
        switch (imageType) {
            case "PUBLIC": //通用镜像
                logger.info("执行通用镜像创建。。。。。。");
                if (osCategory.equalsIgnoreCase(Common.LINUX)) {
                    //指定创建方式 sshkey 还是用户名密码
                    VirtualMachine.DefinitionStages.WithLinuxCreateManagedOrUnmanaged popularLinuxImage = this.withPopularLinuxImage(serverModel, withProximityPlacementGroup);
                    //数据盘大小及可用性区域
                    withCreate = this.publicLinuxCreate(serverModel, popularLinuxImage);
                } else {
                    //指定创建方式 sshkey 还是用户名密码
                    VirtualMachine.DefinitionStages.WithWindowsCreateManagedOrUnmanaged withPopularWindowsImage = this.withPopularWindowsImage(serverModel, withProximityPlacementGroup);
                    //数据盘大小及可用性区域
                    withCreate = this.publicWindowsCreate(serverModel, withPopularWindowsImage);
                }
                break;
            case "PRIVATE"://自定义镜像
                logger.info("执行自定义镜像创建。。。。。。");
                if (osCategory.equalsIgnoreCase(Common.LINUX)) {
                    //指定创建方式 sshkey 还是用户名密码
                    VirtualMachine.DefinitionStages.WithLinuxCreateManaged linuxCustomImage = this.withLinuxCustomImage(serverModel, withProximityPlacementGroup);
                    //数据盘大小及可用性区域
                    withCreate = this.privateLinuxCreate(serverModel, linuxCustomImage);
                } else {
                    //指定创建方式 sshkey 还是用户名密码
                    VirtualMachine.DefinitionStages.WithWindowsCreateManaged withWindowsCustomImage = this.withWindowsCustomImage(serverModel, withProximityPlacementGroup);
                    withCreate = this.privateWindowsCreate(serverModel, withWindowsCustomImage);
                }
                break;
            case "PUBLISHER"://流行/社区镜像
                logger.info("执行社区镜像创建。。。。。。");
                if (osCategory.equalsIgnoreCase(Common.LINUX)) {
                    //指定创建方式 sshkey 还是用户名密码
                    VirtualMachine.DefinitionStages.WithLinuxCreateManagedOrUnmanaged latestLinuxImage = this.withLatestLinuxImage(serverModel, withProximityPlacementGroup);
                    //数据盘大小及可用性区域
                    withCreate = this.publisherLinuxCreate(serverModel, latestLinuxImage);
                } else {
                    //指定创建方式 sshkey 还是用户名密码
                    VirtualMachine.DefinitionStages.WithWindowsCreateManagedOrUnmanaged withLatestWindowsImage = this.withLatestWindowsImage(serverModel, withProximityPlacementGroup);
                    withCreate = this.publisherWindowsCreate(serverModel, withLatestWindowsImage);
                }
                break;
            default:
                break;
        }
        return withCreate;
    }

    //第四步 指定可用性集
    private VirtualMachine.DefinitionStages.WithCreate withCreate(ServerModel serverModel, VirtualMachine.DefinitionStages.WithCreate withCreate) {
        //可用性集
        AvailabilitySet availabilitySet = null;
        if (!StringUtils.isEmpty(serverModel.getAvailabilitySetUuid())) {
            availabilitySet = this.getAzure().availabilitySets().getById(serverModel.getAvailabilitySetUuid());
        }
        //插入可用性集
        if (null != availabilitySet) {
            withCreate.withExistingAvailabilitySet(availabilitySet);//可用性集-和负载均衡有关系
        }
        return withCreate;
    }

    //---------通用镜像 指定创建方式 sshkey 还是用户名密码
    private VirtualMachine.DefinitionStages.WithLinuxCreateManagedOrUnmanaged withPopularLinuxImage(ServerModel serverModel, VirtualMachine.DefinitionStages.WithProximityPlacementGroup withProximityPlacementGroup) {
        VirtualMachine.DefinitionStages.WithLinuxCreateManagedOrUnmanaged withLinuxCreateManagedOrUnmanaged = null;
        String sshKey = serverModel.getSshKey();
        if (!StringUtils.isEmpty(sshKey)) {//采用密钥创建
            withLinuxCreateManagedOrUnmanaged = withProximityPlacementGroup
                    .withPopularLinuxImage(KnownLinuxVirtualMachineImage.valueOf(serverModel.getPopularLinuxImage()))  //设置镜像
                    .withRootUsername(serverModel.getAccount())//设置用户名
                    .withSsh(serverModel.getSshKey());
        } else {
            withLinuxCreateManagedOrUnmanaged = withProximityPlacementGroup
                    .withPopularLinuxImage(KnownLinuxVirtualMachineImage.valueOf(serverModel.getPopularLinuxImage()))  //设置镜像
                    .withRootUsername(serverModel.getAccount())//设置用户名
                    .withRootPassword(serverModel.getPassword());//设置密码
        }
        return withLinuxCreateManagedOrUnmanaged;
    }

    private VirtualMachine.DefinitionStages.WithCreate publicLinuxCreate(ServerModel serverModel, VirtualMachine.DefinitionStages.WithLinuxCreateManagedOrUnmanaged withLinuxCreateManagedOrUnmanaged) {
        VirtualMachine.DefinitionStages.WithCreate withCreate = null;
        String zoneIds = serverModel.getAvailabilityZoneIds();
        if (!StringUtils.isEmpty(zoneIds)) {
            List<String> availabilityZoneIds = JSONArray.parseArray(zoneIds, String.class);
            withCreate = withLinuxCreateManagedOrUnmanaged
                    .withAvailabilityZone(AvailabilityZoneId.fromString(availabilityZoneIds.get(0)))
                    .withOSDiskStorageAccountType(StorageAccountTypes.fromString(serverModel.getSystemDiskCategory()))
                    .withSize(serverModel.getFlavorUuid())//设置规格
                    .withOSDiskName(serverModel.getSystemDiskName())//设置系统盘名称
                    .withOSDiskCaching(CachingTypes.valueOf(serverModel.getSystemDiskCaching()))//设置系统盘缓冲（只读，只写，无）
                    .withOSDiskSizeInGB(serverModel.getSystemDiskSize().intValue());//设置系统盘大小
        } else {
            withCreate = withLinuxCreateManagedOrUnmanaged
                    .withOSDiskStorageAccountType(StorageAccountTypes.fromString(serverModel.getSystemDiskCategory()))
                    .withSize(serverModel.getFlavorUuid())//设置规格
                    .withOSDiskName(serverModel.getSystemDiskName())//设置系统盘名称
                    .withOSDiskCaching(CachingTypes.valueOf(serverModel.getSystemDiskCaching()))//设置系统盘缓冲（只读，只写，无）
                    .withOSDiskSizeInGB(serverModel.getSystemDiskSize());//设置系统盘大小
        }
        return withCreate;
    }

    private VirtualMachine.DefinitionStages.WithWindowsCreateManagedOrUnmanaged withPopularWindowsImage(ServerModel serverModel, VirtualMachine.DefinitionStages.WithProximityPlacementGroup withProximityPlacementGroup) {
        return withProximityPlacementGroup
                .withPopularWindowsImage(KnownWindowsVirtualMachineImage.valueOf(serverModel.getPopularWindowsImage())) //设置通用windows镜像
                .withAdminUsername(serverModel.getAccount())//设置用户名
                .withAdminPassword(serverModel.getPassword());
    }

    private VirtualMachine.DefinitionStages.WithCreate publicWindowsCreate(ServerModel serverModel, VirtualMachine.DefinitionStages.WithWindowsCreateManagedOrUnmanaged withWindowsCreateManagedOrUnmanaged) {
        VirtualMachine.DefinitionStages.WithCreate withCreate;
        String licenseType = serverModel.getLicenseType();
        if (!StringUtils.isEmpty(licenseType)) {
            withCreate = withWindowsCreateManagedOrUnmanaged.withOSDiskStorageAccountType(StorageAccountTypes.fromString(serverModel.getSystemDiskCategory()))
                    .withSize(serverModel.getFlavorUuid())//设置规格
                    .withOSDiskName(serverModel.getSystemDiskName())//设置系统盘名称
                    .withOSDiskCaching(CachingTypes.valueOf(serverModel.getSystemDiskCaching()))//设置系统盘缓冲（只读，只写，无）
                    .withOSDiskSizeInGB(serverModel.getSystemDiskSize().intValue())//设置系统盘大小
                    .withLicenseType(licenseType);//设置授权证书
        } else {
            withCreate = withWindowsCreateManagedOrUnmanaged.withOSDiskStorageAccountType(StorageAccountTypes.fromString(serverModel.getSystemDiskCategory()))
                    .withSize(serverModel.getFlavorUuid())//设置规格
                    .withOSDiskName(serverModel.getSystemDiskName())//设置系统盘名称
                    .withOSDiskCaching(CachingTypes.valueOf(serverModel.getSystemDiskCaching()))//设置系统盘缓冲（只读，只写，无）
                    .withOSDiskSizeInGB(serverModel.getSystemDiskSize().intValue());
        }
        return withCreate;
    }

    //---------自定义镜像
    private VirtualMachine.DefinitionStages.WithLinuxCreateManaged withLinuxCustomImage(ServerModel serverModel, VirtualMachine.DefinitionStages.WithProximityPlacementGroup withProximityPlacementGroup) {
        VirtualMachine.DefinitionStages.WithLinuxCreateManaged withLinuxCreateManaged;
        String sshKey = serverModel.getSshKey();
        if (!StringUtils.isEmpty(sshKey)) {//采用密钥创建
            withLinuxCreateManaged = withProximityPlacementGroup
                    .withLinuxCustomImage(serverModel.getImageUuid())   //linux 镜像
                    .withRootUsername(serverModel.getAccount())//设置用户名
                    .withSsh(serverModel.getSshKey());
        } else {
            withLinuxCreateManaged = withProximityPlacementGroup
                    .withLinuxCustomImage(serverModel.getImageUuid())   //linux 镜像
                    .withRootUsername(serverModel.getAccount())//设置用户名
                    .withRootPassword(serverModel.getPassword());//设置密码
        }
        return withLinuxCreateManaged;
    }

    private VirtualMachine.DefinitionStages.WithCreate privateLinuxCreate(ServerModel serverModel, VirtualMachine.DefinitionStages.WithLinuxCreateManaged withLinuxCreateManaged) {
        String zoneIds = serverModel.getAvailabilityZoneIds();
        VirtualMachine.DefinitionStages.WithCreate withCreate;
        if (!StringUtils.isEmpty(zoneIds)) {
            List<String> availabilityZoneIds = JSONArray.parseArray(zoneIds, String.class);
            withCreate = withLinuxCreateManaged
                    .withAvailabilityZone(AvailabilityZoneId.fromString(availabilityZoneIds.get(0)))
                    .withOSDiskStorageAccountType(StorageAccountTypes.fromString(serverModel.getSystemDiskCategory()))
                    .withSize(serverModel.getFlavorUuid())//设置规格
                    .withOSDiskName(serverModel.getSystemDiskName())//设置系统盘名称
                    .withOSDiskCaching(CachingTypes.valueOf(serverModel.getSystemDiskCaching()))//设置系统盘缓冲（只读，只写，无）
                    .withOSDiskSizeInGB(serverModel.getSystemDiskSize().intValue());//设置系统盘大小
        } else {
            withCreate = withLinuxCreateManaged
                    .withOSDiskStorageAccountType(StorageAccountTypes.fromString(serverModel.getSystemDiskCategory()))
                    .withSize(serverModel.getFlavorUuid())//设置规格
                    .withOSDiskName(serverModel.getSystemDiskName())//设置系统盘名称
                    .withOSDiskCaching(CachingTypes.valueOf(serverModel.getSystemDiskCaching()))//设置系统盘缓冲（只读，只写，无）
                    .withOSDiskSizeInGB(serverModel.getSystemDiskSize().intValue());//设置系统盘大小
        }
        return withCreate;
    }

    private VirtualMachine.DefinitionStages.WithWindowsCreateManaged withWindowsCustomImage(ServerModel serverModel, VirtualMachine.DefinitionStages.WithProximityPlacementGroup withProximityPlacementGroup) {
        return withProximityPlacementGroup
                .withWindowsCustomImage(serverModel.getImageUuid())  //windows 镜像
                .withAdminUsername(serverModel.getAccount())//设置用户名
                .withAdminPassword(serverModel.getPassword());
    }

    private VirtualMachine.DefinitionStages.WithCreate privateWindowsCreate(ServerModel serverModel, VirtualMachine.DefinitionStages.WithWindowsCreateManaged withWindowsCreateManaged) {
        VirtualMachine.DefinitionStages.WithCreate withCreate;
        String licenseType = serverModel.getLicenseType();
        if (!StringUtils.isEmpty(licenseType)) {
            withCreate = withWindowsCreateManaged.withOSDiskStorageAccountType(StorageAccountTypes.fromString(serverModel.getSystemDiskCategory()))
                    .withSize(serverModel.getFlavorUuid())//设置规格
                    .withOSDiskName(serverModel.getSystemDiskName())//设置系统盘名称
                    .withOSDiskCaching(CachingTypes.valueOf(serverModel.getSystemDiskCaching()))//设置系统盘缓冲（只读，只写，无）
                    .withOSDiskSizeInGB(serverModel.getSystemDiskSize().intValue())//设置系统盘大小
                    .withLicenseType(licenseType);//设置授权证书
        } else {
            withCreate = withWindowsCreateManaged.withOSDiskStorageAccountType(StorageAccountTypes.fromString(serverModel.getSystemDiskCategory()))
                    .withSize(serverModel.getFlavorUuid())//设置规格
                    .withOSDiskName(serverModel.getSystemDiskName())//设置系统盘名称
                    .withOSDiskCaching(CachingTypes.valueOf(serverModel.getSystemDiskCaching()))//设置系统盘缓冲（只读，只写，无）
                    .withOSDiskSizeInGB(serverModel.getSystemDiskSize().intValue());
        }
        return withCreate;
    }

    //---------流行/社区镜像PUBLISHER
    private VirtualMachine.DefinitionStages.WithLinuxCreateManagedOrUnmanaged withLatestLinuxImage(ServerModel serverModel, VirtualMachine.DefinitionStages.WithProximityPlacementGroup withProximityPlacementGroup) {
        VirtualMachine.DefinitionStages.WithLinuxCreateManagedOrUnmanaged withLinuxCreateManagedOrUnmanaged = null;
        String sshKey = serverModel.getSshKey();
        if (!StringUtils.isEmpty(sshKey)) {//采用密钥创建
            withLinuxCreateManagedOrUnmanaged = withProximityPlacementGroup
                    .withLatestLinuxImage(serverModel.getPublisher(), serverModel.getOffer(), serverModel.getSku())  //设置镜像
                    .withRootUsername(serverModel.getAccount())//设置用户名
                    .withSsh(serverModel.getSshKey());
        } else {
            withLinuxCreateManagedOrUnmanaged = withProximityPlacementGroup
                    .withLatestLinuxImage(serverModel.getPublisher(), serverModel.getOffer(), serverModel.getSku())  //设置镜像
                    .withRootUsername(serverModel.getAccount())//设置用户名
                    .withRootPassword(serverModel.getPassword());//设置密码
        }
        return withLinuxCreateManagedOrUnmanaged;
    }

    private VirtualMachine.DefinitionStages.WithCreate publisherLinuxCreate(ServerModel serverModel, VirtualMachine.DefinitionStages.WithLinuxCreateManagedOrUnmanaged withLatestLinuxImage) {
        VirtualMachine.DefinitionStages.WithCreate withCreate = null;
        if (!StringUtils.isEmpty(serverModel.getAvailabilityZoneIds())) {
            List<String> availabilityZoneIds = JSONArray.parseArray(serverModel.getAvailabilityZoneIds(), String.class);
            withCreate = withLatestLinuxImage
                    .withAvailabilityZone(AvailabilityZoneId.fromString(availabilityZoneIds.get(0)))
                    .withOSDiskStorageAccountType(StorageAccountTypes.fromString(serverModel.getSystemDiskCategory()))
                    .withSize(serverModel.getFlavorUuid())//设置规格
                    .withOSDiskName(serverModel.getSystemDiskName())//设置系统盘名称
                    .withOSDiskCaching(CachingTypes.valueOf(serverModel.getSystemDiskCaching()))//设置系统盘缓冲（只读，只写，无）
                    .withOSDiskSizeInGB(serverModel.getSystemDiskSize().intValue());//设置系统盘大小
        } else {
            withCreate = withLatestLinuxImage
                    .withOSDiskStorageAccountType(StorageAccountTypes.fromString(serverModel.getSystemDiskCategory()))
                    .withSize(serverModel.getFlavorUuid())//设置规格
                    .withOSDiskName(serverModel.getSystemDiskName())//设置系统盘名称
                    .withOSDiskCaching(CachingTypes.valueOf(serverModel.getSystemDiskCaching()))//设置系统盘缓冲（只读，只写，无）
                    .withOSDiskSizeInGB(serverModel.getSystemDiskSize().intValue());//设置系统盘大小
        }
        return withCreate;
    }

    private VirtualMachine.DefinitionStages.WithWindowsCreateManagedOrUnmanaged withLatestWindowsImage(ServerModel serverModel, VirtualMachine.DefinitionStages.WithProximityPlacementGroup withProximityPlacementGroup) {
        return withProximityPlacementGroup
                .withLatestWindowsImage(serverModel.getPublisher(), serverModel.getOffer(), serverModel.getSku())
                .withAdminUsername(serverModel.getAccount())//设置用户名
                .withAdminPassword(serverModel.getPassword());
    }

    private VirtualMachine.DefinitionStages.WithCreate publisherWindowsCreate(ServerModel serverModel, VirtualMachine.DefinitionStages.WithWindowsCreateManagedOrUnmanaged withLatestWindowsImage) {
        VirtualMachine.DefinitionStages.WithCreate withCreate;
        String licenseType = serverModel.getLicenseType();
        if (!StringUtils.isEmpty(licenseType)) {
            withCreate = withLatestWindowsImage.withOSDiskStorageAccountType(StorageAccountTypes.fromString(serverModel.getSystemDiskCategory()))
                    .withSize(serverModel.getFlavorUuid())//设置规格
                    .withOSDiskName(serverModel.getSystemDiskName())//设置系统盘名称
                    .withOSDiskCaching(CachingTypes.valueOf(serverModel.getSystemDiskCaching()))//设置系统盘缓冲（只读，只写，无）
                    .withOSDiskSizeInGB(serverModel.getSystemDiskSize().intValue())//设置系统盘大小
                    .withLicenseType(licenseType);//设置授权证书
        } else {
            withCreate = withLatestWindowsImage.withOSDiskStorageAccountType(StorageAccountTypes.fromString(serverModel.getSystemDiskCategory()))
                    .withSize(serverModel.getFlavorUuid())//设置规格
                    .withOSDiskName(serverModel.getSystemDiskName())//设置系统盘名称
                    .withOSDiskCaching(CachingTypes.valueOf(serverModel.getSystemDiskCaching()))//设置系统盘缓冲（只读，只写，无）
                    .withOSDiskSizeInGB(serverModel.getSystemDiskSize().intValue());
        }
        return withCreate;
    }

    @Override
    public BocloudResult<?> secondBindNics(ServerModel model) {
        try {
            NetworkInterface networkInterface = this.getAzure()
                    .networkInterfaces()
                    .getById(model.getNetworkInterfaceUuid());
            this.getAzure()
                    .virtualMachines()
                    .getById(model.getInstanceId())
                    .update()
                    .withExistingSecondaryNetworkInterface(networkInterface)
                    .apply();
            return new BocloudResult<>(true, "secondBindNics cloudServer success!");
        } catch (Exception e) {
            logger.error("secondBindNics cloudServer error :", e);
            return new BocloudResult<>(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult secondUnBindNics(ServerModel model) {
        try {
            NetworkInterface networkInterface = this.getAzure()
                    .networkInterfaces()
                    .getById(model.getNetworkInterfaceUuid());
            this.getAzure().virtualMachines().getById(model.getInstanceId())
                    .update()
                    .withoutSecondaryNetworkInterface(networkInterface.name())
                    .apply();
            return new BocloudResult(true, "secondUnBindNics cloudServer success!");
        } catch (Exception e) {
            logger.error("secondUnBindNics cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult osDiskModify(ServerModel model) {
        try {
            VirtualMachine.Update update = this.getAzure()
                    .virtualMachines()
                    .getById(model.getInstanceId())
                    .update();

            if (null != model.getSystemDiskCaching()) {
                update.withOSDiskCaching(CachingTypes.valueOf(model.getSystemDiskCaching()));
            }
            if (null != model.getSystemDiskSize()) {
                update.withOSDiskSizeInGB(model.getSystemDiskSize().intValue());
            }
            update.apply();
            return new BocloudResult(true, "osDiskConfig cloudServer success!");
        } catch (Exception e) {
            logger.error("osDiskConfig cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(ServerModel server) {
        //只修改描述
        return new BocloudResult(true, "修改成功");
    }

    @Override
    public BocloudResult migrate(ServerModel server) {
        return null;
    }

    @Override
    public BocloudResult migrate(OSMigrateModel model) {
        return null;
    }

    @Override
    public BocloudResult remove(String instanceId, String name) {
        try {
            this.getAzure().virtualMachines().deleteById(instanceId);
            return new BocloudResult(true, "remove cloudServer success!");
        } catch (Exception e) {
            logger.error("remove cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult<?> start(String instanceId) {
        try {
            VirtualMachine vm = this.getAzure().virtualMachines().getById(instanceId);
            Assert.notNull(vm,"current vm is absent! it maybe deleted by someone else!");
            this.getAzure().virtualMachines().start(vm.resourceGroupName(), vm.name());
            return new BocloudResult<>(true, "start cloudServer success!");
        } catch (Exception e) {
            logger.error("start cloudServer error ", e);
            return new BocloudResult<>(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult<?> stop(String instanceId) {
        try {
            VirtualMachine vm = this.getAzure().virtualMachines().getById(instanceId);
            Assert.notNull(vm,"current vm is absent! it maybe deleted by someone else!");
            this.getAzure().virtualMachines().powerOff(vm.resourceGroupName(), vm.name());
            return new BocloudResult<>(true, "stop cloudServer success!");
        } catch (Exception e) {
            logger.error("stop cloudServer error :", e);
            return new BocloudResult<>(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult<?> deallocate(String instanceId) {
        try {
            VirtualMachine vm = this.getAzure().virtualMachines().getById(instanceId);
            Assert.notNull(vm,"current vm is absent! it maybe deleted by someone else!");
            this.getAzure().virtualMachines().deallocate(vm.resourceGroupName(), vm.name());
            return new BocloudResult<>(true, "deallocate cloudServer success!");
        } catch (Exception e) {
            logger.error("deallocate cloudServer error :", e);
            return new BocloudResult<>(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult<?> reboot(String instanceId) {
        try {
            VirtualMachine vm = this.getAzure().virtualMachines().getById(instanceId);
            Assert.notNull(vm,"current vm is absent! it maybe deleted by someone else!");
            this.getAzure().virtualMachines().restart(vm.resourceGroupName(), vm.name());
            return new BocloudResult<>(true, "reboot cloudServer success!");
        } catch (Exception e) {
            logger.error("reboot cloudServer error :", e);
            return new BocloudResult<>(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult softReboot(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult pause(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult unPause(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult suspend(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult resume(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult joinSGroup(String instanceId, String group) {
        return null;
    }

    @Override
    public BocloudResult leaveSGroup(String instanceId, String group) {
        return null;
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        return null;
    }

    @Override
    public BocloudResult vncConsole(String vncType, String serverId) {
        return null;
    }

    @Override
    public BocloudResult attachNetwork(ServerModel server) {
        return null;
    }

    @Override
    public BocloudResult detachNetwork(String instanceId, String portUuid) {
        return null;
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId) {
        try {
            this.getAzure().virtualMachines().getById(instanceId).update().withSize(flavorId).apply();
            return new BocloudResult(true, this.detail(instanceId).getData(), "resize cloudServer success!");
        } catch (Exception e) {
            logger.error("resize cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult confirmResize(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult revertResize(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean MemHotEnable) {
        return null;
    }

    @Override
    public BocloudResult getTicket(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult createInResPool(ServerModel server) {
        return null;
    }

    @Override
    public BocloudResult mountToolsInstaller(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult unmountToolsInstaller(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult upgradeToolsTask(String instanceId, String options) {
        return null;
    }

    @Override
    public BocloudResult getNetCard(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult getVmIps(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult markAsTemplate(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
        return null;
    }

    @Override
    public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
        return null;
    }

    @Override
    public BocloudResult getMetric(String type) {
        return null;
    }

    @Override
    public BocloudResult vmrcConsole(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult migrate(H3cMigrateModel model) {
        return null;
    }

    @Override
    public BocloudResult create(H3cServerModel server) {
        return null;
    }

    @Override
    public BocloudResult getJnlpXml(Long vmId, String address) {
        return null;
    }

    @Override
    public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
        return null;
    }

    @Override
    public BocloudResult resetInstance(ServerModel serverModel) {
        return null;
    }

    @Override
    public BocloudResult bindKeypair(ServerModel serverModel) {
        return null;
    }

    @Override
    public BocloudResult unbindKeypair(ServerModel serverModel) {
        return null;
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
        return null;
    }

    @Override
    public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
        return null;
    }

    @Override
    public BocloudResult extension(ServerModel server, String extensionName, LogAnalyticModel analyticModel) {
        try {
            if (server.getOsCategory().equalsIgnoreCase(Common.WINDOWS)) {
                this.getAzure().virtualMachines().getById(server.getInstanceId()).update()
                        .defineNewExtension(extensionName)//名称不能重复
                        .withPublisher("Microsoft.EnterpriseCloud.Monitoring")
                        .withType("MicrosoftMonitoringAgent")
                        .withVersion("1.0")
                        .withProtectedSetting("workspaceKey", analyticModel.getPrimarySharedKey())
                        .withMinorVersionAutoUpgrade()
                        .withPublicSetting("workspaceId", analyticModel.getCustomerId())
                        .attach()
                        .apply();
            } else if (server.getOsCategory().equalsIgnoreCase(Common.LINUX)) {
                this.getAzure().virtualMachines().getById(server.getInstanceId()).update()
                        .defineNewExtension(extensionName)
                        .withPublisher("Microsoft.EnterpriseCloud.Monitoring")
                        .withType("OmsAgentForLinux")
                        .withVersion("1.0")
                        .withProtectedSetting("workspaceKey", analyticModel.getPrimarySharedKey())
                        .withMinorVersionAutoUpgrade()
                        .withPublicSetting("workspaceId", analyticModel.getCustomerId())
                        .attach()
                        .apply();
            }
            return new BocloudResult(true, "extension cloudServer success!");
        } catch (Exception e) {
            logger.error("extension cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult generalize(String instanceId) {
        try {
            this.getAzure().virtualMachines().getById(instanceId).generalize();
            return this.detail(instanceId);
        } catch (Exception e) {
            logger.error("generalize cloudServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

}
