package com.bocloud.cmp.provider.monitor;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.monitor.MetricAlertConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.monitor.MetricAlertModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureMetricAlertProvider
 * @Description:指标警报
 * @author: zhenpingLi/李振平
 * @date: 2020/12/29 15:35
 */
@Provider("bocloud.cmp.azure")
public class AzureMetricAlertProvider extends AzureRestApiProvider implements MetricAlertProvider {

    public AzureMetricAlertProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureMetricAlertProvider.class);

    private BeanJsonConvertor<MetricAlertModel> convertor = new MetricAlertConvertor();

    @Override
    public BocloudResult list() {
        try {
            String api = "subscriptions/%s/providers/microsoft.insights/metricAlerts?api-version=2018-03-01";
            String url = String.format(api, this.getSubscriptionId());
            String response = this.request(AzureRestApiProvider.RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<MetricAlertModel> models = convertor.convertList(jsonObject.getJSONArray("value"));
            return new BocloudResult(true, models, "查询成功!");
        } catch (Exception e) {
            logger.error("list metricAlert error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(MetricAlertModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Insights/metricAlerts/%s?api-version=2018-03-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            JSONObject params = new JSONObject();
            params.put("location", "global");
            JSONObject properties = new JSONObject();
            properties.put("severity", model.getSeverity());//警报严重性 {0，1，2，3，4}
            properties.put("enabled", model.getEnabled());//指示是否启用指标警报的标志。
            properties.put("description", model.getDescription());//将包括在警报电子邮件中的指标警报的说明。
            properties.put("evaluationFrequency", model.getEvaluationFrequency());//指标警报的评估频率，以 ISO 8601 持续时间格式表示。
            properties.put("windowSize", model.getWindowSize());//用于根据阈值监视警报活动的时间段 (ISO 8601 duration 格式) 。
            properties.put("targetResourceType", model.getTargetResourceType());//	 创建/更新警报的目标资源 () 的资源类型。 对于 MultipleResourceMultipleMetricCriteria 是必需的。
            properties.put("autoMitigate", model.getAutoMitigate());//指示警报是否应自动解决的标志。 默认值为 true。
            JSONArray scopes = new JSONArray();//此指标警报的作用域的资源 id 列表。
            model.getResourceUuids().stream().forEach(instanceId -> scopes.add(instanceId));
            properties.put("scopes", scopes);
            JSONObject criteria = new JSONObject();
            //条件集合
            JSONArray allOf = new JSONArray();
            model.getMetricCriteriaModels().stream().forEach(metricCriteria -> {
                JSONObject MetricCriteria = new JSONObject();
                MetricCriteria.put("criterionType", "StaticThresholdCriterion");
                MetricCriteria.put("metricNamespace", metricCriteria.getMetricNamespace());
                MetricCriteria.put("metricName", metricCriteria.getMetricName());
                MetricCriteria.put("name", metricCriteria.getName());
                MetricCriteria.put("operator", metricCriteria.getOperator());
                MetricCriteria.put("timeAggregation", metricCriteria.getTimeAggregation());
                MetricCriteria.put("threshold", metricCriteria.getThreshold());
                allOf.add(MetricCriteria);
            });
            criteria.put("allOf", allOf);
            criteria.put("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria");
            properties.put("criteria", criteria);
            params.put("properties", properties);
            String response = this.request(RequestStatus.PUT, url, params.toJSONString());
            JSONObject object = JSONObject.parseObject(response);
            MetricAlertModel metricAlertModel = convertor.convertModel(object);
            return new BocloudResult(true, metricAlertModel, "创建成功!");
        } catch (Exception e) {
            logger.error("create metricAlert error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(MetricAlertModel model) {
        try {
            String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Insights/metricAlerts/%s?api-version=2018-03-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            String response = this.request(RequestStatus.DELETE, url, null);
            return new BocloudResult(true, response, "查询成功!");
        } catch (Exception e) {
            logger.error("remove metricAlert error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult modify(MetricAlertModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Insights/metricAlerts/%s?api-version=2018-03-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            JSONObject params = new JSONObject();
            params.put("location", "global");
            JSONObject properties = new JSONObject();
            properties.put("severity", model.getSeverity());//警报严重性 {0，1，2，3，4}
            properties.put("enabled", model.getEnabled());//指示是否启用指标警报的标志。
            properties.put("description", model.getDescription());//将包括在警报电子邮件中的指标警报的说明。
            properties.put("evaluationFrequency", model.getEvaluationFrequency());//指标警报的评估频率，以 ISO 8601 持续时间格式表示。
            properties.put("windowSize", model.getWindowSize());//用于根据阈值监视警报活动的时间段 (ISO 8601 duration 格式) 。
            properties.put("targetResourceType", "Microsoft.Compute/virtualMachines");//	 创建/更新警报的目标资源 () 的资源类型。 对于 MultipleResourceMultipleMetricCriteria 是必需的。
            properties.put("autoMitigate", model.getAutoMitigate());//指示警报是否应自动解决的标志。 默认值为 true。
            JSONArray scopes = new JSONArray();//此指标警报的作用域的资源 id 列表。
            model.getResourceUuids().stream().forEach(instanceId -> scopes.add(instanceId));
            properties.put("scopes", scopes);
            JSONObject criteria = new JSONObject();
            //条件集合
            JSONArray allOf = new JSONArray();
            model.getMetricCriteriaModels().stream().forEach(metricCriteria -> {
                JSONObject MetricCriteria = new JSONObject();
                MetricCriteria.put("criterionType", "StaticThresholdCriterion");
                MetricCriteria.put("metricNamespace", metricCriteria.getMetricNamespace());
                MetricCriteria.put("metricName", metricCriteria.getMetricName());
                MetricCriteria.put("name", metricCriteria.getName());
                MetricCriteria.put("operator", metricCriteria.getOperator());
                MetricCriteria.put("timeAggregation", metricCriteria.getTimeAggregation());
                MetricCriteria.put("threshold", metricCriteria.getThreshold());
                allOf.add(MetricCriteria);
            });
            criteria.put("allOf", allOf);
            criteria.put("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria");
            properties.put("criteria", criteria);
            params.put("properties", properties);
            String response = this.request(RequestStatus.PATCH, url, params.toJSONString());
            JSONObject object = JSONObject.parseObject(response);
            MetricAlertModel metricAlertModel = convertor.convertModel(object);
            return new BocloudResult(true, metricAlertModel, "创建成功!");
        } catch (Exception e) {
            logger.error("modify metricAlert error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}