package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.network.NetworkInterfaceConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkInterfaceModel;
import com.bocloud.cmp.model.azure.network.NicIpConfigurationModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureNetworkInterface
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/9/27 14:24
 */
@Provider("bocloud.cmp.azure")
public class AzureNetworkInterfaceProvider extends AzureProvider implements NetworkInterfaceProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureNetworkInterfaceProvider.class);

    private BeanConvertor<NetworkInterfaceModel, NetworkInterface> convertor = new NetworkInterfaceConvertor();

    public AzureNetworkInterfaceProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            List<NetworkInterfaceModel> regionModels = convertor.convertList(this.getAzure().networkInterfaces().list()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            return new BocloudResult(true, regionModels, "list networkInterface success!");
        } catch (Exception e) {
            logger.error("list networkInterface error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String netFaceId) {
        try {
            NetworkInterface networkInterface = this.getAzure().networkInterfaces().getById(netFaceId);
            return new BocloudResult(true, convertor.convertModel(networkInterface), "detail networkInterface success!");
        } catch (Exception e) {
            logger.error("detail networkInterface error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String netFaceId) {
        try {
            this.getAzure().networkInterfaces().deleteById(netFaceId);
            return new BocloudResult(true, "remove networkInterface success!");
        } catch (Exception e) {
            logger.error("remove networkInterface error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(NetworkInterfaceModel model) {
        return null;
    }

    @Override
    public BocloudResult vmChangeInterface(String netFaceId, String instanceId) {
        try {
            NetworkInterface networkInterface = this.getAzure().networkInterfaces().getById(netFaceId);
            this.getAzure().virtualMachines().getById(instanceId).update()
                    .withExistingSecondaryNetworkInterface(networkInterface)
                    .apply();
            return new BocloudResult(true, "vmChangeInterface networkInterface success!");
        } catch (Exception e) {
            logger.error("vmChangeInterface networkInterface error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult changeSubnet(NetworkInterfaceModel model) {
        try {
            NetworkInterface networkInterface = this.getAzure().networkInterfaces().getById(model.getNetworkInterfaceUuid());
            networkInterface.update().withSubnet(model.getSubnetName()).apply();
            return this.detail(model.getNetworkInterfaceUuid());
        } catch (Exception e) {
            logger.error("networkInterface changeSubnet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(NetworkInterfaceModel model) {
        try {
            Network network = this.getAzure().networks().getById(model.getNetworkUuid());
            NetworkInterface networkInterface;
            if (StringUtils.isEmpty(model.getPrimaryPrivateIPAddress())) {
                networkInterface = this.getAzure().networkInterfaces()
                        .define(model.getName())
                        .withRegion(model.getRegionId())
                        .withExistingResourceGroup(model.getResourceGroupName())
                        .withExistingPrimaryNetwork(network)
                        .withSubnet(model.getSubnetName())
                        .withPrimaryPrivateIPAddressDynamic()
                        .create();
            } else {
                networkInterface = this.getAzure().networkInterfaces()
                        .define(model.getName())
                        .withRegion(model.getRegionId())
                        .withExistingResourceGroup(model.getResourceGroupName())
                        .withExistingPrimaryNetwork(network)
                        .withSubnet(model.getSubnetName())
                        .withPrimaryPrivateIPAddressStatic(model.getPrimaryPrivateIPAddress())
                        .create();
            }
            NetworkInterfaceModel networkInterfaceModel = convertor.convertModel(networkInterface);
            return new BocloudResult(true, networkInterfaceModel, "create networkInterface success!");
        } catch (Exception e) {
            logger.error("create networkInterface error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult attachSecurityGroup(NetworkInterfaceModel model) {
        //安全组uuid NetworkInterfaceUuid
        try {
            NetworkSecurityGroup networkSecurityGroup = this.getAzure().networkSecurityGroups().getById(model.getSecurityGroupUuid());
            NetworkInterface networkInterface = this.getAzure().networkInterfaces().getById(model.getNetworkInterfaceUuid());
            networkInterface.update().withExistingNetworkSecurityGroup(networkSecurityGroup).apply();
            return new BocloudResult(true, "networkInterface withExisting securityGroup success!");
        } catch (Exception e) {
            logger.error("networkInterface withExisting securityGroup error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult attachPublicIpAddress(NetworkInterfaceModel model) {
        try {
            PublicIPAddress publicIPAddress = this.getAzure().publicIPAddresses().getById(model.getPublicIpAddressUuid());
            NetworkInterface networkInterface = this.getAzure()
                    .networkInterfaces()
                    .getById(model.getNetworkInterfaceUuid())
                    .update()
                    .withExistingPrimaryPublicIPAddress(publicIPAddress)
                    .apply();
            NetworkInterfaceModel networkInterfaceModel = convertor.convertModel(networkInterface);
            return new BocloudResult(true, networkInterfaceModel, "networkInterface withExisting publicIpAddress success!");
        } catch (Exception e) {
            logger.error("networkInterface withExisting publicIpAddress error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detachPublicIpAddress(NetworkInterfaceModel model) {
        try {
            NetworkInterface networkInterface = this.getAzure()
                    .networkInterfaces()
                    .getById(model.getNetworkInterfaceUuid())
                    .update()
                    .withoutPrimaryPublicIPAddress()
                    .apply();
            NetworkInterfaceModel networkInterfaceModel = convertor.convertModel(networkInterface);
            return new BocloudResult(true, networkInterfaceModel, "networkInterface without publicIpAddress success!");
        } catch (Exception e) {
            logger.error("networkInterface without publicIpAddress error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detachSecurityGroup(NetworkInterfaceModel model) {
        //NetworkInterfaceUuid
        try {
            NetworkInterface networkInterface = this.getAzure().networkInterfaces().getById(model.getNetworkInterfaceUuid());
            networkInterface.update().withoutNetworkSecurityGroup().apply();
            return new BocloudResult(true, "networkInterface without securityGroup success!");
        } catch (Exception e) {
            logger.error("networkInterface without securityGroup error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult attach(NetworkInterfaceModel model) {
        try {
            NetworkInterface networkInterface = this.getAzure().networkInterfaces().getById(model.getNetworkInterfaceUuid());
            this.getAzure().virtualMachines().getById(model.getVirtualMachineUuid()).update()
                    .withExistingSecondaryNetworkInterface(networkInterface)
                    .apply();
            return this.detail(model.getNetworkInterfaceUuid());
        } catch (Exception e) {
            logger.error("secondaryNetworkInterface attach virtualMachine error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detach(NetworkInterfaceModel model) {
        try {
            this.getAzure().virtualMachines().getById(model.getVirtualMachineUuid()).update()
                    .withoutSecondaryNetworkInterface(model.getName())
                    .apply();
            return this.detail(model.getNetworkInterfaceUuid());
        } catch (Exception e) {
            logger.error("secondaryNetworkInterface detach virtualMachine error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    //网络接口增加IP配置
    @Override
    public BocloudResult networkInterfaceAddNicIpConfiguration(NicIpConfigurationModel model) {
        try {
//            NetworkInterface networkInterface = null;
//            Network network = this.getAzure().networks().getById(model.getVpcUuid());
//            if (!StringUtils.isEmpty(model.getPublicIpAddressUuid())) {
//                PublicIPAddress publicIPAddress = this.getAzure().publicIPAddresses().getById(model.getPublicIpAddressUuid());
//                networkInterface = this.getAzure()
//                        .networkInterfaces()
//                        .getById(model.getNetworkInterfaceUuid())
//                        .update()
//                        .defineSecondaryIPConfiguration(model.getName())
//                        .withExistingNetwork(network)
//                        .withSubnet(model.getSubnetName())
//                        .withPrivateIPVersion(IPVersion.IPV4)
//                        .withExistingPublicIPAddress(publicIPAddress)
//                        .attach()
//                        .apply();
//            } else {
//                networkInterface = this.getAzure()
//                        .networkInterfaces()
//                        .getById(model.getNetworkInterfaceUuid())
//                        .update()
//                        .defineSecondaryIPConfiguration(model.getName())
//                        .withExistingNetwork(network)
//                        .withSubnet(model.getSubnetName())
//                        .withPrivateIPVersion(IPVersion.IPV4)
//                        .attach()
//                        .apply();
//            }
//            NetworkInterfaceModel networkInterfaceModel = convertor.convertModel(networkInterface);
//
//            NicIpConfigurationModel ipConfigurationModel = null;
//            List<NicIpConfigurationModel> nicIpConfigurationModels = networkInterfaceModel.getNicIpConfigurationModels();
//            for (NicIpConfigurationModel nicIpConfigurationModel : nicIpConfigurationModels) {
//                if (!nicIpConfigurationModel.getIsPrimary()) {
//                    ipConfigurationModel = nicIpConfigurationModel;
//                }
//
//            }
//            return new BocloudResult(true, ipConfigurationModel, "networkInterface Add NicIpConfiguration success!");

            //第一步：获取网络接口的数据
            BocloudResult detailResult = this.detail(model.getNetworkInterfaceUuid());
            if (detailResult.isFailed()) {
                return new BocloudResult(false, "网络接口不存在");
            }
            NetworkInterfaceModel interfaceModel = JSONObject.parseObject(JSONObject.toJSONString(detailResult.getData()), NetworkInterfaceModel.class);
            //第二步：用API在获取一遍
            AzureRestApiProvider provider = new AzureRestApiProvider(this.getButler());
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/networkInterfaces/%s?api-version=2020-11-01";
            String url = String.format(api, provider.getSubscriptionId(), interfaceModel.getResourceGroupName(), interfaceModel.getName());
            String response = provider.request(AzureRestApiProvider.RequestStatus.GET, url, null);
            JSONObject oldResponse = JSONObject.parseObject(response);
            JSONObject oldProperties = oldResponse.getJSONObject("properties");
            JSONArray oldIpConfigurations = oldProperties.getJSONArray("ipConfigurations");
            //第三步：新增ip配置
            JSONObject params = new JSONObject();
            JSONObject properties = new JSONObject();
            JSONObject ipConfiguration = new JSONObject();
            ipConfiguration.put("name", model.getName());
            JSONObject ipcProperties = new JSONObject();
            ipcProperties.put("privateIPAddressVersion", "IPv4");
            ipcProperties.put("privateIPAllocationMethod", "Dynamic");
            JSONObject subnet = new JSONObject();
            subnet.put("id", model.getSubnetUuid());
            ipcProperties.put("subnet", subnet);
            if (!StringUtils.isEmpty(model.getPublicIpAddressUuid())) {
                JSONObject publicIPAddress = new JSONObject();
                publicIPAddress.put("id", model.getPublicIpAddressUuid());
                ipcProperties.put("publicIPAddress", publicIPAddress);
            }
            ipConfiguration.put("properties", ipcProperties);
            oldIpConfigurations.add(ipConfiguration);
            properties.put("ipConfigurations", oldIpConfigurations);
            params.put("properties", properties);
            params.put("location", interfaceModel.getRegionId());
            String putResponse = provider.request(AzureRestApiProvider.RequestStatus.PUT, url, params.toJSONString());
            JSONObject newResponse = JSONObject.parseObject(putResponse);
            JSONObject newProperties = newResponse.getJSONObject("properties");
            JSONArray newIpConfigurations = newProperties.getJSONArray("ipConfigurations");

            JSONObject ipconfig = null;
            for (int i = 0; i < newIpConfigurations.size(); i++) {
                JSONObject jsonObject = newIpConfigurations.getJSONObject(i);
                String name = jsonObject.getString("name");
                if (name.equalsIgnoreCase(model.getName())) {
                    ipconfig = jsonObject;
                    break;
                }
            }
            if (null == ipconfig) {
                return new BocloudResult(false, "未生效，创建失败！");
            }
            model.setUuid(ipconfig.getString("id"));
            model.setName(ipconfig.getString("name"));
            Optional.ofNullable(ipconfig.getJSONObject("properties")).ifPresent(props -> {
                model.setIsPrimary(props.getBoolean("primary"));
                model.setPrivateIpAddress(props.getString("privateIPAddress"));
                model.setIpMethod(props.getString("privateIPAllocationMethod"));
            });
            return new BocloudResult(true, model, "networkInterface Add NicIpConfiguration success!");
        } catch (Exception e) {
            logger.error("networkInterface Add NicIpConfiguration error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult networkInterfaceModifyNicIpConfiguration(NicIpConfigurationModel model) {
        try {
            NetworkInterface networkInterface;
            if (model.getIpMethod().equalsIgnoreCase("Dynamic")) {
                networkInterface = this.getAzure()
                        .networkInterfaces()
                        .getById(model.getNetworkInterfaceUuid())
                        .update()
                        .updateIPConfiguration(model.getName())
                        .withPrivateIPAddressDynamic()
                        .parent()
                        .apply();
            } else {
                networkInterface = this.getAzure()
                        .networkInterfaces()
                        .getById(model.getNetworkInterfaceUuid())
                        .update()
                        .updateIPConfiguration(model.getName())
                        .withPrivateIPAddressStatic(model.getPrivateIpAddress())
                        .parent()
                        .apply();
            }
            NicIPConfiguration nicIPConfiguration = networkInterface.ipConfigurations().get(model.getName());
            model.setPrivateIpAddress(nicIPConfiguration.privateIPAddress());
            model.setIpMethod(nicIPConfiguration.privateIPAllocationMethod().toString());
            return new BocloudResult(true, model, "networkInterface Modify NicIpConfiguration success!");
        } catch (Exception e) {
            logger.error("networkInterface Modify NicIpConfiguration error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult networkInterfaceRemoveNicIpConfiguration(NicIpConfigurationModel model) {
        try {
            this.getAzure()
                    .networkInterfaces()
                    .getById(model.getNetworkInterfaceUuid())
                    .update()
                    .withoutIPConfiguration(model.getName())
                    .apply();
            return new BocloudResult(true, "networkInterface Remove NicIpConfiguration success!");
        } catch (Exception e) {
            logger.error("networkInterface Remove NicIpConfiguration error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    //网络接口挂载IP
    @Override
    public BocloudResult nicIpConfigurationAttachPublicIpAddress(NicIpConfigurationModel model) {
        try {
            PublicIPAddress publicIPAddress = this.getAzure().publicIPAddresses().getById(model.getPublicIpAddressUuid());
            NetworkInterface networkInterface = this.getAzure()
                    .networkInterfaces()
                    .getById(model.getNetworkInterfaceUuid())
                    .update()
                    .updateIPConfiguration(model.getName())
                    .withExistingPublicIPAddress(publicIPAddress)
                    .parent()
                    .apply();
            NetworkInterfaceModel networkInterfaceModel = convertor.convertModel(networkInterface);
            return new BocloudResult(true, networkInterfaceModel, "nicIpConfiguration withExisting publicIpAddress success!");
        } catch (Exception e) {
            logger.error("nicIpConfiguration withExisting publicIpAddress error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult nicIpConfigurationdetachPublicIpAddress(NicIpConfigurationModel model) {
        try {
            NetworkInterface networkInterface = this.getAzure()
                    .networkInterfaces()
                    .getById(model.getNetworkInterfaceUuid())
                    .update()
                    .updateIPConfiguration(model.getName())
                    .withoutPublicIPAddress()
                    .parent()
                    .apply();
            NetworkInterfaceModel networkInterfaceModel = convertor.convertModel(networkInterface);
            return new BocloudResult(true, networkInterfaceModel, "nicIpConfiguration without publicIpAddress success!");
        } catch (Exception e) {
            logger.error("nicIpConfiguration without publicIpAddress error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult addDnsServer(NetworkInterfaceModel model) {
        try {
            NetworkInterface networkInterface = this.getAzure()
                    .networkInterfaces()
                    .getById(model.getNetworkInterfaceUuid())
                    .update()
                    .withDnsServer(model.getDnsServer())
                    .apply();
            NetworkInterfaceModel networkInterfaceModel = convertor.convertModel(networkInterface);
            return new BocloudResult(true, networkInterfaceModel, "addDnsServer networkInterface success!");
        } catch (Exception e) {
            logger.error("networkInterface addDnsServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult removeDnsServer(NetworkInterfaceModel model) {
        try {
            NetworkInterface networkInterface = this.getAzure()
                    .networkInterfaces()
                    .getById(model.getNetworkInterfaceUuid())
                    .update()
                    .withoutDnsServer(model.getDnsServer())
                    .apply();
            NetworkInterfaceModel networkInterfaceModel = convertor.convertModel(networkInterface);
            return new BocloudResult(true, networkInterfaceModel, "removeDnsServer networkInterface success!");
        } catch (Exception e) {
            logger.error("networkInterface removeDnsServer error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult effectiveRouterTable(NetworkInterfaceModel model) {
        try {
            AzureRestApiProvider provider = new AzureRestApiProvider(this.getButler());
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/networkInterfaces/%s/effectiveRouteTable?api-version=2020-11-01";
            String url = String.format(api, provider.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            String response = provider.request(AzureRestApiProvider.RequestStatus.POST, url, null);
            return new BocloudResult(true, response, "查询成功");
        } catch (Exception e) {
            logger.error("networkInterface effectiveRouterTable error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }

    @Override
    public BocloudResult ipForwarding(NetworkInterfaceModel model) {
        try {
            if (model.getIpForwarding()) {
                this.getAzure()
                        .networkInterfaces()
                        .getById(model.getNetworkInterfaceUuid())
                        .update()
                        .withIPForwarding()
                        .apply();
            } else {
                this.getAzure()
                        .networkInterfaces()
                        .getById(model.getNetworkInterfaceUuid())
                        .update()
                        .withoutIPForwarding()
                        .apply();
            }
            return new BocloudResult(true, "NetworkInterface ipForwarding success!");
        } catch (Exception e) {
            logger.error("NetworkInterface ipForwarding error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


}
