package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.azureconvertor.network.PublicIPAddressConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.PublicIPAddressModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.PublicIPAddress;
import com.microsoft.azure.management.network.PublicIPAddressSku;
import com.microsoft.azure.management.network.PublicIPAddressSkuName;
import com.microsoft.azure.management.network.PublicIPSkuType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @ClassName: AzurePublicIPAddressProvider
 * @Description:
 * @author: zhenpingLi/李振平
 */
@Provider("bocloud.cmp.azure")
public class AzurePublicIPAddressProvider extends AzureProvider implements PublicIPAddressProvider {

    public AzurePublicIPAddressProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzurePublicIPAddressProvider.class);

    private BeanConvertor<PublicIPAddressModel, PublicIPAddress> convertor = new PublicIPAddressConvertor();

    @Override
    public BocloudResult list() {
        try {
            List<PublicIPAddressModel> regionModels = convertor.convertList(this.getAzure().publicIPAddresses().list()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            return new BocloudResult(true, regionModels, "list publicIPAddress success!");
        } catch (Exception e) {
            logger.error("list publicIPAddress error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(PublicIPAddressModel model) {
        try {
//            //poc先写死，不动态区分 回归后在进行区分sku基本和标准
//            PublicIPAddress publicIPAddress = null;
//            switch (model.getIpAllocationMethod()) {
//                case "Dynamic":
//                    publicIPAddress = this.getAzure().publicIPAddresses()
//                            .define(model.getName())
//                            .withRegion(model.getRegionId())
//                            .withExistingResourceGroup(model.getResourceGroupName())
//                            .withSku(PublicIPSkuType.BASIC)
//                            .withIdleTimeoutInMinutes(model.getIdleTimeoutInMinutes())//空闲超时（分钟）
//                            .withDynamicIP()
//                            .create();
//                    break;
//                case "Static":
//                    publicIPAddress = this.getAzure().publicIPAddresses()
//                            .define(model.getName())
//                            .withRegion(model.getRegionId())
//                            .withExistingResourceGroup(model.getResourceGroupName())
//                            .withSku(PublicIPSkuType.STANDARD)
//                            .withIdleTimeoutInMinutes(model.getIdleTimeoutInMinutes())//空闲超时（分钟）
//                            .withStaticIP()
//                            .create();
//                    break;
//                default:
//                    break;
//            }
            PublicIPAddress.DefinitionStages.WithCreate withCreate1 = this.withCreate(model);
            PublicIPAddress.DefinitionStages.WithCreate withCreate2 = this.withCreateAllocationMethod(model, withCreate1);
            PublicIPAddress.DefinitionStages.WithCreate withCreate3 = this.withCreateFqdn(model, withCreate2);
            PublicIPAddress publicIPAddress = withCreate3.create();
            return new BocloudResult(true, convertor.convertModel(publicIPAddress), "create publicIPAddress success!");
        } catch (Exception e) {
            logger.error("create publicIPAddress error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    private PublicIPAddress.DefinitionStages.WithCreate withCreate(PublicIPAddressModel model) {
        //sku
        PublicIPAddressSku sku = new PublicIPAddressSku();
        if (!StringUtils.isEmpty(model.getSku())) {
            sku.withName(PublicIPAddressSkuName.fromString(model.getSku()));
        } else {
            sku.withName(PublicIPAddressSkuName.STANDARD);
        }
        PublicIPAddress.DefinitionStages.WithCreate withCreate = this.getAzure().publicIPAddresses()
                .define(model.getName())
                .withRegion(model.getRegionId())
                .withExistingResourceGroup(model.getResourceGroupName())
                .withSku(PublicIPSkuType.fromSku(sku))
                .withIdleTimeoutInMinutes(model.getIdleTimeoutInMinutes());
        return withCreate;
    }

    private PublicIPAddress.DefinitionStages.WithCreate withCreateAllocationMethod(PublicIPAddressModel model, PublicIPAddress.DefinitionStages.WithCreate withCreate) {
        PublicIPAddress.DefinitionStages.WithCreate withCreate1 = null;
        if (model.getIpAllocationMethod().equalsIgnoreCase("Dynamic")) {
            withCreate1 = withCreate.withDynamicIP();
        } else if (model.getIpAllocationMethod().equalsIgnoreCase("Static")) {
            withCreate1 = withCreate.withStaticIP();
        }
        return withCreate1;
    }

    private PublicIPAddress.DefinitionStages.WithCreate withCreateFqdn(PublicIPAddressModel model, PublicIPAddress.DefinitionStages.WithCreate withCreate) {
        if (!StringUtils.isEmpty(model.getFqdn())) {
            withCreate.withLeafDomainLabel(model.getFqdn());
        }
        return withCreate;
    }

    @Override
    public BocloudResult modify(PublicIPAddressModel model) {
        try {
            PublicIPAddress.Update withUpdate = this.withUpdate(model);
            PublicIPAddress.Update withUpdate1 = this.withUpdateFqdn(model, withUpdate);
            PublicIPAddress publicIPAddress = withUpdate1.apply();
            return new BocloudResult(true, convertor.convertModel(publicIPAddress), "modify publicIPAddress success!");
        } catch (Exception e) {
            logger.error("modify publicIPAddress error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    private PublicIPAddress.Update withUpdate(PublicIPAddressModel model) {
        PublicIPAddress.Update update = this.getAzure()
                .publicIPAddresses()
                .getById(model.getUuid())
                .update()
                .withIdleTimeoutInMinutes(model.getIdleTimeoutInMinutes());
        return update;
    }

    private PublicIPAddress.Update withUpdateFqdn(PublicIPAddressModel model, PublicIPAddress.Update update) {
        PublicIPAddress.Update update1 = null;
        if (!StringUtils.isEmpty(model.getFqdn())) {
            update1 = update.withLeafDomainLabel(model.getFqdn());
        } else {
            update1 = update.withoutLeafDomainLabel();
        }
        return update1;
    }

    @Override
    public BocloudResult remove(String uuid) {
        try {
            this.getAzure().publicIPAddresses().deleteById(uuid);
            return new BocloudResult(true, "remove publicIPAddress success!");
        } catch (Exception e) {
            logger.error("remove publicIPAddress error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String uuid) {
        try {
            PublicIPAddress publicIPAddress = this.getAzure().publicIPAddresses().getById(uuid);
            return new BocloudResult(true, convertor.convertModel(publicIPAddress), "detail publicIPAddress success!");
        } catch (Exception e) {
            logger.error("detail publicIPAddress error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


}