package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.NetworkSecurityGroup;
import com.microsoft.azure.management.network.NetworkSecurityRule;
import com.microsoft.azure.management.network.SecurityRuleProtocol;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureSecuGroupRuleProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/30 17:24
 */
@Provider("bocloud.cmp.azure")
public class AzureSecuGroupRuleProvider extends AzureProvider implements SecurityGroupRuleProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureSecuGroupRuleProvider.class);

    public AzureSecuGroupRuleProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(SecurityGroupRuleModel model) {
        try {
            logger.info("安全组规则创建----------------------------------------------------");
            logger.info("源地址:{}", model.getSourceAddressPrefix());
            logger.info("源端口范围:{}", model.getSourcePortRange());
            logger.info("目标地址:{}", model.getDestinationAddressPrefix());
            logger.info("目标端口范围:{}", model.getDestinationPortRange());
            SecurityGroupRuleModel groupRuleModel = this.getCreateWay(model);
            return new BocloudResult(true, groupRuleModel, "create networkSecurityGroupRule success!");
        } catch (Exception e) {
            logger.error("create networkSecurityGroupRule error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    private SecurityGroupRuleModel getCreateWay(SecurityGroupRuleModel model) {
        NetworkSecurityGroup networkSecurityGroup = this.getAzure().networkSecurityGroups().getById(model.getGroupUuid());
        //获取名称
        NetworkSecurityRule.UpdateDefinitionStages.Blank<NetworkSecurityGroup.Update> defineRule = networkSecurityGroup.update().defineRule(model.getName());
        //获取方向和action
        NetworkSecurityRule.UpdateDefinitionStages.WithSourceAddressOrSecurityGroup<NetworkSecurityGroup.Update> update1 = this.getDirection(defineRule, model);
        //获取源地址
        NetworkSecurityRule.UpdateDefinitionStages.WithSourcePort<NetworkSecurityGroup.Update> updata2 = this.getFromAddress(update1, model);
        //获取源端口范围
        NetworkSecurityRule.UpdateDefinitionStages.WithDestinationAddressOrSecurityGroup<NetworkSecurityGroup.Update> update3 = this.getFromPort(updata2, model);
        //获取目标地址
        NetworkSecurityRule.UpdateDefinitionStages.WithDestinationPort<NetworkSecurityGroup.Update> uodate4 = this.getToAddress(update3, model);
        //获取目标端口范围
        NetworkSecurityRule.UpdateDefinitionStages.WithProtocol<NetworkSecurityGroup.Update> update5 = this.getToPort(uodate4, model);
        //获取协议
        NetworkSecurityRule.UpdateDefinitionStages.WithAttach<NetworkSecurityGroup.Update> update6 = this.getProtocol(update5, model);
        //执行创建
        NetworkSecurityGroup apply = update6.withDescription(model.getRemark())
                .withPriority(model.getPriority())
                .attach()
                .apply();
        //数据确认、转换、返回
        Map<String, NetworkSecurityRule> ruleMap = apply.securityRules();
        NetworkSecurityRule networkSecurityRule = null;
        if (null != ruleMap && !ruleMap.isEmpty()) {
            networkSecurityRule = ruleMap.get(model.getName());
        }
        SecurityGroupRuleModel groupRuleModel = convertorRule(networkSecurityRule, model.getGroupUuid(), false);
        return groupRuleModel;
    }

    //获取规则方向和action
    private NetworkSecurityRule.UpdateDefinitionStages.WithSourceAddressOrSecurityGroup<NetworkSecurityGroup.Update> getDirection(NetworkSecurityRule.UpdateDefinitionStages.Blank<NetworkSecurityGroup.Update> defineRule, SecurityGroupRuleModel model) {
        NetworkSecurityRule.UpdateDefinitionStages.WithSourceAddressOrSecurityGroup<NetworkSecurityGroup.Update> update = null;
        if (model.getDirection().equalsIgnoreCase("Inbound")) { //入站
            if (model.getAction().equalsIgnoreCase("Allow")) {//允许
                update = defineRule.allowInbound();
            } else if (model.getAction().equalsIgnoreCase("Deny")) {//拒绝
                update = defineRule.denyInbound();
            }
        } else if (model.getDirection().equalsIgnoreCase("Outbound")) {//出战
            if (model.getAction().equalsIgnoreCase("Allow")) {//允许
                update = defineRule.allowOutbound();
            } else if (model.getAction().equalsIgnoreCase("Deny")) {//拒绝
                update = defineRule.denyOutbound();
            }
        }
        return update;
    }

    //获取源地址
    private NetworkSecurityRule.UpdateDefinitionStages.WithSourcePort<NetworkSecurityGroup.Update> getFromAddress(NetworkSecurityRule.UpdateDefinitionStages.WithSourceAddressOrSecurityGroup<NetworkSecurityGroup.Update> updateWithSourceAddressOrSecurityGroup, SecurityGroupRuleModel model) {
        NetworkSecurityRule.UpdateDefinitionStages.WithSourcePort<NetworkSecurityGroup.Update> updateWithSourcePort = null;
        if (model.getSourceAddressPrefix().equalsIgnoreCase("Any")) {
            updateWithSourcePort = updateWithSourceAddressOrSecurityGroup.fromAnyAddress();
        } else {
            List<String> sourceAddressPrefixs = JSONArray.parseArray(model.getSourceAddressPrefix(), String.class);
            // updateWithSourcePort = updateWithSourceAddressOrSecurityGroup.fromAddresses(model.getSourceAddressPrefix());
            updateWithSourcePort = updateWithSourceAddressOrSecurityGroup.fromAddresses(sourceAddressPrefixs.toArray(new String[]{}));
        }
        return updateWithSourcePort;
    }

    //获取源端口范围
    private NetworkSecurityRule.UpdateDefinitionStages.WithDestinationAddressOrSecurityGroup<NetworkSecurityGroup.Update> getFromPort(NetworkSecurityRule.UpdateDefinitionStages.WithSourcePort<NetworkSecurityGroup.Update> updateWithSourcePort, SecurityGroupRuleModel model) {
        NetworkSecurityRule.UpdateDefinitionStages.WithDestinationAddressOrSecurityGroup<NetworkSecurityGroup.Update> updateWithDestinationAddressOrSecurityGroup = null;
        if (model.getSourcePortRange().equalsIgnoreCase("*")) {
            updateWithDestinationAddressOrSecurityGroup = updateWithSourcePort.fromAnyPort();
        } else {
            List<String> sourcePortRanges = JSONArray.parseArray(model.getSourcePortRange(), String.class);
//updateWithDestinationAddressOrSecurityGroup = updateWithSourcePort.fromPort(Integer.parseInt(model.getSourcePortRange()));
            updateWithDestinationAddressOrSecurityGroup = updateWithSourcePort.fromPortRanges(sourcePortRanges.toArray(new String[]{}));
        }
        return updateWithDestinationAddressOrSecurityGroup;
    }

    //获取目标地址
    private NetworkSecurityRule.UpdateDefinitionStages.WithDestinationPort<NetworkSecurityGroup.Update> getToAddress(NetworkSecurityRule.UpdateDefinitionStages.WithDestinationAddressOrSecurityGroup<NetworkSecurityGroup.Update> withDestinationAddressOrSecurityGroup, SecurityGroupRuleModel model) {
        NetworkSecurityRule.UpdateDefinitionStages.WithDestinationPort<NetworkSecurityGroup.Update> updateWithDestinationPort = null;
        if (model.getDestinationAddressPrefix().equalsIgnoreCase("Any")) {
            updateWithDestinationPort = withDestinationAddressOrSecurityGroup.toAnyAddress();
        } else {
            List<String> destinationAddressPrefixs = JSONArray.parseArray(model.getDestinationAddressPrefix(), String.class);
            //updateWithDestinationPort = withDestinationAddressOrSecurityGroup.toAddresses(model.getDestinationAddressPrefix());
            updateWithDestinationPort = withDestinationAddressOrSecurityGroup.toAddresses(destinationAddressPrefixs.toArray(new String[]{}));
        }
        return updateWithDestinationPort;
    }

    //获取目标端口范围
    private NetworkSecurityRule.UpdateDefinitionStages.WithProtocol<NetworkSecurityGroup.Update> getToPort(NetworkSecurityRule.UpdateDefinitionStages.WithDestinationPort<NetworkSecurityGroup.Update> withDestinationPort, SecurityGroupRuleModel model) {
        NetworkSecurityRule.UpdateDefinitionStages.WithProtocol<NetworkSecurityGroup.Update> updateWithProtocol = null;
        if (model.getDestinationPortRange().equalsIgnoreCase("*")) {
            updateWithProtocol = withDestinationPort.toAnyPort();
        } else {
            List<String> destinationPortRanges = JSONArray.parseArray(model.getDestinationPortRange(), String.class);
            //updateWithProtocol = withDestinationPort.toPort(Integer.parseInt(model.getDestinationPortRange()));
            updateWithProtocol = withDestinationPort.toPortRanges(destinationPortRanges.toArray(new String[]{}));
        }
        return updateWithProtocol;
    }

    //获取协议
    private NetworkSecurityRule.UpdateDefinitionStages.WithAttach<NetworkSecurityGroup.Update> getProtocol(NetworkSecurityRule.UpdateDefinitionStages.WithProtocol<NetworkSecurityGroup.Update> withProtocol, SecurityGroupRuleModel model) {
        NetworkSecurityRule.UpdateDefinitionStages.WithAttach<NetworkSecurityGroup.Update> updateWithAttach = null;
        if (model.getProtocol().equalsIgnoreCase("*")) {
            updateWithAttach = withProtocol.withAnyProtocol();
        } else {
            updateWithAttach = withProtocol.withProtocol(SecurityRuleProtocol.fromString(model.getProtocol()));
        }
        return updateWithAttach;
    }

    @Override
    public BocloudResult remove(SecurityGroupRuleModel ruleModel) {
        try {
            NetworkSecurityGroup networkSecurityGroup = this.getAzure().networkSecurityGroups().getById(ruleModel.getGroupUuid());
            networkSecurityGroup.update().withoutRule(ruleModel.getName()).apply();
            return new BocloudResult(true, "remove networkSecurityGroupRule success!");
        } catch (Exception e) {
            logger.error("remove networkSecurityGroupRule error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(SecurityGroupRuleModel model) {
        try {
            SecurityGroupRuleModel groupRuleModel = this.getModifyWay(model);
            return new BocloudResult(true, groupRuleModel, "modify networkSecurityGroupRule success!");
        } catch (Exception e) {
            logger.error("modify networkSecurityGroupRule error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    private SecurityGroupRuleModel getModifyWay(SecurityGroupRuleModel model) {
        NetworkSecurityGroup networkSecurityGroup = this.getAzure().networkSecurityGroups().getById(model.getGroupUuid());
        //获取名称
        NetworkSecurityRule.Update updateRule = networkSecurityGroup.update().updateRule(model.getName());
        //获取方向和action
        NetworkSecurityRule.Update update1 = this.getDirection(updateRule, model);
        //获取源地址
        NetworkSecurityRule.Update updata2 = this.getFromAddress(update1, model);
        //获取源端口范围
        NetworkSecurityRule.Update update3 = this.getFromPort(updata2, model);
        //获取目标地址
        NetworkSecurityRule.Update uodate4 = this.getToAddress(update3, model);
        //获取目标端口范围
        NetworkSecurityRule.Update update5 = this.getToPort(uodate4, model);
        //获取协议
        NetworkSecurityRule.Update update6 = this.getProtocol(update5, model);
        //执行修改
        NetworkSecurityGroup apply = update6.withDescription(model.getRemark())
                .withPriority(model.getPriority())
                .parent()
                .apply();
        //数据确认、转换、返回
        Map<String, NetworkSecurityRule> ruleMap = apply.securityRules();
        NetworkSecurityRule networkSecurityRule = null;
        if (null != ruleMap && !ruleMap.isEmpty()) {
            networkSecurityRule = ruleMap.get(model.getName());
        }
        SecurityGroupRuleModel groupRuleModel = convertorRule(networkSecurityRule, model.getGroupUuid(), false);
        return groupRuleModel;
    }

    //获取规则方向和action
    private NetworkSecurityRule.Update getDirection(NetworkSecurityRule.Update updateRule, SecurityGroupRuleModel model) {
        NetworkSecurityRule.Update update = null;
        if (model.getDirection().equalsIgnoreCase("Inbound")) { //入站
            if (model.getAction().equalsIgnoreCase("Allow")) {//允许
                update = updateRule.allowInbound();
            } else if (model.getAction().equalsIgnoreCase("Deny")) {//拒绝
                update = updateRule.denyInbound();
            }
        } else if (model.getDirection().equalsIgnoreCase("Outbound")) {//出战
            if (model.getAction().equalsIgnoreCase("Allow")) {//允许
                update = updateRule.allowOutbound();
            } else if (model.getAction().equalsIgnoreCase("Deny")) {//拒绝
                update = updateRule.denyOutbound();
            }
        }
        return update;
    }

    //获取源地址
    private NetworkSecurityRule.Update getFromAddress(NetworkSecurityRule.Update updateRule, SecurityGroupRuleModel model) {
        NetworkSecurityRule.Update update = null;
        if (model.getSourceAddressPrefix().equalsIgnoreCase("Any")) {
            update = updateRule.fromAnyAddress();
        } else {
            List<String> sourceAddressPrefixs = JSONArray.parseArray(model.getSourceAddressPrefix(), String.class);
            update = updateRule.fromAddresses(sourceAddressPrefixs.toArray(new String[]{}));
        }
        return update;
    }

    //获取源端口范围
    private NetworkSecurityRule.Update getFromPort(NetworkSecurityRule.Update updateRule, SecurityGroupRuleModel model) {
        NetworkSecurityRule.Update update = null;
        if (model.getSourcePortRange().equalsIgnoreCase("*")) {
            update = updateRule.fromAnyPort();
        } else {
            List<String> sourcePortRanges = JSONArray.parseArray(model.getSourcePortRange(), String.class);
            update = updateRule.fromPortRanges(sourcePortRanges.toArray(new String[]{}));
        }
        return update;
    }

    //获取目标地址
    private NetworkSecurityRule.Update getToAddress(NetworkSecurityRule.Update updateRule, SecurityGroupRuleModel model) {
        NetworkSecurityRule.Update update = null;
        if (model.getDestinationAddressPrefix().equalsIgnoreCase("Any")) {
            update = updateRule.toAnyAddress();
        } else {
            List<String> destinationAddressPrefixs = JSONArray.parseArray(model.getDestinationAddressPrefix(), String.class);
            update = updateRule.toAddresses(destinationAddressPrefixs.toArray(new String[]{}));
        }
        return update;
    }

    //获取目标端口范围
    private NetworkSecurityRule.Update getToPort(NetworkSecurityRule.Update updateRule, SecurityGroupRuleModel model) {
        NetworkSecurityRule.Update update = null;
        if (model.getDestinationPortRange().equalsIgnoreCase("*")) {
            update = updateRule.toAnyPort();
        } else {
            List<String> destinationPortRanges = JSONArray.parseArray(model.getDestinationPortRange(), String.class);
            update = updateRule.toPortRanges(destinationPortRanges.toArray(new String[]{}));
        }
        return update;
    }

    //获取协议
    private NetworkSecurityRule.Update getProtocol(NetworkSecurityRule.Update updateRule, SecurityGroupRuleModel model) {
        NetworkSecurityRule.Update update = null;
        if (model.getProtocol().equalsIgnoreCase("*")) {
            update = updateRule.withAnyProtocol();
        } else {
            update = updateRule.withProtocol(SecurityRuleProtocol.fromString(model.getProtocol()));
        }
        return update;
    }

    /**
     * 安全组规则转换
     *
     * @param rule
     * @param groupUuid
     * @return
     */
    private SecurityGroupRuleModel convertorRule(NetworkSecurityRule rule, String groupUuid, Boolean defaultRule) {
        SecurityGroupRuleModel model = new SecurityGroupRuleModel();
        if (null != rule) {
            model.setRuleUuid(rule.inner().id());
            model.setName(rule.name());
            if (null != rule.sourceAddressPrefix()) {
                model.setSourceAddressPrefix(rule.sourceAddressPrefix());//源地址前缀
            } else {
                model.setSourceAddressPrefix(JSONObject.toJSONString(rule.sourceAddressPrefixes()));//源地址前缀
            }
            if (null != rule.sourcePortRange()) {
                model.setSourcePortRange(rule.sourcePortRange()); //源端口范围
            } else {
                model.setSourcePortRange(JSONObject.toJSONString(rule.sourcePortRanges())); //源端口范围
            }
            if (null != rule.destinationAddressPrefix()) {
                model.setDestinationAddressPrefix(rule.destinationAddressPrefix());//终点地址前缀
            } else {
                model.setDestinationAddressPrefix(JSONObject.toJSONString(rule.destinationAddressPrefixes()));//终点地址前缀
            }
            if (null != rule.destinationPortRange()) {
                model.setDestinationPortRange(rule.destinationPortRange());//终点端口范围
            } else {
                model.setDestinationPortRange(JSONObject.toJSONString(rule.destinationPortRanges()));//终点端口范围
            }
            model.setRemark(rule.description()); //描述
            model.setDirection(rule.direction().toString());//方向
            model.setProtocol(rule.protocol().toString()); //协议
            model.setDefaultRule(defaultRule); //是否默认安全组规则
            model.setGroupUuid(groupUuid); //安全组规格id
            model.setAction(rule.access().toString());//操作 允许/拒绝
            model.setPriority(rule.priority()); //优先级
        }
        return model;
    }

}
