package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.azureconvertor.network.SubnetConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.Network;
import com.microsoft.azure.management.network.Subnet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;


/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureSubnetProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/9/23 14:37
 */
@Provider("bocloud.cmp.azure")
public class AzureSubnetProvider extends AzureProvider implements SubnetProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureSubnetProvider.class);

    private BeanConvertor<SubnetModel, Subnet> convertor = new SubnetConvertor();

    public AzureSubnetProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(SubnetModel subnetModel) {
        //所需参数 vpcUuid cidr name  securityGroupUuid
        try {
            //获取网络对象
            Network network = this.getAzure().networks().getById(subnetModel.getVpcUuid());
            //todo 不进行验证命名空间，默认VPC只有一个命名空间
//            获取网络下全部命名空间
//            List<String> addressSpaces = network.addressSpaces();
//            boolean addressSpaceExis = false;
            //验证创建子网的命名空间是否存在
//            for (String addressSpace : addressSpaces) {
//                if (addressSpace.equals(subnetModel.getCidr())) {
//                    addressSpaceExis = true;
//                }
//            }
            Network.Update withAddressSpace = this.withAddressSpace(subnetModel, network, true);
            Subnet.UpdateDefinitionStages.WithAttach<Network.Update> updateWithAttach = this.updateWithAttach(subnetModel, withAddressSpace);
            Subnet.UpdateDefinitionStages.WithAttach<Network.Update> withSecurityGroup = this.withSecurityGroup(subnetModel, updateWithAttach);
            network = withSecurityGroup.attach().apply();
            List<Subnet> list = new ArrayList(network.subnets().values());
            Subnet newSubnet = null;
            for (Subnet subnet : list) {
                if (subnet.name().equals(subnetModel.getName())) {
                    newSubnet = subnet;
                    break;
                }
            }
            SubnetModel model = convertor.convertModel(newSubnet);
            model.setVpcUuid(subnetModel.getVpcUuid());
            return new BocloudResult(true, model, "create subnet success!");
        } catch (Exception e) {
            logger.error("create subnet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    private Network.Update withAddressSpace(SubnetModel subnetModel, Network network, boolean addressSpaceExis) {
        //如果命名空间不存在，需要创建命名空间
        Network.Update update = network.update();
        if (!addressSpaceExis) {
            update = network.update().withAddressSpace(subnetModel.getCidr());
        }
        return update;
    }

    private Subnet.UpdateDefinitionStages.WithAttach<Network.Update> updateWithAttach(SubnetModel subnetModel, Network.Update update) {
        return update.defineSubnet(subnetModel.getName())
                .withAddressPrefix(subnetModel.getCidr());
    }

    private Subnet.UpdateDefinitionStages.WithAttach<Network.Update> withSecurityGroup(SubnetModel subnetModel, Subnet.UpdateDefinitionStages.WithAttach<Network.Update> updateWithAttach) {
        if (null != subnetModel.getSecurityGroupUuid()) {
            updateWithAttach = updateWithAttach.withExistingNetworkSecurityGroup(subnetModel.getSecurityGroupUuid());
        }
        return updateWithAttach;
    }


    @Override
    public BocloudResult modify(SubnetModel subnetModel) {
        //所需参数 vpcUuid cidr name  securityGroupUuid
        try {
            Network network = this.getAzure().networks().getById(subnetModel.getVpcUuid());
            Subnet.Update update = network.update().updateSubnet(subnetModel.getName());
            if (null != subnetModel.getSecurityGroupUuid()) {
                update.withExistingNetworkSecurityGroup(subnetModel.getSecurityGroupUuid());
            } else {
                update.withoutNetworkSecurityGroup();
            }
            update.parent().apply();
            return new BocloudResult(true, "update subnet success!");
        } catch (Exception e) {
            logger.error("update subnet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SubnetModel subnet) {
        //所需参数 vpcUuid cidr name
        try {
            //删除子网同时删除地址空间
            Network network = this.getAzure().networks().getById(subnet.getVpcUuid());
            network.update().withoutSubnet(subnet.getName()).withoutAddressSpace(subnet.getCidr()).apply();
            return new BocloudResult(true, "remove subnet success!");
        } catch (Exception e) {
            logger.error("remove subnet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult associate(SubnetModel subnetModel) {
        //所需参数 vpcUuid  name  routerTableUuid
        try {
            this.getAzure()
                    .networks()
                    .getById(subnetModel.getVpcUuid())
                    .update()
                    .updateSubnet(subnetModel.getName())
                    .withExistingRouteTable(subnetModel.getRouterTableUuid())
                    .parent()
                    .apply();
            return new BocloudResult(true, "associate subnet success!");
        } catch (Exception e) {
            logger.error("associate subnet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult release(SubnetModel subnetModel) {
        //所需参数 vpcUuid  name
        try {
            this.getAzure()
                    .networks()
                    .getById(subnetModel.getVpcUuid())
                    .update()
                    .updateSubnet(subnetModel.getName())
                    .withoutRouteTable()
                    .parent()
                    .apply();
            return new BocloudResult(true, "release subnet success!");
        } catch (Exception e) {
            logger.error("release subnet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult remove(String subnetId) {
        return null;
    }

    @Override
    public BocloudResult detail(String subnetId) {
        return null;
    }


}
