package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.azureconvertor.network.VpcConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.Network;
import io.jsonwebtoken.lang.Assert;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureVpcProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/9/17 15:37
 */
@Provider("bocloud.cmp.azure")
public class AzureVpcProvider extends AzureProvider implements VpcProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureVpcProvider.class);

    private BeanConvertor<VpcModel, Network> convertor = new VpcConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public AzureVpcProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult list() {
        try {
            List<VpcModel> regionModels = convertor.convertList(this.getAzure().networks().list()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            return new BocloudResult(true, regionModels, "list network success!");
        } catch (Exception e) {
            logger.error("list network error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String vpcId, String regionId) {
        try {
            Network network = this.getAzure().networks().getById(vpcId);
            return new BocloudResult(true, convertor.convertModel(network), "detail network success!");
        } catch (Exception e) {
            logger.error("detail network error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(VpcModel model) {
        try {
            Map<String, String> nameCidrPairs = new HashMap<>();
            List<SubnetModel> subnetModels = model.getSubnetModels();
            for (SubnetModel subnetModel : subnetModels) {
                nameCidrPairs.put(subnetModel.getName(), subnetModel.getCidr());
            }
            Network network = this.getAzure().networks()
                    .define(model.getName())
                    .withRegion(model.getRegionId())
                    .withExistingResourceGroup(model.getResourceGroupName())
                    .withAddressSpace(model.getCidr())
                    .withSubnets(nameCidrPairs)
                    .create();
            return new BocloudResult(true, convertor.convertModel(network), "create network success!");
        } catch (Exception e) {
            logger.error("create network error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult dnsServer(VpcModel model) {
        try {
            Network network = this.getAzure().networks().getById(model.getVpcId())
                    .update()
                    .withDnsServer(model.getDnsServer())
                    .apply();
            return new BocloudResult(true, convertor.convertModel(network), "create network success!");
        } catch (Exception e) {
            logger.error("dnsServer network error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult addAddressSpace(VpcModel model) {
        try {
            Network network = this.getAzure().networks().getById(model.getVpcId())
                    .update()
                    .withAddressSpace(model.getCidr())
                    .apply();
            return new BocloudResult(true, convertor.convertModel(network), "create network success!");
        } catch (Exception e) {
            logger.error("addAddressSpace network error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult removeAddressSpace(VpcModel model) {
        try {
            Network network = this.getAzure().networks().getById(model.getVpcId())
                    .update()
                    .withoutAddressSpace(model.getCidr())
                    .apply();
            return new BocloudResult(true, convertor.convertModel(network), "create network success!");
        } catch (Exception e) {
            logger.error("removeAddressSpace network error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String vpcId) {
        try {
            this.getAzure().networks().deleteById(vpcId);
            return new BocloudResult(true, "remove network success!");
        } catch (Exception e) {
            logger.error("remove network error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult withDdos(String vpcId, String planId) {
        try {
            Network network = this.getAzure().networks().getById(vpcId);
            Assert.notNull(network, "VPC不存在");
            network = network.update().withExistingDdosProtectionPlan(planId).apply();
            return new BocloudResult(true, convertor.convertModel(network), "更新成功");
        } catch (Exception e) {
            logger.error("Get error message :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult withoutDdos(String vpcId) {
        try {
            Network network = this.getAzure().networks().getById(vpcId);
            Assert.notNull(network, "VPC不存在");
            network = network.update().withoutDdosProtectionPlan().apply();
            return new BocloudResult(true, convertor.convertModel(network), "更新成功");
        } catch (Exception e) {
            logger.error("Get error message :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult update(VpcModel model) {
        return null;
    }
}
