package com.bocloud.cmp.provider;

import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.conn.ssl.TrustStrategy;
import org.apache.http.ssl.SSLContextBuilder;
import org.openstack4j.api.OSClient;
import org.openstack4j.core.transport.Config;
import org.openstack4j.model.identity.v2.Access;
import org.openstack4j.model.identity.v3.Token;
import org.openstack4j.openstack.OSFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.config.AccessToken;

import io.jsonwebtoken.lang.Assert;

/**
 * 云管客户端工厂类
 */
public class EasyStackClientFactory {

	private static Logger logger = LoggerFactory.getLogger(EasyStackClientFactory.class);
	private static Map<String, AccessToken> tokens = new ConcurrentHashMap<>(8, 0.75f);
	private static SSLContextBuilder contextBuilder = null;
	private static Config config = null;

	static {
		contextBuilder = new SSLContextBuilder();
		// 全部信任 不做身份鉴定
		try {
			contextBuilder.loadTrustMaterial(null, (TrustStrategy) (x509Certificates, s) -> true);
			config = Config.newConfig()
					.withReadTimeout(20 * 1000 * 1000).withConnectionTimeout(20 * 1000 * 1000);
		} catch (NoSuchAlgorithmException e) {
			logger.error("NoSuchAlgorithmException", e);
		} catch (KeyStoreException e) {
			logger.error("KeyStoreException", e);
		}
	}

	/**
	 * 私有构造方法
	 */
	private EasyStackClientFactory() {
		super();
		if (null != FactoryContainer.instance) {// 防止利用反射机制实现多个对象的漏洞
			throw new RuntimeException("ClientFactory遭受反射攻击！");
		}
		tokens = new ConcurrentHashMap<>(10, 0.75f);
	}

	/**
	 * 获取配置实例标准方法
	 *
	 * @return
	 */
	public static EasyStackClientFactory getInstance() {
		return FactoryContainer.instance;
	}

	/**
	 * 获取指定UUID的OpenStack的client
	 *
	 * @param key
	 * @return
	 */
	public OSClient<?> getClient(String key, String projectName) {
		projectName = StringUtils.isEmpty(projectName) ? "" : projectName;
		String tokenKey = key + projectName;
		Assert.hasLength(key, "");
		// Assert.notNull(ButlerRepository.getInstance().get(key), "");
		// 判断当前UUID的OpenStack认证信息是否存在于缓存中
		if (tokens.keySet().contains(tokenKey)) {// 认证信息存在于缓存
			AccessToken accessToken = tokens.get(tokenKey);
			Assert.isTrue(null != accessToken.getAccess() || null != accessToken.getToken(), "");
			Date current = new Date();
			if (null == accessToken.getAccess() && null != accessToken.getToken()) {
				Token token = accessToken.getToken();
				// 判断当前时间是否超过了会话超时时间，如果超时之后进行重新认证
				if (current.after(token.getExpires())) {
					logger.info("Cloud Provider [{}] token is Expired! rebuild one!",
							ButlerRepository.getInstance().get(key).getName());
					return null;
				}
				return OSFactory.clientFromToken(token, config);
			} else {
				Access access = accessToken.getAccess();
				// 判断当前时间是否超过了会话超时时间，如果超时之后进行重新认证
				if (current.after(access.getToken().getExpires())) {
					logger.info("Cloud Provider [{}] token is Expired! rebuild one!",
							ButlerRepository.getInstance().get(key).getName());
					return null;
				}
				return OSFactory.clientFromAccess(access, config);
			}
		}
		return null;
	}

	/**
	 * 刷新认证
	 *
	 * @param butler
	 */
	public void updateToken(Butler butler) {
		Assert.notNull(butler, "butler为空");
		String projectName = butler.getProjectName();
		if (StringUtils.isEmpty(projectName)) {
			projectName = butler.getTenant();
		}
		projectName = StringUtils.isEmpty(projectName) ? "" : projectName;
		if (tokens.containsKey(butler.getUuid() + projectName)) {
			tokens.remove(butler.getUuid() + projectName);
		}
		new EasyStackProvider(butler);
	}

	/**
	 * 刷新认证
	 *
	 * @param key
	 * @param projectName
	 * @param provider
	 */
	public void refreshToken(String key, String projectName, EasyStackProvider provider) {
		projectName = StringUtils.isEmpty(projectName) ? "" : projectName;
		if (provider.isAccessable()) {
			switch (provider.getVersion()) {
			case V2:
				tokens.put(key + projectName, new AccessToken(provider.getClient2().getAccess(), null));
				break;
			case V3:
				tokens.put(key + projectName, new AccessToken(null, provider.getClient3().getToken()));
				break;
			default:
				break;
			}
		}
	}

	public void removeToken(String key, String projectName) {
		projectName = StringUtils.isEmpty(projectName) ? "" : projectName;
		tokens.remove(key + projectName);
	}

	private static class FactoryContainer {
		private static EasyStackClientFactory instance = new EasyStackClientFactory();
	}

}
