package com.bocloud.cmp.provider;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.listener.EasyStackButlerListener;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.CommonResultBuilder;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPatch;
import org.apache.http.conn.ssl.TrustStrategy;
import org.apache.http.entity.StringEntity;
import org.apache.http.ssl.SSLContextBuilder;
import org.openstack4j.api.OSClient;
import org.openstack4j.api.types.Facing;
import org.openstack4j.core.transport.Config;
import org.openstack4j.model.common.Identifier;
import org.openstack4j.model.identity.v2.Access.Service;
import org.openstack4j.model.identity.v3.Endpoint;
import org.openstack4j.openstack.OSFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URI;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * easystack驱动客户端
 */
public class EasyStackProvider extends GenericProvider implements SuperProvider {
    private static final Logger logger = LoggerFactory.getLogger(EasyStackProvider.class);
    private static final String CODE = "bocloud.cmp.openstack";
    private static SSLContextBuilder contextBuilder = null;
    private static Config config = null;
    private String endpoint;// 平台认证路径
    private String username;// 管理员账号
    private String password;// 管理员密码
    private String tenant;// 租户名称
    private String domain;// 域名称
    private String project;// 工程名
    private Version version;// KeyStone的认证API版本
    private OSClient.OSClientV2 client2;
    private OSClient.OSClientV3 client3;
    private OSClient<?> client;
    private boolean accessable;
    private String error; // 错误信息
    private HttpClient httpClient = new HttpClient(20 * 1000 * 1000, PostDataFormat.RAW);
    private Map<String, Object> headers = null;

    static {
        contextBuilder = new SSLContextBuilder();
        // 全部信任 不做身份鉴定
        try {
            contextBuilder.loadTrustMaterial(null, (TrustStrategy) (x509Certificates, s) -> true);
        } catch (NoSuchAlgorithmException e) {
            logger.error("NoSuchAlgorithmException", e);
        } catch (KeyStoreException e) {
            logger.error("KeyStoreException", e);
        }
    }

    public EasyStackProvider(Butler butler) {
        super(butler);
        this.endpoint = butler.getUrl();
        this.username = butler.getUsername();
        this.password = butler.getPassword();
        this.tenant = butler.getTenant();
        this.domain = butler.getDomainName();
        this.project = butler.getProjectName();
        this.version = Version.valueOf(butler.getVersion());
        String projName = butler.getProjectName();
        if (StringUtils.isEmpty(projName)) {
            projName = butler.getTenant();
        }
        this.client = EasyStackClientFactory.getInstance().getClient(butler.getUuid(), projName);
        if (null != this.client) {
            this.accessable = true;
            if (this.client instanceof OSClient.OSClientV2) {
                client2 = (OSClient.OSClientV2) this.client;
            } else if (this.client instanceof OSClient.OSClientV3) {
                client3 = (OSClient.OSClientV3) this.client;
            }
        } else {
            init();
            if (accessable) {
                ButlerRepository repository = ButlerRepository.getInstance();
                if (!repository.contains(CODE)) {
                    repository.addListener(new EasyStackButlerListener(CODE));
                }
                EasyStackClientFactory.getInstance().refreshToken(butler.getUuid(), projName, this);
            }
        }
    }

    /**
     * 初始化客户端连接
     */
    private void init() {
        Result result = paramCheck();
        if (result.isFailed()) {// 基本的参数校验不通过，即用户名，密码或者认证路径为空
            this.accessable = false;
            logger.error("Driver init error : {}", result.getMessage());
            this.error = result.getMessage();
            return;
        }
        decideVersion();// 确定认证版本
        if (logger.isDebugEnabled()) {
            logger.info("Keystone Version:{}", this.version);
        }
        try {
//            config = Config.newConfig().withSSLVerificationDisabled().withSSLContext(contextBuilder.build())
//                    .withReadTimeout(50 * 1000 * 1000).withConnectionTimeout(50 * 1000 * 1000);
            config = Config.newConfig()
                    .withReadTimeout(50 * 1000 * 1000).withConnectionTimeout(50 * 1000 * 1000);
            switch (this.version) {
                case V2:
                    buildV2();
                    break;
                case V3:
                    buildV3();
                    break;
                default:
                    logger.warn("Unsupported Version! Use V2 version to build!");
                    buildV2();
                    break;
            }
        } catch (Exception e) {
            logger.error("OSDriver Init Error:", e);
            this.accessable = false;
            this.error = e.getMessage();
        }
    }

    /**
     * 基本参数信息认证，即确保认证路径，认证账号和认证密码为非空
     *
     * @return
     */
    private Result paramCheck() {
        if (StringUtils.isEmpty(this.endpoint)) {
            return new Result(false, "Endpoint is Empty!");
        }
        if (StringUtils.isEmpty(this.username)) {
            return new Result(false, "Username is Empty!");
        }
        if (StringUtils.isEmpty(this.password)) {
            return new Result(false, "Password is Empty!");
        }
        if (null == this.version) {
            this.version = Version.V2;
        }
        switch (this.version) {
            case V3:
                if (StringUtils.isEmpty(this.project)) {
                    return new Result(false, "Project is Empty!");
                }
                break;
            case V2:
                if (StringUtils.isEmpty(this.tenant)) {
                    return new Result(false, "Tenant is Empty!");
                }
                break;
            default:
                logger.warn("Uknown Version use V2 to build");
                if (StringUtils.isEmpty(this.tenant)) {
                    return new Result(false, "Tenant is Empty!");
                }
                break;
        }

        return new Result(true, "success");
    }

    private void buildEndpoint(String version) {
        if (this.endpoint.endsWith("/")) {
            this.endpoint = this.endpoint + version + "/";
        } else {
            this.endpoint = this.endpoint + "/" + version + "/";
        }
    }

    /**
     * 确定版本信息，根据传入的版本参数或者认证路径endpoint来确定
     */
    private void decideVersion() {
        this.endpoint = this.endpoint.trim();
        if (null == this.version) {
            this.version = Version.V2;
        }
        switch (this.version) {
            case V3:
                buildEndpoint("v3");
                break;
            case V2:
                buildEndpoint("v2.0");
                break;
            default:
                logger.warn("Uknown Version use V2 to build");
                buildEndpoint("v2.0");
                break;
        }
    }

    /**
     * 构建V2版本的客户端
     */
    private void buildV2() {
        this.client2 = OSFactory.builderV2().endpoint(this.endpoint).credentials(this.username, this.password)
                .tenantName(this.tenant).withConfig(config).authenticate();
        this.client = client2;
        this.accessable = true;
    }

    /**
     * 构建V3版本的客户端
     */
    private void buildV3() {
        if (StringUtils.isEmpty(this.domain)) {
            this.accessable = false;
            this.error = "Driver init error : Domain is empty!";
            logger.error(error);
            return;
        }
        Identifier domainIdentifier = Identifier.byName(this.domain);
        if (StringUtils.isEmpty(project)) {
            this.client3 = OSFactory.builderV3().endpoint(this.endpoint)
                    .credentials(this.username, this.password, domainIdentifier).withConfig(config).authenticate();
        } else {
            this.client3 = OSFactory.builderV3().endpoint(this.endpoint)
                    .credentials(this.username, this.password, domainIdentifier)
                    .scopeToProject(Identifier.byName(this.project), Identifier.byName(this.domain)).withConfig(config)
                    .authenticate();
        }
        this.client = this.client3;
        this.accessable = true;
    }

    public void setAccessable(boolean accessable) {
        this.accessable = accessable;
    }

    public String getEndpoint() {
        return endpoint;
    }

    public void setEndpoint(String endpoint) {
        this.endpoint = endpoint;
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getTenant() {
        return tenant;
    }

    public void setTenant(String tenant) {
        this.tenant = tenant;
    }

    public String getDomain() {
        return domain;
    }

    public void setDomain(String domain) {
        this.domain = domain;
    }

    public String getProject() {
        return project;
    }

    public void setProject(String project) {
        this.project = project;
    }

    public Version getVersion() {
        return version;
    }

    public void setVersion(Version version) {
        this.version = version;
    }

    public OSClient.OSClientV2 getClient2() {
        return client2;
    }

    public OSClient.OSClientV3 getClient3() {
        return client3;
    }

    public boolean isAccessable() {
        return accessable;
    }

    public OSClient<?> getClient() {
        return client;
    }

    public String getError() {
        return error;
    }

    /**
     * 获取api调用网关
     *
     * @param client
     * @param serviceName
     * @param serviceType
     * @return
     */
    private String findApi(OSClient<?> client, String serviceName, String serviceType) {
        String api = null;
        if (client instanceof OSClient.OSClientV2) {
            OSClient.OSClientV2 client2 = (OSClient.OSClientV2) client;
            this.headers = buildHeader(client2.getAccess().getToken().getId(), null);
            List<? extends Service> services = client2.getAccess().getServiceCatalog();
            for (Service service : services) {
                String name = service.getName();
                String type = service.getType();
                if (name.equals(serviceName) && type.equals(serviceType)) {
                    URI url = service.getEndpoints().get(0).getPublicURL();
                    api = url.getPath() + ":" + url.getPort();
                    break;
                }
            }
        } else if (client instanceof OSClient.OSClientV3) {
            OSClient.OSClientV3 client3 = (OSClient.OSClientV3) client;
            this.headers = buildHeader(client3.getToken().getId(), null);
            List<? extends org.openstack4j.model.identity.v3.Service> services = client3.getToken().getCatalog();
            for (org.openstack4j.model.identity.v3.Service service : services) {
                String name = service.getName();
                String type = service.getType();
                if (name.equals(serviceName) && type.equals(serviceType)) {
                    List<? extends Endpoint> publicEndpoints = service.getEndpoints().stream().filter(endpointObj -> Facing.PUBLIC.equals(endpointObj.getIface())).collect(Collectors.toList());
                    Endpoint endpoint;
                    if (publicEndpoints.isEmpty()) {
                        endpoint = service.getEndpoints().get(0);
                    } else {
                        endpoint = publicEndpoints.get(0);
                    }
                    api = endpoint.getUrl() + "";
                    break;
                }
            }
        }
        return api;
    }

    /**
     * 请求头信息
     *
     * @param tokenId
     * @param projectId
     * @return
     */
    private Map<String, Object> buildHeader(String tokenId, String projectId) {
        Map<String, Object> header = new HashMap<String, Object>();
        header.put("accept", "application/json");
        header.put("content-type", "application/json");
        header.put("X-Auth-Token", tokenId);
        if (!StringUtils.isEmpty(projectId)) {
            header.put("X-Project-id", projectId);
        }
        return header;
    }

    public Result doGet(String url, Map<String, Object> params, String serviceName, String serviceType) {
        try {
            httpClient.setDataFormat(null);
            String apiUrl = findApi(this.getClient(), serviceName, serviceType);
            if (StringUtils.isEmpty(apiUrl)) {
                return Result.FAILED("Neuturn service is unavailable");
            }
            return this.httpClient.get(this.headers, params, apiUrl + url);
        } finally {
            this.httpClient.close();
        }
    }

    public Result doPost(String url, Map<String, Object> params, String serviceName, String serviceType) {
        try {
            httpClient.setDataFormat(PostDataFormat.RAW);
            String apiUrl = findApi(this.getClient(), serviceName, serviceType);
            if (StringUtils.isEmpty(apiUrl)) {
                return Result.FAILED("Neuturn service is unavailable");
            }
            return this.httpClient.post(this.headers, params, apiUrl + url);
        } finally {
            this.httpClient.close();
        }
    }

    public Result doPut(String url, Map<String, Object> params, String serviceName, String serviceType) {
        try {
            httpClient.setDataFormat(PostDataFormat.RAW);
            String apiUrl = findApi(this.getClient(), serviceName, serviceType);
            if (StringUtils.isEmpty(apiUrl)) {
                return Result.FAILED("Neuturn service is unavailable");
            }
            return this.httpClient.put(this.headers, params, apiUrl + url);
        } finally {
            this.httpClient.close();
        }
    }

    public Result doPatch(String url, Map<String, Object> params, String serviceName, String serviceType) {
        try {
            httpClient.setDataFormat(PostDataFormat.RAW);
            String apiUrl = findApi(this.getClient(), serviceName, serviceType);
            if (StringUtils.isEmpty(apiUrl)) {
                return Result.FAILED("Neuturn service is unavailable");
            }
            return this.httpClient.patch(this.headers, params, apiUrl + url);
        } finally {
            this.httpClient.close();
        }
    }

    public Result doPatch(String url, String params, String serviceName, String serviceType) {
        try {
            httpClient.setDataFormat(PostDataFormat.RAW);
            String apiUrl = findApi(this.getClient(), serviceName, serviceType);
            if (StringUtils.isEmpty(apiUrl)) {
                return Result.FAILED("service is unavailable");
            }
            HttpPatch patch = new HttpPatch(apiUrl + url);
            if (client instanceof OSClient.OSClientV2) {
                OSClient.OSClientV2 client2 = (OSClient.OSClientV2) client;
                patch.addHeader("X-Auth-Token", client2.getAccess().getToken().getId());
            } else if (client instanceof OSClient.OSClientV3) {
                OSClient.OSClientV3 client3 = (OSClient.OSClientV3) client;
                patch.addHeader("X-Auth-Token", client3.getToken().getId());
            }
            patch.addHeader("Accept", "application/json");
            patch.addHeader("Content-Type", "application/openstack-images-v2.1-json-patch");
            StringEntity httpEntity = new StringEntity(JSONObject.toJSON(params).toString(), "UTF-8");
            patch.setEntity(httpEntity);
            CloseableHttpResponse httpResponse = httpClient.getHttpClient().execute(patch);
            Result result = new CommonResultBuilder().build(httpResponse.getStatusLine().getStatusCode());
            return result;
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return Result.FAILED("请求失败");
        } finally {
            this.httpClient.close();
        }
    }

    public Result doDelete(String url, Map<String, Object> params, String serviceName, String serviceType) {
        try {
            httpClient.setDataFormat(null);
            String apiUrl = findApi(this.getClient(), serviceName, serviceType);
            if (StringUtils.isEmpty(apiUrl)) {
                return Result.FAILED("Neuturn service is unavailable");
            }
            return this.httpClient.delete(this.headers, params, apiUrl + url);
        } finally {
            this.httpClient.close();
        }
    }

    public Result get(String url, Map<String, Object> headers, Map<String, Object> params) {
        try {
            httpClient.setDataFormat(null);
            logger.info("get url:{}", this.endpoint + url);
            return this.httpClient.get(headers, params, this.endpoint + url);
        } finally {
            this.httpClient.close();
        }
    }

    public Result blockStoragePost(String url, Map<String, Object> params) {
        try {
            httpClient.setDataFormat(PostDataFormat.RAW);
            String apiUrl = findVolumeApi(this.getClient());
            if (StringUtils.isEmpty(apiUrl)) {
                return Result.FAILED("Block storage service is unavailable");
            }
            apiUrl = apiUrl.replace("/v1", "/v3");
            logger.info("block storage访问地址：{}", (apiUrl + url));
            return this.httpClient.post(this.headers, params, apiUrl + url);
        } finally {
            this.httpClient.close();
        }
    }

    public Result blockStoragePut(String url, Map<String, Object> params) {
        try {
            httpClient.setDataFormat(PostDataFormat.RAW);
            String apiUrl = findVolumeApi(this.getClient());
            if (StringUtils.isEmpty(apiUrl)) {
                return Result.FAILED("Block storage service is unavailable");
            }
            apiUrl = apiUrl.replace("/v1", "/v3");
            logger.info("block storage访问地址：{}", (apiUrl + url));
            return this.httpClient.put(this.headers, params, apiUrl + url);
        } finally {
            this.httpClient.close();
        }
    }

    /**
     * 获取硬盘，快照API地址
     *
     * @param client
     * @return
     */
    private String findVolumeApi(OSClient<?> client) {
        String api = null;
        if (client instanceof OSClient.OSClientV2) {
            OSClient.OSClientV2 client2 = (OSClient.OSClientV2) client;
            this.headers = buildHeader(client2.getAccess().getToken().getId(), null);
            List<? extends Service> services = client2.getAccess().getServiceCatalog();
            for (Service service : services) {
                String name = service.getName();
                String type = service.getType();
                if (name.contains("cinder") && type.contains("volume")) {
                    List<? extends org.openstack4j.model.identity.v2.Endpoint> endpoints = service.getEndpoints();
                    if (ListTool.isEmpty(endpoints)) {
                        continue;
                    }
                    URI url = endpoints.get(0).getPublicURL();
                    api = url.getPath() + ":" + url.getPort();
                    if (!api.contains("/v3")) {
                        continue;
                    }
                    break;
                }
            }
        } else if (client instanceof OSClient.OSClientV3) {
            OSClient.OSClientV3 client3 = (OSClient.OSClientV3) client;
            this.headers = buildHeader(client3.getToken().getId(), null);
            List<? extends org.openstack4j.model.identity.v3.Service> services = client3.getToken().getCatalog();
            for (org.openstack4j.model.identity.v3.Service service : services) {
                String name = service.getName();
                String type = service.getType();
                if (name.contains("cinder") && type.contains("volume")) {
                    List<? extends Endpoint> endpoints = service.getEndpoints();
                    if (ListTool.isEmpty(endpoints)) {
                        continue;
                    }
                    Endpoint endpoint = endpoints.get(0);
                    api = endpoint.getUrl() + "";
                    if (!api.contains("/v3")) {
                        continue;
                    }
                    break;
                }
            }
        }
        return api;
    }

    @Override
    public boolean accessable() {
        return this.accessable;
    }

    public enum Version {
        // V2版本
        V2,
        // V3版本
        V3;
    }

    public static void sleepSomeTime(int sleepTime) {
        try {
            Thread.sleep(sleepTime);
        } catch (InterruptedException e) {
            logger.error("睡眠失败: " + e.getMessage());
        }
    }
}
