package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.OSTenantModel;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import org.apache.http.ssl.SSLContextBuilder;
import org.openstack4j.api.Builders;
import org.openstack4j.api.OSClient;
import org.openstack4j.core.transport.Config;
import org.openstack4j.model.common.Identifier;
import org.openstack4j.model.identity.v3.Domain;
import org.openstack4j.model.identity.v3.Project;
import org.openstack4j.model.identity.v3.Role;
import org.openstack4j.model.identity.v3.User;
import org.openstack4j.openstack.OSFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * easystack认证操作类
 */
@Provider("bocloud.cmp.easystack")
public class EasyStackAuthProvider extends EasyStackProvider implements AuthProvider {

    private static Logger logger = LoggerFactory.getLogger(EasyStackAuthProvider.class);
    private Butler butler = null;

    public EasyStackAuthProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult auth() {
        if (this.isAccessable()) {
            return new BocloudResult(true, "" + this.isAccessable());
        }
        return new BocloudResult(this.isAccessable(), "认证信息失败");
    }

    @Override
    public BocloudResult createProject(OSTenantModel model) {
        try {
            String projectName = model.getName();
            OSTenantModel osTenant;
            Identifier domainIdentifier = Identifier.byName(butler.getDomainName());
            Config config = Config.newConfig().withSSLVerificationDisabled().withSSLContext(new SSLContextBuilder().build())
                    .withReadTimeout(50 * 1000 * 1000).withConnectionTimeout(50 * 1000 * 1000);
            OSClient.OSClientV3 client3 = OSFactory.builderV3().endpoint(butler.getUrl() + "/v3")
                    .credentials(butler.getUsername(), butler.getPassword(), domainIdentifier)
                    .scopeToDomain(Identifier.byName(butler.getDomainName())).withConfig(config)
                    .authenticate();
            // 查询域
            List<? extends Domain> domains = client3.identity().domains().getByName(butler.getDomainName());
            Domain domain = domains.get(0);
            // 查询用户
            User user = client3.identity().users().getByName(this.butler.getUsername(), domain.getId());
            // 查询user的角色列表
            List<? extends Role> roles = client3.identity().roles().list();
            List<? extends Project> projects = client3.identity().projects().getByName(projectName);
            if (!ListTool.isEmpty(projects)) {
                for (Role role : roles) {
                    for (Project project : projects) {
                        client3.identity().roles().grantProjectUserRole(project.getId(), user.getId(),
                                role.getId());
                    }
                }
                Project project = projects.get(0);
                Domain projectDomain = project.getDomain();
                String domainName = null;
                if (null != projectDomain) {
                    domainName = projectDomain.getName();
                } else {
                    domainName = project.getDomainId();
                }
                osTenant = new OSTenantModel(project.getName(), project.getDescription(), project.getId(),
                        project.isEnabled(), project.getDomainId(), domainName);
                return new BocloudResult(true, osTenant, "项目查询成功");
            }
            Project project = client3.identity().projects().create(
                    client3.identity().domains().getByName(this.butler.getDomainName()).get(0).getId(), projectName,
                    model.getRemark(), true);
            if (null == project) {
                return new BocloudResult(false, "项目创建失败");
            }
            Domain projectDomain = project.getDomain();
            String domainName = null;
            if (null != projectDomain) {
                domainName = projectDomain.getName();
            } else {
                domainName = project.getDomainId();
            }
            osTenant = new OSTenantModel(project.getName(), project.getDescription(), project.getId(),
                    project.isEnabled(), project.getDomainId(), domainName);
            for (Role role : roles) { // domain_admin admin
                if ("admin".equalsIgnoreCase(role.getName()) || "domain_admin".equalsIgnoreCase(role.getName())) {
                    client3.identity().roles().grantProjectUserRole(project.getId(), user.getId(), role.getId());
                }
            }
            return new BocloudResult(true, osTenant, "项目创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "项目创建失败");
        }
    }

    @Override
    public BocloudResult removeProject(String projectName) {
        try {
            Identifier domainIdentifier = Identifier.byName(butler.getDomainName());
            Config config = Config.newConfig().withSSLVerificationDisabled().withSSLContext(new SSLContextBuilder().build())
                    .withReadTimeout(50 * 1000 * 1000).withConnectionTimeout(50 * 1000 * 1000);
            OSClient.OSClientV3 client3 = OSFactory.builderV3().endpoint(butler.getUrl() + "/v3")
                    .credentials(butler.getUsername(), butler.getPassword(), domainIdentifier)
                    .scopeToDomain(Identifier.byName(butler.getDomainName())).withConfig(config)
                    .authenticate();
            List<? extends Project> projects = client3.identity().projects().getByName(projectName);
            if (ListTool.isEmpty(projects)) {
                return new BocloudResult(true, "项目删除成功");
            }
            for (Project project : projects) {
                client3.identity().projects().delete(project.getId());
            }
            return new BocloudResult(true, "项目移除成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "项目移除失败");
        }
    }

    @Override
    public BocloudResult modifyProject(OSTenantModel model) {
        try {
            String projectName = model.getName();
            OSTenantModel osTenant = null;
            Identifier domainIdentifier = Identifier.byName(butler.getDomainName());
            Config config = Config.newConfig().withSSLVerificationDisabled().withSSLContext(new SSLContextBuilder().build())
                    .withReadTimeout(50 * 1000 * 1000).withConnectionTimeout(50 * 1000 * 1000);
            OSClient.OSClientV3 client3 = OSFactory.builderV3().endpoint(butler.getUrl() + "/v3")
                    .credentials(butler.getUsername(), butler.getPassword(), domainIdentifier)
                    .scopeToDomain(Identifier.byName(butler.getDomainName())).withConfig(config)
                    .authenticate();


            Project build = Builders.identityV3().project().name(projectName).description(model.getRemark())
                    .id(model.getTenantUuid()).build();
            Project update = client3.identity().projects().update(build);
            osTenant = new OSTenantModel(update.getName(), update.getDescription(), null, null, null, null);
            return new BocloudResult(true, osTenant, "项目编辑成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "项目编辑失败");
        }
    }

}
