package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.V2ImageConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.openstack4j.api.Builders;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.common.Payload;
import org.openstack4j.model.common.Payloads;
import org.openstack4j.model.image.v2.ContainerFormat;
import org.openstack4j.model.image.v2.DiskFormat;
import org.openstack4j.model.image.v2.Image;
import org.openstack4j.model.image.v2.builder.ImageBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * easystack镜像操作类
 */
@Provider("bocloud.cmp.easystack")
public class EasyStackImageProvider extends EasyStackProvider implements ImageProvider {

    private static Logger logger = LoggerFactory.getLogger(EasyStackImageProvider.class);

    public EasyStackImageProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        List<ImageModel> imageModels = new ArrayList<>();
        List<? extends Image> v2Images = this.getClient().imagesV2().list();
        if (!ListTool.isEmpty(v2Images)) {
            BeanConvertor<ImageModel, Image> convertImage = new V2ImageConvertor();
            imageModels = convertImage.convertList(v2Images);
        }
        return new BocloudResult(true, imageModels, "success");
    }

    @Override
    public BocloudResult detail(String imageId) {
        BocloudResult result = null;
        Image v2Image = this.getClient().imagesV2().get(imageId);
        if (null != v2Image) {
            BeanConvertor<ImageModel, Image> convertImage = new V2ImageConvertor();
            result = new BocloudResult(true, convertImage.convertModel(v2Image), "success");
        } else {
            result = new BocloudResult(false, "fail");
        }
        return result;
    }

    @Override
    public BocloudResult remove(String imageId) {
        try {
            ActionResponse response = this.getClient().imagesV2().delete(imageId);
            if (response.isSuccess()) {
                return new BocloudResult(true, "删除成功!");
            } else {
                logger.error("remove image [{}] error: {}", imageId, response.getFault());
                return new BocloudResult(false, response.getFault());
            }
        } catch (Exception e) {
            logger.error("删除失败", e);
            return new BocloudResult(false, "删除失败!");
        }
    }

    @Override
    public BocloudResult modify(ImageModel image) {
        try {
            List<JSONObject> params = new ArrayList<>();
            JSONObject nameObj = new JSONObject();
            nameObj.put("op", "replace");
            nameObj.put("path", "/name");
            nameObj.put("value", image.getName());
            params.add(nameObj);
            Result updateResult = this.doPatch("/v2/images/" + image.getImageUuid(), JSONArray.toJSONString(params),
                    "glance", "image");
            if (updateResult.isFailed()) {
                return new BocloudResult(false, updateResult.getMessage());
            }
            Image newImage = this.getClient().imagesV2().get(image.getImageUuid());
            if (null == newImage) {
                logger.error("Update image fail");
                return new BocloudResult(false, "修改失败！");
            }
            BeanConvertor<ImageModel, Image> v2ImageConvertor = new V2ImageConvertor();
            ImageModel model = v2ImageConvertor.convertModel(newImage);
            model.setOsCategory(image.getOsCategory());
            model.setOsName(image.getOsName());
            model.setOsVersion(image.getOsVersion());
            model.setDefaultUser(image.getDefaultUser());
            model.setName(image.getName());
            return new BocloudResult(true, model, "修改成功！");
        } catch (Exception e) {
            logger.error("修改失败", e);
            return new BocloudResult(false, "修改失败!");
        }
    }

    @Override
    public BocloudResult create(ImageModel image) {
        Payload<File> payload = null;
        try {
            String filePath = image.getFilePath();
            File file = new File(filePath);
            logger.info("镜像上传路径：" + filePath);
            if (!file.exists()) {
                logger.warn("Image file does not exist");
                return new BocloudResult(false, "镜像文件不存在");
            }
            payload = Payloads.create(file);
            BocloudResult result = null;
            ImageBuilder v2Builder = Builders.imageV2().name(image.getName()).containerFormat(ContainerFormat.BARE)
                    .visibility(Image.ImageVisibility.PRIVATE).diskFormat(DiskFormat.valueOf(image.getOsType()))
                    .minDisk(image.getMinDisk());
            if (image.getQgaInstalled()) {
                v2Builder.additionalProperty("hw_qemu_guest_agent", "yes");
            }
            if (!StringUtils.isEmpty(image.getOsCategory())) {
                v2Builder.additionalProperty("oscategory", image.getOsCategory());
            }
            if (!StringUtils.isEmpty(image.getOsName())) {
                v2Builder.additionalProperty("osname", image.getOsName());
            }
            if (!StringUtils.isEmpty(image.getOsVersion())) {
                v2Builder.additionalProperty("osversion", image.getOsVersion());
            }
            if (!StringUtils.isEmpty(image.getDefaultUser())) {
                v2Builder.additionalProperty("default_user", image.getDefaultUser());
            }
            Image imageV2 = this.getClient().imagesV2().create(v2Builder.build());
            ActionResponse response = this.getClient().imagesV2().upload(imageV2.getId(), payload, imageV2);
            imageV2 = this.getClient().imagesV2().get(imageV2.getId());
            if (null != imageV2) {
                BeanConvertor<ImageModel, Image> imageConvertor = new V2ImageConvertor();
                result = new BocloudResult(response.isSuccess(), imageConvertor.convertModel(imageV2), "镜像创建成功");
            } else {
                image.setStatus("TIMEOUT");
                result = new BocloudResult(response.isSuccess(), image, "镜像创建失败");
            }
            // 删除服务器镜像文件
            boolean flag = file.delete();
            logger.info("删除镜像临时文件状态：{}", flag);
            return result;
        } catch (Exception e) {
            logger.error("Get error message:", e);
            image.setStatus("TIMEOUT");
            return new BocloudResult(false, image, "镜像上传失败：" + e.getMessage());
        } finally {
            if (null != payload) {
                try {
                    payload.close();
                } catch (Exception e) {
                    logger.error("close payload error:", e);
                }
            }
        }
    }

    @Override
    public BocloudResult shareImage(ImageModel imageModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult syncImages(ImageModel imageModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult download(String imageId, String path) {
        try {
            Image image = this.getClient().imagesV2().get(imageId);
            if (null == image) {
                return new BocloudResult(false, "镜像不存在");
            }
            File file = new File(path);
            ActionResponse response = this.getClient().imagesV2().download(imageId, file);
            if (!response.isSuccess()) {
                return new BocloudResult(false, response.getFault());
            }
            logger.info("image download in path {}", path);
            return new BocloudResult(true, file.getPath(), "镜像下载成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "镜像下载失败：" + e.getMessage());
        }
    }
}
