package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.cmp.provider.utils.ResourcePercolator;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.storage.block.Volume;
import org.openstack4j.model.storage.block.VolumeSnapshot;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * easystack快照操作类
 */
@Provider("bocloud.cmp.easystack")
public class EasyStackSnapshotProvider extends EasyStackProvider implements SnapshotProvider {

    private static Logger logger = LoggerFactory.getLogger(EasyStackSnapshotProvider.class);

    public EasyStackSnapshotProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        List<SnapshotModel> models = new ArrayList<>();
        List<? extends VolumeSnapshot> snapshots = new ArrayList<>();
        snapshots = this.getClient().blockStorage().snapshots().list();
        if (null != snapshots) {
            for (VolumeSnapshot snapshot : snapshots) {
                models.add(convert(snapshot));
            }
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        VolumeSnapshot snapshot = this.getClient().blockStorage().snapshots().get(model.getSnapshotUuid());
        if (null == snapshot) {
            logger.error("snapshot [{}] doesnot exsit!", model.getSnapshotUuid());
            return new BocloudResult(true, "快照不存在！");
        }
        ActionResponse response = this.getClient().blockStorage().snapshots().delete(model.getSnapshotUuid());
        if (response.isSuccess()) {
            return new BocloudResult(true, "删除成功!");
        } else {
            logger.error("remove snapshot [{}] error: {}", model.getSnapshotUuid(), response.getFault());
            return new BocloudResult(false, response.getFault());
        }
    }

    private SnapshotModel convert(VolumeSnapshot snapshot) {
        if (null == snapshot) {
            return null;
        }
        SnapshotModel model = new SnapshotModel();
        model.setCreateTime(snapshot.getCreated());
        model.setName(snapshot.getName());
        model.setSize(snapshot.getSize());
        model.setSnapshotUuid(snapshot.getId());
        model.setVolumeUuid(snapshot.getVolumeId());
        model.setStatus(snapshot.getStatus().name());
        return model;
    }

    @Override
    public BocloudResult modify(SnapshotModel model) {
       /* ActionResponse response = this.getClient().blockStorage().snapshots().update(model.getSnapshotUuid(),
                model.getName(), model.getRemark());
        if (response.isSuccess()) {
            return new BocloudResult(true, "success");
        } else {
            return new BocloudResult(false, response.getFault());
        }*/
        try {
            JSONObject params = new JSONObject();
            params.put("name", model.getName());
            params.put("description", model.getRemark());
            JSONObject snapshotParam = new JSONObject();
            snapshotParam.put("snapshot", params);
            Result result = blockStoragePut("/snapshots/" + model.getSnapshotUuid(), snapshotParam);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, this.detail(model).getData(), "快照修改成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
        }
        return new BocloudResult(false, "快照修改失败");
    }

    @Override
    public BocloudResult detail(SnapshotModel model) {
        VolumeSnapshot snapshot = this.getClient().blockStorage().snapshots().get(model.getSnapshotUuid());
        if (null == snapshot) {
            logger.error("snapshot [{}] doesnot exsit!", model.getSnapshotUuid());
            return new BocloudResult(false, "快照不存在！");
        }
        return new BocloudResult(true, convert(snapshot), "success");
    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        return null;
    }

    @Override
    public BocloudResult removeByVolumeId(String volumeId) {
        List<? extends VolumeSnapshot> snapshots = this.getClient().blockStorage().snapshots().list();
        if (snapshots.isEmpty()) {
            return new BocloudResult(true, "remove success");
        }
        for (VolumeSnapshot snapshot : snapshots) {
            // 删除指定磁盘下的所有快照
            if (volumeId.equals(snapshot.getVolumeId())) {
                this.getClient().blockStorage().snapshots().delete(snapshot.getId());
            }
        }
        return new BocloudResult(true, "快照删除成功");
    }

    @Override
    public BocloudResult getSnapshot(String snapshotId) {
        VolumeSnapshot snapshot = this.getClient().blockStorage().snapshots().get(snapshotId);
        if (null == snapshot) {
            logger.error("snapshot [{}] doesnot exsit!", snapshotId);
            return new BocloudResult(false, "快照不存在！");
        }
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();
        while (flag) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            if (null == snapshot) {
                flag = false;
                break;
            }
            if (ResourcePercolator.snapshotFilter(snapshot.getStatus())) {
                flag = false;
                break;
            }
            Long endTime = System.currentTimeMillis();
            // 等待5分钟
            if (endTime - startTime > 5 * 60 * 1000) {
                flag = false;
                snapshot = this.getClient().blockStorage().snapshots().get(snapshotId);
                break;
            }
            snapshot = this.getClient().blockStorage().snapshots().get(snapshotId);
        }
        if (null == snapshot) {
            return new BocloudResult(false, "快照不存在！");
        }
        return new BocloudResult(true, convert(snapshot), "success");
    }

    @Override
    public BocloudResult getAvaliableSnap(String snapshotId) {
        VolumeSnapshot snapshot = this.getClient().blockStorage().snapshots().get(snapshotId);
        if (null == snapshot) {
            logger.error("snapshot [{}] doesnot exsit!", snapshotId);
            return new BocloudResult(false, "快照不存在！");
        }
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();
        while (flag) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            if (null == snapshot) {
                flag = false;
                break;
            }
            if (Volume.Status.AVAILABLE == snapshot.getStatus()) {
                flag = false;
                snapshot = this.getClient().blockStorage().snapshots().get(snapshotId);
                break;
            }
            Long endTime = System.currentTimeMillis();
            // 等待5分钟
            if (endTime - startTime > 5 * 60 * 1000) {
                flag = false;
                snapshot = this.getClient().blockStorage().snapshots().get(snapshotId);
                break;
            }
            snapshot = this.getClient().blockStorage().snapshots().get(snapshotId);
        }
        if (null == snapshot) {
            return new BocloudResult(false, "快照不存在！");
        }
        return new BocloudResult(true, convert(snapshot), "success");
    }

    @Override
    public BocloudResult list(Long aLong) {
        return null;
    }

}
