package com.bocloud.cmp.provider.network;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.GwQosModel;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.QosPolicyConvertor;
import com.bocloud.cmp.provider.convertor.QosRuleConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * qos
 */
@Provider("bocloud.cmp.easystack")
public class EasyStackGwQosProvider extends EasyStackProvider implements GwQosProvider {

    private static Logger logger = LoggerFactory.getLogger(EasyStackGwQosProvider.class);

    private BeanConvertor<GwQosModel, JSONObject> ruleConvertor = new QosRuleConvertor();

    private BeanConvertor<GwQosModel, JSONObject> policyConvertor = new QosPolicyConvertor();

    public EasyStackGwQosProvider(Butler butler) {
        super(butler);
    }

    /**
     * 列出带宽限制规则 get /v2.0/qos/policies/{policy_id}/bandwidth_limit_rules
     * 
     * @return
     */
    @Override
    public BocloudResult qosRulesList(GwQosModel qosModel) {
        try {
            Result result = this.doGet("/v2.0/qos/policies/" + qosModel.getPolicyUuid() + "/bandwidth_limit_rules",
                    null, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "数据不存在");
            }
            List<JSONObject> list = JSONArray.parseArray(object.getString("bandwidth_limit_rules"), JSONObject.class);
            return new BocloudResult(true, ruleConvertor.convertList(list), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    /**
     * 创建带宽限制规则 post /v2.0/qos/policies/{policy_id}/bandwidth_limit_rules
     * 
     * @return
     */
    @Override
    public BocloudResult qosRulesCreate(GwQosModel qosModel) {
        try {
            JSONObject model = new JSONObject();
            if (null != qosModel.getMaxKbps()) {
                model.put("max_kbps", qosModel.getMaxKbps() * 1000);
            }
            if (null != qosModel.getMaxBurstKbps()) {
                model.put("max_burst_kbps", qosModel.getMaxBurstKbps() * 1000);
            }
            if (null != qosModel.getDirection()) {
                model.put("direction", qosModel.getDirection());
            }
            JSONObject gwQos = new JSONObject();
            gwQos.put("bandwidth_limit_rule", model);
            Result result = this.doPost("/v2.0/qos/policies/" + qosModel.getPolicyUuid() + "/bandwidth_limit_rules",
                    gwQos, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "创建失败");
            }
            JSONObject parseObject = JSONObject.parseObject(object.getString("bandwidth_limit_rule"), JSONObject.class);
            return new BocloudResult(true, ruleConvertor.convertModel(parseObject), "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    /**
     * 显示带宽限制详细规则 get
     * /v2.0/qos/policies/{policy_id}/bandwidth_limit_rules/{rule_id}
     * 
     * @return
     */
    @Override
    public BocloudResult qosRulesDetail(GwQosModel qosModel) {
        try {
            Result result = this.doGet("/v2.0/qos/policies/" + qosModel.getPolicyUuid() + "/bandwidth_limit_rules/"
                    + qosModel.getRuleUuid(), null, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "数据不存在");
            }
            JSONObject parseObject = JSONObject.parseObject(object.getString("bandwidth_limit_rule"), JSONObject.class);
            return new BocloudResult(true, ruleConvertor.convertModel(parseObject), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    /**
     * 更新带宽限制规则 put
     * /v2.0/qos/policies/{policy_id}/bandwidth_limit_rules/{rule_id}
     * 
     * @return
     */
    @Override
    public BocloudResult qosRulesModify(GwQosModel qosModel) {
        try {
            JSONObject model = new JSONObject();
            if (null != qosModel.getMaxKbps()) {
                model.put("max_kbps", qosModel.getMaxKbps());
            }
            if (null != qosModel.getMaxBurstKbps()) {
                model.put("max_burst_kbps", qosModel.getMaxBurstKbps());
            }
            if (null != qosModel.getDirection()) {
                model.put("direction", qosModel.getDirection());
            }
            JSONObject gwQos = new JSONObject();
            gwQos.put("bandwidth_limit_rule", model);
            Result result = this.doPut("/v2.0/qos/policies/" + qosModel.getPolicyUuid() + "/bandwidth_limit_rules/"
                    + qosModel.getRuleUuid(), gwQos, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "编辑失败");
            }
            JSONObject parseObject = JSONObject.parseObject(object.getString("bandwidth_limit_rule"), JSONObject.class);
            return new BocloudResult(true, ruleConvertor.convertModel(parseObject), "编辑成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    /**
     * 删除带宽限制规则 delete
     * /v2.0/qos/policies/{policy_id}/bandwidth_limit_rules/{rule_id}
     * 
     * @return
     */
    @Override
    public BocloudResult qosRulesRemove(GwQosModel qosModel) {
        try {
            BocloudResult bocloudResult = this.qosRulesDetail(qosModel);
            if (null == bocloudResult.getData()) {
                return new BocloudResult(true, "删除成功");
            }
            logger.info("gwQosId : " + qosModel.getRuleUuid());
            Result result = this.doDelete("/v2.0/qos/policies/" + qosModel.getPolicyUuid() + "/bandwidth_limit_rules/"
                    + qosModel.getRuleUuid(), null, "neutron", "network");
            logger.info("result : " + JSON.toJSONString(result));
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    /**
     * get /v2.0/qos/policies
     * 
     * @return
     */
    @Override
    public BocloudResult qosPoliciesList(GwQosModel qosModel) {
        try {
            Result result = this.doGet("/v2.0/qos/policies", null, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "数据不存在");
            }
            List<JSONObject> parseArray = JSONArray.parseArray(object.getString("policies"), JSONObject.class);
            return new BocloudResult(true, policyConvertor.convertList(parseArray), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    /**
     * post /v2.0/qos/policies
     * 
     * @return
     */
    @Override
    public BocloudResult qosPolicyCreate(GwQosModel qosModel) {
        try {
            JSONObject policyModel = new JSONObject();
            policyModel.put("name", qosModel.getName());
            policyModel.put("description", qosModel.getRemark());
            policyModel.put("shared", qosModel.getShared());
            JSONObject gwQos = new JSONObject();
            gwQos.put("policy", policyModel);
            Result result = this.doPost("/v2.0/qos/policies", gwQos, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "创建失败");
            }
            JSONObject parseObject = JSONObject.parseObject(object.getString("policy"), JSONObject.class);
            return new BocloudResult(true, policyConvertor.convertModel(parseObject), "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    /**
     * get /v2.0/qos/policies/{policy_id}
     * 
     * @return
     */
    @Override
    public BocloudResult qosPolicyDetail(GwQosModel qosModel) {
        try {
            Result result = this.doGet("/v2.0/qos/policies/" + qosModel.getPolicyUuid(), null, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "数据不存在");
            }
            JSONObject parseObject = JSONObject.parseObject(object.getString("policy"), JSONObject.class);
            return new BocloudResult(true, policyConvertor.convertModel(parseObject), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    /**
     * put /v2.0/qos/policies/{policy_id}
     * 
     * @return
     */
    @Override
    public BocloudResult qosPolicyModify(GwQosModel qosModel) {
        try {
            JSONObject policyModel = new JSONObject();
            policyModel.put("name", qosModel.getName());
            policyModel.put("description", qosModel.getRemark());
            policyModel.put("shared", qosModel.getShared());
            JSONObject gwQos = new JSONObject();
            gwQos.put("policy", policyModel);
            Result result = this.doPut("/v2.0/qos/policies/" + qosModel.getPolicyUuid(), gwQos, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "修改失败");
            }
            JSONObject parseObject = JSONObject.parseObject(object.getString("policy"), JSONObject.class);
            return new BocloudResult(true, policyConvertor.convertModel(parseObject), "修改成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    /**
     * delete /v2.0/qos/policies/{policy_id}
     * 
     * @return
     */
    @Override
    public BocloudResult qosPolicyRemove(GwQosModel qosModel) {
        try {
            BocloudResult bocloudResult = this.qosPolicyDetail(qosModel);
            if (null == bocloudResult.getData()) {
                return new BocloudResult(true, "删除成功");
            }
            logger.info("policyId : " + qosModel.getPolicyUuid());
            Result result = this.doDelete("/v2.0/qos/policies/" + qosModel.getPolicyUuid(), null, "neutron", "network");
            logger.info("result : " + JSON.toJSONString(result));
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult list() {
        try {
            Result result = this.doGet("/v2.0/gw-qoses/", null, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "数据不存在");
            }
            return new BocloudResult(true, JSONArray.parseArray(object.getString("gw_qoses"), GwQosModel.class),
                    "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(GwQosModel qosModel) {
        try {
            JSONObject gwQos = new JSONObject();
            gwQos.put("gw_qos", qosModel);
            Result result = this.doPost("/v2.0/gw-qoses", gwQos, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "创建失败");
            }
            return new BocloudResult(true, JSONObject.parseObject(object.getString("gw_qos"), GwQosModel.class),
                    "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult modify(GwQosModel qosModel) {
        try {
            String qosUuid = qosModel.getUuid();
            qosModel.setUuid(null);
            JSONObject gwQos = new JSONObject();
            gwQos.put("gw_qos", qosModel);
            Result result = this.doPut("/v2.0/gw-qoses/" + qosUuid, gwQos, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "创建失败");
            }
            return new BocloudResult(true, JSONObject.parseObject(object.getString("gw_qos"), GwQosModel.class),
                    "修改成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    @Override
    public BocloudResult remove(String gwQosId) {
        try {
            BocloudResult bocloudResult = this.detail(gwQosId);
            if (null == bocloudResult.getData()) {
                return new BocloudResult(true, "删除成功");
            }
            logger.info("gwQosId : " + gwQosId);
            Result result = this.doDelete("/v2.0/gw-qoses/" + gwQosId, null, "neutron", "network");
            logger.info("result : " + JSON.toJSONString(result));
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult detail(String gwQosId) {
        try {
            Result result = this.doGet("/v2.0/gw-qoses/" + gwQosId, null, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "数据不存在");
            }
            return new BocloudResult(true, JSONObject.parseObject(object.getString("gw_qos"), GwQosModel.class),
                    "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

}
