package com.bocloud.cmp.provider.network;

import org.apache.commons.lang3.StringUtils;
import org.openstack4j.api.Builders;
import org.openstack4j.api.exceptions.ClientResponseException;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.network.AttachInterfaceType;
import org.openstack4j.model.network.Port;
import org.openstack4j.model.network.Router;
import org.openstack4j.model.network.RouterInterface;
import org.openstack4j.model.network.builder.RouterBuilder;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.model.RouterModel;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.PortConvertor;
import com.bocloud.cmp.provider.convertor.RouterConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.IDFactory;

/**
 * easystack路由操作类
 */
@Provider("bocloud.cmp.easystack")
public class EasyStackRouterProvider extends EasyStackProvider implements RouterProvider {

    public EasyStackRouterProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(RouterModel model) {
        try {
            RouterBuilder builder = Builders.router().name(model.getName()).adminStateUp(model.getAdminStateup());
            if (!StringUtils.isEmpty(model.getNetworkUuid())) {
                builder.externalGateway(model.getNetworkUuid());
            }
            Router router = this.getClient().networking().router().create(builder.build());
            if (null != router) {
                BeanConvertor<RouterModel, Router> convertRoute = new RouterConvertor();
                return new BocloudResult(true, convertRoute.convertModel(router), "创建成功!");
            }
            return new BocloudResult(false, model, "创建路由失败");
        } catch (ClientResponseException e) {
            return new BocloudResult(false, "创建路由失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String routerId) {
        try {
            ActionResponse response = this.getClient().networking().router().delete(routerId);
            if (response.isSuccess()) {
                return new BocloudResult(true, "success");
            }
            return new BocloudResult(false, response.getFault());
        } catch (ClientResponseException e) {
            return new BocloudResult(false, "删除路由失败：" + e.getMessage());
        }

    }

    @Override
    public BocloudResult detail(String routerId) {
        Router router = this.getClient().networking().router().get(routerId);
        if (null == router) {
            return new BocloudResult(false, "路由不存在");
        }
        BeanConvertor<RouterModel, Router> convertRoute = new RouterConvertor();
        return new BocloudResult(true, convertRoute.convertModel(router), "Success");
    }

    @Override
    public BocloudResult modify(RouterModel model) {
        try {
            Router router = this.getClient().networking().router().get(model.getRouterUuid());
            if (null == router) {
                return new BocloudResult(false, "路由不存在");
            }
            router = router.toBuilder().name(model.getName()).adminStateUp(model.getAdminStateup()).build();
            router = this.getClient().networking().router().update(router);
            BeanConvertor<RouterModel, Router> convertRouter = new RouterConvertor();
            return new BocloudResult(true, convertRouter.convertModel(router), "修改成功");
        } catch (ClientResponseException e) {
            return new BocloudResult(false, "修改路由失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult attach(RouterModel model) {
        RouterInterface inter = null;
        PortModel portModel = null;
        try {
            String address = model.getAddress();
            if (StringUtils.isEmpty(address)) {
                inter = this.getClient().networking().router().attachInterface(model.getRouterUuid(),
                        AttachInterfaceType.SUBNET, model.getSubnetUuid());
            } else {
                Port port = this.getClient().networking().port()
                        .create(Builders.port().name(IDFactory.instance().uuid())
                                .networkId(model.getAttachNetworkUuid()).fixedIp(address, model.getSubnetUuid())
                                .build());
                if (null == port) {
                    return new BocloudResult(false, "指定的IP创建端口时发生错误");
                }
                inter = this.getClient().networking().router().attachInterface(model.getRouterUuid(),
                        AttachInterfaceType.PORT, port.getId());
            }
            if (null == inter) {
                return new BocloudResult(false, "路由或子网不存在！");
            }
            String portId = inter.getPortId();
            Port port = this.getClient().networking().port().get(portId);
            portModel = new PortConvertor().convertModel(port);
        } catch (ClientResponseException e) {
            if (400 == e.getStatus()) {
                if (null != inter) {
                    return new BocloudResult(true, detail(inter.getId()).getData(), "子网连接失败");
                }
            }
            return new BocloudResult(false, "子网连接失败：" + e.getMessage());
        }
        return new BocloudResult(true, portModel, "success");
    }

    @Override
    public BocloudResult detach(RouterModel model) {
        try {
            RouterInterface inter = this.getClient().networking().router().detachInterface(model.getRouterUuid(),
                    model.getSubnetUuid(), model.getPortUuid());
            if (null == inter) {
                return new BocloudResult(false, "路由或子网不存在！");
            } else {
                return new BocloudResult(true, "success");
            }
        } catch (ClientResponseException e) {
            return new BocloudResult(false, "断开子网失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult config(RouterModel model) {
        Router router = this.getClient().networking().router().get(model.getRouterUuid()).toBuilder()
                .externalGateway(model.getNetworkUuid()).build();
        router = this.getClient().networking().router().update(router);
        if (null == router) {
            return new BocloudResult(false, "路由不存在！");
        } else {
            BeanConvertor<RouterModel, Router> convertRouter = new RouterConvertor();
            return new BocloudResult(true, convertRouter.convertModel(router), "success");
        }
    }

    @Override
    public BocloudResult clear(String routerId) {
        try {
            Router router = this.getClient().networking().router().get(routerId).toBuilder().clearExternalGateway()
                    .build();
            router = this.getClient().networking().router().update(router);
            if (null == router) {
                return new BocloudResult(false, "路由不存在！");
            } else {
                return new BocloudResult(true, router, "success");
            }
        } catch (ClientResponseException e) {
            return new BocloudResult(false, "清除网关失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult list() {
        return null;
    }

}
