package com.bocloud.cmp.provider.network;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VpnConnModel;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.VpnConnConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * easystack conn
 */
@Provider("bocloud.cmp.easystack")
public class EasyStackVpnConnProvider extends EasyStackProvider implements VpnConnProvider {

	private static Logger logger = LoggerFactory.getLogger(EasyStackVpnConnProvider.class);

	private BeanConvertor<VpnConnModel, JSONObject> convertor = new VpnConnConvertor();

	public EasyStackVpnConnProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			Result result = this.doGet("/v2.0/vpn/ipsec-site-connections", null, "neutron", "network");
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "数据不存在");
			}
			List<JSONObject> list = JSONArray.parseArray(object.getString("ipsec_site_connections"), JSONObject.class);
			List<VpnConnModel> convertList = convertor.convertList(list);
			return new BocloudResult(true, convertList, "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult detail(String vpnConnId) {
		try {
			Result result = this.doGet("/v2.0/vpn/ipsec-site-connections/" + vpnConnId, null, "neutron", "network");
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "数据不存在");
			}
			VpnConnModel convertModel = convertor
					.convertModel(JSONObject.parseObject(object.getString("ipsec_site_connection")));
			return new BocloudResult(true, convertModel, "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult create(VpnConnModel model) {
		try {
			JSONObject param = new JSONObject();
			param.put("psk", model.getPsk());
			// bi-directional
			param.put("initiator", model.getInitiator());
			param.put("ipsecpolicy_id", model.getIpsecUuid());
			param.put("admin_state_up", model.getAdminStateUp());
			param.put("mtu", model.getMtu());
			param.put("dpd",JSON.parseObject(model.getDpd()));
			param.put("peer_ep_group_id", model.getPeerEpGroupId());
			param.put("ikepolicy_id", model.getIkeUuid());
			param.put("vpnservice_id", model.getVpnUuid());
			param.put("local_ep_group_id", model.getLocalEpGroupId());
			param.put("peer_address", model.getPeerAddress());
			param.put("peer_id", model.getPeerAddress());
			param.put("name", model.getName());
			String peerCidrs = model.getPeerCidrs();
			if(null != peerCidrs) {
				String[] split = peerCidrs.split(",");
				param.put("peer_cidrs", split);
			}
			param.put("description", model.getRemark());
			JSONObject ipsecSiteConnection = new JSONObject();
			ipsecSiteConnection.put("ipsec_site_connection", param);
			logger.info("请求参数 ： " + ipsecSiteConnection);
			Result result = this.doPost("/v2.0/vpn/ipsec-site-connections", ipsecSiteConnection, "neutron", "network");
			logger.info("请求结果 ： " + JSON.toJSONString(result));
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "创建失败");
			}
			VpnConnModel convertModel = convertor
					.convertModel(JSONObject.parseObject(object.getString("ipsec_site_connection")));
			return new BocloudResult(true, convertModel, "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "创建失败");
		}
	}

	@Override
	public BocloudResult delete(String vpnUuid) {
		try {
			BocloudResult bocloudResult = this.detail(vpnUuid);
			if (null == bocloudResult.getData()) {
				return new BocloudResult(true, "删除成功");
			}
			Result result = this.doDelete("/v2.0/vpn/ipsec-site-connections/" + vpnUuid, null, "neutron", "network");
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "删除失败");
		}
	}

	@Override
	public BocloudResult modify(VpnConnModel model) {
		try {
			JSONObject param = new JSONObject();
			param.put("psk", model.getPsk());
			param.put("initiator", model.getInitiator());
			param.put("ipsecpolicy_id", model.getIpsecUuid());
			param.put("admin_state_up", model.getAdminStateUp());
			param.put("mtu", model.getMtu());
			param.put("ikepolicy_id", model.getIkeUuid());
			param.put("vpnservice_id", model.getVpnUuid());
			param.put("peer_address", model.getPeerAddress());
			param.put("peer_id", model.getPeerAddress());
			String peerCidrs = model.getPeerCidrs();
			if(null != peerCidrs) {
				String[] split = peerCidrs.split(",");
				param.put("peer_cidrs", split);
			}
			param.put("name", model.getName());
			param.put("description", model.getRemark());

			JSONObject ipsecSiteConnection = new JSONObject();
			ipsecSiteConnection.put("ipsec_site_connection", param);
			Result result = this.doPost("/v2.0/vpn/ipsec-site-connections/" + model.getVpnConnUuid(),
					ipsecSiteConnection, "neutron", "network");
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "编辑失败");
			}
			VpnConnModel convertModel = convertor
					.convertModel(JSONObject.parseObject(object.getString("ipsec_site_connection")));
			return new BocloudResult(true, convertModel, "编辑成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "创建失败");
		}
	}

}
