package com.bocloud.cmp.provider.storage;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.commons.lang3.StringUtils;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.common.DLPayload;
import org.openstack4j.model.common.Payload;
import org.openstack4j.model.common.payloads.InputStreamPayload;
import org.openstack4j.model.common.payloads.URLPayload;
import org.openstack4j.model.storage.object.SwiftContainer;
import org.openstack4j.model.storage.object.SwiftObject;
import org.openstack4j.model.storage.object.options.ContainerListOptions;
import org.openstack4j.model.storage.object.options.ObjectPutOptions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.*;

/**
 * easystack对象存储操作类
 */
@Provider("bocloud.cmp.easystack")
public class EasyStackObjectStorageProvider extends EasyStackProvider implements ObjectStorageProvider {

    private static Logger logger = LoggerFactory.getLogger(EasyStackObjectStorageProvider.class);

    public EasyStackObjectStorageProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult createContainer(ObjectStorageContainerModel container) {
        ActionResponse actionResponse = this.getClient().objectStorage().containers().create(container.getName());
        if (actionResponse.isSuccess()) {
            if (actionResponse.getCode() == 202) {
                return new BocloudResult(false, "数据已存在");
            }
            return this.detailContainer(container);
        } else {
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult removeContainer(ObjectStorageContainerModel container) {
        ActionResponse actionResponse = this.getClient().objectStorage().containers().delete(container.getName());
        if (actionResponse.isSuccess()) {
            return new BocloudResult(true, "删除成功");
        } else {
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult existsContainer(ObjectStorageContainerModel container) {
        return detailContainer(container);
    }

    @Override
    public BocloudResult detailContainer(ObjectStorageContainerModel container) {
        ContainerListOptions containerListOptions = ContainerListOptions.create();
        containerListOptions.startsWith(container.getName());
        List<? extends SwiftContainer> list = this.getClient().objectStorage().containers().list(containerListOptions);
        for (SwiftContainer swiftContainer : list) {
            if (swiftContainer != null && StringUtils.equals(container.getName(), swiftContainer.getName())) {
                return new BocloudResult(true, converToDto(swiftContainer), "创建成功");
            }
        }
        return new BocloudResult(false, "不存在");
    }

    @Override
    public BocloudResult listContainer() {
        List<? extends SwiftContainer> list = this.getClient().objectStorage().containers().list();
        List<ObjectStorageContainerBean> result = new ArrayList<>();
        for (SwiftContainer swiftContainer : list) {
            ObjectStorageContainerBean container = converToDto(swiftContainer);
            result.add(container);
        }
        return new BocloudResult(true, result, "查询成功");
    }

    @Override
    public BocloudResult listItem(String containerName) {
        List<? extends SwiftObject> list = this.getClient().objectStorage().objects().list(containerName);
        List<ObjectStorageItemBean> items = new ArrayList<>();
        for (SwiftObject swiftObject : list) {
            if (swiftObject != null) {
                ObjectStorageItemBean storageItem = converToDto(swiftObject);
                items.add(storageItem);
            }
        }
        return new BocloudResult(true, items, "查询成功");

    }

    private ObjectStorageItemBean converToDto(SwiftObject swiftObject) {
        ObjectStorageItemBean storageItem = new ObjectStorageItemBean();
        storageItem.setContainerName(swiftObject.getContainerName());
        storageItem.setContentType(swiftObject.getMimeType());
        storageItem.setDirectoryName(swiftObject.getDirectoryName());
        storageItem.setSize(swiftObject.getSizeInBytes());
        // storageItem.setIsDirectory(swiftObject.isDirectory()?1:0);
        // 这个openstack4j返回的一直是false,不能用这个判断 FIXME
        storageItem.setIsDirectory(
                (StringUtils.isBlank(swiftObject.getMimeType()) && swiftObject.getSizeInBytes() == 0) ? 1 : 0);
        storageItem.setName(swiftObject.getName()); // 目录的名称最后一位是/
        storageItem.setUpdateTime(swiftObject.getLastModified());
        storageItem.setHash(swiftObject.getETag());
        return storageItem;
    }

    @Override
    public BocloudResult createItem(ObjectStorageItemModel itemModel, String bspPath) {

        if (itemModel.getIsDirectory() == 0) { // 文件
            ObjectPutOptions options = ObjectPutOptions.create();
            URL url = null;
            URL removeUrl = null;
            try {
                url = new URL("http://" + bspPath + "/files/download?filePath="
                        + URLEncoder.encode(itemModel.getUploadUrl(), "UTF-8"));
                removeUrl = new URL("http://" + bspPath + "/files/removeDownload?filePath="
                        + URLEncoder.encode(itemModel.getUploadUrl(), "UTF-8"));
            } catch (MalformedURLException e) {
                return new BocloudResult(false, "上传失败");
            } catch (UnsupportedEncodingException e) {
                return new BocloudResult(false, "不支持的文件名");
            }
            String checkSum = this.getClient().objectStorage().objects().put(itemModel.getContainerName(),
                    itemModel.getName(), new URLPayload(url), options);
            SwiftObject swiftObject = this.getClient().objectStorage().objects().get(itemModel.getContainerName(),
                    itemModel.getName());
            ObjectStorageItemModel retItemModel = convertSwiftObjectToItem(swiftObject);
            retItemModel.setOrigFilename(itemModel.getOrigFilename());
            callRemoveFileUrl(removeUrl);
            if (StringUtils.isNotBlank(checkSum)) {
                return new BocloudResult(true, retItemModel, "上传成功");
            } else {
                return new BocloudResult(false, "上传失败");
            }
        } else { // 文件夹
            String folderName = itemModel.getName();
            folderName = folderName.replaceAll("[/]+", "/");
            if (folderName.indexOf("/") == 0) { // 99cloud的目录是伪目录,第一个不是/ ,如果第一位是/,会把它当成真目录,目录会有问题
                folderName = folderName.substring(1);
            }
            ObjectPutOptions option = ObjectPutOptions.create().contentType("application/pseudo-folder");
            String path = this.getClient().objectStorage().objects().put(itemModel.getContainerName(), folderName,
                    new InputStreamPayload(new ByteArrayInputStream(new byte[0])), option); // 传个空的数据,是为了把大小为0的当作目录处理
            if (StringUtils.isNotBlank(path)) {
                return new BocloudResult(true, itemModel, "创建文件夹成功");
            } else {
                return new BocloudResult(false, "上传文件夹失败");
            }
        }
    }

    @Override
    public BocloudResult existsItem(ObjectStorageItemModel itemModel) {
        return detailItem(itemModel);
    }

    @Override
    public BocloudResult removeItem(ObjectStorageItemModel itemModel) {
        ActionResponse delete = this.getClient().objectStorage().objects().delete(itemModel.getContainerName(),
                itemModel.getName());
        if (delete.isSuccess()) {
            return new BocloudResult(true, "删除成功");
        } else if (delete.getCode() == 404) { // 文件已经找不到了
            return new BocloudResult(true, "删除成功");
        } else {
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult detailItem(ObjectStorageItemModel itemModel) {
        SwiftObject swiftObject = this.getClient().objectStorage().objects().get(itemModel.getContainerName(),
                itemModel.getName());
        if (swiftObject != null) {
            ObjectStorageItemBean storageItem = converToDto(swiftObject);
            return new BocloudResult(true, storageItem, "该对象已经存在");
        } else {
            return new BocloudResult(false, "该对象不存在");
        }
    }

    @Override
    public InputStream download(ObjectStorageItemModel itemModel) {
        DLPayload download = this.getClient().objectStorage().objects().download(itemModel.getContainerName(),
                itemModel.getName());
        return download.getInputStream();
    }

    @Override
    public BocloudResult uploadItem(ObjectStorageItemModel itemModel, InputStream inputStream) {
        Payload<?> payLoad = new InputStreamPayload(inputStream);
        String put = this.getClient().objectStorage().objects().put(itemModel.getContainerName(), itemModel.getName(),
                payLoad);
        return new BocloudResult(true, put, "上传对象成功");
    }

    private ObjectStorageContainerBean converToDto(SwiftContainer swiftContainer) {
        ObjectStorageContainerBean container = new ObjectStorageContainerBean();
        container.setName(swiftContainer.getName());
        container.setItemCount(swiftContainer.getTotalSize());
        Map<String, String> metadata = swiftContainer.getMetadata();
        container.setCreateTime(new Date(Double.valueOf(metadata.get("X-Timestamp")).longValue() * 1000));
        container.setItemCount(Long.parseLong(metadata.get("X-Container-Object-Count")));
        container.setSize(Long.parseLong(metadata.get("X-Container-Bytes-Used")));
        return container;
    }

    /**
     * 删除本地文件的方法
     *
     * @param url
     */
    private void callRemoveFileUrl(URL url) {
        try {
            HttpClient httpClient = new HttpClient();
            Map<String, Object> header = new HashMap<>();
            Map<String, Object> params = header;
            Result result = httpClient.get(header, params, url.toString());
            if (!result.isSuccess()) {
                logger.error("文件删除失败,url=" + url);
            }
        } catch (Exception e) {
            logger.error("文件删除失败,url=" + url);
        }
    }

    private ObjectStorageItemModel convertSwiftObjectToItem(SwiftObject swiftObject) {
        ObjectStorageItemModel storageItem = new ObjectStorageItemModel();
        storageItem.setContainerName(swiftObject.getContainerName());
        storageItem.setContentType(swiftObject.getMimeType());
        storageItem.setDirectoryName(swiftObject.getDirectoryName());
        storageItem.setSize(swiftObject.getSizeInBytes());
        // storageItem.setIsDirectory(swiftObject.isDirectory()?1:0); 99cloud不能用这个判断
        // FIXME
        storageItem
                .setIsDirectory(((StringUtils.isEmpty(swiftObject.getMimeType()) && swiftObject.getSizeInBytes() == 0)
                        || "application/pseudo-folder".equals(swiftObject.getMimeType())) ? 1 : 0);
        String name = swiftObject.getName();

        if (name.indexOf("/") == 0) { // 处理首位 "/" 99cloud那边,对首位/的目录处理有问题
            name = name.substring(1);
        }
        storageItem.setName(name); // 目录的名称最后一位是/
        storageItem.setUpdateTime(
                new Date(Double.valueOf(swiftObject.getMetadata().get("X-Timestamp")).longValue() * 1000));
        storageItem.setHash(swiftObject.getETag());
        Map<String, String> storageMeta = swiftObject.getMetadata();
        if (storageMeta != null) {
            storageItem.setOrigFilename(storageMeta.get("Orig-Filename"));
        }
        return storageItem;
    }

}
