package com.bocloud.cmp.provider.storage;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.ImageConvertor;
import com.bocloud.cmp.provider.convertor.VolumeConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import org.apache.commons.lang3.StringUtils;
import org.openstack4j.api.Builders;
import org.openstack4j.api.EndpointTokenProvider;
import org.openstack4j.api.exceptions.ClientResponseException;
import org.openstack4j.api.types.ServiceType;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.compute.VolumeAttachment;
import org.openstack4j.model.image.DiskFormat;
import org.openstack4j.model.image.Image;
import org.openstack4j.model.storage.block.Volume;
import org.openstack4j.model.storage.block.VolumeSnapshot;
import org.openstack4j.model.storage.block.VolumeType;
import org.openstack4j.model.storage.block.VolumeUploadImage;
import org.openstack4j.model.storage.block.builder.VolumeBuilder;
import org.openstack4j.model.storage.block.builder.VolumeTypeBuilder;
import org.openstack4j.model.storage.block.options.UploadImageData;
import org.openstack4j.openstack.internal.OSClientSession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * easystack卷操作类
 */
@Provider("bocloud.cmp.easystack")
public class EasyStackVolumeProvider extends EasyStackProvider implements VolumeProvider {

    private static Logger logger = LoggerFactory.getLogger(EasyStackVolumeProvider.class);

    public EasyStackVolumeProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        List<VolumeModel> models = new ArrayList<>();
        List<? extends Volume> volumes = this.getClient().blockStorage().volumes().list();
        if (null != volumes) {
            for (Volume volume : volumes) {
                BeanConvertor<VolumeModel, Volume> volumeConvert = new VolumeConvertor();
                models.add(volumeConvert.convertModel(volume));
            }
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult create(VolumeModel volume) {
        try {
            boolean bootable = false;
            if (null != volume.getBootVolume()) {
                bootable = volume.getBootVolume();
            }
            VolumeBuilder builder = Builders.volume();
            builder.name(volume.getName()).description(volume.getRemark()).size(volume.getSize());
            String sourceImgId = volume.getImageUuid();
            if (!StringUtils.isEmpty(sourceImgId)) {
                bootable = true;
                builder.imageRef(sourceImgId);
            }
            String sourceVolidId = volume.getSourceVolidUuid();
            if (!StringUtils.isEmpty(sourceVolidId)) {
                builder.source_volid(sourceVolidId);
            }
            String sourceSnapId = volume.getSourceSnapUuid();
            if (!StringUtils.isEmpty(sourceSnapId)) {
                builder.snapshot(sourceSnapId);
            }
            String volumeType = volume.getVolumeType();
            if (!StringUtils.isEmpty(volumeType)) {
                builder.volumeType(volumeType);
            }
            Volume volumeCreated = this.getClient().blockStorage().volumes().create(builder.bootable(bootable).build());
            if (null == volumeCreated) {
                return new BocloudResult(false, "create fail");
            }
            volume.setBootVolume(volumeCreated.bootable());
            volume.setVolumeUuid(volumeCreated.getId());
            volume.setCreateTime(volumeCreated.getCreated());
            volume.setStatus(volumeCreated.getStatus().name());
            volume.setZone(volumeCreated.getZone());
            // 等待启动盘创建完毕
            if (bootable && !"ERROR".equals(volumeCreated.getStatus().name())) {
                Long startTime = System.currentTimeMillis();
                Boolean flag = true;
                while (flag) {
                    volumeCreated = this.getClient().blockStorage().volumes().get(volumeCreated.getId());
                    if (null == volumeCreated) {
                        return new BocloudResult(false, "create fail");
                    }
                    if ("ERROR".equals(volumeCreated.getStatus().name())) {
                        this.getClient().blockStorage().volumes().forceDelete(volumeCreated.getId());
                        return new BocloudResult(false, volume, "根磁盘创建错误");
                    }
                    Long endTime = System.currentTimeMillis();
                    if (endTime - startTime > 60 * 60 * 1000) {// 等待1小时
                        flag = false;
                        volume.setStatus(volumeCreated.getStatus().name());
                        return new BocloudResult(false, volume, "create bootable volume timeout");
                    }
                    if ("available".equalsIgnoreCase(volumeCreated.getStatus().name()) && volumeCreated.bootable()
                            && !StringUtils.isEmpty(volumeCreated.getImageRef())) {
                        logger.info("Block device is ready...");
                        volume.setStatus(volumeCreated.getStatus().name());
                        flag = false;
                    }
                }
            } else if (bootable && "ERROR".equals(volumeCreated.getStatus().name())) {
                this.getClient().blockStorage().volumes().forceDelete(volumeCreated.getId());
                return new BocloudResult(false, volume, "启动盘创建错误");
            }
            return new BocloudResult(true, this.detail(volumeCreated.getId()).getData(), "create success");
        } catch (ClientResponseException e) {
            logger.error("云硬盘创建失败:", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String volumeId) {
        Volume volume = this.getClient().blockStorage().volumes().get(volumeId);
        if (null == volume) {
            logger.info("volume not exist...");
            return new BocloudResult(true, "云硬盘不存在");
        }
        if ("available".equalsIgnoreCase(volume.getStatus().name()) || "error".equalsIgnoreCase(volume.getStatus().name())
                || "error_restoring".equalsIgnoreCase(volume.getStatus().name())
                || "error_extending".equalsIgnoreCase(volume.getStatus().name())
                || "error_managing".equalsIgnoreCase(volume.getStatus().name())) {
            ActionResponse response = this.getClient().blockStorage().volumes().forceDelete(volumeId);
            if (response.isSuccess()) {
                return new BocloudResult(true, "云硬盘删除成功！");
            } else {
                logger.error("remove instance [{}] failed :{}", volumeId, response.getFault());
                return new BocloudResult(false, "云硬盘删除失败！【" + response.getFault() + "】");
            }
        }
        return new BocloudResult(true, "云硬盘删除成功！");
    }

    @Override
    public BocloudResult detail(String volumeId) {
        Volume volume = this.getClient().blockStorage().volumes().get(volumeId);
        if (null == volume) {
            return new BocloudResult(false, "云硬盘不存在！");
        }
        BeanConvertor<VolumeModel, Volume> volumeConvert = new VolumeConvertor();
        return new BocloudResult(true, volumeConvert.convertModel(volume), "success");
    }

    @Override
    public BocloudResult modify(VolumeModel volume) {
        ActionResponse response = this.getClient().blockStorage().volumes().update(volume.getVolumeUuid(), volume.getName(),
                volume.getRemark());
        if (null != response) {
            return new BocloudResult(true, "操作成功！");
        } else {
            logger.error("");
            return new BocloudResult(false, "操作失败！");
        }
    }

    @Override
    public BocloudResult snapshot(SnapshotModel model) {
        /*try {
            VolumeSnapshot volumnBuilder = Builders.volumeSnapshot().description(model.getRemark()).name(model.getName())
                    .volume(model.getVolumeUuid()).force(true).build(); // force =
            // true可以进行强制创建快照
            VolumeSnapshot snapshot = this.getClient().blockStorage().snapshots().create(volumnBuilder);
            if (null == snapshot) {
                return new BocloudResult(false, "failed!");
            } else {
                return new BocloudResult(true, convert(snapshot), "success");
            }
        } catch (ClientResponseException e) {
            logger.error("创建快照失败！", e);
            if (400 == e.getStatus()) {
                return new BocloudResult(false, e.getMessage());
            }
            return new BocloudResult(false, e.getMessage());
        }*/
        try {
            JSONObject params = new JSONObject();
            params.put("name", model.getName());
            params.put("description", model.getRemark());
            params.put("volume_id", model.getVolumeUuid());
            JSONObject snapshotParam = new JSONObject();
            snapshotParam.put("snapshot", params);
            Result result = blockStoragePost("/snapshots", snapshotParam);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("snapshot");
            VolumeSnapshot snapshot = this.getClient().blockStorage().snapshots().get(jsonObject.getString("id"));
            if (null == snapshot) {
                return new BocloudResult(false, "快照创建失败");
            }
            return new BocloudResult(true, convert(snapshot), "快照创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
        }
        return new BocloudResult(false, "快照创建失败");
    }

    @Override
    public BocloudResult attach(String serverId, String volumeId) {
        List<String> list = this.getClient().compute().servers().get(serverId).getOsExtendedVolumesAttached();
        String device = "/dev/vda";
        List<String> strs = new ArrayList<>();
        for (int i = 0; i < 26; i++) {
            strs.add((char) (97 + i) + "");
        }
        if (!ListTool.isEmpty(list)) {
            device = "/dev/vd" + strs.get(list.size());
        }
        VolumeAttachment attachment = this.getClient().compute().servers().attachVolume(serverId, volumeId, device);
        if (null == attachment) {
            return new BocloudResult(false, "failed!");
        } else {
            return new BocloudResult(true, attachment.getDevice(), "success");
        }
    }

    @Override
    public BocloudResult detach(String serverId, String volumeId) {
        try {
            List<String> list = this.getClient().compute().servers().get(serverId).getOsExtendedVolumesAttached();
            if (ListTool.isEmpty(list)) {
                return new BocloudResult(true, "success");
            }
            if (!list.contains(volumeId)) {
                return new BocloudResult(true, "success");
            }
            this.getClient().compute().servers().detachVolume(serverId, volumeId);
            return new BocloudResult(true, "success");
        } catch (Exception e) {
            logger.error("卸载硬盘失败:", e);
            return new BocloudResult(false, "卸载硬盘失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult resetStatus(String volumeId, String status) {
        if (StringUtils.isEmpty(volumeId) || StringUtils.isEmpty(status)) {
            return new BocloudResult(false, "No required param");
        }
        Volume.Status targetStatus = Volume.Status.valueOf(status);
        if (targetStatus == null) {
            return new BocloudResult(false, "param not right");
        }
        ActionResponse actionResponse = this.getClient().blockStorage().volumes().resetState(volumeId, targetStatus);
        if (actionResponse.isSuccess()) {
            return new BocloudResult(true, "");
        } else {
            return new BocloudResult(false, actionResponse.getFault());
        }

    }

    @Override
    public BocloudResult uploadToImage(String volumeId, ImageModel imageModel) {
        UploadImageData data = UploadImageData.create(imageModel.getName());
        if (!StringUtils.isEmpty(imageModel.getOsType())) {
            data.diskFormat(DiskFormat.value(imageModel.getOsType()));
        }
        Boolean force = imageModel.getForce();
        if (null == force) {
            force = false;
        }
        data.force(force);
        ActionResponse response = this.getClient().blockStorage().volumes().resetState(volumeId, Volume.Status.AVAILABLE);
        if (response.isSuccess()) {
            VolumeUploadImage image = this.getClient().blockStorage().volumes().uploadToImage(volumeId, data);
            if (null == image) {
                return new BocloudResult(false, "upload to image fail");
            }
            ImageModel model = this.getActiveImage(image.getImageId());
            return new BocloudResult(true, model, "upload to image success");
        } else {
            return new BocloudResult(false, response.getFault());
        }
    }

    @Override
    public BocloudResult extend(String volumeId, Integer newSize) {
        Volume volume = this.getClient().blockStorage().volumes().get(volumeId);
        if (null == volume) {
            return new BocloudResult(false, "云硬盘不存在！");
        }
        HttpClient httpClient = new HttpClient();
        httpClient.setTimeout(5000);
        httpClient.setDataFormat(PostDataFormat.RAW);
        Map<String, Object> params = MapTools.simpleMap("os-extend", MapTools.simpleMap("new_size", newSize));
        Map<String, Object> header = MapTools.simpleMap("Openstack-API-version", "volume 3.42");
        EndpointTokenProvider provider = OSClientSession.getCurrent();
        header.put("X-Auth-Token",provider.getTokenId());
        String endpoint = provider.getEndpoint(ServiceType.BLOCK_STORAGE);
        endpoint = endpoint.replace("v1", "v3");
        Result post = httpClient.post(header, params, "" + endpoint + "/volumes/" + volumeId + "/action");
        //ActionResponse response = this.getClient().blockStorage().volumes().extend(volumeId, newSize);
        if (post.isSuccess()) {
            return new BocloudResult(true, "卷扩展成功");
        } else {
            return new BocloudResult(false, post.getErrorMsg());
        }
    }

    @Override
    public BocloudResult list(String s, String s1) {
        return null;
    }

    @Override
    public BocloudResult remove(H3cVolumeModel h3cVolumeModel) {
        return null;
    }

    @Override
    public BocloudResult createVolumeType(VolumeTypeModel model) {
        try {
            VolumeTypeBuilder builder = Builders.volumeType().name(model.getName());
            if (null != model.getExtraSpecMap() && !model.getExtraSpecMap().isEmpty()) {
                builder.extraSpecs(model.getExtraSpecMap());
            }
            VolumeType type = this.getClient().blockStorage().volumes().createVolumeType(builder.build());
            if (null == type) {
                return new BocloudResult(false, "创建云硬盘类型失败！");
            }
            model.setVolumeTypeUuid(type.getId());
            model.setName(type.getName());
            if (null != type.getExtraSpecs() && !type.getExtraSpecs().isEmpty()) {
                model.setExtraSpecs(JSONObject.toJSONString(type.getExtraSpecs()));
            }
            return new BocloudResult(true, model, "success");
        } catch (Exception e) {
            logger.error("创建云硬盘类型失败：", e);
            return new BocloudResult(false, "创建云硬盘类型失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult deleteVolumeType(String volumeTypeUuid) {
        try {
            this.getClient().blockStorage().volumes().deleteVolumeType(volumeTypeUuid);
            return new BocloudResult(true, "success");
        } catch (Exception e) {
            logger.error("删除云硬盘类型失败：", e);
            return new BocloudResult(false, "删除云硬盘类型失败：" + e.getMessage());
        }
    }

    private ImageModel getActiveImage(String imageId) {
        ImageModel imageModel = null;
        Image image = null;
        boolean flag = true;
        BeanConvertor<ImageModel, Image> convertImage = new ImageConvertor();
        Long startTime = System.currentTimeMillis();
        while (flag) {
            image = this.getClient().images().get(imageId);
            // 镜像已经为运行中
            if ("active".equalsIgnoreCase(image.getStatus().name())) {
                imageModel = convertImage.convertModel(image);
                flag = false;
            }
            Long endTime = System.currentTimeMillis();
            // 因为备份镜像的过程比较漫长，因此此处超时时间设置为1小时
            if (endTime - startTime > 60 * 60 * 1000) {
                imageModel = convertImage.convertModel(image);
                flag = false;
            }
        }
        return imageModel;
    }

    private SnapshotModel convert(VolumeSnapshot snapshot) {
        if (null == snapshot) {
            return null;
        }
        SnapshotModel model = new SnapshotModel();
        model.setCreateTime(snapshot.getCreated());
        model.setName(snapshot.getName());
        model.setSize(snapshot.getSize());
        model.setSnapshotUuid(snapshot.getId());
        model.setVolumeUuid(snapshot.getVolumeId());
        model.setStatus(snapshot.getStatus().name());
        model.setRemark(snapshot.getDescription());
        return model;
    }

    @Override
    public BocloudResult attach(String serverId, String mountpoint, String volumeId) {
        // TODO Auto-generated method stub
        return null;
    }
}
