package com.bocloud.cmp.provider.f5;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.f5.VirtualServerModel;
import com.bocloud.cmp.provider.F5Provider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * VirtualServer操作接口实现类
 *
 * @author liuyuanyuan
 */
@Provider("bocloud.cmp.f5")
public class F5VirtualServerProvider extends F5Provider implements VirtualServerProvider {
    private static final Logger logger = LoggerFactory.getLogger(F5VirtualServerProvider.class);
    private Butler butler;

    public F5VirtualServerProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult create(VirtualServerModel virtualServerModel) {
        // 开始创建virtualServer
        // 处理请求路径
        String url = "/mgmt/tm/ltm/virtual/";
        url = butler.getUrl() + url;
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(virtualServerModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.post(header, body, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "创建Virtual Server失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "创建成功");
    }

    @Override
    public BocloudResult remove(String name) {
        // 执行删除VirtualServer
        // 处理请求路径
        String url = "/mgmt/tm/ltm/virtual/" + name;
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.delete(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "删除Virtual Server失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "删除成功");
    }

    @Override
    public BocloudResult update(VirtualServerModel virtualServerModel) {
        // 开始更新virtualServer
        // 处理请求路径
        String url = "/mgmt/tm/ltm/virtual/" + virtualServerModel.getName();
        url = butler.getUrl() + url;
        buildVirtualServerModel(virtualServerModel);
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(virtualServerModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.patch(header, body, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "更新Virtual Server失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "更新成功");
    }

    @Override
    public BocloudResult list() {
        // 获取virtualServer列表。。。
        // 处理请求路径
        String url = "/mgmt/tm/ltm/virtual/";
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取Virtual Server数据失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    private void buildVirtualServerModel(VirtualServerModel virtualServerModel) {
        virtualServerModel.setName(null);
    }

    @Override
    public BocloudResult stats(String vsName) {
        // 获取virtualServer的状态。。。
        // 处理请求路径
        try {
            vsName = URLEncoder.encode(vsName, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            logger.error("url编码转换异常", e);
        }
        String url = "/mgmt/tm/ltm/virtual/" + vsName + "/stats";
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取Virtual Server状态失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    @Override
    public BocloudResult getProfiles(String vsName) {
        // 获取virtual server关联的profiles
        // 处理请求路径
        String url = "/mgmt/tm/ltm/virtual/" + vsName + "/profiles";
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取Virtual Server关联下的Profile失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }
}
