package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.common.model.FusionCloudDomain;
import com.bocloud.cmp.common.model.FusionCloudProject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * 华为私有云认证driver
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年9月17日
 */
@Provider("bocloud.cmp.fusioncloud")
public class FusionCloudAuthProvider extends FusionCloudProvider implements AuthProvider {

    private static final Logger logger = LoggerFactory.getLogger(FusionCloudAuthProvider.class);

    /**
     * @param butler
     */
    public FusionCloudAuthProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult auth() {
        if (this.isAccessable()) {
            return new BocloudResult(true, "" + this.isAccessable());
        }
        return new BocloudResult(this.isAccessable(), "认证信息失败");
    }

    @Override
    public BocloudResult createProject(String projectName) {
        try {
            BocloudResult bocloudResult = this.listDomains();
            if (bocloudResult.isFailed()) {
                return new BocloudResult(false, bocloudResult.getMessage());
            }
            List<FusionCloudDomain> domains = JSON.parseArray(JSON.toJSONString(bocloudResult.getData()), FusionCloudDomain.class);
            Optional<FusionCloudDomain> domainOpt = domains.stream().filter(FusionCloudDomain::getEnabled).findFirst();
            if (!domainOpt.isPresent()) {
                return new BocloudResult(false, "创建失败，未找到domain");
            }
            FusionCloudProject project = new FusionCloudProject();
            project.setName("projectName");
            project.setTenantId(domainOpt.get().getId());
            Map<String, Object> params = new HashMap<>();
            params.put("project", project);

            Result result = this.doPost(this.getEndpoint() + "/rest/vdc/v3.1/projects", params);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            FusionCloudProject object = JSON.parseObject(result.getMessage(), FusionCloudProject.class);
            return new BocloudResult(true, object, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult removeProject(String projectName) {
        try {
            BocloudResult bocloudResult = this.listProjects();
            if (bocloudResult.isFailed()) {
                return new BocloudResult(false, bocloudResult.getMessage());
            }
            List<FusionCloudProject> projects = JSON.parseArray(JSON.toJSONString(bocloudResult.getData()), FusionCloudProject.class);
            Optional<FusionCloudProject> projectOpt = projects.stream()
                    .filter(FusionCloudProject::getEnabled)
                    .filter(project -> Objects.equals(projectName, project.getName()))
                    .findFirst();
            if (!projectOpt.isPresent()) {
                return new BocloudResult(false, "删除失败，未找到project");
            }

            Result result = this.doDelete(this.getEndpoint() + "/rest/vdc/v3.1/projects/" + projectOpt.get().getId(), null);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除成功");
        }
    }

    @Override
    public BocloudResult listProjects() {
        try {
            Result result = this.doGet("/auth/projects", null, FusionCloudApiName.keystone,
                    FusionCloudApiType.identity);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "查询失败");
            }
            List<FusionCloudProject> objects = JSONArray.parseArray(object.getString("domains"), FusionCloudProject.class);
            if (null == objects) {
                return new BocloudResult(false, "查询失败");
            }
            return new BocloudResult(true, result.getMessage(), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult listDomains() {
        try {
            Result result = this.doGet("/auth/domains", null, FusionCloudApiName.keystone,
                    FusionCloudApiType.identity);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "查询失败");
            }
            List<FusionCloudDomain> objects = JSONArray.parseArray(object.getString("domains"), FusionCloudDomain.class);
            if (null == objects) {
                return new BocloudResult(false, "查询失败");
            }
            return new BocloudResult(true, objects, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult listUsers() {
        try {
            Result result = this.doGet("/users", null, FusionCloudApiName.keystone,
                    FusionCloudApiType.identity);

               if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, result.getMessage(), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }
}
