package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.cmp.provider.compute.*;
import com.bocloud.cmp.provider.network.*;
import com.bocloud.cmp.provider.security.FusionCloudKeypairProvider;
import com.bocloud.cmp.provider.security.KeypairProvider;
import com.bocloud.cmp.provider.storage.FusionCloudVolumeProvider;
import com.bocloud.cmp.provider.storage.VolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ResultTools;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 华为云平台数据组装类
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年9月25日
 */
@Provider("bocloud.cmp.fusioncloud")
public class FusionCloudLocationProvider extends FusionCloudProvider implements LocationProvider {

    private static final Logger logger = LoggerFactory.getLogger(FusionCloudLocationProvider.class);
    private ZoneProvider regionProvider = null;
    private FlavorProvider flavorProvider = null;
    private ImageProvider imageProvider = null;
    private KeypairProvider keypairProvider = null;
    private NetworkProvider networkProvider = null;
    private PortProvider portProvider = null;
    private RouterProvider routerProvider = null;
    private SecurityGroupProvider secgProvider = null;
    private ServerProvider serverProvider = null;
    private SnapshotProvider snapshotProvider = null;
    private SubnetProvider subnetProvider = null;
    private VolumeProvider volumeProvider = null;
    private FloatingIPProvider floatingIPProvider = null;
    private Butler butler;

    /**
     * @param butler
     */
    public FusionCloudLocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;

        regionProvider = new FusionCloudRegionProvider(butler);
        flavorProvider = new FusionCloudFlavorProvider(butler);
        imageProvider = new FusionCloudImageProvider(butler);
        keypairProvider = new FusionCloudKeypairProvider(butler);
        networkProvider = new FusionCloudNetworkProvider(butler);
        portProvider = new FusionCloudPortProvider(butler);
        routerProvider = new FusionCloudRouterProvider(butler);
        secgProvider = new FusionCloudSecurityGroupProvider(butler);
        serverProvider = new FusionCloudServerProvider(butler);
        snapshotProvider = new FusionCloudSnapshotProvider(butler);
        subnetProvider = new FusionCloudSubnetProvider(butler);
        volumeProvider = new FusionCloudVolumeProvider(butler);
        floatingIPProvider = new FusionCloudFloatingIPProvider(butler);
    }

    @Override
    public BocloudResult listRegions() {
        return ResultTools.noImplementedResult();
    }

    @Override
    public BocloudResult sync() {
        FusionCloudSyncModel model = new FusionCloudSyncModel();

        logger.info("Start assembling region data...");
        BocloudResult regionResult = regionProvider.list();
        List<RegionModel> regions;
        if (regionResult.isSuccess()) {
            regions = JSONArray.parseArray(JSONArray.toJSONString(regionResult.getData()), RegionModel.class);
        } else {
            return regionResult;
        }
        model.setRegions(regions);
        logger.info("End assembling region data...");

        logger.info("Start assembling flavor data...");
        BocloudResult flavorResult = flavorProvider.sync();
        List<FlavorModel> flavors;
        if (flavorResult.isSuccess()) {
            flavors = JSONArray.parseArray(JSONArray.toJSONString(flavorResult.getData()), FlavorModel.class);
        } else {
            return flavorResult;
        }
        model.setFlavorModels(flavors);
        logger.info("End assembling flavor data...");

        logger.info("Start assembling image data...");
        BocloudResult imageResult = imageProvider.list();
        List<ImageModel> images;
        if (imageResult.isSuccess()) {
            images = JSONArray.parseArray(JSONArray.toJSONString(imageResult.getData()), ImageModel.class);
        } else {
            return imageResult;
        }
        model.setImageModels(images);
        logger.info("End assembling image data...");

        logger.info("Start assembling keypair data...");
        BocloudResult keypairResult = keypairProvider.list();
        List<KeypairModel> keypairs;
        if (keypairResult.isSuccess()) {
            keypairs = JSONArray.parseArray(JSONArray.toJSONString(keypairResult.getData()), KeypairModel.class);
        } else {
            return keypairResult;
        }
        model.setKeypairModels(keypairs);
        logger.info("End assembling keypair data...");

        logger.info("Start assembling port data...");
        BocloudResult portResult = portProvider.sync();
        List<PortModel> portModels;
        if (portResult.isSuccess()) {
            portModels = JSONArray.parseArray(JSONArray.toJSONString(portResult.getData()), PortModel.class);
        } else {
            return portResult;
        }
        model.setPortModels(portModels);
        logger.info("End assembling port data...");

        logger.info("Start assembling Network data...");
        BocloudResult networkResult = networkProvider.list();
        List<NetworkModel> networkModels;
        if (networkResult.isSuccess()) {
            networkModels = JSONArray.parseArray(JSONArray.toJSONString(networkResult.getData()), NetworkModel.class);
            for (NetworkModel networkModel : networkModels) {
                List<SubnetModel> subnetModels = networkModel.getSubnets();
                List<SubnetModel> newSubModels = new ArrayList<>();
                for (SubnetModel subnetModel : subnetModels) {
                    BocloudResult subnetResult = subnetProvider.detail(subnetModel.getSubnetUuid());
                    if (subnetResult.isSuccess()) {
                        newSubModels.add((SubnetModel) subnetResult.getData());
                    }
                }
                networkModel.setSubnets(newSubModels);
            }
        } else {
            return networkResult;
        }
        model.setNetworkModels(networkModels);
        logger.info("End assembling port data...");

        logger.info("Start assembling router data...");
        BocloudResult routerResult = routerProvider.list();
        List<RouterModel> routerModels;
        if (routerResult.isSuccess()) {
            routerModels = JSONArray.parseArray(JSONArray.toJSONString(routerResult.getData()), RouterModel.class);
        } else {
            return routerResult;
        }
        model.setRouterModels(routerModels);
        logger.info("End assembling router data...");

        logger.info("Start assembling security group data...");
        BocloudResult secgResult = secgProvider.list();
        List<SecurityGroupModel> secgModels;
        if (secgResult.isSuccess()) {
            secgModels = JSONArray.parseArray(JSONArray.toJSONString(secgResult.getData()), SecurityGroupModel.class);
        } else {
            return secgResult;
        }
        model.setGroupModels(secgModels);
        logger.info("End assembling security group data...");

        logger.info("Start assembling server data...");
        BocloudResult serverResult = serverProvider.list();
        List<ServerModel> serverModels;
        if (serverResult.isSuccess()) {
            serverModels = JSONArray.parseArray(JSONArray.toJSONString(serverResult.getData()), ServerModel.class);
        }  else {
            return serverResult;
        }
        model.setServerModels(serverModels);
        logger.info("End assembling server data...");

        logger.info("Start assembling volume type data...");
        BocloudResult volumeTypeResult = volumeProvider.listVolumeTypes();
        List<VolumeTypeModel> volumeTypeModels;
        if (volumeTypeResult.isSuccess()) {
            volumeTypeModels = JSONArray.parseArray(JSONArray.toJSONString(volumeTypeResult.getData()), VolumeTypeModel.class);
        } else {
            return volumeTypeResult;
        }
        model.setVolumeTypeModels(volumeTypeModels);
        logger.info("End assembling volume type data...");

        logger.info("Start assembling volume data...");
        BocloudResult volumeResult = volumeProvider.list();
        List<VolumeModel> volumeModels;
        if (volumeResult.isSuccess()) {
            volumeModels = JSONArray.parseArray(JSONArray.toJSONString(volumeResult.getData()), VolumeModel.class);
        } else {
            return volumeResult;
        }
        model.setVolumeModels(volumeModels);
        logger.info("End assembling volume data...");

        logger.info("Start assembling snapshot data...");
        BocloudResult snapResult = snapshotProvider.list();
        List<SnapshotModel> snapshotModels;
        if (snapResult.isSuccess()) {
            snapshotModels = JSONArray.parseArray(JSONArray.toJSONString(snapResult.getData()), SnapshotModel.class);
        } else {
            return snapResult;
        }
        model.setSnapshotModels(snapshotModels);
        logger.info("End assembling snapshot data...");

        logger.info("Start assembling floatingIp data...");
        BocloudResult floatIpResult = floatingIPProvider.list();
        List<FloatingIpModel> floatingIpModels;
        if (floatIpResult.isSuccess()) {
            floatingIpModels = JSONArray.parseArray(JSONArray.toJSONString(floatIpResult.getData()), FloatingIpModel.class);
        } else {
            return floatIpResult;
        }
        model.setFloatingIpModels(floatingIpModels);
        logger.info("End assembling floatingIp data...");

        return new BocloudResult(true, model, "同步成功");
    }

    @Override
    public BocloudResult listZones(String region) {
        return ResultTools.noImplementedResult();
    }

    @Override
    public BocloudResult syncPortByNetworkUuid(String networkUuid) {
        FusionCloudSyncModel model = new FusionCloudSyncModel();
        BocloudResult portResult = portProvider.sync();
        List<PortModel> portModels = new ArrayList<PortModel>();
        if (portResult.isSuccess()) {
            portModels = JSONArray.parseArray(JSONArray.toJSONString(portResult.getData()), PortModel.class);
        }
        List<PortModel> result = new ArrayList<PortModel>();
        for (PortModel portModel : portModels) {
            if (portModel.getNetworkUuid().contains(networkUuid)) {
                result.add(portModel);
            }
        }
        model.setPortModels(result);
        return new BocloudResult(true, model, "success");
    }

    @Override
    public BocloudResult getHostModel(HostModel host) {
        // TODO Auto-generated method stub
        return null;
    }

}
