package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.fusioncloudconvertor.ImageConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.google.common.base.Objects;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

@Provider("bocloud.cmp.fusioncloud")
public class FusionCloudImageProvider extends FusionCloudProvider implements ImageProvider {
    private static final Logger logger = LoggerFactory.getLogger(FusionCloudImageProvider.class);
    private BeanConvertor<ImageModel, JSONObject> convertor = new ImageConvertor();

    public FusionCloudImageProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            Result result = doGet("/v2/images", null, FusionCloudApiName.ims, FusionCloudApiType.image);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "查询失败");
            }
            List<JSONObject> objects = JSONArray.parseArray(object.getString("images"), JSONObject.class);
            if (null == objects) {
                return new BocloudResult(false, "查询失败");
            }
            return new BocloudResult(true, this.convertor.convertList(objects), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message: ", e);
        }
        return new BocloudResult(false, "查询失败");
    }

    @Override
    public BocloudResult detail(String imageId) {
        try {
            Result result = doGet("/v2/images/" + imageId, null, FusionCloudApiName.ims, FusionCloudApiType.image);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "查询失败");
            }
            return new BocloudResult(true, this.convertor.convertModel(object), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message: ", e);
        }
        return new BocloudResult(false, "查询失败");
    }

    @Override
    public BocloudResult remove(String imageId) {
        try {
            BocloudResult bocloudResult = detail(imageId);
            if (null == bocloudResult.getData()) {
                return new BocloudResult(true, "删除失败");
            }
            Result result = doDelete("/v2/images/" + imageId, null, FusionCloudApiName.ims, FusionCloudApiType.image);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message: ", e);
        }
        return new BocloudResult(false, "删除失败");
    }

    @Override
    public BocloudResult modify(ImageModel newImage) {
        try {
            List<JSONObject> params = new ArrayList<>();
            JSONObject nameObj = new JSONObject();
            nameObj.put("op", "replace"); // 所需进行的更新操作的类型：替换、添加、删除。取值范围：replace、add、remove
            nameObj.put("path", "/name"); // 所要操作的属性名称
            nameObj.put("value", newImage.getName()); // 所需更新/添加属性的值,若是删除则不需要
            params.add(nameObj);
            if (null != newImage.getRemark()) {
                BocloudResult getResult = this.detail(newImage.getImageUuid());
                if (getResult.isFailed()) {
                    return getResult;
                }
                ImageModel oldImage = (ImageModel) getResult.getData();

                if (!Objects.equal(oldImage.getRemark(), newImage.getRemark())) {
                    JSONObject descObj = new JSONObject();
                    descObj.put("path", "/__description"); // 所要操作的属性名称
                    if (StringUtils.isEmpty(oldImage.getRemark())) {
                        descObj.put("op", "add"); // 新增操作
                    } else if (StringUtils.isEmpty(newImage.getRemark())) {
                        descObj.put("op", "remove"); // 删除操作
                    } else {
                        descObj.put("op", "replace"); // 修改操作
                    }
                    descObj.put("value", newImage.getRemark());
                    params.add(descObj);
                }
            }
            Result result = doPatch("/v2/cloudimages/" + newImage.getImageUuid(), JSONArray.toJSONString(params),
                    FusionCloudApiName.ims, FusionCloudApiType.image);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject imageObj = (JSONObject) JSONObject.parseObject(result.getMessage(), JSONObject.class);
            return new BocloudResult(true, this.convertor.convertModel(imageObj), "修改成功");
        } catch (Exception e) {
            logger.error("Get error message: ", e);
        }
        return new BocloudResult(false, "修改失败");
    }

    @Override
    public BocloudResult create(ImageModel imageModel) {
        if (imageModel.getImageSource().equals("makeImage")) {
            return createImage(imageModel);
        } else if (imageModel.getImageSource().equals("uploadImage")){
            return uploadImage(imageModel);
        } else {
            return new BocloudResult(false, "创建镜像失败，暂不支持其他镜像创建方法");
        }
    }

    private BocloudResult createImage(ImageModel imageModel) {
        try {
            JSONObject params = new JSONObject();
            params.put("name", imageModel.getName());
            params.put("description", imageModel.getRemark());
            params.put("instance_id", imageModel.getInstanceId());
            Result result = doPost("/v2/cloudimages/action", params, FusionCloudApiName.ims, FusionCloudApiType.image);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            String jobId = object.getString("job_id");
            ImageModel model = waitTaskResult(jobId);
            if (null == model) {
                return new BocloudResult(false, "创建失败");
            }
            return new BocloudResult(true, model, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message: ", e);
        }
        return new BocloudResult(false, "创建失败");
    }

    // IMS 创建镜像-创建镜像元数据
    private BocloudResult uploadImage(ImageModel model) {
        try {
            JSONObject params = new JSONObject();
            params.put("_os_version", model.getOsVersion());
            params.put("visibility", "private"); // 其他租户是否可见。 默认取值为private。创建镜像元数据时，visibility取值只能为private。
            params.put("name", model.getName());
            params.put("protected", false); // 镜像是否被保护，保护后的镜像不可删除。默认取值为false。
            params.put("container_format", "bare"); // 容器格式。 默认取值为bare。
            params.put("disk_format", model.getDiskFormat()); // 镜像文件格式。目前支持vhd、zvhd、raw、qcow2。默认取值为vhd。
            params.put("tags", new ArrayList<>()); // 镜像标签列表。长度为1～255位。默认为空
            params.put("min_ram", model.getMinRam()); // 镜像运行需要的最小内存，单位为MB。参数取值依据云服务器的规格限制。默认取值为0。
            params.put("min_disk", model.getMinDisk());// 镜像运行需要的最小磁盘，单位为GB 。取值为40～1024GB。必须大于镜像系统盘容量，否则创建云服务器可能失败。
            params.put("__description", model.getRemark());
            Result result = doPost("/v2/images", params, FusionCloudApiName.ims, FusionCloudApiType.image);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "Failed to upload image ,Please check the network connection!");
            }
            ImageModel imageModel = this.convertor.convertModel(object);
            BocloudResult uploadResult = upload(imageModel.getImageUuid(), model.getFilePath());
            if (uploadResult.isFailed()) {
                return new BocloudResult(false, uploadResult.getMessage());
            }

            return new BocloudResult(true, JSONObject.parseObject(JSONObject.toJSONString(uploadResult.getData()), ImageModel.class), "upload image success !");
        } catch (Exception e) {
            logger.error("upload image error message:", e.getMessage());
        }
        return new BocloudResult(false, "upload image error!");
    }

    private ImageModel waitTaskResult(String jobId) {
        ImageModel model = new ImageModel();
        Boolean flag = Boolean.valueOf(true);
        Long startTime = Long.valueOf(System.currentTimeMillis());
        Result result = null;
        JSONObject jobResult = null;
        while (flag.booleanValue()) {
            try {
                Thread.sleep(8000L);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            result = waitImageTask(jobId);
            if (result.isFailed()) {
                flag = Boolean.valueOf(false);
                break;
            }
            jobResult = JSONObject.parseObject(result.getMessage());
            logger.info("jobResult : " + JSON.toJSONString(jobResult));
            if ("FAIL".equalsIgnoreCase(jobResult.getString("status"))) {
                flag = Boolean.valueOf(false);
                break;
            }
            if ("SUCCESS".equalsIgnoreCase(jobResult.getString("status"))) {
                JSONObject imageObj = JSONObject.parseObject(jobResult.getString("entities"));
                String imageId = imageObj.getString("image_id");
                BocloudResult detailResult = detail(imageId);
                if (detailResult.isSuccess()) {
                    model = (ImageModel) JSONObject.parseObject(JSONObject.toJSONString(detailResult.getData()),
                            ImageModel.class);
                }
                flag = Boolean.valueOf(false);
                break;
            }
            Long endTime = Long.valueOf(System.currentTimeMillis());
            if (endTime.longValue() - startTime.longValue() > 1800000L) {
                flag = Boolean.valueOf(false);
                break;
            }
        }
        return model;
    }

    @Override
    public BocloudResult upload(String imageId, String path) {
        try {
            File file = new File(path);
            if (!file.exists()) {
                return new BocloudResult(false, "需上传的镜像路径未找到");
            }
            Result result = doPutFile("/v2/images/" + imageId + "/file", "key", file,
                    FusionCloudApiName.ims, FusionCloudApiType.image);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            //查询镜像状态，状态为active则为上传成功，超过10分钟为失败
            BocloudResult detailResult = detail(imageId);
            ImageModel imageModel = (ImageModel) detailResult.getData();
            long start = System.currentTimeMillis();
            while (imageModel != null && imageModel.getStatus().equals("active") && System.currentTimeMillis() - start < 1000 * 60 * 10) {
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    logger.info("查询镜像状态异常", e);
                    break;
                }
                BocloudResult detail = detail(imageModel.getImageUuid());
                imageModel = this.convertor.convertModel(JSONObject.parseObject(detail.getMessage(), JSONObject.class));
            }
            if (System.currentTimeMillis() - start >= 10 * 60 * 1000) {
                return new BocloudResult(false, "镜像上传失败,镜像上传10分钟后，该镜像状态仍未是active状态");
            }
            return new BocloudResult(true, imageModel, "镜像上传成功");
        } catch (Exception e) {
            logger.error("Get error message: ", e);
        }
        return new BocloudResult(false, "镜像上传失败");
    }
    @Override
    public BocloudResult shareImage(ImageModel imageModel) {
        return null;
    }

    @Override
    public BocloudResult syncImages(ImageModel imageModel) {
        return null;
    }
}
