package com.bocloud.cmp.provider.compute;

import ch.ethz.ssh2.crypto.Base64;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.fusioncloudconvertor.PortConvertor;
import com.bocloud.cmp.fusioncloudconvertor.ServerConvertor;
import com.bocloud.cmp.fusioncloudconvertor.VolumeConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkBean;
import com.bocloud.cmp.model.OSMigrateModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.OSCategory;
import com.bocloud.common.http.CommonResultBuilder;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.util.Asserts;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

/**
 * 华为私有云云主机操作driver
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年9月17日
 */
@Provider("bocloud.cmp.fusioncloud")
public class FusionCloudServerProvider extends FusionCloudProvider implements ServerProvider {

	private static final Logger logger = LoggerFactory.getLogger(FusionCloudServerProvider.class);
	private BeanConvertor<ServerModel, JSONObject> convertor = new ServerConvertor();
	private BeanConvertor<PortModel, JSONObject> portConvertor = new PortConvertor();
	private BeanConvertor<VolumeModel, JSONObject> volumeConvertor = new VolumeConvertor();
	private Butler butler;

	/**
	 * @param butler
	 */
	public FusionCloudServerProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult list() {
		try {
			Result result = this.doGet("/servers", null, FusionCloudApiName.ecs, FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "云主机列表为空");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("servers"), JSONObject.class);
			if (null == objects) {
				return new BocloudResult(false, "云主机列表为空");
			}
			List<ServerModel> models = new ArrayList<>();
			for (JSONObject obj : objects) {
				BocloudResult detailResult = this.detail(obj.getString("id"));
				if (detailResult.isSuccess()) {
					ServerModel vm = JSONObject.parseObject(JSONObject.toJSONString(detailResult.getData()),
							ServerModel.class);
					models.add(vm);
				}
			}
			return new BocloudResult(true, models, "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult create(ServerModel server) {
		try {
			JSONObject vmBuild = new JSONObject();
			vmBuild.put("flavorRef", server.getFlavorUuid());
			vmBuild.put("name", server.getName());
			List<JSONObject> personality = new ArrayList<>();
			JSONObject personal = new JSONObject();
			personal.put("path", "/etc/motd");
			personal.put("contents",
					new String(Base64.encode("Welcome to the new VM! Restricted access only".getBytes())));
			personality.add(personal);
			vmBuild.put("personality", personality);
			String osCategory = server.getOsCategory();
			if (StringUtils.isEmpty(osCategory)) {
				osCategory = OSCategory.LINUX.name();
			}
			// 使用支持Cloud-init或Cloudbase-init功能的镜像创建云服务器时
			// 对于Linux弹性云服务器，如果需要注入密码，只能使用user_data进行注入
			// 对于Windows弹性云服务器，如果需要注入密码，只能通过元数据adminPass进行注入
			if (OSCategory.LINUX.name().equalsIgnoreCase(osCategory)) {
				StringBuffer buffer = new StringBuffer();
				buffer.append("#cloud-config\n");
				buffer.append("hostname: " + server.getName() + "\n");
				String password = server.getPassword();
				if (!StringUtils.isEmpty(password)) {
					buffer.append("password: " + server.getPassword() + "\n");
				}
				buffer.append("chpasswd: { expire: False }\n");
				buffer.append("ssh_pwauth: True");
				vmBuild.put("user_data", new String(Base64.encode(buffer.toString().getBytes())));
			} else {
				vmBuild.put("adminPass", server.getPassword());
			}
			String sourceValidUuid = server.getSourceValidUuid();
			String sourceSnapUuid = server.getSourceSnapUuid();
			List<JSONObject> blockDevices = new ArrayList<>();
			if (!StringUtils.isEmpty(sourceValidUuid)) {
				JSONObject sourceVol = new JSONObject();
				sourceVol.put("source_type", "volume");
				sourceVol.put("uuid", sourceValidUuid);
				sourceVol.put("delete_on_termination", "true");
				sourceVol.put("boot_index", "0");
				blockDevices.add(sourceVol);
				vmBuild.put("block_device_mapping_v2", blockDevices);
			} else if (!StringUtils.isEmpty(sourceSnapUuid)) {
				JSONObject sourceSnap = new JSONObject();
				sourceSnap.put("source_type", "snapshot");
				sourceSnap.put("uuid", sourceSnapUuid);
				sourceSnap.put("delete_on_termination", "true");
				sourceSnap.put("boot_index", "0");
				blockDevices.add(sourceSnap);
				vmBuild.put("block_device_mapping_v2", blockDevices);
			} else if (null != server.getBootFromVolume() && server.getBootFromVolume()) {
				// 从image中创建卷
				String volumeName = Optional.ofNullable(server.getName()).map(name -> name + "_boot")
						.orElse(UUID.randomUUID().toString().replace("-", ""));
				JSONObject vlBuilder = new JSONObject();
				vlBuilder.put("size", server.getRootSize());
				vlBuilder.put("availability_zone", server.getZone());
				vlBuilder.put("name", volumeName);
				vlBuilder.put("multiattach", false);
				vlBuilder.put("imageRef", server.getImageUuid());
				JSONObject params = new JSONObject();
				params.put("volume", vlBuilder);
				Result result = this.doPost("/volumes", params, FusionCloudApiName.evsv2, FusionCloudApiType.volumev2);
				if (result.isFailed()) {
					return new BocloudResult(false, result.getMessage());
				}
				JSONObject volumeObject = JSONObject.parseObject(result.getMessage()).getJSONObject("volume");
				VolumeModel volumeModel = volumeConvertor.convertModel(volumeObject);
				// 查询卷状态直到为available
				long start = System.currentTimeMillis();
				while (System.currentTimeMillis() - start < 15 * 60 * 1000
						&& !volumeModel.getStatus().equalsIgnoreCase("AVAILABLE")) {
					logger.debug("The status of volume[{}] is :{}", volumeModel.getName(), volumeModel.getStatus());
					try {
						Thread.sleep(5000);
					} catch (InterruptedException e) {
						Thread.currentThread().interrupt();
					}
					result = this.doGet("/volumes/" + volumeModel.getVolumeUuid(), params, FusionCloudApiName.evsv2,
							FusionCloudApiType.volumev2);
					if (result.isFailed()) {
						return new BocloudResult(false, result.getMessage());
					}
					volumeObject = JSONObject.parseObject(result.getMessage()).getJSONObject("volume");
					volumeModel = volumeConvertor.convertModel(volumeObject);
				}

				JSONObject source = new JSONObject();
				source.put("uuid", volumeModel.getVolumeUuid());
				source.put("source_type", "volume");
				source.put("destination_type", "volume");
				source.put("delete_on_termination", true);
				source.put("boot_index", 0);
				blockDevices.add(source);
				vmBuild.put("block_device_mapping_v2", blockDevices);
			} else {
				vmBuild.put("imageRef", server.getImageUuid());
			}
			// 设置密钥
			if (!StringUtils.isEmpty(server.getKeypairName())) {
				vmBuild.put("key_name", server.getKeypairName());
			}
			// 设置网卡配置
			List<NetworkBean> networkBeans = server.getNetworkBeans();
			List<JSONObject> networks = new ArrayList<>();
			if (ListTool.isEmpty(networkBeans)) {
				return new BocloudResult(false, "至少需要一块网卡");
			}
			for (NetworkBean networkBean : networkBeans) {
				JSONObject network = new JSONObject();
				if (!StringUtils.isEmpty(networkBean.getPortUuid())) {
					network.put("port", networkBean.getPortUuid());
				} else {
					network.put("uuid", networkBean.getNetworkUuid());
					if (!StringUtils.isEmpty(networkBean.getFixIp())) {
						network.put("fixed_ip", networkBean.getFixIp());
					}
				}
				networks.add(network);
			}
			vmBuild.put("networks", networks);
			List<JSONObject> secgList = new ArrayList<>();
			List<String> groups = server.getSecurityGroups();
			if (!ListTool.isEmpty(groups)) {
				for (String group : groups) {
					JSONObject secg = new JSONObject();
					secg.put("name", group);
					secgList.add(secg);
				}
			}
			// 设置安全组
			vmBuild.put("security_groups", secgList);
			// 可用域
			vmBuild.put("availability_zone", server.getZone());
			JSONObject params = new JSONObject();
			params.put("server", vmBuild);
			Result result = this.doPost("/servers", params, FusionCloudApiName.ecs, FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject createdResult = JSONObject.parseObject(result.getMessage());
			JSONObject vmObj = JSONObject.parseObject(createdResult.getString("server"));
			if (null == vmObj) {
				return new BocloudResult(false, "创建失败");
			}
			String serverId = vmObj.getString("id");
			ServerModel model = waitForTask(serverId);
			if (null == model) {
				this.remove(serverId, null);
				return new BocloudResult(false, "云主机创建失败");
			}
			// 查询云主机端口
			model.setPorts(listPortByInstanceId(serverId));
			return new BocloudResult(true, model, "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "创建失败");
		}
	}

	private List<PortModel> listPortByInstanceId(String instanceId) {
		// 查询云主机端口
		List<PortModel> portModels = new ArrayList<>();
		Result portResult = this.doGet("/v2.0/ports?device_id=" + instanceId, null, FusionCloudApiName.vpc,
				FusionCloudApiType.network);
		if (portResult.isSuccess()) {
			JSONObject object = JSONObject.parseObject(portResult.getMessage());
			List<JSONObject> ports = JSONArray.parseArray(object.getString("ports"), JSONObject.class);
			for (JSONObject port : ports) {
				portModels.add(portConvertor.convertModel(port));
			}
		}
		return portModels;
	}

	private ServerModel waitForTask(String serverId) {
		ServerModel model = null;
		BocloudResult result = this.detail(serverId);
		if (result.isFailed()) {
			return model;
		}
		model = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), ServerModel.class);
		if (null == model) {
			return model;
		}
		Boolean flag = true;
		Long startTime = System.currentTimeMillis();
		while (flag) {
			try {
				Thread.sleep(7000);
			} catch (InterruptedException e) {
				logger.error("Get error message:", e);
				Thread.currentThread().interrupt();
			}
			result = this.detail(serverId);
			if (result.isFailed()) {
				flag = false;
				break;
			}
			model = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), ServerModel.class);
			if (null == model) {
				flag = false;
				break;
			}
			if ((!BocloudStatus.Server.BUILDING.name().equalsIgnoreCase(model.getStatus()))) {
				flag = false;
				break;
			}
			Long endTime = System.currentTimeMillis();
			// 等待三分钟
			if (endTime - startTime > 15 * 60 * 1000) {
				flag = false;
				break;
			}
		}
		return model;
	}

	@Override
	public BocloudResult migrate(ServerModel server) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult migrate(OSMigrateModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult snapshot(SnapshotModel snapshotModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult modify(ServerModel server) {
		try {
			JSONObject objectParams = new JSONObject();
			objectParams.put("name", server.getName());
			JSONObject params = new JSONObject();
			params.put("server", objectParams);
			Result result = this.doPut("/servers/" + server.getInstanceId(), params, FusionCloudApiName.ecs,
					FusionCloudApiType.compute);
			Asserts.check(result.isSuccess(), result.getMessage());
			JSONObject updateResult = JSONObject.parseObject(result.getMessage());
			JSONObject vmObj = JSONObject.parseObject(updateResult.getString("server"));
			if (null == vmObj) {
				return new BocloudResult(false, "修改失败");
			}
			String serverId = vmObj.getString("id");
			return new BocloudResult(true, this.detail(serverId).getData(), "修改成功");
		} catch (Exception e) {
			logger.error("Get error message：", e.getMessage());
			return new BocloudResult(false, "修改失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult start(String instanceId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject type = new JSONObject();
			params.put("os-start", type);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, FusionCloudApiName.ecs,
					FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "启动成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "启动失败");
		}
	}

	@Override
	public BocloudResult stop(String instanceId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject type = new JSONObject();
			type.put("type", "HARD");
			params.put("os-stop", type);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, FusionCloudApiName.ecs,
					FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "关闭成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "关闭失败");
		}
	}

	@Override
	public BocloudResult reboot(String instanceId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject type = new JSONObject();
			type.put("type", "HARD");
			params.put("reboot", type);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, FusionCloudApiName.ecs,
					FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "重启成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "重启失败");
		}
	}

	@Override
	public BocloudResult softReboot(String instanceId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject type = new JSONObject();
			type.put("type", "SOFT");
			params.put("reboot", type);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, FusionCloudApiName.ecs,
					FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "软重启成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "软重启失败");
		}
	}

	@Override
	public BocloudResult pause(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult unPause(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult suspend(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult resume(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult remove(String instanceId, String name) {
		try {
			Result result = this.doDelete("/servers/" + instanceId, null, FusionCloudApiName.ecs,
					FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "删除失败");
		}
	}

	@Override
	public BocloudResult detail(String instanceId) {
		try {
			Result result = this.doGet("/servers/" + instanceId, null, FusionCloudApiName.ecs,
					FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			JSONObject vmObj = object.getJSONObject("server");
			return new BocloudResult(true, convertor.convertModel(vmObj), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult vncConsole(String vncType, String serverId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject output = new JSONObject();
			output.put("length", "50");
			params.put("os-getConsoleOutput", output);
			Result result = this.doPost("/servers/" + serverId + "/action", params, FusionCloudApiName.ecs,
					FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, result.getMessage(), "获取控制台成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "获取控制台失败");
		}
	}

	@Override
	public BocloudResult joinSGroup(String instanceId, String groupId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject name = new JSONObject();
			name.put("name", groupId);
			params.put("addSecurityGroup", name);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, FusionCloudApiName.ecs,
					FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "安全组添加成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "安全组添加失败");
		}
	}

	@Override
	public BocloudResult leaveSGroup(String instanceId, String groupId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject name = new JSONObject();
			name.put("name", groupId);
			params.put("removeSecurityGroup", name);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, FusionCloudApiName.ecs,
					FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "安全组移除成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "安全组移除失败");
		}
	}

	@Override
	public BocloudResult changePassword(String instanceId, String password) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult attachNetwork(ServerModel server) {
		try {
			JSONObject params = new JSONObject();
			JSONObject attach = new JSONObject();
			NetworkBean networkBean = server.getNetworkBeans().get(0);
			if (!StringUtils.isEmpty(networkBean.getPortUuid())) {
				attach.put("port_id", networkBean.getPortUuid());
			} else {
				attach.put("net_id", networkBean.getNetworkUuid());
				if (!StringUtils.isEmpty(networkBean.getFixIp())) {
					List<JSONObject> fixedIps = new ArrayList<JSONObject>();
					com.alibaba.fastjson.JSONObject ipOjbect = new com.alibaba.fastjson.JSONObject();
					ipOjbect.put("ip_address", networkBean.getFixIp());
					fixedIps.add(ipOjbect);
					attach.put("fixed_ips", fixedIps);
				}
			}
			params.put("interfaceAttachment", attach);
			logger.info("params : " + params);
			Result result = this.doPost("/servers/" + server.getInstanceId() + "/os-interface", params,
					FusionCloudApiName.ecs, FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			logger.info("result : " + result);
			// 查询云主机端口
			server.setPorts(listPortByInstanceId(server.getInstanceId()));
			return new BocloudResult(true, server, "网卡添加成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "网卡添加失败");
		}
	}

	@Override
	public BocloudResult detachNetwork(String instanceId, String portUuid) {
		try {
			Result result = this.doDelete("/servers/" + instanceId + "/os-interface/" + portUuid, null,
					FusionCloudApiName.ecs, FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			ServerModel model = new ServerModel();
			model.setPorts(listPortByInstanceId(instanceId));
			return new BocloudResult(true, model, "网卡卸载成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "网卡卸载失败");
		}
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject resize = new JSONObject();
			resize.put("flavorRef", flavorId);
			params.put("resize", resize);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, FusionCloudApiName.ecs,
					FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			ServerModel model = waitForResizeTask(instanceId, "RESIZING");
			return new BocloudResult(true, model, "修改配置成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "修改配置失败");
		}
	}

	@Override
	public BocloudResult confirmResize(String instanceId) {
		try {
			String jsonStr = "{\"confirmResize\":null}";
			net.sf.json.JSONObject json = net.sf.json.JSONObject.fromObject(jsonStr);
			@SuppressWarnings("unchecked")
			Result result = this.doPost("/servers/" + instanceId + "/action", json, FusionCloudApiName.ecs,
					FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			ServerModel model = waitForResizeTask(instanceId, "STOPPED");
			return new BocloudResult(true, model, "确认配置成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "确认配置失败");
		}
	}

	@Override
	public BocloudResult revertResize(String instanceId) {
		try {
			String jsonStr = "{\"revertResize\":null}";
			net.sf.json.JSONObject json = net.sf.json.JSONObject.fromObject(jsonStr);
			@SuppressWarnings("unchecked")
			Result result = this.doPost("/servers/" + instanceId + "/action", json, FusionCloudApiName.ecs,
					FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			ServerModel model = waitForResizeTask(instanceId, "STOPPED");
			return new BocloudResult(true, model, "回退变更配置成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "回退变更配置失败");
		}
	}

	@Override
	public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean memHotEnable) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult getTicket(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult createInResPool(ServerModel server) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult mountToolsInstaller(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult unmountToolsInstaller(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult upgradeToolsTask(String instanceId, String options) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult getNetCard(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult getVmIps(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult markAsTemplate(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
		// TODO Auto-generated method stub
		return null;
	}

	private List<JSONObject> getCloudVm() {
		List<JSONObject> list = new ArrayList<>();
		String url = this.getUrl() + "/rest/tenant-resource/v1/tenant/resources/CLOUD_VM?limit=1000";
		JSONObject params = new JSONObject();
		Result result = this.getClient().get(buildMonitorHeader(), params, url);
		JSONObject vmObj = JSON.parseObject(result.getMessage(), JSONObject.class);
		String objList = vmObj.getString("objList");
		List<JSONObject> vmList = JSON.parseArray(objList, JSONObject.class);
		if (null != vmList && vmList.size() > 0) {
			for (JSONObject jsonObject : vmList) {
				String projectId = jsonObject.getString("projectId");
				if (null != projectId && projectId.equals(butler.getProjectId())) {
					list.add(jsonObject);
				}
			}
		}
		return list;
	}

	private String getObjTypeId() {
		String url = this.getUrl() + "/rest/performance/v1/mgr-svc/obj-types";
		JSONObject params = new JSONObject();
		Result result = this.getClient().get(buildMonitorHeader(), params, url);
		JSONObject serverTypes = JSON.parseObject(result.getMessage(), JSONObject.class);
		List<JSONObject> dataList = JSON.parseArray(serverTypes.getString("data"), JSONObject.class);
		String objTypeId = "";
		for (JSONObject jsonObject : dataList) {
			String resourceCategory = jsonObject.getString("resource_category");
			if ("CLOUD_VM".equalsIgnoreCase(resourceCategory)) {
				objTypeId = jsonObject.getString("obj_type_id");
				break;
			}
		}
		return objTypeId;
	}

	private List<String> getIndicators(String objTypeId) {
		// String indicatorIdsUrl = this.getUrl() +
		// "/rest/performance/v1/mgr-svc/obj-types/" + objTypeId +
		// "/indicators";
		// JSONObject indicatorIdsParams = new JSONObject();
		// Result indicatorIdsResult =
		// this.getClient().get(buildMonitorHeader(), indicatorIdsParams,
		// indicatorIdsUrl);
		// JSONObject parseObject =
		// JSON.parseObject(indicatorIdsResult.getMessage(), JSONObject.class);
		// JSONObject parseObject2 =
		// JSON.parseObject(parseObject.getString("data"), JSONObject.class);
		// JSONArray jsonArray = parseObject2.getJSONArray("indicator_ids");
		// return jsonArray;
		List<String> list = new ArrayList<>();
		list.add("562958543486979"); // 内存使用率
		list.add("562958543421441"); // CPU使用率
		return list;
	}

	@Override
	public BocloudResult getMetric(String type) { // type 前端参数传过来的
		try {
			String objTypeId = getObjTypeId();
			logger.info("objTypeId : " + objTypeId);
			List<JSONObject> cloudVm = getCloudVm();
			if (!"".equals(objTypeId)) {
				List<String> indicatorsList = getIndicators(objTypeId);
				List<String> vmResids = new ArrayList<>();
				for (JSONObject jsonObject : cloudVm) {
					vmResids.add(jsonObject.getString("resId"));
				}
				String historyDataUrl = this.getUrl() + "/rest/performance/v1/data-svc/history-data/action/query";
				JSONObject historyDataParams = new JSONObject();
				historyDataParams.put("obj_type_id", objTypeId);
				historyDataParams.put("indicator_ids", indicatorsList);
				historyDataParams.put("range", "LAST_5_MINUTE");
				historyDataParams.put("obj_ids", vmResids);
				HttpPost httpPost = new HttpPost(historyDataUrl);
				httpPost.addHeader("accept", "application/json");
				httpPost.addHeader("content-type", "application/json");
				httpPost.addHeader("X-Auth-Token", this.getMonitorToken());
				httpPost.addHeader("roaRand", this.getMonitorRoaRand());
				StringEntity httpEntity = new StringEntity(JSONObject.toJSON(historyDataParams).toString(), "UTF-8");
				httpPost.setEntity(httpEntity);
				CloseableHttpResponse httpResponse = this.getClient().getHttpClient().execute(httpPost);
				Result monitorResult = new CommonResultBuilder().build(httpResponse.getStatusLine().getStatusCode());
				JSONObject content = JSONObject
						.parseObject(IOUtils.toString(httpResponse.getEntity().getContent(), "utf-8"));
				// logger.info("content : " + content);
				String monitorData = content.getString("data");
				JSONObject dataObj = JSON.parseObject(monitorData, JSONObject.class);
				// logger.info("dataObj : " + dataObj);
				for (JSONObject vm : cloudVm) {
					String resId = vm.getString("resId");
					JSONObject monitorInfo = JSON.parseObject(dataObj.getString(resId), JSONObject.class);
					// logger.info("monitorInfo : " + monitorInfo);
					if (null != monitorInfo) {
						if ("CPU".equalsIgnoreCase(type)) {
							JSONObject cpuUsed = JSON.parseObject(monitorInfo.getString("562958543486979"),
									JSONObject.class);
							String maxCpuVal = cpuUsed.getString("max");
							logger.info("maxCpuVal : " + maxCpuVal);
							if (null != maxCpuVal && maxCpuVal.contains(":")) {
								String[] split = maxCpuVal.split(":");
								String max = split[1].replaceAll("\"", "").replaceAll("}", "");
								BigDecimal bg = new BigDecimal(max);
								double f1 = bg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
								vm.put("value", f1);
							}
						}
						if ("内存".equals(type)) {
							JSONObject memoryInfo = JSON.parseObject(monitorInfo.getString("562958543421441"),
									JSONObject.class);
							String maxMemoryVal = memoryInfo.getString("max");
							logger.info("maxMemoryVal : " + maxMemoryVal);
							if (null != maxMemoryVal && maxMemoryVal.contains(":")) {
								String[] split = maxMemoryVal.split(":");
								String max = split[1].replaceAll("\"", "").replaceAll("}", "");
								BigDecimal bg = new BigDecimal(max);
								double f1 = bg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
								vm.put("value", f1);
							}
						}
					}
					Double val = vm.getDouble("value");
					if (null == val) {
						vm.put("value", 0d);
					}
				}
				// logger.info("cloudVm : " + JSON.toJSONString(cloudVm));
			}
			return new BocloudResult(true, cloudVm, "查询成功");
		} catch (Exception e) {
			logger.error("查询Error : " + e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult vmrcConsole(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult migrate(H3cMigrateModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult list(String hostId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult create(H3cServerModel server) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult getJnlpXml(Long vmId, String address) {
		// TODO Auto-generated method stub
		return null;
	}

	private ServerModel waitForResizeTask(String serverId, String status) {
		ServerModel model = null;
		Boolean flag = Boolean.valueOf(true);
		Long startTime = Long.valueOf(System.currentTimeMillis());
		while (flag.booleanValue()) {
			try {
				Thread.sleep(5000L);
			} catch (InterruptedException e) {
				logger.error("Get error message:", e);
				Thread.currentThread().interrupt();
			}
			BocloudResult result = detail(serverId);
			if (result.isFailed()) {
				flag = Boolean.valueOf(false);
				break;
			}
			model = (ServerModel) com.alibaba.fastjson.JSONObject
					.parseObject(com.alibaba.fastjson.JSONObject.toJSONString(result.getData()), ServerModel.class);
			if (null == model) {
				flag = Boolean.valueOf(false);
				break;
			}
			if (status.equalsIgnoreCase(model.getStatus())) {
				flag = Boolean.valueOf(false);
				break;
			}
			Long endTime = Long.valueOf(System.currentTimeMillis());
			if (endTime.longValue() - startTime.longValue() > 480000L) {
				flag = Boolean.valueOf(false);
				model.setStatus("EXCEPTION");
				break;
			}
		}
		return model;
	}

	/**
	 * 调整网络上限
	 *
	 * @param instanceId
	 * @param internetMaxBandwidthOut
	 * @return
	 */
	@Override
	public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
		return null;
	}

	/**
	 * 重装系统
	 *
	 * @param serverModel
	 * @return
	 */
	@Override
	public BocloudResult resetInstance(ServerModel serverModel) {
		try {
			JSONObject params = new JSONObject();
			// 云服务器管理员帐户的初始登录密码。
			// 其中，Windows管理员帐户的用户名为Administrator。
			// 建议密码复杂度如下：长度为8-26位。密码至少必须包含大写字母、小写字母、数字和特殊字符（!@$%^-_=+[{}]:,./?）中的三种。
			// 说明：对于Windows弹性云服务器，密码不能包含用户名或用户名的逆序，不能包含用户名中超过两个连续字符的部分。
			// adminpass和keyname不能同时有值。
			// adminpass和keyname如果同时为空，则metadata中的user_data属性必须有值。
			params.put("adminpass", serverModel.getPassword());
			// 密钥名称。
			params.put("keyname", serverModel.getKeypairName());
			// 用户ID。 说明 如果使用秘钥方式重装操作系统，则该字段为必选字段。
			// params.put("userid", "");
			JSONObject serverParams = new JSONObject();
			serverParams.put("os-reinstall", params);
			Result result = this.doPost("/cloudservers/" + serverModel.getInstanceId() + "/reinstallos", serverParams,
					FusionCloudApiName.ecs, FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject createdResult = JSONObject.parseObject(result.getMessage());
			JSONObject vmObj = createdResult.getJSONObject("server");
			if (null == vmObj) {
				return new BocloudResult(false, "创建失败");
			}
			String imageId = vmObj.getJSONObject("image").getString("id");
			if (serverModel.getImageUuid().equalsIgnoreCase(imageId)) {
				return new BocloudResult(true, this.detail(serverModel.getInstanceId()),
						"server resetInstance success!");
			} else {
				return new BocloudResult(false, "server resetInstance fail!");
			}
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "server resetInstance error!");
		}
	}

	/**
	 * 切换系统
	 *
	 * @param serverModel
	 * @return
	 */
	@Override
	public BocloudResult changeInstance(ServerModel serverModel) {
		try {
			JSONObject params = new JSONObject();
			// 云服务器管理员帐户的初始登录密码。
			// 其中，Windows管理员帐户的用户名为Administrator。
			// 建议密码复杂度如下：长度为8-26位。密码至少必须包含大写字母、小写字母、数字和特殊字符（!@$%^-_=+[{}]:,./?）中的三种。
			// 说明：对于Windows弹性云服务器，密码不能包含用户名或用户名的逆序，不能包含用户名中超过两个连续字符的部分。
			// adminpass和keyname不能同时有值。
			// adminpass和keyname如果同时为空，则metadata中的user_data属性必须有值。
			params.put("adminpass", serverModel.getPassword());
			// 密钥名称。
			params.put("keyname", serverModel.getKeypairName());
			// 用户ID。 说明 如果使用秘钥方式重装操作系统，则该字段为必选字段。
			// params.put("userid", "");
			// 切换系统所使用的新镜像的uuid。
			params.put("imageid", serverModel.getImageUuid());
			JSONObject serverParams = new JSONObject();
			serverParams.put("os-change", params);
			Result result = this.doPost("/cloudservers/" + serverModel.getInstanceId() + "/changeos", serverParams,
					FusionCloudApiName.ecs, FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject createdResult = JSONObject.parseObject(result.getMessage());
			JSONObject vmObj = createdResult.getJSONObject("server");
			if (null == vmObj) {
				return new BocloudResult(false, "创建失败");
			}
			String imageId = vmObj.getJSONObject("image").getString("id");
			if (serverModel.getImageUuid().equalsIgnoreCase(imageId)) {
				return new BocloudResult(true, this.detail(serverModel.getInstanceId()),
						"server resetInstance success!");
			} else {
				return new BocloudResult(false, "server resetInstance fail!");
			}
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "server resetInstance error!");
		}
	}

	@Override
	public BocloudResult bindKeypair(ServerModel serverModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult unbindKeypair(ServerModel serverModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
		return null;
	}

	@Override
	public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
		// TODO Auto-generated method stub
		return null;
	}

}
