package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.fusioncloudconvertor.PortConvertor;
import com.bocloud.cmp.fusioncloudconvertor.RouterConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.model.RouterModel;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 华为私有云路由操作driver
 *
 * @author wangyu
 * @version 1.0
 * @complete gp
 * @since 2018年9月18日
 */
@Provider("bocloud.cmp.fusioncloud")
public class FusionCloudRouterProvider extends FusionCloudProvider implements RouterProvider {

    private static final Logger logger = LoggerFactory.getLogger(FusionCloudRouterProvider.class);
    private BeanConvertor<RouterModel, JSONObject> convertor = new RouterConvertor();

    /**
     * @param butler
     */
    public FusionCloudRouterProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(RouterModel model) {
        try {
            JSONObject params = new JSONObject();
            params.put("name", model.getName());
            params.put("admin_state_up", model.getAdminStateup());
            JSONObject network = new JSONObject();
            if (StringUtils.isNotEmpty(model.getNetworkUuid())) {
                network.put("network_id", model.getNetworkUuid());
                params.put("external_gateway_info", network);
            }
            JSONObject routerParam = new JSONObject();
            routerParam.put("router", params);
            Result result = this.doPost("/v2.0/routers", routerParam, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "创建失败");
            }
            return new BocloudResult(true, convertor.convertModel(object.getJSONObject("router")), "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");

        }
    }

    @Override
    public BocloudResult remove(String routerId) {
        try {
            BocloudResult bocloudResult = this.detail(routerId);
            if (null == bocloudResult.getData()) {
                return new BocloudResult(true, "删除成功");
            }
            Result result = this.doDelete("/v2.0/routers/" + routerId, null, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败");
        }

    }

    @Override
    public BocloudResult detail(String routerId) {
        try {
            Result result = this.doGet("/v2.0/routers/" + routerId, null, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "路由不存在");
            }
            return new BocloudResult(true, convertor.convertModel(object.getJSONObject("router")), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult modify(RouterModel model) {
        try {
            JSONObject params = new JSONObject();
            JSONObject router = new JSONObject();
            router.put("name", model.getName());
            params.put("router", router);
            Result result = this.doPut("/v2.0/routers/" + model.getRouterUuid(), params, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject routerObject = JSONObject.parseObject(result.getMessage(), JSONObject.class);
            return new BocloudResult(true, convertor.convertModel(routerObject.getJSONObject("router")), "修改成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    @Override
    public BocloudResult attach(RouterModel model) {
        try {
            String subnetId = model.getSubnetUuid();
            JSONObject params = new JSONObject();

            if (StringUtils.isNotEmpty(subnetId)) {
                params.put("subnet_id", subnetId);
            } else {
                params.put("port_id", model.getPortUuid());
            }
            Result result = this.doPut("/v2.0/routers/" + model.getRouterUuid() + "/add_router_interface", params,
                    FusionCloudApiName.vpc, FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject portObject = JSONObject.parseObject(result.getMessage(), JSONObject.class);
            String portUuid = portObject.getString("port_id");
            Result portResult = this.doGet("/v2.0/ports/" + portUuid, null, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            PortModel portModel = null;
            if (portResult.isSuccess()) {
                JSONObject portObj = JSONObject.parseObject(portResult.getMessage());
                BeanConvertor<PortModel, JSONObject> portConvertor = new PortConvertor();
                portModel = portConvertor.convertModel(portObj.getJSONObject("port"));
            }
            return new BocloudResult(true, portModel, "添加接口成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "添加接口失败");
        }
    }

    @Override
    public BocloudResult detach(RouterModel model) {
        try {
            String subnetId = model.getSubnetUuid();
            JSONObject params = new JSONObject();
            if (StringUtils.isNotEmpty(subnetId)) {
                params.put("subnet_id", subnetId);
            } else {
                params.put("port_id", model.getPortUuid());
            }
            Result result = this.doPut("/v2.0/routers/" + model.getRouterUuid() + "/remove_router_interface", params,
                    FusionCloudApiName.vpc, FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject routerObject = JSONObject.parseObject(result.getMessage(), JSONObject.class);
            return new BocloudResult(true, convertor.convertModel(routerObject), "添加接口成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除接口失败");
        }
    }

    @Override
    public BocloudResult config(RouterModel model) {
        try {
            JSONObject externalGatewayInfo = new JSONObject();
//            externalGatewayInfo.put("enable_snat", "false");
            externalGatewayInfo.put("network_id", model.getNetworkUuid());
            JSONObject externalGatewayParams = new JSONObject();
            externalGatewayParams.put("external_gateway_info",externalGatewayInfo);
            JSONObject routerParam = new JSONObject();
            routerParam.put("router", externalGatewayParams);
            Result result = this.doPut("/v2.0/routers/" + model.getRouterUuid(), routerParam, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "设置网关失败");
            }
            return new BocloudResult(true, convertor.convertModel(object.getJSONObject("router")), "设置网关成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "设置网关失败");

        }
    }

    @Override
    public BocloudResult clear(String routerId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult list() {

        try {
            Result result = this.doGet("/v2.0/routers", null, FusionCloudApiName.vpc, FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "路由列表为空");
            }
            List<JSONObject> objects = JSONArray.parseArray(object.getString("routers"), JSONObject.class);
            if (null == objects) {
                return new BocloudResult(false, "路由列表为空");
            }
            return new BocloudResult(true, convertor.convertList(objects), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

}
