package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.fusioncloudconvertor.SubnetConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 华为私有云子网操作driver
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年9月17日
 */
@Provider("bocloud.cmp.fusioncloud")
public class FusionCloudSubnetProvider extends FusionCloudProvider implements SubnetProvider {

    private static final Logger logger = LoggerFactory.getLogger(FusionCloudSubnetProvider.class);

    private BeanConvertor<SubnetModel, JSONObject> convertor = new SubnetConvertor();

    /**
     * @param butler
     */
    public FusionCloudSubnetProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            logger.info("find subnet !");
            Result result = this.doGet("/v2.0/subnets", null, FusionCloudApiName.vpc, FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "子网列表为空");
            }
            List<JSONObject> objects = JSONArray.parseArray(object.getString("subnets"), JSONObject.class);
            if (null == objects) {
                return new BocloudResult(false, "子网列表为空");
            }
            return new BocloudResult(true, convertor.convertList(objects), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(SubnetModel subnet) {
        try {
            JSONObject subnetModel = new JSONObject();
            subnetModel.put("tenant_id", this.getProject());
            subnetModel.put("name", subnet.getName());
            subnetModel.put("enable_dhcp", subnet.getDhcpEnabled());
            subnetModel.put("network_id", subnet.getNetworkUuid());
            List<String> dnsServers = new ArrayList<String>();
            if (!StringUtils.isEmpty(subnet.getDnsNames())) {
                dnsServers = JSON.parseArray(subnet.getDnsNames()).toJavaList(String.class);
            }
            subnetModel.put("dns_nameservers", dnsServers);
            List<JSONObject> ipPools = new ArrayList<JSONObject>();
            if (!StringUtils.isEmpty(subnet.getIpPools())) {
                List<String> list = JSONArray.parseArray(subnet.getIpPools(), String.class);
                for (String ipPool : list) {
                    String[] startEndIp = ipPool.split("-");
                    JSONObject object = new JSONObject();
                    object.put("start", startEndIp[0]);
                    object.put("end", startEndIp[1]);
                    ipPools.add(object);
                }
            }
            subnetModel.put("allocation_pools", ipPools);
            subnetModel.put("host_routes", subnet.getHostRoutes());
            subnetModel.put("ip_version", null == subnet.getIpVersion() ? "4" : subnet.getIpVersion());
            subnetModel.put("gateway_ip", subnet.getGateway());
            subnetModel.put("cidr", subnet.getCidr());
            JSONObject params = new JSONObject();
            params.put("subnet", subnetModel);
            Result result = this.doPost("/v2.0/subnets", params, FusionCloudApiName.vpc, FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("subnet");
            return new BocloudResult(true, convertor.convertModel(jsonObject), "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult modify(SubnetModel subnet) {
        try {
            logger.info("modify subnet ");
            JSONObject subnetParam = new JSONObject();
            subnetParam.put("name", subnet.getName());
            subnetParam.put("enable_dhcp", subnet.getDhcpEnabled());
            if (!StringUtils.isEmpty(subnet.getDnsNames())) {
                String[] dnsServers = JSON.parseObject(subnet.getDnsNames(), String[].class);
                subnetParam.put("dns_nameservers", dnsServers);
            }
            JSONObject params = new JSONObject();
            params.put("subnet", subnetParam);
            Result result = this.doPut("/v2.0/subnets/" + subnet.getSubnetUuid(), params, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("subnet");
            return new BocloudResult(true, convertor.convertModel(jsonObject), "修改成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "修改失败！");
        }
    }

    @Override
    public BocloudResult remove(String subnetId) {
        try {
            Result result = this.doDelete("/v2.0/subnets/" + subnetId, null, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, result.getMessage(), "删除成功！");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败！");
        }
    }

    @Override
    public BocloudResult detail(String subnetId) {
        try {
            Result result = this.doGet("/v2.0/subnets/" + subnetId, null, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("subnet");
            return new BocloudResult(true, convertor.convertModel(jsonObject), "查询详情成功！");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败！");
        }
    }

	@Override
	public BocloudResult remove(SubnetModel subnet) {
		// TODO Auto-generated method stub
		return null;
	}

}
