package com.bocloud.cmp.provider;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.io.IOUtils;
import org.apache.http.Header;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSphereRestApiProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/10/14 11:10
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereRestApiProvider extends GenericProvider implements SuperProvider {
    private static final Logger LOGGER = LoggerFactory.getLogger(FusionSphereRestApiProvider.class);
    private static final Integer TIMEOUT = 20 * 60 * 1000;//连接超时时间
    protected static final Integer LIMIT = 50;
    private String host;//https://<ip>:<port>
    private String version; //版本FusionCompute 6.3.0/6.3.1版本的 <version>为 6.3，FusionCompute 6.5.0/6.5.1版本的<version>为6.5，FusionCompute 8.0.0版本的<version>为8.0。
    private String username;//user为用户名
    private String password;//password为user对应的密码
    private Butler butler;
    private static Map<String, Token> tokenCache = new ConcurrentHashMap<>();


    public FusionSphereRestApiProvider(Butler butler) {
        super(butler);
        this.butler = butler;
        this.host = butler.getUrl();
        this.username = butler.getUsername();
        this.password = butler.getPassword();
        this.version = butler.getVersion();
//        this.token = new Token();
    }

    @Override
    public boolean accessable() {
        Token token = getFusionSphereAuthToken();
        if (null != token) {
            if (null != token.getAuthToken()) {
                return true;
            }
        }
        return false;
    }

    public String request(RequestStatus method, String url, String params) {
        Result result = this.connection(method, url, null, JSON.parseObject(params, Map.class));
        if (result.isFailed()) {
            throw new IllegalStateException("[" + unicodeToString(result.getErrorMsg()) + "]" + unicodeToString(result.getMessage()));
        }
        return unicodeToString(result.getMessage());
    }

    public String requestMonitor(RequestStatus method, String url, String params) {
        url = this.host + url;
        Map<String, Object> header = this.buildHeader();
        HttpClient client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
        try {
            Map<String, Object> auth = new HashMap<>();
            auth.put("X-Auth-Token", getFusionSphereAuthToken().getAuthToken());
            if (null != header) {
                auth.putAll(header);
            }
            Result result = client.post(auth, JSONArray.parseArray(params), url);
            client.close();
            if (result.isFailed()) {
                throw new IllegalStateException("[" + unicodeToString(result.getErrorMsg()) + "]" + unicodeToString(result.getMessage()));
            }
            return unicodeToString(result.getMessage());
        } finally {
            client.close();
        }


    }

    Result connection(RequestStatus method, String url, Map<String, Object> header, Map<String, Object> params) {
        if (MapUtils.isEmpty(header)) {
            header = this.buildHeader();
            header.put("X-Auth-Token", getFusionSphereAuthToken().getAuthToken());
        }
        url = this.host + url;
        log.info("接受到method：{}", method.name());
        log.info("接受到url：{}", url);
        log.info("接受到params：{}", params);
        if (null != params) {
            log.info("接受到jsonParams：{}", JSONObject.toJSONString(params));
        }
        log.info("接受到X-Auth-Token：{}", header.get("X-Auth-Token"));
        Result result;
        HttpClient client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
        try {
            switch (method) {
                case GET:
                    result = client.get(header, params, url);
                    break;
                case PUT:
                    result = client.put(header, params, url);
                    break;
                case POST:
                    result = client.post(header, params, url);
                    break;
                case DELETE:
                    result = client.delete(header, params, url);
                    break;
                case PATCH:
                    result = client.patch(header, params, url);
                    break;
                default:
                    result = new Result(false, "请求方式不存在！", "请求方式不存在！");
                    break;
            }
        } finally {
            client.close();
        }
        log.info("result：{}", result);
        return result;
    }

    private String unicodeToString(String string) {
        if (null == string) {
            return string;
        }
        StringBuffer stringBuffer = new StringBuffer();
        String pattern = "\\\\u[0-9,a-f,A-F]{4}";
        Matcher matcher = Pattern.compile(pattern).matcher(string);
        while (matcher.find()) {
            char c = (char) Integer.parseInt(matcher.group().replaceAll("\\\\u", ""), 16);
            matcher.appendReplacement(stringBuffer, new String(new char[]{c}));
        }
        matcher.appendTail(stringBuffer);
        return stringBuffer.toString();
    }

    protected Token getFusionSphereAuthToken() {
        Token token;
        if (tokenCache.containsKey(this.butler.getUuid())) {
            token = tokenCache.get(this.butler.getUuid());
            long entTime = System.currentTimeMillis();
            if (entTime - token.getStartTime() < token.getValidity()) {
                return token;
            }
        }
        token = getApiToken();
        return token;
    }

    private Token getApiToken() {
        log.info("开始执行底层api获取token.....");
        Token token = null;
        HttpClient client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
        try {
            String url = this.host + "/service/session";
            HttpPost httpPost = new HttpPost(url);
            Map<String, Object> header = this.buildHeader();
            header.put("X-Auth-User", this.username);
            header.put("X-Auth-Key", this.password);
            header.put("X-Auth-UserType", "0");
            header.put("X-ENCRIPT-ALGORITHM", "1");
            client.buildHeader(httpPost, header);
            CloseableHttpResponse response = client.getHttpClient().execute(httpPost);
            token = getBody(response).toJavaObject(Token.class);
            Header firstHeader = response.getFirstHeader("X-Auth-Token");
            String tokenstr = firstHeader.getValue();
            token.setAuthToken(tokenstr);
            token.setStartTime(System.currentTimeMillis());
            tokenCache.put(this.butler.getUuid(), token);
        } catch (Exception e) {
            LOGGER.error("令牌请求失败:" + e.getMessage());
        } finally {
            client.close();
        }
        return token;
    }

    public JSONObject getBody(CloseableHttpResponse httpResponse) throws Exception {
        JSONObject content = JSONObject.parseObject(IOUtils.toString(httpResponse.getEntity().getContent(), "UTF-8"));
        return content;
    }

    private Map<String, Object> buildHeader() {
        Map<String, Object> header = new HashMap<>();
        header.put("Content-Type", "application/json");
        header.put("Accept", "application/json");
        header.put("version", this.version);
        header.put("charset", "UTF-8");
        return header;
    }

    public static class Token {
        private Long startTime;
        private Integer validity;
        private String userId;
        private String userName;
        private String[] privilegeIds;
        private String[] roleList;
        private Integer rightType;
        private String authToken;

        public Long getStartTime() {
            return startTime;
        }

        public void setStartTime(Long startTime) {
            this.startTime = startTime;
        }

        public Integer getValidity() {
            return validity;
        }

        public void setValidity(Integer validity) {
            this.validity = validity;
        }

        public String getUserId() {
            return userId;
        }

        public void setUserId(String userId) {
            this.userId = userId;
        }

        public String getUserName() {
            return userName;
        }

        public void setUserName(String userName) {
            this.userName = userName;
        }

        public String[] getPrivilegeIds() {
            return privilegeIds;
        }

        public void setPrivilegeIds(String[] privilegeIds) {
            this.privilegeIds = privilegeIds;
        }

        public String[] getRoleList() {
            return roleList;
        }

        public void setRoleList(String[] roleList) {
            this.roleList = roleList;
        }

        public Integer getRightType() {
            return rightType;
        }

        public void setRightType(Integer rightType) {
            this.rightType = rightType;
        }

        public String getAuthToken() {
            return authToken;
        }

        public void setAuthToken(String authToken) {
            this.authToken = authToken;
        }
    }

    public enum RequestStatus {
        GET,
        POST,
        DELETE,
        PUT,
        PATCH
    }

    public Butler getButler() {
        return butler;
    }

}
