package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.fusionsphere.FusionSphereOsVersionModel;
import com.bocloud.cmp.model.fusionsphere.FusionSphereUpLinkPortModel;
import com.bocloud.cmp.model.fusionsphere.HostPortModel;
import com.bocloud.cmp.model.fusionsphere.StorageDeviceModel;
import com.bocloud.cmp.model.fusionsphere.sync.FusionSphereSyncModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.compute.*;
import com.bocloud.cmp.provider.compute.HostProvider;
import com.bocloud.cmp.provider.fusionsphere.*;
import com.bocloud.cmp.provider.monitor.FusionSphereMonitorProvider;
import com.bocloud.cmp.provider.monitor.MonitorProvider;
import com.bocloud.cmp.provider.network.*;
import com.bocloud.cmp.provider.storage.*;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSphereLocationProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/10/15 14:59
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereLocationProvider extends FusionSphereRestApiProvider implements LocationProvider {

    private Butler butler;
    private VdcProvider vdcProvider;
    private ClusterProvider clusterProvider;
    private HostProvider hostProvider;
    private HostPortProvider hostPortProvider;
    private ServerProvider serverProvider;

    private StorageDeviceProvider storageDeviceProvider;
    private DataStoreProvider dataStoreProvider;
    private VolumeProvider volumeProvider;

    private SecurityGroupProvider securityGroupProvider;
    private SecurityGroupRuleProvider securityGroupRuleProvider;
    private SwitchProvider switchProvider;
    private UpLinkPortProvider upLinkPortProvider;
    private PortGroupProvider portGroupProvider;
    private SnapshotProvider snapshotProvider;
    private OsVersionProvider osVersionProvider;
    private MonitorProvider monitorProvider;


    public FusionSphereLocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
        vdcProvider = new FusionSphereVdcProvider(butler);
        clusterProvider = new FusionSphereClusterProvider(butler);
        hostProvider = new FusionSphereHostProvider(butler);
        hostPortProvider = new FusionSphereHostPortProvider(butler);
        serverProvider = new FusionSphereServerProvider(butler);

        storageDeviceProvider = new FusionSphereStorageDeviceProvider(butler);
        dataStoreProvider = new FusionSphereDataStoreProvider(butler);
        volumeProvider = new FusionSphereVolumeProvider(butler);

        securityGroupProvider = new FusionSphereSecurityGroupProvider(butler);
        securityGroupRuleProvider = new FusionSphereSecurityGroupRuleProvider(butler);
        switchProvider = new FusionSphereSwitchProvider(butler);
        upLinkPortProvider = new FusionSphereUpLinkPortProvider(butler);
        portGroupProvider = new FusionSpherePortGroupProvider(butler);
        snapshotProvider = new FusionSphereSnapshotProvider(butler);
        osVersionProvider = new FusionSphereOsVersionProvider(butler);
        monitorProvider = new FusionSphereMonitorProvider(butler);
    }

    @Override
    public BocloudResult sync() {
        try {
            BocloudResult bocloudResult;
            FusionSphereSyncModel syncModel = new FusionSphereSyncModel();

            log.info("{} - site -->Start", this.butler.getName());
            bocloudResult = vdcProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setVdcModels(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), VDCModel.class));
            log.info("{} - site -->End", this.butler.getName());

            log.info("{} - OsVersion -->Start", this.butler.getName());
            List<FusionSphereOsVersionModel> osVersionModels = new ArrayList<>();
            for (VDCModel vdcModel : syncModel.getVdcModels()) {
                bocloudResult = osVersionProvider.listByVdc(vdcModel.getUri());
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                osVersionModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), FusionSphereOsVersionModel.class));
            }
            syncModel.setOsVersionModels(osVersionModels);
            log.info("{} - OsVersion -->End", this.butler.getName());

            log.info("{} - cluster -->Start", this.butler.getName());
            List<ClusterModel> clusterModels = new ArrayList<>();
            for (VDCModel vdcModel : syncModel.getVdcModels()) {
                bocloudResult = clusterProvider.listByVdc(vdcModel.getUri());
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                clusterModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), ClusterModel.class));
            }
            syncModel.setClusterModels(clusterModels);
            log.info("{} - cluster -->End", this.butler.getName());

            log.info("{} - host -->Start", this.butler.getName());
            List<HostModel> hostModels = new ArrayList<>();
            for (VDCModel vdcModel : syncModel.getVdcModels()) {
                bocloudResult = hostProvider.listByVdc(vdcModel.getUri());
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                hostModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), HostModel.class));
            }
            //查询服务器网口
            for (HostModel hostModel : hostModels) {
                bocloudResult = hostPortProvider.list(hostModel.getUri());
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                List<HostPortModel> hostPortModels = JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), HostPortModel.class);
                hostModel.setHostPortModels(hostPortModels);
            }
            syncModel.setHostModels(hostModels);
            log.info("{} - host -->End", this.butler.getName());

            log.info("{} - StorageDevice -->Start", this.butler.getName());
            List<StorageDeviceModel> storageDeviceModels = new ArrayList<>();
            for (VDCModel vdcModel : syncModel.getVdcModels()) {
                bocloudResult = storageDeviceProvider.listByVdc(vdcModel.getUri());
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                storageDeviceModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), StorageDeviceModel.class));
            }
            syncModel.setStorageDeviceModels(storageDeviceModels);
            log.info("{} - StorageDevice -->End", this.butler.getName());

            log.info("{} - dataStorage -->Start", this.butler.getName());
            List<DataStoreModel> dataStoreModels = new ArrayList<>();
            for (VDCModel vdcModel : syncModel.getVdcModels()) {
                bocloudResult = dataStoreProvider.listByVdc(vdcModel.getUri());
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                dataStoreModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), DataStoreModel.class));
            }
            syncModel.setDataStoreModels(dataStoreModels);
            log.info("{} - dataStorage -->End", this.butler.getName());

            log.info("{} - volume -->Start", this.butler.getName());
            List<VolumeModel> volumeModels = new ArrayList<>();
            for (VDCModel vdcModel : syncModel.getVdcModels()) {
                bocloudResult = volumeProvider.listByVdc(vdcModel.getUri());
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                volumeModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), VolumeModel.class));
            }
            syncModel.setVolumeModels(volumeModels);
            log.info("{} - volume -->End", this.butler.getName());

            log.info("{} - securityGroup -->Start", this.butler.getName());
            List<SecurityGroupModel> securityGroupModels = new ArrayList<>();
            for (VDCModel vdcModel : syncModel.getVdcModels()) {
                bocloudResult = securityGroupProvider.listByVdc(vdcModel.getUri());
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                securityGroupModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), SecurityGroupModel.class));
            }
            log.info("{} - securityGroup -->End", this.butler.getName());

            log.info("{} - securityGroupRule -->Start", this.butler.getName());
            for (SecurityGroupModel securityGroupModel : securityGroupModels) {
                bocloudResult = securityGroupRuleProvider.list(securityGroupModel.getGroupUuid(), securityGroupModel.getVdcUri());
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                securityGroupModel.setModels(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), SecurityGroupRuleModel.class));
            }
            syncModel.setSecurityGroupModels(securityGroupModels);
            log.info("{} - securityGroupRule -->End", this.butler.getName());

            log.info("{} - switcher -->Start", this.butler.getName());
            List<SwitchModel> switchModels = new ArrayList<>();
            for (VDCModel vdcModel : syncModel.getVdcModels()) {
                bocloudResult = switchProvider.listByVdc(vdcModel.getUri());
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                switchModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), SwitchModel.class));
            }

            for (SwitchModel switchModel : switchModels) {
                //获取详情
                bocloudResult = switchProvider.detail(switchModel.getUri());
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                SwitchModel newSwitcherModel = JSONObject.parseObject(JSONObject.toJSONString(bocloudResult.getData()), SwitchModel.class);
                switchModel.setVlanPoolModels(newSwitcherModel.getVlanPoolModels());
                switchModel.setUpLinkModels(newSwitcherModel.getUpLinkModels());
                //获取端口组
                bocloudResult = portGroupProvider.listBySwitcher(switchModel.getUri());
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                switchModel.setGroups(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), PortGroupModel.class));
                //获取上行链路组
                bocloudResult = upLinkPortProvider.listBySwitcher(switchModel.getVdcUri(), switchModel.getSwitcherVal());
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                switchModel.setUpLinkPortModels(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), FusionSphereUpLinkPortModel.class));
            }
            syncModel.setSwitchModels(switchModels);
            log.info("{} - switcher -->End", this.butler.getName());

            log.info("{} - vm -->Start", this.butler.getName());
            List<ServerModel> serverModels = new ArrayList<>();
            for (VDCModel vdcModel : syncModel.getVdcModels()) {
                bocloudResult = serverProvider.listByVdc(vdcModel.getUri());
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                serverModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), ServerModel.class));
            }
            //云主机快照
            if (!ListTool.isEmpty(serverModels)) {
                for (ServerModel server : serverModels) {
                    bocloudResult = snapshotProvider.list(server.getUri());
                    if (bocloudResult.isFailed()) {
                        return bocloudResult;
                    }
                    server.setSnapshots(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), SnapshotModel.class));
                }
            }
            syncModel.setServerModels(serverModels);
            log.info("{} - vm -->End", this.butler.getName());

            //todo 测试环境拿数据用
            String jsonString = JSONObject.toJSONString(syncModel);
            System.out.println(jsonString);

            return new BocloudResult(true, syncModel, "success");
        } catch (Exception e) {
            log.error("FusionCompute sync failure :", e);
            return new BocloudResult(false, "FusionCompute sync failure :" + e.getMessage());
        }
    }


}