package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.SnapshotConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.cmp.utils.TaskProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

/**
 * @ClassName: FusionSphereSnapshotProvider
 * @Description: 云主机快照
 * @Author lxy
 * @Date 2021/10/28 15:07
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereSnapshotProvider extends FusionSphereRestApiProvider implements SnapshotProvider {

    private BeanJsonConvertor<SnapshotModel> convertor = new SnapshotConvertor();

    public FusionSphereSnapshotProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list(String serverUri) {
        try {
            String api = "%s/snapshots";
            String url = String.format(api, serverUri);
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<SnapshotModel> models = convertor.convertListParams(jsonObject.getJSONArray("rootSnapshots"), null);
            return new BocloudResult(true, traverseSnapshotTree(models), LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(SnapshotModel model) {
        try {
            //需要先调用云主机快照列表接口，得到当前parentUuid
            String api = "%s/snapshots";
            String url = String.format(api, model.getServerUri());

            String parentUuid = null;
            String responseList = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObjectList = JSONObject.parseObject(responseList);
            JSONObject currentSnapshot = jsonObjectList.getJSONObject("currentSnapshot");
            if (null != currentSnapshot) {
                parentUuid = currentSnapshot.getString("urn");
            }
            JSONObject params = new JSONObject();
            params.put("name", model.getName());//必选
            params.put("description", model.getRemark());
            params.put("needMemoryShot", model.getNeedMemoryShot());
            params.put("isConsistent", model.getIsConsistent());
            params.put("type", model.getSnapshotType().toLowerCase());
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            if (taskStatus) {
                BocloudResult snapshot = this.getSnapshot(jsonObject.getString("uri"));
                if (snapshot.isFailed()) {
                    return snapshot;
                }
                SnapshotModel snapshotModel = JSONObject.parseObject(JSONObject.toJSONString(snapshot.getData()), SnapshotModel.class);
                snapshotModel.setParentSnapshotUuid(parentUuid);
                return new BocloudResult(true, snapshotModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
            } else {
                return new BocloudResult(false, "任务失败");
            }
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult getSnapshot(String snapshotUri) {
        try {
            String response = this.request(RequestStatus.GET, snapshotUri, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            SnapshotModel models = convertor.convertModelParams(jsonObject, null);
            return new BocloudResult(true, models, LogMapUtils.MAP.get("detail"));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    /**
     * 遍历快照树，塞入快照详细信息
     *
     * @param trees
     * @return
     */
    public List<SnapshotModel> traverseSnapshotTree(List<SnapshotModel> trees) throws Exception {
        if (null != trees) {
            for (SnapshotModel tree : trees) {
                BocloudResult result = getSnapshot(tree.getUri());
                if (result.isFailed()) {
                    throw new Exception(result.getMessage());
                }
                SnapshotModel model = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), SnapshotModel.class);
                //快照详细信息
                tree.setSize(model.getSize());
                if (null != tree && null != tree.getChildSnapshots()) {
                    traverseSnapshotTree(tree.getChildSnapshots());
                }
            }
        }
        return trees;
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        try {
            String response = this.request(RequestStatus.DELETE, model.getUri(), null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        try {
            String api = "%s/action/resumevm";
            String url = String.format(api, model.getUri());
            String response = this.request(RequestStatus.POST, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get("operation"));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

}
