package com.bocloud.cmp.provider.compute;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VDCModel;
import com.bocloud.cmp.model.h3c.H3cClusterModel;
import com.bocloud.cmp.model.h3c.H3cHostModel;
import com.bocloud.cmp.model.h3c.HostPoolModel;
import com.bocloud.cmp.model.h3c.entity.RsCommList;
import com.bocloud.cmp.model.h3c.entity.RsHostPool;
import com.bocloud.cmp.model.h3c.utils.RestUtil;
import com.bocloud.cmp.provider.H3cProvider;
import com.bocloud.cmp.provider.utils.XmlUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

/**
 * 华三云主机池操作类
 * 
 * @author ymh
 *
 */
@Provider("bocloud.cmp.h3c")
public class H3cHostPoolProvider extends H3cProvider implements HostPoolProvider {

	private static Logger logger = LoggerFactory.getLogger(H3cHostPoolProvider.class);

	public H3cHostPoolProvider(Butler butler) {
		super(butler);
	}

	/**
	 * 查询CAS系统中所有主机池列表
	 */
	public static final String LIST_ALL = "/cas/casrs/hostpool/all";

	/**
	 * 查询主机及集群
	 */
	public static final String LIST_HOST_CLUSTER = "/cas/casrs/hostpool/{hpId}/allChildNode";
	/**
	 * 查询主机池下所有云主机
	 */
	public static final String LIST_ALL_VM = "/cas/casrs/vm/vmList?hpId={hpId}&offset={offset}&limit={limit}&sortField={sortField}&sortDir={sortDir}";
	/**
	 * 查询指定主机池的概要信息
	 */
	public static final String LIST_SUMMARY = "/cas/casrs/hostpool/summary/{hpId}";
	/**
	 * 创建主机池
	 */
	public static final String CREATION = "/cas/casrs/hostpool/add/{name}";

	/**
	 * 删除主机池
	 */
	public static final String DELETE = "/cas/casrs/hostpool/delete/{hpId}";
	/**
	 * 查询CAS系统中指定主机池下的共享文件系统列表
	 */
	public static final String LIST_SHARE_FILE = "/cas/casrs/hostpool/shareFile/{hpId}";

	/**
	 * 查询CAS系统中指定主机池下的主机列表
	 */
	public static final String LIST_HOST = "/cas/casrs/hostpool/host/{hpId}?hostName={hostName}&offset={offset}&limit={limit}";

	/**
	 * 删除主机池下的共享文件系统
	 */
	public static final String DELETE_SHARE_FILE = "/cas/casrs/hostpool/delete/{id}?hpId={hpId}";

	/**
	 * 连接主机池下的所有主机
	 */
	public static final String CONNECT_ALL_HOST = "/cas/casrs/hostpool/connectAllHost/{hpId}";

	private Result getResult(String url) {
		Result result = null;
		try {
			result = doGet(url);
		} catch (Exception e) {
			logger.error("访问失败", e);
		} finally {
			try {
				this.getClient().close();
			} catch (IOException e) {
				logger.error("出现io异常", e);
			}
		}
		return result;
	}

	/**
	 * 查询所有主机池
	 *
	 * @return BocloudResult
	 */
	@Override
	public BocloudResult<List<VDCModel>> list() {
		String casuri = "/cas/casrs/hostpool/all";
		@SuppressWarnings("unchecked")
		RsCommList<RsHostPool> httpGet = RestUtil.httpGet(defaultHttpClient, casuri, RsCommList.class);
		if (null == httpGet || null == httpGet.getData()) {
			return new BocloudResult<List<VDCModel>>(false, "查询VDC异常");
		}
		List<RsHostPool> data = httpGet.getData();
		List<VDCModel> convertCluster = convertVdc(data);
		return new BocloudResult<List<VDCModel>>(true, convertCluster, "查询VDC成功！");
	}

	private List<VDCModel> convertVdc(List<RsHostPool> rsHostPools) {
		List<VDCModel> vdcModels = new ArrayList<>();
		if (!ListTool.isEmpty(rsHostPools)) {
			logger.info("start sync cluster summary ...");
			for (RsHostPool rsHostPool : rsHostPools) {
				VDCModel vdcModel = new VDCModel();
				vdcModel.setName(rsHostPool.getName());
				vdcModel.setVdcVal(String.valueOf(rsHostPool.getId()));
				getMoreClusterInfo(vdcModel);
				vdcModels.add(vdcModel);
			}
			logger.info("end sync cluster summary ...");
		}
		return vdcModels;
	}

	private void getMoreClusterInfo(VDCModel vdcModel) {
		BocloudResult<VDCModel> summary = this.summary(vdcModel.getVdcVal());
		if (summary.isSuccess()) {
			VDCModel summaryData = (VDCModel) summary.getData();
			vdcModel.setMemTotal(summaryData.getMemTotal());
			vdcModel.setCpuTotal(summaryData.getCpuTotal());
			vdcModel.setDiskTotal(summaryData.getDiskTotal());
			vdcModel.setDiskUsed(summaryData.getDiskUsed());
		}
	}

	/**
	 * 创建主机池
	 *
	 * @param name
	 *            名称
	 * @return BocloudResult
	 */
	@Override
	public BocloudResult  create(String name) {
		try {
			String uri = "/cas/casrs/hostpool/add/" + name;
			RestUtil.httpPost(defaultHttpClient, uri, null, String.class);
			return new BocloudResult (true, "主机池创建成功！");
		} catch (Exception e) {
			logger.error("主机池创建失败：" + e.getMessage());
			return new BocloudResult (false, "主机池创建失败！");
		}
	}

	/**
	 * 查询查询CAS系统中主机池下的主机及集群列表
	 *
	 * @param hpId
	 * @return BocloudResult
	 */
	@Override
	public BocloudResult listHostAndCluster(Long hpId) {
		Result result = getResult(LIST_HOST_CLUSTER.replace("{hpId}", String.valueOf(hpId)));
		if (result != null && result.isSuccess()) {
			String xmlStr = result.getMessage();
			HostPoolModel h3cHostPoolModel = new HostPoolModel();
			List<Element> clusterElements = XmlUtils.getNodesByXpath(xmlStr, "nodeList/clusterList");
			List<Element> hostElements = XmlUtils.getNodesByXpath(xmlStr, "nodeList/hostList");
			List<H3cClusterModel> clusterList = new ArrayList<>();
			List<H3cHostModel> hostList = new ArrayList<>();
			if (!ListTool.isEmpty(clusterElements)) {
				for (Element clusterElement : clusterElements) {
					String s = XmlUtils.xml2JSON(clusterElement.asXML());
					JSONObject jsonObject = JSONObject.parseObject(s);
					JSONObject obj = ((JSONObject) jsonObject.get("clusterList"));
					clusterList.add(JSONObject.toJavaObject(obj, H3cClusterModel.class));
				}
				h3cHostPoolModel.setClusters(clusterList);
			}
			if (!ListTool.isEmpty(hostElements)) {
				for (Element hostElement : hostElements) {
					String s = XmlUtils.xml2JSON(hostElement.asXML());
					JSONObject jsonObject = JSONObject.parseObject(s);
					JSONObject obj = ((JSONObject) jsonObject.get("hostList"));
					hostList.add(JSONObject.toJavaObject(obj, H3cHostModel.class));
				}
				h3cHostPoolModel.setHosts(hostList);
			}
			return new BocloudResult(true, h3cHostPoolModel, "查询成功");
		} else {
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult<VDCModel> summary(String vdcVal) {
		String url = LIST_SUMMARY.replace("{hpId}", vdcVal);
		String vdcSummary = RestUtil.httpGet(defaultHttpClient, url, String.class);
		if (!StringUtils.isEmpty(vdcSummary)) {
			String xmlStr = vdcSummary;
			List<Element> keyvalues = XmlUtils.getNodesByXpath(xmlStr, "//list/keyValue");
			VDCModel vdcModel = new VDCModel();
			Long cpuTotal = 0L;
			Double memTotal = 0d;
			Double diskAvail = 0d;
			Double diskTotal = 0d;
			if (!ListTool.isEmpty(keyvalues)) {
				for (Element keyvalue : keyvalues) {
					if ("CPU总核数".equals(keyvalue.element("key").getTextTrim())) {
						cpuTotal = Long.valueOf(keyvalue.element("value").getTextTrim());
					} else if ("总内存".equals(keyvalue.element("key").getTextTrim())) {
						memTotal = kbMbGbToGb(keyvalue.element("value").getTextTrim());
					} else if ("本地总存储".equals(keyvalue.element("key").getTextTrim())) {
						diskTotal = kbMbGbToGb(keyvalue.element("value").getTextTrim());
					} else if ("本地总可用存储".equals(keyvalue.element("key").getTextTrim())) {
						diskAvail = kbMbGbToGb(keyvalue.element("value").getTextTrim());
					}
				}
			}
			vdcModel.setDiskUsed(new BigDecimal(diskTotal).subtract(new BigDecimal(diskAvail))
					.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
			vdcModel.setDiskTotal(diskTotal);
			vdcModel.setCpuTotal(cpuTotal);
			vdcModel.setMemTotal(memTotal);
			return new BocloudResult<VDCModel>(true, vdcModel, "查询成功");
		} else {
			return new BocloudResult<VDCModel>(false, "查询失败");
		}
	}

	@Override
	public BocloudResult  remove(String vdcVal) {
		String uri = "/cas/casrs/hostpool/delete/" + vdcVal;
		try {
			RestUtil.httpDelete(defaultHttpClient, uri);
			return new BocloudResult (true, "删除主机池成功！");
		} catch (Exception e) {
			logger.error("删除主机池失败:" + e.getMessage());
			return new BocloudResult (false, "查询失败");
		}
	}

	@Override
	public BocloudResult<VDCModel> detail(String name) {
		BocloudResult<List<VDCModel>> listResult = this.list();
		if (listResult.isSuccess()) {
			List<VDCModel> list = (List<VDCModel>) listResult.getData();
			VDCModel model = null;
			for (VDCModel hostPoolModel : list) {
				if (name.equals(hostPoolModel.getName())) {
					model = hostPoolModel;
					break;
				}
			}
			return new BocloudResult<VDCModel>(true, model, "查询成功！");
		} else {
			return new BocloudResult<VDCModel>(false, "查询出错");
		}
	}

}
