package com.bocloud.cmp.provider;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcVersion;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

public abstract class HmcDispatcherProvider extends HmcAccessableProvider {

    private final Logger logger = LoggerFactory.getLogger(HmcDispatcherProvider.class);

    private final HmcVersion defaultVersion = HmcVersion.V8;
    private final Butler butler;
    private static final Map<ProviderCacheKey, Object> providerCache = new ConcurrentHashMap<>();

    public HmcDispatcherProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    protected synchronized  <T extends SuperProvider, N extends T> T getHmcProviderImpl(Class<T> serviceClass, Class<N> dispatcherClass) {
        ProviderCacheKey providerCacheKey = new ProviderCacheKey(serviceClass, dispatcherClass, butler);
        if (providerCache.containsKey(providerCacheKey)) {
            return (T) providerCache.get(providerCacheKey);
        }

        String providerImplVersionPackageName = serviceClass.getPackage().getName() + "." + butler.getVersion().toLowerCase();
        String defaultImplVersionPackageName = serviceClass.getPackage().getName() + "." + defaultVersion.name().toLowerCase();
        Class<?> serviceImplClass;
        try {
            serviceImplClass = Class.forName(providerImplVersionPackageName + "." + dispatcherClass.getSimpleName());
        } catch (ClassNotFoundException e) {
            try {
                serviceImplClass = Class.forName(defaultImplVersionPackageName + "." + dispatcherClass.getSimpleName());
            } catch (ClassNotFoundException e1) {
                throw new RuntimeException(String.format("未找到类：%s",
                        providerImplVersionPackageName + "." + dispatcherClass.getSimpleName())
                );
            }
        }
        if (!serviceClass.isAssignableFrom(serviceImplClass)) {
            throw new RuntimeException(String.format("类：%s并未继承或实现%s",
                    providerImplVersionPackageName + "." + dispatcherClass.getSimpleName(),
                    serviceClass.getName()
            ));
        }
        Constructor<?> constructor;
        try {
            constructor = serviceImplClass.getConstructor(new Class[]{Butler.class});
        } catch (NoSuchMethodException e) {
            throw new RuntimeException(String.format("类：%s并未包含以Butler为唯一参数的构造方法",
                    providerImplVersionPackageName + "." + dispatcherClass.getSimpleName())
            );
        }
        T serviceImplInstance;
        try {
            serviceImplInstance = (T) constructor.newInstance(butler);
        } catch (InstantiationException | IllegalAccessException | InvocationTargetException e) {
            throw new RuntimeException(String.format("实例化类：%s失败%s",
                    providerImplVersionPackageName + "." + dispatcherClass.getSimpleName(), e.getMessage())
            );
        }
        logger.info(serviceClass.getSimpleName() + "[" + butler.getVersion() + "][" + butler.getName() + "] ---> " + serviceImplClass.getName());
        providerCache.put(providerCacheKey, serviceImplInstance);
        return serviceImplInstance;
    }

    private class ProviderCacheKey {
        private final Class serviceClass;
        private final Class dispatcherClass;
        private final Butler butler;
        public ProviderCacheKey(Class serviceClass, Class dispatcherClass, Butler butler) {
            this.serviceClass = serviceClass;
            this.dispatcherClass = dispatcherClass;
            this.butler = butler;
        }
        public Class getServiceClass() {
            return serviceClass;
        }
        public Class getDispatcherClass() {
            return dispatcherClass;
        }
        public Butler getButler() {
            return butler;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) {
                return true;
            }
            if (o == null || getClass() != o.getClass()) {
                return false;
            }
            ProviderCacheKey that = (ProviderCacheKey) o;
            return Objects.equals(serviceClass, that.serviceClass) &&
                    Objects.equals(dispatcherClass, that.dispatcherClass) &&
                    Objects.equals(butler.getUrl(), that.butler.getUrl());
        }

        @Override
        public int hashCode() {
            return Objects.hash(serviceClass, dispatcherClass, butler.getUrl());
        }
    }
}
