package com.bocloud.cmp.provider.hmc.sdk.api.v7;

import com.bocloud.cmp.provider.hmc.sdk.api.BaseService;
import com.bocloud.cmp.provider.hmc.sdk.enums.RequestMethod;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcSdkConstant;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.common.V7ErrorContent;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.common.V7ErrorResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.common.V7HttpErrorResponse;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.Result;
import org.apache.http.HttpHeaders;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.function.Function;

/**
 * HTTP调用的基础服务类
 * 
 * @author sxp
 * @since May 2, 2018
 */
public class V7BaseService extends BaseService {

    private static final Logger logger = LoggerFactory.getLogger(V7BaseService.class);

    /**
     * 查询
     *
     * @author sxp
     * @since May 2, 2018
     */
    public static Result httpGet(HmcAuthInfo auth, String url, String contentType) {
        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, contentType);
        return invokeWith401RetryOfDefaultCnt3(RequestMethod.GET, auth, url, header, "");
    }

    /**
     * 查询
     * 
     * @author sxp
     * @since May 2, 2018
     */
    public static Result httpGet(HmcAuthInfo auth, String url, Map<String, Object> header, String body) {
        return invokeWith401RetryOfDefaultCnt3(RequestMethod.GET, auth, url, header, body);
    }

    /**
     * 创建
     * 
     * @author sxp
     * @since May 2, 2018
     */
    public static Result httpPost(HmcAuthInfo auth, String url, Map<String, Object> header, String body) {
        return invokeWith401RetryOfDefaultCnt3(RequestMethod.POST, auth, url, header, body);
    }


    /**
     * 修改
     * 
     * @author sxp
     * @since May 2, 2018
     */
    public static Result httpPut(HmcAuthInfo auth, String url, Map<String, Object> header, String body) {
        return invokeWith401RetryOfDefaultCnt3(RequestMethod.PUT, auth, url, header, body);
    }

    /**
     * 删除
     * 
     * @author sxp
     * @since May 2, 2018
     */
    public static Result httpDelete(HmcAuthInfo auth, String url, Map<String, Object> header, String body) {
        return invokeWith401RetryOfDefaultCnt3(RequestMethod.DELETE, auth, url, header, body);
    }

    /**
     * 当调用结果满足某些条件时，再次尝试调用
     * @param method GET POST ...
     * @param auth hmc认证信息
     * @param url 地址
     * @param header 头
     * @param body 体
     * @param condition 需要再次尝试调用的条件
     * @param tryCnt 共尝试几次
     * @return 调用结果
     */
    public static Result invokeWithConditionRetry(RequestMethod method, HmcAuthInfo auth, String url, Map<String, Object> header, String body, Function<Result, Boolean> condition, int tryCnt) {
        int currTryCnt = 1;
        Result result = invokeWith401RetryOfDefaultCnt3(method, auth, url, header, body);
        while (condition.apply(result) && currTryCnt <= tryCnt) {
            result = invokeWith401RetryOfDefaultCnt3(method, auth, url, header, body);
            currTryCnt++;
        }
        return result;
    }

    /**
     * 如果返回结果是401，则可能是Session过期，则再次尝试，总共尝试3次
     * @param method GET POST ...
     * @param auth hmc认证信息
     * @param url 地址
     * @param header 头
     * @param body 体
     * @return 调用结果
     */
    public static Result invokeWith401RetryOfDefaultCnt3(RequestMethod method, HmcAuthInfo auth, String url, Map<String, Object> header, String body) {
        return invokeWith401Retry(method, auth, url, header, body, 3);
    }

    /**
     * 如果返回结果是401，则可能是Session过期，则再次尝试
     * @param method GET POST ...
     * @param auth hmc认证信息
     * @param url 地址
     * @param header 头
     * @param body 体
     * @param tryCnt 尝试次数
     * @return 调用结果
     */
    public static Result invokeWith401Retry(RequestMethod method, HmcAuthInfo auth, String url, Map<String, Object> header, String body, int tryCnt) {
        int currTryCnt = 1;
        Result result = invokeWithSession(method, auth, url, header, body);
        while ("401".equals(result.getErrorMsg()) && currTryCnt <= tryCnt) {
            V7LogonService.clearSession(auth);
            result = invokeWithSession(method, auth, url, header, body);
            currTryCnt++;
        }
        return result;
    }

    /**
     * 获取Session并执行调用，不考虑Session是否失效
     * @param method GET POST ...
     * @param auth hmc认证信息
     * @param url 地址
     * @param header 头
     * @param body 体
     * @return 调用结果
     */
    public static Result invokeWithSession(RequestMethod method, HmcAuthInfo auth, String url, Map<String, Object> header, String body) {
        header = Optional.ofNullable(header).orElse(new HashMap<>());
        if (!header.containsKey(HttpHeaders.ACCEPT)) {
            header.put(HttpHeaders.ACCEPT, "*/*");
        }
        if (!header.containsKey(HmcSdkConstant.HEADER_X_API_SESSION)) {
            header.put(HmcSdkConstant.HEADER_X_API_SESSION, V7LogonService.getSession(auth));
        }
        return invokeWithErrorParse(method, url, header, body);
    }

    /**
     * 调用Http请求，如果发生错误，则解析错误信息
     * @param method GET POST ...
     * @param url 地址
     * @param header 头
     * @param body 体
     * @return 调用结果
     */
    public static Result invokeWithErrorParse(RequestMethod method, String url, Map<String, Object> header, String body) {
        Result result = invokeWithLog(method, url, header, body);
        if (result.isSuccess()) {
            return result;
        }
        StringBuilder errMsg = new StringBuilder();
        try {
            Optional<V7HttpErrorResponse> errorResponseOpt = Optional.ofNullable(result.getMessage())
                    .map(msg -> msg.replaceAll("HttpErrorResponse:HttpErrorResponse", "HttpErrorResponse"))
                    .map(msg -> XmlUtils.parseObject(V7ErrorResponse.class, msg))
                    .map(V7ErrorResponse::getContent)
                    .map(V7ErrorContent::getHttpErrorResponse);
            errorResponseOpt.map(V7HttpErrorResponse::getHttpStatus).ifPresent(status -> errMsg.append("[").append(status).append("]"));
            errorResponseOpt.map(V7HttpErrorResponse::getReasonCode).ifPresent(reason -> errMsg.append(": ").append(reason));
            errorResponseOpt.map(V7HttpErrorResponse::getMessage).ifPresent(msg -> errMsg.append(", ").append(msg));
            errorResponseOpt.map(V7HttpErrorResponse::getStackTrace).ifPresent(logger::error);
        } catch (Exception e) {
            errMsg.append("[").append(result.getStatus()).append("]: ").append(result.getMessage());
        }
        result.setMessage(errMsg.toString());
        return result;
    }
}
