package com.bocloud.cmp.provider.hmc.sdk.api.v7;

import com.bocloud.cmp.provider.hmc.sdk.enums.HostEthernetAdapterQuickproperty;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7HostEthernetAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7HostEthernetAdapterEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7HostEthernetAdapterForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7HostEthernetAdapterResponse;
import com.bocloud.cmp.provider.hmc.sdk.util.EnhancedBeanUtils;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.http.HttpHeaders;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * 主机以太网适配器服务
 * 
 * @author sxp
 * @since May 15, 2018
 */
public class V7HostEthernetAdapterService extends V7BaseService {
    private static final String TYPE = "HostEthernetAdapter";
    private static final String API = "/rest/api/uom/ManagedSystem/%s/HostEthernetAdapter";
    private static final String SPECIFIED_API = API + "/%s";
    private static final String QUICK_API = "/rest/api/uom/ManagedSystem/%s/HostEthernetAdapter/%s/quick/%s";
    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.uom+xml; Type=HostEthernetAdapter";

    /**
     * 获取系统的以太网适配器信息
     * 
     * @author sxp
     * @since May 15, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth, String managedSystemUuid) {
        String api = String.format(API, managedSystemUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7HostEthernetAdapterResponse response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7HostEthernetAdapterResponse.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 获取系统的以太网适配器信息
     * 
     * @author sxp
     * @since May 15, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth, String managedSystemUuid, String adapterId) {
        String api = String.format(SPECIFIED_API, managedSystemUuid, adapterId);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7HostEthernetAdapterEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7HostEthernetAdapterEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 快速查询指定以太网适配器的某个属性信息
     * <p>
     * 只获取指定属性的值
     * </p>
     * 
     * @author sxp
     * @since May 15, 2018
     */
    public static BocloudResult getQuickProperty(HmcAuthInfo auth, String managedSystemUuid, String adapterId, HostEthernetAdapterQuickproperty property) {
        String api = String.format(QUICK_API, managedSystemUuid, adapterId, property.name());
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        String value = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace("\"", ""))
                .orElse(null);
        return new BocloudResult(true, value, "执行成功");
    }

    /**
     * 修改虚拟网络交换机
     */
    public static BocloudResult update(HmcAuthInfo auth, String serverUuid, String heaUuid, V7HostEthernetAdapterForUpdate heaForUpdate) {
        BocloudResult getResult = get(auth, serverUuid, heaUuid);
        if (getResult.isFailed()) {
            return new BocloudResult(false, getResult.getMessage());
        }

        V7HostEthernetAdapterEntry entry = (V7HostEthernetAdapterEntry) getResult.getData();
        V7HostEthernetAdapter target = entry.getContent().getHostEthernetAdapter();
        if (target == null) {
            return new BocloudResult(false, "系统中不存在唯一标识为[" + heaUuid + "]的主机以太网");
        }

        EnhancedBeanUtils.copyNotEmptyProperties(target, heaForUpdate);

        String api = String.format(SPECIFIED_API, serverUuid, heaUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = XmlUtils.toXmlString(target);
        xmlText = XmlUtils.appendHmcNamespace(target.getClass(), xmlText);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);
        Result updateResult = httpPost(auth, url, header, xmlText);
        if (updateResult.isFailed()) {
            return new BocloudResult(false, updateResult.getMessage());
        }
        V7HostEthernetAdapterEntry response = Optional.ofNullable(updateResult.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7HostEthernetAdapterEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }
}
