package com.bocloud.cmp.provider.hmc.sdk.api.v7;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.hmc.sdk.common.ManagementConsoleEntry;
import com.bocloud.cmp.model.hmc.sdk.common.ManagementConsoleResponse;
import com.bocloud.cmp.model.hmc.sdk.module.ManagedSystemEntry;
import com.bocloud.cmp.model.hmc.sdk.module.ManagedSystemResponse;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcJobType;
import com.bocloud.cmp.provider.hmc.sdk.enums.ManagedSystemQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcJobConstant;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.job.V7JobRequest;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.*;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpHeaders;

import java.util.*;

/**
 * Managed System 概览服务
 * 
 * @author sxp
 * @since May 7, 2018
 */
public class V7ManagedSystemService extends V7BaseService {
    private static final String TYPE = "ManagedSystem";
    private static final String API = "/rest/api/uom/ManagedSystem";
    private static final String UUID_API = "/rest/api/uom/ManagedSystem/%s";
    private static final String QUICK_PROPERTY_API = "/rest/api/uom/ManagedSystem/%s/quick/%s";
    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.uom+xml; Type=ManagedSystem";

    /**
     * 查询HMC控制台管理的所有系统信息
     * 
     * @author sxp
     * @since May 7, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth) {
        Result result = httpGet(auth, auth.getEndpoint() + API, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7ManagedSystemResponse response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7ManagedSystemResponse.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 查询HMC控制台管理的指定系统信息
     * 
     * @author sxp
     * @since May 7, 2018
     */
    public static BocloudResult getByUuid(HmcAuthInfo auth, String uuid) {
        String api = String.format(UUID_API, uuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7ManagedSystemEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7ManagedSystemEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }


    /**
     * v7版本服务器状态查询接口无数据返回，这边根据uuid查询出状态返回
     *
     * @author sxp
     * @since May 7, 2018
     */
    public static BocloudResult getStateByUuid(HmcAuthInfo auth, String uuid) {
        String api = String.format(UUID_API, uuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7ManagedSystemEntry responseV7 = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7ManagedSystemEntry.class, msg))
                .orElse(null);
        ManagedSystemEntry response = JSON.parseObject(JSON.toJSONString(responseV7), ManagedSystemEntry.class);
        return new BocloudResult(true, response.getContent().getManagedSystem().getState(), "执行成功");
    }

    /**
     * 快速查询指定系统的某个属性信息
     * <p>
     * 只获取指定属性的值
     * </p>
     * 
     * @author sxp
     * @since May 7, 2018
     */
    public static BocloudResult getQuickProperty(HmcAuthInfo auth, String uuid, ManagedSystemQuickProperty property) {
        String api = String.format(QUICK_PROPERTY_API, uuid, property.name());
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        String value = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace("\"", ""))
                .orElse(null);
        return new BocloudResult(true, value, "执行成功");
    }

    /**
     * 系统开机
     * 
     * @author sxp
     * @since Jun 7, 2018
     */
    public static BocloudResult powerOnManagedSystem(HmcAuthInfo auth, String managedSystemUuid) {
        return V7JobService.powerOnManagedSystem(auth, managedSystemUuid);
    }

    /**
     * 系统开机
     *
     * @author sxp
     * @since Jun 7, 2018
     */
    public static BocloudResult powerOffManagedSystem(HmcAuthInfo auth, String managedSystemUuid) {
        return V7JobService.powerOffManagedSystem(auth, managedSystemUuid);
    }

    /**
     * 修改主机信息
     * @param auth
     * @param managedSystemUuid
     * @param sourceManagedSystem
     * @return
     */
    public static BocloudResult update(HmcAuthInfo auth, String managedSystemUuid, V7ManagedSystem sourceManagedSystem) {
        BocloudResult getResult = getByUuid(auth, managedSystemUuid);
        if (getResult.isFailed()) {
            return getResult;
        }
        V7ManagedSystemEntry entry = (V7ManagedSystemEntry) getResult.getData();
        Optional<V7ManagedSystem> managedSystemOpt = Optional.ofNullable(entry)
                .map(V7ManagedSystemEntry::getContent)
                .map(V7ManagedSystemContent::getManagedSystem);
        if (!managedSystemOpt.isPresent()) {
            return new BocloudResult(false, "系统中不存在唯一标识为[" + managedSystemUuid + "]的主机");
        }
        V7ManagedSystem managedSystem = managedSystemOpt.get();

        copyProperties(sourceManagedSystem, managedSystem);

        String api = String.format(UUID_API, managedSystemUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = XmlUtils.toXmlString(managedSystem);
        xmlText = XmlUtils.appendHmcNamespace(managedSystem.getClass(), xmlText);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPost(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7ManagedSystemEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7ManagedSystemEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");

    }

    /**
     * 执行服务器job请求
     *
     */
    public static BocloudResult doJobRequest(HmcAuthInfo auth, String uuid, V7JobRequest jobRequest, HmcJobType type) {
        Objects.requireNonNull(jobRequest, "jobRequest");
        Boolean hasJobNs;
        Boolean hasNs2;
        String apiOrigin;
        if (HmcJobConstant.getHasJobNamespaceMap().containsKey(type.name())) {
            hasJobNs = HmcJobConstant.getHasJobNamespaceMap().get(type.name());
        } else {
            return new BocloudResult(false, null, "没有定义的操作类型:" + type.name());
        }

        if (HmcJobConstant.getHasJobNs2Map().containsKey(type.name())) {
            hasNs2 = HmcJobConstant.getHasJobNs2Map().get(type.name());
        } else {
            return new BocloudResult(false, null, "没有定义的操作类型:" + type.name());
        }

        if (HmcJobConstant.getJobRequestResource().containsKey(type.name())) {
            apiOrigin = HmcJobConstant.getJobRequestResource().get(type.name());
        } else {
            return new BocloudResult(false, null, "没有定义的操作类型:" + type.name());
        }

        String api;
        if (type == HmcJobType.ResetConnection_ManagedSystem || type == HmcJobType.RemoveConnection_ManagedSystem) {
            // Api需要ManagedSystem_UUID
            BocloudResult result = V7ManagementConsoleService.get(auth);
            if (result.isFailed()) {
                return new BocloudResult(false, null, "不能获得ManagementConsole uuid:" + result.getErrorMsg());
            }

            ManagementConsoleResponse response = (ManagementConsoleResponse) result.getData();
            List<ManagementConsoleEntry> entries = response.getEntries();

            ManagementConsoleEntry entry;
            if (entries == null || entries.isEmpty() || (entry = entries.get(0)) == null) {
                return new BocloudResult(false, "系统中未获取到HMC控制台信息");
            }

            String consoleUuid = entry.getId();
            if (StringUtils.isEmpty(consoleUuid)) {
                return new BocloudResult(false, "系统中未获取到HMC控制台唯一标识");
            }
            api = String.format(apiOrigin, consoleUuid, uuid);
        } else {
            api = String.format(apiOrigin, uuid);
        }
        return V7JobService.doJobRequest(auth, api, jobRequest, hasJobNs, hasNs2);
    }

    // 支持修改
    // 1. SystemName  系统名称
    // 2. SystemLocation  系统位置
    // 3. Description  描述
    // 4. ServicePartition.Href  服务分区
    // 5. AssociatedIPLConfiguration.PowerOffWhenLastLogicalPartitionIsShutdown  最后一个逻辑分区关闭时断电
    // 6. AssociatedIPLConfiguration.PendingPowerOnSide  供电端
    // 7. AssociatedIPLConfiguration.pendingSystemKeylock  键锁位置
    // 8. AssociatedIPLConfiguration.pendingPowerOnLogicalPartitionStartPolicy  分区启动策略
    private static void copyProperties(V7ManagedSystem sourceManagedSystem, V7ManagedSystem targetManagedSystem) {
        if (sourceManagedSystem.getSystemName() != null) {
            targetManagedSystem.setSystemName(sourceManagedSystem.getSystemName());
        }
        if (sourceManagedSystem.getSystemLocation() != null) {
            if (targetManagedSystem.getSystemLocation() == null) {
                targetManagedSystem.setSystemLocation(new V7SystemLocation());
                targetManagedSystem.getSystemLocation().setKsv("V1_2_0");
            }
            targetManagedSystem.getSystemLocation().setValue(sourceManagedSystem.getSystemLocation().getValue());
        }
        if (sourceManagedSystem.getDescription() != null) {
            if (targetManagedSystem.getDescription() == null) {
                targetManagedSystem.setDescription(new V7Description());
                targetManagedSystem.getDescription().setKsv("V1_2_0");
            }
            targetManagedSystem.getDescription().setValue(sourceManagedSystem.getDescription().getValue());
        }
        if (sourceManagedSystem.getServicePartition() != null) {
            if (targetManagedSystem.getServicePartition() == null) {
                targetManagedSystem.setServicePartition(new V7ServicePartition());
                targetManagedSystem.getServicePartition().setRel("related");
            }
            if (sourceManagedSystem.getServicePartition().getHref() != null) {
                targetManagedSystem.getServicePartition().setHref(sourceManagedSystem.getServicePartition().getHref());
            }
            if (sourceManagedSystem.getServicePartition().getRel() != null) {
                targetManagedSystem.getServicePartition().setRel(sourceManagedSystem.getServicePartition().getRel());
            }
        }
        if (sourceManagedSystem.getAssociatedIPLConfiguration() != null) {
            if (targetManagedSystem.getAssociatedIPLConfiguration() == null) {
                targetManagedSystem.setAssociatedIPLConfiguration(new V7AssociatedIPLConfiguration());
            }
            if (sourceManagedSystem.getAssociatedIPLConfiguration().getPowerOffWhenLastLogicalPartitionIsShutdown() != null) {
                targetManagedSystem.getAssociatedIPLConfiguration().setPowerOffWhenLastLogicalPartitionIsShutdown(
                        sourceManagedSystem.getAssociatedIPLConfiguration().getPowerOffWhenLastLogicalPartitionIsShutdown()
                );
            }
            if (sourceManagedSystem.getAssociatedIPLConfiguration().getPendingPowerOnSide() != null) {
                targetManagedSystem.getAssociatedIPLConfiguration().setPendingPowerOnSide(
                        sourceManagedSystem.getAssociatedIPLConfiguration().getPendingPowerOnSide()
                );
            }
            if (sourceManagedSystem.getAssociatedIPLConfiguration().getPendingSystemKeylock() != null) {
                targetManagedSystem.getAssociatedIPLConfiguration().setPendingSystemKeylock(
                        sourceManagedSystem.getAssociatedIPLConfiguration().getPendingSystemKeylock()
                );
            }
            if (sourceManagedSystem.getAssociatedIPLConfiguration().getPendingPowerOnLogicalPartitionStartPolicy() != null) {
                if (targetManagedSystem.getAssociatedIPLConfiguration().getPendingPowerOnLogicalPartitionStartPolicy() == null) {
                    targetManagedSystem.getAssociatedIPLConfiguration().setPendingPowerOnLogicalPartitionStartPolicy(new V7PendingPowerOnLogicalPartitionStartPolicy());
                    targetManagedSystem.getAssociatedIPLConfiguration().getPendingPowerOnLogicalPartitionStartPolicy().setKsv("V1_3_0");
                }
                if (sourceManagedSystem.getAssociatedIPLConfiguration().getPendingPowerOnLogicalPartitionStartPolicy().getKsv() != null) {
                    targetManagedSystem.getAssociatedIPLConfiguration().getPendingPowerOnLogicalPartitionStartPolicy().setKsv(
                            sourceManagedSystem.getAssociatedIPLConfiguration().getPendingPowerOnLogicalPartitionStartPolicy().getKsv()
                    );
                }
                if (sourceManagedSystem.getAssociatedIPLConfiguration().getPendingPowerOnLogicalPartitionStartPolicy().getValue() != null) {
                    targetManagedSystem.getAssociatedIPLConfiguration().getPendingPowerOnLogicalPartitionStartPolicy().setValue(
                            sourceManagedSystem.getAssociatedIPLConfiguration().getPendingPowerOnLogicalPartitionStartPolicy().getValue()
                    );
                }
            }
        }
    }
}
