package com.bocloud.cmp.provider.hmc.sdk.api.v7;

import com.bocloud.cmp.provider.hmc.sdk.enums.HmcGroupName;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcJobType;
import com.bocloud.cmp.provider.hmc.sdk.enums.VirtualIOServerQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcJobConstant;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcSdkConstant;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.job.V7JobRequest;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.*;
import com.bocloud.cmp.provider.hmc.sdk.util.EnhancedBeanUtils;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpHeaders;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.concurrent.*;

/**
 * 虚拟IO服务器服务
 *
 * @author sxp
 * @since May 11, 2018
 */
public class V7VirtualIOServerService extends V7BaseService {

    private static final Logger logger = LoggerFactory.getLogger(V7VirtualIOServerService.class);

    private static final String TYPE = "VirtualIOServer";
    private static final String API_GET_ALL = "/rest/api/uom/VirtualIOServer";
    private static final String API_GET = "/rest/api/uom/VirtualIOServer/%s";
    private static final String API_SYS_ALL = "/rest/api/uom/ManagedSystem/%s/VirtualIOServer";
    private static final String API_SYS = "/rest/api/uom/ManagedSystem/%s/VirtualIOServer/%s";
    private static final String QUICK_PROPERTY_API = "/rest/api/uom/VirtualIOServer/%s/quick/%s";

    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.uom+xml; Type=VirtualIOServer";

    // HMC790，虚拟适配器那块，如果槽号是任意插槽，接口返回的是65535，这里处理成null，保持和870一致
    private static V7VirtualIOServerEntry setVirtualAdapterSlotNullWhen65535(V7VirtualIOServerEntry vios) {
        Optional.ofNullable(vios)
                .map(V7VirtualIOServerEntry::getContent)
                .map(V7VirtualIOServerContent::getVirtualIOServer)
                .map(V7VirtualIOServer::getVirtualSCSIMappings)
                .map(V7VirtualSCSIMappings::getVirtualScsiMapping)
                .ifPresent(mappings -> mappings.forEach(mapping -> {
                    Optional.ofNullable(mapping)
                            .map(V7VirtualSCSIMapping::getServerAdapter)
                            .ifPresent(serverAdapter -> {
                                if ("65535".equalsIgnoreCase(serverAdapter.getVirtualSlotNumber())) {
                                    serverAdapter.setVirtualSlotNumber(null);
                                }
                                if ("65535".equalsIgnoreCase(serverAdapter.getRemoteSlotNumber())) {
                                    serverAdapter.setRemoteSlotNumber(null);
                                }
                            });
                    Optional.ofNullable(mapping)
                            .map(V7VirtualSCSIMapping::getClientAdapter)
                            .ifPresent(clientAdapter -> {
                                if ("65535".equalsIgnoreCase(clientAdapter.getVirtualSlotNumber())) {
                                    clientAdapter.setVirtualSlotNumber(null);
                                }
                                if ("65535".equalsIgnoreCase(clientAdapter.getRemoteSlotNumber())) {
                                    clientAdapter.setRemoteSlotNumber(null);
                                }
                            });
                }));
        return vios;
    }

    // 将RMC状态转换为小写，和HMC870保持一致
    private static V7VirtualIOServerEntry setRmcStatusLowercase(V7VirtualIOServerEntry vios) {
        Optional.ofNullable(vios)
                .map(V7VirtualIOServerEntry::getContent)
                .map(V7VirtualIOServerContent::getVirtualIOServer)
                .ifPresent(v7VirtualIOServer ->
                        Optional.of(v7VirtualIOServer)
                                .map(V7VirtualIOServer::getResourceMonitoringControlState)
                                .map(String::toLowerCase)
                                .ifPresent(v7VirtualIOServer::setResourceMonitoringControlState)
                );
        return vios;
    }


    /**
     * 获取虚拟IO服务器信息
     *
     * @author sxp
     * @since May 11, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth) {
        Result result = httpGet(auth, auth.getEndpoint() + API_GET_ALL, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7VirtualIOServerResponse response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7VirtualIOServerResponse.class, msg))
                .map(rep -> {
                    Optional.of(rep).map(V7VirtualIOServerResponse::getEntries).ifPresent(entries -> {
                        entries.forEach(V7VirtualIOServerService::setVirtualAdapterSlotNullWhen65535);
                        entries.forEach(V7VirtualIOServerService::setRmcStatusLowercase);
                    });
                    return rep;
                }).orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 获取指定的虚拟IO服务器信息
     *
     * @param auth
     * @param viosUuid
     * @return
     * @author sxp
     * @since May 11, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth, String viosUuid) {
        String api = String.format(API_GET, viosUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7VirtualIOServerEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7VirtualIOServerEntry.class, msg))
                .map(V7VirtualIOServerService::setVirtualAdapterSlotNullWhen65535)
                .map(V7VirtualIOServerService::setRmcStatusLowercase)
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 快速查询指定虚拟IO服务器的某个属性信息
     * <p>
     * 只获取指定属性的值
     * </p>
     *
     * @author sxp
     * @since May 11, 2018
     */
    public static BocloudResult getQuickProperty(HmcAuthInfo auth, String uuid, VirtualIOServerQuickProperty property) {
        String api = String.format(QUICK_PROPERTY_API, uuid, property.name());
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        String value = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace("\"", ""))
                .orElse(null);
        return new BocloudResult(true, value, "执行成功");
    }

    /**
     * 创建VIOS
     * <p>
     * 创建VIOS，必须指定在哪个受管系统下创建。
     * </p>
     *
     * @param managedSystemUuid 管理系统的唯一标识
     * @author 007
     * @since Jun 6, 2018
     */
    public static BocloudResult create(HmcAuthInfo auth, String managedSystemUuid, V7VirtualIOServer vios) {
        return create(auth, managedSystemUuid, vios, null);
    }

    /**
     * 创建VIOS(创建VIOS的同时，支持传入概要文件信息)
     *
     * @author sxp
     * @since Jul 5, 2018
     */
    public static BocloudResult create(HmcAuthInfo auth, String managedSystemUuid, V7VirtualIOServer vios, V7LogicalPartitionProfileForUpdate updateProfile) {
        String viosName = Optional.ofNullable(vios).map(V7VirtualIOServer::getPartitionName).orElse("未知的VIOS名称");

        String schemaVersion = vios.getSchemaVersion();
        if (StringUtils.isEmpty(schemaVersion)) {
            vios.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V10);
        }

        String api = String.format(API_SYS_ALL, managedSystemUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = XmlUtils.toXmlString(vios);
        xmlText = XmlUtils.appendHmcNamespace(vios.getClass(), xmlText);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPut(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        Optional<V7VirtualIOServerEntry> serverEntryOpt = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7VirtualIOServerEntry.class, msg));
        if (!serverEntryOpt.isPresent()) {
            return new BocloudResult(false, "创建VIOS失败：执行创建后未返回VIOS信息");
        }
        V7VirtualIOServerEntry serverEntryV7 = serverEntryOpt.get();
        String voisUuid = serverEntryV7.getId();
        String profileUuid = getProfileUuid(serverEntryV7);
        logger.info("创建VIOS[{}]返回该VIOS的UUID：{}", viosName, voisUuid);
        logger.info("创建VIOS[{}]返回该VIOS的概要文件的UUID：{}", viosName, profileUuid);

        // 再更新一下VIOS概要文件
        if (updateProfile != null) {
            if (StringUtils.isEmpty(profileUuid)) {
                // 删除已经创建的VIOS
                logger.error("创建VIOS[{}]后未找到其概要文件的UUID，即将删除该VIOS", viosName);
                delete(auth, managedSystemUuid, voisUuid);
                logger.info("已删除VIOS[{}]", viosName);
                return new BocloudResult(false, "创建VIOS后未找到其概要文件的UUID");
            }
            RunnableFuture<BocloudResult> future = new FutureTask<>(() -> {
                logger.info("正在修改VIOS[{}]的概要文件：{}", viosName, profileUuid);
                BocloudResult updateProfileResult = V7LogicalPartitionProfileService.update(auth, true, voisUuid, profileUuid, updateProfile);
                if (updateProfileResult.isSuccess()) {
                    logger.info("成功修改VIOS[{}]的概要文件：{}", viosName, profileUuid);
                    return new BocloudResult(true, "执行成功");
                }
                //HMC790修改完概要文件可能返回404，且在返回结果中明确说明：Unable to refetch an equivalent CHILD object.  Note that this does not mean that the POST operation failed.  Perhaps a previous operation changed the identity of the desired object.
                if ("404".equals(updateProfileResult.getStatus())) {
                    logger.warn("创建VIOS[{}]后修改概要文件返回404，HMC V7R790发现此问题，但无影响，仅做警告", viosName);
                    return new BocloudResult(true, "执行成功");
                }

                // 删除已经创建的VIOS
                logger.error("创建VIOS[{}]后修改概要文件错误：{}", viosName, updateProfileResult.getMessage());
                delete(auth, managedSystemUuid, voisUuid);
                logger.info("已删除创建成功的VIOS[{}]", viosName);
                return new BocloudResult(false, "更新VIOS概要文件出错：" + updateProfileResult.getMessage());
            });
            new Thread(future).start();
            try {
                BocloudResult bocloudResult = future.get(10, TimeUnit.SECONDS);
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
            } catch (ExecutionException e) {
                logger.error("创建VIOS[{}]后修改概要文件错误：", viosName, e);
                return new BocloudResult(false, e.getMessage());
            } catch (TimeoutException e) {
                logger.info("创建VIOS[{}]后等待修改概要文件超时，放弃等待", viosName);
            }
        }
        V7VirtualIOServerService.setVirtualAdapterSlotNullWhen65535(serverEntryV7);
        V7VirtualIOServerService.setRmcStatusLowercase(serverEntryV7);
        return new BocloudResult(true, serverEntryV7, "执行成功");
    }

    public static BocloudResult update(HmcAuthInfo auth, String managedSystemUuid, String viosUuid, V7VirtualIOServerForUpdate updateVios) {
        return update(auth, managedSystemUuid, viosUuid, updateVios, 1000 * 60 * 10);

    }

    /**
     * 修改VIOS
     *
     * @author lkx
     * @since Jun 7, 2018
     */
    public static BocloudResult update(HmcAuthInfo auth, String managedSystemUuid, String viosUuid, V7VirtualIOServerForUpdate updateVios, int timeout) {
        BocloudResult getResult = get(auth, viosUuid);
        if (getResult.isFailed()) {
            return getResult;
        }

        V7VirtualIOServerEntry entry = (V7VirtualIOServerEntry) getResult.getData();
        Optional<V7VirtualIOServer> virtualIOServerOpt = Optional.ofNullable(entry)
                .map(V7VirtualIOServerEntry::getContent)
                .map(V7VirtualIOServerContent::getVirtualIOServer);
        if (!virtualIOServerOpt.isPresent()) {
            return new BocloudResult(false, "系统中不存在唯一标识为[" + viosUuid + "]的VIOS");
        }
        V7VirtualIOServer virtualIOServer = virtualIOServerOpt.get();

        copyProperties(updateVios, virtualIOServer);

        String api = String.format(API_SYS, managedSystemUuid, viosUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = XmlUtils.toXmlString(virtualIOServer);
        xmlText = XmlUtils.appendHmcNamespace(virtualIOServer.getClass(), xmlText);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPost(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7VirtualIOServerEntry serverEntryV7 = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7VirtualIOServerEntry.class, msg))
                .orElse(null);
        V7VirtualIOServerService.setVirtualAdapterSlotNullWhen65535(serverEntryV7);
        V7VirtualIOServerService.setRmcStatusLowercase(serverEntryV7);
        return new BocloudResult(true, serverEntryV7, "执行成功");
    }

    /**
     * 将VirtualIOServerForUpdate中的属性拷贝至VirtualIOServer
     *
     * @author 007
     * @since Jun 7, 2018
     */
    private static void copyProperties(V7VirtualIOServerForUpdate source, V7VirtualIOServer target) {
        V7PartitionIOConfiguration partitionIOConfiguration = source.getPartitionIOConfiguration();
        V7PartitionMemoryConfiguration partitionMemoryConfiguration = source.getPartitionMemoryConfiguration();
        V7PartitionProcessorConfiguration partitionProcessorConfiguration = source.getPartitionProcessorConfiguration();
        V7HostEthernetAdapterLogicalPorts hostEthernetAdapterLogicalPorts = source.getHostEthernetAdapterLogicalPorts();
        V7VirtualSCSIMappings virtualSCSIMappings = source.getVirtualSCSIMappings();
        V7VirtualFibreChannelMappings virtualFibreChannelMappings = source.getVirtualFibreChannelMappings();

        Set<String> ignoreProperties = new HashSet<>();
        ignoreProperties.add("partitionIOConfiguration");
        ignoreProperties.add("partitionMemoryConfiguration");
        ignoreProperties.add("partitionProcessorConfiguration");
        ignoreProperties.add("hostEthernetAdapterLogicalPorts");
        ignoreProperties.add("virtualSCSIMappings");
        ignoreProperties.add("virtualFibreChannelMappings");
        EnhancedBeanUtils.copyNotEmptyProperties(target, source, ignoreProperties);

        if (partitionIOConfiguration != null) {
            if (target.getPartitionIOConfiguration() == null) {
                target.setPartitionIOConfiguration(partitionIOConfiguration);
            } else {
                V7PartitionIOConfiguration dest = target.getPartitionIOConfiguration();
                EnhancedBeanUtils.copyNotEmptyProperties(dest, partitionIOConfiguration);
            }
        }

        if (partitionMemoryConfiguration != null) {
            if (target.getPartitionMemoryConfiguration() == null) {
                target.setPartitionMemoryConfiguration(partitionMemoryConfiguration);
            } else {
                V7PartitionMemoryConfiguration dest = target.getPartitionMemoryConfiguration();
                EnhancedBeanUtils.copyNotEmptyProperties(dest, partitionMemoryConfiguration);
            }
        }

        if (partitionProcessorConfiguration != null) {
            if (target.getPartitionProcessorConfiguration() == null) {
                target.setPartitionProcessorConfiguration(partitionProcessorConfiguration);
            } else {
                V7PartitionProcessorConfiguration dest = target.getPartitionProcessorConfiguration();
                EnhancedBeanUtils.copyNotEmptyProperties(dest, partitionProcessorConfiguration);
            }
        }

        target.setHostEthernetAdapterLogicalPorts(null);

        if (virtualSCSIMappings != null) {
            if (target.getVirtualSCSIMappings() == null) {
                target.setVirtualSCSIMappings(virtualSCSIMappings);
            } else {
                V7VirtualSCSIMappings dest = target.getVirtualSCSIMappings();
                EnhancedBeanUtils.copyNotEmptyProperties(dest, virtualSCSIMappings);
            }
        }

        if (virtualFibreChannelMappings != null) {
            if (target.getVirtualFibreChannelMappings() == null) {
                target.setVirtualFibreChannelMappings(virtualFibreChannelMappings);
            } else {
                V7VirtualFibreChannelMappings dest = target.getVirtualFibreChannelMappings();
                EnhancedBeanUtils.copyNotEmptyProperties(dest, virtualFibreChannelMappings);
            }
        }

        // 这两句代码没什么卵用，就是为了避免报错
        Optional.of(target).map(V7VirtualIOServer::getStoragePools).ifPresent(o -> o.setGroup(null));
        Optional.of(target).map(V7VirtualIOServer::getVirtualSCSIMappings).ifPresent(o -> o.setGroup(null));
    }

    /**
     * 删除VIOS
     *
     * @param managedSystemUuid 管理系统的唯一标识
     * @param viosUuid          VIOS的唯一标识
     * @author 007
     * @since Jun 6, 2018
     */
    public static BocloudResult delete(HmcAuthInfo auth, String managedSystemUuid, String viosUuid) {
        if (StringUtils.isEmpty(managedSystemUuid)) {
            return new BocloudResult(false, "管理系统的唯一标识不能为空");
        }
        if (StringUtils.isEmpty(viosUuid)) {
            return new BocloudResult(false, "VIOS的唯一标识不能为空");
        }

        String endpoint = auth.getEndpoint();
        String api = String.format(API_SYS, managedSystemUuid, viosUuid);
        String url = endpoint + api;

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpDelete(auth, url, header, null);
        String message = (result.isSuccess()) ? "succeed" : result.getMessage();
        return new BocloudResult(result.isSuccess(), "", message);
    }

    /**
     * VIOS开机(激活)
     *
     * @author sxp
     * @since Jun 8, 2018
     */
    public static BocloudResult powerOnPartition(HmcAuthInfo auth, String viosUuid) {
        return V7JobService.powerOnPartition(auth, HmcGroupName.VirtualIOServer, viosUuid);
    }

    /**
     * VIOS开机(激活)(指定概要文件)
     *
     * @author sxp
     * @since Jun 15, 2018
     */
    public static BocloudResult powerOnPartition(HmcAuthInfo auth, String viosUuid, String profileUuid) {
        return V7JobService.powerOnPartition(auth, HmcGroupName.VirtualIOServer, viosUuid, profileUuid);
    }

    /**
     * VIOS关机
     *
     * @author sxp
     * @since Jun 8, 2018
     */
    public static BocloudResult powerOffPartition(HmcAuthInfo auth, String viosUuid) {
        return V7JobService.powerOffPartition(auth, HmcGroupName.VirtualIOServer, viosUuid);
    }

    public static BocloudResult doJobRequest(HmcAuthInfo auth, String uuid, V7JobRequest jobRequest, HmcJobType type) {
        Boolean hasJobNs;
        Boolean hasNs2;
        String apiOrigin;
        if (HmcJobConstant.getHasJobNamespaceMap().containsKey(type.name())) {
            hasJobNs = HmcJobConstant.getHasJobNamespaceMap().get(type.name());
        } else {
            return new BocloudResult(false, null, "没有定义的操作类型:" + type.name());
        }

        if (HmcJobConstant.getHasJobNs2Map().containsKey(type.name())) {
            hasNs2 = HmcJobConstant.getHasJobNs2Map().get(type.name());
        } else {
            return new BocloudResult(false, null, "没有定义的操作类型:" + type.name());
        }

        if (HmcJobConstant.getJobRequestResource().containsKey(type.name())) {
            apiOrigin = HmcJobConstant.getJobRequestResource().get(type.name());
        } else {
            return new BocloudResult(false, null, "没有定义的操作类型:" + type.name());
        }

        String api = String.format(apiOrigin, uuid);
        return V7JobService.doJobRequest(auth, api, jobRequest, hasJobNs, hasNs2);
    }

    private static String getProfileUuid(V7VirtualIOServerEntry entry) {
        return Optional.ofNullable(entry)
                .map(V7VirtualIOServerEntry::getContent)
                .map(V7VirtualIOServerContent::getVirtualIOServer)
                .map(V7VirtualIOServer::getAssociatedPartitionProfile)
                .map(V7AssociatedPartitionProfile::getHref)
                .map(href -> href.substring(href.lastIndexOf('/') + 1))
                .orElse(null);
    }

    /**
     * 获取VIOS的概要文件唯一标识
     *
     * @author sxp
     * @since Jul 5, 2018
     */
    public static BocloudResult getProfileUuid(HmcAuthInfo auth, String viosUuid) {
        BocloudResult result = get(auth, viosUuid);
        if (result.isFailed()) {
            return result;
        }
        V7VirtualIOServerEntry entry = (V7VirtualIOServerEntry) result.getData();
        String profileUuid = getProfileUuid(entry);
        return new BocloudResult(StringUtils.isNotEmpty(profileUuid), profileUuid, "执行完成");
    }
}
