package com.bocloud.cmp.provider.hmc.sdk.api.v7;

import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.*;
import com.bocloud.cmp.provider.hmc.sdk.util.EnhancedBeanUtils;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.http.HttpHeaders;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * 物理卷组服务
 * 
 * @author sxp
 * @since Jul 4, 2018
 */
public class V7VolumeGroupService extends V7BaseService {
    private static final String TYPE = "VolumeGroup";
    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.uom+xml; type=" + TYPE;
    private static final String API = "/rest/api/uom/VirtualIOServer/%s/" + TYPE;
    private static final String SPECIFIED_API = API + "/%s";

    /**
     * 查询所有卷组
     * 
     * @author sxp
     * @since Jul 4, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth, String viosUuid) {
       String api = String.format(API, viosUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7VolumeGroupResponse response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7VolumeGroupResponse.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 查询指定唯一标识的卷组
     * 
     * @author sxp
     * @since Jul 4, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth, String viosUuid, String volumeGroupUuid) {
        String api = String.format(SPECIFIED_API, viosUuid, volumeGroupUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7VolumeGroupEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7VolumeGroupEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 修改卷组
     * 
     * @author sxp
     * @since Jul 4, 2018
     */
    public static BocloudResult update(HmcAuthInfo auth, String viosUuid, String volumeGroupUuid, V7VolumeGroupForUpdate updateBean) {
        BocloudResult getResult = get(auth, viosUuid);
        if (getResult.isFailed()) {
            return getResult;
        }
        V7VolumeGroupResponse response = (V7VolumeGroupResponse) getResult.getData();

        Optional<V7VolumeGroup> volumeGroupOpt = Optional.ofNullable(response)
                .map(V7VolumeGroupResponse::getEntries)
                .map(entries -> entries.stream()
                        .filter(entry -> volumeGroupUuid.equals(entry.getId()))
                        .findFirst()
                        .orElse(null))
                .map(V7VolumeGroupEntry::getContent)
                .map(V7VolumeGroupContent::getVolumeGroup);
        if (!volumeGroupOpt.isPresent()) {
            return new BocloudResult(false, "系统中不存在唯一标识为[" + volumeGroupUuid + "]的卷组");
        }
        V7VolumeGroup volumeGroup = volumeGroupOpt.get();

        EnhancedBeanUtils.copyNotEmptyProperties(volumeGroup, updateBean);

        String api = String.format(SPECIFIED_API, viosUuid, volumeGroupUuid);
        String url = auth.getEndpoint() + api;

        // 执行跟新的时候，不提交虚拟光驱相关信息，以免报错
        volumeGroup.setMediaRepositories(null);
        String xmlText = XmlUtils.toXmlString(volumeGroup);
        xmlText = XmlUtils.appendHmcNamespace(volumeGroup.getClass(), xmlText);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPost(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7VolumeGroupEntry entry = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7VolumeGroupEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, entry, "执行成功");
    }

    public static BocloudResult delete(HmcAuthInfo auth, String viosUuid, String volumeGroupUuid) {
        String api = String.format(SPECIFIED_API, viosUuid, volumeGroupUuid);
        String url = auth.getEndpoint() + api;
        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);
        Result result = httpDelete(auth, url, header, null);
        return new BocloudResult(result.isSuccess(), result.getMessage(), result.getMessage());
    }

    public static BocloudResult create(HmcAuthInfo auth, String viosUuid, V7VolumeGroup volumeGroup) {
        String xmlText = XmlUtils.toXmlString(volumeGroup);
        xmlText = XmlUtils.appendHmcNamespace(volumeGroup.getClass(), xmlText);

        String api = String.format(API, viosUuid);
        String url = auth.getEndpoint() + api;

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPut(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7VolumeGroupEntry entry = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7VolumeGroupEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, entry, "执行成功");
    }

}
