package com.bocloud.cmp.provider.hmc.sdk.api.v8;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcGroupName;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcJobConstant;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcSdkConstant;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.job.*;
import com.bocloud.cmp.provider.hmc.sdk.util.ResourceUtils;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpHeaders;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 作业服务
 * 
 * @author sxp
 * @since Jun 7, 2018
 */
public class V8JobService extends V8BaseService {
    private static Logger logger = LoggerFactory.getLogger(V8JobService.class);

    private static final String TYPE = "JobResponse";

    /** 查询任务默认超时时间(10分钟) */
    public static final long DEFAULT_TIMEOUT_SECONDS = 60;

    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.web+xml; type=JobRequest";

    /** 查询任务执行状态的接口 */
    private static final String GET_API = "/rest/api/uom/jobs/%s";

    /** 向HMC添加系统 */
    private static final String ADD_MS_API = "/rest/api/uom/ManagementConsole/%s/do/AddManagedSystem";

    /** 从HMC删除系统 */
    private static final String REMOVE_MS_API = "/rest/api/uom/ManagementConsole/%s/ManagedSystem/%s/do/RemoveConnection";

    /** 系统开机 */
    private static final String MS_POWER_ON_API = "/rest/api/uom/ManagedSystem/%s/do/PowerOn";

    /** 系统关机 */
    private static final String MS_POWER_OFF_API = "/rest/api/uom/ManagedSystem/%s/do/PowerOff";

    /** 逻辑分区(/VIOS)开机(激活) */
    private static final String LPAR_POWER_ON_API = "/rest/api/uom/%s/%s/do/PowerOn";

    /** 逻辑分区(/VIOS)关机 */
    private static final String LPAR_POWER_OFF_API = "/rest/api/uom/%s/%s/do/PowerOff";

    /**
     * 增加服务器(ManagedSystem)到HMC平台
     * 
     * @author sxp
     * @since Jun 7, 2018
     */
    public static BocloudResult addManagedSystem(HmcAuthInfo auth, String consoleUuid, String hostIp, String password) {
        String api = String.format(ADD_MS_API, consoleUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = ResourceUtils.readContent("config/addManagedSystem.xml", StandardCharsets.UTF_8);
        xmlText = String.format(xmlText, hostIp, password);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPut(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        JSONObject json = acquireJobId(result);
        return new BocloudResult(true, json, "执行成功");
    }

    /**
     * 从HMC平台删除服务器(ManagedSystem)的连接
     * 
     * @author sxp
     * @since Jun 7, 2018
     */
    public static BocloudResult removeManagedSystemConnection(HmcAuthInfo auth, String consoleUuid, String managedSystemUuid) {
        String api = String.format(REMOVE_MS_API, consoleUuid, managedSystemUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = ResourceUtils.readContent("config/removeManagedSystemConnection.xml", StandardCharsets.UTF_8);
        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPut(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        JSONObject json = acquireJobId(result);
        return new BocloudResult(true, json, "执行成功");
    }

    /**
     * 系统开机
     * 
     * @author sxp
     * @since Jun 7, 2018
     */
    public static BocloudResult powerOnManagedSystem(HmcAuthInfo auth, String managedSystemUuid) {
        String api = String.format(MS_POWER_ON_API, managedSystemUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = ResourceUtils.readContent("config/powerOnManagedSystem.xml", StandardCharsets.UTF_8);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPut(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        JSONObject json = acquireJobId(result);
        return new BocloudResult(true, json, "执行成功");
    }

    /**
     * 系统开机
     * 
     * @author sxp
     * @since Jun 7, 2018
     */
    public static BocloudResult powerOffManagedSystem(HmcAuthInfo auth, String managedSystemUuid) {
        String api = String.format(MS_POWER_OFF_API, managedSystemUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = ResourceUtils.readContent("config/powerOffManagedSystem.xml", StandardCharsets.UTF_8);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPut(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        JSONObject json = acquireJobId(result);
        return new BocloudResult(true, json, "执行成功");
    }

    /**
     * 分区开机(激活)
     * 
     * @param groupName
     *            组名，用来区分AIX逻辑分区和VIOS逻辑分区
     * @author sxp
     * @since Jun 8, 2018
     */
    public static BocloudResult powerOnPartition(HmcAuthInfo auth, HmcGroupName groupName, String lparUuid) {
        return powerOnPartition(auth, groupName, lparUuid, null);
    }

    /**
     * 分区开机(激活)
     * 
     * @param groupName
     *            组名，用来区分AIX逻辑分区和VIOS逻辑分区
     * @author sxp
     * @since Jun 15, 2018
     */
    public static BocloudResult powerOnPartition(HmcAuthInfo auth, HmcGroupName groupName, String partitionUuid, String profileUuid) {
        if (StringUtils.isEmpty(profileUuid)) {
            BocloudResult result;

            switch (groupName) {
            case LogicalPartition:
                result = V8LogicalPartitionService.getProfileUuid(auth, partitionUuid);
                break;
            case VirtualIOServer:
                result = V8VirtualIOServerService.getProfileUuid(auth, partitionUuid);
                break;
            default:
                return new BocloudResult(false, "不支持激活[" + groupName.name() + "]类型的逻辑分区");
            }
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            profileUuid = (String) result.getData();
        }

        if (StringUtils.isEmpty(profileUuid)) {
            return new BocloudResult(false, "未获取到逻辑分区的概要文件的唯一标识");
        }

        String api = String.format(LPAR_POWER_ON_API, groupName.name(), partitionUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = ResourceUtils.readContent("config/powerOnPartition.xml", StandardCharsets.UTF_8);
        xmlText = String.format(xmlText, groupName.name(), profileUuid);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPut(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        JSONObject json = acquireJobId(result);
        return new BocloudResult(true, json, "执行成功");
    }

    /**
     * 分区关机
     * 
     * @param groupName
     *            组名，用来区分AIX逻辑分区和VIOS逻辑分区
     * @author sxp
     * @since Jun 8, 2018
     */
    public static BocloudResult powerOffPartition(HmcAuthInfo auth, HmcGroupName groupName, String lparUuid) {
        String api = String.format(LPAR_POWER_OFF_API, groupName.name(), lparUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = ResourceUtils.readContent("config/powerOffPartition.xml", StandardCharsets.UTF_8);
            xmlText = String.format(xmlText, groupName.name());

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPut(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        JSONObject json = acquireJobId(result);
        return new BocloudResult(true, json, "执行成功");
    }

    public static BocloudResult doJobRequest(HmcAuthInfo auth, String api, V8JobRequest jobRequest, Boolean hasJobNs, Boolean hasNs2) {
        String xmlText = XmlUtils.toXmlString(jobRequest);
        xmlText = XmlUtils.appendJobRequestNamespace(jobRequest.getClass(), xmlText, hasJobNs, hasNs2);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        
        String url = auth.getEndpoint() + api;
        Result result = httpPut(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        JSONObject json = acquireJobId(result);
        return new BocloudResult(true, json, "执行成功");
    }

    /**
     * 从XML中提取jobId放入JSONObject对象返回
     * 
     * @author sxp
     * @since Jun 8, 2018
     */
    private static JSONObject acquireJobId(Result result) {
        return Optional.ofNullable(result).map(Result::getMessage)
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V8JobResultEntry.class, msg))
                .map(V8JobResultEntry::getContent)
                .map(V8JobResultContent::getJobResponse)
                .map(V8InnerJobResponse::getJobId)
                .map(jobId -> {
                    JSONObject json = new JSONObject();
                    json.put(HmcSdkConstant.JOB_ID, jobId);
                    return json;
                }).orElse(null);
    }

    /**
     * 根据任务唯一标识查询任务执行状态
     * 
     * @param jobId
     *            任务唯一标识
     * @param timeoutSeconds
     *            超时时间(单位:秒)
     * @author sxp
     * @since Jun 8, 2018
     */
    public static BocloudResult getJobStatus(HmcAuthInfo auth, String jobId, long timeoutSeconds) {
        if (timeoutSeconds == 0) {
            timeoutSeconds = DEFAULT_TIMEOUT_SECONDS;
        }
        long timeoutMillisSeconds = TimeUnit.SECONDS.toMillis(timeoutSeconds);

        String api = String.format(GET_API, jobId);

        logger.info("Geting job({}) status ...", jobId);
        long start = System.currentTimeMillis();
        while (System.currentTimeMillis() - start < timeoutMillisSeconds) {
            Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
            if (result.isFailed()) {
                logger.warn("Geting job({}) result failed, {}", jobId, result);
                continue;
            }

            Optional<V8InnerJobResponse> jobResponseOpt = Optional.ofNullable(result.getMessage())
                    .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                    .map(msg -> XmlUtils.parseObject(V8JobResultEntry.class, msg))
                    .map(V8JobResultEntry::getContent)
                    .map(V8JobResultContent::getJobResponse);

            Optional<String> statusOpt = jobResponseOpt.map(V8InnerJobResponse::getStatus);
            if (!jobResponseOpt.isPresent() || !statusOpt.isPresent()) {
                logger.warn("Geting job({}) status empty", jobId);
                continue;
            }

            String status = statusOpt.get();
            V8InnerJobResponse jobResponse = jobResponseOpt.get();
            logger.info("job({}) status {}, spent {}s ...", jobId, status, (System.currentTimeMillis() - start) / 1000);
            // 作业执行完成的状态: COMPLETED_OK COMPLETED_WITH_WARNINGS
            // 作业执行失败的状态: COMPLETED_WITH_ERROR FAILED_BEFORE_COMPLETION
            if (HmcJobConstant.JOB_COMPLETE.equals(status) || HmcJobConstant.JOB_COMPLETED_WITH_WARNINGS.equals(status)) {
                logger.warn("Geting job({}) status succeed:{}", jobId, status);
                return new BocloudResult(true, "任务执行成功");
            } else if (HmcJobConstant.JOB_COMPLETED_WITH_ERROR.equals(status) || HmcJobConstant.JOB_FAILED_BEFORE_COMPLETION.equals(status)) {
                List<V8JobParameter> jobParameters = Optional.ofNullable(jobResponse.getResults())
                        .map(V8JobResults::getJobParameters)
                        .orElse(new ArrayList<>());
                Optional<String> errorDataOpt = jobParameters.stream()
                        .filter(param -> "ErrorData".equals(param.getParameterName()))
                        .findFirst().map(V8JobParameter::getParameterValue);
                Optional<String> resultOpt = jobParameters.stream()
                        .filter(param -> "result".equals(param.getParameterName()))
                        .findFirst().map(V8JobParameter::getParameterValue);
                String errorMsg = errorDataOpt.orElse(resultOpt.orElse("未知原因"));
                logger.info("Geting job({}) status failed:{}", jobId, errorMsg);
                return new BocloudResult(false, "任务执行失败:" + errorMsg);
            }

            try {
                TimeUnit.SECONDS.sleep(10);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
            }
        }

        // 在一些状态下，这个job HMC不会返回，所以也不能认定就是失败, 最终还是需要从服务器的状态去判断
        return new BocloudResult(true, "任务执行完成");
    }

}
