package com.bocloud.cmp.provider.hmc.sdk.api.v8;

import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7LogicalPartitionService;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcGroupName;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcJobType;
import com.bocloud.cmp.provider.hmc.sdk.enums.LogicalPartitionQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcJobConstant;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcSdkConstant;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.job.V8JobRequest;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.*;
import com.bocloud.cmp.provider.hmc.sdk.util.EnhancedBeanUtils;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpHeaders;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * 逻辑分区服务
 * <p>
 * Logical Partition provides information about AIX, LINUX or IBMi partitions
 * </p>
 * 
 * @author sxp
 * @since May 8, 2018
 */
public class V8LogicalPartitionService extends V8BaseService {

    private static final Logger logger = LoggerFactory.getLogger(V7LogicalPartitionService.class);

    private static final String PARTITION_TYPE = "LogicalPartition";
    private static final String VFC_TYPE = "VirtualFibreChannelClientAdapter";
    private static final String API_1 = "/rest/api/uom/ManagedSystem/%s/LogicalPartition";
    private static final String API_2 = "/rest/api/uom/ManagedSystem/%s/LogicalPartition/%s";
    private static final String API_3 = "/rest/api/uom/LogicalPartition";
    private static final String API_4 = "/rest/api/uom/LogicalPartition/%s";
    private static final String API_5 = "/rest/api/uom/LogicalPartition/%s/VirtualFibreChannelClientAdapter";
    private static final String API_6 = "/rest/api/uom/LogicalPartition/%s/VirtualFibreChannelClientAdapter/%s";
    private static final String QUICK_PROPERTY_API = "/rest/api/uom/LogicalPartition/%s/quick/%s";

    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.uom+xml; Type=LogicalPartition";

    /**
     * 查询所有指定逻辑分区信息
     * 
     * @author sxp
     * @since May 9, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth) {
        String api = API_3;
        return getResponse(auth, api);
    }

    /**
     * 查询指定逻辑分区信息
     * 
     * @author sxp
     * @since May 9, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth, String logicalPartitionUuid) {
        String api = String.format(API_4, logicalPartitionUuid);
        return getEntry(auth, api);
    }

    /**
     * 查询指定管理系统的所有逻辑分区信息
     * 
     * @author sxp
     * @since May 8, 2018
     */
    public static BocloudResult getByManagedSystemUuid(HmcAuthInfo auth, String managedSystemUuid) {
        String api = String.format(API_1, managedSystemUuid);
        return getResponse(auth, api);
    }

    /**
     * 查询指定管理系统的指定逻辑分区信息
     * 
     * @author sxp
     * @since May 9, 2018
     */
    public static BocloudResult getByManagedSystemUuid(HmcAuthInfo auth, String managedSystemUuid, String logicalPartitionUuid) {
        String api = String.format(API_2, managedSystemUuid, logicalPartitionUuid);
        return getEntry(auth, api);
    }

    private static BocloudResult getResponse(HmcAuthInfo auth, String api) {
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8LogicalPartitionResponse response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(PARTITION_TYPE + ":" + PARTITION_TYPE, PARTITION_TYPE))
                .map(msg -> XmlUtils.parseObject(V8LogicalPartitionResponse.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    private static BocloudResult getEntry(HmcAuthInfo auth, String api) {
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8LogicalPartitionEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(PARTITION_TYPE + ":" + PARTITION_TYPE, PARTITION_TYPE))
                .map(msg -> XmlUtils.parseObject(V8LogicalPartitionEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 快速查询指定逻辑分区的某个属性信息
     * <p>
     * 只获取指定属性的值
     * </p>
     * 
     * @author sxp
     * @since May 9, 2018
     */
    public static BocloudResult getQuickProperty(HmcAuthInfo auth, String uuid, LogicalPartitionQuickProperty property) {
        String api = String.format(QUICK_PROPERTY_API, uuid, property.name());
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        String value = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace("\"", ""))
                .orElse(null);
        return new BocloudResult(true, value, "执行成功");
    }

    /**
     * 创建逻辑分区
     * <p>
     * 创建逻辑分许，必须指定在哪个受管系统下创建。
     * </p>
     * 
     * @param managedSystemUuid
     *            管理系统的唯一标识
     * @author sxp
     * @since Jun 6, 2018
     */
    public static BocloudResult create(HmcAuthInfo auth, String managedSystemUuid, V8LogicalPartition lpar) {
        return create(auth, managedSystemUuid, lpar, null);
    }

    /**
     * 创建逻辑分区(创建逻辑分区的同时，支持传入概要文件信息)
     * 
     * @param managedSystemUuid
     *            管理系统的唯一标识
     * @updateProfile 概要文件需要修改的内容
     * @author sxp
     * @since Jul 2, 2018
     */
    public static BocloudResult create(HmcAuthInfo auth, String managedSystemUuid, V8LogicalPartition lpar, V8LogicalPartitionProfileForUpdate updateProfile) {
        String lparName = Optional.ofNullable(lpar).map(V8LogicalPartition::getPartitionName).orElse("未知的逻辑分区名称");

        V8PartitionIOConfiguration ioConfiguration = lpar.getPartitionIOConfiguration();
        if (null != ioConfiguration) {
            String partitionSchemaVersion = ioConfiguration.getSchemaVersion();
            if (StringUtils.isEmpty(partitionSchemaVersion)) {
                ioConfiguration.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
            }

            if (null != ioConfiguration.getProfileIOSlots()) {
                String profileSchemaVersion = ioConfiguration.getProfileIOSlots().getSchemaVersion();
                if (StringUtils.isEmpty(profileSchemaVersion)) {
                    ioConfiguration.getProfileIOSlots().setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
                }
            }
        }

        String api = String.format(API_1, managedSystemUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = XmlUtils.toXmlString(lpar);
        xmlText = XmlUtils.appendHmcNamespace(lpar.getClass(), xmlText);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPut(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        Optional<V8LogicalPartitionEntry> partitionEntryOpt = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(PARTITION_TYPE + ":" + PARTITION_TYPE, PARTITION_TYPE))
                .map(msg -> XmlUtils.parseObject(V8LogicalPartitionEntry.class, msg));
        if (!partitionEntryOpt.isPresent()) {
            return new BocloudResult(false, "创建逻辑分区失败：执行创建后未返回逻辑分区信息");
        }
        V8LogicalPartitionEntry partitionEntry = partitionEntryOpt.get();
        String lparUuid = partitionEntry.getId();
        String profileUuid = getProfileUuid(partitionEntry);
        logger.info("创建逻辑分区[{}]返回该逻辑分区的UUID：{}", lparName, lparUuid);
        logger.info("创建逻辑分区[{}]返回该逻辑分区的概要文件的UUID：{}", lparName, profileUuid);

        // 再更新一下分区概要文件
        if (updateProfile != null) {
            if (StringUtils.isEmpty(profileUuid)) {
                // 删除已经创建的分区
                delete(auth, managedSystemUuid, lparUuid);
                return new BocloudResult(false, "创建逻辑分区后未找到其概要文件的UUID");
            }
            BocloudResult updateProfileResult = V8LogicalPartitionProfileService.update(auth, false, lparUuid, profileUuid, updateProfile);
            if (updateProfileResult.isFailed()) {
                // 删除已经创建的分区
                delete(auth, managedSystemUuid, lparUuid);
                return new BocloudResult(false, "更新逻辑分区概要文件出错：" + updateProfileResult.getMessage());
            }
        }
        return new BocloudResult(true, partitionEntry, "执行成功");
    }

    /**
     * 删除逻辑分区
     * 
     * @param managedSystemUuid
     *            管理系统的唯一标识
     * @param lparUuid
     *            逻辑分区的唯一标识
     * @author sxp
     * @since Jun 6, 2018
     */
    public static BocloudResult delete(HmcAuthInfo auth, String managedSystemUuid, String lparUuid) {
        String api = String.format(API_2, managedSystemUuid, lparUuid);
        String url = auth.getEndpoint() + api;

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpDelete(auth, url, header, null);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    /**
     * 修改逻辑分区
     * 
     * @author sxp
     * @since Jun 7, 2018
     */
    public static BocloudResult update(HmcAuthInfo auth, String managedSystemUuid, String lparUuid, V8LogicalPartitionForUpdate updateLpar) {
       BocloudResult lparResult = getByManagedSystemUuid(auth, managedSystemUuid, lparUuid);
        if (lparResult.isFailed()) {
            return new BocloudResult(false, lparResult.getMessage());
        }

        V8LogicalPartitionEntry entry = (V8LogicalPartitionEntry) lparResult.getData();
        Optional<V8LogicalPartition> logicalPartitionOpt = Optional.ofNullable(entry)
                .map(V8LogicalPartitionEntry::getContent)
                .map(V8LogicalPartitionContent::getLogicalPartition);
        if (!logicalPartitionOpt.isPresent()) {
            return new BocloudResult(false, "系统中不存在唯一标识为[" + lparUuid + "]的逻辑分区");
        }

        V8LogicalPartition logicalPartition = logicalPartitionOpt.get();

        copyProperties(updateLpar, logicalPartition);

        String api = String.format(API_2, managedSystemUuid, lparUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = XmlUtils.toXmlString(logicalPartition);
        xmlText = XmlUtils.appendHmcNamespace(logicalPartition.getClass(), xmlText);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPost(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8LogicalPartitionEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(PARTITION_TYPE + ":" + PARTITION_TYPE, PARTITION_TYPE))
                .map(msg -> XmlUtils.parseObject(V8LogicalPartitionEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 将LogicalPartitionForUpdate中的属性拷贝至LogicalPartition中
     * 
     * @author sxp
     * @since Jun 7, 2018
     */
    private static void copyProperties(V8LogicalPartitionForUpdate source, V8LogicalPartition target) {
        V8PartitionIOConfiguration partitionIOConfiguration = source.getPartitionIOConfiguration();
        V8PartitionMemoryConfiguration partitionMemoryConfiguration = source.getPartitionMemoryConfiguration();
        V8PartitionProcessorConfiguration partitionProcessorConfiguration = source.getPartitionProcessorConfiguration();
        V8HostEthernetAdapterLogicalPorts hostEthernetAdapterLogicalPorts = source.getHostEthernetAdapterLogicalPorts();

        Set<String> ignoreProperties = new HashSet<>();
        ignoreProperties.add("partitionIOConfiguration");
        ignoreProperties.add("partitionMemoryConfiguration");
        ignoreProperties.add("partitionProcessorConfiguration");
        ignoreProperties.add("hostEthernetAdapterLogicalPorts");
        EnhancedBeanUtils.copyNotEmptyProperties(target, source, ignoreProperties);

        if (partitionIOConfiguration != null) {
            if (target.getPartitionIOConfiguration() == null) {
                target.setPartitionIOConfiguration(partitionIOConfiguration);
            } else {
                V8PartitionIOConfiguration dest = target.getPartitionIOConfiguration();
                EnhancedBeanUtils.copyNotEmptyProperties(dest, partitionIOConfiguration);
            }
        }

        if (partitionMemoryConfiguration != null) {
            if (target.getPartitionMemoryConfiguration() == null) {
                target.setPartitionMemoryConfiguration(partitionMemoryConfiguration);
            } else {
                V8PartitionMemoryConfiguration dest = target.getPartitionMemoryConfiguration();
                EnhancedBeanUtils.copyNotEmptyProperties(dest, partitionMemoryConfiguration);
            }
        }

        if (partitionProcessorConfiguration != null) {
            if (target.getPartitionProcessorConfiguration() == null) {
                target.setPartitionProcessorConfiguration(partitionProcessorConfiguration);
            } else {
                V8PartitionProcessorConfiguration dest = target.getPartitionProcessorConfiguration();
                EnhancedBeanUtils.copyNotEmptyProperties(dest, partitionProcessorConfiguration);
            }
        }

        if (hostEthernetAdapterLogicalPorts != null) {
            if (target.getHostEthernetAdapterLogicalPorts() == null) {
                target.setHostEthernetAdapterLogicalPorts(hostEthernetAdapterLogicalPorts);
            } else {
                V8HostEthernetAdapterLogicalPorts dest = target.getHostEthernetAdapterLogicalPorts();
                EnhancedBeanUtils.copyNotEmptyProperties(dest, hostEthernetAdapterLogicalPorts);
            }
        }
    }

    /**
     * 逻辑分区开机(激活)
     * 
     * @author sxp
     * @since Jun 8, 2018
     */
    public static BocloudResult powerOnPartition(HmcAuthInfo auth, String lparUuid) {
        return V8JobService.powerOnPartition(auth, HmcGroupName.LogicalPartition, lparUuid);
    }

    /**
     * 逻辑分区开机(激活)(指定概要文件)
     * 
     * @author sxp
     * @since Jun 15, 2018
     */
    public static BocloudResult powerOnPartition(HmcAuthInfo auth, String lparUuid, String profileUuid) {
        return V8JobService.powerOnPartition(auth, HmcGroupName.LogicalPartition, lparUuid, profileUuid);
    }

    /**
     * 逻辑分区关机
     * 
     * @author sxp
     * @since Jun 8, 2018
     */
    public static BocloudResult powerOffPartition(HmcAuthInfo auth, String lparUuid) {
        return V8JobService.powerOffPartition(auth, HmcGroupName.LogicalPartition, lparUuid);
    }

    public static BocloudResult doJobRequest(HmcAuthInfo auth, String uuid, V8JobRequest jobRequest, HmcJobType type) {
        Boolean hasJobNs;
        Boolean hasNs2;
        String apiOrigin;
        if (HmcJobConstant.getHasJobNamespaceMap().containsKey(type.name())) {
            hasJobNs = HmcJobConstant.getHasJobNamespaceMap().get(type.name());
        } else {
            return new BocloudResult(false, null, "没有定义的操作类型:" + type.name());
        }

        if (HmcJobConstant.getHasJobNs2Map().containsKey(type.name())) {
            hasNs2 = HmcJobConstant.getHasJobNs2Map().get(type.name());
        } else {
            return new BocloudResult(false, null, "没有定义的操作类型:" + type.name());
        }

        if (HmcJobConstant.getJobRequestResource().containsKey(type.name())) {
            apiOrigin = HmcJobConstant.getJobRequestResource().get(type.name());
        } else {
            return new BocloudResult(false, null, "没有定义的操作类型:" + type.name());
        }

        String api = String.format(apiOrigin, uuid);
        return V8JobService.doJobRequest(auth, api, jobRequest, hasJobNs, hasNs2);
    }

    /**
     * 查询逻辑分区的虚拟光纤适配
     * @param auth
     * @param uuid 逻辑分区uuid
     * @return
     */
    public static BocloudResult getVfc(HmcAuthInfo auth, String uuid) {
        String api = String.format(API_5, uuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8LogicalPartitionVfcResponse response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(VFC_TYPE + ":" + VFC_TYPE, VFC_TYPE))
                .map(msg -> XmlUtils.parseObject(V8LogicalPartitionVfcResponse.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    private static String getProfileUuid(V8LogicalPartitionEntry entry) {
        return Optional.ofNullable(entry)
                .map(V8LogicalPartitionEntry::getContent)
                .map(V8LogicalPartitionContent::getLogicalPartition)
                .map(V8LogicalPartition::getAssociatedPartitionProfile)
                .map(V8AssociatedPartitionProfile::getHref)
                .map(href -> href.substring(href.lastIndexOf('/') + 1))
                .orElse(null);
    }
    
    /**
     * 获取逻辑分区的概要文件唯一标识
     *
     * @author sxp
     * @since Jul 2, 2018
     */
    public static BocloudResult getProfileUuid(HmcAuthInfo auth, String lparUuid) {
        BocloudResult result = V8LogicalPartitionService.get(auth, lparUuid);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8LogicalPartitionEntry entry = (V8LogicalPartitionEntry) result.getData();
        String profileUuid = getProfileUuid(entry);
        return new BocloudResult(StringUtils.isNotEmpty(profileUuid), profileUuid, "执行完成");
    }
}
