package com.bocloud.cmp.provider.hmc.sdk.api.v8;

import com.bocloud.cmp.provider.hmc.sdk.enums.HmcGroupName;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcJobType;
import com.bocloud.cmp.provider.hmc.sdk.enums.VirtualIOServerQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcJobConstant;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcSdkConstant;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.job.V8JobRequest;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.*;
import com.bocloud.cmp.provider.hmc.sdk.util.EnhancedBeanUtils;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpHeaders;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * 虚拟IO服务器服务
 * 
 * @author sxp
 * @since May 11, 2018
 */
public class V8VirtualIOServerService extends V8BaseService {

    private static final Logger logger = LoggerFactory.getLogger(V8LogicalPartitionService.class);

    private static final String TYPE = "VirtualIOServer";
    private static final String API_GET_ALL = "/rest/api/uom/VirtualIOServer";
    private static final String API_GET = "/rest/api/uom/VirtualIOServer/%s";
    private static final String API_SYS_ALL = "/rest/api/uom/ManagedSystem/%s/VirtualIOServer";
    private static final String API_SYS = "/rest/api/uom/ManagedSystem/%s/VirtualIOServer/%s";
    private static final String QUICK_PROPERTY_API = "/rest/api/uom/VirtualIOServer/%s/quick/%s";

    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.uom+xml; Type=VirtualIOServer";

    /**
     * 获取虚拟IO服务器信息
     * 
     * @author sxp
     * @since May 11, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth) {
        Result result = httpGet(auth, auth.getEndpoint() + API_GET_ALL, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8VirtualIOServerResponse response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V8VirtualIOServerResponse.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 获取指定的虚拟IO服务器信息
     * 
     * @author sxp
     * @since May 11, 2018
     * 
     * @param auth
     * @param viosUuid
     * @return
     */
    public static BocloudResult get(HmcAuthInfo auth, String viosUuid) {
        String api = String.format(API_GET, viosUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8VirtualIOServerEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V8VirtualIOServerEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 快速查询指定虚拟IO服务器的某个属性信息
     * <p>
     * 只获取指定属性的值
     * </p>
     * 
     * @author sxp
     * @since May 11, 2018
     */
    public static BocloudResult getQuickProperty(HmcAuthInfo auth, String uuid, VirtualIOServerQuickProperty property) {
        String api = String.format(QUICK_PROPERTY_API, uuid, property.name());
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        String value = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace("\"", ""))
                .orElse(null);
        return new BocloudResult(true, value, "执行成功");
    }

    /**
     * 创建VIOS
     * <p>
     * 创建VIOS，必须指定在哪个受管系统下创建。
     * </p>
     * 
     * @param managedSystemUuid
     *            管理系统的唯一标识
     * @author 007
     * @since Jun 6, 2018
     */
    public static BocloudResult create(HmcAuthInfo auth, String managedSystemUuid, V8VirtualIOServer vios) {
        return create(auth, managedSystemUuid, vios, null);
    }

    /**
     * 创建VIOS(创建VIOS的同时，支持传入概要文件信息)
     * 
     * @author sxp
     * @since Jul 5, 2018
     */
    public static BocloudResult create(HmcAuthInfo auth, String managedSystemUuid, V8VirtualIOServer vios, V8LogicalPartitionProfileForUpdate updateProfile) {
        String lparName = Optional.ofNullable(vios).map(V8VirtualIOServer::getPartitionName).orElse("未知的VIOS名称");

        String schemaVersion = vios.getSchemaVersion();
        if (StringUtils.isEmpty(schemaVersion)) {
            vios.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
        }

        String api = String.format(API_SYS_ALL, managedSystemUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = XmlUtils.toXmlString(vios);
        xmlText = XmlUtils.appendHmcNamespace(vios.getClass(), xmlText);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPut(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        Optional<V8VirtualIOServerEntry> serverEntryOpt = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V8VirtualIOServerEntry.class, msg));
        if (!serverEntryOpt.isPresent()) {
            return new BocloudResult(false, "创建VIOS失败：执行创建后未返回VIOS信息");
        }
        V8VirtualIOServerEntry serverEntryV8 = serverEntryOpt.get();
        String voisUuid = serverEntryV8.getId();
        String profileUuid = getProfileUuid(serverEntryV8);
        logger.info("创建逻辑分区[{}]返回该逻辑分区的UUID：{}", lparName, voisUuid);
        logger.info("创建逻辑分区[{}]返回该逻辑分区的概要文件的UUID：{}", lparName, profileUuid);
        
        // 再更新一下分区概要文件
        if (updateProfile != null) {
            if (StringUtils.isEmpty(profileUuid)) {
                // 删除已经创建的分区
                delete(auth, managedSystemUuid, voisUuid);
                return new BocloudResult(false, "创建VIOS后未找到其概要文件的UUID");
            }
            
            BocloudResult updateProfileResult = V8LogicalPartitionProfileService.update(auth, true, voisUuid, profileUuid, updateProfile);
            if (updateProfileResult.isFailed()) {
                // 删除已经创建的分区
                delete(auth, managedSystemUuid, voisUuid);
                return new BocloudResult(false, "更新VIOS分区概要文件出错：" + updateProfileResult.getMessage());
            }
        }

        return new BocloudResult(true, serverEntryV8, "执行成功");
    }

    public static BocloudResult update(HmcAuthInfo auth, String managedSystemUuid, String viosUuid, V8VirtualIOServerForUpdate updateVios) {
        return update(auth, managedSystemUuid, viosUuid, updateVios, 1000 * 60 * 10);

    }

    /**
     * 修改VIOS
     * 
     * @author lkx
     * @since Jun 7, 2018
     */
    public static BocloudResult update(HmcAuthInfo auth, String managedSystemUuid, String viosUuid, V8VirtualIOServerForUpdate updateVios, int timeout) {
        BocloudResult getResult = get(auth, viosUuid);
        if (getResult.isFailed()) {
            return getResult;
        }

        V8VirtualIOServerEntry entry = (V8VirtualIOServerEntry) getResult.getData();
        Optional<V8VirtualIOServer> virtualIOServerOpt = Optional.ofNullable(entry)
                .map(V8VirtualIOServerEntry::getContent)
                .map(V8VirtualIOServerContent::getVirtualIOServer);
        if (!virtualIOServerOpt.isPresent()) {
            return new BocloudResult(false, "系统中不存在唯一标识为[" + viosUuid + "]的VIOS");
        }
        V8VirtualIOServer virtualIOServer = virtualIOServerOpt.get();

        copyProperties(updateVios, virtualIOServer);

        String api = String.format(API_SYS, managedSystemUuid, viosUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = XmlUtils.toXmlString(virtualIOServer);
        xmlText = XmlUtils.appendHmcNamespace(virtualIOServer.getClass(), xmlText);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPost(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8VirtualIOServerEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V8VirtualIOServerEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 将VirtualIOServerForUpdate中的属性拷贝至VirtualIOServer
     * 
     * @author 007
     * @since Jun 7, 2018
     */
    private static void copyProperties(V8VirtualIOServerForUpdate source, V8VirtualIOServer target) {
        V8PartitionIOConfiguration partitionIOConfiguration = source.getPartitionIOConfiguration();
        V8PartitionMemoryConfiguration partitionMemoryConfiguration = source.getPartitionMemoryConfiguration();
        V8PartitionProcessorConfiguration partitionProcessorConfiguration = source.getPartitionProcessorConfiguration();
        V8HostEthernetAdapterLogicalPorts hostEthernetAdapterLogicalPorts = source.getHostEthernetAdapterLogicalPorts();
        V8VirtualSCSIMappings virtualSCSIMappings = source.getVirtualSCSIMappings();
        V8VirtualFibreChannelMappings virtualFibreChannelMappings = source.getVirtualFibreChannelMappings();

        Set<String> ignoreProperties = new HashSet<>();
        ignoreProperties.add("partitionIOConfiguration");
        ignoreProperties.add("partitionMemoryConfiguration");
        ignoreProperties.add("partitionProcessorConfiguration");
        ignoreProperties.add("hostEthernetAdapterLogicalPorts");
        ignoreProperties.add("virtualSCSIMappings");
        ignoreProperties.add("virtualFibreChannelMappings");
        EnhancedBeanUtils.copyNotEmptyProperties(target, source, ignoreProperties);

        if (partitionIOConfiguration != null) {
            if (target.getPartitionIOConfiguration() == null) {
                target.setPartitionIOConfiguration(partitionIOConfiguration);
            } else {
                V8PartitionIOConfiguration dest = target.getPartitionIOConfiguration();
                EnhancedBeanUtils.copyNotEmptyProperties(dest, partitionIOConfiguration);
            }
        }

        if (partitionMemoryConfiguration != null) {
            if (target.getPartitionMemoryConfiguration() == null) {
                target.setPartitionMemoryConfiguration(partitionMemoryConfiguration);
            } else {
                V8PartitionMemoryConfiguration dest = target.getPartitionMemoryConfiguration();
                EnhancedBeanUtils.copyNotEmptyProperties(dest, partitionMemoryConfiguration);
            }
        }

        if (partitionProcessorConfiguration != null) {
            if (target.getPartitionProcessorConfiguration() == null) {
                target.setPartitionProcessorConfiguration(partitionProcessorConfiguration);
            } else {
                V8PartitionProcessorConfiguration dest = target.getPartitionProcessorConfiguration();
                EnhancedBeanUtils.copyNotEmptyProperties(dest, partitionProcessorConfiguration);
            }
        }

        target.setHostEthernetAdapterLogicalPorts(null);

        if (virtualSCSIMappings != null) {
            if (target.getVirtualSCSIMappings() == null) {
                target.setVirtualSCSIMappings(virtualSCSIMappings);
            } else {
                V8VirtualSCSIMappings dest = target.getVirtualSCSIMappings();
                EnhancedBeanUtils.copyNotEmptyProperties(dest, virtualSCSIMappings);
            }
        }

        if (virtualFibreChannelMappings != null) {
            if (target.getVirtualFibreChannelMappings() == null) {
                target.setVirtualFibreChannelMappings(virtualFibreChannelMappings);
            } else {
                V8VirtualFibreChannelMappings dest = target.getVirtualFibreChannelMappings();
                EnhancedBeanUtils.copyNotEmptyProperties(dest, virtualFibreChannelMappings);
            }
        }
    }

    /**
     * 删除VIOS
     * 
     * @param managedSystemUuid
     *            管理系统的唯一标识
     * @param viosUuid
     *            逻辑分区的唯一标识
     * @author 007
     * @since Jun 6, 2018
     */
    public static BocloudResult delete(HmcAuthInfo auth, String managedSystemUuid, String viosUuid) {
        if (StringUtils.isEmpty(managedSystemUuid)) {
            return new BocloudResult(false, "管理系统的唯一标识不能为空");
        }
        if (StringUtils.isEmpty(viosUuid)) {
            return new BocloudResult(false, "VIOS的唯一标识不能为空");
        }

        String endpoint = auth.getEndpoint();
        String api = String.format(API_SYS, managedSystemUuid, viosUuid);
        String url = endpoint + api;

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpDelete(auth, url, header, null);
        String message = (result.isSuccess()) ? "succeed" : result.getMessage();

        return new BocloudResult(result.isSuccess(), "", message);
    }

    /**
     * VIOS分区开机(激活)
     * 
     * @author sxp
     * @since Jun 8, 2018
     */
    public static BocloudResult powerOnPartition(HmcAuthInfo auth, String viosUuid) {
        return V8JobService.powerOnPartition(auth, HmcGroupName.VirtualIOServer, viosUuid);
    }

    /**
     * VIOS分区开机(激活)(指定概要文件)
     * 
     * @author sxp
     * @since Jun 15, 2018
     */
    public static BocloudResult powerOnPartition(HmcAuthInfo auth, String viosUuid, String profileUuid) {
        return V8JobService.powerOnPartition(auth, HmcGroupName.VirtualIOServer, viosUuid, profileUuid);
    }

    /**
     * VIOS分区关机
     * 
     * @author sxp
     * @since Jun 8, 2018
     */
    public static BocloudResult powerOffPartition(HmcAuthInfo auth, String viosUuid) {
        return V8JobService.powerOffPartition(auth, HmcGroupName.VirtualIOServer, viosUuid);
    }

    public static BocloudResult doJobRequest(HmcAuthInfo auth, String uuid, V8JobRequest jobRequest, HmcJobType type) {
        Boolean hasJobNs;
        Boolean hasNs2;
        String apiOrigin;
        if (HmcJobConstant.getHasJobNamespaceMap().containsKey(type.name())) {
            hasJobNs = HmcJobConstant.getHasJobNamespaceMap().get(type.name());
        } else {
            return new BocloudResult(false, null, "没有定义的操作类型:" + type.name());
        }

        if (HmcJobConstant.getHasJobNs2Map().containsKey(type.name())) {
            hasNs2 = HmcJobConstant.getHasJobNs2Map().get(type.name());
        } else {
            return new BocloudResult(false, null, "没有定义的操作类型:" + type.name());
        }

        if (HmcJobConstant.getJobRequestResource().containsKey(type.name())) {
            apiOrigin = HmcJobConstant.getJobRequestResource().get(type.name());
        } else {
            return new BocloudResult(false, null, "没有定义的操作类型:" + type.name());
        }

        String api = String.format(apiOrigin, uuid);
        return V8JobService.doJobRequest(auth, api, jobRequest, hasJobNs, hasNs2);
    }


    private static String getProfileUuid(V8VirtualIOServerEntry entry) {
        return Optional.ofNullable(entry)
                .map(V8VirtualIOServerEntry::getContent)
                .map(V8VirtualIOServerContent::getVirtualIOServer)
                .map(V8VirtualIOServer::getAssociatedPartitionProfile)
                .map(V8AssociatedPartitionProfile::getHref)
                .map(href -> href.substring(href.lastIndexOf('/') + 1))
                .orElse(null);
    }

    /**
     * 获取逻辑分区的概要文件唯一标识
     *
     * @author sxp
     * @since Jul 5, 2018
     */
    public static BocloudResult getProfileUuid(HmcAuthInfo auth, String viosUuid) {
        BocloudResult result = get(auth, viosUuid);
        if (result.isFailed()) {
            return result;
        }
        V8VirtualIOServerEntry entry = (V8VirtualIOServerEntry) result.getData();
        String profileUuid = getProfileUuid(entry);
        return new BocloudResult(StringUtils.isNotEmpty(profileUuid), profileUuid, "执行完成");
    }
}
