package com.bocloud.cmp.provider.hmc.sdk.api.v8;

import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.*;
import com.bocloud.cmp.provider.hmc.sdk.util.EnhancedBeanUtils;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.http.HttpHeaders;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * 物理卷组服务
 * 
 * @author sxp
 * @since Jul 4, 2018
 */
public class V8VolumeGroupService extends V8BaseService {
    private static final String TYPE = "VolumeGroup";
    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.uom+xml; type=" + TYPE;
    private static final String API = "/rest/api/uom/VirtualIOServer/%s/" + TYPE;
    private static final String SPECIFIED_API = API + "/%s";

    /**
     * 查询所有卷组
     * 
     * @author sxp
     * @since Jul 4, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth, String viosUuid) {
       String api = String.format(API, viosUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8VolumeGroupResponse response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V8VolumeGroupResponse.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 查询指定唯一标识的卷组
     * 
     * @author sxp
     * @since Jul 4, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth, String viosUuid, String volumeGroupUuid) {
        String api = String.format(SPECIFIED_API, viosUuid, volumeGroupUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8VolumeGroupEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V8VolumeGroupEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 修改卷组
     * 
     * @author sxp
     * @since Jul 4, 2018
     */
    public static BocloudResult update(HmcAuthInfo auth, String viosUuid, String volumeGroupUuid, V8VolumeGroupForUpdate updateBean) {
        BocloudResult getResult = get(auth, viosUuid);
        if (getResult.isFailed()) {
            return getResult;
        }
        V8VolumeGroupResponse response = (V8VolumeGroupResponse) getResult.getData();

        Optional<V8VolumeGroup> volumeGroupOpt = Optional.ofNullable(response)
                .map(V8VolumeGroupResponse::getEntries)
                .map(entries -> entries.stream()
                        .filter(entry -> volumeGroupUuid.equals(entry.getId()))
                        .findFirst()
                        .orElse(null))
                .map(V8VolumeGroupEntry::getContent)
                .map(V8VolumeGroupContent::getVolumeGroup);
        if (!volumeGroupOpt.isPresent()) {
            return new BocloudResult(false, "系统中不存在唯一标识为[" + volumeGroupUuid + "]的卷组");
        }
        V8VolumeGroup volumeGroup = volumeGroupOpt.get();

        EnhancedBeanUtils.copyNotEmptyProperties(volumeGroup, updateBean);

        String api = String.format(SPECIFIED_API, viosUuid, volumeGroupUuid);
        String url = auth.getEndpoint() + api;

        // 执行跟新的时候，不提交虚拟光驱相关信息，以免报错
        // 2021-05-12 未能在 CEC 8203-E4A*10F8563 - 针对 Virtual Optical Media 的 RMC 命令失败，发生错误 - Unable to perform operation while the virtual DVD is loaded.针对 Virtual Media Repository 的 RMC 命令失败，发生错误 - DVD repository contains file backed DVD media. Use -f to remove 中标识为 1 的 VIOS 上对 Volume Group 执行操作
        // volumeGroup.setMediaRepositories(null);
        String xmlText = XmlUtils.toXmlString(volumeGroup);
        xmlText = XmlUtils.appendHmcNamespace(volumeGroup.getClass(), xmlText);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPost(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8VolumeGroupEntry entry = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V8VolumeGroupEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, entry, "执行成功");
    }

    public static BocloudResult delete(HmcAuthInfo auth, String viosUuid, String volumeGroupUuid) {
        String api = String.format(SPECIFIED_API, viosUuid, volumeGroupUuid);
        String url = auth.getEndpoint() + api;
        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);
        Result result = httpDelete(auth, url, header, null);
        return new BocloudResult(result.isSuccess(), result.getMessage(), result.getMessage());
    }

    public static BocloudResult create(HmcAuthInfo auth, String viosUuid, V8VolumeGroup volumeGroup) {
        String xmlText = XmlUtils.toXmlString(volumeGroup);
        xmlText = XmlUtils.appendHmcNamespace(volumeGroup.getClass(), xmlText);

        String api = String.format(API, viosUuid);
        String url = auth.getEndpoint() + api;

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPut(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8VolumeGroupEntry entry = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V8VolumeGroupEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, entry, "执行成功");
    }

}
