package com.bocloud.cmp.provider.hmc.sdk.util;

import org.apache.commons.lang3.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.chrono.IsoChronology;
import java.util.Calendar;
import java.util.Date;

/**
 * 日期工具类
 * 
 * @author sxp
 * @since Mar 19, 2018
 */
public class DateUtils {
    /**
     * 一天的毫秒总数
     */
    public static final long DAY_IN_MILLIS = 24L * 3600 * 1000;

    /**
     * 将字符串转成时间
     * 
     * @param dateString
     *            时间的字符串，例如：2016-07-22
     * @param format
     *            字符串匹配模式，例如：yyyy-MM-dd
     * @return 返回dateString对应的Date对象
     * @author sxp
     * @since 2018-03-19
     */
    public static Date stringToDate(String dateString, String format) {
        if (StringUtils.isEmpty(dateString) || StringUtils.isEmpty(format)) {
            return null;
        }

        SimpleDateFormat df = new SimpleDateFormat(format);
        df.setLenient(false);// 将2017-13-01看做是一个非法的日期字符串
        try {
            return df.parse(dateString);
        } catch (ParseException e) {
            throw new RuntimeException(e.getMessage());
        }
    }

    /**
     * 将时间转成字符串
     * 
     * @param date
     *            时间
     * @param format
     *            字符串匹配模式，例如：yyyy-MM-dd
     * @author sxp
     * @since 2018-03-19
     */
    public static String dateToString(Date date, String format) {
        if (date == null || StringUtils.isEmpty(format)) {
            return null;
        }
        SimpleDateFormat df = new SimpleDateFormat(format);
        df.setLenient(false);// 将2017-13-01看做是一个非法的日期字符串
        return df.format(date);
    }

    /**
     * 求两个日期之间相差的天数
     * 
     * @param beginDate
     *            被减数
     * @param endDate
     *            减数
     * @return 返回endDate-beginDate(减数-被减数)的结果，小数部分直接舍去
     * @author sxp
     * @since V1.0 Jul 22, 2016
     */
    public static long getDayDifference(Date beginDate, Date endDate) {
        if (beginDate == null) {
            throw new NullPointerException("传入参数[开始时间]为空");
        }
        if (endDate == null) {
            throw new NullPointerException("传入参数[结束时间]为空");
        }
        long beginMilliseconds = beginDate.getTime();
        long endMilliseconds = endDate.getTime();
        return (endMilliseconds - beginMilliseconds) / DAY_IN_MILLIS;
    }

    /**
     * 求两个日期之间相差的秒数
     * 
     * @param beginDate
     *            被减数
     * @param endDate
     *            减数
     * @return 返回endDate-beginDate(减数-被减数)的结果，小数部分直接舍去
     * @author sxp
     * @since V1.0 Jul 22, 2016
     */
    public static long getSecondDifference(Date beginDate, Date endDate) {
        if (beginDate == null) {
            throw new NullPointerException("传入参数[开始时间]为空");
        }
        if (endDate == null) {
            throw new NullPointerException("传入参数[结束时间]为空");
        }
        long beginMilliseconds = beginDate.getTime();
        long endMilliseconds = endDate.getTime();
        return ((endMilliseconds - beginMilliseconds) / 1000);
    }

    /**
     * 求两个日期之间相差的月数
     * 
     * @param beginDate
     *            被减数
     * @param endDate
     *            减数
     * @return 返回值小数部分直接舍去
     * @author sxp
     * @since V1.0 Aug 21, 2016
     */
    public static long getMonthDifference(Date beginDate, Date endDate) {
        if (beginDate == null) {
            throw new NullPointerException("传入参数[开始时间]为空");
        }
        if (endDate == null) {
            throw new NullPointerException("传入参数[结束时间]为空");
        }

        // yyyy-MM-dd HH:mm:ss
        int year1 = Integer.parseInt(dateToString(beginDate, "yyyy"));
        int year2 = Integer.parseInt(dateToString(endDate, "yyyy"));

        int month1 = Integer.parseInt(dateToString(beginDate, "MM"));
        int month2 = Integer.parseInt(dateToString(endDate, "MM"));

        int day1 = Integer.parseInt(dateToString(beginDate, "dd"));
        int day2 = Integer.parseInt(dateToString(endDate, "dd"));

        int months = (year2 - year1) * 12 + month2 - month1;

        if (day1 == day2) {
            // do nothing
        } else if (day1 == getLastDayIntegerOfMonth(beginDate) && day2 == getLastDayIntegerOfMonth(endDate)) {
            // do nothing
        } else {
            months += (day2 - day1) / 31.00;
        }

        return months;
    }

    /**
     * 是否为闰年
     * <p>
     * (1)非整百年能被4整除的为闰年。（如2004年就是闰年,2010年不是闰年）<br>
     * (2)能被400整除的是闰年。(如2000年是闰年，1900年不是闰年)<br>
     * </p>
     * 
     * @param yearNum
     *            年度的整数，例如：2016
     * @author sxp
     * @since V1.0 Jul 22, 2016
     */
    public static boolean isLeapYear(int yearNum) {
           return IsoChronology.INSTANCE.isLeapYear(yearNum);
    }

    /**
     * 取某个日期所属月份的最后一天(整数形式)
     * 
     * @author sxp
     * @since V1.0 Aug 21, 2016
     */
    public static int getLastDayIntegerOfMonth(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        return cal.getActualMaximum(Calendar.DAY_OF_MONTH);
    }
}
