/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.bocloud.cmp.provider.hmc.sdk.util;

import org.apache.commons.beanutils.*;
import org.apache.commons.beanutils.expression.Resolver;

import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashSet;
import java.util.Map;
import java.util.Objects;
import java.util.Set;

/**
 * 增强型的BeanUtils
 * 
 * @author sxp
 * @since Jun 7, 2018
 */
public class EnhancedBeanUtils {
    private static final BeanUtilsBean INSTANTCE = BeanUtilsBean.getInstance();

    /**
     * 将名称相同的非空属性从origin拷贝至destination
     * <p>
     * 参考代码：org.apache.commons.beanutils.BeanUtilsBean
     * </p>
     * <b>请注意，该拷贝方法为浅拷贝</b> 2018-12-19 lyc
     * 
     * @author sxp
     * @since Jun 7, 2018
     */
    public static void copyNotEmptyProperties(Object dest, Object orig) {
        copyNotEmptyProperties(dest, orig, null);
    }

    /**
     * 将名称相同的非空属性从origin拷贝至destination
     * <p>
     * 参考代码：org.apache.commons.beanutils.BeanUtilsBean
     * </p>
     * <b>请注意，该拷贝方法为浅拷贝</b> 2018-12-19 lyc
     * 
     * @author sxp
     * @since Jun 7, 2018
     */
    public static void copyNotEmptyProperties(Object dest, Object orig, Set<String> ignoreProperties) {
        Objects.requireNonNull(dest, "dest");
        Objects.requireNonNull(orig, "orig");

        if (ignoreProperties == null) {
            ignoreProperties = new HashSet<>(0);
        }

        try {
            // Copy the properties, converting as necessary
            if (orig instanceof DynaBean) {
                final DynaProperty[] origDescriptors = ((DynaBean) orig).getDynaClass().getDynaProperties();
                for (DynaProperty origDescriptor : origDescriptors) {
                    final String name = origDescriptor.getName();
                    if (ignoreProperties.contains(name)) {
                        continue;
                    }
                    // Need to check isReadable() for WrapDynaBean
                    if (getPropertyUtils().isReadable(orig, name) && getPropertyUtils().isWriteable(dest, name)) {
                        final Object value = ((DynaBean) orig).get(name);
                        copyNotEmptyProperty(dest, name, value);
                    }
                }
            } else if (orig instanceof Map) {
                @SuppressWarnings("unchecked")
                final
                // Map properties are always of type <String, Object>
                Map<String, Object> propMap = (Map<String, Object>) orig;
                for (final Map.Entry<String, Object> entry : propMap.entrySet()) {
                    final String name = entry.getKey();
                    if (ignoreProperties.contains(name)) {
                        continue;
                    }
                    if (getPropertyUtils().isWriteable(dest, name)) {
                        copyNotEmptyProperty(dest, name, entry.getValue());
                    }
                }
            } else {
                /* if (orig is a standard JavaBean) */
                final PropertyDescriptor[] origDescriptors = getPropertyUtils().getPropertyDescriptors(orig);
                for (PropertyDescriptor origDescriptor : origDescriptors) {
                    final String name = origDescriptor.getName();
                    if ("class".equals(name)) {
                        // No point in trying to set an object's class
                        continue;
                    }
                    if (ignoreProperties.contains(name)) {
                        continue;
                    }
                    if (getPropertyUtils().isReadable(orig, name) && getPropertyUtils().isWriteable(dest, name)) {
                        try {
                            final Object value = getPropertyUtils().getSimpleProperty(orig, name);
                            copyNotEmptyProperty(dest, name, value);
                        } catch (final NoSuchMethodException e) {
                            // Should not happen
                        }
                    }
                }
            }
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage(), e);
        }
    }

    private static void copyNotEmptyProperty(Object bean, String name, Object value) throws Exception {
        // ----my codes begin----
        if (value == null) {
            return;
        }
        if (value instanceof String) {
            if (((String) value).length() == 0) {
                return;
            }
        }
        // ----my codes ends----

        // Resolve any nested expression to get the actual target bean
        Object target = bean;
        Resolver resolver = getPropertyUtils().getResolver();
        while (resolver.hasNested(name)) {
            try {
                target = getPropertyUtils().getProperty(target, resolver.next(name));
                name = resolver.remove(name);
            } catch (NoSuchMethodException e) {
                return; // Skip this property setter
            }
        }

        // Declare local variables we will require
        String propName = resolver.getProperty(name);
        Class<?> type = null;
        int index = resolver.getIndex(name);
        String key = resolver.getKey(name);

        // Calculate the target property type
        if (target instanceof DynaBean) {
            DynaClass dynaClass = ((DynaBean) target).getDynaClass();
            DynaProperty dynaProperty = dynaClass.getDynaProperty(propName);
            if (dynaProperty == null) {
                return;
            }
            type = dynaProperty.getType();
        } else {
            PropertyDescriptor descriptor = null;
            try {
                descriptor = getPropertyUtils().getPropertyDescriptor(target, name);
                if (descriptor == null) {
                    return;
                }
            } catch (NoSuchMethodException e) {
                return;
            }
            type = descriptor.getPropertyType();
            if (type == null) {
                return;
            }
        }

        // Convert the specified value to the required type and store it
        if (index >= 0) {
            value = convert(value, type.getComponentType());
            try {
                getPropertyUtils().setIndexedProperty(target, propName, index, value);
            } catch (NoSuchMethodException e) {
                throw new InvocationTargetException(e, "Cannot set " + propName);
            }
        } else if (key != null) {
            // Maps do not know what the preferred data type is,
            // so perform no conversions at all
            try {
                getPropertyUtils().setMappedProperty(target, propName, key, value);
            } catch (NoSuchMethodException e) {
                throw new InvocationTargetException(e, "Cannot set " + propName);
            }
        } else {
            value = convert(value, type);
            try {
                getPropertyUtils().setSimpleProperty(target, propName, value);
            } catch (NoSuchMethodException e) {
                throw new InvocationTargetException(e, "Cannot set " + propName);
            }
        }
    }

    /**
     * <p>
     * Convert the value to an object of the specified class (if possible).
     * </p>
     *
     * @param value
     *            Value to be converted (may be null)
     * @param type
     *            Class of the value to be converted to
     * @return The converted value
     *
     * @exception ConversionException
     *                if thrown by an underlying Converter
     */
    protected static Object convert(Object value, Class<?> type) {
        Converter converter = getConvertUtils().lookup(type);
        if (converter != null) {
            return converter.convert(type, value);
        } else {
            return value;
        }
    }

    private static PropertyUtilsBean getPropertyUtils() {
        return INSTANTCE.getPropertyUtils();
    }

    private static ConvertUtilsBean getConvertUtils() {
        return INSTANTCE.getConvertUtils();
    }

}
