package com.bocloud.cmp.provider.hmc.sdk.util;

import com.bocloud.cmp.provider.hmc.sdk.model.HmcSdkConstant;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.Source;
import javax.xml.transform.sax.SAXSource;
import java.io.StringReader;
import java.io.StringWriter;
import java.nio.charset.StandardCharsets;
import java.util.Objects;
import java.util.Optional;

/**
 * 基于JDK-JAXB的Java对象与XML互转工具类
 *
 * @author sxp
 * @since 2018/5/3
 */
public class XmlUtils {
    private static final Logger logger = LoggerFactory.getLogger(XmlUtils.class);

    /**
     * 将java对象转换成XML字符串
     *
     * @author sxp
     * @since 2018/5/3
     */
    public static String toXmlString(Object object) {
        try {
            JAXBContext context = JAXBContext.newInstance(object.getClass());
            Marshaller marshaller = context.createMarshaller();

            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            marshaller.setProperty(Marshaller.JAXB_ENCODING, StandardCharsets.UTF_8.name());

            StringWriter sw = new StringWriter();
            marshaller.marshal(object, sw);

            return sw.toString();
        } catch (Exception e) {
            String message = "将java对象转换为XML字符串出错[" + e.getMessage() + "]";
            logger.error(message, e);
            throw new RuntimeException(message, e);
        }
    }

    /**
     * 将XML字符串转换成java对象
     *
     * @author sxp
     * @since 2018/5/3
     */
    @SuppressWarnings("unchecked")
    public static <T> T parseObject(Class<T> clazz, String text) {
        if (StringUtils.isEmpty(text)) {
            return null;
        }

        String className = null;

        try {
            className = clazz.getSimpleName();

            JAXBContext context = JAXBContext.newInstance(clazz);
            Unmarshaller unmarshaller = context.createUnmarshaller();

            StringReader sr = new StringReader(text);

            // 忽略命名空间
            SAXParserFactory sax = SAXParserFactory.newInstance();
            sax.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true); //首选
            sax.setNamespaceAware(false);
            XMLReader xmlReader = sax.newSAXParser().getXMLReader();
            Source source = new SAXSource(xmlReader, new InputSource(sr));

            Object object = unmarshaller.unmarshal(source);

            sr.close();

            return (T) object;
        } catch (Exception e) {
            String message = "将XML字符串转换成java对象(" + className + ")出错[" + e.getMessage() + "]";
            logger.error(message, e);
            throw new RuntimeException(message, e);
        }
    }

    /**
     * 追加HMC使用的命名空间
     * 
     * <pre>
     *  <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
     *  <LogicalPartition schemaVersion="V1_2_0">
     *      <PartitionName>sxp-test-aix-1</PartitionName>
     *  </LogicalPartition>
     *  
     *  修改为:
     *  
     *  <?xml version="1.0" encoding="UTF-8"?>
     *  <LogicalPartition:LogicalPartition schemaVersion="V1_2_0"
     *  	xmlns=
    "http://www.ibm.com/xmlns/systems/power/firmware/uom/mc/2012_10/"
     *  	xmlns:LogicalPartition=
    "http://www.ibm.com/xmlns/systems/power/firmware/uom/mc/2012_10/">		
     *  	<PartitionName>sxp-test-aix</PartitionName>	
     *  </LogicalPartition:LogicalPartition>
     * </pre>
     * 
     * @param text
     *            不命名空间的XML字符串
     * @param clazz
     *            XML字符串对应的实体类
     * @author sxp
     * @since Jun 6, 2018
     */
    public static <T> String appendHmcNamespace(Class<T> clazz, String text) {
        Objects.requireNonNull(clazz, "clazz");

        if (StringUtils.isEmpty(text)) {
            return null;
        }

        XmlRootElement annotation = clazz.getAnnotation(XmlRootElement.class);
        String className = Optional.ofNullable(annotation).map(XmlRootElement::name).orElse(clazz.getSimpleName());
        String rootName = className + ":" + className;
        text = text.replace("<" + className, "<" + rootName);
        text = text.replace("</" + className, "</" + rootName);

        int index = text.indexOf(rootName);
        if (index < 0) {
            throw new RuntimeException("追加HMC使用的命名空间时，发现XML字符串中没有[" + className + "]，XML字符串为[" + text + "]");
        }
        int len = rootName.length();

        StringBuilder sb = new StringBuilder(text.length() + 250);
        sb.append(text.substring(0, index));
        sb.append(rootName);
        sb.append(" xmlns=\"").append(HmcSdkConstant.NAMESPACE).append("\"");
        sb.append(" xmlns:").append(className).append("=\"").append(HmcSdkConstant.NAMESPACE).append("\"");
        sb.append(text.substring(index + len));

        return sb.toString();
    }

    public static <T> String appendJobRequestNamespace(Class<T> clazz, String text, Boolean hasJobNs, Boolean hasNs2) {
        Objects.requireNonNull(clazz, "clazz");

        if (StringUtils.isEmpty(text)) {
            return null;
        }

        XmlRootElement annotation = clazz.getAnnotation(XmlRootElement.class);
        String className = Optional.ofNullable(annotation).map(XmlRootElement::name).orElse(clazz.getSimpleName());
        String rootName = className;
        if (hasJobNs) {
            rootName = className + ":" + className;
            text = text.replace("<" + className, "<" + rootName);
            text = text.replace("</" + className, "</" + rootName);
        }

        int index = text.indexOf(rootName);
        if (index < 0) {
            throw new RuntimeException("追加HMC使用的命名空间时，发现XML字符串中没有[" + className + "]，XML字符串为[" + text + "]");
        }
        int len = rootName.length();

        StringBuilder sb = new StringBuilder(text.length() + 250);
        sb.append(text.substring(0, index));
        sb.append(rootName);
        if (hasJobNs) {
            sb.append(" xmlns:").append(className).append("=\"").append(HmcSdkConstant.JOBREQUEST_NAMESPACE).append("\"");
        }
        sb.append(" xmlns=\"").append(HmcSdkConstant.JOBREQUEST_NAMESPACE).append("\"");
        if (hasNs2) {
            sb.append(" xmlns:ns2").append("=\"").append(HmcSdkConstant.NS2_NAMESPACE).append("\"");
        }
        sb.append(text.substring(index + len));

        return sb.toString();
    }
}
