package com.bocloud.cmp.provider.hmc.v7;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.hmc.HmcPartitionModel;
import com.bocloud.cmp.model.hmc.HmcProfileLheaModel;
import com.bocloud.cmp.model.hmc.HmcProfileModel;
import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionProfileEntry;
import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionProfileResponse;
import com.bocloud.cmp.provider.HmcAccessableProvider;
import com.bocloud.cmp.provider.hmc.ProfileProvider;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7LogicalPartitionProfileService;
import com.bocloud.cmp.provider.hmc.sdk.helper.v7.V7HmcDriverHelper;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7DedicatedProcessorConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7HostEthernetAdapterLogicalPort;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7HostEthernetAdapterLogicalPorts;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7IOConfigurationInstance;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartitionProfile;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartitionProfileEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartitionProfileForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartitionProfileResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProcessorAttributes;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProfileMemory;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7SharedProcessorConfiguration;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Optional;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.FutureTask;
import java.util.concurrent.RunnableFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

public class HmcProfileProvider extends HmcAccessableProvider implements ProfileProvider {
    private static Logger logger = LoggerFactory.getLogger(HmcProfileProvider.class);

    public HmcProfileProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(HmcPartitionModel partition, HmcProfileModel profile) {
        V7LogicalPartitionProfile hmcProfile = new V7LogicalPartitionProfile();
        hmcProfile.setProfileName(profile.getName());

        boolean isAssignAllResources = Optional.ofNullable(profile.getAssignAllResources()).orElse(false);
        hmcProfile.setAssignAllResources(String.valueOf(isAssignAllResources));

        if (!isAssignAllResources) {
            // 设置处理器
            V7ProcessorAttributes processorAttributes = new V7ProcessorAttributes();
            setProcessorAttributes(processorAttributes, profile);
            hmcProfile.setProcessorAttributes(processorAttributes);

            // 设置内存
            V7ProfileMemory memory = new V7ProfileMemory();
            memory.setMaximumMemory(profile.getMaxMemory().toString());
            memory.setMinimumMemory(profile.getMinMemory().toString());
            memory.setDesiredMemory(profile.getDesMemory().toString());
            hmcProfile.setProfileMemory(memory);

            // 概要文件虚拟适配器设置
            V7IOConfigurationInstance ioConfig = new V7IOConfigurationInstance();
            if (V7HmcDriverHelper.updateProfileIOConfiguration(ioConfig, profile.getMaxVirtualIoslots(),
                    profile.getHmcProfileNetworkAdapters(), profile.getHmcProfileFibreAdapters(),
                    profile.getHmcProfileScsiAdapters(), profile.getHmcProfileIoslot())) {
                hmcProfile.setIoConfigurationInstance(ioConfig);
            }

            // 设置hea
            if (profile.getHmcProfileLheas() != null) {
                V7HostEthernetAdapterLogicalPorts logicalPorts = new V7HostEthernetAdapterLogicalPorts();
                logicalPorts.setHostEthernetAdapterLogicalPortList(new ArrayList<>());
                for (HmcProfileLheaModel hmcProfileLhea : profile.getHmcProfileLheas()) {
                    V7HostEthernetAdapterLogicalPort logicalPort = new V7HostEthernetAdapterLogicalPort();
                    logicalPort.setAdapterID(hmcProfileLhea.getAdapterId() == null ? null :hmcProfileLhea.getAdapterId().toString());
                    logicalPort.setAllowedVLANIDs(StringUtils.isEmpty(hmcProfileLhea.getAllowVlanIds()) ? null : hmcProfileLhea.getAllowVlanIds());
                    logicalPort.setAllowedOSMACAddresses(StringUtils.isEmpty(hmcProfileLhea.getAllowMacAddresses()) ? null : hmcProfileLhea.getAllowMacAddresses());
                    logicalPort.setLogicalPortID(hmcProfileLhea.getLogicalPortId() == null ? null : hmcProfileLhea.getLogicalPortId().toString());
                    logicalPort.setPhysicalPortID(hmcProfileLhea.getPhysicalPortId() == null ? null : hmcProfileLhea.getPhysicalPortId().toString());
                    logicalPort.setPortGroupID(hmcProfileLhea.getPortGroupId() == null ? null : hmcProfileLhea.getPortGroupId().toString());
                    logicalPorts.getHostEthernetAdapterLogicalPortList().add(logicalPort);
                }
                hmcProfile.setHostEthernetAdapterLogicalPorts(logicalPorts);
            }

            // 设置其它
            hmcProfile.setBootMode(profile.getBootMode().toString());
            hmcProfile.setAutoStart(profile.getAutoStart().toString());
            hmcProfile.setConnectionMonitoringEnabled(profile.getConnMonitor().toString());
            hmcProfile.setRedundantErrorPathReportingEnabled(profile.getRedundantErrorPathReport().toString());
            if (null != profile.getProcessorCompatibilityMode()) {
                hmcProfile.setDesiredProcessorCompatibilityMode(profile.getProcessorCompatibilityMode());
            }
        }

        boolean isVios = false;
        if ("Virtual IO Server".equals(partition.getPartitionType())) {
            isVios = true;
        }
        BocloudResult result = V7LogicalPartitionProfileService.create(getAuth(), isVios, partition.getUuid(), hmcProfile);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely create partition");

        V7LogicalPartitionProfileEntry responseV7 = (V7LogicalPartitionProfileEntry) result.getData();
        LogicalPartitionProfileEntry response = JSON.parseObject(JSON.toJSONString(responseV7), LogicalPartitionProfileEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    private void setProcessorAttributes(V7ProcessorAttributes processorAttributes, HmcProfileModel profile) {
        processorAttributes.setSharingMode(profile.getProSharingMode());
        if (profile.getSharedProcessor()) {
            processorAttributes.setHasDedicatedProcessors("false");
            V7SharedProcessorConfiguration sharedConfig = new V7SharedProcessorConfiguration();
            sharedConfig.setDesiredProcessingUnits(profile.getDesProUnits().toString());
            sharedConfig.setDesiredVirtualProcessors(profile.getDesProVirt().toString());
            sharedConfig.setMaximumProcessingUnits(profile.getMaxProUnits().toString());
            sharedConfig.setMaximumVirtualProcessors(profile.getMaxProVirt().toString());
            sharedConfig.setMinimumProcessingUnits(profile.getMinProUnits().toString());
            sharedConfig.setMinimumVirtualProcessors(profile.getMinProVirt().toString());
            sharedConfig.setSharedProcessorPoolID(profile.getSharedProPoolId().toString());
            if (profile.getUncappedWeight() != null) {
                sharedConfig.setUncappedWeight(profile.getUncappedWeight().toString());
            }

            processorAttributes.setSharedProcessorConfiguration(sharedConfig);
        } else {
            processorAttributes.setHasDedicatedProcessors("true");
            V7DedicatedProcessorConfiguration dedicatedConfig = new V7DedicatedProcessorConfiguration();
            dedicatedConfig.setDesiredProcessors(profile.getDesPro().toString());
            dedicatedConfig.setMaximumProcessors(profile.getMaxPro().toString());
            dedicatedConfig.setMinimumProcessors(profile.getMinPro().toString());

            processorAttributes.setDedicatedProcessorConfiguration(dedicatedConfig);
        }
    }

    /**
     * Hmc概要文件编辑
     * 
     * @param partitionUuid
     * @return
     */
    @Override
    public BocloudResult update(HmcProfileModel profile, boolean isVIOS, String partitionUuid, String profileUuid) {
        V7LogicalPartitionProfileForUpdate updateProfile = new V7LogicalPartitionProfileForUpdate();
        if (null != profile.getName()) {
            updateProfile.setProfileName(profile.getName());
        }
        if (null != profile.getConnMonitor()) {
            updateProfile.setConnectionMonitoringEnabled(profile.getConnMonitor().toString());
        }
        if (null != profile.getBootMode()) {
            updateProfile.setBootMode(profile.getBootMode());
        }
        if (null != profile.getAutoStart()) {
            updateProfile.setAutoStart(profile.getAutoStart().toString());
        }
        if (null != profile.getRedundantErrorPathReport()) {
            updateProfile.setRedundantErrorPathReportingEnabled(profile.getRedundantErrorPathReport().toString());
        }
        if (null != profile.getProcessorCompatibilityMode()) {
            updateProfile.setDesiredProcessorCompatibilityMode(profile.getProcessorCompatibilityMode());
        }

        boolean isAssignAllResources = Optional.ofNullable(profile.getAssignAllResources()).orElse(false);
        updateProfile.setAssignAllResources(String.valueOf(isAssignAllResources));

        if (!isAssignAllResources) {
            // 设置处理器
            if (null != profile.getSharedProcessor()) {
                V7ProcessorAttributes processorAttributes = new V7ProcessorAttributes();
                setProcessorAttributes(processorAttributes, profile);
                updateProfile.setProcessorAttributes(processorAttributes);
            }

            // 设置内存
            // 内存如果要修改的话，所有的值都要传过来
            if (null != profile.getMaxMemory()) {
                V7ProfileMemory memory = new V7ProfileMemory();
                memory.setMinimumMemory(profile.getMinMemory().toString());
                memory.setMaximumMemory(profile.getMaxMemory().toString());
                memory.setDesiredMemory(profile.getDesMemory().toString());
                memory.setBarrierSynchronizationRegisterArrayCount(profile.getMemoryBsrCount().toString());
                memory.setMinimumHugePageCount(profile.getMemoryMinPageCount().toString());
                memory.setDesiredHugePageCount(profile.getMemoryDesPageCount().toString());
                memory.setMaximumHugePageCount(profile.getMemoryMaxPageCount().toString());
                updateProfile.setProfileMemory(memory);
            }

            V7IOConfigurationInstance ioConfig = new V7IOConfigurationInstance();
            if (V7HmcDriverHelper.updateProfileIOConfiguration(ioConfig, profile.getMaxVirtualIoslots(),
                    profile.getHmcProfileNetworkAdapters(), profile.getHmcProfileFibreAdapters(),
                    profile.getHmcProfileScsiAdapters(), profile.getHmcProfileIoslot())) {
                updateProfile.setIoConfigurationInstance(ioConfig);
            }

            // 主机以太网逻辑端口设置
            V7HostEthernetAdapterLogicalPorts logicalPorts = new V7HostEthernetAdapterLogicalPorts();
            if (null != profile.getHmcProfileLheas()) {
                V7HmcDriverHelper.updateProfileHEALogicalPort(logicalPorts, profile.getHmcProfileLheas());
                updateProfile.setHostEthernetAdapterLogicalPorts(logicalPorts);
            }
        }

        RunnableFuture<BocloudResult> future = new FutureTask<>(() -> {
            BocloudResult bocloudResult = V7LogicalPartitionProfileService.update(getAuth(), isVIOS, partitionUuid, profileUuid, updateProfile);
            if (bocloudResult.isSuccess()) {
                logger.info("成功修改概要文件：{}", profile.getName());
                return new BocloudResult(true, "执行成功");
            }
            //HMC790修改完概要文件可能返回404，且在返回结果中明确说明：Unable to refetch an equivalent CHILD object.  Note that this does not mean that the POST operation failed.  Perhaps a previous operation changed the identity of the desired object.
            if ("404".equals(bocloudResult.getStatus())) {
                logger.warn("修改概要文件[{}]返回404，HMC V7R790发现此问题，但无影响，仅做警告", profile.getName());
                return new BocloudResult(true, "执行成功");
            }
            return new BocloudResult(false, bocloudResult.getMessage());
        });
        new Thread(future).start();
        try {
            BocloudResult bocloudResult = future.get(30, TimeUnit.SECONDS);
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
        } catch (ExecutionException e) {
            logger.error("修改概要文件[{}]错误：", profile.getName(), e);
            return new BocloudResult(false, e.getMessage());
        } catch (TimeoutException e) {
            logger.info("修改概要文件[{}]错误超时，放弃等待", profile.getName());
        }

        BocloudResult bocloudResult = V7LogicalPartitionProfileService.getByLogicalPartitionUuid(getAuth(), partitionUuid, profileUuid);
        V7LogicalPartitionProfileEntry responseV7 = (V7LogicalPartitionProfileEntry) bocloudResult.getData();
        LogicalPartitionProfileEntry response = JSON.parseObject(JSON.toJSONString(responseV7), LogicalPartitionProfileEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * Hmc分区概要文件删除
     * 
     * @param partitionUuid
     * @return
     */
    @Override
    public BocloudResult delete(boolean isVios, String partitionUuid, String profileUuid) {
        BocloudResult result = V7LogicalPartitionProfileService.delete(getAuth(), isVios, partitionUuid, profileUuid);
        logger.info("Hmc driver completely delete partition profile");
        return result;
    }

    /**
     * 获得Hmc分区概要文件
     * 
     * @param partitionUuid
     * @return
     */
    @Override
    public BocloudResult getProfile(boolean isVios, String partitionUuid, String profileUuid) {
        BocloudResult result;
        if (isVios) {
            result = V7LogicalPartitionProfileService.getByVirtualIOServerUuid(getAuth(), partitionUuid, profileUuid);
        } else {
            result = V7LogicalPartitionProfileService.getByLogicalPartitionUuid(getAuth(), partitionUuid, profileUuid);
        }
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely get partition profile");
        V7LogicalPartitionProfileEntry responseV7 = (V7LogicalPartitionProfileEntry) result.getData();
        LogicalPartitionProfileEntry response = JSON.parseObject(JSON.toJSONString(responseV7), LogicalPartitionProfileEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 获得Hmc分区概要文件
     * 
     * @param partitionUuid
     * @return
     */
    @Override
    public BocloudResult getProfile(boolean isVios, String partitionUuid) {
        BocloudResult result;
        if (isVios) {
            result = V7LogicalPartitionProfileService.getByVirtualIOServerUuid(getAuth(), partitionUuid);
        } else {
            result = V7LogicalPartitionProfileService.getByLogicalPartitionUuid(getAuth(), partitionUuid);
        }
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely get partition profile");
        V7LogicalPartitionProfileResponse responseV7 = (V7LogicalPartitionProfileResponse) result.getData();
        LogicalPartitionProfileResponse response = JSON.parseObject(JSON.toJSONString(responseV7), LogicalPartitionProfileResponse.class);
        return new BocloudResult(true, response, "执行成功");
    }
}
