package com.bocloud.cmp.provider;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.horizon.VmwareHorizonToken;
import com.bocloud.cmp.provider.horizon.VmwareHorizonFarmProvider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.Result;
import io.jsonwebtoken.lang.Assert;
import lombok.extern.slf4j.Slf4j;

import java.util.HashMap;
import java.util.Map;

/**
 * vmware云桌面认证类
 *
 * @author HP
 */
@Slf4j
public class VmwareHorizonProvider extends GenericProvider implements SuperProvider {

    private String endpoint;
    private String username;
    private String password;
    private String domain;
    private String version;
    private VmwareHorizonToken token;
    private HttpClient httpClient = new HttpClient(20 * 1000 * 1000, PostDataFormat.RAW);

    public VmwareHorizonProvider(Butler butler) {
        super(butler);
        this.endpoint = butler.getUrl();
        this.username = butler.getUsername();
        this.password = butler.getPassword();
        this.domain = butler.getDomainName();
        this.version = butler.getVersion();
        // 认证云桌面连接服务器
        this.authenticate(username, password, domain);
    }

    /**
     * 认证
     *
     * @param username
     * @param password
     * @param domain
     */
    private void authenticate(String username, String password, String domain) {
        try {
            Map<String, Object> paramMap = new HashMap<>();
            paramMap.put("username", username);
            paramMap.put("password", password);
            paramMap.put("domain", domain);
            Result result = this.doPost("/login", paramMap);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            String message = result.getMessage();
            Assert.notNull(message, "获取access token异常");
            JSONObject token = JSONObject.parseObject(message);
            this.token = new VmwareHorizonToken(token.getString("access_token"), token.getString("refresh_token"));
        } catch (Exception e) {
            log.error("Get error message:{}", e);
        }
    }

    /**
     * 请求头信息
     *
     * @return
     */
    private Map<String, Object> buildHeader() {
        Map<String, Object> header = new HashMap<String, Object>();
        header.put("Content-type", "application/json");
        if (null != token) {
            header.put("Authorization", "Bearer " + token.getAccessToken());
        }
        return header;
    }

    public Result doGet(String url, Map<String, Object> params) {
        try {
            httpClient.setDataFormat(null);
            return this.httpClient.get(buildHeader(), params, endpoint + "/rest" + url);
        } finally {
            this.httpClient.close();
        }
    }

    public Result doPost(String url, Map<String, Object> params) {
        try {
            httpClient.setDataFormat(PostDataFormat.RAW);
            return this.httpClient.post(buildHeader(), params, endpoint + "/rest" + url);
        } finally {
            this.httpClient.close();
        }
    }

    public Result doPut(String url, Map<String, Object> params) {
        try {
            httpClient.setDataFormat(PostDataFormat.RAW);
            return this.httpClient.put(buildHeader(), params, endpoint + "/rest" + url);
        } finally {
            this.httpClient.close();
        }
    }

    public Result doPatch(String url, Map<String, Object> params) {
        try {
            httpClient.setDataFormat(PostDataFormat.RAW);
            return this.httpClient.patch(buildHeader(), params, endpoint + "/rest" + url);
        } finally {
            this.httpClient.close();
        }
    }

    public Result doDelete(String url, Map<String, Object> params) {
        try {
            httpClient.setDataFormat(null);
            return this.httpClient.delete(buildHeader(), params, endpoint + "/rest" + url);
        } finally {
            this.httpClient.close();
        }
    }

    @Override
    public boolean accessable() {
        try {
            // 查询桌面池判断连接是否正常
            Result result = this.doGet("/external/v1/ad-users-or-groups", null);
            return result.isSuccess();
        } catch (Exception e) {
            log.error("Get error message:{}", e);
            return false;
        }
    }

    public String getVersion() {
        return version;
    }

    public static void main(String[] args) {
        Butler butler = new Butler();
//		butler.setUrl("https://192.168.1.159");
//		butler.setUsername("administrator");
//		butler.setPassword("Passw0rd;/");
//		butler.setDomainName("HORIZON");
//		butler.setVersion("7.13");
        butler.setUrl("https://192.168.1.158");
        butler.setUsername("administrator");
        butler.setPassword("Passw0rd;/");
        butler.setDomainName("CMP0");
        butler.setVersion("8.0");
        VmwareHorizonFarmProvider provider = new VmwareHorizonFarmProvider(butler);
        System.out.println(provider.list().getData());
    }

}
