package com.bocloud.cmp.provider;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.model.Butler;
import com.bocloud.common.utils.ListTool;
import com.huaweicloud.sdk.bss.v2.BssClient;
import com.huaweicloud.sdk.bss.v2.region.BssRegion;
import com.huaweicloud.sdk.cbr.v1.CbrClient;
import com.huaweicloud.sdk.cbr.v1.region.CbrRegion;
import com.huaweicloud.sdk.core.ClientBuilder;
import com.huaweicloud.sdk.core.auth.BasicCredentials;
import com.huaweicloud.sdk.core.auth.GlobalCredentials;
import com.huaweicloud.sdk.core.auth.ICredential;
import com.huaweicloud.sdk.core.http.HttpConfig;
import com.huaweicloud.sdk.ecs.v2.EcsClient;
import com.huaweicloud.sdk.ecs.v2.region.EcsRegion;
import com.huaweicloud.sdk.eip.v2.EipClient;
import com.huaweicloud.sdk.eip.v2.region.EipRegion;
import com.huaweicloud.sdk.elb.v2.ElbClient;
import com.huaweicloud.sdk.elb.v2.region.ElbRegion;
import com.huaweicloud.sdk.eps.v1.EpsClient;
import com.huaweicloud.sdk.evs.v2.EvsClient;
import com.huaweicloud.sdk.evs.v2.region.EvsRegion;
import com.huaweicloud.sdk.iam.v3.IamClient;
import com.huaweicloud.sdk.iam.v3.model.Domains;
import com.huaweicloud.sdk.iam.v3.model.KeystoneListAuthDomainsRequest;
import com.huaweicloud.sdk.iam.v3.model.KeystoneListAuthDomainsResponse;
import com.huaweicloud.sdk.iam.v3.model.KeystoneListProjectsRequest;
import com.huaweicloud.sdk.iam.v3.model.KeystoneListProjectsResponse;
import com.huaweicloud.sdk.iam.v3.model.ProjectResult;
import com.huaweicloud.sdk.iam.v3.region.IamRegion;
import com.huaweicloud.sdk.ims.v2.ImsClient;
import com.huaweicloud.sdk.ims.v2.region.ImsRegion;
import com.huaweicloud.sdk.nat.v2.NatClient;
import com.huaweicloud.sdk.nat.v2.region.NatRegion;
import com.huaweicloud.sdk.vpc.v2.VpcClient;
import com.huaweicloud.sdk.vpc.v2.region.VpcRegion;

import lombok.Data;

/**
 * @ Description： @ Author ：丁明威 @ Date ：Created in 16:12 2018/11/15 @ Modified
 * By：
 */
@Data
public class HuaweiProvider extends GenericProvider implements SuperProvider {

	private static final Logger logger = LoggerFactory.getLogger(HuaweiProvider.class);

	private String regionId = "cn-central-220";
	private String accessKey;
	private String secretKey;
	private Butler butler;
	private boolean accessable;
	private String endpoint;
	private HttpConfig httpConfig = null;
	
	public static final int PAGE_SIZE = 1000;

	public HuaweiProvider(Butler butler) {
		super(butler);
		this.butler = butler;
		this.accessKey = butler.getAccessKey();
		this.secretKey = butler.getSecretKey();
		this.regionId = butler.getRegionId() == null ? this.regionId : butler.getRegionId();
		this.endpoint = butler.getUrl();

		httpConfig = HttpConfig.getDefaultHttpConfig();
		httpConfig.setIgnoreSSLVerification(true);
		httpConfig.setTimeout(50000);
		if (!StringUtils.isEmpty(butler.getProxyUri())) {
			String host = butler.getProxyUri().substring(0, butler.getProxyUri().lastIndexOf(":"));
			int port = Integer.valueOf(butler.getProxyUri().substring(butler.getProxyUri().lastIndexOf(":") + 1));
			httpConfig.withProxyHost(host).withProxyPort(port);
		}
	}

	public Map<String, String> regionMap() {
		Map<String, String> regionMap = new HashMap<>();
		try {
			IamClient client = (IamClient) getClient(HuaweiIndex.service.IAM);
			KeystoneListProjectsRequest request = new KeystoneListProjectsRequest();
			KeystoneListProjectsResponse response = client.keystoneListProjects(request);
			if (!ListTool.isEmpty(response.getProjects())) {
				for (ProjectResult projectResult : response.getProjects()) {
					regionMap.put(projectResult.getName(), projectResult.getId());
				}
			}
		} catch (Exception e) {
			logger.error("list regions error:", e);
		}
		return regionMap;
	}

	public String fullEndpoint(String endpoint, HuaweiIndex.service service) {
		if (StringUtils.isEmpty(endpoint)) {
			return null;
		}
		String serviceName = "";
		if (HuaweiIndex.service.IAM == service) {
			serviceName = service.name().toLowerCase() + "-pub";
		} else if (HuaweiIndex.service.EIP_VPC == service || HuaweiIndex.service.ELB == service) {
			serviceName = HuaweiIndex.service.VPC.name().toLowerCase();
		} else {
			serviceName = service.name().toLowerCase();
		}
		return HuaweiIndex.PROTOCOL + serviceName + "." + this.regionId + "." + endpoint;
	}

	public String getProjectId() {
		Map<String, String> regionMap = regionMap();
		return regionMap.get(regionId);
	}

	@Override
	public boolean accessable() {
		try {
			getClient(HuaweiIndex.service.IAM);
			return true;
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("获取华为连接错误 ： " + e);
			return false;
		}
	}

	public Object getClient(HuaweiIndex.service service) {
		ICredential auth = new BasicCredentials().withAk(accessKey).withSk(secretKey);
		ICredential globalCredential = null;
		switch (service) {
		case IAM:
			globalCredential = new GlobalCredentials().withAk(accessKey).withSk(secretKey);
			ClientBuilder<IamClient> iamBuilder = IamClient.newBuilder().withCredential(globalCredential)
					.withHttpConfig(httpConfig);
			if (StringUtils.isEmpty(endpoint)) {
				iamBuilder.withRegion(IamRegion.valueOf(regionId));
			} else {
				iamBuilder.withEndpoint(this.fullEndpoint(endpoint, service));
			}
			return iamBuilder.build();
		case VPC:
			ClientBuilder<VpcClient> vpcBuilder = VpcClient.newBuilder().withHttpConfig(httpConfig);
			if (StringUtils.isEmpty(endpoint)) {
				vpcBuilder.withRegion(VpcRegion.valueOf(regionId));
			} else {
				auth = new BasicCredentials().withAk(accessKey).withSk(secretKey).withProjectId(this.getProjectId());
				vpcBuilder.withEndpoint(this.fullEndpoint(endpoint, service));
			}
			vpcBuilder.withCredential(auth);
			return vpcBuilder.build();
		case ECS:
			ClientBuilder<EcsClient> ecsBuilder = EcsClient.newBuilder().withHttpConfig(httpConfig);
			if (StringUtils.isEmpty(endpoint)) {
				ecsBuilder.withRegion(EcsRegion.valueOf(regionId));
			} else {
				auth = new BasicCredentials().withAk(accessKey).withSk(secretKey).withProjectId(this.getProjectId());
				ecsBuilder.withEndpoint(this.fullEndpoint(endpoint, service));
			}
			ecsBuilder.withCredential(auth);
			return ecsBuilder.build();
		case IMS:
			ClientBuilder<ImsClient> imsBuilder = ImsClient.newBuilder().withHttpConfig(httpConfig);
			if (StringUtils.isEmpty(endpoint)) {
				imsBuilder.withRegion(ImsRegion.valueOf(regionId));
			} else {
				auth = new BasicCredentials().withAk(accessKey).withSk(secretKey).withProjectId(this.getProjectId());
				imsBuilder.withEndpoint(this.fullEndpoint(endpoint, service));
			}
			imsBuilder.withCredential(auth);
			return imsBuilder.build();
		case EVS:
			ClientBuilder<EvsClient> evsBuilder = EvsClient.newBuilder().withHttpConfig(httpConfig);
			if (StringUtils.isEmpty(endpoint)) {
				evsBuilder.withRegion(EvsRegion.valueOf(regionId));
			} else {
				auth = new BasicCredentials().withAk(accessKey).withSk(secretKey).withProjectId(this.getProjectId());
				evsBuilder.withEndpoint(this.fullEndpoint(endpoint, service));
			}
			evsBuilder.withCredential(auth);
			return evsBuilder.build();
		case EIP:
			ClientBuilder<EipClient> eipBuilder = EipClient.newBuilder().withHttpConfig(httpConfig);
			if (StringUtils.isEmpty(endpoint)) {
				eipBuilder.withRegion(EipRegion.valueOf(regionId));
			} else {
				auth = new BasicCredentials().withAk(accessKey).withSk(secretKey).withProjectId(this.getProjectId());
				eipBuilder.withEndpoint(this.fullEndpoint(endpoint, service));
			}
			eipBuilder.withCredential(auth);
			return eipBuilder.build();
		case NAT:
			ClientBuilder<NatClient> netBuilder = NatClient.newBuilder().withHttpConfig(httpConfig);
			if (StringUtils.isEmpty(endpoint)) {
				netBuilder.withRegion(NatRegion.valueOf(regionId));
			} else {
				auth = new BasicCredentials().withAk(accessKey).withSk(secretKey).withProjectId(this.getProjectId());
				netBuilder.withEndpoint(this.fullEndpoint(endpoint, service));
			}
			netBuilder.withCredential(auth);
			return netBuilder.build();
		case ELB:
			ClientBuilder<ElbClient> elbBuilder = ElbClient.newBuilder().withHttpConfig(httpConfig);
			if (StringUtils.isEmpty(endpoint)) {
				elbBuilder.withRegion(ElbRegion.valueOf(regionId));
			} else {
				auth = new BasicCredentials().withAk(accessKey).withSk(secretKey).withProjectId(this.getProjectId());
				elbBuilder.withEndpoint(this.fullEndpoint(endpoint, service));
			}
			elbBuilder.withCredential(auth);
			return elbBuilder.build();
		case EIP_VPC:
			ClientBuilder<EipClient> eipVpcbuilder = EipClient.newBuilder().withHttpConfig(httpConfig);
			if (StringUtils.isEmpty(endpoint)) {
				eipVpcbuilder.withRegion(VpcRegion.valueOf(regionId));
			} else {
				auth = new BasicCredentials().withAk(accessKey).withSk(secretKey).withProjectId(this.getProjectId());
				eipVpcbuilder.withEndpoint(this.fullEndpoint(endpoint, service));
			}
			eipVpcbuilder.withCredential(auth);
			return eipVpcbuilder.build();
		case CBR:
			ClientBuilder<CbrClient> cbrBuilder = CbrClient.newBuilder().withHttpConfig(httpConfig);
			if (StringUtils.isEmpty(endpoint)) {
				cbrBuilder.withRegion(CbrRegion.valueOf(regionId));
			} else {
				auth = new BasicCredentials().withAk(accessKey).withSk(secretKey).withProjectId(this.getProjectId());
				cbrBuilder.withEndpoint(this.fullEndpoint(endpoint, service));
			}
			cbrBuilder.withCredential(auth);
			return cbrBuilder.build();
		case EPS:
			String domainId = getDomainId();
			if (!StringUtils.isEmpty(domainId)) {
				globalCredential = new GlobalCredentials().withAk(accessKey).withSk(secretKey).withDomainId(domainId);
			} else {
				globalCredential = new GlobalCredentials().withAk(accessKey).withSk(secretKey);
			}
			ClientBuilder<EpsClient> epsBuilder = EpsClient.newBuilder().withCredential(globalCredential)
					.withHttpConfig(httpConfig);
			if (StringUtils.isEmpty(endpoint)) {
				epsBuilder.withRegion(IamRegion.valueOf(regionId));
			} else {
				epsBuilder.withEndpoint(this.fullEndpoint(endpoint, service));
			}
			return epsBuilder.build();
		case BSS:
			globalCredential = new GlobalCredentials().withAk(accessKey).withSk(secretKey);
			return BssClient.newBuilder().withCredential(globalCredential).withRegion(BssRegion.valueOf("cn-north-1"))
					.withHttpConfig(httpConfig).build();
		default:
			return null;
		}
	}

	public String getDomainId() {
		String domainId = null;
		try {
			IamClient client = (IamClient) getClient(HuaweiIndex.service.IAM);
			KeystoneListAuthDomainsRequest keystoneListAuthDomainsRequest = new KeystoneListAuthDomainsRequest();
			KeystoneListAuthDomainsResponse keystoneListAuthDomains = client
					.keystoneListAuthDomains(keystoneListAuthDomainsRequest);
			List<Domains> domains = keystoneListAuthDomains.getDomains();
			if (!ListTool.isEmpty(domains)) {
				domainId = domains.get(0).getId();
			}
		} catch (Exception e) {
			logger.error("查询用户DomainId异常：" + e);
		}
		return domainId;
	}

	public static void sleepSomeTime(int sleepTime) {
		try {
			Thread.sleep(sleepTime);
		} catch (InterruptedException e) {
			logger.error("睡眠失败: " + e.getMessage());
		}
	}


	public String getChineseMessage(String message) {
		if (message.contains("No more IP addresses available on network")) {
			return "eip容量不足，无法分配";
		} else if (message.contains("The subnet has already existed in the VPC")) {
			return "该网段已包含在所属VPC中，或者该网段与所属VPC已有子网网段冲突";
		} else {
			return message;
		}
	}
}