package com.bocloud.cmp.provider.backup;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.format.DateTimeFormatter;
import java.util.*;

import com.bocloud.cmp.model.*;
import com.huaweicloud.sdk.cbr.v1.model.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.BackupConvertor;
import com.bocloud.cmp.huaweiconvertor.BackupPolicyConvertor;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.compute.HuaweiSnapshotProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.huaweicloud.sdk.cbr.v1.CbrClient;

/**
 * 华为云存储库
 *
 * @author ymh
 * @时间 2018年11月29日
 */
@Provider("bocloud.cmp.huawei")
public class HuaweiBackupProvider extends HuaweiProvider implements BackupProvider {

	private Butler butler;

	private static DateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", Locale.getDefault());

	private static Logger logger = LoggerFactory.getLogger(HuaweiBackupProvider.class);

	private static BackupPolicyConvertor policyConvertor = new BackupPolicyConvertor();

	private static BackupConvertor backupConvertor = new BackupConvertor();

	public HuaweiBackupProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult list(BackupModel model) {
		try {
			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			ListBackupsRequest request = new ListBackupsRequest();
			// 查询条件
			if (null != model && !StringUtils.isEmpty(model.getCheckpointUuid())) {
				request.setCheckpointId(model.getCheckpointUuid());
			}
			ListBackupsResponse listBackups = client.listBackups(request);
			List<BackupResp> backups = listBackups.getBackups();
			if (ListTool.isEmpty(backups)) {
				return new BocloudResult(true, "备份列表为空！");
			}
			List<BackupModel> convertList = backupConvertor.convertList(backups);
			List<BackupModel> subBackupModels = getSubBackupModels(convertList);
			return new BocloudResult(true, getSnapshotModel(subBackupModels), "查询列表成功");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	public List<BackupModel> getSnapshotModel(List<BackupModel> backupModels) {
		if (null == backupModels) {
			return backupModels;
		}
		HuaweiSnapshotProvider snapshotProvider = new HuaweiSnapshotProvider(butler);
		for (BackupModel backupModel : backupModels) {
			if ("OS::Cinder::Volume".equalsIgnoreCase(backupModel.getResourceType())) {
				SnapshotModel model = new SnapshotModel();
				model.setSnapshotUuid(backupModel.getSnapshotUuid());
				BocloudResult detail = snapshotProvider.detail(model);
				if (detail.isSuccess()) {
					backupModel.setSnapshotModel((SnapshotModel) detail.getData());
				}
			}
		}
		return backupModels;
	}

	public List<BackupModel> getSubBackupModels(List<BackupModel> backupModels) {
		if (null == backupModels) {
			return backupModels;
		}
		List<BackupModel> allBackModels = new ArrayList<>();
		for (BackupModel backupModel : backupModels) {
			allBackModels.add(backupModel);
			if (!"OS::Nova::Server".equalsIgnoreCase(backupModel.getResourceType())) {
				continue;
			}
			BocloudResult detail = this.detail(backupModel);
			if (detail.isFailed()) {
				continue;
			}
			BackupModel serverBackupModel = (BackupModel) detail.getData();
			List<BackupResp> subBackupModels = JSONObject.parseArray(serverBackupModel.getChildren(), BackupResp.class);
			if (ListTool.isEmpty(subBackupModels)) {
				continue;
			}
			for (BackupResp subObject : subBackupModels) {
				BackupModel convertModel = backupConvertor.convertModel(subObject);
				convertModel.setResourceType("SERVER_VOLUME");
				convertModel.setParentResourceUuid(backupModel.getBackupUuid());
				allBackModels.add(convertModel);
			}
		}
		return allBackModels;
	}

	public BocloudResult detail(BackupModel model) {
		try {
			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			ShowBackupRequest request = new ShowBackupRequest();
			request.setBackupId(model.getBackupUuid());
			ShowBackupResponse response = client.showBackup(request);
			BackupDetail backupDetail = response.getBackup();
			if (null == backupDetail) {
				return new BocloudResult(false, "查询备份详情异常！");
			}
			return new BocloudResult(true, convertModel(backupDetail), "查询成功");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "查询失败", e.getMessage());
		}
	}

	private BackupModel convertModel(BackupDetail backupDetail) {
		BackupModel backupModel = new BackupModel();
		if (null != backupDetail) {
			backupModel.setBackupUuid(backupDetail.getId());
			backupModel.setName(backupDetail.getName());
			backupModel.setCheckpointUuid(backupDetail.getCheckpointId());
			backupModel.setResourceType(backupDetail.getResourceType().toString());
			backupModel.setRemark(backupDetail.getDescription());
			backupModel.setCreateTime(stringToDateTime(
				backupDetail.getCreatedAt().format(DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss"))));
			backupModel.setUpdateTime(stringToDateTime(
				backupDetail.getUpdatedAt().format(DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss"))));
			BackupExtendInfo extendInfo = backupDetail.getExtendInfo();
			backupModel.setExtendInfo(JSON.toJSONString(extendInfo));
			if (null != extendInfo) {
				backupModel.setSnapshotUuid(extendInfo.getSnapshotId());
				backupModel.setBootable(extendInfo.getBootable());
			}
			backupModel.setResourceSize(backupDetail.getResourceSize());
			backupModel.setResourceUuid(backupDetail.getResourceId());
			backupModel.setResourceName(backupDetail.getResourceName());
			backupModel.setResourceAz(backupDetail.getResourceAz());
			backupModel.setProjectUuid(backupDetail.getProjectId());
			backupModel.setVaultsUuid(backupDetail.getVaultId());
			backupModel.setImageType(backupDetail.getImageType().getValue());
			backupModel.setStatus(
				backupDetail.getStatus() == null ? null : backupDetail.getStatus().toString().toUpperCase());
			backupModel.setChildren(JSON.toJSONString(backupDetail.getChildren()));
		}
		return backupModel;
	}

	public static Date stringToDateTime(String dateString) {
		Date date = null;
		try {
			if ((dateString != null) && (dateString.length() != 0)) {
				format.setTimeZone(TimeZone.getTimeZone("GMT+0"));
				date = format.parse(dateString);
			}
		} catch (ParseException e) {
		}
		return date;
	}

	@Override
	public BocloudResult remove(BackupModel model) {
		try {
			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			DeleteBackupRequest request = new DeleteBackupRequest();
			request.setBackupId(model.getBackupUuid());
			client.deleteBackup(request);
			return new BocloudResult(true, waitForDelete(model.getBackupUuid()), "删除成功！");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "查询失败", e.getMessage());
		}
	}

	public BocloudResult waitForDelete(String backupUuid) {
		Long startTime = System.currentTimeMillis();
		BackupModel model = new BackupModel();
		model.setBackupUuid(backupUuid);
		while (true) {
			sleepSomeTime(10000);
			BocloudResult detail = this.detail(model);
			JSONObject object = JSONObject.parseObject(JSON.toJSONString(detail.getData()));
			if (null == object) {
				return new BocloudResult(true, "资源删除成功！");
			}
			Long endTime = System.currentTimeMillis();
			// 等待15分钟
			if (endTime - startTime > 15 * 60 * 1000) {
				return new BocloudResult(false, "资源删除成功！");
			}
		}
	}

	/**
	 * 备份恢复
	 */
	@Override
	public BocloudResult restore(BackupModel model) {
		try {
			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			RestoreBackupRequest request = new RestoreBackupRequest();
			request.setBackupId(model.getBackupUuid());
			BackupRestoreReq body = new BackupRestoreReq();
			BackupRestore restore = new BackupRestore();
			String resourceType = model.getResourceType();
			if ("OS::Cinder::Volume".equalsIgnoreCase(resourceType)) {
				restore.setVolumeId(model.getResourceUuid());
			} else if ("OS::Nova::Server".equalsIgnoreCase(resourceType)) {
				List<BackupRestoreServerMapping> mappings = new ArrayList<>();
				List<VolumeModel> volumeModels = model.getVolumeModels();
				for (VolumeModel volumeModel : volumeModels) {
					BackupRestoreServerMapping mapping = new BackupRestoreServerMapping();
					mapping.setBackupId(volumeModel.getBackupUuid());
					mapping.setVolumeId(volumeModel.getVolumeUuid());
					mappings.add(mapping);
				}
				restore.setMappings(mappings);
				restore.setServerId(model.getResourceUuid());
			} else {
				return new BocloudResult(false, "恢复类型错误！");
			}
			body.setRestore(restore);
			request.setBody(body);
			RestoreBackupResponse response = client.restoreBackup(request);
			return waitForBackupRestore(model);
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	public BocloudResult waitForBackupRestore(BackupModel model) {
		Long startTime = System.currentTimeMillis();
		while (true) {
			sleepSomeTime(5000);
			BocloudResult detail = this.detail(model);
			if (detail.isFailed()) {
				return detail;
			}
			BackupModel backupModel = (BackupModel) detail.getData();
			if ("available".equalsIgnoreCase(backupModel.getStatus())) {
				return new BocloudResult(true, backupModel, "备份恢复成功！");
			}
			if ("FAIL".equalsIgnoreCase(backupModel.getStatus())) {
				return new BocloudResult(false, backupModel, "备份恢复失败！");
			}
			Long endTime = System.currentTimeMillis();
			// 等待15分钟
			if (endTime - startTime > 15 * 60 * 1000) {
				return new BocloudResult(false, "备份恢复超时！");
			}
		}
	}

	@Override
	public BocloudResult listBackupPolicy(BackupPolicyModel policyModel) {
		try {
			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			ListPoliciesRequest request = new ListPoliciesRequest();
			if (null != policyModel && !StringUtils.isEmpty(policyModel.getVaultsUuid())) {
				request.setVaultId(policyModel.getVaultsUuid());
			}
			ListPoliciesResponse response = client.listPolicies(request);
			List<Policy> policies = response.getPolicies();
			if (ListTool.isEmpty(policies)) {
				return new BocloudResult(true, "存储库列表为空！");
			}
			return new BocloudResult(true, policyConvertor.convertList(policies), "查询列表成功");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult createPolicy(BackupPolicyModel policyModel) {
		try {

			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			CreatePolicyRequest request = new CreatePolicyRequest();
			PolicyCreate policyCreate = new PolicyCreate();
			policyCreate.setEnabled(policyModel.getEnabled());
			policyCreate.setName(policyModel.getName());
			policyCreate.setOperationType(PolicyCreate.OperationTypeEnum.BACKUP);

			// 策略执行时间规则
			PolicyTriggerReq trigger = new PolicyTriggerReq();
			PolicyTriggerPropertiesReq properties = new PolicyTriggerPropertiesReq();
			properties.setPattern(Collections.singletonList(policyModel.getTrigger()));
			trigger.setProperties(properties);
			policyCreate.setTrigger(trigger);
			// 调度参数

			policyCreate.setOperationDefinition(JSONObject.parseObject(policyModel.getOperationDefinition(), PolicyoODCreate.class));

			PolicyCreateReq policyCreateReq = new PolicyCreateReq();
			policyCreateReq.setPolicy(policyCreate);

			request.setBody(policyCreateReq);
			CreatePolicyResponse response = client.createPolicy(request);

			Policy policy = response.getPolicy();
			if (null == policy) {
				return new BocloudResult(false, "备份策略创建失败！");
			}
			BackupPolicyModel backupPolicyModel = new BackupPolicyConvertor().convertModel(policy);
			backupPolicyModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, backupPolicyModel, "创建成功");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult modifyPolicy(BackupPolicyModel policyModel) {
		try {

			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			UpdatePolicyRequest request = new UpdatePolicyRequest();
			PolicyUpdate policyUpdate = new PolicyUpdate();
			policyUpdate.setEnabled(policyModel.getEnabled());
			policyUpdate.setName(policyModel.getName());


			// 策略执行时间规则
			PolicyTriggerReq trigger = new PolicyTriggerReq();
			PolicyTriggerPropertiesReq properties = new PolicyTriggerPropertiesReq();
			properties.setPattern(Collections.singletonList(policyModel.getTrigger()));
			trigger.setProperties(properties);
			policyUpdate.setTrigger(trigger);
			// 调度参数

			policyUpdate.setOperationDefinition(JSONObject.parseObject(policyModel.getOperationDefinition(), PolicyoODCreate.class));

			PolicyUpdateReq policyUpdateReq = new PolicyUpdateReq();
			policyUpdateReq.setPolicy(policyUpdate);

			request.setBody(policyUpdateReq);
			UpdatePolicyResponse response = client.updatePolicy(request);

			Policy policy = response.getPolicy();
			if (null == policy) {
				return new BocloudResult(false, "备份策略修改失败！");
			}
			BackupPolicyModel backupPolicyModel = new BackupPolicyConvertor().convertModel(policy);
			backupPolicyModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, backupPolicyModel, "修改成功");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "修改失败", e.getMessage());
		}
	}


	@Override
	public BocloudResult removePolicy(BackupPolicyModel policyModel) {
		try {
			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			DeletePolicyRequest request = new DeletePolicyRequest();
			request.setPolicyId(policyModel.getBackupPolicyUuid());
			client.deletePolicy(request);
			return new BocloudResult(true, "删除成功！");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "查询失败", e.getMessage());
		}
	}

}
