package com.bocloud.cmp.provider.backup;

import java.util.ArrayList;
import java.util.List;

import com.huaweicloud.sdk.cbr.v1.model.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.VaultsConvertor;
import com.bocloud.cmp.model.BackupPolicyModel;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VaultsModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.huaweicloud.sdk.cbr.v1.CbrClient;
import com.huaweicloud.sdk.cbr.v1.model.BillingCreate.ObjectTypeEnum;
import com.huaweicloud.sdk.cbr.v1.model.BillingCreate.ProtectTypeEnum;

/**
 *
 * @ClassName: HuaweiVaultsProvider.java
 * @Description: 华为云存储库
 *
 * @author: ymh
 * @date: 2021年3月19日 上午11:03:27
 */
@Provider("bocloud.cmp.huawei")
public class HuaweiVaultsProvider extends HuaweiProvider implements VaultsProvider {

	private Butler butler;

	private static Logger logger = LoggerFactory.getLogger(HuaweiVaultsProvider.class);

	private static VaultsConvertor convertor = new VaultsConvertor();

	public HuaweiVaultsProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult list(VaultsModel model) {
		try {
			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			ListVaultRequest request = new ListVaultRequest();
			ListVaultResponse response = client.listVault(request);
			List<Vault> vaults = response.getVaults();

			if (ListTool.isEmpty(vaults)) {
				return new BocloudResult(false, "存储库列表为空！");
			}
			// 查询备份策略
			List<VaultsModel> vaultsModels = convertor.convertList(vaults);

			HuaweiBackupProvider backupProvider = new HuaweiBackupProvider(butler);
			for (VaultsModel vaultsModel : vaultsModels) {
				vaultsModel.setRegionId(this.getRegionId());
				BocloudResult listBackupPolicy = backupProvider
					.listBackupPolicy(new BackupPolicyModel(vaultsModel.getVaultsUuid()));
				if (listBackupPolicy.isSuccess()) {
					@SuppressWarnings("unchecked")
					List<BackupPolicyModel> backupPolicyModels = (List<BackupPolicyModel>) listBackupPolicy.getData();
					if (!ListTool.isEmpty(backupPolicyModels)) {
						BackupPolicyModel backupPolicyModel = backupPolicyModels.get(0);
						vaultsModel.setBackupPolicyUuid(backupPolicyModel.getBackupPolicyUuid());
					}
				}
			}
			return new BocloudResult(true, vaultsModels, "查询列表成功");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(VaultsModel model) {
		try {
			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			ShowVaultRequest request = new ShowVaultRequest();
			request.setVaultId(model.getVaultsUuid());
			ShowVaultResponse response = client.showVault(request);
			Vault vault = response.getVault();
			if (null == vault) {
				return new BocloudResult<>(false, "资源不存在！");
			}
			VaultsModel convertModel = convertor.convertModel(vault);
			return new BocloudResult(true, convertModel, "查询列表成功");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult create(VaultsModel model) {
		try {

			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			CreateVaultRequest request = new CreateVaultRequest();
			VaultCreateReq body = new VaultCreateReq();
			VaultCreate vault = new VaultCreate();

			BillingCreate billing = new BillingCreate();
			// 规格，崩溃一致性（crash_consistent）或应用一致性（app_consistent）
			billing.setConsistentLevel("crash_consistent");
			billing.setObjectType(ObjectTypeEnum.valueOf(model.getObjectType()));
			// 保护类型：备份（backup）、复制(replication)
			billing.setProtectType(ProtectTypeEnum.BACKUP);
			billing.setSize(model.getVaultSize());
			billing.setConsistentLevel(getAccessKey());
			vault.setBilling(billing);

			vault.setName(model.getName());
			if (!StringUtils.isEmpty(model.getRemark())) {
				vault.setDescription(model.getRemark());
			}
			// 添加资源
			List<ResourceCreate> resources = new ArrayList<>();
			if (!ListTool.isEmpty(model.getResources())) {
				String type = "OS::Cinder::Volume";
				String objectType = model.getObjectType();
				if ("server".equalsIgnoreCase(objectType)) {
					type = "OS::Nova::Server";
				}

				for (String resourceUuid : model.getResources()) {
					ResourceCreate resouorcesItem = new ResourceCreate();
					resouorcesItem.setType(type);
					resouorcesItem.setId(resourceUuid);
					resources.add(resouorcesItem);
				}
			}
			vault.setResources(resources);
			if (!StringUtils.isEmpty(model.getBackupPolicyUuid())) {
				vault.setBackupPolicyId(model.getBackupPolicyUuid());
			}
			// 是否自动扩容
			vault.setAutoExpand(model.getAutoExpand());
			body.setVault(vault);
			request.setBody(body);
			CreateVaultResponse response = client.createVault(request);

			Vault createVault = response.getVault();
			if (null == createVault) {
				return new BocloudResult(false, "存储库创建失败！");
			}
			VaultsModel convertModel = convertor.convertModel(createVault);
			convertModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, convertModel, "创建成功");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	/**
	 * 删除存储库。若删除储存库，将一并删除存储库中的所有备份。
	 */
	@Override
	public BocloudResult remove(VaultsModel model) {
		try {
			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			RemoveVaultResourceRequest request = new RemoveVaultResourceRequest();
			request.setVaultId(model.getVaultsUuid());
			client.removeVaultResource(request);
			return new BocloudResult(true, "删除成功！");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "删除失败：", e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(VaultsModel model) {
		try {
			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			UpdateVaultRequest request = new UpdateVaultRequest();
			request.setVaultId(model.getVaultsUuid());
			VaultUpdateReq body = new VaultUpdateReq();
			VaultUpdate vault = new VaultUpdate();
			if (!StringUtils.isEmpty(model.getName())) {
				vault.setName(model.getName());
			}
			Integer vaultSize = model.getVaultSize();
			if (null != vaultSize && vaultSize > 0) {
				BillingUpdate billing = new BillingUpdate();
				billing.setSize(vaultSize);
				vault.setBilling(billing);
			}
			body.setVault(vault);
			request.setBody(body);
			UpdateVaultResponse response = client.updateVault(request);
			return new BocloudResult(true, convertor.convertModel(response.getVault()), "编辑成功");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "编辑失败：", e.getMessage());
		}
	}

	/**
	 * 存储库设置策略
	 *
	 * @param model
	 * @return
	 */
	public BocloudResult associatePolicy(VaultsModel model) {
		try {
			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			AssociateVaultPolicyRequest request = new AssociateVaultPolicyRequest();
			request.setVaultId(model.getVaultsUuid());
			VaultAssociate vaultAssociate = new VaultAssociate();
			vaultAssociate.setDestinationVaultId(model.getVaultsUuid());
			vaultAssociate.setPolicyId(model.getBackupPolicyUuid());
			request.setBody(vaultAssociate);
			client.associateVaultPolicy(request);
			return new BocloudResult(true, "存储库设置策略成功！");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "存储库设置策略失败：", e.getMessage());
		}
	}

	/**
	 * 存储库解除策略
	 *
	 * @param model
	 * @return
	 */
	public BocloudResult dissociatePolicy(VaultsModel model) {
		try {
			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			DisassociateVaultPolicyRequest request = new DisassociateVaultPolicyRequest();
			request.setVaultId(model.getVaultsUuid());
			VaultDissociate vaultDissociate = new VaultDissociate();
			vaultDissociate.setPolicyId(model.getBackupPolicyUuid());
			request.setBody(vaultDissociate);
			client.disassociateVaultPolicy(request);
			return new BocloudResult(true, "存储库解除策略成功！");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "存储库解除策略失败：", e.getMessage());
		}
	}

	/**
	 * 添加资源
	 *
	 * @param model
	 * @return
	 */
	@Override
	public BocloudResult addResources(VaultsModel model) {
		try {
			BocloudResult detail = this.detail(model);
			if (detail.isFailed()) {
				return detail;
			}
			List<String> newResourceUuids = model.getResources();
			VaultsModel detailModel = (VaultsModel) detail.getData();
			List<String> oldResourceUuids = model.getRemoveResources();
			// 先移除
			if (!ListTool.isEmpty(oldResourceUuids)) {
				VaultsModel removeModel = new VaultsModel();
				removeModel.setVaultsUuid(detailModel.getVaultsUuid());
				removeModel.setResources(oldResourceUuids);
				BocloudResult removeResources = this.removeResources(removeModel);
				if (removeResources.isFailed()) {
					return new BocloudResult(false, "资源绑定失败：" + removeResources.getMessage());
				}
			}
			if (ListTool.isEmpty(newResourceUuids)) {
				return new BocloudResult(true, this.detail(model).getData(), "资源绑定成功！");
			}
			// 绑定

			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			AddVaultResourceRequest request = new AddVaultResourceRequest();

			String type = "OS::Cinder::Volume";
			String objectType = model.getObjectType();
			if ("server".equalsIgnoreCase(objectType)) {
				type = "OS::Nova::Server";
			}
			request.setVaultId(model.getVaultsUuid());
			VaultAddResourceReq body = new VaultAddResourceReq();

			if (!ListTool.isEmpty(newResourceUuids)) {
				for (String resourceUuid : newResourceUuids) {
					ResourceCreate resouorcesItem = new ResourceCreate();
					resouorcesItem.setType(type);
					resouorcesItem.setId(resourceUuid);
					body.addResourcesItem(resouorcesItem);
				}
			}
			request.setBody(body);

			AddVaultResourceResponse response = client.addVaultResource(request);
			List<String> addResourceIds = response.getAddResourceIds();
			if (ListTool.isEmpty(addResourceIds)) {
				return new BocloudResult(false, "资源添加失败！");
			}
			return waitForAddProcess(addResourceIds, model.getVaultsUuid());
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "资源添加失败:" + e.getMessage());
		}
	}

	/**
	 * 移除资源
	 *
	 * @param model
	 * @return
	 */
	@Override
	public BocloudResult removeResources(VaultsModel model) {
		try {

			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			RemoveVaultResourceRequest request = new RemoveVaultResourceRequest();

			request.setVaultId(model.getVaultsUuid());
			VaultRemoveResourceReq body = new VaultRemoveResourceReq();
			body.setResourceIds(model.getResources());
			request.setBody(body);
			RemoveVaultResourceResponse response = client.removeVaultResource(request);
			List<String> removeResourceIds = response.getRemoveResourceIds();
			return waitForRemoveProcess(removeResourceIds, model.getVaultsUuid());
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "资源移除失败", e.getMessage());
		}
	}

	private BocloudResult waitForAddProcess(List<String> resourceUuids, String vaultsUuid) {
		Long startTime = System.currentTimeMillis();
		VaultsModel model = new VaultsModel();
		model.setVaultsUuid(vaultsUuid);
		for (;;) {
			sleepSomeTime(2000);
			BocloudResult detail = this.detail(model);
			if (detail.isFailed()) {
				return detail;
			}
			VaultsModel vaultsModels = (VaultsModel) detail.getData();
			List<String> resources = vaultsModels.getResources();

			if (ListTool.isEmpty(resources)) {
				continue;
			} else {
				for (String resourceStr : resources) {
					if (resourceUuids.contains(resourceStr)) {
						resourceUuids.remove(resourceStr);
						logger.info(String.format("id %s operate successfully ", resourceStr));
					}
				}
			}
			if (resourceUuids.size() == 0) {
				logger.info("resource add success !!! ");
				return new BocloudResult(true, detail.getData(), "操作成功！");
			}
			Long timeStamp = System.currentTimeMillis();
			if (timeStamp - startTime > 60 * 1000 * 5 * resourceUuids.size()) {
				return new BocloudResult(false, "操作超时！");
			}
		}
	}

	private BocloudResult waitForRemoveProcess(List<String> resourceUuids, String vaultsUuid) {
		Long startTime = System.currentTimeMillis();
		VaultsModel model = new VaultsModel();
		model.setVaultsUuid(vaultsUuid);
		for (;;) {
			sleepSomeTime(1000);
			BocloudResult detail = this.detail(model);
			if (detail.isFailed()) {
				return detail;
			}
			VaultsModel vaultsModels = (VaultsModel) detail.getData();
			List<String> resources = vaultsModels.getResources();
			Long timeStamp = System.currentTimeMillis();
			if (ListTool.isEmpty(resources)) {
				resourceUuids.clear();
				logger.info("resource add success !!! ");
				return new BocloudResult(true, this.detail(model).getData(), "操作成功！");
			}
			for (int i = resourceUuids.size() - 1; i >= 0; i--) {
				if (resources.contains(resourceUuids.get(i))) {
					continue;
				} else {
					resourceUuids.remove(resourceUuids.get(i));
				}
			}
			if (resourceUuids.size() == 0) {
				logger.info("resource add success !!! ");
				return new BocloudResult(true, detail.getData(), "操作成功！");
			}
			if (timeStamp - startTime > 60 * 1000 * 5 * resourceUuids.size()) {
				return new BocloudResult(false, "操作超时！");
			}
		}
	}

	/**
	 * 创建备份还原点
	 *
	 * @param model
	 * @return
	 */
	@Override
	public BocloudResult checkpoints(VaultsModel model) {
		try {
			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			CreateCheckpointRequest request = new CreateCheckpointRequest();

			VaultBackupReq body = new VaultBackupReq();
			VaultBackup checkPoint = new VaultBackup();

			CheckpointParam parameters = new CheckpointParam();

			// incremental 是否增量备份，true:增量备份，false：非增量备份。 缺省值：true
			parameters.setIncremental(true);
			// 是否自动触发,true:自动触发，false：非自动触发。 缺省值：false
			parameters.setAutoTrigger(true);
			parameters.setName(model.getBackupName());
			parameters.setDescription(model.getBackupRemark());
			List<String> resources = model.getResources() == null ? new ArrayList<String>() : model.getResources();
			parameters.setResources(resources);
			checkPoint.setParameters(parameters);

			checkPoint.setVaultId(model.getVaultsUuid());
			body.setCheckpoint(checkPoint);
			request.setBody(body);

			CreateCheckpointResponse response = client.createCheckpoint(request);
			CheckpointCreate ckPoint = response.getCheckpoint();

			VaultsModel checkPointModel = new VaultsModel();
			checkPointModel.setCheckpointUuid(ckPoint.getId());
			return new BocloudResult(true, this.checkpointsDetail(checkPointModel).getData(), "创建备份成功！");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "备份创建失败：", e.getMessage());
		}
	}

	/**
	 * 备份还原点详情
	 *
	 * @param model
	 * @return
	 */
	public BocloudResult checkpointsDetail(VaultsModel model) {
		try {

			CbrClient client = (CbrClient) getClient(HuaweiIndex.service.CBR);
			ShowCheckpointRequest request = new ShowCheckpointRequest();
			request.setCheckpointId(model.getCheckpointUuid());
			ShowCheckpointResponse response = client.showCheckpoint(request);
			CheckpointCreate checkpoint = response.getCheckpoint();
			if (null == checkpoint) {
				return new BocloudResult(false, "查询失败！");
			}
			return new BocloudResult(true, checkpoint, "创建成功！");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "资源移除失败", e.getMessage());
		}
	}

}
