package com.bocloud.cmp.provider.compute;

import java.util.*;

import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.ImageConvertor;
import com.huaweicloud.sdk.ims.v2.ImsClient;
import com.huaweicloud.sdk.ims.v2.model.*;
import com.huaweicloud.sdk.ims.v2.model.ListImagesRequest.VirtualEnvTypeEnum;

import io.jsonwebtoken.lang.Assert;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

@Provider("bocloud.cmp.huawei")
public class HuaweiImageProvider extends HuaweiProvider implements ImageProvider {

	public HuaweiImageProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(HuaweiImageProvider.class);

	private BeanConvertor<ImageModel, ImageInfo> convertor = new ImageConvertor();

	private static Map<String, ListImagesRequest.ImagetypeEnum> imageType = new HashMap<>();
	private static Map<String, ListImagesRequest.DiskFormatEnum> diskFormat = new HashMap<>();
	static {
		imageType.put("gold", ListImagesRequest.ImagetypeEnum.GOLD);
		imageType.put("private", ListImagesRequest.ImagetypeEnum.PRIVATE);

		diskFormat.put("vhd", ListImagesRequest.DiskFormatEnum.VHD);
		diskFormat.put("zvhd", ListImagesRequest.DiskFormatEnum.ZVHD);
		diskFormat.put("zvhd2", ListImagesRequest.DiskFormatEnum.ZVHD2);
		diskFormat.put("raw", ListImagesRequest.DiskFormatEnum.RAW);
		diskFormat.put("qcow2", ListImagesRequest.DiskFormatEnum.QCOW2);
	}

	@Override
	public BocloudResult list() {
		try {
			// 镜像列表查询不传imageType，从数据返回上来看是只返回market类型的镜像，文档未说明这个类型是啥玩意，这个类型的镜像数据原生页面也不做展示
			// 所以调用镜像列表查询接口需要传imageTyep，并且需要传diskFormat(不传就查询的VHD类型的镜像)
			ImsClient client = (ImsClient) getClient(HuaweiIndex.service.IMS);
			List<ImageModel> imageModels = new ArrayList<>();
			Set<Map.Entry<String, ListImagesRequest.ImagetypeEnum>> entriesType = imageType.entrySet();
			Set<Map.Entry<String, ListImagesRequest.DiskFormatEnum>> entriesFormat = diskFormat.entrySet();
			for (Map.Entry<String, ListImagesRequest.ImagetypeEnum> entryType : entriesType) {
				for (Map.Entry<String, ListImagesRequest.DiskFormatEnum> entryFormat : entriesFormat) {
					ListImagesRequest request = new ListImagesRequest();
					request.withLimit(Integer.MAX_VALUE);
					request.withImagetype(entryType.getValue());
					request.withDiskFormat(entryFormat.getValue());
					// 只获取云主机
					request.withVirtualEnvType(VirtualEnvTypeEnum.FUSIONCOMPUTE);
					ListImagesResponse response = client.listImages(request);
					imageModels.addAll(convertor.convertList(response.getImages()));
				}
			}
			imageModels.forEach(imageModel -> imageModel.setRegionId(this.getRegionId()));
			return new BocloudResult(true, imageModels, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询列表失败 ： " + e);
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}
	
	public BocloudResult listImagesByEnterpriseProject(ImsClient client, String enterpriseProjectId, String imageType) {
		try {
			// 镜像列表查询不传imageType，从数据返回上来看是只返回market类型的镜像，文档未说明这个类型是啥玩意，这个类型的镜像数据原生页面也不做展示
			// 所以调用镜像列表查询接口需要传imageTyep，并且需要传diskFormat(不传就查询的VHD类型的镜像)
			List<ImageModel> imageModels = new ArrayList<>();
			ListImagesRequest request = new ListImagesRequest();
			request.withLimit(Integer.MAX_VALUE);
			if (!StringUtils.isEmpty(imageType)) {
				request.withImagetype(ListImagesRequest.ImagetypeEnum.valueOf(imageType));
			}
			if (!StringUtils.isEmpty(enterpriseProjectId)) {
				request.setEnterpriseProjectId(enterpriseProjectId);
			}
			// 只获取云主机
			request.withVirtualEnvType(VirtualEnvTypeEnum.FUSIONCOMPUTE);
			ListImagesResponse response = client.listImages(request);
			imageModels.addAll(convertor.convertList(response.getImages()));
			imageModels.forEach(imageModel -> imageModel.setRegionId(this.getRegionId()));
			return new BocloudResult(true, imageModels, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询列表失败 ： " + e);
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}
	
	@Override
	public BocloudResult detail(String imageId) {
		try {
			// sdk暂未发现查询镜像详情的方法，先以一下方法代替
			ImageModel model = getModel(imageId, null);

			return new BocloudResult(true, model, "查询详情成功");
		} catch (Exception e) {
			logger.error("查询详情失败 ： " + e);
			return new BocloudResult(false, "查询详情失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String imageId) {
		try {
			// 使用的镜像(openstack原生端)的sdk方法，测试可以删除镜像，后续有问题再看
			ImsClient client = (ImsClient) getClient(HuaweiIndex.service.IMS);
			GlanceDeleteImageRequest request = new GlanceDeleteImageRequest();
			request.withImageId(imageId);
			GlanceDeleteImageRequestBody body = new GlanceDeleteImageRequestBody();
			request.withBody(body);
			client.glanceDeleteImage(request);
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			logger.error("删除失败 ： " + e.getMessage());
			return new BocloudResult(false, "删除失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(ImageModel image) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult create(ImageModel imageModel) {
		try {
			// 系统盘创建镜像和数据盘创建镜像是同一个方法，只是data_images传与不传的区别
			// 系统盘创建镜像不需要传data_images，数据盘创建镜像需要
			// 硬盘接口未返回卷是系统盘还是数据盘的区分，所以暂时用系统盘去创建镜像
			Assert.isTrue(StringUtils.isNotEmpty(imageModel.getName()), "参数name不能为空");
			Assert.isTrue(StringUtils.isNotEmpty(imageModel.getInstanceId()), "参数instanceId不能为空");
			ImsClient client = (ImsClient) getClient(HuaweiIndex.service.IMS);
			CreateImageRequest request = new CreateImageRequest();
			CreateImageRequestBody createImageRequestBody = new CreateImageRequestBody();
			createImageRequestBody.withName(imageModel.getName()).withInstanceId(imageModel.getInstanceId())
					.withDescription(imageModel.getRemark());
			
			String enterpriseProjectUuid = imageModel.getEnterpriseProjectUuid();
			if (!StringUtils.isEmpty(enterpriseProjectUuid)) {
				createImageRequestBody.setEnterpriseProjectId(enterpriseProjectUuid);
			}

			request.withBody(createImageRequestBody);
			CreateImageResponse response = client.createImage(request);
			// 整机镜像
//			CreateWholeImageRequest request = new CreateWholeImageRequest();
//			CreateWholeImageRequestBody createWholeImageRequestBody = new CreateWholeImageRequestBody();
//			createWholeImageRequestBody.withName(imageModel.getName()).
//					withInstanceId(imageModel.getInstanceId()).
//					withDescription(imageModel.getRemark());
//			request.withBody(createWholeImageRequestBody);
//			CreateWholeImageResponse response = client.createWholeImage(request);
			Assert.isTrue(StringUtils.isNotEmpty(response.getJobId()), "镜像创建失败，创建镜像未返回jobId");
			return this.waitForImageStatus(response.getJobId(), enterpriseProjectUuid);
		} catch (Exception e) {
			logger.error("创建镜像失败 ： " + e.getMessage());
			return new BocloudResult(false, "创建镜像失败:" + e.getMessage());
		}
	}

	private BocloudResult waitForImageStatus(String jobId, String enterpriseProjectUuid) {
		BocloudResult result = new BocloudResult(false, "创建失败");
		Boolean flag = true;
		Long startTime = System.currentTimeMillis();
		while (flag) {
			result = getEvsJobStatus(jobId);
			if (result.isFailed()) {
				return result;
			}
			ShowJobResponse resultData = (ShowJobResponse) result.getData();
			String status = resultData.getStatus().getValue();
			if ("SUCCESS".equalsIgnoreCase(status)) {
				JobEntities entities = resultData.getEntities();
				String imageId = entities.getImageId();
				if (StringUtils.isEmpty(imageId)) {
					return result;
				}
				flag = false;
				result.setSuccess(true);
				result.setData(getModel(imageId, enterpriseProjectUuid));
			} else if ("FAIL".equalsIgnoreCase(status)) {
				flag = false;
				break;
			} else {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
			}
			Long endTime = System.currentTimeMillis();
			// 等待三分钟
			if (endTime - startTime > 15 * 60 * 1000) {
				flag = false;
				break;
			}
		}
		return result;
	}

	private BocloudResult getEvsJobStatus(String jobId) {
		try {
			ImsClient client = (ImsClient) getClient(HuaweiIndex.service.IMS);

			ShowJobRequest request = new ShowJobRequest();
			request.withJobId(jobId);
			request.withProjectId(this.getProjectId());
			ShowJobResponse response = client.showJob(request);

			return new BocloudResult(true, response, "查询任务详情成功");

		} catch (Exception e) {
			logger.error("查询任务详情失败： " + e);
			return new BocloudResult(false, "查询任务详情失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult shareImage(ImageModel imageModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult syncImages(ImageModel imageModel) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * 查询企业项目下的私有镜像
	 * 
	 * @param uuid
	 * @param enterpriseProjectUuid
	 * @return
	 */
	private ImageModel getModel(String uuid, String enterpriseProjectUuid) {
		try {
			// 通过查询list 过滤
			ImsClient client = (ImsClient) getClient(HuaweiIndex.service.IMS);
			List<ImageModel> imageModels = new ArrayList<>();
			Set<Map.Entry<String, ListImagesRequest.ImagetypeEnum>> entriesType = imageType.entrySet();
			Set<Map.Entry<String, ListImagesRequest.DiskFormatEnum>> entriesFormat = diskFormat.entrySet();
			for (Map.Entry<String, ListImagesRequest.ImagetypeEnum> entryType : entriesType) {
				for (Map.Entry<String, ListImagesRequest.DiskFormatEnum> entryFormat : entriesFormat) {
					ListImagesRequest request = new ListImagesRequest();
					request.withLimit(Integer.MAX_VALUE);
					if (!StringUtils.isEmpty(enterpriseProjectUuid)) {
						request.setEnterpriseProjectId(enterpriseProjectUuid);
					}
					request.withImagetype(entryType.getValue());
					request.withDiskFormat(entryFormat.getValue());
					// 只获取云主机
					request.withVirtualEnvType(VirtualEnvTypeEnum.FUSIONCOMPUTE);
					ListImagesResponse response = client.listImages(request);
					imageModels.addAll(convertor.convertList(response.getImages()));
				}
			}
			imageModels.forEach(imageModel -> imageModel.setRegionId(this.getRegionId()));

			Map<String, ImageModel> map = new HashMap<>();
			Optional.ofNullable(imageModels).map(lists -> {
				lists.stream().forEach(model -> map.put(model.getImageUuid(), model));
				return map;
			});
			if (map.containsKey(uuid)) {
				return map.get(uuid);
			} else {
				return null;
			}
		} catch (Exception e) {
			logger.error("查询私有镜像异常：" + e);
			return null;
		}
	}

}
