package com.bocloud.cmp.provider.compute;

import java.util.*;
import java.util.stream.Collectors;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.PortConvertor;
import com.bocloud.cmp.huaweiconvertor.ServerConvertor;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.network.HuaweiElasticIpProvider;
import com.bocloud.cmp.provider.storage.HuaweiVolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.huaweicloud.sdk.ecs.v2.EcsClient;
import com.huaweicloud.sdk.ecs.v2.model.*;
import com.huaweicloud.sdk.ecs.v2.model.GetServerRemoteConsoleOption.ProtocolEnum;
import com.huaweicloud.sdk.ecs.v2.model.GetServerRemoteConsoleOption.TypeEnum;
import com.huaweicloud.sdk.ecs.v2.model.PrePaidServerEipExtendParam.ChargingModeEnum;
import com.huaweicloud.sdk.evs.v2.model.Attachment;
import com.huaweicloud.sdk.evs.v2.model.VolumeDetail;
import com.huaweicloud.sdk.vpc.v2.VpcClient;
import com.huaweicloud.sdk.vpc.v2.model.*;

/**
 * 云主机
 *
 * @author ymh
 * @时间 2018年11月29日
 */

@Provider("bocloud.cmp.huawei")
public class HuaweiServerProvider extends HuaweiProvider implements ServerProvider {

	private static Logger logger = LoggerFactory.getLogger(HuaweiServerProvider.class);

	private BeanConvertor<ServerModel, ServerDetail> convertor = new ServerConvertor();

	private BeanConvertor<PortModel, Port> portConvertor = new PortConvertor();

	public HuaweiServerProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		List<ServerModel> serverModels = new ArrayList<>();
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			ListServersDetailsRequest request = new ListServersDetailsRequest();
			request.setOffset(0);
			request.withLimit(PAGE_SIZE);
			ListServersDetailsResponse response = client.listServersDetails(request);
			Integer totalCount = response.getCount();
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, serverModels, "云主机列表为空");
			}
			List<ServerDetail> servers = response.getServers();
			serverModels = convertor.convertList(servers);
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				for (ServerModel serverModel : serverModels) {
					serverModel.setRegionId(this.getRegionId());
				}
				serverModels = this.getSystemDiskSize(serverModels);
				serverModels = this.getPortInfo(serverModels);
				logger.info("云主机数量 ： " + serverModels.size());
				return new BocloudResult(true, serverModels, "列表查询成功");
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setOffset(i);
				request.withLimit(PAGE_SIZE);
				response = client.listServersDetails(request);
				servers = response.getServers();
				serverModels.addAll(convertor.convertList(servers));
			}
			for (ServerModel serverModel : serverModels) {
				serverModel.setRegionId(this.getRegionId());
			}
			serverModels = this.getSystemDiskSize(serverModels);
			serverModels = this.getPortInfo(serverModels);
			logger.info("云主机数量 ： " + serverModels.size());
			return new BocloudResult(true, serverModels, "查询列表成功");
		} catch (Exception e) {
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	private List<ServerModel> getPortInfo(List<ServerModel> convertList) {
		try {
			if (ListTool.isEmpty(convertList)) {
				return convertList;
			}
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			for (ServerModel serverModel : convertList) {
				List<JSONObject> parseArray = JSONObject.parseArray(serverModel.getPrivateIps(), JSONObject.class);
				if (ListTool.isEmpty(parseArray)) {
					continue;
				}
				for (JSONObject object : parseArray) {
					String portId = object.getString("portId");
					String type = object.getString("type");
					String address = object.getString("address");
					String subnetUuid = "";
					if ("fixed".equalsIgnoreCase(type)) {
						ShowPortRequest request = new ShowPortRequest();
						request.setPortId(portId);
						Port resultPort = client.showPort(request).getPort();
						List<FixedIp> fixedIps = resultPort.getFixedIps();
						for (FixedIp fixedIp : fixedIps) {
							if (address.equalsIgnoreCase(fixedIp.getIpAddress())) {
								subnetUuid = fixedIp.getSubnetId();
								break;
							}
						}
					}
					object.put("subnetUuid", subnetUuid);
				}
				serverModel.setPrivateIps(JSONObject.toJSONString(parseArray));
			}
		} catch (Exception e) {
			logger.error("云主机查询PORT异常：" + e.getMessage());
		}
		return convertList;
	}

	private List<ServerModel> getSystemDiskSize(List<ServerModel> convertList) {
		try {
			HuaweiVolumeProvider volumeProvider = new HuaweiVolumeProvider(this.getButler());
			List<VolumeDetail> details = volumeProvider.listAllVolumes();
			if (ListTool.isEmpty(details)) {
				return convertList;
			}
			Map<String, List<VolumeDetail>> volMap = new HashMap<String, List<VolumeDetail>>();
			for (VolumeDetail detail : details) {
				List<Attachment> attachments = detail.getAttachments();
				for (Attachment attachment : attachments) {
					if (volMap.containsKey(attachment.getServerId())) {
						List<VolumeDetail> list = volMap.get(attachment.getServerId());
						list.add(detail);
						volMap.put(attachment.getServerId(), list);
					} else {
						List<VolumeDetail> list = new ArrayList<>();
						list.add(detail);
						volMap.put(attachment.getServerId(), list);
					}
				}
			}
			for (ServerModel serverModel : convertList) {
				Integer diskSize = 0;
				if (volMap.containsKey(serverModel.getInstanceId())) {
					List<VolumeDetail> list = volMap.get(serverModel.getInstanceId());
					for (VolumeDetail volumeDetail : list) {
						if("true".equals(volumeDetail.getBootable())) {
							diskSize += volumeDetail.getSize();
						}
					}
				}
				serverModel.setDisk(diskSize);
			}
			return convertList;
		} catch (Exception e) {
			return convertList;
		}
	}

	@Override
	public BocloudResult list(String hostId) {
		return null;
	}

	@Override
	public BocloudResult create(ServerModel serverModel) {
		try {
			return createPrePaidServer(serverModel);
		} catch (Exception e) {
			logger.error("创建云主机失败 ： " + e);
			return new BocloudResult(false, this.getChineseMessage(e.getMessage()));
		}
	}

	private BocloudResult createPrePaidServer(ServerModel serverModel) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);

			CreateServersRequest request = new CreateServersRequest();

			CreateServersRequestBody body = new CreateServersRequestBody();
			PrePaidServer server = new PrePaidServer();
			server.withName(serverModel.getName()).withVpcid(serverModel.getVpcId())
					.withImageRef(serverModel.getImageUuid()).withFlavorRef(serverModel.getFlavorUuid())
					.withAvailabilityZone(serverModel.getZone());

			// 网卡信息
			List<PrePaidServerNic> nics = new ArrayList();
			for (NetworkInterfaceModel networkInterfaceModel : serverModel.getNetworkInterfaces()) {
				PrePaidServerNic nic = new PrePaidServerNic();
				if (!StringUtils.isEmpty(networkInterfaceModel.getPortUuid())) {
					nic.withportId(networkInterfaceModel.getPortUuid());
					if (server.getOsSchedulerHints() == null || server.getOsSchedulerHints().getIpv6Enable() == null) {
						server.withOsSchedulerHints(new PrePaidServerSchedulerHints()
								.withIpv6Enable(PrePaidServerSchedulerHints.Ipv6EnableEnum.TRUE));
					}
				} else {
					nic.withSubnetId(networkInterfaceModel.getSubnetUuid());
					if (!StringUtils.isEmpty(networkInterfaceModel.getPrimaryPrivateIPAddress())) {
						nic.withIpAddress(networkInterfaceModel.getPrimaryPrivateIPAddress());
					}
					if (!StringUtils.isEmpty(networkInterfaceModel.getIpv6Address())
							&& Boolean.parseBoolean(networkInterfaceModel.getIpv6Address())) {
						nic.withIpv6Enable(true);
					}
				}
				nics.add(nic);
			}
			server.withNics(nics);

			// 安全组
			List<String> securityGroups = serverModel.getSecurityGroups();
			if (null != securityGroups && !securityGroups.isEmpty()) {

				List<PrePaidServerSecurityGroup> securityGroupsList = new ArrayList<>();
				PrePaidServerSecurityGroup security = null;
				for (String securityG : securityGroups) {
					security = new PrePaidServerSecurityGroup();
					security.setId(securityG);
					securityGroupsList.add(security);
				}

				server.withSecurityGroups(securityGroupsList);
			}

			// 登录凭证
			String keypairName = serverModel.getKeypairName();
			if (!StringUtils.isEmpty(keypairName)) {
				server.withKeyName(serverModel.getKeypairName());
			} else if (!StringUtils.isEmpty(serverModel.getPassword())) {
				server.withAdminPass(serverModel.getPassword());
			}

			// 公网IP
			this.prepaidPublicIp(server, serverModel);

			// 系统盘
			PrePaidServerRootVolume rootVolume = new PrePaidServerRootVolume();

			PrePaidServerRootVolume.VolumetypeEnum rootVolumeType = PrePaidServerRootVolume.VolumetypeEnum
					.valueOf(serverModel.getRootVolumeType());
			if (rootVolumeType == null) {
				rootVolumeType = PrePaidServerRootVolume.VolumetypeEnum.GPSSD;
			}
			rootVolume.withVolumetype(rootVolumeType).withSize(serverModel.getRootSize());
			server.withRootVolume(rootVolume);

			// 添加数据盘
			this.prePaidVolumeParams(server, serverModel);

			PrePaidServerExtendParam extendParam = new PrePaidServerExtendParam();
			extendParam.withSupportAutoRecovery(true);
			String instanceChargeType = serverModel.getInstanceChargeType();
			if (StringUtils.isEmpty(instanceChargeType)) {
				instanceChargeType = "PostPaid";
			}
			switch (instanceChargeType) {
			case "PrePaid":
				extendParam.withChargingMode(PrePaidServerExtendParam.ChargingModeEnum.PREPAID)
						.withPeriodType(PrePaidServerExtendParam.PeriodTypeEnum.valueOf(serverModel.getPeriodUnit()))
						.withPeriodNum(serverModel.getPeriod())
						.withIsAutoRenew(PrePaidServerExtendParam.IsAutoRenewEnum.FALSE);
				break;
			case "PostPaid":
				extendParam.withChargingMode(PrePaidServerExtendParam.ChargingModeEnum.POSTPAID);
				break;
			default:
				break;
			}
			String enterpriseProjectUuid = serverModel.getEnterpriseProjectUuid();
			if (!StringUtils.isEmpty(enterpriseProjectUuid)) {
				extendParam.setEnterpriseProjectId(enterpriseProjectUuid);
			}
			server.withExtendparam(extendParam);
			server.withCount(serverModel.getCount());

			// 标签
			if (!ListTool.isEmpty(serverModel.getTagModels())) {
				List<PrePaidServerTag> serverTags = new ArrayList<>();
				for (TagModel tagModel : serverModel.getTagModels()) {
					PrePaidServerTag prePaidServerTag = new PrePaidServerTag();
					prePaidServerTag.withKey(tagModel.getKey()).withValue(tagModel.getValue());
					serverTags.add(prePaidServerTag);
				}
				server.withServerTags(serverTags);
			}

			body.withServer(server);
			request.withBody(body);

			CreateServersResponse response = client.createServers(request);

			BocloudResult bocloudResult = this.waitForJobStatus(response.getJobId(), null,
					(serverModel.getIsUsePublicIp() == null ? false : serverModel.getIsUsePublicIp()), "创建");

			if (bocloudResult.isSuccess()) {
				ServerModel sModel = (ServerModel) bocloudResult.getData();
				VpcClient vpcClient = (VpcClient) getClient(HuaweiIndex.service.VPC);

				ListPortsRequest listPortsRequest = new ListPortsRequest();
				listPortsRequest.withDeviceId(sModel.getInstanceId());
				ListPortsResponse listPortsResponse = vpcClient.listPorts(listPortsRequest);
				List<PortModel> portModels = portConvertor.convertList(listPortsResponse.getPorts());
				if (!ListTool.isEmpty(portModels)) {
					for (PortModel portModel : portModels) {
						portModel.setRegionId(getRegionId());
					}
				}
				sModel.setPorts(portModels);
			}

			return bocloudResult;
		} catch (Exception e) {
			logger.error("创建云主机失败 ： " + e);
			return new BocloudResult(false, this.getChineseMessage(e.getMessage()));
		}
	}

	private BocloudResult createPostPaidServer(ServerModel serverModel) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);

			CreatePostPaidServersRequest request = new CreatePostPaidServersRequest();

			CreatePostPaidServersRequestBody body = new CreatePostPaidServersRequestBody();
			PostPaidServer server = new PostPaidServer();

			server.withName(serverModel.getName()).withVpcid(serverModel.getVpcId())
					.withImageRef(serverModel.getImageUuid()).withFlavorRef(serverModel.getFlavorUuid())
					.withAvailabilityZone(serverModel.getZone());

			// 网卡信息
			List<PostPaidServerNic> nics = new ArrayList();
			for (NetworkInterfaceModel networkInterfaceModel : serverModel.getNetworkInterfaces()) {
				PostPaidServerNic nic = new PostPaidServerNic();
				nic.withSubnetId(networkInterfaceModel.getSubnetUuid());
				if (!StringUtils.isEmpty(networkInterfaceModel.getPrimaryPrivateIPAddress())) {
					nic.withIpAddress(networkInterfaceModel.getPrimaryPrivateIPAddress());
				}
				if (!StringUtils.isEmpty(networkInterfaceModel.getIpv6Address())
						&& Boolean.parseBoolean(networkInterfaceModel.getIpv6Address())) {
					nic.withIpv6Enable(true);
				}
				nics.add(nic);
			}
			server.withNics(nics);

			// 安全组
			List<String> securityGroups = serverModel.getSecurityGroups();
			if (null != securityGroups && !securityGroups.isEmpty()) {

				List<PostPaidServerSecurityGroup> securityGroupsList = new ArrayList<>();
				PostPaidServerSecurityGroup security = null;
				for (String securityG : securityGroups) {
					security = new PostPaidServerSecurityGroup();
					security.setId(securityG);
					securityGroupsList.add(security);
				}

				server.withSecurityGroups(securityGroupsList);
			}

			// 登录凭证
			String keypairName = serverModel.getKeypairName();
			if (!StringUtils.isEmpty(keypairName)) {
				server.withKeyName(serverModel.getKeypairName());
			} else if (!StringUtils.isEmpty(serverModel.getPassword())) {
				server.withAdminPass(serverModel.getPassword());
			}

			// 公网IP
			this.postpaidPublicIp(server, serverModel);

			// 系统盘
			PostPaidServerRootVolume rootVolume = new PostPaidServerRootVolume();

			PostPaidServerRootVolume.VolumetypeEnum rootVolumeType = PostPaidServerRootVolume.VolumetypeEnum
					.valueOf(serverModel.getRootVolumeType());
			if (rootVolumeType == null) {
				rootVolumeType = PostPaidServerRootVolume.VolumetypeEnum.GPSSD;
			}
			rootVolume.withVolumetype(rootVolumeType).withSize(serverModel.getRootSize());
			server.withRootVolume(rootVolume);

			// 添加数据盘
			this.postPaidVolumeParams(server, serverModel);

			PostPaidServerExtendParam extendParam = new PostPaidServerExtendParam();
			extendParam.withSupportAutoRecovery(true);
			server.withExtendparam(extendParam);
			server.withCount(serverModel.getCount());

			// 标签
			if (!ListTool.isEmpty(serverModel.getTagModels())) {
				List<PostPaidServerTag> serverTags = new ArrayList<>();
				for (TagModel tagModel : serverModel.getTagModels()) {
					PostPaidServerTag postPaidServerTag = new PostPaidServerTag();
					postPaidServerTag.withKey(tagModel.getKey()).withValue(tagModel.getValue());
					serverTags.add(postPaidServerTag);
				}
				server.withServerTags(serverTags);
			}

			body.withServer(server);
			request.withBody(body);

			CreatePostPaidServersResponse response = client.createPostPaidServers(request);
			return this.waitForJobStatus(response.getJobId(), null,
					serverModel.getIsUsePublicIp() == null ? false : serverModel.getIsUsePublicIp(), "创建");
		} catch (Exception e) {
			logger.error("创建云主机失败 ： " + e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	private void prePaidVolumeParams(PrePaidServer server, ServerModel serverModel) {
		List<VolumeModel> diskDevices = serverModel.getDiskDevices();
		if (null != diskDevices && diskDevices.size() > 0) {
			List<PrePaidServerDataVolume> dataVolumes = new ArrayList<>();
			PrePaidServerDataVolume volume = null;
			for (VolumeModel volumeModel : diskDevices) {
				volume = new PrePaidServerDataVolume();

				PrePaidServerDataVolume.VolumetypeEnum type = PrePaidServerDataVolume.VolumetypeEnum
						.valueOf(volumeModel.getType());
				if (type == null) {
					type = PrePaidServerDataVolume.VolumetypeEnum.GPSSD;
				}
				volume.withVolumetype(type).withSize(volumeModel.getSize()).withMultiattach(false);
				dataVolumes.add(volume);

			}

			server.withDataVolumes(dataVolumes);
		}
	}

	private void postPaidVolumeParams(PostPaidServer server, ServerModel serverModel) {
		List<VolumeModel> diskDevices = serverModel.getDiskDevices();
		if (null != diskDevices && diskDevices.size() > 0) {
			List<PostPaidServerDataVolume> dataVolumes = new ArrayList<>();
			PostPaidServerDataVolume volume = null;
			for (VolumeModel volumeModel : diskDevices) {
				volume = new PostPaidServerDataVolume();

				PostPaidServerDataVolume.VolumetypeEnum type = PostPaidServerDataVolume.VolumetypeEnum
						.valueOf(volumeModel.getType());
				if (type == null) {
					type = PostPaidServerDataVolume.VolumetypeEnum.GPSSD;
				}
				volume.withVolumetype(type).withSize(volumeModel.getSize()).withMultiattach(false);
				dataVolumes.add(volume);

			}

			server.withDataVolumes(dataVolumes);
		}
	}

	private void prepaidPublicIp(PrePaidServer server, ServerModel serverModel) {
		Boolean isUsePublicIp = serverModel.getIsUsePublicIp() == null ? false : serverModel.getIsUsePublicIp();
		if (isUsePublicIp) {
			PrePaidServerPublicip publicip = new PrePaidServerPublicip();
			PrePaidServerEip eip = new PrePaidServerEip();
			String type = "5_bgp";
			if (serverModel.getIpType() != null) {
				type = serverModel.getIpType();
			}
			eip.setIptype(type);
			PrePaidServerEipBandwidth bandwidth = new PrePaidServerEipBandwidth();
			bandwidth.withSize(serverModel.getBandWidth()).withSharetype(PrePaidServerEipBandwidth.SharetypeEnum.PER);
			eip.withBandwidth(bandwidth);

			PrePaidServerEipExtendParam extendParam = new PrePaidServerEipExtendParam();
			extendParam.setChargingMode(ChargingModeEnum.POSTPAID);
			eip.withExtendparam(extendParam);

			publicip.withEip(eip);
			server.withPublicip(publicip);

		} else if (serverModel.getPublicIps() != null) {
			PrePaidServerPublicip publicip = new PrePaidServerPublicip();
			publicip.withId(serverModel.getPublicIps());
			server.withPublicip(publicip);
		}
	}

	private void postpaidPublicIp(PostPaidServer server, ServerModel serverModel) {
		Boolean isUsePublicIp = serverModel.getIsUsePublicIp() == null ? false : serverModel.getIsUsePublicIp();
		if (isUsePublicIp) {

			PostPaidServerPublicip publicip = new PostPaidServerPublicip();
			PostPaidServerEip eip = new PostPaidServerEip();
			PostPaidServerEipBandwidth bandwidth = new PostPaidServerEipBandwidth();
			bandwidth.withSize(serverModel.getBandWidth()).withSharetype(PostPaidServerEipBandwidth.SharetypeEnum.PER);
			// bandwidth.withChargemode("traffic");
			eip.withBandwidth(bandwidth);
			publicip.withEip(eip);
			server.withPublicip(publicip);
		}
	}

	public BocloudResult waitForJobStatus(String jobId, String instanceId, Boolean isUsePublicIp, String message) {
		logger.info("wait job ==  jobId :" + jobId);
		BocloudResult result = new BocloudResult(false, message + "失败");
		Boolean flag = true;
		Long startTime = System.currentTimeMillis();
		EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);

		while (flag) {
			ShowJobRequest request = new ShowJobRequest();
			request.setJobId(jobId);
			ShowJobResponse response = client.showJob(request);
			if (null == response) {
				return result;
			}
			String status = response.getStatus().getValue();
			logger.info("wait for response : " + JSON.toJSONString(response));
			if ("SUCCESS".equalsIgnoreCase(status) && null != response.getEntities()) {
				JobEntities entities = response.getEntities();
				List<SubJob> subJobs = entities.getSubJobs();
				if (subJobs != null) {
					SubJob subJob = subJobs.get(0);
					String serverId = subJob.getEntities().getServerId();
					result = new BocloudResult(true, this.detail(serverId).getData(), message + "成功");
				} else {
					result = new BocloudResult(true, this.detail(instanceId).getData(), message + "成功");
				}
				if (isUsePublicIp) {
					ServerModel serverModel = (ServerModel) result.getData();
					String managerIp = serverModel.getManagerIp();
					BocloudResult list = new HuaweiElasticIpProvider(getButler()).list();
					if (list.isSuccess()) {
						List<ElasticIpModel> elasticIpModels = (List<ElasticIpModel>) list.getData();
						for (ElasticIpModel elasticIpModel : elasticIpModels) {
							if (!StringUtils.isEmpty(managerIp)
									&& managerIp.equalsIgnoreCase(elasticIpModel.getPrivateIpAddress())) {
								serverModel.setElasticIpModel(elasticIpModel);
								break;
							}
						}
					}
					result.setData(serverModel);
				}

				flag = false;
				break;
			} else if ("FAIL".equalsIgnoreCase(status)) {
				flag = false;
				String errorMessage = response.getFailReason();
				JobEntities entities = response.getEntities();
				if (null != entities) {
					List<SubJob> subJobs = entities.getSubJobs();
					if (!ListTool.isEmpty(subJobs)) {
						SubJob subJob = subJobs.get(0);
						if (null != subJob) {
							errorMessage = this.getChineseMessage(subJob.getFailReason());
						}
					}
				}
				result.setMessage(errorMessage);
				break;
			} else {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					Thread.currentThread().interrupt();
				}
			}
			Long endTime = System.currentTimeMillis();
			// 等待三分钟
			if (endTime - startTime > 20 * 60 * 1000) {
				flag = false;
				break;
			}
		}
		return result;
	}

	@Override
	public BocloudResult modify(ServerModel server) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			UpdateServerRequest request = new UpdateServerRequest();
			UpdateServerRequestBody body = new UpdateServerRequestBody();
			UpdateServerOption option = new UpdateServerOption();
			if (!StringUtils.isEmpty(server.getName())) {
				option.withName(server.getName());
			}
			if (!StringUtils.isEmpty(server.getRemark())) {
				option.withDescription(server.getRemark());
			}
			body.withServer(option);
			request.withServerId(server.getInstanceId()).withBody(body);
			UpdateServerResponse response = client.updateServer(request);
			BocloudResult result = detail(response.getServer().getId());
			if (result.isFailed()) {
				return result;
			}
			return new BocloudResult(true, (ServerModel) result.getData(), "修改成功");
		} catch (Exception e) {
			return new BocloudResult(false, "修改失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult migrate(ServerModel server) {
		return null;
	}

	@Override
	public BocloudResult migrate(OSMigrateModel model) {
		return null;
	}

	@Override
	public BocloudResult remove(String instanceId, Boolean deletePublicIp, Boolean deleteVolume) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);

			DeleteServersRequest request = new DeleteServersRequest();
			DeleteServersRequestBody body = new DeleteServersRequestBody();
			ServerId serverId = new ServerId();
			serverId.withId(instanceId);
			body.withServers(Arrays.asList(serverId)).withDeletePublicip(deletePublicIp).withDeleteVolume(deleteVolume);
			request.withBody(body);
			DeleteServersResponse response = client.deleteServers(request);
			return waitForJobStatus(response.getJobId(), instanceId, false, "删除");
		} catch (Exception e) {
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String instanceId) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			ShowServerRequest request = new ShowServerRequest();
			request.withServerId(instanceId);
			ShowServerResponse response = client.showServer(request);
			ServerModel convertModel = convertor.convertModel(response.getServer());
			convertModel.setRegionId(this.getRegionId());
			List<ServerModel> portInfo = getPortInfo(Collections.singletonList(convertModel));
			convertModel = portInfo.get(0);
			return new BocloudResult(true, convertModel, "查询成功");
		} catch (Exception e) {
			return new BocloudResult(false, "查询失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult start(String instanceId) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			BatchStartServersRequest request = new BatchStartServersRequest();
			BatchStartServersRequestBody body = new BatchStartServersRequestBody();
			List<ServerId> listOsStartServers = new ArrayList<>();
			listOsStartServers.add(new ServerId().withId(instanceId));
			BatchStartServersOption osStartbody = new BatchStartServersOption();
			osStartbody.withServers(listOsStartServers);
			body.withOsStart(osStartbody);
			request.withBody(body);

			BatchStartServersResponse response = client.batchStartServers(request);
			return waitForJobStatus(response.getJobId(), instanceId, false, "开机");
		} catch (Exception e) {
			e.printStackTrace();
			return new BocloudResult(false, "开机失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult stop(String instanceId) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);

			BatchStopServersRequest request = new BatchStopServersRequest();
			BatchStopServersRequestBody body = new BatchStopServersRequestBody();
			BatchStopServersOption option = new BatchStopServersOption();
			ServerId serverId = new ServerId();
			serverId.withId(instanceId);
			option.withServers(Arrays.asList(serverId));
			body.withOsStop(option);
			request.withBody(body);
			BatchStopServersResponse response = client.batchStopServers(request);
			return waitForJobStatus(response.getJobId(), instanceId, false, "关机");
		} catch (Exception e) {
			return new BocloudResult(false, "关机失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult reboot(String instanceId) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);

			BatchRebootServersRequest request = new BatchRebootServersRequest();
			BatchRebootServersRequestBody body = new BatchRebootServersRequestBody();
			BatchRebootSeversOption option = new BatchRebootSeversOption();
			ServerId serverId = new ServerId();
			serverId.withId(instanceId);
			option.withServers(Arrays.asList(serverId)).withType(BatchRebootSeversOption.TypeEnum.HARD);
			body.withReboot(option);
			request.withBody(body);
			BatchRebootServersResponse response = client.batchRebootServers(request);
			return waitForJobStatus(response.getJobId(), instanceId, false, "硬重启");
		} catch (Exception e) {
			return new BocloudResult(false, "硬重启失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult softReboot(String instanceId) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);

			BatchRebootServersRequest request = new BatchRebootServersRequest();
			BatchRebootServersRequestBody body = new BatchRebootServersRequestBody();
			BatchRebootSeversOption option = new BatchRebootSeversOption();
			ServerId serverId = new ServerId();
			serverId.withId(instanceId);
			option.withServers(Arrays.asList(serverId)).withType(BatchRebootSeversOption.TypeEnum.SOFT);
			body.withReboot(option);
			request.withBody(body);
			BatchRebootServersResponse response = client.batchRebootServers(request);
			return waitForJobStatus(response.getJobId(), instanceId, false, "软重启");
		} catch (Exception e) {
			return new BocloudResult(false, "软重启失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult pause(String instanceId) {
		try {
//            ActionResponse action = this.getClient().compute().servers().action(instanceId, Action.PAUSE);
//            if (action.isSuccess()) {
//                return new BocloudResult(true, action.isSuccess(), "关机成功");
//            } else {
//                return new BocloudResult(false, null, action.getFault());
//            }

			return new BocloudResult(true, "关机成功");

		} catch (Exception e) {
			return new BocloudResult(false, "关机失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult unPause(String instanceId) {
		try {
//            ActionResponse action = this.getClient().compute().servers().action(instanceId, Action.UNPAUSE);
//            return new BocloudResult(true, action.isSuccess(), "关机成功");
			return new BocloudResult(true, "关机成功");

		} catch (Exception e) {
			return new BocloudResult(false, "关机失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult suspend(String instanceId) {
		try {
//            ActionResponse action = this.getClient().compute().servers().action(instanceId, Action.SUSPEND);
//            return new BocloudResult(true, action.isSuccess(), "关机成功");
			return new BocloudResult(true, "关机成功");
		} catch (Exception e) {
			return new BocloudResult(false, "关机失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult resume(String instanceId) {
		try {
//            ActionResponse action = this.getClient().compute().servers().action(instanceId, Action.RESUME);
//            return new BocloudResult(true, action.isSuccess(), "关机成功");
			return new BocloudResult(true, "关机成功");
		} catch (Exception e) {
			return new BocloudResult(false, "关机失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult joinSGroup(String instanceId, String group) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			NovaAssociateSecurityGroupRequest request = new NovaAssociateSecurityGroupRequest();
			request.withServerId(instanceId);
			NovaAssociateSecurityGroupRequestBody requestBody = new NovaAssociateSecurityGroupRequestBody();
			NovaAddSecurityGroupOption option = new NovaAddSecurityGroupOption();
			option.withName(group);
			requestBody.withAddSecurityGroup(option);
			request.withBody(requestBody);
			client.novaAssociateSecurityGroup(request);
			return new BocloudResult(true, "安全组加入成功");
		} catch (Exception e) {
			return new BocloudResult(false, "安全组加入失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult leaveSGroup(String instanceId, String group) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			NovaDisassociateSecurityGroupRequest request = new NovaDisassociateSecurityGroupRequest();
			request.withServerId(instanceId);
			NovaDisassociateSecurityGroupRequestBody requestBody = new NovaDisassociateSecurityGroupRequestBody();
			NovaRemoveSecurityGroupOption option = new NovaRemoveSecurityGroupOption();
			option.withName(group);
			requestBody.withRemoveSecurityGroup(option);
			request.withBody(requestBody);
			client.novaDisassociateSecurityGroup(request);
			return new BocloudResult(true, "安全组移除成功");
		} catch (Exception e) {
			return new BocloudResult(false, "安全组移除失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult snapshot(SnapshotModel snapshotModel) {
		/*
		 * try { String createSnapshot =
		 * this.getClient().compute().servers().createSnapshot(snapshotModel.
		 * getInstanceId(), snapshotModel.getName()); return new BocloudResult(true,
		 * createSnapshot, "创建快照成功"); } catch (Exception e) { return new
		 * BocloudResult(false, "创建快照失败", e.getMessage()); }
		 */
		return null;
	}

	@Override
	public BocloudResult vncConsole(String vncType, String serverId) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			ShowServerRemoteConsoleRequest request = new ShowServerRemoteConsoleRequest();
			request.setServerId(serverId);
			ShowServerRemoteConsoleRequestBody body = new ShowServerRemoteConsoleRequestBody();
			GetServerRemoteConsoleOption remoteConsole = new GetServerRemoteConsoleOption();
			remoteConsole.setProtocol(ProtocolEnum.VNC);
			remoteConsole.setType(TypeEnum.NOVNC);
			body.setRemoteConsole(remoteConsole);
			request.setBody(body);
			ShowServerRemoteConsoleResponse showServerRemoteConsole = client.showServerRemoteConsole(request);
			if (null != showServerRemoteConsole && null != showServerRemoteConsole.getRemoteConsole()) {
				return new BocloudResult<>(true, showServerRemoteConsole.getRemoteConsole().getUrl(), "查询成功");
			}
			return new BocloudResult<>(false, "获取控制台url异常：底层接口异常");
		} catch (Exception e) {
			return new BocloudResult<>(false, "获取控制台url异常：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult changePassword(String instanceId, String password) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			ResetServerPasswordRequest request = new ResetServerPasswordRequest();
			request.withServerId(instanceId);
			ResetServerPasswordRequestBody requestBody = new ResetServerPasswordRequestBody();
			ResetServerPasswordOption option = new ResetServerPasswordOption();
			option.withNewPassword(password);
			requestBody.withResetPassword(option);
			request.withBody(requestBody);
			client.resetServerPassword(request);
			return new BocloudResult(true, "修改成功");
		} catch (Exception e) {
			return new BocloudResult(false, "修改失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult attachNetwork(ServerModel server) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			BatchAddServerNicsRequest request = new BatchAddServerNicsRequest();
			request.withServerId(server.getInstanceId());
			BatchAddServerNicsRequestBody body = new BatchAddServerNicsRequestBody();
			List<BatchAddServerNicOption> options = new ArrayList<>();
			server.getNicses().forEach(nics -> {
				BatchAddServerNicOption nicOption = new BatchAddServerNicOption();
				nicOption.withSubnetId(nics.getSubnetUuid());
				List<ServerNicSecurityGroup> groups = nics.getScGroupUuids().stream()
						.map(s -> new ServerNicSecurityGroup().withId(s)).collect(Collectors.toList());
				nicOption.withSecurityGroups(groups);
				options.add(nicOption);
			});
			body.withNics(options);
			request.withBody(body);
			BatchAddServerNicsResponse response = client.batchAddServerNics(request);
			return waitForJobStatus(response.getJobId(), server.getInstanceId(), false, "挂载网卡");
		} catch (Exception e) {
			return new BocloudResult(false, "挂载失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detachNetwork(String instanceId, String portUuid) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			BatchDeleteServerNicsRequest request = new BatchDeleteServerNicsRequest();
			request.withServerId(instanceId);
			BatchDeleteServerNicsRequestBody body = new BatchDeleteServerNicsRequestBody();
			BatchDeleteServerNicOption option = new BatchDeleteServerNicOption().withId(portUuid);
			body.withNics(Collections.singletonList(option));
			request.withBody(body);
			BatchDeleteServerNicsResponse response = client.batchDeleteServerNics(request);
			return waitForJobStatus(response.getJobId(), instanceId, false, "卸载网卡");
		} catch (Exception e) {
			return new BocloudResult(false, "卸载网卡失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			ResizeServerRequest request = new ResizeServerRequest();
			request.withServerId(instanceId);
			ResizeServerRequestBody body = new ResizeServerRequestBody();
			ResizePrePaidServerOption resizebody = new ResizePrePaidServerOption();
			resizebody.withFlavorRef(flavorId);
			body.withResize(resizebody);
			request.withBody(body);
			ResizeServerResponse response = client.resizeServer(request);
			if (StringUtils.isEmpty(response.getJobId())) {
				return new BocloudResult(false, "创建失败，获取的任务id未空");
			}
			return waitForJobStatus(response.getJobId(), instanceId, false, "变更规格");
		} catch (Exception e) {
			logger.error("变更规格失败" + e.getMessage());
			return new BocloudResult(false, "变更规格失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult confirmResize(String instanceId) {
		/*
		 * try { BocloudResult resizeDetail = this.serverResizeDetail(instanceId,
		 * "VERIFY_RESIZE"); if (resizeDetail.isFailed()) { return resizeDetail; }
		 * ActionResponse resize =
		 * this.getClient().compute().servers().confirmResize(instanceId); if
		 * (resize.isSuccess()) { BocloudResult bocloudResult =
		 * this.serverResizeDetail(instanceId, "SHUTOFF"); return new
		 * BocloudResult(true, bocloudResult.getData(), "修改成功"); } else { return new
		 * BocloudResult(false, "修改失败", resize.getFault()); } } catch (Exception e) {
		 * return new BocloudResult(false, "修改失败", e.getMessage()); }
		 */
		return null;
	}

	@Override
	public BocloudResult revertResize(String instanceId) { // REVERT_RESIZE
		/*
		 * try { BocloudResult resizeDetail = this.serverResizeDetail(instanceId,
		 * "VERIFY_RESIZE"); if (resizeDetail.isFailed()) { return resizeDetail; }
		 * ActionResponse resize =
		 * this.getClient().compute().servers().revertResize(instanceId); if
		 * (resize.isSuccess()) { BocloudResult bocloudResult =
		 * this.serverResizeDetail(instanceId, "SHUTOFF"); return new
		 * BocloudResult(true, bocloudResult.getData(), "修改成功"); } else { return new
		 * BocloudResult(false, "修改失败", resize.getFault()); } } catch (Exception e) {
		 * return new BocloudResult(false, "修改失败", e.getMessage()); }
		 */
		return null;
	}

	@Override
	public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean MemHotEnable) {
		return null;
	}

	@Override
	public BocloudResult getTicket(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult createInResPool(ServerModel server) {
		return null;
	}

	@Override
	public BocloudResult mountToolsInstaller(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult unmountToolsInstaller(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult upgradeToolsTask(String instanceId, String options) {
		return null;
	}

	@Override
	public BocloudResult getNetCard(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult getVmIps(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult markAsTemplate(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			ResetServerPasswordRequest request = new ResetServerPasswordRequest();
			request.withServerId(instanceId);
			ResetServerPasswordRequestBody requestBody = new ResetServerPasswordRequestBody();
			ResetServerPasswordOption option = new ResetServerPasswordOption();
			option.withNewPassword(newPasswd);
			requestBody.withResetPassword(option);
			request.withBody(requestBody);
			client.resetServerPassword(request);
			return new BocloudResult(true, "修改成功");
		} catch (Exception e) {
			return new BocloudResult(false, "修改失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
		return null;
	}

	@Override
	public BocloudResult getMetric(String type) {
		return null;
	}

	@Override
	public BocloudResult vmrcConsole(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult getJnlpXml(Long vmId, String address) {
		return null;
	}

	@Override
	public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
		return null;
	}

	@Override
	public BocloudResult resetInstance(ServerModel serverModel) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			ChangeServerOsWithCloudInitRequest request = new ChangeServerOsWithCloudInitRequest();
			request.withServerId(serverModel.getInstanceId());
			ChangeServerOsWithCloudInitRequestBody body = new ChangeServerOsWithCloudInitRequestBody();
			ChangeServerOsWithCloudInitOption osChangebody = new ChangeServerOsWithCloudInitOption();
			osChangebody.withImageid(serverModel.getImageUuid()).withAdminpass(serverModel.getPassword());
			body.withOsChange(osChangebody);
			request.withBody(body);
			ChangeServerOsWithCloudInitResponse response = client.changeServerOsWithCloudInit(request);
			if (StringUtils.isEmpty(response.getJobId())) {
				return new BocloudResult(false, "重置系统失败，获取的任务id未空");
			}
			return waitForJobStatus(response.getJobId(), serverModel.getInstanceId(), false, "重置系统");
		} catch (Exception e) {
			logger.error("重置系统失败" + e.getMessage());
			return new BocloudResult(false, "重置系统失败：" + e.getMessage());
		}
	}

	/*
	 * private BocloudResult waitTaskResult(String jobId) { BocloudResult model =
	 * new BocloudResult(); Boolean flag = true; Long startTime =
	 * System.currentTimeMillis(); Result result = null; JSONObject jobResult =
	 * null; while (flag) { try { Thread.sleep(10000); } catch (InterruptedException
	 * e) { Thread.currentThread().interrupt(); } result =
	 * doGet(buildUrl(serviceName, "/v1/" + currentProjectId() + "/jobs/" + jobId),
	 * null); if (result.isFailed()) { model = new BocloudResult(false,
	 * result.getMessage()); flag = false; break; } jobResult =
	 * JSONObject.parseObject(result.getMessage()); if
	 * ("FAIL".equalsIgnoreCase(jobResult.getString("status"))) { model = new
	 * BocloudResult(false, "任务查询失败"); flag = false; break; } else if
	 * ("SUCCESS".equalsIgnoreCase(jobResult.getString("status"))) { JSONObject
	 * serverOjb = JSONObject.parseObject(jobResult.getString("entities")); String
	 * serverId = serverOjb.getString("server_id"); model =
	 * serverResizeDetail(serverId, "ACTIVE"); flag = false; break; } Long endTime =
	 * System.currentTimeMillis(); // 等待20分钟 if (endTime - startTime > 30 * 60 *
	 * 1000) { flag = false; model = new BocloudResult(false, "任务超时"); break; } }
	 * return model; }
	 */

	@Override
	public BocloudResult bindKeypair(ServerModel serverModel) {
		return null;
	}

	@Override
	public BocloudResult unbindKeypair(ServerModel serverModel) {
		return null;
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
		return null;
	}

	@Override
	public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
		return null;
	}

	@Override
	public BocloudResult createTag(ServerModel model) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			BatchCreateServerTagsRequest request = new BatchCreateServerTagsRequest();
			request.withServerId(model.getInstanceId());
			BatchCreateServerTagsRequestBody body = new BatchCreateServerTagsRequestBody();
			body.withAction(BatchCreateServerTagsRequestBody.ActionEnum.CREATE)
					.withTags(JSONObject.parseArray(JSONObject.toJSONString(model.getTagModels()), ServerTag.class));
			request.withBody(body);
			BatchCreateServerTagsResponse response = client.batchCreateServerTags(request);
			return new BocloudResult(true, "创建卷标签成功");
		} catch (Exception e) {
			logger.error("创建卷标签失败： " + e);
			return new BocloudResult(false, "创建卷标签失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult removeTag(ServerModel model) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			BatchDeleteServerTagsRequest request = new BatchDeleteServerTagsRequest();
			request.withServerId(model.getInstanceId());
			BatchDeleteServerTagsRequestBody body = new BatchDeleteServerTagsRequestBody();
			body.withAction(BatchDeleteServerTagsRequestBody.ActionEnum.DELETE)
					.withTags(JSONObject.parseArray(JSONObject.toJSONString(model.getTagModels()), ServerTag.class));
			request.withBody(body);
			BatchDeleteServerTagsResponse response = client.batchDeleteServerTags(request);
			return new BocloudResult(true, "删除卷标签成功");
		} catch (Exception e) {
			logger.error("删除卷标签失败： " + e);
			return new BocloudResult(false, "删除卷标签失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String instanceId, String name) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			DeleteServersRequest request = new DeleteServersRequest();
			DeleteServersRequestBody body = new DeleteServersRequestBody();
			ServerId serverId = new ServerId();
			serverId.withId(instanceId);
			body.withServers(Arrays.asList(serverId));
			request.withBody(body);
			DeleteServersResponse response = client.deleteServers(request);
			return waitForJobStatus(response.getJobId(), instanceId, false, "删除");
		} catch (Exception e) {
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

	/**
	 * @Author: lxy
	 * @Description: 查询租户配额信息-云主机。
	 * @Date 2021/8/2 10:32
	 * @Param
	 * @return com.bocloud.common.model.BocloudResult
	 * @throws:
	 **/
	@Override
	public BocloudResult showServerLimitsSolution() {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			ShowServerLimitsRequest request = new ShowServerLimitsRequest();
			ShowServerLimitsResponse response = client.showServerLimits(request);
			ServerLimits absolute = response.getAbsolute();
			ServerLimitsModel serverLimitsModel = new ServerLimitsModel();
			BeanUtils.copyProperties(serverLimitsModel, absolute);
			return new BocloudResult(true, serverLimitsModel, "查询云主机租户配额信息成功");
		} catch (Exception e) {
			e.printStackTrace();
			return new BocloudResult(false, "查询云主机租户配额信息失败", e.getMessage());
		}
	}
}
