package com.bocloud.cmp.provider.network;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.NatGatewayConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NatGatewayModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.huaweicloud.sdk.nat.v2.NatClient;
import com.huaweicloud.sdk.nat.v2.model.CreateNatGatewayOption;
import com.huaweicloud.sdk.nat.v2.model.CreateNatGatewayRequest;
import com.huaweicloud.sdk.nat.v2.model.CreateNatGatewayRequestBody;
import com.huaweicloud.sdk.nat.v2.model.CreateNatGatewayResponse;
import com.huaweicloud.sdk.nat.v2.model.DeleteNatGatewayRequest;
import com.huaweicloud.sdk.nat.v2.model.ListNatGatewaysRequest;
import com.huaweicloud.sdk.nat.v2.model.ListNatGatewaysResponse;
import com.huaweicloud.sdk.nat.v2.model.NatGatewayResponseBody;
import com.huaweicloud.sdk.nat.v2.model.ShowNatGatewayRequest;
import com.huaweicloud.sdk.nat.v2.model.ShowNatGatewayResponse;
import com.huaweicloud.sdk.nat.v2.model.UpdateNatGatewayOption;
import com.huaweicloud.sdk.nat.v2.model.UpdateNatGatewayRequest;
import com.huaweicloud.sdk.nat.v2.model.UpdateNatGatewayRequestBody;
import com.huaweicloud.sdk.nat.v2.model.UpdateNatGatewayResponse;
import io.jsonwebtoken.lang.Assert;

@Provider("bocloud.cmp.huawei")
public class HuaweiNatGatewayProvider extends HuaweiProvider implements NatGatewayProvider {

    private Butler butler;
    
	public HuaweiNatGatewayProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	private static Logger logger = LoggerFactory.getLogger(HuaweiNatGatewayProvider.class);

	private BeanConvertor<NatGatewayModel, NatGatewayResponseBody> convetor = new NatGatewayConvertor();

	@Override
	public BocloudResult list() {
		try {
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			ListNatGatewaysRequest request = new ListNatGatewaysRequest();
			request.withLimit(2000);
			ListNatGatewaysResponse response = client.listNatGateways(request);
			return new BocloudResult(true, convetor.convertList(response.getNatGateways()), "查询列表成功");
		} catch (Exception e) {
			logger.error("查询列表失败： " + e);
			return new BocloudResult(false, "查询列表失败:" + e);
		}
	}

	@Override
	public BocloudResult create(NatGatewayModel model) {
		try {
			Assert.isTrue(StringUtils.isNotEmpty(model.getName()), "参数name不能为空");
			Assert.isTrue(StringUtils.isNotEmpty(model.getVpcUuid()), "参数vpcUuid不能为空");
			Assert.isTrue(StringUtils.isNotEmpty(model.getSubnetUuid()), "参数subnetUuid不能为空");
			Assert.isTrue(StringUtils.isNotEmpty(model.getSpec()), "参数spec不能为空");
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			CreateNatGatewayRequest request = new CreateNatGatewayRequest();
			CreateNatGatewayRequestBody body = new CreateNatGatewayRequestBody();
			CreateNatGatewayOption natGatewaybody = new CreateNatGatewayOption();
			natGatewaybody.withName(model.getName()).withRouterId(model.getVpcUuid())
					.withInternalNetworkId(model.getSubnetUuid()) // 感觉这个是子网,这个就是子网
					.withSpec(CreateNatGatewayOption.SpecEnum.fromValue(model.getSpec()))
					.withDescription(model.getRemark());
			body.withNatGateway(natGatewaybody);
			request.withBody(body);
			CreateNatGatewayResponse response = client.createNatGateway(request);
			BocloudResult waitForServerStatus = waitForServerStatus(response.getNatGateway().getId());
			if(waitForServerStatus.isFailed()) {
			    return waitForServerStatus;
			}
			NatGatewayModel  natGatewayModel = (NatGatewayModel)waitForServerStatus.getData();
			natGatewayModel.setRegionId(this.getRegionId());
            // 同步获取nat的port
            HuaweiPortProvider portProvider = new HuaweiPortProvider(butler);
            BocloudResult portResult = portProvider.list();
            if (portResult.isSuccess()) {
                List<PortModel> portModels = (List<PortModel>)portResult.getData();
                if (!ListTool.isEmpty(portModels)) {
                    for (PortModel portModel : portModels) {
                        if (natGatewayModel.getUuid().equalsIgnoreCase(portModel.getDeviceUuid())) {
                            natGatewayModel.setPortModel(portModel);
                            break;
                        }
                    }
                }
            }
			return new BocloudResult(true, natGatewayModel, "创建nat成功");
		} catch (Exception e) {
			logger.info("创建失败" + e.getMessage());
			return new BocloudResult(false, "创建失败:" + e.getMessage());
		}
	}

	private BocloudResult waitForServerStatus(String natGatewayId) {
		BocloudResult result = new BocloudResult(false, "创建失败");
		Boolean flag = true;
		Long startTime = System.currentTimeMillis();
		while (flag) {
			result = this.detail(natGatewayId);
			if (result.isFailed()) {
				return result;
			}
			NatGatewayModel gateway = JSON.parseObject(JSON.toJSONString(result.getData()), NatGatewayModel.class);
			if (null == gateway) {
				return result;
			}
			if ("ACTIVE".equalsIgnoreCase(gateway.getStatus())) {
				flag = false;
				break;
			} else {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
			}
			Long endTime = System.currentTimeMillis();
			// 等待三分钟
			if (endTime - startTime > 15 * 60 * 1000) {
				flag = false;
				break;
			}
		}
		return result;
	}

	/*
	 * 在admin_state_up = True & status = ACTIVE 时允许更新,支持更新名称、描述、规格。"
	 */
	@Override
	public BocloudResult modify(NatGatewayModel model) {
		try {
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			UpdateNatGatewayRequest request = new UpdateNatGatewayRequest();
			request.withNatGatewayId(model.getUuid());
			UpdateNatGatewayRequestBody body = new UpdateNatGatewayRequestBody();
			UpdateNatGatewayOption natGatewaybody = new UpdateNatGatewayOption();
			natGatewaybody.withName(model.getName()).withDescription(model.getRemark())
					.withSpec(UpdateNatGatewayOption.SpecEnum.fromValue(model.getSpec()));
			body.withNatGateway(natGatewaybody);
			request.withBody(body);
			UpdateNatGatewayResponse response = client.updateNatGateway(request);
			return new BocloudResult(true, waitForServerStatus(response.getNatGateway().getId()).getData(), "修改nat成功");
		} catch (Exception e) {
			logger.info("修改失败" + e.getMessage());
			return new BocloudResult(false, "修改失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String gatewayId) {
		try {
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			DeleteNatGatewayRequest request = new DeleteNatGatewayRequest();
			request.withNatGatewayId(gatewayId);
			client.deleteNatGateway(request);
			return new BocloudResult(true, "删除nat成功");
		} catch (Exception e) {
			logger.info("删除失败" + e.getMessage());
			return new BocloudResult(false, "删除失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String gateway) {
		try {
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			ShowNatGatewayRequest request = new ShowNatGatewayRequest();
			request.withNatGatewayId(gateway);
			ShowNatGatewayResponse response = client.showNatGateway(request);
			NatGatewayModel natGatewayModel = convetor.convertModel(response.getNatGateway());
			natGatewayModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, natGatewayModel, "查询详情成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败:" + e.getMessage());
		}
	}

}
