package com.bocloud.cmp.provider.network;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.NatGatewaySnatRuleConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NatGatewayDnatRuleModel;
import com.bocloud.cmp.model.NatGatewaySnatRuleModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.huaweicloud.sdk.nat.v2.NatClient;
import com.huaweicloud.sdk.nat.v2.model.CreateNatGatewaySnatRuleOption;
import com.huaweicloud.sdk.nat.v2.model.CreateNatGatewaySnatRuleRequest;
import com.huaweicloud.sdk.nat.v2.model.CreateNatGatewaySnatRuleRequestOption;
import com.huaweicloud.sdk.nat.v2.model.CreateNatGatewaySnatRuleResponse;
import com.huaweicloud.sdk.nat.v2.model.DeleteNatGatewaySnatRuleRequest;
import com.huaweicloud.sdk.nat.v2.model.ListNatGatewaySnatRulesRequest;
import com.huaweicloud.sdk.nat.v2.model.ListNatGatewaySnatRulesResponse;
import com.huaweicloud.sdk.nat.v2.model.NatGatewaySnatRuleResponseBody;
import com.huaweicloud.sdk.nat.v2.model.ShowNatGatewaySnatRuleRequest;
import com.huaweicloud.sdk.nat.v2.model.ShowNatGatewaySnatRuleResponse;
import com.huaweicloud.sdk.nat.v2.model.UpdateNatGatewaySnatRuleOption;
import com.huaweicloud.sdk.nat.v2.model.UpdateNatGatewaySnatRuleRequest;
import com.huaweicloud.sdk.nat.v2.model.UpdateNatGatewaySnatRuleRequestOption;
import com.huaweicloud.sdk.nat.v2.model.UpdateNatGatewaySnatRuleResponse;

import io.jsonwebtoken.lang.Assert;

@Provider("bocloud.cmp.huawei")
public class HuaweiNatGatewaySnatRuleProvider extends HuaweiProvider implements NatGatewaySnatRuleProvider {

	public HuaweiNatGatewaySnatRuleProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(HuaweiNatGatewaySnatRuleProvider.class);

	private BeanConvertor<NatGatewaySnatRuleModel, NatGatewaySnatRuleResponseBody> convetor = new NatGatewaySnatRuleConvertor();

	@Override
	public BocloudResult list() {
		try {
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			ListNatGatewaySnatRulesRequest request = new ListNatGatewaySnatRulesRequest();
			request.withLimit(2000);
			ListNatGatewaySnatRulesResponse response = client.listNatGatewaySnatRules(request);
			return new BocloudResult(true, convetor.convertList(response.getSnatRules()), "查询列表成功");
		} catch (Exception e) {
			logger.error("查询列表失败： " + e);
			return new BocloudResult(false, "查询列表失败:" + e);
		}
	}

	/*
	 * 创建规则时,要求网关状态status = ACTIVE,要求网关管理员状态admin_state_up = True。
	 * port_id和private_ip不能同时生效。 对于all port类型的规则,要求internal_service_port =
	 * 0,external_service_port = 0, protocol = ANY.
	 * 
	 * SNAT规则和DNAT规则一般面向不同的业务，如果使用相同的EIP，会面临业务相互抢占问题，请尽量避免。 SNAT规则不能和全端口的DNAT规则共用EIP
	 */
	@Override
	public BocloudResult create(NatGatewaySnatRuleModel model) {
		try {
			Assert.isTrue(StringUtils.isNotEmpty(model.getNatGatewayId()), "参数natGatewayId不能为空");
			Assert.isTrue(StringUtils.isNotEmpty(model.getFloatingIpId()), "参数floatingIpId不能为空");
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			CreateNatGatewaySnatRuleRequest request = new CreateNatGatewaySnatRuleRequest();
			CreateNatGatewaySnatRuleRequestOption body = new CreateNatGatewaySnatRuleRequestOption();
			CreateNatGatewaySnatRuleOption snatRulebody = new CreateNatGatewaySnatRuleOption();
			snatRulebody.withNatGatewayId(model.getNatGatewayId()).withCidr(model.getCidr())
					.withNetworkId(model.getSubnetUuid()) // 子网, 只能在未设置SNAT规则的子网下创建SNAT规则
					.withDescription(model.getRemark()).withSourceType(model.getSourceType())
					.withFloatingIpId(model.getFloatingIpId()); // 多个弹性公网IP使用逗号分隔
			body.withSnatRule(snatRulebody);
			request.withBody(body);
			CreateNatGatewaySnatRuleResponse response = client.createNatGatewaySnatRule(request);
			return new BocloudResult(true, waitForServerStatus(response.getSnatRule().getId()).getData(), "创建snat规则成功");
		} catch (Exception e) {
			logger.info("创建snat规则失败" + e.getMessage());
			return new BocloudResult(false, "创建snat规则失败:" + e.getMessage());
		}
	}

	private BocloudResult waitForServerStatus(String dnatRuleId) {
		BocloudResult result = new BocloudResult(false, "创建失败");
		Boolean flag = true;
		Long startTime = System.currentTimeMillis();
		while (flag) {
			result = this.detail(dnatRuleId);
			if (result.isFailed()) {
				return result;
			}
			NatGatewayDnatRuleModel gateway = JSON.parseObject(JSON.toJSONString(result.getData()),
					NatGatewayDnatRuleModel.class);
			if (null == gateway) {
				return result;
			}
			if ("ACTIVE".equalsIgnoreCase(gateway.getStatus())) {
				flag = false;
				break;
			} else {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
			}
			Long endTime = System.currentTimeMillis();
			// 等待三分钟
			if (endTime - startTime > 15 * 60 * 1000) {
				flag = false;
				break;
			}
		}
		return result;
	}

	/*
	 * 更新操作时,要求DNAT规则状态status = ACTIVE,要求网关管理员状态admin_state_up = True
	 */
	@Override
	public BocloudResult modify(NatGatewaySnatRuleModel model) {
		try {
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			UpdateNatGatewaySnatRuleRequest request = new UpdateNatGatewaySnatRuleRequest();
			request.withSnatRuleId(model.getUuid());
			UpdateNatGatewaySnatRuleRequestOption body = new UpdateNatGatewaySnatRuleRequestOption();
			UpdateNatGatewaySnatRuleOption snatRulebody = new UpdateNatGatewaySnatRuleOption();
			snatRulebody.withNatGatewayId(model.getNatGatewayId()).withPublicIpAddress(model.getFloatingIpAddress())
					.withDescription(model.getRemark());
			body.withSnatRule(snatRulebody);
			request.withBody(body);
			UpdateNatGatewaySnatRuleResponse response = client.updateNatGatewaySnatRule(request);
			return new BocloudResult(true, waitForServerStatus(response.getSnatRule().getId()).getData(), "修改snat规则成功");
		} catch (Exception e) {
			logger.info("修改snat规则失败" + e.getMessage());
			return new BocloudResult(false, "修改snat规则失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String gatewayId, String snanRuleId) {
		try {
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			DeleteNatGatewaySnatRuleRequest request = new DeleteNatGatewaySnatRuleRequest();
			request.withNatGatewayId(gatewayId);
			request.withSnatRuleId(snanRuleId);
			client.deleteNatGatewaySnatRule(request);
			return new BocloudResult(true, "删除snat规则成功");
		} catch (Exception e) {
			logger.info("删除snat规则失败" + e.getMessage());
			return new BocloudResult(false, "删除snat规则失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String ruleId) {
		try {
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			ShowNatGatewaySnatRuleRequest request = new ShowNatGatewaySnatRuleRequest();
			request.withSnatRuleId(ruleId);
			ShowNatGatewaySnatRuleResponse response = client.showNatGatewaySnatRule(request);
			NatGatewaySnatRuleModel snatRuleModel = convetor.convertModel(response.getSnatRule());
			snatRuleModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, snatRuleModel, "查询详情成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败:" + e.getMessage());
		}
	}

}
