package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.PortConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.PortFixedIpModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.huaweicloud.sdk.vpc.v2.VpcClient;
import com.huaweicloud.sdk.vpc.v2.model.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

@Provider("bocloud.cmp.huawei")
public class HuaweiPortProvider extends HuaweiProvider implements PortProvider {

	private static final Logger logger = LoggerFactory.getLogger(HuaweiPortProvider.class);
	private BeanConvertor<PortModel, Port> convertor = new PortConvertor();

	public HuaweiPortProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult sync() {
		return null;
	}

	@Override
	public BocloudResult list() {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			List<Port> allPorts = new ArrayList<>();
			ListPortsRequest request = new ListPortsRequest();
			request.withLimit(PAGE_SIZE);
			ListPortsResponse response = client.listPorts(request);
			List<Port> ports = response.getPorts();
			allPorts.addAll(ports);
			while (null != ports && ports.size() == PAGE_SIZE) {
				Port port = ports.get(ports.size() - 1);
				request.setMarker(port.getId());
				response = client.listPorts(request);
				ports = response.getPorts();
				allPorts.addAll(ports);
			}
			List<PortModel> portModels = convertor.convertList(allPorts);
			if (!ListTool.isEmpty(portModels)) {
				for (PortModel portModel : portModels) {
					portModel.setRegionId(getRegionId());
				}
			}
			return new BocloudResult(true, portModels, "查询成功");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "查询失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult create(PortModel model) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);

			CreatePortRequest request = new CreatePortRequest();
			CreatePortRequestBody body = new CreatePortRequestBody();
			CreatePortOption option = new CreatePortOption();
			option.setName(model.getName());
			option.setNetworkId(model.getNetworkUuid());
			option.setTenantId(getProjectId());
			option.setAdminStateUp(true);
			List<PortFixedIpModel> portFixedIps = model.getPortFixedIps();
			if (!ListTool.isEmpty(portFixedIps)) {
				List<FixedIp> fixedIps = JSONObject.parseArray(JSONObject.toJSONString(portFixedIps), FixedIp.class);
				for (int i = 0; i < portFixedIps.size(); i++) {
					fixedIps.get(i).setSubnetId(portFixedIps.get(i).getSubnetUuid());
				}
				option.setFixedIps(fixedIps);
			}
			List<SecurityGroupModel> securityGroupModels = model.getSecurityGroupModels();
			if (!ListTool.isEmpty(securityGroupModels)) {
				List<String> sgs = new ArrayList<>();
				for (SecurityGroupModel securityGroupModel : securityGroupModels) {
					sgs.add(securityGroupModel.getGroupUuid());
				}
				option.setSecurityGroups(sgs);
			}
			body.setPort(option);
			request.setBody(body);
			CreatePortResponse response = client.createPort(request);
			PortModel convertModel = convertor.convertModel(response.getPort());
			convertModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, convertModel, "创建成功");
		} catch (Exception e) {
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(PortModel portModel) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);

			UpdatePortRequest request = new UpdatePortRequest();
			request.setPortId(portModel.getPortUuid());
			UpdatePortOption option = new UpdatePortOption();
			option.setName(portModel.getName());
			UpdatePortRequestBody body = new UpdatePortRequestBody();
			body.setPort(option);
			request.setBody(body);
			UpdatePortResponse response = client.updatePort(request);
			PortModel convertModel = convertor.convertModel(response.getPort());
			convertModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, convertModel, "修改成功");
		} catch (Exception e) {
			return new BocloudResult(false, "修改失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String portId) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);

			DeletePortRequest request = new DeletePortRequest();
			request.setPortId(portId);
			client.deletePort(request);
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String portId) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			ShowPortRequest request = new ShowPortRequest();
			request.setPortId(portId);
			Port resultPort = client.showPort(request).getPort();
			PortModel convertModel = convertor.convertModel(resultPort);
			convertModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, convertModel, "查询详情成功");
		} catch (Exception e) {
			return new BocloudResult(false, "查询详情失败", e.getMessage());
		}
	}

}
