package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.SecurityGroupConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.vpc.v2.VpcClient;
import com.huaweicloud.sdk.vpc.v2.model.*;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;

@Provider("bocloud.cmp.huawei")
public class HuaweiSecurityGroupProvider extends HuaweiProvider implements SecurityGroupProvider {

	public HuaweiSecurityGroupProvider(Butler butler) {
		super(butler);
	}

	private BeanConvertor<SecurityGroupModel, SecurityGroup> groupConvertor = new SecurityGroupConvertor();

	private static Logger logger = LoggerFactory.getLogger(HuaweiSecurityGroupProvider.class);

	@Override
	public BocloudResult list() {
		try {
			List<SecurityGroup> allSecurityGroups = new ArrayList<>();
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			ListSecurityGroupsRequest request = new ListSecurityGroupsRequest();
			request.withLimit(PAGE_SIZE);
			ListSecurityGroupsResponse response = client.listSecurityGroups(request);
			List<SecurityGroup> securityGroups = response.getSecurityGroups();
			allSecurityGroups.addAll(securityGroups);
			while (null != securityGroups && securityGroups.size() == PAGE_SIZE) {
				SecurityGroup securityGroup = securityGroups.get(securityGroups.size() - 1);
				request.setMarker(securityGroup.getId());
				response = client.listSecurityGroups(request);
				securityGroups = response.getSecurityGroups();
				allSecurityGroups.addAll(securityGroups);
			}
			List<SecurityGroupModel> convertList = groupConvertor.convertList(allSecurityGroups);
			return new BocloudResult(true, convertList, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询列表失败： " + e);
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult create(SecurityGroupModel group) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			CreateSecurityGroupRequest request = new CreateSecurityGroupRequest();
			CreateSecurityGroupRequestBody body = new CreateSecurityGroupRequestBody();
			CreateSecurityGroupOption securityGroupbody = new CreateSecurityGroupOption();
			securityGroupbody.withName(group.getName());
			if (!StringUtils.isEmpty(group.getEnterpriseProjectUuid())) {
				securityGroupbody.setEnterpriseProjectId(group.getEnterpriseProjectUuid());
			}
			body.withSecurityGroup(securityGroupbody);
			request.withBody(body);
			CreateSecurityGroupResponse response = client.createSecurityGroup(request);
			SecurityGroupModel securityGroupModel = groupConvertor.convertModel(response.getSecurityGroup());
			this.removeDefaultIngressRules(securityGroupModel);
			securityGroupModel.setRegionId(this.getRegionId());
			List<SecurityGroupRuleModel> models = group.getModels();
			if (!ListTool.isEmpty(models)) {
				addRules(securityGroupModel, models);
			}
			return new BocloudResult(true, securityGroupModel, "创建成功");
		} catch (Exception e) {
			logger.error("创建失败： " + e);
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	private void removeDefaultIngressRules(SecurityGroupModel securityGroupModel) {
		try {
			List<SecurityGroupRuleModel> createRuleModels = securityGroupModel.getModels();
			if (!ListTool.isEmpty(createRuleModels)) {
				HuaweiSecuGroupRuleProvider ruleProvider = new HuaweiSecuGroupRuleProvider(this.getButler());
				for (int i = createRuleModels.size() - 1; i >= 0; i--) {
					SecurityGroupRuleModel securityGroupRuleModel = createRuleModels.get(i);
					String direction = securityGroupRuleModel.getDirection();
					if ("ingress".equalsIgnoreCase(direction)) {
						BocloudResult removeResult = ruleProvider.remove(securityGroupRuleModel.getRuleUuid());
						if (removeResult.isSuccess()) {
							createRuleModels.remove(securityGroupRuleModel);
						}
					}
				}
			}
			securityGroupModel.setModels(createRuleModels);
		} catch (Exception e) {
			logger.error("移除默认如方向规则异常：" + e);
		}
	}

	private void addRules(SecurityGroupModel securityGroupModel, List<SecurityGroupRuleModel> models) {
		try {
			HuaweiSecuGroupRuleProvider ruleProvider = new HuaweiSecuGroupRuleProvider(getButler());
			List<SecurityGroupRuleModel> securityGroupRules = securityGroupModel.getModels() == null ? new ArrayList<>()
					: securityGroupModel.getModels();
			for (SecurityGroupRuleModel securityGroupRuleModel : models) {
				securityGroupRuleModel.setGroupUuid(securityGroupModel.getGroupUuid());
				BocloudResult bocloudResult = ruleProvider.create(securityGroupRuleModel);
				if (bocloudResult.isSuccess()) {
					SecurityGroupRuleModel model = (SecurityGroupRuleModel) bocloudResult.getData();
					securityGroupRules.add(model);
				}
			}
			securityGroupModel.setModels(securityGroupRules);
		} catch (Exception e) {
			logger.error("添加安全组规则失败：" + e);
		}
	}

	@Override
	public BocloudResult remove(String sgUuid) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			DeleteSecurityGroupRequest request = new DeleteSecurityGroupRequest();
			request.withSecurityGroupId(sgUuid);
			client.deleteSecurityGroup(request);
			return new BocloudResult(true, "安全组删除成功");
		} catch (Exception e) {
			logger.error("删除失败： " + e.getMessage());
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String sgUuid) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			ShowSecurityGroupRequest request = new ShowSecurityGroupRequest();
			request.withSecurityGroupId(sgUuid);
			ShowSecurityGroupResponse response = client.showSecurityGroup(request);
			return new BocloudResult(true, groupConvertor.convertModel(response.getSecurityGroup()), "查询详情成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(SecurityGroupModel model) {
		return null;
	}

}
