package com.bocloud.cmp.provider.network;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.SubnetConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.huaweicloud.sdk.vpc.v2.VpcClient;
import com.huaweicloud.sdk.vpc.v2.model.CreateSubnetOption;
import com.huaweicloud.sdk.vpc.v2.model.CreateSubnetRequest;
import com.huaweicloud.sdk.vpc.v2.model.CreateSubnetRequestBody;
import com.huaweicloud.sdk.vpc.v2.model.CreateSubnetResponse;
import com.huaweicloud.sdk.vpc.v2.model.DeleteSubnetRequest;
import com.huaweicloud.sdk.vpc.v2.model.ListSubnetsRequest;
import com.huaweicloud.sdk.vpc.v2.model.ListSubnetsResponse;
import com.huaweicloud.sdk.vpc.v2.model.ShowSubnetRequest;
import com.huaweicloud.sdk.vpc.v2.model.ShowSubnetResponse;
import com.huaweicloud.sdk.vpc.v2.model.Subnet;
import com.huaweicloud.sdk.vpc.v2.model.UpdateSubnetOption;
import com.huaweicloud.sdk.vpc.v2.model.UpdateSubnetRequest;
import com.huaweicloud.sdk.vpc.v2.model.UpdateSubnetRequestBody;

import io.jsonwebtoken.lang.Assert;

@Provider("bocloud.cmp.huawei")
public class HuaweiSubnetProvider extends HuaweiProvider implements SubnetProvider {

	public HuaweiSubnetProvider(Butler butler) {
		super(butler);
	}

	private static final Logger logger = LoggerFactory.getLogger(HuaweiSubnetProvider.class);

	private BeanConvertor<SubnetModel, Subnet> convertor = new SubnetConvertor();

	@Override
	public BocloudResult list(String vpcId) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			ListSubnetsRequest request = new ListSubnetsRequest();
			if (vpcId != null) {
				request.withVpcId(vpcId);
			}
			request.withLimit(Integer.MAX_VALUE);
			ListSubnetsResponse response = client.listSubnets(request);
			return new BocloudResult(true, convertor.convertList(response.getSubnets()), "查询列表成功");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "查询列表失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult create(SubnetModel subnetModel) {
		try {
			Assert.isTrue(StringUtils.isNotEmpty(subnetModel.getName()), "参数name不能为空");
			Assert.isTrue(StringUtils.isNotEmpty(subnetModel.getVpcUuid()), "参数vpcUuid不能为空");
			Assert.isTrue(StringUtils.isNotEmpty(subnetModel.getCidr()), "参数cidr不能为空");
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			CreateSubnetRequest request = new CreateSubnetRequest();
			CreateSubnetRequestBody body = new CreateSubnetRequestBody();
			CreateSubnetOption subnetbody = new CreateSubnetOption();
			subnetbody.withVpcId(subnetModel.getVpcUuid()).withAvailabilityZone(subnetModel.getZone())
					.withCidr(subnetModel.getCidr()).withName(subnetModel.getName())
					.withGatewayIp(subnetModel.getGateway())
					.withDnsList(JSONObject.parseArray(subnetModel.getDnsNames(), String.class))
					.withIpv6Enable(subnetModel.getIpV6Enable());
			body.withSubnet(subnetbody);
			request.withBody(body);
			CreateSubnetResponse response = client.createSubnet(request);
			return new BocloudResult(true, convertor.convertModel(response.getSubnet()), "子网创建成功");
		} catch (Exception e) {
			logger.error("子网创建失败：" + e);
			if(!StringUtils.isEmpty(e.getMessage()) && e.getMessage().contains("not in the range of VPC")) {
				return new BocloudResult<>(false, "创建失败：子网超出VPC网段范围");
			}
			return new BocloudResult(false, "创建失败:" + this.getChineseMessage(e.getMessage()));
		}
	}

	@Override
	public BocloudResult modify(SubnetModel subnetModel) {
		try {
			Assert.isTrue(StringUtils.isNotEmpty(subnetModel.getName()), "参数name不能为空");
			Assert.isTrue(StringUtils.isNotEmpty(subnetModel.getSubnetUuid()), "参数subnetUuid不能为空");
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			UpdateSubnetRequest request = new UpdateSubnetRequest();
			request.withVpcId(subnetModel.getVpcUuid());
			request.withSubnetId(subnetModel.getSubnetUuid());
			UpdateSubnetRequestBody body = new UpdateSubnetRequestBody();
			UpdateSubnetOption subnetbody = new UpdateSubnetOption();
			subnetbody.withName(subnetModel.getName())
					.withDnsList(JSONObject.parseArray(subnetModel.getDnsNames(), String.class));
			subnetbody.withIpv6Enable(subnetModel.getIpV6Enable());
			body.withSubnet(subnetbody);
			request.withBody(body);
			client.updateSubnet(request);
			return new BocloudResult(true, detail(subnetModel.getSubnetUuid()).getData(), "子网修改成功");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "子网修改失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(SubnetModel subnet) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			DeleteSubnetRequest request = new DeleteSubnetRequest();
			request.withVpcId(subnet.getVpcUuid());
			request.withSubnetId(subnet.getSubnetUuid());
			client.deleteSubnet(request);
			return new BocloudResult(true, "子网删除成功");
		} catch (Exception e) {
			logger.error(e.getMessage());
			String message = e.getMessage();
			if (!StringUtils.isEmpty(message) && message.contains("subnet is still used")) {
				return new BocloudResult<>(false, "VPC内含关联云资源，请全部释放后再进行删除操作");
			}
			return new BocloudResult(false, "子网删除失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String subnetId) {
		return null;
	}

	@Override
	public BocloudResult detail(String subnetUuid) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			ShowSubnetRequest request = new ShowSubnetRequest();
			request.withSubnetId(subnetUuid);
			ShowSubnetResponse response = client.showSubnet(request);
			return new BocloudResult(true, convertor.convertModel(response.getSubnet()), "查询详情成功");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "查询详情失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult list() {
		// TODO Auto-generated method stub
		return null;
	}

}
