package com.bocloud.cmp.provider.network;


import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.VpcConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.huaweicloud.sdk.vpc.v2.VpcClient;
import com.huaweicloud.sdk.vpc.v2.model.CreateVpcOption;
import com.huaweicloud.sdk.vpc.v2.model.CreateVpcRequest;
import com.huaweicloud.sdk.vpc.v2.model.CreateVpcRequestBody;
import com.huaweicloud.sdk.vpc.v2.model.CreateVpcResponse;
import com.huaweicloud.sdk.vpc.v2.model.DeleteVpcRequest;
import com.huaweicloud.sdk.vpc.v2.model.ListVpcsRequest;
import com.huaweicloud.sdk.vpc.v2.model.ListVpcsResponse;
import com.huaweicloud.sdk.vpc.v2.model.ShowVpcRequest;
import com.huaweicloud.sdk.vpc.v2.model.ShowVpcResponse;
import com.huaweicloud.sdk.vpc.v2.model.Vpc;

import io.jsonwebtoken.lang.Assert;


@Provider("bocloud.cmp.huawei")
public class HuaweiVpcProvider extends HuaweiProvider implements VpcProvider {

	private static Logger logger = LoggerFactory.getLogger(HuaweiVpcProvider.class);

	private BeanConvertor<VpcModel, Vpc> convertor = new VpcConvertor();

	private HuaweiSubnetProvider subnetProvider;
	
	public HuaweiVpcProvider(Butler butler) {
		super(butler);
		subnetProvider = new HuaweiSubnetProvider(butler);
	}

	@Override
	public BocloudResult create(VpcModel model) {
		try {
			Assert.isTrue(StringUtils.isNotEmpty(model.getName()), "参数name不能为空");
			Assert.isTrue(StringUtils.isNotEmpty(model.getCidr()), "参数cidr不能为空");
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			CreateVpcRequest request = new CreateVpcRequest();
			CreateVpcRequestBody body = new CreateVpcRequestBody();
			CreateVpcOption vpcbody = new CreateVpcOption();
			vpcbody.withCidr(model.getCidr())
					.withName(model.getName());
			String enterpriseProjectUuid = model.getEnterpriseProjectUuid();
			if (!StringUtils.isEmpty(enterpriseProjectUuid)) {
				vpcbody.withEnterpriseProjectId(enterpriseProjectUuid);
			}
			body.withVpc(vpcbody);
			request.withBody(body);
			CreateVpcResponse response = client.createVpc(request);
			BocloudResult result = waitForServerStatus(response.getVpc().getId());
			if (result.isFailed()) {
				return new BocloudResult(false, "创建vpc成功,但查询vpc失败");
			}
			BocloudResult subnetResult = subnetProvider.list(response.getVpc().getId());
			if (subnetResult.isFailed()) {
				return new BocloudResult(false, "创建vpc成功,但查询vpc的默认子网失败");
			}
			VpcModel vpcModel = (VpcModel) result.getData();
			vpcModel.setSubnetModels((List<SubnetModel>) subnetResult.getData());
			return new BocloudResult(true, vpcModel, "创建vpc成功");
		} catch (Exception e) {
			logger.info("创建vpc失败" + e.getMessage());
			return new BocloudResult(false, "创建失败:" + e.getMessage());
		}
	}

	private BocloudResult waitForServerStatus(String vpcId) {
		BocloudResult result = new BocloudResult(false, "创建失败");
		Boolean flag = true;
		Long startTime = System.currentTimeMillis();
		while (flag) {
			result = this.detail(vpcId, null);
			if (result.isFailed()) {
				return result;
			}
			VpcModel vpcModel = JSON.parseObject(JSON.toJSONString(result.getData()), VpcModel.class);
			if (null == vpcModel) {
				return result;
			}
			if ("OK".equalsIgnoreCase(vpcModel.getStatus()) || "ACTIVE".equalsIgnoreCase(vpcModel.getStatus())) {
				flag = false;
				break;
			} else {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
			}
			Long endTime = System.currentTimeMillis();
			// 等待三分钟
			if (endTime - startTime > 15 * 60 * 1000) {
				flag = false;
				break;
			}
		}
		return result;
	}

	@Override
	public BocloudResult remove(String vpcId) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			DeleteVpcRequest request = new DeleteVpcRequest();
			request.withVpcId(vpcId);
			client.deleteVpc(request);
			return new BocloudResult(true, "vpc删除成功");
		} catch (Exception e) {
			return new BocloudResult(false, "删除失败:" + e.getMessage());
		}
	}


	@Override
	public BocloudResult list() {
		try {
			List<Vpc> allVpcs = new ArrayList<>();
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			ListVpcsRequest request = new ListVpcsRequest();
			request.withLimit(PAGE_SIZE);
			ListVpcsResponse response = client.listVpcs(request);
			List<Vpc> vpcs = response.getVpcs();
			allVpcs.addAll(vpcs);
			while (null != vpcs && vpcs.size() == PAGE_SIZE) {
				Vpc vpc = vpcs.get(vpcs.size() - 1);
				request.setMarker(vpc.getId());
				response = client.listVpcs(request);
				vpcs = response.getVpcs();
				allVpcs.addAll(vpcs);
			}
			List<VpcModel> convertList = convertor.convertList(allVpcs);
			return new BocloudResult(true, convertList, "查询列表成功");
		} catch (Exception e) {
			return new BocloudResult(false, "查询失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String vpcId, String region) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			ShowVpcRequest request = new ShowVpcRequest();
			request.withVpcId(vpcId);
			ShowVpcResponse response = client.showVpc(request);
			VpcModel vpcModel = convertor.convertModel(response.getVpc());
			vpcModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, vpcModel, "查询详情成功");
		} catch (Exception e) {
			return new BocloudResult(false, "查询详情失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult update(VpcModel model) {
		return null;
	}

}
