package com.bocloud.cmp.provider.storage;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.SnapshotConvertor;
import com.bocloud.cmp.huaweiconvertor.VolumeConvertor;
import com.bocloud.cmp.huaweiconvertor.VolumeTypeConvertor;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.compute.HuaweiSnapshotProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.huaweicloud.sdk.ecs.v2.EcsClient;
import com.huaweicloud.sdk.ecs.v2.model.*;
import com.huaweicloud.sdk.evs.v2.EvsClient;
import com.huaweicloud.sdk.evs.v2.model.ShowJobRequest;
import com.huaweicloud.sdk.evs.v2.model.ShowJobResponse;
import com.huaweicloud.sdk.evs.v2.model.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 华为私有云卷操作driver
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年9月17日
 */

@Provider("bocloud.cmp.huawei")
public class HuaweiVolumeProvider extends HuaweiProvider implements VolumeProvider {

	private static final Logger logger = LoggerFactory.getLogger(HuaweiVolumeProvider.class);
	private BeanConvertor<VolumeModel, VolumeDetail> convertor = new VolumeConvertor();
	private BeanConvertor<SnapshotModel, SnapshotList> snapshotConvertor = new SnapshotConvertor();
	private BeanConvertor<VolumeTypeModel, VolumeType> volumeTypeConvertor = new VolumeTypeConvertor();

	/**
	 * @param butler
	 */

	public HuaweiVolumeProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		List<VolumeModel> volumeModels = new ArrayList<>();
		try {
			List<VolumeDetail> listAllVolumes = listAllVolumes();
			volumeModels = convertor.convertList(listAllVolumes);
			volumeModels.forEach(volumeModel -> volumeModel.setRegionId(this.getRegionId()));
			return new BocloudResult(true, volumeModels, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询列表失败： " + e);
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	public List<VolumeDetail> listAllVolumes() {
		List<VolumeDetail> volumes = new ArrayList<>();
		try {
			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);
			ListVolumesRequest request = new ListVolumesRequest();
			request.setOffset(0);
			request.withLimit(PAGE_SIZE);
			ListVolumesResponse response = client.listVolumes(request);
			Integer totalCount = response.getCount();
			if (null == totalCount || totalCount == 0) {
				return volumes;
			}
			volumes = response.getVolumes();
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				logger.info("硬盘数量 ： " + volumes.size());
				return volumes;
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setOffset(PAGE_SIZE * (i - 1));
				request.withLimit(PAGE_SIZE);
				response = client.listVolumes(request);
				volumes.addAll(response.getVolumes());
			}
			logger.info("硬盘数量 ： " + volumes.size());
		} catch (Exception e) {
			logger.error("查询硬盘列表失败： " + e);
		}
		return volumes;

	}
	
	@Override
	public BocloudResult create(VolumeModel volume) {
		// SSD 超高IO
		// SATA 普通IO
		// SAS 高IO
		try {

			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);

			CreateVolumeRequest request = new CreateVolumeRequest();

			CreateVolumeRequestBody body = new CreateVolumeRequestBody();

			/*
			 * BssParamForCreateVolume bssParam = new BssParamForCreateVolume();
			 *//*
				 * 功能说明:计费模式。默认值为postPaid。
				 * 
				 * prePaid:包年包月 postPaid:按需
				 *//*
					 * bssParam.setChargingMode(BssParamForCreateVolume.ChargingModeEnum.POSTPAID);
					 */

			CreateVolumeOption createVolume = new CreateVolumeOption();
			createVolume.withAvailabilityZone(volume.getZone()).withCount(volume.getCopyCount());

			if (null != volume.getSourceVolidUuid() && !"".equals(volume.getSourceVolidUuid())) {
				// 备份ID,从备份创建云硬盘时为必选。
				createVolume.withBackupId(volume.getSourceVolidUuid());
			}
			String enterpriseProjectUuid = volume.getEnterpriseProjectUuid();
			if(!StringUtils.isEmpty(enterpriseProjectUuid)) {
				createVolume.withEnterpriseProjectId(enterpriseProjectUuid);
			}

			createVolume.withDescription(volume.getRemark());

			if (null != volume.getImageUuid() && !"".equals(volume.getImageUuid())) {
				createVolume.withImageRef(volume.getImageUuid());
			}

			createVolume.withMultiattach(volume.getMultiattach()).withName(volume.getName()).withSize(volume.getSize());

			if (null != volume.getSourceSnapUuid() && !"".equals(volume.getSourceSnapUuid())) {
				createVolume.setSnapshotId(volume.getSourceSnapshotId());
			}

			/*
			 * 目前支持“SSD”,“SAS”和“SATA”三种。 “SSD”为超高IO云硬盘 "GPSSD"为通用型SSD云硬盘 “SAS”为高IO云硬盘
			 * “SATA”为普通IO云硬盘 当指定的云硬盘类型在avaliability_zone内不存在时,则创建云硬盘失败。
			 * 
			 * 说明: 从快照创建云硬盘时,volume_type字段必须和快照源云硬盘保持一致。
			 */

			createVolume.setVolumeType(CreateVolumeOption.VolumeTypeEnum.fromValue(volume.getType()));
			body.setVolume(createVolume);
			request.setBody(body);
			CreateVolumeResponse response = client.createVolume(request);
			String jobId = response.getJobId();

			if (StringUtils.isEmpty(jobId)) {
				return new BocloudResult(false, "创建失败", "");
			}

			return waitForEvsJobStatus(jobId, "创建硬盘");

		} catch (Exception e) {
			logger.error("创建失败： " + e);
			// is sold out
			if(e.getMessage().contains("is sold out")) {
				return new BocloudResult<>(false, "创建失败：当前类型磁盘已售罄");
			}
			return new BocloudResult(false, "创建失败:" + e.getMessage());
		}
	}

	public BocloudResult waitForServerStatus(String volumeId) {
		BocloudResult result = new BocloudResult(false, "创建失败");
		Boolean flag = true;
		Long startTime = System.currentTimeMillis();
		while (flag) {
			result = this.detail(volumeId);
			if (result.isFailed()) {
				return result;
			}
			VolumeModel volumeModel = JSON.parseObject(JSON.toJSONString(result.getData()), VolumeModel.class);
			if (null == volumeModel) {
				return result;
			}
			if ("OK".equalsIgnoreCase(volumeModel.getStatus()) || "ACTIVE".equalsIgnoreCase(volumeModel.getStatus())) {
				flag = false;
				break;
			} else {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
			}
			Long endTime = System.currentTimeMillis();
			// 等待三分钟
			if (endTime - startTime > 15 * 60 * 1000) {
				flag = false;
				break;
			}
		}
		return result;
	}

	@Override
	public BocloudResult modify(VolumeModel volume) {
		try {

			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);
			UpdateVolumeRequest request = new UpdateVolumeRequest();
			request.withVolumeId(volume.getVolumeUuid());
			UpdateVolumeRequestBody body = new UpdateVolumeRequestBody();
			UpdateVolumeOption updateVolume = new UpdateVolumeOption();
			updateVolume.withDescription(volume.getRemark()).withName(volume.getName());
			body.setVolume(updateVolume);
			request.setBody(body);
			UpdateVolumeResponse response = client.updateVolume(request);
			return new BocloudResult(true, this.detail(volume.getVolumeUuid()).getData(), "查询详情成功");

		} catch (Exception e) {
			logger.error("修改失败： " + e);
			return new BocloudResult(false, "修改失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String volumeId) {
		try {

			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);

			DeleteVolumeRequest request = new DeleteVolumeRequest();
			request.withVolumeId(volumeId);
			DeleteVolumeResponse response = client.deleteVolume(request);

			String jobId = response.getJobId();

			BocloudResult result = waitForEvsJobStatus(jobId, "删除硬盘");

			if (result.isSuccess()) {
				return new BocloudResult(true, result.isSuccess(), "删除成功");
			} else {
				return new BocloudResult(false, "删除失败", result.getMessage());
			}
		} catch (Exception e) {
			logger.error("删除失败： " + e);
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

	public BocloudResult waitForEvsJobStatus(String jobId, String message) {
		BocloudResult result = new BocloudResult(false, "任务失败");
		Boolean flag = true;
		Long startTime = System.currentTimeMillis();
		while (flag) {
			result = this.getEvsJobStatus(jobId);
			if (result.isFailed()) {
				return result;
			}
			ShowJobResponse jobResponse = (ShowJobResponse) result.getData();

			if ("SUCCESS".equalsIgnoreCase(jobResponse.getStatus().toString())) {
				flag = false;
				if ("deleteVolume".equals(jobResponse.getJobType())) {
					result = new BocloudResult(true, message + "成功");
				} else {
					result = new BocloudResult(true, detail(jobResponse.getEntities().getVolumeId()).getData(),
							message + "成功");
				}
				break;
			} else if ("FAIL".equalsIgnoreCase(jobResponse.getStatus().toString())) {
				flag = false;
				result.setSuccess(false);
				result.setMessage(jobResponse.getFailReason());
				break;
			} else {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
			}
			Long endTime = System.currentTimeMillis();
			// 等待三分钟
			if (endTime - startTime > 15 * 60 * 1000) {
				flag = false;
				break;
			}
		}
		return result;
	}

	// @Override
	public BocloudResult getEvsJobStatus(String jobId) {
		try {
			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);

			ShowJobRequest request = new ShowJobRequest();
			request.withJobId(jobId);
			ShowJobResponse response = client.showJob(request);

			return new BocloudResult(true, response, "查询任务详情成功");

		} catch (Exception e) {
			logger.error("查询任务详情失败： " + e);
			return new BocloudResult(false, "查询任务详情失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String volumeId) {
		try {
			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);

			ShowVolumeRequest request = new ShowVolumeRequest();
			request.setVolumeId(volumeId);
			ShowVolumeResponse response = client.showVolume(request);
			VolumeModel convertModel = convertor.convertModel(response.getVolume());
			convertModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, convertModel, "查询详情成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult snapshot(SnapshotModel snapshotModel) {
		try {

			// 创建磁盘快照
			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);

			CreateSnapshotRequest request = new CreateSnapshotRequest();

			CreateSnapshotRequestBody body = new CreateSnapshotRequestBody();

			CreateSnapshotOption snapshot = new CreateSnapshotOption();
			snapshot.withVolumeId(snapshotModel.getVolumeUuid()).withForce(true).withName(snapshotModel.getName())
					.withDescription(snapshotModel.getRemark());

			body.withSnapshot(snapshot);
			request.withBody(body);
			CreateSnapshotResponse response = client.createSnapshot(request);

			SnapshotDetails snapshotDetails = response.getSnapshot();

			// 查询磁盘快照
			ListSnapshotsRequest request1 = new ListSnapshotsRequest();
			request1.withId(snapshotDetails.getId());

			ListSnapshotsResponse response1 = client.listSnapshots(request1);

			if (response1.getCount() < 1) {
				return new BocloudResult(false, "快照创建失败");
			}

			SnapshotModel convertModel = snapshotConvertor.convertModel(response1.getSnapshots().get(0));
			convertModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, convertModel, "快照创建成功");
		} catch (Exception e) {
			logger.error("快照创建失败： " + e);
			return new BocloudResult(false, "快照创建失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult attach(String serverId, String volumeId) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			String device = getDevice(serverId, client);

			AttachServerVolumeRequest attaRequest = new AttachServerVolumeRequest();
			attaRequest.withServerId(serverId);

			AttachServerVolumeRequestBody body = new AttachServerVolumeRequestBody();

			AttachServerVolumeOption volumeAttachment = new AttachServerVolumeOption();
			volumeAttachment.withVolumeId(volumeId);
			volumeAttachment.withDevice(device);
			body.withVolumeAttachment(volumeAttachment);
			attaRequest.withBody(body);

			client.attachServerVolume(attaRequest);

			BocloudResult waitForStatus = waitForStatus(volumeId, "in-use");
			if (waitForStatus.isFailed()) {
				return waitForStatus;
			}
			// 重新查询弹性云服务器磁盘信息
			ListServerBlockDevicesRequest newRequest = new ListServerBlockDevicesRequest();
			newRequest.withServerId(serverId);
			ListServerBlockDevicesResponse newResponse = client.listServerBlockDevices(newRequest);
			// 挂载盘列表
			List<ServerBlockDevice> newVolumeAttachments = newResponse.getVolumeAttachments();

			if (ListTool.isEmpty(newVolumeAttachments)) {
				return new BocloudResult(false, "failed!");
			}

			ServerBlockDevice attachment = null;

			for (ServerBlockDevice blockDevice : newVolumeAttachments) {

				if (volumeId.equals(blockDevice.getVolumeId())) {
					attachment = blockDevice;
					break;
				}

			}

			if (null == attachment) {
				return new BocloudResult(false, "failed!");
			} else {
				return new BocloudResult(true, attachment.getDevice(), "success");
			}

		} catch (Exception e) {
			logger.error("云服务器挂载磁盘失败： " + e);
			return new BocloudResult(false, "云服务器挂载磁盘失败", e.getMessage());
		}

	}

	private String getDevice(String serverId, EcsClient client) {
		String device = "/dev/vd";
		try {
			// 查询弹性云服务器磁盘信息
			ListServerBlockDevicesRequest request = new ListServerBlockDevicesRequest();
			request.withServerId(serverId);
			ListServerBlockDevicesResponse response = client.listServerBlockDevices(request);
			// 挂载盘列表
			List<ServerBlockDevice> volumeAttachments = response.getVolumeAttachments();
			List<String> oldDevice = new ArrayList<>();
			if (!ListTool.isEmpty(volumeAttachments)) {
				for (ServerBlockDevice serverBlockDevice : volumeAttachments) {
					String blockDevice = serverBlockDevice.getDevice();
					oldDevice.add(blockDevice.substring(blockDevice.length() - 1, blockDevice.length()));
				}
			}
			List<String> strs = new ArrayList();
			for (int i = 0; i < 26; i++) {
				strs.add((char) (97 + i) + "");
			}
			strs.removeAll(oldDevice);
			if (!ListTool.isEmpty(strs)) {
				device = device + strs.get(0);
			}
		} catch (Exception e) {
			logger.error("获取挂载点异常：" + e.getMessage());
		}
		return device;
	}

	public BocloudResult waitForEcsJobStatus(String jobId) {
		BocloudResult result = new BocloudResult(false, "任务失败");
		Boolean flag = true;
		Long startTime = System.currentTimeMillis();
		while (flag) {
			result = this.getEcsJobStatus(jobId);
			if (result.isFailed()) {
				return result;
			}
			String status = result.getData().toString();

			if ("SUCCESS".equalsIgnoreCase(status)) {
				flag = false;
				result.setSuccess(true);
				break;
			} else if ("FAIL".equalsIgnoreCase(status)) {
				flag = false;
				result.setSuccess(false);
				break;
			} else {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
			}
			Long endTime = System.currentTimeMillis();
			// 等待三分钟
			if (endTime - startTime > 15 * 60 * 1000) {
				flag = false;
				break;
			}
		}
		return result;
	}

	// @Override
	public BocloudResult getEcsJobStatus(String jobId) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);

			com.huaweicloud.sdk.ecs.v2.model.ShowJobRequest request = new com.huaweicloud.sdk.ecs.v2.model.ShowJobRequest();
			request.withJobId(jobId);
			com.huaweicloud.sdk.ecs.v2.model.ShowJobResponse response = client.showJob(request);
			return new BocloudResult(true, response.getStatus(), "查询任务详情成功");

		} catch (Exception e) {
			logger.error("查询任务详情失败： " + e);
			return new BocloudResult(false, "查询任务详情失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult attach(String serverId, String mountpoint, String volumeId) {
		// os.blockStorage().volumes().get(volume.getId()).getStatus().toString().equals("in-use")
		// 挂载成功
		try {

			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			// 查询弹性云服务器磁盘信息
			ListServerBlockDevicesRequest request = new ListServerBlockDevicesRequest();
			request.withServerId(serverId);
			ListServerBlockDevicesResponse response = client.listServerBlockDevices(request);
			// 挂载盘列表
			List<ServerBlockDevice> volumeAttachments = response.getVolumeAttachments();
			String device = "/dev/vda";
			List<String> strs = new ArrayList<String>();
			for (int i = 0; i < 26; i++) {
				strs.add((char) (97 + i) + "");
			}
			if (!ListTool.isEmpty(volumeAttachments)) {
				device = "/dev/vd" + strs.get(volumeAttachments.size());
			}

			AttachServerVolumeRequest attaRequest = new AttachServerVolumeRequest();
			attaRequest.withServerId(serverId);
			AttachServerVolumeRequestBody body = new AttachServerVolumeRequestBody();
			AttachServerVolumeOption volumeAttachment = new AttachServerVolumeOption();
			volumeAttachment.withVolumeId(volumeId);
			volumeAttachment.withDevice(device);
			body.withVolumeAttachment(volumeAttachment);
			attaRequest.withBody(body);
			AttachServerVolumeResponse attaResponse = client.attachServerVolume(attaRequest);
			BocloudResult waitForStatus = waitForStatus(volumeId, "in-use");
			if (waitForStatus.isFailed()) {
				return waitForStatus;
			}
			// 重新查询弹性云服务器磁盘信息
			ListServerBlockDevicesRequest newRequest = new ListServerBlockDevicesRequest();
			newRequest.withServerId(serverId);
			ListServerBlockDevicesResponse newResponse = client.listServerBlockDevices(newRequest);
			// 挂载盘列表
			List<ServerBlockDevice> newVolumeAttachments = newResponse.getVolumeAttachments();

			if (ListTool.isEmpty(newVolumeAttachments)) {
				return new BocloudResult(false, "failed!");
			}
			ServerBlockDevice attachment = null;
			for (ServerBlockDevice blockDevice : newVolumeAttachments) {
				if (volumeId.equals(blockDevice.getVolumeId())) {
					attachment = blockDevice;
					break;
				}
			}
			if (null == attachment) {
				return new BocloudResult(false, "failed!");
			} else {
				return new BocloudResult(true, attachment.getDevice(), "挂载成功");
			}
		} catch (Exception e) {
			logger.error("挂载失败： " + e);
			return new BocloudResult(false, "挂载失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detach(String serverId, String volumeId) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			DetachServerVolumeRequest request = new DetachServerVolumeRequest();
			request.withServerId(serverId).withVolumeId(volumeId);
			DetachServerVolumeResponse response = client.detachServerVolume(request);
			// String jobId = attaResponse.getJobId();

			// return waitForEcsJobStatus(jobId);
			BocloudResult waitForStatus = waitForStatus(volumeId, "AVAILABLE");
			if (waitForStatus.isFailed()) {
				return waitForStatus;
			}
			return new BocloudResult(true, "卸载成功", "卸载成功");
		} catch (Exception e) {
			logger.error("卸载失败： " + e);
			return new BocloudResult(false, "卸载失败", e.getMessage());
		}
	}

	public BocloudResult waitForStatus(String volumeId, String status) {
		BocloudResult result = new BocloudResult(false, "操作失败!");
		Boolean flag = true;
		Long startTime = System.currentTimeMillis();
		while (flag) {

			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);

			ShowVolumeRequest request = new ShowVolumeRequest();
			request.setVolumeId(volumeId);
			ShowVolumeResponse response = client.showVolume(request);
			VolumeDetail volume = response.getVolume();
			if (null == volume) {
				return result;
			}
			String volumeStatus = volume.getStatus();
			if (!StringUtils.isEmpty(volumeStatus) && volumeStatus.equalsIgnoreCase(status)) {
				flag = false;
				result = new BocloudResult(true, convertor.convertModel(volume), "success");
				break;
			} else {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
			}
			Long endTime = System.currentTimeMillis();
			// 等待三分钟
			if (endTime - startTime > 15 * 60 * 1000) {
				flag = false;
				break;
			}
		}
		return result;
	}

	@Override
	public BocloudResult resetStatus(String volumeId, String status) {

		// todo 新sdk没有找到对应的接口
		/*
		 * try { Status paramStatus = null; Status[] values = Status.values(); for
		 * (Status st : values) { if (st.toString().equalsIgnoreCase(status)) {
		 * paramStatus = st; break; } } ActionResponse actionResponse =
		 * this.getClient().compute().servers().resetState(volumeId, paramStatus);
		 * return new BocloudResult(true, actionResponse.isSuccess(), "卸载成功"); } catch
		 * (Exception e) { logger.error("卸载失败： " + e); return new BocloudResult(false,
		 * "卸载失败", e.getMessage()); }
		 */

		return null;
	}

	@Override
	public BocloudResult uploadToImage(String volumeId, ImageModel imageModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult extend(String volumeId, Integer newSize) {
		try {
			// ActionResponse actionResponse =
			// this.getClient().blockStorage().volumes().extend(volumeId, newSize);

			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);
			ResizeVolumeRequest request = new ResizeVolumeRequest();

			ResizeVolumeRequestBody body = new ResizeVolumeRequestBody();

			OsExtend osExtend = new OsExtend();
			osExtend.withNewSize(newSize);
			body.withOsExtend(osExtend);
			request.withVolumeId(volumeId).withBody(body);

			ResizeVolumeResponse response = client.resizeVolume(request);

			String jobId = response.getJobId();

			BocloudResult result = waitForEvsJobStatus(jobId, "扩容硬盘");

			if (result.isSuccess()) {
				return new BocloudResult(true, result.isSuccess(), "扩容成功");
			} else {
				return new BocloudResult(false, "扩容失败", result.getMessage());
			}
		} catch (Exception e) {
			logger.error("扩容失败： " + e);
			return new BocloudResult(false, "扩容失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult listVolumeTypes() {
		try {
			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);
			CinderListVolumeTypesRequest request = new CinderListVolumeTypesRequest();
			CinderListVolumeTypesResponse response = client.cinderListVolumeTypes(request);
			List<VolumeTypeModel> volumeTypeModels = volumeTypeConvertor.convertList(response.getVolumeTypes());
			volumeTypeModels.stream().forEach(volumeTypeModel -> volumeTypeModel.setRegionId(this.getRegionId()));
			return new BocloudResult(true, volumeTypeModels, "查询卷类型成功");
		} catch (Exception e) {
			logger.error("查询卷类型失败： " + e);
			return new BocloudResult(false, "查询卷类型失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult list(String hostId, String storagePoolName) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult createTag(VolumeModel model) {
		try {
			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);
			BatchCreateVolumeTagsRequest request = new BatchCreateVolumeTagsRequest();
			request.withVolumeId(model.getVolumeUuid());
			BatchCreateVolumeTagsRequestBody body = new BatchCreateVolumeTagsRequestBody();
			body.withAction(BatchCreateVolumeTagsRequestBody.ActionEnum.CREATE)
					.withTags(JSONObject.parseArray(JSONObject.toJSONString(model.getTagModels()), Tag.class));
			request.withBody(body);
			BatchCreateVolumeTagsResponse response = client.batchCreateVolumeTags(request);
			return new BocloudResult(true, "创建卷标签成功");
		} catch (Exception e) {
			logger.error("创建卷标签失败： " + e);
			return new BocloudResult(false, "创建卷标签失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult removeTag(VolumeModel model) {
		try {
			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);
			BatchDeleteVolumeTagsRequest request = new BatchDeleteVolumeTagsRequest();
			request.withVolumeId(model.getVolumeUuid());
			BatchDeleteVolumeTagsRequestBody body = new BatchDeleteVolumeTagsRequestBody();
			body.withAction(BatchDeleteVolumeTagsRequestBody.ActionEnum.DELETE).withTags(
					JSONObject.parseArray(JSONObject.toJSONString(model.getTagModels()), DeleteTagsOption.class));
			request.withBody(body);
			BatchDeleteVolumeTagsResponse response = client.batchDeleteVolumeTags(request);
			return new BocloudResult(true, "删除卷标签成功");
		} catch (Exception e) {
			logger.error("删除卷标签失败： " + e);
			return new BocloudResult(false, "删除卷标签失败：" + e.getMessage());
		}
	}

	/** @Author: lxy
	 * @Description: 查询云硬盘租户配额信息
	 * @Date 2021/8/2 16:58
	 * @Param
	 * @return com.bocloud.common.model.BocloudResult
	 * @throws:
	 **/
	@Override
	public BocloudResult cinderListQuotasSolution() {
		try {
			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);
			CinderListQuotasRequest request = new CinderListQuotasRequest();
			request.withTargetProjectId(getProjectId());
			request.withUsage(CinderListQuotasRequest.UsageEnum.fromValue("true"));
			CinderListQuotasResponse response = client.cinderListQuotas(request);
			QuotaList quotaSet = response.getQuotaSet();
			QuotaDetailGigabytes gigabytes = quotaSet.getGigabytes();
			return new BocloudResult(true,JSONObject.toJSONString(gigabytes), "查询云硬盘租户配额信息成功");
		} catch (Exception e) {
			e.printStackTrace();
			return new BocloudResult(false, "查询云硬盘租户配额信息失败", e.getMessage());
		}
	}
}
