package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.provider.JdCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.jdconvertor.InstanceConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import com.jdcloud.sdk.service.disk.model.DiskSpec;
import com.jdcloud.sdk.service.vm.model.*;
import com.jdcloud.sdk.service.vpc.model.NetworkInterfaceSpec;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @ Description： @ Author ：丁明威 @ Date ：Created in 17:23 2018/11/14 @ Modified
 * By：郭鹏
 */
@Provider("bocloud.cmp.jdcloud")
public class JdCloudServerProvider extends JdCloudProvider implements ServerProvider {

	private static Logger logger = LoggerFactory.getLogger(JdCloudServerProvider.class);

	private BeanConvertor<ServerModel, Instance> instanceConvertor = new InstanceConvertor();
	private static final int PAGE_SIZE = 100;
	private Butler butler;

	/**
	 * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
	 *
	 * @param butler 云平台认证信息
	 */
	public JdCloudServerProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult list() {
		try {
			int pageNumber = 1;
			// 云主机
			DescribeInstancesRequest request = new DescribeInstancesRequest();
			request.setRegionId(butler.getRegionId());
			request.setPageSize(PAGE_SIZE);
			request.setPageNumber(pageNumber);
			DescribeInstancesResponse instanceResponse = this.getVmClient().describeInstances(request);
			DescribeInstancesResult instanceResult = instanceResponse.getResult();
			if (null == instanceResult) {
				return new BocloudResult(false, "查询云主机失败");
			}
			Integer totalCount = instanceResult.getTotalCount().intValue();
			if (totalCount == 0) {
				logger.info("查询云主机列表:totalCount为0");
				return new BocloudResult(true, new ArrayList<KeypairModel>(), "empty");
			}
			List<Instance> instances = instanceResult.getInstances();
			int totalPage = (instanceResult.getTotalCount().intValue() % PAGE_SIZE) == 0
					? instanceResult.getTotalCount().intValue() / PAGE_SIZE : (instanceResult.getTotalCount().intValue() / PAGE_SIZE + 1);
			if (totalPage > pageNumber) {
				for (int i = pageNumber + 1; i <= totalPage; i++) {
					request.setPageNumber(i);
					request.setPageSize(PAGE_SIZE);
					instanceResponse = this.getVmClient().describeInstances(request);
					instances.addAll(instanceResponse.getResult().getInstances());
				}
			}

			List<ServerModel> convertList = instanceConvertor.convertList(instances);
			for (ServerModel serverModel : convertList) {
				serverModel.setRegionId(butler.getRegionId());
			}
			return new BocloudResult(true, convertList, "查询成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult list(String hostId) {
		return null;
	}

	/**
	 * 查询镜像详情 因为创建云主机时 不通镜像 的系统盘类型 不同
	 * 
	 * @param imageId
	 * @return
	 */
	private Image imageDetail(String imageId) {
		try {
			if (null == imageId) {
				return null;
			}
			DescribeImageRequest request = new DescribeImageRequest();
			request.setRegionId(this.getRegion());
			request.setImageId(imageId);
			DescribeImageResponse response = this.getVmClient().describeImage(request);
			if (null == response) {
				return null;
			}
			DescribeImageResult result = response.getResult();
			if (null == result) {
				return null;
			}
			Image image = result.getImage();
			return image;
		} catch (Exception e) {
			return null;
		}
	}

	@Override
	public BocloudResult create(ServerModel server) {
		try {
			String az = server.getZone();// "cn-north-1a"; // 默认第一个去 后续优化
			CreateInstancesRequest createInstancesRequest = new CreateInstancesRequest();
			createInstancesRequest.setRegionId(this.getRegion());
			createInstancesRequest.setMaxCount(1);
			// 设置基本信息
			InstanceSpec instanceSpec = new InstanceSpec();
			instanceSpec.setAz(server.getZone());
			instanceSpec.setInstanceType(server.getInstanceType());
			instanceSpec.setImageId(server.getImageUuid());
			instanceSpec.setName(server.getName());
			// 配置网络
			InstanceNetworkInterfaceAttachmentSpec primaryNetworkInterface = new InstanceNetworkInterfaceAttachmentSpec();
			NetworkInterfaceSpec networkInterface = new NetworkInterfaceSpec();
			networkInterface.setAz(az);
			networkInterface.setSubnetId(server.getSubnetUuid());
			primaryNetworkInterface.setNetworkInterface(networkInterface);
			instanceSpec.setPrimaryNetworkInterface(primaryNetworkInterface);
			// 系统盘默认window40G,linux30G,无法改变
			InstanceDiskAttachmentSpec systemDisk = new InstanceDiskAttachmentSpec();
			// 是否自动删除
			systemDisk.setAutoDelete(true);
			Image image = imageDetail(server.getImageUuid());
			if (null == image) {
				return new BocloudResult(false, "云主机创建失败：镜像不存在！");
			}
			String diskCategory = "cloud";
			if ("localDisk".equalsIgnoreCase(image.getRootDeviceType())) {
				diskCategory = "local";
			}
			systemDisk.setDiskCategory(diskCategory);
			instanceSpec.setSystemDisk(systemDisk);
			// 设置数据盘
			List<VolumeModel> dataDiskModels = server.getDiskDevices();
			if (!ListTool.isEmpty(dataDiskModels)) {
				List<InstanceDiskAttachmentSpec> dataDisks = Lists.newArrayList();
				for (VolumeModel dataDiskModel : dataDiskModels) {
					InstanceDiskAttachmentSpec dataDisk = new InstanceDiskAttachmentSpec();
					DiskSpec cloudDiskSpec = new DiskSpec();
					cloudDiskSpec.setAz(az);
					cloudDiskSpec.setDescription(dataDiskModel.getName());
					cloudDiskSpec.setDiskSizeGB(dataDiskModel.getSize());
					cloudDiskSpec.setDiskType(dataDiskModel.getType());
					cloudDiskSpec.setName(dataDiskModel.getName());
					dataDisk.setCloudDiskSpec(cloudDiskSpec);
					dataDisk.setDiskCategory("cloud");
					dataDisks.add(dataDisk);
				}
				instanceSpec.setDataDisks(dataDisks);
			}
			createInstancesRequest.setInstanceSpec(instanceSpec);
			CreateInstancesResponse response = this.getVmClient().createInstances(createInstancesRequest);
			if (null == response) {
				return new BocloudResult(false, "创建云主机失败");
			}
			CreateInstancesResult result = response.getResult();
			if (null == result) {
				return new BocloudResult(false, response.getError().getMessage());
			}
			ServerModel serverModel = getInstance(result.getInstanceIds().get(0), server);
			serverModel.setRegionId(butler.getRegionId());
			return new BocloudResult(true, serverModel, "创建云主机成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "创建云主机失败");
		}
	}

	@Override
	public BocloudResult modify(ServerModel server) {
		try {
			ModifyInstanceAttributeRequest request = new ModifyInstanceAttributeRequest();
			request.setRegionId(butler.getRegionId());
			request.setInstanceId(server.getInstanceId());
			if (!StringUtils.isEmpty(server.getRemark())) {
				request.setDescription(server.getRemark());
			}
			if (!StringUtils.isEmpty(server.getName())) {
				request.setName(server.getName());
			}
			this.getVmClient().modifyInstanceAttribute(request);
			return new BocloudResult(true, server, "修改成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "修改失败");
		}
	}

	@Override
	public BocloudResult migrate(ServerModel server) {
		return null;
	}

	@Override
	public BocloudResult migrate(OSMigrateModel model) {
		return null;
	}

	@Override
	public BocloudResult remove(String instanceId, String name) {
		try {
			DeleteInstanceRequest request = new DeleteInstanceRequest();
			request.setRegionId(butler.getRegionId());
			request.setInstanceId(instanceId);
			DeleteInstanceResponse response = this.getVmClient().deleteInstance(request);
			if (null == response) {
				return new BocloudResult(false, "删除云主机失败");
			}
			return new BocloudResult(true, "删除云主机成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "删除云主机失败");
		}
	}

	@Override
	public BocloudResult detail(String instanceId) {
		try {
			DescribeInstanceRequest request = new DescribeInstanceRequest();
			request.regionId(butler.getRegionId());
			request.setInstanceId(instanceId);
			DescribeInstanceResponse response = this.getVmClient().describeInstance(request);
			DescribeInstanceResult result = response.getResult();
			if (null == result) {
				return new BocloudResult(false, "查询云主机详情失败");
			}
			Instance instance = result.getInstance();
			if (null == instance) {
				return new BocloudResult(false, "云主机不存在");
			}
			BeanConvertor<ServerModel, Instance> instanceConvertor = new InstanceConvertor();
			return new BocloudResult(true, instanceConvertor.convertModel(instance), "查询云主机详情成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "查询云主机详情失败");
		}
	}

	@Override
	public BocloudResult start(String instanceId) {
		try {
			StartInstanceRequest startInstanceRequest = new StartInstanceRequest();
			startInstanceRequest.setInstanceId(instanceId);
			startInstanceRequest.setRegionId(butler.getRegionId());
			StartInstanceResponse response = this.getVmClient().startInstance(startInstanceRequest);
			if (null == response) {
				return new BocloudResult(false, "开启云主机失败");
			}
			return new BocloudResult(true, "开启云主机成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "开启云主机失败");
		}
	}

	@Override
	public BocloudResult stop(String instanceId) {
		try {
			StopInstanceRequest stopInstanceRequest = new StopInstanceRequest();
			stopInstanceRequest.setInstanceId(instanceId);
			stopInstanceRequest.setRegionId(butler.getRegionId());
			StopInstanceResponse response = this.getVmClient().stopInstance(stopInstanceRequest);
			if (null == response) {
				return new BocloudResult(false, "停止云主机失败");
			}
			return new BocloudResult(true, "停止云主机成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "停止云主机失败");
		}
	}

	@Override
	public BocloudResult reboot(String instanceId) {
		try {
			RebootInstanceRequest rebootInstanceRequest = new RebootInstanceRequest();
			rebootInstanceRequest.setInstanceId(instanceId);
			rebootInstanceRequest.setRegionId(butler.getRegionId());
			RebootInstanceResponse response = this.getVmClient().rebootInstance(rebootInstanceRequest);
			if (null == response) {
				return new BocloudResult(false, "重启云主机失败");
			}
			return new BocloudResult(true, "重启云主机成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "重启云主机失败");
		}
	}

	@Override
	public BocloudResult softReboot(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult pause(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult unPause(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult suspend(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult resume(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult joinSGroup(String instanceId, String group) {
		return null;
	}

	@Override
	public BocloudResult leaveSGroup(String instanceId, String group) {
		return null;
	}

	@Override
	public BocloudResult snapshot(SnapshotModel snapshotModel) {
		return null;
	}

	@Override
	public BocloudResult vncConsole(String vncType, String serverId) {
		return null;
	}

	@Override
	public BocloudResult changePassword(String instanceId, String password) {
		return null;
	}

	@Override
	public BocloudResult attachNetwork(ServerModel server) {
		return null;
	}

	@Override
	public BocloudResult detachNetwork(String instanceId, String portUuid) {
		return null;
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId) {
		return null;
	}

	@Override
	public BocloudResult confirmResize(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult revertResize(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean MemHotEnable) {
		return null;
	}

	@Override
	public BocloudResult getTicket(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult createInResPool(ServerModel server) {
		return null;
	}

	@Override
	public BocloudResult mountToolsInstaller(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult unmountToolsInstaller(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult upgradeToolsTask(String instanceId, String options) {
		return null;
	}

	@Override
	public BocloudResult getNetCard(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult getVmIps(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult markAsTemplate(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
		return null;
	}

	@Override
	public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
		return null;
	}

	@Override
	public BocloudResult getMetric(String type) {
		return null;
	}

	@Override
	public BocloudResult vmrcConsole(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult migrate(H3cMigrateModel model) {
		return null;
	}

	@Override
	public BocloudResult create(H3cServerModel server) {
		return null;
	}

	@Override
	public BocloudResult getJnlpXml(Long vmId, String address) {
		return null;
	}

	/**
	 * 获取新创建的实例
	 *
	 * @param instanceId
	 * @return
	 */
	private ServerModel getInstance(String instanceId, ServerModel server) {
		ServerModel serverModel = null;
		DescribeInstanceRequest request = new DescribeInstanceRequest();
		request.regionId(this.getRegion());
		request.setInstanceId(instanceId);
		DescribeInstanceResponse response = this.getVmClient().describeInstance(request);
		DescribeInstanceResult result = response.getResult();
		if (null == result) {
			return new ServerModel();
		}
		Instance instance = result.getInstance();
		if (null == instance) {
			return new ServerModel();
		}
		BeanConvertor<ServerModel, Instance> instanceConvertor = new InstanceConvertor();
		Boolean flag = true;
		Long startTime = System.currentTimeMillis();
		while (flag) {
			try {
				Thread.sleep(5000);
			} catch (InterruptedException e) {
				logger.error("Get error message:", e);
				Thread.currentThread().interrupt();
			}
			response = this.getVmClient().describeInstance(request);
			result = response.getResult();
			if (null == result) {
				serverModel = new ServerModel();
				flag = false;
				break;
			}
			instance = result.getInstance();
			if (null == instance) {
				serverModel = new ServerModel();
				flag = false;
				break;
			}
			if ("stopped".equalsIgnoreCase(instance.getStatus()) || "running".equalsIgnoreCase(instance.getStatus())) {
				List<VolumeModel> diskDevices = server.getDiskDevices();
				if (null != diskDevices && diskDevices.size() > 0) {
					int size = diskDevices.size();
					if (null != instance.getDataDisks() && size == instance.getDataDisks().size()) {
						serverModel = instanceConvertor.convertModel(instance);
						flag = false;
						break;
					} else {
						continue;
					}
				}
				serverModel = instanceConvertor.convertModel(instance);
				flag = false;
				break;
			}
			Long endTime = System.currentTimeMillis();
			// 等待5分钟
			if (endTime - startTime > 5 * 60 * 1000) {
				flag = false;
				serverModel = instanceConvertor.convertModel(instance);
				break;
			}
		}
		return serverModel;
	}

	@Override
	public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult resetInstance(ServerModel serverModel) {
		try {

			return new BocloudResult(true, "停止云主机成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "停止云主机失败");
		}
	}

	@Override
	public BocloudResult bindKeypair(ServerModel serverModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult unbindKeypair(ServerModel serverModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
		return null;
	}

	@Override
	public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
		// TODO Auto-generated method stub
		return null;
	}

}