package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.List;

import org.apache.http.util.Asserts;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ElasticIpModel;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.JdCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.jdconvertor.ElasticIpConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.jdcloud.sdk.service.vm.model.AssociateElasticIpRequest;
import com.jdcloud.sdk.service.vm.model.DisassociateElasticIpRequest;
import com.jdcloud.sdk.service.vpc.model.*;

/**
 * @author : guopeng create at: 2018/11/15 4:18 PM
 * @description:
 */
@Provider("bocloud.cmp.jdcloud")

public class JdCloudElasticIpProvider extends JdCloudProvider implements ElasticIpProvider {

    private static Logger logger = LoggerFactory.getLogger(JdCloudElasticIpProvider.class);

    private static final int PAGE_SIZE = 100;

    private BeanConvertor<ElasticIpModel, ElasticIp> convertor = new ElasticIpConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler
     *            云平台认证信息
     */
    public JdCloudElasticIpProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeElasticIpsRequest request = new DescribeElasticIpsRequest();
            request.setRegionId(this.getRegionId());
            request.setPageNumber(1);
            request.setPageSize(PAGE_SIZE);
            DescribeElasticIpsResponse response = this.getVpcClient().describeElasticIps(request);
            if (response == null || response.getResult() == null) {
                return new BocloudResult(false, "查询弹性公网IP列表失败, 接口返回response或response.getResult()为空");
            }
            Integer totalCount = response.getResult().getTotalCount().intValue();
            if (totalCount == 0) {
                logger.info("查询弹性公网IP列表: totalCount为0");
                return new BocloudResult(true, new ArrayList<VpcModel>(), "empty");
            }
            DescribeElasticIpsResult result = response.getResult();
            List<ElasticIp> totalVpcs = new ArrayList<>(result.getElasticIps());
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1 || totalCount == PAGE_SIZE) {
                return new BocloudResult(true, convertor.convertList(totalVpcs), "弹性公网IP列表查询成功");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getVpcClient().describeElasticIps(request);
                totalVpcs.addAll(response.getResult().getElasticIps());
            }
            return new BocloudResult(true, convertor.convertList(totalVpcs), "弹性公网IP列表查询成功");
        } catch (Exception e) {
            logger.error("查询弹性公网IP列表失败 ： " + e);
            return new BocloudResult(false, "弹性公网IP列表查询失败", e.getMessage());
        }
    }

    // 创建
    @Override
    public BocloudResult allocate(ElasticIpModel model) {
        try {
            Asserts.notNull(model.getBandwidth(), "Params bandwidth can't be null");
            CreateElasticIpsRequest request = new CreateElasticIpsRequest();
            request.setRegionId(this.getRegionId());
            request.setMaxCount(1);
            ElasticIpSpec elasticIpSpec = new ElasticIpSpec();
            elasticIpSpec.setBandwidthMbps(Integer.valueOf(model.getBandwidth()));
            elasticIpSpec.setProvider("bgp");
            request.setElasticIpSpec(elasticIpSpec);
            CreateElasticIpsResponse response = getVpcClient().createElasticIps(request);
            if (response == null || response.getResult() == null) {
                return new BocloudResult(false, "创建弹性公网IP失败, 接口返回response或response.getResult()为空");
            }
            if (response.getResult().getElasticIpIds() == null || response.getResult().getElasticIpIds().size() == 0) {
                return new BocloudResult(false, "创建弹性公网IP失败, response.getResult().getElasticIpIds()为空");
            }
            BocloudResult detail = detail(response.getResult().getElasticIpIds().get(0));
            if (detail.isFailed()) {
                return new BocloudResult(false, detail.getMessage());
            }
            return new BocloudResult(true, detail.getData(), "创建vpc成功");
        } catch (Exception e) {
            logger.error("创建vpc失败 ： " + e);
            return new BocloudResult(false, "创建vpc失败", e.getMessage());
        }
    }

    // 删除
    @Override
    public BocloudResult release(String allocationId) {
        try {
            DeleteElasticIpRequest request = new DeleteElasticIpRequest();
            request.setElasticIpId(allocationId);
            request.setRegionId(this.getRegionId());
            DeleteElasticIpResponse response = this.getVpcClient().deleteElasticIp(request);
            if (null == response) {
                return new BocloudResult(false, "弹性公网IP删除失败");
            }
            return new BocloudResult(true, "删除弹性公网IP成功");
        } catch (Exception e) {
            logger.error("删除弹性公网IP失败 ： " + e);
            return new BocloudResult(false, "删除弹性公网IP失败", e.getMessage());
        }
    }

    // 关联实例
    @Override
    public BocloudResult associate(ElasticIpModel model) {
        try {
            Asserts.notNull(model.getInstanceId(), "Params InstanceId can't be null");
            Asserts.notNull(model.getElasticIpUuid(), "Params ElasticIpUuid can't be null");
            AssociateElasticIpRequest request = new AssociateElasticIpRequest();
            request.setRegionId(this.getRegionId());
            request.setElasticIpId(model.getElasticIpUuid());
            request.setInstanceId(model.getInstanceId());
            getVmClient().associateElasticIp(request);
            return new BocloudResult(true, "云主机绑定弹性公网IP成功");
        } catch (Exception e) {
            logger.error("云主机绑定弹性公网IP失败 ： " + e);
            return new BocloudResult(false, "云主机绑定弹性公网IP失败", e.getMessage());
        }
    }

    // 解除实例关联
    @Override
    public BocloudResult disassociate(ElasticIpModel model) {
        try {
            Asserts.notNull(model.getInstanceId(), "Params InstanceId can't be null");
            Asserts.notNull(model.getElasticIpUuid(), "Params ElasticIpUuid can't be null");
            DisassociateElasticIpRequest request = new DisassociateElasticIpRequest();
            request.setRegionId(this.getRegionId());
            request.setElasticIpId(model.getElasticIpUuid());
            request.setInstanceId(model.getInstanceId());
            getVmClient().disassociateElasticIp(request);
            return new BocloudResult(true, "云主机解绑弹性公网IP成功");
        } catch (Exception e) {
            logger.error("云主机解绑弹性公网IP失败 ： " + e);
            return new BocloudResult(false, "云主机解绑弹性公网IP失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String allocationId) {
        try {
            DescribeElasticIpRequest request = new DescribeElasticIpRequest();
            request.setRegionId(this.getRegionId());
            request.setElasticIpId(allocationId);
            DescribeElasticIpResponse response = getVpcClient().describeElasticIp(request);
            if (response == null || response.getResult() == null || response.getResult().getElasticIp() == null) {
                return new BocloudResult(false, "查询弹性公网IP详情失败, 接口返回response或response.getResult()为空");
            }
            ElasticIpModel elasticIpModel = convertor.convertModel(response.getResult().getElasticIp());
            elasticIpModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, elasticIpModel, "查询详情成功");
        } catch (Exception e) {
            logger.error("查询详情失败 ： " + e);
            return new BocloudResult(false, "查询详情失败", e.getMessage());
        }
    }

    // 修改带宽
    @Override
    public BocloudResult updateBandwidth(ElasticIpModel model) {
        try {
            Asserts.notNull(model.getBandwidth(), "Params bandwidth can't be null");
            ModifyElasticIpRequest request = new ModifyElasticIpRequest();
            request.setRegionId(this.getRegionId());
            request.setBandwidthMbps(Integer.valueOf(model.getBandwidth()));
            getVpcClient().modifyElasticIp(request);
            BocloudResult detail = detail(model.getElasticIpUuid());
            if (detail.isFailed()) {
                return new BocloudResult(false, detail.getMessage());
            }
            return new BocloudResult(true, detail.getData(), "修改弹性公网IP成功");
        } catch (Exception e) {
            logger.error("修改弹性公网IP失败 ： " + e);
            return new BocloudResult(false, "修改弹性公网IP失败", e.getMessage());
        }
    }
}
