package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.http.util.Asserts;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.RouteTableModel;
import com.bocloud.cmp.provider.JdCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.jdconvertor.RouteTableConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.jdcloud.sdk.service.vpc.model.*;

/**
 * @author : guopeng create at: 2018/11/15 4:22 PM
 * @description:
 */
@Provider("bocloud.cmp.jdcloud")
public class JdCloudRouterTableProvider extends JdCloudProvider implements RouteTableProvider {

    private static Logger logger = LoggerFactory.getLogger(JdCloudRouterTableProvider.class);

    private static final int PAGE_SIZE = 100;

    private BeanConvertor<RouteTableModel, RouteTable> convertor = new RouteTableConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler
     *            云平台认证信息
     */
    public JdCloudRouterTableProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeRouteTablesRequest request = new DescribeRouteTablesRequest();
            request.setRegionId(this.getRegionId());
            request.setPageNumber(1);
            request.setPageSize(PAGE_SIZE);
            DescribeRouteTablesResponse response = this.getVpcClient().describeRouteTables(request);
            if (response == null || response.getResult() == null) {
                return new BocloudResult(false, "查询路由表列表失败, 接口返回response或response.getResult()为空");
            }
            Integer totalCount = response.getResult().getTotalCount().intValue();
            if (totalCount == 0) {
                logger.info("查询路由表列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<RouteTableModel>(), "empty");
            }
            DescribeRouteTablesResult result = response.getResult();
            List<RouteTable> totalVpcs = new ArrayList<>(result.getRouteTables());
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1 || totalCount == PAGE_SIZE) {
                return new BocloudResult(true, convertor.convertList(totalVpcs), "路由表列表查询成功");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getVpcClient().describeRouteTables(request);
                totalVpcs.addAll(response.getResult().getRouteTables());
            }
            return new BocloudResult(true, convertor.convertList(totalVpcs), "路由表列表查询成功");
        } catch (Exception e) {
            logger.error("查询列表失败 ： " + e);
            return new BocloudResult(false, "路由表列表查询失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult create(RouteTableModel model) {
        try {
            Asserts.notNull(model.getName(), "Params name can't be null");
            Asserts.notNull(model.getVpcUuid(), "Params vpcUuid can't be null");
            CreateRouteTableRequest request = new CreateRouteTableRequest();
            request.setRegionId(this.getRegionId());
            request.setVpcId(model.getVpcUuid());
            request.setRouteTableName(model.getName());
            request.setDescription(model.getRemark());
            CreateRouteTableResponse response = getVpcClient().createRouteTable(request);
            if (response == null || response.getResult() == null) {
                return new BocloudResult(false, "创建路由表失败, 接口返回response或response.getResult()为空");
            }
            if (response.getResult().getRouteTableId() == null) {
                return new BocloudResult(false, "创建路由表失败, response.getResult().getRouteTableId()为空");
            }
            BocloudResult detail = detail(response.getResult().getRouteTableId());
            if (detail.isFailed()) {
                return new BocloudResult(false, detail.getMessage());
            }
            return new BocloudResult(true, detail.getData(), "创建路由表成功");
        } catch (Exception e) {
            logger.error("创建路由表失败 ： " + e);
            return new BocloudResult(false, "创建路由表失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult delete(String routeTableId) {
        try {
            DeleteRouteTableRequest request = new DeleteRouteTableRequest();
            request.setRouteTableId(routeTableId);
            request.setRegionId(this.getRegionId());
            DeleteRouteTableResponse response = this.getVpcClient().deleteRouteTable(request);
            if (null == response) {
                return new BocloudResult(false, "路由表删除失败");
            }
            return new BocloudResult(true, "删除路由表成功");
        } catch (Exception e) {
            logger.error("删除路由表失败 ： " + e);
            return new BocloudResult(false, "删除路由表失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(RouteTableModel model) {
        try {
            Asserts.notNull(model.getName(), "Params name can't be null");
            Asserts.notNull(model.getUuid(), "Params uuid can't be null");
            ModifyRouteTableRequest request = new ModifyRouteTableRequest();
            request.setRegionId(this.getRegionId());
            request.setRouteTableId(model.getUuid());
            request.setRouteTableName(model.getName());
            request.setDescription(model.getRemark());
            getVpcClient().modifyRouteTable(request);
            BocloudResult detail = detail(model.getUuid());
            if (detail.isFailed()) {
                return new BocloudResult(false, detail.getMessage());
            }
            return new BocloudResult(true, detail.getData(), "修改路由表成功");
        } catch (Exception e) {
            logger.error("修改路由表失败 ： " + e);
            return new BocloudResult(false, "修改路由表失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String routeTableId) {
        try {
            DescribeRouteTableRequest request = new DescribeRouteTableRequest();
            request.setRegionId(this.getRegionId());
            request.setRouteTableId(routeTableId);
            DescribeRouteTableResponse response = getVpcClient().describeRouteTable(request);
            if (response == null || response.getResult() == null || response.getResult().getRouteTable() == null) {
                return new BocloudResult(false, "查询路由表详情失败, 接口返回response或response.getResult()为空");
            }
            RouteTableModel routeTableModel = convertor.convertModel(response.getResult().getRouteTable());
            routeTableModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, routeTableModel, "查询详情成功");
        } catch (Exception e) {
            logger.error("查询详情失败 ： " + e);
            return new BocloudResult(false, "查询详情失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult associateSubnet(List<String> subnetUuids, String routeTableId) {
        try {
            AssociateRouteTableRequest request = new AssociateRouteTableRequest();
            request.setRegionId(this.getRegionId());
            request.setRouteTableId(routeTableId);
            request.setSubnetIds(subnetUuids);
            getVpcClient().associateRouteTable(request);
            return new BocloudResult(true, "路由表关联子网成功");
        } catch (Exception e) {
            logger.error("路由表关联子网失败 ： " + e);
            return new BocloudResult(false, "路由表关联子网失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult unAssociateSubnet(String subnetUuid, String routeTableId) {
        try {
            DisassociateRouteTableRequest request = new DisassociateRouteTableRequest();
            request.setRegionId(this.getRegionId());
            request.setRouteTableId(routeTableId);
            request.setSubnetId(subnetUuid);
            getVpcClient().disassociateRouteTable(request);
            return new BocloudResult(true, "路由表解除子网关联成功");
        } catch (Exception e) {
            logger.error("路由表解除子网关联失败 ： " + e);
            return new BocloudResult(false, "路由表解除子网关联失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult getByVpc(String vpcUuid) {
        try {
            BocloudResult list = list();
            if (list.isFailed()) {
                return new BocloudResult(false, list.getMessage());
            }
            List<RouteTableModel> models = (List<RouteTableModel>)list.getData();
            List<RouteTableModel> finalSgs = models.stream().filter(sg -> sg.getVpcUuid() != null)
                .filter(routeTableModel -> routeTableModel.getVpcUuid().equals(vpcUuid)).collect(Collectors.toList());
            return new BocloudResult(true, finalSgs, "根据vpc查询路由表成功");
        } catch (Exception e) {
            logger.error("list route table by vpc error:", e);
            return new BocloudResult(false, "根据vpc查询路由表成功失败");
        }
    }
}
