package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.provider.JdCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.jdconvertor.SecurityGroupConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.jdcloud.sdk.JdcloudSdkException;
import com.jdcloud.sdk.service.vpc.model.*;
import com.jdcloud.sdk.utils.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author : guopeng create at: 2018/11/15 4:23 PM
 * @description:
 */
@Provider("bocloud.cmp.jdcloud")
public class JdCloudSecurityProvider extends JdCloudProvider implements SecurityGroupProvider {

    private static Logger logger = LoggerFactory.getLogger(JdCloudSecurityProvider.class);
    private BeanConvertor<SecurityGroupModel, NetworkSecurityGroup> securityGroupConvertor =
            new SecurityGroupConvertor();
    private static final int PAGE_SIZE = 100;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public JdCloudSecurityProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            int pageNumber = 1;
            DescribeNetworkSecurityGroupsRequest request = new DescribeNetworkSecurityGroupsRequest();
            request.setRegionId(this.getRegion());
            request.setPageSize(PAGE_SIZE);
            request.setPageNumber(pageNumber);
            DescribeNetworkSecurityGroupsResponse response = this.getVpcClient().describeNetworkSecurityGroups(request);
            DescribeNetworkSecurityGroupsResult result = response.getResult();
            if (null == result) {
                return new BocloudResult(false, "查询NetworkSecurityGroup列表失败，数据为空");
            }
            Integer totalCount = result.getTotalCount().intValue();
            if (totalCount == 0) {
                logger.info("查询安全组列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<SecurityGroupModel>(), "empty");
            }
            List<NetworkSecurityGroup> networkModels = result.getNetworkSecurityGroups();
            int totalPage = (result.getTotalCount().intValue() % PAGE_SIZE) == 0
                    ? result.getTotalCount().intValue() / PAGE_SIZE : (result.getTotalCount().intValue() / PAGE_SIZE + 1);
            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    request.setPageNumber(i);
                    request.setPageSize(PAGE_SIZE);
                    response = this.getVpcClient().describeNetworkSecurityGroups(request);
                    networkModels.addAll(response.getResult().getNetworkSecurityGroups());
                }
            }
            List<SecurityGroupModel> convertList = securityGroupConvertor.convertList(networkModels);
            for (SecurityGroupModel securityGroupModel : convertList) {
                securityGroupModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "NetworkSecurityGroup列表查询成功");

        } catch (JdcloudSdkException e) {
            logger.error("list SecurityGroup error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(SecurityGroupModel group) {
        try {
            CreateNetworkSecurityGroupRequest request = new CreateNetworkSecurityGroupRequest();
            request.setRegionId(this.getRegion());
            request.setDescription(StringUtils.isNotBlank(group.getRemark()) ? group.getRemark() : null);
            request.setNetworkSecurityGroupName(group.getName());
            request.setVpcId(group.getVpcUuid());
            CreateNetworkSecurityGroupResponse response = this.getVpcClient().createNetworkSecurityGroup(request);
            CreateNetworkSecurityGroupResult result = response.getResult();
            if (null == result) {
                return new BocloudResult(false, "创建失败");
            }
            SecurityGroupModel securityGroupModel =
                    securityGroupConvertor.convertModel(getById(result.getNetworkSecurityGroupId()));
            securityGroupModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, securityGroupModel, "创建成功");
        } catch (Exception e) {
            logger.error("创建失败： " + e);
            return new BocloudResult(false, "创建失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String sgId) {
        try {
            DeleteNetworkSecurityGroupRequest request = new DeleteNetworkSecurityGroupRequest();
            request.setRegionId(this.getRegion());
            request.setNetworkSecurityGroupId(sgId);
            DeleteNetworkSecurityGroupResponse response = this.getVpcClient().deleteNetworkSecurityGroup(request);

            if (null == response.getRequestId()) {
                return new BocloudResult(false, "删除失败");
            }
            return new BocloudResult(true, "安全组删除成功");
        } catch (Exception e) {
            logger.error("删除失败： " + e.getMessage());
            return new BocloudResult(false, "删除失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String sgId) {
        return null;
    }

    @Override
    public BocloudResult modify(SecurityGroupModel model) {
        return null;
    }

    private NetworkSecurityGroup getById(String networkSecurityGroupId) {
        DescribeNetworkSecurityGroupRequest request = new DescribeNetworkSecurityGroupRequest();
        request.setRegionId(this.getRegion());
        request.setNetworkSecurityGroupId(networkSecurityGroupId);
        DescribeNetworkSecurityGroupResponse describeNetworkSecurityGroupResponse =
                this.getVpcClient().describeNetworkSecurityGroup(request);
        DescribeNetworkSecurityGroupResult result = describeNetworkSecurityGroupResponse.getResult();
        if (null == result) {
            return null;
        }
        return result.getNetworkSecurityGroup();
    }

    @Override
    public BocloudResult getByVpc(String vpcId) {
        try {
            BocloudResult list = list();
            if (list.isFailed()) {
                return new BocloudResult(false, list.getMessage());
            }
            List<SecurityGroupModel> sgs = (List<SecurityGroupModel>) list.getData();
            List<SecurityGroupModel> finalSgs = sgs.stream().filter(sg -> sg.getVpcUuid() != null)
                    .filter(securityGroupModel -> securityGroupModel.getVpcUuid().equals(vpcId))
                    .collect(Collectors.toList());
            return new BocloudResult(true, finalSgs, "根据vpc查询安全组成功");
        } catch (Exception e) {
            logger.error("list SecurityGroup by vpc error:", e);
            return new BocloudResult(false, "根据vpc查询安全组成功失败");
        }
    }
}
