package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.List;

import org.apache.http.util.Asserts;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.JdCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.jdconvertor.SubnetConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.jdcloud.sdk.service.vpc.model.*;

/**
 * @author : guopeng create at: 2018/11/15 4:32 PM
 * @description:
 */
@Provider("bocloud.cmp.jdcloud")
public class JdCloudSubnetProvider extends JdCloudProvider implements SubnetProvider {

    private static Logger logger = LoggerFactory.getLogger(JdCloudSubnetProvider.class);
    private static final int PAGE_SIZE = 100;

    private BeanConvertor<SubnetModel, Subnet> convertor = new SubnetConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler
     *            云平台认证信息
     */
    public JdCloudSubnetProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeSubnetsRequest request = new DescribeSubnetsRequest();
            request.setRegionId(this.getRegionId());
            request.setPageNumber(1);
            request.setPageSize(PAGE_SIZE);
            DescribeSubnetsResponse response = this.getVpcClient().describeSubnets(request);
            if (response == null || response.getResult() == null) {
                return new BocloudResult(false, "查询子网列表失败, 接口返回response或response.getResult()为空");
            }
            Integer totalCount = response.getResult().getTotalCount().intValue();
            if (totalCount == 0) {
                logger.info("查询子网列表: totalCount为0");
                return new BocloudResult(true, new ArrayList<SubnetModel>(), "empty");
            }
            DescribeSubnetsResult result = response.getResult();
            List<Subnet> totalVpcs = new ArrayList<>(result.getSubnets());
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1 || totalCount == PAGE_SIZE) {
                return new BocloudResult(true, convertor.convertList(totalVpcs), "子网列表查询成功");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getVpcClient().describeSubnets(request);
                totalVpcs.addAll(response.getResult().getSubnets());
            }
            return new BocloudResult(true, convertor.convertList(totalVpcs), "子网列表查询成功");
        } catch (Exception e) {
            logger.error("查询列表失败 ： " + e);
            return new BocloudResult(false, "子网列表查询失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult create(SubnetModel model) {
        try {
            Asserts.notNull(model.getName(), "Params name can't be null");
            Asserts.notNull(model.getCidr(), "Params cidr can't be null");
            Asserts.notNull(model.getVpcUuid(), "Params vpcUuid can't be null");
            CreateSubnetRequest request = new CreateSubnetRequest();
            request.setRegionId(this.getRegionId());
            request.setSubnetName(model.getName());
            request.setVpcId(model.getVpcUuid());
            request.setAddressPrefix(model.getCidr());
            request.setDescription(model.getRemark());
            request.setRouteTableId(model.getRouteTableId());
            CreateSubnetResponse response = getVpcClient().createSubnet(request);
            if (response == null || response.getResult() == null) {
                return new BocloudResult(false, "创建子网失败, 接口返回response或response.getResult()为空");
            }
            if (response.getResult().getSubnetId() == null) {
                return new BocloudResult(false, "创建子网失败, response.getResult().getSubnetId()为空");
            }
            BocloudResult detail = detail(response.getResult().getSubnetId());
            if (detail.isFailed()) {
                return new BocloudResult(false, detail.getMessage());
            }
            return new BocloudResult(true, detail.getData(), "创建子网成功");
        } catch (Exception e) {
            logger.error("创建子网失败 ： " + e);
            return new BocloudResult(false, "创建子网失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(SubnetModel model) {
        try {
            Asserts.notNull(model.getName(), "Params name can't be null");
            Asserts.notNull(model.getSubnetUuid(), "Params subnetUuid can't be null");
            ModifySubnetRequest request = new ModifySubnetRequest();
            request.setRegionId(this.getRegionId());
            request.setSubnetId(model.getSubnetUuid());
            request.setSubnetName(model.getName());
            request.setDescription(model.getRemark());
            getVpcClient().modifySubnet(request);
            BocloudResult detail = detail(model.getSubnetUuid());
            if (detail.isFailed()) {
                return new BocloudResult(false, detail.getMessage());
            }
            return new BocloudResult(true, detail.getData(), "修改子网成功");
        } catch (Exception e) {
            logger.error("修改子网失败 ： " + e);
            return new BocloudResult(false, "修改子网失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String subnetId) {
        try {
            DeleteSubnetRequest request = new DeleteSubnetRequest();
            request.setSubnetId(subnetId);
            request.setRegionId(this.getRegionId());
            DeleteSubnetResponse response = this.getVpcClient().deleteSubnet(request);
            if (null == response) {
                return new BocloudResult(false, "子网删除失败");
            }
            return new BocloudResult(true, "删除子网成功");
        } catch (Exception e) {
            logger.error("删除子网失败 ： " + e);
            return new BocloudResult(false, "删除子网失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String subnetId) {
        try {
            DescribeSubnetRequest request = new DescribeSubnetRequest();
            request.setRegionId(this.getRegionId());
            request.setSubnetId(subnetId);
            DescribeSubnetResponse response = getVpcClient().describeSubnet(request);
            if (response == null || response.getResult() == null || response.getResult().getSubnet() == null) {
                return new BocloudResult(false, "查询子网详情失败, 接口返回response或response.getResult()为空");
            }
            SubnetModel subnetModel = convertor.convertModel(response.getResult().getSubnet());
            subnetModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, subnetModel, "查询详情成功");
        } catch (Exception e) {
            logger.error("查询详情失败 ： " + e);
            return new BocloudResult(false, "查询详情失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SubnetModel subnet) {
        // TODO Auto-generated method stub
        return null;
    }

}
