package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.List;

import org.apache.http.util.Asserts;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.JdCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.jdconvertor.VpcConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.jdcloud.sdk.service.vpc.model.*;

/**
 * 京东VPC
 * 
 * @author ymh
 * @时间 2019年2月15日
 */
@Provider("bocloud.cmp.jdcloud")
public class JdCloudVpcProvider extends JdCloudProvider implements VpcProvider {

    private static Logger logger = LoggerFactory.getLogger(JdCloudVpcProvider.class);

    private static final int PAGE_SIZE = 100;

    private BeanConvertor<VpcModel, Vpc> convertor = new VpcConvertor();

    public JdCloudVpcProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(VpcModel model) {
        try {
            Asserts.notNull(model.getName(), "Params name can't be null");
            Asserts.notNull(model.getCidr(), "Params cidr can't be null");
            CreateVpcRequest createVpcRequest = new CreateVpcRequest();
            createVpcRequest.setRegionId(this.getRegionId());
            createVpcRequest.setVpcName(model.getName());
            createVpcRequest.setAddressPrefix(model.getCidr());
            createVpcRequest.setDescription(model.getRemark());
            CreateVpcResponse response = getVpcClient().createVpc(createVpcRequest);
            if (response == null || response.getResult() == null) {
                return new BocloudResult(false, "创建vpc失败, 接口返回response或response.getResult()为空");
            }
            if (response.getResult().getVpcId() == null) {
                return new BocloudResult(false, "创建vpc失败, response.getResult().getVpcId()为空");
            }
            BocloudResult detail = detail(response.getResult().getVpcId(), this.getRegionId());
            if (detail.isFailed()) {
                return new BocloudResult(false, detail.getMessage());
            }
            return new BocloudResult(true, detail.getData(), "创建vpc成功");
        } catch (Exception e) {
            logger.error("创建vpc失败 ： " + e);
            return new BocloudResult(false, "创建vpc失败", e.getMessage());
        }
    }

    // public BocloudResult getVpcDetail(String vpcId, String regionId) {
    // BocloudResult result = null;
    // try {
    // Boolean flag = true;
    // Long startTime = System.currentTimeMillis();
    // while (flag) {
    // Thread.sleep(100);
    // VpcClient client = (VpcClient) this.getClient(TencentEnum.vpc);
    // client.setRegion(regionId);
    // DescribeVpcsRequest req = new DescribeVpcsRequest();
    // req.setVpcIds(new String[] { vpcId });
    // DescribeVpcsResponse resp = client.DescribeVpcs(req);
    // Vpc[] vpcSet = resp.getVpcSet();
    //
    // if (vpcSet == null || vpcSet.length <= 0) {
    // flag = false;
    // result = new BocloudResult(false, "失败");
    // break;
    // }
    // Vpc vpc = vpcSet[0];
    // if (null != vpc.getCreatedTime() &&
    // !vpc.getCreatedTime().equals("0000-00-00 00:00:00")) {
    // flag = false;
    // result = this.detail(vpcId, regionId);
    // break;
    // }
    // Long endTime = System.currentTimeMillis();
    // // 等待60s
    // if (endTime - startTime > 3 * 60 * 1000) {
    // flag = false;
    // result = this.detail(vpcId, regionId);
    // break;
    // }
    // }
    // return result;
    // } catch (Exception e) {
    // logger.error("查询状态失败： " + e);
    // return new BocloudResult(false, "查询状态失败", e.getMessage());
    // }
    // }

    @Override
    public BocloudResult remove(String vpcId) {
        try {
            DeleteVpcRequest deleteVpcRequest = new DeleteVpcRequest();
            deleteVpcRequest.setVpcId(vpcId);
            deleteVpcRequest.setRegionId(this.getRegionId());
            DeleteVpcResponse response = this.getVpcClient().deleteVpc(deleteVpcRequest);
            if (null == response) {
                return new BocloudResult(false, "vpc删除失败");
            }
            return new BocloudResult(true, "删除vpc成功");
        } catch (Exception e) {
            logger.error("删除vpc失败 ： " + e);
            return new BocloudResult(false, "删除vpc失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeVpcsRequest request = new DescribeVpcsRequest();
            request.setRegionId(this.getRegionId());
            request.setPageNumber(1);
            request.setPageSize(PAGE_SIZE);
            DescribeVpcsResponse response = this.getVpcClient().describeVpcs(request);
            if (response == null || response.getResult() == null) {
                return new BocloudResult(false, "查询vpc列表失败, 接口返回response或response.getResult()为空");
            }
            Integer totalCount = response.getResult().getTotalCount().intValue();
            if (totalCount == 0) {
                logger.info("查询vpc列表: totalCount为0");
                return new BocloudResult(true, new ArrayList<VpcModel>(), "empty");
            }
            DescribeVpcsResult result = response.getResult();
            List<Vpc> totalVpcs = new ArrayList<>(result.getVpcs());
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1 || totalCount == PAGE_SIZE) {
                return new BocloudResult(true, convertor.convertList(totalVpcs), "vpc列表查询成功");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getVpcClient().describeVpcs(request);
                totalVpcs.addAll(response.getResult().getVpcs());
            }
            return new BocloudResult(true, convertor.convertList(totalVpcs), "vpc列表查询成功");
        } catch (Exception e) {
            logger.error("查询列表失败 ： " + e);
            return new BocloudResult(false, "vpc列表查询失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String vpcId, String regionId) {
        try {
            DescribeVpcRequest request = new DescribeVpcRequest();
            request.setRegionId(regionId);
            request.setVpcId(vpcId);
            DescribeVpcResponse response = getVpcClient().describeVpc(request);
            if (response == null || response.getResult() == null || response.getResult().getVpc() == null) {
                return new BocloudResult(false, "查询vpc详情失败, 接口返回response或response.getResult()为空");
            }
            VpcModel vpcModel = convertor.convertModel(response.getResult().getVpc());
            vpcModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, vpcModel, "查询详情成功");
        } catch (Exception e) {
            logger.error("查询详情失败 ： " + e);
            return new BocloudResult(false, "查询详情失败", e.getMessage());
        }

    }

    @Override
    public BocloudResult update(VpcModel model) {
        try {
            Asserts.notNull(model.getName(), "Params name can't be null");
            Asserts.notNull(model.getVpcId(), "Params vpcId can't be null");
            ModifyVpcRequest request = new ModifyVpcRequest();
            request.setRegionId(this.getRegionId());
            request.setVpcId(model.getVpcId());
            request.setVpcName(model.getName());
            request.setDescription(model.getRemark());
            getVpcClient().modifyVpc(request);
            BocloudResult detail = detail(model.getVpcId(), this.getRegionId());
            if (detail.isFailed()) {
                return new BocloudResult(false, detail.getMessage());
            }
            return new BocloudResult(true, detail.getData(), "修改vpc成功");
        } catch (Exception e) {
            logger.error("修改vpc失败 ： " + e);
            return new BocloudResult(false, "修改vpc失败", e.getMessage());
        }
    }

}
