package com.bocloud.cmp.provider.security;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.JdCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.jdconvertor.KeypairConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.jdcloud.sdk.JdcloudSdkException;
import com.jdcloud.sdk.service.vm.model.*;
import com.jdcloud.sdk.utils.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName: JdCloudKeypairProvider
 * @Description: 秘钥
 * @Author lxy
 * @Date 2021/6/17 20:21
 */
@Provider("bocloud.cmp.jdcloud")
public class JdCloudKeypairProvider extends JdCloudProvider implements KeypairProvider {
    private static Logger logger = LoggerFactory.getLogger(JdCloudKeypairProvider.class);
    private static final int PAGE_SIZE = 100;
    private BeanConvertor<KeypairModel, Keypair> keypairConvertor =
            new KeypairConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public JdCloudKeypairProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            int pageNumber = 1;
            DescribeKeypairsRequest request = new DescribeKeypairsRequest();
            request.setPageSize(PAGE_SIZE);
            request.setPageNumber(pageNumber);
            request.setRegionId(this.getRegion());
            DescribeKeypairsResponse response = this.getVmClient().describeKeypairs(request);
            DescribeKeypairsResult result = response.getResult();
            if (null == result) {
                return new BocloudResult(false, "查询Keypair列表失败，数据为空");
            }
            Integer totalCount = result.getTotalCount().intValue();
            if (totalCount == 0) {
                logger.info("查询秘钥列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<KeypairModel>(), "empty");
            }
            List<Keypair> keyList = result.getKeypairs();
            int totalPage = (result.getTotalCount().intValue() % PAGE_SIZE) == 0
                    ? result.getTotalCount().intValue() / PAGE_SIZE : (result.getTotalCount().intValue() / PAGE_SIZE + 1);
            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    request.setPageNumber(i);
                    request.setPageSize(PAGE_SIZE);
                    response = this.getVmClient().describeKeypairs(request);
                    keyList.addAll(response.getResult().getKeypairs());
                }
            }
            List<KeypairModel> convertList = keypairConvertor.convertList(keyList);
            for (KeypairModel keypairModel : convertList) {
                keypairModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "keypair列表查询成功");
        } catch (JdcloudSdkException e) {
            logger.error("list keypair error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult detail(String keypairName) {
        return null;
    }

    @Override
    public BocloudResult create(String keypairName, String publicKey) {
        if (StringUtils.isBlank(publicKey)) {
            //创建
            try {
                CreateKeypairRequest request = new CreateKeypairRequest();
                request.setRegionId(this.getRegion());
                request.setKeyName(keypairName);
                CreateKeypairResponse response = this.getVmClient().createKeypair(request);
                CreateKeypairResult result = response.getResult();
                if (null == result) {
                    return new BocloudResult(false, "创建失败");
                }
                Keypair keypair = new Keypair();
                keypair.setKeyName(result.getKeyName());
                keypair.setKeyFingerprint(result.getKeyFingerprint());
                KeypairModel keypairModel = keypairConvertor.convertModelParams(keypair, result.getPrivateKey(), null);
                keypairModel.setRegionId(this.getRegionId());
                return new BocloudResult(true, keypairModel, "创建成功");
            } catch (Exception e) {
                logger.error("创建失败 ： " + e);
                return new BocloudResult(false, "创建失败", e.getMessage());
            }
        } else {
            //导入
            try {
                ImportKeypairRequest request = new ImportKeypairRequest();
                request.setRegionId(this.getRegion());
                request.setKeyName(keypairName);
                request.setPublicKey(publicKey);
                ImportKeypairResponse response = this.getVmClient().importKeypair(request);
                ImportKeypairResult result = response.getResult();
                if (null == result) {
                    return new BocloudResult(false, "导入失败");
                }
                Keypair keypair = new Keypair();
                keypair.setKeyName(result.getKeyName());
                keypair.setKeyFingerprint(result.getKeyFingerprint());
                KeypairModel keypairModel = keypairConvertor.convertModel(keypair);
                keypairModel.setPublicKey(publicKey);
                keypairModel.setRegionId(this.getRegionId());
                return new BocloudResult(true, keypairModel, "导入成功");
            } catch (Exception e) {
                logger.error("导入失败 ： " + e);
                return new BocloudResult(false, "导入失败", e.getMessage());
            }
        }
    }

    @Override
    public BocloudResult delete(String keypairName) {
        try {
            DeleteKeypairRequest request = new DeleteKeypairRequest();
            request.setRegionId(this.getRegion());
            request.setKeyName(keypairName);
            DeleteKeypairResponse response = this.getVmClient().deleteKeypair(request);
            if (response.getJdcloudHttpResponse().getStatusCode() != 200) {
                return new BocloudResult(false, "删除keypair失败，原因：" + response.getJdcloudHttpResponse().getStatusCode());
            }
            return new BocloudResult(true, "删除keypair成功");
        } catch (JdcloudSdkException e) {
            logger.error("删除keypair失败 ： " + e);
            return new BocloudResult(false, "删除keypair失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(KeypairModel model) {
        return null;
    }

}
