package com.bocloud.cmp.provider.storage;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.provider.JdCloudProvider;
import com.bocloud.cmp.provider.compute.JdCloudSnapshotProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.jdconvertor.JdDiskConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.jdcloud.sdk.service.disk.model.*;
import com.jdcloud.sdk.service.vm.model.*;

/**
 * 京东云
 * 
 * @author ymh
 * @时间 2019年1月23日
 */
@Provider("bocloud.cmp.jdcloud")
public class JdCloudVolumeProvider extends JdCloudProvider implements VolumeProvider {

    private static Logger logger = LoggerFactory.getLogger(JdCloudVolumeProvider.class);

    private BeanConvertor<VolumeModel, Disk> diskConvertor = new JdDiskConvertor();

    private static final int PAGE_SIZE = 100;

    private JdCloudSnapshotProvider jdCloudSnapshotProvider = null;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler
     *            云平台认证信息
     */
    public JdCloudVolumeProvider(Butler butler) {
        super(butler);
        jdCloudSnapshotProvider = new JdCloudSnapshotProvider(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeDisksRequest request = new DescribeDisksRequest();
            request.setRegionId(this.getRegionId());
            request.setPageNumber(1);
            request.setPageSize(PAGE_SIZE);
            DescribeDisksResponse response = this.getDiskClient().describeDisks(request);
            if (response == null || response.getResult() == null) {
                return new BocloudResult(false, "查询云硬盘列表失败, 接口返回response或response.getResult()为空");
            }
            int totalCount = response.getResult().getTotalCount() == null ? 0 : response.getResult().getTotalCount();
            if (totalCount == 0) {
                logger.info("查询云硬盘列表: totalCount为0");
                return new BocloudResult(true, new ArrayList<VolumeModel>(), "empty");
            }
            DescribeDisksResult result = response.getResult();
            List<Disk> totalVpcs = new ArrayList<>(result.getDisks());
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1 || totalCount == PAGE_SIZE) {
                return new BocloudResult(true, diskConvertor.convertList(totalVpcs), "云硬盘列表查询成功");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getDiskClient().describeDisks(request);
                totalVpcs.addAll(response.getResult().getDisks());
            }
            return new BocloudResult(true, diskConvertor.convertList(totalVpcs), "云硬盘列表查询成功");
        } catch (Exception e) {
            logger.error("查询云硬盘列表失败 ： " + e);
            return new BocloudResult(false, "云硬盘列表查询失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult create(VolumeModel volume) {
        try {
            CreateDisksRequest request = new CreateDisksRequest();
            request.setRegionId(this.getRegionId());
            DiskSpec diskSpec = new DiskSpec();
            diskSpec.setAz(volume.getZone());
            diskSpec.setDescription(volume.getRemark());
            diskSpec.setName(volume.getName());
            if (null != volume.getSourceSnapUuid() && !"".equals(volume.getSourceSnapUuid())) {
                diskSpec.setSnapshotId(volume.getSourceSnapUuid());
            } else {
                // 云硬盘大小，单位为 GiB；
                // ssd.io1 类型取值范围[20,16000]GB,步长为10G;
                // ssd.gp1 类型取值范围[20,16000]GB,步长为10G;
                // hdd.std1 类型取值范围[20,16000]GB,步长为10G
                diskSpec.setDiskSizeGB(volume.getSize());
            }
            // 云硬盘类型，取值为ssd.gp1(通用型ssd)、ssd.io1(性能型ssd)、hdd.std1(容量型hdd)
            // hdd.std1: 容量型hdd，范围[20,16000]GiB，步长为10GiB, iops为计算得出，与购买磁盘的容量成正比
            // ssd.gp1: 通用型ssd，范围[20,16000]GiB，步长为10GiB, iops为计算得出，与购买磁盘的容量成正比。
            // ssd.io1: 性能型ssd，范围[20,16000]GiB，步长为10GiB, iops可以通过购买容量计算得出，或者为用户指定。
            String diskType = "ssd.gp1"; // 云硬盘无类型时，默认为通用性
            if (null != volume.getType() && !"".equals(volume.getType())) {
                diskType = volume.getType();
            }
            // 云硬盘IOPS的大小，当且仅当云盘类型是ssd.io1型的云盘有效，步长是10,默认值为容量30，最大值为容量50
            if ("ssd.io1".equals(diskType)) {
                diskSpec.setIops(volume.getIops());
            }
            diskSpec.setDiskType(diskType);
            // 云硬盘是否加密，默认为false（不加密）
            diskSpec.setEncrypt(volume.getEncrypted());
            request.setDiskSpec(diskSpec);
            CreateDisksResponse response = this.getDiskClient().createDisks(request);
            if (response == null || response.getResult() == null) {
                return new BocloudResult(false, "创建云硬盘失败, 接口返回response或response.getResult()为空");
            }
            if (response.getResult().getDiskIds() == null || response.getResult().getDiskIds().size() == 0) {
                return new BocloudResult(false, "创建云硬盘失败, response.getResult().getDiskIds()为空");
            }
            CreateDisksResult result = response.getResult();
            List<String> diskIds = result.getDiskIds();
            return new BocloudResult(true, getAvailable(diskIds.get(0)), "创建云硬盘成功");
        } catch (Exception e) {
            logger.error("create volume error :", e);
            return new BocloudResult(false, "创建云硬盘失败");
        }
    }

    private VolumeModel getAvailable(String volumeId) {
        VolumeModel model = new VolumeModel();
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                try {
                    Thread.sleep(10000);
                } catch (InterruptedException e) {
                    logger.error("Get error message:", e);
                    Thread.currentThread().interrupt();
                }
                BocloudResult detail = detail(volumeId);
                if (detail.isFailed()) {
                    throw new Exception("查询云硬盘详情失败");
                }
                if (detail.getData() == null) {
                    flag = false;
                    break;
                }
                model = (VolumeModel)detail.getData();
                if ("available".equalsIgnoreCase(model.getStatus())) {
                    flag = false;
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 5 * 60 * 1000) {
                    flag = false;
                    break;
                }
            }
        } catch (Exception e) {
            logger.error("get disk error:", e);
        }
        return model;
    }

    @Override
    public BocloudResult modify(VolumeModel volume) {
        try {
            ModifyDiskAttributeRequest request = new ModifyDiskAttributeRequest();
            request.setRegionId(this.getRegionId());
            request.setDiskId(volume.getVolumeUuid());
            request.setDescription(volume.getRemark());
            request.setName(volume.getName());
            this.getDiskClient().modifyDiskAttribute(request);
            BocloudResult detail = detail(volume.getVolumeUuid());
            if (detail.isFailed()) {
                return new BocloudResult(false, detail.getMessage());
            }
            return new BocloudResult(true, detail.getData(), "修改成功");
        } catch (Exception e) {
            logger.error("modify volume error :", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    @Override
    public BocloudResult remove(String volumeId) {
        try {
            DeleteDiskRequest request = new DeleteDiskRequest();
            request.setRegionId(this.getRegionId());
            request.setDiskId(volumeId);
            DeleteDiskResponse deleteDisk = this.getDiskClient().deleteDisk(request);
            return new BocloudResult(true, deleteDisk.getResult(), "删除成功");
        } catch (Exception e) {
            logger.error("delete volume error :", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult detail(String volumeId) {
        try {
            DescribeDiskRequest request = new DescribeDiskRequest();
            request.setDiskId(volumeId);
            request.setRegionId(this.getRegionId());
            DescribeDiskResponse response = this.getDiskClient().describeDisk(request);
            if (response == null || response.getResult() == null || response.getResult().getDisk() == null) {
                return new BocloudResult(false, "查询云硬盘详情失败, 接口返回response或response.getResult()为空");
            }
            DescribeDiskResult result = response.getResult();
            Disk disk = result.getDisk();
            VolumeModel volumeModel = diskConvertor.convertModel(disk);
            volumeModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, volumeModel, "查询云硬盘详情成功");
        } catch (Exception e) {
            logger.error("qeury volume error :", e);
            return new BocloudResult(false, "查询云硬盘详情失败");
        }
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        try {
            CreateSnapshotRequest request = new CreateSnapshotRequest();
            request.setRegionId(this.getRegionId());
            SnapshotSpec snapshotSpec = new SnapshotSpec();
            snapshotSpec.setDiskId(snapshotModel.getVolumeUuid());
            snapshotSpec.setDescription(snapshotModel.getRemark());
            snapshotSpec.setName(snapshotModel.getName());
            request.setSnapshotSpec(snapshotSpec);
            CreateSnapshotResponse response = this.getDiskClient().createSnapshot(request);
            if (response == null || response.getResult() == null) {
                return new BocloudResult(false, "创建快照失败, 接口返回response或response.getResult()为空");
            }
            if (response.getResult().getSnapshotId() == null) {
                return new BocloudResult(false, "创建快照失败, response.getResult().getSnapshotId()为空");
            }
            CreateSnapshotResult result = response.getResult();
            return new BocloudResult(true, getAvailableSnapshot(result.getSnapshotId()), "创建快照成功");
        } catch (Exception e) {
            logger.error("create volume error :", e);
            return new BocloudResult(false, "创建快照失败");
        }
    }

    private SnapshotModel getAvailableSnapshot(String snapId) {
        SnapshotModel model = new SnapshotModel();
        model.setSnapshotUuid(snapId);
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                try {
                    Thread.sleep(10000);
                } catch (InterruptedException e) {
                    logger.error("Get error message:", e);
                    Thread.currentThread().interrupt();
                }
                BocloudResult detail = jdCloudSnapshotProvider.detail(model);
                if (detail.isFailed()) {
                    throw new Exception("查询快照详情失败");
                }
                if (detail.getData() == null) {
                    flag = false;
                    break;
                }
                model = (SnapshotModel)detail.getData();
                if ("available".equalsIgnoreCase(model.getStatus())) {
                    flag = false;
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 5 * 60 * 1000) {
                    flag = false;
                    break;
                }
            }
        } catch (Exception e) {
            logger.error("get disk error:", e);
        }
        return model;
    }

    @Override
    public BocloudResult attach(String serverId, String volumeId) {
        try {
            AttachDiskRequest request = new AttachDiskRequest();
            request.setDiskId(volumeId);
            request.setInstanceId(serverId);
            request.setRegionId(this.getRegionId());
            this.getVmClient().attachDisk(request);
            return new BocloudResult(true, this.getDiskDevice(serverId, volumeId), "挂载成功");
        } catch (Exception e) {
            logger.error("AttachDiskRequest volume error :", e);
            return new BocloudResult(false, "挂载失败");
        }
    }

    private String getDiskDevice(String serverId, String volumeId) {
        DescribeInstanceRequest request = new DescribeInstanceRequest();
        request.regionId(this.getRegion());
        request.setInstanceId(serverId);
        DescribeInstanceResponse response = this.getVmClient().describeInstance(request);
        DescribeInstanceResult result = response.getResult();
        Instance instance = result.getInstance();
        List<InstanceDiskAttachment> dataDisks = instance.getDataDisks();
        for (InstanceDiskAttachment instanceDiskAttachment : dataDisks) {
            Disk cloudDisk = instanceDiskAttachment.getCloudDisk();
            String diskId = cloudDisk.getDiskId();
            if (volumeId.equals(diskId)) {
                return instanceDiskAttachment.getDeviceName();
            }
        }
        return null;
    }

    @Override
    public BocloudResult attach(String serverId, String mountpoint, String volumeId) {
        try {
            AttachDiskRequest request = new AttachDiskRequest();
            request.setDiskId(volumeId);
            request.setInstanceId(serverId);
            request.setRegionId(this.getRegionId());
            request.setDeviceName(mountpoint);
            this.getVmClient().attachDisk(request);
            return new BocloudResult(true, this.getDiskDevice(serverId, volumeId), "挂载成功");
        } catch (Exception e) {
            logger.error("AttachDiskRequest volume error :", e);
            return new BocloudResult(false, "挂载失败");
        }
    }

    @Override
    public BocloudResult detach(String serverId, String volumeId) {
        try {
            DetachDiskRequest request = new DetachDiskRequest();
            request.setDiskId(volumeId);
            request.setForce(true);
            request.setInstanceId(serverId);
            request.setRegionId(this.getRegionId());
            DetachDiskResponse detachDisk = this.getVmClient().detachDisk(request);
            detachDisk.getResult();
            return new BocloudResult(true, "卸载成功");
        } catch (Exception e) {
            logger.error("DetachDiskRequest volume error :", e);
            return new BocloudResult(false, "卸载失败");
        }
    }

    @Override
    public BocloudResult resetStatus(String volumeId, String status) {
        return null;
    }

    @Override
    public BocloudResult uploadToImage(String volumeId, ImageModel imageModel) {
        return null;
    }

    @Override
    public BocloudResult extend(String volumeId, Integer newSize) {
        try {
            ExtendDiskRequest request = new ExtendDiskRequest();
            request.setDiskId(volumeId);
            request.setDiskSizeGB(newSize);
            ExtendDiskResponse response = this.getDiskClient().extendDisk(request);
            if (null == response) {
                return new BocloudResult(false, "硬盘扩容失败");
            }
            BocloudResult bocloudResult = this.detail(volumeId);
            if (bocloudResult.isFailed()) {
                return new BocloudResult(false, "硬盘扩容失败");
            }
            return new BocloudResult(true, bocloudResult.getData(), "云硬盘扩容成功");
        } catch (Exception e) {
            logger.info("云硬盘扩容失败：" + e);
            return new BocloudResult(false, "云硬盘扩容失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult list(String hostId, String storagePoolName) {
        return null;
    }

    @Override
    public BocloudResult remove(H3cVolumeModel volume) {
        return null;
    }

    // 仅可对制作快照的源硬盘进行数据恢复操作。
    // 仅源硬盘处于可用状态时才能使用快照进行数据恢复操作。
    // 云硬盘恢复后，当前数据将被清除，请您谨慎操作。
    @Override
    public BocloudResult restore(VolumeModel model) {
        try {
            RestoreDiskRequest request = new RestoreDiskRequest();
            request.setDiskId(model.getVolumeUuid());
            request.setRegionId(this.getRegionId());
            request.setSnapshotId(model.getSnapshotUuid());
            RestoreDiskResponse response = this.getDiskClient().restoreDisk(request);
            if (null == response) {
                return new BocloudResult(false, "恢复硬盘失败");
            }
            BocloudResult bocloudResult = this.detail(model.getVolumeUuid());
            if (bocloudResult.isFailed()) {
                return new BocloudResult(false, "恢复硬盘失败");
            }
            return new BocloudResult(true, bocloudResult.getData(), "恢复硬盘成功");
        } catch (Exception e) {
            logger.info("恢复硬盘失败：" + e);
            return new BocloudResult(false, "恢复硬盘失败：" + e.getMessage());
        }
    }
}
