package com.bocloud.cmp.provider;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPatch;
import org.apache.http.entity.StringEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.CommonResultBuilder;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.Result;
import io.fabric8.kubernetes.client.Config;
import io.fabric8.kubernetes.client.ConfigBuilder;
import io.fabric8.kubernetes.client.DefaultKubernetesClient;
import io.fabric8.kubernetes.client.KubernetesClient;

/**
 * kubernets provider
 * 
 * @时间 2019年8月20日
 */
public class KubernetesProvider extends GenericProvider implements SuperProvider {

	private static final Logger logger = LoggerFactory.getLogger(KubernetesProvider.class);

	private String endpoint;
	private String username;
	private String password;
	private String token;

	private KubernetesClient client = null;
	private HttpClient httpClient = new HttpClient(20 * 1000 * 1000, PostDataFormat.RAW);
	private String apiAuthToken = null;

	public KubernetesProvider(Butler butler) {
		super(butler);
		this.endpoint = butler.getUrl();
		this.username = butler.getUsername();
		this.password = butler.getPassword();
		this.token = butler.getToken();
		initClient();
	}

	protected void initClient() {
		// KubernetesClient
		ConfigBuilder withMasterUrl = new ConfigBuilder().withTrustCerts(true).withMasterUrl(endpoint);
		if (!StringUtils.isEmpty(token)) {
			withMasterUrl.withOauthToken(token);
		}
		Config config = withMasterUrl.build();
		client = new DefaultKubernetesClient(config);
	}

	@Override
	public boolean accessable() {
		try {
			client.namespaces().list();
			return true;
		} catch (Exception e) {
			logger.error("k8s连接失败 ： " + e.getMessage());
		}
		return false;
	}

	public String getEndpoint() {
		return endpoint;
	}

	public void setEndpoint(String endpoint) {
		this.endpoint = endpoint;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getToken() {
		return token;
	}

	public void setToken(String token) {
		this.token = token;
	}

	public KubernetesClient getClient() {
		return client;
	}

	public void setClient(KubernetesClient client) {
		this.client = client;
	}

	/**
	 * 请求头信息
	 *
	 * @param tokenId
	 * @param projectId
	 * @return
	 */
	private Map<String, Object> buildHeader(String tokenId) {
		Map<String, Object> header = new HashMap<String, Object>();
		header.put("accept", "application/json");
		header.put("content-type", "application/json");
		if (!StringUtils.isEmpty(tokenId)) {
			header.put("X-Auth-Token", tokenId);
		}
		return header;
	}

	public Result doGet(String apiUrl, Map<String, Object> params, String serviceName, String serviceType) {
		try {
			httpClient.setDataFormat(null);
			if (StringUtils.isEmpty(apiUrl)) {
				return Result.FAILED("Neuturn service is unavailable");
			}
			return this.httpClient.get(buildHeader(apiAuthToken), params, apiUrl);
		} finally {
			this.httpClient.close();
		}
	}

	public Result doPost(String apiUrl, Map<String, Object> params, String serviceName, String serviceType) {
		try {
			if (StringUtils.isEmpty(apiUrl)) {
				return Result.FAILED("Neuturn service is unavailable");
			}
			return this.httpClient.post(buildHeader(apiAuthToken), params, apiUrl);
		} finally {
			this.httpClient.close();
		}
	}

	public Result doPut(String apiUrl, Map<String, Object> params, String serviceName, String serviceType) {
		try {
			if (StringUtils.isEmpty(apiUrl)) {
				return Result.FAILED("Neuturn service is unavailable");
			}
			return this.httpClient.put(buildHeader(apiAuthToken), params, apiUrl);
		} finally {
			this.httpClient.close();
		}
	}

	public Result doPatch(String apiUrl, Map<String, Object> params, String serviceName, String serviceType) {
		try {
			if (StringUtils.isEmpty(apiUrl)) {
				return Result.FAILED("Neuturn service is unavailable");
			}
			return this.httpClient.patch(buildHeader(apiAuthToken), params, apiUrl);
		} finally {
			this.httpClient.close();
		}
	}

	public Result doPatch(String apiUrl, String params, String serviceName, String serviceType) {
		try {
			if (StringUtils.isEmpty(apiUrl)) {
				return Result.FAILED("service is unavailable");
			}
			HttpPatch patch = new HttpPatch(apiUrl);
			patch.addHeader("Accept", "application/json");
			patch.addHeader("Content-Type", "application/openstack-images-v2.1-json-patch");
			StringEntity httpEntity = new StringEntity(JSONObject.toJSON(params).toString(), "UTF-8");
			patch.setEntity(httpEntity);
			CloseableHttpResponse httpResponse = httpClient.getHttpClient().execute(patch);
			Result result = new CommonResultBuilder().build(httpResponse.getStatusLine().getStatusCode());
			return result;
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return Result.FAILED("请求失败");
		} finally {
			this.httpClient.close();
		}
	}

	public Result doDelete(String apiUrl, Map<String, Object> params, String serviceName, String serviceType) {
		try {
			if (StringUtils.isEmpty(apiUrl)) {
				return Result.FAILED("Neuturn service is unavailable");
			}
			return this.httpClient.delete(buildHeader(apiAuthToken), params, apiUrl);
		} finally {
			this.httpClient.close();
		}
	}

	public static String strRandom(int count) {
		String str = "";
		if (count <= 0) {
			return str;
		}
		String letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890";
		Random r = new Random(1);
		for (int i = 0; i < count; i++) {
			int ran1 = r.nextInt(letters.length());
			str += letters.charAt(ran1);
		}
		return str;
	}

}