package com.bocloud.cmp.provider.common;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.kubernetes.DaemonSetModel;
import com.bocloud.cmp.model.kubernetes.DeploymentModel;
import com.bocloud.cmp.model.kubernetes.KubernetesSyncModel;
import com.bocloud.cmp.model.kubernetes.NamespaceModel;
import com.bocloud.cmp.model.kubernetes.NodeModel;
import com.bocloud.cmp.model.kubernetes.PersistentVolumeClaimModel;
import com.bocloud.cmp.model.kubernetes.PersistentVolumeModel;
import com.bocloud.cmp.model.kubernetes.PodModel;
import com.bocloud.cmp.model.kubernetes.ReplicaModel;
import com.bocloud.cmp.model.kubernetes.SecretModel;
import com.bocloud.cmp.model.kubernetes.ServiceModel;
import com.bocloud.cmp.provider.KubernetesProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.DaemonSetConvertor;
import com.bocloud.cmp.provider.convertor.DeploymentConvertor;
import com.bocloud.cmp.provider.convertor.NamespaceConvertor;
import com.bocloud.cmp.provider.convertor.NodeConvertor;
import com.bocloud.cmp.provider.convertor.PersistentVolumeClaimConvertor;
import com.bocloud.cmp.provider.convertor.PersistentVolumeConvertor;
import com.bocloud.cmp.provider.convertor.PodConvertor;
import com.bocloud.cmp.provider.convertor.ReplicaConvertor;
import com.bocloud.cmp.provider.convertor.SecretConvertor;
import com.bocloud.cmp.provider.convertor.ServiceConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;

import io.fabric8.kubernetes.api.model.LabelSelector;
import io.fabric8.kubernetes.api.model.Namespace;
import io.fabric8.kubernetes.api.model.NamespaceList;
import io.fabric8.kubernetes.api.model.Node;
import io.fabric8.kubernetes.api.model.NodeList;
import io.fabric8.kubernetes.api.model.PersistentVolume;
import io.fabric8.kubernetes.api.model.PersistentVolumeClaim;
import io.fabric8.kubernetes.api.model.PersistentVolumeClaimList;
import io.fabric8.kubernetes.api.model.PersistentVolumeList;
import io.fabric8.kubernetes.api.model.Pod;
import io.fabric8.kubernetes.api.model.PodList;
import io.fabric8.kubernetes.api.model.Secret;
import io.fabric8.kubernetes.api.model.SecretList;
import io.fabric8.kubernetes.api.model.Service;
import io.fabric8.kubernetes.api.model.ServiceList;
import io.fabric8.kubernetes.api.model.apps.DaemonSet;
import io.fabric8.kubernetes.api.model.apps.DaemonSetList;
import io.fabric8.kubernetes.api.model.apps.Deployment;
import io.fabric8.kubernetes.api.model.apps.DeploymentList;
import io.fabric8.kubernetes.api.model.apps.ReplicaSet;
import io.fabric8.kubernetes.api.model.apps.ReplicaSetList;
import io.fabric8.kubernetes.client.KubernetesClient;

/**
 * k8s同步数据组装类
 * 
 * @时间 2019年8月22日
 */
@Provider("bocloud.cmp.kubernetes")
public class KubeLocationProvider extends KubernetesProvider implements LocationProvider {

	public KubeLocationProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(KubeLocationProvider.class);

	private BeanConvertor<PodModel, Pod> podConvertor = new PodConvertor();
	private BeanConvertor<NamespaceModel, Namespace> namespaceConvertor = new NamespaceConvertor();
	private BeanConvertor<NodeModel, Node> nodeConvertor = new NodeConvertor();
	private BeanConvertor<ServiceModel, Service> serviceConvertor = new ServiceConvertor();
	private BeanConvertor<SecretModel, Secret> secretConvertor = new SecretConvertor();
	private BeanConvertor<DeploymentModel, Deployment> deploymentConvertor = new DeploymentConvertor();
	private BeanConvertor<DaemonSetModel, DaemonSet> daemonSetConvertor = new DaemonSetConvertor();
	private BeanConvertor<PersistentVolumeModel, PersistentVolume> persistentVolumeConvertor = new PersistentVolumeConvertor();
	private BeanConvertor<ReplicaModel, ReplicaSet> replicaConvertor = new ReplicaConvertor(); // 副本集
	private BeanConvertor<PersistentVolumeClaimModel, PersistentVolumeClaim> persistentVolumeClaimConvertor = new PersistentVolumeClaimConvertor();

	@Override
	public BocloudResult sync() {
		try {
			KubernetesSyncModel syncModel = new KubernetesSyncModel();
			KubernetesClient kubernetesClient = this.getClient();
			// 获取namespace
			List<String> namespaceNames = new ArrayList<>();
			List<NamespaceModel> syncNamespaces = this.syncNamespaces(syncModel);
			if (null == syncNamespaces) {
				return new BocloudResult(false, "查询命名空间异常！");
			}
			// 同步node
			List<NodeModel> nodeModels = this.syncNodes();
			if (null == nodeModels) {
				return new BocloudResult(false, "查询节点信息异常！");
			}
			syncModel.setNodeModels(nodeModels);

			List<ServiceModel> allServiceModels = new ArrayList<>();
			List<SecretModel> allSecretModels = new ArrayList<>();
			List<PersistentVolumeClaimModel> persistentVolumeClaimModels = new ArrayList<>();

			for (NamespaceModel namespaceModel : syncNamespaces) {
				namespaceNames.add(namespaceModel.getName());
				// 同步service
				ServiceList serviceList = kubernetesClient.services().inNamespace(namespaceModel.getName()).list();
				List<Service> serviceItems = serviceList.getItems();
				List<ServiceModel> serviceModels = serviceConvertor.convertList(serviceItems);
				for (ServiceModel serviceModel : serviceModels) {
					serviceModel.setNamespace(namespaceModel.getName());
				}
				// 同步pvc
				PersistentVolumeClaimList persistentVolumeClaimList = kubernetesClient.persistentVolumeClaims()
						.inNamespace(namespaceModel.getName()).list();
				List<PersistentVolumeClaim> volumeClaimItems = persistentVolumeClaimList.getItems();
				List<PersistentVolumeClaimModel> convertList = persistentVolumeClaimConvertor
						.convertList(volumeClaimItems);

				// 同步Secret
				SecretList secretList = kubernetesClient.secrets().inNamespace(namespaceModel.getName()).list();
				List<Secret> items = secretList.getItems();
				List<SecretModel> secretModels = secretConvertor.convertList(items);

				allSecretModels.addAll(secretModels);
				allServiceModels.addAll(serviceModels);
				persistentVolumeClaimModels.addAll(convertList);
			}
			syncModel.setSecretModels(allSecretModels);
			syncModel.setServiceModels(allServiceModels);
			syncModel.setPersistentVolumeClaimModels(persistentVolumeClaimModels);
			List<PodModel> podModels = new ArrayList<>();

			// 同步Pods
			List<PodModel> syncPods = this.syncPods(allServiceModels, podModels);
			if (null == syncPods) {
				return new BocloudResult(false, "POD查询失败！");
			}
			Map<String, PodModel> podMap = new HashMap<>();
			for (int i = podModels.size() - 1; i >= 0; i--) {
				PodModel podModel = podModels.get(i);
				if (namespaceNames.contains(podModel.getNamespace())) {
					podMap.put(podModel.getUid(), podModel);
				} else {
					podModels.remove(podModel);
				}
			}
			// 同步daemonSet
			List<DaemonSetModel> daemonSetModelModels = this.syncDaemonSet(syncPods);
			for (int i = daemonSetModelModels.size() - 1; i >= 0; i--) {
				DaemonSetModel daemonSetModel = daemonSetModelModels.get(i);
				if (!namespaceNames.contains(daemonSetModel.getNamespace())) {
					daemonSetModelModels.remove(daemonSetModel);
				}
			}
			syncModel.setDaemonSetModels(daemonSetModelModels);
			// 同步部署
			List<DeploymentModel> deploymentModels = this.syncDeployments(podMap);
			for (int i = deploymentModels.size() - 1; i >= 0; i--) {
				DeploymentModel deploymentModel = deploymentModels.get(i);
				if (!namespaceNames.contains(deploymentModel.getNamespace())) {
					deploymentModels.remove(deploymentModel);
				}
			}
			syncModel.setPodModels(new ArrayList<>(podMap.values()));
			syncModel.setDeploymentModels(deploymentModels);
			// 持久化存储卷
			List<PersistentVolumeModel> syncPersistentVolumes = this.syncPersistentVolumes();
			syncModel.setPersistentModels(syncPersistentVolumes);
			// 节点统计信息
			this.statisticalNodeInfo(syncModel);
			return new BocloudResult(true, syncModel, "k8s同步成功！");
		} catch (Exception e) {
			logger.error("k8s 同步失败 ： " + e.getMessage());
			return new BocloudResult(false, "同步失败：" + e.getMessage());
		}
	}

	/**
	 * 统计node
	 * 
	 * @param syncModel
	 */
	private void statisticalNodeInfo(KubernetesSyncModel syncModel) {
		try {
			// 节点信息
			List<NodeModel> nodeModels = syncModel.getNodeModels();
			Map<String, NodeModel> nodeMap = new HashMap<>();
			for (NodeModel nodeModel : nodeModels) {
				nodeMap.put(nodeModel.getName(), nodeModel);
			}
			// DaemonSet信息
			List<DaemonSetModel> daemonSetModels = syncModel.getDaemonSetModels();
			Map<String, DaemonSetModel> daemonSetModelsMap = new HashMap<>();
			for (DaemonSetModel daemonSetModel : daemonSetModels) {
				daemonSetModelsMap.put(daemonSetModel.getUid(), daemonSetModel);
			}
			// ReplicaSet 信息
			ReplicaSetList replicaSetList = this.getClient().apps().replicaSets().list();
			List<ReplicaModel> replicaModelList = replicaConvertor.convertList(replicaSetList.getItems());
			Map<String, ReplicaModel> replicaModelListMap = new HashMap<>();
			for (ReplicaModel replicaModel : replicaModelList) {
				replicaModelListMap.put(replicaModel.getUid(), replicaModel);
			}

			List<PodModel> podModels = syncModel.getPodModels();
			Map<String, List<PodModel>> podModelsMap = new HashMap<>();
			for (PodModel podModel : podModels) {
				if (null == podModel.getNodeName()) {
					continue;
				}
				List<PodModel> list = new ArrayList<>();
				if (podModelsMap.containsKey(podModel.getNodeName())) {
					list = podModelsMap.get(podModel.getNodeName());
				}
				list.add(podModel);
				podModelsMap.put(podModel.getNodeName(), list);
			}
			Set<Entry<String, List<PodModel>>> entrySet = podModelsMap.entrySet();
			for (Entry<String, List<PodModel>> entry : entrySet) {
				String nodeName = entry.getKey();
				NodeModel nodeModel = nodeMap.get(nodeName);
				Integer cpu = nodeModel.getCpu();
				BigDecimal memory = nodeModel.getMemory();
				List<PodModel> value = entry.getValue();
				Integer requestsCpu = 0;
				Integer requestsMemory = 0;
				Integer limitsCpu = 0;
				Integer limitsMemory = 0;
				for (PodModel podModel : value) {
					String parentResourceType = podModel.getParentResourceType();
					if ("ReplicaSet".equalsIgnoreCase(parentResourceType)) {
						ReplicaModel replicaModel = replicaModelListMap.get(podModel.getParentResourceUid());
						if (null != replicaModel) {
							requestsCpu += Integer.valueOf(unitConversion(replicaModel.getRequestsCpu(), "CPU"));
							requestsMemory += Integer
									.valueOf(unitConversion(replicaModel.getRequestsMemory(), "MEMORY"));
							limitsCpu += Integer.valueOf(unitConversion(replicaModel.getLimitsCpu(), "CPU"));
							limitsMemory += Integer.valueOf(unitConversion(replicaModel.getLimitsMemory(), "MEMORY"));
						}
					}
					if ("DaemonSet".equalsIgnoreCase(parentResourceType)) {
						DaemonSetModel daemonSetModel = daemonSetModelsMap.get(podModel.getParentResourceUid());
						if (null != daemonSetModel) {
							requestsCpu += Integer.valueOf(unitConversion(daemonSetModel.getRequestsCpu(), "CPU"));
							requestsMemory += Integer
									.valueOf(unitConversion(daemonSetModel.getRequestsMemory(), "MEMORY"));
							limitsCpu += Integer.valueOf(unitConversion(daemonSetModel.getLimitsCpu(), "CPU"));
							limitsMemory += Integer.valueOf(unitConversion(daemonSetModel.getLimitsMemory(), "MEMORY"));
						}
					}
				}
				nodeModel.setCpu(cpu / 1000);
				double rint = Math
						.rint(memory.divide(new BigDecimal(1024 * 1024), 2, BigDecimal.ROUND_HALF_UP).doubleValue());
				nodeModel.setMemory(new BigDecimal(rint));
				nodeModel.setCpuUsed(requestsCpu + "m");
				nodeModel.setMemoryUsed((requestsMemory / 1024) + "Mi");
				BigDecimal cpuPercentDivide = new BigDecimal(requestsCpu).multiply(new BigDecimal(100))
						.divide(BigDecimal.valueOf(cpu.longValue()), 2, BigDecimal.ROUND_HALF_UP);
				nodeModel.setCpuPercent(cpuPercentDivide.toString());
				BigDecimal memoryPercent = new BigDecimal(requestsMemory).multiply(new BigDecimal(100))
						.divide(BigDecimal.valueOf(memory.longValue()), 2, BigDecimal.ROUND_HALF_UP);
				nodeModel.setMemoryPercent(memoryPercent.toString());
			}

		} catch (Exception e) {
			logger.error("node统计异常：" + e.getMessage());
		}
	}

	public String unitConversion(String oldValue, String category) {
		String newValue = "0";
		try {
			if (StringUtils.isEmpty(oldValue)) {
				return newValue;
			}
			if ("CPU".equalsIgnoreCase(category)) {
				if (oldValue.contains("m")) {
					newValue = oldValue.substring(0, oldValue.indexOf("m"));
				} else {
					newValue = Integer.valueOf(oldValue) * 1000 + "";
				}
			}
			if ("MEMORY".equalsIgnoreCase(category)) {
				if (oldValue.contains("Ki")) {
					newValue = oldValue.substring(0, oldValue.indexOf("Ki"));
				} else if (oldValue.contains("Mi") || oldValue.contains("M") || oldValue.contains("G")) {
					String unit = "";
					Integer val = 1024;
					if (oldValue.contains("Mi")) {
						unit = "Mi";
					}
					if (oldValue.contains("M")) {
						unit = "M";
					}
					if (oldValue.contains("G")) {
						unit = "G";
						val = val * 1024;
					}
					oldValue = oldValue.substring(0, oldValue.indexOf(unit));
					Integer limitsM = Integer.valueOf(oldValue) * val;
					newValue = String.valueOf(limitsM);
				}
			}
		} catch (Exception e) {
			logger.error("单位换算失败：" + e.getMessage());
		}
		return newValue;
	}

	private List<NodeModel> syncNodes() {
		try {
			NodeList nodeList = this.getClient().nodes().list();
			List<Node> nodeItems = nodeList.getItems();
			return nodeConvertor.convertList(nodeItems);
		} catch (Exception e) {
			logger.error("同步节点异常：" + e.getMessage());
			return null;
		}
	}

	private List<NamespaceModel> syncNamespaces(KubernetesSyncModel syncModel) {
		try {
			NamespaceList namespaceList = this.getClient().namespaces().list();
			List<Namespace> namespaceItems = namespaceList.getItems();
			List<NamespaceModel> namespaceModels = namespaceConvertor.convertList(namespaceItems);
			syncModel.setNamespaceModels(namespaceModels);
			return namespaceModels;
		} catch (Exception e) {
			logger.error("同步命名空间异常：" + e.getMessage());
			return null;
		}
	}

	private List<DaemonSetModel> syncDaemonSet(List<PodModel> syncPods) {
		try {
			DaemonSetList list = this.getClient().apps().daemonSets().list();
			List<DaemonSetModel> daemonSetModelModels = daemonSetConvertor.convertList(list.getItems());
			Map<String, DaemonSetModel> deploymentModelMap = new HashMap<>();
			for (DaemonSetModel daemonSetModel : daemonSetModelModels) {
				deploymentModelMap.put(daemonSetModel.getUid(), daemonSetModel);
			}
			for (PodModel podModel : syncPods) {
				if (deploymentModelMap.containsKey(podModel.getParentResourceUid())) {
					DaemonSetModel daemonSetModel = deploymentModelMap.get(podModel.getParentResourceUid());
					podModel.setDaemonSetName(daemonSetModel.getName());
					podModel.setParentResourceUid(daemonSetModel.getUid());
					podModel.setParentResourceType(daemonSetModel.getKind());
					podModel.setParentResourceName(daemonSetModel.getName());
					podModel.setParentResourceVersion(daemonSetModel.getApiVersion());
					Integer podCount = daemonSetModel.getPodCount();
					if (null == podCount || 0 == podCount) {
						daemonSetModel.setPodCount(1);
					} else {
						daemonSetModel.setPodCount(daemonSetModel.getPodCount() + 1);
					}
				}
			}
			return daemonSetModelModels;
		} catch (Exception e) {
			logger.error("同步DaemonSet异常：" + e.getMessage());
			return null;
		}
	}

	private List<DeploymentModel> syncDeployments(Map<String, PodModel> podMap) {
		List<DeploymentModel> deploymentModels = new ArrayList<>();
		try {
			// Deployment
			DeploymentList deploymentList = this.getClient().apps().deployments().list();
			deploymentModels = deploymentConvertor.convertList(deploymentList.getItems());
			if (ListTool.isEmpty(deploymentModels)) {
				return deploymentModels;
			}
			for (DeploymentModel deploymentModel : deploymentModels) {
				List<ReplicaModel> replicaModels = listReplica(deploymentModel.getUid(),
						deploymentModel.getNamespace());
				if (ListTool.isEmpty(replicaModels)) {
					continue;
				}
				deploymentModel.setReplicaModels(replicaModels);
				for (ReplicaModel replicaModel : replicaModels) {
					PodList podList = this.getClient().pods().inNamespace(replicaModel.getNamespace()).list();
					List<PodModel> podConvertors = podConvertor.convertList(podList.getItems());
					for (PodModel podModel : podConvertors) {
						String key = podModel.getUid();
						if (replicaModel.getKind().equalsIgnoreCase(podModel.getParentResourceType())
								&& replicaModel.getUid().equalsIgnoreCase(podModel.getParentResourceUid())
								&& podMap.containsKey(key)) {
							PodModel model = podMap.get(key);
							model.setDeploymentName(deploymentModel.getName());
							model.setDeploymentUid(deploymentModel.getUid());
							model.setDeploymentApiVersion(deploymentModel.getApiVersion());
							model.setReplicaName(replicaModel.getName());
							podMap.put(key, model);
							Integer podCount = deploymentModel.getPodCount();
							if (null == podCount || 0 == podCount) {
								deploymentModel.setPodCount(1);
							} else {
								deploymentModel.setPodCount(deploymentModel.getPodCount() + 1);
							}
						}
					}
				}
			}
		} catch (Exception e) {
			logger.error("同步部署异常：" + e.getMessage());
		}
		return deploymentModels;
	}

	private List<PodModel> syncPods(List<ServiceModel> serviceModels, List<PodModel> podModels) {
		try {
			for (ServiceModel serviceModel : serviceModels) {
				Map<String, String> selectorMap = serviceModel.getSelectorMap();
				if (null != selectorMap) {
					LabelSelector selector = new LabelSelector();
					selector.setMatchLabels(serviceModel.getSelectorMap());
					PodList podList = this.getClient().pods().inNamespace(serviceModel.getNamespace())
							.withLabelSelector(selector).list();
					List<PodModel> podConvertors = podConvertor.convertList(podList.getItems());
					for (PodModel podModel : podConvertors) {
						podModel.setNamespace(serviceModel.getNamespace());
						podModel.setServiceName(serviceModel.getName());
					}
					podModels.addAll(podConvertors);
				}
			}
			Map<String, PodModel> podMap = new HashMap<>();
			for (PodModel podModel : podModels) {
				podMap.put(podModel.getUid(), podModel);
			}

			PodList list = this.getClient().pods().list();
			List<PodModel> allPodConvertors = podConvertor.convertList(list.getItems());
			for (PodModel podModel : allPodConvertors) {
				String key = podModel.getUid();
				if (!podMap.containsKey(key)) {
					podModels.add(podModel);
				}
			}
			return podModels;
		} catch (Exception e) {
			logger.error("同步Pod异常：" + e.getMessage());
			return null;
		}
	}

	private List<PersistentVolumeModel> syncPersistentVolumes() {
		try {
			// 持久化存储卷
			PersistentVolumeList persistentVolumeList = this.getClient().persistentVolumes().list();
			return persistentVolumeConvertor.convertList(persistentVolumeList.getItems());
		} catch (Exception e) {
			logger.error("同步存储卷异常：" + e.getMessage());
			return new ArrayList<>();
		}
	}

	public List<ReplicaModel> listReplica(String depUuid, String namespace) {
		List<ReplicaModel> replicaModelList = new ArrayList<>();
		try {
			ReplicaSetList replicaSetList = this.getClient().apps().replicaSets().inNamespace(namespace).list();
			List<ReplicaModel> convertList = replicaConvertor.convertList(replicaSetList.getItems());
			if (!ListTool.isEmpty(convertList)) {
				for (ReplicaModel replicaModel : convertList) {
					if (depUuid.equalsIgnoreCase(replicaModel.getParentResourceUid())) {
						replicaModelList.add(replicaModel);
					}
				}
			}
			return replicaModelList;
		} catch (Exception e) {
			logger.error("查询副本集ERROR:" + e.getMessage());
			return replicaModelList;
		}
	}

	protected void getRelation(ServiceModel serviceModel, List<PodModel> allPodConvertors) {
		Map<String, String> selectorMap = serviceModel.getSelectorMap();
		if (null != selectorMap) {
			String namespace = serviceModel.getNamespace();
			for (PodModel podModel : allPodConvertors) {
				String podNs = podModel.getNamespace();
				Map<String, String> labelsMap = podModel.getLabelsMap();
				for (Entry<String, String> entry : selectorMap.entrySet()) {
					if (podNs.equals(namespace) && labelsMap.containsKey(entry.getKey())
							&& labelsMap.get(entry.getKey()).equals(entry.getValue())) {
						podModel.setServiceName(serviceModel.getName());
					}
				}
			}
		}
	}

	@Override
	public BocloudResult listRegions() {
		try {
			// 获取namespace
			NamespaceList namespaceList = this.getClient().namespaces().list();
			List<Namespace> namespaceItems = namespaceList.getItems();
			List<NamespaceModel> namespaceModels = namespaceConvertor.convertList(namespaceItems);
			return new BocloudResult(true, namespaceModels, "查询成功！");
		} catch (Exception e) {
			logger.error("同步命名空间异常：" + e.getMessage());
			return new BocloudResult(false, "同步命名空间异常：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult listZones(String region) {
		return null;
	}

	@Override
	public BocloudResult syncPortByNetworkUuid(String networkUuid) {
		return null;
	}

	@Override
	public BocloudResult getHostModel(HostModel host) {
		return null;
	}

}
