package com.bocloud.cmp.provider.kubernetes;

import java.io.ByteArrayInputStream;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.kubernetes.PodModel;
import com.bocloud.cmp.provider.KubernetesProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.PodConvertor;
import com.bocloud.cmp.provider.utils.CommonUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;

import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.Pod;

/**
 * k8s pod
 * 
 * @时间 2019年8月21日
 */
@Provider("bocloud.cmp.kubernetes")
public class KubePodProvider extends KubernetesProvider implements PodProvider {

	public KubePodProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(KubePodProvider.class);

	private BeanConvertor<PodModel, Pod> convertor = new PodConvertor();

	@Override
	public BocloudResult list() {
		try {
			List<Pod> items = this.getClient().pods().list().getItems();
			List<PodModel> convertList = convertor.convertList(items);
			return new BocloudResult(true, convertList, "查询Node成功！");
		} catch (Exception e) {
			logger.error("query node list fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	public BocloudResult list(Map<String, String> labelsMap) {
		try {
			Long startTime = System.currentTimeMillis();
			boolean flag = false;
			while (true) {
				Thread.sleep(10 * 1000);
				List<Pod> items = this.getClient().pods().withLabels(labelsMap).list().getItems();
				if (!ListTool.isEmpty(items)) {
					boolean podStatus = true;
					for (Pod pod : items) {
						if (!"running".equalsIgnoreCase(pod.getStatus().getPhase())) {
							podStatus = false;
						}
					}
					flag = podStatus;
				} else {
					return new BocloudResult(false, "Pod创建异常！");
				}
				// 等待三分钟
				if (flag || System.currentTimeMillis() - startTime > 5 * 60 * 1000) {
					break;
				}
			}
			List<Pod> items = this.getClient().pods().withLabels(labelsMap).list().getItems();
			List<PodModel> convertList = convertor.convertList(items);
			return new BocloudResult(true, convertList, "查询成功");
		} catch (Exception e) {
			logger.error("query pod list fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult create(PodModel model) {
		try {
			String yamlText = CommonUtils.yamlAnalysis(model.getYamlText());
			ByteArrayInputStream bais = new ByteArrayInputStream(yamlText.getBytes());
			Pod pod = this.getClient().pods().load(bais).get();
			ObjectMeta metadata = pod.getMetadata();
			if (null != metadata && null != metadata.getResourceVersion()) {
				metadata.setResourceVersion(null);
			}
			if (null != metadata && null != metadata.getUid()) {
				metadata.setUid(null);
			}
			if (null != metadata && null != metadata.getSelfLink()) {
				metadata.setSelfLink(null);
			}
			pod.setMetadata(metadata);
			Pod create = this.getClient().pods().create(pod);
			PodModel convertModel = convertor.convertModel(create);
			return new BocloudResult(true, this.detail(convertModel).getData(), "新增成功！");
		} catch (Exception e) {
			logger.error("新增失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

	@Override
	public BocloudResult remove(PodModel model) {
		try {
			Pod pod = this.getClient().pods().inNamespace(model.getNamespace()).withName(model.getName()).get();
			if (null == pod) {
				return new BocloudResult(true, "删除成功！");
			}
			Boolean delete = this.getClient().pods().delete(pod);
			return new BocloudResult(delete, "", delete ? "删除成功！" : "删除失败！");
		} catch (Exception e) {
			logger.error("delete node fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(PodModel model) {
		try {
			Pod pod = this.getClient().pods().inNamespace(model.getNamespace()).withName(model.getName()).get();
			if (null == pod) {
				return new BocloudResult(false, "pod不存在！");
			}
			Long startTime = System.currentTimeMillis();
			while (true) {
				Thread.sleep(2000);
				pod = this.getClient().pods().inNamespace(model.getNamespace()).withName(model.getName()).get();
				PodModel convertModel = convertor.convertModel(pod);
				if ("Running".equalsIgnoreCase(convertModel.getStatus())
						|| System.currentTimeMillis() - startTime > 2 * 60 * 1000) {
					break;
				}
			}
			return new BocloudResult(true, convertor.convertModel(pod), "查询成功！");
		} catch (Exception e) {
			logger.error("get pod fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(PodModel model) {
		try {
			String yamlText = model.getYamlText();
			ByteArrayInputStream bais = new ByteArrayInputStream(yamlText.getBytes());
			Pod pod = this.getClient().pods().load(bais).get();
			ObjectMeta metadata = pod.getMetadata();
			if (StringUtils.isEmpty(metadata.getNamespace()) || StringUtils.isEmpty(metadata.getName())) {
				return new BocloudResult(false, "参数缺失！");
			}
			Pod replace = this.getClient().pods().inNamespace(metadata.getNamespace()).withName(metadata.getName())
					.replace(pod);
			return new BocloudResult(true, convertor.convertModel(replace), "编辑成功！");
		} catch (Exception e) {
			logger.error("编辑失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

	@Override
	public BocloudResult logDetail(PodModel model) {
		try {
			Pod pod = this.getClient().pods().inNamespace(model.getNamespace()).withName(model.getName()).get();
			if (null == pod) {
				return new BocloudResult(false, "资源不存在！");
			}
			String log = this.getClient().pods().inNamespace(model.getNamespace()).withName(model.getName())
					.getLog(true);
			return new BocloudResult(true, log, "日志查询成功！");
		} catch (Exception e) {
			logger.error("日志查询失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

}
