package com.bocloud.cmp.provider.kubernetes;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.kubernetes.ServiceModel;
import com.bocloud.cmp.provider.KubernetesProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.ServiceConvertor;
import com.bocloud.cmp.provider.utils.CommonUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

import io.fabric8.kubernetes.api.model.IntOrString;
import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.Service;
import io.fabric8.kubernetes.api.model.ServiceList;
import io.fabric8.kubernetes.api.model.ServicePort;
import io.fabric8.kubernetes.api.model.ServiceSpec;

/**
 * k8s service
 * 
 * @时间 2019年8月21日
 */
@Provider("bocloud.cmp.kubernetes")
public class KubeServiceProvider extends KubernetesProvider implements ServiceProvider {

	public KubeServiceProvider(Butler butler) {
		super(butler);
	}

	BeanConvertor<ServiceModel, Service> convertor = new ServiceConvertor();

	private static Logger logger = LoggerFactory.getLogger(KubeServiceProvider.class);

	@Override
	public BocloudResult list() {
		try {
			ServiceList list = this.getClient().services().list();
			List<Service> items = list.getItems();
			List<ServiceModel> convertList = convertor.convertList(items);
			return new BocloudResult(true, convertList, "查询Node成功！");
		} catch (Exception e) {
			logger.error("query node list fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult create(ServiceModel model) {
		try {
			String yamlText = CommonUtils.yamlAnalysis(model.getYamlText());
			if (StringUtils.isEmpty(yamlText)) {
				return createService(model);
			}
			ByteArrayInputStream bais = new ByteArrayInputStream(yamlText.getBytes());
			Service service = this.getClient().services().load(bais).get();
			ObjectMeta metadata = service.getMetadata();
			if (null != metadata.getResourceVersion()) {
				metadata.setResourceVersion(null);
			}
			if (null != metadata.getUid()) {
				metadata.setUid(null);
			}
			if (null != metadata.getSelfLink()) {
				metadata.setSelfLink(null);
			}
			service.setMetadata(metadata);
			Service create = this.getClient().services().create(service);
			return new BocloudResult(true, convertor.convertModel(create), "新增成功！");
		} catch (Exception e) {
			logger.error("新增Services失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

	private BocloudResult createService(ServiceModel model) {
		try {
			Service item = new Service();
			ObjectMeta metadata = new ObjectMeta();
			metadata.setName(model.getName());
			metadata.setNamespace(model.getNamespace());
			metadata.setLabels(model.getLabelsMap());
			ServiceSpec spec = new ServiceSpec();
			String type = model.getType();
			spec.setType(type);
			spec.setSelector(model.getSelectorMap());
			String serviceConfig = model.getServiceConfig();
			if (StringUtils.isEmpty(serviceConfig)) {
				return new BocloudResult(false, "参数格式错误！");
			}
			List<ServicePort> ports = new ArrayList<ServicePort>();
			List<JSONObject> serviceConfigArrays = JSONObject.parseArray(serviceConfig, JSONObject.class);
			ServicePort port = null;
			for (JSONObject jsonObject : serviceConfigArrays) {
				port = new ServicePort();
				port.setName(jsonObject.getString("configName"));
				port.setProtocol(jsonObject.getString("protocol"));
				port.setPort(jsonObject.getInteger("port"));
				port.setTargetPort(new IntOrString(jsonObject.getInteger("port")));
				if ("ClusterIP".equalsIgnoreCase(type)) {
				} else {
					port.setNodePort(jsonObject.getInteger("nodePort"));
				}
				ports.add(port);
			}
			spec.setPorts(ports);
			item.setSpec(spec);
			item.setMetadata(metadata);
			Service create = this.getClient().services().create(item);
			return new BocloudResult(true, convertor.convertModel(create), "新增成功！");
		} catch (Exception e) {
			logger.error("服务创建失败：" + e.getMessage());
			return new BocloudResult(false, "服务创建失败: " + CommonUtils.getMessage(e.getMessage()));
		}
	}

	@Override
	public BocloudResult remove(ServiceModel model) {
		try {
			Service service = this.getClient().services().inNamespace(model.getNamespace()).withName(model.getName())
					.get();
			if (null == service) {
				return new BocloudResult(true, "删除成功！");
			}
			Boolean delete = this.getClient().services().delete(service);
			return new BocloudResult(true,  "删除成功！");
		} catch (Exception e) {
			logger.error("delete service fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(ServiceModel model) {
		return null;
	}

	@Override
	public BocloudResult modify(ServiceModel model) {
		try {
			String yamlText = model.getYamlText();
			if (StringUtils.isEmpty(yamlText)) {
				return modifyService(model);
			}
			ByteArrayInputStream bais = new ByteArrayInputStream(yamlText.getBytes());
			Service service = this.getClient().services().load(bais).get();
			ObjectMeta metadata = service.getMetadata();
			Service createOrReplace = this.getClient().services().inNamespace(metadata.getNamespace())
					.withName(metadata.getName()).replace(service);
			return new BocloudResult(true, convertor.convertModel(createOrReplace), "编辑成功！");
		} catch (Exception e) {
			logger.error("编辑失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

	private BocloudResult modifyService(ServiceModel model) {
		try {
			String serviceConfig = model.getServiceConfig();
			if (StringUtils.isEmpty(serviceConfig)) {
				return new BocloudResult(false, "参数格式错误！");
			}
			Service service = this.getClient().services().inNamespace(model.getNamespace()).withName(model.getName())
					.get();
			if (null == service) {
				return new BocloudResult(false, "资源不存在！");
			}
			ServiceSpec serviceSpec = service.getSpec();
			String type = serviceSpec.getType();
			List<ServicePort> ports = new ArrayList<ServicePort>();
			List<JSONObject> serviceConfigArrays = JSONObject.parseArray(serviceConfig, JSONObject.class);
			ServicePort port = null;
			for (JSONObject jsonObject : serviceConfigArrays) {
				port = new ServicePort();
				String name = jsonObject.getString("name");
				if (StringUtils.isEmpty(name)) {
					name = jsonObject.getString("configName");
				}
				port.setName(name);
				port.setProtocol(jsonObject.getString("protocol"));
				port.setPort(jsonObject.getInteger("port"));
				port.setTargetPort(new IntOrString(jsonObject.getInteger("port")));
				if ("ClusterIP".equalsIgnoreCase(type)) {
				} else {
					port.setNodePort(jsonObject.getInteger("nodePort"));
				}
				ports.add(port);
			}
			serviceSpec.setPorts(ports);
			service.setSpec(serviceSpec);
			Service createOrReplace = this.getClient().services().inNamespace(model.getNamespace())
					.withName(model.getName()).replace(service);
			return new BocloudResult(true, convertor.convertModel(createOrReplace), "编辑成功！");
		} catch (Exception e) {
			logger.error("编辑失败：" + e.getMessage());
			return new BocloudResult(false, "编辑失败：" + e.getMessage());
		}
	}

}
