package com.bocloud.cmp.provider.compute;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.kingconvertor.LocalVolumeConvertor;
import com.bocloud.cmp.kingconvertor.ServerConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.OSMigrateModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.provider.KingCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.storage.KingCloudVolumeProvider;
import com.bocloud.cmp.provider.util.KingCloudConstants;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.ksc.HttpMethod;
import com.ksc.kec.model.LocalVolume;
import com.ksc.kec.model.ModifyInstanceAttributeRequest;
import com.ksc.kec.model.ModifyInstanceAttributeResult;
import com.ksc.kec.model.ModifyInstanceTypeRequest;
import com.ksc.kec.model.ModifyInstanceTypeResult;
import com.ksc.kec.model.RebootInstancesRequest;
import com.ksc.kec.model.RebootInstancesResult;
import com.ksc.kec.model.StartInstancesRequest;
import com.ksc.kec.model.StartInstancesResult;
import com.ksc.kec.model.StopInstancesRequest;
import com.ksc.kec.model.StopInstancesResult;
import com.ksc.kec.model.TerminateInstancesRequest;
import com.ksc.kec.model.TerminateInstancesResult;

/**
 * 云主机操作
 * 
 * @author ymh
 * @时间 2019年9月24日
 */
@Provider("bocloud.cmp.kingcloud")
public class KingCloudServerProvider extends KingCloudProvider implements ServerProvider {

    private Butler butler;

    private static final String INSTANCESSET = "InstancesSet";

    public KingCloudServerProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    private static Logger logger = LoggerFactory.getLogger(KingCloudServerProvider.class);
    private BeanConvertor<ServerModel, JSONObject> serverConvertor = new ServerConvertor();
    private BeanConvertor<VolumeModel, LocalVolume> localVolumeConvertor = new LocalVolumeConvertor();
    private static final String ENDPOINT = KingCloudConstants.KEC_ENDPOINT;
    private static final String VERSION = KingCloudConstants.VERSION_20160304;
    private static final String SREVICE_NAME = KingCloudConstants.KEC;

    @Override
    public BocloudResult list() {
        try {
            String action = "DescribeInstances";
            Map<String, String> parameters = new HashMap<>();
            String call = call(ENDPOINT, parameters, action, VERSION, SREVICE_NAME, HttpMethod.GET);
            JSONObject parseObject = JSONObject.parseObject(call, JSONObject.class);
            String instancesSet = parseObject.getString(INSTANCESSET);
            if (null == instancesSet) {
                return new BocloudResult(true, "no instances！");
            }
            List<JSONObject> servers = JSONObject.parseArray(instancesSet, JSONObject.class);
            List<ServerModel> convertList = serverConvertor.convertList(servers);
            for (ServerModel serverModel : convertList) {
                serverModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询成功！");
        } catch (Exception e) {
            logger.error(e.getMessage());
            return new BocloudResult(false, "查询云主机失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult list(String hostId) {
        return null;
    }

    @Override
    public BocloudResult create(ServerModel server) {
        try {
            String action = "RunInstances";
            Map<String, String> parameters = new HashMap<>();
            parameters.put("MaxCount", "1");
            parameters.put("MinCount", "1");
            parameters.put("ImageId", server.getImageUuid());
            parameters.put("InstanceType", server.getFlavorUuid());
            // 本地硬盘大小
            if (null != server.getLocalVolumeSize() && server.getLocalVolumeSize() > 0) {
                parameters.put("DataDiskGb", String.valueOf(server.getLocalVolumeSize()));
            }

            // 系统盘大小，当前仅N2、N3机型支持调整大小； 否则系统默认本地硬盘 20G
            if (null != server.getSystemDiskSize() && server.getSystemDiskSize() > 0) {
                JSONObject systemDisk = new JSONObject();
                systemDisk.put("DiskType",
                        server.getSystemDiskCategory() == null ? "SSD3.0" : server.getSystemDiskCategory());
                systemDisk.put("DiskSize", server.getSystemDiskSize());
                parameters.put("SystemDisk", systemDisk.toJSONString());
            }
            String dataDisk = server.getDataDisks();
            @SuppressWarnings("unchecked")
            HashMap<String, String> dataDisks = JSONObject.parseObject(dataDisk, HashMap.class);
            if (null != dataDisks && dataDisks.keySet().size() > 0) {
                /**
                 * DataDisk.n.Type ===== 数据盘（云盘）的类型，数据盘n的类型，n 的取值范围为 [1,
                 * 8]。只支持I2、I2联网增强、N1、N2、N3、S3和I3。DataDisk.n.Type与DataDisk.n.Size必须都填写才有效。
                 * 
                 * DataDisk.n.Size ===== 第 n 个数据盘（云盘）的容量大小，n 的取值范围为 [1,
                 * 8]。只支持I2、I2联网增强、N1、N2、N3、S3和I3。DataDisk.n.Type与DataDisk.n.Size必须都填写才有效。
                 * 
                 * DataDisk.n.SnapshotId
                 * 
                 * DataDisk.n.DeleteWithInstance 数据盘是否随实例释放
                 */
                Set<Entry<String, String>> entrySet = dataDisks.entrySet();
                for (Entry<String, String> entry : entrySet) {
                    parameters.put(entry.getKey(), entry.getValue());
                }
            }
            parameters.put("SubnetId", server.getSubnetUuid());
            if (null != server.getPassword()) {
                parameters.put("InstancePassword", server.getPassword());
            }
            /**
             * Monthly(包年包月）、Daily（按日月结)、 HourlyInstantSettlement（按小时实时结算）
             */
            parameters.put("ChargeType", server.getInstanceChargeType());
            /**
             * 实例绑定的安全组，目前仅支持绑定一个安全组
             */
            List<String> securityGroups = server.getSecurityGroups();
            if (null == securityGroups || securityGroups.size() < 1) {
                return new BocloudResult(false, "创建云主机失败，缺失安全组信息！");
            }
            parameters.put("SecurityGroupId", securityGroups.get(0));
            /**
             * 私有IP地址，指定子网IP地址范围内的任意有效值，代表实例的主IP地址，只能选择一个，绑定到主网卡；如果未指定该参数，系统自动从有效地址池中随机选取一个
             */
            String privateIps = server.getPrivateIps();
            if (StringUtils.isNotEmpty(privateIps)) {
                parameters.put("PrivateIpAddress", privateIps);
            }
            /**
             * 实例名称，如果未指定，则自动生成，形如KSC-IN-[A-Z0-9]{10}
             */
            parameters.put("InstanceName", server.getName());
            String[] keypairIds = server.getKeypairIds();
            if (null != keypairIds && keypairIds.length > 0) {
                for (int i = 0; i < keypairIds.length; i++) {
                    parameters.put("KeyId." + i, keypairIds[i]);
                }
            }
            String call = call(ENDPOINT, parameters, action, VERSION, SREVICE_NAME, HttpMethod.POST);
            JSONObject createResult = JSONObject.parseObject(call, JSONObject.class);
            if (null == createResult) {
                return new BocloudResult(false, "云主机创建失败！");
            }
            String instancesSet = createResult.getString(INSTANCESSET);
            List<JSONObject> parseArray = JSONObject.parseArray(instancesSet, JSONObject.class);
            if (null == parseArray || parseArray.size() == 0) {
                return new BocloudResult(false, "云主机创建失败！");
            }
            JSONObject jsonObject = parseArray.get(0);
            String instanceId = jsonObject.getString("InstanceId");
            BocloudResult instanceProcess = instanceProcess(instanceId);
            ServerModel serverModel = (ServerModel) instanceProcess.getData();
            BocloudResult bocloudResult = volumesDetail(serverModel.getInstanceId());
            if (bocloudResult.isSuccess()) {
                @SuppressWarnings("unchecked")
                List<VolumeModel> volumeModels = (List<VolumeModel>) bocloudResult.getData();
                if (null != volumeModels && volumeModels.size() > 0) {
                    serverModel.setDiskDevices(volumeModels);
                }
            }
            return new BocloudResult(true, serverModel, "创建成功！");
        } catch (Exception e) {
            logger.error("新建云主机失败：" + e.getMessage());
            return new BocloudResult(false, "新建云主机失败：" + e.getMessage());
        }
    }

    /**
     * 根据云主机ID查询 云硬盘
     * 
     * @return
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    private BocloudResult volumesDetail(String instanceId) throws Exception {
        // 同步云硬盘
        List<VolumeModel> volumeModels = new ArrayList<VolumeModel>();
        KingCloudVolumeProvider volumeProvider = new KingCloudVolumeProvider(butler);
        Map<String, String> parameters = new HashMap<>();
        parameters.put("InstanceId", instanceId);
        BocloudResult volumeList = volumeProvider.listVolumes(parameters);
        if (volumeList.isFailed()) {
            return new BocloudResult(false, volumeList.getMessage());
        }
        List<VolumeModel> cloudVolumeModels = (List<VolumeModel>) volumeList.getData();
        if (null != cloudVolumeModels && cloudVolumeModels.size() > 0) {
            volumeModels.addAll(cloudVolumeModels);
        }
        // 同步本地硬盘
        List<VolumeModel> describeLocalVolumes = describeLocalVolumes(instanceId);
        if (null != describeLocalVolumes) {
            volumeModels.addAll(describeLocalVolumes);
        }
        return new BocloudResult(true, volumeModels, "查询成功！");
    }

    /**
     * 根据云主机ID查询 本地硬盘
     * 
     * @param instanceId
     * @return
     * @throws Exception
     */
    private List<VolumeModel> describeLocalVolumes(String instanceId) throws Exception {
        String action = "DescribeLocalVolumes";
        String endpoint = "http://kec.api.ksyun.com";
        String serviceName = "kec";
        String version = "2016-03-04";
        Map<String, String> parameters = new HashMap<String, String>();
        parameters.put("InstanceId", instanceId);
        String call = call(endpoint, parameters, action, version, serviceName, HttpMethod.GET);
        JSONObject createResult = JSONObject.parseObject(call, JSONObject.class);
        if (null == createResult) {
            return null;
        }
        String instancesSet = createResult.getString("LocalVolumeSet");
        List<LocalVolume> localVolumes = JSONObject.parseArray(instancesSet, LocalVolume.class);
        if (null == localVolumes || localVolumes.size() == 0) {
            return null;
        }
        List<VolumeModel> convertList = localVolumeConvertor.convertList(localVolumes);
        for (VolumeModel volumeModel : convertList) {
            volumeModel.setRegionId(this.getRegionId());
        }
        return convertList;
    }

    private BocloudResult instanceProcess(String instanceId) {
        ServerModel serverModel = null;
        try {
            BocloudResult detail = new BocloudResult(false, "查询云主机失败！");
            Boolean createResult = false;
            String message = "查询失败或超时！";
            Integer flag = 10 * 10; // 5 min
            while (flag > 0) {
                detail = this.detail(instanceId);
                serverModel = (ServerModel) detail.getData();
                if (null != serverModel && ("active".equalsIgnoreCase(serverModel.getStatus())
                        || "error".equalsIgnoreCase(serverModel.getStatus())
                        || "exception".equalsIgnoreCase(serverModel.getStatus()))) {
                    flag = 0;
                    createResult = true;
                    message = "查询云主机成功！";
                    break;
                } else {
                    try {
                        Thread.sleep(6000);
                    } catch (Exception e) {
                        logger.error("线程睡觉异常 : " + e);
                    }
                    flag--;
                }
            }
            return new BocloudResult(createResult, serverModel, message);
        } catch (Exception e) {
            logger.error("查询云主机失败：" + e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }

    }

    @Override
    public BocloudResult modify(ServerModel server) {
        try {
            ModifyInstanceAttributeRequest modifyInstanceAttributeRequest = new ModifyInstanceAttributeRequest();
            modifyInstanceAttributeRequest.setInstanceId(server.getInstanceId());
            modifyInstanceAttributeRequest.setInstanceName(server.getName());
            ModifyInstanceAttributeResult modifyInstanceAttribute = getKecClient()
                    .modifyInstanceAttribute(modifyInstanceAttributeRequest);
            return new BocloudResult(true, modifyInstanceAttribute.getReturn(), "云主机修改成功！");
        } catch (Exception e) {
            logger.error("云主机修改失败：" + e.getMessage());
            return new BocloudResult(false, "云主机修改失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult migrate(ServerModel server) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult migrate(OSMigrateModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult remove(String instanceId, String name) {
        try {
            TerminateInstancesRequest terminateInstancesRequest = new TerminateInstancesRequest();
            terminateInstancesRequest.withInstanceIds(new String[] { instanceId });
            TerminateInstancesResult terminateInstances = getKecClient().terminateInstances(terminateInstancesRequest);
            return new BocloudResult(true, terminateInstances.getInstancesSet(), "云主机删除成功！");
        } catch (Exception e) {
            logger.error("云主机删除失败：" + e.getMessage());
            return new BocloudResult(false, "云主机删除失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String instanceId) {
        try {
            String action = "DescribeInstances";
            Map<String, String> parameters = new HashMap<String, String>();
            parameters.put("InstanceId", instanceId);
            String call = call(ENDPOINT, parameters, action, VERSION, SREVICE_NAME, HttpMethod.GET);
            JSONObject parseObject = JSONObject.parseObject(call, JSONObject.class);
            String instancesSet = parseObject.getString(INSTANCESSET);
            if (null == instancesSet) {
                return new BocloudResult(true, "云主机不存在！");
            }
            List<JSONObject> servers = JSONObject.parseArray(instancesSet, JSONObject.class);
            if (null == servers || servers.size() == 0) {
                return new BocloudResult(true, "云主机不存在！");
            }
            JSONObject jsonObject = servers.get(0);
            ServerModel convertModel = serverConvertor.convertModel(jsonObject);
            convertModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, convertModel, "查询成功！");
        } catch (Exception e) {
            logger.error("查询云主机失败：" + e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult start(String instanceId) {
        try {
            StartInstancesRequest startInstancesRequest = new StartInstancesRequest();
            startInstancesRequest.withInstanceIds(new String[] { instanceId });
            StartInstancesResult startInstances = getKecClient().startInstances(startInstancesRequest);
            return new BocloudResult(true, startInstances.getInstancesSet(), "云主机启动成功！");
        } catch (Exception e) {
            logger.error("云主机启动失败：" + e.getMessage());
            return new BocloudResult(false, "云主机启动失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult stop(String instanceId) {
        try {
            StopInstancesRequest stopInstancesRequest = new StopInstancesRequest();
            stopInstancesRequest.withInstanceIds(new String[] { instanceId });
            StopInstancesResult stopInstances = getKecClient().stopInstances(stopInstancesRequest);
            return new BocloudResult(true, stopInstances.getInstancesSet(), "云主机停止成功！");
        } catch (Exception e) {
            logger.error("云主机停止失败：" + e.getMessage());
            return new BocloudResult(false, "云主机停止失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult reboot(String instanceId) {
        try {
            RebootInstancesRequest rebootInstancesRequest = new RebootInstancesRequest();
            rebootInstancesRequest.withInstanceIds(new String[] { instanceId });
            RebootInstancesResult rebootInstances = getKecClient().rebootInstances(rebootInstancesRequest);
            return new BocloudResult(true, rebootInstances.getInstancesSet(), "云主机重启成功！");
        } catch (Exception e) {
            logger.error("云主机重启失败：" + e.getMessage());
            return new BocloudResult(false, "云主机重启失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult softReboot(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult pause(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult unPause(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult suspend(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult resume(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult joinSGroup(String instanceId, String group) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult leaveSGroup(String instanceId, String group) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult vncConsole(String vncType, String serverId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult changePassword(String instanceId, String password) {
        try {
            // 修改实例开机密码需要关闭实例
            ModifyInstanceAttributeRequest modifyInstanceAttributeRequest = new ModifyInstanceAttributeRequest();
            modifyInstanceAttributeRequest.setInstanceId(instanceId);
            modifyInstanceAttributeRequest.setInstancePassword(password);
            ModifyInstanceAttributeResult modifyInstanceAttribute = getKecClient()
                    .modifyInstanceAttribute(modifyInstanceAttributeRequest);
            return new BocloudResult(true, modifyInstanceAttribute.getReturn(), "密码修改成功！");
        } catch (Exception e) {
            logger.error("密码修改失败：" + e.getMessage());
            return new BocloudResult(false, "密码修改失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult attachNetwork(ServerModel server) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult detachNetwork(String instanceId, String portUuid) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId) {
        try {
            ModifyInstanceTypeRequest modifyInstanceTypeRequest = new ModifyInstanceTypeRequest();
            modifyInstanceTypeRequest.setInstanceId(instanceId);
            modifyInstanceTypeRequest.setInstanceType(instanceId);
            // TODO 后续需要处理 DataDiskGb 的值 数据卷容量，单位GB，数据卷只能扩容或者保持不变，且不能低于 实例套餐类型定义
            // 的最小值
            modifyInstanceTypeRequest.setDataDiskGb(0);
            ModifyInstanceTypeResult modifyInstanceType = getKecClient().modifyInstanceType(modifyInstanceTypeRequest);
            return new BocloudResult(true, modifyInstanceType.getReturn(), "规格变更成功！");
        } catch (Exception e) {
            logger.error("规格变更成功：" + e.getMessage());
            return new BocloudResult(false, "规格变更成功：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult confirmResize(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult revertResize(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean MemHotEnable) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getTicket(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult createInResPool(ServerModel server) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult mountToolsInstaller(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult unmountToolsInstaller(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult upgradeToolsTask(String instanceId, String options) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getNetCard(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getVmIps(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult markAsTemplate(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getMetric(String type) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult vmrcConsole(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult migrate(H3cMigrateModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult create(H3cServerModel server) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getJnlpXml(Long vmId, String address) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult resetInstance(ServerModel serverModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult bindKeypair(ServerModel serverModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult unbindKeypair(ServerModel serverModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
        // TODO Auto-generated method stub
        return null;
    }

}