package com.bocloud.cmp.provider.storage;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.kingconvertor.SnapshotConvertor;
import com.bocloud.cmp.kingconvertor.VolumeConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.provider.KingCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.util.KingCloudConstants;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.ksc.HttpMethod;
import com.ksc.kec.model.CreateLocalVolumeSnapshotRequest;
import com.ksc.kec.model.CreateLocalVolumeSnapshotResult;
import com.ksc.kec.model.LocalVolumeSnapshot;

/**
 * volume 操作
 * 
 * @author ymh
 * @时间 2019年9月24日
 */
@Provider("bocloud.cmp.kingcloud")
public class KingCloudVolumeProvider extends KingCloudProvider implements VolumeProvider {

    public KingCloudVolumeProvider(Butler butler) {
        super(butler);
    }

    private static String endpoint = KingCloudConstants.EBS_ENDPOINT;
    private static String serviceName = KingCloudConstants.EBS;
    private static String version = KingCloudConstants.VERSION_20160304;
    private static Logger logger = LoggerFactory.getLogger(KingCloudVolumeProvider.class);
    private BeanConvertor<SnapshotModel, LocalVolumeSnapshot> snapshotConvertor = new SnapshotConvertor();
    private BeanConvertor<VolumeModel, JSONObject> volumeConvertor = new VolumeConvertor();

    @Override
    public BocloudResult list() {
        return listVolumes(null);
    }

    public BocloudResult listVolumes(Map<String, String> parameters) {
        try {
            String action = "DescribeVolumes";
            String call = call(endpoint, parameters, action, version, serviceName, HttpMethod.GET);
            JSONObject parseObject = JSONObject.parseObject(call);
            String volumes = parseObject.getString("Volumes");
            if (null == volumes) {
                return new BocloudResult(true, "no volumes!");
            }
            List<JSONObject> volumeArray = JSONObject.parseArray(volumes, JSONObject.class);
            List<VolumeModel> convertList = volumeConvertor.convertList(volumeArray);
            for (VolumeModel volumeModel : convertList) {
                volumeModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询云硬盘列表成功！");
        } catch (Exception e) {
            logger.error("查询云硬盘列表失败：" + e.getMessage());
            return new BocloudResult(false, "查询云硬盘列表失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult create(VolumeModel volume) {
        try {
            String action = "CreateVolume";
            Map<String, String> parameters = new HashMap<String, String>();
            parameters.put("VolumeName", "VolumeName");
            parameters.put("VolumeType", "SATA2.0");
            parameters.put("VolumeDesc", "VolumeDesc");
            parameters.put("Size", "10");
            /**
             * Monthly（预付费，包年包月）HourlyInstantSettlement（后付费，按小时实时结算）
             * Daily（后付费，按日月结）
             */
            parameters.put("ChargeType", "HourlyInstantSettlement");
            parameters.put("AvailabilityZone", "cn-shanghai-2b");
            String call = call(endpoint, parameters, action, version, serviceName, HttpMethod.GET);
            JSONObject parseObject = JSONObject.parseObject(call);
            if (null == parseObject) {
                return new BocloudResult(false, "创建失败");
            }
            String volumeId = parseObject.getString("VolumeId");
            return new BocloudResult(true, this.detail(volumeId).getData(), "云硬盘创建成功！");
        } catch (Exception e) {
            logger.error("创建硬盘失败：" + e.getMessage());
            return new BocloudResult(false, "创建硬盘失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(VolumeModel volume) {
        try {
            String action = "ModifyVolume";
            Map<String, String> parameters = new HashMap<String, String>();
            parameters.put("VolumeId", volume.getVolumeUuid());
            parameters.put("VolumeName", volume.getName());
            parameters.put("VolumeDesc", volume.getRemark());
            if (null != volume.getDeleteWithInstance()) {
                parameters.put("DeleteWithInstance", String.valueOf(volume.getDeleteWithInstance()));
            }
            String call = call(endpoint, parameters, action, version, serviceName, HttpMethod.GET);
            return new BocloudResult(true, call, "编辑云硬盘成功！");
        } catch (Exception e) {
            logger.error("编辑云硬盘失败：" + e.getMessage());
            return new BocloudResult(false, "编辑云硬盘失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String volumeId) {
        try {
            String action = "DeleteVolume";
            Map<String, String> parameters = new HashMap<String, String>();
            parameters.put("VolumeId", volumeId);
            call(endpoint, parameters, action, version, serviceName, HttpMethod.GET);
            return new BocloudResult(true, "删除云硬盘成功！");
        } catch (Exception e) {
            logger.error("删除云硬盘失败：" + e.getMessage());
            return new BocloudResult(false, "删除云硬盘失败：" + e.getMessage());
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    public BocloudResult detail(String volumeId) {
        try {
            Map<String, String> parameters = new HashMap<>();
            parameters.put("VolumeId", volumeId);
            BocloudResult listVolumes = listVolumes(parameters);
            List<VolumeModel> volumeModels = (List<VolumeModel>) listVolumes.getData();
            if (null == volumeModels || volumeModels.size() == 0) {
                return new BocloudResult(false, "云硬盘不存在！");
            }
            VolumeModel volumeModel = volumeModels.get(0);
            volumeModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, volumeModel, "查询成功！");
        } catch (Exception e) {
            logger.error("查询云硬盘失败：" + e.getMessage());
            return new BocloudResult(false, "查询云硬盘失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        try {
            String volumeType = snapshotModel.getVolumeType();
            if (null == volumeType) {
                return new BocloudResult(false, "快照创建失败！无效类型");
            }
            if ("cloud".equalsIgnoreCase(volumeType)) {
                return createCloudVolumeSnapshot(snapshotModel);
            }
            if ("local".equalsIgnoreCase(volumeType)) {
                return createLocalVolumeSnapshot(snapshotModel);
            }
            return new BocloudResult(false, "快照创建失败！无效类型");
        } catch (Exception e) {
            logger.error("创建快照失败：" + e.getMessage());
            return new BocloudResult(false, "创建快照失败：" + e.getMessage());
        }
    }

    public BocloudResult createCloudVolumeSnapshot(SnapshotModel snapshotModel) {
        try {
            return new BocloudResult(true, "创建成功！");
        } catch (Exception e) {
            logger.error("创建快照失败：" + e.getMessage());
            return new BocloudResult(false, "创建快照失败：" + e.getMessage());
        }
    }

    private BocloudResult createLocalVolumeSnapshot(SnapshotModel snapshotModel) {
        try {
            CreateLocalVolumeSnapshotRequest createLocalVolumeSnapshotRequest = new CreateLocalVolumeSnapshotRequest();
            createLocalVolumeSnapshotRequest.setLocalVolumeId(snapshotModel.getVolumeUuid());
            createLocalVolumeSnapshotRequest.setLocalVolumeSnapshotDesc(snapshotModel.getRemark());
            createLocalVolumeSnapshotRequest.setLocalVolumeSnapshotName(snapshotModel.getName());
            CreateLocalVolumeSnapshotResult createLocalVolumeSnapshot = getKecClient()
                    .createLocalVolumeSnapshot(createLocalVolumeSnapshotRequest);
            String localVolumeSnapshotId = createLocalVolumeSnapshot.getLocalVolumeSnapshotId();
            BocloudResult snapshotDetail = snapshotDetail(localVolumeSnapshotId);
            if (snapshotDetail.isFailed()) {
                return snapshotDetail;
            }
            LocalVolumeSnapshot snapshot = (LocalVolumeSnapshot) snapshotDetail.getData();
            SnapshotModel convertModel = snapshotConvertor.convertModel(snapshot);
            convertModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, convertModel, "创建成功！");
        } catch (Exception e) {
            logger.error("创建失败：" + e.getMessage());
            return new BocloudResult(false, "创建失败：" + e.getMessage());
        }
    }

    private BocloudResult snapshotDetail(String localVolumeSnapshotId) {
        try {
            if (null == localVolumeSnapshotId) {
                return new BocloudResult(false, "快照不存在！");
            }
            String action = "DescribeLocalVolumeSnapshots";
            String endpoint = "http://kec.api.ksyun.com";
            String serviceName = "kec";
            String version = "2016-03-04";
            Map<String, String> parameters = new HashMap<String, String>();
            parameters.put("LocalVolumeSnapshotId", localVolumeSnapshotId);
            Boolean createResult = false;
            String message = "查询失败或超时！";
            Integer flag = 10 * 10; // 5 min
            LocalVolumeSnapshot snapshot = null;
            while (flag > 0) {
                String call = call(endpoint, parameters, action, version, serviceName, HttpMethod.GET);
                JSONObject result = JSONObject.parseObject(call);
                String localVolumeSnapshotSet = result.getString("LocalVolumeSnapshotSet");
                List<LocalVolumeSnapshot> snapshots = JSONObject.parseArray(localVolumeSnapshotSet,
                        LocalVolumeSnapshot.class);
                if (null == snapshots || snapshots.size() == 0) {
                    flag = 0;
                    break;
                }
                snapshot = snapshots.get(0);
                System.out.println(JSON.toJSONString(snapshot));
                if (null != snapshot && null != snapshot.getState() && "active".equalsIgnoreCase(snapshot.getState())) {
                    flag = 0;
                    createResult = true;
                    message = "查询云主机成功！";
                    break;
                } else {
                    try {
                        Thread.sleep(6000);
                    } catch (Exception e) {
                        logger.error("线程睡觉异常 : " + e);
                    }
                    flag--;
                }
            }
            return new BocloudResult(createResult, snapshot, message);
        } catch (Exception e) {
            logger.error("查询快照失败：" + e.getMessage());
            return new BocloudResult(false, "查询快照失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult attach(String serverId, String volumeId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult attach(String serverId, String mountpoint, String volumeId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult detach(String serverId, String volumeId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult resetStatus(String volumeId, String status) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult uploadToImage(String volumeId, ImageModel imageModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult extend(String volumeId, Integer newSize) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult list(String hostId, String storagePoolName) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult remove(H3cVolumeModel volume) {
        // TODO Auto-generated method stub
        return null;
    }

}
