package com.bocloud.cmp.provider.compute;

import io.jsonwebtoken.lang.Assert;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.ManageOneApiName;
import com.bocloud.cmp.common.enums.ManageOneApiType;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.manageoneconvertor.PortConvertor;
import com.bocloud.cmp.manageoneconvertor.ServerConvertor;
import com.bocloud.cmp.manageoneconvertor.StoragePoolConvertor;
import com.bocloud.cmp.model.AddressBean;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkBean;
import com.bocloud.cmp.model.OSMigrateModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.storage.ManageOneVolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.http.CommonResultBuilder;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;

/**
 * 华为manage one云主机操作driver
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年9月17日
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneServerProvider extends ManageOneProvider implements ServerProvider {

	private static final Logger logger = LoggerFactory.getLogger(ManageOneServerProvider.class);
	private BeanConvertor<ServerModel, JSONObject> convertor = new ServerConvertor();
	private BeanConvertor<PortModel, JSONObject> portConvertor = new PortConvertor();
	private ManageOneVolumeProvider volumeProvider = null;
	private Butler butler;

	/**
	 * @param butler
	 */
	public ManageOneServerProvider(Butler butler) {
		super(butler);
		this.butler = butler;
		volumeProvider = new ManageOneVolumeProvider(butler);
	}

	/**
	 * 查询存储容量
	 * 
	 * @return
	 */
	public BocloudResult storageCapacity() {
		try {
			JSONObject params = new JSONObject();
			params.put("detail", true);
			Result result = this.doGet("/scheduler-stats/get_pools", params, ManageOneApiName.evsv2,
					ManageOneApiType.volumev2);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			logger.info("storageCapacity : " + object.toJSONString());
			if (null == object) {
				return new BocloudResult(false, "云主机列表为空");
			}
			logger.info("storageCapacity : " + object.toJSONString());
			List<JSONObject> objects = JSONArray.parseArray(object.getString("pools"), JSONObject.class);
			StoragePoolConvertor convertor = new StoragePoolConvertor();
			return new BocloudResult(true, convertor.convertList(objects), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult list() {
		try {
			Result result = this.doGet("/servers", null, ManageOneApiName.ecs, ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "云主机列表为空");
			}
			logger.info("servers : " + object.toJSONString());
			List<JSONObject> objects = JSONArray.parseArray(object.getString("servers"), JSONObject.class);
			List<ServerModel> models = new ArrayList<>();
			for (JSONObject obj : objects) {
				BocloudResult detailResult = this.detail(obj.getString("id"));
				if (detailResult.isSuccess()) {
					ServerModel vm = JSONObject.parseObject(JSONObject.toJSONString(detailResult.getData()),
							ServerModel.class);
					models.add(vm);
				}
			}
			return new BocloudResult(true, models, "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}
	
	@Override
	public BocloudResult create(ServerModel server) {
		try {
			JSONObject serverParam = new JSONObject();
			serverParam.put("imageRef", server.getImageUuid());
			serverParam.put("flavorRef", server.getFlavorUuid());
			serverParam.put("name", server.getName());
			// 可用域
			serverParam.put("availability_zone", server.getZone());
			if(!StringUtils.isEmpty(server.getPassword())) {
				serverParam.put("adminPass", server.getPassword());
			} else {
				serverParam.put("key_name", server.getKeypairName());
			}
			serverParam.put("vpcid", server.getRouterUuid());
			
			// 设置网卡配置
			List<NetworkBean> networkBeans = server.getNetworkBeans();
			List<JSONObject> nics = new ArrayList<>();
			if (ListTool.isEmpty(networkBeans)) {
				return new BocloudResult(false, "至少需要一块网卡");
			}
			for (NetworkBean networkBean : networkBeans) {
				List<AddressBean> addresses = networkBean.getAddresses();
				if (ListTool.isEmpty(addresses)) {
					continue;
				}
				for (AddressBean addresse : addresses) {
					JSONObject network = new JSONObject();
					network.put("subnet_id", addresse.getNetworkUuid());
					if (!StringUtils.isEmpty(addresse.getAddress())) {
						network.put("ip_address", addresse.getAddress());
					}
					nics.add(network);
				}
			}
			serverParam.put("nics", nics);
			JSONObject rootVolume = new JSONObject();
			Integer rootSize = server.getRootSize();
			if(null != rootSize && rootSize > 0) {
				rootVolume.put("size", rootSize);
			}
			String rootVolumeType = server.getRootVolumeType();
			if(!StringUtils.isEmpty(rootVolumeType)) {
				rootVolume.put("volumetype", rootVolumeType);
			} else {
				rootVolume.put("volumetype", "SATA");
			}
			serverParam.put("root_volume", rootVolume);

			List<JSONObject> secgList = new ArrayList<>();
			List<String> groups = server.getSecurityGroups();
			if (!ListTool.isEmpty(groups)) {
				for (String group : groups) {
					JSONObject secg = new JSONObject();
					secg.put("id", group);
					secgList.add(secg);
				}
			}
			// 设置安全组
			serverParam.put("security_groups", secgList);
			
			JSONObject params = new JSONObject();
			params.put("server", serverParam);
			System.out.println(params.toJSONString());
			Result result = this.doPostForV1("/cloudservers", params, ManageOneApiName.ecs, ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject createdResult = JSONObject.parseObject(result.getMessage());
			if (null == createdResult) {
				return new BocloudResult(false, "创建失败:接口返回值异常");
			}
			BocloudResult waitForJobs = waitForJobs(createdResult.getString("job_id"));
			if(waitForJobs.isFailed()) {
				return waitForJobs;
			}
			String serverId = waitForJobs.getData().toString();
			ServerModel model = buildServerDetail(serverId);
			if (null == model) {
				this.remove(serverId, null);
				return new BocloudResult(false, "云主机创建失败");
			}
			// 查询云主机端口
			model.setPorts(listPortByInstanceId(serverId));
			return new BocloudResult(true, model, "创建成功");
			
			
			
//			JSONObject vmBuild = new JSONObject();
//			vmBuild.put("flavorRef", server.getFlavorUuid());
//			vmBuild.put("name", server.getName());
//			List<JSONObject> personality = new ArrayList<>();
//			JSONObject personal = new JSONObject();
//			personal.put("path", "/etc/motd");
//			personal.put("contents",
//					new String(Base64.encode("Welcome to the new VM! Restricted access only".getBytes())));
//			personality.add(personal);
//			vmBuild.put("personality", personality);
//			String osCategory = server.getOsCategory();
//			if (StringUtils.isEmpty(osCategory)) {
//				osCategory = OSCategory.LINUX.name();
//			}
//			// 使用支持Cloud-init或Cloudbase-init功能的镜像创建云服务器时
//			// 对于Linux弹性云服务器，如果需要注入密码，只能使用user_data进行注入
//			// 对于Windows弹性云服务器，如果需要注入密码，只能通过元数据adminPass进行注入
//			if (OSCategory.LINUX.name().equalsIgnoreCase(osCategory)) {
//				StringBuffer buffer = new StringBuffer();
//				buffer.append("#cloud-config\n");
//				buffer.append("hostname: " + server.getName() + "\n");
//				String password = server.getPassword();
//				if (!StringUtils.isEmpty(password)) {
//					buffer.append("password: " + server.getPassword() + "\n");
//				}
//				buffer.append("chpasswd: { expire: False }\n");
//				buffer.append("ssh_pwauth: True");
//				vmBuild.put("user_data", new String(Base64.encode(buffer.toString().getBytes())));
//			} else {
//				vmBuild.put("adminPass", server.getPassword());
//			}
//			String sourceValidUuid = server.getSourceValidUuid();
//			String sourceSnapUuid = server.getSourceSnapUuid();
//			List<JSONObject> blockDevices = new ArrayList<>();
//			if (!StringUtils.isEmpty(sourceValidUuid)) {
//				JSONObject sourceVol = new JSONObject();
//				sourceVol.put("source_type", "volume");
//				sourceVol.put("uuid", sourceValidUuid);
//				sourceVol.put("delete_on_termination", "true");
//				sourceVol.put("boot_index", "0");
//				blockDevices.add(sourceVol);
//				vmBuild.put("block_device_mapping_v2", blockDevices);
//			} else if (!StringUtils.isEmpty(sourceSnapUuid)) {
//				JSONObject sourceSnap = new JSONObject();
//				sourceSnap.put("source_type", "snapshot");
//				sourceSnap.put("uuid", sourceSnapUuid);
//				sourceSnap.put("delete_on_termination", "true");
//				sourceSnap.put("boot_index", "0");
//				blockDevices.add(sourceSnap);
//				vmBuild.put("block_device_mapping_v2", blockDevices);
//			} else if (null != server.getBootFromVolume() && server.getBootFromVolume()) {
//				JSONObject source = new JSONObject();
//				source.put("device_name", "/dev/vda");
//				source.put("source_type", "image");
//				source.put("destination_type", "volume");
//				source.put("volume_size", server.getRootSize());
//				source.put("uuid", server.getImageUuid());
//				source.put("delete_on_termination", "true");
//				source.put("boot_index", "0");
//				blockDevices.add(source);
//				vmBuild.put("block_device_mapping_v2", blockDevices);
//			} else {
//				vmBuild.put("imageRef", server.getImageUuid());
//			}
//			// 设置密钥
//			if (!StringUtils.isEmpty(server.getKeypairName())) {
//				vmBuild.put("key_name", server.getKeypairName());
//			}
//			// 设置网卡配置
//			List<NetworkBean> networkBeans = server.getNetworkBeans();
//			List<JSONObject> networks = new ArrayList<>();
//			if (ListTool.isEmpty(networkBeans)) {
//				return new BocloudResult(false, "至少需要一块网卡");
//			}
//			for (NetworkBean networkBean : networkBeans) {
//				List<AddressBean> addresses = networkBean.getAddresses();
//				if (ListTool.isEmpty(addresses)) {
//					return new BocloudResult(false, "至少需要一块网卡");
//				}
//				for (AddressBean addresse : addresses) {
//					JSONObject network = new JSONObject();
//					network.put("uuid", addresse.getNetworkUuid());
//					if (!StringUtils.isEmpty(addresse.getAddress())) {
//						network.put("fixed_ip", addresse.getAddress());
//					}
//					networks.add(network);
//				}
//			}
//			vmBuild.put("networks", networks);
//			List<JSONObject> secgList = new ArrayList<>();
//			List<String> groups = server.getSecurityGroups();
//			if (!ListTool.isEmpty(groups)) {
//				for (String group : groups) {
//					JSONObject secg = new JSONObject();
//					secg.put("name", group);
//					secgList.add(secg);
//				}
//			}
//			// 设置安全组
//			vmBuild.put("security_groups", secgList);
//			// 可用域
//			vmBuild.put("availability_zone", server.getZone());
//			JSONObject params = new JSONObject();
//			params.put("server", vmBuild);
//			Result result = this.doPost("/servers", params, ManageOneApiName.ecs, ManageOneApiType.compute);
//			if (result.isFailed()) {
//				return new BocloudResult(false, result.getMessage());
//			}
//			JSONObject createdResult = JSONObject.parseObject(result.getMessage());
//			JSONObject vmObj = JSONObject.parseObject(createdResult.getString("server"));
//			if (null == vmObj) {
//				return new BocloudResult(false, "创建失败");
//			}
//			String serverId = vmObj.getString("id");
//			ServerModel model = waitForTask(serverId);
//			if (null == model) {
//				this.remove(serverId, null);
//				return new BocloudResult(false, "云主机创建失败");
//			}
//			// 查询云主机端口
//			model.setPorts(listPortByInstanceId(serverId));
//			return new BocloudResult(true, model, "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "创建失败 : " + e);
		}
	}
	
	public BocloudResult waitForJobs(String jobId) {
		try {
			Long startTime = System.currentTimeMillis();
			while (true) {
				Thread.sleep(5000);
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 15 * 60 * 1000) {
					return new BocloudResult(false, "任务执行超时：任务ID = " + jobId);
				}
				Result waitImageTask = this.waitImageTask(jobId);
				JSONObject createdResult = JSONObject.parseObject(waitImageTask.getMessage());
				if (null == createdResult) {
					return new BocloudResult(false, "获取任务异常：任务ID = " + jobId);
				}
				logger.info("jobId = " + jobId + " result = " + createdResult.toJSONString());
				JSONObject entitiesObject = createdResult.getJSONObject("entities");
				if (null == entitiesObject) {
					continue;
				}
				JSONArray subJobsArrays = entitiesObject.getJSONArray("sub_jobs");
				if (subJobsArrays.size() <= 0) {
					continue;
				}
				JSONObject entitiesDetail = JSON.parseObject(JSON.toJSONString(subJobsArrays.get(0)), JSONObject.class);
				if (null == entitiesDetail || null == entitiesDetail.getJSONObject("entities")) {
					continue;
				}
				String serverId = entitiesDetail.getJSONObject("entities").getString("server_id");
				if (StringUtils.isEmpty(serverId)) {
					continue;
				} else {
					return new BocloudResult(true, serverId, "任务执行成功");
				}
			}
		} catch (Exception e) {
			logger.error("任务执行失败：" + e);
		}
		return new BocloudResult(false, "获取任务异常：任务ID = " + jobId);
	}
	
	

	private List<PortModel> listPortByInstanceId(String instanceId) {
		// 查询云主机端口
		List<PortModel> portModels = new ArrayList<>();
		Result portResult = this.doGet("/v2.0/ports?device_id=" + instanceId, null, ManageOneApiName.vpc,
				ManageOneApiType.network);
		if (portResult.isSuccess()) {
			JSONObject object = JSONObject.parseObject(portResult.getMessage());
			List<JSONObject> ports = JSONArray.parseArray(object.getString("ports"), JSONObject.class);
			for (JSONObject port : ports) {
				portModels.add(portConvertor.convertModel(port));
			}
		}
		return portModels;
	}
	
	private ServerModel buildServerDetail(String serverId) {
		ServerModel model = null;
		BocloudResult result = this.detail(serverId);
		if (result.isFailed()) {
			return model;
		}
		model = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), ServerModel.class);
		if (null == model) {
			return model;
		}
		Result vmResult = this.doGet("/servers/" + serverId, null, ManageOneApiName.ecs, ManageOneApiType.compute);
		logger.info("vmResult : " + JSON.toJSONString(vmResult));
		if (vmResult.isSuccess() && !StringUtils.isEmpty(vmResult.getMessage())) {
			JSONObject object = JSONObject.parseObject(vmResult.getMessage());
			logger.info("server create success : " + object.toJSONString());
			JSONObject vmObj = object.getJSONObject("server");
			List<JSONObject> volumeIds = new ArrayList<>();
			List<VolumeModel> volumeModels = new ArrayList<>();
			if (vmObj.containsKey("os-extended-volumes:volumes_attached")
					&& !StringUtils.isEmpty(vmObj.getString("os-extended-volumes:volumes_attached"))) {
				volumeIds = JSONArray.parseArray(vmObj.getString("os-extended-volumes:volumes_attached"),
						JSONObject.class);
				for (JSONObject diskObj : volumeIds) {
					String volumeId = diskObj.getString("id");
					BocloudResult detail = volumeProvider.detail(volumeId);
					if (detail.isSuccess() && null != detail.getData()) {
						volumeModels.add((VolumeModel) detail.getData());
					}
				}
			}
			model.setDiskDevices(volumeModels);
		}
		return model;
	}

	private ServerModel waitForTask(String serverId) {
		ServerModel model = null;
		BocloudResult result = this.detail(serverId);
		if (result.isFailed()) {
			return model;
		}
		model = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), ServerModel.class);
		if (null == model) {
			return model;
		}
		Boolean flag = true;
		Long startTime = System.currentTimeMillis();
		while (flag) {
			try {
				Thread.sleep(7000);
			} catch (InterruptedException e) {
				logger.error("Get error message:", e);
				Thread.currentThread().interrupt();
			}
			result = this.detail(serverId);
			if (result.isFailed()) {
				flag = false;
				break;
			}
			model = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), ServerModel.class);
			if (null == model) {
				flag = false;
				break;
			}
			if ((!BocloudStatus.Server.BUILDING.name().equalsIgnoreCase(model.getStatus()))
					&& (!BocloudStatus.Server.EXCEPTION.name().equalsIgnoreCase(model.getStatus()))
					&& (!"STOPPED".equalsIgnoreCase(model.getStatus()))) {
				flag = false;
				break;
			}
			Long endTime = System.currentTimeMillis();
			// 等待三分钟
			if (endTime - startTime > 30 * 60 * 1000) {
				flag = false;
				break;
			}
		}
		Result vmResult = this.doGet("/servers/" + serverId, null, ManageOneApiName.ecs, ManageOneApiType.compute);
		logger.info("vmResult : " + JSON.toJSONString(vmResult));
		if (vmResult.isSuccess() && !StringUtils.isEmpty(vmResult.getMessage())) {
			JSONObject object = JSONObject.parseObject(vmResult.getMessage());
			logger.info("server create success : " + object.toJSONString());
			JSONObject vmObj = object.getJSONObject("server");
			List<JSONObject> volumeIds = new ArrayList<>();
			List<VolumeModel> volumeModels = new ArrayList<>();
			if (vmObj.containsKey("os-extended-volumes:volumes_attached")
					&& !StringUtils.isEmpty(vmObj.getString("os-extended-volumes:volumes_attached"))) {
				volumeIds = JSONArray.parseArray(vmObj.getString("os-extended-volumes:volumes_attached"),
						JSONObject.class);
				for (JSONObject diskObj : volumeIds) {
					String volumeId = diskObj.getString("id");
					BocloudResult detail = volumeProvider.detail(volumeId);
					if (detail.isSuccess() && null != detail.getData()) {
						volumeModels.add((VolumeModel) detail.getData());
					}
				}
			}
			model.setDiskDevices(volumeModels);
		}
		return model;
	}

	@Override
	public BocloudResult migrate(ServerModel model) {
		return new BocloudResult(false, "暂未开发！");
	}

	@SuppressWarnings("unchecked")
	@Override
	public BocloudResult migrate(OSMigrateModel model) {
		String serverId = model.getInstanceId();
		try {
			JSONObject params = new JSONObject();
			JSONObject object = new JSONObject();
			if (model.getLive()) {
				object.put("host", model.getHost());
				object.put("disk_over_commit", "False"); // 检测目的主机可用磁盘空间时，是否允许overcommit。
				object.put("block_migration", false); // 是否有本地磁盘，如果是则设置为True,否则设置为False。
				params.put("os-migrateLive", object);
			} else {
				String jsonStr = "{\"migrate\":null}";
				net.sf.json.JSONObject json = net.sf.json.JSONObject.fromObject(jsonStr);
				params.putAll(json);
			}
			Result result = doPost("/servers/" + serverId + "/action", params, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			if (model.getLive()) {
				return new BocloudResult(true, waitForResizeTask(serverId, "RUNNING"), "迁移成功！");
			} else {
				ServerModel waitForResizeTask = waitForResizeTask(serverId, "RESIZING");
				if (null == waitForResizeTask) {
					return new BocloudResult(false, "迁移失败！");
				}
				Thread.sleep(5000);
				String jsonStr = "{\"confirmResize\":null}";
				net.sf.json.JSONObject json = net.sf.json.JSONObject.fromObject(jsonStr);
				Result confirmResult = this.doPost("/servers/" + serverId + "/action", json, ManageOneApiName.ecs,
						ManageOneApiType.compute);
				if (confirmResult.isFailed()) {
					return new BocloudResult(false, result.getMessage());
				}
				return new BocloudResult(true, waitForResizeTask(serverId, "RUNNING"), "迁移成功！");
			}
		} catch (Exception e) {
			logger.error("migrate vm error:", e);
			return new BocloudResult(false, "migrate fail");
		}

	}

	@Override
	public BocloudResult snapshot(SnapshotModel snapshotModel) {
		try {
			JSONObject objectParams = new JSONObject();
			objectParams.put("name", snapshotModel.getName());
			objectParams.put("instance_snapshot", "true");
			JSONObject params = new JSONObject();
			params.put("createImage", objectParams);
			Result result = this.doPost("/servers/" + snapshotModel.getInstanceId() + "/action", params,
					ManageOneApiName.ecs, ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "快照创建失败！");
			}
			String jobId = object.getString("job_id");
			return waitSnapshot(jobId, snapshotModel.getInstanceId());
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}

	}

	/**
	 * 获取云主机的整机快照信息
	 * 
	 * @param snapshotModel
	 * @return
	 */
	public BocloudResult getServerSnapshot(String instanceId) {
		try {
			JSONObject params = new JSONObject();
			params.put("__snapshot_from_instance", instanceId);
			params.put("sort_key", "created_at");
			params.put("sort_dir", "desc");
			Result result = this.doGet("/v2/images", params, ManageOneApiName.ims, ManageOneApiType.image);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(true, "整机快照列表为空！");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("images"), JSONObject.class);
			if (ListTool.isEmpty(objects)) {
				return new BocloudResult(true, "整机快照列表为空！");
			}
			List<SnapshotModel> convertList = convertList(objects);
			return new BocloudResult(true, convertList, "查询成功！");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}

	}

	public List<SnapshotModel> convertList(List<? extends JSONObject> objects) {
		List<SnapshotModel> snapshotModels = new ArrayList<SnapshotModel>();
		for (JSONObject object : objects) {
			snapshotModels.add(convertSnapshotModel(object));
		}
		return snapshotModels;
	}

	/**
	 * 整机快照转换类
	 * 
	 * @param jsonObject
	 * @return
	 */
	public SnapshotModel convertSnapshotModel(JSONObject jsonObject) {
		SnapshotModel snapshot = new SnapshotModel();
		snapshot.setSnapshotUuid(jsonObject.getString("id"));
		snapshot.setSize(jsonObject.getInteger("size"));
		snapshot.setInstanceId(jsonObject.getString("__snapshot_from_instance"));
		snapshot.setName(jsonObject.getString("name"));
		String status = jsonObject.getString("status");
		if (!StringUtils.isEmpty(status) && "ACTIVE".equalsIgnoreCase(status)) {
			snapshot.setStatus("AVAILABLE");
		} else {
			snapshot.setStatus("EXCEPTION");
		}
		String creationTime = jsonObject.getString("created_at");
		if (!creationTime.contains("+")) {
			creationTime += "+0000";
		}
		Date transfer = DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", creationTime, "Z");
		if (null != creationTime && !"".equals(creationTime)) {
			snapshot.setCreateTime(transfer);
		}
		snapshot.setBlockDeviceMapping(jsonObject.getString("block_device_mapping"));
		snapshot.setSnapshotType("VM");
		return snapshot;
	}

	@SuppressWarnings("unchecked")
	private BocloudResult waitSnapshot(String jobId, String serverId) {
		// 获取当前云主机的硬盘信息
		Long startTime = Long.valueOf(System.currentTimeMillis());
		while (true) {
			try {
				Thread.sleep(5000L);
			} catch (InterruptedException e) {
				logger.error("Get error message:", e);
				Thread.currentThread().interrupt();
			}
			Result waitImageTask = waitImageTask(jobId);
			if (waitImageTask.isFailed()) {
				return new BocloudResult(false, "快照创建失败！");
			}
			JSONObject jobObject = JSONObject.parseObject(waitImageTask.getMessage());
			if ("SUCCESS".equalsIgnoreCase(jobObject.getString("status"))) {
				List<SnapshotModel> models = new ArrayList<>();
				// 查询云主机的整机快照
				BocloudResult serverSnapshot = getServerSnapshot(serverId);
				if (serverSnapshot.isFailed()) {
					return serverSnapshot;
				}
				List<SnapshotModel> convertList = (List<SnapshotModel>) serverSnapshot.getData();
				if (ListTool.isEmpty(convertList)) {
					return new BocloudResult(false, "快照创建失败！");
				}
				SnapshotModel snapshotModel = convertList.get(0);
				models.add(snapshotModel);
				String blockDeviceMapping = snapshotModel.getBlockDeviceMapping();
				// 获取整机快照 下面 硬盘的快照
				List<JSONObject> parseArray = JSONObject.parseArray(blockDeviceMapping, JSONObject.class);
				ManageOneSnapshotProvider snapshotProvider = new ManageOneSnapshotProvider(butler);
				for (JSONObject obj : parseArray) {
					SnapshotModel model = new SnapshotModel();
					model.setSnapshotUuid(obj.getString("snapshot_id"));
					BocloudResult bocloudResult = snapshotProvider.detail(model);
					if (bocloudResult.isFailed()) {
						continue;
					}
					models.add((SnapshotModel) bocloudResult.getData());
				}
				return new BocloudResult(true, models, "快照创建成功！");
			}
			Long endTime = Long.valueOf(System.currentTimeMillis());
			if (endTime.longValue() - startTime.longValue() > 480000L) {
				return new BocloudResult(false, "快照创建超时！");
			}
		}
	}

	/**
	 * 整机快照回滚
	 * 
	 * @param snapshotModel
	 * @return
	 */
	@Override
	public BocloudResult rebuildServer(SnapshotModel snapshotModel) {
		try {
			JSONObject paramObject = new JSONObject();
			paramObject.put("imageRef", snapshotModel.getSnapshotUuid());
			JSONObject params = new JSONObject();
			params.put("rebuild", paramObject);
			Result result = this.doPost("/servers/" + snapshotModel.getInstanceId() + "/action", params,
					ManageOneApiName.ecs, ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "云主机列表为空");
			}
			String jobId = object.getString("job_id");
			return waitServerStatus(jobId);
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}

	}

	private BocloudResult waitServerStatus(String jobId) {
		// 获取当前云主机的硬盘信息
		Long startTime = Long.valueOf(System.currentTimeMillis());
		while (true) {
			try {
				Thread.sleep(5000L);
			} catch (InterruptedException e) {
				logger.error("Get error message:", e);
				Thread.currentThread().interrupt();
			}
			Result waitImageTask = waitImageTask(jobId);
			if (waitImageTask.isFailed()) {
				return new BocloudResult(false, "快照回滚失败！");
			}
			JSONObject jobObject = JSONObject.parseObject(waitImageTask.getMessage());
			if ("SUCCESS".equalsIgnoreCase(jobObject.getString("status"))) {
				return new BocloudResult(true, "快照回滚成功！");
			}
			Long endTime = Long.valueOf(System.currentTimeMillis());
			if (endTime.longValue() - startTime.longValue() > 480000L) {
				return new BocloudResult(false, "快照回滚超时！");
			}
		}
	}

	@Override
	public BocloudResult modify(ServerModel server) {
		try {
			JSONObject objectParams = new JSONObject();
			objectParams.put("name", server.getName());
			if (!StringUtils.isEmpty(server.getRemark())) {
				objectParams.put("description", server.getRemark());
			}
			JSONObject params = new JSONObject();
			params.put("server", objectParams);
			Result result = this.doPut("/servers/" + server.getInstanceId(), params, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			JSONObject updateResult = JSONObject.parseObject(result.getMessage());
			JSONObject vmObj = JSONObject.parseObject(updateResult.getString("server"));
			if (null == vmObj) {
				return new BocloudResult(false, "修改失败");
			}
			String serverId = vmObj.getString("id");
			return new BocloudResult(true, this.detail(serverId).getData(), "修改成功");
		} catch (Exception e) {
			logger.error("Get error message：", e.getMessage());
			return new BocloudResult(false, "修改失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult start(String instanceId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject type = new JSONObject();
			params.put("os-start", type);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "启动成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "启动失败");
		}
	}

	@Override
	public BocloudResult stop(String instanceId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject type = new JSONObject();
			type.put("type", "HARD");
			params.put("os-stop", type);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "关闭成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "关闭失败");
		}
	}

	@Override
	public BocloudResult reboot(String instanceId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject type = new JSONObject();
			type.put("type", "HARD");
			params.put("reboot", type);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "重启成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "重启失败");
		}
	}

	@Override
	public BocloudResult softReboot(String instanceId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject type = new JSONObject();
			type.put("type", "SOFT");
			params.put("reboot", type);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "软重启成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "软重启失败");
		}
	}

	@Override
	public BocloudResult pause(String instanceId) {
		try {
			String jsonStr = "{\"pause\":null}";
			net.sf.json.JSONObject json = net.sf.json.JSONObject.fromObject(jsonStr);
			@SuppressWarnings("unchecked")
			Result result = this.doPost("/servers/" + instanceId + "/action", json, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "重启成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "重启失败");
		}
	}

	@Override
	public BocloudResult unPause(String instanceId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject type = new JSONObject();
			type.put("type", "HARD");
			params.put("unpause", type);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "重启成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "重启失败");
		}
	}

	@Override
	public BocloudResult suspend(String instanceId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject type = new JSONObject();
			type.put("type", "HARD");
			params.put("suspend", type);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "重启成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "重启失败");
		}
	}

	@Override
	public BocloudResult resume(String instanceId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject type = new JSONObject();
			type.put("type", "HARD");
			params.put("resume", type);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "重启成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "重启失败");
		}
	}

	@Override
	public BocloudResult remove(String instanceId, String name) {
		try {
			JSONObject params = new JSONObject();
			List<JSONObject> servers = new ArrayList<JSONObject>();
			JSONObject serverObject = new JSONObject();
			serverObject.put("id", instanceId);
			servers.add(serverObject);
			params.put("servers", servers);
			Result result = this.doPostForV1("/cloudservers/delete", params, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "删除失败");
		}
	}

	@Override
	public BocloudResult detail(String instanceId) {
		try {
			Result result = this.doGet("/servers/" + instanceId, null, ManageOneApiName.ecs, ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			JSONObject vmObj = object.getJSONObject("server");
			ServerModel convertModel = convertor.convertModel(vmObj);
			convertModel.setPorts(listPortByInstanceId(instanceId));
			return new BocloudResult(true, convertModel, "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult vncConsole(String vncType, String serverId) {
		try {
			Result result = this.getVncConsole(serverId);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject console = JSONObject
					.parseObject(JSONObject.parseObject(result.getMessage()).getString("remote_console"));
			return new BocloudResult(true, console.getString("url"), "获取控制台成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "获取控制台失败");
		}
	}

	@Override
	public BocloudResult joinSGroup(String instanceId, String groupId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject name = new JSONObject();
			name.put("name", groupId);
			params.put("addSecurityGroup", name);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "安全组添加成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "安全组添加失败");
		}
	}

	@Override
	public BocloudResult leaveSGroup(String instanceId, String groupId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject name = new JSONObject();
			name.put("name", groupId);
			params.put("removeSecurityGroup", name);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "安全组移除成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "安全组移除失败");
		}
	}

	@Override
	public BocloudResult changePassword(String instanceId, String password) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult attachNetwork(ServerModel server) {
		try {
			JSONObject params = new JSONObject();
			JSONObject attach = new JSONObject();
			NetworkBean networkBean = server.getNetworkBeans().get(0);
			if (!StringUtils.isEmpty(networkBean.getPortUuid())) {
				attach.put("port_id", networkBean.getPortUuid());
			} else {
				attach.put("net_id", networkBean.getNetworkUuid());
				List<JSONObject> fixedIps = new ArrayList<JSONObject>();
				List<AddressBean> addresses = networkBean.getAddresses();
				if (!ListTool.isEmpty(addresses)) {
					AddressBean addressBean = addresses.get(0);
					if (!StringUtils.isEmpty(addressBean.getAddress())) {
						com.alibaba.fastjson.JSONObject ipOjbect = new com.alibaba.fastjson.JSONObject();
						ipOjbect.put("ip_address", addressBean.getAddress());
						fixedIps.add(ipOjbect);
					}
				}
				if (!ListTool.isEmpty(fixedIps)) {
					attach.put("fixed_ips", fixedIps);
				}
			}
			params.put("interfaceAttachment", attach);
			logger.info("params : " + params);
			Result result = this.doPost("/servers/" + server.getInstanceId() + "/os-interface", params,
					ManageOneApiName.ecs, ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			logger.info("result : " + result);
			// 查询云主机端口
			server.setPorts(listPortByInstanceId(server.getInstanceId()));
			return new BocloudResult(true, server, "网卡添加成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "网卡添加失败");
		}
	}

	@Override
	public BocloudResult detachNetwork(String instanceId, String portUuid) {
		try {
			Result result = this.doDelete("/servers/" + instanceId + "/os-interface/" + portUuid, null,
					ManageOneApiName.ecs, ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			ServerModel model = new ServerModel();
			model.setPorts(listPortByInstanceId(instanceId));
			return new BocloudResult(true, model, "网卡卸载成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "网卡卸载失败");
		}
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId) {
		try {
			JSONObject params = new JSONObject();
			JSONObject resize = new JSONObject();
			resize.put("flavorRef", flavorId);
			params.put("resize", resize);
			Result result = this.doPost("/servers/" + instanceId + "/action", params, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			ServerModel model = waitForResizeTask(instanceId, "RESIZING");
			return new BocloudResult(true, model, "修改配置成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "修改配置失败");
		}
	}

	@Override
	public BocloudResult confirmResize(String instanceId) {
		try {
			String jsonStr = "{\"confirmResize\":null}";
			net.sf.json.JSONObject json = net.sf.json.JSONObject.fromObject(jsonStr);
			@SuppressWarnings("unchecked")
			Result result = this.doPost("/servers/" + instanceId + "/action", json, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			ServerModel model = waitForResizeTask(instanceId, new String[] { "STOPPED", "RUNNING" });
			return new BocloudResult(true, model, "确认配置成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "确认配置失败");
		}
	}

	@Override
	public BocloudResult revertResize(String instanceId) {
		try {
			String jsonStr = "{\"revertResize\":null}";
			net.sf.json.JSONObject json = net.sf.json.JSONObject.fromObject(jsonStr);
			@SuppressWarnings("unchecked")
			Result result = this.doPost("/servers/" + instanceId + "/action", json, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			ServerModel model = waitForResizeTask(instanceId, "STOPPED");
			return new BocloudResult(true, model, "回退变更配置成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "回退变更配置失败");
		}
	}

	@Override
	public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean memHotEnable) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult getTicket(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult createInResPool(ServerModel server) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult mountToolsInstaller(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult unmountToolsInstaller(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult upgradeToolsTask(String instanceId, String options) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult getNetCard(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult getVmIps(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult markAsTemplate(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
		// TODO Auto-generated method stub
		return null;
	}

	private List<JSONObject> getCloudVm() {
		List<JSONObject> list = new ArrayList<>();
		String url = this.getUrl() + "/rest/tenant-resource/v1/tenant/resources/CLOUD_VM?limit=1000";
		JSONObject params = new JSONObject();
		Result result = this.getClient().get(buildMonitorHeader(), params, url);
		JSONObject vmObj = JSON.parseObject(result.getMessage(), JSONObject.class);
		String objList = vmObj.getString("objList");
		List<JSONObject> vmList = JSON.parseArray(objList, JSONObject.class);
		if (null != vmList && vmList.size() > 0) {
			for (JSONObject jsonObject : vmList) {
				String projectId = jsonObject.getString("projectId");
				if (null != projectId && projectId.equals(butler.getProjectId())) {
					list.add(jsonObject);
				}
			}
		}
		return list;
	}

	private String getObjTypeId() {
		String url = this.getUrl() + "/rest/performance/v1/mgr-svc/obj-types";
		JSONObject params = new JSONObject();
		Result result = this.getClient().get(buildMonitorHeader(), params, url);
		JSONObject serverTypes = JSON.parseObject(result.getMessage(), JSONObject.class);
		List<JSONObject> dataList = JSON.parseArray(serverTypes.getString("data"), JSONObject.class);
		String objTypeId = "";
		for (JSONObject jsonObject : dataList) {
			String resourceCategory = jsonObject.getString("resource_category");
			if ("CLOUD_VM".equalsIgnoreCase(resourceCategory)) {
				objTypeId = jsonObject.getString("obj_type_id");
				break;
			}
		}
		return objTypeId;
	}

	private List<String> getIndicators(String objTypeId) {
		// String indicatorIdsUrl = this.getUrl() +
		// "/rest/performance/v1/mgr-svc/obj-types/" + objTypeId +
		// "/indicators";
		// JSONObject indicatorIdsParams = new JSONObject();
		// Result indicatorIdsResult =
		// this.getClient().get(buildMonitorHeader(), indicatorIdsParams,
		// indicatorIdsUrl);
		// JSONObject parseObject =
		// JSON.parseObject(indicatorIdsResult.getMessage(), JSONObject.class);
		// JSONObject parseObject2 =
		// JSON.parseObject(parseObject.getString("data"), JSONObject.class);
		// JSONArray jsonArray = parseObject2.getJSONArray("indicator_ids");
		// return jsonArray;
		List<String> list = new ArrayList<>();
		list.add("562958543486979"); // 内存使用率
		list.add("562958543421441"); // CPU使用率
		return list;
	}

	@Override
	public BocloudResult getMetric(String type) { // type 前端参数传过来的
		try {
			String objTypeId = getObjTypeId();
			logger.info("objTypeId : " + objTypeId);
			List<JSONObject> cloudVm = getCloudVm();
			if (!"".equals(objTypeId)) {
				List<String> indicatorsList = getIndicators(objTypeId);
				List<String> vmResids = new ArrayList<>();
				for (JSONObject jsonObject : cloudVm) {
					vmResids.add(jsonObject.getString("resId"));
				}
				String historyDataUrl = this.getUrl() + "/rest/performance/v1/data-svc/history-data/action/query";
				JSONObject historyDataParams = new JSONObject();
				historyDataParams.put("obj_type_id", objTypeId);
				historyDataParams.put("indicator_ids", indicatorsList);
				historyDataParams.put("range", "LAST_5_MINUTE");
				historyDataParams.put("obj_ids", vmResids);
				HttpPost httpPost = new HttpPost(historyDataUrl);
				httpPost.addHeader("accept", "application/json");
				httpPost.addHeader("content-type", "application/json");
				httpPost.addHeader("X-Auth-Token", this.getMonitorToken());
				httpPost.addHeader("roaRand", this.getMonitorRoaRand());
				StringEntity httpEntity = new StringEntity(JSONObject.toJSON(historyDataParams).toString(), "UTF-8");
				httpPost.setEntity(httpEntity);
				CloseableHttpResponse httpResponse = this.getClient().getHttpClient().execute(httpPost);
				Result monitorResult = new CommonResultBuilder().build(httpResponse.getStatusLine().getStatusCode());
				logger.info("monitor result:{}", JSONObject.toJSON(monitorResult));
				JSONObject content = JSONObject
						.parseObject(IOUtils.toString(httpResponse.getEntity().getContent(), "utf-8"));
				// logger.info("content : " + content);
				String monitorData = content.getString("data");
				JSONObject dataObj = JSON.parseObject(monitorData, JSONObject.class);
				// logger.info("dataObj : " + dataObj);
				for (JSONObject vm : cloudVm) {
					String resId = vm.getString("resId");
					JSONObject monitorInfo = JSON.parseObject(dataObj.getString(resId), JSONObject.class);
					// logger.info("monitorInfo : " + monitorInfo);
					if (null != monitorInfo) {
						if ("CPU".equalsIgnoreCase(type)) {
							JSONObject cpuUsed = JSON.parseObject(monitorInfo.getString("562958543486979"),
									JSONObject.class);
							String maxCpuVal = cpuUsed.getString("max");
							logger.info("maxCpuVal : " + maxCpuVal);
							if (null != maxCpuVal && maxCpuVal.contains(":")) {
								String[] split = maxCpuVal.split(":");
								String max = split[1].replaceAll("\"", "").replaceAll("}", "");
								BigDecimal bg = new BigDecimal(max);
								double f1 = bg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
								vm.put("value", f1);
							}
						}
						if ("内存".equals(type)) {
							JSONObject memoryInfo = JSON.parseObject(monitorInfo.getString("562958543421441"),
									JSONObject.class);
							String maxMemoryVal = memoryInfo.getString("max");
							logger.info("maxMemoryVal : " + maxMemoryVal);
							if (null != maxMemoryVal && maxMemoryVal.contains(":")) {
								String[] split = maxMemoryVal.split(":");
								String max = split[1].replaceAll("\"", "").replaceAll("}", "");
								BigDecimal bg = new BigDecimal(max);
								double f1 = bg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
								vm.put("value", f1);
							}
						}
					}
					Double val = vm.getDouble("value");
					if (null == val) {
						vm.put("value", 0d);
					}
				}
				// logger.info("cloudVm : " + JSON.toJSONString(cloudVm));
			}
			return new BocloudResult(true, cloudVm, "查询成功");
		} catch (Exception e) {
			logger.error("查询Error : " + e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult vmrcConsole(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult migrate(H3cMigrateModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult list(String hostId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult create(H3cServerModel server) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult getJnlpXml(Long vmId, String address) {
		// TODO Auto-generated method stub
		return null;
	}

	private ServerModel waitForResizeTask(String serverId, String... status) {
		ServerModel model = null;
		Boolean flag = Boolean.valueOf(true);
		Long startTime = Long.valueOf(System.currentTimeMillis());
		while (flag.booleanValue()) {
			try {
				Thread.sleep(5000L);
			} catch (InterruptedException e) {
				logger.error("Get error message:", e);
				Thread.currentThread().interrupt();
			}
			BocloudResult result = detail(serverId);
			if (result.isFailed()) {
				flag = Boolean.valueOf(false);
				break;
			}
			model = (ServerModel) com.alibaba.fastjson.JSONObject
					.parseObject(com.alibaba.fastjson.JSONObject.toJSONString(result.getData()), ServerModel.class);
			logger.info("model : " + JSON.toJSONString(model));
			if (null == model) {
				flag = Boolean.valueOf(false);
				break;
			}
			if (Arrays.asList(status).contains(model.getStatus())) {
				flag = Boolean.valueOf(false);
				break;
			}
			Long endTime = Long.valueOf(System.currentTimeMillis());
			if (endTime.longValue() - startTime.longValue() > 480000L) {
				flag = Boolean.valueOf(false);
				model.setStatus("EXCEPTION");
				break;
			}
		}
		return model;
	}

	/**
	 * 调整网络上限
	 *
	 * @param instanceId
	 * @param internetMaxBandwidthOut
	 * @return
	 */
	@Override
	public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
		return null;
	}

	/**
	 * 重装系统
	 *
	 * @param serverModel
	 * @return
	 */
	@Override
	public BocloudResult resetInstance(ServerModel serverModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult bindKeypair(ServerModel serverModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult unbindKeypair(ServerModel serverModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
		return null;
	}

	@Override
	public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
		// TODO Auto-generated method stub
		return null;
	}

}
