package com.bocloud.cmp.provider.compute;

import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.ManageOneApiName;
import com.bocloud.cmp.common.enums.ManageOneApiType;
import com.bocloud.cmp.manageoneconvertor.SnapshotConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

@Provider("bocloud.cmp.manageone")
public class ManageOneSnapshotProvider extends ManageOneProvider implements SnapshotProvider {
	private static final Logger logger = LoggerFactory.getLogger(ManageOneSnapshotProvider.class);
	private BeanConvertor<SnapshotModel, JSONObject> snapshotConvertor = new SnapshotConvertor();

	public ManageOneSnapshotProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			Result result = doGet("/snapshots", null, ManageOneApiName.evsv2, ManageOneApiType.volumev2);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "查询失败");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("snapshots"), JSONObject.class);
			return new BocloudResult(true, this.snapshotConvertor.convertList(objects), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
		}
		return new BocloudResult(false, "查询失败");
	}

	@Override
	public BocloudResult remove(SnapshotModel model) {
		try {
			// 查询详情 如果底层没有数据则返回成功
			Result detailResult = doGet("/snapshots/" + model.getSnapshotUuid(), null, ManageOneApiName.evsv2,
					ManageOneApiType.volumev2);
			if (detailResult.isFailed()) {
				String message = detailResult.getMessage();
				if (null != message && message.contains("itemNotFound")) {
					return new BocloudResult(false, "删除成功！");
				}
				return new BocloudResult(false, detailResult.getMessage());
			}
			Result result = doDelete("/snapshots/" + model.getSnapshotUuid(), null, ManageOneApiName.evsv2,
					ManageOneApiType.volumev2);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, result.getMessage(), "删除成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
		}
		return new BocloudResult(false, "删除失败");
	}

	@Override
	public BocloudResult removeServerSnapshot(SnapshotModel snapshotModel) {
		try {
			List<String> images = new ArrayList<>();
			images.add(snapshotModel.getSnapshotUuid());
			JSONObject params = new JSONObject();
			params.put("images", images);
			params.put("is_snapshot_image", "true");
			params.put("available_zone", "az1.dc1");
			params.put("region", "sa-fb-1");
			Result result = this.doDeleteWithParams("/v2/cloudimages", params, ManageOneApiName.ims,
					ManageOneApiType.image);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "云主机列表为空");
			}
			String jobId = object.getString("job_id");
			return waitServerStatus(jobId);
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	private BocloudResult waitServerStatus(String jobId) {
		// 获取当前云主机的硬盘信息
		Long startTime = Long.valueOf(System.currentTimeMillis());
		while (true) {
			try {
				Thread.sleep(5000L);
			} catch (InterruptedException e) {
				logger.error("Get error message:", e);
				Thread.currentThread().interrupt();
			}
			Result waitImageTask = waitImageTask(jobId);
			if (waitImageTask.isFailed()) {
				return new BocloudResult(false, "快照删除成功！");
			}
			JSONObject jobObject = JSONObject.parseObject(waitImageTask.getMessage());
			if ("SUCCESS".equalsIgnoreCase(jobObject.getString("status"))) {
				return new BocloudResult(true, "快照删除成功！");
			}
			Long endTime = Long.valueOf(System.currentTimeMillis());
			if (endTime.longValue() - startTime.longValue() > 480000L) {
				return new BocloudResult(false, "快照删除超时！");
			}
		}
	}

	@Override
	public BocloudResult detail(SnapshotModel model) {
		try {
			Result result = doGet("/snapshots/" + model.getSnapshotUuid(), null, ManageOneApiName.evsv2,
					ManageOneApiType.volumev2);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("snapshot");
			return new BocloudResult(true, this.snapshotConvertor.convertModel(jsonObject), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
		}
		return new BocloudResult(false, "查询失败");
	}

	@Override
	public BocloudResult modify(SnapshotModel model) {
		try {
			JSONObject params = new JSONObject();
			params.put("description", model.getRemark());
			params.put("name", model.getName());
			JSONObject volumeParam = new JSONObject();
			volumeParam.put("snapshot", params);

			Result result = doPut("/snapshots/" + model.getSnapshotUuid(), volumeParam, ManageOneApiName.evsv2,
					ManageOneApiType.volumev2);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("snapshot");
			return new BocloudResult(true, this.snapshotConvertor.convertModel(jsonObject), "修改成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
		}
		return new BocloudResult(false, "修改失败");
	}

	@Override
	public BocloudResult revert(SnapshotModel model) {
		try {
			JSONObject jsonObject = new JSONObject();
			jsonObject.put("volume_id", model.getVolumeUuid());
			JSONObject params = new JSONObject();
			params.put("rollback", jsonObject);
			Result result = doPost("/os-vendor-snapshots/" + model.getSnapshotUuid() + "/rollback", params,
					ManageOneApiName.evsv2, ManageOneApiType.volumev2);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, result.getMessage(), "success");
		} catch (Exception e) {
			logger.error("Get error message:", e);
		}
		return new BocloudResult(false, "fail");
	}

	@Override
	public BocloudResult removeByVolumeId(String volumeId) {
		return null;
	}

	@Override
	public BocloudResult getSnapshot(String snapshotId) {
		SnapshotModel model = new SnapshotModel();
		model.setSnapshotUuid(snapshotId);
		BocloudResult result = new BocloudResult(false, "操作失败");
		Boolean flag = Boolean.valueOf(true);
		Long startTime = Long.valueOf(System.currentTimeMillis());
		while (flag.booleanValue()) {
			try {
				Thread.sleep(5000L);
			} catch (InterruptedException e) {
				logger.error("Get error message:", e);
				Thread.currentThread().interrupt();
			}
			result = detail(model);
			if (result.isFailed()) {
				flag = Boolean.valueOf(false);
				break;
			}
			SnapshotModel snapshotModel = (SnapshotModel) JSONObject
					.parseObject(JSONObject.toJSONString(result.getData()), SnapshotModel.class);
			if (null == snapshotModel) {
				flag = Boolean.valueOf(false);
				result = new BocloudResult(false, "fail");
				break;
			}
			if (!"creating".equalsIgnoreCase(snapshotModel.getStatus())) {
				flag = Boolean.valueOf(false);
				break;
			}
			Long endTime = Long.valueOf(System.currentTimeMillis());
			if (endTime.longValue() - startTime.longValue() > 900000L) {
				flag = Boolean.valueOf(false);
				model.setStatus("EXCEPTION");
				break;
			}
		}
		return result;
	}

	@Override
	public BocloudResult getAvaliableSnap(String snapshotId) {
		return null;
	}

	@Override
	public BocloudResult list(Long cloudServerId) {
		return null;
	}
}
