
package com.bocloud.cmp.provider.manageone;

import io.jsonwebtoken.lang.Assert;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.manageoneconvertor.VdcProjectConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.manageone.VdcProjectModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

/**
 * 华为manage one 项目管理
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年6月17日
 *
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneVdcProjectProvider extends ManageOneProvider implements VdcProjectProvider {

	private static Logger logger = LoggerFactory.getLogger(ManageOneVdcProjectProvider.class);
	private BeanConvertor<VdcProjectModel, JSONObject> convertor = new VdcProjectConvertor();
	/**
	 * 华为manage one limit取值范围：0-1000，默认20
	 */
	private static final Integer LIMIT = 1000;

	public ManageOneVdcProjectProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list(String vdcId) {
		try {
			Map<String, Object> paramMap = new HashMap<>();
			paramMap.put("start", 0);
			paramMap.put("limit", LIMIT);
			Result result = this.doGet("/rest/vdc/v3.0/vdcs/" + vdcId + "/projects", paramMap, null, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			JSONObject object = JSONObject.parseObject(result.getMessage());
			// 返回总条数
			Long total = object.getLong("total");
			if (null == total || total == 0L) {
				return new BocloudResult(true, "查询成功");
			}
			List<VdcProjectModel> vdcModels = new ArrayList<>();
			// 第一次查询的结果集
			List<JSONObject> objects = JSONArray.parseArray(object.getString("projects"), JSONObject.class);
			vdcModels.addAll(convertor.convertList(objects));
			Long totalPage = total / LIMIT + 1;
			if (totalPage.equals(1L)) {
				return new BocloudResult(true, vdcModels, "查询成功");
			}
			for (int i = 1; i <= totalPage; i++) {
				paramMap.clear();
				paramMap.put("start", i);
				paramMap.put("limit", LIMIT);
				result = this.doGet("/rest/vdc/v3.0/vdcs/" + vdcId + "/projects", paramMap, null, null);
				Assert.isTrue(result.isSuccess(), result.getMessage());
				object = JSONObject.parseObject(result.getMessage());
				objects = JSONArray.parseArray(object.getString("projects"), JSONObject.class);
				if (!ListTool.isEmpty(objects)) {
					vdcModels.addAll(convertor.convertList(objects));
				}
			}
			return new BocloudResult(true, vdcModels, "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "查询失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult create(VdcProjectModel model) {
		try {
			JSONObject projectParam = new JSONObject();
			projectParam.put("name", model.getName());
			projectParam.put("display_name", model.getDisplayName());
			projectParam.put("description", model.getRemark());
			projectParam.put("group_id", model.getGroupId());
			projectParam.put("tenant_id", model.getVdcTenantId());
			projectParam.put("vrm_project_id", model.getVrmProjectId());
			projectParam.put("contract_number", model.getContractNumber());
			projectParam.put("attachment_id", model.getAttachmentId());
			projectParam.put("is_shared", model.getShared());
			projectParam.put("is_bind_external_network", model.getIsBindExternalNetwork());
			List<JSONObject> regions = this.getRegionList(model.getVdcTenantId());
			if (!ListTool.isEmpty(regions)) {
				List<JSONObject> regionList = new ArrayList<>();
				for (JSONObject region : regions) {
					JSONObject newRegion = new JSONObject();
					newRegion.put("region_id", region.getString("region_id"));
					regionList.add(newRegion);
				}
				projectParam.put("regions", regionList);
			}
			JSONObject params = new JSONObject();
			params.put("project", projectParam);
			Result result = this.doPost("/rest/vdc/v3.1/projects", params, null, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("project");
			return new BocloudResult(true, this.detail(jsonObject.getString("id")).getData(), "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "创建失败：" + e.getMessage());
		}
	}

	/**
	 * 获取地域列表
	 * 
	 * @param vdcId
	 * @return
	 */
	private List<JSONObject> getRegionList(String vdcId) {
		Result regionResult = this.doGet("/rest/vdc/v3.1/vdcs/" + vdcId + "/regions", null, null, null);
		if (regionResult.isFailed()) {
			return null;
		}
		List<JSONObject> regions = JSONArray
				.parseArray(JSONObject.parseObject(regionResult.getMessage()).getString("regions"), JSONObject.class);
		return regions;
	}

	@Override
	public BocloudResult remove(String projectId) {
		try {
			Result result = this.doDelete("/rest/vdc/v3.1/projects/" + projectId, null, null, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			return new BocloudResult(true, "删除成功！");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "删除失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String projectId) {
		try {
			Result result = this.doGet("/rest/vdc/v3.1/projects/" + projectId, null, null, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("project");
			return new BocloudResult(true, convertor.convertModel(jsonObject), "查询详情成功！");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "查询失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult update(VdcProjectModel model) {
		try {
			JSONObject projectParam = new JSONObject();
			projectParam.put("name", model.getName());
			projectParam.put("display_name", model.getDisplayName());
			projectParam.put("description", model.getRemark());
			projectParam.put("contract_number", model.getContractNumber());
			projectParam.put("attachment_id", model.getAttachmentId());
			projectParam.put("is_shared", model.getShared());
			JSONObject params = new JSONObject();
			params.put("project", projectParam);
			Result result = this.doPost("/rest/vdc/v3.1/projects/" + model.getProjectId(), params, null, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("project");
			return new BocloudResult(true, this.detail(jsonObject.getString("id")).getData(), "修改成功");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "修改失败：" + e.getMessage());
		}
	}
}
