package com.bocloud.cmp.provider.network;

import java.util.List;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.ManageOneApiName;
import com.bocloud.cmp.common.enums.ManageOneApiType;
import com.bocloud.cmp.manageoneconvertor.FloatingIpConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FloatingIpModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

/**
 * 华为manage one浮动IP操作driver
 *
 * @Date: Created in 2018/9/20
 * @Author: guopeng
 * @Description:
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneFloatingIPProvider extends ManageOneProvider implements FloatingIPProvider {

	private static final Logger logger = LoggerFactory.getLogger(ManageOneFloatingIPProvider.class);
	private BeanConvertor<FloatingIpModel, JSONObject> convertor = new FloatingIpConvertor();
	private Butler butler;

	/**
	 * @param butler
	 */
	public ManageOneFloatingIPProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult list() {
		try {
			Result result = this.doGet("/v2.0/floatingips", null, ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "浮动IP列表为空");
			}
			logger.info("获取浮动IP数据：" + object.toJSONString());
			List<JSONObject> objects = JSONArray.parseArray(object.getString("floatingips"), JSONObject.class);
			return new BocloudResult(true, convertor.convertList(objects), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult create(FloatingIpModel floatingIpModel) {
		try {
			String publicType = floatingIpModel.getPublicType();
			if (StringUtils.isEmpty(publicType)) {
				ManageOneNetworkProvider networkProvider = new ManageOneNetworkProvider(butler);
				BocloudResult listPublicipTypes = networkProvider.listPublicipTypes();
				if (listPublicipTypes.isFailed()) {
					return new BocloudResult(false, "获取外部网络异常！");
				}
				List<JSONObject> parseArray = JSONObject.parseArray(JSON.toJSONString(listPublicipTypes.getData()),
						JSONObject.class);
				if (ListTool.isEmpty(parseArray)) {
					return new BocloudResult(false, "获取外部网络异常！");
				}
				JSONObject jsonObject = parseArray.get(0);
				publicType = jsonObject.getString("type");
			}
			JSONObject params = new JSONObject();
			params.put("type", publicType);
			JSONObject floatingIpParams = new JSONObject();
			floatingIpParams.put("publicip", params);

			JSONObject bandwidth = new JSONObject();
			bandwidth.put("name", UUID.randomUUID().toString().replaceAll("-", ""));
			bandwidth.put("size", 10);
			bandwidth.put("share_type", "PER");
			floatingIpParams.put("bandwidth", bandwidth);

			Result result = this.doPost("/v1/" + butler.getProjectId() + "/publicips", floatingIpParams,
					ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "Failed to create publicip ,Please check the network connection!");
			}
			JSONObject jsonObject = object.getJSONObject("publicip");
			if (null == jsonObject) {
				return new BocloudResult(false, "创建失败");
			}
			return new BocloudResult(true, convertModel(jsonObject), "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "创建失败");
		}
	}

	public FloatingIpModel convertModel(JSONObject model) {
		FloatingIpModel floatingIpModel = new FloatingIpModel();
		floatingIpModel.setTenantUuid(model.getString("tenant_id"));
		floatingIpModel.setName(model.getString("public_ip_address"));
		floatingIpModel.setBandwidthId(model.getString("bandwidth_id"));
		floatingIpModel.setBandwidthSize(model.getInteger("bandwidth_size"));
		floatingIpModel.setFloatingIpAddress(model.getString("public_ip_address"));
		floatingIpModel.setFloatingIpUuid(model.getString("id"));
		floatingIpModel.setFloatingNetworkUuid(model.getString("external_net_id"));
		floatingIpModel.setStatus(model.getString("status"));
		return floatingIpModel;
	}

	@Override
	public BocloudResult modify(FloatingIpModel floatingIpModel) {
		try {
			JSONObject params = new JSONObject();
			params.put("port_id", floatingIpModel.getPortUuid());
			JSONObject floatingIpParams = new JSONObject();
			floatingIpParams.put("floatingip", params);
			Result result = this.doPut("/v2.0/floatingips/" + floatingIpModel.getFloatingIpUuid(), floatingIpParams,
					ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "修改失败");
			}
			JSONObject jsonObject = object.getJSONObject("floatingip");
			if (null == jsonObject) {
				return new BocloudResult(false, "修改失败");
			}
			return new BocloudResult(true, convertor.convertModel(jsonObject), "修改成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "删除失败");
		}
	}

	@Override
	public BocloudResult remove(String fipId) {
		try {
			Result result = this.doDelete("/v2.0/floatingips/" + fipId, null, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "删除失败");
		}
	}

	@Override
	public BocloudResult detail(String fipId) {
		return null;
	}

	@Override
	public BocloudResult associate(String fipId, String portId) {
		try {
			JSONObject params = new JSONObject();
			params.put("port_id", portId);
			JSONObject floatingIpParams = new JSONObject();
			floatingIpParams.put("floatingip", params);
			Result result = this.doPut("/v2.0/floatingips/" + fipId, floatingIpParams, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "Failed to associate floatingIp ,Please check the network connection!");
			}
			JSONObject jsonObject = object.getJSONObject("floatingip");
			if (null == jsonObject) {
				return new BocloudResult(false, "分配失败");
			}
			return new BocloudResult(true, this.convertor.convertModel(jsonObject), "floatingIp associate success!");
		} catch (Exception e) {
			logger.error("floatingIp associate error:", e.getMessage());
			return new BocloudResult(false, "floatingIp associate error!");
		}
	}

	@Override
	public BocloudResult disassociate(String fipId) {
		try {
			JSONObject params = new JSONObject();
			// port_id 为空为解绑
			params.put("port_id", "");
			JSONObject floatingIpParams = new JSONObject();
			floatingIpParams.put("floatingip", params);
			Result result = this.doPut("/v2.0/floatingips/" + fipId, floatingIpParams, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false,
						"Failed to disassociate floatingIp ,Please check the network connection!");
			}
			JSONObject jsonObject = object.getJSONObject("floatingip");
			if (null == jsonObject) {
				return new BocloudResult(false, "解绑失败失败");
			}
			return new BocloudResult(true, this.convertor.convertModel(jsonObject), "floatingIp disassociate success!");
		} catch (Exception e) {
			logger.error("floatingIp disassociate error:", e.getMessage());
			return new BocloudResult(false, "floatingIp disassociate error!");
		}
	}

	@Override
	public BocloudResult associate(FloatingIpModel modeld) {
		try {
			JSONObject params = new JSONObject();
			params.put("port_id", modeld.getPortUuid());
			JSONObject floatingIpParams = new JSONObject();
			floatingIpParams.put("floatingip", params);
			Result result = this.doPut("/v2.0/floatingips/" + modeld.getFloatingIpUuid(), floatingIpParams,
					ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "Failed to associate floatingIp ,Please check the network connection!");
			}
			JSONObject jsonObject = object.getJSONObject("floatingip");
			if (null == jsonObject) {
				return new BocloudResult(false, "分配失败");
			}
			return new BocloudResult(true, this.convertor.convertModel(jsonObject), "floatingIp associate success!");
		} catch (Exception e) {
			logger.error("floatingIp associate error:", e.getMessage());
			return new BocloudResult(false, "floatingIp associate error!");
		}
	}
}
