package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.ManageOneApiName;
import com.bocloud.cmp.common.enums.ManageOneApiType;
import com.bocloud.cmp.manageoneconvertor.NetworkConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkModel;
import com.bocloud.cmp.model.PortGroupModel;
import com.bocloud.cmp.model.SwitchModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

/**
 * 华为manage one网络操作driver
 *
 * @author wangyu
 * @version 1.0
 * @complete gp
 * @since 2018年9月17日
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneNetworkProvider extends ManageOneProvider implements NetworkProvider {

	private static final Logger logger = LoggerFactory.getLogger(ManageOneNetworkProvider.class);
	private BeanConvertor<NetworkModel, JSONObject> convertor = new NetworkConvertor();

	private Butler butler;

	/**
	 * @param butler
	 */
	public ManageOneNetworkProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@SuppressWarnings("unchecked")
	@Override
	public BocloudResult list() {
		try {
			Result result = doGet("/v2.0/networks", null, ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "查询失败");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("networks"), JSONObject.class);
			List<JSONObject> currentNets = new ArrayList<>();
			for (JSONObject jsonObject : objects) {
				String tenantId = jsonObject.getString("tenant_id");
				if ((null != tenantId) && (tenantId.equals(this.butler.getProjectId()))) {
					currentNets.add(jsonObject);
				}
				// 添加共享网络
				Boolean shared = jsonObject.getBoolean("shared");
				if (null != shared && shared) {
					currentNets.add(jsonObject);
				}
			}
			List<NetworkModel> convertList = this.convertor.convertList(currentNets);
			Map<String, NetworkModel> networkModelMap = new HashMap<>();
			for (NetworkModel networkModel : convertList) {
				networkModelMap.put(networkModel.getNetworkUuid(), networkModel);
			}
			if (!ListTool.isEmpty(convertList)) {
				// 查询external_networks
				BocloudResult listVpcExternalNetworks = this.listVpcExternalNetworks();
				if (listVpcExternalNetworks.isSuccess()) {
					List<JSONObject> vpcExternalNetworks = (List<JSONObject>) listVpcExternalNetworks.getData();
					for (JSONObject vpcExternalNetwork : vpcExternalNetworks) {
						NetworkModel networkModel = networkModelMap.get(vpcExternalNetwork.getString("id"));
						if (null != networkModel) {
							networkModel.setType("external_network");
							networkModelMap.put(vpcExternalNetwork.getString("id"), networkModel);
						}
					}
				}
				BocloudResult listPublicipTypes = this.listPublicipTypes();
				if (listPublicipTypes.isSuccess() && null != listPublicipTypes.getData()) {
					List<JSONObject> publicipTypes = (List<JSONObject>) listPublicipTypes.getData();
					if (!ListTool.isEmpty(publicipTypes)) {
						for (JSONObject publicipType : publicipTypes) {
							NetworkModel networkModel = networkModelMap.get(publicipType.getString("id"));
							if (null != networkModel) {
								networkModel.setType("eip_external_net");
								networkModelMap.put(publicipType.getString("id"), networkModel);
							}
						}
					}
				}
			}
			return new BocloudResult(true, networkModelMap.values(), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message��", e);
		}
		return new BocloudResult(false, "查询失败");
	}

	public BocloudResult listVpcExternalNetworks() {
		try {
			Result result = doGet("/v1/" + butler.getProjectId() + "/external_networks", null, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, "查询VPC外部网络异常：" + result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "暂无可以外部VPC网络！");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("vpc_external_networks"),
					JSONObject.class);
			return new BocloudResult(true, objects, "查询成功！");
		} catch (Exception e) {
			logger.error("Get error message��", e);
			return new BocloudResult(false, "查询VPC外部网络异常：" + e.getMessage());
		}
	}

	public BocloudResult listPublicipTypes() {
		try {
			Result result = doGet("/v1/" + butler.getProjectId() + "/publicip_types", null, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "查询失败");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("publicip_types"), JSONObject.class);
			if (ListTool.isEmpty(objects)) {
				return new BocloudResult(true, "暂无可以外部IP");
			}
			return new BocloudResult(true, objects, "查询成功");
		} catch (Exception e) {
			logger.error("Get error message��", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult create(NetworkModel model) {
		try {
			JSONObject network = new JSONObject();
			JSONObject param = new JSONObject();
			param.put("shared", model.getShared());
			param.put("name", model.getName());
			param.put("admin_state_up", model.getAdminStateup());
			network.put("network", param);
			Result result = this.doPost("/v2.0/networks", network, ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "创建失败");
			}
			return new BocloudResult(true, convertor.convertModel(object.getJSONObject("network")), "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "创建失败");

		}
	}

	@Override
	public BocloudResult remove(String networkId) {
		try {
			BocloudResult bocloudResult = this.detail(networkId);
			if (null == bocloudResult.getData()) {
				return new BocloudResult(true, "删除成功");
			}
			Result result = this.doDelete("/v2.0/networks/" + networkId, null, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "删除失败");
		}
	}

	@Override
	public BocloudResult detail(String networkId) {
		try {
			Result result = this.doGet("/v2.0/networks/" + networkId, null, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "路由不存在");
			}
			return new BocloudResult(true, convertor.convertModel(object.getJSONObject("network")), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult modify(NetworkModel model) {
		try {
			JSONObject params = new JSONObject();
			JSONObject network = new JSONObject();
			network.put("name", model.getName());
			network.put("shared", model.getShared());
			network.put("admin_state_up", model.getAdminStateup());
			params.put("network", network);
			Result result = this.doPut("/v2.0/networks/" + model.getNetworkUuid(), params, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject routerObject = JSONObject.parseObject(result.getMessage(), JSONObject.class);
			return new BocloudResult(true, convertor.convertModel(routerObject.getJSONObject("network")), "修改成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "修改失败");
		}
	}

	@Override
	public BocloudResult createDSwitch(SwitchModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult createVSwitch(SwitchModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult removeDSwitch(String name) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult renameDSwitch(String oldName, String newName) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult updateDVSwitch(SwitchModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult removeVSwitch(String hostName, String vswitchName, List<String> pgNames) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult updateVSwitch(String hostName, String vswitchName, SwitchModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult createPortGroup(String hostName, String vswitchName, PortGroupModel pgModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult removePortGroup(String hostName, String pgName, String type) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult updatePortGroup(String hostName, PortGroupModel pgModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult addHost(SwitchModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult listVSwitch(SwitchModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult listProfile() {
		// TODO Auto-generated method stub
		return null;
	}

}
