package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.ManageOneApiName;
import com.bocloud.cmp.common.enums.ManageOneApiType;
import com.bocloud.cmp.manageoneconvertor.PortConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.PortFixedIpModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

/**
 * 华为manage one端口操作driver
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年9月18日
 */
@Provider("bocloud.cmp.manageone")
public class ManageOnePortProvider extends ManageOneProvider implements PortProvider {

	private static final Logger logger = LoggerFactory.getLogger(ManageOnePortProvider.class);
	private BeanConvertor<PortModel, JSONObject> convertor = new PortConvertor();

	/**
	 * @param butler
	 */
	public ManageOnePortProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult sync() {
		try {
			Result result = this.doGet("/v2.0/ports", null, ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "查询结果为空");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("ports"), JSONObject.class);
			return new BocloudResult(true, convertor.convertList(objects), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult create(PortModel portModel) {
		try {
			JSONObject port = new JSONObject();
			port.put("name", portModel.getName());
			port.put("network_id", portModel.getNetworkUuid());
			port.put("tenant_id", this.getProject());
			port.put("admin_state_up", true);
			List<JSONObject> ips = new ArrayList<JSONObject>();
			if (!ListTool.isEmpty(portModel.getPortFixedIps())) {
				List<PortFixedIpModel> fixedIpModels = portModel.getPortFixedIps();
				for (PortFixedIpModel fixedIpModel : fixedIpModels) {
					JSONObject ip = new JSONObject();
					ip.put("ip_address", fixedIpModel.getIpAddress());
					ip.put("subnet_id", fixedIpModel.getSubnetUuid());
					ips.add(ip);
				}
			}
			port.put("fixed_ips", ips);
			JSONObject params = new JSONObject();
			params.put("port", port);
			Result result = this.doPost("/v2.0/ports", params, ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "创建失败");
			}
			return new BocloudResult(true, convertor.convertModel(object.getJSONObject("port")), "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "创建失败");
		}
	}

	@Override
	public BocloudResult modify(PortModel portModel) {
		try {
			Result portDetail = this.doGet("/v2.0/ports/" + portModel.getPortUuid(), null, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (portDetail.isFailed()) {
				return new BocloudResult(false, portDetail.getMessage());
			}
			JSONObject detailObject = JSONObject.parseObject(portDetail.getMessage());
			if (null == detailObject) {
				return new BocloudResult(false, "端口数据不存在");
			}
			JSONObject port = new JSONObject();
			JSONObject portObject = detailObject.getJSONObject("port");
			List<PortFixedIpModel> portFixedIps = portModel.getPortFixedIps();
			if (!ListTool.isEmpty(portFixedIps)) {
				List<JSONObject> list = JSON.parseArray(portObject.getString("fixed_ips"), JSONObject.class);
				if (ListTool.isEmpty(list)) {
					return new BocloudResult(false, "网卡信息异常！");
				}
				JSONObject jsonObject = list.get(0);
				String subnetId = jsonObject.getString("subnet_id");
				List<JSONObject> ips = new ArrayList<JSONObject>();
				List<PortFixedIpModel> fixedIpModels = portModel.getPortFixedIps();
				for (PortFixedIpModel fixedIpModel : fixedIpModels) {
					JSONObject ip = new JSONObject();
					ip.put("subnet_id", subnetId);
					ip.put("ip_address", fixedIpModel.getIpAddress());
					ips.add(ip);
				}
				port.put("fixed_ips", ips);
			}
			String name = portModel.getName();
			if (!StringUtils.isEmpty(name)) {
				port.put("name", name);
			}
			JSONObject params = new JSONObject();
			params.put("port", port);
			Result result = this.doPut("/v2.0/ports/" + portModel.getPortUuid(), params, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "创建失败");
			}
			return new BocloudResult(true, convertor.convertModel(object.getJSONObject("port")), "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "创建失败");
		}
	}

	@Override
	public BocloudResult remove(String portId) {
		try {
			BocloudResult bocloudResult = this.detail(portId);
			if (null == bocloudResult.getData()) {
				return new BocloudResult(true, "删除成功");
			}
			Result result = this
					.doDelete("/v2.0/ports/" + portId, null, ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "删除失败");
		}
	}

	@Override
	public BocloudResult detail(String portId) {
		try {
			Result result = this.doGet("/v2.0/ports/" + portId, null, ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "端口不存在");
			}
			return new BocloudResult(true, convertor.convertModel(object), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

}
