package com.bocloud.cmp.provider.network;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.ManageOneApiName;
import com.bocloud.cmp.common.enums.ManageOneApiType;
import com.bocloud.cmp.manageoneconvertor.PortConvertor;
import com.bocloud.cmp.manageoneconvertor.RouterConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.model.RouterModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * 华为manage one路由操作driver
 *
 * @author wangyu
 * @version 1.0
 * @complete gp
 * @since 2018年9月18日
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneRouterProvider extends ManageOneProvider implements RouterProvider {

	private static final Logger logger = LoggerFactory.getLogger(ManageOneRouterProvider.class);
	private BeanConvertor<RouterModel, JSONObject> convertor = new RouterConvertor();

	/**
	 * @param butler
	 */
	public ManageOneRouterProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult create(RouterModel model) {
		try {
			JSONObject params = new JSONObject();
			params.put("name", model.getName());
			params.put("admin_state_up", model.getAdminStateup() == null ? true : model.getAdminStateup());
			JSONObject network = new JSONObject();
			if (!StringUtils.isEmpty(model.getNetworkUuid())) {
				network.put("network_id", model.getNetworkUuid());
				params.put("external_gateway_info", network);
			}
			JSONObject routerParam = new JSONObject();
			routerParam.put("router", params);
			Result result = this.doPost("/v2.0/routers", routerParam, ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "创建失败");
			}
			return new BocloudResult(true, convertor.convertModel(object.getJSONObject("router")), "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "创建失败");

		}
	}

	@Override
	public BocloudResult remove(String routerId) {
		try {
			BocloudResult bocloudResult = this.detail(routerId);
			if (null == bocloudResult.getData()) {
				return new BocloudResult(true, "删除成功");
			}
			Result result = this.doDelete("/v2.0/routers/" + routerId, null, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "删除失败");
		}

	}

	@Override
	public BocloudResult detail(String routerId) {
		try {
			Result result = this.doGet("/v2.0/routers/" + routerId, null, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "路由不存在");
			}
			return new BocloudResult(true, convertor.convertModel(object.getJSONObject("router")), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult modify(RouterModel model) {
		try {
			JSONObject params = new JSONObject();
			JSONObject router = new JSONObject();
			router.put("name", model.getName());
			params.put("router", router);
			Result result = this.doPut("/v2.0/routers/" + model.getRouterUuid(), params, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject routerObject = JSONObject.parseObject(result.getMessage(), JSONObject.class);
			return new BocloudResult(true, convertor.convertModel(routerObject.getJSONObject("router")), "修改成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "修改失败");
		}
	}

	@Override
	public BocloudResult attach(RouterModel model) {
		try {
			String subnetId = model.getSubnetUuid();
			JSONObject params = new JSONObject();

			if (!StringUtils.isEmpty(subnetId)) {
				params.put("subnet_id", subnetId);
			} else {
				params.put("port_id", model.getPortUuid());
			}
			Result result = this.doPut("/v2.0/routers/" + model.getRouterUuid() + "/add_router_interface", params,
					ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject portObject = JSONObject.parseObject(result.getMessage(), JSONObject.class);
			String portUuid = portObject.getString("port_id");
			Result portResult = this.doGet("/v2.0/ports/" + portUuid, null, ManageOneApiName.vpc,
					ManageOneApiType.network);
			PortModel portModel = null;
			if (portResult.isSuccess()) {
				JSONObject portObj = JSONObject.parseObject(portResult.getMessage());
				BeanConvertor<PortModel, JSONObject> portConvertor = new PortConvertor();
				portModel = portConvertor.convertModel(portObj.getJSONObject("port"));
			}
			return new BocloudResult(true, portModel, "添加接口成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "添加接口失败");
		}
	}

	@Override
	public BocloudResult detach(RouterModel model) {
		try {
			String subnetId = model.getSubnetUuid();
			JSONObject params = new JSONObject();
			if (!StringUtils.isEmpty(subnetId)) {
				params.put("subnet_id", subnetId);
			} else {
				params.put("port_id", model.getPortUuid());
			}
			Result result = this.doPut("/v2.0/routers/" + model.getRouterUuid() + "/remove_router_interface", params,
					ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject routerObject = JSONObject.parseObject(result.getMessage(), JSONObject.class);
			return new BocloudResult(true, convertor.convertModel(routerObject), "添加接口成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "删除接口失败");
		}
	}

	@Override
	public BocloudResult config(RouterModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult clear(String routerId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult list() {

		try {
			Result result = this.doGet("/v2.0/routers", null, ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "路由列表为空");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("routers"), JSONObject.class);
			return new BocloudResult(true, convertor.convertList(objects), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

}
