package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.ManageOneApiName;
import com.bocloud.cmp.common.enums.ManageOneApiType;
import com.bocloud.cmp.manageoneconvertor.SubnetConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

/**
 * 华为manage one子网操作driver
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年9月17日
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneSubnetProvider extends ManageOneProvider implements SubnetProvider {

	private static final Logger logger = LoggerFactory.getLogger(ManageOneSubnetProvider.class);

	private BeanConvertor<SubnetModel, JSONObject> convertor = new SubnetConvertor();
	private Butler butler;

	/**
	 * @param butler
	 */
	public ManageOneSubnetProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult list() {
		try {
			logger.info("find subnet !");
			JSONObject params = new JSONObject();
			params.put("network_id", "9d2f8b4c-969f-49ba-bbfb-c26824b27284");
			Result result = this.doGet("/v2.0/subnets", params, ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "子网列表为空");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("subnets"), JSONObject.class);
			System.out.println(JSON.toJSONString(objects));
			return new BocloudResult(true, convertor.convertList(objects), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult create(SubnetModel subnet) {
		try {
			JSONObject subnetModel = new JSONObject();
			subnetModel.put("name", subnet.getName());
			subnetModel.put("enable_dhcp", subnet.getDhcpEnabled());
			subnetModel.put("vpc_id", subnet.getRouterUuid());
			// 设置dns
			List<String> dnsServers = new ArrayList<String>();
			if (!StringUtils.isEmpty(subnet.getDnsNames())) {
				dnsServers = JSON.parseArray(subnet.getDnsNames()).toJavaList(String.class);
				subnetModel.put("dnsList", dnsServers);
			}
			List<JSONObject> ipPools = new ArrayList<JSONObject>();
			if (!StringUtils.isEmpty(subnet.getIpPools())) {
				List<String> list = JSONArray.parseArray(subnet.getIpPools(), String.class);
				for (String ipPool : list) {
					String[] startEndIp = ipPool.split("-");
					JSONObject object = new JSONObject();
					object.put("start", startEndIp[0]);
					object.put("end", startEndIp[1]);
					ipPools.add(object);
				}
			}
			subnetModel.put("allocation_pools", ipPools);
			String gateway = subnet.getGateway();
			if (!StringUtils.isEmpty(gateway)) {
				subnetModel.put("gateway_ip", gateway);
			}
			String ipVersion = subnet.getIpVersion();
			if (StringUtils.isEmpty(ipVersion) || "V4".equalsIgnoreCase(ipVersion)) {
				subnetModel.put("ipv4_enable", true);
			} else {
				subnetModel.put("pv6_enable", true);
			}
			subnetModel.put("cidr", subnet.getCidr());

			JSONObject params = new JSONObject();
			params.put("subnet", subnetModel);
			Result result = this.doPost("/v1/" + butler.getProjectId() + "/subnets", params, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("subnet");
			SubnetModel convertModel = convertModel(jsonObject);
			String subnetUuid = getSubnetUuid(convertModel.getNetworkUuid());
			if (null != subnetUuid) {
				convertModel.setSubnetUuid(subnetUuid);
			}
			return new BocloudResult(true, convertModel, "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "创建失败");
		}
	}

	public String getSubnetUuid(String networkUuid) {
		try {
			JSONObject params = new JSONObject();
			params.put("network_id", networkUuid);
			Result result = this.doGet("/v2.0/subnets", params, ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return null;
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return null;
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("subnets"), JSONObject.class);
			if (!ListTool.isEmpty(objects)) {
				JSONObject jsonObject = objects.get(0);
				return jsonObject.getString("id");
			}
		} catch (Exception e) {
			logger.error("Get error message：", e);
		}
		return null;
	}

	public SubnetModel convertModel(JSONObject model) {
		SubnetModel subnet = new SubnetModel();
		subnet.setRouterUuid(model.getString("vpc_id"));
		subnet.setVpcUuid(model.getString("vpc_id"));
		subnet.setName(model.getString("name"));
		subnet.setCidr(model.getString("cidr"));
		subnet.setSubnetUuid(model.getString("neutron_subnet_id"));

		List<String> ipPool = new ArrayList<String>();
		String ipPools = model.getString("allocation_pools");
		if (!StringUtils.isEmpty(ipPools)) {
			List<JSONObject> pools = JSONArray.parseArray(ipPools, JSONObject.class);
			for (JSONObject pool : pools) {
				ipPool.add(pool.getString("start") + "-" + pool.getString("end"));
			}
		}
		subnet.setIpPools(JSONArray.toJSONString(ipPool));
		subnet.setDhcpEnabled(model.getBoolean("dhcp_enable"));
		// manageone 组合接口 返回的ID 实际是底层network的id
		subnet.setNetworkUuid(model.getString("id"));
		subnet.setGateway(model.getString("gateway_ip"));
		subnet.setStatus(model.getString("status"));
		subnet.setDnsNames(model.getString("dnsList"));
		String ipVersion = model.getString("ip_version");
		if (StringUtils.isEmpty(ipVersion)) {
			subnet.setIpVersion("4");
		}
		subnet.setHostRoutes(model.getString("host_routes")); // 类型为数组类型

		return subnet;
	}

	@Override
	public BocloudResult modify(SubnetModel subnet) {
		try {
			logger.info("modify subnet ");
			JSONObject subnetParam = new JSONObject();
			subnetParam.put("name", subnet.getName());
			subnetParam.put("enable_dhcp", subnet.getDhcpEnabled());
			if (!StringUtils.isEmpty(subnet.getDnsNames())) {
				String[] dnsServers = JSON.parseObject(subnet.getDnsNames(), String[].class);
				subnetParam.put("dns_nameservers", dnsServers);
			}
			JSONObject params = new JSONObject();
			params.put("subnet", subnetParam);
			Result result = this.doPut("/v2.0/subnets/" + subnet.getSubnetUuid(), params, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("subnet");
			return new BocloudResult(true, convertor.convertModel(jsonObject), "修改成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "修改失败！");
		}
	}

	@Override
	public BocloudResult remove(String subnetId) {
		try {
			// vpc组合接口
			// Result result = this.doDelete("/v1/" + butler.getProjectId() +
			// "/subnets/" + subnetId, null,
			// ManageOneApiName.vpc, ManageOneApiType.network);
			Result result = this.doDelete("/v2.0/subnets/" + subnetId, null, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, result.getMessage(), "删除成功！");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "删除失败！");
		}
	}

	@Override
	public BocloudResult detail(String subnetId) {
		try {
			Result result = this.doGet("/v2.0/subnets/" + subnetId, null, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("subnet");
			return new BocloudResult(true, convertor.convertModel(jsonObject), "查询详情成功！");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败！");
		}
	}

	@Override
	public BocloudResult remove(SubnetModel subnet) {
		try {
			// Result result = this.doDelete("/v2.0/subnets/" + subnetId, null,
			// ManageOneApiName.vpc,
			// ManageOneApiType.network);
			// vpc组合接口
			Result result = this.doDelete("/v1/" + butler.getProjectId() + "/subnets/" + subnet.getNetworkUuid(), null,
					ManageOneApiName.vpc, ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, result.getMessage(), "删除成功！");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "删除失败！");
		}
	}

}
