package com.bocloud.cmp.provider.security;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.ManageOneApiName;
import com.bocloud.cmp.common.enums.ManageOneApiType;
import com.bocloud.cmp.manageoneconvertor.FirewallConvertor;
import com.bocloud.cmp.manageoneconvertor.FirewallPolicyConvertor;
import com.bocloud.cmp.manageoneconvertor.FirewallRuleConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FirewallModel;
import com.bocloud.cmp.model.FirewallPolicyModel;
import com.bocloud.cmp.model.FirewallRuleModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

/**
 * managerone firewall
 * 
 * @author admin
 *
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneFirewallProvider extends ManageOneProvider implements FirewallProvider {

    public ManageOneFirewallProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(ManageOneFirewallProvider.class);
    private FirewallConvertor firewallConvertor = new FirewallConvertor();
    private FirewallPolicyConvertor firewallPolicyConvertor = new FirewallPolicyConvertor();
    private FirewallRuleConvertor firewallRuleConvertor = new FirewallRuleConvertor();

    @Override
    public BocloudResult list() {
        try {
            Result result = doGet("/v2.0/fw/firewalls", null, ManageOneApiName.vpc, ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            logger.info("firewalls : " + object.toJSONString());
            if (null == object) {
                return new BocloudResult(true, "防火墙列表为空");
            }
            List<JSONObject> objects = JSONArray.parseArray(object.getString("firewalls"), JSONObject.class);
            return new BocloudResult(true, firewallConvertor.convertList(objects), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(FirewallModel firewall) {
        try {
            String policyUuid = firewall.getPolicyUuid();
            FirewallPolicyModel firewallPolicyModel = null;
            if (StringUtils.isEmpty(policyUuid)) {
                BocloudResult createPolicy = this.createPolicy(new FirewallPolicyModel());
                if (createPolicy.isFailed()) {
                    return createPolicy;
                }
                firewallPolicyModel = (FirewallPolicyModel)createPolicy.getData();
                policyUuid = firewallPolicyModel.getPolicyUuid();
            }
            JSONObject params = new JSONObject();
            JSONObject fw = new JSONObject();
            fw.put("name", firewall.getName());
            List<String> routerUuids = firewall.getRouterUuids();
            if (ListTool.isEmpty(routerUuids)) {
                routerUuids = new ArrayList<>();
            }
            fw.put("router_ids", routerUuids);
            String remark = firewall.getRemark();
            if (!StringUtils.isEmpty(remark)) {
                fw.put("description", remark);
            }
            fw.put("admin_state_up", true);
            fw.put("firewall_policy_id", policyUuid);
            params.put("firewall", fw);
            logger.info("Create firewall params:{}", JSONObject.toJSON(params));
            Result result = this.doPost("/v2.0/fw/firewalls", params, ManageOneApiName.vpc, ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "创建失败");
            }
            JSONObject fwObj = object.getJSONObject("firewall");
            BocloudResult waitFirewallStatus = this.waitFirewallStatus(fwObj.getString("id"));
            if (waitFirewallStatus.isFailed()) {
                return new BocloudResult(false, "防火墙创建失败：" + waitFirewallStatus.getMessage());
            }
            FirewallModel model = (FirewallModel)waitFirewallStatus.getData();
            model.setFirewallPolicyModel(firewallPolicyModel);
            return new BocloudResult(true, model, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult remove(String firewallId) {
        try {
            Result result =
                this.doGet("/v2.0/fw/firewalls/" + firewallId, null, ManageOneApiName.vpc, ManageOneApiType.network);
            if (result.isFailed()) {
                String message = result.getMessage();
                if (!StringUtils.isEmpty(message)
                    && message.contains("Firewall " + firewallId + " could not be found")) {
                    return new BocloudResult(true, "删除成功！");
                }
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(true, "删除成功！");
            }
            FirewallModel firewallModel = firewallConvertor.convertModel(object.getJSONObject("firewall"));
            Result delResult =
                doDelete("/v2.0/fw/firewalls/" + firewallId, null, ManageOneApiName.vpc, ManageOneApiType.network);
            if (delResult.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            removePolicy(firewallModel.getPolicyUuid());
            return new BocloudResult(true, "删除成功！");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            e.printStackTrace();
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult update(FirewallModel firewall) {
        try {
            JSONObject params = new JSONObject();
            JSONObject fw = new JSONObject();
            fw.put("name", firewall.getName());
            String remark = firewall.getRemark();
            if (!StringUtils.isEmpty(remark)) {
                fw.put("description", remark);
            }
            // 添加/移除路由
            String operation = firewall.getOperation();
            if (!StringUtils.isEmpty(operation)) {
                BocloudResult detail = this.detail(firewall.getFirewallUuid());
                if (detail.isFailed()) {
                    return detail;
                }
                FirewallModel firewallModel = (FirewallModel)detail.getData();
                List<String> routerUuids =
                    firewallModel.getRouterUuids() == null ? new ArrayList<>() : firewallModel.getRouterUuids();
                if ("add".equalsIgnoreCase(operation)) { // 关联路由
                    if (null != routerUuids && routerUuids.contains(firewall.getRouterUuid())) {
                        return new BocloudResult(false, "该VPC已经被关联！");
                    }
                    routerUuids.add(firewall.getRouterUuid());
                }
                if ("remove".equalsIgnoreCase(operation)) { // 取消关联
                    routerUuids.remove(firewall.getRouterUuid());
                }
                fw.put("router_ids", routerUuids);
            }
            params.put("firewall", fw);
            Result result = this.doPut("/v2.0/fw/firewalls/" + firewall.getFirewallUuid(), params, ManageOneApiName.vpc,
                ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "创建失败");
            }
            JSONObject fwObj = object.getJSONObject("firewall");
            return new BocloudResult(true, this.waitFirewallStatus(fwObj.getString("id")).getData(), "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult relevanceRouter(FirewallModel firewall) {
        try {
            JSONObject params = new JSONObject();
            JSONObject fw = new JSONObject();
            // 添加/移除路由
            String operation = firewall.getOperation();
            String message = "关联操作";
            if (!StringUtils.isEmpty(operation)) {
                BocloudResult detail = this.detail(firewall.getFirewallUuid());
                if (detail.isFailed()) {
                    return detail;
                }
                FirewallModel firewallModel = (FirewallModel)detail.getData();
                List<String> routerUuids =
                    firewallModel.getRouterUuids() == null ? new ArrayList<>() : firewallModel.getRouterUuids();
                if ("add".equalsIgnoreCase(operation)) { // 关联路由
                    routerUuids.addAll(firewall.getRouterUuids());
                    message = "关联操作";
                }
                if ("remove".equalsIgnoreCase(operation)) { // 取消关联
                    routerUuids.removeAll(firewall.getRouterUuids());
                    message = "取消关联";
                }
                fw.put("router_ids", routerUuids);
            }
            params.put("firewall", fw);
            Result result = this.doPut("/v2.0/fw/firewalls/" + firewall.getFirewallUuid(), params, ManageOneApiName.vpc,
                ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, message + "失败！");
            }
            JSONObject fwObj = object.getJSONObject("firewall");
            return new BocloudResult(true, this.waitFirewallStatus(fwObj.getString("id")).getData(), message + "成功！");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "操作失败！");
        }
    }

    @Override
    public BocloudResult detail(String firewallId) {
        try {
            Result result =
                this.doGet("/v2.0/fw/firewalls/" + firewallId, null, ManageOneApiName.vpc, ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "数据不存在");
            }
            return new BocloudResult(true, firewallConvertor.convertModel(object.getJSONObject("firewall")), "查询成功！");
        } catch (Exception e) {
            logger.error("查询防火墙详情异常：" + e.getMessage());
            return new BocloudResult(false, "查询防火墙详情异常：" + e.getMessage());
        }
    }

    public BocloudResult waitFirewallStatus(String firewallId) {
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();
        FirewallModel model = null;
        String content = "查询详情成功";
        Boolean resultFlag = false;
        while (flag) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                logger.error("线程睡觉bug： " + e.getMessage());
            }
            Result result =
                this.doGet("/v2.0/fw/firewalls/" + firewallId, null, ManageOneApiName.vpc, ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "数据不存在");
            }
            model = firewallConvertor.convertModel(object.getJSONObject("firewall"));
            if ("ACTIVE".equalsIgnoreCase(model.getStatus()) || "ERROR".equalsIgnoreCase(model.getStatus())
                || "RUNNING".equalsIgnoreCase(model.getStatus())) {
                flag = false;
                resultFlag = true;
                break;
            }
            Long endTime = System.currentTimeMillis();
            // 等待5分钟
            if (endTime - startTime > 30 * 60 * 1000) {
                flag = false;
                content = "操作超时！";
            }
        }
        return new BocloudResult(resultFlag, model, content);
    }

    @Override
    public BocloudResult listPolicy() {
        try {
            Result result = doGet("/v2.0/fw/firewall_policies", null, ManageOneApiName.vpc, ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(true, "防火墙策略列表为空");
            }
            List<JSONObject> objects = JSONArray.parseArray(object.getString("firewall_policies"), JSONObject.class);
            if (ListTool.isEmpty(objects)) {
                return new BocloudResult(true, "防火墙策略列表为空");
            }
            return new BocloudResult(true, firewallPolicyConvertor.convertList(objects), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult createPolicy(FirewallPolicyModel policyModel) {
        try {
            JSONObject paramObject = new JSONObject();
            paramObject.put("shared", Boolean.FALSE);
            paramObject.put("audited", Boolean.FALSE);
            String policyName = policyModel.getName();
            if (StringUtils.isEmpty(policyName)) {
                policyName = "firewall_policy";
            }
            paramObject.put("name", policyName);
            JSONObject params = new JSONObject();
            params.put("firewall_policy", paramObject);
            Result result =
                this.doPost("/v2.0/fw/firewall_policies", params, ManageOneApiName.vpc, ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "创建失败");
            }
            JSONObject parseObject = JSONObject.parseObject(object.getString("firewall_policy"));
            if (null == parseObject) {
                return new BocloudResult(false, "防火墙策略创建异常!");
            }
            return new BocloudResult(true, firewallPolicyConvertor.convertModel(parseObject), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult insertRule(String policyId, String ruleId, String type, String referRuleId) {
        try {
            JSONObject object = new JSONObject();
            object.put("firewall_rule_id", ruleId);
            if ("BEFORE".equalsIgnoreCase(type)) {
                object.put("insert_after", "");
                object.put("insert_before", referRuleId);
            } else if ("AFTER".equalsIgnoreCase(type)) {
                object.put("insert_after", referRuleId);
                object.put("insert_before", "");
            } else {
                object.put("insert_before", "");
                object.put("insert_after", "");
            }
            logger.info("policyId : " + policyId);
            logger.info("params : " + object);
            Result result = this.doPut("/v2.0/fw/firewall_policies/" + policyId + "/insert_rule", object,
                ManageOneApiName.vpc, ManageOneApiType.network);
            logger.info("result : " + result.getMessage());
            if (null == result || result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            BocloudResult detail = detailPolicy(policyId);
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "insert firewall rule in policy success");
            } else {
                return new BocloudResult(false, "插入规则失败！");
            }
        } catch (Exception e) {
            logger.error("插入规则失败:{}", e);
            return new BocloudResult(false, "插入规则失败！【" + e.getMessage() + "】");
        }

    }

    @Override
    public BocloudResult removePolicy(String policyId) {
        try {
            BocloudResult detailPolicy = this.detailPolicy(policyId);
            FirewallPolicyModel firewallPolicyModel = null;
            if (detailPolicy.isSuccess()) {
                firewallPolicyModel = (FirewallPolicyModel)detailPolicy.getData();
            }
            Result result = this.doDelete("/v2.0/fw/firewall_policies/" + policyId, null, ManageOneApiName.vpc,
                ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            // 删除当前策略绑定的rule
            if (null != firewallPolicyModel) {
                List<String> ruleUuids = firewallPolicyModel.getRuleUuids();
                if (!ListTool.isEmpty(ruleUuids)) {
                    for (String ruleUuid : ruleUuids) {
                        removeRule(ruleUuid);
                    }
                }
            }
            return new BocloudResult(true, "防火墙策略删除成功！");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult removeRule(String policyId, String ruleId) {
        try {
            JSONObject params = new JSONObject();
            params.put("firewall_rule_id", ruleId);
            Result result = this.doPut("/v2.0/fw/firewall_policies/" + policyId + "/remove_rule", params,
                ManageOneApiName.vpc, ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            this.removeRule(ruleId);
            return new BocloudResult(true, "防火墙规则移除成功！");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult updatePolicy(FirewallPolicyModel policyModel) {
        return null;
    }

    @Override
    public BocloudResult detailPolicy(String policyId) {
        try {
            Result result =
                doGet("/v2.0/fw/firewall_policies/" + policyId, null, ManageOneApiName.vpc, ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "防火墙策略为空");
            }
            JSONObject policyObject = JSONArray.parseObject(object.getString("firewall_policy"), JSONObject.class);
            if (null == policyObject) {
                return new BocloudResult(false, "防火墙策略为空");
            }
            return new BocloudResult(true, firewallPolicyConvertor.convertModel(policyObject), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult listRule() {
        try {
            Result result = doGet("/v2.0/fw/firewall_rules", null, ManageOneApiName.vpc, ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(true, "防火墙规则列表为空");
            }
            List<JSONObject> objects = JSONArray.parseArray(object.getString("firewall_rules"), JSONObject.class);
            if (ListTool.isEmpty(objects)) {
                return new BocloudResult(true, "防火墙规则列表为空");
            }
            return new BocloudResult(true, firewallRuleConvertor.convertList(objects), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult createRule(FirewallRuleModel ruleModel) {
        try {
            JSONObject object = new JSONObject();
            object.put("name", ruleModel.getName());
            object.put("description", ruleModel.getRemark());
            object.put("shared", ruleModel.getShared() == null ? false : ruleModel.getShared());
            String protocol = ruleModel.getProtocol();
            if (null != protocol && !"any".equalsIgnoreCase(protocol)) {
                object.put("protocol", protocol.toLowerCase());
            }
            object.put("ip_version", "V6".equalsIgnoreCase(ruleModel.getIpVersion()) ? 6 : 4);

            List<String> sourceIpAddresses = new ArrayList<>();
            sourceIpAddresses.add(ruleModel.getSourceIp());
            object.put("source_ip_addresses", sourceIpAddresses);
            List<String> destinationIpAddresses = new ArrayList<>();
            destinationIpAddresses.add(ruleModel.getDestIp());
            object.put("destination_ip_addresses", destinationIpAddresses);

            String sourcePorts = ruleModel.getSourcePort();
            if (!StringUtils.isEmpty(sourcePorts)) {
                if (sourcePorts.contains("-")) {
                    sourcePorts = sourcePorts.replaceAll("-", ":");
                }
                object.put("source_ports", sourcePorts);
            }
            String destinationPorts = ruleModel.getDestPort();
            if (!StringUtils.isEmpty(destinationPorts)) {
                if (destinationPorts.contains("-")) {
                    destinationPorts = destinationPorts.replaceAll("-", ":");
                }
                object.put("destination_ports", destinationPorts);
            }

            object.put("action", ruleModel.getAction() == null ? "allow" : ruleModel.getAction().toLowerCase());
            object.put("enabled", ruleModel.getEnabled());
            JSONObject params = new JSONObject();
            params.put("firewall_rule", object);
            Result result =
                this.doPost("/v2.0/fw/firewall_rules", params, ManageOneApiName.vpc, ManageOneApiType.network);
            logger.info("result : " + result.getMessage());
            if (null == result || result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject resultObject = JSONObject.parseObject(result.getMessage());
            if (null == resultObject) {
                return new BocloudResult(false, "创建防火规则失败");
            }
            JSONObject fwObj = resultObject.getJSONObject("firewall_rule");
            if (null == fwObj) {
                return new BocloudResult(false, "创建防火墙规则失败！");
            }
            String id = fwObj.getString("id");
            BocloudResult detail = detailRule(id);
            BocloudResult insertRule =
                insertRule(ruleModel.getPolicyUuid(), id, ruleModel.getType(), ruleModel.getReferRuleId());
            if (insertRule.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "防火墙规则创建成功！");
            } else {
                return new BocloudResult(false, "创建失败！");
            }
        } catch (Exception e) {
            logger.error("防火墙规则创建异常：" + e.getMessage());
            return new BocloudResult(false, "防火墙规则创建异常：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult removeRule(String ruleId) {
        try {
            Result result = this.doDelete("/v2.0/fw/firewall_rules/" + ruleId, null, ManageOneApiName.vpc,
                ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, "防火墙规则删除成功！");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "防火墙规则删除失败");
        }
    }

    @Override
    public BocloudResult updateRule(FirewallRuleModel ruleModel) {
        try {
            JSONObject objectParam = new JSONObject();
            if (null != ruleModel.getEnabled()) {
                objectParam.put("enabled", ruleModel.getEnabled());
            }
            if (null != ruleModel.getRemark()) {
                objectParam.put("description", ruleModel.getRemark());
            }
            String protocol = ruleModel.getProtocol();
            if (null != protocol && !"any".equalsIgnoreCase(protocol)) {
                objectParam.put("protocol", protocol.toLowerCase());
            }

            if (null != ruleModel.getSourceIp()) {
                List<String> sourceIpAddresses = new ArrayList<>();
                sourceIpAddresses.add(ruleModel.getSourceIp());
                objectParam.put("source_ip_addresses", sourceIpAddresses);
            }
            if (null != ruleModel.getDestIp()) {
                List<String> destinationIpAddresses = new ArrayList<>();
                destinationIpAddresses.add(ruleModel.getDestIp());
                objectParam.put("destination_ip_addresses", destinationIpAddresses);
            }

            String sourcePorts = ruleModel.getSourcePort();
            if (!StringUtils.isEmpty(sourcePorts)) {
                if (sourcePorts.contains("-")) {
                    sourcePorts = sourcePorts.replaceAll("-", ":");
                }
                objectParam.put("source_ports", sourcePorts);
            }
            String destinationPorts = ruleModel.getDestPort();
            if (!StringUtils.isEmpty(destinationPorts)) {
                if (destinationPorts.contains("-")) {
                    destinationPorts = destinationPorts.replaceAll("-", ":");
                }
                objectParam.put("destination_ports", destinationPorts);
            }
            if (null != ruleModel.getAction()) {
                objectParam.put("action", ruleModel.getAction().toLowerCase());
            }

            JSONObject params = new JSONObject();
            params.put("firewall_rule", objectParam);
            Result result = doPut("/v2.0/fw/firewall_rules/" + ruleModel.getRuleUuid(), params, ManageOneApiName.vpc,
                ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "防火墙规则不存在！");
            }
            JSONObject objects = JSONArray.parseObject(object.getString("firewall_rule"), JSONObject.class);
            if (null == objects) {
                return new BocloudResult(false, "防火墙规则不存在！");
            }
            return new BocloudResult(true, firewallRuleConvertor.convertModel(objects), "修改成功！");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult detailRule(String ruleId) {
        try {
            Result result =
                doGet("/v2.0/fw/firewall_rules/" + ruleId, null, ManageOneApiName.vpc, ManageOneApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "防火墙规则不存在！");
            }
            JSONObject objects = JSONArray.parseObject(object.getString("firewall_rule"), JSONObject.class);
            if (null == objects) {
                return new BocloudResult(false, "防火墙规则不存在！");
            }
            return new BocloudResult(true, firewallRuleConvertor.convertModel(objects), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public boolean accessable() {
        return super.accessable();
    }

}
