package com.bocloud.cmp.provider.storage;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.ManageOneApiName;
import com.bocloud.cmp.common.enums.ManageOneApiType;
import com.bocloud.cmp.manageoneconvertor.SnapshotConvertor;
import com.bocloud.cmp.manageoneconvertor.VolumeConvertor;
import com.bocloud.cmp.manageoneconvertor.VolumeTypeConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.VolumeTypeModel;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ResultTools;

/**
 * 华为私有云卷操作driver
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年9月17日
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneVolumeProvider extends ManageOneProvider implements VolumeProvider {

	public ManageOneVolumeProvider(Butler butler) {
		super(butler);
	}

	private static final Logger logger = LoggerFactory.getLogger(ManageOneVolumeProvider.class);

	private BeanConvertor<VolumeTypeModel, JSONObject> volumeTypeConvertor = new VolumeTypeConvertor();
	private BeanConvertor<VolumeModel, JSONObject> volumeConvertor = new VolumeConvertor();
	private BeanConvertor<SnapshotModel, JSONObject> snapshotConvertor = new SnapshotConvertor();

	@Override
	public BocloudResult list() {
		try {
			Result result = this.doGet("/volumes/detail", null, ManageOneApiName.evsv2, ManageOneApiType.volumev2);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "硬盘列表为空");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("volumes"), JSONObject.class);
			return new BocloudResult(true, volumeConvertor.convertList(objects), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult create(VolumeModel volume) {
		try {
			JSONObject params = new JSONObject();
			params.put("availability_zone", volume.getZone());
			params.put("description", volume.getRemark());
			if (!StringUtils.isEmpty(volume.getType())) {
				params.put("volume_type", volume.getType());
			}
			params.put("name", volume.getName());
			params.put("size", volume.getSize());
			params.put("multiattach", volume.getMultiattach());
			params.put("source_volid", volume.getSourceVolidId()); // 源卷id
			params.put("snapshot_id", volume.getSourceSnapshotId());
			params.put("imageRef", volume.getSourceImgId());
			JSONObject volumeParam = new JSONObject();
			volumeParam.put("volume", params);
			Result result = this.doPost("/volumes", volumeParam, ManageOneApiName.evsv2, ManageOneApiType.volumev2);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("volume");
			return new BocloudResult(true, volumeConvertor.convertModel(jsonObject), "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "创建失败");
		}
	}

	@Override
	public BocloudResult modify(VolumeModel volume) {
		try {
			JSONObject params = new JSONObject();
			params.put("description", volume.getRemark());
			params.put("name", volume.getName());
			JSONObject volumeParam = new JSONObject();
			volumeParam.put("volume", params);
			Result result = this.doPut("/volumes/" + volume.getVolumeUuid(), volumeParam, ManageOneApiName.evsv2,
					ManageOneApiType.volumev2);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("volume");
			return new BocloudResult(true, volumeConvertor.convertModel(jsonObject), "修改成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "修改失败！");
		}
	}

	@Override
	public BocloudResult remove(String volumeId) {
		try {
			Result result = this.doDelete("/volumes/" + volumeId, null, ManageOneApiName.evsv2,
					ManageOneApiType.volumev2);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, result.getMessage(), "删除成功！");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "删除失败！");
		}
	}

	@Override
	public BocloudResult detail(String volumeId) {
		try {
			Result result = this.doGet("/volumes/" + volumeId, null, ManageOneApiName.evsv2, ManageOneApiType.volumev2);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("volume");
			return new BocloudResult(true, volumeConvertor.convertModel(jsonObject), "查询详情成功！");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败！");
		}
	}

	@Override
	public BocloudResult snapshot(SnapshotModel snapshotModel) {
		try {
			JSONObject obj = new JSONObject();
			obj.put("name", snapshotModel.getName());
			obj.put("description", snapshotModel.getRemark());
			obj.put("volume_id", snapshotModel.getVolumeUuid());
			obj.put("force", false);
			JSONObject params = new JSONObject();
			params.put("snapshot", obj);
			Result result = this.doPost("/snapshots", params, ManageOneApiName.evsv2, ManageOneApiType.volumev2);
			if (result.isFailed()) {
				JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("badRequest");
				if (null != jsonObject) {
					return new BocloudResult(false, jsonObject.getString("message"));
				} else {
					return new BocloudResult(false, "快照创建失败");
				}
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("snapshot");
			return new BocloudResult(true, snapshotConvertor.convertModel(jsonObject), "快照创建成功！");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "创建硬盘快照失败！");
		}
	}

	@Override
	public BocloudResult attach(String serverId, String volumeId) {
		return attach(serverId, null, volumeId);
	}

	@Override
	public BocloudResult detach(String serverId, String volumeId) {
		try {
			Result result = this.doDelete("/servers/" + serverId + "/os-volume_attachments/" + volumeId, null,
					ManageOneApiName.ecs, ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, result.getMessage(), "卸载成功！");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "卸载失败！");
		}
	}

	@Override
	public BocloudResult resetStatus(String volumeId, String status) {
		return ResultTools.noImplementedResult();
	}

	@Override
	public BocloudResult uploadToImage(String volumeId, ImageModel imageModel) {
		return null;
	}

	/**
	 * 扩容
	 */
	@Override
	public BocloudResult extend(String volumeId, Integer newSize) {
		try {
			JSONObject params = new JSONObject();
			params.put("new_size", newSize);
			JSONObject volumeParam = new JSONObject();
			volumeParam.put("os-extend", params);
			Result result = this.doPost("/volumes/" + volumeId + "/action", volumeParam, ManageOneApiName.evsv2,
					ManageOneApiType.volumev2);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, result.getMessage(), "扩容成功！");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "扩容失败！");
		}
	}

	/**
	 * mountpoint 挂载点
	 */
	@Override
	public BocloudResult attach(String serverId, String mountpoint, String volumeId) {
		try {
			JSONObject jsonObject = new JSONObject();
			jsonObject.put("volumeId", volumeId);
			jsonObject.put("device", mountpoint);
			JSONObject params = new JSONObject();
			params.put("volumeAttachment", jsonObject);
			Result result = this.doPost("/servers/" + serverId + "/os-volume_attachments", params, ManageOneApiName.ecs,
					ManageOneApiType.compute);
			if (result.isFailed()) {
				String message = "";
				JSONObject errRes = JSONObject.parseObject(result.getMessage()).getJSONObject("badRequest");
				if (null == errRes) {
					errRes = JSONObject.parseObject(result.getMessage()).getJSONObject("conflictingRequest");
				}
				if (null == errRes) {
					message = "硬盘挂载失败";
				} else {
					message = errRes.getString("message");
				}
				return new BocloudResult(false, message);
			}
			JSONObject res = JSONObject.parseObject(result.getMessage()).getJSONObject("volumeAttachment");
			return new BocloudResult(true, res.get("device"), "挂载成功！");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "挂载失败！");
		}
	}

	@Override
	public BocloudResult list(String hostId, String storagePoolName) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult remove(H3cVolumeModel volume) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult listVolumeTypes() {
		try {
			Result result = this.doGet("/types", null, ManageOneApiName.evsv2, ManageOneApiType.volumev2);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "硬盘列表为空");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("volume_types"), JSONObject.class);
			if (null == objects) {
				return new BocloudResult(false, "硬盘列表为空");
			}

			return new BocloudResult(true, volumeTypeConvertor.convertList(objects), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}
}
