package com.bocloud.cmp.provider.common;

import java.util.List;

import org.openstack4j.api.Builders;
import org.openstack4j.api.OSClient;
import org.openstack4j.model.identity.v2.Tenant;
import org.openstack4j.model.identity.v2.builder.TenantBuilder;
import org.openstack4j.model.identity.v3.Domain;
import org.openstack4j.model.identity.v3.Project;
import org.openstack4j.model.identity.v3.Role;
import org.openstack4j.model.identity.v3.User;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.OSTenantModel;
import com.bocloud.cmp.provider.OpenStackProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;

/**
 * openstack认证操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/15
 */
@Provider("bocloud.cmp.openstack")
public class OpenStackAuthProvider extends OpenStackProvider implements AuthProvider {

	private static Logger logger = LoggerFactory.getLogger(OpenStackAuthProvider.class);
	private Butler butler = null;

	public OpenStackAuthProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult auth() {
		if (this.isAccessable()) {
			return new BocloudResult(true, "" + this.isAccessable());
		}
		return new BocloudResult(this.isAccessable(), "认证信息失败");
	}

	@Override
	public BocloudResult createProject(OSTenantModel model) {
		try {
			String projectName = model.getName();
			OSTenantModel osTenant = null;
			OSClient<?> client = this.getClient();
			if (client instanceof OSClient.OSClientV2) {
				OSClient.OSClientV2 client2 = (OSClient.OSClientV2) client;
				// 查询用户
				org.openstack4j.model.identity.v2.User user = client2.identity().users()
						.getByName(this.butler.getUsername());
				// 查询用户角色列表
				List<? extends org.openstack4j.model.identity.v2.Role> roles = client2.identity().roles()
						.listRolesForUser(user.getId());
				Tenant oldTenant = client2.identity().tenants().getByName(projectName);
				if (null != oldTenant) {
					for (org.openstack4j.model.identity.v2.Role role : roles) {
						client2.identity().roles().addUserRole(oldTenant.getId(), user.getId(), role.getId());
					}
					osTenant = new OSTenantModel(oldTenant.getName(), oldTenant.getDescription(), oldTenant.getId(),
							oldTenant.isEnabled(), null, null);
					return new BocloudResult(true, osTenant, "租户查询成功");
				}
				Tenant tenant = client2.identity().tenants().create(Builders.identityV2().tenant()
						.description(model.getRemark()).enabled(true).name(projectName).build());
				if (null == tenant) {
					return new BocloudResult(false, "租户创建失败");
				}
				osTenant = new OSTenantModel(tenant.getName(), tenant.getDescription(), tenant.getId(),
						tenant.isEnabled(), null, null);
				for (org.openstack4j.model.identity.v2.Role role : roles) {
					client2.identity().roles().addUserRole(tenant.getId(), user.getId(), role.getId());
				}
			} else if (client instanceof OSClient.OSClientV3) {
				OSClient.OSClientV3 client3 = (OSClient.OSClientV3) client;
				// 查询域
				List<? extends Domain> domains = client3.identity().domains().getByName(butler.getDomainName());
				Domain domain = domains.get(0);
				// 查询用户
				User user = client3.identity().users().getByName(this.butler.getUsername(), domain.getId());
				// 查询user的角色列表
				List<? extends Role> roles = client3.identity().roles().list();
				List<? extends Project> projects = client3.identity().projects().getByName(projectName);
				if (!ListTool.isEmpty(projects)) {
					for (Role role : roles) {
						for (Project project : projects) {
							client3.identity().roles().grantProjectUserRole(project.getId(), user.getId(),
									role.getId());
						}
					}
					Project project = projects.get(0);
					Domain projectDomain = project.getDomain();
					String domainName = null;
					if (null != projectDomain) {
						domainName = projectDomain.getName();
					} else {
						domainName = project.getDomainId();
					}
					osTenant = new OSTenantModel(project.getName(), project.getDescription(), project.getId(),
							project.isEnabled(), project.getDomainId(), domainName);
					return new BocloudResult(true, osTenant, "项目查询成功");
				}
				Project project = client3.identity().projects().create(
						client3.identity().domains().getByName(this.butler.getDomainName()).get(0).getId(), projectName,
						model.getRemark(), true);
				if (null == project) {
					return new BocloudResult(false, "项目创建失败");
				}
				Domain projectDomain = project.getDomain();
				String domainName = null;
				if (null != projectDomain) {
					domainName = projectDomain.getName();
				} else {
					domainName = project.getDomainId();
				}
				osTenant = new OSTenantModel(project.getName(), project.getDescription(), project.getId(),
						project.isEnabled(), project.getDomainId(), domainName);
				for (Role role : roles) { // domain_admin admin
					if ("admin".equalsIgnoreCase(role.getName()) || "domain_admin".equalsIgnoreCase(role.getName())) {
						client3.identity().roles().grantProjectUserRole(project.getId(), user.getId(), role.getId());
					}
				}
			}
			return new BocloudResult(true, osTenant, "项目创建成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "项目创建失败");
		}
	}

	@Override
	public BocloudResult removeProject(String projectName) {
		try {
			OSClient<?> client = this.getClient();
			if (client instanceof OSClient.OSClientV2) {
				OSClient.OSClientV2 client2 = (OSClient.OSClientV2) client;
				Tenant oldTenant = client2.identity().tenants().getByName(projectName);
				if (null == oldTenant) {
					return new BocloudResult(true, "租户删除成功");
				}
				client2.identity().tenants().delete(oldTenant.getId());
			} else if (client instanceof OSClient.OSClientV3) {
				OSClient.OSClientV3 client3 = (OSClient.OSClientV3) client;
				List<? extends Project> projects = client3.identity().projects().getByName(projectName);
				if (ListTool.isEmpty(projects)) {
					return new BocloudResult(true, "项目删除成功");
				}
				for (Project project : projects) {
					client3.identity().projects().delete(project.getId());
				}
			}
			return new BocloudResult(true, "项目移除成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "项目移除失败");
		}
	}

	@Override
	public BocloudResult modifyProject(OSTenantModel model) {
		try {
			String projectName = model.getName();
			OSTenantModel osTenant = null;
			OSClient<?> client = this.getClient();
			if (client instanceof OSClient.OSClientV2) {
				OSClient.OSClientV2 client2 = (OSClient.OSClientV2) client;
				Tenant tenant = Builders.identityV2().tenant().id(model.getTenantUuid()).name(projectName)
						.description(model.getRemark()).build();
				Tenant update = client2.identity().tenants().update(tenant);
				osTenant = new OSTenantModel(update.getName(), update.getDescription(), null, null, null, null);
				return new BocloudResult<>(true, osTenant, "项目编辑成功");
			} else if (client instanceof OSClient.OSClientV3) {
				OSClient.OSClientV3 client3 = (OSClient.OSClientV3) client;
				Project build = Builders.identityV3().project().name(projectName).description(model.getRemark())
						.id(model.getTenantUuid()).build();
				Project update = client3.identity().projects().update(build);
				osTenant = new OSTenantModel(update.getName(), update.getDescription(), null, null, null, null);
				return new BocloudResult(true, osTenant, "项目编辑成功");
			}
			return new BocloudResult<>(false, "项目编辑失败");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "项目创建失败");
		}
	}

}
