package com.bocloud.cmp.provider.common;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.openstack4j.api.Builders;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.heat.Event;
import org.openstack4j.model.heat.Resource;
import org.openstack4j.model.heat.Stack;
import org.openstack4j.model.heat.builder.StackCreateBuilder;
import org.openstack4j.model.heat.builder.StackUpdateBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.yaml.snakeyaml.Yaml;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.HeatStackModel;
import com.bocloud.cmp.provider.OpenStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.HeatStackConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * op heat service stack
 * 
 * @author ymh
 * @时间 2019年11月13日
 */
@Provider("bocloud.cmp.openstack")
public class OpenStackHeatProvider extends OpenStackProvider implements HeatProvider {

    private static Logger logger = LoggerFactory.getLogger(OpenStackHeatProvider.class);

    private BeanConvertor<HeatStackModel, Stack> convertor = new HeatStackConvertor();

    public OpenStackHeatProvider(Butler butler) {
        super(butler);
    }

    /**
     * post /v1/{tenant_id}/stacks
     */
    @Override
    public BocloudResult createStack(HeatStackModel heatStackModel) {
        try {
            if (null == heatStackModel.getTemplate()) {
                return new BocloudResult(false, "模板数据格式错误！");
            }
            StackCreateBuilder stackCreateBuilder = Builders.heat().stack();
            stackCreateBuilder.name(heatStackModel.getName()).template(heatStackModel.getTemplate());
            String environment = heatStackModel.getEnvironment();
            if (!StringUtils.isEmpty(environment)) {
                Map<String, String> files = new HashMap<String, String>();
                files.put("lb_env.yaml", environment);
                stackCreateBuilder.files(files);
            }
            Map<String, String> parameters = heatStackModel.getParameters();
            if (null != parameters && parameters.keySet().size() > 0) {
                stackCreateBuilder.parameters(parameters);
            }
            Stack create = this.getClient().heat().stacks().create(stackCreateBuilder.build());
            heatStackModel.setStackUuid(create.getId());
            BocloudResult createProcess = createProcess(heatStackModel);
            heatStackModel = (HeatStackModel) createProcess.getData();
            resources(heatStackModel);
            events(heatStackModel);
            templateInfo(heatStackModel);
            return new BocloudResult(true, heatStackModel, "创建成功！");
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("创建stack异常：" + e.getMessage());
            return new BocloudResult(false, "创建stack异常：" + e.getMessage());
        }
    }

    public BocloudResult createProcess(HeatStackModel heatStackModel) {
        BocloudResult bocloudResult = new BocloudResult(false, "查询失败！");
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    logger.error("Get error message:", e);
                    Thread.currentThread().interrupt();
                }
                bocloudResult = detailStack(heatStackModel);
                if (bocloudResult.isFailed()) {
                    flag = false;
                    break;
                }
                HeatStackModel model = (HeatStackModel) bocloudResult.getData();
                if ("CREATE_COMPLETE".equalsIgnoreCase(model.getStatus())
                        || "CREATE_FAILED".equalsIgnoreCase(model.getStatus())
                        || "UPDATE_FAILED".equalsIgnoreCase(model.getStatus())
                        || "UPDATE_COMPLETE".equalsIgnoreCase(model.getStatus())
                        || "SUSPEND_COMPLETE".equalsIgnoreCase(model.getStatus())
                        || "RESUME_COMPLETE".equalsIgnoreCase(model.getStatus())
                        || "CHECK_COMPLETE".equalsIgnoreCase(model.getStatus())
                        || "CHECK_FAILED".equalsIgnoreCase(model.getStatus())
                        || "SUSPEND_FAILED".equalsIgnoreCase(model.getStatus())
                        || "RESUME_FAILED".equalsIgnoreCase(model.getStatus())){
                    flag = false;
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 60 * 1000 * 30) {
                    flag = false;
                    break;
                }
            }
        } catch (Exception e) {
            logger.error("查询stack detail error : " + e.getMessage());
        }
        return bocloudResult;
    }

    /**
     * get /v1/{tenant_id}/stacks
     */
    @Override
    public BocloudResult listStack(HeatStackModel heatStackModel) {
        try {
            List<HeatStackModel> convertList = new ArrayList<>();
            List<? extends Stack> list = this.getClient().heat().stacks().list();
            for (Stack stack : list) {
                Stack details = this.getClient().heat().stacks().getDetails(stack.getName(), stack.getId());
                HeatStackModel convertModel = convertor.convertModel(details);
                resources(convertModel);
                events(convertModel);
                templateInfo(convertModel);
                convertList.add(convertModel);
            }
            return new BocloudResult(true, convertList, "查询成功！");
        } catch (Exception e) {
            logger.error("查询stack异常：" + e.getMessage());
            return new BocloudResult(false, "查询stack异常：" + e.getMessage());
        }
    }

    public HeatStackModel resources(HeatStackModel heatStackModel) {
        try {
            List<? extends Resource> resources = this.getClient().heat().resources()
                    .list(heatStackModel.getStackUuid());
            // OS::Nova::Server
            if (null != resources && resources.size() > 0) {
                heatStackModel.setResources(JSON.toJSONString(resources));
            }
        } catch (Exception e) {
            logger.error("获取栈Resource异常：" + e.getMessage());
        }
        return heatStackModel;
    }

    public HeatStackModel events(HeatStackModel heatStackModel) {
        try {
            List<? extends Event> list = this.getClient().heat().events().list(heatStackModel.getName(),
                    heatStackModel.getStackUuid());
            if (null != list && list.size() > 0) {
                heatStackModel.setEvents(JSON.toJSONString(list));
            }
        } catch (Exception e) {
            logger.error("获取栈Event异常：" + e.getMessage());
        }
        return heatStackModel;
    }

    public HeatStackModel templateInfo(HeatStackModel heatStackModel) {
        try {
            Map<String, Object> templateAsMap = this.getClient().heat().templates()
                    .getTemplateAsMap(heatStackModel.getName(), heatStackModel.getStackUuid());
            Yaml yaml = new Yaml();
            String dump = yaml.dump(templateAsMap);
            if (null != dump) {
                heatStackModel.setTemplate(dump);
            }
        } catch (Exception e) {
            logger.error("获取栈Template异常：" + e.getMessage());
        }
        return heatStackModel;
    }

    /**
     * get /v1/{tenant_id}/stacks/{stack_name}/{stack_id}
     */
    @Override
    public BocloudResult detailStack(HeatStackModel heatStackModel) {
        try {
            Stack details = this.getClient().heat().stacks().getDetails(heatStackModel.getName(),
                    heatStackModel.getStackUuid());
            if (null == details) {
                return new BocloudResult(false, "栈信息不存在！");
            }
            HeatStackModel convertModel = convertor.convertModel(details);
            resources(convertModel);
            events(convertModel);
            templateInfo(convertModel);
            return new BocloudResult(true, convertModel, "查询成功！");
        } catch (Exception e) {
            logger.error("查询栈信息异常：" + e.getMessage());
            return new BocloudResult(false, "查询栈信息异常：" + e.getMessage());
        }
    }

    /**
     * put /v1/{tenant_id}/stacks/{stack_name}/{stack_id}
     */
    @Override
    public BocloudResult modifyStack(HeatStackModel heatStackModel) {
        try {
            Stack details = this.getClient().heat().stacks().getDetails(heatStackModel.getName(),
                    heatStackModel.getStackUuid());
            if (null == details) {
                return new BocloudResult(false, "资源不存在！");
            }
            if ("UPDATE_IN_PROGRESS".equalsIgnoreCase(details.getStatus())
                    || "DELETE_IN_PROGRESS".equalsIgnoreCase(details.getStatus())) {
                return new BocloudResult(false, "当前资源存在任务...请稍后再试！");
            }
            StackUpdateBuilder stackUpdate = Builders.heat().stackUpdate();
            String environment = heatStackModel.getEnvironment();
            if (!StringUtils.isEmpty(environment)) {
                Map<String, String> files = new HashMap<String, String>();
                files.put("lb_env.yaml", environment);
                stackUpdate.files(files);
            }
            Map<String, String> parameters = heatStackModel.getParameters();
            if (null != parameters && parameters.keySet().size() > 0) {
                stackUpdate.parameters(parameters);
            }
            stackUpdate.template(heatStackModel.getTemplate());
            ActionResponse update = this.getClient().heat().stacks().update(heatStackModel.getName(),
                    heatStackModel.getStackUuid(), stackUpdate.build());
            BocloudResult createProcess = createProcess(heatStackModel);
            Thread.sleep(5000);
            heatStackModel = (HeatStackModel) createProcess.getData();
            resources(heatStackModel);
            events(heatStackModel);
            templateInfo(heatStackModel);
            return new BocloudResult(update.isSuccess(), heatStackModel, update.getFault());
        } catch (Exception e) {
            logger.error("编辑stack异常：" + e.getMessage());
            return new BocloudResult(false, "编辑stack异常：" + e.getMessage());
        }
    }

    /**
     * delete /v1/{tenant_id}/stacks/{stack_name}/{stack_id}
     */
    @Override
    public BocloudResult removeStack(HeatStackModel heatStackModel) {
        try {
            Stack details = this.getClient().heat().stacks().getDetails(heatStackModel.getName(),
                    heatStackModel.getStackUuid());
            if (null != details && "DELETE_COMPLETE".equalsIgnoreCase(details.getStatus())) {
                return new BocloudResult(true, "删除成功！");
            }
            ActionResponse delete = this.getClient().heat().stacks().delete(heatStackModel.getName(),
                    heatStackModel.getStackUuid());
            return new BocloudResult(delete.isSuccess(), heatStackModel, delete.getFault());
        } catch (Exception e) {
            logger.error("删除stack异常：" + e.getMessage());
            return new BocloudResult(false, "删除stack异常：" + e.getMessage());
        }
    }

    /**
     * get /v1/{tenant_id}/template_versions
     */
    @Override
    public BocloudResult templateVersions(HeatStackModel heatStackModel) {
        try {
            Result result = this.doGet("/template_versions", null, "heat", "orchestration");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "数据不存在");
            }
            List<JSONObject> list = JSONArray.parseArray(object.getString("template_versions"), JSONObject.class);
            List<HeatStackModel> versions = null;
            if (null != list && list.size() > 0) {
                versions = new ArrayList<HeatStackModel>();
                for (JSONObject jsonObject : list) {
                    HeatStackModel templateConvertor = templateConvertor(jsonObject);
                    versions.add(templateVersionFunctions(templateConvertor));
                }
            }
            return new BocloudResult(true, versions, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    public HeatStackModel templateVersionFunctions(HeatStackModel heatStackModel) {
        try {
            Result result = this.doGet("/template_versions/" + heatStackModel.getVersion() + "/functions", null, "heat",
                    "orchestration");
            if (result.isFailed()) {
                logger.error("获取模板参数异常！");
                return heatStackModel;
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                logger.error("获取模板参数异常！");
                return heatStackModel;
            }
            List<JSONObject> list = JSONArray.parseArray(object.getString("template_functions"), JSONObject.class);
            if (null != list && list.size() > 0) {
                heatStackModel.setFunctions(JSON.toJSONString(list));
            }
        } catch (Exception e) {
            logger.error("Get error message：", e);
        }
        return heatStackModel;
    }

    public HeatStackModel templateConvertor(JSONObject object) {
        HeatStackModel model = new HeatStackModel();
        if (null != object) {
            model.setVersion(object.getString("version"));
            model.setType(object.getString("type"));
            model.setAliases(object.getString("aliases"));
        }
        return model;
    }

    /**
     * get /v1/{tenant_id}/stacks/{stack_name}/{stack_id}/export
     */
    @Override
    public BocloudResult exportStack(HeatStackModel heatStackModel) {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * get /v1/{tenant_id}/stacks/{stack_name}/{stack_id}/template
     */
    @Override
    public BocloudResult detailStackTemplate(HeatStackModel heatStackModel) {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * get /v1/{tenant_id}/stacks/{stack_name}/{stack_id}/environment
     */
    @Override
    public BocloudResult detailStackEnvironment(HeatStackModel heatStackModel) {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * get /v1/{tenant_id}/stacks/{stack_name}/{stack_id}/files
     */
    @Override
    public BocloudResult stackFiles(HeatStackModel heatStackModel) {
        return null;
    }

    @SuppressWarnings("unchecked")
    @Override
    public BocloudResult suspendStack(HeatStackModel heatStackModel) {
        try {
            String jsonStr = "{\"suspend\":null}";
            net.sf.json.JSONObject model = net.sf.json.JSONObject.fromObject(jsonStr);
            Result result = this.doPost(
                    "/stacks/" + heatStackModel.getName() + "/" + heatStackModel.getStackUuid() + "/actions", model,
                    "heat", "orchestration");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, createProcess(heatStackModel).getData(), "挂起成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "挂起失败");
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    public BocloudResult resumeStack(HeatStackModel heatStackModel) {
        try {
            String jsonStr = "{\"resume\":null}";
            net.sf.json.JSONObject model = net.sf.json.JSONObject.fromObject(jsonStr);
            Result result = this.doPost(
                    "/stacks/" + heatStackModel.getName() + "/" + heatStackModel.getStackUuid() + "/actions", model,
                    "heat", "orchestration");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, createProcess(heatStackModel).getData(), "恢复成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "恢复失败");
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    public BocloudResult cheakStack(HeatStackModel heatStackModel) {
        try {
            String jsonStr = "{\"check\":null}";
            net.sf.json.JSONObject model = net.sf.json.JSONObject.fromObject(jsonStr);
            Result result = this.doPost(
                    "/stacks/" + heatStackModel.getName() + "/" + heatStackModel.getStackUuid() + "/actions", model,
                    "heat", "orchestration");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, createProcess(heatStackModel).getData(), "检查栈成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "检查栈失败");
        }
    }

    public BocloudResult listResourceTypes(HeatStackModel heatStackModel) {
        try {
            Result result = this.doGet("/resource_types", null, "heat", "orchestration");
            if (result.isFailed()) {
                logger.error("获取参数类型异常！");
                return new BocloudResult(false, "获取参数类型异常：" + result.getErrorMsg());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                logger.error("获取参数类型异常！");
                return new BocloudResult(false, "获取参数类型异常：" + result.getErrorMsg());
            }
            List<String> resourceTypes = JSONArray.parseArray(object.getString("resource_types"), String.class);
            return new BocloudResult(true, resourceTypes, "查询成功！");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询资源类型异常：" + e.getMessage());
        }
    }
}
