package com.bocloud.cmp.provider.compute;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.openstack4j.api.Builders;
import org.openstack4j.api.exceptions.ClientResponseException;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.compute.Action;
import org.openstack4j.model.compute.BDMDestType;
import org.openstack4j.model.compute.BDMSourceType;
import org.openstack4j.model.compute.Fault;
import org.openstack4j.model.compute.InterfaceAttachment;
import org.openstack4j.model.compute.RebootType;
import org.openstack4j.model.compute.Server;
import org.openstack4j.model.compute.ServerCreate;
import org.openstack4j.model.compute.ServerUpdateOptions;
import org.openstack4j.model.compute.VNCConsole;
import org.openstack4j.model.compute.actions.LiveMigrateOptions;
import org.openstack4j.model.compute.builder.ServerCreateBuilder;
import org.openstack4j.model.network.Port;
import org.openstack4j.model.network.builder.PortBuilder;
import org.openstack4j.model.network.options.PortListOptions;
import org.openstack4j.model.storage.block.Volume;
import org.openstack4j.model.storage.block.VolumeSnapshot;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.AddressBean;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.HeatStackModel;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.NetworkBean;
import com.bocloud.cmp.model.OSMigrateModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.provider.OpenStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.PortConvertor;
import com.bocloud.cmp.provider.convertor.ServerConvertor;
import com.bocloud.cmp.provider.convertor.SnapshotConvertor;
import com.bocloud.cmp.provider.convertor.V2ImageConvertor;
import com.bocloud.cmp.provider.convertor.VolumeConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import ch.ethz.ssh2.crypto.Base64;

/**
 * openstack云主机操作类
 *
 * @ Description： @ Author ：丁明威 @ Date ：Created in 16:36 2018/11/14 @ Modified
 * By：
 */
@Provider("bocloud.cmp.openstack")
public class OpenStackServerProvider extends OpenStackProvider implements ServerProvider {

    private static Logger logger = LoggerFactory.getLogger(OpenStackServerProvider.class);

    public OpenStackServerProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(ServerModel server) {
        Server result = null;
        List<? extends Port> newPorts = new ArrayList<>();
        try {
            // 此处需要安装镜像中已包含cloud_init，在云主机创建成功之后运行，包含设置云主机密码，是否允许ssh连接等
            StringBuffer buffer = new StringBuffer();
            buffer.append("#cloud-config\n");
            buffer.append("hostname: " + server.getName() + "\n");
            buffer.append("#manage_etc_hosts: true\n");
            String password = server.getPassword();
            if (!StringUtils.isEmpty(password)) {
                buffer.append("chpasswd:\n");
                buffer.append(" list: |\n");
                buffer.append("   root:").append(password).append("\n");
                buffer.append(" expire: false\n");
            } else {
                buffer.append("chpasswd: { expire: False }\n");
            }
            buffer.append("ssh_pwauth: True");
            logger.info("cloud-init脚本：{}", buffer.toString());
            logger.info("Set vm【" + server.getName() + "】 user data start");
            ServerCreateBuilder serverCreateBuilder = Builders.server()
                    .userData(new String(Base64.encode(buffer.toString().getBytes())));
//            serverCreateBuilder.addAdminPass(password);
            logger.info("Set vm【" + server.getName() + "】 user data end");
            ServerCreate serverCreate = null;
            String sourceValidUuid = server.getSourceValidUuid();
            String sourceSnapUuid = server.getSourceSnapUuid();
            logger.info("Build vm【" + server.getName() + "】 block device mapping start");
            if (!StringUtils.isEmpty(sourceValidUuid)) {
                serverCreateBuilder = serverCreateBuilder
                        .blockDevice(Builders.blockDeviceMapping().sourceType(BDMSourceType.VOLUME).bootIndex(0)
                                .uuid(sourceValidUuid).deleteOnTermination(true).build());
            } else if (!StringUtils.isEmpty(sourceSnapUuid)) {
                serverCreateBuilder = serverCreateBuilder
                        .blockDevice(Builders.blockDeviceMapping().sourceType(BDMSourceType.SNAPSHOT).bootIndex(0)
                                .uuid(sourceSnapUuid).deleteOnTermination(true).build());
            } else if (null != server.getBootFromVolume() && server.getBootFromVolume()) {
                serverCreateBuilder = serverCreateBuilder.blockDevice(
                        Builders.blockDeviceMapping().deviceName("/dev/vda").sourceType(BDMSourceType.IMAGE)
                                .destinationType(BDMDestType.VOLUME).volumeSize(server.getRootSize()).bootIndex(0)
                                .uuid(server.getImageUuid()).deleteOnTermination(true).build());
            } else {
                serverCreateBuilder = serverCreateBuilder.image(server.getImageUuid());
            }
            logger.info("Build vm【" + server.getName() + "】block device mapping end");
            serverCreateBuilder = serverCreateBuilder.name(server.getName()).flavor(server.getFlavorUuid())
                    .addPersonality("/etc/motd", "Welcome to the new VM! Restricted access only");
            logger.info("Create openstack vm【" + server.getName() + "】 start...");
            // 添加密钥
            if (!StringUtils.isEmpty(server.getKeypairName())) {
                logger.info("bind openstack vm【" + server.getName() + "】 keypair start...");
                serverCreateBuilder.keypairName(server.getKeypairName());
                logger.info("bind openstack vm【" + server.getName() + "】 keypair end...");
            }
            // 端口设置安全组
            List<String> sGroups = server.getSecurityGroups();
            // 挂载网卡
            List<NetworkBean> networkBeans = server.getNetworkBeans();
            if (!ListTool.isEmpty(networkBeans)) {
                logger.info("Create openstack vm【" + server.getName() + "】 network port start...");
                List<Port> ports = new ArrayList<>();
                logger.info("networkBeans : " + JSON.toJSONString(networkBeans));
                for (NetworkBean networkBean : networkBeans) {
                    List<AddressBean> addresses = networkBean.getAddresses();
                    // 如果未指定IP，则通过dhcp获取IP并创建端口
                    if (!ListTool.isEmpty(addresses)) {
                        for (AddressBean address : addresses) {
                            PortBuilder portBuilder = Builders.port().name(IDFactory.instance().uuid())
                                    .networkId(networkBean.getNetworkUuid());
                            portBuilder.fixedIp(address.getAddress(), address.getSubnetUuid());
                            for (String sGroup : sGroups) {
                                portBuilder.securityGroup(sGroup);
                            }
                            ports.add(portBuilder.build());
                        }
                    } else {
                        PortBuilder portBuilder = Builders.port().name(IDFactory.instance().uuid())
                                .networkId(networkBean.getNetworkUuid());
                        for (String sGroup : sGroups) {
                            portBuilder.securityGroup(sGroup);
                        }
                        ports.add(portBuilder.build());
                    }
                }
                // 批量创建端口
                newPorts = this.getClient().networking().port().create(ports);
                if (ListTool.isEmpty(newPorts)) {
                    return new BocloudResult(false, "创建网络端口时发生错误，创建云主机失败");
                }
                // 设置带宽
                for (Port port : newPorts) {
                    serverCreateBuilder.addNetworkPort(port.getId());
                    if (!StringUtils.isEmpty(server.getPolicyUuid())) {
                        logger.info("port绑定带宽start....");
                        BocloudResult portPolicy = addPortPolicy(server.getPolicyUuid(), port.getId());
                        logger.info("port绑定带宽end...." + portPolicy.isSuccess() + " == " + portPolicy.getMessage());
                        logger.info("result : " + JSON.toJSONString(portPolicy));
                    }
                }
                logger.info("Create openstack vm【" + server.getName() + "】 network port End...");
            }
            if (!StringUtils.isEmpty(server.getRegionId())) {
                serverCreateBuilder.availabilityZone(server.getRegionId());
            }
            serverCreate = serverCreateBuilder.build();
            result = this.getClient().compute().servers().boot(serverCreate);
        } catch (ClientResponseException e) {
            logger.error("创建失败！", e);
            this.removeFailPort(newPorts);
            return new BocloudResult(false, "创建失败【" + e.getMessage() + "】");
        }
        if (null == result) {
            return new BocloudResult(false, "创建失败");
        }
        logger.info("Create new vm detail：{}", JSONObject.toJSON(result));
        Boolean bootFromVolume = server.getBootFromVolume() == null ? false : server.getBootFromVolume();
        BocloudResult detail = waitForVmStatus(result.getId(), bootFromVolume);
        ServerModel model = (ServerModel) detail.getData();
        BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
        model.setPorts(convertPort.convertList(newPorts));
        model.setAccount(server.getAccount());
        model.setPassword(server.getPassword());
        model.setOsCategory(server.getOsCategory());
        model.setOsName(server.getOsName());
        model.setOsVersion(server.getOsVersion());
        if(StringUtils.isEmpty(model.getImageUuid())) {
        	model.setImageUuid(server.getImageUuid());
        }
        logger.info("create openstack vm【" + server.getName() + "】 end!");
        return new BocloudResult(true, model, "create success");
    }
    
    public BocloudResult waitForVmStatus(String serverId, Boolean bootFromVolume) {
        BeanConvertor<ServerModel, Server> convertServer = new ServerConvertor();
        Server server = this.getClient().compute().servers().get(serverId);
        if (null == server) {
            return new BocloudResult(false, "云主机不存在");
        }
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();
        while (flag) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
			if (!("BUILD".equalsIgnoreCase(server.getStatus().name())
					|| "MIGRATING".equalsIgnoreCase(server.getStatus().name()))) {
				if (bootFromVolume) {
			        List<String> volumeIds = server.getOsExtendedVolumesAttached();
			        if (!ListTool.isEmpty(volumeIds)) {
			        	flag = false;
						break;
			        }
				} else {
					flag = false;
					break;
				}
			}
            Long endTime = System.currentTimeMillis();
            // 等待60s
            if (endTime - startTime > 15 * 60 * 1000) {
                flag = false;
                server = this.getClient().compute().servers().get(serverId);
                break;
            }
            server = this.getClient().compute().servers().get(serverId);
        }
        ServerModel model = convertServer.convertModel(server);
        List<VolumeModel> volumeModels = Lists.newArrayList();
        // 是否是卷创建
        List<String> volumeIds = server.getOsExtendedVolumesAttached();
        if (!ListTool.isEmpty(volumeIds)) {
            BeanConvertor<VolumeModel, Volume> volumeConvert = new VolumeConvertor();
            for (String volumeId : volumeIds) {
                Volume volume = this.getClient().blockStorage().volumes().get(volumeId);
                volumeModels.add(volumeConvert.convertModel(volume));
            }
        }
        model.setDiskDevices(volumeModels);
        return new BocloudResult(true, model, "success");
    
    }

    @Override
    public BocloudResult addPortPolicy(String policyId, String portId) {
        try {
            JSONObject portModel = new JSONObject();
            portModel.put("qos_policy_id", policyId);
            JSONObject model = new JSONObject();
            model.put("port", portModel);
            Result result = this.doPut("/v2.0/ports/" + portId, model, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "port绑定失败");
            }
            return new BocloudResult(true, "success!");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "port绑定失败：" + e.getMessage());
        }
    }

    /**
     * 删除创建失败的云主机端口
     *
     * @param failPorts
     */
    private void removeFailPort(List<? extends Port> failPorts) {
        if (!ListTool.isEmpty(failPorts)) {
            logger.info("Remove error virtual machine port start");
            // 删除创建失败的云主机的port
            for (Port port : failPorts) {
                this.getClient().networking().port().delete(port.getId());
            }
            logger.info("Remove error virtual machine port end");
        }
    }

    @Override
    public BocloudResult start(String instanceId) {
        ActionResponse response = this.getClient().compute().servers().action(instanceId, Action.START);
        if (response.isSuccess()) {
            return new BocloudResult(true, "启动成功！");
        } else {
            logger.error("start instance [{}] failed :{}", instanceId, response.getFault());
            return new BocloudResult(false, "启动失败！");
        }
    }

    @Override
    public BocloudResult stop(String instanceId) {
        ActionResponse response = this.getClient().compute().servers().action(instanceId, Action.STOP);
        if (response.isSuccess()) {
            return new BocloudResult(true, "关机成功！");
        } else {
            logger.error("stop instance [{}] failed :{}", instanceId, response.getFault());
            return new BocloudResult(false, "关机失败！");
        }
    }

    @Override
    public BocloudResult reboot(String instanceId) {
        ActionResponse response = this.getClient().compute().servers().reboot(instanceId, RebootType.HARD);
        if (response.isSuccess()) {
            return new BocloudResult(true, "硬重启成功！");
        } else {
            logger.error("Hard reboot instance [{}] failed :{}", instanceId, response.getFault());
            return new BocloudResult(false, "硬重启失败！");
        }
    }

    @Override
    public BocloudResult softReboot(String instanceId) {
        ActionResponse response = this.getClient().compute().servers().reboot(instanceId, RebootType.SOFT);
        if (response.isSuccess()) {
            return new BocloudResult(true, "软重启成功！");
        } else {
            logger.error("Soft reboot instance [{}] failed :{}", instanceId, response.getFault());
            return new BocloudResult(false, "软重启失败！");
        }
    }

    @Override
    public BocloudResult remove(String instanceId, String name) {
        if (StringUtils.isEmpty(instanceId)) {
            return new BocloudResult(true, "云主机【" + name + "】不存在");
        }
        Server server = this.getClient().compute().servers().get(instanceId);
        if (null == server) {
            logger.info("cloud server not exist!");
            return new BocloudResult(true, "云主机【" + name + "】不存在");
        }
        ActionResponse response = this.getClient().compute().servers().action(instanceId, Action.FORCEDELETE);
        if (response.isSuccess()) {
            return new BocloudResult(true, "删除成功！");
        } else {
            logger.error("remove instance [{}] failed :{}", instanceId, response.getFault());
            return new BocloudResult(false, "删除失败！");
        }
    }

    @Override
    public BocloudResult list() {
        List<ServerModel> models = Lists.newArrayList();
        List<? extends Server> servers = Lists.newArrayList();
        servers = this.getClient().compute().servers().listAll(true);
        if (null != servers) {
            for (Server server : servers) {
                BeanConvertor<ServerModel, Server> convertServer = new ServerConvertor();
                models.add(convertServer.convertModel(server));
            }
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult list(String s) {
        return null;
    }

    @Override
    public BocloudResult serverDetail(ServerModel params) {
        try {
            Server server = this.getClient().compute().servers().get(params.getInstanceId());
            BeanConvertor<ServerModel, Server> convertServer = new ServerConvertor();
            ServerModel model = convertServer.convertModel(server);
            PortListOptions options = PortListOptions.create().deviceId(model.getInstanceId());
            List<? extends Port> ports = this.getClient().networking().port().list(options);
            BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
            model.setPorts(convertPort.convertList(ports));
            return new BocloudResult(true, model, "查询成功！");
        } catch (Exception e) {
            logger.error("查询失败：" + e.getMessage());
            return new BocloudResult(false, "查询失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String serverId) {
        BeanConvertor<ServerModel, Server> convertServer = new ServerConvertor();
        Server server = this.getClient().compute().servers().get(serverId);
        if (null == server) {
            return new BocloudResult(false, "云主机不存在");
        }
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();
        while (flag) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            if (!("BUILD".equalsIgnoreCase(server.getStatus().name())
                    || "MIGRATING".equalsIgnoreCase(server.getStatus().name()))) {
                flag = false;
                break;
            }
            Long endTime = System.currentTimeMillis();
            // 等待60s
            if (endTime - startTime > 60 * 1000) {
                flag = false;
                server = this.getClient().compute().servers().get(serverId);
                break;
            }
            server = this.getClient().compute().servers().get(serverId);
        }
        ServerModel model = convertServer.convertModel(server);
        List<VolumeModel> volumeModels = Lists.newArrayList();
        // 是否是卷创建
        List<String> volumeIds = server.getOsExtendedVolumesAttached();
        if (!ListTool.isEmpty(volumeIds)) {
            BeanConvertor<VolumeModel, Volume> volumeConvert = new VolumeConvertor();
            for (String volumeId : volumeIds) {
                Volume volume = this.getClient().blockStorage().volumes().get(volumeId);
                volumeModels.add(volumeConvert.convertModel(volume));
            }
        }
        model.setDiskDevices(volumeModels);
        return new BocloudResult(true, model, "success");
    }

    @Override
    public BocloudResult detailFault(String instanceId) {
        Server server = this.getClient().compute().servers().get(instanceId);
        if (null == server) {
            return new BocloudResult(false, "云主机不存在");
        }
        Fault fault = server.getFault();
        if (null == fault) {
            return new BocloudResult(false, "云主机异常信息未获取到");
        }
        return new BocloudResult(true, JSONObject.toJSONString(fault), "true");
    }

    @Override
    public BocloudResult joinSGroup(String serverId, String groupId) {
        ActionResponse response = this.getClient().compute().servers().addSecurityGroup(serverId, groupId);
        if (response.isSuccess()) {
            return new BocloudResult(true, "操作成功！");
        } else {
            logger.error("joinSGroup [{}] failed :{}", serverId, response.getFault());
            return new BocloudResult(false, "操作失败！");
        }
    }

    @Override
    public BocloudResult leaveSGroup(String serverId, String groupId) {
        ActionResponse response = this.getClient().compute().servers().removeSecurityGroup(serverId, groupId);
        if (response.isSuccess()) {
            return new BocloudResult(true, "操作成功！");
        } else {
            logger.error("joinSGroup [{}] failed :{}", serverId, response.getFault());
            return new BocloudResult(false, "操作失败！");
        }
    }

    @Override
    public BocloudResult modify(ServerModel server) {
        ServerUpdateOptions options = new ServerUpdateOptions();
        options.name(server.getName());
        Server response = this.getClient().compute().servers().update(server.getInstanceId(), options);
        if (null != response) {
            BeanConvertor<ServerModel, Server> convertServer = new ServerConvertor();
            return new BocloudResult(true, convertServer.convertModel(response), "操作成功！");
        } else {
            logger.error("joinSGroup [{}] failed!", server.getInstanceId());
            return new BocloudResult(false, "操作失败！");
        }
    }

    @Override
    public BocloudResult migrate(ServerModel server) {
        String serverId = server.getInstanceId();
        ActionResponse response = this.getClient().compute().servers().migrateServer(serverId);
        if (response.isSuccess()) {
            return new BocloudResult(true, "操作成功！");
        } else {
            logger.error("migrate [{}] failed :{}", serverId, response.getFault());
            return new BocloudResult(false, "操作失败！");
        }
    }

    @Override
    public BocloudResult migrate(OSMigrateModel model) {
        try {
            String serverId = model.getInstanceId();
            ActionResponse response = null;
            if (model.getLive()) {
                response = this.getClient().compute().servers().liveMigrate(serverId,
                        LiveMigrateOptions.create().host(model.getHost()).blockMigration(false).diskOverCommit(false));
            } else {
                response = this.getClient().compute().servers().migrateServer(serverId);
                boolean flag = true;
                Long startTime = System.currentTimeMillis();
                while (flag) {
                    try {
                        Thread.sleep(5000);
                    } catch (InterruptedException e) {
                        logger.error("Get error message:", e);
                        Thread.currentThread().interrupt();
                    }
                    Server.Status status = this.getClient().compute().servers().get(serverId).getStatus();
                    if (Server.Status.VERIFY_RESIZE.equals(status)) {
                        flag = false;
                        response = this.getClient().compute().servers().confirmResize(serverId);
                    }
                    Long endTime = System.currentTimeMillis();
                    // 等待60s
                    if (endTime - startTime > 60 * 1000) {
                        flag = false;
                    }
                }
            }
            if (response.isSuccess()) {
                return new BocloudResult(true, detailMigrate(serverId).getData(), "操作成功！");
            } else {
                logger.error("migrate [{}] failed :{}", serverId, response.getFault());
                return new BocloudResult(false, response.getFault());
            }
        } catch (Exception e) {
            logger.error("migrate vm error:", e);
            return new BocloudResult(false, "migrate fail");
        }
    }

    public BocloudResult detailMigrate(String serverId) {
        BeanConvertor<ServerModel, Server> convertServer = new ServerConvertor();
        Server server = null;
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();
        while (flag) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            server = this.getClient().compute().servers().get(serverId);
            if (!"MIGRATING".equalsIgnoreCase(server.getStatus().name())) {
                flag = false;
            }
            Long endTime = System.currentTimeMillis();
            // 等待30s
            if (endTime - startTime > 60 * 1000 * 5) {
                flag = false;
            }
        }
        return new BocloudResult(true, convertServer.convertModel(server), "success");
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        try {
            Map<String, Object> resultMap = Maps.newHashMap();
            List<SnapshotModel> snapshotModels = Lists.newArrayList();
            String imageId = this.getClient().compute().servers().createSnapshot(snapshotModel.getInstanceId(),
                    snapshotModel.getName());
            if (!StringUtils.isEmpty(imageId)) {
                ImageModel imageModel = getActiveImage(imageId);
                if (null == imageModel) {
                    return new BocloudResult(false, "创建快照失败");
                }
                // 云主机打快照生成的镜像
                resultMap.put("image", imageModel);
                Server server = this.getClient().compute().servers().get(snapshotModel.getInstanceId());
                List<String> volumeIds = server.getOsExtendedVolumesAttached();
                if (!ListTool.isEmpty(volumeIds)) {
                    List<String> snapshotIds = Lists.newArrayList();
                    for (String volumeId : volumeIds) {
                        Map<String, String> filteringParams = Maps.newHashMap();
                        filteringParams.put("volume_id", volumeId);
                        List<? extends VolumeSnapshot> snapshots = this.getClient().blockStorage().snapshots()
                                .list(filteringParams);
                        if (!ListTool.isEmpty(snapshots)) {
                            for (VolumeSnapshot snapshot : snapshots) {
                                snapshotIds.add(snapshot.getId());
                            }
                        }
                    }
                    if (!ListTool.isEmpty(snapshotIds)) {
                        for (String snapshotId : snapshotIds) {
                            snapshotModels.add(getAvaliable(snapshotId));
                        }
                    }
                }
                resultMap.put("snapshots", snapshotModels);
                return new BocloudResult(true, resultMap, "添加快照成功");
            }
            return new BocloudResult(false, "添加快照失败！");
        } catch (Exception e) {
            logger.error("添加快照失败！", e);
            return new BocloudResult(false, "添加快照失败！");
        }
    }

    private ImageModel getActiveImage(String imageId) {
        BeanConvertor<ImageModel, org.openstack4j.model.image.v2.Image> converImage = new V2ImageConvertor();
        org.openstack4j.model.image.v2.Image image = this.getClient().imagesV2().get(imageId);
        ImageModel imageModel = null;
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();
        while (flag) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            image = this.getClient().imagesV2().get(imageId);
            if (null == image) {
                flag = false;
                break;
            }
            if ((null != image.getStatus() && "ACTIVE".equalsIgnoreCase(image.getStatus().name()))
                    || "DELETED".equalsIgnoreCase(image.getStatus().name())) {
                flag = false;
                imageModel = converImage.convertModel(image);
                break;
            }
            Long endTime = System.currentTimeMillis();
            // 等待30分钟
            if (endTime - startTime > 30 * 60 * 1000) {
                flag = false;
                imageModel = converImage.convertModel(image);
                imageModel.setStatus("CREATE_TIMEOUT");
                break;
            }
        }
        if (null == image) {
            return null;
        }
        return imageModel;
    }

    private SnapshotModel getAvaliable(String snapshotId) {
        BeanConvertor<SnapshotModel, VolumeSnapshot> converSnapshot = new SnapshotConvertor();
        SnapshotModel snapshotModel = null;
        VolumeSnapshot snapshot = this.getClient().blockStorage().snapshots().get(snapshotId);
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();
        while (flag) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            snapshot = this.getClient().blockStorage().snapshots().get(snapshotId);
            if (null != snapshot.getStatus() && "AVAILABLE".equalsIgnoreCase(snapshot.getStatus().name())) {
                flag = false;
                snapshotModel = converSnapshot.convertModel(snapshot);
                break;
            }
            Long endTime = System.currentTimeMillis();
            // 等待60s
            if (endTime - startTime > 60 * 1000 * 20) {
                flag = false;
                snapshotModel = converSnapshot.convertModel(snapshot);
                snapshotModel.setStatus("CREATE_TIMEOUT");
                break;
            }
        }
        return snapshotModel;
    }

    @Override
    public BocloudResult pause(String instanceId) {
        ActionResponse actionResponse = this.getClient().compute().servers().action(instanceId, Action.PAUSE);
        if (actionResponse.isSuccess()) {
            return new BocloudResult(true, "休眠成功");
        } else {
            return new BocloudResult(false, "休眠失败");
        }
    }

    @Override
    public BocloudResult unPause(String instanceId) {
        ActionResponse actionResponse = this.getClient().compute().servers().action(instanceId, Action.UNPAUSE);
        if (actionResponse.isSuccess()) {
            return new BocloudResult(true, "取消休眠成功");
        } else {
            return new BocloudResult(false, "取消休眠失败");
        }
    }

    @Override
    public BocloudResult suspend(String instanceId) {
        ActionResponse actionResponse = this.getClient().compute().servers().action(instanceId, Action.SUSPEND);
        if (actionResponse.isSuccess()) {
            return new BocloudResult(true, "睡眠成功");
        } else {
            return new BocloudResult(false, "睡眠失败");
        }
    }

    @Override
    public BocloudResult resume(String instanceId) {
        ActionResponse actionResponse = this.getClient().compute().servers().action(instanceId, Action.RESUME);
        if (actionResponse.isSuccess()) {
            return new BocloudResult(true, "恢复成功");
        } else {
            return new BocloudResult(false, "恢复失败");
        }
    }

    @Override
    public BocloudResult vncConsole(String vncType, String serverId) {
        try {
            VNCConsole vncConsole = this.getClient().compute().servers().getVNCConsole(serverId,
                    VNCConsole.Type.value(vncType));
            return new BocloudResult(true, vncConsole, "访问成功");
        } catch (Exception e) {
            logger.error("vncConsole Exception", e);
            return new BocloudResult(false, "访问失败");
        }
    }

    @Override
    public BocloudResult changePassword(String instanceId, String password) {
        ActionResponse actionResponse = this.getClient().compute().servers().changeAdminPassword(instanceId, password);
        if (actionResponse.isSuccess()) {
            return new BocloudResult(true, "修改成功");
        } else {
            return new BocloudResult(false, "修改失败[" + actionResponse.getFault() + "]");
        }
    }

    @Override
    public BocloudResult attachNetwork(ServerModel server) {
        List<NetworkBean> networkBeans = server.getNetworkBeans();
        if (ListTool.isEmpty(networkBeans)) {
            return new BocloudResult(false, "挂载网卡为空");
        }
        NetworkBean networkBean = networkBeans.get(0);
        logger.info("Attach port to server start...");
        // 网络下子网
        List<AddressBean> addresses = networkBean.getAddresses();
        // 创建端口
        Port port = null;
        try {
            PortBuilder builder = Builders.port().name(IDFactory.instance().uuid())
                    .networkId(networkBean.getNetworkUuid());
            if (!ListTool.isEmpty(addresses)) {
                AddressBean address = addresses.get(0);
                if (!StringUtils.isEmpty(address.getAddress())) {
                    builder.fixedIp(address.getAddress(), address.getSubnetUuid());
                } else {
                    builder.fixedIp(null, address.getSubnetUuid());
                }
            }
            port = this.getClient().networking().port().create(builder.build());
            if (null == port) {
                return new BocloudResult(false, "网卡挂载失败: 端口创建失败,可能配额不足");
            }
            if (org.apache.commons.lang3.StringUtils.isNotEmpty(server.getPolicyUuid())) {
                logger.info("port绑定带宽start....");
                BocloudResult portPolicy = addPortPolicy(server.getPolicyUuid(), port.getId());
                logger.info("port绑定带宽end...." + portPolicy.isSuccess() + " == " + portPolicy.getMessage());
                logger.info("result : " + JSON.toJSONString(portPolicy));
            }
        } catch (ClientResponseException e) {
            logger.error("create port fail:" + e.getMessage());
            return new BocloudResult(false, "网卡挂载失败:" + e.getMessage());
        }
        sleepSomeTime(200);
        // 挂载网卡
        InterfaceAttachment attachment = this.getClient().compute().servers().interfaces()
                .create(server.getInstanceId(), port.getId());
        if (null == attachment) {
            logger.info("Attach port to server fail...");
            if (null != port) {
                logger.info("Remove attach fail port start...");
                this.getClient().networking().port().delete(port.getId());
                logger.info("Remove attach fail port end...");
            }
            return new BocloudResult(false, "网卡挂载失败: 端口创建失败,可能配额不足");
        }
        logger.info("Attach port to server end...");
        sleepSomeTime(200);
        port = this.getClient().networking().port().update(this.getClient().networking().port()
                .get(attachment.getPortId()).toBuilder().deviceOwner("compute:nova").build());
        logger.info("update port : " + JSON.toJSONString(port));
        Server result = this.getClient().compute().servers().get(server.getInstanceId());
        if (null != result) {
            BeanConvertor<ServerModel, Server> convertServer = new ServerConvertor();
            ServerModel serverModel = convertServer.convertModel(result);
            BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
            List<Port> attachPort = Lists.newArrayList();
            attachPort.add(port);
            logger.info("attachPort : " + JSON.toJSONString(attachPort));
            serverModel.setPorts(convertPort.convertList(attachPort));
            logger.info("serverModel : " + JSON.toJSONString(serverModel));
            return new BocloudResult(true, serverModel, "操作成功！");
        } else {
            if (null != port) {
                logger.info("Remove attach fail port start...");
                this.getClient().networking().port().delete(port.getId());
                logger.info("Remove attach fail port end...");
            }
            logger.error("openstack mount network [{}] failed!", server.getInstanceId());
            return new BocloudResult(false, "操作失败！");
        }
    }

    @Override
    public BocloudResult detachNetwork(String instanceId, String portUuid) {
        ActionResponse response = this.getClient().compute().servers().interfaces().detach(instanceId, portUuid);
        if (response.isSuccess()) {
            ActionResponse actionResponse = this.getClient().networking().port().delete(portUuid);
            if (actionResponse.isSuccess()) {
                logger.info("remove detach port success...");
            }
            return new BocloudResult(true, "网卡卸载成功");
        } else {
            return new BocloudResult(false, "网卡卸载失败");
        }
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId) {
        ActionResponse response = this.getClient().compute().servers().resize(instanceId, flavorId);
        if (response.isSuccess()) {
            BeanConvertor<ServerModel, Server> convertServer = new ServerConvertor();
            Server server = null;
            Long startTime = System.currentTimeMillis();
            boolean flag = true;
            while (flag) {
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    logger.error("Get error message:", e);
                    Thread.currentThread().interrupt();
                }
                server = this.getClient().compute().servers().get(instanceId);
                if ("VERIFY_RESIZE".equalsIgnoreCase(server.getStatus().name())
                        || "error".equalsIgnoreCase(server.getStatus().name())) {
                    flag = false;
                }
                Long endTime = System.currentTimeMillis();
                // 等待10分钟
                if (endTime - startTime > 10 * 60 * 1000) {
                    flag = false;
                }
            }
            return new BocloudResult(true, convertServer.convertModel(server), "修改配置成功");
        } else {
            return new BocloudResult(false, "修改配置失败：" + response.getFault());
        }
    }

    @Override
    public BocloudResult confirmResize(String instanceId) {
        ActionResponse response = this.getClient().compute().servers().confirmResize(instanceId);
        if (response.isSuccess()) {
            ServerModel serverModel = this.getServer(instanceId);
            return new BocloudResult(true, serverModel, "确认修改配置成功");
        } else {
            return new BocloudResult(false, "确认修改配置失败：" + response.getFault());
        }
    }

    @Override
    public BocloudResult revertResize(String instanceId) {
        ActionResponse response = this.getClient().compute().servers().revertResize(instanceId);
        if (response.isSuccess()) {
            ServerModel serverModel = this.getServer(instanceId);
            return new BocloudResult(true, serverModel, "回滚配置成功");
        } else {
            return new BocloudResult(false, "回滚配置失败" + response.getFault());
        }
    }

    /**
     * 获取运行中的云主机
     *
     * @param instanceId
     * @return
     */
    private ServerModel getServer(String instanceId) {
        BeanConvertor<ServerModel, Server> convertServer = new ServerConvertor();
        Server server = null;
        Long startTime = System.currentTimeMillis();
        boolean flag = true;
        ServerModel serverModel = null;
        while (flag) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            server = this.getClient().compute().servers().get(instanceId);
            if ("active".equalsIgnoreCase(server.getStatus().name())
                    || "error".equalsIgnoreCase(server.getStatus().name())) {
                serverModel = convertServer.convertModel(server);
                flag = false;
            }
            if ("shutoff".equalsIgnoreCase(server.getStatus().name())) {
                serverModel = convertServer.convertModel(server);
                serverModel.setStatus(BocloudStatus.Server.STOPPED.name());
                flag = false;
            }
            Long endTime = System.currentTimeMillis();
            // 等待10分钟
            if (endTime - startTime > 10 * 60 * 1000) {
                serverModel = convertServer.convertModel(server);
                flag = false;
            }
        }
        return serverModel;
    }

    @Override
    public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean memHotEnable) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getTicket(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult createInResPool(ServerModel server) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult mountToolsInstaller(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult unmountToolsInstaller(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult upgradeToolsTask(String instanceId, String options) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getNetCard(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getVmIps(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult markAsTemplate(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
        return null;
    }

    @Override
    public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getMetric(String type) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult vmrcConsole(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult migrate(H3cMigrateModel h3cMigrateModel) {
        return null;
    }

    @Override
    public BocloudResult create(H3cServerModel h3cServerModel) {
        return null;
    }

    @Override
    public BocloudResult getJnlpXml(Long aLong, String s) {
        return null;
    }

    @Override
    public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult resetInstance(ServerModel serverModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult bindKeypair(ServerModel serverModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult unbindKeypair(ServerModel serverModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
        return null;
    }

    @Override
    public BocloudResult listByStack(HeatStackModel stackModel) {
        try {
            String stackUuid = stackModel.getStackUuid();
            List<ServerModel> models = Lists.newArrayList();
            if (StringUtils.isEmpty(stackUuid)) {
                return new BocloudResult(true, models, "success");
            }
            List<? extends Server> list = this.getClient().compute().servers().listAll(true);
            if (null != list && !list.isEmpty()) {
                BeanConvertor<ServerModel, Server> convertServer = new ServerConvertor();
                BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
                for (Server server : list) {
                    Map<String, String> metadata = server.getMetadata();
                    if (metadata.containsKey("metering.server_group")
                            && metadata.get("metering.server_group").equalsIgnoreCase(stackUuid)) {
                        ServerModel model = convertServer.convertModel(server);
                        PortListOptions options = PortListOptions.create().deviceId(model.getInstanceId());
                        List<? extends Port> ports = this.getClient().networking().port().list(options);
                        model.setPorts(convertPort.convertList(ports));
                        models.add(model);
                    }
                }
            }
            return new BocloudResult(true, models, "success");
        } catch (Exception e) {
            logger.error("查询列表失败：" + e.getMessage());
            return new BocloudResult(false, "查询列表失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getVmIps(String instanceId, String address) {
        // TODO Auto-generated method stub
        return null;
    }

}