package com.bocloud.cmp.provider.loadbalance;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.OpenStackProvider;
import com.bocloud.cmp.provider.convertor.*;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.openstack4j.api.Builders;
import org.openstack4j.api.exceptions.ClientResponseException;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.network.Port;
import org.openstack4j.model.network.ext.*;
import org.openstack4j.model.network.ext.builder.*;
import org.openstack4j.openstack.networking.domain.ext.ListItem;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * openstack负载均衡器操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/15
 */
@Provider("bocloud.cmp.openstack")
public class OpenStackLbProvider extends OpenStackProvider implements LoadBalanceProvider {

    private static Logger logger = LoggerFactory.getLogger(OpenStackLbProvider.class);
    private BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
    private BeanConvertor<LbModel, LoadBalancerV2> lbConvertor = new LbConvertor();
    private BeanConvertor<LbListenerModel, ListenerV2> listenerConvertor = new LbListenerConvertor();
    private BeanConvertor<LbMemberModel, MemberV2> memberConvertor = new LbMemberConvertor();
    private BeanConvertor<LbPoolModel, LbPoolV2> poolConvertor = new LbPoolConvertor();
    private BeanConvertor<HealthMonitorModel, HealthMonitorV2> monitorConvertor = new HealthMonitorConvertor();

    public OpenStackLbProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        List<LbModel> models = Lists.newArrayList();
        List<? extends LoadBalancerV2> loadBalancerV2s = this.getClient().networking().lbaasV2().loadbalancer().list();
        if (!loadBalancerV2s.isEmpty()) {
            for (LoadBalancerV2 balancer : loadBalancerV2s) {
                models.add(lbConvertor.convertModel(balancer));
            }
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult create(LbModel lb) {
        String lbId = null;
        String listenerId = null;
        String poolId = null;
        String hmId = null;
        try {
            logger.info("Start to create load balancer");
            LoadBalancerV2Builder lBuilder = Builders.loadbalancerV2().name(lb.getName()).description(lb.getRemark())
                    .subnetId(lb.getSubnetUuid());
            if (!StringUtils.isEmpty(lb.getVipAddress())) {
                lBuilder.address(lb.getVipAddress());
            }
            LoadBalancerV2 balancer = this.getClient().networking().lbaasV2().loadbalancer().create(lBuilder.build());
            if (null == balancer) {
                return new BocloudResult(false, "创建失败！");
            }
            lbId = balancer.getId();
            boolean active = isActive(lbId);
            if (!active) {
                this.getClient().networking().lbaasV2().loadbalancer().delete(lbId);
                return new BocloudResult(false, "创建失败！");
            }
            logger.info("Finish Create load balancer");
            if (ListTool.isEmpty(lb.getModels())) {
                balancer = this.getClient().networking().lbaasV2().loadbalancer().get(balancer.getId());
                LbModel model = lbConvertor.convertModel(balancer);
                return new BocloudResult(true, model, "success");
            }
            logger.info("Start to create load balancer listener");
            LbListenerModel listenerModel = lb.getModels().get(0);
            ListenerV2Builder listenerBuilder = Builders.listenerV2().name(listenerModel.getName())
                    .description(listenerModel.getRemark())
                    .protocol(ListenerProtocol.valueOf(listenerModel.getProtocol()))
                    .protocolPort(listenerModel.getPort()).adminStateUp(true).loadBalancerId(balancer.getId());
            ListenerV2 listener = this.getClient().networking().lbaasV2().listener().create(listenerBuilder.build());
            if (null == listener) {
                this.getClient().networking().lbaasV2().loadbalancer().delete(lbId);
                return new BocloudResult(false, "创建失败！");
            }
            listenerId = listener.getId();
            active = isActive(lbId);
            if (!active) {
                this.getClient().networking().lbaasV2().listener().delete(listenerId);
                this.getClient().networking().lbaasV2().loadbalancer().delete(lbId);
                return new BocloudResult(false, "创建失败！");
            }
            logger.info("Finish Create load balancer listener");
            logger.info("Start to create load balancer pool");
            LbPoolModel poolModel = listenerModel.getPoolModel();
            LbPoolV2Builder lpBuilder = Builders.lbpoolV2().name(poolModel.getName()).description(poolModel.getRemark())
                    .protocol(Protocol.valueOf(poolModel.getProtocol()))
                    .lbMethod(LbMethod.valueOf(poolModel.getLbMethod())).listenerId(listener.getId());
            if (!StringUtils.isEmpty(poolModel.getType())) {
                SessionPersistenceBuilder persistenceBuilder = Builders.sessionPersistence()
                        .type(SessionPersistenceType.valueOf(poolModel.getType()));
                if (!StringUtils.isEmpty(poolModel.getCookieName())) {
                    persistenceBuilder.cookieName(poolModel.getCookieName());
                }
                lpBuilder.sessionPersistence(persistenceBuilder.build());
            }
            LbPoolV2 pool = this.getClient().networking().lbaasV2().lbPool().create(lpBuilder.build());
            if (null == pool) {
                this.getClient().networking().lbaasV2().listener().delete(listenerId);
                this.getClient().networking().lbaasV2().loadbalancer().delete(lbId);

                return new BocloudResult(false, "创建失败！");
            }
            poolId = pool.getId();
            active = isActive(lbId);
            if (!active) {
                this.getClient().networking().lbaasV2().lbPool().delete(poolId);
                this.getClient().networking().lbaasV2().listener().delete(listenerId);
                this.getClient().networking().lbaasV2().loadbalancer().delete(lbId);
                return new BocloudResult(false, "创建失败！");
            }
            logger.info("Finish Create load balancer pool");
            logger.info("Start to create load balancer health monitor");
            HealthMonitorModel hmModel = poolModel.getModel();
            HealthMonitorV2Builder hmBuilder = Builders.healthmonitorV2()
                    .type(HealthMonitorType.valueOf(hmModel.getType())).delay(hmModel.getDelay())
                    .timeout(hmModel.getTimeout()).maxRetries(hmModel.getMaxRetries()).adminStateUp(true)
                    .poolId(pool.getId());
            if (HealthMonitorType.HTTP.name().equals(hmModel.getType())) {
                hmBuilder.httpMethod(hmModel.getHttpMethod()).urlPath(hmModel.getUrlPath())
                        .expectedCodes(hmModel.getExpectedCodes());
            }
            HealthMonitorV2 monitor = this.getClient().networking().lbaasV2().healthMonitor().create(hmBuilder.build());
            if (null == monitor) {
                this.getClient().networking().lbaasV2().lbPool().delete(poolId);
                this.getClient().networking().lbaasV2().listener().delete(listenerId);
                this.getClient().networking().lbaasV2().loadbalancer().delete(lbId);
                return new BocloudResult(false, "创建失败！");
            }
            hmId = monitor.getId();
            active = isActive(lbId);
            if (!active) {
                this.getClient().networking().lbaasV2().healthMonitor().delete(hmId);
                this.getClient().networking().lbaasV2().lbPool().delete(poolId);
                this.getClient().networking().lbaasV2().listener().delete(listenerId);
                this.getClient().networking().lbaasV2().loadbalancer().delete(lbId);
                return new BocloudResult(false, "创建失败！");
            }
            logger.info("Finish create load balancer health monitor");
            logger.info("Start to create load balancer listener members");
            List<LbMemberModel> memberModels = listenerModel.getModels();
            for (LbMemberModel memberModel : memberModels) {
                active = isActive(lbId);
                if (!active) {
                    break;
                }
                this.getClient().networking().lbaasV2().lbPool().createMember(pool.getId(),
                        Builders.memberV2().address(memberModel.getAddress()).protocolPort(memberModel.getPort())
                                .weight(memberModel.getWeight()).subnetId(memberModel.getSubnetUuid())
                                .adminStateUp(true).build());
            }
            logger.info("Finish load balancer listener members");
            listener = this.getClient().networking().lbaasV2().listener().get(listener.getId());
            LbListenerModel lbListenerModel = listenerConvertor.convertModel(listener);
            pool = this.getClient().networking().lbaasV2().lbPool().get(pool.getId());
            LbPoolModel lbPoolModel = poolConvertor.convertModel(pool);
            monitor = this.getClient().networking().lbaasV2().healthMonitor().get(monitor.getId());
            HealthMonitorModel monitorModel = monitorConvertor.convertModel(monitor);
            List<? extends MemberV2> members = this.getClient().networking().lbaasV2().lbPool()
                    .listMembers(pool.getId());
            active = isActive(lbId);
            if (!active) {
                this.getClient().networking().lbaasV2().healthMonitor().delete(hmId);
                this.getClient().networking().lbaasV2().lbPool().delete(poolId);
                this.getClient().networking().lbaasV2().listener().delete(listenerId);
                this.getClient().networking().lbaasV2().loadbalancer().delete(lbId);
                return new BocloudResult(false, "创建失败！");
            }
            balancer = this.getClient().networking().lbaasV2().loadbalancer().get(balancer.getId());
            LbModel model = lbConvertor.convertModel(balancer);
            if (!ListTool.isEmpty(members)) {
                List<LbMemberModel> mList = memberConvertor.convertList(members);
                for (LbMemberModel memberModel : mList) {
                    memberModel.setPoolUuid(pool.getId());
                }
                lbListenerModel.setModels(mList);
            }
            lbPoolModel.setModel(monitorModel);
            lbListenerModel.setPoolModel(lbPoolModel);
            List<LbListenerModel> listeners = Lists.newArrayList();
            listeners.add(lbListenerModel);
            model.setModels(listeners);
            return new BocloudResult(true, model, "success");
        } catch (Exception e) {
            logger.error("create load balancer error:", e);
            if (null != hmId) {
                this.getClient().networking().lbaasV2().healthMonitor().delete(hmId);
            }
            if (null != poolId) {
                this.getClient().networking().lbaasV2().lbPool().delete(poolId);
            }
            if (null != listenerId) {
                this.getClient().networking().lbaasV2().listener().delete(listenerId);
            }
            if (null != lbId) {
                this.getClient().networking().lbaasV2().loadbalancer().delete(lbId);
            }
            return new BocloudResult(false, e.getMessage());
        }

    }

    private boolean isActive(String lbUuid) {
        LoadBalancerV2 balancer = this.getClient().networking().lbaasV2().loadbalancer().get(lbUuid);
        Long start = System.currentTimeMillis();
        while (true) {
            // 超时和延时
            if (System.currentTimeMillis() - start > 1000 * 60 * 5) {
                return false;
            }
            if (LbProvisioningStatus.ACTIVE.name().equals(balancer.getProvisioningStatus().name())) {
                break;
            }
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                return false;
            }
            balancer = this.getClient().networking().lbaasV2().loadbalancer().get(lbUuid);
        }
        return true;
    }

    @Override
    public BocloudResult update(LbModel lb) {
        try {
            LoadBalancerV2 result = this.getClient().networking().lbaasV2().loadbalancer().update(lb.getLbUuid(),
                    Builders.loadBalancerV2Update().name(lb.getName()).description(lb.getRemark())
                            .adminStateUp(lb.getAdminStateUp()).build());
            BocloudResult detail = detail(result.getId());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "update success");
            }
            return new BocloudResult(false, "修改失败！");
        } catch (Exception e) {
            logger.error("Update load balancer error:", e);
            return new BocloudResult(false, "修改失败");
        }

    }

    @Override
    public BocloudResult remove(String lbId) {
        try {
            LoadBalancerV2 lb = this.getClient().networking().lbaasV2().loadbalancer().get(lbId);
            if (null == lb) {
                return new BocloudResult(true, "delete success");
            }
            List<ListItem> listeners = lb.getListeners();
            // 删除关联的所有监听器
            if (!ListTool.isEmpty(listeners)) {
                for (ListItem listener : listeners) {
                    ActionResponse response = this.getClient().networking().lbaasV2().listener()
                            .delete(listener.getId());
                    if (!response.isSuccess()) {
                        return new BocloudResult(false, "delete fail");
                    }
                }
            }
            ActionResponse response = this.getClient().networking().lbaasV2().loadbalancer().delete(lbId);
            if (response.isSuccess()) {
                return new BocloudResult(true, "success");
            }
            return new BocloudResult(false, response.getFault());
        } catch (ClientResponseException e) {
            logger.error("Remove load balancer error:", e);
            return new BocloudResult(false, "删除失败：" + e.getMessage());
        } catch (Exception e) {
            logger.error("Remove load balancer error:", e);
            return new BocloudResult(false, "删除失败");
        }

    }

    @Override
    public BocloudResult detail(String lbId) {
        try {
            LoadBalancerV2 balancer = this.getClient().networking().lbaasV2().loadbalancer().get(lbId);
            if (null == balancer) {
                return new BocloudResult(false, "no such load balancer");
            } else {
                return new BocloudResult(true, lbConvertor.convertModel(balancer), "success");
            }
        } catch (Exception e) {
            logger.error("Detail load balancer error:", e);
            return new BocloudResult(false, "查询失败");
        }

    }

    @Override
    public BocloudResult listListener() {
        List<LbListenerModel> models = Lists.newArrayList();
        List<? extends ListenerV2> listenerV2s = Lists.newArrayList();
        listenerV2s = this.getClient().networking().lbaasV2().listener().list();
        if (!listenerV2s.isEmpty()) {
            for (ListenerV2 listenerV2 : listenerV2s) {
                models.add(listenerConvertor.convertModel(listenerV2));
            }
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult createListener(LbListenerModel model) {
        String listenerId = null;
        String poolId = null;
        String hmId = null;
        String lbId = model.getLbUuid();
        try {
            logger.info("Start to create load balancer listener");
            ListenerV2Builder listenerBuilder = Builders.listenerV2().name(model.getName())
                    .description(model.getRemark()).protocol(ListenerProtocol.valueOf(model.getProtocol()))
                    .protocolPort(model.getPort()).adminStateUp(true).loadBalancerId(model.getLbUuid());
            String tlsContainerRef = model.getDefTlsContainerRef();
            if (!StringUtils.isEmpty(model.getDefTlsContainerRef())) {
                // https证书引用路径，需要OpenStack支持key-manager
                listenerBuilder.defaultTlsContainerRef(tlsContainerRef);
                // 设置证书引用列表
                List<String> sniContainerRefs = new ArrayList<>();
                sniContainerRefs.add(tlsContainerRef);
                listenerBuilder.sniContainerRefs(sniContainerRefs);
            }
            ListenerV2 listener = this.getClient().networking().lbaasV2().listener().create(listenerBuilder.build());
            listenerId = listener.getId();
            boolean active = isActive(lbId);
            if (!active) {
                this.getClient().networking().lbaasV2().listener().delete(listenerId);
                return new BocloudResult(false, "创建失败！");
            }
            logger.info("Finish Create load balancer listener");
            logger.info("Start to create load balancer pool");
            LbPoolModel poolModel = model.getPoolModel();
            LbPoolV2Builder lpBuilder = Builders.lbpoolV2().name(poolModel.getName()).description(poolModel.getRemark())
                    .protocol(Protocol.valueOf(poolModel.getProtocol()))
                    .lbMethod(LbMethod.valueOf(poolModel.getLbMethod())).listenerId(listener.getId());
            if (!StringUtils.isEmpty(poolModel.getType())) {
                SessionPersistenceBuilder persistenceBuilder = Builders.sessionPersistence()
                        .type(SessionPersistenceType.valueOf(poolModel.getType()));
                if (!StringUtils.isEmpty(poolModel.getCookieName())) {
                    persistenceBuilder.cookieName(poolModel.getCookieName());
                }
                lpBuilder.sessionPersistence(persistenceBuilder.build());
            }
            LbPoolV2 pool = this.getClient().networking().lbaasV2().lbPool().create(lpBuilder.build());
            poolId = pool.getId();
            active = isActive(lbId);
            if (!active) {
                this.getClient().networking().lbaasV2().lbPool().delete(poolId);
                this.getClient().networking().lbaasV2().listener().delete(listenerId);
                return new BocloudResult(false, "创建失败！");
            }
            logger.info("Finish Create load balancer pool");
            logger.info("Start to create load balancer health monitor");
            HealthMonitorModel hmModel = poolModel.getModel();
            HealthMonitorV2Builder hmBuilder = Builders.healthmonitorV2()
                    .type(HealthMonitorType.valueOf(hmModel.getType())).delay(hmModel.getDelay())
                    .timeout(hmModel.getTimeout()).maxRetries(hmModel.getMaxRetries()).adminStateUp(true)
                    .poolId(pool.getId());
            if (HealthMonitorType.HTTP.name().equals(hmModel.getType())) {
                hmBuilder.httpMethod(hmModel.getHttpMethod()).urlPath(hmModel.getUrlPath())
                        .expectedCodes(hmModel.getExpectedCodes());
            }
            HealthMonitorV2 monitor = this.getClient().networking().lbaasV2().healthMonitor().create(hmBuilder.build());
            hmId = monitor.getId();
            active = isActive(lbId);
            if (!active) {
                this.getClient().networking().lbaasV2().healthMonitor().delete(hmId);
                this.getClient().networking().lbaasV2().lbPool().delete(poolId);
                this.getClient().networking().lbaasV2().listener().delete(listenerId);
                return new BocloudResult(false, "创建失败！");
            }
            logger.info("Finish create load balancer health monitor");
            logger.info("Start to create load balancer listener members");
            List<LbMemberModel> memberModels = model.getModels();
            for (LbMemberModel memberModel : memberModels) {
                active = isActive(lbId);
                if (!active) {
                    break;
                }
                this.getClient().networking().lbaasV2().lbPool().createMember(pool.getId(),
                        Builders.memberV2().address(memberModel.getAddress()).protocolPort(memberModel.getPort())
                                .weight(memberModel.getWeight()).subnetId(memberModel.getSubnetUuid())
                                .adminStateUp(true).build());
            }
            logger.info("Finish load balancer listener members");
            listener = this.getClient().networking().lbaasV2().listener().get(listener.getId());
            LbListenerModel lbListenerModel = listenerConvertor.convertModel(listener);
            pool = this.getClient().networking().lbaasV2().lbPool().get(pool.getId());
            LbPoolModel lbPoolModel = poolConvertor.convertModel(pool);
            monitor = this.getClient().networking().lbaasV2().healthMonitor().get(monitor.getId());
            HealthMonitorModel monitorModel = monitorConvertor.convertModel(monitor);
            List<? extends MemberV2> members = this.getClient().networking().lbaasV2().lbPool()
                    .listMembers(pool.getId());
            if (!ListTool.isEmpty(members)) {
                List<LbMemberModel> mList = memberConvertor.convertList(members);
                for (LbMemberModel memberModel : mList) {
                    memberModel.setPoolUuid(pool.getId());
                }
                lbListenerModel.setModels(mList);
            }
            lbPoolModel.setModel(monitorModel);
            lbListenerModel.setPoolModel(lbPoolModel);
            return new BocloudResult(true, lbListenerModel, "success");
        } catch (Exception e) {
            logger.error("create listener error:", e);
            if (null != hmId) {
                this.getClient().networking().lbaasV2().healthMonitor().delete(hmId);
            }
            if (null != poolId) {
                this.getClient().networking().lbaasV2().lbPool().delete(poolId);
            }
            if (null != listenerId) {
                this.getClient().networking().lbaasV2().listener().delete(listenerId);
            }
            return new BocloudResult(false, "创建失败");
        }

    }

    @Override
    public BocloudResult updateListener(LbListenerModel model) {
        try {
            ListenerV2 listener =
                    this.getClient().networking().lbaasV2().listener().update(model.getListenerUuid(),
                            Builders.listenerV2Update().name(model.getName()).connectionLimit(model.getConnLimit())
                                    .description(model.getRemark()).adminStateUp(model.getAdminStateUp())
                                    .defaultTlsContainerRef(model.getDefTlsContainerRef()).build());
            if (null == listener) {
                return new BocloudResult(false, "修改失败！");
            } else {
                BocloudResult detail = detailListener(listener.getId());
                if (detail.isSuccess()) {
                    return new BocloudResult(true, detail.getData(), "update success");
                }
                return new BocloudResult(false, "修改失败！");
            }
        } catch (Exception e) {
            logger.error("Update load balancer listener error:", e);
            return new BocloudResult(false, "修改失败");
        }

    }

    @Override
    public BocloudResult removeListener(String listenerId) {
        try {
            ListenerV2 listener = this.getClient().networking().lbaasV2().listener().get(listenerId);
            if (null == listener) {
                return new BocloudResult(true, "success");
            }
            LbPoolV2 pool = null;
            if (!StringUtils.isEmpty(listener.getDefaultPoolId())) {
                pool = this.getClient().networking().lbaasV2().lbPool().get(listener.getDefaultPoolId());
            }
            if (null != pool && !StringUtils.isEmpty(pool.getHealthMonitorId())) {
                this.getClient().networking().lbaasV2().healthMonitor().delete(pool.getHealthMonitorId());
            }
            if (null != pool) {
                List<ListItem> items = pool.getMembers();
                if (!ListTool.isEmpty(items)) {
                    for (ListItem item : items) {
                        this.getClient().networking().lbaasV2().lbPool().deleteMember(pool.getId(), item.getId());
                    }
                }
                this.getClient().networking().lbaasV2().lbPool().delete(pool.getId());
            }
            ActionResponse response = this.getClient().networking().lbaasV2().listener().delete(listenerId);
            if (response.isSuccess()) {
                return new BocloudResult(true, "success");
            }
            return new BocloudResult(false, response.getFault());
        } catch (Exception e) {
            logger.error("Remove load balancer listener error:", e);
            return new BocloudResult(false, "删除失败");
        }

    }

    @Override
    public BocloudResult detailListener(String listenerId) {
        try {
            ListenerV2 listenerV2 = this.getClient().networking().lbaasV2().listener().get(listenerId);
            if (null == listenerV2) {
                return new BocloudResult(false, "no such listener");
            } else {
                return new BocloudResult(true, listenerConvertor.convertModel(listenerV2), "success");
            }
        } catch (Exception e) {
            logger.error("Detail load balancer listener error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult listHealMonitor() {
        List<HealthMonitorModel> models = Lists.newArrayList();
        List<? extends HealthMonitorV2> healthMonitorV2s = Lists.newArrayList();
        healthMonitorV2s = this.getClient().networking().lbaasV2().healthMonitor().list();
        for (HealthMonitorV2 healthMonitorV2 : healthMonitorV2s) {
            models.add(monitorConvertor.convertModel(healthMonitorV2));
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult createHealMonitor(HealthMonitorModel model) {
        HealthMonitorV2 result = this.getClient().networking().lbaasV2().healthMonitor()
                .create(Builders.healthmonitorV2().type(HealthMonitorType.valueOf(model.getType()))
                        .delay(model.getDelay()).timeout(model.getTimeout()).maxRetries(model.getMaxRetries())
                        .httpMethod(model.getHttpMethod()).adminStateUp(true).poolId(model.getPoolUuid()).build());
        if (null == result) {
            return new BocloudResult(false, "创建失败！");
        } else {
            BocloudResult detail = detailHealMonitor(result.getId());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "create success");
            } else {
                return new BocloudResult(false, "创建失败！");
            }
        }
    }

    @Override
    public BocloudResult updateHealMonitor(HealthMonitorModel model) {
        HealthMonitorV2 result = this.getClient().networking().lbaasV2().healthMonitor().update(model.getHealthUuid(),
                Builders.healthMonitorV2Update().delay(model.getDelay()).timeout(model.getTimeout())
                        .maxRetries(model.getMaxRetries()).httpMethod(model.getHttpMethod()).urlPath(model.getUrlPath())
                        .expectedCodes(model.getExpectedCodes()).adminStateUp(model.getAdminStateUp()).build());
        if (null == result) {
            return new BocloudResult(false, "修改失败！");
        } else {
            BocloudResult detail = detailHealMonitor(result.getId());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "update success");
            } else {
                return new BocloudResult(false, "修改失败！");
            }
        }
    }

    @Override
    public BocloudResult removeHealMonitor(String hmId) {
        ActionResponse response = this.getClient().networking().lbaasV2().healthMonitor().delete(hmId);
        if (response.isSuccess()) {
            return new BocloudResult(true, "success");
        } else {
            logger.error("remove health monitor [{}] error: {}", hmId, response.getFault());
            return new BocloudResult(false, response.getFault());
        }
    }

    @Override
    public BocloudResult detailHealMonitor(String hmId) {
        HealthMonitorV2 vMonitorV2 = this.getClient().networking().lbaasV2().healthMonitor().get(hmId);
        if (null != vMonitorV2) {
            return new BocloudResult(true, monitorConvertor.convertModel(vMonitorV2), "success");
        } else {
            return new BocloudResult(false, "fail");
        }
    }

    @Override
    public BocloudResult listMember(String poolId) {
        List<LbMemberModel> models = Lists.newArrayList();
        List<? extends MemberV2> members = this.getClient().networking().lbaasV2().lbPool().listMembers(poolId);
        if (!members.isEmpty()) {
            for (MemberV2 member : members) {
                models.add(convertMember(member, poolId));
            }
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult createMember(LbMemberModel model) {
        try {
            boolean active = isActive(model.getLbUuid());
            if (!active) {
                return new BocloudResult(false, "创建失败！");
            }
            String lbPoolId = model.getPoolUuid();
            MemberV2 result = this.getClient().networking().lbaasV2().lbPool().createMember(lbPoolId,
                    Builders.memberV2().address(model.getAddress()).protocolPort(model.getPort())
                            .weight(model.getWeight()).subnetId(model.getSubnetUuid()).adminStateUp(true).build());
            if (null == result) {
                return new BocloudResult(false, "创建失败！");
            } else {
                BocloudResult detail = detailMember(lbPoolId, result.getId());
                if (detail.isSuccess()) {
                    return new BocloudResult(true, detail.getData(), "create success");
                }
                return new BocloudResult(false, "创建失败！");
            }
        } catch (Exception e) {
            logger.error("Create member error:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult updateMember(LbMemberModel model) {
        try {
            MemberV2 result = this.getClient().networking().lbaasV2().lbPool().updateMember(model.getPoolUuid(),
                    model.getMemberUuid(),
                    Builders.memberV2Update().weight(model.getWeight()).adminStateUp(model.getAdminStateUp()).build());
            if (null == result) {
                return new BocloudResult(false, "修改失败！");
            }
            return new BocloudResult(true, "update success");
        } catch (Exception e) {
            logger.error("Update member error:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult removeMember(String poolId, String mbId) {
        try {
            ActionResponse response = this.getClient().networking().lbaasV2().lbPool().deleteMember(poolId, mbId);
            if (response.isSuccess()) {
                return new BocloudResult(true, "success");
            }
            return new BocloudResult(false, response.getFault());
        } catch (Exception e) {
            logger.error("Remove member error:", e);
            return new BocloudResult(false, "删除失败");
        }

    }

    @Override
    public BocloudResult detailMember(String poolId, String mbId) {
        MemberV2 member = this.getClient().networking().lbaasV2().lbPool().getMember(poolId, mbId);
        if (null == member) {
            return new BocloudResult(false, "no such lb member");
        } else {
            return new BocloudResult(true, convertMember(member, poolId), "success");
        }
    }

    @Override
    public BocloudResult listPool() {
        List<LbPoolModel> models = Lists.newArrayList();
        List<? extends LbPoolV2> lbPoolV2s = Lists.newArrayList();
        lbPoolV2s = this.getClient().networking().lbaasV2().lbPool().list();
        if (!lbPoolV2s.isEmpty()) {
            for (LbPoolV2 lbPoolV2 : lbPoolV2s) {
                models.add(poolConvertor.convertModel(lbPoolV2));
            }
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult createPool(LbPoolModel poolModel) {
        LbPoolV2 result = this.getClient().networking().lbaasV2().lbPool()
                .create(Builders.lbpoolV2().name(poolModel.getName()).description(poolModel.getRemark())
                        .protocol(Protocol.valueOf(poolModel.getProtocol()))
                        .lbMethod(LbMethod.valueOf(poolModel.getLbMethod())).listenerId(poolModel.getListenerUuid())
                        .build());
        if (null == result) {
            return new BocloudResult(false, "创建失败！");
        } else {
            BocloudResult detail = detailPool(result.getId());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "create success");
            } else {
                return new BocloudResult(false, "创建失败！");
            }
        }
    }

    @Override
    public BocloudResult removePool(String poolId) {
        ActionResponse response = this.getClient().networking().lbaasV2().lbPool().delete(poolId);
        if (response.isSuccess()) {
            return new BocloudResult(true, "success");
        } else {
            logger.error("remove lb pool [{}] error: {}", poolId, response.getFault());
            return new BocloudResult(false, response.getFault());
        }
    }

    @Override
    public BocloudResult updatePool(LbPoolModel poolModel) {
        LbPoolV2 result = this.getClient().networking().lbaasV2().lbPool().update(poolModel.getPoolUuid(),
                Builders.lbPoolV2Update().name(poolModel.getName()).description(poolModel.getRemark())
                        .lbMethod(LbMethod.valueOf(poolModel.getLbMethod())).build());
        if (null == result) {
            return new BocloudResult(false, "修改失败！");
        } else {
            BocloudResult detail = detailPool(result.getId());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "update success");
            } else {
                return new BocloudResult(false, "修改失败！");
            }
        }
    }

    @Override
    public BocloudResult detailPool(String poolId) {
        LbPoolV2 lbPoolV2 = this.getClient().networking().lbaasV2().lbPool().get(poolId);
        if (null == lbPoolV2) {
            return new BocloudResult(false, "no such lb pool");
        } else {
            return new BocloudResult(true, poolConvertor.convertModel(lbPoolV2), "success");
        }
    }

    private LbMemberModel convertMember(MemberV2 member, String poolId) {
        LbMemberModel model = new LbMemberModel();
        model.setAddress(member.getAddress());
        model.setMemberUuid(member.getId());
        model.setPort(member.getProtocolPort());
        model.setSubnetUuid(member.getSubnetId());
        model.setTenantUuid(member.getTenantId());
        model.setAdminStateUp(member.isAdminStateUp());
        model.setWeight(member.getWeight());
        model.setPoolUuid(poolId);
        return model;
    }

    @Override
    public BocloudResult listListener(String lbId) {
        // TODO Auto-generated method stub
        return null;
    }

}
