package com.bocloud.cmp.provider.network;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.openstack4j.api.Builders;
import org.openstack4j.api.exceptions.ClientResponseException;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.network.NetFloatingIP;
import org.openstack4j.model.network.Port;
import org.openstack4j.model.network.options.PortListOptions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FloatingIpModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.provider.OpenStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.FloatingIpConvertor;
import com.bocloud.cmp.provider.convertor.PortConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;

/**
 * openstack浮动IP操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/15
 */
@Provider("bocloud.cmp.openstack")
public class OpenStackFloatingIpProvider extends OpenStackProvider implements FloatingIPProvider {

    private static Logger logger = LoggerFactory.getLogger(OpenStackFloatingIpProvider.class);

    public OpenStackFloatingIpProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        List<FloatingIpModel> models = Lists.newArrayList();
        List<? extends NetFloatingIP> floatingIPs = this.getClient().networking().floatingip().list();
        if (null != floatingIPs) {
            BeanConvertor<FloatingIpModel, NetFloatingIP> fipConvertor = new FloatingIpConvertor();
            models.addAll(fipConvertor.convertList(floatingIPs));
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult create(FloatingIpModel model) {
        try {
            NetFloatingIP result = this.getClient().networking().floatingip()
                    .create(Builders.netFloatingIP().floatingNetworkId(model.getFloatingNetworkUuid()).build());
            if (null == result) {
                return new BocloudResult(false, "创建失败！");
            } else {
                BocloudResult detail = detail(result.getId());
                if (detail.isSuccess()) {
                    return new BocloudResult(true, detail.getData(), "create success");
                } else {
                    return new BocloudResult(false, detail.getMessage());
                }
            }
        } catch (ClientResponseException e) {
            logger.error("create floating ip error:", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String fipId) {
        NetFloatingIP floatingIP = this.getClient().networking().floatingip().get(fipId);
        if (null == floatingIP) {
            return new BocloudResult(true, "floating ip not exists");
        }
        ActionResponse response = this.getClient().networking().floatingip().delete(fipId);
        if (response.isSuccess()) {
            return new BocloudResult(true, "success");
        } else {
            logger.error("remove floatingIp [{}] error: {}", fipId, response.getFault());
            return new BocloudResult(false, response.getFault());
        }
    }

    @Override
    public BocloudResult detail(String fipId) {
        NetFloatingIP floatingIP = this.getClient().networking().floatingip().get(fipId);
        if (null == floatingIP) {
            return new BocloudResult(false, "no such floatingIP");
        } else {
            BeanConvertor<FloatingIpModel, NetFloatingIP> fipConvertor = new FloatingIpConvertor();
            FloatingIpModel floatingIpModel = fipConvertor.convertModel(floatingIP);
            if (!StringUtils.isEmpty(floatingIpModel.getFloatingIpUuid())) {
                PortListOptions options = PortListOptions.create();
                options.deviceId(floatingIpModel.getFloatingIpUuid());
                List<? extends Port> ports = this.getClient().networking().port().list(options);
                if (!ListTool.isEmpty(ports)) {
                    BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
                    floatingIpModel.setPorts(convertPort.convertList(ports));
                }
            }
            return new BocloudResult(true, floatingIpModel, "success");
        }
    }

    @Override
    public BocloudResult associate(String fipId, String portId) {
        NetFloatingIP floatingIP = this.getClient().networking().floatingip().associateToPort(fipId, portId);
        if (null == floatingIP) {
            return new BocloudResult(false, "Associates a Floating IP to a Port fail");
        } else {
            boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                floatingIP = this.getClient().networking().floatingip().get(fipId);
                if ("ACTIVE".equalsIgnoreCase(floatingIP.getStatus().toString())) {
                    flag = false;
                }
                Long endTime = System.currentTimeMillis();
                if (endTime - startTime > 30 * 1000) {// 等待30秒
                    flag = false;
                }
            }
            BeanConvertor<FloatingIpModel, NetFloatingIP> fipConvertor = new FloatingIpConvertor();
            return new BocloudResult(true, fipConvertor.convertModel(floatingIP), "success");
        }
    }

    @Override
    public BocloudResult disassociate(String fipId) {
        NetFloatingIP floatingIP = this.getClient().networking().floatingip().disassociateFromPort(fipId);
        if (null == floatingIP) {
            return new BocloudResult(false, "Deassociate a Floating IP from port fail");
        } else {
            boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                floatingIP = this.getClient().networking().floatingip().get(fipId);
                if ("DOWN".equalsIgnoreCase(floatingIP.getStatus().toString())) {
                    flag = false;
                }
                Long endTime = System.currentTimeMillis();
                if (endTime - startTime > 30 * 1000) {// 等待30秒
                    flag = false;
                }
            }
            BeanConvertor<FloatingIpModel, NetFloatingIP> fipConvertor = new FloatingIpConvertor();
            return new BocloudResult(true, fipConvertor.convertModel(floatingIP), "success");
        }
    }

	@Override
	public BocloudResult associate(FloatingIpModel modeld) {
		// TODO Auto-generated method stub
		return null;
	}

}
