package com.bocloud.cmp.provider.network;

import java.util.List;

import org.openstack4j.api.Builders;
import org.openstack4j.api.exceptions.ClientResponseException;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.network.Network;
import org.openstack4j.model.network.NetworkType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkModel;
import com.bocloud.cmp.model.PortGroupModel;
import com.bocloud.cmp.model.SwitchModel;
import com.bocloud.cmp.provider.OpenStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.NetworkConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.google.common.collect.Lists;

/**
 * openstack网络操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/15
 */
@Provider("bocloud.cmp.openstack")
public class OpenStackNetworkProvider extends OpenStackProvider implements NetworkProvider {

    private static Logger logger = LoggerFactory.getLogger(OpenStackNetworkProvider.class);

    public OpenStackNetworkProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        List<NetworkModel> models = Lists.newArrayList();
        List<? extends Network> networks = this.getClient().networking().network().list();
        if (null != networks) {
            BeanConvertor<NetworkModel, Network> convertNetworks = new NetworkConvertor();
            for (Network network : networks) {
                models.add(convertNetworks.convertModel(network));
            }
        }

        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult create(NetworkModel model) {
        try {
            Network networkBuilder = buildNetwork(model);
            if (null == networkBuilder) {
                return new BocloudResult(false, "创建失败！");
            }
            Network network = this.getClient().networking().network().create(networkBuilder);
            if (null == network) {
                return new BocloudResult(false, "创建失败！");
            }
            BocloudResult detail = detail(network.getId());
            if (detail.isFailed()) {
                return new BocloudResult(false, "创建失败！");
            }
            return new BocloudResult(true, detail.getData(), "create success");
        } catch (ClientResponseException e) {
            logger.error("创建网络失败:" + e.getMessage());
            if (409 == e.getStatus()) {
                return new BocloudResult(false, "创建失败 : " + e.getMessage());
            }
            return new BocloudResult(false, "创建失败 : " + e.getMessage());
        }

    }

    @Override
    public BocloudResult remove(String networkId) {
        Network network = this.getClient().networking().network().get(networkId);
        if (null == network) {
            return new BocloudResult(true, "删除成功！");
        }
        ActionResponse response = this.getClient().networking().network().delete(networkId);
        if (response.isSuccess()) {
            return new BocloudResult(true, "DELETE SUCCESS!");
        } else {
            return new BocloudResult(false, response.getFault());
        }
    }

    @Override
    public BocloudResult detail(String networkId) {
        Network network = this.getClient().networking().network().get(networkId);
        if (null == network) {
            return new BocloudResult(false, "network is not exist!");
        } else {
            BeanConvertor<NetworkModel, Network> convertNetworks = new NetworkConvertor();
            return new BocloudResult(true, convertNetworks.convertModel(network), "success");
        }
    }

    @Override
    public BocloudResult modify(NetworkModel model) {
        Network network = this.getClient().networking().network().update(model.getNetworkUuid(),
                Builders.networkUpdate().name(model.getName()).adminStateUp(model.getAdminStateup())
                        .shared(model.getShared()).build());
        if (null == network) {
            return new BocloudResult(false, "修改失败！");
        } else {
            if (network.getName().equals(model.getName())) {
                return new BocloudResult(true, "modify success");
            } else {
                return new BocloudResult(false, "修改失败！");
            }
        }
    }

    private Network buildNetwork(NetworkModel model) {
        Network networkBuilder = null;
        if (!model.getIsTenant()) {
            if (null != model.getPhyNetwork() && null != model.getSegmentationId()) {// vlan
                networkBuilder = Builders.network().name(model.getName())
                        .networkType(NetworkType.forValue(model.getType())).physicalNetwork(model.getPhyNetwork())
                        .isRouterExternal(null == model.getIsRouterExternal() ? false : model.getIsRouterExternal())
                        .isShared(null == model.getShared() ? false : model.getShared())
                        .adminStateUp(null == model.getAdminStateup() ? true : model.getAdminStateup())
                        .segmentId(model.getSegmentationId()).build();
            } else if (null == model.getPhyNetwork() && null == model.getSegmentationId()) {// local
                networkBuilder = Builders.network().name(model.getName())
                        .networkType(NetworkType.forValue(model.getType()))
                        .isRouterExternal(null == model.getIsRouterExternal() ? false : model.getIsRouterExternal())
                        .isShared(null == model.getShared() ? false : model.getShared())
                        .adminStateUp(null == model.getAdminStateup() ? true : model.getAdminStateup()).build();
            } else if (null != model.getPhyNetwork() && null == model.getSegmentationId()) {// flat
                networkBuilder = Builders.network().name(model.getName())
                        .networkType(NetworkType.forValue(model.getType()))
                        .isRouterExternal(null == model.getIsRouterExternal() ? false : model.getIsRouterExternal())
                        .isShared(null == model.getShared() ? false : model.getShared())
                        .adminStateUp(null == model.getAdminStateup() ? true : model.getAdminStateup())
                        .physicalNetwork(model.getPhyNetwork()).build();
            } else if (null == model.getPhyNetwork() && null != model.getSegmentationId()) {// gre
                                                                                            // or
                                                                                            // vxlan
                networkBuilder = Builders.network().name(model.getName())
                        .networkType(NetworkType.forValue(model.getType()))
                        .isRouterExternal(null == model.getIsRouterExternal() ? false : model.getIsRouterExternal())
                        .isShared(null == model.getShared() ? false : model.getShared())
                        .adminStateUp(null == model.getAdminStateup() ? true : model.getAdminStateup())
                        .segmentId(model.getSegmentationId()).build();
            } else {
                return null;
            }
        } else {
            networkBuilder = Builders.network().name(model.getName()).isRouterExternal(false).isShared(false)
                    .adminStateUp(true).build();
        }
        return networkBuilder;
    }

    @Override
    public BocloudResult createDSwitch(SwitchModel switchModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult createVSwitch(SwitchModel switchModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult removeDSwitch(String name) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult renameDSwitch(String oldName, String newName) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult removeVSwitch(String hostName, String vswitchName, List<String> pgNames) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult createPortGroup(String hostName, String vswitchName, PortGroupModel pgModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult updateVSwitch(String hostName, String vswitchName, SwitchModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult removePortGroup(String hostName, String pgName, String type) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult updatePortGroup(String hostName, PortGroupModel pgModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult updateDVSwitch(SwitchModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult addHost(SwitchModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult listVSwitch(SwitchModel model) {
        return null;
    }

    @Override
    public BocloudResult listProfile() {
        return null;
    }

}
