package com.bocloud.cmp.provider.network;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.openstack4j.api.Builders;
import org.openstack4j.api.exceptions.ClientResponseException;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.network.SecurityGroup;
import org.openstack4j.model.network.SecurityGroupRule;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.OpenStackProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;

/**
 * openstack安全组操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/15
 */
@Provider("bocloud.cmp.openstack")
public class OpenStackSecuGroupProvider extends OpenStackProvider implements SecurityGroupProvider {

    private static Logger logger = LoggerFactory.getLogger(OpenStackSecuGroupProvider.class);

    public OpenStackSecuGroupProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        List<SecurityGroupModel> models = Lists.newArrayList();
        List<? extends SecurityGroup> sgroups = this.getClient().networking().securitygroup().list();
        if (null != sgroups) {
            for (SecurityGroup group : sgroups) {
                models.add(convert(group));
            }
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult create(SecurityGroupModel group) {
        try {
            SecurityGroup result = this.getClient().networking().securitygroup()
                    .create(Builders.securityGroup().name(group.getName()).description(group.getRemark()).build());
            if (null == result) {
                return new BocloudResult(false, group, "fail");
            }
            group.setCreateTime(new Date());
            group.setGroupUuid(result.getId());
            group.setTenantUuid(result.getTenantId());
            // 创建完安全组会创建两个默认安全组规则
            List<? extends SecurityGroupRule> rules = result.getRules();
            List<SecurityGroupRuleModel> ruleModels = Lists.newArrayList();
            if (!ListTool.isEmpty(rules)) {
                rules.forEach(rule -> {
                    SecurityGroupRuleModel ruleModel = new SecurityGroupRuleModel();
                    ruleModel.setPortMin(rule.getPortRangeMin());
                    ruleModel.setPortMax(rule.getPortRangeMax());
                    ruleModel.setProtocol(rule.getProtocol());
                    ruleModel.setRuleUuid(rule.getId());
                    ruleModel.setGroupUuid(result.getId());
                    ruleModel.setDirection(rule.getDirection());
                    ruleModel.setEtherType(rule.getEtherType());
                    ruleModel.setRemoteIpPrefix(rule.getRemoteIpPrefix());
                    ruleModel.setRemoteGroupUuid(rule.getRemoteGroupId());
                    ruleModels.add(ruleModel);
                });
            }
            group.setModels(ruleModels);
            return new BocloudResult(true, group, "succsss");
        } catch (ClientResponseException e) {
            logger.error("create security group error:", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String sgId) {
        SecurityGroup group = this.getClient().networking().securitygroup().get(sgId);
        if (null == group) {
            return new BocloudResult(true, "安全组不存在");
        }
        ActionResponse response = this.getClient().networking().securitygroup().delete(sgId);
        if (response.isSuccess()) {
            return new BocloudResult(true, "删除成功！");
        } else {
            logger.error("remove securitygroup [{}] failed :{}", sgId, response.getFault());
            return new BocloudResult(false, "删除失败！");
        }
    }

    @Override
    public BocloudResult detail(String sgId) {
        SecurityGroup group = this.getClient().networking().securitygroup().get(sgId);
        if (null == group) {
            return new BocloudResult(false, "安全组不存在！");
        } else {
            return new BocloudResult(true, convert(group), "success");
        }
    }

    @Override
    public BocloudResult modify(SecurityGroupModel model) {
        SecurityGroup group = this.getClient().networking().securitygroup().get(model.getGroupUuid());
        if (null == group) {
            return new BocloudResult(false, "安全组不存在");
        }
        group = this.getClient().networking().securitygroup().update(model.getGroupUuid(),
                Builders.securityGroupUpdate().name(model.getName()).description(model.getRemark()).build());
        if (null == group) {
            return new BocloudResult(false, "修改安全组失败");
        } else {
            return new BocloudResult(true, "success");
        }
    }

    @Override
    public BocloudResult getProjectDefaultSecg() {
        String tenantId = null;
        if (this.getVersion().equals(Version.V2)) {
            tenantId = this.getClient2().identity().tenants().getByName(this.getTenant()).getId();
        } else if (this.getVersion().equals(Version.V3)) {
            tenantId = this.getClient3().getToken().getProject().getId();
        }
        // 过滤当前租户下的安全组
        Map<String, String> filteringParams = new HashMap<>();
        if (this.getVersion().equals(Version.V2)) {
            filteringParams.put("tenant_id", tenantId);
        } else if (this.getVersion().equals(Version.V3)) {
            filteringParams.put("project_id", tenantId);
        }
        List<? extends SecurityGroup> sGroups = this.getClient().networking().securitygroup().list(filteringParams);
        for (SecurityGroup group : sGroups) {
            if ("Default".equalsIgnoreCase(group.getName())) {
                return new BocloudResult(true, this.convert(group), "success");
            }
        }
        return new BocloudResult(false, "fail");
    }

    private SecurityGroupModel convert(SecurityGroup group) {
        if (null == group) {
            return null;
        }
        SecurityGroupModel model = new SecurityGroupModel(group.getName(), group.getId(), group.getDescription(),
                new Date(), null);
        return model;
    }

}
