package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.openstack4j.api.Builders;
import org.openstack4j.api.exceptions.ClientResponseException;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.network.IPVersionType;
import org.openstack4j.model.network.Pool;
import org.openstack4j.model.network.Subnet;
import org.openstack4j.model.network.builder.SubnetBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.OpenStackProvider;
import com.bocloud.cmp.provider.utils.IPAddress;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;

/**
 * openstack子网操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/15
 */
@Provider("bocloud.cmp.openstack")
public class OpenStackSubnetProvider extends OpenStackProvider implements SubnetProvider {

    private static Logger logger = LoggerFactory.getLogger(OpenStackSubnetProvider.class);

    public OpenStackSubnetProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        List<SubnetModel> models = Lists.newArrayList();
        List<? extends Subnet> subnets = Lists.newArrayList();
        subnets = this.getClient().networking().subnet().list();
        if (null != subnets) {
            for (Subnet subnet : subnets) {
                models.add(convertSubnet(subnet));
            }
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult create(SubnetModel subnet) {
        Subnet subnetInfo = buildSubnetBuilder(subnet);
        logger.info("创建的子网信息：" + JSONObject.toJSONString(subnetInfo));
        try {
            Subnet result = this.getClient().networking().subnet().create(subnetInfo);
            if (null == result) {
                return new BocloudResult(false, "create fail！");
            } else {
                BocloudResult detail = detail(result.getId());
                if (detail.isSuccess()) {
                    return new BocloudResult(true, detail.getData(), "create success");
                } else {
                    return new BocloudResult(false, detail.getMessage());
                }
            }
        } catch (ClientResponseException e) {
            return new BocloudResult(false, e.getMessage());
        }
    }

    /**
     * 构建 subnet
     *
     * @param subnet
     * @return
     */
    private Subnet buildSubnetBuilder(SubnetModel subnet) {
        String cidr = subnet.getCidr();
        IPVersionType type = IPAddress.ipv6Validate(cidr.substring(0, cidr.indexOf("/"))) ? IPVersionType.V6
                : IPVersionType.V4;
        SubnetBuilder builder = Builders.subnet().name(subnet.getName()).networkId(subnet.getNetworkUuid())
                .ipVersion(type).cidr(cidr).enableDHCP(subnet.getDhcpEnabled());
        if (subnet.getGateway() != null && !"".equals(subnet.getGateway())) {
            builder.gateway(subnet.getGateway());
        }
        if (subnet.getIpPools() != null && !"".equals(subnet.getIpPools())) {
            @SuppressWarnings("unchecked")
            List<String> list = JSONObject.parseObject(subnet.getIpPools(), ArrayList.class);
            for (String ipPool : list) {
                String[] startEndIp = ipPool.split("-");
                builder.addPool(startEndIp[0], startEndIp[1]);
            }
        }
        if (!StringUtils.isEmpty(subnet.getHostRoutes())) {
            @SuppressWarnings("unchecked")
            List<String> list = JSONObject.parseObject(subnet.getHostRoutes(), ArrayList.class);
            if (!ListTool.isEmpty(list)) {
                for (String hostRoute : list) {
                    String[] value = hostRoute.split(",");
                    if (null != value && value.length == 2) {
                        builder.addHostRoute(value[0], value[1]);
                    }
                }
            }
        }
        if (subnet.getDnsNames() != null && !"".equals(subnet.getDnsNames())) {
            @SuppressWarnings("unchecked")
            List<String> list = JSONObject.parseObject(subnet.getDnsNames(), ArrayList.class);
            for (String dns : list) {
                builder.addDNSNameServer(dns);
            }
        }
        return builder.build();

    }

    @Override
    public BocloudResult modify(SubnetModel subnet) {
        Subnet subnetInfo = buildSubnetBuilder(subnet);
        Subnet result = this.getClient().networking().subnet().update(subnet.getSubnetUuid(), subnetInfo);
        if (null == result) {
            return new BocloudResult(false, "修改失败！");
        } else {
            BocloudResult detail = detail(result.getId());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "create success");
            } else {
                return new BocloudResult(false, "修改失败！");
            }
        }
    }

    @Override
    public BocloudResult remove(String subnetId) {
        ActionResponse response = this.getClient().networking().subnet().delete(subnetId);
        if (response.isSuccess()) {
            return new BocloudResult(true, "success");
        } else {
            logger.error("remove subnet [{}] error: {}", subnetId, response.getFault());
            return new BocloudResult(false, response.getFault());
        }
    }

    @Override
    public BocloudResult detail(String subnetId) {
        Subnet subnet = this.getClient().networking().subnet().get(subnetId);
        if (null == subnet) {
            return new BocloudResult(false, "no such subnet");
        } else {
            return new BocloudResult(true, convertSubnet(subnet), "success");
        }
    }

    private SubnetModel convertSubnet(Subnet subnet) {
        SubnetModel model = new SubnetModel();
        model.setName(subnet.getName());
        model.setCidr(subnet.getCidr());
        model.setDhcpEnabled(subnet.isDHCPEnabled());
        model.setDnsNames(JSONObject.toJSONString(subnet.getDnsNames()));
        model.setSubnetUuid(subnet.getId());
        model.setIpVersion(subnet.getIpVersion().name());
        model.setNetworkUuid(subnet.getNetworkId());
        model.setGateway(subnet.getGateway());
        model.setHostRoutes(JSONObject.toJSONString(subnet.getHostRoutes().toString()));
        if (!ListTool.isEmpty(subnet.getAllocationPools())) {
            List<String> ipPools = new ArrayList<>();
            for (Pool pool : subnet.getAllocationPools()) {
                ipPools.add(pool.getStart() + "-" + pool.getEnd());
            }
            model.setIpPools(JSONObject.toJSONString(ipPools));
        }
        return model;
    }

    @Override
    public BocloudResult remove(SubnetModel subnet) {
        // TODO Auto-generated method stub
        return null;
    }

}
