package com.bocloud.cmp.provider.security;

import java.util.ArrayList;
import java.util.List;

import org.openstack4j.api.Builders;
import org.openstack4j.api.exceptions.ClientResponseException;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.network.IPVersionType;
import org.openstack4j.model.network.ext.Firewall;
import org.openstack4j.model.network.ext.FirewallPolicy;
import org.openstack4j.model.network.ext.FirewallRule;
import org.openstack4j.openstack.networking.domain.ext.FirewallRuleStrategy;
import org.openstack4j.openstack.networking.domain.ext.NeutronFirewallRule;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FirewallModel;
import com.bocloud.cmp.model.FirewallPolicyModel;
import com.bocloud.cmp.model.FirewallRuleModel;
import com.bocloud.cmp.provider.OpenStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.FirewallConvertor;
import com.bocloud.cmp.provider.convertor.FirewallPolicyConvertor;
import com.bocloud.cmp.provider.convertor.FirewallRuleConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;

/**
 * openstack防火墙操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/14
 */
@Provider("bocloud.cmp.openstack")
public class OpenStackFirewallProvider extends OpenStackProvider implements FirewallProvider {

    private static Logger logger = LoggerFactory.getLogger(OpenStackFirewallProvider.class);

    public OpenStackFirewallProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        List<FirewallModel> models = Lists.newArrayList();
        List<? extends Firewall> firewalls = this.getClient().networking().firewalls().firewall().list();
        if (null != firewalls) {
            BeanConvertor<FirewallModel, Firewall> convertFirewall = new FirewallConvertor();
            for (Firewall firewall : firewalls) {
                models.add(convertFirewall.convertModel(firewall));
            }
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult create(FirewallModel firewall) {
        try {
            Firewall result = this.getClient().networking().firewalls().firewall()
                    .create(Builders.firewall().name(firewall.getName()).description(firewall.getRemark())
                            .policy(firewall.getPolicyUuid()).shared(firewall.getShared())
                            .routerIds(firewall.getRouterUuids()).adminStateUp(firewall.getAdminStateUp()).build());
            if (null == result) {
                return new BocloudResult(false, "防火墙创建失败");
            }
            BocloudResult detail = detail(result.getId());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "防火墙创建成功");
            } else {
                return new BocloudResult(false, "防火墙创建失败");
            }
        } catch (ClientResponseException e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "防火墙创建失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String firewallId) {
        try {
            Firewall firewall = this.getClient().networking().firewalls().firewall().get(firewallId);
            if (null == firewall) {
                return new BocloudResult(true, "");
            }
            ActionResponse response = this.getClient().networking().firewalls().firewall().delete(firewallId);
            if (response.isSuccess()) {
                return new BocloudResult(true, "success");
            } else {
                logger.error("remove firewall [{}] error: {}", firewallId, response.getFault());
                return new BocloudResult(false, response.getFault());
            }
        } catch (ClientResponseException e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "防火墙删除失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult update(FirewallModel firewall) {
        try {
            Firewall result = this.getClient().networking().firewalls().firewall().update(firewall.getFirewallUuid(),
                    Builders.firewallUpdate().name(firewall.getName()).description(firewall.getRemark())
                            .policy(firewall.getPolicyUuid()).adminStateUp(firewall.getAdminStateUp()).build());
            if (null == result) {
                return new BocloudResult(false, "修改失败！");
            }
            BocloudResult detail = detail(result.getId());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "update success");
            } else {
                return new BocloudResult(false, "修改失败！");
            }
        } catch (ClientResponseException e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "防火墙修改失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult relevanceRouter(FirewallModel firewall) {
        try {
            Firewall oldFw = this.getClient().networking().firewalls().firewall().get(firewall.getFirewallUuid());
            if (null == oldFw) {
                return new BocloudResult(false, "防火墙不存在");
            }
            JSONObject params = new JSONObject();
            JSONObject fw = new JSONObject();
            fw.put("name", oldFw.getName());
            fw.put("description", oldFw.getDescription());
            fw.put("firewall_policy_id", oldFw.getPolicy());
            fw.put("admin_state_up", oldFw.isAdminStateUp());
            if (ListTool.isEmpty(firewall.getRouterUuids())) {
                firewall.setRouterUuids(new ArrayList<>());
            }
            fw.put("router_ids", firewall.getRouterUuids());
            params.put("firewall", fw);
            Result result = this.doPut("/v2.0/fw/firewalls/" + firewall.getFirewallUuid(), params, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "路由更新失败");
            }
            JSONObject fwObj = object.getJSONObject("firewall");
            if (null == fwObj) {
                return new BocloudResult(false, "路由更新失败：防火墙不存在");
            }
            BocloudResult detail = detail(firewall.getFirewallUuid());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "关联路由成功");
            } else {
                return new BocloudResult(false, "路由更新失败！");
            }
        } catch (ClientResponseException e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "路由更新失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String firewallId) {
        BeanConvertor<FirewallModel, Firewall> convertFirewall = new FirewallConvertor();
        Firewall firewall = null;
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();
        while (flag) {
            firewall = this.getClient().networking().firewalls().firewall().get(firewallId);
            if ("ACTIVE".equalsIgnoreCase(firewall.getStatus().name())
                    || "DOWN".equalsIgnoreCase(firewall.getStatus().name())
                    || "ERROR".equalsIgnoreCase(firewall.getStatus().name())
                    || "UNRECOGNIZED".equalsIgnoreCase(firewall.getStatus().name())) {
                flag = false;
            }
            Long endTime = System.currentTimeMillis();
            // 等待5分钟
            if (endTime - startTime > 5 * 60 * 1000) {
                flag = false;
            }
        }
        return new BocloudResult(true, convertFirewall.convertModel(firewall), "success");
    }

    @Override
    public BocloudResult listPolicy() {
        List<FirewallPolicyModel> models = Lists.newArrayList();
        List<? extends FirewallPolicy> firewallPolicys = Lists.newArrayList();
        firewallPolicys = this.getClient().networking().firewalls().firewallpolicy().list();
        if (null != firewallPolicys) {
            BeanConvertor<FirewallPolicyModel, FirewallPolicy> convertFirewallPolicy = new FirewallPolicyConvertor();
            for (FirewallPolicy firewallPolicy : firewallPolicys) {
                models.add(convertFirewallPolicy.convertModel(firewallPolicy));
            }
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult createPolicy(FirewallPolicyModel policyModel) {
        try {
            FirewallPolicy result = this.getClient().networking().firewalls().firewallpolicy()
                    .create(Builders.firewallPolicy().name(policyModel.getName()).description(policyModel.getRemark())
                            .shared(policyModel.getShared()).audited(policyModel.getAudited())
                            .firewallRules(policyModel.getRuleUuids()).build());
            if (null == result) {
                return new BocloudResult(false, "防火墙策略创建失败！");
            }
            BocloudResult detail = detailPolicy(result.getId());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "create success");
            } else {
                return new BocloudResult(false, "防火墙策略创建失败！");
            }
        } catch (ClientResponseException e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "防火墙策略创建失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult insertRule(String policyId, String ruleId, String type, String referRuleId) {
        try {
            FirewallPolicy result = this.getClient().networking().firewalls().firewallpolicy()
                    .insertFirewallRuleInPolicy(policyId, ruleId, FirewallRuleStrategy.RuleInsertStrategyType.valueOf(type), referRuleId);
            if (null == result) {
                return new BocloudResult(false, "插入规则失败！");
            }
            BocloudResult detail = detailPolicy(result.getId());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "insert firewall rule in policy success");
            } else {
                return new BocloudResult(false, "插入规则失败！");
            }
        } catch (ClientResponseException e) {
            logger.error("插入规则失败:{}", e);
            return new BocloudResult(false, "插入规则失败！【" + e.getMessage() + "】");
        }
    }

    @Override
    public BocloudResult removePolicy(String policyId) {
        try {
            FirewallPolicy firewallPolicy = this.getClient().networking().firewalls().firewallpolicy().get(policyId);
            if (null == firewallPolicy) {
                return new BocloudResult(true, "");
            }
            ActionResponse response = this.getClient().networking().firewalls().firewallpolicy().delete(policyId);
            if (response.isSuccess()) {
                return new BocloudResult(true, "success");
            } else {
                logger.error("remove firewall policy [{}] error: {}", policyId, response.getFault());
                return new BocloudResult(false, response.getFault());
            }
        } catch (ClientResponseException e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "移除防火墙策略失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult removeRule(String policyId, String ruleId) {
        try {
            FirewallPolicy result = this.getClient().networking().firewalls().firewallpolicy()
                    .removeFirewallRuleFromPolicy(policyId, ruleId);
            if (null == result) {
                return new BocloudResult(false, "防火墙策略移除规则失败！");
            }
            BocloudResult detail = detailPolicy(result.getId());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "remove firewall rule from firewall policy success");
            } else {
                return new BocloudResult(false, "防火墙策略移除规则失败！");
            }
        } catch (ClientResponseException e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "防火墙策略移除规则失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult updatePolicy(FirewallPolicyModel policyModel) {
        try {
            FirewallPolicy result = this.getClient().networking().firewalls().firewallpolicy()
                    .update(policyModel.getPolicyUuid(),
                            Builders.firewallPolicyUpdate().name(policyModel.getName())
                                    .description(policyModel.getRemark()).shared(policyModel.getShared())
                                    .audited(policyModel.getAudited()).build());
            if (null == result) {
                return new BocloudResult(false, "更新失败！");
            }
            BocloudResult detail = detailPolicy(result.getId());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "update success");
            } else {
                return new BocloudResult(false, "更新失败！");
            }
        } catch (ClientResponseException e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "更新策略失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detailPolicy(String policyId) {
        FirewallPolicy firewallPolicy = this.getClient().networking().firewalls().firewallpolicy().get(policyId);
        if (null == firewallPolicy) {
            return new BocloudResult(false, "no such firewall policy");
        } else {
            BeanConvertor<FirewallPolicyModel, FirewallPolicy> convertFirewallPolicy = new FirewallPolicyConvertor();
            return new BocloudResult(true, convertFirewallPolicy.convertModel(firewallPolicy), "success");
        }
    }

    @Override
    public BocloudResult listRule() {
        List<FirewallRuleModel> models = Lists.newArrayList();
        List<? extends FirewallRule> firewallRules = Lists.newArrayList();
        firewallRules = this.getClient().networking().firewalls().firewallrule().list();
        if (null != firewallRules) {
            BeanConvertor<FirewallRuleModel, FirewallRule> convertFirewallRule = new FirewallRuleConvertor();
            for (FirewallRule firewallRule : firewallRules) {
                models.add(convertFirewallRule.convertModel(firewallRule));
            }
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult createRule(FirewallRuleModel ruleModel) {
        try {
            if (null == ruleModel.getProtocol() || "".equals(ruleModel.getProtocol())) {
                ruleModel.setProtocol(null);
            }
            FirewallRule result = this.getClient().networking().firewalls().firewallrule()
                    .create(Builders.firewallRule().name(ruleModel.getName()).description(ruleModel.getRemark())
                            .shared(ruleModel.getShared())
                            .protocol(NeutronFirewallRule.IPProtocol.value(ruleModel.getProtocol()))
                            .ipVersion("V4".equals(ruleModel.getIpVersion()) ? IPVersionType.V4 : IPVersionType.V6)
                            .sourceIpAddress("".equals(ruleModel.getSourceIp()) ? null : ruleModel.getSourceIp())
                            .destinationIpAddress("".equals(ruleModel.getDestIp()) ? null : ruleModel.getDestIp())
                            .sourcePort("".equals(ruleModel.getSourcePort()) ? null : ruleModel.getSourcePort())
                            .destinationPort("".equals(ruleModel.getDestPort()) ? null : ruleModel.getDestPort())
                            .action(NeutronFirewallRule.FirewallRuleAction.value(ruleModel.getAction()))
                            .enabled(ruleModel.getEnabled()).build());
            if (null == result) {
                return new BocloudResult(false, "创建失败！");
            }
            BocloudResult detail = detailRule(result.getId());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "create success");
            } else {
                return new BocloudResult(false, "创建失败！");
            }
        } catch (ClientResponseException e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "创建规则失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult removeRule(String ruleId) {
        try {
            FirewallRule firewallRule = this.getClient().networking().firewalls().firewallrule().get(ruleId);
            if (null == firewallRule) {
                return new BocloudResult(true, "");
            }
            ActionResponse response = this.getClient().networking().firewalls().firewallrule().delete(ruleId);
            if (response.isSuccess()) {
                return new BocloudResult(true, "success");
            } else {
                logger.error("remove firewall rule [{}] error: {}", ruleId, response.getFault());
                return new BocloudResult(false, response.getFault());
            }
        } catch (ClientResponseException e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "删除规则失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult updateRule(FirewallRuleModel ruleModel) {
        try {
            if (null == ruleModel.getProtocol() || "".equals(ruleModel.getProtocol())) {
                ruleModel.setProtocol(null);
            }
            FirewallRule result = this.getClient().networking().firewalls().firewallrule()
                    .update(ruleModel.getRuleUuid(), Builders.firewallRuleUpdate().name(ruleModel.getName())
                            .description(ruleModel.getRemark()).shared(ruleModel.getShared())
                            .protocol("UNRECOGNIZED".equals(ruleModel.getProtocol())
                                    ? NeutronFirewallRule.IPProtocol.value("NONE")
                                    : NeutronFirewallRule.IPProtocol.value(ruleModel.getProtocol()))
                            .ipVersion("V4".equals(ruleModel.getIpVersion()) ? IPVersionType.V4 : IPVersionType.V6)
                            .sourceIpAddress("".equals(ruleModel.getSourceIp()) ? null : ruleModel.getSourceIp())
                            .destinationIpAddress("".equals(ruleModel.getDestIp()) ? null : ruleModel.getDestIp())
                            .sourcePort("".equals(ruleModel.getSourcePort()) ? null : ruleModel.getSourcePort())
                            .destinationPort("".equals(ruleModel.getDestPort()) ? null : ruleModel.getDestPort())
                            .action(NeutronFirewallRule.FirewallRuleAction.value(ruleModel.getAction()))
                            .enabled(ruleModel.getEnabled()).build());
            if (null == result) {
                return new BocloudResult(false, "更新失败！");
            }
            BocloudResult detail = detailRule(result.getId());
            if (detail.isSuccess()) {
                return new BocloudResult(true, detail.getData(), "修改成功！");
            } else {
                return new BocloudResult(false, "更新失败！");
            }
        } catch (ClientResponseException e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "更新规则失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detailRule(String ruleId) {
        FirewallRule firewallRule = this.getClient().networking().firewalls().firewallrule().get(ruleId);
        if (null == firewallRule) {
            return new BocloudResult(false, "no such firewall rule");
        } else {
            BeanConvertor<FirewallRuleModel, FirewallRule> convertFirewallRule = new FirewallRuleConvertor();
            return new BocloudResult(true, convertFirewallRule.convertModel(firewallRule), "success");
        }
    }

}
