package com.bocloud.cmp.provider;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ClusterModel;
import com.bocloud.cmp.model.FlavorExtraSpecsModel;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.model.HmcHostModel;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.NetworkModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.model.SanFabricModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.StorageModel;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.VolumePoolModel;
import com.bocloud.cmp.model.VolumeTemplateModel;
import com.bocloud.cmp.model.powervc.HostSeaAdapterModel;
import com.bocloud.cmp.model.powervc.NetworkSeaAdaterModel;
import com.bocloud.cmp.model.powervc.PowervcConstant;
import com.bocloud.cmp.provider.powervc4j.api.PVClient;
import com.bocloud.cmp.provider.powervc4j.impl.identity.Authenticator;
import com.bocloud.cmp.provider.powervc4j.model.identity.Token;
import com.bocloud.common.model.Result;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * PowerVC驱动
 * <p>
 * PowerVC: IBM Power Virtualization Center
 * </p>
 *
 * @author sxp
 * @since Mar 14, 2018
 */
public class PowervcProvider extends GenericProvider implements SuperProvider {
    private static final Logger logger = LoggerFactory.getLogger(PowervcProvider.class);

    private PowervcConstant.Version version;

    private Butler butler;

    /**
     * 平台认证路径
     * <p>
     * 例如https://192.168.2.61，注意powervc使用的是https协议
     * </p>
     */
    private String endpoint;

    /**
     * 管理员账号
     */
    private String username;

    /**
     * 管理员密码
     */
    private String password;

    /**
     * 域名称
     */
    private String domain;

    /**
     * 工程名
     */
    private String project;

    // ------------------

    /**
     * 授权令牌
     */
    private Token token;

    /**
     * 是否可以访问
     */
    private boolean accessable;

    /**
     * 错误信息
     */
    private String error;

    /**
     * 是否只显示当前租户的资源
     */
    private boolean onlyShowCurrentTenantResource = true;

    // ------------------

    /**
     * 客户端
     */
    private PVClient client;
    private boolean prepared = false;

    public PowervcProvider(Butler butler) {
        super(butler);
        Objects.requireNonNull(butler.getVersion());
        this.version = PowervcConstant.Version.getByValue(butler.getVersion());
        this.butler = butler;
        this.endpoint = butler.getUrl();
        this.username = butler.getUsername();
        this.password = butler.getPassword();
        this.domain = butler.getDomainName();
        this.project = butler.getProjectName();
        init();
        prepare();
    }


    @Override
    public boolean accessable() {
        return accessable;
    }

    private void init() {
        if (endpoint != null && endpoint.endsWith("/")) {
            endpoint = endpoint.substring(0, endpoint.length() - 1);
        }

        Result result = paramCheck();

        if (result.isFailed()) {
            this.accessable = false;
            logger.error("PowervcDriver初始化失败: {}", result.getMessage());
            this.error = result.getMessage();
            return;
        }

        refreshToken();

        if (accessable) {
            client = new PVClient(this);
        }
    }

    /**
     * 刷新PowerVC授权令牌
     *
     * @author sxp
     * @since Mar 19, 2018
     */
    public boolean refreshToken() {
        // 2019-07-24 : 该方法用来刷新token，所以不考虑缓存
        Token newToken = Authenticator.getToken(endpoint, username, password, domain, project);

        if (newToken != null) {
            accessable = true;
            error = null;
        } else {
            accessable = false;
            error = "PowervcDriver无法获得有效的token信息";
        }

        token = newToken;

        return accessable;
    }

    private Result paramCheck() {
        if (StringUtils.isEmpty(endpoint)) {
            return new Result(false, "平台认证路径不能为空!");
        }
        if (StringUtils.isEmpty(username)) {
            return new Result(false, "用户名不能为空!");
        }
        if (StringUtils.isEmpty(password)) {
            return new Result(false, "密码不能为空!");
        }
        if (StringUtils.isEmpty(project)) {
            return new Result(false, "工程名(project)不能为空!");
        }
        if (StringUtils.isEmpty(domain)) {
            return new Result(false, "区域名(domain)不能为空!");
        }
        return new Result(true, "success");
    }

    // ---- getters and setters

    /**
     * 获取有效的Token信息
     *
     * @author sxp
     * @since Mar 20, 2018
     */
    public Token getValidToken() {
        boolean accessable = this.isAccessable();
        if (!accessable) {
            return null;
        }

        boolean valid = Authenticator.validateToken(token);
        if (!valid) {
            refreshToken();
        }

        if (token == null) {
            return null;
        }

        return token;
    }

    /**
     * 获取API前缀
     *
     * @author sxp
     * @since Mar 20, 2018
     */
    public Map<String, String> getApiPrefix() {
        // 如果当前token过去，自动刷新token
        Token token = getValidToken();
        if (token == null) {
            return null;
        }

        return token.getApiPrefix();
    }

    public String getEndpoint() {
        return endpoint;
    }

    public String getUsername() {
        return username;
    }

    public String getPassword() {
        return password;
    }

    public String getDomain() {
        return domain;
    }

    public String getProject() {
        return project;
    }

    public Token getToken() {
        return token;
    }

    public boolean isAccessable() {
        return accessable;
    }

    public String getError() {
        return error;
    }

    public void setError(String error) {
        this.error = error;
    }

    public PVClient getClient() {
        return client;
    }

    public boolean isOnlyShowCurrentTenantResource() {
        return onlyShowCurrentTenantResource;
    }

    public void setOnlyShowCurrentTenantResource(boolean onlyShowCurrentTenantResource) {
        this.onlyShowCurrentTenantResource = onlyShowCurrentTenantResource;
    }

    public void resetOnlyShowCurrentTenantResource() {
        onlyShowCurrentTenantResource = true;
    }

    public Butler getButler() {
        return butler;
    }

    public PowervcConstant.Version getVersion() {
        return version;
    }

    /**
     * 为获取数据做好准备
     * <p>
     * 获得有效的token
     * </p>
     *
     * @author sxp
     * @since Mar 27, 2018
     */
    protected void prepare() {
        try {
            boolean accessable = this.isAccessable();
            if (!accessable) {
                return;
            }

            Token token = this.getValidToken();
            Objects.requireNonNull(token, "token");

            String tenantId = token.getTenantId();
            if (StringUtils.isEmpty(tenantId)) {
                this.setError("当前认证账号下租户不存在");
                return;
            }

            PVClient client = this.getClient();
            if (client == null) {
                this.setError("当前PowerVC Driver的客户端未初始化");
                return;
            }

            prepared = true;
        } catch (Exception e) {
            String errorMsg = "初始化失败:" + e.getMessage();
            this.setError(errorMsg);
        }
    }

    /**
     * 获取宿主机列表
     *
     * @author sxp
     * @since Mar 27, 2018
     */
    public List<HostModel> listHypervisor() {
        if (!prepared) {
            return null;
        }
        return this.getClient().compute(version).hypervisors(version).list();
    }

    /**
     * 获取云主机列表
     *
     * @author sxp
     * @since Mar 27, 2018
     */
    public List<ServerModel> listServer(boolean allTenants) {
        if (!prepared) {
            return null;
        }
        List<ServerModel> server = this.getClient().compute(version).servers(version).list(allTenants);
        for (int i = 0; i < server.size(); i++) {
                // 查询该云主机挂载的卷信息
                List<VolumeModel> diskDevices = new ArrayList<>(server.get(i).getDiskDevices().size());
                for (VolumeModel diskDevice : server.get(i).getDiskDevices()) {
                    String volumeUuid = diskDevice.getVolumeUuid();
                    VolumeModel volumeModel = this.getClient().blockStorage(getVersion()).volumes(getVersion()).get(volumeUuid);
                    diskDevices.add(volumeModel);
                }
                server.get(i).setDiskDevices(diskDevices);
            }
        return server;
    }
    public List<ServerModel> listServer() {
        return listServer(true);
    }

    /**
     * 获取网络列表
     *
     * @author sxp
     * @since Mar 27, 2018
     */
    public List<NetworkModel> listNetwork() {
        if (!prepared) {
            return null;
        }
        List<NetworkModel> networks = this.getClient().networking(version).network(version).list();
        for (NetworkModel network : networks) {
            List<SubnetModel> subnets = new ArrayList<>(network.getSubnetIds().size());
            for (String subnetId : network.getSubnetIds()) {
                SubnetModel subnetModel = this.getClient().networking(getVersion()).subnet(getVersion()).get(subnetId);
                subnets.add(subnetModel);
            }
            network.setSubnets(subnets);
        }
        return networks;
    }

    /**
     * 获取子网列表
     * @return
     */
    public List<SubnetModel> listSubnet() {
        if (!prepared) {
            return null;
        }
        return this.getClient().networking(version).subnet(version).list();
    }

    /**
     * 获取存储卷列表
     * @return
     */
    public List<VolumeModel> listVolume() {
        if (!prepared) {
            return null;
        }
        return this.getClient().blockStorage(version).volumes(version).list();
    }

    /**
     * 获取存储器模版列表
     * @return
     */
    public List<VolumeTemplateModel> listVolumeTemplate() {
        if (!prepared) {
            return null;
        }
        return this.getClient().blockStorage(version).templates(version).list();
    }

    /**
     * 获取存储服务器列表
     * @return
     */
    public List<StorageModel> listStorageDevice() {
        if (!prepared) {
            return null;
        }
        List<StorageModel> storageModels = this.getClient().blockStorage(version).storageDevices(version).list();
        if (storageModels != null && storageModels.size() > 0) {
            for (StorageModel storageModel : storageModels) {
                List<VolumePoolModel> poolModels = this.getClient().blockStorage(version).pools(version).list(storageModel.getHost());
                storageModel.setPools(poolModels);
            }
        }
        return storageModels;
    }

    /**
     * 获取映像列表
     * @return
     */
    public List<ImageModel> listImage() {
        if (!prepared) {
            return null;
        }
        return this.getClient().image(version).images(version).list();
    }

    /**
     * 获取规格列表
     * @return
     */
    public List<FlavorModel> listFlavor() {
        if (!prepared) {
            return null;
        }
        List<FlavorModel> flavors = this.getClient().compute(version).flavors(version).list(true);
        if (flavors != null) {
            for (FlavorModel flavor : flavors) {
                FlavorExtraSpecsModel extra = this.getClient().compute(version).flavorExtras(version).get(flavor.getFlavorUuid());
                flavor.setExtraSpecs(extra);
            }
        }
        return flavors;
    }

    /**
     * 获取端口列表
     * @return
     */
    public List<PortModel> listPort() {
        if (!prepared) {
            return null;
        }
        return this.getClient().networking(version).port(version).list();
    }

    /**
     * 获取HMC连接列表
     * @return
     */
    public List<HmcHostModel> listHmc() {
        if (!prepared) {
            return null;
        }
        return this.getClient().compute(version).hmcs(version).list();
    }

    /**
     * 获取HEA列表
     * @return
     */
    public List<HostSeaAdapterModel> listHostSea() {
        if (!prepared) {
            return null;
        }
        return this.getClient().compute(version).hostSea(version).list();
    }

    /**
     * 获取主机组列表
     * @return
     */
    public List<ClusterModel> listCluster() {
        if (!prepared) {
            return null;
        }
        return this.getClient().compute(version).osAggregate(version).list();
    }

    /**
     * 获取存储池列表
     * @param storageHostName
     * @return
     */
    public List<VolumePoolModel> listVolumePool(String storageHostName) {
        if (!prepared) {
            return null;
        }
        return this.getClient().blockStorage(version).pools(version).list(storageHostName);
    }

    /**
     * 获取光纤网列表
     * @return
     */
    public List<SanFabricModel> listFabrics() {
        if (!prepared) {
            return null;
        }
        return this.getClient().blockStorage(version).fabrics(version).list();
    }

    /**
     * 获取网络配置的sea
     * @return
     */
    public List<NetworkSeaAdaterModel> listNetworkSea() {
        if (!prepared) {
            return null;
        }
        return this.getClient().compute(version).hostSea(version).listMapping();
    }

    public boolean isPrepared() {
        return prepared;
    }

}

