package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.VolumeMetadataModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * PowerVC镜像驱动
 *
 * @author sxp
 * @since Mar 28, 2018
 */
@Provider("bocloud.cmp.powervc")
public class PowervcImageProvider extends PowervcProvider implements ImageProvider {
    private static Logger logger = LoggerFactory.getLogger(PowervcImageProvider.class);

    public PowervcImageProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult list() {
        List<ImageModel> images = listImage();
        return new BocloudResult(true, images, "success");
    }

    @Override
    public BocloudResult detail(String imageId) {
        ImageModel image = this.getClient().image(getVersion()).images(getVersion()).get(imageId);
        return new BocloudResult(true, image, "success");
    }

    @Override
    public BocloudResult remove(String imageId) {
        return this.getClient().image(getVersion()).images(getVersion()).delete(imageId);
    }

    @Override
    public BocloudResult create(ImageModel imageModel) {
        List<VolumeModel> diskDeviceMappings = imageModel.getDiskDeviceMappings();

        JSONArray volumes = new JSONArray();
        for (int i = 0; i < diskDeviceMappings.size(); i++) {
            VolumeModel volumeModel = diskDeviceMappings.get(i);
            JSONObject volume = new JSONObject();
            volume.put("volume_id", volumeModel.getVolumeUuid());
            volume.put("source_type", "volume");
            volume.put("device_type", "disk");
            volume.put("destination_type", "volume");
            volume.put("delete_on_termination", true);
            volume.put("boot_index", i);
            volumes.add(volume);
        }

        JSONObject params = new JSONObject();
        params.put("block_device_mapping", JSON.toJSONString(volumes));
        params.put("architecture", imageModel.getArchitecture() == null ? "ppc64" : imageModel.getArchitecture());
        params.put("bdm_v2", "true");
        params.put("container_format", "bare");
        params.put("disk_format", "raw");
        params.put("hypervisor_type", "phyp");
        params.put("min_disk", imageModel.getMinDisk() == null ? 1 : imageModel.getMinDisk());
        params.put("name", imageModel.getName());
        params.put("os_distro", imageModel.getOsName());
        params.put("powervc_comments", imageModel.getRemark());
        params.put("root_device_name", "/dev/sda");
        params.put("visibility", "private");

        BocloudResult result = this.getClient().image(getVersion()).images(getVersion()).create(params);
        if (result.isFailed()) {
            logger.error("创建image失败:" + result.getMessage());
            return new BocloudResult(false, "创建image失败");
        }
        imageModel = (ImageModel) result.getData();
        result = this.getClient().image(getVersion()).images(getVersion()).file(imageModel.getImageUuid());
        if (result.isFailed()) {
            logger.error("创建image失败:" + result.getMessage());
            return new BocloudResult(false, "创建image失败");
        }

        for (int i = 0; i < diskDeviceMappings.size(); i++) {
            VolumeModel volumeModel = diskDeviceMappings.get(i);
            String volumeUuid = volumeModel.getVolumeUuid();
            Boolean bootVolume = volumeModel.getBootVolume();
            volumeModel = this.getClient().blockStorage(getVersion()).volumes(getVersion()).get(volumeUuid);
            if (!volumeModel.getBootVolume().equals(bootVolume)) {
                VolumeMetadataModel metadataModel = new VolumeMetadataModel();
                metadataModel.setWwn(volumeModel.getWwn());
                metadataModel.setBoot(bootVolume);
                metadataModel.setImage(true);
                metadataModel.setImageId(imageModel.getImageUuid());
                result = this.getClient().blockStorage(getVersion()).volumes(getVersion()).setMetadata(volumeUuid, metadataModel);
                if (result.isFailed()) {
                    logger.error("创建image失败:" + result.getMessage());
                    return new BocloudResult(false, "创建image失败");
                }
            }
        }

        imageModel = this.getClient().image(getVersion()).images(getVersion()).get(imageModel.getImageUuid());
        return new BocloudResult(true, imageModel, "创建image成功");
    }

    @Override
    public BocloudResult modify(ImageModel image) {
        return new BocloudResult(false, "PowerVC不支持该API");
    }

    @Override
    public BocloudResult file(String imageId) {
        return this.getClient().image(getVersion()).images(getVersion()).file(imageId);
    }

    @Override
    public BocloudResult shareImage(ImageModel imageModel) {
        return new BocloudResult(false, "PowerVC不支持该API");
    }

    @Override
    public BocloudResult syncImages(ImageModel imageModel) {
        return new BocloudResult(false, "PowerVC不支持该API");
    }
}
