package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

@Provider("bocloud.cmp.powervc")
public class PowervcOsHostProvider extends PowervcProvider implements OsHostProvider {
    private static Logger logger = LoggerFactory.getLogger(PowervcOsHostProvider.class);

    public PowervcOsHostProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult get() {
        List<OsHost> hosts = this.getClient().compute(getVersion()).oshost(getVersion()).list();
        return new BocloudResult(true, hosts, "success");
    }

    @Override
    public BocloudResult get(String hostName) {
        OsHost osHost = this.getClient().compute(getVersion()).oshost(getVersion()).get(hostName);
        return new BocloudResult(true, osHost, "success");
    }

    @Override
    public BocloudResult create(OsHost osHost) {
        return this.getClient().compute(getVersion()).oshost(getVersion()).create(osHost);
    }

    @Override
    public BocloudResult delete(String uuid) {
        return this.getClient().compute(getVersion()).oshost(getVersion()).delete(uuid);
    }

    @Override
    public BocloudResult onboard(String hostName, List<String> serverUuids) {
        BocloudResult bocloudResult = this.getClient().compute(getVersion()).oshost(getVersion()).onboard(hostName, serverUuids);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }

        List<ServerModel> serverModels = new ArrayList<>();
        List<String> successUuids = new ArrayList<>();
        long start = System.currentTimeMillis();
        while (System.currentTimeMillis() - start < 60 * 1000 * serverUuids.size()) {
            for (String serverUuid : serverUuids) {
                if (!successUuids.contains(serverUuid)) {
                    ServerModel serverModel = this.getClient().compute(getVersion()).servers(getVersion()).get(serverUuid);
                    if (serverModel != null) {
                        successUuids.add(serverUuid);
                        List<VolumeModel> volumeModels = new ArrayList<>();
                        for (VolumeModel diskDevice : serverModel.getDiskDevices()) {
                            VolumeModel volumeModel = this.getClient().blockStorage(getVersion()).volumes(getVersion()).get(diskDevice.getVolumeUuid());
                            volumeModels.add(volumeModel);
                        }
                        serverModel.setDiskDevices(volumeModels);
                        serverModels.add(serverModel);
                    }
                }
            }
            if (successUuids.size() == serverUuids.size()) {
                return new BocloudResult(true, serverModels, "执行成功");
            }
            logger.info("正在管理现有云主机" + successUuids.size() + "/" + serverUuids.size() + "，3秒后将重新检测执行结果...");
            try {
                Thread.sleep(3000);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                logger.info("管理现有云主机异常", e);
                break;
            }
        }
        return new BocloudResult(false, serverModels,"执行超时");
    }

    @Override
    public BocloudResult unmanage(String hostName, List<String> serverUuids) {
        List<String> successServerUuids = new ArrayList<>();
        List<String> successPortUuids = new ArrayList<>();
        List<String> failurePortUuids = new ArrayList<>();
        Set<String> errorMessage = new HashSet<>();
        // 在取消管理之前先查询所有的端口，取消管理后需要将该云主机使用的且没有fixedIp的端口删除掉
        // 为什么会这样呢？因为Powervc管理现有云主机后，可能会在现有的某个网络中创建一个port且没有ip地址，取消管理该云主机这厮竟然不给删掉，所以我们帮他删
        List<PortModel> portModels = this.getClient().networking(getVersion()).port(getVersion()).list();
        // 执行取消管理操作
        BocloudResult unmanageResult = this.getClient().compute(getVersion()).oshost(getVersion()).unmanage(hostName, serverUuids);
        if (unmanageResult.isSuccess()) {
            // 循环检测是否取消管理成功
            long start = System.currentTimeMillis();
            while (System.currentTimeMillis() - start < 60 * 1000 * serverUuids.size()) {
                for (String serverUuid : serverUuids) {
                    if (!successServerUuids.contains(serverUuid)) {
                        ServerModel serverModel = this.getClient().compute(getVersion()).servers(getVersion()).get(serverUuid);
                        if (serverModel == null) {
                            successServerUuids.add(serverUuid);
                        }
                    }
                }
                if (successServerUuids.size() == serverUuids.size()) {
                    break;
                }
                logger.info("正在取消管理云主机，3秒后将重新检测执行结果...");
                try {
                    Thread.sleep(3000);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    logger.info("取消管理现有云主机异常", e);
                    break;
                }

            }
            for (PortModel portModel : portModels) {
                if (successServerUuids.contains(portModel.getDeviceUuid()) && (portModel.getPortFixedIps() == null || portModel.getPortFixedIps().size() == 0)) {
                    // 重新查询一下port是不是还在，如果不在了就直接添加到成功列表里，如果还在就要执行删除
                    // 为什么要这样呢？因为Powervc管理现有云主机后，可能创建一个新的网络，且取消管理该云主机后会将该网络删除，删除网络的时候该网络下的port也删除了，所以这里要判断一下免得报错
                    PortModel portResearch = this.getClient().networking(getVersion()).port(getVersion()).get(portModel.getPortUuid());
                    if (portResearch == null) {
                        successPortUuids.add(portModel.getPortUuid());
                    } else {
                        BocloudResult deletePortResult = this.getClient().networking(getVersion()).port(getVersion()).remove(portModel.getPortUuid());
                        if (deletePortResult.isFailed()) {
                            failurePortUuids.add(portModel.getPortUuid());
                            errorMessage.add(deletePortResult.getMessage());
                        } else {
                            successPortUuids.add(portModel.getPortUuid());
                        }
                    }
                }
            }
        } else {
            errorMessage.add(unmanageResult.getMessage());
        }
        Map<String, List> resultMap = new HashMap<>();
        resultMap.put("successServerUuids", successServerUuids);
        resultMap.put("successPortUuids", successPortUuids);
        resultMap.put("failurePortUuids", failurePortUuids);
        return new BocloudResult(
                serverUuids.size() == successServerUuids.size() && failurePortUuids.size() == 0,
                resultMap, errorMessage.toString()
        );
    }

    @Override
    public BocloudResult allServers(String hostName) {
        return this.getClient().compute(getVersion()).oshost(getVersion()).allServers(hostName);
    }
}
