package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.AddressBean;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorExtraSpecsModel;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.NetworkBean;
import com.bocloud.cmp.model.NetworkModel;
import com.bocloud.cmp.model.OSMigrateModel;
import com.bocloud.cmp.model.PortFixedIpModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VmNetworkBean;
import com.bocloud.cmp.model.VolumeMetadataModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.cmp.provider.powervc4j.common.util.MapUtils;
import com.bocloud.cmp.provider.powervc4j.model.compute.RebootType;
import com.bocloud.cmp.provider.powervc4j.model.compute.ServerAction;
import com.bocloud.cmp.provider.powervc4j.model.compute.TaskStateType;
import com.bocloud.cmp.provider.storage.PowervcVolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * PowerVC云主机驱动
 *
 * @author sxp
 * @since Mar 30, 2018
 */
@Provider("bocloud.cmp.powervc")
public class PowervcServerProvider extends PowervcProvider implements ServerProvider {
    private static final String NEW_LINE = "\n";
    private static Logger logger = LoggerFactory.getLogger(PowervcServerProvider.class);

    public PowervcServerProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult list() {
        List<ServerModel> servers = listServer(true);
        return new BocloudResult(servers != null, servers, "");
    }

    @Override
    public BocloudResult list(String hostId) {
        return null;
    }

    @Override
    public BocloudResult detail(String instanceId) {
        ServerModel server = this.getClient().compute(getVersion()).servers(getVersion()).get(instanceId);
        if (server != null) {
            // 查询该云主机挂载的卷信息
            List<VolumeModel> diskDevices = new ArrayList<>(server.getDiskDevices().size());
            for (VolumeModel diskDevice : server.getDiskDevices()) {
                String volumeUuid = diskDevice.getVolumeUuid();
                VolumeModel volumeModel = this.getClient().blockStorage(getVersion()).volumes(getVersion()).get(volumeUuid);
                if (volumeModel != null) {
                    // 如果云主机由于卷的某些问题创建失败，会在云主机中记录着卷的uuid，但是在卷列表中却没有这个卷，所以这里判断下非空
                    diskDevices.add(volumeModel);
                }
            }
            server.setDiskDevices(diskDevices);
        }
        return new BocloudResult(true, server, "查询成功");
    }

    @Override
    public BocloudResult remove(String instanceId, String name) {
        BocloudResult removeResult = this.getClient().compute(getVersion()).servers(getVersion()).delete(instanceId);
        if (removeResult.isFailed()) {
            return removeResult;
        }
        long start = System.currentTimeMillis();
        ServerModel serverModel = null;
        while (System.currentTimeMillis() - start < 3 * 60 * 1000) {
            serverModel = this.getClient().compute(getVersion()).servers(getVersion()).get(instanceId);
            if (serverModel == null) {
                return new BocloudResult(true, serverModel, "删除成功");
            }
            logger.info("正在删除云主机，3秒后将再次检测...");
            try {
                Thread.sleep(3000);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                logger.info("删除云主机异常", e);
                break;
            }
        }
        return new BocloudResult(false, serverModel, "删除超时");
    }

    @Override
    public BocloudResult modify(ServerModel server) {
        BocloudResult bocloudResult = this.getClient().compute(getVersion()).servers(getVersion()).update(server);
        if (bocloudResult.isSuccess()){
            return new BocloudResult(true,"修改成功");
        }
        return new BocloudResult(false,bocloudResult.getMessage());
    }

    @Override
    public BocloudResult create(ServerModel server) {
        JSONObject serverJson = new JSONObject();
        // 规格
        FlavorModel flavorModel = this.getClient().compute(getVersion()).flavors(getVersion()).get(server.getFlavorUuid());
        if (flavorModel == null) {
            return new BocloudResult(false, "未找到规格信息：" + server.getFlavorUuid());
        }
        FlavorExtraSpecsModel flavorExtraSpecsModel = this.getClient().compute(getVersion()).flavorExtras(getVersion()).get(flavorModel.getFlavorUuid());
        if (flavorExtraSpecsModel == null) {
            return new BocloudResult(false, "未找到规格扩展信息：" + flavorModel.getFlavorUuid());
        }
        JSONObject flavor = new JSONObject();
        flavor.put("disk", flavorModel.getDisk());
        // 2019-07-17 : 需要统一将内存代为转换成GB
        flavor.put("ram", server.getMemory().multiply(new BigDecimal(1024)).intValue());
        flavor.put("vcpus", server.getCpu());
        JSONObject flavorExtra = new JSONObject();
        flavorExtra.put("powervm:proc_units", server.getHmcCpu());
        flavorExtra.put("powervm:availability_priority", flavorExtraSpecsModel.getPriority());
        flavorExtra.put("powervm:dedicated_proc", flavorExtraSpecsModel.getDedicatedProc());
        // 2019-07-17 : 需要统一将内存代为转换成GB
        flavorExtra.put("powervm:max_mem", flavorExtraSpecsModel.getMaxMem() == null ? null : flavorExtraSpecsModel.getMaxMem().multiply(new BigDecimal(1024)).intValue());
        flavorExtra.put("powervm:min_mem", flavorExtraSpecsModel.getMinMem() == null ? null : flavorExtraSpecsModel.getMinMem().multiply(new BigDecimal(1024)).intValue());
        flavorExtra.put("powervm:max_proc_units", flavorExtraSpecsModel.getMaxProcUnits());
        flavorExtra.put("powervm:min_proc_units", flavorExtraSpecsModel.getMinProcUnits());
        flavorExtra.put("powervm:max_vcpu", flavorExtraSpecsModel.getMaxVcpu());
        flavorExtra.put("powervm:min_vcpu", flavorExtraSpecsModel.getMinVcpu());
        flavorExtra.put("powervm:shared_weight", flavorExtraSpecsModel.getSharedWeight());
        flavorExtra.put("powervm:uncapped", flavorExtraSpecsModel.getUncapped());
        flavorExtra.put("powervm:processor_compatibility", flavorExtraSpecsModel.getProcCompatibility());
        flavor.put("extra_specs", flavorExtra);
        serverJson.put("flavor", flavor);
        // 镜像
        serverJson.put("imageRef", server.getImageUuid());
        // 云主机名称
        String name = server.getName();
        if (StringUtils.isEmpty(name)) {
            return new BocloudResult(false, "云主机名称不能为空");
        }
        serverJson.put("name", name);
        // 云主机描述
        serverJson.put("description", server.getRemark());
        // 主机名称
        String hostName = server.getHostName();
        if (StringUtils.isNotEmpty(hostName)) {
            serverJson.put("availability_zone", ":" + hostName);
        }
        // 磁盘
        JSONArray mappingArray = new JSONArray();
        if (server.getDiskDevices() != null) {
            for (int i = 0; i < server.getDiskDevices().size(); i++) {
                VolumeModel disk = server.getDiskDevices().get(i);
                if (disk.getImageVolume()) {
                    flavorExtra.put("powervm:image_volume_type_" + disk.getVolumeUuid(), disk.getVolumeTemplateUuid());
                } else {
                    PowervcVolumeProvider volumeProvider = new PowervcVolumeProvider(this.getButler());
                    // 创建或查询卷信息
                    BocloudResult volumeResult;
                    if (StringUtils.isEmpty(disk.getVolumeUuid())) {
                        volumeResult = volumeProvider.create(disk);
                    } else {
                        volumeResult = volumeProvider.detail(disk.getVolumeUuid());
                    }
                    if (volumeResult.isFailed()) {
                        return volumeResult;
                    }
                    // 设定卷是否为引导卷
                    VolumeModel volume = (VolumeModel) volumeResult.getData();
                    if (!disk.getBootVolume().equals(volume.getBootVolume())) {
                        VolumeMetadataModel metadata = new VolumeMetadataModel();
                        metadata.setBoot(true);
                        metadata.setImage(volume.getImageVolume());
                        metadata.setWwn(volume.getWwn());
                        BocloudResult metadataResult = volumeProvider.setMetadata(volume.getVolumeUuid(), metadata);
                        if (metadataResult.isFailed()) {
                            return metadataResult;
                        }
                    }
                    // 填充卷相关参数
                    JSONObject mapping = new JSONObject(MapUtils.capacity(7));
                    mapping.put("source_type", "volume");
                    mapping.put("destination_type", "volume");
                    mapping.put("boot_index", disk.getBootIndex());
                    mapping.put("delete_on_termination", disk.getDeleteWithInstance());
                    mapping.put("uuid", volume.getVolumeUuid());
                    mappingArray.add(mapping);
                }
            }
        }
        serverJson.put("block_device_mapping_v2", mappingArray);
        // 密钥对
        String keypairName = server.getKeypairName();
        if (StringUtils.isNotEmpty(keypairName)) {
            serverJson.put("key_name", keypairName);
        }
        // 网络
        List<JSONObject> networks = Lists.newArrayList();
        for (NetworkModel networkModel : server.getNetworks()) {
            JSONObject network = new JSONObject();
            network.put("uuid", networkModel.getNetworkUuid());
            network.put("fixed_ip", networkModel.getFixedIp());
            networks.add(network);
        }
        serverJson.put("networks", networks);
        // 激活输入
        serverJson.put("user_data", server.getActiveCmd());

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("server", serverJson);

        // 该云主机所属主机组
        if (StringUtils.isNotEmpty(server.getClusterUuid())) {
            JSONObject osSchedulerHintsJson = new JSONObject();
            osSchedulerHintsJson.put("host_aggregate_id", server.getClusterUuid());
            params.put("os:scheduler_hints", osSchedulerHintsJson);
        }

        // 调用API创建云主机
        BocloudResult bocloudResult = this.getClient().compute(getVersion()).servers(getVersion()).create(params);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        ServerModel createdServer = (ServerModel) bocloudResult.getData();

        long start = System.currentTimeMillis();
        while (System.currentTimeMillis() - start < 10 * 60 * 1000) {
            ServerModel detailServer = this.getClient().compute(getVersion()).servers(getVersion()).get(createdServer.getInstanceId());
            if (detailServer != null) {
                if (BocloudStatus.Server.ERROR.name().equals(detailServer.getStatus())){
                    JSONObject props = JSON.parseObject(detailServer.getProps());
                    String errorMessage = props != null && props.get("errorMessage") != null && StringUtils.isNotEmpty((String) props.get("errorMessage")) ?
                            (String) props.get("errorMessage") : "Powervc中云主机状态为" + detailServer.getStatus();
                    return new BocloudResult(false, detailServer, errorMessage);
                } else if (!BocloudStatus.Server.BUILDING.name().equals(detailServer.getStatus())) {
                    return new BocloudResult(true, detailServer, "创建成功");
                }
            }
            logger.info("云主机[" + name + "]正在在构建中，5秒后会重新尝试获取该云主机信息...");
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                logger.info("创建云主机异常", e);
                break;
            }
        }
        return new BocloudResult(false, "创建超时");
    }

    @Override
    public BocloudResult start(String instanceId) {
        BocloudResult result = this.getClient().compute(getVersion()).servers(getVersion()).action(instanceId, ServerAction.START);
        if (result.isFailed()) {
            String msg = "启动云主机失败，" + result.getMessage();
            logger.error(msg);
            return new BocloudResult(false, msg);
        }
        // 返回执行结果
        List<String> expectedStatuses = Lists.newArrayList();
        expectedStatuses.add(BocloudStatus.Server.RUNNING.name());
        result = checkServerStatus(instanceId, TaskStateType.POWERON, expectedStatuses);
        return result;
    }

    private BocloudResult checkServerStatus(String instanceId, TaskStateType taskType, List<String> expectedStatuses) {
        long start = System.currentTimeMillis();
        while (System.currentTimeMillis() - start < 1000 * 60 * 10) {
            ServerModel server = this.getClient().compute(getVersion()).servers(getVersion()).get(instanceId);
            if (server == null) {
                logger.info("云主机不存在：" + instanceId);
                return new BocloudResult(false, "云主机不存在");
            }
            if (BocloudStatus.Server.ERROR.name().equals(server.getStatus())) {
                JSONObject props = JSON.parseObject(server.getProps());
                String errorMessage = props != null && props.get("errorMessage") != null && StringUtils.isNotEmpty((String) props.get("errorMessage")) ?
                        (String) props.get("errorMessage") : "Powervc中云主机状态为" + server.getStatus();
                return new BocloudResult(false, errorMessage);
            }
            // 检查任务还再不再执行
            String currTaskState = server.getTaskState();
            if (null != currTaskState && currTaskState.contains(taskType.getTaskState())) {
                logger.info("5秒后会重新尝试获取该云主机信息：" + instanceId);
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    logger.info("获取云主机信息异常", e);
                    break;
                }
                continue;
            }
            // 执行结束，检查是否是期望的状态
            if (expectedStatuses.contains(server.getStatus())) {
                return new BocloudResult(true, server, "");
            } else {
                logger.error("期望的状态是" + expectedStatuses + "实际的状态是" + server.getStatus());
                return new BocloudResult(false, server, "不是期望的云主机状态" + server.getStatus());
            }
        }
        logger.info("检查云主机状态超时：" + instanceId);
        return new BocloudResult(false, "检查云主机状态超时");
    }

    @Override
    public BocloudResult stop(String instanceId) {
        BocloudResult result = this.getClient().compute(getVersion()).servers(getVersion()).action(instanceId, ServerAction.STOP);
        if (result.isFailed()) {
            String msg = "关闭云主机失败，" + result.getMessage();
            logger.error(msg);
            return new BocloudResult(false, msg);
        }
        // 返回执行结果
        List<String> expectedStatuses = Lists.newArrayList();
        expectedStatuses.add(BocloudStatus.Server.STOPPED.name());
        result = checkServerStatus(instanceId, TaskStateType.POWEROFF, expectedStatuses);
        return result;
    }

    @Override
    public BocloudResult reboot(String instanceId) {
        BocloudResult result = this.getClient().compute(getVersion()).servers(getVersion()).reboot(instanceId, RebootType.HARD);
        if (result.isFailed()) {
            String msg = "硬重启云主机失败，" + result.getMessage();
            logger.error(msg);
            return new BocloudResult(false, msg);
        }
        // 返回执行结果
        List<String> expectedStatuses = Lists.newArrayList();
        expectedStatuses.add(BocloudStatus.Server.RUNNING.name());
        result = checkServerStatus(instanceId, TaskStateType.REBOOT, expectedStatuses);
        return result;
    }

    @Override
    public BocloudResult softReboot(String instanceId) {
        BocloudResult result = this.getClient().compute(getVersion()).servers(getVersion()).reboot(instanceId, RebootType.SOFT);
        if (result.isFailed()) {
            String msg = "软重启云主机失败，" + result.getMessage();
            logger.error(msg);
            return new BocloudResult(false, msg);
        }
        // 返回执行结果
        List<String> expectedStatuses = Lists.newArrayList();
        expectedStatuses.add(BocloudStatus.Server.RUNNING.name());
        result = checkServerStatus(instanceId, TaskStateType.REBOOT, expectedStatuses);
        return result;
    }

    @Override
    public BocloudResult migrate(ServerModel server) {
        return new BocloudResult(false, "PowerVC不支持该迁移接口，只支持动态迁移接口");
    }

    @Override
    public BocloudResult migrate(OSMigrateModel model) {
        Boolean live = model.getLive();
        if (live == null) {
            live = false;
        }
        if (!live) {
            return new BocloudResult(false, "PowerVC不支持该迁移接口，只支持动态迁移接口");
        }

        JSONObject json = new JSONObject();
        if (model.getHost() == null) {
            model.setHost("");
        }
        json.put("host", model.getHost());
        json.put("block_migration", false);
        json.put("disk_over_commit", false);

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put(ServerAction.LIVE_MIGRATE.getUrl(), json);
        BocloudResult result = this.getClient().compute(getVersion()).servers(getVersion()).liveMigrate(model.getInstanceId(), params);
        if (result.isSuccess()) {
            result.setData(detail(model.getInstanceId()).getData());
        }
        return result;
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        return new BocloudResult(false, "PowerVC不支持快照接口");
    }

    @Override
    public BocloudResult pause(String instanceId) {
        return new BocloudResult(false, "PowerVC不支持休眠接口");
    }

    @Override
    public BocloudResult unPause(String instanceId) {
        return new BocloudResult(false, "PowerVC不支持取消休眠接口");
    }

    @Override
    public BocloudResult suspend(String instanceId) {
        return new BocloudResult(false, "PowerVC不支持睡眠接口");
    }

    @Override
    public BocloudResult resume(String instanceId) {
        return new BocloudResult(false, "PowerVC不支持睡眠恢复接口");
    }

    @Override
    public BocloudResult vncConsole(String vncType, String serverId) {
        return new BocloudResult(false, "PowerVC不支持VNC控制台接口");
    }

    @Override
    public BocloudResult joinSGroup(String instanceId, String groupId) {
        return new BocloudResult(false, "PowerVC不支持安全组接口");
    }

    @Override
    public BocloudResult leaveSGroup(String instanceId, String groupId) {
        return new BocloudResult(false, "PowerVC不支持安全组接口");
    }

    @Override
    public BocloudResult changePassword(String instanceId, String password) {
        return new BocloudResult(false, "PowerVC不支持修改密码接口");
    }

    @Override
    public BocloudResult attachNetwork(ServerModel server) {
        // 定义一个list 用来收集创建成功的port
        List<PortModel> portModels = new ArrayList<>();
        // 创建port并附加到云主机
        for (NetworkBean networkBean : server.getNetworkBeans()) {
            PortModel portModel = new PortModel();
            portModel.setNetworkUuid(networkBean.getNetworkUuid());
            portModel.setPortFixedIps(new ArrayList<>());
            for (AddressBean addressBean : networkBean.getAddresses()) {
                PortFixedIpModel portFixedIpModel = new PortFixedIpModel();
                if (addressBean.getAddress() != null && addressBean.getSubnetUuid() != null){
                    portFixedIpModel.setIpAddress(addressBean.getAddress());
                    portFixedIpModel.setSubnetUuid(addressBean.getSubnetUuid());
                    portModel.getPortFixedIps().add(portFixedIpModel);
                }
            }
            // 创建port
            BocloudResult bocloudResult = this.getClient().networking(getVersion()).port(getVersion()).create(portModel);
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            // 附加到云主机
            String portId = ((PortModel) bocloudResult.getData()).getPortUuid();
            bocloudResult = this.getClient().compute(getVersion()).servers(getVersion()).attachNetwork(server.getInstanceId(), portId);
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            // 重新获取port并加入list
            portModel = this.getClient().networking(getVersion()).port(getVersion()).get(portId);
            portModels.add(portModel);
        }

        //附加动态网络获取port的PortFixedIps为0，所以直接返回成功，不做fixIp的判断
        if (portModels.get(0).getPortFixedIps().size() == 0){
            server = this.getClient().compute(getVersion()).servers(getVersion()).get(server.getInstanceId());
            server.setPorts(portModels);
            return new BocloudResult(true,server,"附加网络成功");
        }
        // 获取所有创建成功的port的fixIp
        List<PortFixedIpModel> portFixedIpModels = new ArrayList<>();
        for (PortModel portModel : portModels) {
            portFixedIpModels.addAll(portModel.getPortFixedIps());
        }
        // 下面要循环所有创建成功的port的fixIp是否已经附加到云主机，该变量用来存储检测成功的ip
        List<PortFixedIpModel> successIpModels = new ArrayList<>();
        // 开始检测
        long start = System.currentTimeMillis();
        while (System.currentTimeMillis() - start < 60 * 1000) {
            server = this.getClient().compute(getVersion()).servers(getVersion()).get(server.getInstanceId());
            if (StringUtils.isNotEmpty(server.getPrivateIps())) {
                List<VmNetworkBean> networkBeans = JSON.parseArray(server.getPrivateIps(), VmNetworkBean.class);
                for (VmNetworkBean networkBean : networkBeans) {
                    for (PortFixedIpModel portFixedIpModel : portFixedIpModels) {
                        if (networkBean.getAddress().equals(portFixedIpModel.getIpAddress()) && !successIpModels.contains(portFixedIpModel)) {
                            successIpModels.add(portFixedIpModel);
                            break;
                        }
                    }
                }
            }
            if (portFixedIpModels.size() == successIpModels.size()) {
                server.setPorts(portModels);
                return new BocloudResult(true, server, "附加网络成功");
            }
            logger.info("正在附加网络，3秒后将重新检测...");
            try {
                Thread.sleep(3000);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                logger.info("云主机附加网络异常", e);
                break;
            }
        }
        return new BocloudResult(false, "附加网络超时");
    }

    @Override
    public BocloudResult detachNetwork(String instanceId, String portUuid) {
        // 先查询获取当前port的所有ip
        PortModel model = this.getClient().networking(getVersion()).port(getVersion()).get(portUuid);
        List<PortFixedIpModel> portFixedIpModels = model.getPortFixedIps();
        // 定义一个变量，用来保存检测已经从云主机移除的ip
        List<PortFixedIpModel> successIpModels = new ArrayList<>();
        // 拆离网络
        BocloudResult bocloudResult = this.getClient().compute(getVersion()).servers(getVersion()).detachNetworik(instanceId, portUuid);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        // 循环检测
        long start = System.currentTimeMillis();
        while (System.currentTimeMillis() - start <= 60 * 1000) {
            PortModel portModel = this.getClient().networking(getVersion()).port(getVersion()).get(portUuid);
            if (portModel == null) {
                ServerModel serverModel = this.getClient().compute(getVersion()).servers(getVersion()).get(instanceId);
                if (StringUtils.isEmpty(serverModel.getPrivateIps())) {
                    return new BocloudResult(true, serverModel, "移除网络成功");
                } else {
                    List<VmNetworkBean> networkBeans = JSON.parseArray(serverModel.getPrivateIps(), VmNetworkBean.class);
                    for (PortFixedIpModel portFixedIpModel : portFixedIpModels) {
                        if (!successIpModels.contains(portFixedIpModel)) {
                            boolean finded = false;
                            for (VmNetworkBean networkBean : networkBeans) {
                                if (networkBean.getAddress().equals(portFixedIpModel.getIpAddress())) {
                                    finded = true;
                                    break;
                                }
                            }
                            if (!finded) {
                                successIpModels.add(portFixedIpModel);
                            }
                        }
                    }
                    if (portFixedIpModels.size() == successIpModels.size()) {
                        return new BocloudResult(true, serverModel, "移除网络成功");
                    }
                }
            }
            logger.info("正在移除网络，2秒后将重新检测");
            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                logger.info("云主机移除网络异常", e);
                break;
            }
        }
        return new BocloudResult(false, "移除网络失败");
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorUuid) {
        return new BocloudResult(false, "PowerVC不支持通过flavor Uuid调整大小");
    }

    @Override
    public BocloudResult resize(ServerModel server, FlavorModel flavor) {
        // flavor
        Map<String, Object> flavorInfo = new HashMap<>(MapUtils.capacity(4));
        flavorInfo.put("vcpus", flavor.getCpu());
        // 2019-07-17 : 需要统一将内存代为转换成GB
        flavorInfo.put("ram", flavor.getMemory().multiply(new BigDecimal(1024)));
        // 必须提供磁盘信息，否则失败
        flavorInfo.put("disk", server.getDisk());

        if (null != flavor.getExtraSpecs()) {
            // extra_specs
            Map<String, Object> extraSpecs = new HashMap<>(MapUtils.capacity(1));
            extraSpecs.put("powervm:proc_units", flavor.getExtraSpecs().getProcUnits());

            flavorInfo.put("extra_specs", extraSpecs);
        }

        Map<String, Object> flavorParams = new HashMap<>(MapUtils.capacity(1));
        flavorParams.put("flavor", flavorInfo);

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("resize", flavorParams);

        BocloudResult bocloudResult = this.getClient().compute(getVersion()).servers(getVersion()).resize(server.getInstanceId(), params);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }

        long start = System.currentTimeMillis();
        ServerModel serverModel = null;
        while (System.currentTimeMillis() - start < 3 * 60 * 1000) {
            serverModel = this.getClient().compute(getVersion()).servers(getVersion()).get(server.getInstanceId());
            if (serverModel.getStatus().equals(BocloudStatus.Server.RESIZING.name())) {
                try {
                    Thread.sleep(3000);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    logger.info("云主机调整大小异常", e);
                    break;
                }
                logger.info("正在执行调整大小，3秒后重新查询...");
                continue;
            }
            if (serverModel.getStatus().equals(BocloudStatus.Server.ERROR.name())) {
                JSONObject props = JSON.parseObject(serverModel.getProps());
                String errorMessage = props != null && props.get("errorMessage") != null && StringUtils.isNotEmpty((String) props.get("errorMessage")) ?
                        (String) props.get("errorMessage") : "Powervc中云主机状态为" + serverModel.getStatus();
                return new BocloudResult(false, serverModel, errorMessage);
            } else {
                return new BocloudResult(true, serverModel, "执行成功");
            }
        }
        return new BocloudResult(false, serverModel, "执行超时");
    }

    @Override
    public BocloudResult confirmResize(String instanceId) {
        return this.getClient().compute(getVersion()).servers(getVersion()).action(instanceId, ServerAction.CONFIRM_RESIZE);
    }

    @Override
    public BocloudResult revertResize(String instanceId) {
        return this.getClient().compute(getVersion()).servers(getVersion()).action(instanceId, ServerAction.REVERT_RESIZE);
    }

    @Override
    public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean memHotEnable) {
        return new BocloudResult(false, "PowerVC不支持接口");
    }

    @Override
    public BocloudResult getTicket(String instanceId) {
        return new BocloudResult(false, "PowerVC不支持接口");
    }

    @Override
    public BocloudResult createInResPool(ServerModel server) {
        return new BocloudResult(false, "PowerVC不支持接口");
    }

    @Override
    public BocloudResult mountToolsInstaller(String instanceId) {
        return new BocloudResult(false, "PowerVC不支持接口");
    }

    @Override
    public BocloudResult unmountToolsInstaller(String instanceId) {
        return new BocloudResult(false, "PowerVC不支持接口");
    }

    @Override
    public BocloudResult upgradeToolsTask(String instanceId, String options) {
        return new BocloudResult(false, "PowerVC不支持接口");
    }

    @Override
    public BocloudResult getNetCard(String instanceId) {
        return new BocloudResult(false, "PowerVC不支持接口");
    }

    @Override
    public BocloudResult getVmIps(String instanceId) {
        return new BocloudResult(false, "PowerVC不支持接口");
    }

    @Override
    public BocloudResult markAsTemplate(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
        return null;
    }

    @Override
    public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
        return null;
    }

    @Override
    public BocloudResult getMetric(String type) {
        return null;
    }

    @Override
    public BocloudResult vmrcConsole(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult migrate(H3cMigrateModel model) {
        return null;
    }

    @Override
    public BocloudResult create(H3cServerModel server) {
        return null;
    }

    @Override
    public BocloudResult getJnlpXml(Long vmId, String address) {
        return null;
    }

    @Override
    public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
        return null;
    }

    @Override
    public BocloudResult resetInstance(ServerModel serverModel) {
        return null;
    }

    @Override
    public BocloudResult bindKeypair(ServerModel serverModel) {
        return null;
    }

    @Override
    public BocloudResult unbindKeypair(ServerModel serverModel) {
        return null;
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
        return null;
    }

    @Override
    public BocloudResult attachVolume(String instanceId, VolumeModel volumeModel) {
        BocloudResult bocloudResult = this.getClient().compute(getVersion()).servers(getVersion()).attachVolume(instanceId, volumeModel);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        long start = System.currentTimeMillis();
        while (System.currentTimeMillis() - start < 60 * 1000) {
            VolumeModel newModel = this.getClient().blockStorage(getVersion()).volumes(getVersion()).get(volumeModel.getVolumeUuid());
            if (!newModel.getStatus().equals("attaching") &&
                    newModel.getModels() != null && newModel.getModels().size() > 0 &&
                    newModel.getModels().removeIf(serverVolumeModel -> serverVolumeModel.getInstanceId().equals(instanceId))) {
                break;
            }
            logger.info("卷[" + volumeModel.getVolumeUuid() + "]正在附加中，3秒后将再次确认");
            try {
                Thread.sleep(3000);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                logger.info("云主机附加卷异常", e);
                break;
            }
        }
        return bocloudResult;
    }

    @Override
    public BocloudResult retachVolume(String instanceId, VolumeModel volumeModel) {
        BocloudResult bocloudResult = this.getClient().compute(getVersion()).servers(getVersion()).retachVolume(instanceId, volumeModel);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        return bocloudResult;
    }

    @Override
    public BocloudResult detachVolume(String instanceId, VolumeModel volumeModel) {
        BocloudResult bocloudResult = this.getClient().compute(getVersion()).servers(getVersion()).detachVolume(instanceId, volumeModel);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        long start = System.currentTimeMillis();
        while (System.currentTimeMillis() - start < 60 * 1000) {
            VolumeModel newModel = this.getClient().blockStorage(getVersion()).volumes(getVersion()).get(volumeModel.getVolumeUuid());
            if (!newModel.getStatus().equals("detaching") &&
                    (newModel.getModels() == null || newModel.getModels().size() == 0 ||
                            !newModel.getModels().removeIf(serverVolumeModel -> serverVolumeModel.getInstanceId().equals(instanceId)))
            ) {
                break;
            }
            logger.info("卷[" + volumeModel.getVolumeUuid() + "]正在拆离中，2秒后将再次确认");
            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                logger.info("云主机拆离卷异常", e);
                break;

            }
        }
        return bocloudResult;
    }

    @Override
    public BocloudResult beforeCaptureImage(String instanceId, ImageModel imageModel){
        List<String> bootVolumeIds = new ArrayList<>();
        List<String> volumeUuids = new ArrayList<>();
        for (VolumeModel volume : imageModel.getDiskDeviceMappings()) {
            volumeUuids.add(volume.getVolumeUuid());
            if (volume.getBootVolume()){
                bootVolumeIds.add(volume.getVolumeUuid());
            }
        }
        Map<String, Object> finishOnboard = new HashMap<>(MapUtils.capacity(2));
        finishOnboard.put("attach_order", volumeUuids);
        finishOnboard.put("boot_volume_ids", bootVolumeIds);
        //finishOnboard.put("endianness","big-endian");
        finishOnboard.put("os_distro", imageModel.getOsName());
        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("finishOnboard", finishOnboard);
        BocloudResult result = this.getClient().compute(getVersion()).servers(getVersion()).beforeCaptureImage(instanceId, params);
        return result;
    }

    @Override
    public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
        Map<String, Object> imageInfo = new HashMap<>(MapUtils.capacity(2));
        imageInfo.put("name", imageName);

        int volumeSize = 0;
        if (null != volumes) {
            volumeSize = volumes.size();
        }
        Map<String, String> volumeInfo = new HashMap<>(MapUtils.capacity(volumeSize));
        if (null != volumes && volumes.size() > 0) {
            for (int i = 0; i < volumes.size(); i++) {
                String diskName = "powervc_capture_disk_" + i;
                volumeInfo.put(diskName, volumes.get(i));
            }
        }
        imageInfo.put("metadata", volumeInfo);

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("createImage", imageInfo);

        BocloudResult result = this.getClient().compute(getVersion()).servers(getVersion()).createImage(instanceId, params);
        return result;
    }

    @Override
    public BocloudResult resetStatus(String instanceId) {
        BocloudResult bocloudResult = this.getClient().compute(getVersion()).servers(getVersion()).resetStatus(instanceId);
        if(bocloudResult.isFailed()){
            return bocloudResult;
        }
        long startTime = System.currentTimeMillis();
        while (System.currentTimeMillis() - startTime < 10 * 1000) {
            ServerModel server = this.getClient().compute(getVersion()).servers(getVersion()).get(instanceId);
            if (server == null) {
                logger.info("云主机不存在：" + instanceId);
                return new BocloudResult(false, "云主机不存在");
            }
            if (!BocloudStatus.Server.ERROR.name().equals(server.getStatus()) &&
                    !BocloudStatus.Server.EXCEPTION.name().equals(server.getStatus())) {
                return new BocloudResult(true, server, "重置状态成功");
            }
            logger.info("云主机状态重置中，2秒后会重新尝试获取该云主机信息：" + instanceId);
            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                logger.info("云主机重置状态异常", e);
                break;
            }
        }
        return new BocloudResult(false, "重置状态超时");

    }
}
