package com.bocloud.cmp.provider.powervc4j.common.util;

import com.bocloud.cmp.provider.powervc4j.exception.DriverException;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * 日期工具类
 * 
 * @author sxp
 * @since Mar 19, 2018
 */
public class DateUtils {
    /**
     * 将字符串转成时间
     * 
     * @param dateString
     *            时间的字符串，例如：2016-07-22
     * @param format
     *            字符串匹配模式，例如：yyyy-MM-dd
     * @return 返回dateString对应的Date对象
     * @author sxp
     * @since 2018-03-19
     */
    public static Date stringToDate(String dateString, String format) {
        if (dateString == null || "".equals(dateString)) {
            throw new DriverException("传入参数中的[时间串]不能为空！");
        }
        if (format == null || "".equals(format)) {
            throw new DriverException("传入参数中的[时间格式]不能为空！");
        }

        SimpleDateFormat df = new SimpleDateFormat(format);
        df.setLenient(false);// 将2017-13-01看做是一个非法的日期字符串

        Date date;

        try {
            date = df.parse(dateString);
        } catch (ParseException e) {
            throw new DriverException(e.getMessage());
        }

        return date;
    }

    /**
     * 将时间转成字符串
     * 
     * @param date
     *            时间
     * @param format
     *            字符串匹配模式，例如：yyyy-MM-dd
     * @author sxp
     * @since 2018-03-19
     */
    public static String dateToString(Date date, String format) {
        if (date == null) {
            return null;
        }
        if (format == null || "".equals(format)) {
            throw new DriverException("传入参数中的[时间格式]不能为空！");
        }

        SimpleDateFormat df = new SimpleDateFormat(format);
        df.setLenient(false);// 将2017-13-01看做是一个非法的日期字符串

        String dateString = df.format(date);

        return dateString;
    }

    /**
     * 将UTC格式的时间字符串转换成日期
     * 
     * <pre>
     * 将2016-10-20T16:00:00.041117Z这样的时间字符串转换成日期。
     * 
     * UTC :协调世界时，又称世界统一时间，世界标准时间，国际协调时间。
     * 中国时区为东八区，所以中国时间与UTC的时差均为+8，也就是UTC+8。
     * 
     * 2016-10-20T16:00:00.000000000Z对应的中国时间为2016-10-21 00:00:00。
     * </pre>
     * 
     * @author sxp
     * @since 2018-03-19
     */
    public static Date getDateFromUTCString(String dateStringOfUTC) {
        String prefix = "调用getDateFromUTCString";
        Date date = null;
        boolean hasNano;

        String dStr = dateStringOfUTC;

        if (dStr == null || dStr.length() == 0) {
            throw new DriverException(prefix + "传入的时间字符串不能为空！");
        }

        // 将纳秒值转换为毫秒值
        int pos1 = dStr.lastIndexOf(".");
        int pos2 = dStr.lastIndexOf("Z");
        if (pos2 < 0) {
            throw new DriverException(prefix + "传入的时间字符串无效，不含有字符Z！");
        }

        int nano;// 纳秒
        String nanoS;
        if (pos1 >= 0) {
            hasNano = true;
            nanoS = dStr.substring(pos1 + 1, pos2);
            // 1985-04-12T23:20:50.52Z 表示1985年4月12日23时20分50.52秒
            nanoS = nanoS + "000000000";
            nanoS = nanoS.substring(0, 9);// 保证长度为9位

            nano = Integer.parseInt(nanoS);

            int milli = nano / (1000 * 1000);// 毫秒
            if (milli > 999) {
                milli = 999;
            }

            dStr = dStr.substring(0, pos1 + 1) + String.valueOf(milli) + "Z";
        } else {
            hasNano = false;
        }

        // 增加UTC后缀，否则无法转换
        dStr = dStr.replace("Z", " UTC");// 空格+UTC

        SimpleDateFormat df;

        if (hasNano) {
            df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS Z");
        } else {
            df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss Z");
        }
        df.setLenient(false);// 将2017-13-01看做是一个非法的日期字符串

        try {
            date = df.parse(dStr);
        } catch (ParseException e) {
            throw new DriverException(prefix + "传入的时间字符串不是有效的时间！");
        }

        return date;
    }

    /**
     * 根据本地时间获取UTC时间的字符串
     * 
     * @author sxp
     * @since 2018-03-19
     */
    public static String getUTCStringFromLocalDate(Date localDate) {
        // 本地时间
        Calendar cal = Calendar.getInstance();
        cal.setTime(localDate);

        // 时间偏移量
        int zoneOffset = cal.get(Calendar.ZONE_OFFSET);

        // 夏令时差
        int dstOffset = cal.get(Calendar.DST_OFFSET);

        // 从本地时间里扣除这些差量，即可以取得UTC时间
        cal.add(Calendar.MILLISECOND, -(zoneOffset + dstOffset));

        // 毫秒转换成纳秒
        int millisecond = cal.get(Calendar.MILLISECOND);
        int nano = millisecond * 1000 * 1000;
        if (nano > 999999999) {
            nano = 999999999;
        }
        String nanoS = String.valueOf(nano);
        nanoS = nanoS + "000000000";
        nanoS = nanoS.substring(0, 9);// 保证长度为9位

        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
        df.setLenient(false);// 将2017-13-01看做是一个非法的日期字符串

        String dateString = df.format(cal.getTime()) + "." + nanoS + "Z";

        return dateString;
    }

}
