package com.bocloud.cmp.provider.powervc4j.impl.base;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.cmp.provider.powervc4j.api.PVClient;
import com.bocloud.cmp.provider.powervc4j.common.enums.RequestMethod;
import com.bocloud.cmp.provider.powervc4j.common.util.MapUtils;
import com.bocloud.cmp.provider.powervc4j.core.Constant;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.model.identity.Token;
import com.bocloud.cmp.provider.powervc4j.utils.ApiPrefixUtils;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.Result;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Base服务实现类
 *
 * @author sxp
 * @since Mar 20, 2018
 */
public abstract class BaseServiceImpl<T> {
    private static Logger logger = LoggerFactory.getLogger(BaseServiceImpl.class);

    /**
     * 当前服务所属的顶层服务类型(例如ComputeService.class)
     */
    private Class<?> serviceType = initServiceType();

    /**
     * 初始化服务类型
     *
     * @author sxp
     * @since Mar 20, 2018
     */
    protected abstract Class<?> initServiceType();

    // ----CRUD操作----

    /**
     * 获取详细信息列表
     *
     * @author sxp
     * @since Mar 20, 2018
     */
    public List<T> doList(String apiSuffix) {
        return doList(apiSuffix, null);
    }

    /**
     * 获取详细信息列表
     *
     * @author sxp
     * @since Mar 20, 2018
     */
    public List<T> doList(String apiSuffix, Map<String, Object> params) {
        PowervcProvider driver = getCurrentDriverSafely();

        Token token = driver.getValidToken();
        Objects.requireNonNull(token, "token");

        Map<String, String> apiPrefix = driver.getApiPrefix();
        String prefix = ApiPrefixUtils.getPrefix(apiPrefix, serviceType);
        if (StringUtils.isEmpty(prefix)) {
            throw new DriverException("无法从Token信息中提取服务API前缀");
        }

        String uri = prefix + apiSuffix;
        String functionDescr = String.format("调用API[%s]获取数据", uri);

        Result result = httpRequest(RequestMethod.GET, driver, token, uri, params);
        String message = result.getMessage();
        if (result.isSuccess()) {
            return processListResult(message);
        } else {
            throw new DriverException(functionDescr + "失败，原因是：{" + message + "} {" + result.getErrorMsg() + "}。");
        }
    }

    /**
     * 转换list请求的结果为指定的类型
     *
     * @author sxp
     * @since Mar 20, 2018
     */
    protected abstract List<T> processListResult(String message);

    /**
     * 获取详细信息
     *
     * @author sxp
     * @since Mar 21, 2018
     */
    public T doGet(String apiSuffix) {
        String functionDescr = String.format("调用API[%s]获取数据", apiSuffix);

        PowervcProvider driver = getCurrentDriverSafely();

        Token token = driver.getValidToken();
        Objects.requireNonNull(token, "token");

        Map<String, String> apiPrefix = driver.getApiPrefix();
        String prefix = ApiPrefixUtils.getPrefix(apiPrefix, serviceType);
        if (StringUtils.isEmpty(prefix)) {
            throw new DriverException("无法从Token信息中提取服务API前缀");
        }

        String uri = prefix + apiSuffix;

        try {
            Result result = httpRequest(RequestMethod.GET, driver, token, uri, null);
            String message = result.getMessage();
            if (result.isSuccess()) {
                return processGetResult(message);
            } else {
                logger.warn(functionDescr + "失败，原因是：{} {}。", message, result.getErrorMsg());
                return null;
            }
        } catch (RuntimeException e) {
            logger.error(functionDescr + "失败，" + e.getMessage(), e);
            return null;
        }
    }

    /**
     * 转换请求的结果为指定的类型
     *
     * @author sxp
     * @since Mar 21, 2018
     */
    protected abstract T processGetResult(String message);

    /**
     * 自定义Get方法
     *
     * @author sxp
     * @since Mar 28, 2018
     */
    public <R> R customGet(Class<R> returnType, String apiSuffix) {
        String functionDescr = String.format("调用API[%s]获取数据", apiSuffix);

        PowervcProvider driver = getCurrentDriverSafely();

        Token token = driver.getValidToken();
        Objects.requireNonNull(token, "token");

        Map<String, String> apiPrefix = driver.getApiPrefix();
        String prefix = ApiPrefixUtils.getPrefix(apiPrefix, serviceType);
        if (StringUtils.isEmpty(prefix)) {
            throw new DriverException("无法从Token信息中提取服务API前缀");
        }

        String uri = prefix + apiSuffix;

        try {
            Result result = httpRequest(RequestMethod.GET, driver, token, uri, null);
            String message = result.getMessage();
            if (result.isSuccess()) {
                return JSON.parseObject(message, returnType);
            } else {
                logger.error(functionDescr + "失败，原因是：{} {}。", message, result.getErrorMsg());
                return null;
            }
        } catch (RuntimeException e) {
            logger.error(functionDescr + "失败，" + e.getMessage(), e);
            return null;
        }
    }

    /**
     * 创建
     *
     * @author sxp
     * @since Mar 28, 2018
     */
    public Result doCreate(String apiSuffix, Map<String, Object> params) {
        return work(RequestMethod.POST, apiSuffix, params);
    }

    /**
     * 创建
     *
     * @author sxp
     * @since Mar 28, 2018
     */
    public Result doCreate(String apiSuffix, Map<String, Object> params, int timeout) {
        return work(RequestMethod.POST, apiSuffix, params, timeout);
    }

    /**
     * 修改
     *
     * @author sxp
     * @since Mar 29, 2018
     */
    public Result doUpdate(String apiSuffix, Map<String, Object> params) {
        return work(RequestMethod.PUT, apiSuffix, params);
    }

    /**
     * 修改
     *
     * @author sxp
     * @since Mar 29, 2018
     */
    public Result doUpdate(String apiSuffix, Map<String, Object> params, Map<String, Object> header) {
        return work(RequestMethod.PUT, apiSuffix, params, header, 60 * 1000);
    }

    /**
     * 删除
     *
     * @author sxp
     * @since Mar 28, 2018
     */
    public Result doDelete(String apiSuffix, Map<String, Object> params) {
        return work(RequestMethod.DELETE, apiSuffix, params);
    }

    public Result work(RequestMethod method, String apiSuffix, Map<String, Object> params) {
        return work(method, apiSuffix, params, 60 * 1000);
    }

    public Result work(RequestMethod method, String apiSuffix, Map<String, Object> params, int timeout) {
        return work(method, apiSuffix, params, null, timeout);
    }


    /**
     * 获取token和api后调用API
     *
     * @author sxp
     * @since Mar 30, 2018
     */
    public Result work(RequestMethod method, String apiSuffix, Map<String, Object> params, Map<String, Object> header, int timeout) {
        String functionDescr = String.format("调用API[%s]获取数据", apiSuffix);

        try {
            PowervcProvider driver = getCurrentDriverSafely();

            Token token = driver.getValidToken();
            Objects.requireNonNull(token, "token");

            Map<String, String> apiPrefix = driver.getApiPrefix();
            String prefix = ApiPrefixUtils.getPrefix(apiPrefix, serviceType);
            if (StringUtils.isEmpty(prefix)) {
                throw new DriverException("无法从Token信息中提取服务API前缀");
            }

            String uri = prefix + apiSuffix;
            // TODO
            //System.out.println("uri = " + uri);

            return httpRequest(method, driver, token, uri, params, header, timeout);
        } catch (RuntimeException e) {
            String message = functionDescr + "失败，" + e.getMessage();
            logger.error(message, e);
            return new Result(false, message);
        }
    }

    protected Result httpRequest(RequestMethod method, PowervcProvider driver, Token token, String uri,
                                 Map<String, Object> params) {
        return httpRequest(method, driver, token, uri, params, 60 * 1000);
    }

    protected Result httpRequest(RequestMethod method, PowervcProvider driver, Token token, String uri,
                                 Map<String, Object> params, int timeout) {
        return httpRequest(method, driver, token, uri, params, null, timeout);
    }

    protected Result httpRequest(RequestMethod method, PowervcProvider driver, Token token, String uri,
                                 Map<String, Object> params, Map<String, Object> header, int timeout) {
        HttpClient client = new HttpClient(timeout);
        Result result;

        if (header == null) {
            header = new HashMap<>(MapUtils.capacity(1));
        }
        header.put(Constant.HEADER_X_AUTH_TOKEN, token.getTokenId());
        header.put(Constant.HEADER_LANGUAGE, Constant.DEFAULT_LANGUAGE);

        result = invoke(method, params, uri, client, header);
        logger.debug("==================================================");
        logger.debug("== uri: " + uri);
        logger.debug("== method: " + method.name());
        logger.debug("== params: " + params);
        logger.debug("== result: " + result);
        logger.debug("==================================================");

        if (result.isFailed()) {
            if (Constant.HTTP_CLIENT_401_ERROR.equals(result.getErrorMsg())) {
                String message = result.getMessage();
                logger.warn("调用PowerVC的API是出现{}错误，系统将自动刷新token后重试", message);

                driver.refreshToken();
                token = driver.getToken();

                header.clear();
                header.put(Constant.HEADER_X_AUTH_TOKEN, token.getTokenId());

                result = invoke(method, params, uri, client, header);
            }
        }

        return result;
    }

    private Result invoke(RequestMethod method, Map<String, Object> params, String uri, HttpClient client,
                          Map<String, Object> header) {
        Result result;
        switch (method) {
            case GET:
                result = invoke(RequestMethod.GET, params, uri, client, header, null);
                break;
            case POST:
                result = invoke(RequestMethod.POST, params, uri, client, header, PostDataFormat.RAW);
                break;
            case PUT:
                result = invoke(RequestMethod.PUT, params, uri, client, header, PostDataFormat.RAW);
                break;
            case DELETE:
                result = invoke(RequestMethod.DELETE, params, uri, client, header, null);
                break;
            default:
                result = new Result(false, "不支持" + method.name() + "请求方法");
                break;
        }
        return result;
    }

    private String unicodeToString(String string) {
        if (string == null) {
            return null;
        }
        if (string.equals("")) {
            return "";
        }
        StringBuffer stringBuffer = new StringBuffer() ;
        String pattern = "\\\\u[0-9,a-f,A-F]{4}";
        Matcher matcher = Pattern.compile(pattern).matcher(string);
        while (matcher.find()) {
            char c = (char) Integer.parseInt(matcher.group().replaceAll("\\\\u", ""), 16);
            matcher.appendReplacement(stringBuffer, new String(new char[]{c}));
        }
        matcher.appendTail(stringBuffer);
        return stringBuffer.toString();
    }

    private Result invoke(RequestMethod method, Map<String, Object> params, String uri, HttpClient client,
                          Map<String, Object> header, PostDataFormat dataFormat) {
        client.setDataFormat(dataFormat);
        Result result;
        switch (method) {
            case GET:
                result = client.get(header, params, uri);
                break;
            case POST:
                result = client.post(header, params, uri);
                break;
            case PUT:
                result = client.put(header, params, uri);
                break;
            case DELETE:
                result = client.delete(header, params, uri);
                break;
            default:
                result = new Result(false, "不支持" + method.name() + "请求方法");
                break;
        }
        result.setMessage(unicodeToString(result.getMessage()));
        result.setErrorMsg(unicodeToString(result.getErrorMsg()));
        return result;
    }

    // ----其他工具方法-------

    /**
     * 安全地获取当前的PowervcDriver
     *
     * @author sxp
     * @since Mar 22, 2018
     */
    public PowervcProvider getCurrentDriverSafely() {
        PowervcProvider driver = PVClient.getCurrent();
        if (driver == null) {
            throw new DriverException("无法获取当前当前线程中的PowervcDriver对象");
        }
        return driver;
    }

    /**
     * 安全地获取当前租户的唯一标识
     *
     * @author sxp
     * @since Mar 22, 2018
     */
    public String getCurrentTenantIdSafely() {
        PowervcProvider driver = getCurrentDriverSafely();

        Token token = driver.getToken();
        Objects.requireNonNull(token, "token");

        String tenantId = token.getTenantId();
        if (StringUtils.isEmpty(tenantId)) {
            throw new NullPointerException("tenantId");
        }

        return tenantId;
    }

}
