package com.bocloud.cmp.provider.powervc4j.impl.compute.v14;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.PowervcOsAggregateInitialPolicy;
import com.bocloud.cmp.model.ClusterModel;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.compute.OsAggregateService;
import com.bocloud.cmp.provider.powervc4j.common.enums.RequestMethod;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 主机组
 *
 * @author luyanchao
 * @since 2019/1/30 15:06
 */
public class OsAggregateServiceImpl extends BaseServiceImpl<ClusterModel> implements OsAggregateService {

    private static Logger logger = LoggerFactory.getLogger(OsAggregateServiceImpl.class);

    @Override
    public BocloudResult get(String uuid) {
        if (StringUtils.isEmpty(uuid)) {
            throw new DriverException("查询主机组信息时，唯一标识不能为空");
        }
        String apiSuffix = String.format("/os-aggregates/%s", uuid);

        ClusterModel model = doGet(apiSuffix);

        return new BocloudResult(true, model, "success");
    }

    @Override
    public BocloudResult create(String name) {
        String apiSuffix = "/os-aggregates";

        Map<String, Object> map = new HashMap<>();
        map.put("name", name);
        Map<String, Object> params = new HashMap<>();
        params.put("aggregate", map);

        Result result = doCreate(apiSuffix, params);
        if (result.isFailed()) {
            logger.error("创建主机组[" + name + "]失败：" + result.getMessage());
            return new BocloudResult(false, result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "执行成功");
    }

    @Override
    public BocloudResult setMetadata(String uuid, String initialPolicy) {
        if (StringUtils.isEmpty(uuid)) {
            return new BocloudResult(false, "修改主机组metadata时其uuid不能为空值");
        }
        String apiSuffix = String.format("/os-aggregates/%s/action", uuid);

        Integer initialPolicyId = PowervcOsAggregateInitialPolicy.getId(initialPolicy);
        if (initialPolicyId == null) {
            return new BocloudResult(false, "主机组布置策略不合法");
        }
        Map<String, Object> metadata = new HashMap<>();
        metadata.put("initialpolicy-id", initialPolicyId.toString());
        Map<String, Object> setMetadata = new HashMap<>();
        setMetadata.put("metadata", metadata);
        Map<String, Object> params = new HashMap<>();
        params.put("set_metadata", setMetadata);

        Result result = work(RequestMethod.POST, apiSuffix, params);
        if (result.isFailed()) {
            logger.error("修改主机组[" + uuid + "]metadata数据失败：" + result.getMessage());
            return new BocloudResult(false, result.getMessage());
        }

        JSONObject cluster = JSON.parseObject(result.getMessage()).getJSONObject("aggregate");
        ClusterModel model = convert(cluster);
        return new BocloudResult(true, model, "执行成功");
    }

    @Override
    public BocloudResult modify(String uuid, String name) {
        String apiSuffix = String.format("/os-aggregates/%s", uuid);

        Map<String, Object> map = new HashMap<>();
        map.put("name", name);
        Map<String, Object> params = new HashMap<>();
        params.put("aggregate", map);

        Result result = doUpdate(apiSuffix, params);
        if (result.isFailed()) {
            logger.error("修改主机组[" + name + "]失败：" + result.getMessage());
            return new BocloudResult(false, result.getMessage());
        }

        JSONObject cluster = JSON.parseObject(result.getMessage()).getJSONObject("aggregate");
        ClusterModel model = convert(cluster);
        return new BocloudResult(true, model, "执行成功");
    }

    @Override
    public BocloudResult remove(String uuid) {
        String apiSuffix = String.format("/os-aggregates/%s", uuid);
        Result result = doDelete(apiSuffix, null);
        if (result.isFailed()) {
            logger.error("删除主机组[" + uuid + "]失败：" + result.getMessage());
            return new BocloudResult(false, result.getMessage());
        }
        return new BocloudResult(true, "执行成功");
    }

    @Override
    public BocloudResult addServer(String uuid, String hostName) {
        String apiSuffix = String.format("/os-aggregates/%s/action", uuid);

        Map<String, Object> map = new HashMap<>();
        map.put("host", hostName);
        Map<String, Object> params = new HashMap<>();
        params.put("add_host", map);

        Result result = work(RequestMethod.POST, apiSuffix, params);
        if (result.isFailed()) {
            logger.error("为主机组[" + uuid + "]中添加主机[ " + hostName + " ]失败：" + result.getMessage());
            return new BocloudResult(false, result.getMessage());
        }

        JSONObject cluster = JSON.parseObject(result.getMessage()).getJSONObject("aggregate");
        ClusterModel model = convert(cluster);
        return new BocloudResult(true, model, "执行成功");
    }

    @Override
    public BocloudResult removeServer(String uuid, String hostName) {
        String apiSuffix = String.format("/os-aggregates/%s/action", uuid);

        Map<String, Object> map = new HashMap<>();
        map.put("host", hostName);
        Map<String, Object> params = new HashMap<>();
        params.put("remove_host", map);

        Result result = work(RequestMethod.POST, apiSuffix, params);
        if (result.isFailed()) {
            logger.error("从主机组[" + uuid + "]中移除主机[ " + hostName + " ]失败：" + result.getMessage());
            return new BocloudResult(false, result.getMessage());
        }

        JSONObject cluster = JSON.parseObject(result.getMessage()).getJSONObject("aggregate");
        ClusterModel model = convert(cluster);
        return new BocloudResult(true, model, "执行成功");
    }

    @Override
    public List<ClusterModel> list() {
        String apiSuffix = "/os-aggregates";
        return doList(apiSuffix);
    }

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(OsAggregateService.class);
    }

    @Override
    protected List<ClusterModel> processListResult(String message) {
        logger.debug("powervc cluster list {}", message);

        try {
            JSONObject json = JSONTools.isJSONObj(message);
            JSONArray clusters = json.getJSONArray("aggregates");

            List<ClusterModel> models = new ArrayList<>(clusters.size());

            JSONObject cluster;
            ClusterModel model;

            for (Object obj : clusters) {
                cluster = (JSONObject) obj;
                model = convert(cluster);

                models.add(model);
            }

            return models;
        } catch (Exception e) {
            String msg = "解析数据出错。" + e.getMessage();
            logger.error(msg);

            throw new DriverException(msg, e);
        }
    }

    @Override
    protected ClusterModel processGetResult(String message) {
        logger.debug("powervc cluster info {}", message);

        try {
            JSONObject json = JSONTools.isJSONObj(message);
            JSONObject cluster = json.getJSONObject("aggregate");

            ClusterModel model = convert(cluster);

            return model;
        } catch (Exception e) {
            String msg = "解析数据出错。" + e.getMessage();
            logger.error(msg);

            throw new DriverException(msg, e);
        }
    }

    private ClusterModel convert(JSONObject cluster) {
        ClusterModel clusterModel = new ClusterModel();
        clusterModel.setUuid(cluster.getString("id"));
        clusterModel.setName(cluster.getString("name"));
        if (cluster.getBoolean("powervc_default") != null && cluster.getBoolean("powervc_default")) {
            clusterModel.setStatus("default");
        }
        clusterModel.setHostNames(cluster.getJSONArray("hosts").toJavaList(String.class));
        if (cluster.get("metadata") != null) {
            int initialpolicy = cluster.getJSONObject("metadata").getIntValue("initialpolicy-id");
            clusterModel.setInitialPolicy(PowervcOsAggregateInitialPolicy.getText(initialpolicy));
        }
        return clusterModel;
    }
}
