package com.bocloud.cmp.provider.powervc4j.impl.identity;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.common.util.DateUtils;
import com.bocloud.cmp.provider.powervc4j.common.util.MapUtils;
import com.bocloud.cmp.provider.powervc4j.core.Constant;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.model.identity.Token;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.ExtendHttpClient;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.ReentrantLock;

/**
 * PowerVC认证器
 * 
 * @author sxp
 * @since Mar 19, 2018
 */
public class Authenticator {
    private static Logger logger = LoggerFactory.getLogger(Authenticator.class);

    /**
     * 获取token的API
     */
    private static final String AUTH_API = Apis.API_PREFIX + "/identity/v3/auth/tokens";

    /**
     * 获取Token时使用的锁
     */
    private static ReentrantLock lock = new ReentrantLock();

    /**
     * PowerVC授权令牌
     */
    private  static ConcurrentHashMap<String, Token> tokens = new ConcurrentHashMap<>(MapUtils.capacity(2));

    /**
     * 获取PowerVC授权令牌信息
     * 
     * @author sxp
     * @since Mar 19, 2018
     */
    public static Token getToken(String endpoint) {
        Token token = tokens.get(endpoint);

        if (validateToken(token)) {
            return token;
        }

        return null;
    }

    public static void removeToken(String endpoint) {
        if (tokens.get(endpoint) != null) {
            tokens.remove(endpoint);
        }
    }


    /**
     * 判断token的有效性
     * <p>
     * 即判断token是否超过有效期
     * </p>
     * 
     * @author sxp
     * @since Mar 20, 2018
     */
    public static boolean validateToken(Token token) {
        if (token == null) {
            return false;
        }

        Date expires = token.getExpires();
        Date now = Calendar.getInstance().getTime();

        if (now.before(expires)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 获取PowerVC授权令牌信息
     * <p>
     * 调用PowerVC的API获取PowerVC授权令牌信息
     * </p>
     * 
     * @author sxp
     * @since Mar 19, 2018
     */
    public static Token getToken(String endpoint, String username, String password, String domain, String project) {
        Token token;
        lock.lock();
        try {
            String uri = endpoint + AUTH_API;

            ExtendHttpClient client = new ExtendHttpClient();
            client.setDataFormat(PostDataFormat.RAW);

            Map<String, Object> header = new HashMap<>(MapUtils.capacity(1));
            header.put(Constant.HEADER_CONTENT_TYPE, Constant.CONTENT_TYPE_JSON);

            Map<String, Object> params = genAuthParams(username, password, domain, project);

            Result result = client.postWithHeaders(header, params, uri);

            if (result.isSuccess()) {
                String message = result.getMessage();
                logger.debug("token: [" + endpoint + "]" + message);
                try {
                    token = processMessage(message);
                    tokens.put(endpoint, token);
                } catch (Exception e) {
                    token = null;
                    logger.error("从返回结果中提取Token信息失败", e);
                }
            } else {
                logger.error("获取Token信息失败:{}", result);
                token = null;
            }
        } catch (RuntimeException e) {
            logger.error("调用PowerVC的API获取PowerVC授权令牌信息失败", e);
            throw e;
        } finally {
            lock.unlock();
        }

        return token;
    }

    /**
     * 生成参数
     * 
     * @author sxp
     * @since Mar 19, 2018
     */
    private static Map<String, Object> genAuthParams(String username, String password, String domain, String project) {
        Map<String, Object> domainMap = new HashMap<>(MapUtils.capacity(1));
        domainMap.put("name", domain);

        Map<String, Object> projectMap = new HashMap<>(MapUtils.capacity(2));
        projectMap.put("name", project);
        projectMap.put("domain", domainMap);

        Map<String, Object> scopeMap = new HashMap<>(MapUtils.capacity(1));
        scopeMap.put("project", projectMap);

        Map<String, Object> userMap = new HashMap<>(MapUtils.capacity(3));
        userMap.put("domain", new HashMap<>(domainMap));
        userMap.put("name", username);
        userMap.put("password", password);

        Map<String, Object> passwordMap = new HashMap<>(MapUtils.capacity(1));
        passwordMap.put("user", userMap);

        List<String> methods = Arrays.asList("password");

        Map<String, Object> identityMap = new HashMap<>(MapUtils.capacity(2));
        identityMap.put("password", passwordMap);
        identityMap.put("methods", methods);

        Map<String, Object> authMap = new HashMap<>(MapUtils.capacity(2));
        authMap.put("scope", scopeMap);
        authMap.put("identity", identityMap);

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("auth", authMap);

        return params;
    }

    /**
     * 从返回结果中提取Token信息
     * 
     * @author sxp
     * @since Mar 19, 2018
     */
    private static Token processMessage(String message) {
        JSONObject json = JSONTools.isJSONObj(message);
        if (json == null) {
            throw new DriverException("调用ExtendHttpClient的返回结果不是json字符串");
        }

        String headersStr = json.getString("headers");
        // JSONObject contentJson = json.getJSONObject("body");// 这个方法会报错
        String content = json.getString("body");
        JSONObject contentJson = JSON.parseObject(content, JSONObject.class);

        // 提取tokenId
        List<JSONObject> headers = JSON.parseArray(headersStr, JSONObject.class);
        String tokenId = null;

        for (JSONObject item : headers) {
            if (Constant.HEADER_X_SUBJECT_TOKEN.equals(item.getString("name"))) {
                tokenId = item.getString("value");
            }
        }

        if (StringUtils.isEmpty(tokenId)) {
            throw new DriverException("未获取到的token唯一标识");
        }

        // 提取token过期时间
        JSONObject tokenJson = contentJson.getJSONObject("token");
        String expiresOfUTC = tokenJson.getString("expires_at");
        Date expires = DateUtils.getDateFromUTCString(expiresOfUTC);

        Date now = Calendar.getInstance().getTime();
        if (expires.before(now)) {
            throw new DriverException("获取到token的过期时间小于云管平台服务器的当前时间，请检查PowerVC、云管平台" + "服务器的时间是否正确");
        }

        // 提取各个服务的apiPrefix
        Map<String, String> apiPrefix = new HashMap<>(MapUtils.capacity(10));
        JSONArray services = tokenJson.getJSONArray("catalog");
        JSONObject service, endpoint;
        JSONArray endpoints;
        String type, url;

        for (Object obj : services) {
            service = (JSONObject) obj;
            endpoints = service.getJSONArray("endpoints");
            for (Object item : endpoints) {
                endpoint = (JSONObject) item;
                if ("public".equals(endpoint.getString("interface"))) {
                    type = service.getString("type");
                    url = endpoint.getString("url");
                    apiPrefix.put(type, url);
                }
            }
        }

        if (apiPrefix.size() == 0) {
            throw new DriverException("未获取到的PowerVC的服务URL前缀");
        }

        // 提取租户唯一标识
        JSONObject projectJson = tokenJson.getJSONObject("project");
        String tenantId = projectJson.getString("id");

        Token token = new Token(tokenId, expires, apiPrefix, tenantId);

        if (logger.isDebugEnabled()) {
            logger.debug(token.toString());
        }

        return token;
    }
}
