package com.bocloud.cmp.provider.powervc4j.impl.image.v14;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.image.PowervcImageService;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.MapTools;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * PowerVC镜像服务实现
 *
 * @author sxp
 * @since Mar 27, 2018
 */
public class PvImageServiceImpl extends BaseServiceImpl<ImageModel> implements PowervcImageService {
    private static Logger logger = LoggerFactory.getLogger(PvImageServiceImpl.class);

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(PowervcImageService.class);
    }

    @Override
    public List<ImageModel> list() {
        String apiSuffix = "/images";
        return doList(apiSuffix);
    }

    @Override
    protected List<ImageModel> processListResult(String message) {
        JSONArray images = JSON.parseObject(message).getJSONArray("images");
        List<ImageModel> models = new ArrayList<>(images.size());
        for (int i = 0; i < images.size(); i++) {
            models.add(convert(images.getJSONObject(i)));
        }
        return models;
    }

    @Override
    public ImageModel get(String imageId) {
        String apiSuffix = String.format("/images/%s", imageId);
        return doGet(apiSuffix);
    }

    @Override
    protected ImageModel processGetResult(String message) {
        return convert(JSON.parseObject(message));
    }

    /**
     * 将JSON对象转换为ImageModel
     *
     * @author sxp
     * @since Mar 28, 2018
     */
    private ImageModel convert(JSONObject image) {
        ImageModel model = new ImageModel();

        model.setGmtCreate(image.getDate("created_at"));
        model.setImageUuid(image.getString("id"));
        model.setMinDisk(image.getLong("min_disk"));
        model.setName(image.getString("name"));
        model.setOsName(image.getString("os_distro"));
        model.setRemark(image.getString("powervc_comments"));

        JSONObject props = new JSONObject();
        props.put("architecture", image.getString("architecture"));
        props.put("containerFormat", image.get("container_format") == null ? null : image.getString("container_format").toUpperCase());
        props.put("diskFormat", image.get("disk_format") == null ? null : image.getString("disk_format").toUpperCase());
        if (image.getString("hypervisor_type").equals("QEMU")) {
            props.put("hypervisorType", "KVM");
        } else if (image.getString("hypervisor_type").equals("phyp")) {
            props.put("hypervisorType", "PowerVM");
        }
        model.setProps(props.toJSONString());

        long sizeValue = image.getLongValue("size");
        if (sizeValue != 0) {
            BigDecimal size = new BigDecimal(sizeValue).divide(new BigDecimal(1024 * 1024 * 1024), 2, RoundingMode.HALF_UP);
            model.setSize(size.doubleValue());
        }
        model.setStatus(image.getString("status"));
        model.setFilePath(image.getString("file"));
        model.setInstanceUuid(image.getString("instance_uuid"));

        JSONObject addition = image.getJSONObject("additionalProperties");
        if (addition != null) {
            model.setOsCategory(addition.getString("oscategory"));
            model.setOsName(addition.getString("osname"));
            model.setOsVersion(addition.getString("osversion"));
            model.setDefaultUser(addition.getString("default_user"));

            String agent = addition.getString("hw_qemu_guest_agent");

            if (StringUtils.isEmpty(agent)) {
                model.setQgaInstalled(false);
            } else if (agent.equals("yes")) {
                model.setQgaInstalled(true);
            } else if (agent.equals("no")) {
                model.setQgaInstalled(false);
            }
        }

        JSONArray volumes = JSON.parseArray(image.getString("block_device_mapping"));
        if (volumes != null) {
            model.setDiskDeviceMappings(new ArrayList<>(volumes.size()));
            for (int i = 0; i < volumes.size(); i++) {
                JSONObject volume = volumes.getJSONObject(i);
                VolumeModel volumeModel = new VolumeModel();
                volumeModel.setVolumeUuid(volume.getString("volume_id"));
                model.getDiskDeviceMappings().add(volumeModel);
            }
        }

        return model;
    }

    @Override
    public BocloudResult delete(String imageId) {
        if (StringUtils.isEmpty(imageId)) {
            return new BocloudResult(false, "镜像的唯一标识不能为空");
        }

        String apiSuffix = String.format("/images/%s", imageId);
        Result result = doDelete(apiSuffix, null);

        return new BocloudResult(result.isSuccess(), result.getMessage());
    }


    @Override
    public BocloudResult create(JSONObject params) {
        if (params == null) {
            return new BocloudResult(false, "镜像信息不能为空");
        }

        String apiSuffix = "/images";
        Result result = doCreate(apiSuffix, params);

        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        ImageModel imageModel = convert(JSON.parseObject(result.getMessage()));

        return new BocloudResult(true, imageModel, "执行成功");
    }

    @Override
    public BocloudResult file(String imageId) {
        if (imageId == null) {
            return new BocloudResult(false, "镜像的唯一标识不能为空");
        }

        String apiSuffix = String.format("/images/%s/file", imageId);
        Map<String, Object> header = MapTools.simpleMap("Content-Type", "application/octet-stream");
        Result result = doUpdate(apiSuffix, null, header);

        return new BocloudResult(result.isSuccess(), result.getMessage());
    }
}
