package com.bocloud.cmp.provider.powervc4j.impl.networking.v14;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.PortFixedIpModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.networking.PortService;
import com.bocloud.cmp.provider.powervc4j.api.networking.SubnetService;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * PowerVC子网服务实现类
 * 
 * @author sxp
 * @since Mar 21, 2018
 */
public class PortServiceImpl extends BaseServiceImpl<PortModel> implements PortService
{
    private static Logger logger = LoggerFactory.getLogger(PortServiceImpl.class);

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(SubnetService.class);
    }

    @Override
    public List<PortModel> list() {
        String apiSuffix = "/ports";
        return doList(apiSuffix);
    }

    @Override
    protected List<PortModel> processListResult(String message) {
        JSONArray ports = JSON.parseObject(message).getJSONArray("ports");
        List<PortModel> models = new ArrayList<>(ports.size());
        for (int i = 0; i < ports.size(); i++) {
            models.add(convert(ports.getJSONObject(i)));
        }
        return models;
    }

    @Override
    protected PortModel processGetResult(String message) {
        return convert(JSON.parseObject(message).getJSONObject("port"));
    }

    private PortModel convert(JSONObject port) {
        PortModel model = new PortModel();
        model.setPortUuid(port.getString("id"));
        model.setStatus(port.getString("status"));
        model.setDeviceUuid(port.getString("device_id") == null || port.getString("device_id").equals("") || port.getString("device_id").equals("Lock") ? "Lock" : port.getString("device_id"));
        model.setMacAddress(port.getString("mac_address"));
        model.setNetworkUuid(port.getString("network_id"));
        String deviceOwner = port.getString("device_owner");
        model.setBindingVifType(port.getString("binding:vif_type"));
        if (StringUtils.isNotEmpty(deviceOwner)) {
            String diviceOwnerPart1 = deviceOwner.split(":")[0];
            String diviceOwnerPart2 = deviceOwner.split(":")[1];
            if (diviceOwnerPart1.equals("PowerVC")) {
                model.setDeviceOwner(diviceOwnerPart2.trim());
            } else if (diviceOwnerPart1.equals("compute")) {
                model.setDeviceOwner("");
            } else {
                model.setDeviceOwner(deviceOwner.trim());
            }
        }

        JSONArray fixedIps = port.getJSONArray("fixed_ips");
        if (fixedIps != null) {
            List<PortFixedIpModel> ipModels = new ArrayList<>(fixedIps.size());
            for (int i = 0; i < fixedIps.size(); i++) {
                JSONObject fixedIpObject = fixedIps.getJSONObject(i);
                PortFixedIpModel ipModel = new PortFixedIpModel();
                ipModel.setIpAddress(fixedIpObject.getString("ip_address"));
                ipModel.setSubnetUuid(fixedIpObject.getString("subnet_id"));
                ipModels.add(ipModel);
            }
            model.setPortFixedIps(ipModels);
        }
        return model;
    }

    @Override
    public PortModel get(String portId) {
        String apiSuffix = "/ports/" + portId;
        return doGet(apiSuffix);
    }

    @Override
    public BocloudResult create(PortModel model) {
        String apiSuffix = "/ports";
        JSONArray fixedIps = new JSONArray();
        for (PortFixedIpModel portFixedIp : model.getPortFixedIps()) {
            JSONObject fixedIp = new JSONObject();
            fixedIp.put("ip_address", StringUtils.isEmpty(portFixedIp.getIpAddress()) ? null : portFixedIp.getIpAddress());
            fixedIp.put("subnet_id", portFixedIp.getSubnetUuid());
            fixedIps.add(fixedIp);
        }
        JSONObject profile = new JSONObject();
        profile.put("delete_with_instance", model.getDeleteWithInstance() == null || model.getDeleteWithInstance() ? 1 : 0);

        JSONObject port = new JSONObject();
        port.put("fixed_ips", fixedIps);
        port.put("device_id", model.getDeviceUuid());
        port.put("device_owner", model.getDeviceOwner() == null ? null : "PowerVC:" + model.getDeviceOwner());
        port.put("network_id", model.getNetworkUuid());
        port.put("binding:profile", profile);
        JSONObject params = new JSONObject();
        params.put("port", port);
        Result result = doCreate(apiSuffix, params);
        if (result.isFailed()) {
            return new BocloudResult(result.isSuccess(), result.getMessage());
        }
        model = convert(JSON.parseObject(result.getMessage()).getJSONObject("port"));
        return new BocloudResult(true, model, "创建成功");
    }

    @Override
    public BocloudResult modify(PortModel model) {
        String apiSuffix = String.format("/ports/%s", model.getPortUuid());
        JSONObject port = new JSONObject();
        if (StringUtils.isNotEmpty(model.getDeviceOwner())) {
            port.put("device_owner", "PowerVC:" + model.getDeviceOwner());
        }
        if (model.getPortFixedIps() != null) {
            JSONArray fixedIps = new JSONArray();
            for (PortFixedIpModel portFixedIp : model.getPortFixedIps()) {
                JSONObject fixedIp = new JSONObject();
                fixedIp.put("ip_address", portFixedIp.getIpAddress());
                fixedIp.put("subnet_id", portFixedIp.getSubnetUuid());
                fixedIps.add(fixedIp);
            }
            port.put("fixed_ips", fixedIps);
        }
        JSONObject params = new JSONObject();
        params.put("port", port);
        Result result = doUpdate(apiSuffix, params);
        if (result.isFailed()) {
            return new BocloudResult(result.isSuccess(), result.getMessage());
        }
        model = convert(JSON.parseObject(result.getMessage()).getJSONObject("port"));
        return new BocloudResult(true, model, "修改成功");
    }

    @Override
    public BocloudResult remove(String uuid) {
        String apiSuffix = "/ports/" + uuid;
        Result result = doDelete(apiSuffix, null);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }
}
