package com.bocloud.cmp.provider.powervc4j.impl.networking.v14;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.networking.SubnetService;
import com.bocloud.cmp.provider.powervc4j.common.util.MapUtils;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * PowerVC子网服务实现类
 * 
 * @author sxp
 * @since Mar 21, 2018
 */
public class SubnetServiceImpl extends BaseServiceImpl<SubnetModel> implements SubnetService {
    private static Logger logger = LoggerFactory.getLogger(SubnetServiceImpl.class);

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(SubnetService.class);
    }

    @Override
    public List<SubnetModel> list() {
        String apiSuffix = "/subnets";
        return doList(apiSuffix);
    }

    @Override
    protected List<SubnetModel> processListResult(String message) {
        logger.debug(message);

        try {
            JSONObject json = JSONTools.isJSONObj(message);
            JSONArray subnets = json.getJSONArray("subnets");

            List<SubnetModel> models = new ArrayList<>(subnets.size());

            PowervcProvider driver = getCurrentDriverSafely();
            boolean filter = driver.isOnlyShowCurrentTenantResource();
            String currentTenantId = getCurrentTenantIdSafely();

            JSONObject subnetJson;
            SubnetModel model;
            String tenantId;

            for (Object obj : subnets) {
                subnetJson = (JSONObject) obj;

                tenantId = subnetJson.getString("tenant_id");
                if (filter && StringUtils.isNotEmpty(tenantId) && !tenantId.equals(currentTenantId)) {
                    continue;
                }

                model = convert(subnetJson);

                models.add(model);
            }

            return models;
        } catch (Exception e) {
            String msg = "解析数据出错。" + e.getMessage();
            logger.error(msg);

            throw new DriverException(msg, e);
        }
    }

    // ----get

    @Override
    public SubnetModel get(String id) {
        String apiSuffix = String.format("/subnets/%s", id);
        return doGet(apiSuffix);
    }

    @Override
    protected SubnetModel processGetResult(String message) {
        logger.debug(message);

        try {
            JSONObject json = JSONTools.isJSONObj(message);
            JSONObject subnet = json.getJSONObject("subnet");

            SubnetModel model = convert(subnet);

            return model;
        } catch (Exception e) {
            String msg = "解析数据出错。" + e.getMessage();
            logger.error(msg);

            throw new DriverException(msg, e);
        }
    }

    private SubnetModel convert(JSONObject subnet) {
        SubnetModel model = new SubnetModel();

        model.setName(subnet.getString("name"));
        model.setCidr(subnet.getString("cidr"));
        model.setDhcpEnabled(subnet.getBoolean("enable_dhcp"));
        model.setDnsNames(JSON.toJSONString(subnet.get("dns_nameservers")));
        model.setSubnetUuid(subnet.getString("id"));
        model.setIpVersion("V" + subnet.getString("ip_version"));
        model.setNetworkUuid(subnet.getString("network_id"));
        model.setGateway(subnet.getString("gateway_ip"));

        JSONArray hostRoutes = subnet.getJSONArray("host_routes");
        JSONObject hostRoute;
        int size;

        if (hostRoutes != null && (size = hostRoutes.size()) > 0) {
            List<String> hostRouteList = new ArrayList<>(size);

            for (Object obj : hostRoutes) {
                if (obj instanceof JSONObject) {
                    hostRoute = (JSONObject) obj;
                    hostRouteList.add(hostRoute.getString("destination") + "," + hostRoute.getString("nexthop"));
                }
            }

            model.setHostRoutes(JSONObject.toJSONString(hostRouteList));
        }

        JSONArray pools = subnet.getJSONArray("allocation_pools");
        JSONObject pool;

        if (pools != null && (size = pools.size()) > 0) {
            List<String> ipPools = new ArrayList<>(size);

            for (Object poolObj : pools) {
                if (poolObj instanceof JSONObject) {
                    pool = (JSONObject) poolObj;
                    ipPools.add(pool.getString("start") + "-" + pool.getString("end"));
                }
            }

            model.setIpPools(JSON.toJSONString(ipPools));
        }
        return model;
    }

    @Override
    public BocloudResult create(SubnetModel model) {
        String apiSuffix = "/subnets";

        Map<String, Object> params = getSubnetParams(model);

        Result result = doCreate(apiSuffix, params);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        String message = result.getMessage();

        try {
            JSONObject json = JSONTools.isJSONObj(message);
            JSONObject subnetJson = json.getJSONObject("subnet");
            SubnetModel responseModel = convert(subnetJson);

            return new BocloudResult(true, responseModel, result.getMessage());
        } catch (Exception e) {
            String msg = "解析子网网络创建的返回数据出错，" + e.getMessage();
            logger.error(msg);

            return new BocloudResult(false, msg);
        }
    }

    private Map<String, Object> getSubnetParams(SubnetModel model) {
        JSONObject subnet = new JSONObject();
        subnet.put("name", model.getName());
        subnet.put("network_id", model.getNetworkUuid());

        String version = model.getIpVersion();
        if (version == null || version.length() != 2) {
            version = "4";
        } else {
            version = version.substring(1, 2);
        }
        subnet.put("ip_version", version);

        subnet.put("cidr", model.getCidr());
        subnet.put("enable_dhcp", model.getDhcpEnabled());

        String gateway = model.getGateway();
        if (StringUtils.isNotEmpty(gateway)) {
            subnet.put("gateway_ip", gateway);
        }

        String ipPools = model.getIpPools();
        String[] startEndIp;

        if (StringUtils.isNotEmpty(ipPools)) {
            List<String> ips = JSON.parseArray(ipPools, String.class);

            if (ips != null && ips.size() > 0) {
                List<Map<String, String>> ipPoolResult = new ArrayList<>();
                Map<String, String> ipItem;

                for (String ipPool : ips) {
                    startEndIp = ipPool.split("-");

                    if (startEndIp != null && startEndIp.length == 2) {
                        ipItem = new HashMap<>(MapUtils.capacity(2));
                        ipItem.put("start", startEndIp[0]);
                        ipItem.put("end", startEndIp[1]);

                        ipPoolResult.add(ipItem);
                    }
                }

                subnet.put("allocation_pools", ipPoolResult);
            }
        }

        String HostRoutesStr = model.getHostRoutes();
        String[] hostRouteArray;

        if (StringUtils.isNotEmpty(HostRoutesStr)) {
            List<String> list = JSON.parseArray(HostRoutesStr, String.class);

            if (list != null && list.size() > 0) {
                List<Map<String, String>> routerResult = new ArrayList<>();
                Map<String, String> routerItem;

                for (String hostRoute : list) {
                    hostRouteArray = hostRoute.split(",");
                    if (hostRouteArray != null && hostRouteArray.length == 2) {
                        routerItem = new HashMap<>(MapUtils.capacity(2));
                        routerItem.put("destination", hostRouteArray[0]);
                        routerItem.put("nexthop", hostRouteArray[1]);

                        routerResult.add(routerItem);
                    }
                }

                subnet.put("host_routes", routerResult);
            }
        }

        String namesStr = model.getDnsNames();
        if (StringUtils.isNotEmpty(namesStr)) {
            List<String> dnsNames = JSON.parseArray(namesStr, String.class);
            subnet.put("dns_nameservers", dnsNames);
        }

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("subnet", subnet);
        return params;
    }

    @Override
    public BocloudResult update(SubnetModel model) {
        Objects.requireNonNull(model, "model");

        String subnetId = model.getSubnetUuid();
        if (StringUtils.isEmpty(subnetId)) {
            return new BocloudResult(false, "子网唯一标识不能为空");
        }

        String apiSuffix = String.format("/subnets/%s", subnetId);
        Map<String, Object> params = getSubnetUpdateParams(model);

        Result result = doUpdate(apiSuffix, params);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        String message = result.getMessage();

        try {
            JSONObject json = JSONTools.isJSONObj(message);
            JSONObject subnetJson = json.getJSONObject("subnet");
            SubnetModel responseModel = convert(subnetJson);

            return new BocloudResult(true, responseModel, result.getMessage());
        } catch (Exception e) {
            String msg = "解析子网更新的返回数据出错，" + e.getMessage();
            logger.error(msg);

            return new BocloudResult(false, msg);
        }
    }

    private Map<String, Object> getSubnetUpdateParams(SubnetModel model) {
        JSONObject subnet = new JSONObject();
        subnet.put("name", model.getName());
        subnet.put("enable_dhcp", model.getDhcpEnabled());

        String gateway = model.getGateway();
        if (StringUtils.isNotEmpty(gateway)) {
            subnet.put("gateway_ip", gateway);
        }

        String ipPools = model.getIpPools();
        String[] startEndIp;

        if (StringUtils.isNotEmpty(ipPools)) {
            List<String> ips = JSON.parseArray(ipPools, String.class);

            if (ips != null && ips.size() > 0) {
                List<Map<String, String>> ipPoolResult = new ArrayList<>();
                Map<String, String> ipItem;

                for (String ipPool : ips) {
                    startEndIp = ipPool.split("-");

                    if (startEndIp != null && startEndIp.length == 2) {
                        ipItem = new HashMap<>(MapUtils.capacity(2));
                        ipItem.put("start", startEndIp[0]);
                        ipItem.put("end", startEndIp[1]);

                        ipPoolResult.add(ipItem);
                    }
                }

                subnet.put("allocation_pools", ipPoolResult);
            }
        }

        String HostRoutesStr = model.getHostRoutes();
        String[] hostRouteArray;

        if (StringUtils.isNotEmpty(HostRoutesStr)) {
            List<String> list = JSON.parseArray(HostRoutesStr, String.class);

            if (list != null && list.size() > 0) {
                List<Map<String, String>> routerResult = new ArrayList<>();
                Map<String, String> routerItem;

                for (String hostRoute : list) {
                    hostRouteArray = hostRoute.split(",");
                    if (hostRouteArray != null && hostRouteArray.length == 2) {
                        routerItem = new HashMap<>(MapUtils.capacity(2));
                        routerItem.put("destination", hostRouteArray[0]);
                        routerItem.put("nexthop", hostRouteArray[1]);

                        routerResult.add(routerItem);
                    }
                }

                subnet.put("host_routes", routerResult);
            }
        }

        String namesStr = model.getDnsNames();
        if (StringUtils.isNotEmpty(namesStr)) {
            List<String> dnsNames = JSON.parseArray(namesStr, String.class);
            subnet.put("dns_nameservers", dnsNames);
        }

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("subnet", subnet);
        return params;
    }

    @Override
    public BocloudResult delete(String subnetId) {
        if (StringUtils.isEmpty(subnetId)) {
            return new BocloudResult(false, "子网唯一标识不能为空");
        }

        String apiSuffix = String.format("/subnets/%s", subnetId);
        Result result = doDelete(apiSuffix, null);

        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

}
