package com.bocloud.cmp.provider.powervc4j.impl.storage.v13;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.ServerVolumeModel;
import com.bocloud.cmp.model.VolumeMetadataModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.storage.BlockVolumeService;
import com.bocloud.cmp.provider.powervc4j.common.enums.RequestMethod;
import com.bocloud.cmp.provider.powervc4j.common.util.DateUtils;
import com.bocloud.cmp.provider.powervc4j.common.util.MapUtils;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 存储卷管理服务实现类
 *
 * @author sxp
 * @since Mar 21, 2018
 */
public class BlockVolumeServiceImpl extends BaseServiceImpl<VolumeModel> implements BlockVolumeService {
    private static Logger logger = LoggerFactory.getLogger(BlockVolumeServiceImpl.class);

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(BlockVolumeService.class);
    }

    // ----list

    @Override
    public List<VolumeModel> list() {
        String apiSuffix = "/volumes/detail";
        return doList(apiSuffix);
    }

    @Override
    protected List<VolumeModel> processListResult(String message) {
        logger.debug(message);

        try {
            JSONObject json = JSONTools.isJSONObj(message);
            JSONArray volumes = json.getJSONArray("volumes");

            List<VolumeModel> models = new ArrayList<>(volumes.size());

            JSONObject volume;
            VolumeModel model;

            for (Object obj : volumes) {
                volume = (JSONObject) obj;
                model = convert(volume);

                models.add(model);
            }

            return models;
        } catch (Exception e) {
            String msg = "解析数据出错。" + e.getMessage();
            logger.error(msg);

            throw new DriverException(msg, e);
        }
    }

    private VolumeModel convert(JSONObject volume) {
        JSONArray attachments;
        JSONObject imageMetadata, att;
        String imageRef, imageId, name;
        int size;

        VolumeModel model = new VolumeModel();
        model.setStorageHost(volume.getString("os-vol-host-attr:host"));
        model.setCategory(volume.getString("volume_type"));
        model.setMultiattach(volume.getBoolean("multiattach"));
        String createStr = volume.getString("created_at");
        if (createStr != null) {
            createStr = createStr + "Z";
            model.setCreateTime(DateUtils.getDateFromUTCString(createStr));
        }

        imageRef = volume.getString("imageRef");
        if (StringUtils.isNotEmpty(imageRef)) {
            model.setImageUuid(imageRef);
        } else {
            imageId = volume.getString("image_id");
            if (StringUtils.isEmpty(imageId)) {
                imageMetadata = volume.getJSONObject("volume_image_metadata");
                if (imageMetadata != null && imageMetadata.containsKey("image_id")) {
                    imageId = imageMetadata.getString("image_id");
                    model.setImageUuid(imageId);
                }
            }
        }

        name = volume.getString("name");
        if (StringUtils.isEmpty(name)) {
            model.setName(volume.getString("id"));
        } else {
            model.setName(name);
        }

        model.setRemark(volume.getString("description"));
        model.setSize(volume.getIntValue("size"));
        model.setStatus(volume.getString("status"));
        model.setType(volume.getString("volume_type"));
        model.setVolumeUuid(volume.getString("id"));
        // model.setUuid(volume.getString("source_volid"));
        model.setZone(volume.getString("availability_zone"));
        model.setBootVolume("true".equalsIgnoreCase(volume.getJSONObject("metadata").getString("is_boot_volume")));
        model.setWwn(volume.getJSONObject("metadata").getString("volume_wwn"));
        model.setPoolName(volume.getString("storage_pool"));

        attachments = volume.getJSONArray("attachments");
        if (attachments != null && (size = attachments.size()) > 0) {
            List<ServerVolumeModel> serverModels = new ArrayList<>(size);

            for (Object attachmentObj : attachments) {
                if (attachmentObj instanceof JSONObject) {
                    att = (JSONObject) attachmentObj;
                    ServerVolumeModel serverVolumeModel = new ServerVolumeModel();
                    serverVolumeModel.setUuid(att.getString("id"));
                    serverVolumeModel.setDevice(att.getString("device"));
                    serverVolumeModel.setVolumeUuid(att.getString("volume_id"));
                    serverVolumeModel.setInstanceId(att.getString("server_id"));
                    serverModels.add(serverVolumeModel);
                }
            }

            model.setModels(serverModels);
        }
        // add support info
        JSONObject support = volume.getJSONObject("support");
        if (support != null) {
            model.setSupport("supported".equals(support.getString("status")));
            List<String> reasons = JSONObject.parseArray(support.getString("reasons"), String.class);
            if (reasons != null) {
                model.setReason(reasons.get(0));
            }
        }
        model.setManaged(volume.getBoolean("managed"));

        return model;
    }

    // ----get

    @Override
    public VolumeModel get(String volumeId) {
        if (StringUtils.isEmpty(volumeId)) {
            throw new DriverException("查询存储卷信息时，唯一标识不能为空");
        }
        String apiSuffix = String.format("/volumes/%s", volumeId);

        return doGet(apiSuffix);
    }

    @Override
    protected VolumeModel processGetResult(String message) {
        logger.debug("powervc volume info {}", message);

        try {
            JSONObject json = JSONTools.isJSONObj(message);
            JSONObject volume = json.getJSONObject("volume");

            VolumeModel model = convert(volume);

            return model;
        } catch (Exception e) {
            String msg = "解析数据出错。" + e.getMessage();
            logger.error(msg);

            throw new DriverException(msg, e);
        }
    }

    @Override
    public BocloudResult create(VolumeModel model) {
        String apiSuffix = "/volumes";

        Map<String, Object> params = getVolumeParams(model);

        Result result = doCreate(apiSuffix, params);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        String message = result.getMessage();

        try {
            JSONObject json = JSONTools.isJSONObj(message);
            JSONObject volumeJson = json.getJSONObject("volume");
            VolumeModel responseModel = convert(volumeJson);

            return new BocloudResult(true, responseModel, result.getMessage());
        } catch (Exception e) {
            String msg = "解析存储卷创建的返回数据出错，" + e.getMessage();
            logger.error(msg);

            return new BocloudResult(false, msg);
        }
    }

    private Map<String, Object> getVolumeParams(VolumeModel model) {
        JSONObject volume = new JSONObject();
        volume.put("name", model.getName());
        volume.put("description", model.getRemark());
        volume.put("multiattach", model.getMultiattach());
        volume.put("volume_type", model.getVolumeTemplateUuid());
        Integer size = model.getSize();
        if (size == null) {
            size = 1;
        }
        volume.put("size", size);

        Boolean bootable = model.getBootVolume();
        if (bootable == null) {
            bootable = false;
        }

        if (bootable) {
            volume.put("imageRef", model.getImageUuid());
        }

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("volume", volume);
        return params;
    }

    /*
     * private Map<String, Object> getVolumeUpdateParams(VolumeModel model) {
     * JSONObject volume = new JSONObject(); volume.put("name",
     * model.getName()); volume.put("description", model.getRemark());
     *
     * Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
     * params.put("volume", volume); return params; }
     */

    @Override
    public BocloudResult delete(String volumeId) {
        if (StringUtils.isEmpty(volumeId)) {
            throw new DriverException("存储卷唯一标识不能为空");
        }
        String apiSuffix = String.format("/volumes/%s", volumeId);

        Result result = doDelete(apiSuffix, null);

        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    @Override
    public BocloudResult getMetadata(String volumeId) {
        try {
            String apiSuffix = String.format("/volumes/%s/restricted-metadata", volumeId);

            //{"metadata": {"volume_wwn": "60050768029E0EB8C800000000000081", "vdisk_name": "AIX-NOT-REMOVE",
            // "vdisk_uid": "60050768029E0EB8C800000000000081", "vdisk_id": "5"}}
            JSONObject jsonObject = customGet(JSONObject.class, apiSuffix);
            if (jsonObject == null) {
                logger.error("metadata查询失败");
                return new BocloudResult(false, "查询卷metadata失败");
            } else {
                return new BocloudResult(true, jsonObject.getJSONObject("metadata"), "查询卷metadata成功");
            }
        } catch (Exception e) {
            String msg = "解析查询卷的metadata的返回数据出错，" + e.getMessage();
            logger.error(msg);
            return new BocloudResult(false, msg);
        }
    }

    @Override
    public BocloudResult setMetadata(String volumeId, VolumeMetadataModel metadataModel) {
        try {
            String apiSuffix = String.format("/volumes/%s/metadata", volumeId);
            JSONObject metadata = new JSONObject();
            metadata.put("image_id", metadataModel.getImageId());
            metadata.put("volume_wwn", metadataModel.getWwn());
            metadata.put("is_boot_volume", metadataModel.getBoot() != null && metadataModel.getBoot() ? "True" :"False");
            metadata.put("is_image_volume", metadataModel.getImage() != null && metadataModel.getImage() ? "True" :"False");

            JSONObject params = new JSONObject();
            params.put("metadata", metadata);

            Result result = doUpdate(apiSuffix, params);
            if (result.isFailed()) {
                logger.error("更新volume的metadata信息失败:{}", result);
                return new BocloudResult(false, "更新metadat失败:" + result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            metadataModel.setWwn(jsonObject.getJSONObject("metadata").getString("volume_wwn"));
            metadataModel.setImageId(jsonObject.getJSONObject("metadata").getString("image_id"));
            metadataModel.setBoot(jsonObject.getJSONObject("metadata").getBoolean("is_boot_volume"));
            metadataModel.setImage(jsonObject.getJSONObject("metadata").getBoolean("is_image_volume"));

            return new BocloudResult(true, metadataModel, "更新卷metadata成功");
        } catch (Exception e) {
            logger.error("更新卷的metadata出错", e);
            return new BocloudResult(false, "更新卷的metadata出错，" + e.getMessage());
        }
    }

    @Override
    public BocloudResult onboard(String storageName, List<String> volumeIds) {
        if (storageName == null) {
            throw new DriverException("存储名称不能为空");
        }
        // 不传volumeIds，默认管理所有powervc支持的卷
        if (volumeIds == null || volumeIds.size() == 0) {
            volumeIds = Lists.newArrayList();
            volumeIds.add("*ALL");
        }
        String apiSuffix = String.format("/os-hosts/%s/onboard", storageName);
        JSONObject params = new JSONObject();
        params.put("volumes", volumeIds);
        Result result = doCreate(apiSuffix, params);
        if (result.isSuccess()) {
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage(), JSONObject.class);
            return new BocloudResult(true, jsonObject, "管理存储器现有卷成功");
        } else {
            return new BocloudResult(false, "管理存储器现有卷失败:" + result.getMessage());
        }
    }

    @Override
    public BocloudResult unmanage(String storageName, List<String> volumeIds) {
        if (storageName == null) {
            throw new DriverException("存储名称不能为空");
        }
        if (volumeIds == null || volumeIds.isEmpty()) {
            throw new DriverException("存储卷信息不能为空");
        }
        String apiSuffix = String.format("/os-hosts/%s/unmanage", storageName);
        JSONObject params = new JSONObject();
        params.put("volumes", volumeIds);
        Result result = doCreate(apiSuffix, params);
        if (result.isSuccess()) {
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage(), JSONObject.class);
            return new BocloudResult(true, jsonObject, "取消管理存储上的卷任务下发成功, 正在执行...");
        } else {
            return new BocloudResult(false, "取消管理存储上的卷失败:" + result.getMessage());
        }
    }

    @Override
    public BocloudResult modify(VolumeModel model) {
        String apiSuffix = String.format("/volumes/%s", model.getVolumeUuid());
        JSONObject volume = new JSONObject();
        volume.put("multiattach", model.getMultiattach());
        JSONObject params = new JSONObject();
        params.put("volume", volume);
        Result result = doUpdate(apiSuffix, params);
        if (result.isFailed()) {
            return new BocloudResult(result.isSuccess(), result.getMessage());
        }
        VolumeModel volumeModel = convert(JSON.parseObject(result.getMessage()).getJSONObject("volume"));
        return new BocloudResult(true, volumeModel, "修改成功");
    }

    @Override
    public BocloudResult extend(String volumeId, int size) {
        String apiSuffix = String.format("/volumes/%s/action", volumeId);
        JSONObject extend = new JSONObject();
        extend.put("new_size", size);
        JSONObject params = new JSONObject();
        params.put("ibm-extend", extend);
        Result result = work(RequestMethod.POST, apiSuffix, params);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    @Override
    public BocloudResult allVolumes(String storageName) {
        if (storageName == null) {
            throw new DriverException("存储名称不能为空");
        }
        String apiSuffix = String.format("/os-hosts/%s/all-volumes", storageName);

        JSONObject json = customGet(JSONObject.class,apiSuffix);
        if (json!=null) {
            JSONArray volumes = json.getJSONArray("volumes");

            List<VolumeModel> models = new ArrayList<>(volumes.size());

            JSONObject volume;
            VolumeModel model;

            for (Object obj : volumes) {
                volume = (JSONObject) obj;
                model = convert(volume);

                models.add(model);
            }
            return new BocloudResult(true, models, "查询存储器上的所有卷成功");
        } else {
            return new BocloudResult(false, "查询存储器上的所有卷失败");
        }
    }

}
