package com.bocloud.cmp.provider.powervc4j.impl.storage.v14;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.VolumeTemplateModel;
import com.bocloud.cmp.model.powervc.PowervcConstant;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.storage.BlockVolumeTemplateService;
import com.bocloud.cmp.provider.powervc4j.common.enums.RequestMethod;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

;

/**
 * 存储器模板
 * @author luyanchao
 * @since 2019/1/24 15:47
 */
public class BlockVolumeTemplateServiceImpl extends BaseServiceImpl<VolumeTemplateModel> implements BlockVolumeTemplateService {

    private static Logger logger = LoggerFactory.getLogger(BlockVolumeTemplateServiceImpl.class);

    @Override
    public List<VolumeTemplateModel> list() {
        String apiSuffix = "/types";
        return doList(apiSuffix);
    }

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(BlockVolumeTemplateService.class);
    }

    private VolumeTemplateModel convertVolumeTemplate(JSONObject template) {
        VolumeTemplateModel model = new VolumeTemplateModel();

        model.setOriginName(template.getString("name"));
        model.setUuid(template.getString("id"));
        model.setDescription(template.getString("description"));
        model.setIsPublic(template.getBoolean("is_public"));

        JSONObject extraSpecs = template.getJSONObject("extra_specs");
        setVolumeTemplateExtra(model, extraSpecs);

        return model;
    }

    private void setVolumeTemplateExtra(VolumeTemplateModel model, JSONObject extraSpecs) {
        model.setName(extraSpecs.getString("drivers:display_name"));
        model.setAutoExpand(extraSpecs.getBoolean("drivers:autoexpand"));
        model.setCompress(extraSpecs.getBoolean("drivers:compression"));
        model.setIoGroup(extraSpecs.getString("drivers:iogrp"));
        model.setStorageName(extraSpecs.getString("capabilities:volume_backend_name"));
        model.setPoolName(extraSpecs.getString("drivers:storage_pool"));
        model.setCopyCount(extraSpecs.getInteger("drivers:replication"));
        model.setCopyPoolName(extraSpecs.getString("drivers:mirror_pool"));
        model.setMultipath(BooleanUtils.toBoolean(extraSpecs.getString("drivers:multipath")));
        model.setCapacityPercent(extraSpecs.getFloat("drivers:rsize"));
        model.setWarningPercent(extraSpecs.getFloat("drivers:warning"));
        model.setGrainsize(extraSpecs.getFloat("drivers:grainsize"));
    }

    @Override
    protected List<VolumeTemplateModel> processListResult(String message) {
        logger.debug(message);

        try {
            JSONObject json = JSONTools.isJSONObj(message);
            JSONArray templates = json.getJSONArray("volume_types");
            List<VolumeTemplateModel> models = new ArrayList<>(templates.size());
            for (Object obj : templates) {
                models.add(convertVolumeTemplate((JSONObject) obj));
            }
            return models;
        } catch (Exception e) {
            String msg = "解析数据出错。" + e.getMessage();
            logger.error(msg);

            throw new DriverException(msg, e);
        }
    }

    @Override
    protected VolumeTemplateModel processGetResult(String message) {
        return convertVolumeTemplate(JSONTools.isJSONObj(message).getJSONObject("volume_type"));
    }

    @Override
    public BocloudResult get(String uuid) {
        if (StringUtils.isEmpty(uuid)) {
            throw new DriverException("查询存储卷模板信息时，唯一标识不能为空");
        }
        String apiSuffix = String.format("/types/%s", uuid);
        VolumeTemplateModel model = doGet(apiSuffix);
        return new BocloudResult(true, model, "success");
    }

    private Map<String, Object> getVolumeTemplateParams(VolumeTemplateModel model) {
        Map<String, Object> extraSpecs = getVolumeTemplateExtraParams(model);

        Map<String, Object> volumeType = new HashMap<>();
        volumeType.put("name", model.getName());
        volumeType.put("extra_specs", extraSpecs);

        Map<String, Object> params = new HashMap<>();
        params.put("volume_type", volumeType);

        return params;
    }

    private Map<String, Object> getVolumeTemplateExtraParams(VolumeTemplateModel model) {
        Map<String, Object> extraSpecs = new HashMap<>();
        extraSpecs.put("drivers:display_name", model.getName());
        extraSpecs.put("drivers:autoexpand", model.getAutoExpand() != null && model.getAutoExpand() ? "1" : "0");
        extraSpecs.put("drivers:compression", model.getCompress() != null && model.getCompress() ? "1" : "0");
        extraSpecs.put("drivers:iogrp", model.getIoGroup() == null ? "0" : model.getIoGroup());
        extraSpecs.put("capabilities:volume_backend_name", model.getStorageName());
        extraSpecs.put("drivers:storage_pool", model.getPoolName());
        extraSpecs.put("drivers:replication", model.getCopyCount() == null ? "0" : model.getCopyCount());
        extraSpecs.put("drivers:mirror_pool", model.getCopyPoolName());
        extraSpecs.put("drivers:multipath", model.getMultipath() != null && model.getMultipath() ? "True" : "False");
        extraSpecs.put("drivers:rsize", model.getCapacityPercent() == null ?
                String.valueOf(PowervcConstant.VOLUME_TEMPLATE_CAPACITY_PERCENT.intValue()) : String.valueOf(model.getCapacityPercent().intValue()));
        extraSpecs.put("drivers:warning", model.getWarningPercent() == null ?
                String.valueOf(PowervcConstant.VOLUME_TEMPLATE_WARNING_PERCENT.intValue()) : String.valueOf(model.getWarningPercent().intValue()));
        extraSpecs.put("drivers:grainsize", model.getGrainsize() == null ?
                String.valueOf(PowervcConstant.VOLUME_TEMPLATE_GRAINSIZE.intValue()) : String.valueOf(model.getGrainsize().intValue()));
        return extraSpecs;
    }

    @Override
    public BocloudResult create(VolumeTemplateModel model) {
        String apiSuffix ="/types";
        Result result = doCreate(apiSuffix, getVolumeTemplateParams(model));
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        model = convertVolumeTemplate(JSON.parseObject(result.getMessage()).getJSONObject("volume_type"));
        return new BocloudResult(true, model, result.getMessage());
    }

    @Override
    public BocloudResult modify(VolumeTemplateModel model) {
        String apiSuffix = String.format("/types/%s/extra_specs", model.getUuid());
        Map<String, Object> extraSpecs = getVolumeTemplateExtraParams(model);
        Map<String, Object> params = new HashMap<>();
        params.put("extra_specs", extraSpecs);
        Result result = work(RequestMethod.POST, apiSuffix, params);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        setVolumeTemplateExtra(model, JSON.parseObject(result.getMessage()).getJSONObject("extra_specs"));
        return new BocloudResult(true, model, result.getMessage());
    }

    @Override
    public BocloudResult remove(String uuid) {
        String apiSuffix = String.format("/types/%s", uuid);
        Result result = doDelete(apiSuffix, null);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }
}
