package com.bocloud.cmp.provider.powervc4j.provider;

import com.bocloud.cmp.model.powervc.PowervcConstant;
import com.bocloud.cmp.provider.powervc4j.api.APIProvider;
import com.bocloud.cmp.provider.powervc4j.api.compute.ComputeService;
import com.bocloud.cmp.provider.powervc4j.api.compute.FlavorExtraService;
import com.bocloud.cmp.provider.powervc4j.api.compute.FlavorService;
import com.bocloud.cmp.provider.powervc4j.api.compute.HmcService;
import com.bocloud.cmp.provider.powervc4j.api.compute.HostSeaService;
import com.bocloud.cmp.provider.powervc4j.api.compute.HostService;
import com.bocloud.cmp.provider.powervc4j.api.compute.MessageService;
import com.bocloud.cmp.provider.powervc4j.api.compute.OsAggregateService;
import com.bocloud.cmp.provider.powervc4j.api.compute.OsHostService;
import com.bocloud.cmp.provider.powervc4j.api.compute.ServerService;
import com.bocloud.cmp.provider.powervc4j.api.compute.ViosGroupService;
import com.bocloud.cmp.provider.powervc4j.api.compute.ext.HypervisorService;
import com.bocloud.cmp.provider.powervc4j.api.image.ImageService;
import com.bocloud.cmp.provider.powervc4j.api.image.PowervcImageService;
import com.bocloud.cmp.provider.powervc4j.api.networking.NetworkService;
import com.bocloud.cmp.provider.powervc4j.api.networking.NetworkingService;
import com.bocloud.cmp.provider.powervc4j.api.networking.PortService;
import com.bocloud.cmp.provider.powervc4j.api.networking.SubnetService;
import com.bocloud.cmp.provider.powervc4j.api.notification.NotificationService;
import com.bocloud.cmp.provider.powervc4j.api.notification.WebEventService;
import com.bocloud.cmp.provider.powervc4j.api.storage.BlockStorageDeviceService;
import com.bocloud.cmp.provider.powervc4j.api.storage.BlockStorageService;
import com.bocloud.cmp.provider.powervc4j.api.storage.BlockVolumePoolService;
import com.bocloud.cmp.provider.powervc4j.api.storage.BlockVolumeService;
import com.bocloud.cmp.provider.powervc4j.api.storage.BlockVolumeTemplateService;
import com.bocloud.cmp.provider.powervc4j.api.storage.FabircService;
import com.bocloud.cmp.provider.powervc4j.api.validation.PowervcValidateService;
import com.bocloud.cmp.provider.powervc4j.api.validation.ValidateService;
import com.bocloud.cmp.provider.powervc4j.common.util.MapUtils;
import com.bocloud.cmp.provider.powervc4j.exception.ApiNotFoundException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

;

/**
 * PowerVC API 提供者的默认实现类
 *
 * @author sxp
 * @since Mar 20, 2018
 */
public class DefaultAPIProvider implements APIProvider {

    private final Logger logger = LoggerFactory.getLogger(DefaultAPIProvider.class);
    /**
     * serviceBindings这个map里面记录了每个接口类在不用的Powervc版本下的实现。
     * 由于最开始Powervc是基于1.3版本开发的，所以在寻找其他版本的实现类的时候，如果找不到则会默认使用1.3版本的实现类。
     */
    private static final Map<Class<?>, Map<PowervcConstant.Version, Class<?>>> serviceBindings = new ConcurrentHashMap<>(MapUtils.capacity(10));
    private static final Map<Class<?>, Map<PowervcConstant.Version, Object>> serviceInstances = new ConcurrentHashMap<>(MapUtils.capacity(10));

    private static final Map<Class<?>, Class<?>> topServices = new ConcurrentHashMap<>(MapUtils.capacity(10));

    @Override
    public void initialize() {
        bindServices();
        bindTopServices();
    }

    @SuppressWarnings("unchecked")
    @Override
    public <T> T get(Class<T> api, PowervcConstant.Version version) {
        if (serviceInstances.containsKey(api) && serviceInstances.get(api).containsKey(version)) {
            return (T) serviceInstances.get(api).get(version);
        }

        if (serviceBindings.containsKey(api)) {
            T impl;

            if (serviceBindings.get(api).containsKey(version)) {
                try {
                    impl = (T) serviceBindings.get(api).get(version).newInstance();
                } catch (InstantiationException | IllegalAccessException e) {
                    throw new ApiNotFoundException("API Not found for: " + api.getName(), e);
                }
            } else if (serviceBindings.get(api).containsKey(PowervcConstant.Version.v14)) {
                try {
                    impl = (T) serviceBindings.get(api).get(PowervcConstant.Version.v14).newInstance();
                } catch (InstantiationException | IllegalAccessException e) {
                    throw new ApiNotFoundException("API Not found for: " + api.getName(), e);
                }
            } else {
                throw new ApiNotFoundException("API Not found for: " + api.getName());
            }

            Map<PowervcConstant.Version, Object> instanceMap = serviceInstances.get(api);
            if (instanceMap == null) {
                instanceMap = new ConcurrentHashMap<>();
            }
            instanceMap.put(version, impl);
            serviceInstances.put(api, instanceMap);
            logger.info("注册Powervc接口实例：" + api.getSimpleName() + "[" + version.getValue() + "] --> " + impl.getClass().getName());
            return impl;
        }

        throw new ApiNotFoundException("API Not found for: " + api.getName());
    }

    @Override
    public Class<?> getTop(Class<?> api) {
        if (topServices.containsKey(api)) {
            return topServices.get(api);
        }

        return api;
    }

    /**
     * 绑定服务接口类与实现类
     *
     * @author sxp
     * @since Mar 20, 2018
     */
    private void bindServices() {
        // 1.4版本 该版本为最开始开发的版本，寻找接口实现类的时候，如果要寻找的版本不存在相应的实现类，则默认按照该版本处理
        bind(ComputeService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.compute.v14.ComputeServiceImpl.class);
        bind(HypervisorService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.compute.v14.HypervisorServiceImpl.class);
        bind(ServerService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.compute.v14.ServerServiceImpl.class);
        bind(FlavorService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.compute.v14.FlavorServiceImpl.class);
        bind(FlavorExtraService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.compute.v14.FlavorExtraServiceImpl.class);
        bind(HostService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.compute.v14.HostServiceImpl.class);
        bind(HmcService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.compute.v14.HmcServiceImpl.class);
        bind(OsHostService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.compute.v14.OsHostServiceImpl.class);
        bind(MessageService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.compute.v14.MessageServiceImpl.class);
        bind(OsAggregateService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.compute.v14.OsAggregateServiceImpl.class);
        bind(HostSeaService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.compute.v14.HostSeaServiceImpl.class);
        bind(ViosGroupService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.compute.v14.ViosGroupServiceImpl.class);

        bind(NetworkingService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.networking.v14.NetworkingServiceImpl.class);
        bind(NetworkService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.networking.v14.NetworkServiceImpl.class);
        bind(SubnetService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.networking.v14.SubnetServiceImpl.class);
        bind(PortService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.networking.v14.PortServiceImpl.class);

        bind(BlockStorageService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.storage.v14.BlockStorageServiceImpl.class);
        bind(BlockStorageDeviceService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.storage.v14.BlockStorageDeviceServiceImpl.class);
        bind(BlockVolumeService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.storage.v14.BlockVolumeServiceImpl.class);
        bind(BlockVolumeTemplateService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.storage.v14.BlockVolumeTemplateServiceImpl.class);
        bind(BlockVolumePoolService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.storage.v14.BlockVolumePoolServiceImpl.class);
        bind(FabircService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.storage.v14.FabircServiceImpl.class);

        bind(ImageService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.image.v14.ImageServiceImpl.class);
        bind(PowervcImageService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.image.v14.PvImageServiceImpl.class);

        bind(ValidateService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.validation.v14.ValidateServiceImpl.class);
        bind(PowervcValidateService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.validation.v14.PowervcValidateServiceImpl.class);

        bind(NotificationService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.notification.v14.NotificationServiceImpl.class);
        bind(WebEventService.class, PowervcConstant.Version.v14, com.bocloud.cmp.provider.powervc4j.impl.notification.v14.WebEventServiceImpl.class);

        // 1.3版本
        bind(BlockVolumeService.class, PowervcConstant.Version.v13, com.bocloud.cmp.provider.powervc4j.impl.storage.v13.BlockVolumeServiceImpl.class);
    }

    /**
     * 绑定服务接口类与与其从属的顶层服务接口类
     *
     * @author sxp
     * @since Mar 20, 2018
     */
    private void bindTopServices() {
        bindTop(HypervisorService.class, ComputeService.class);
        bindTop(ServerService.class, ComputeService.class);
        bindTop(FlavorService.class, ComputeService.class);
        bindTop(FlavorExtraService.class, ComputeService.class);
        bindTop(HostService.class, ComputeService.class);
        bindTop(HmcService.class, ComputeService.class);
        bindTop(OsHostService.class, ComputeService.class);
        bindTop(MessageService.class, ComputeService.class);
        bindTop(OsAggregateService.class, ComputeService.class);
        bindTop(HostSeaService.class, ComputeService.class);
        bindTop(ViosGroupService.class, ComputeService.class);

        bindTop(NetworkService.class, NetworkingService.class);
        bindTop(SubnetService.class, NetworkingService.class);
        bindTop(PortService.class, NetworkingService.class);

        bindTop(BlockStorageDeviceService.class, BlockStorageService.class);
        bindTop(BlockVolumeService.class, BlockStorageService.class);
        bindTop(BlockVolumeTemplateService.class, BlockStorageService.class);
        bindTop(BlockVolumePoolService.class, BlockStorageService.class);
        bindTop(FabircService.class, BlockStorageService.class);

        bindTop(PowervcImageService.class, ImageService.class);

        bindTop(PowervcValidateService.class, ValidateService.class);

        bindTop(NotificationService.class, WebEventService.class);
    }

    private void bind(Class<?> api, PowervcConstant.Version version, Class<?> impl) {
        Map<PowervcConstant.Version, Class<?>> implMap = serviceBindings.get(api);
        if (implMap == null) {
            implMap = new ConcurrentHashMap<>();
        }
        implMap.put(version, impl);
        serviceBindings.put(api, implMap);
    }

    private void bindTop(Class<?> api, Class<?> topApi) {
        topServices.put(api, topApi);
    }

}
