package com.bocloud.cmp.provider.storage;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * PowerVC存储卷驱动
 *
 * @author sxp
 * @since Apr 4, 2018
 */
@Provider("bocloud.cmp.powervc")
public class PowervcVolumeProvider extends PowervcProvider implements VolumeProvider {
    private static Logger logger = LoggerFactory.getLogger(PowervcVolumeProvider.class);

    public PowervcVolumeProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult list() {
        List<VolumeModel> volumes = listVolume();
        return new BocloudResult(true, volumes, "查询成功");
    }

    @Override
    public BocloudResult create(VolumeModel volume) {
        BocloudResult result = this.getClient().blockStorage(getVersion()).volumes(getVersion()).create(volume);
        if (result.isFailed()) {
            return result;
        }
        VolumeModel volumeModel = (VolumeModel) result.getData();
        while (true) {
            volumeModel = this.getClient().blockStorage(getVersion()).volumes(getVersion()).get(volumeModel.getVolumeUuid());
            if (volumeModel.getStatus().equals("creating")) {
                logger.info("卷[" + volumeModel.getVolumeUuid() + "]正在创建中，3秒后将再次确认");
                try {
                    Thread.sleep(3000);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    logger.info("卷创建异常", e);
                }
                continue;
            }
            if (volumeModel.getStatus().equals("error")) {
                return new BocloudResult(false, volumeModel, JSON.parseObject(volumeModel.getProps()).getString("errorMessage"));
            }
            return new BocloudResult(true, volumeModel, result.getMessage());
        }
    }

    @Override
    public BocloudResult modify(VolumeModel volume) {
        VolumeModel volumeOld = this.getClient().blockStorage(getVersion()).volumes(getVersion()).get(volume.getVolumeUuid());
        BocloudResult bocloudResult = null;
        if (!volumeOld.getSize().equals(volume.getSize())) {
            bocloudResult = this.getClient().blockStorage(getVersion()).volumes(getVersion()).extend(volume.getVolumeUuid(), volume.getSize());
            if (bocloudResult.isFailed()) {
                // 重新查询一下卷详情以便获取真正的错误信息
                VolumeModel volumeAfterExtend = this.getClient().blockStorage(getVersion()).volumes(getVersion()).get(volume.getVolumeUuid());
                JSONObject props = JSON.parseObject(volumeAfterExtend.getProps());
                return new BocloudResult(false, volumeAfterExtend, props.containsKey("errorMessage") ? props.getString("errorMessage") : bocloudResult.getMessage());
            }
        }
        if (!volumeOld.getMultiattach().equals(volume.getMultiattach())) {
            bocloudResult = this.getClient().blockStorage(getVersion()).volumes(getVersion()).modify(volume);
        }
        return bocloudResult == null ? new BocloudResult(true, volumeOld, "修改成功") : bocloudResult;
    }

    @Override
    public BocloudResult remove(String volumeId) {
        BocloudResult result = this.getClient().blockStorage(getVersion()).volumes(getVersion()).delete(volumeId);
        if (result.isFailed()) {
            return result;
        }
        while (true) {
            VolumeModel detailVolume = this.getClient().blockStorage(getVersion()).volumes(getVersion()).get(volumeId);
            if (detailVolume == null || !detailVolume.getStatus().equals("deleting")) {
                return new BocloudResult(true, result.getMessage());
            }
            logger.info("卷[" + volumeId + "]正在删除中，1秒后将再次确认");
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                logger.info("卷删除异常", e);
            }
        }
    }

    @Override
    public BocloudResult detail(String volumeId) {
        VolumeModel model = this.getClient().blockStorage(getVersion()).volumes(getVersion()).get(volumeId);
        return new BocloudResult(true, model, "查询成功");
    }

    @Override
    public BocloudResult setMetadata(String volumeId, VolumeMetadataModel metadata) {
        return this.getClient().blockStorage(getVersion()).volumes(getVersion()).setMetadata(volumeId, metadata);
    }

    @Override
    public BocloudResult getMetadata(String volumeId) {
        return this.getClient().blockStorage(getVersion()).volumes(getVersion()).getMetadata(volumeId);
    }

    @Override
    public BocloudResult onboard(String storageName, List<String> volumeIds) {
        return this.getClient().blockStorage(getVersion()).volumes(getVersion()).onboard(storageName, volumeIds);
    }

    @Override
    public BocloudResult unmanage(String storageName, List<String> volumeIds) {
        return this.getClient().blockStorage(getVersion()).volumes(getVersion()).unmanage(storageName, volumeIds);
    }

    @Override
    public BocloudResult allVolumes(String storageName) {
        return this.getClient().blockStorage(getVersion()).volumes(getVersion()).allVolumes(storageName);
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        return new BocloudResult(false, "PowerVC不支持该接口");
    }

    @Override
    public BocloudResult attach(String serverId, String volumeId) {
        return new BocloudResult(false, "PowerVC不支持该接口");
    }

    @Override
    public BocloudResult attach(String serverId, String mountpoint, String volumeId) {
        return new BocloudResult(false, "PowerVC不支持该接口");
    }

    @Override
    public BocloudResult detach(String serverId, String volumeId) {
        return new BocloudResult(false, "PowerVC不支持该接口");
    }

    @Override
    public BocloudResult resetStatus(String volumeId, String status) {
        return new BocloudResult(false, "PowerVC不支持该接口");
    }

    @Override
    public BocloudResult uploadToImage(String volumeId, ImageModel imageModel) {
        return new BocloudResult(false, "PowerVC不支持该接口");
    }

    @Override
    public BocloudResult extend(String volumeId, Integer newSize) {
        return new BocloudResult(false, "PowerVC不支持该接口");
    }

    @Override
    public BocloudResult list(String hostId, String storagePoolName) {
        return new BocloudResult(false, "PowerVC不支持该接口");
    }

    @Override
    public BocloudResult remove(H3cVolumeModel volume) {
        return new BocloudResult(false, "PowerVC不支持该接口");
    }

}
