package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.pqcloudconvertor.ServerConvertor;
import com.bocloud.cmp.provider.PqcloudRestApiProvider;
import com.bocloud.cmp.provider.common.PqcloudJobProvider;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: QcloudServerProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.pqcloud")
public class PqcloudServerProvider extends PqcloudRestApiProvider implements ServerProvider {

    private static Logger logger = LoggerFactory.getLogger(PqcloudServerProvider.class);

    private BeanJsonConvertor<ServerModel> convertor = new ServerConvertor();

    private Butler butler;

    public PqcloudServerProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult listByZone(String zone) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        JSONObject object = this.connection("DescribeInstances", zone, params);
        if (object.getInteger("ret_code") == 0) {
            List<ServerModel> serverModels = convertor.convertList(object.getJSONArray("instance_set"));
            return new BocloudResult(true, serverModels, "查询成功");
        } else {
            logger.error("list server error ： " + object.getString("message"));
            return new BocloudResult(false, "查询失败", object.getString("message"));
        }
    }

    @Override
    public BocloudResult create(ServerModel server) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("image_id", server.getImageUuid());
        params.put("instance_type", server.getFlavorUuid());
        params.put("os_disk_size", server.getSystemDiskSize());
        params.put("instance_name", server.getName());
        params.put("zone", server.getZone());
        /**
         * 指定登录方式。当为 linux 主机时，有效值为 keypair 和 passwd; 当为 windows 主机时，只能选用
         * passwd 登录方式。 当登录方式为 keypair 时，需要指定 login_keypair 参数； 当登录方式为
         * passwd 时，需要指定 login_passwd 参数。
         */
        if (StringUtils.isEmpty(server.getKeypairName())) {
            params.put("login_mode", "passwd");
            params.put("login_passwd", server.getPassword());
        } else {
            params.put("login_mode", "keypair");
            params.put("login_keypair", server.getKeypairName());
        }
        // 网络 "vxnet-f8b2mgr|192.168.45.12"
        List<NetworkBean> networkBeans = server.getNetworkBeans();
        for (int i = 0; i < networkBeans.size(); i++) {
            String vxnet = networkBeans.get(i).getNetworkUuid();
            if (StringUtils.isNotEmpty(networkBeans.get(i).getFixIp())) {
                vxnet += ("|" + networkBeans.get(i).getFixIp());
            }
            params.put("vxnets." + i, vxnet);
        }
        List<String> securityGroups = server.getSecurityGroups();
        if (null != securityGroups && securityGroups.size() > 0) {
            params.put("security_group", securityGroups.get(0));
        }

        JSONObject object = this.connection("RunInstances", params);
        if (object.getInteger("ret_code") == 0) {
            JSONArray instances = object.getJSONArray("instances");
            return getInstance(instances.get(0).toString());
        } else {
            logger.error("create image error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    public BocloudResult getInstance(String instanceId) {
        BocloudResult detailResult = null;
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                Thread.sleep(6000);
                BocloudResult detail = this.detail(instanceId);
                ServerModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), ServerModel.class);
                if (model == null) {
                    flag = false;
                    break;
                }
                if (("RUNNING".equalsIgnoreCase(model.getStatus()))) {
                    flag = false;
                    detailResult = this.detail(instanceId);
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 5 * 60 * 1000) {
                    flag = false;
                    detailResult = this.detail(instanceId);
                    break;
                }
            }
            return detailResult;
        } catch (Exception e) {
            logger.error("查询状态失败： " + e);
            return new BocloudResult(false, "查询状态失败", e.getMessage());
        }
    }


    @Override
    public BocloudResult modify(ServerModel server) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("instance", server.getInstanceId());
        params.put("instance_name", server.getName());
        params.put("description", server.getRemark());
        JSONObject object = this.connection("ModifyInstanceAttributes", params);
        if (object.getInteger("ret_code") == 0) {
            return new BocloudResult(true, "instance modify success!");
        } else {
            logger.error("instance modify error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult migrate(ServerModel server) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult migrate(OSMigrateModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult remove(String instanceId, String name) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("instances.0", instanceId);
        JSONObject object = this.connection("TerminateInstances", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "instance remove fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "instance remove success!");
        } else {
            logger.error("instance remove error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult detail(String instanceId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("instances.0", instanceId);
        JSONObject object = this.connection("DescribeInstances", params);
        if (object.getInteger("ret_code") == 0) {
            List<ServerModel> serverModels = convertor.convertList(object.getJSONArray("instance_set"));
            return new BocloudResult(true, serverModels.get(0), "instance detail success!");
        } else {
            logger.error("instance detail error ：" + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    public boolean waitJob(String jobId) {
        PqcloudJobProvider provider = new PqcloudJobProvider(this.getButler());
        return provider.waitJob(jobId);
    }


    @Override
    public BocloudResult start(String instanceId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("instances.0", instanceId);
        JSONObject object = this.connection("StartInstances", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "instance start fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "instance start success!");
        } else {
            logger.error("instance start error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult stop(String instanceId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("instances.0", instanceId);
        JSONObject object = this.connection("StopInstances", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "instance stop fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "instance stop success!");
        } else {
            logger.error("instance stop error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult reboot(String instanceId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("instances.0", instanceId);
        JSONObject object = this.connection("RestartInstances", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "instance restart fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "instance restart success!");
        } else {
            logger.error("instance restart error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult softReboot(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult pause(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult unPause(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult suspend(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult resume(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult joinSGroup(String instanceId, String group) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("instances.0", instanceId);
        params.put("instance_group", group);
        JSONObject object = this.connection("JoinInstanceGroup", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "instance joinSGroup fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "instance joinSGroup success!");
        } else {
            logger.error("instance joinSGroup error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult leaveSGroup(String instanceId, String group) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("instances.0", instanceId);
        params.put("instance_group", group);
        JSONObject object = this.connection("LeaveInstanceGroup", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "instance leaveSGroup fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "instance leaveSGroup success!");
        } else {
            logger.error("instance leaveSGroup error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult vncConsole(String vncType, String serverId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult changePassword(String instanceId, String password) {
        return null;
    }

    @Override
    public BocloudResult attachNetwork(ServerModel server) {
        return null;
    }

    @Override
    public BocloudResult detachNetwork(String instanceId, String portUuid) {
        return null;
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("instances.0", instanceId);
        params.put("instance_type", flavorId);
        JSONObject object = this.connection("ResizeInstances", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "instance resize fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "instance resize success!");
        } else {
            logger.error("instance resize error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
        return null;
    }

    @Override
    public BocloudResult confirmResize(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult revertResize(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean MemHotEnable) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getTicket(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult createInResPool(ServerModel server) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult mountToolsInstaller(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult unmountToolsInstaller(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult upgradeToolsTask(String instanceId, String options) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getNetCard(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getVmIps(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult markAsTemplate(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
        return null;
    }

    /**
     * 云主机磁盘扩容
     */
    @Override
    public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
        return null;
    }

    @Override
    public BocloudResult getMetric(String type) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult vmrcConsole(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult migrate(H3cMigrateModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult list(String hostId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult create(H3cServerModel arg0) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getJnlpXml(Long arg0, String arg1) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult resetInstance(ServerModel serverModel) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("instances.0", serverModel.getInstanceId());
        if (StringUtils.isEmpty(serverModel.getKeypairName())) {
            params.put("login_mode", "passwd");
            params.put("login_passwd", serverModel.getPassword());
        } else {
            params.put("login_mode", "keypair");
            params.put("login_keypair", serverModel.getKeypairName());
        }
        JSONObject object = this.connection("ResetInstances", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "instance reset fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "instance reset success!");
        } else {
            logger.error("instance reset error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }


    @Override
    public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
        return null;
    }

    @Override
    public BocloudResult bindKeypair(ServerModel serverModel) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("keypairs.0", serverModel.getKeypairIds()[0]);
        params.put("instances.0", serverModel.getInstanceIds()[0]);
        JSONObject object = this.connection("AttachKeyPairs", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "instance bindKeypair fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "instance bindKeypair success!");
        } else {
            logger.error("instance bindKeypair error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult unbindKeypair(ServerModel serverModel) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("keypairs.0", serverModel.getKeypairIds()[0]);
        params.put("instances.0", serverModel.getInstanceIds()[0]);
        JSONObject object = this.connection("DetachKeyPairs", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "instance unbindKeypair fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "instance unbindKeypair success!");
        } else {
            logger.error("instance unbindKeypair error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }


    @Override
    public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
        // TODO Auto-generated method stub
        return null;
    }

}