package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.QcloudProvider;
import com.bocloud.cmp.provider.compute.QcloudImageProvider;
import com.bocloud.cmp.provider.compute.QcloudServerProvider;
import com.bocloud.cmp.provider.compute.QcloudSnapshotProvider;
import com.bocloud.cmp.provider.network.*;
import com.bocloud.cmp.provider.qcloudconvertor.ZoneConvertor;
import com.bocloud.cmp.provider.security.QcloudKeypairProvider;
import com.bocloud.cmp.provider.storage.QcloudVolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.qingcloud.sdk.exception.QCException;
import com.qingcloud.sdk.service.QingCloudService;
import com.qingcloud.sdk.service.Types.ZoneModel;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 同步
 *
 * @author ymh
 * @时间 2019年10月23日
 */
@Provider("bocloud.cmp.qcloud")
public class QcloudLocationProvider extends QcloudProvider implements LocationProvider {

    private static Logger logger = LoggerFactory.getLogger(QcloudLocationProvider.class);
    private Butler butler;

    public QcloudLocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult listRegions() {
        List<RegionModel> models = new ArrayList<>();
        // 北京3区
        RegionModel pekModel = new RegionModel();
        pekModel.setCode("pek3");
        pekModel.setRegionId("pek3");
        pekModel.setName("北京3区");
        models.add(pekModel);
        // 广东2区
        RegionModel gdModel = new RegionModel();
        gdModel.setCode("gd2");
        gdModel.setRegionId("gd2");
        gdModel.setName("广东2区");
        models.add(gdModel);
        // 上海1区
        RegionModel shModel = new RegionModel();
        shModel.setCode("sh1");
        shModel.setRegionId("sh1");
        shModel.setName("上海1区");
        models.add(shModel);
        // 亚太2区
        RegionModel apModel = new RegionModel();
        apModel.setCode("ap2");
        apModel.setRegionId("ap2");
        apModel.setName("亚太2区");
        models.add(apModel);
        return new BocloudResult(true, models, "查询成功！");
    }

    @SuppressWarnings("unchecked")
    private RegionModel getRegion(String region) {
        BocloudResult listRegions = this.listRegions();
        if (listRegions.isFailed()) {
            return null;
        }
        List<RegionModel> models = (List<RegionModel>) listRegions.getData();
        for (RegionModel regionModel : models) {
            if (region.equalsIgnoreCase(regionModel.getRegionId())) {
                return regionModel;
            }
        }
        return null;
    }

    /**
     * 查询所有可用域信息
     *
     * @return
     */
    @SuppressWarnings("unused")
    private List<com.bocloud.cmp.model.ZoneModel> zones() {
        try {
            QingCloudService service = new QingCloudService(this.getContext());
            QingCloudService.DescribeZonesInput input = new QingCloudService.DescribeZonesInput();
            QingCloudService.DescribeZonesOutput output = service.describeZones(input);
            List<ZoneModel> zoneModels = output.getZoneSet();
            if (ListTool.isEmpty(zoneModels)) {
                return new ZoneConvertor().convertList(zoneModels);
            }
            return new ArrayList<>();
        } catch (QCException e) {
            logger.error("Get error message：{}", e);
            return new ArrayList<>();
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    public BocloudResult sync() {
        QcloudSyncModel model = new QcloudSyncModel();

        QcloudServerProvider serverProvider = new QcloudServerProvider(butler);
        QcloudSubnetProvider subnetProvider = new QcloudSubnetProvider(butler);
        String region = butler.getRegionId();
        if(StringUtils.isEmpty(region)) {
        	region = "pek3";
        }
        // 只获取当前可以区下的资源
        RegionModel regionModel = getRegion(region);
        if (null == regionModel) {
            logger.error("所选区域不存在！同步失败！");
            return new BocloudResult(false, "所选区域不存在！同步失败！");
        }
        // 获取region 下的zone
        BocloudResult listZones = listZones(region);
        if (listZones.isSuccess()) {
            List<com.bocloud.cmp.model.ZoneModel> zoneSet = (List<com.bocloud.cmp.model.ZoneModel>) listZones.getData();
            regionModel.setZones(zoneSet);

            List<SubnetModel> subnetModels = new ArrayList<>();
            List<VolumeModel> volumeModels = new ArrayList<>();
            List<ServerModel> serverModels = new ArrayList<>();
            List<FlavorModel> flavorModels = new ArrayList<>();

            QcloudVolumeProvider volumeProvider = new QcloudVolumeProvider(butler);

            for (com.bocloud.cmp.model.ZoneModel zoneModel : zoneSet) {
                // 同步子网数据
                BocloudResult subnetResult = subnetProvider.list(zoneModel.getZoneId());
                if (subnetResult.isSuccess()) {
                    subnetModels.addAll((List<SubnetModel>) subnetResult.getData());
                }
                // 同步云硬盘
                BocloudResult volumeResult = volumeProvider.list(zoneModel.getZoneId());
                if (volumeResult.isSuccess()) {
                    volumeModels.addAll((List<VolumeModel>) volumeResult.getData());
                }
                // 同步云主机
                BocloudResult serverResult = serverProvider.listByZone(zoneModel.getZoneId());
                if (serverResult.isSuccess()) {
                    serverModels.addAll((List<ServerModel>) serverResult.getData());
                }
                // 同步规格
                QcloudFlavorProvider flavorProvider = new QcloudFlavorProvider(butler);
                BocloudResult flavorResult = flavorProvider.list(zoneModel.getZoneId());
                if (flavorResult.isSuccess()) {
                    flavorModels.addAll((List<FlavorModel>) flavorResult.getData());
                }
            }
            model.setSubnetModels(subnetModels);
            model.setVolumeModels(volumeModels);
            model.setServerModels(serverModels);
            model.setFlavorModels(flavorModels);
        }
        model.setRegionModels(Arrays.asList(new RegionModel[] { regionModel }));
        // 同步VPC
        QcloudVpcProvider vpcProvider = new QcloudVpcProvider(butler);
        BocloudResult vpcResult = vpcProvider.list();
        if (vpcResult.isSuccess()) {
            model.setVpcModels((List<VpcModel>) vpcResult.getData());
        } else {
            model.setVpcModels(new ArrayList<>());
        }
        // 同步快照
        QcloudSnapshotProvider snapshotProvider = new QcloudSnapshotProvider(butler);
        BocloudResult snapshotResult = snapshotProvider.list();
        if (snapshotResult.isSuccess()) {
            model.setSnapshotModels((List<SnapshotModel>) snapshotResult.getData());
        } else {
            model.setSnapshotModels(new ArrayList<>());
        }
        // 同步镜像
        QcloudImageProvider imageProvider = new QcloudImageProvider(butler);
        BocloudResult imageResult = imageProvider.list();
        if (imageResult.isSuccess()) {
            model.setImageModels((List<ImageModel>) imageResult.getData());
        } else {
            model.setImageModels(new ArrayList<>());
        }
        // 同步安全组 and 安全组规则
        QcloudSecuGroupProvider secuGroupProvider = new QcloudSecuGroupProvider(butler);
        QcloudSecuGroupRuleProvider secuGroupRuleProvider = new QcloudSecuGroupRuleProvider(butler);
        BocloudResult groupResult = secuGroupProvider.list();
        if (groupResult.isSuccess()) {
            List<SecurityGroupModel> groups = (List<SecurityGroupModel>) groupResult.getData();
            if (null != groups && groups.size() > 0) {
                for (SecurityGroupModel securityGroupModel : groups) {
                    BocloudResult groupRuleResult = secuGroupRuleProvider.list(securityGroupModel.getGroupUuid());
                    if (groupRuleResult.isSuccess()) {
                        securityGroupModel.setModels((List<SecurityGroupRuleModel>) groupRuleResult.getData());
                    } else {
                        securityGroupModel.setModels(new ArrayList<>());
                    }
                }
            }
            model.setGroupModels(groups);
        } else {
            model.setGroupModels(new ArrayList<>());
        }
        // 同步秘钥
        QcloudKeypairProvider keypairProvider = new QcloudKeypairProvider(butler);
        BocloudResult keypairResult = keypairProvider.list();
        if (keypairResult.isSuccess()) {
            model.setKeypairModels((List<KeypairModel>) keypairResult.getData());
        } else {
            model.setKeypairModels(new ArrayList<>());
        }
        // 同步浮动IP
        QcloudFloatingIpProvider floatingIpProvider = new QcloudFloatingIpProvider(butler);
        BocloudResult fipResult = floatingIpProvider.list();
        if (fipResult.isSuccess()) {
            model.setFloatingIpModels((List<FloatingIpModel>) fipResult.getData());
        } else {
            model.setFloatingIpModels(new ArrayList<>());
        }
        return new BocloudResult(true, model, "success");
    }

    @Override
    public BocloudResult listZones(String region) {
        if (StringUtils.isEmpty(region)) {
            return new BocloudResult(false, "当前区域下不存在可用域！");
        }
        List<com.bocloud.cmp.model.ZoneModel> zoneModes = new ArrayList<>();
        QingCloudService service = new QingCloudService(this.getContext());
        QingCloudService.DescribeZonesInput input = new QingCloudService.DescribeZonesInput();
        try {
            RegionModel regionModel = getRegion(region);
            if (null == regionModel) {
                return new BocloudResult(false, "当前区域下不存在可用域！");
            }
            QingCloudService.DescribeZonesOutput output = service.describeZones(input);
            List<ZoneModel> zoneSet = output.getZoneSet();
            if (null == zoneSet) {
                zoneSet = new ArrayList<ZoneModel>();
            }
            com.bocloud.cmp.model.ZoneModel zoneModel = null;
            for (com.qingcloud.sdk.service.Types.ZoneModel model : zoneSet) {
                zoneModel = new com.bocloud.cmp.model.ZoneModel();
                if ("active".equalsIgnoreCase(model.getStatus())
                        && model.getZoneID().startsWith(region.toLowerCase())) {
                    String zoneID = model.getZoneID();
                    zoneModel.setRegionId(region);
                    zoneModel.setZoneId(zoneID);
                    // 名称 regionname + 最后字符变大写
                    zoneModel.setName(regionModel.getName() + "-"
                            + zoneID.substring(zoneID.length() - 1, zoneID.length()).toUpperCase());
                    zoneModes.add(zoneModel);
                }
            }
            return new BocloudResult(true, zoneModes, "获取可用域成功");
        } catch (QCException e) {
            logger.error("获取可用域失败：{}", e);
            return new BocloudResult(false, "获取可用域失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult syncPortByNetworkUuid(String networkUuid) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getHostModel(HostModel host) {
        // TODO Auto-generated method stub
        return null;
    }

}
