package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.QcloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.qcloudconvertor.ImageConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.qingcloud.sdk.exception.QCException;
import com.qingcloud.sdk.service.ImageService;
import com.qingcloud.sdk.service.Types;
import com.qingcloud.sdk.service.ImageService.CaptureInstanceInput;
import com.qingcloud.sdk.service.ImageService.CaptureInstanceOutput;
import com.qingcloud.sdk.service.ImageService.DescribeImagesOutput;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 镜像
 *
 * @时间 2019年9月25日
 */
@Provider("bocloud.cmp.qcloud")
public class QcloudImageProvider extends QcloudProvider implements ImageProvider {

    private static Logger logger = LoggerFactory.getLogger(QcloudImageProvider.class);
    private BeanConvertor<ImageModel, Types.ImageModel> convertor = new ImageConvertor();
    private ImageService service = new ImageService(this.getContext());
    private static final int PAGE_SIZE = 100;

    public QcloudImageProvider(Butler butler) {
        super(butler);
    }

    @Override
    public boolean accessable() {
        return super.accessable();
    }

    @Override
    public BocloudResult list() { // self
        try {
            List<ImageModel> models = new ArrayList<>();
            // 系统镜像
            models.addAll(listImages("system"));
            // 自定义镜像
            models.addAll(listImages("self"));
            return new BocloudResult(true, models, "查询列表成功！");
        } catch (Exception e) {
            logger.error("查询镜像列表异常：" + e.getMessage());
            return new BocloudResult(false, "查询镜像列表异常：" + e.getMessage());
        }
    }

    private List<ImageModel> listImages(String provider) {
        List<ImageModel> models = new ArrayList<>();
        ImageService.DescribeImagesInput input = new ImageService.DescribeImagesInput();
        input.setProvider(provider);
        input.setOffset(0);
        input.setLimit(PAGE_SIZE);
        try {
            ImageService.DescribeImagesOutput output = service.describeImages(input);
            if (null == output || null == output.getTotalCount()) {
                return models;
            }
            int totalCount = output.getTotalCount();
            if (null != output.getImageSet() && !ListTool.isEmpty(output.getImageSet())) {
                models.addAll(convertor.convertList(output.getImageSet()));
            }
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1) {
                return models;
            }
            for (int i = 2; i <= totalPage; i++) {
                input.setOffset((i - 1) * PAGE_SIZE);
                input.setLimit(PAGE_SIZE);
                input.setProvider(provider);
                output = service.describeImages(input);
                if (null != output.getImageSet() && !ListTool.isEmpty(output.getImageSet())) {
                    models.addAll(convertor.convertList(output.getImageSet()));
                }
            }
            for (ImageModel imageModel : models) {
                imageModel.setRegionId(this.getRegionId());
            }
        } catch (Exception e) {
            logger.error("list image error:", e);
        }
        return models;
    }

    @Override
    public BocloudResult detail(String imageId) {
        try {
            ImageService.DescribeImagesInput input = new ImageService.DescribeImagesInput();
            List<String> images = new ArrayList<>();
            images.add(imageId);
            input.setImages(images);
            ImageService.DescribeImagesOutput output = service.describeImages(input);
            if (ListTool.isEmpty(output.getImageSet())) {
                return new BocloudResult(false, "查询失败，镜像不存在");
            }
            return new BocloudResult(true, convertor.convertModel(output.getImageSet().get(0)), "查询成功");
        } catch (QCException e) {
            logger.error(e.getMessage());
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult remove(String imageId) {
        try {
            ImageService.DeleteImagesInput input = new ImageService.DeleteImagesInput();
            List<String> images = new ArrayList<>();
            images.add(imageId);
            input.setImages(images);
            ImageService.DeleteImagesOutput output = service.deleteImages(input);
            if (!this.waitJob(output.getJobID())) {
                return new BocloudResult(false, "镜像删除失败：" + output.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (QCException e) {
            logger.error(e.getMessage());
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult modify(ImageModel image) {
        try {
            ImageService.ModifyImageAttributesInput input = new ImageService.ModifyImageAttributesInput();
            input.setImage(image.getImageUuid());
            input.setImageName(image.getName());
            input.setDescription(image.getRemark());
            ImageService.ModifyImageAttributesOutput output = service.modifyImageAttributes(input);
            if (0 != output.getRetCode()) {
                return new BocloudResult(false, "镜像修改失败：" + output.getMessage());
            }
            return new BocloudResult(true, "修改成功");
        } catch (QCException e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    @Override
    public BocloudResult create(ImageModel imageModel) {
        try {
            ImageService service = new ImageService(this.getContext());
            CaptureInstanceInput input = new CaptureInstanceInput();
            input.setImageName(imageModel.getName());
            input.setInstance(imageModel.getInstanceId());
            CaptureInstanceOutput captureInstance = service.captureInstance(input);
            if (0 == captureInstance.getRetCode()) {
                BocloudResult imageDetail = imageDetail(captureInstance.getImageID());
                return new BocloudResult(imageDetail.isSuccess(), imageDetail.getData(), "镜像创建成功！");
            } else {
                return new BocloudResult(false, captureInstance.getMessage());
            }
        } catch (Exception e) {
            logger.error("查询异常：" + e.getMessage());
            return new BocloudResult(false, "查询异常：" + e.getMessage());
        }
    }

    private BocloudResult imageDetail(String imageId) {
        ImageService service = new ImageService(this.getContext());
        ImageService.DescribeImagesInput input = new ImageService.DescribeImagesInput();
        input.setImages(Arrays.asList(new String[] { imageId }));
        BocloudResult bocloudResult = new BocloudResult(false, "查询失败！");
        try {
            boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    logger.error("Get error message:", e);
                    Thread.currentThread().interrupt();
                }
                DescribeImagesOutput describeImages = service.describeImages(input);
                List<com.qingcloud.sdk.service.Types.ImageModel> imageSet = describeImages.getImageSet();
                if (null == imageSet || imageSet.size() == 0) {
                    flag = false;
                    break;
                }
                ImageModel convertModel = convertor.convertModel(imageSet.get(0));
                if ("ACTIVE".equalsIgnoreCase(convertModel.getStatus())) {
                    flag = true;
                    convertModel.setRegionId(this.getRegionId());
                    bocloudResult = new BocloudResult(true, convertModel, "查询详情成功！");
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待10分钟
                if (endTime - startTime > 10 * 60 * 1000) {
                    flag = false;
                    bocloudResult.setMessage("查询超时！");
                    break;
                }
            }
        } catch (QCException e) {
            logger.error(e.getMessage());
        }
        return bocloudResult;
    }

    @Override
    public BocloudResult list(String zone) {
        List<ImageModel> models = new ArrayList<>();
        ImageService.DescribeImagesInput input = new ImageService.DescribeImagesInput();
        input.setZone(zone);
        input.setOffset(1);
        input.setLimit(10000);
        try {
            ImageService.DescribeImagesOutput output = service.describeImages(input);
            int totalCount = output.getTotalCount();
            if (null != output.getImageSet() && !ListTool.isEmpty(output.getImageSet())) {
                models.addAll(convertor.convertList(output.getImageSet()));
            }
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1) {
                return new BocloudResult(true, models, "success");
            }
            for (int i = 2; i <= totalPage; i++) {
                input.setOffset(i);
                input.setLimit(10000);
                input.setZone(zone);
                output = service.describeImages(input);
                if (null != output.getImageSet() && !ListTool.isEmpty(output.getImageSet())) {
                    models.addAll(convertor.convertList(output.getImageSet()));
                }
            }
            return new BocloudResult(true, models, "查询列表成功！");
        } catch (Exception e) {
            logger.error("list image error:", e);
            return new BocloudResult(false, "list image error!");
        }

    }

}
