package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FloatingIpModel;
import com.bocloud.cmp.provider.QcloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.qcloudconvertor.FloatingIpConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.qingcloud.sdk.exception.QCException;
import com.qingcloud.sdk.service.EIPService;
import com.qingcloud.sdk.service.Types;
import com.qingcloud.sdk.service.Types.EIPModel;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

/**
 * 青云公网IP操作类
 *
 * @author wangyu
 * @时间 2019年10月24日
 */
@Provider("bocloud.cmp.qcloud")
public class QcloudFloatingIpProvider extends QcloudProvider implements FloatingIPProvider {

	public QcloudFloatingIpProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(QcloudFloatingIpProvider.class);
	private BeanConvertor<FloatingIpModel, Types.EIPModel> convertor = new FloatingIpConvertor();
	private EIPService service = new EIPService(this.getContext());

	@Override
	public BocloudResult list() {
		try {
			EIPService.DescribeEIPsInput input = new EIPService.DescribeEIPsInput();
			EIPService.DescribeEIPsOutput output = service.describeEIPs(input);
			List<EIPModel> eipSet = output.getEIPSet();
			if (null == eipSet) {
				eipSet = new ArrayList<EIPModel>();
			}
			List<FloatingIpModel> convertList = convertor.convertList(eipSet);
			for (FloatingIpModel floatingIpModel : convertList) {
				floatingIpModel.setRegionId(this.getRegionId());
			}
			return new BocloudResult(true, convertList, "查询成功");
		} catch (QCException e) {
			logger.error("查询列表异常：{}", e);
			return new BocloudResult(false, "查询失败 :" + e.getMessage());
		}
	}

	@Override
	public BocloudResult create(FloatingIpModel model) {
		try {
			EIPService.AllocateEIPsInput input = new EIPService.AllocateEIPsInput();
			input.setCount(1);
			String name = model.getName();
			if (StringUtils.isEmpty(name)) {
				name = UUID.randomUUID().toString().replaceAll("-", "");
			}
			input.setEIPName(name);
			String bandwidth = model.getBandwidth();
			if (StringUtils.isEmpty(bandwidth)) {
				bandwidth = "10";
			}
			input.setBandwidth(Integer.parseInt(bandwidth));
			// bandwidth按带宽计费,traffic按流量计费
			input.setBillingMode("traffic");
			// 0不需要,1需要
			input.setNeedICP(0);
			EIPService.AllocateEIPsOutput output = service.allocateEIPs(input);
			if (ListTool.isEmpty(output.getEIPs())) {
				return new BocloudResult(false, "创建失败：" + output.getMessage());
			}
			Thread.sleep(100000);
			BocloudResult detail = this.detail(output.getEIPs().get(0));
			if (detail.isFailed()) {
				return detail;
			}
			List<FloatingIpModel> models = new ArrayList<>();
			FloatingIpModel resultModel = (FloatingIpModel) detail.getData();
			models.add(resultModel);
			return new BocloudResult(true, models, "创建成功");
		} catch (Exception e) {
			logger.error("创建失败：{}", e);
			return new BocloudResult(false, "创建失败 :" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String fipId) {
		try {
			EIPService.ReleaseEIPsInput input = new EIPService.ReleaseEIPsInput();
			input.setEIPs(Arrays.asList(new String[] { fipId }));
			EIPService.ReleaseEIPsOutput output = service.releaseEIPs(input);
			String jobID = output.getJobID();
			if (StringUtils.isEmpty(jobID)) {
				return new BocloudResult(false, "当前浮动IP状态不允许删除！");
			}
			if (!this.waitJob(jobID)) {
				return new BocloudResult(false, "删除失败：" + output.getMessage());
			}
			return new BocloudResult(true, "删除成功");
		} catch (QCException e) {
			logger.error("删除失败：{}", e);
			return new BocloudResult(false, "删除失败 : " + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String fipId) {
		try {
			EIPService.DescribeEIPsInput input = new EIPService.DescribeEIPsInput();
			List<String> eips = new ArrayList<>();
			eips.add(fipId);
			input.setEIPs(eips);
			EIPService.DescribeEIPsOutput output = service.describeEIPs(input);
			if (ListTool.isEmpty(output.getEIPSet())) {
				return new BocloudResult(false, "查询失败：" + output.getMessage());
			}
			return new BocloudResult(true, convertor.convertModel(output.getEIPSet().get(0)), "查询成功");
		} catch (QCException e) {
			logger.error("查询详情异常：{}", e);
			return new BocloudResult(false, "查询失败 : " + e.getMessage());
		}
	}

	@Override
	public BocloudResult associate(FloatingIpModel model) {
		try {
			EIPService.AssociateEIPInput input = new EIPService.AssociateEIPInput();
			input.setEIP(model.getFloatingIpUuid());
			input.setInstance(model.getInstanceId());
			EIPService.AssociateEIPOutput output = service.associateEIP(input);
			if (!this.waitJob(output.getJobID())) {
				return new BocloudResult(false, "绑定失败：" + output.getMessage());
			}
			BocloudResult bocloudResult = this.detail(model.getFloatingIpUuid());
			if (bocloudResult.isFailed()) {
				return bocloudResult;
			}
			FloatingIpModel floatingIpModel = (FloatingIpModel) bocloudResult.getData();
			floatingIpModel.setInstanceId(model.getInstanceId());
			return new BocloudResult(true, floatingIpModel, "绑定成功");
		} catch (QCException e) {
			logger.error("绑定IP异常：{}", e);
			return new BocloudResult(false, "绑定失败 :" + e.getMessage());
		}
	}

	@Override
	public BocloudResult associate(String fipId, String portId) {
		try {
			EIPService.DissociateEIPsInput input = new EIPService.DissociateEIPsInput();
			List<String> eips = new ArrayList<>();
			eips.add(fipId);
			input.setEIPs(eips);
			EIPService.DissociateEIPsOutput output = service.dissociateEIPs(input);
			if (!this.waitJob(output.getJobID())) {
				return new BocloudResult(false, "解绑失败：" + output.getMessage());
			}
			return new BocloudResult(true, this.detail(fipId).getData(), "解绑成功");
		} catch (QCException e) {
			logger.error("解绑IP异常：{}", e);
			return new BocloudResult(false, "解绑失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult disassociate(FloatingIpModel modeld) {
		try {
			EIPService.DissociateEIPsInput input = new EIPService.DissociateEIPsInput();
			List<String> eips = new ArrayList<>();
			eips.add(modeld.getFloatingIpUuid());
			input.setEIPs(eips);
			EIPService.DissociateEIPsOutput output = service.dissociateEIPs(input);
			if (!this.waitJob(output.getJobID())) {
				return new BocloudResult(false, "解绑失败：" + output.getMessage());
			}
			return new BocloudResult(true, this.detail(modeld.getFloatingIpUuid()).getData(), "解绑成功");
		} catch (QCException e) {
			logger.error("解绑IP异常：{}", e);
			return new BocloudResult(false, "解绑失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult disassociate(String fipId) {
		try {
			EIPService.DissociateEIPsInput input = new EIPService.DissociateEIPsInput();
			List<String> eips = new ArrayList<>();
			eips.add(fipId);
			input.setEIPs(eips);
			EIPService.DissociateEIPsOutput output = service.dissociateEIPs(input);
			if (!this.waitJob(output.getJobID())) {
				return new BocloudResult(false, "解绑失败：" + output.getMessage());
			}
			return new BocloudResult(true, this.detail(fipId).getData(), "解绑成功");
		} catch (QCException e) {
			logger.error("解绑IP异常：{}", e);
			return new BocloudResult(false, "解绑失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult list(String zone) {
		try {
			EIPService.DescribeEIPsInput input = new EIPService.DescribeEIPsInput();
			input.setZone(zone);
			EIPService.DescribeEIPsOutput output = service.describeEIPs(input);
			if (ListTool.isEmpty(output.getEIPSet())) {
				return new BocloudResult(false, "查询失败：" + output.getMessage());
			}
			return new BocloudResult(true, convertor.convertList(output.getEIPSet()), "查询成功");
		} catch (QCException e) {
			logger.error("查询列表异常：{}", e);
			return new BocloudResult(false, "查询失败:" + e.getMessage());
		}
	}
}
