package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.provider.QcloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.qcloudconvertor.SecurityGroupConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.qingcloud.sdk.exception.QCException;
import com.qingcloud.sdk.service.SecurityGroupService;
import com.qingcloud.sdk.service.Types;

/**
 * 安全组
 *
 * @时间 2019年9月24日
 */
@Provider("bocloud.cmp.qcloud")
public class QcloudSecuGroupProvider extends QcloudProvider implements SecurityGroupProvider {

    private static Logger logger = LoggerFactory.getLogger(QcloudSecuGroupProvider.class);
    private BeanConvertor<SecurityGroupModel, Types.SecurityGroupModel> convertor = new SecurityGroupConvertor();
    private SecurityGroupService service = new SecurityGroupService(this.getContext());

    public QcloudSecuGroupProvider(Butler butler) {
        super(butler);
    }

    @Override
    public boolean accessable() {
        return super.accessable();
    }

    @Override
    public BocloudResult list() {
        try {
            SecurityGroupService.DescribeSecurityGroupsInput input = new SecurityGroupService.DescribeSecurityGroupsInput();
            SecurityGroupService.DescribeSecurityGroupsOutput output = service.describeSecurityGroups(input);
            if (ListTool.isEmpty(output.getSecurityGroupSet())) {
                return new BocloudResult(false, "查询失败：" + output.getMessage());
            }
            return new BocloudResult(true, convertor.convertList(output.getSecurityGroupSet()), "查询成功");
        } catch (QCException e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "查询列表失败");
        }
    }

    @Override
    public BocloudResult create(SecurityGroupModel group) {
        try {
            SecurityGroupService.CreateSecurityGroupInput input = new SecurityGroupService.CreateSecurityGroupInput();
            input.setSecurityGroupName(group.getName());
            SecurityGroupService.CreateSecurityGroupOutput output = service.createSecurityGroup(input);
            if (StringUtils.isEmpty(output.getSecurityGroupID())) {
                return new BocloudResult(false, "创建失败：" + output.getMessage());
            }
            return new BocloudResult(true, this.detail(output.getSecurityGroupID()).getData(), "创建成功");
        } catch (QCException e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult remove(String sgId) {
        try {
            SecurityGroupService.DeleteSecurityGroupsInput input = new SecurityGroupService.DeleteSecurityGroupsInput();
            List<String> securityGroups = new ArrayList<>();
            securityGroups.add(sgId);
            input.setSecurityGroups(securityGroups);
            SecurityGroupService.DeleteSecurityGroupsOutput output = service.deleteSecurityGroups(input);
            if (0 != output.getRetCode()) {
                return new BocloudResult(false, "删除失败：" + output.getMessage());
            }
            return new BocloudResult(true, this.detail(sgId).getData(), "删除成功");
        } catch (QCException e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult detail(String sgId) {
        try {
            SecurityGroupService.DescribeSecurityGroupsInput input = new SecurityGroupService.DescribeSecurityGroupsInput();
            List<String> securityGroups = new ArrayList<>();
            securityGroups.add(sgId);
            input.setSecurityGroups(securityGroups);
            SecurityGroupService.DescribeSecurityGroupsOutput output = service.describeSecurityGroups(input);
            if (ListTool.isEmpty(output.getSecurityGroupSet())) {
                return new BocloudResult(false, "查询详情失败：" + output.getMessage());
            }
            SecurityGroupModel convertModel = convertor.convertModel(output.getSecurityGroupSet().get(0));
            convertModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, convertModel, "查询成功");
        } catch (QCException e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "查询详情失败");
        }
    }

    @Override
    public BocloudResult modify(SecurityGroupModel model) {
        try {
            SecurityGroupService.ModifySecurityGroupAttributesInput input = new SecurityGroupService.ModifySecurityGroupAttributesInput();
            input.setSecurityGroup(model.getGroupUuid());
            input.setSecurityGroupName(model.getName());
            input.setDescription(model.getRemark());
            SecurityGroupService.ModifySecurityGroupAttributesOutput output = service
                    .modifySecurityGroupAttributes(input);
            if (StringUtils.isEmpty(output.getSecurityGroupID())) {
                return new BocloudResult(false, "修改失败：" + output.getMessage());
            }
            return new BocloudResult(true, this.detail(output.getSecurityGroupID()).getData(), "修改成功");
        } catch (QCException e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    @Override
    public BocloudResult list(String zone) {
        try {
            SecurityGroupService.DescribeSecurityGroupsInput input = new SecurityGroupService.DescribeSecurityGroupsInput();
            input.setZone(zone);
            SecurityGroupService.DescribeSecurityGroupsOutput output = service.describeSecurityGroups(input);
            if (ListTool.isEmpty(output.getSecurityGroupSet())) {
                return new BocloudResult(false, "查询失败：" + output.getMessage());
            }
            return new BocloudResult(true, convertor.convertList(output.getSecurityGroupSet()), "查询成功");
        } catch (QCException e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "列表查询失败ByZone");
        }
    }

}
