package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.QcloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.qcloudconvertor.SecurityGroupRuleConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.qingcloud.sdk.exception.QCException;
import com.qingcloud.sdk.service.SecurityGroupService;
import com.qingcloud.sdk.service.Types;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.util.ArrayList;
import java.util.List;

/**
 * 安全组规则
 *
 * @时间 2019年10月23日
 */
@Provider("bocloud.cmp.qcloud")
public class QcloudSecuGroupRuleProvider extends QcloudProvider implements SecurityGroupRuleProvider {

    private static Logger logger = LoggerFactory.getLogger(QcloudSecuGroupRuleProvider.class);
    private BeanConvertor<SecurityGroupRuleModel, Types.SecurityGroupRuleModel> convertor = new SecurityGroupRuleConvertor();
    private SecurityGroupService service = new SecurityGroupService(this.getContext());

    public QcloudSecuGroupRuleProvider(Butler butler) {
        super(butler);
    }

    @Override
    public boolean accessable() {
        return super.accessable();
    }

    @Override
    public BocloudResult create(SecurityGroupRuleModel model) {
        try {
            SecurityGroupService.AddSecurityGroupRulesInput input = new SecurityGroupService.AddSecurityGroupRulesInput();
            input.setSecurityGroup(model.getGroupUuid());
            List<Types.SecurityGroupRuleModel> ruleModels = new ArrayList<>();
            Types.SecurityGroupRuleModel ruleModel = new Types.SecurityGroupRuleModel();
            ruleModel.setSecurityGroupRuleName(model.getName() == null ? "" : model.getName());
            if (null == model.getAction()) {
                return new BocloudResult(false, "行为不允许空！");
            }
            ruleModel.setAction(model.getAction().toLowerCase());
            ruleModel.setPriority(1);
            String direction = model.getDirection();
            if ("ingress".equalsIgnoreCase(direction)) {
                ruleModel.setDirection(0);
            }
            if ("egress".equalsIgnoreCase(direction)) {
                ruleModel.setDirection(1);
            }
            String lowerCase = model.getProtocol().toLowerCase();
            ruleModel.setProtocol(model.getProtocol().toLowerCase());
            Integer portMin = model.getPortMin();
            Integer portMax = model.getPortMax();
            // icmp特殊处理 青云ICMP 默认min 8 max 0
            if ("icmp".equalsIgnoreCase(lowerCase)) {
                portMin = 8;
                portMax = 0;
            }
            if (null == portMax || null == portMin) {
                return new BocloudResult(false, "起始端口、结束端口不允许为空！");
            }
            ruleModel.setVal2(portMax + "");
            ruleModel.setVal1(portMin + "");
            ruleModel.setVal3(model.getRemoteIpPrefix());
            ruleModel.setSecurityGroupID(model.getGroupUuid());
            ruleModels.add(ruleModel);
            input.setRules(ruleModels);
            SecurityGroupService.AddSecurityGroupRulesOutput output = service.addSecurityGroupRules(input);
            if (0 == output.getRetCode()) {
                if (ListTool.isEmpty(output.getSecurityGroupRules())) {
                    return new BocloudResult(true, "安全组添加成功！");
                }
                SecurityGroupService.DescribeSecurityGroupRulesInput rulesInput = new SecurityGroupService.DescribeSecurityGroupRulesInput();
                rulesInput.setSecurityGroupRules(output.getSecurityGroupRules());
                SecurityGroupService.DescribeSecurityGroupRulesOutput rulesOutput = service
                        .describeSecurityGroupRules(rulesInput);
                if (ListTool.isEmpty(rulesOutput.getSecurityGroupRuleSet())) {
                    return new BocloudResult(false, "创建失败：" + rulesOutput.getMessage());
                }
                SecurityGroupRuleModel groupRuleModel = convertor
                        .convertModel(rulesOutput.getSecurityGroupRuleSet().get(0));
                return new BocloudResult(true, groupRuleModel, "创建成功");
            } else {
                return new BocloudResult(false, output.getMessage());
            }
        } catch (QCException e) {
            logger.error("查询列表异常：", e);
            return new BocloudResult(false, "创建失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String ruleId) {
        try {
            SecurityGroupService.DeleteSecurityGroupRulesInput input = new SecurityGroupService.DeleteSecurityGroupRulesInput();
            List<String> securityGroupRules = new ArrayList<>();
            securityGroupRules.add(ruleId);
            input.setSecurityGroupRules(securityGroupRules);
            SecurityGroupService.DeleteSecurityGroupRulesOutput output = service.deleteSecurityGroupRules(input);
            if (0 == output.getRetCode()) {
                return new BocloudResult(true, "删除成功！");
            } else {
                return new BocloudResult(false, "删除失败：" + output.getMessage());
            }
        } catch (QCException e) {
            logger.error("删除失败：{}", e);
            return new BocloudResult(false, "删除失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SecurityGroupRuleModel ruleModel) {
        return null;
    }

    @Override
    public BocloudResult list() {
        try {
            SecurityGroupService.DescribeSecurityGroupRulesInput input = new SecurityGroupService.DescribeSecurityGroupRulesInput();
            SecurityGroupService.DescribeSecurityGroupRulesOutput output = service.describeSecurityGroupRules(input);
            if (ListTool.isEmpty(output.getSecurityGroupRuleSet())) {
                return new BocloudResult(false, "查询失败：" + output.getMessage());
            }
            return new BocloudResult(true, convertor.convertList(output.getSecurityGroupRuleSet()), "查询成功");
        } catch (QCException e) {
            logger.error("查询列表失败：{}", e);
            return new BocloudResult(false, "查询失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult list(String sgId) {
        try {
            SecurityGroupService.DescribeSecurityGroupRulesInput input = new SecurityGroupService.DescribeSecurityGroupRulesInput();
            input.setSecurityGroup(sgId);
            SecurityGroupService.DescribeSecurityGroupRulesOutput output = service.describeSecurityGroupRules(input);
            if (0 == output.getRetCode()) {
                return new BocloudResult(true, convertor.convertList(output.getSecurityGroupRuleSet()), "查询成功");
            } else {
                return new BocloudResult(false, output.getMessage());
            }
        } catch (QCException e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "查询失败：" + e.getMessage());
        }
    }
}
