package com.bocloud.cmp.provider.security;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.QcloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.qcloudconvertor.KeypairConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.qingcloud.sdk.exception.QCException;
import com.qingcloud.sdk.service.KeyPairService;
import com.qingcloud.sdk.service.KeyPairService.CreateKeyPairInput;
import com.qingcloud.sdk.service.KeyPairService.CreateKeyPairOutput;
import com.qingcloud.sdk.service.Types;

/**
 * 秘钥
 *
 * @时间 2019年9月25日
 */
@Provider("bocloud.cmp.qcloud")
public class QcloudKeypairProvider extends QcloudProvider implements KeypairProvider {

	private static Logger logger = LoggerFactory.getLogger(QcloudKeypairProvider.class);
	private BeanConvertor<KeypairModel, Types.KeyPairModel> convertor = new KeypairConvertor();
	private KeyPairService service = new KeyPairService(this.getContext());

	public QcloudKeypairProvider(Butler butler) {
		super(butler);
	}

	@Override
	public boolean accessable() {
		return super.accessable();
	}

	@Override
	public BocloudResult list() {
		try {
			KeyPairService.DescribeKeyPairsInput input = new KeyPairService.DescribeKeyPairsInput();
			KeyPairService.DescribeKeyPairsOutput output = service.describeKeyPairs(input);
			List<KeypairModel> keypairModels = new ArrayList<>();
			if (!ListTool.isEmpty(output.getKeyPairSet())) {
				keypairModels = convertor.convertList(output.getKeyPairSet());
			}
			return new BocloudResult(true, keypairModels, "查询成功");
		} catch (QCException e) {
			logger.error("查询列表异常：{}", e);
			return new BocloudResult(false, "查询列表异常：{}", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String keypairName) {
		try {
			KeyPairService.DescribeKeyPairsInput input = new KeyPairService.DescribeKeyPairsInput();
			List<String> keypairs = new ArrayList<>();
			keypairs.add(keypairName);
			input.setKeyPairs(keypairs);
			KeyPairService.DescribeKeyPairsOutput output = service.describeKeyPairs(input);
			if (ListTool.isEmpty(output.getKeyPairSet())) {
				return new BocloudResult(false, "查询失败，密钥不存在");
			}
			KeypairModel convertModel = convertor.convertModel(output.getKeyPairSet().get(0));
			convertModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, convertModel, "查询成功");
		} catch (QCException e) {
			logger.error("查询详情异常：{}", e);
			return new BocloudResult(false, "查询详情异常：{}", e.getMessage());
		}
	}

	@Override
	public BocloudResult create(String keypairName, String publicKey) {
		try {
			KeyPairService service = new KeyPairService(this.getContext());
			CreateKeyPairInput input = new CreateKeyPairInput();
			if (null != keypairName) {
				input.setKeyPairName(keypairName);
			}
			if (null != publicKey) {
				if (!publicKey.startsWith("ssh-rsa ")) {
					return new BocloudResult(false, "访问被拒绝, 非法的公钥格式");
				}
				input.setPublicKey(publicKey);
			}
			CreateKeyPairOutput createKeyPair = service.createKeyPair(input);
			if (0 == createKeyPair.getRetCode()) {
				KeypairModel convertModel = (KeypairModel) this.detail(createKeyPair.getKeyPairID()).getData();
				convertModel.setPrivateKey(createKeyPair.getPrivateKey());
				return new BocloudResult(true, convertModel, "创建成功！");
			} else {
				return new BocloudResult(false, createKeyPair.getMessage());
			}
		} catch (QCException e) {
			logger.error("创建失败：{}", e);
			return new BocloudResult(false, "创建失败：{}", e.getMessage());
		}
	}

	@Override
	public BocloudResult delete(String keypairName) {
		try {
			KeyPairService.DeleteKeyPairsInput input = new KeyPairService.DeleteKeyPairsInput();
			List<String> keypairs = new ArrayList<>();
			keypairs.add(keypairName);
			input.setKeyPairs(keypairs);
			KeyPairService.DeleteKeyPairsOutput output = service.deleteKeyPairs(input);
			if (0 == output.getRetCode()) {
				return new BocloudResult(true, "删除成功！");
			} else {
				return new BocloudResult(false, output.getMessage());
			}
		} catch (QCException e) {
			logger.error("删除失败：{}", e);
			return new BocloudResult(false, "删除失败：{}", e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(KeypairModel model) {
		try {
			KeyPairService.ModifyKeyPairAttributesInput input = new KeyPairService.ModifyKeyPairAttributesInput();
			input.setKeyPair(model.getKeyId());
			input.setKeyPairName(model.getName());
			input.setDescription(model.getRemark());
			KeyPairService.ModifyKeyPairAttributesOutput output = service.modifyKeyPairAttributes(input);
			if (0 == output.getRetCode()) {
				return new BocloudResult(true, "编辑成功！");
			} else {
				return new BocloudResult(false, output.getMessage());
			}
		} catch (QCException e) {
			logger.error("编辑失败：{}", e);
			return new BocloudResult(false, "编辑失败：{}", e.getMessage());
		}
	}

	@Override
	public BocloudResult create(String keypairName, String publicKey, String encryptMethod) {
		try {
			KeyPairService.CreateKeyPairInput input = new KeyPairService.CreateKeyPairInput();
			input.setKeyPairName(keypairName);
			input.setPublicKey(publicKey);
			// ssh-rsa,ssh-dss,ecdsa-sha2-nistp521,ssh-ed25519
			if (!StringUtils.isEmpty(encryptMethod)) {
				input.setEncryptMethod(encryptMethod);
			}
			KeyPairService.CreateKeyPairOutput output = service.createKeyPair(input);
			if (!StringUtils.isEmpty(output.getKeyPairID())) {
				return new BocloudResult(false, "创建失败：" + output.getMessage());
			}
			return new BocloudResult(true, this.detail(output.getKeyPairID()), "创建成功");
		} catch (QCException e) {
			logger.error("Get error message：{}", e);
			return new BocloudResult(false, "创建失败");
		}
	}

	@Override
	public BocloudResult list(String zone) {
		try {
			KeyPairService.DescribeKeyPairsInput input = new KeyPairService.DescribeKeyPairsInput();
			input.setZone(zone);
			KeyPairService.DescribeKeyPairsOutput output = service.describeKeyPairs(input);
			List<KeypairModel> keypairModels = new ArrayList<>();
			if (!ListTool.isEmpty(output.getKeyPairSet())) {
				keypairModels = convertor.convertList(output.getKeyPairSet());
			}
			return new BocloudResult(true, keypairModels, "查询成功");
		} catch (QCException e) {
			logger.error("Get error message：{}", e);
			return new BocloudResult(false, "查询失败");
		}
	}

}
