package com.bocloud.cmp.provider.storage;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.provider.QcloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.qcloudconvertor.VolumeConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.qingcloud.sdk.exception.QCException;
import com.qingcloud.sdk.service.Types;
import com.qingcloud.sdk.service.VolumeService;
import com.qingcloud.sdk.service.VolumeService.CreateVolumesInput;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * volume 操作
 *
 * @时间 2019年9月24日
 */
@Provider("bocloud.cmp.qcloud")
public class QcloudVolumeProvider extends QcloudProvider implements VolumeProvider {

    private static Logger logger = LoggerFactory.getLogger(QcloudVolumeProvider.class);
    private BeanConvertor<VolumeModel, Types.VolumeModel> convertor = new VolumeConvertor();
    private VolumeService service = new VolumeService(this.getContext());

    public QcloudVolumeProvider(Butler butler) {
        super(butler);
    }

    @Override
    public boolean accessable() {
        return super.accessable();
    }

    @Override
    public BocloudResult list() {
        try {
            VolumeService.DescribeVolumesInput input = new VolumeService.DescribeVolumesInput();
            VolumeService.DescribeVolumesOutput output = service.describeVolumes(input);
            List<VolumeModel> volumeModelList = new ArrayList<>();
            if (!ListTool.isEmpty(output.getVolumeSet())) {
                volumeModelList = convertor.convertList(output.getVolumeSet());
            }
            return new BocloudResult(true, volumeModelList, "查询成功");
        } catch (QCException e) {
            logger.error("列表查询成功：{}", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(VolumeModel volume) {
        try {
            VolumeService service = new VolumeService(this.getContext(), volume.getZone());
            CreateVolumesInput input = new CreateVolumesInput();
            input.setCount(1);
            input.setSize(volume.getSize());
            input.setVolumeName(volume.getName());
            input.setVolumeType(Integer.valueOf(volume.getVolumeType()));// 200
                                                                         // SSD企业级,100
                                                                         // 基础型,2
                                                                         // 容量型,5
                                                                         // 企业级分布式SAN(NeonSAN)
            VolumeService.CreateVolumesOutput output = service.createVolumes(input);
            if (ListTool.isEmpty(output.getVolumes())) {
                return new BocloudResult(false, "创建失败：" + output.getMessage());
            }
            return new BocloudResult(true, this.detail(output.getVolumes().get(0)).getData(), "创建成功");
        } catch (QCException e) {
            logger.error("创建硬盘失败：{}", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult modify(VolumeModel volume) {
        try {
            VolumeService.ModifyVolumeAttributesInput input = new VolumeService.ModifyVolumeAttributesInput();
            input.setVolume(volume.getVolumeUuid());
            input.setVolumeName(volume.getName());
            input.setDescription(volume.getRemark());
            VolumeService.ModifyVolumeAttributesOutput output = service.modifyVolumeAttributes(input);
            if (0 != output.getRetCode()) {
                return new BocloudResult(false, "修改失败：" + output.getMessage());
            }
            return new BocloudResult(true, this.detail(volume.getVolumeUuid()).getData(), "修改成功");
        } catch (QCException e) {
            logger.error("编辑硬盘失败：{}", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    @Override
    public BocloudResult remove(String volumeId) {
        try {
            VolumeService.DeleteVolumesInput input = new VolumeService.DeleteVolumesInput();
            List<String> volumeIds = new ArrayList<>();
            volumeIds.add(volumeId);
            input.setVolumes(volumeIds);
            VolumeService.DeleteVolumesOutput output = service.deleteVolumes(input);
            if (!this.waitJob(output.getJobID())) {
                return new BocloudResult(false, "删除失败：" + output.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (QCException e) {
            logger.error("硬盘删除失败：{}", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult detail(String volumeId) {
        try {
            VolumeService.DescribeVolumesInput input = new VolumeService.DescribeVolumesInput();
            List<String> volumeIds = new ArrayList<>();
            volumeIds.add(volumeId);
            input.setVolumes(volumeIds);
            VolumeService.DescribeVolumesOutput output = service.describeVolumes(input);
            if (ListTool.isEmpty(output.getVolumeSet())) {
                return new BocloudResult(false, "查询失败，硬盘不存在");
            }
            return new BocloudResult(true, convertor.convertModel(output.getVolumeSet().get(0)), "查询成功");
        } catch (QCException e) {
            logger.error("查询硬盘详情失败：{}", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        return null;
    }

	@Override
	public BocloudResult attach(String serverId, String volumeId) {
		try {
			VolumeService.AttachVolumesInput input = new VolumeService.AttachVolumesInput();
			List<String> volumeIds = new ArrayList<>();
			volumeIds.add(volumeId);
			input.setVolumes(volumeIds);
			input.setInstance(serverId);
			VolumeService.AttachVolumesOutput output = service.attachVolumes(input);
			if (!this.waitJob(output.getJobID())) {
				return new BocloudResult(false, "挂载失败：" + output.getMessage());
			}
			VolumeModel volumeModel = (VolumeModel) this.detail(volumeId).getData();
			return new BocloudResult(true, volumeModel.getDevice(), "挂载成功");
		} catch (QCException e) {
			logger.error("硬盘挂载失败：{}", e);
			return new BocloudResult(false, "挂载失败");
		}
	}

    @Override
    public BocloudResult attach(String serverId, String mountpoint, String volumeId) {
        return null;
    }

    @Override
    public BocloudResult detach(String serverId, String volumeId) {
        try {
            VolumeService.DetachVolumesInput input = new VolumeService.DetachVolumesInput();
            List<String> volumeIds = new ArrayList<>();
            volumeIds.add(volumeId);
            input.setVolumes(volumeIds);
            input.setInstance(serverId);
            VolumeService.DetachVolumesOutput output = service.detachVolumes(input);
            if (!this.waitJob(output.getJobID())) {
                return new BocloudResult(false, "卸载失败：" + output.getMessage());
            }
            return new BocloudResult(true, this.detail(volumeId).getData(), "卸载成功");
        } catch (QCException e) {
            logger.error("硬盘卸载失败：{}", e);
            return new BocloudResult(false, "卸载失败");
        }
    }

    @Override
    public BocloudResult resetStatus(String volumeId, String status) {
        return null;
    }

    @Override
    public BocloudResult uploadToImage(String volumeId, ImageModel imageModel) {
        return null;
    }

    @Override
    public BocloudResult extend(String volumeId, Integer newSize) {
        try {
            VolumeService.ResizeVolumesInput input = new VolumeService.ResizeVolumesInput();
            List<String> volumeIds = new ArrayList<>();
            volumeIds.add(volumeId);
            input.setVolumes(volumeIds);
            input.setSize(newSize);
            VolumeService.ResizeVolumesOutput output = service.resizeVolumes(input);
            if (!this.waitJob(output.getJobID())) {
                return new BocloudResult(false, "硬盘大小修改失败：" + output.getMessage());
            }
            return new BocloudResult(true, this.detail(volumeId).getData(), "硬盘大小修改成功");
        } catch (QCException e) {
            logger.error("硬盘扩容失败：{}", e);
            return new BocloudResult(false, "硬盘大小修改失败");
        }
    }

    @Override
    public BocloudResult list(String hostId, String storagePoolName) {
        return null;
    }

    @Override
    public BocloudResult remove(H3cVolumeModel volume) {
        return null;
    }

    @Override
    public BocloudResult list(String zone) {
        try {
            VolumeService volumeService = new VolumeService(this.getContext(), zone);
            VolumeService.DescribeVolumesInput input = new VolumeService.DescribeVolumesInput();
            VolumeService.DescribeVolumesOutput output = volumeService.describeVolumes(input);
            List<VolumeModel> volumeModelList = new ArrayList<>();
            if (!ListTool.isEmpty(output.getVolumeSet())) {
                volumeModelList = convertor.convertList(output.getVolumeSet());
                for (VolumeModel volumeModel : volumeModelList) {
                    volumeModel.setZone(zone);
                    volumeModel.setRegionId(this.getRegionId());
                }
            }
            return new BocloudResult(true, volumeModelList, "查询成功");
        } catch (QCException e) {
            logger.error("根据zone查询失败：{}", e);
            return new BocloudResult(false, "查询失败");
        }
    }
}
