package com.bocloud.cmp.provider;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.common.SangforHttpClient;
import com.bocloud.cmp.provider.v63.util.RSAUtils;
import com.bocloud.cmp.provider.v68.SangForProvider68;
import com.bocloud.cmp.provider.v68.compute.SangforServerProvider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.ssl.TrustStrategy;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import java.math.BigInteger;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * @Author : wangjian
 * @Description
 * @Date : 2022/8/10 11:08
 * @Version : 1.0
 */

public class SangforProvider extends GenericProvider implements SuperProvider {
    private static Logger logger = LoggerFactory.getLogger(SangforProvider.class);

    protected static String CSRFPreventionToken = "CSRFPreventionToken";
    protected static String Cookie = "Cookie";
    protected static final int PAGE_SIZE = 20;// 最大值为100
    protected static final String SANGFOR_TYPE = "hci";// 深信服HCI平台

    protected Butler butler;
    private String prefix;
    private String path;
    private String method;
    private String body;
    private String username;
    private String password;
    private HashMap<String, Object> header = null;

    //6.8.0的参数
    // scp颁发的ak sk
    private String accessKey;
    private String secretKey;
    // 以下均为默认值
    private String region;
    private String version;
    private String service;
    private String strSignedHeader;
    // 认证范围,为了配合scp端，可以指定，不指定的话自动生成
    private String credentialScope;
    //仅仅用于过验证，值任意
    private static final String ec2Cookie = "aCMPAuthToken=EC2TOKEN";

    public SangforProvider(Butler butler) {
        super(butler);
        this.butler = butler;
        this.prefix = butler.getUrl();
        this.username = butler.getUsername();
        this.password = butler.getPassword();
        this.version = butler.getVersion();
        if (null != version && version.equals("6.8")) {
            this.accessKey = butler.getAccessKey();
            this.secretKey = butler.getSecretKey();
            this.region = "RegionOne";
            this.service = "DSP-Client";
            this.strSignedHeader = "path;";
            this.credentialScope = region + "/" + service + "/" + "dmp_request";
        }
    }

    @Override
    public boolean accessable() {
        try {
            String version = this.butler.getVersion();
            if (null != version && version.equals("6.8")) {
                //6.8.0版本走EC2认证
                //logger.info("===========SangforProvider 6.8.0版本走【EC2认证】");
                SangForProvider68 sangForProvider68 = new SangForProvider68(this.butler);
                return sangForProvider68.accessable();
            }
            //logger.info("===========SangforProvider 6.3.0版本走【密码Token认证】方式");
            BocloudResult result = getToken();
            if (result.isFailed()) {
                return false;
            }
            return true;
        } catch (Exception var5) {
            logger.error("平台认证失败 ： " + var5.getMessage());
            return false;
        }
    }

    public static HttpClient getHttpClient() {
        return new HttpClient(30 * 1000, PostDataFormat.FORM_URLENCODED);
    }

    public String buildUrl(String apiUrl) {
        // sxp:2023.5.25
        return this.prefix.contains("https") ? this.prefix + apiUrl : "https://" + this.prefix + apiUrl;
    }

    protected Ec2Signer getEc2Signer() {
        if (method.equals("GET")) {
            Ec2Signer ec2Signer = new Ec2Signer.Builder(accessKey, secretKey)
                    .regionName(region)
                    .serviceName(service)
                    .httpMethodName(method)
                    .canonicalURI(path)
                    .strSignedHeader(strSignedHeader)
                    .credentialScope(credentialScope)
                    //.debug()  //开启打印
                    .build();
            return ec2Signer;
        } else {
            return new Ec2Signer.Builder(accessKey, secretKey)
                    .regionName(region)
                    .serviceName(service)
                    .httpMethodName(method)
                    .canonicalURI(path)
                    .payload(body)
                    .strSignedHeader(strSignedHeader)
                    .credentialScope(credentialScope)
                    //.debug()  //开启打印
                    .build();
        }
    }

    protected Map<String, Object> buildHeader() {
        if (null != version && version.equals("6.8")) {
            Ec2Signer ec2Signer = getEc2Signer();
            Map<String, String> headers = ec2Signer.getHeaders();
            if (header == null) {
                header = new HashMap<>();
            }
            header.put("Cookie", ec2Cookie);
            header.put("Authorization", headers.get("Authorization"));
            return header;
        } else {
            if (header != null && header.containsKey(CSRFPreventionToken) && header.containsKey(Cookie)) {
                return header;
            }
            header = new HashMap<String, Object>();
            BocloudResult tokenData = getToken();
            if (tokenData.isSuccess()) {
                JSONObject capJO = JSON.parseObject(tokenData.getData().toString());
                String csrfPreventionToken = capJO.getString("CSRFPreventionToken");
                String ticket = capJO.getString("ticket");
                header.put(CSRFPreventionToken, csrfPreventionToken);
                header.put(Cookie, "LoginAuthCookie=" + ticket);
                return header;
            }
        }
        return header;
    }

    private Map<String, Object> buildHeaderToken() {
        Map<String, Object> header = new HashMap<String, Object>();
        header.put("Content-type", "application/json");
        return header;
    }

    public Result doGet(String urlStr, Map<String, Object> params) {
        HttpClient httpClient = getHttpClient();
        try {
            UUID uuid = UUID.randomUUID();
            httpClient.setDataFormat(null);
            this.path = urlStr;
            this.method = "GET";
            Result result = httpClient.get(buildHeader(), params, buildUrl(urlStr));
            String message = result.getMessage();
            if (result.isFailed() && message != null && message.contains("Read timed out")) {
                // 重试
                logger.error("GET请求【{}】SSLException,2秒后重试...", uuid);
                try {
                    Thread.sleep(1000 * 2);
                } catch (InterruptedException e) {
                    logger.error(e.getMessage(), e);
                }
                result = httpClient.get(buildHeader(), params, buildUrl(urlStr));
                logger.info("GET请求【{}】SSLException, 重试结果：{}", uuid, result.isSuccess());
            }
            return result;
        } finally {
            httpClient.close();
        }
    }

    public Result doPost(String url, Map<String, Object> params) {
        return this.doPost(url, params, null, null);
    }

    public Result doPost(String url, Map<String, Object> params, String contentType, PostDataFormat postDataFormat) {
        HttpClient httpClient = getHttpClient();
        try {
            this.path = url;
            this.method = "POST";
            if (params == null) {
                params = new HashMap<>(0);
            }
            this.body = JSON.toJSONString(params);
            Map<String, Object> header = buildHeader();
            if (contentType != null) {
                header.put("Content-Type", contentType);
            }
            if (postDataFormat != null) {
                httpClient.setDataFormat(postDataFormat);
            }
            return httpClient.post(header, params, buildUrl(url));
        } finally {
            httpClient.close();
        }
    }


    public Result doPut(String url, Map<String, Object> params) {
        HttpClient httpClient = getHttpClient();
        try {
            this.path = url;
            return httpClient.put(buildHeader(), params, buildUrl(url));
        } finally {
            httpClient.close();
        }
    }

    public Result doPut(String url, Map<String, Object> params, String contentType, PostDataFormat postDataFormat) {
        HttpClient httpClient = getHttpClient();
        try {
            this.path = url;
            this.method = "PUT";
            if (params == null) {
                params = new HashMap<>(0);
            }
            this.body = JSON.toJSONString(params);
            Map<String, Object> httpHeader = buildHeader();
            if (contentType != null) {
                httpHeader.put("Content-Type", contentType);
            }
            if (postDataFormat != null) {
                httpClient.setDataFormat(postDataFormat);
            }
            return httpClient.put(httpHeader, params, buildUrl(url));
        } finally {
            httpClient.close();
        }
    }

    public Result doDelete(String url, Map<String, Object> params, String contentType, PostDataFormat postDataFormat) {
        SangforHttpClient sangforHttpClient = new SangforHttpClient(30 * 1000, PostDataFormat.FORM_URLENCODED);

        try {
            this.path = url;
            this.method = "DELETE";
            if (params == null) {
                params = new HashMap<>(0);
            }
            this.body = JSON.toJSONString(params);
            Map<String, Object> httpHeader = buildHeader();
            if (contentType != null) {
                httpHeader.put("Content-Type", contentType);
            }
            if (postDataFormat != null) {
                sangforHttpClient.setDataFormat(postDataFormat);
            }
            return sangforHttpClient.delete(httpHeader, params, buildUrl(url));
        } finally {
            sangforHttpClient.close();
        }
    }

    public Result doPostLogin(String url, Map<String, Object> params) {
        HttpClient httpClient = getHttpClient();
        try {
            this.path = url;
            return httpClient.post(buildHeaderToken(), params, buildUrl(url));
        } finally {
            httpClient.close();
        }
    }

    public Result doGetPublicKey(String url) {
        HttpClient httpClient = getHttpClient();
        try {
            httpClient.setDataFormat(null);
            Result result = httpClient.get(new HashMap<>(), new Object(), buildUrl(url));
            return result;
        } finally {
            httpClient.close();
        }
    }

    public BocloudResult getToken() {
        try {
            // 获取公钥
            Result publicKeyResult = this.doGetPublicKey("/vapi/json/public_key");
            if (publicKeyResult.isFailed()) {
                logger.info(JSONObject.toJSONString(publicKeyResult));
                return new BocloudResult(false, publicKeyResult.getMessage());
            }
            JSONObject publicKeyJO = JSONObject.parseObject(publicKeyResult.getMessage());
            if (null == publicKeyJO || publicKeyJO.getInteger("success") != 1 || null == publicKeyJO.getString("data")) {
                logger.info(JSONObject.toJSONString(publicKeyResult));
                return new BocloudResult(false, "获取公钥失败");
            }
            String public_key = publicKeyJO.getString("data");
            // 接口获取的公钥public_key是 rsa 的模数
            // 10001 是 指数
            BigInteger publicModules = new BigInteger(public_key, 16);
            BigInteger publicExponent = new BigInteger("10001", 16);
            String encrypt = RSAUtils.encryptByPublicKey(password, RSAUtils.getPublicKey(publicModules, publicExponent), true);
            JSONObject params = new JSONObject();
            params.put("username", username);
            params.put("password", encrypt);
            // 获取token
            Result result = doPostLogin("/vapi/json/access/ticket", params);
            if (result.isFailed()) {
                logger.info("==========>>>获取认证信息失败...");
                logger.info(JSONObject.toJSONString(result));
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object || object.getInteger("success") != 1 || null == object.getJSONObject("data")) {
                logger.info("==========>>>获取认证信息失败...");
                logger.info(JSONObject.toJSONString(result));
                return new BocloudResult(false, "验证失败");
            }
            JSONObject jsonObject = object.getJSONObject("data");
            return new BocloudResult(true, jsonObject.toJSONString(), "验证成功");
        } catch (Exception e) {
            logger.error("获取认证信息失败", e);
            return new BocloudResult(false, "验证失败");
        }
    }

    /**
     * 获得信任证书的httpclient
     *
     * @return
     */
    public static CloseableHttpClient createSSLClientDefault() {
        try {
            //使用 loadTrustMaterial() 方法实现一个信任策略，信任所有证书
            SSLContext sslContext = new SSLContextBuilder().loadTrustMaterial(null, new TrustStrategy() {
                // 信任所有
                public boolean isTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                    return true;
                }
            }).build();
            //NoopHostnameVerifier类:  作为主机名验证工具，实质上关闭了主机名验证，它接受任何
            //有效的SSL会话并匹配到目标主机。
            HostnameVerifier hostnameVerifier = NoopHostnameVerifier.INSTANCE;
            SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sslContext, hostnameVerifier);
            return HttpClients.custom().setSSLSocketFactory(sslsf).build();
        } catch (KeyManagementException e) {
            logger.error(e.getMessage(), e);
        } catch (NoSuchAlgorithmException e) {
            logger.error(e.getMessage(), e);
        } catch (KeyStoreException e) {
            logger.error(e.getMessage(), e);
        }
        return HttpClients.createDefault();

    }

    public static void main(String[] args) {
        Butler butler = new Butler();
        butler.setUrl("https://172.22.150.20:4430");
        butler.setVersion("6.8");
        butler.setAccessKey("263eafbb6df24213ad67873f24dbccee");
        butler.setSecretKey("2cfc7e770f8d49688415bf5a9c62c093");

        String instanceId = "3a8ea86a-43c1-4b55-b0cb-80da0304ff23";

        SangforServerProvider provider = new SangforServerProvider(butler);
        BocloudResult result = provider.detail(instanceId);
        System.out.println(JSON.toJSONString(result.getData()));
    }
}
