package com.bocloud.cmp.provider.v68.common;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.sangfor.SangforAzModel;
import com.bocloud.cmp.params.Sangfor.SangforAzParams;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.cmp.provider.v68.model.SangforResult;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 通用
 *
 * @author sxp
 * @since 2023/6/2
 */
@Slf4j
@Provider("bocloud.cmp.sangfor.v68")
public class BaseProvider extends SangforProvider {
    private static Logger logger = LoggerFactory.getLogger(BaseProvider.class);

    public BaseProvider(Butler butler) {
        super(butler);
    }

    /**
     * 获取加密公钥的公钥模数
     *
     * @author sxp
     * @since 2023/6/2
     */
    public BocloudResult<String> getPublicKey() {
        HashMap<String, Object> params = new HashMap<>(0);
        String url = "/janus/v2/public-key";

        Result result;
        try {
            result = this.doGet(url, params);
            if (result.isFailed()) {
                return new BocloudResult(false, "获取加密公钥失败，失败原因为：" + result.getMessage());
            }
            String message = result.getMessage();
            SangforResult sfResult = JSON.parseObject(message, SangforResult.class);
            if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                Object data = sfResult.getData();
                JSONObject json = JSON.parseObject(JSON.toJSONString(data), JSONObject.class);
                String publicKey = json.getString("public_key");
                return new BocloudResult(true, publicKey, "success");
            } else {
                return new BocloudResult(false, "获取加密公钥失败，失败原因为：" + sfResult.getMessage());
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new BocloudResult(false, "获取加密公钥失败，失败原因为：" + e.getMessage());
        }
    }


    /**
     * 获取资源池列表
     *
     * @author sxp
     * @since 2023/6/20
     */
    public BocloudResult<List<SangforAzParams>> listAzs() {
        try {
            Map<String, Object> params = new HashMap<>();
            params.put("type", "hci");
            Result result = this.doGet("/janus/20180725/azs", params);
            List<SangforAzModel> azList = new ArrayList<>();
            if (result.isSuccess()) {
                List<SangforAzModel> sangforAzModels =
                        JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data")).getString("data"),
                                SangforAzModel.class);
                azList.addAll(sangforAzModels);
            }

            List<SangforAzParams> azParamsList = new ArrayList<>();
            for (SangforAzModel model : azList) {
                SangforAzParams item = new SangforAzParams();
                BeanUtils.copyProperties(model, item);
                item.setUuid(model.getId());
                azParamsList.add(item);
            }

            return new BocloudResult<>(true, azParamsList, "获取资源池列表成功");
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return new BocloudResult<>(false, "获取资源池列表失败:" + e.getMessage());
        }
    }

    /**
     * 获取资源池对应的HCI版本
     *
     * @author sxp
     * @since 2023/6/20
     */
    public String getVersionByAzId(String azId) {
        if (StringUtils.isEmpty(azId)) {
            logger.warn("传递过来的azId为空");
            return null;
        }

        try {
            BocloudResult<List<SangforAzParams>> result = listAzs();
            if (result.isFailed()) {
                logger.error(result.getMessage());
                return null;
            }
            List<SangforAzParams> azParamsList = result.getData();
            Map<String, String> versionMap = new HashMap<>();
            for (SangforAzParams az : azParamsList) {
                versionMap.put(az.getUuid(), az.getVersion());
            }

            return versionMap.get(azId);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return null;
        }
    }




}
