package com.bocloud.cmp.provider.v68.compute;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.sangfor.SangforImage;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.cmp.provider.common.FastjsonUtils;
import com.bocloud.cmp.provider.compute.ImageProvider;
import com.bocloud.cmp.provider.v68.model.SangforResult;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * 镜像
 *
 * @author sxp
 * @since 2023/5/30
 */
@Provider("bocloud.cmp.sangfor.v68")
public class SangforImageProvider extends SangforProvider implements ImageProvider {
    private static Logger logger = LoggerFactory.getLogger(SangforImageProvider.class);

    private static final Map<String, Object> INIT_PARAMS = new HashMap<>();

    static {
        INIT_PARAMS.put("image_type", "public");// 公共镜像，全平台所有用户可见
        INIT_PARAMS.put("status", "active");// 可用
    }


    public SangforImageProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult<List<SangforImage>> list() {
        return this.list(Collections.emptyMap());
    }


    public BocloudResult<List<SangforImage>> list(Map<String, Object> params) {
        String url = "/janus/20180725/images";

        Map<String, Object> queryParams = new HashMap<>(INIT_PARAMS);
        if (params != null) {
            queryParams.putAll(params);
        }

        List<SangforImage> allImageList = new ArrayList<>();

        try {
            queryParams.put("page_num", 0);
            queryParams.put("page_size", PAGE_SIZE);
            Result result = this.doGet(url, queryParams);

            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }
            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                    Object data = sfResult.getData();
                    JSONObject dataJson = JSON.parseObject(JSON.toJSONString(data));
                    List<SangforImage> images = JSON.parseArray(dataJson.getString("data"), SangforImage.class);
                    if (images != null) {
                        allImageList.addAll(images);
                    }
                    Integer totalSize = dataJson.getInteger("total_size");
                    String nextPageString = dataJson.getString("next_page_num");

                    Integer totalPage, nextPage;
                    // 下一页数，整型的字符串，为""表示当前是最后一页
                    if (StringUtils.isEmpty(nextPageString)) {
                        nextPage = 0;
                        totalPage = 0;
                    } else {
                        nextPage = dataJson.getInteger("next_page_num");
                        totalPage = totalSize / PAGE_SIZE;
                        if (totalSize % PAGE_SIZE > 0) {
                            totalPage++;
                        }
                    }

                    if (totalPage > 0) {
                        while (nextPage < totalPage) {
                            queryParams.put("page_num", nextPage);
                            queryParams.put("page_size", PAGE_SIZE);
                            result = this.doGet(url, queryParams);
                            message = result.getMessage();
                            sfResult = null;
                            if (FastjsonUtils.isJSONObject(message)) {
                                sfResult = JSON.parseObject(message, SangforResult.class);
                            }
                            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                                    data = sfResult.getData();
                                    dataJson = JSON.parseObject(JSON.toJSONString(data));
                                    images = JSON.parseArray(dataJson.getString("data"), SangforImage.class);
                                    if (images != null) {
                                        allImageList.addAll(images);
                                    }
                                }
                            }
                            nextPage++;
                        }
                    }
                }

                return new BocloudResult(true, allImageList, "success");
            }

            String errorMessage;
            if (sfResult != null) {
                errorMessage = sfResult.getMessage();
            } else {
                errorMessage = message;
            }
            return new BocloudResult(false, "查询镜像出错，失败原因为：" + errorMessage);
        } catch (Exception e) {
            logger.error("查询镜像出错", e);
            return new BocloudResult(false, "查询镜像出错，失败原因为：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult<List<SangforImage>> detail(String imageUuid) {
        String url = "/janus/20180725/images";

        Map<String, Object> queryParams = new HashMap<>();
        queryParams.put("image_ids", imageUuid);

        try {
            Result result = this.doPost(url, queryParams, "application/json", PostDataFormat.RAW);

            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }

            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                    Object data = sfResult.getData();
                    List<SangforImage> images = JSON.parseArray(JSON.toJSONString(data), SangforImage.class);
                    if (images != null) {
                        return new BocloudResult(true, images, "success");
                    } else {
                        return new BocloudResult(false, "查询镜像出错，返回数据为：" + JSON.toJSONString(data));
                    }
                }
            }

            String errorMessage;
            if (sfResult != null) {
                errorMessage = sfResult.getMessage();
            } else {
                errorMessage = message;
            }
            return new BocloudResult(false, "查询镜像出错，失败原因为：" + errorMessage);
        } catch (Exception e) {
            logger.error("查询镜像出错", e);
            return new BocloudResult(false, "查询镜像出错，失败原因为：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String s) {
        return new BocloudResult(false, "暂不支持");
    }

    @Override
    public BocloudResult modify(ImageModel imageModel) {
        return new BocloudResult(false, "暂不支持");
    }

    @Override
    public BocloudResult create(ImageModel imageModel) {
        return new BocloudResult(false, "暂不支持");
    }
}
