package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.smart.SyncModel;
import com.bocloud.cmp.provider.SmartProvider;
import com.bocloud.cmp.provider.smart.sdk.MapBuilder;
import com.bocloud.cmp.provider.smart.sdk.api.*;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.convert.*;
import com.bocloud.cmp.provider.smart.sdk.model.ComputeSummaryModel;
import com.bocloud.cmp.provider.smart.sdk.model.MetricModel;
import com.bocloud.cmp.provider.smart.sdk.model.StorageSummaryModel;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;

@Provider("bocloud.cmp.smartx")
public class SmartLocationProvider extends SmartProvider implements LocationProvider {

    private static final Logger logger = LoggerFactory.getLogger(SmartLocationProvider.class);

    public SmartLocationProvider(Butler butler) {
        super(butler);
    }

    @Override
    public boolean accessable() {
        return super.accessable();
    }

    @Override
    @SuppressWarnings("unchecked")
    public BocloudResult sync() {
        SyncModel syncModel = new SyncModel();
        logger.info("开始获取Smart-X数据...");
        // 存储策略数据
        logger.info("正在获取存储策略数据...");
        List<VolumeTemplateModel> volumeTemplateModels = new StoragePolicyConvert().convert(
                new StoragePolicyApi(this.getSessionParams()).list()
        );
        syncModel.setVolumeTemplateModels(volumeTemplateModels);
        // 镜像数据
        logger.info("正在获取镜像数据...");
        List<ImageModel> imageModels = new ImageConvert().convert(
                new ImageApi(this.getSessionParams()).list()
        );
        syncModel.setImageModels(imageModels);
        // 主机，主机包含的网卡、磁盘
        logger.info("正在获取主机数据...");
        List<HostModel> hostModels = new HostConvert().convert(
                new ClusterApi(this.getSessionParams()).storageSummary().getHosts(),
                new HostApi(this.getSessionParams()).list(),
                new VdsApi(this.getSessionParams()).listAvailableNic()
        );
        syncModel.setHostModels(hostModels);
        // 虚拟交换机，虚拟交换机关联的主机网卡，虚拟交换机包含的网络
        logger.info("正在获取虚拟交换机数据...");
        List<SwitcherModel> switcherModels = new VdsConvert().convert(
                new VdsApi(this.getSessionParams()).list()
        );
        for (SwitcherModel switcherModel : switcherModels) {
            logger.info("正在获取虚拟交换机[{}]的虚拟网络数据...", switcherModel.getName());
            List<NetworkModel> networkModels = new VlanConvert().convert(
                    new VlanApi(this.getSessionParams()).list(switcherModel.getUuid())
            );
            switcherModel.setNetworks(networkModels);
        }
        syncModel.setSwitcherModels(switcherModels);
        // 云硬盘，云硬盘关联的云主机
        logger.info("正在获取虚拟卷数据...");
        List<VolumeModel> volumeModels = new VolumeConvert().convert(
                new VolumeApi(this.getSessionParams()).list()
        );
        syncModel.setVolumeModels(volumeModels);
        // 云主机，云主机包含的网卡
        logger.info("正在获取云主机数据...");
        List<ServerModel> serverModels = new VmConvert().convert(
                new VmApi(this.getSessionParams()).list()
        );
        syncModel.setServerModels(serverModels);
        // 云主机模板
        logger.info("正在获取云主机模板数据...");
        syncModel.setServerTemplateModels(new VmTemplateConvert().convert(
                new VmTemplateApi(this.getSessionParams()).list())
        );
        // 云主机快照
        logger.info("正在获取云主机快照数据...");
        List<SnapshotModel> serverSnapshotModels = new VmSnapshotConvert().convert(
                new VmSnapshotApi(this.getSessionParams()).list()
        );
        syncModel.setServerSnapshotModels(serverSnapshotModels);
        // 虚拟卷快照
        logger.info("正在获取虚拟卷快照数据...");
        List<SnapshotModel> volumeSnapshotModels = new VolumeSnapshotConvert().convert(
                new VolumeSnapshotApi(this.getSessionParams()).list()
        );
        syncModel.setVolumeSnapshotModels(volumeSnapshotModels);
        // 任务
        logger.info("正在获取任务数据...");
        List<TaskMsgModel> taskMsgModels = new JobConvert().convert(
                new JobApi(this.getSessionParams()).list()
        );
        syncModel.setTaskMsgModels(taskMsgModels);
        logger.info("成功获取Smart-X数据!!!");
        return new BocloudResult(true, syncModel, "success");
    }

    @Override
    public BocloudResult summary() {
        try {
            ClusterApi clusterApi = new ClusterApi(this.getSessionParams());

            StorageSummaryModel storageSummary = clusterApi.storageSummary();
            Long remainDataSpace = storageSummary.getTotalDataCapacity() - storageSummary.getUsedDataSpace() - storageSummary.getFailureDataSpace();
            Double usedDataPercent = 1.0 * storageSummary.getUsedDataSpace() / storageSummary.getTotalDataCapacity();

            ComputeSummaryModel computeSummary = clusterApi.computeSummary();
            Integer unknownVmCount = computeSummary.getTotalVms() - computeSummary.getVmStat().getPoweredOn() -
                    computeSummary.getVmStat().getSuspended() - computeSummary.getVmStat().getPoweredOff();

            MetricModel cpuUsageMetric = clusterApi.metric(Constant.METRIC_CLUSTER_CPU_USAGE_PERCENT);
            MetricModel memoryUsageMetric = clusterApi.metric(Constant.METRIC_CLUSTER_MEMORY_USAGE_PERCENT);

            long cpuUsed = Math.round(computeSummary.getHz() * cpuUsageMetric.getValue() / 100);
            long cpuRemain = computeSummary.getHz() - cpuUsed;

            long memoryUsed = Math.round(computeSummary.getTotalMemoryInByte() * memoryUsageMetric.getValue() / 100);
            long memoryRemain = computeSummary.getTotalMemoryInByte() - memoryUsed;

            Map<String, Object> storage = MapBuilder.init()
                    .put("total", storageSummary.getTotalDataCapacity())
                    .put("used", storageSummary.getUsedDataSpace())
                    .put("failed", storageSummary.getFailureDataSpace())
                    .put("percent", Math.round(usedDataPercent * 1000D) / 10D)
                    .put("remain", remainDataSpace)
                    .put("remain2", remainDataSpace / 2)
                    .put("remain3", remainDataSpace / 3)
                    .get();

            Map<String, Object> vm = MapBuilder.init()
                    .put("total", computeSummary.getTotalVms())
                    .put("running", computeSummary.getVmStat().getPoweredOn())
                    .put("paused", computeSummary.getVmStat().getSuspended())
                    .put("stoped", computeSummary.getVmStat().getPoweredOff())
                    .put("unknown", unknownVmCount)
                    .get();

            Map<String, Object> cpu = MapBuilder.init()
                    .put("hz", computeSummary.getHz())
                    .put("cores", computeSummary.getTotalCpuCores())
                    .put("used", cpuUsed)
                    .put("remain", cpuRemain)
                    .put("percent", Math.round(cpuUsageMetric.getValue() * 10D) / 10D)
                    .get();

            Map<String, Object> memory = MapBuilder.init()
                    .put("total", computeSummary.getTotalMemoryInByte())
                    .put("percent", Math.round(memoryUsageMetric.getValue() * 10D) / 10D)
                    .put("used", memoryUsed)
                    .put("remain", memoryRemain)
                    .get();

            Map<String, Object> summary = MapBuilder.init()
                    .put("storage", storage)
                    .put("vm", vm)
                    .put("cpu", cpu)
                    .put("memory", memory)
                    .get();

            return new BocloudResult(true, summary, "success");
        } catch (Exception e) {
            logger.error("查询集群概览失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
