package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.SmartProvider;
import com.bocloud.cmp.provider.smart.sdk.api.ImageApi;
import com.bocloud.cmp.provider.smart.sdk.convert.ImageConvert;
import com.bocloud.cmp.provider.smart.sdk.model.ImageUploadModel;
import com.bocloud.cmp.provider.smart.sdk.model.create.ImageCreateModel;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.List;

@Provider("bocloud.cmp.smartx")
public class SmartImageProvider extends SmartProvider implements ImageProvider {

    private static final Logger logger = LoggerFactory.getLogger(SmartImageProvider.class);

    private final ImageApi imageApi;
    private final ImageConvert imageConvert = new ImageConvert();

    public SmartImageProvider(Butler butler) {
        super(butler);
        this.imageApi = new ImageApi(this.getSessionParams());
    }

    @Override
    public BocloudResult list() {
        try {
            List<ImageModel> imageModels = imageConvert.convert(imageApi.list());
            return new BocloudResult(true, imageModels, "success");
        } catch (Exception e) {
            logger.error("查询ISO映像失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String imageId) {
        try {
            ImageModel imageModel = imageConvert.convert(imageApi.get(imageId));
            return new BocloudResult(true, imageModel, "success");
        } catch (Exception e) {
            logger.error("查询ISO映像失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(ImageModel imageModel) {
        File file = new File(imageModel.getFilePath());
        logger.info("上传的文件大小：{}", file.length());

        FileInputStream fileInputStream;
        try {
            fileInputStream = new FileInputStream(file);
        } catch (FileNotFoundException e) {
            logger.error("创建ISO映像失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }

        ImageUploadModel imageUploadModel;
        try {
            ImageCreateModel imageCreateModel = new ImageCreateModel();
            imageCreateModel.setName(imageModel.getName());
            imageCreateModel.setDescription(imageModel.getRemark());
            imageCreateModel.setSize(file.length());
            imageCreateModel.setDevice("nfs");
            imageUploadModel = imageApi.create(imageCreateModel);
            logger.info("上传的块大小：{}", imageUploadModel.getChunkSize());
            logger.info("上传的块数：{}", imageUploadModel.getToUpload().size());
        } catch (Exception e) {
            logger.error("创建ISO映像失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }

        try {
            List<Long> uploadIds = imageUploadModel.getToUpload();
            Integer chunkSize = Math.toIntExact(imageUploadModel.getChunkSize());
            for (Long uploadId : uploadIds) {
                byte[] bytes = new byte[chunkSize];
                int read = fileInputStream.read(bytes);
                if (read == 0) break;
                logger.info("第[{}]块文件正在上传....", uploadId + 1);
                imageApi.upload(bytes, imageUploadModel.getZbsVolumeId(), imageUploadModel.getImageUuid(), uploadId);
            }
        } catch (Exception e) {
            imageApi.remove(imageUploadModel.getImageUuid());
            logger.error("创建ISO映像失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }

        try {
            ImageModel resultModel = imageConvert.convert(imageApi.get(imageUploadModel.getImageUuid()));
            return new BocloudResult(true, resultModel, "success");
        } catch (Exception e) {
            logger.error("查询ISO映像失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(ImageModel image) {
        try {
            imageApi.modify(image.getImageUuid(), image.getName(), image.getRemark());
            ImageModel resultModel = imageConvert.convert(imageApi.get(image.getImageUuid()));
            return new BocloudResult(true, resultModel, "success");
        } catch (Exception e) {
            logger.error("修改ISO映像失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String imageId) {
        try {
            imageApi.remove(imageId);
            return new BocloudResult(true, "success");
        } catch (Exception e) {
            logger.error("删除ISO映像失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
