package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.SmartProvider;
import com.bocloud.cmp.provider.smart.sdk.MapBuilder;
import com.bocloud.cmp.provider.smart.sdk.api.*;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.convert.VmConvert;
import com.bocloud.cmp.provider.smart.sdk.model.VmModel;
import com.bocloud.cmp.provider.smart.sdk.model.create.VmCreateModel;
import com.bocloud.cmp.provider.smart.sdk.model.modify.VmModifyModel;
import com.bocloud.cmp.provider.smart.sdk.model.other.VmCloneModel;
import com.bocloud.cmp.provider.smart.sdk.model.other.VmCreateVmTemplateModel;
import com.bocloud.cmp.provider.smart.sdk.session.SessionManager;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

@Provider("bocloud.cmp.smartx")
public class SmartServerProvider extends SmartProvider implements ServerProvider {

    private static final Logger logger = LoggerFactory.getLogger(SmartServerProvider.class);

    private final HostApi hostApi;
    private final VmApi vmApi;
    private final VolumeApi volumeApi;
    private final VmSnapshotApi vmSnapshotApi;
    private final VmTemplateApi vmTemplateApi;
    private final VmConvert vmConvert = new VmConvert();

    public SmartServerProvider(Butler butler) {
        super(butler);
        hostApi = new HostApi(this.getSessionParams());
        vmApi = new VmApi(this.getSessionParams());
        volumeApi = new VolumeApi(this.getSessionParams());
        vmSnapshotApi = new VmSnapshotApi(this.getSessionParams());
        vmTemplateApi = new VmTemplateApi(this.getSessionParams());
    }

    @Override
    public BocloudResult list() {
        try {
            List<ServerModel> serverModels = vmConvert.convert(vmApi.list());
            return new BocloudResult(true, serverModels, "success");
        } catch (Exception e) {
            logger.error("查询云主机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(ServerModel server) {
        try {
            BigDecimal d1024 = new BigDecimal(1024);

            VmCreateModel params = new VmCreateModel();
            params.setVmName(server.getName());
            params.setHa(server.getHa());
            params.setAutoSchedule(server.getAutoSchedule());
            params.setNodeIp(server.getHostIp());
            params.setVcpu(server.getCpu());
            params.setCpu(new VmCreateModel.Cpu(new VmCreateModel.Cpu.Topology(1L, server.getCpu().longValue())));
            params.setMemory(server.getMemory().multiply(d1024).multiply(d1024).multiply(d1024).longValue());
            params.setFirmware(server.getFirmware());
            params.setStatus(server.getStatus());
            params.setDescription(server.getRemark());

            List<VmCreateModel.Disk> disks = new ArrayList<>();
            for (VolumeModel diskDevice : server.getDiskDevices()) {
                if ("newDisk".equals(diskDevice.getType())) {
                    VmCreateModel.Disk disk = new VmCreateModel.Disk();
                    disk.setType("disk");
                    disk.setName(diskDevice.getName());
                    disk.setSizeInByte(diskDevice.getBlockCount());
                    if (diskDevice.getBus() != null) {
                        disk.setBus(diskDevice.getBus().toLowerCase());
                    }
                    disk.setStoragePolicyUuid(diskDevice.getVolumeTemplateUuid());
                    disks.add(disk);
                } else if ("mountDisk".equals(diskDevice.getType())) {
                    VmCreateModel.Disk disk = new VmCreateModel.Disk();
                    disk.setType("disk");
                    disk.setPath(diskDevice.getPath());
                    if (diskDevice.getBus() != null) {
                        disk.setBus(diskDevice.getBus().toLowerCase());
                    }
                    disks.add(disk);
                } else if ("mountISO".equals(diskDevice.getType())) {
                    VmCreateModel.Disk disk = new VmCreateModel.Disk();
                    disk.setType("cdrom");
                    disk.setPath(diskDevice.getPath());
                    if (diskDevice.getBus() != null) {
                        disk.setBus(diskDevice.getBus().toLowerCase());
                    }
                    disk.setDisabled(diskDevice.getDisabled());
                    disks.add(disk);
                }
            }
            params.setDisks(disks);

            List<VmCreateModel.Nic> nics = new ArrayList<>();
            for (NetworkCardModel networkCard : server.getNetworkCards()) {
                VmCreateModel.Nic nic = new VmCreateModel.Nic();
                nic.setOvs(networkCard.getOvsbrName());
                nic.setVlanUuid(networkCard.getNetworkUuid());
                VmCreateModel.Nic.Vlan vlan = new VmCreateModel.Nic.Vlan();
                vlan.setVlanId(networkCard.getVlan());
                nic.setVlans(Collections.singletonList(vlan));
                nic.setLink(networkCard.getUsed() ? "up" : "down");
                nics.add(nic);
            }
            params.setNics(nics);

            String taskUuid = vmApi.create(params);

            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("创建云主机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(ServerModel server) {
        try {
            BigDecimal d1024 = new BigDecimal(1024);
            VmModel vmModel = vmApi.get(server.getInstanceId());
            // 基本信息
            VmModifyModel param = new VmModifyModel();
            param.setVmName(server.getName());
            if (server.getCpu() != null) {
                param.setVcpu(server.getCpu());
                param.setCpu(new VmModifyModel.Cpu(new VmModifyModel.Cpu.Topology(1L, server.getCpu().longValue())));
            }
            if (server.getMemory() != null) {
                param.setMemory(server.getMemory().multiply(d1024).multiply(d1024).multiply(d1024).longValue());
            }
            param.setHa(server.getHa() != null ? server.getHa() : vmModel.getHa());
            param.setDescription(server.getRemark());

            String taskUuid = vmApi.modify(server.getInstanceId(), param);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("修改云主机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String instanceId, String name) {
        try {
            String taskUuid = vmApi.remove(instanceId, true, true, true);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("迁移云主机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult migrate(ServerModel server) {
        try {
            String taskUuid = vmApi.migrate(server.getInstanceId(), server.getHostIp(), server.getAutoSchedule());
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("迁移云主机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult cloneAsVm(ServerModel server) {
        try {
            VmModel vmModel = vmApi.get(server.getInstanceId());

            BigDecimal d1024 = new BigDecimal(1024);

            VmCloneModel params = new VmCloneModel();
            params.setVmName(server.getName());
            params.setHa(server.getHa());
            params.setAutoSchedule(server.getAutoSchedule());
            params.setNodeIp(server.getHostIp());
            params.setVcpu(server.getCpu());
            params.setCpu(new VmCloneModel.Cpu(new VmCloneModel.Cpu.Topology(1L, server.getCpu().longValue())));
            params.setMemory(server.getMemory().multiply(d1024).multiply(d1024).multiply(d1024).longValue());
            params.setFirmware(server.getFirmware());
            params.setStatus(server.getStatus());
            params.setDescription(server.getRemark());
            params.setIsFullCopy(server.getFullClone());

            List<VmCloneModel.NewDisk> newDisks = new ArrayList<>();
            List<VmCloneModel.ModifyDisk> modifyDisks = new ArrayList<>();
            List<VmCloneModel.DeleteDisk> deleteDisks = new ArrayList<>();

            for (VolumeModel diskDevice : server.getDiskDevices()) {
                if ("reserveDisk".equals(diskDevice.getType())) {
                    VmCloneModel.ModifyDisk modifyDisk = new VmCloneModel.ModifyDisk();
                    modifyDisk.setType("disk");
                    modifyDisk.setPath(diskDevice.getPath());
                    modifyDisk.setName(diskDevice.getName());
                    if (diskDevice.getBus() != null) {
                        modifyDisk.setBus(diskDevice.getBus().toLowerCase());
                    }
                    modifyDisks.add(modifyDisk);
                } else if ("removeDisk".equals(diskDevice.getType())) {
                    VmCloneModel.DeleteDisk deleteDisk = new VmCloneModel.DeleteDisk();
                    deleteDisk.setType("disk");
                    deleteDisk.setPath(diskDevice.getPath());
                    deleteDisks.add(deleteDisk);
                } else if ("reserveISO".equals(diskDevice.getType())) {
                    VmCloneModel.ModifyDisk modifyDisk = new VmCloneModel.ModifyDisk();
                    modifyDisk.setType("cdrom");
                    modifyDisk.setKey(diskDevice.getKey());
                    modifyDisk.setPath(diskDevice.getPath());
                    if (diskDevice.getBus() != null) {
                        modifyDisk.setBus(diskDevice.getBus().toLowerCase());
                    }
                    modifyDisk.setDisabled(diskDevice.getDisabled());
                    modifyDisks.add(modifyDisk);
                } else if ("removeISO".equals(diskDevice.getType())) {
                    VmCloneModel.DeleteDisk deleteDisk = new VmCloneModel.DeleteDisk();
                    deleteDisk.setType("cdrom");
                    deleteDisk.setKey(diskDevice.getKey());
                    deleteDisk.setPath(diskDevice.getPath());
                    deleteDisks.add(deleteDisk);
                } else if ("newDisk".equals(diskDevice.getType())) {
                    VmCloneModel.NewDisk newDisk = new VmCloneModel.NewDisk();
                    newDisk.setType("disk");
                    if (diskDevice.getBus() != null) {
                        newDisk.setBus(diskDevice.getBus().toLowerCase());
                    }
                    newDisk.setName(diskDevice.getName());
                    newDisk.setSizeInByte(diskDevice.getBlockCount());
                    newDisk.setStoragePolicyUuid(diskDevice.getVolumeTemplateUuid());
                    newDisks.add(newDisk);
                } else if ("mountDisk".equals(diskDevice.getType())) {
                    VmCloneModel.NewDisk newDisk = new VmCloneModel.NewDisk();
                    newDisk.setType("disk");
                    if (diskDevice.getBus() != null) {
                        newDisk.setBus(diskDevice.getBus().toLowerCase());
                    }
                    newDisk.setPath(diskDevice.getPath());
                    newDisks.add(newDisk);
                } else if ("mountISO".equals(diskDevice.getType())) {
                    VmCloneModel.NewDisk newDisk = new VmCloneModel.NewDisk();
                    newDisk.setType("cdrom");
                    if (diskDevice.getBus() != null) {
                        newDisk.setBus(diskDevice.getBus().toLowerCase());
                    }
                    newDisk.setPath(diskDevice.getPath());
                    newDisk.setDisabled(diskDevice.getDisabled());
                    newDisks.add(newDisk);
                }
            }
            for (VmModel.Disk disk : vmModel.getDisks()) {
                if ("cdrom".equals(disk.getType()) && StringUtils.isEmpty(disk.getPath())) {
                    VmCloneModel.DeleteDisk deleteDisk = new VmCloneModel.DeleteDisk();
                    deleteDisk.setType("cdrom");
                    deleteDisk.setKey(disk.getKey());
                    deleteDisks.add(deleteDisk);
                }
            }
            params.setNewDisks(newDisks);
            params.setModifyDisks(modifyDisks);
            params.setDeleteDisks(deleteDisks);

            List<VmCloneModel.Nic> nics = new ArrayList<>();
            for (NetworkCardModel networkCard : server.getNetworkCards()) {
                VmCloneModel.Nic nic = new VmCloneModel.Nic();
                nic.setOvs(networkCard.getOvsbrName());
                nic.setVlanUuid(networkCard.getNetworkUuid());
                VmCloneModel.Nic.Vlan vlan = new VmCloneModel.Nic.Vlan();
                vlan.setVlanId(networkCard.getVlan());
                nic.setVlans(Collections.singletonList(vlan));
                nic.setLink(networkCard.getUsed() ? "up" : "down");
                nics.add(nic);
            }
            params.setNics(nics);

            String taskUuid = vmApi.clone(server.getInstanceId(), params);

            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("创建云主机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult cloneAsTemplate(ServerModel server) {
        try {
            VmCreateVmTemplateModel params = new VmCreateVmTemplateModel();
            params.setVmUuid(server.getInstanceId());
            params.setName(server.getName());
            params.setDescription(server.getRemark());
            params.setCloudInitSupported(server.getCloudInitSupported());
            String taskUuid = vmTemplateApi.create(params);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("云主机克隆为模板失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult convertAsTemplate(ServerModel server) {
        try {
            vmApi.asTemplate(server.getInstanceId(), server.getName(), server.getCloudInitSupported());
            return new BocloudResult(true, "success");
        } catch (Exception e) {
            logger.error("云主机转化为模板失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String instanceId) {
        try {
            ServerModel serverModel = vmConvert.convert(vmApi.get(instanceId));
            return new BocloudResult(true, serverModel, "success");
        } catch (Exception e) {
            logger.error("查询云主机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult start(String instanceId) {
        try {
            String taskUuid = vmApi.start(instanceId);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("云主机开机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult stop(String instanceId) {
        try {
            String taskUuid = vmApi.stop(instanceId, true);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("云主机关机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult softStop(String instanceId) {
        try {
            String taskUuid = vmApi.stop(instanceId, false);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("云主机关机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult reboot(String instanceId) {
        try {
            String taskUuid = vmApi.reboot(instanceId, true);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("云主机重启失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult softReboot(String instanceId) {
        try {
            String taskUuid = vmApi.reboot(instanceId, false);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("云主机重启失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult pause(String instanceId) {
        try {
            String taskUuid = vmApi.pause(instanceId);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("云主机挂起失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult unPause(String instanceId) {
        try {
            String taskUuid = vmApi.resume(instanceId);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("云主机恢复失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult attachVolume(String instanceId, VolumeModel volumeModel) {
        try {
            VmModel vmModel = vmApi.get(instanceId);

            VmModifyModel param = new VmModifyModel();
            param.setDiskPathsDel(new ArrayList<>());

            // 添加原有的卷数据
            List<VmModifyModel.Disk> paramDisks = new ArrayList<>();
            for (VmModel.Disk vmDisk : vmModel.getDisks()) {
                VmModifyModel.Disk paramDisk = new VmModifyModel.Disk();
                if ("disk".equals(vmDisk.getType())) {
                    paramDisk.setType(vmDisk.getType());
                    if (vmDisk.getBus() != null) {
                        paramDisk.setBus(vmDisk.getBus().toLowerCase());
                    }
                    paramDisk.setPath(vmDisk.getPath());
                    paramDisk.setStoragePolicyUuid(vmDisk.getStoragePolicyUuid());
                    volumeApi.list().stream().filter(v -> vmDisk.getPath().equals(v.getPath())).findFirst()
                            .map(v -> v.getSizeInByte()).ifPresent(paramDisk::setSizeInByte);
                } else if ("cdrom".equals(vmDisk.getType())) {
                    paramDisk.setType(vmDisk.getType());
                    if (vmDisk.getBus() != null) {
                        paramDisk.setBus(vmDisk.getBus().toLowerCase());
                    }
                    paramDisk.setPath(vmDisk.getPath());
                }
                paramDisks.add(paramDisk);
            }

            // 添加新增的卷数据
            VmModifyModel.Disk paramDisk = new VmModifyModel.Disk();
            paramDisk.setType(volumeModel.getType());
            paramDisk.setName(volumeModel.getName());
            paramDisk.setPath(volumeModel.getPath());
            paramDisk.setSizeInByte(volumeModel.getBlockCount());
            if (volumeModel.getBus() != null) {
                paramDisk.setBus(volumeModel.getBus().toLowerCase());
            }
            paramDisk.setStoragePolicyUuid(volumeModel.getVolumeTemplateUuid());
            paramDisk.setDisabled(volumeModel.getDisabled());
            paramDisks.add(paramDisk);

            param.setDisks(paramDisks);

            String taskUuid = vmApi.modify(instanceId, param);
            return new BocloudResult(true, taskUuid, "success");

        } catch (Exception e) {
            logger.error("云主机挂载虚拟卷失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detachVolume(String instanceId, VolumeModel volumeModel) {
        try {
            VmModel vmModel = vmApi.get(instanceId);

            VmModifyModel param = new VmModifyModel();
            param.setDiskPathsDel(Collections.singletonList(volumeModel.getPath()));

            // 添加原有的卷数据，要删除的卷除外
            List<VmModifyModel.Disk> paramDisks = new ArrayList<>();
            for (VmModel.Disk vmDisk : vmModel.getDisks()) {
                if (volumeModel.getPath().equals(vmDisk.getPath())) {
                    continue;
                }
                VmModifyModel.Disk paramDisk = new VmModifyModel.Disk();
                if ("disk".equals(vmDisk.getType())) {
                    paramDisk.setType(vmDisk.getType());
                    if (vmDisk.getBus() != null) {
                        paramDisk.setBus(vmDisk.getBus().toLowerCase());
                    }
                    paramDisk.setPath(vmDisk.getPath());
                    paramDisk.setStoragePolicyUuid(vmDisk.getStoragePolicyUuid());
                    volumeApi.list().stream().filter(v -> vmDisk.getPath().equals(v.getPath())).findFirst()
                            .map(v -> v.getSizeInByte()).ifPresent(paramDisk::setSizeInByte);
                } else if ("cdrom".equals(vmDisk.getType())) {
                    paramDisk.setType(vmDisk.getType());
                    if (vmDisk.getBus() != null) {
                        paramDisk.setBus(vmDisk.getBus().toLowerCase());
                    }
                    paramDisk.setPath(vmDisk.getPath());
                }
                paramDisks.add(paramDisk);
            }
            param.setDisks(paramDisks);

            String taskUuid = vmApi.modify(instanceId, param);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("云主机卸载虚拟卷失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult attachNetwork(ServerModel server) {
        try {
            BigDecimal d1024 = new BigDecimal(1024);
            VmModel vmModel = vmApi.get(server.getInstanceId());

            VmModifyModel param = new VmModifyModel();
            List<VmModifyModel.Nic> paramNics = new ArrayList<>();
            for (VmModel.Nic nic : vmModel.getNics()) {
                VmModifyModel.Nic paramNic = new VmModifyModel.Nic();
                paramNic.setOvs(nic.getOvs());
                paramNic.setVlanUuid(nic.getVlanUuid());
                paramNic.setLink(nic.getLink());
                VmModifyModel.Nic.Vlan vlan = new VmModifyModel.Nic.Vlan();
                vlan.setVlanId(nic.getVlans().get(0).getVlanId());
                paramNic.setVlans(Collections.singletonList(vlan));
                paramNics.add(paramNic);
            }
            for (NetcardOperation nic : server.getOperations()) {
                VmModifyModel.Nic paramNic = new VmModifyModel.Nic();
                paramNic.setOvs(nic.getOvs());
                paramNic.setVlanUuid(nic.getNetworkUuid());
                paramNic.setLink(nic.getLink());
                VmModifyModel.Nic.Vlan vlan = new VmModifyModel.Nic.Vlan();
                vlan.setVlanId(nic.getVlan());
                paramNic.setVlans(Collections.singletonList(vlan));
                paramNics.add(paramNic);
            }
            param.setNics(paramNics);

            String taskUuid = vmApi.modify(server.getInstanceId(), param);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("修改云主机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detachNetwork(String instanceId, String macAddresses) {
        try {
            BigDecimal d1024 = new BigDecimal(1024);
            VmModel vmModel = vmApi.get(instanceId);

            VmModifyModel param = new VmModifyModel();
            List<VmModifyModel.Nic> paramNics = new ArrayList<>();
            for (VmModel.Nic nic : vmModel.getNics()) {
                if (macAddresses.contains(nic.getMacAddress())) {
                    continue;
                }
                VmModifyModel.Nic paramNic = new VmModifyModel.Nic();
                paramNic.setOvs(nic.getOvs());
                paramNic.setVlanUuid(nic.getVlanUuid());
                paramNic.setLink(nic.getLink());
                VmModifyModel.Nic.Vlan vlan = new VmModifyModel.Nic.Vlan();
                vlan.setVlanId(nic.getVlans().get(0).getVlanId());
                paramNic.setVlans(Collections.singletonList(vlan));
                paramNics.add(paramNic);
            }
            param.setNics(paramNics);

            String taskUuid = vmApi.modify(instanceId, param);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("修改云主机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        try {
            String taskUuid = vmSnapshotApi.create(snapshotModel.getInstanceId(),
                    snapshotModel.getName(), snapshotModel.getRemark(), Constant.DEFAULT_VM_SNAPSHOT_CONSISTENT_TYPE);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("云主机创建快照失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult vncConsole(String vncType, String serverId) {
        return this.vmrcConsole(serverId);
    }

    @Override
    public BocloudResult vmrcConsole(String instanceId) {
        try {
            VmModel vm = vmApi.get(instanceId);
            List<com.bocloud.cmp.provider.smart.sdk.model.HostModel> hosts = hostApi.list();

            URL url = new URL(this.getButler().getUrl());
            String ip = url.getHost();
            int port = url.getPort() == -1 ? url.getDefaultPort() : url.getPort();
            String uuid = vm.getUuid();
            String name = vm.getVmName();
            String host = hosts.stream().filter(h ->
                    vm.getNodeIp().equals(h.getAccessIp()) ||
                            vm.getNodeIp().equals(h.getDataIp()) ||
                            vm.getNodeIp().equals(h.getHypervisorIp()) ||
                            vm.getNodeIp().equals(h.getManagementIp()) ||
                            vm.getNodeIp().equals(h.getRpcIp()) ||
                            vm.getNodeIp().equals(h.getIpmiIp())
            ).findFirst().map(com.bocloud.cmp.provider.smart.sdk.model.HostModel::getManagementIp).orElse(null);
            String token = new SessionManager().getSession(this.getSessionParams()).getToken();
            String origin = this.getButler().getUrl() + "/terminal.html" +
                    "?uuid=" + URLEncoder.encode(uuid, StandardCharsets.UTF_8.name()) +
                    "&vmname=" + URLEncoder.encode(name, StandardCharsets.UTF_8.name()) +
                    "&host=" + URLEncoder.encode(host, StandardCharsets.UTF_8.name());

            Map<String, Object> result = MapBuilder.init().put("ip", ip).put("port", port).put("uuid", uuid)
                    .put("name", name).put("host", host).put("origin", origin).put("token", token).get();

            return new BocloudResult(true, result, "success");
        } catch (Exception e) {
            logger.error("打开云主机控制台失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
