package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.SmartProvider;
import com.bocloud.cmp.provider.smart.sdk.api.VmTemplateApi;
import com.bocloud.cmp.provider.smart.sdk.convert.VmTemplateConvert;
import com.bocloud.cmp.provider.smart.sdk.model.VmModel;
import com.bocloud.cmp.provider.smart.sdk.model.create.VmTemplateCreateModel;
import com.bocloud.cmp.provider.smart.sdk.model.other.VmTemplateCreateVmsModel;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

@Provider("bocloud.cmp.smartx")
public class SmartServerTemplateProvider extends SmartProvider implements ServerTemplateProvider {

    private static final Logger logger = LoggerFactory.getLogger(SmartServerTemplateProvider.class);

    private final VmTemplateApi vmTemplateApi;
    private final VmTemplateConvert vmTemplateConvert = new VmTemplateConvert();

    public SmartServerTemplateProvider(Butler butler) {
        super(butler);
        vmTemplateApi = new VmTemplateApi(this.getSessionParams());
    }

    @Override
    public BocloudResult list(ServerTemplateModel serverTemplateModel) {
        try {
            List<ServerModel> serverModels = vmTemplateConvert.convert(vmTemplateApi.list());
            return new BocloudResult(true, serverModels, "success");
        } catch (Exception e) {
            logger.error("查询云主机模板失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String uuid) {
        try {
            ServerModel serverModel = vmTemplateConvert.convert(vmTemplateApi.get(uuid));
            return new BocloudResult(true, serverModel, "success");
        } catch (Exception e) {
            logger.error("查询云主机模板失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(ServerTemplateModel serverTemplateModel) {
        try {
            BigDecimal d1024 = new BigDecimal(1024);

            VmTemplateCreateModel params = new VmTemplateCreateModel();
            params.setName(serverTemplateModel.getName());
            params.setHa(serverTemplateModel.getHa());
            params.setVcpu(serverTemplateModel.getCpu());
            params.setCpu(new VmTemplateCreateModel.Cpu(new VmTemplateCreateModel.Cpu.Topology(1L, serverTemplateModel.getCpu().longValue())));
            params.setMemory(serverTemplateModel.getMemory().multiply(d1024).multiply(d1024).multiply(d1024).longValue());
            params.setFirmware(serverTemplateModel.getFirmware());
            params.setDescription(serverTemplateModel.getRemark());

            List<VmTemplateCreateModel.Disk> disks = new ArrayList<>();
            for (VolumeModel diskDevice : serverTemplateModel.getVolumeModels()) {
                VmTemplateCreateModel.Disk disk = new VmTemplateCreateModel.Disk();
                disk.setType(diskDevice.getType());
                disk.setName(diskDevice.getName());
                disk.setPath(diskDevice.getPath());
                disk.setSizeInByte(diskDevice.getBlockCount());
                if (diskDevice.getBus() != null) {
                    disk.setBus(diskDevice.getBus().toLowerCase());
                }
                disk.setStoragePolicyUuid(diskDevice.getVolumeTemplateUuid());
                disk.setDisabled(diskDevice.getDisabled());
                disks.add(disk);
            }
            params.setDisks(disks);

            List<VmTemplateCreateModel.Nic> nics = new ArrayList<>();
            for (NetworkCardModel networkCard : serverTemplateModel.getNetworkCardModels()) {
                VmTemplateCreateModel.Nic nic = new VmTemplateCreateModel.Nic();
                nic.setOvs(networkCard.getOvsbrName());
                nic.setVlanUuid(networkCard.getNetworkUuid());
                VmTemplateCreateModel.Nic.Vlan vlan = new VmTemplateCreateModel.Nic.Vlan();
                vlan.setVlanId(networkCard.getVlan());
                nic.setVlans(Collections.singletonList(vlan));
                nic.setLink(networkCard.getUsed() ? "up" : "down");
                nics.add(nic);
            }
            params.setNics(nics);

            String taskUuid = vmTemplateApi.create(params);

            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("创建云主机模板失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(ServerTemplateModel serverTemplateModel) {
        try {
            String taskUuid = vmTemplateApi.remove(serverTemplateModel.getTemplateUuid());
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("删除云主机模板失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(ServerTemplateModel serverTemplateModel) {
        try {
            vmTemplateApi.modify(
                    serverTemplateModel.getTemplateUuid(),
                    serverTemplateModel.getName(),
                    serverTemplateModel.getRemark()
            );
            return new BocloudResult(true, "success");
        } catch (Exception e) {
            logger.error("修改云主机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult createServer(String uuid, ServerModel server) {
        try {
            VmModel vmModel = vmTemplateApi.get(uuid);

            BigDecimal d1024 = new BigDecimal(1024);

            VmTemplateCreateVmsModel.Vm vm = new VmTemplateCreateVmsModel.Vm();
            vm.setVmName(server.getName());
            vm.setHa(server.getHa());
            vm.setAutoSchedule(server.getAutoSchedule());
            vm.setNodeIp(server.getHostIp());
            vm.setVcpu(server.getCpu());
            vm.setCpu(new VmTemplateCreateVmsModel.Vm.Cpu(new VmTemplateCreateVmsModel.Vm.Cpu.Topology(1L, server.getCpu().longValue())));
            vm.setMemory(server.getMemory().multiply(d1024).multiply(d1024).multiply(d1024).longValue());
            vm.setFirmware(server.getFirmware());
            vm.setStatus(server.getStatus());
            vm.setDescription(server.getRemark());
            vm.setIsFullCopy(server.getFullClone());

            List<VmTemplateCreateVmsModel.Vm.NewDisk> newDisks = new ArrayList<>();
            List<VmTemplateCreateVmsModel.Vm.ModifyDisk> modifyDisks = new ArrayList<>();
            List<VmTemplateCreateVmsModel.Vm.DeleteDisk> deleteDisks = new ArrayList<>();

            for (VolumeModel diskDevice : server.getDiskDevices()) {
                if ("reserveDisk".equals(diskDevice.getType())) {
                    VmTemplateCreateVmsModel.Vm.ModifyDisk modifyDisk = new VmTemplateCreateVmsModel.Vm.ModifyDisk();
                    modifyDisk.setType("disk");
                    modifyDisk.setPath(diskDevice.getPath());
                    modifyDisk.setName(diskDevice.getName());
                    if (diskDevice.getBus() != null) {
                        modifyDisk.setBus(diskDevice.getBus().toLowerCase());
                    }
                    modifyDisk.setVolumeTemplateUuid(diskDevice.getTemplateUuid());
                    modifyDisks.add(modifyDisk);
                } else if ("removeDisk".equals(diskDevice.getType())) {
                    VmTemplateCreateVmsModel.Vm.DeleteDisk deleteDisk = new VmTemplateCreateVmsModel.Vm.DeleteDisk();
                    deleteDisk.setType("disk");
                    deleteDisk.setPath(diskDevice.getPath());
                    deleteDisks.add(deleteDisk);
                } else if ("reserveISO".equals(diskDevice.getType())) {
                    VmTemplateCreateVmsModel.Vm.ModifyDisk modifyDisk = new VmTemplateCreateVmsModel.Vm.ModifyDisk();
                    modifyDisk.setType("cdrom");
                    modifyDisk.setKey(diskDevice.getKey());
                    modifyDisk.setPath(diskDevice.getPath());
                    if (diskDevice.getBus() != null) {
                        modifyDisk.setBus(diskDevice.getBus().toLowerCase());
                    }
                    modifyDisk.setDisabled(diskDevice.getDisabled());
                    modifyDisks.add(modifyDisk);
                } else if ("removeISO".equals(diskDevice.getType())) {
                    VmTemplateCreateVmsModel.Vm.DeleteDisk deleteDisk = new VmTemplateCreateVmsModel.Vm.DeleteDisk();
                    deleteDisk.setType("cdrom");
                    deleteDisk.setKey(diskDevice.getKey());
                    deleteDisk.setPath(diskDevice.getPath());
                    deleteDisks.add(deleteDisk);
                } else if ("newDisk".equals(diskDevice.getType())) {
                    VmTemplateCreateVmsModel.Vm.NewDisk newDisk = new VmTemplateCreateVmsModel.Vm.NewDisk();
                    newDisk.setType("disk");
                    if (diskDevice.getBus() != null) {
                        newDisk.setBus(diskDevice.getBus().toLowerCase());
                    }
                    newDisk.setName(diskDevice.getName());
                    newDisk.setSizeInByte(diskDevice.getBlockCount());
                    newDisk.setStoragePolicyUuid(diskDevice.getVolumeTemplateUuid());
                    newDisks.add(newDisk);
                } else if ("mountDisk".equals(diskDevice.getType())) {
                    VmTemplateCreateVmsModel.Vm.NewDisk newDisk = new VmTemplateCreateVmsModel.Vm.NewDisk();
                    newDisk.setType("disk");
                    if (diskDevice.getBus() != null) {
                        newDisk.setBus(diskDevice.getBus().toLowerCase());
                    }
                    newDisk.setPath(diskDevice.getPath());
                    newDisks.add(newDisk);
                } else if ("mountISO".equals(diskDevice.getType())) {
                    VmTemplateCreateVmsModel.Vm.NewDisk newDisk = new VmTemplateCreateVmsModel.Vm.NewDisk();
                    newDisk.setType("cdrom");
                    if (diskDevice.getBus() != null) {
                        newDisk.setBus(diskDevice.getBus().toLowerCase());
                    }
                    newDisk.setPath(diskDevice.getPath());
                    newDisk.setDisabled(diskDevice.getDisabled());
                    newDisks.add(newDisk);
                }
            }
            vm.setNewDisks(newDisks);
            vm.setModifyDisks(modifyDisks);
            vm.setDeleteDisks(deleteDisks);

            List<VmTemplateCreateVmsModel.Vm.Nic> nics = new ArrayList<>();
            List<VmTemplateCreateVmsModel.Vm.CloudInit.NetworkInfo.Network> networks = new ArrayList<>();
            for (int i = 0; i < server.getNetworkCards().size(); i++) {
                NetworkCardModel networkCard = server.getNetworkCards().get(i);

                VmTemplateCreateVmsModel.Vm.Nic nic = new VmTemplateCreateVmsModel.Vm.Nic();
                nic.setOvs(networkCard.getOvsbrName());
                nic.setVlanUuid(networkCard.getNetworkUuid());
                VmTemplateCreateVmsModel.Vm.Nic.Vlan vlan = new VmTemplateCreateVmsModel.Vm.Nic.Vlan();
                vlan.setVlanId(networkCard.getVlan());
                nic.setVlans(Collections.singletonList(vlan));
                nic.setLink(networkCard.getUsed() ? "up" : "down");
                nics.add(nic);


                if (networkCard.getDhcp() != null) {
                    VmTemplateCreateVmsModel.Vm.CloudInit.NetworkInfo.Network network = new VmTemplateCreateVmsModel.Vm.CloudInit.NetworkInfo.Network();
                    network.setNicIndex(i);
                    if (networkCard.getDhcp()) {
                        network.setType("ipv4_dhcp");
                    } else {
                        network.setType("ipv4");
                        network.setIpAddress(networkCard.getIpAddress());
                        network.setNetmask(networkCard.getNetmask());
                    }
                    networks.add(network);
                }

            }
            vm.setNics(nics);

            if (CollectionUtils.isNotEmpty(networks)) {
                vm.getCloudInitSafely().getNetworkInfoSafely().setNetworks(networks);
            }
            if (StringUtils.isNotEmpty(server.getGateway())) {
                vm.getCloudInitSafely().getNetworkInfoSafely().setGateway(server.getGateway());
            }
            if (StringUtils.isNotEmpty(server.getDnsServers())) {
                vm.getCloudInitSafely().getNetworkInfoSafely().setNameservers(Arrays.asList(server.getDnsServers().split(",")));
            }
            if (StringUtils.isNotEmpty(server.getVmHostName())) {
                vm.getCloudInitSafely().setHostname(server.getVmHostName());
            }
            if (StringUtils.isNotEmpty(server.getPassword())) {
                vm.getCloudInitSafely().setDefaultUserPassword(server.getPassword());
            }

            VmTemplateCreateVmsModel params = new VmTemplateCreateVmsModel();
            params.setVms(Collections.singletonList(vm));

            String taskUuid = vmTemplateApi.createVms(uuid, params);

            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("从模板创建云主机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult asServer(String uuid, ServerModel serverModel) {
        try {
            vmTemplateApi.asVm(uuid, serverModel.getName());
            return new BocloudResult(true, "success");
        } catch (Exception e) {
            logger.error("将模板转化云主机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
