package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.SmartProvider;
import com.bocloud.cmp.provider.smart.sdk.api.VmSnapshotApi;
import com.bocloud.cmp.provider.smart.sdk.api.VolumeSnapshotApi;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.convert.VmSnapshotConvert;
import com.bocloud.cmp.provider.smart.sdk.convert.VolumeSnapshotConvert;
import com.bocloud.cmp.provider.smart.sdk.model.VmModel;
import com.bocloud.cmp.provider.smart.sdk.model.other.VmSnapshotRebuildModel;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Provider("bocloud.cmp.smartx")
public class SmartSnapshotProvider extends SmartProvider implements SnapshotProvider {

    private static final Logger logger = LoggerFactory.getLogger(SmartSnapshotProvider.class);

    private final VmSnapshotApi vmSnapshotApi;
    private final VolumeSnapshotApi volumeSnapshotApi;

    private final VmSnapshotConvert vmSnapshotConvert = new VmSnapshotConvert();
    private final VolumeSnapshotConvert volumeSnapshotConvert = new VolumeSnapshotConvert();


    public SmartSnapshotProvider(Butler butler) {
        super(butler);
        vmSnapshotApi = new VmSnapshotApi(this.getSessionParams());
        volumeSnapshotApi = new VolumeSnapshotApi(this.getSessionParams());
    }

    @Override
    public BocloudResult list() {
        try {
            List<SnapshotModel> allSnapshots = new ArrayList<>();
            allSnapshots.addAll(vmSnapshotConvert.convert(vmSnapshotApi.list()));
            allSnapshots.addAll(volumeSnapshotConvert.convert(volumeSnapshotApi.list()));
            return new BocloudResult(true, allSnapshots, "success");
        } catch (Exception e) {
            logger.error("查询快照失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(SnapshotModel model) {
        try {
            if ("VM".equals(model.getSnapshotType())) {
                SnapshotModel snapshotModel = vmSnapshotConvert.convert(vmSnapshotApi.get(model.getSnapshotUuid()));
                return new BocloudResult(true, snapshotModel, "success");
            } else if ("VOLUME".equals(model.getSnapshotType())) {
                SnapshotModel snapshotModel = volumeSnapshotConvert.convert(volumeSnapshotApi.get(model.getSnapshotUuid()));
                return new BocloudResult(true, snapshotModel, "success");
            } else {
                return new BocloudResult(false, "未知的参数snapshotType的值：" + model.getSnapshotType());
            }
        } catch (Exception e) {
            logger.error("查询快照失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(SnapshotModel model) {
        try {
            if ("VM".equals(model.getSnapshotType())) {
                String taskUuid = vmSnapshotApi.create(model.getInstanceId(), model.getName(),
                        model.getRemark(), Constant.DEFAULT_VM_SNAPSHOT_CONSISTENT_TYPE);
                return new BocloudResult(true, taskUuid, "success");
            } else if ("VOLUME".equals(model.getSnapshotType())) {
                String taskUuid = volumeSnapshotApi.create(model.getVolumeUuid(), model.getName(), model.getRemark());
                return new BocloudResult(true, taskUuid, "success");
            } else {
                return new BocloudResult(false, "未知的参数snapshotType的值：" + model.getSnapshotType());
            }
        } catch (Exception e) {
            logger.error("创建快照失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        try {
            if ("VM".equals(model.getSnapshotType())) {
                String taskUuid = vmSnapshotApi.remove(model.getSnapshotUuid());
                return new BocloudResult(true, taskUuid, "success");
            } else if ("VOLUME".equals(model.getSnapshotType())) {
                String taskUuid = volumeSnapshotApi.remove(model.getSnapshotUuid());
                return new BocloudResult(true, taskUuid, "success");
            } else {
                return new BocloudResult(false, "未知的参数snapshotType的值：" + model.getSnapshotType());
            }
        } catch (Exception e) {
            logger.error("删除快照失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        try {
            if ("VM".equals(model.getSnapshotType())) {
                String taskUuid = vmSnapshotApi.rollback(model.getSnapshotUuid());
                return new BocloudResult(true, taskUuid, "success");
            } else if ("VOLUME".equals(model.getSnapshotType())) {
                String taskUuid = volumeSnapshotApi.rollback(model.getSnapshotUuid());
                return new BocloudResult(true, taskUuid, "success");
            } else {
                return new BocloudResult(false, "未知的参数snapshotType的值：" + model.getSnapshotType());
            }
        } catch (Exception e) {
            logger.error("恢复快照失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult rebuild(SnapshotModel model) {
        try {
            if ("VM".equals(model.getSnapshotType())) {
                VmModel vmModel = vmSnapshotApi.get(model.getSnapshotUuid());
                ServerModel server = model.getRebuildServer();

                BigDecimal d1024 = new BigDecimal(1024);

                VmSnapshotRebuildModel params = new VmSnapshotRebuildModel();
                params.setVmName(server.getName());
                params.setHa(server.getHa());
                params.setAutoSchedule(server.getAutoSchedule());
                params.setNodeIp(server.getHostIp());
                params.setVcpu(server.getCpu());
                params.setCpu(new VmSnapshotRebuildModel.Cpu(new VmSnapshotRebuildModel.Cpu.Topology(1L, server.getCpu().longValue())));
                params.setMemory(server.getMemory().multiply(d1024).multiply(d1024).multiply(d1024).longValue());
                params.setStatus(server.getStatus());
                params.setDescription(server.getRemark());
                params.setIsFullCopy(server.getFullClone());

                List<VmSnapshotRebuildModel.Disk> newDisks = new ArrayList<>();
                List<VmSnapshotRebuildModel.Disk> modifyDisks = new ArrayList<>();

                for (VolumeModel diskDevice : server.getDiskDevices()) {
                    if ("reserveDisk".equals(diskDevice.getType())) {
                        VmSnapshotRebuildModel.Disk modifyDisk = new VmSnapshotRebuildModel.Disk();
                        modifyDisk.setType("disk");
                        modifyDisk.setPath(diskDevice.getPath());
                        modifyDisk.setName(diskDevice.getName());
                        if (diskDevice.getBus() != null) {
                            modifyDisk.setBus(diskDevice.getBus().toLowerCase());
                        }
                        modifyDisks.add(modifyDisk);
                    } else if ("removeDisk".equals(diskDevice.getType())) {
                        // 云主机快照重建云主机不支持删除虚拟卷
                    } else if ("reserveISO".equals(diskDevice.getType())) {
                        VmSnapshotRebuildModel.Disk modifyDisk = new VmSnapshotRebuildModel.Disk();
                        modifyDisk.setType("cdrom");
                        modifyDisk.setKey(diskDevice.getKey());
                        modifyDisk.setPath(diskDevice.getPath());
                        if (diskDevice.getBus() != null) {
                            modifyDisk.setBus(diskDevice.getBus().toLowerCase());
                        }
                        modifyDisk.setDisabled(diskDevice.getDisabled());
                        modifyDisks.add(modifyDisk);
                    } else if ("removeISO".equals(diskDevice.getType())) {
                        // 云主机快照重建云主机不支持删除ISO映像
                    } else if ("newDisk".equals(diskDevice.getType())) {
                        VmSnapshotRebuildModel.Disk newDisk = new VmSnapshotRebuildModel.Disk();
                        newDisk.setType("disk");
                        if (diskDevice.getBus() != null) {
                            newDisk.setBus(diskDevice.getBus().toLowerCase());
                        }
                        newDisk.setName(diskDevice.getName());
                        newDisk.setSizeInByte(diskDevice.getBlockCount());
                        newDisk.setStoragePolicyUuid(diskDevice.getVolumeTemplateUuid());
                        newDisks.add(newDisk);
                    } else if ("mountDisk".equals(diskDevice.getType())) {
                        VmSnapshotRebuildModel.Disk newDisk = new VmSnapshotRebuildModel.Disk();
                        newDisk.setType("disk");
                        if (diskDevice.getBus() != null) {
                            newDisk.setBus(diskDevice.getBus().toLowerCase());
                        }
                        newDisk.setPath(diskDevice.getPath());
                        newDisks.add(newDisk);
                    } else if ("mountISO".equals(diskDevice.getType())) {
                        VmSnapshotRebuildModel.Disk newDisk = new VmSnapshotRebuildModel.Disk();
                        newDisk.setType("cdrom");
                        if (diskDevice.getBus() != null) {
                            newDisk.setBus(diskDevice.getBus().toLowerCase());
                        }
                        newDisk.setPath(diskDevice.getPath());
                        newDisk.setDisabled(diskDevice.getDisabled());
                        newDisks.add(newDisk);
                    }
                }
                for (VmModel.Disk disk : vmModel.getDisks()) {
                    if ("cdrom".equals(disk.getType()) && StringUtils.isEmpty(disk.getPath())) {
                        VmSnapshotRebuildModel.Disk modifyDisk = new VmSnapshotRebuildModel.Disk();
                        modifyDisk.setType("cdrom");
                        modifyDisk.setKey(disk.getKey());
                        modifyDisk.setPath(disk.getPath());
                        if (disk.getBus() != null) {
                            modifyDisk.setBus(disk.getBus().toLowerCase());
                        }
                        modifyDisk.setDisabled(disk.getDisabled());
                        modifyDisks.add(modifyDisk);
                    }
                }
                params.setNewDisks(newDisks);
                params.setModifyDisks(modifyDisks);

                List<VmSnapshotRebuildModel.Nic> nics = new ArrayList<>();
                for (NetworkCardModel networkCard : server.getNetworkCards()) {
                    VmSnapshotRebuildModel.Nic nic = new VmSnapshotRebuildModel.Nic();
                    nic.setOvs(networkCard.getOvsbrName());
                    nic.setVlanUuid(networkCard.getNetworkUuid());
                    VmSnapshotRebuildModel.Nic.Vlan vlan = new VmSnapshotRebuildModel.Nic.Vlan();
                    vlan.setVlanId(networkCard.getVlan());
                    nic.setVlans(Collections.singletonList(vlan));
                    nic.setLink(networkCard.getUsed() ? "up" : "down");
                    nics.add(nic);
                }
                params.setNics(nics);

                String taskUuid = vmSnapshotApi.rebuild(model.getSnapshotUuid(), params);
                return new BocloudResult(true, taskUuid, "success");
            } else if ("VOLUME".equals(model.getSnapshotType())) {
                VolumeModel rebuildVolume = model.getRebuildVolume();
                String taskUuid = volumeSnapshotApi.rebuild(model.getSnapshotUuid(),
                        rebuildVolume.getName(), rebuildVolume.getRemark());
                return new BocloudResult(true, taskUuid, "success");
            } else {
                return new BocloudResult(false, "未知的参数snapshotType的值：" + model.getSnapshotType());
            }
        } catch (Exception e) {
            logger.error("重建快照失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
