package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.SmartProvider;
import com.bocloud.cmp.provider.smart.sdk.api.VdsApi;
import com.bocloud.cmp.provider.smart.sdk.api.VlanApi;
import com.bocloud.cmp.provider.smart.sdk.convert.VdsConvert;
import com.bocloud.cmp.provider.smart.sdk.convert.VlanConvert;
import com.bocloud.cmp.provider.smart.sdk.model.VlanModel;
import com.bocloud.cmp.provider.smart.sdk.model.create.VlanCreateModel;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Provider("bocloud.cmp.smartx")
public class SmartNetworkProvider extends SmartProvider implements NetworkProvider {

    private static final Logger logger = LoggerFactory.getLogger(SmartNetworkProvider.class);
    private final VlanApi vlanApi;
    private final VdsApi vdsApi;
    private final VlanConvert vlanConvert = new VlanConvert();
    private final VdsConvert vdsConvert = new VdsConvert();

    public SmartNetworkProvider(Butler butler) {
        super(butler);
        this.vdsApi = new VdsApi(this.getSessionParams());
        this.vlanApi = new VlanApi(this.getSessionParams());
    }

    @Override
    public BocloudResult list() {
        try {
            List<SwitcherModel> switcherModels = vdsConvert.convert(vdsApi.list());
            for (SwitcherModel switcherModel : switcherModels) {
                List<NetworkModel> networkModels = vlanConvert.convert(vlanApi.list(switcherModel.getUuid()));
                switcherModel.setNetworks(networkModels);
            }
            return new BocloudResult<>(true, switcherModels, "success");
        } catch (Exception e) {
            logger.error("查询虚拟网络失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult listVSwitch(SwitchModel model) {
        try {
            List<NetworkModel> networkModels = vlanConvert.convert(vlanApi.list(model.getSwitchUuid()));
            return new BocloudResult<>(true, networkModels, "success");
        } catch (Exception e) {
            logger.error("查询虚拟网络失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String switcherUuid, String networkId) {
        try {
            NetworkModel networkModel = vlanConvert.convert(vlanApi.get(switcherUuid, networkId));
            return new BocloudResult<>(true, networkModel, "success");
        } catch (Exception e) {
            logger.error("查询虚拟网络失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(NetworkModel network) {
        try {
            SubnetModel subnet = network.getSubnets().get(0);

            VlanCreateModel param = new VlanCreateModel();
            param.setName(network.getName());
            param.setVlanId(subnet.getVlan());

            if (subnet.getDhcpEnabled()) {
                VlanCreateModel.DhcpConfig dhcpConfig = new VlanCreateModel.DhcpConfig();
                dhcpConfig.setCidr(subnet.getCidr());
                dhcpConfig.setGatewayIp(subnet.getGateway());
                dhcpConfig.setDhcpInterfaceIp(subnet.getDhcpServerIp());

                if (StringUtils.isNotEmpty(subnet.getIpPools())) {
                    List<String> ipPools = JSON.parseArray(subnet.getIpPools(), String.class);
                    List<VlanCreateModel.DhcpConfig.IpRange> ipRanges = new ArrayList<>();
                    for (String ipPool : ipPools) {
                        String[] ipDuan = ipPool.split("-");
                        String ipStart = ipDuan[0].trim();
                        String ipEnd = ipDuan[1].trim();
                        VlanCreateModel.DhcpConfig.IpRange ipRange = new VlanCreateModel.DhcpConfig.IpRange();
                        ipRange.setStartIp(ipStart);
                        ipRange.setEndIp(ipEnd);
                        ipRanges.add(ipRange);
                    }
                    dhcpConfig.setIpRanges(ipRanges);
                }

                if (StringUtils.isNotEmpty(subnet.getDnsNames())) {
                    String[] dnsNames = subnet.getDnsNames().split(",");
                    dhcpConfig.setDnsServers(Arrays.asList(dnsNames));
                }
                param.setDhcpConfig(dhcpConfig);
            }

            VlanModel vlanModel = vlanApi.create(network.getSwitchUUid(), param);
            NetworkModel networkModel = vlanConvert.convert(vlanModel);
            return new BocloudResult<>(true, networkModel, "success");
        } catch (Exception e) {
            logger.error("创建虚拟网络失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(NetworkModel model) {
        try {
            VlanModel vlanModel = vlanApi.get(model.getSwitchUUid(), model.getNetworkUuid());
            NetworkModel networkModel = vlanConvert.convert(
                    vlanApi.modify(model.getSwitchUUid(), model.getNetworkUuid(), model.getName(), vlanModel.getVlanId())
            );
            return new BocloudResult<>(true, networkModel, "success");
        } catch (Exception e) {
            logger.error("修改虚拟网络失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String switcherUuid, String networkId) {
        try {
            vlanApi.remove(switcherUuid, networkId);
            return new BocloudResult<>(true, "success");
        } catch (Exception e) {
            logger.error("删除虚拟网络失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
