package com.bocloud.cmp.provider.smart.sdk.http;

import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.util.EntityUtils;
import org.slf4j.LoggerFactory;

import java.io.IOException;

public class LoggingInvoker implements HttpInvoker {

    private interface Logger {
        void log(String format, Object... arguments);
    }

    private static org.slf4j.Logger logger = LoggerFactory.getLogger(LoggingInvoker.class);

    private final HttpInvoker target;

    public LoggingInvoker(HttpInvoker target) {
        this.target = target;
    }

    private void log(Header[] headers, String prefix, Logger logger) {
        if (headers == null) {
            return;
        }
        for (Header header : headers) {
            logger.log("{} [HEADER] {}={}", prefix, header.getName(), header.getValue());
        }
    }

    private void log(HttpEntity entity, String prefix, Logger logger) {
        if (entity == null) {
            return;
        }
        if (entity.getContentType() != null) {
            logger.log("{} [HEADER] {}={}", prefix, entity.getContentType().getName(), entity.getContentType().getValue());
        }
        if (entity.getContentLength() > 0) {
            if (entity.isRepeatable()) {
                String body;
                try {
                    body = EntityUtils.toString(entity).replaceAll("\r", " ").replace("\n", " ");
                } catch (IOException e) {
                    body = "read entity error: " + e.getMessage();
                }
                logger.log("{} [ENTITY] {}", prefix, body);
            } else {
                logger.log("{} [ENTITY] entity is unrepeatable", prefix);
            }
        }
    }

    private void log(HttpResponse response, Logger logger) {
        if (response == null) {
            return;
        }
        logger.log("==<< [CODE] {}", response.getStatusLine().getStatusCode());
        log(response.getAllHeaders(), "==<<", logger);
        log(response.getEntity(), "==<<", logger);
    }

    private void log(String url, String method, Header[] headers, HttpEntity entity, HttpResponse response, Logger logger) {
        logger.log("========调用Rest API========");
        logger.log("==>> [{}] {} ", method, url);
        log(headers, "==>>", logger);
        log(entity, "==>>", logger);
        log(response, logger);
    }

    @Override
    public HttpResponse invoke(String url, String method, Header[] headers, HttpEntity entity) {
        try {
            HttpResponse response = target.invoke(url, method, headers, entity);
            log(url, method, headers, entity, response, logger::debug);
            return response;
        } catch (Throwable e) {
            log(url, method, headers, entity, null, logger::warn);
            logger.warn("==<< [ERROR] {}", e.getMessage());
            throw e;
        }
    }
}
