package com.bocloud.cmp.provider.storage;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SwitcherModel;
import com.bocloud.cmp.provider.SmartProvider;
import com.bocloud.cmp.provider.smart.sdk.api.HostApi;
import com.bocloud.cmp.provider.smart.sdk.api.VdsApi;
import com.bocloud.cmp.provider.smart.sdk.convert.VdsConvert;
import com.bocloud.cmp.provider.smart.sdk.model.HostModel;
import com.bocloud.cmp.provider.smart.sdk.model.create.VdsCreateModel;
import com.bocloud.cmp.provider.smart.sdk.model.modify.VdsModifyModel;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@Provider("bocloud.cmp.smartx")
public class SmartSwitcherProvider extends SmartProvider implements SwitcherProvider {

    private static final Logger logger = LoggerFactory.getLogger(SmartSwitcherProvider.class);

    private final HostApi hostApi;
    private final VdsApi vdsApi;
    private final VdsConvert vdsConvert = new VdsConvert();

    public SmartSwitcherProvider(Butler butler) {
        super(butler);
        this.vdsApi = new VdsApi(this.getSessionParams());
        this.hostApi = new HostApi(this.getSessionParams());
    }

    @Override
    public BocloudResult list() {
        try {
            List<SwitcherModel> switcherModels = vdsConvert.convert(vdsApi.list());
            return new BocloudResult<>(true, switcherModels, "success");
        } catch (Exception e) {
            logger.error("查询虚拟交换机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String uuid) {
        try {
            SwitcherModel switcherModel = vdsConvert.convert(vdsApi.get(uuid));
            return new BocloudResult<>(true, switcherModel, "success");
        } catch (Exception e) {
            logger.error("查询虚拟交换机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(SwitcherModel model) {
        try {
            List<HostModel> hostModels = hostApi.list();

            VdsCreateModel param = new VdsCreateModel();
            param.setName(model.getName());
            param.setBondMode(model.getBondMode());
            param.setDescription(model.getRemark());

            List<VdsCreateModel.Host> paramHosts = new ArrayList<>();
            for (Map.Entry<String, List<String>> entry : model.getHostNetworkCards().entrySet()) {
                VdsCreateModel.Host paramHost = new VdsCreateModel.Host();
                Optional<HostModel> hostOpt =
                        hostModels.stream().filter(h -> entry.getKey().equals(h.getHostUuid())).findFirst();
                Assert.isTrue(hostOpt.isPresent(), "未找到uuid为" + entry.getKey() + "的主机");
                paramHost.setHostUuid(entry.getKey());
                paramHost.setDataIp(hostOpt.get().getDataIp());
                paramHost.setNicsAssociated(entry.getValue());
                paramHosts.add(paramHost);
            }
            param.setHostsAssociated(paramHosts);

            String taskUuid = vdsApi.create(param);

            return new BocloudResult<>(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("创建虚拟交换机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(SwitcherModel model) {
        try {
            List<HostModel> hostModels = hostApi.list();

            VdsModifyModel param = new VdsModifyModel();
            param.setName(model.getName());
            param.setBondMode(model.getBondMode());
            param.setDescription(model.getRemark());

            List<VdsModifyModel.Host> paramHosts = new ArrayList<>();
            for (Map.Entry<String, List<String>> entry : model.getHostNetworkCards().entrySet()) {
                VdsModifyModel.Host paramHost = new VdsModifyModel.Host();
                Optional<HostModel> hostOpt =
                        hostModels.stream().filter(h -> entry.getKey().equals(h.getHostUuid())).findFirst();
                Assert.isTrue(hostOpt.isPresent(), "未找到uuid为" + entry.getKey() + "的主机");
                paramHost.setHostUuid(entry.getKey());
                paramHost.setDataIp(hostOpt.get().getDataIp());
                paramHost.setNicsAssociated(entry.getValue());
                paramHosts.add(paramHost);
            }
            param.setHostsAssociated(paramHosts);

            String taskUuid = vdsApi.modify(model.getUuid(), param);

            return new BocloudResult<>(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("修改虚拟交换机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SwitcherModel model) {
        try {
            String taskUuid = vdsApi.remove(model.getUuid());
            return new BocloudResult<>(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("删除虚拟交换机失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
