package com.bocloud.cmp.provider.storage;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.SmartProvider;
import com.bocloud.cmp.provider.smart.sdk.api.VmApi;
import com.bocloud.cmp.provider.smart.sdk.api.VolumeApi;
import com.bocloud.cmp.provider.smart.sdk.api.VolumeSnapshotApi;
import com.bocloud.cmp.provider.smart.sdk.convert.VolumeConvert;
import com.bocloud.cmp.provider.smart.sdk.exception.SmartException;
import com.bocloud.cmp.provider.smart.sdk.model.VmModel;
import com.bocloud.cmp.provider.smart.sdk.model.create.VolumeCreateModel;
import com.bocloud.cmp.provider.smart.sdk.model.modify.VmModifyModel;
import com.bocloud.cmp.provider.smart.sdk.model.modify.VolumeModifyModel;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Provider("bocloud.cmp.smartx")
public class SmartVolumeProvider extends SmartProvider implements VolumeProvider {

    private static final Logger logger = LoggerFactory.getLogger(SmartVolumeProvider.class);

    private final VmApi vmApi;
    private final VolumeApi volumeApi;
    private final VolumeSnapshotApi volumeSnapshotApi;
    private final VolumeConvert volumeConvert = new VolumeConvert();

    public SmartVolumeProvider(Butler butler) {
        super(butler);
        vmApi = new VmApi(this.getSessionParams());
        volumeApi = new VolumeApi(this.getSessionParams());
        volumeSnapshotApi = new VolumeSnapshotApi(this.getSessionParams());
    }

    @Override
    public BocloudResult list() {
        try {
            List<VolumeModel> volumeModels = volumeConvert.convert(volumeApi.list());
            return new BocloudResult(true, volumeModels, "success");
        } catch (Exception e) {
            logger.error("查询虚拟卷失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String volumeId) {
        try {
            VolumeModel volumeModel = volumeConvert.convert(volumeApi.get(volumeId));
            return new BocloudResult(true, volumeModel, "success");
        } catch (SmartException e) {
            if ("VOLUME_NOT_FOUND".equals(e.getResult().getEc())) {
                return new BocloudResult(true, null, "success");
            } else {
                logger.error("查询虚拟卷失败: {}", e.getMessage(), e);
                return new BocloudResult(false, e.getMessage());
            }
        } catch (Exception e) {
            logger.error("查询虚拟卷失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }

    }

    @Override
    public BocloudResult create(VolumeModel volume) {
        try {
            VolumeCreateModel param = new VolumeCreateModel();
            param.setType("KVM_VOLUME_ISCSI");
            param.setName(volume.getName());
            param.setSharing(volume.getMultiattach());
            param.setSizeInByte(volume.getBlockCount());
            param.setDescription(volume.getRemark());
            param.setStoragePolicyUuid(volume.getVolumeTemplateUuid());
            String taskUuid = volumeApi.create(param);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("创建虚拟卷失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(VolumeModel volume) {
        try {
            VolumeModifyModel param = new VolumeModifyModel();
            param.setName(volume.getName());
            param.setSizeInByte(volume.getBlockCount());
            param.setDescription(volume.getRemark());
            String taskUuid = volumeApi.modify(volume.getVolumeUuid(), param);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("修改虚拟卷失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String volumeId) {
        try {
            String taskUuid = volumeApi.remove(volumeId, true);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("删除虚拟卷失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        try {
            String taskUuid = volumeSnapshotApi.create(snapshotModel.getVolumeUuid(), snapshotModel.getName(), snapshotModel.getRemark());
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("虚拟卷创建快照失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult attach(String serverId, String volumeId) {
        try {
            VmModel vmModel = vmApi.get(serverId);
            com.bocloud.cmp.provider.smart.sdk.model.VolumeModel volumeModel = volumeApi.get(volumeId);

            VmModifyModel param = new VmModifyModel();
            param.setDiskPathsDel(new ArrayList<>());

            // 添加原有的卷数据
            List<VmModifyModel.Disk> paramDisks = new ArrayList<>();
            for (VmModel.Disk vmDisk : vmModel.getDisks()) {
                VmModifyModel.Disk paramDisk = new VmModifyModel.Disk();
                if ("disk".equals(vmDisk.getType())) {
                    paramDisk.setType(vmDisk.getType());
                    if (vmDisk.getBus() != null) {
                        paramDisk.setBus(vmDisk.getBus().toLowerCase());
                    }
                    paramDisk.setPath(vmDisk.getPath());
                    paramDisk.setStoragePolicyUuid(vmDisk.getStoragePolicyUuid());
                    volumeApi.list().stream().filter(v -> vmDisk.getPath().equals(v.getPath())).findFirst()
                            .map(v -> v.getSizeInByte()).ifPresent(paramDisk::setSizeInByte);
                } else if ("cdrom".equals(vmDisk.getType())) {
                    paramDisk.setType(vmDisk.getType());
                    if (vmDisk.getBus() != null) {
                        paramDisk.setBus(vmDisk.getBus().toLowerCase());
                    }
                    paramDisk.setPath(vmDisk.getPath());
                }
                paramDisks.add(paramDisk);
            }

            // 添加新增的卷数据
            VmModifyModel.Disk paramDisk = new VmModifyModel.Disk();
            paramDisk.setType("disk");
            paramDisk.setBus("virtio");
            paramDisk.setPath(volumeModel.getPath());
            paramDisks.add(paramDisk);

            param.setDisks(paramDisks);

            String taskUuid = vmApi.modify(serverId, param);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("云主机挂载虚拟卷失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detach(String serverId, String volumeId) {
        try {
            VmModel vmModel = vmApi.get(serverId);
            com.bocloud.cmp.provider.smart.sdk.model.VolumeModel volumeModel = volumeApi.get(volumeId);

            VmModifyModel param = new VmModifyModel();
            param.setDiskPathsDel(Collections.singletonList(volumeModel.getPath()));

            // 添加原有的卷数据，要删除的卷除外
            List<VmModifyModel.Disk> paramDisks = new ArrayList<>();
            for (VmModel.Disk vmDisk : vmModel.getDisks()) {
                if (volumeModel.getPath().equals(vmDisk.getPath())) {
                    continue;
                }
                VmModifyModel.Disk paramDisk = new VmModifyModel.Disk();
                if ("disk".equals(vmDisk.getType())) {
                    paramDisk.setType(vmDisk.getType());
                    if (vmDisk.getBus() != null) {
                        paramDisk.setBus(vmDisk.getBus().toLowerCase());
                    }
                    paramDisk.setPath(vmDisk.getPath());
                    paramDisk.setStoragePolicyUuid(vmDisk.getStoragePolicyUuid());
                    volumeApi.list().stream().filter(v -> vmDisk.getPath().equals(v.getPath())).findFirst()
                            .map(v -> v.getSizeInByte()).ifPresent(paramDisk::setSizeInByte);
                } else if ("cdrom".equals(vmDisk.getType())) {
                    paramDisk.setType(vmDisk.getType());
                    if (vmDisk.getBus() != null) {
                        paramDisk.setBus(vmDisk.getBus().toLowerCase());
                    }
                    paramDisk.setPath(vmDisk.getPath());
                }
                paramDisks.add(paramDisk);
            }
            param.setDisks(paramDisks);

            String taskUuid = vmApi.modify(serverId, param);
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("云主机卸载虚拟卷失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult copyVolume(VolumeModel volume) {
        try {
            String taskUuid = volumeApi.clone(volume.getVolumeUuid(), volume.getName(), volume.getRemark());
            return new BocloudResult(true, taskUuid, "success");
        } catch (Exception e) {
            logger.error("云主机卸载虚拟卷失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
