package com.bocloud.cmp.provider;

import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.xml.ws.BindingProvider;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.handler.HeaderCookieExtractionHandler;
import com.bocloud.cmp.provider.handler.HeaderHandlerResolver;
import com.bocloud.cmp.provider.handler.VcSessionHandler;
import com.bocloud.cmp.provider.utils.PbmUtil;
import com.bocloud.common.annotations.Provider;
import com.vmware.pbm.PbmPortType;
import com.vmware.pbm.PbmProfileCategoryEnum;
import com.vmware.pbm.PbmService;
import com.vmware.pbm.PbmServiceInstanceContent;
import com.vmware.vim25.ManagedObjectReference;
import com.vmware.vim25.ServiceContent;
import com.vmware.vim25.UserSession;
import com.vmware.vim25.VimPortType;
import com.vmware.vim25.VimService;

import io.jsonwebtoken.lang.Assert;

/**
 * vcenter存储策略认证类
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年6月15日
 *
 */
@Provider("bocloud.cmp.vmware")
public class PbmProvider extends GenericProvider implements SuperProvider {

	private static Logger logger = LoggerFactory.getLogger(PbmProvider.class);
	public static final String SERVICE_INSTANCE_TYPE = "ServiceInstance";
	public static final String SERVICE_INSTANCE_VALUE = "ServiceInstance";
	public static final String PBM_SERVICE_INSTANCE_TYPE = "PbmServiceInstance";
	private String endpoint;
	private String username;
	private String password;
	private ServiceContent serviceContent;
	private VimService vimService;
	private VimPortType vimPort;
	private UserSession userSession;
	private PbmService pbmService;
	private PbmPortType pbmPort;
	private PbmServiceInstanceContent pbmServiceContent;

	static {
		TrustManager[] trustAllCerts = new TrustManager[] { new X509TrustManager() {
			public X509Certificate[] getAcceptedIssuers() {
				return new X509Certificate[] {};
			}

			public void checkClientTrusted(X509Certificate[] chain, String authType) {
				logger.info("check client trusted");
			}

			public void checkServerTrusted(X509Certificate[] chain, String authType) {
				logger.info("check server trusted");
			}
		} };
		try {
			SSLContext sc = SSLContext.getInstance("TLSv1.2");
			sc.init(null, trustAllCerts, new SecureRandom());
			HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
		}
	}

	public PbmProvider(Butler butler) {
		super(butler);
		this.endpoint = butler.getUrl();
		this.username = butler.getUsername();
		this.password = butler.getPassword();
		if (null == userSession || !validSession()) {
			logger.warn("User Session is not active,init one");
			this.init(endpoint, username, password);
		}
	}

	private void init(String endpoint, String username, String password) {
		try {
			vimService = new VimService();
			HeaderCookieExtractionHandler cookieExtractor = new HeaderCookieExtractionHandler();
			HeaderHandlerResolver handlerResolver = new HeaderHandlerResolver();
			handlerResolver.addHandler(cookieExtractor);
			vimService.setHandlerResolver(handlerResolver);
			vimPort = vimService.getVimPort();
			Map<String, Object> ctxt = ((BindingProvider) vimPort).getRequestContext();
			ctxt.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://" + endpoint + "/sdk/vimService");
			ctxt.put(BindingProvider.SESSION_MAINTAIN_PROPERTY, true);
			ManagedObjectReference ref = new ManagedObjectReference();
			ref.setType(SERVICE_INSTANCE_TYPE);
			ref.setValue(SERVICE_INSTANCE_VALUE);
			serviceContent = vimPort.retrieveServiceContent(ref);
			userSession = vimPort.login(serviceContent.getSessionManager(), username, password, null);
			Assert.notNull(userSession, "获取vcenter用户会话异常");
			String cookie = cookieExtractor.getCookie();
			ctxt = ((BindingProvider) vimPort).getRequestContext();
			String[] tokens = cookie.split(";");
			tokens = tokens[0].split("=");
			String extractedCookie = tokens[1];
			pbmService = new PbmService();
			handlerResolver = new HeaderHandlerResolver();
			handlerResolver.addHandler(new VcSessionHandler(extractedCookie));
			pbmService.setHandlerResolver(handlerResolver);
			pbmPort = pbmService.getPbmPort();
			Map<String, Object> pbmCtxt = ((BindingProvider) pbmPort).getRequestContext();
			pbmCtxt.put(BindingProvider.SESSION_MAINTAIN_PROPERTY, true);
			pbmCtxt.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://" + endpoint + "/pbm");
			ManagedObjectReference pbmRef = new ManagedObjectReference();
			pbmRef.setType(PBM_SERVICE_INSTANCE_TYPE);
			pbmRef.setValue(SERVICE_INSTANCE_VALUE);
			pbmServiceContent = pbmPort.pbmRetrieveServiceContent(pbmRef);
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
		}
	}

	/**
	 * 校验用户session是否已过期
	 * 
	 * @return
	 */
	private boolean validSession() {
		try {
			return vimPort.sessionIsActive(serviceContent.getSessionManager(), userSession.getKey(),
					userSession.getUserName());
		} catch (Exception e) {
			logger.error("Valid session error：{}", e.getMessage());
			return false;
		}
	}

	/**
	 * 终止用户会话
	 */
	public void terminateSession() {
		try {
			if (null != userSession) {
				List<String> sessionIds = new ArrayList<>();
				sessionIds.add(userSession.getKey());
				vimPort.terminateSession(serviceContent.getSessionManager(), sessionIds);
			}
		} catch (Exception e) {
			logger.error("Terminate session error：{}", e.getMessage());
		}
	}

	public ServiceContent getServiceContent() {
		return serviceContent;
	}

	public VimService getVimService() {
		return vimService;
	}

	public VimPortType getVimPort() {
		return vimPort;
	}

	public UserSession getUserSession() {
		return userSession;
	}

	public PbmService getPbmService() {
		return pbmService;
	}

	public PbmPortType getPbmPort() {
		return pbmPort;
	}

	public PbmServiceInstanceContent getPbmServiceContent() {
		return pbmServiceContent;
	}

	@Override
	public boolean accessable() {
		try {
			pbmPort.pbmQueryProfile(pbmServiceContent.getProfileManager(), PbmUtil.getStorageResourceType(),
					PbmProfileCategoryEnum.REQUIREMENT.name());
			return true;
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return false;
		}
	}
}
