package com.bocloud.cmp.provider;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.utils.Util;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Package: com.bocloud.cmp.provider
 * @ClassName: TianyiProvider
 * @Author: 胡文涛
 * @CreateTime: 2020/9/18 10:31
 * @Description: 认证
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiProvider implements SuperProvider {

	private String accessKey;
	private String secretKey;
	private String projectId;
	private String platform = "3";
	private Butler butler;
	private String regionId = "cn-jssz1";
	private HttpClient httpClient = new HttpClient(20 * 1000, PostDataFormat.RAW);
	public static final String DATE_FORMAT = "yyyy-MM-dd'T'HH:mmZ";
	public static final Integer SUCCESS_STATUS_CODE = 800;
	public static final Integer FILE_STATUS_CODE = 900;

	public static final String PROTOCOL = "https://api.ctyun.cn";
	public static final String CONTENT_TYPE = "application/x-www-form-urlencoded;charset=utf-8";
	private static Logger logger = LoggerFactory.getLogger(TianyiProvider.class);

	/**
	 * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
	 *
	 * @param butler
	 *            云平台认证信息
	 */
	public TianyiProvider(Butler butler) {
		this.butler = butler;
		this.accessKey = butler.getAccessKey().trim();
		this.secretKey = butler.getSecretKey().trim();
		if (StringUtils.isEmpty(butler.getProjectId())) {
			throw new RuntimeException("缺少平台项目信息！");
		}
		this.projectId = butler.getProjectId().trim();
		this.regionId = butler.getRegionId() == null ? this.regionId : butler.getRegionId().trim();
	}

	@Override
	public boolean accessable() {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/queryProjectIds";
			JSONObject headers = new JSONObject();

			Result result = doGet(apiUrl, headers);

			if (result.isFailed()) {
				logger.error("查询项目列表失败:" + result.getMessage());
				return false;
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return false;
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("查询项目列表失败：" + object.getString("message"));
				return false;
			}
			List<JSONObject> returnObj = JSONArray.parseArray(object.getString("returnObj"), JSONObject.class);
			if (ListTool.isEmpty(returnObj)) {
				logger.error("项目列表为空");
				return false;
			}
			for (JSONObject project : returnObj) {
				if (project.getString("id").equals(this.getProjectId())) {
					return true;
				}
			}
			logger.error("此项目不存在");
			return false;
		} catch (Exception var5) {
			logger.error("平台认证失败 ： " + var5.getMessage());
			return false;
		}
	}

	private Map<String, Object> buildHeader(String url, Map<String, Object> headers) {
		Map<String, Object> header = new HashMap<String, Object>(8);
		header.put("Content-type", "application/x-www-form-urlencoded");
		Map<String, Object> getHeader = Util.sendGet(url, headers, this.accessKey, this.secretKey, this.projectId,
				this.platform);

		header.putAll(getHeader);
		if (!MapTools.isEmpty(headers)) {
			header.putAll(headers);
		}

		return header;
	}

	public String buildUrl(String apiUrl) {
		return PROTOCOL + apiUrl;
	}

	public Result doGet(String url, Map<String, Object> params) {
		try {
			httpClient.setDataFormat(null);
			return this.httpClient.get(buildHeader(url, params), params, buildUrl(url));
		} finally {
			this.httpClient.close();
		}
	}

	public Result doPut(String url, Map<String, Object> params) {
		try {
			httpClient.setDataFormat(PostDataFormat.RAW);
			return this.httpClient.put(buildHeader(url, null), params, buildUrl(url));
		} finally {
			this.httpClient.close();
		}
	}

	public Result doPost(String url, Map<String, Object> params) {
		try {
			httpClient.setDataFormat(PostDataFormat.FORM_URLENCODED);
			params.put("projectId", this.getProjectId());
			return this.httpClient.post(buildHeader(url, null), params, buildUrl(url));
		} finally {
			this.httpClient.close();
		}
	}

	public Result doDelete(String url, Map<String, Object> params) {
		try {
			httpClient.setDataFormat(PostDataFormat.RAW);
			return this.httpClient.delete(buildHeader(url, null), params, buildUrl(url));
		} finally {
			this.httpClient.close();
		}
	}

	public String getAccessKey() {
		return accessKey;
	}

	public void setAccessKey(String accessKey) {
		this.accessKey = accessKey;
	}

	public String getSecretKey() {
		return secretKey;
	}

	public void setSecretKey(String secretKey) {
		this.secretKey = secretKey;
	}

	public String getProjectId() {
		return projectId;
	}

	public void setProjectId(String projectId) {
		this.projectId = projectId;
	}

	public String getPlatform() {
		return platform;
	}

	public void setPlatform(String platform) {
		this.platform = platform;
	}

	public Butler getButler() {
		return butler;
	}

	public void setButler(Butler butler) {
		this.butler = butler;
	}

	public String getRegionId() {
		return regionId;
	}

	public void setRegionId(String regionId) {
		this.regionId = regionId;
	}

	public static void sleepSomeTime(int sleepTime) {
		try {
			Thread.sleep(sleepTime);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
}
