package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.TianyiProvider;
import com.bocloud.cmp.provider.compute.TianyiClusterProvider;
import com.bocloud.cmp.provider.compute.TianyiImageProvider;
import com.bocloud.cmp.provider.compute.TianyiServerProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.network.*;
import com.bocloud.cmp.provider.security.TianyiKeypairProvider;
import com.bocloud.cmp.provider.storage.TianyiVolumeProvider;
import com.bocloud.cmp.provider.tianyiconvertor.RegionConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import javax.print.attribute.HashAttributeSet;

/**
 * @author Hu Wentao
 * @create 2020-10-13 14:55
 * @Description: 同步数据组装类
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiLocationProvider extends TianyiProvider implements LocationProvider {

    private static Logger logger = LoggerFactory.getLogger(TianyiLocationProvider.class);

    private BeanConvertor<RegionModel, JSONObject> convertor = new RegionConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public TianyiLocationProvider(Butler butler) {
        super(butler);
    }

    // 查询资源池列表
    // GET http://ctyun-api-url/apiproxy/v3/order/getZoneConfig
    public BocloudResult getZoneConfig() {
        try {
            String apiUrl = "/apiproxy/v3/order/getZoneConfig";
            JSONObject headers = new JSONObject();

            Result result = doGet(apiUrl, headers);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "查询资源池列表失败");
            }
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                return new BocloudResult(false, "查询资源池列表失败：" + object.getString("message"));
            }
            List<JSONObject> jsonObjects = JSONArray.parseArray(object.getString("returnObj"), JSONObject.class);
            if (ListTool.isEmpty(jsonObjects)) {
                return new BocloudResult(true, "地域列表为空");
            }
            List<RegionModel> regionModels = convertor.convertList(jsonObjects);
            return new BocloudResult(true, regionModels, "查询资源池列表成功");
        } catch (Exception e) {
            logger.error("查询资源池列表失败 ： " + e);
            return new BocloudResult(false, "查询资源池列表失败", e.getMessage());
        }
    }

    // 查询资源池列表（融合）
    // GET http://ctyun-api-url/apiproxy/v5/order/queryRegions
    public BocloudResult queryRegions() {
        try {
            String apiUrl = "/apiproxy/v5/order/queryRegions";
            JSONObject headers = new JSONObject();
            headers.put("projectId", this.getProjectId());

            Result result = doGet(apiUrl, headers);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "查询资源池列表失败");
            }
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                return new BocloudResult(false, "查询资源池列表失败：" + object.getString("message"));
            }
            List<JSONObject> jsonObjects = JSONArray.parseArray(object.getString("returnObj"), JSONObject.class);
            if (ListTool.isEmpty(jsonObjects)) {
                return new BocloudResult(true, "地域列表为空");
            }
            List<RegionModel> regionModels = convertor.convertList(jsonObjects);
            return new BocloudResult(true, regionModels, "查询资源池列表成功");
        } catch (Exception e) {
            logger.error("查询资源池列表失败 ： " + e);
            return new BocloudResult(false, "查询资源池列表失败", e.getMessage());
        }
    }

    // 查询可用资源池以及可用区(支持多可用区)
    // GET http://ctyun-api-url/apiproxy/v4/order/queryRegionInfo
    @Override
    public BocloudResult listRegions() {
        try {
            String apiUrl = "/apiproxy/v4/order/queryRegionInfo";
            JSONObject headers = new JSONObject();
            headers.put("regionId", this.getRegionId());
            Result result = doGet(apiUrl, headers);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "查询资源池列表失败");
            }
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                return new BocloudResult(false, "查询资源池列表失败：" + object.getString("message"));
            }
            List<JSONObject> jsonObjects = JSONArray.parseArray(object.getString("returnObj"), JSONObject.class);
            if (ListTool.isEmpty(jsonObjects)) {
                return new BocloudResult(true, "地域列表为空");
            }

            BocloudResult computeColony = getComputeColony();
            JSONObject data = (JSONObject) computeColony.getData();
            String zone = data.getString("availability_zone");
            for (JSONObject jsonObject : jsonObjects) {
                if (jsonObject.getString("regionId").equals(this.getRegionId())) {
                    List<String> zoneIds = JSONArray.parseArray(jsonObject.getString("zoneId"), String.class);
                    JSONArray zoneList = new JSONArray();
                    zoneIds.stream().filter(zoneId -> {
                        return zoneId.equals(zone);
                    }).map((Function<String, Object>) zoneList::add).collect(Collectors.toList());
                    jsonObject.put("zoneId", zoneList);
                }
            }

            List<RegionModel> regionModels = convertor.convertList(jsonObjects);

            return new BocloudResult(true, regionModels, "查询资源池列表成功");
        } catch (Exception e) {
            logger.error("查询资源池列表失败 ： " + e);
            return new BocloudResult(false, "查询资源池列表失败", e.getMessage());
        }

    }

    /**
     * @Author: Hu Wentao
     * @Date: 2020/10/13 15:23
     * @Description: 同步数据 private List<ServerModel> serverModels; private
     * List<ImageModel> imageModels; private List<FlavorModel>
     * flavorModels; private List<VolumeModel> volumeModels;
     * private List<SnapshotModel> snapshotModels; private
     * List<SecurityGroupModel> securityGroupModels; private
     * List<RegionModel> regionModels; private List<VpcModel>
     * vpcModels; private List<KeypairModel> keypairModels;
     * private List<SubnetModel> subnetModels; private
     * List<ServerTemplateModel> serverTemplateModel; private
     * List<ResourceGroupModel> resourceGroupModels; private
     * List<FloatingIpModel> floatingIpModels;
     **/
    @SuppressWarnings("unchecked")
    @Override
    public BocloudResult sync() {
        try {
            TianyiSyncModel syncModel = new TianyiSyncModel();

            //同步集群
            TianyiClusterProvider tianyiClusterProvider = new TianyiClusterProvider(this.getButler());
            BocloudResult clusterList = tianyiClusterProvider.list();
            if (clusterList.isFailed()) {
                return new BocloudResult(false, clusterList.getMessage(), "同步集群失败");
            }
            syncModel.setClusterModels((List<ClusterModel>) clusterList.getData());
            // 同步镜像
            TianyiImageProvider tianyiImageProvider = new TianyiImageProvider(this.getButler());
            BocloudResult imageList = tianyiImageProvider.list();
            if (imageList.isFailed()) {
                return new BocloudResult(false, imageList.getMessage(), "同步镜像失败");
            }
            syncModel.setImageModels((List<ImageModel>) imageList.getData());
            // 同步规格
            TianyiFlavorProvider tianyiFlavorProvider = new TianyiFlavorProvider(this.getButler());
            BocloudResult flavorList = tianyiFlavorProvider.sync();
            if (flavorList.isFailed()) {
                return new BocloudResult(false, flavorList.getMessage(), "同步规格失败");
            }

            List<FlavorModel> flavorModels = (List<FlavorModel>) flavorList.getData();
            BocloudResult computeColony = getComputeColony();
            JSONObject data = (JSONObject) computeColony.getData();
            JSONArray flavors = JSONArray.parseArray(data.getString("flavors"));
            List<FlavorModel> flavorModelList = flavorModels.stream().filter(flavorModel -> {
                return flavors.contains(flavorModel.getFlavorUuid());
            }).collect(Collectors.toList());

            syncModel.setFlavorModels(flavorModelList);
            // 同步云硬盘
            TianyiVolumeProvider tianyiVolumeProvider = new TianyiVolumeProvider(this.getButler());
            BocloudResult volumeList = tianyiVolumeProvider.list();
            if (volumeList.isFailed()) {
                return new BocloudResult(false, volumeList.getMessage(), "同步云硬盘失败");
            }
            Map<String, VolumeModel> bootVolumesMap = new HashMap<String, VolumeModel>(8);
            List<VolumeModel> volumeModels = (List<VolumeModel>) volumeList.getData();
            if (!ListTool.isEmpty(volumeModels)) {
                for (VolumeModel volumeModel : volumeModels) {
                    if (volumeModel.getBootVolume()) {
                        bootVolumesMap.put(volumeModel.getInstanceId(), volumeModel);
                    }
                }
            }
            syncModel.setVolumeModels(volumeModels);
            // 同步云主机
            TianyiServerProvider tianyiServerProvider = new TianyiServerProvider(this.getButler());
            BocloudResult serverList = tianyiServerProvider.list();
            if (serverList.isFailed()) {
                return new BocloudResult(false, serverList.getMessage(), "同步云主机失败");
            }
            List<ServerModel> serverModels = (List<ServerModel>) serverList.getData();
            for (ServerModel serverModel : serverModels) {
                if (bootVolumesMap.containsKey(serverModel.getInstanceId())) {
                    VolumeModel volumeModel = bootVolumesMap.get(serverModel.getInstanceId());
                    if (null != volumeModel) {
                        serverModel.setDisk(volumeModel.getSize());
                    }
                }
            }
            syncModel.setServerModels(serverModels);
            // 同步安全组
            TianyiSecurityGroupProvider tianyiSecurityGroupProvider = new TianyiSecurityGroupProvider(this.getButler());
            BocloudResult securityGroupList = tianyiSecurityGroupProvider.list();
            if (securityGroupList.isFailed()) {
                return new BocloudResult(false, securityGroupList.getMessage(), "同步安全组失败");
            }
            syncModel.setSecurityGroupModels((List<SecurityGroupModel>) securityGroupList.getData());
            // 同步地域
            BocloudResult regionList = this.listRegions();
            if (regionList.isFailed()) {
                return new BocloudResult(false, regionList.getMessage(), "同步地域失败");
            }
            syncModel.setRegionModels((List<RegionModel>) regionList.getData());
            // 同步vpc
            TianyiVpcProvider tianyiVpcProvider = new TianyiVpcProvider(this.getButler());
            BocloudResult vpcList = tianyiVpcProvider.list();
            if (vpcList.isFailed()) {
                return new BocloudResult(false, vpcList.getMessage(), "同步vpc失败");
            }
            syncModel.setVpcModels((List<VpcModel>) vpcList.getData());
            // 同步密钥
            TianyiKeypairProvider tianyiKeypairProvider = new TianyiKeypairProvider(this.getButler());
            BocloudResult keypairList = tianyiKeypairProvider.list();
            if (keypairList.isFailed()) {
                return new BocloudResult(false, keypairList.getMessage(), "同步密钥失败");
            }
            syncModel.setKeypairModels((List<KeypairModel>) keypairList.getData());
            // 同步子网
            TianyiSubnetProvider tianyiSubnetProvider = new TianyiSubnetProvider(this.getButler());
            BocloudResult subnetList = tianyiSubnetProvider.list();
            if (subnetList.isFailed()) {
                return new BocloudResult(false, subnetList.getMessage(), "同步子网失败");
            }
            syncModel.setSubnetModels((List<SubnetModel>) subnetList.getData());
            // 同步浮动ip
            TianyiFloatingIpProvider tianyiFloatingIpProvider = new TianyiFloatingIpProvider(this.getButler());
            BocloudResult floatingIpList = tianyiFloatingIpProvider.list();
            if (floatingIpList.isFailed()) {
                return new BocloudResult(false, floatingIpList.getMessage(), "同步浮动ip失败");
            }
            syncModel.setFloatingIpModels((List<FloatingIpModel>) floatingIpList.getData());

            logger.info("end sync ... current region is " + this.getRegionId());
            return new BocloudResult(true, syncModel, "数据同步成功！");
        } catch (Exception e) {
            logger.error("数据同步失败 ： " + e);
            return new BocloudResult(false, "数据同步失败", e.getMessage());
        }
    }

    // 查询projectId
    //GET http://ctyun-api-url/apiproxy/v3/ondemand/queryProjectIds
    public BocloudResult getProject() {
        try {
            String apiUrl = "/apiproxy/v3/ondemand/queryProjectIds";
            JSONObject headers = new JSONObject();

            Result result = doGet(apiUrl, headers);

            if (result.isFailed()) {
                logger.error("查询项目列表失败:" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "查询规则列表失败");
            }
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("查询项目列表失败：" + object.getString("message"));
                return new BocloudResult(false, "查询项目列表失败：" + object.getString("message"));
            }
            List<JSONObject> returnObj = JSONArray.parseArray(object.getString("returnObj"), JSONObject.class);
            if (ListTool.isEmpty(returnObj)) {
                logger.error("项目列表为空");
                return new BocloudResult(true, "项目列表为空");
            }
            for (JSONObject project : returnObj) {
                if (project.getString("id").equals(this.getProjectId())) {
                    return new BocloudResult(true, project, "查询项目列表成功");
                }
            }
            logger.error("此项目不存在");
            return new BocloudResult(false, "此项目不存在");
        } catch (Exception e) {
            logger.error("查询项目列表失败 ： " + e);
            return new BocloudResult(false, "查询项目列表失败", e.getMessage());
        }
    }

    //获取专属计算集群资源
    //GET http://ctyun-api-url /apiproxy/v3/hws/getComputeColony
    public BocloudResult getComputeColony() {
        try {
            String apiUrl = "/apiproxy/v3/hws/getComputeColony";
            JSONObject headers = new JSONObject();

            BocloudResult project = this.getProject();
            if (project.isFailed()) {
                logger.error("查询项目列表失败");
                return new BocloudResult(true, "查询项目列表失败");
            }

            JSONObject projectData = (JSONObject) project.getData();
            headers.put("regionId", projectData.getString("name"));

            Result result = doGet(apiUrl, headers);

            if (result.isFailed()) {
                logger.error("获取专属计算集群资源失败:" + result.getMessage());
                return new BocloudResult(false, "获取专属计算集群资源失败:" + result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                logger.error("获取专属计算集群资源失败");
                return new BocloudResult(false, "获取专属计算集群资源失败");
            }
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("获取专属计算集群资源失败：" + object.getString("message"));
                return new BocloudResult(false, "获取专属计算集群资源失败：" + object.getString("message"));
            }
            JSONObject returnObj = JSONObject.parseObject(object.getString("returnObj"));
            List<JSONObject> projects = JSONArray.parseArray(returnObj.getString("dedicated_cluster"), JSONObject.class);
            for (JSONObject data : projects) {
                if (data.getString("project_id").equals(this.getProjectId())) {
                    return new BocloudResult(true, data, "获取专属计算集群资源成功");
                }
            }

            logger.error("获取专属计算集群资源失败");
            return new BocloudResult(false, "获取专属计算集群资源失败");
        } catch (Exception e) {
            logger.error("获取专属计算集群资源失败 ： " + e);
            return new BocloudResult(false, "获取专属计算集群资源失败", e.getMessage());
        }
    }

}
