package com.bocloud.cmp.provider.compute;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.TianyiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.ImageConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

/**
 * @author Hu Wentao
 * @create 2020-10-10 17:50
 * @Description: 镜像   目前只有一个查询镜像列表的接口
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiImageProvider extends TianyiProvider implements ImageProvider {

    private static Logger logger = LoggerFactory.getLogger(TianyiImageProvider.class);

    private BeanConvertor<ImageModel, JSONObject> convertor = new ImageConvertor();


    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public TianyiImageProvider(Butler butler) {
        super(butler);
    }

    //查询镜像列表
    //GET http://ctyun-api-url/apiproxy/v3/order/getImages
    @Override
    public BocloudResult list() {
        try {
        	String apiUrl = "/apiproxy/v3/order/getImages";
        	JSONObject headers = new JSONObject();
        	headers.put("regionId", this.getRegionId());

        	Result result = doGet(apiUrl, headers);
        	if (result.isFailed()) {
        		return new BocloudResult(false, result.getMessage());
        	}
        	JSONObject object = JSONObject.parseObject(result.getMessage());
        	if (null == object) {
        		logger.error("查询镜像列表失败");
        		return new BocloudResult(false, "查询镜像列表失败");
        	}
        	if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
        		logger.error("查询镜像列表失败：" + object.getString("message"));
        		return new BocloudResult(false, "查询镜像列表失败：" + object.getString("message"));
        	}
            List<JSONObject> returnObj = JSONArray.parseArray(object.getString("returnObj"), JSONObject.class);
            if (ListTool.isEmpty(returnObj)) {
        		return new BocloudResult(true, "镜像列表为空");
        	}
            List<ImageModel> imageModels = convertor.convertList(returnObj);
            for (ImageModel imageModel : imageModels) {
            	imageModel.setRegionId(this.getRegionId());
			}
            return new BocloudResult(true, imageModels, "查询镜像列表成功");
        } catch (Exception e) {
        	logger.error("查询镜像列表失败 ： " + e);
        	return new BocloudResult(false, "查询镜像列表失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String imageId) {
		try {
			Long startTime = System.currentTimeMillis();
			while (true) {
				sleepSomeTime(1000);
				List<ImageModel> imageModelList = (List<ImageModel>) list().getData();
				if (imageModelList != null && !"".equals(imageModelList)) {
					for (ImageModel imageModel : imageModelList) {
						if (imageModel.getImageUuid() == imageId || imageId.equals(imageModel.getImageUuid())) {
							return new BocloudResult(true, imageModel, "查询镜像详情成功");
						}
					}
				}
				Long endTime = System.currentTimeMillis();
				// 等待三分钟
				if (endTime - startTime > 15 * 60 * 1000) {
					return new BocloudResult(false, "查询资源详情超时！");
				}
			}
		} catch (Exception e) {
			logger.error("查询镜像详情失败 ： " + e);
			return new BocloudResult(false, "查询镜像详情失败", e.getMessage());
		}
    }

    //删除私有镜像
    //POST http://ctyun-api-url/apiproxy/v3/removeImagine
    @Override
    public BocloudResult remove(String imageId) {
        try {
        	String apiUrl = "/apiproxy/v3/removeImagine";
        	JSONObject params = new JSONObject();
        	JSONObject image = new JSONObject();
        	image.put("regionId", this.getRegionId());
        	image.put("imageId", imageId);
        	params.put("image", image);

        	Result result = doPost(apiUrl, params);
        	if (result.isFailed()) {
        		logger.error("删除私有镜像失败: " + result.getMessage());
        		return new BocloudResult(false, result.getMessage());
        	}
        	JSONObject object = JSONObject.parseObject(result.getMessage());
        	if (null == object) {
        		return new BocloudResult(false, "删除私有镜像失败");
        	}
        	if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
        		logger.error("删除私有镜像失败：" + object.getString("message"));
        		return new BocloudResult(false, "删除私有镜像失败：" + object.getString("message"));
        	}
        	return new BocloudResult(true, "删除私有镜像成功");
        } catch (Exception e) {
        	logger.error("删除私有镜像失败 ： " + e);
        	return new BocloudResult(false, "删除私有镜像失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(ImageModel imageModel) {
        return null;
    }

    //创建私有镜像
    //POST http://ctyun-api-url/apiproxy/v3/customImagine
    @Override
    public BocloudResult create(ImageModel imageModel) {
        try {
        	String apiUrl = "/apiproxy/v3/customImagine";
        	JSONObject params = new JSONObject();
        	JSONObject image = new JSONObject();
        	image.put("regionId", imageModel.getRegionId());
        	image.put("projectId",this.getProjectId());
        	image.put("vmId", imageModel.getInstanceUuid());
        	image.put("name", imageModel.getName());
        	image.put("description", imageModel.getRemark());

        	params.put("image", image);


        	Result result = doPost(apiUrl, params);
        	if (result.isFailed()) {
        		logger.error("创建私有镜像失败" + result.getMessage());
        		return new BocloudResult(false, result.getMessage());
        	}
        	JSONObject object = JSONObject.parseObject(result.getMessage());
        	if (null == object) {
        		logger.error("创建私有镜像失败");
        		return new BocloudResult(false, "创建私有镜像失败");
        	}
        	if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
        		logger.error("创建私有镜像失败：" + object.getString("message"));
        		return new BocloudResult(false, "创建私有镜像失败：" + object.getString("message"));
        	}
			JSONObject returnObj = JSONObject.parseObject(object.getString("returnObj"));
			Long startTime = System.currentTimeMillis();
			while (true) {
				String imageId = (String) this.queryServerJob(this.getRegionId(), returnObj.getString("data")).getData();
				ImageModel model = (ImageModel) detail(imageId).getData();
				if (model != null && !"".equals(model)) {
					return new BocloudResult(true, model, "创建私有镜像成功");
				}
				Long endTime = System.currentTimeMillis();
				// 等待三分钟
				if (endTime - startTime > 15 * 60 * 1000) {
					logger.error("查询资源详情超时");
					return new BocloudResult(false, "查询资源详情超时！");
				}
			}
        } catch (Exception e) {
        	logger.error("创建私有镜像失败 ： " + e);
        	return new BocloudResult(false, "创建私有镜像失败", e.getMessage());
        }
    }

	//查询JOB状态信息
	//GET http://ctyun-api-url/apiproxy/v3/queryJobStatus
	public BocloudResult queryServerJob(String regionId, String jobId) {
		try {
			Long startTime = System.currentTimeMillis();
			while (true) {
				sleepSomeTime(100);
				String apiUrl = "/apiproxy/v3/queryJobStatus";
				JSONObject params = new JSONObject();
				params.put("regionId", regionId);
				params.put("jobId", jobId);

				Result result = doGet(apiUrl, params);
				if (result.isFailed()) {
					return new BocloudResult(false, result.getMessage());
				}
				JSONObject object = JSONObject.parseObject(result.getMessage());
				if (null == object) {
					return new BocloudResult(false, "根据jobId查询JOB状态信息失败");
				}
				JSONObject returnObj = JSONObject.parseObject(object.getString("returnObj"));
				if (object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE) && "SUCCESS".equals(returnObj.getString("status"))) {
					JSONObject entities = JSONObject.parseObject(JSONObject.parseObject(object.getString("returnObj")).getString("entities"));
					List<JSONObject> subJobs = JSONArray.parseArray(entities.getString("sub_jobs"), JSONObject.class);
					if (ListTool.isEmpty(subJobs)) {
						return new BocloudResult(true, entities.getString("image_id"), "根据jobId查询任务成功");
					} else {
						for (JSONObject jsonObject : subJobs) {
							if ("SUCCESS".equals(jsonObject.getString("status")) && !"".equals(jsonObject.getString("entities")) && null != jsonObject.getString("entities")) {
								return new BocloudResult(true, JSONObject.parseObject(jsonObject.getString("entities")).getString("server_id"), "根据jobId查询JOB状态信息成功");
							}
						}
					}
				}
				Long endTime = System.currentTimeMillis();
				// 等待15分钟
				if (endTime - startTime > 15 * 60 * 1000) {
					return new BocloudResult(false, "查询资源详情超时！");
				}
			}

		} catch (Exception e) {
			logger.error("根据jobId查询JOB状态信息失败 ： " + e);
			return new BocloudResult(false, "根据jobId查询磁盘JOB状态信息失败", e.getMessage());
		}
	}
}
