package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FloatingIpModel;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.provider.TianyiProvider;
import com.bocloud.cmp.provider.common.TianyiFlavorProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.FloatingIpConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * @author Hu Wentao
 * @create 2020-10-10 16:08
 * @Description: 浮动ip
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiFloatingIpProvider extends TianyiProvider implements FloatingIPProvider {

	private static Logger logger = LoggerFactory.getLogger(TianyiFlavorProvider.class);

	private BeanConvertor<FloatingIpModel, JSONObject> convertor = new FloatingIpConvertor();

	Butler butler = null;

	/**
	 * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
	 *
	 * @param butler
	 *            云平台认证信息
	 */
	public TianyiFloatingIpProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	// 查询公网IP列表
	// GET http://ctyun-api-url/apiproxy/v3/ondemand/queryIps
	@Override
	public BocloudResult list() {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/queryIps";
			JSONObject headers = new JSONObject();
			headers.put("regionId", this.getRegionId());

			Result result = doGet(apiUrl, headers);
			if (result.isFailed()) {
				logger.error("查询公网Ip失败:" + result.getMessage());
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "查询公网Ip失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("查询公网Ip失败：" + object.getString("message"));
				return new BocloudResult(false, "查询公网Ip失败：" + object.getString("message"));
			}
			List<JSONObject> publicIps = JSONArray.parseArray(JSONObject.parseObject(object.getString("returnObj"))
					.getString("publicips"), JSONObject.class);
			if (ListTool.isEmpty(publicIps)) {
				return new BocloudResult(true, "公网IP列表为空");
			}
			List<FloatingIpModel> floatingIpModels = convertor.convertList(publicIps);
			return new BocloudResult(true, floatingIpModels, "查询公网Ip成功");
		} catch (Exception e) {
			logger.error("查询公网Ip失败 ： " + e);
			return new BocloudResult(false, "查询公网Ip失败", e.getMessage());
		}
	}

	// 创建弹性IP（按需开通）
	// POST http://ctyun-api-url/apiproxy/v3/ondemand/createIp
	@Override
	public BocloudResult create(FloatingIpModel floatingIpModel) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/createIp";
			JSONObject params = new JSONObject();
			JSONObject createIpInfo = new JSONObject();
			createIpInfo.put("regionId", this.getRegionId());
			createIpInfo.put("zoneId", floatingIpModel.getRegionId() + "a");
			createIpInfo.put("name", UUID.randomUUID().toString());
			createIpInfo.put("size",
					floatingIpModel.getBandwidthSize() == null ? 5 : floatingIpModel.getBandwidthSize());
			params.put("createIpInfo", createIpInfo);

			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				logger.error("创建公网Ip失败");
				return new BocloudResult(false, "创建公网Ip失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("创建公网Ip失败：" + object.getString("message"));
				return new BocloudResult(false, "创建公网Ip失败：" + object.getString("message"));
			}
			// 获取返回值
			JSONObject jsonObject = object.getJSONObject("returnObj");
			BocloudResult waitForProcess = waitForProcess(jsonObject.getString("id"), "UNBIND");
			if (waitForProcess.isFailed()) {
				return waitForProcess;
			}
			List<FloatingIpModel> models = new ArrayList<FloatingIpModel>();
			models.add((FloatingIpModel) waitForProcess.getData());
			return new BocloudResult(true, models, "创建公网Ip成功");
		} catch (Exception e) {
			logger.error("创建公网Ip失败 ： " + e);
			return new BocloudResult(false, "创建公网Ip失败", e.getMessage());
		}
	}

	public BocloudResult waitForProcess(String publicIpId, String status) {
		Long startTime = System.currentTimeMillis();
		while (true) {
			sleepSomeTime(100);
			BocloudResult detail = this.detail(publicIpId);
			if (detail.isFailed()) {
				return detail;
			}
			FloatingIpModel ipModel = (FloatingIpModel) detail.getData();
			if (status.equalsIgnoreCase(ipModel.getStatus())) {
				return new BocloudResult(true, ipModel, "浮动IP创建成功！");
			}
			Long endTime = System.currentTimeMillis();
			// 等待三分钟
			if (endTime - startTime > 3 * 60 * 1000) {
				logger.error("创建超时！");
				return new BocloudResult(false, ipModel, "创建超时！");
			}
		}

	}

	// 删除公网ip（按需开通）
	// POST http://ctyun-api-url/apiproxy/v3/ondemand/deleteIp
	@Override
	public BocloudResult remove(String publicIpId) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/deleteIp";
			JSONObject params = new JSONObject();
			params.put("regionId", this.getRegionId());
			params.put("publicIpId", publicIpId);

			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				logger.error("删除公网Ip失败:" + result.getMessage());
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "删除公网Ip失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("删除公网Ip失败：" + object.getString("message"));
				return new BocloudResult(false, "删除公网Ip失败：" + object.getString("message"));
			}
			if (!object.getBoolean("returnObj")) {
				return new BocloudResult(false, "该公网Ip不存在");
			}
			return new BocloudResult(true, "删除公网Ip成功");
		} catch (Exception e) {
			logger.error("删除公网Ip失败 ： " + e);
			return new BocloudResult(false, "删除公网Ip失败", e.getMessage());
		}
	}

	// 查询公网IP列表
	// GET http://ctyun-api-url/apiproxy/v3/ondemand/queryIps
	@Override
	public BocloudResult detail(String publicIpId) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/queryIps";
			JSONObject headers = new JSONObject();
			headers.put("regionId", this.getRegionId());
			headers.put("publicIpId", publicIpId);

			Result result = doGet(apiUrl, headers);
			if (result.isFailed()) {
				logger.error("查询公网Ip失败：" + result.getMessage());
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "查询公网Ip失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("查询公网Ip失败：" + object.getString("message"));
				return new BocloudResult(false, "查询公网Ip失败：" + object.getString("message"));
			}
			List<JSONObject> publicIps = JSONArray.parseArray(JSONObject.parseObject(object.getString("returnObj"))
					.getString("publicips"), JSONObject.class);
			if (ListTool.isEmpty(publicIps)) {
				return new BocloudResult(true, "公网IP不存在");
			}
			return new BocloudResult(true, convertor.convertModel(publicIps.get(0)), "查询公网Ip成功");
		} catch (Exception e) {
			logger.error("查询公网Ip失败 ： " + e);
			return new BocloudResult(false, "查询公网Ip失败", e.getMessage());
		}
	}

	// 绑定按需带宽
	// POST http://ctyun-api-url/apiproxy/v3/ondemand/bindIp
	@Override
	public BocloudResult associate(FloatingIpModel modeld) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/bindIp";
			JSONObject params = new JSONObject();
			params.put("regionId", this.getRegionId());
			params.put("publicIpId", modeld.getFloatingIpUuid());
			params.put("networkCardId", modeld.getNetworkCardUuid());
			if (modeld.getNetworkCardUuid() == null || "".equals(modeld.getNetworkCardUuid())) {
				String fixedIpAddress = modeld.getFixedIpAddress();
				TianyiNetworkCardProvider tianyiNetworkCardProvider = new TianyiNetworkCardProvider(butler);
				BocloudResult list = tianyiNetworkCardProvider.list(modeld.getInstanceId());
				if (list.isSuccess()) {
					List<NetworkCardModel> networkCards = (List<NetworkCardModel>) list.getData();
					for (NetworkCardModel networkCard : networkCards) {
						String ipAddress = networkCard.getIpAddress();
						if (ipAddress == fixedIpAddress || ipAddress.equals(fixedIpAddress)) {
							params.put("networkCardId", networkCard.getNetworkcardUuid());
						}
					}
				}
			}
			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				logger.error("带宽绑定网卡失败:" + result.getMessage());
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				logger.error("带宽绑定网卡失败");
				return new BocloudResult(false, "带宽绑定网卡失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("带宽绑定失败：" + object.getString("message"));
				return new BocloudResult(false, "带宽绑定失败：" + object.getString("message"));
			}
			JSONObject returnObj = object.getJSONObject("returnObj");
			if (null == returnObj) {
				logger.error("浮动IP绑定失败");
				return new BocloudResult(false, "浮动IP绑定失败！");
			}
			FloatingIpModel publicip = convertor.convertModel(returnObj.getJSONObject("publicip"));
			publicip.setInstanceId(modeld.getInstanceId());
			return new BocloudResult(true, publicip, "带宽绑定网卡成功");
		} catch (Exception e) {
			logger.error("带宽绑定网卡失败 ： " + e);
			return new BocloudResult(false, "带宽绑定网卡失败", e.getMessage());
		}
	}

	// 解绑按需带宽
	// POST http://ctyun-api-url/apiproxy/v3/ondemand/unbindIp
	@Override
	public BocloudResult disassociate(String publicIpId) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/unbindIp";
			JSONObject params = new JSONObject();
			params.put("regionId", this.getRegionId());
			params.put("publicIpId", publicIpId);

			Result result = doPost(apiUrl, params);

			if (result.isFailed()) {
				logger.error("解绑按需带宽失败:" + result.getMessage());
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				logger.error("解绑按需带宽失败");
				return new BocloudResult(false, "解绑按需带宽失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("解绑按需带宽失败：" + object.getString("message"));
				return new BocloudResult(false, "解绑按需带宽失败：" + object.getString("message"));
			}
			return new BocloudResult(true, "解绑按需带宽成功");
		} catch (Exception e) {
			logger.error("解绑按需带宽失败 ： " + e);
			return new BocloudResult(false, "解绑按需带宽失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult disassociate(FloatingIpModel model) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/unbindIp";
			JSONObject params = new JSONObject();
			params.put("regionId", this.getRegionId());
			params.put("publicIpId", model.getFloatingIpUuid());

			Result result = doPost(apiUrl, params);

			if (result.isFailed()) {
				logger.error("浮动IP解绑失败:" + result.getMessage());
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				logger.error("浮动IP解绑失败");
				return new BocloudResult(false, "浮动IP解绑失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("浮动IP解绑失败：" + object.getString("message"));
				return new BocloudResult(false, "浮动IP解绑失败：" + object.getString("message"));
			}
			return new BocloudResult(true, this.detail(model.getFloatingIpUuid()).getData(), "浮动IP解绑成功");
		} catch (Exception e) {
			logger.error("浮动IP解绑失败 ： " + e);
			return new BocloudResult(false, "浮动IP解绑失败", e.getMessage());
		}
	}

	// 带宽大小变更（按需开通）
	// POST http://ctyun-api-url/apiproxy/v3/ondemand/upgradeNetwork
	@Override
	public BocloudResult modify(FloatingIpModel floatingIpModel) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/upgradeNetwork";
			JSONObject params = new JSONObject();
			params.put("regionId", floatingIpModel.getRegionId());
			params.put("publicIpId", floatingIpModel.getFloatingIpUuid());
			// 这里必须把name传过来，否则name会设置为null
			params.put("name", floatingIpModel.getName());
			params.put("size", floatingIpModel.getBandwidthSize());

			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				logger.error("带宽大小变更（按需开通）失败:" + result.getMessage());
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				logger.error("带宽大小变更（按需开通）失败");
				return new BocloudResult(false, "带宽大小变更（按需开通）失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("带宽大小变更（按需开通）失败：" + object.getString("message"));
				return new BocloudResult(false, "带宽大小变更（按需开通）失败：" + object.getString("message"));
			}
			return new BocloudResult(true, "带宽大小变更（按需开通）成功");
		} catch (Exception e) {
			logger.error("带宽大小变更（按需开通）失败 ： " + e);
			return new BocloudResult(false, "带宽大小变更（按需开通）失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult associate(String fipId, String portId) {
		// TODO Auto-generated method stub
		return null;
	}
}
