package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.provider.TianyiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.NetworkCardConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Hu Wentao
 * @Description: 天翼网卡
 * @date 2020/10/30 11:20
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiNetworkCardProvider extends TianyiProvider implements NetworkCardProvider {

    private static Logger logger = LoggerFactory.getLogger(TianyiNetworkCardProvider.class);

    private BeanConvertor<NetworkCardModel, JSONObject> convertor = new NetworkCardConvertor();

	/**
	 * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
	 *
	 * @param butler
	 *            云平台认证信息
	 */
	public TianyiNetworkCardProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		return null;
	}

	// 根据主机ID查询网卡列表
	// GET http://ctyun-api-url/apiproxy/v3/queryNetworkCards
	public BocloudResult list(String vmId) {
		try {
			String apiUrl = "/apiproxy/v3/queryNetworkCards";
			JSONObject headers = new JSONObject();
			headers.put("regionId", this.getRegionId());
			headers.put("vmId", vmId);

			Result result = doGet(apiUrl, headers);
			if (result.isFailed()) {
				logger.error("根据主机ID查询网卡列表失败:" + result.getMessage());
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				logger.error("根据主机ID查询网卡列表失败");
				return new BocloudResult(false, "根据主机ID查询网卡列表失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("根据主机ID查询网卡列表失败：" + object.getString("message"));
				return new BocloudResult(false, "根据主机ID查询网卡列表失败：" + object.getString("message"));
			}
			List<JSONObject> jsonObjects = JSONArray.parseArray(JSONObject.parseObject(object.getString("returnObj"))
					.getString("interfaceAttachments"), JSONObject.class);
			if (ListTool.isEmpty(jsonObjects)) {
				return new BocloudResult(true, "该云主机下暂无网卡");
			}
			List<NetworkCardModel> networkCards = convertor.convertList(jsonObjects);
			for (NetworkCardModel networkCard : networkCards) {
				networkCard.setRegionId(this.getRegionId());
				networkCard.setInstanceId(vmId);
			}
			return new BocloudResult(true, networkCards, "根据主机ID查询网卡列表成功");
		} catch (Exception e) {
			logger.error("根据主机ID查询网卡列表失败 ： " + e);
			return new BocloudResult(false, "根据主机ID查询网卡列表失败", e.getMessage());
		}
	}

    @Override
    public BocloudResult remove(String s) {
        return null;
    }

	// 删除网卡
	// POST http://ctyun-api-url/apiproxy/v3/deleteNetworkCard
	public BocloudResult remove(String vmId, String networkCardId) {
		try {
			String apiUrl = "/apiproxy/v3/deleteNetworkCard";
			JSONObject params = new JSONObject();
			params.put("regionId", this.getRegionId());
			params.put("vmId", vmId);
			params.put("networkCardId", networkCardId);

			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				logger.error("删除网卡失败:" + result.getMessage());
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				logger.error("删除网卡失败");
				return new BocloudResult(false, "删除网卡失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("删除网卡失败：" + object.getString("message"));
				return new BocloudResult(false, "删除网卡失败：" + object.getString("message"));
			}
			return new BocloudResult(true, "删除网卡成功");
		} catch (Exception e) {
			logger.error("删除网卡失败 ： " + e);
			return new BocloudResult(false, "删除网卡失败", e.getMessage());
		}
	}

	// 针对云主机批量添加网卡
	// POST http://ctyun-api-url/apiproxy/v3/addNetworkCards
	@Override
	public BocloudResult create(NetworkCardModel networkCardModel) {
		try {
			String apiUrl = "/apiproxy/v3/addNetworkCards";
			JSONObject params = new JSONObject();
			params.put("regionId", networkCardModel.getRegionId());
			params.put("vmId", networkCardModel.getInstanceId());
			List<JSONObject> networkCardsInfo = new ArrayList<>();
			JSONObject networkCard = new JSONObject();
			networkCard.put("subnetId", networkCardModel.getSubnetUuid());
			List<JSONObject> securityGroups = new ArrayList<>();

			networkCardsInfo.add(networkCard);
			params.put("networkCardsInfo", networkCardsInfo);

			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				logger.error("针对云主机批量添加网卡失败:" + result.getMessage());
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				logger.error("针对云主机批量添加网卡失败");
				return new BocloudResult(false, "针对云主机批量添加网卡失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("针对云主机批量添加网卡失败：" + object.getString("message"));
				return new BocloudResult(false, "针对云主机批量添加网卡失败：" + object.getString("message"));
			}
			Boolean status = JSONObject.parseObject(object.getString("returnObj")).getBoolean("status");
			if (!status) {
				logger.error("针对云主机批量添加网卡失败");
				return new BocloudResult(false, "针对云主机批量添加网卡失败");
			}
			return new BocloudResult(true, "针对云主机批量添加网卡成功");
		} catch (Exception e) {
			logger.error("针对云主机批量添加网卡失败 ： " + e);
			return new BocloudResult(false, "针对云主机批量添加网卡失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String s) {
		return null;
	}

	@Override
	public BocloudResult attachNetworkCard(NetworkCardModel networkCardModel) {
		return null;
	}

	@Override
	public BocloudResult detachNetworkCard(NetworkCardModel networkCardModel) {
		return null;
	}
}
