package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.TianyiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.SecurityGroupConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.SecurityGroupRuleConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Package: com.bocloud.cmp.provider.network
 * @ClassName: TianyiSecurityGroupProvider
 * @Author: 胡文涛
 * @CreateTime: 2020/9/21 14:18
 * @Description: 安全组
 * @PS: 这里的安全组没有和安全组规则绑定，安全组详情中不包含安全组规则，若需要安全组规则，可用安全组id去查询安全组规则
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiSecurityGroupProvider extends TianyiProvider implements SecurityGroupProvider {

    private Logger logger = LoggerFactory.getLogger(TianyiSecurityGroupProvider.class);

    BeanConvertor<SecurityGroupModel, JSONObject> convertor = new SecurityGroupConvertor();

    BeanConvertor<SecurityGroupRuleModel, JSONObject> ruleConvertor = new SecurityGroupRuleConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public TianyiSecurityGroupProvider(Butler butler) {
        super(butler);
    }

    //查询安全组列表
    //GET http://ctyun-api-url/apiproxy/v3/getSecurityGroups
    @Override
    public BocloudResult list() {
        try {
            String apiUrl = "/apiproxy/v3/getSecurityGroups";


            JSONObject headers = new JSONObject();
            headers.put("regionId", this.getRegionId());

            Result result = doGet(apiUrl, headers);
            if (result.isFailed()) {
                logger.error("查询安全组列表失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("查询安全组列表失败");
                return new BocloudResult(false, "查询安全组列表失败");
            }
            List<JSONObject> returnObj = JSONObject.parseArray(jsonObject.getString("returnObj"), JSONObject.class);
            if (ListTool.isEmpty(returnObj)) {
                return new BocloudResult(true, "安全组列表为空");
            }
            for (JSONObject securityGroup : returnObj) {
                BocloudResult securityGroupRules = securityGroupRuleList(securityGroup.getString("resSecurityGroupId"));
                List<SecurityGroupRuleModel> securityGroupRuleModels = (List<SecurityGroupRuleModel>) securityGroupRules.getData();
                securityGroup.put("security_group_rules", securityGroupRuleModels);
            }
            List<SecurityGroupModel> securityGroupModels = convertor.convertList(returnObj);
            return new BocloudResult(true, securityGroupModels, "查询安全组列表成功");
        } catch (Exception e) {
        	logger.error("查询安全组列表失败 ： " + e);
        	return new BocloudResult(false, "查询安全组列表失败", e.getMessage());
        }
    }

    public BocloudResult securityGroupRuleList(String securityGroupId) {
        try {
            String apiUrl = "/apiproxy/v3/getSecurityGroupRules";
            JSONObject headers = new JSONObject();
            headers.put("regionId", this.getRegionId());
            headers.put("securityGroupId", securityGroupId);
            Result result = doGet(apiUrl, headers);
            if (result.isFailed()) {
                logger.error("查询安全组规则列表失败:" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("查询安全组规则列表失败");
                return new BocloudResult(false, "查询安全组规则列表失败");
            }
            JSONObject returnObj = JSONObject.parseObject(jsonObject.getString("returnObj"));
            List<JSONObject> securityGroupRules = JSONObject.parseArray(returnObj.getString("security_group_rules"), JSONObject.class);
            if (ListTool.isEmpty(securityGroupRules)) {
                return new BocloudResult(true, "安全组规则列表为空");
            }
            List<SecurityGroupRuleModel> securityGroupRuleModels = ruleConvertor.convertList(securityGroupRules);

            return new BocloudResult(true, securityGroupRuleModels, "查询安全组规则列表成功");
        } catch (Exception e) {
            logger.error("查询安全组规则列表失败 ： " + e);
            return new BocloudResult(false, "查询安全组规则列表失败", e.getMessage());
        }
    }

    //创建安全组
    //POST http://ctyun-api-url/apiproxy/v3/createSecurityGroup
    @Override
    public BocloudResult create(SecurityGroupModel securityGroupModel) {
        try {
            String apiUrl = "/apiproxy/v3/createSecurityGroup";
            JSONObject params = new JSONObject();
            params.put("regionId", this.getRegionId());

            params.put("description", securityGroupModel.getRemark());
            params.put("name", securityGroupModel.getName());

            Result result = doPost(apiUrl, params);
            if (result.isFailed()) {
                logger.error("创建安全组失败:" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("创建安全组失败");
                return new BocloudResult(false, "创建安全组失败");
            }
            String id = JSONObject.parseObject(jsonObject.getString("returnObj")).getString("id");
            Long startTime = System.currentTimeMillis();
            while (true) {
                sleepSomeTime(500);
                SecurityGroupModel model = (SecurityGroupModel) detail(id).getData();
                if (model != null && !"".equals(model)) {
                    return new BocloudResult(true, model, "创建安全组成功");
                }
                Long endTime = System.currentTimeMillis();
                // 等待一分钟
                if (endTime - startTime > 3 * 60 * 1000) {
                    logger.error("查询安全组超时");
                    return new BocloudResult(false, "查询安全组超时！");
                }
            }
        } catch (Exception e) {
        	logger.error("创建安全组失败 ： " + e);
        	return new BocloudResult(false, "创建安全组失败", e.getMessage());
        }
    }

    //删除安全组
    //POST http://ctyun-api-url/apiproxy/v3/deleteSecurityGroup
    @Override
    public BocloudResult remove(String securityGroupId) {
        try {
            String apiUrl = "/apiproxy/v3/deleteSecurityGroup";
            JSONObject params = new JSONObject();
            params.put("regionId", this.getRegionId());
            params.put("securityGroupId", securityGroupId);

            Result result = doPost(apiUrl, params);
            if (result.isFailed()) {
                logger.error("删除安全组失败:" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("删除安全组失败");
                return new BocloudResult(false, "删除安全组失败");
            }
            return new BocloudResult(true, "删除安全组成功");
        } catch (Exception e) {
        	logger.error("删除安全组失败 ： " + e);
        	return new BocloudResult(false, "删除安全组失败", e.getMessage());
        }
    }

    //查询安全组详情
    //GET http://ctyun-api-url/apiproxy/v3/querySecurityGroupDetail
    @Override
    public BocloudResult detail(String securityGroupId) {
        try {
            String apiUrl = "/apiproxy/v3/querySecurityGroupDetail";
            JSONObject headers = new JSONObject();
            headers.put("regionId", this.getRegionId());
            headers.put("securityGroupId", securityGroupId);

            Result result = doGet(apiUrl, headers);
            if (result.isFailed()) {
                logger.error("查询安全组详情失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("查询安全组详情失败");
                return new BocloudResult(false, "查询安全组详情失败");
            }
            JSONObject returnObj = JSONObject.parseObject(jsonObject.getString("returnObj"), JSONObject.class);
            if (null == returnObj) {
                return new BocloudResult(false, "该安全组不存在");
            }
            BocloudResult securityGroupRules = securityGroupRuleList(returnObj.getString("resSecurityGroupId"));
            List<SecurityGroupRuleModel> securityGroupRuleModels = (List<SecurityGroupRuleModel>) securityGroupRules.getData();
            returnObj.put("security_group_rules", securityGroupRuleModels);
            SecurityGroupModel securityGroupModel = convertor.convertModel(returnObj);
            return new BocloudResult(true, securityGroupModel, "查询安全组详情成功");
        } catch (Exception e) {
        	logger.error("查询安全组详情失败 ： " + e);
        	return new BocloudResult(false, "查询安全组详情失败", e.getMessage());
        }
    }


    //TODO 修改安全组信息
    @Override
    public BocloudResult modify(SecurityGroupModel securityGroupModel) {
        return null;
    }

    @Override
    public BocloudResult list(String zone) {
        return null;
    }

    @Override
    public BocloudResult getProjectDefaultSecg() {
        return null;
    }
}
