package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.TianyiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.SecurityGroupRuleConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Package: com.bocloud.cmp.provider.network
 * @ClassName: TianyiSecurityGroupRuleProvider
 * @Author: 胡文涛
 * @CreateTime: 2020/9/21 17:36
 * @Description: 安全组规则
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiSecurityGroupRuleProvider extends TianyiProvider implements SecurityGroupRuleProvider {

	private Logger logger = LoggerFactory.getLogger(TianyiSecurityGroupRuleProvider.class);

	BeanConvertor<SecurityGroupRuleModel, JSONObject> convertor = new SecurityGroupRuleConvertor();

	/**
	 * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
	 *
	 * @param butler
	 *            云平台认证信息
	 */
	public TianyiSecurityGroupRuleProvider(Butler butler) {
		super(butler);
	}

	// 创建安全组规则
	// POST http://ctyun-api-url/apiproxy/v3/createSecurityGroupRule
	@Override
	public BocloudResult create(SecurityGroupRuleModel securityGroupRuleModel) {
		try {
			String apiUrl = "/apiproxy/v3/createSecurityGroupRule";
			JSONObject params = new JSONObject();
			JSONObject jsonStr = new JSONObject();
			jsonStr.put("regionId", this.getRegionId());
			jsonStr.put("projectId", this.getProjectId());
			jsonStr.put("securityGroupId", securityGroupRuleModel.getGroupUuid());
			jsonStr.put("direction", securityGroupRuleModel.getDirection());
			jsonStr.put("ethertype",
					securityGroupRuleModel.getEtherType() == null ? "IPv4" : securityGroupRuleModel.getEtherType());
			jsonStr.put("protocol", securityGroupRuleModel.getProtocol());
			jsonStr.put("portRangeMin", securityGroupRuleModel.getPortMin());
			jsonStr.put("portRangeMax", securityGroupRuleModel.getPortMax());
			jsonStr.put("remoteIpPrefix", securityGroupRuleModel.getRemoteIpPrefix());
			params.put("jsonStr", jsonStr);
			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				logger.error("创建安全组规则失败：" + result.getMessage());
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
			if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				if (jsonObject.getInteger("statusCode").equals(900) && jsonObject.getString("message") != null) {
					String message = jsonObject.getString("message");
					message = message.replace(';',' ');
					return new BocloudResult(false, message);
				}
				logger.error("创建安全组规则失败");
				return new BocloudResult(false, "创建安全组规则失败");
			}
			JSONObject returnObj = JSONObject.parseObject(jsonObject.getString("returnObj"));
			SecurityGroupRuleModel model = convertor.convertModel(JSONObject.parseObject(returnObj
					.getString("security_group_rule")));
			return new BocloudResult(true, model, "创建安全组规则成功");
		} catch (Exception e) {
			logger.error("创建安全组规则失败 ： " + e);
			return new BocloudResult(false, "创建安全组规则失败", e.getMessage());
		}
	}

	// 删除安全组规则
	// POST http://ctyun-api-url/apiproxy/v3/deleteSecurityGroupRule
	@Override
	public BocloudResult remove(String securityGroupRuleId) {
		try {
			String apiUrl = "/apiproxy/v3/deleteSecurityGroupRule";

			JSONObject params = new JSONObject();
			params.put("regionId", this.getRegionId());
			params.put("securityGroupRuleId", securityGroupRuleId);

			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				logger.error("删除安全组规则失败");
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("删除安全组规则失败");
				return new BocloudResult(false, "删除安全组规则失败");
			}
			return new BocloudResult(true, "删除安全组规则成功");
		} catch (Exception e) {
			logger.error("删除安全组规则失败 ： " + e);
			return new BocloudResult(false, "删除安全组规则失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(SecurityGroupRuleModel securityGroupRuleModel) {
		return null;
	}

	// 查询安全组规则
	// GET http://ctyun-api-url/apiproxy/v3/getSecurityGroupRules
	@Override
	public BocloudResult list() {
		try {
			String apiUrl = "/apiproxy/v3/getSecurityGroupRules";
			JSONObject headers = new JSONObject();
			headers.put("regionId", this.getRegionId());
			Result result = doGet(apiUrl, headers);
			if (result.isFailed()) {
				logger.error("查询安全组规则列表失败");
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
			if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("查询安全组规则列表失败");
				return new BocloudResult(false, "查询安全组规则列表失败");
			}
			JSONObject returnObj = JSONObject.parseObject(jsonObject.getString("returnObj"));
			List<JSONObject> securityGroupRules = JSONObject.parseArray(returnObj.getString("security_group_rules"),
					JSONObject.class);
			if (ListTool.isEmpty(securityGroupRules)) {
				return new BocloudResult(true, "安全组规则列表为空");
			}
			List<SecurityGroupRuleModel> securityGroupRuleModels = convertor.convertList(securityGroupRules);

			return new BocloudResult(true, securityGroupRuleModels, "查询安全组规则列表成功");
		} catch (Exception e) {
			logger.error("查询安全组规则列表失败 ： " + e);
			return new BocloudResult(false, "查询安全组规则列表失败", e.getMessage());
		}
	}

	// 查询安全组规则
	// GET http://ctyun-api-url/apiproxy/v3/getSecurityGroupRules
	@Override
	public BocloudResult list(String securityGroupId) {
		try {
			String apiUrl = "/apiproxy/v3/getSecurityGroupRules";
			JSONObject headers = new JSONObject();
			headers.put("regionId", this.getRegionId());
			headers.put("securityGroupId", securityGroupId);
			Result result = doGet(apiUrl, headers);
			if (result.isFailed()) {
				logger.error("查询安全组规则列表失败");
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
			if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("查询安全组规则列表失败");
				return new BocloudResult(false, "查询安全组规则列表失败");
			}
			JSONObject returnObj = JSONObject.parseObject(jsonObject.getString("returnObj"));
			List<JSONObject> securityGroupRules = JSONObject.parseArray(returnObj.getString("security_group_rules"),
					JSONObject.class);
			if (ListTool.isEmpty(securityGroupRules)) {
				return new BocloudResult(true, "安全组规则列表为空");
			}
			List<SecurityGroupRuleModel> securityGroupRuleModels = convertor.convertList(securityGroupRules);

			return new BocloudResult(true, securityGroupRuleModels, "查询安全组规则列表成功");
		} catch (Exception e) {
			logger.error("查询安全组规则列表失败 ： " + e);
			return new BocloudResult(false, "查询安全组规则列表失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String sgrlId) {
		return null;
	}

	@Override
	public BocloudResult list(String sgId, String zone) {
		return null;
	}
}
