package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.TianyiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.SubnetConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @Package: com.bocloud.cmp.provider.network
 * @ClassName: TianyiSubnetProvider
 * @Author: 胡文涛
 * @CreateTime: 2020/9/18 16:29
 * @Description: 子网
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiSubnetProvider extends TianyiProvider implements SubnetProvider {

    private Logger logger = LoggerFactory.getLogger(TianyiSubnetProvider.class);

    BeanConvertor<SubnetModel, JSONObject> convertor = new SubnetConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public TianyiSubnetProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            String apiUrl = "/apiproxy/v3/getSubnets";

            JSONObject headers = new JSONObject();
            headers.put("regionId", this.getRegionId());

            Result result = doGet(apiUrl, headers);
            if (result.isFailed()) {
                logger.error("查询子网列表失败:" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("查询子网列表失败");
                return new BocloudResult(false, "查询子网列表失败");
            }
            List<JSONObject> returnObj = JSONObject.parseArray(jsonObject.getString("returnObj"), JSONObject.class);
            if (ListTool.isEmpty(returnObj)) {
                return new BocloudResult(true, "子网列表为空");
            }
            List<SubnetModel> subnetModels = convertor.convertList(returnObj);

            return new BocloudResult(true, subnetModels, "查询子网列表成功");
        } catch (Exception e) {
            logger.error("查询子网列表失败 ： " + e);
            return new BocloudResult(false, "查询子网列表失败", e.getMessage());
        }
    }

    //查询子网列表
    //GET http://ctyun-api-url/apiproxy/v3/getSubnets
    @Override
    public BocloudResult list(String vpcId) {
        try {
            String apiUrl = "/apiproxy/v3/getSubnets";

            JSONObject headers = new JSONObject();
            headers.put("regionId", this.getRegionId());
            headers.put("vpcId", vpcId);

            Result result = doGet(apiUrl, headers);
            if (result.isFailed()) {
                logger.error("查询子网列表失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("查询子网列表失败");
                return new BocloudResult(false, "查询子网列表失败");
            }
            List<JSONObject> returnObj = JSONObject.parseArray(jsonObject.getString("returnObj"), JSONObject.class);
            if (ListTool.isEmpty(returnObj)) {
                return new BocloudResult(true, "子网列表为空");
            }
            List<SubnetModel> subnetModels = convertor.convertList(returnObj);

            return new BocloudResult(true, subnetModels, "查询子网列表成功");
        } catch (Exception e) {
            logger.error("查询子网列表失败 ： " + e);
            return new BocloudResult(false, "查询子网列表失败", e.getMessage());
        }
    }

    //创建子网
    //POST http://ctyun-api-url/apiproxy/v3/createSubnet
    @Override
    public BocloudResult create(SubnetModel subnetModel) {
        try {
            String apiUrl = "/apiproxy/v3/createSubnet";

            JSONObject params = new JSONObject();
            JSONObject jsonStr = new JSONObject();
            jsonStr.put("projectId", this.getProjectId());
            jsonStr.put("regionId", subnetModel.getRegionId());
            jsonStr.put("zoneId", subnetModel.getZone());
            jsonStr.put("name", subnetModel.getName());
            jsonStr.put("cidr", subnetModel.getCidr());
            jsonStr.put("gatewayIp", subnetModel.getGateway());
            jsonStr.put("dhcpEnable", subnetModel.getDhcpEnabled());
            jsonStr.put("vpcId", subnetModel.getVpcUuid());
            jsonStr.put("primaryDns", subnetModel.getPrimaryDns());
            jsonStr.put("secondaryDns", subnetModel.getSecondaryDns());
            params.put("jsonStr", jsonStr);

            Result result = doPost(apiUrl, params);
            if (result.isFailed()) {
                logger.error("创建子网失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                if (jsonObject.getInteger("statusCode").equals(900) && jsonObject.getString("message") != null) {
                    String message = jsonObject.getString("message");
                    message = message.replace(';',' ');
                    JSONObject object = JSONObject.parseObject(message);
                    if (object != null && object.getString("message") != null) {
                        String errorMessage = object.getString("message");
                        return new BocloudResult(false, errorMessage);
                    }
                    return new BocloudResult(false, jsonObject.getString("message"));
                }
                logger.error("创建子网失败");
                return new BocloudResult(false, "创建子网失败");
            }
            String subnetId = JSONObject.parseObject(jsonObject.getString("returnObj"), JSONObject.class).getString("id");

            Long startTime = System.currentTimeMillis();
            while (true) {
                sleepSomeTime(2000);
                SubnetModel model = (SubnetModel) detail(subnetId).getData();
                if (model != null && !"".equals(model)) {
                    return new BocloudResult(true, model, "创建子网成功");
                }
                Long endTime = System.currentTimeMillis();
                // 等待一分钟
                if (endTime - startTime > 3 * 60 * 1000) {
                    logger.error("查询资源超时");
                    return new BocloudResult(false, "查询资源超时！");
                }
            }
        } catch (Exception e) {
            logger.error("创建子网失败 ： " + e);
            return new BocloudResult(false, "创建子网失败", e.getMessage());
        }
    }

    public SubnetModel convertModel(JSONObject model) {
        SubnetModel subnet = new SubnetModel();
        if (null != model) {
            subnet.setSubnetUuid(model.getString("id"));
            subnet.setName(model.getString("name"));
            subnet.setVpcUuid(model.getString("vpc_id"));
            subnet.setCidr(model.getString("cidr"));
            subnet.setGateway(model.getString("gateway_ip"));
            subnet.setStatus(model.getString("status"));
            subnet.setDhcpEnabled(model.getBoolean("dhcp_enable"));
            subnet.setZone(model.getString("availability_zone"));
            subnet.setPrimaryDns(model.getString("primary_dns"));
            subnet.setSecondaryDns(model.getString("secondary_dns"));
            List<String> dns = new ArrayList<>();
            dns.add(model.getString("primary_dns"));
            dns.add(model.getString("secondary_dns"));
            subnet.setDnsNames(String.valueOf(dns));
        }
        return subnet;
    }


    //修改子网
    //POST http://ctyun-api-url/apiproxy/v3/updateSubnet
    @Override
    public BocloudResult modify(SubnetModel subnetModel) {
        try {
            String apiUrl = "/apiproxy/v3/updateSubnet";

            JSONObject params = new JSONObject();
            JSONObject subnetInfo = new JSONObject();
            subnetInfo.put("regionId", subnetModel.getRegionId());
            subnetInfo.put("projectId", this.getProjectId());
            subnetInfo.put("vpcId", subnetModel.getVpcUuid());
            subnetInfo.put("name", subnetModel.getName());
            subnetInfo.put("subnetId", subnetModel.getSubnetUuid());
            subnetInfo.put("secondaryDns", subnetModel.getSecondaryDns());
            subnetInfo.put("primaryDns", subnetModel.getPrimaryDns());
            if (subnetModel.getDnsNames() != null && !"".equals(subnetModel.getDnsNames())) {
                List<String> dns = JSONArray.parseArray(subnetModel.getDnsNames(), String.class);

                if (dns.size() > 2) {
                    return new BocloudResult(false, "最多可设置两个dns");
                } else if (dns.size() > 1) {
                    subnetInfo.put("secondaryDns", subnetModel.getSecondaryDns() != null ? subnetModel.getSecondaryDns() : dns.get(1));
                    subnetInfo.put("primaryDns", subnetModel.getPrimaryDns() != null ? subnetModel.getPrimaryDns() : dns.get(0));
                } else if (dns.size() > 0) {
                    subnetInfo.put("primaryDns", subnetModel.getPrimaryDns() != null ? subnetModel.getPrimaryDns() : dns.get(0));
                }
            }

            params.put("subnetInfo", subnetInfo);

            Result result = doPost(apiUrl, params);
            if (result.isFailed()) {
                logger.error("修改子网信息失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("修改子网信息失败！");
                return new BocloudResult(false, "修改子网信息失败");
            }
            String subnetId = JSONObject.parseObject(JSONObject.parseObject(jsonObject.getString("returnObj")).getString("subnet")).getString("id");
            SubnetModel model = (SubnetModel) detail(subnetId).getData();
            return new BocloudResult(true, model, "修改子网信息成功");
        } catch (Exception e) {
            logger.error("修改子网信息失败 ： " + e);
            return new BocloudResult(false, "修改子网信息失败", e.getMessage());
        }
    }

    //查询子网详情
    //GET http://ctyun-api-url/apiproxy/v3/querySubnetDetail
    @Override
    public BocloudResult detail(String subnetId) {
        try {
            String apiUrl = "/apiproxy/v3/querySubnetDetail";

            JSONObject headers = new JSONObject();
            headers.put("regionId", this.getRegionId());
            headers.put("subnetId", subnetId);

            Result result = doGet(apiUrl, headers);
            if (result.isFailed()) {
                logger.error("查询子网详情失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (null == jsonObject) {
                logger.error("查询子网详情失败");
                return new BocloudResult(false, "查询子网详情失败");
            }
            JSONObject returnObj = JSONObject.parseObject(jsonObject.getString("returnObj"), JSONObject.class);
            if (null == returnObj) {
                return new BocloudResult(false, "该子网不存在");
            }
            SubnetModel subnetModel = convertor.convertModel(returnObj);
            return new BocloudResult(true, subnetModel, "查询子网详情成功");
        } catch (Exception e) {
            logger.error("查询子网详情失败 ： " + e);
            return new BocloudResult(false, "查询子网详情失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String subnetId) {
        return null;
    }

    //删除子网
    //POST http://ctyun-api-url/apiproxy/v3/deleteSubnet
    @Override
    public BocloudResult remove(SubnetModel subnetModel) {
        try {
            String apiUrl = "/apiproxy/v3/deleteSubnet";

            JSONObject params = new JSONObject();
            params.put("vpcId", subnetModel.getVpcUuid());
            params.put("subnetId", subnetModel.getSubnetUuid());
            params.put("regionId", subnetModel.getRegionId());

            Result result = doPost(apiUrl, params);
            if (result.isFailed()) {
                logger.error("删除子网失败");
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("删除子网失败");
                return new BocloudResult(false, "删除子网失败");
            }
            return new BocloudResult(true, "删除子网成功");
        } catch (Exception e) {
            logger.error("删除子网失败 ： " + e);
            return new BocloudResult(false, "删除子网失败", e.getMessage());
        }
    }
}
