package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.TianyiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.VpcConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Package: com.bocloud.cmp.provider.network
 * @ClassName: TianyiVpcProvider
 * @Author: 胡文涛
 * @CreateTime: 2020/9/18 10:11
 * @Description: vpc
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiVpcProvider extends TianyiProvider implements VpcProvider {

    private static Logger logger = LoggerFactory.getLogger(TianyiVpcProvider.class);

    BeanConvertor<VpcModel, JSONObject> convertor = new VpcConvertor();


    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public TianyiVpcProvider(Butler butler) {
        super(butler);
    }

    // 创建VPC
    //POST http://ctyun-api-url/apiproxy/v3/createVPC
    @Override
    public BocloudResult create(VpcModel vpcModel) {
        try {
            String apiUrl = "/apiproxy/v3/createVPC";

            JSONObject params = new JSONObject();
            params.put("regionId", vpcModel.getRegionId());
            params.put("name", vpcModel.getName());
            params.put("cidr", vpcModel.getCidr());

            Result result = doPost(apiUrl, params);
            if (result.isFailed()) {
                logger.error("创建vpc失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                if (jsonObject.getInteger("statusCode").equals(900) && jsonObject.getString("message") != null) {
                    String message = jsonObject.getString("message");
                    message = message.replace(';',' ');
                    JSONObject object = JSONObject.parseObject(message);
                    if (object != null && object.getString("message") != null) {
                        String errorMessage = JSONObject.parseObject(JSONObject.parseObject(object.getString("message")).getString("NeutronError")).getString("message");
                        return new BocloudResult(false, errorMessage);
                    }
                }
                logger.error("vpc创建失败");
                return new BocloudResult(false, "vpc创建失败");
            }
            String vpcId = JSONObject.parseObject(jsonObject.getString("returnObj"), JSONObject.class).getString("id");

            Long startTime = System.currentTimeMillis();
            while (true) {
                sleepSomeTime(2000);
                VpcModel model = (VpcModel) detail(vpcModel.getRegionId(), vpcId).getData();
                if (model != null && !"".equals(model)) {
                    return new BocloudResult(true, model, "创建vpc成功");
                }
                Long endTime = System.currentTimeMillis();
                // 等待一分钟
                if (endTime - startTime > 3 * 60 * 1000) {
                    logger.error("查询创建vpc任务超时");
                    return new BocloudResult(false, "查询vpc超时！");
                }
            }

        } catch (Exception e) {
            logger.error("创建vpc失败 ： " + e);
            return new BocloudResult(false, "创建vpc失败", e.getMessage());
        }
    }

    //删除VPC
    //POST http://ctyun-api-url/apiproxy/v3/deleteVPC
    @Override
    public BocloudResult remove(String vpcId) {
        try {
            String apiUrl = "/apiproxy/v3/deleteVPC";
            JSONObject params = new JSONObject();
            params.put("regionId", this.getRegionId());
            params.put("vpcId", vpcId);

            Result result = doPost(apiUrl, params);
            if (result.isFailed()) {
                logger.error("删除vpc失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (null == jsonObject) {
                logger.error("删除vpc失败");
                return new BocloudResult(false, "删除VPC失败");
            }
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("删除VPC失败：" + jsonObject.getString("message"));
                return new BocloudResult(false, "删除VPC失败：" + jsonObject.getString("message"));
            }
            return new BocloudResult(true, "删除vpc成功");
        } catch (Exception e) {
            logger.error("删除vpc失败 ： " + e);
            return new BocloudResult(false, "删除vpc失败", e.getMessage());
        }
    }

    //查询VPC列表
    //GET http://ctyun-api-url/apiproxy/v3/getVpcs
    @Override
    public BocloudResult list() {
        try {
            String apiUrl = "/apiproxy/v3/getVpcs";

            JSONObject headers = new JSONObject();
            headers.put("regionId", this.getRegionId());
            Result result = doGet(apiUrl, headers);
            if (result.isFailed()) {
                logger.error("查询vpc列表失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("查询vpc列表失败");
                return new BocloudResult(false, "查询vpc列表失败");
            }
            List<JSONObject> objects = JSONArray.parseArray(object.getString("returnObj"), JSONObject.class);
            if (ListTool.isEmpty(objects)) {
                return new BocloudResult(true, "vpc列表为空");
            }
            //查询vpc下的subnet
            for (JSONObject jsonObject : objects) {
                String resVpcId = jsonObject.getString("resVpcId");
                String regionId = jsonObject.getString("regionId");
                List<JSONObject> subnets = subnetsByVpcId(regionId, resVpcId);
                jsonObject.put("subnets", subnets);
            }
            return new BocloudResult(true, convertor.convertList(objects), "查询vpc列表成功");
        } catch (Exception e) {
            logger.error("查询vpc列表失败 ： " + e);
            return new BocloudResult(false, "查询vpc列表失败", e.getMessage());
        }
    }

    //查询子网列表
    //GET http://ctyun-api-url/apiproxy/v3/getSubnets
    public List<JSONObject> subnetsByVpcId(String regionId, String vpcId) {
        try {
            String apiUrl = "/apiproxy/v3/getSubnets";
            JSONObject headers = new JSONObject();
            headers.put("regionId", regionId);
            headers.put("vpcId", vpcId);
            Result result = doGet(apiUrl, headers);
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                return null;
            }
            List<JSONObject> returnObj = JSONObject.parseArray(object.getString("returnObj"), JSONObject.class);
            return returnObj;
        } catch (Exception e) {
            logger.error("查询子网列表失败 ： " + e);
            return null;
        }
    }


    //查询VPC详情
    //GET http://ctyun-api-url/apiproxy/v3/queryVPCDetail
    @Override
    public BocloudResult detail(String regionId, String vpcId) {
        try {
            String apiUrl = "/apiproxy/v3/queryVPCDetail";

            Map<String, Object> headers = new HashMap<>(8);
            headers.put("regionId", regionId);
            headers.put("vpcId", vpcId);
            Result result = doGet(apiUrl, headers);
            if (result.isFailed()) {
                logger.error("查询vpc详情失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("查询vpc详情失败");
                return new BocloudResult(false, "查询vpc详情失败");
            }
            JSONObject vpcObject = JSONObject.parseObject(jsonObject.getString("returnObj"), JSONObject.class);
            if (null == vpcObject) {
                return new BocloudResult(false, "该vpc不存在");
            }
            //子网
            List<JSONObject> subnets = subnetsByVpcId(regionId, vpcId);
            vpcObject.put("subnets", subnets);
            return new BocloudResult(true, convertor.convertModel(vpcObject), "查询vpc详情成功");
        } catch (Exception e) {
            logger.error("查询vpc详情失败 ： " + e);
            return new BocloudResult(false, "查询vpc详情失败", e.getMessage());
        }
    }

    // 更新VPC
    //POST http://ctyun-api-url/apiproxy/v3/updateVPC
    //PS: 不能改名字
    @Override
    public BocloudResult update(VpcModel vpcModel) {
        try {
            String apiUrl = "/apiproxy/v3/updateVPC";

            JSONObject params = new JSONObject();
            params.put("regionId", vpcModel.getRegionId());
            params.put("name", vpcModel.getName());
            params.put("vpcId", vpcModel.getVpcId());
            params.put("cidr", vpcModel.getCidr());

            Result result = doPost(apiUrl, params);
            if (result.isFailed()) {
                logger.error("更新vpc失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("更新vpc失败");
                return new BocloudResult(false, "更新VPC失败");
            }
            JSONObject returnObject = JSONObject.parseObject(jsonObject.getString("returnObj"), JSONObject.class);
            JSONObject vpcObject = JSONObject.parseObject(returnObject.getString("vpc"), JSONObject.class);

            List<JSONObject> subnets = subnetsByVpcId(vpcModel.getRegionId(), vpcObject.getString("id"));
            vpcObject.put("subnets", subnets);
            VpcModel model = convertor.convertModel(vpcObject);
            return new BocloudResult(true, model, "更新vpc成功");
        } catch (Exception e) {
            logger.error("更新vpc失败 ： " + e);
            return new BocloudResult(false, "更新vpc失败", e.getMessage());
        }
    }
}
