package com.bocloud.cmp.provider.storage;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.TianyiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.VolumeConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Package: com.bocloud.cmp.provider.com.tianyi.cmp.provider.storage
 * @ClassName: TianyiVolumeProvider
 * @Author: 胡文涛
 * @CreateTime: 2020/9/23 10:46
 * @Description: 云硬盘
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiVolumeProvider extends TianyiProvider implements VolumeProvider {

    private Logger logger = LoggerFactory.getLogger(TianyiVolumeProvider.class);

    BeanConvertor<VolumeModel, JSONObject> convertor = new VolumeConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public TianyiVolumeProvider(Butler butler) {
        super(butler);
    }

    //查询磁盘列表
    //GET http://ctyun-api-url/apiproxy/v3/ondemand/queryVolumes
    @Override
    public BocloudResult list() {
        try {
            String apiUrl = "/apiproxy/v3/ondemand/queryVolumes";
            JSONObject headers = new JSONObject();
            headers.put("regionId", this.getRegionId());
            Result result = doGet(apiUrl, headers);
            if (result.isFailed()) {
                logger.error("查询云硬盘列表失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (null == jsonObject) {
                logger.error("查询云硬盘列表失败");
                return new BocloudResult(false, "查询云硬盘列表失败");
            }
            JSONObject returnObj = JSONObject.parseObject(jsonObject.getString("returnObj"));
            if (null == jsonObject) {
                logger.error("查询云硬盘列表失败");
                return new BocloudResult(false, "查询云硬盘列表失败");
            }
            List<JSONObject> volumes = JSONObject.parseArray(returnObj.getString("volumes"), JSONObject.class);
            if (ListTool.isEmpty(volumes)) {
                return new BocloudResult(true, "云硬盘列表为空");
            }
            List<VolumeModel> volumeModels = convertor.convertList(volumes);

            return new BocloudResult(true, volumeModels, "查询云硬盘列表成功");
        } catch (Exception e) {
        	logger.error("查询云硬盘列表失败 ： " + e);
        	return new BocloudResult(false, "查询云硬盘列表失败", e.getMessage());
        }
    }

    //根据磁盘ID查看磁盘详细 调不通
    //GET http://ctyun-api-url/apiproxy/v3/queryDataDiskDetail
    //现在采用查询列表，匹配的方法
    @Override
    public BocloudResult detail(String volumeId) {
        try {
            Long startTime = System.currentTimeMillis();
            while (true) {
                sleepSomeTime(100);
                List<VolumeModel> volumeModelList = (List<VolumeModel>) list().getData();
                for (VolumeModel volumeModel : volumeModelList) {
                    if (volumeModel.getVolumeUuid() == volumeId || volumeModel.getVolumeUuid().equals(volumeId)) {
                        return new BocloudResult(true, volumeModel, "查询云硬盘详情成功");
                    }
                }
                Long endTime = System.currentTimeMillis();
                // 等待一分钟
                if (endTime - startTime > 3 * 60 * 1000) {
                    logger.error("查询云硬盘详情超时");
                    return new BocloudResult(false, "查询云硬盘详情超时！");
                }
            }
        } catch (Exception e) {
        	logger.error("查询云硬盘详情失败 ： " + e);
        	return new BocloudResult(false, "查询云硬盘详情失败", e.getMessage());
        }
    }

    //创建磁盘（按需开通）
    //POST http://ctyun-api-url/apiproxy/v3/ondemand/createVolume
    @Override
    public BocloudResult create(VolumeModel volumeModel) {
        try {
            String apiUrl = "/apiproxy/v3/ondemand/createVolume";
            JSONObject params = new JSONObject();
            JSONObject createVolumeInfo = new JSONObject();
            createVolumeInfo.put("regionId", volumeModel.getRegionId());
            createVolumeInfo.put("zoneId", volumeModel.getZone());
            createVolumeInfo.put("name", volumeModel.getName());
            createVolumeInfo.put("type", volumeModel.getVolumeType());
            createVolumeInfo.put("size", volumeModel.getSize());
            createVolumeInfo.put("count", volumeModel.getBlockCount());
            createVolumeInfo.put("backupId", volumeModel.getBlockVolumeId());
            params.put("createVolumeInfo", createVolumeInfo);
            Result result = doPost(apiUrl, params);
            if (result.isFailed()) {
                logger.error("创建云硬盘失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (null == jsonObject) {
                logger.error("创建云硬盘失败");
                return new BocloudResult(false, "创建云硬盘失败");
            }
            if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                if (jsonObject.getInteger("statusCode").equals(900) && jsonObject.getString("message") != null) {
                    String message = jsonObject.getString("message");
                    message = message.replace(';',' ');
                    JSONObject object = JSONObject.parseObject(message);
                    if (object != null && object.getString("error") != null) {
                        String errorMessage = JSONObject.parseObject(object.getString("error")).getString("message");
                        return new BocloudResult(false, errorMessage);
                    }
                }
                logger.error("创建云硬盘失败：" + jsonObject.getString("message"));
                return new BocloudResult(false, "创建云硬盘失败：" + jsonObject.getString("message"));
            }
            Boolean status = JSONObject.parseObject(jsonObject.getString("returnObj")).getBoolean("status");
            if (!status) {
                logger.error("创建云硬盘失败");
                return new BocloudResult(false, "创建云硬盘失败");
            }
            BocloudResult bocloudResult = this.queryVolumeJob(volumeModel.getRegionId(), JSONObject.parseObject(jsonObject.getString("returnObj")).getString("data"));
            JSONObject volumeObject = (JSONObject) bocloudResult.getData();
            VolumeModel model = convertor.convertModel(volumeObject);
            return new BocloudResult(true, model, "创建云硬盘成功");
        } catch (Exception e) {
        	logger.error("创建云硬盘失败 ： " + e);
        	return new BocloudResult(false, "创建云硬盘失败", e.getMessage());
        }
    }

    //查询磁盘JOB状态信息
    //GET /apiproxy/v3/queryVolumeJob
    public BocloudResult queryVolumeJob(String regionId, String jobId) {
        try {
            Long startTime = System.currentTimeMillis();
            while (true) {
                sleepSomeTime(2000);
                String apiUrl = "/apiproxy/v3/queryVolumeJob";
                JSONObject params = new JSONObject();
                params.put("regionId", regionId);
                params.put("jobId", jobId);

                Result result = doGet(apiUrl, params);
                if (result.isFailed()) {
                    logger.error("根据jobId查询磁盘JOB状态信息失败");
                    return new BocloudResult(false, result.getMessage());
                }
                JSONObject object = JSONObject.parseObject(result.getMessage());
                if (null == object) {
                    logger.error("根据jobId查询磁盘JOB状态信息失败");
                    return new BocloudResult(false, "根据jobId查询磁盘JOB状态信息失败");
                }
                JSONObject returnObj = JSONObject.parseObject(object.getString("returnObj"));
                if (object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE) && "SUCCESS".equals(returnObj.getString("status"))) {
                    JSONObject entities = JSONObject.parseObject(returnObj.getString("entities"));
                    JSONObject volume = new JSONObject();
                    volume.putAll(entities);
                    volume.put("id", entities.getString("volume_id"));
                    return new BocloudResult(true, volume, "根据jobId查询磁盘JOB状态信息成功");
                }
                Long endTime = System.currentTimeMillis();
                // 等待三分钟
                if (endTime - startTime > 3* 60 * 1000) {
                    logger.error("查询云硬盘详情超时");
                    return new BocloudResult(false, "查询云硬盘详情超时！");
                }
            }

        } catch (Exception e) {
            logger.error("根据jobId查询磁盘JOB状态信息失败 ： " + e);
            return new BocloudResult(false, "根据jobId查询磁盘JOB状态信息失败", e.getMessage());
        }
    }


    //TODO 根据磁盘ID修改磁盘名称
    //POST http://ctyun-api-url/apiproxy/v3/renameDataDiskName
    @Override
    public BocloudResult modify(VolumeModel volumeModel) {
        try {
        	String apiUrl = "/apiproxy/v3/renameDataDiskName";
        	JSONObject params = new JSONObject();
        	params.put("regionId", volumeModel.getRegionId());
        	params.put("name", volumeModel.getName());
        	params.put("volumeId", volumeModel.getVolumeUuid());
        	params.put("description", volumeModel.getRemark());

        	Result result = doPost(apiUrl, params);
        	if (result.isFailed()) {
        	    logger.error("根据磁盘ID修改磁盘名称失败:" + result.getMessage());
        		return new BocloudResult(false, result.getMessage());
        	}
        	JSONObject object = JSONObject.parseObject(result.getMessage());
        	if (null == object) {
        	    logger.error("根据磁盘ID修改磁盘名称失败");
        		return new BocloudResult(false, "根据磁盘ID修改磁盘名称失败");
        	}
        	if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
        	    logger.error("根据磁盘ID修改磁盘名称失败：" + object.getString("message"));
        		return new BocloudResult(false, "根据磁盘ID修改磁盘名称失败：" + object.getString("message"));
        	}
        	return new BocloudResult(true, "根据磁盘ID修改磁盘名称成功");
        } catch (Exception e) {
        	logger.error("根据磁盘ID修改磁盘名称失败 ： " + e);
        	return new BocloudResult(false, "根据磁盘ID修改磁盘名称失败", e.getMessage());
        }
    }

    //删除数据盘（按需开通）
    //POST http://ctyun-api-url/apiproxy/v3/ondemand/deleteVolume
    @Override
    public BocloudResult remove(String volumeId) {
        try {
            String apiUrl = "/apiproxy/v3/ondemand/deleteVolume";
            JSONObject params = new JSONObject();
            params.put("regionId", this.getRegionId());
            params.put("volumeId", volumeId);
            Result result = doPost(apiUrl, params);
            if (result.isFailed()) {
                logger.error("删除数据盘失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                logger.error("删除数据盘失败");
                return new BocloudResult(false, "删除数据盘失败");
            }
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("删除数据盘失败");
                return new BocloudResult(false, "删除数据盘失败");
            }
            Boolean status = JSONObject.parseObject(object.getString("returnObj")).getBoolean("status");
            if (!status) {
                logger.error("删除数据盘失败");
                return new BocloudResult(false, "删除数据盘失败");
            }
            return new BocloudResult(true, "删除数据盘成功");
        } catch (Exception e) {
        	logger.error("删除数据盘失败 ： " + e);
        	return new BocloudResult(false, "删除数据盘失败", e.getMessage());
        }
    }

    //磁盘扩容（按需开通）
    //POST http://ctyun-api-url/apiproxy/v3/ondemand/expandVolumeSize
    @Override
    public BocloudResult extend(String volumeId, Integer newSize) {
        try {
            String apiUrl = "/apiproxy/v3/ondemand/expandVolumeSize";
            JSONObject params = new JSONObject();
            params.put("regionId", this.getRegionId());
            params.put("volumeId", volumeId);
            params.put("newSize", newSize);
            Result result = doPost(apiUrl, params);
            if (result.isFailed()) {
                logger.error("磁盘扩容失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                logger.error("磁盘扩容失败");
                return new BocloudResult(false, "磁盘扩容失败");
            }
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("磁盘扩容失败：" + object.getString("message"));
                return new BocloudResult(false, "磁盘扩容失败：" + object.getString("message"));
            }
            Boolean status = JSONObject.parseObject(object.getString("returnObj")).getBoolean("status");
            if (!status) {
                logger.error("磁盘扩容失败");
                return new BocloudResult(false, "磁盘扩容失败");
            }

            return new BocloudResult(true, "磁盘扩容成功");
        } catch (Exception e) {
        	logger.error("磁盘扩容失败 ： " + e);
        	return new BocloudResult(false, "磁盘扩容失败", e.getMessage());
        }
    }

    // 绑定磁盘
    //POST http://ctyun-api-url/apiproxy/v3/ondemand/attachVolume
    @Override
    public BocloudResult attach(String serverId, String mountpoint, String volumeId) {
        try {
            String apiUrl = "/apiproxy/v3/ondemand/attachVolume";
            JSONObject params = new JSONObject();
            params.put("regionId", this.getRegionId());
            params.put("volumeId", volumeId);
            params.put("vmId", serverId);
            params.put("device", mountpoint);
            Result result = doPost(apiUrl, params);
            if (result.isFailed()) {
                logger.error("绑定磁盘失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                logger.error("绑定磁盘失败");
                return new BocloudResult(false, "绑定磁盘失败");
            }
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("绑定磁盘失败：" + object.getString("message"));
                return new BocloudResult(false, "绑定磁盘失败：" + object.getString("message"));
            }
            Boolean status = JSONObject.parseObject(object.getString("returnObj")).getBoolean("status");
            if (!status) {
                logger.error("绑定磁盘失败");
                return new BocloudResult(false, "绑定磁盘失败");
            }

            return new BocloudResult(true, "绑定磁盘成功");
        } catch (Exception e) {
        	logger.error("绑定磁盘失败 ： " + e);
        	return new BocloudResult(false, "绑定磁盘失败", e.getMessage());
        }
    }

    //解绑磁盘
    //POST http://ctyun-api-url/apiproxy/v3/ondemand/uninstallVolume
    @Override
    public BocloudResult detach(String serverId, String volumeId) {
        try {
            String apiUrl = "/apiproxy/v3/ondemand/uninstallVolume";
            JSONObject params = new JSONObject();
            params.put("regionId", this.getRegionId());
            params.put("volumeId", volumeId);
            params.put("vmId", serverId);
            Result result = doPost(apiUrl, params);
            if (result.isFailed()) {
                logger.error("解绑磁盘失败：" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                logger.error("解绑磁盘失败");
                return new BocloudResult(false, "解绑磁盘失败");
            }
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("解绑磁盘失败：" + object.getString("message"));
                return new BocloudResult(false, "解绑磁盘失败：" + object.getString("message"));
            }
            Boolean status = JSONObject.parseObject(object.getString("returnObj")).getBoolean("status");
            if (!status) {
                logger.error("解绑磁盘失败");
                return new BocloudResult(false, "解绑磁盘失败");
            }

            return new BocloudResult(true, "解绑磁盘成功");
        } catch (Exception e) {
        	logger.error("解绑磁盘失败 ： " + e);
        	return new BocloudResult(false, "解绑磁盘失败", e.getMessage());
        }
    }
}
