package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.contentlibrary.ItemModel;
import com.bocloud.cmp.model.contentlibrary.LibraryModel;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.cmp.provider.contentlibrary.VmwareLibraryProvider;
import com.bocloud.cmp.provider.utils.AssembleTools;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.vmware.vim25.*;
import com.bocloud.vmware.vim25.mo.*;
import io.jsonwebtoken.lang.Assert;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.rmi.RemoteException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * vmware同步数据组装类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/14
 */
@Provider("bocloud.cmp.vmware")
public class VmwareLocationProvider extends VmwareProvider implements LocationProvider {

    private static Logger logger = LoggerFactory.getLogger(VmwareLocationProvider.class);

    private VmwareLibraryProvider provider = null;

    public VmwareLocationProvider(Butler butler) {
        super(butler);
        provider = new VmwareLibraryProvider(butler);
    }

    @Override
    public BocloudResult sync() {
        List<VDCModel> dcList = new ArrayList<>();
        try {
            ManagedEntity[] dcEntities = new InventoryNavigator(this.getRootFolder()).searchManagedEntities(DC);
            if (null != dcEntities) {
                for (ManagedEntity dcEntity : dcEntities) {
                    VDCModel model = AssembleTools.getInstance().assembleDc(dcEntity);
                    dcList.add(model);
                }
            }
        } catch (Exception e) {
            logger.error("sync info error:", e);
            return new BocloudResult(false, "同步失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
        return new BocloudResult(true, dcList, "success");
    }

    @Override
    public BocloudResult getHostModel(HostModel host) {
        return null;
    }

    @Override
    public BocloudResult<List<DataStoreModel>> syncStoreImages(String vdcName) {
        try {
            ManagedEntity datacenter = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, vdcName);
            Assert.notNull(datacenter, "数据中心不存在！");
            Datacenter center = (Datacenter) datacenter;
            InventoryNavigator navigator = new InventoryNavigator(center);
            ManagedEntity[] dsEntities = navigator.searchManagedEntities(DT);
            List<DataStoreModel> storeModels = new ArrayList<>();
            if (null != dsEntities) {
                for (ManagedEntity dsEntity : dsEntities) {
                    Datastore datastore = (Datastore) dsEntity;
                    DatastoreSummary datastoreSummary = datastore.getSummary();
                    Long diskCapacity = datastoreSummary.getCapacity();
                    Long freeCapacity = datastoreSummary.getFreeSpace();
                    BigDecimal capacity = new BigDecimal(diskCapacity);
                    BigDecimal freedisk = new BigDecimal(freeCapacity);
                    Double diskTotal = capacity.divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP)
                            .doubleValue();
                    Double diskFree = freedisk.divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP)
                            .doubleValue();
                    // 获取存储下的云主机与模板
                    VirtualMachine[] vms = datastore.getVms();
                    Long templateNum = Arrays.stream(vms)
                            .filter(vm -> Objects.nonNull(vm.getConfig()) && vm.getConfig().isTemplate()).count();
                    Long vmNum = Arrays.stream(vms)
                            .filter(vm -> Objects.nonNull(vm.getConfig()) && !vm.getConfig().isTemplate()).count();
                    int hostNum = Optional.ofNullable(datastore.getHost()).orElse(new DatastoreHostMount[0]).length;
                    DataStoreModel dataStoreModel = new DataStoreModel(datastoreSummary.getName(),
                            datastoreSummary.getType(), datastoreSummary.getUrl(), diskTotal, diskFree, hostNum,
                            vmNum.intValue(), templateNum.intValue(), datastoreSummary.getDatastore().getVal());
                    // 判断数据存储是否是活动状态
                    if (!datastore.getSummary().accessible) {
                        dataStoreModel.setStatus("inactive");
                    } else {
                        dataStoreModel.setStatus("green");
                    }
                    List<ImageModel> images = new ArrayList<>();
                    // 获取数据存储中的镜像文件
                    HostDatastoreBrowserSearchSpec searchSpec = new HostDatastoreBrowserSearchSpec();
                    FileQueryFlags flags = new FileQueryFlags();
                    flags.setFileSize(true);
                    flags.setModification(true);
                    flags.setFileOwner(true);
                    flags.setFileType(true);
                    searchSpec.setDetails(flags);
                    // 搜索不区分大小写
                    searchSpec.setSearchCaseInsensitive(true);
                    // 搜索之前是否先排序文件夹
                    searchSpec.setSortFoldersFirst(true);
                    // 查询.iso结尾的文件
                    String[] matchPattern = new String[]{"*.iso"};
                    searchSpec.setMatchPattern(matchPattern);
                    IsoImageFileQuery isoQuery = new IsoImageFileQuery();
                    FileQuery[] query = new FileQuery[]{isoQuery};
                    searchSpec.setQuery(query);
                    Task task = null;
                    try {
                        task = datastore.getBrowser().searchDatastoreSubFolders_Task("[" + datastore.getName() + "]",
                                searchSpec);
                    } catch (FileFault e) {
                        logger.error("Get error message：", e);
                    } catch (InvalidDatastore e) {
                        logger.error("Get error message：", e);
                    } catch (RuntimeFault e) {
                        logger.error("Get error message：", e);
                    } catch (RemoteException e) {
                        logger.error("Get error message：", e);
                    }
                    String res = "";
                    if (null != task) {
                        try {
                            res = task.waitForTask();
                        } catch (RuntimeFault e) {
                            logger.error("Get error message：", e);
                        } catch (RemoteException e) {
                            logger.error("Get error message：", e);
                        } catch (InterruptedException e) {
                            logger.error("Get error message：", e);
                            Thread.currentThread().interrupt();
                        }
                        if (res == Task.SUCCESS) {
                            Object object = null;
                            try {
                                object = task.getTaskInfo().getResult();
                            } catch (InvalidProperty e) {
                                logger.error("Get error message：", e);
                            } catch (RuntimeFault e) {
                                logger.error("Get error message：", e);
                            } catch (RemoteException e) {
                                logger.error("Get error message：", e);
                            }
                            if (object instanceof ArrayOfHostDatastoreBrowserSearchResults) {
                                ArrayOfHostDatastoreBrowserSearchResults searchResults = (ArrayOfHostDatastoreBrowserSearchResults) object;
                                HostDatastoreBrowserSearchResults[] results = searchResults
                                        .getHostDatastoreBrowserSearchResults();
                                Arrays.stream(
                                        Optional.ofNullable(results).orElse(new HostDatastoreBrowserSearchResults[0]))
                                        .filter(result -> Objects.nonNull(result.getFile())).map(result -> {
                                    FileInfo[] fileInfos = result.getFile();
                                    return Arrays.stream(Optional.ofNullable(fileInfos).orElse(new FileInfo[0]))
                                            .filter(fileInfo -> fileInfo instanceof IsoImageFileInfo)
                                            .map(isoImage -> {
                                                IsoImageFileInfo info = (IsoImageFileInfo) isoImage;
                                                return build(info, datastore.getMOR().getVal(),
                                                        result.getFolderPath());
                                            }).collect(Collectors.toList());
                                }).collect(Collectors.toList()).forEach(imageModels -> {
                                    images.addAll(imageModels);
                                });

                                // if (null != results && results.length > 0) {
                                // for (HostDatastoreBrowserSearchResults result : results) {
                                // FileInfo[] fileInfos = result.getFile();
                                // if (null != fileInfos && fileInfos.length > 0) {
                                // for (FileInfo fileInfo : fileInfos) {
                                // if (fileInfo instanceof IsoImageFileInfo) {
                                // IsoImageFileInfo info = (IsoImageFileInfo) fileInfo;
                                // images.add(build(info, datastore.getMOR().getVal(), result.getFolderPath()));
                                // }
                                // }
                                // }
                                // }
                                // }
                            }
                        }
                    }
                    dataStoreModel.setImages(images);
                    storeModels.add(dataStoreModel);
                }
            }
            return new BocloudResult<>(true, storeModels, "同步数据中心下镜像成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult<>(false, "同步数据中心下镜像失败");
        }
    }

    @Override
    public BocloudResult<List<LibraryModel>> listLibraries() {
        BocloudResult<List<LibraryModel>> list = provider.list();
        if (list.isFailed() || null == list.getData()) {
            return BocloudResult.FAILED("查询失败");
        }
        List<LibraryModel> models = list.getData();
        logger.info("library size : " + models.size());
        logger.info("LibraryModel models : " + JSON.toJSONString(models));
        for (LibraryModel model : models) {
            BocloudResult<List<ItemModel>> result = provider.listItems(model.getLibraryUuid());
            logger.info("ItemModel result : " + JSON.toJSONString(result));
            if (result.isSuccess() && null != result.getData()) {
                model.setItems(result.getData());
            }
        }
        return new BocloudResult<>(true, models, "查询成功");
    }

    ImageModel build(IsoImageFileInfo iso, String storeVal, String folder) {
        ImageModel imageModel = new ImageModel();
        String name = iso.getPath();
        BigDecimal bigDecimal = new BigDecimal(0);
        if (null != iso.getFileSize() && 0L != iso.getFileSize()) {
            bigDecimal = new BigDecimal(iso.getFileSize()).divide(new BigDecimal(1024 * 1024 * 1024L), 2,
                    RoundingMode.HALF_UP);
        }
        imageModel.setSize(bigDecimal.doubleValue());
        imageModel.setName(name);
        imageModel.setStatus("ACTIVE");
        imageModel.setOsType("ISO");
        imageModel.setStoreVal(storeVal);
        imageModel.setFilePath(folder + iso.getPath());
        return imageModel;
    }
}
