package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.TaskMsgModel;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.cmp.provider.utils.DateUtil;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.bocloud.vmware.vim25.*;
import com.bocloud.vmware.vim25.mo.ManagedEntity;
import com.bocloud.vmware.vim25.mo.Task;
import com.bocloud.vmware.vim25.mo.TaskHistoryCollector;
import com.bocloud.vmware.vim25.mo.TaskManager;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * vmware平台任务
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/14
 */
@Provider("bocloud.cmp.vmware")
public class VmwareTaskMsgProvider extends VmwareProvider implements TaskMsgProvider {
    private static Logger logger = LoggerFactory.getLogger(VmwareTaskMsgProvider.class);

    public VmwareTaskMsgProvider(Butler butler) {
        super(butler);
    }

    private static ElementDescription[] methodInfo;

    @Override
    public BocloudResult<List<TaskMsgModel>> list() {
        try {
            TaskManager taskManager = this.getServiceInstance().getTaskManager();
            Task[] recentTasks = taskManager.getRecentTasks();
            TaskDescription description = taskManager.getDescription();
            TypeDescription[] reason = description.getReason();
            methodInfo = description.getMethodInfo();
            List<TaskMsgModel> taskMsgModels = new ArrayList<>();
            if (null != recentTasks && recentTasks.length > 0) {
                for (Task task : recentTasks) {
                    TaskInfo taskInfo = task.getTaskInfo();
                    if (null != taskInfo) {
                        taskMsgModels.add(convertor(taskInfo));
                    }
                }
            }
            return new BocloudResult<>(true, taskMsgModels, "查询成功");
        } catch (Exception e) {
            logger.error("查询失败：" + e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    @Override
    public BocloudResult<List<TaskMsgModel>> historyList() {
        try {
            TaskManager taskManager = this.getServiceInstance().getTaskManager();
            TaskDescription description = taskManager.getDescription();
            TypeDescription[] reason = description.getReason();
            methodInfo = description.getMethodInfo();
            TaskFilterSpec tfs = createTaskFilterSpec(this.getRootFolder());
            TaskHistoryCollector thc = taskManager.createCollectorForTasks(tfs);
            TaskInfo[] tis = null;
            List<TaskMsgModel> taskMsgModels = new ArrayList<>();
            while (true) {
                tis = thc.readNextTasks(50);
                if (tis == null) {
                    break;
                }
                if (null != tis && tis.length > 0) {
                    for (TaskInfo taskInfo : tis) {
                        taskMsgModels.add(convertor(taskInfo));
                    }
                }
            }
            BocloudResult<List<TaskMsgModel>> listBocloudResult = this.list();
            if (listBocloudResult.isSuccess() && !ListTool.isEmpty(listBocloudResult.getData())) {
                taskMsgModels.addAll(listBocloudResult.getData());
            }
            taskMsgModels.stream().distinct().collect(Collectors.toList());
            thc.destroyCollector();

            return new BocloudResult<>(true, taskMsgModels, "查询成功");
        } catch (Exception e) {
            logger.error("查询失败：" + e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    static TaskFilterSpec createTaskFilterSpec(ManagedEntity ent) {
        TaskFilterSpec tfs = new TaskFilterSpec();

        // only the root initiated tasks
        TaskFilterSpecByUsername nameFilter
                = new TaskFilterSpecByUsername();
        nameFilter.setUserList(new String[]{"Administrator"});
        // include tasks initiated by non-users,
        // for example, by ScheduledTaskManager.
        nameFilter.setSystemUser(true);
        tfs.setUserName(nameFilter);

        // only the tasks with one entity itself
        TaskFilterSpecByEntity entFilter =
                new TaskFilterSpecByEntity();
        entFilter.setEntity(ent.getMOR());
        entFilter.setRecursion(TaskFilterSpecRecursionOption.all);
        tfs.setEntity(entFilter);

        // only successfully finished tasks
        tfs.setState(new TaskInfoState[]{TaskInfoState.success});

        // only tasks started within last one month
        // strictly speaking, time should be retrieved from server
        TaskFilterSpecByTime tFilter = new TaskFilterSpecByTime();
        Calendar cal = Calendar.getInstance();
        cal.roll(Calendar.MONTH, -1);
        tFilter.setBeginTime(cal);
        //we ignore the end time here so it gets the latest.
        tFilter.setTimeType(TaskFilterSpecTimeOption.startedTime);
        tfs.setTime(tFilter);

        // Optionally, you limits tasks initiated by scheduled task
        // with the setScheduledTask() method.
        return tfs;
    }

    private TaskMsgModel convertor(TaskInfo taskInfo) {
        TaskMsgModel taskMsgModel = new TaskMsgModel();
        taskMsgModel.setKey(taskInfo.getKey());
        String descriptionId = taskInfo.getDescriptionId();
        String description = "";
        taskMsgModel.setDescriptionId(descriptionId);
        for (ElementDescription e : methodInfo) {
            if (e.getKey().equals(descriptionId)) {
                description = e.getLabel();
            }
        }
        taskMsgModel.setDescription(description);
        LocalizableMessage details = taskInfo.getDescription();
        if (null != details) {
            Optional.ofNullable(details.getMessage()).ifPresent(result -> {
                taskMsgModel.setDetails(result.replace("\'", "\\'"));
            });
        }
        taskMsgModel.setName(taskInfo.getName());
        taskMsgModel.setEntityName(taskInfo.getEntityName());
        taskMsgModel.setState(null == taskInfo.getState() ? null : taskInfo.getState().name());
        Integer progress = taskInfo.getProgress();
        if (null == progress) {
            if (taskInfo.getState() == TaskInfoState.success) {
                progress = 100;
            } else if (taskInfo.getState() == TaskInfoState.error) {
                progress = 100;
            } else if (taskInfo.getState() == TaskInfoState.queued) {
                progress = 0;
            }
        }
        taskMsgModel.setProgress(progress);
        taskMsgModel.setError(null == taskInfo.getError() ? null : taskInfo.getError().getLocalizedMessage().replace("\'", "\\'"));
        TaskReason reason = taskInfo.getReason();
        boolean reasonRes = null == reason;
        taskMsgModel.setReason(reasonRes ? null : reason.toString());
        taskMsgModel.setQueueTime(null == taskInfo.getQueueTime() ? null : taskInfo.getQueueTime().getTime());
        Calendar startTime = taskInfo.getStartTime();
        taskMsgModel.setStartTime(null == startTime ? null : startTime.getTime());
        Calendar completeTime = taskInfo.getCompleteTime();
        taskMsgModel.setCompleteTime(null == completeTime ? null : completeTime.getTime());
        taskMsgModel.setEventChainId(taskInfo.getEventChainId());
        taskMsgModel.setChangeTag(taskInfo.getChangeTag());
        taskMsgModel.setParentTaskKey(taskInfo.getParentTaskKey());
        taskMsgModel.setRootTaskKey(taskInfo.getRootTaskKey());
        taskMsgModel.setActivationId(taskInfo.getActivationId());
        if (null != taskInfo.getQueueTime() && null != startTime) {
            Long waitTime = startTime.getTimeInMillis() - taskInfo.getQueueTime().getTimeInMillis();
            taskMsgModel.setWaitTime(Math.abs(waitTime));
        } else {
            taskMsgModel.setWaitTime(0L);
        }
        if (null != completeTime && null != startTime) {
            String timeInExecution = DateUtil.formatTime(completeTime.getTime().getTime() - startTime.getTime().getTime());
            taskMsgModel.setTimeInExecution(timeInExecution);
        }
        String initiator = "";

        if (reason instanceof TaskReasonSchedule) {
            TaskReasonSchedule taskReasonSchedule = (TaskReasonSchedule) reason;
            initiator = taskReasonSchedule.getName();
        }
        if (StringUtils.isBlank(initiator)) {
            if (!reasonRes) {
                initiator = taskReason(taskInfo.getReason());
            }
        }
        taskMsgModel.setInitiator(initiator);
        return taskMsgModel;
    }

    static String taskReason(TaskReason tr) {
        if (tr instanceof TaskReasonAlarm) {
            return "Alarm";
        } else if (tr instanceof TaskReasonSchedule) {
            return "ScheduledTask";
        } else if (tr instanceof TaskReasonSystem) {
            return "System";
        } else if (tr instanceof TaskReasonUser) {
            return ((TaskReasonUser) tr).getUserName();
        }
        return "Unknown";
    }

}
