package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VAppModel;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.cmp.provider.utils.AssembleTools;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.bocloud.vmware.vim25.*;
import com.bocloud.vmware.vim25.mo.*;
import io.jsonwebtoken.lang.Assert;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * vmware vApp操作接口实现类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/14
 */
@Provider("bocloud.cmp.vmware")
public class VmwareVAppProvider extends VmwareProvider implements VAppProvider {

    private static Logger logger = LoggerFactory.getLogger(VmwareVAppProvider.class);

    public VmwareVAppProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(VAppModel model) {
        try {
            ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    model.getVdcName());
            Assert.notNull(entity, "数据中心不存在！");
            Datacenter center = (Datacenter) entity;
            InventoryNavigator navigator = new InventoryNavigator(center);
            String parentName = model.getParentName();
            VirtualApp app = null;
            if (!StringUtils.isEmpty(parentName)) {
                ManagedEntity parentEntity = new InventoryNavigator(center).searchManagedEntity(VAPP, parentName);
                Assert.notNull(parentEntity, "父vApp不存在！");
                VirtualApp parent = (VirtualApp) parentEntity;
                // 资源池配置
                ResourceConfigSpec resSpec = JSONObject.parseObject(model.getConfig(), ResourceConfigSpec.class);
                app = parent.createVApp(model.getName(), resSpec, new VAppConfigSpec(), null);
            } else {
                String poolVal = model.getPoolVal();
                Assert.notNull(poolVal, "资源池不存在");
                ManagedEntity[] entities = navigator.searchManagedEntities(RP);
                // 如果资源池的父集群开启了drs，才可以创建vapp
                String targetCategory = model.getTargetCategory();
                if ("CLUSTER".equalsIgnoreCase(targetCategory)) {
                    ManagedEntity[] hostEntites = center.getHostFolder().getChildEntity();
                    if (entities == null || entities.length == 0) {
                        return new BocloudResult(false, "获取vcenter集群信息失败");
                    }
                    for (int i = 0; i < hostEntites.length; i++) {
                        ManagedEntity hostEntity = entities[i];
                        if (hostEntity instanceof ClusterComputeResource) {
                            ClusterComputeResource clusterResource = (ClusterComputeResource) hostEntity;
                            if (clusterResource.getName().equalsIgnoreCase(model.getTargetName())) {
                                ClusterDrsConfigInfo drsConfigInfo = clusterResource.getConfiguration().getDrsConfig();
                                if (null == drsConfigInfo) {
                                    return new BocloudResult(false, "未查询到所在集群的DRS配置");
                                }
                                if (!drsConfigInfo.getEnabled()) {
                                    return new BocloudResult(false, "所在集群的DRS配置未启用，创建失败");
                                }
                            }
                        }
                    }
                }
                ResourcePool resourcePool = null;
                // 查询资源池
                if (null != entities && entities.length > 0) {
                    for (ManagedEntity managedEntity : entities) {
                        ResourcePool pool = (ResourcePool) managedEntity;
                        if (pool.getMOR().getVal().equalsIgnoreCase(model.getPoolVal())) {
                            resourcePool = pool;
                            break;
                        }
                    }
                }
                Assert.notNull(resourcePool, "资源池不存在");
                // 资源池配置
                ResourceConfigSpec resSpec = JSONObject.parseObject(model.getConfig(), ResourceConfigSpec.class);
                app = resourcePool.createVApp(model.getName(), resSpec, new VAppConfigSpec(), center.getVmFolder());
            }
            Assert.notNull(app, "创建失败");
            return new BocloudResult(true, AssembleTools.getInstance().assembleVapp(app), "vApp创建成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "vApp创建失败：" + e.getMessage());
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult modify(VAppModel bean) {
        try {
            ManagedEntity dcEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    bean.getVdcName());
            Assert.notNull(dcEntity, "数据中心不存在");
            Datacenter center = (Datacenter) dcEntity;
            ManagedEntity entity = new InventoryNavigator(center).searchManagedEntity(VAPP, bean.getOldName());
            Assert.notNull(entity, "vApp不存在！");
            VirtualApp vApp = (VirtualApp) entity;
            // 资源池配置
            ResourceConfigSpec resSpec = JSONObject.parseObject(bean.getConfig(), ResourceConfigSpec.class);
            vApp.updateConfig(bean.getName(), resSpec);
            VAppConfigSpec vAppConfigSpec = new VAppConfigSpec();
            String vappConfig = bean.getVappConfig();
            JSONObject vappConfigObj = JSONObject.parseObject(vappConfig);
            vAppConfigSpec.setInstallBootRequired(false);
            vAppConfigSpec.setInstallBootStopDelay(60);
            vAppConfigSpec.setAnnotation(vappConfigObj.getString("annotation"));
            vAppConfigSpec.setInstanceUuid(vappConfigObj.getString("instanceUuid"));
            List<VAppEntityConfigInfo> vAppEntityConfigInfos = JSONArray.parseArray(vappConfigObj.getString("entityConfig"), VAppEntityConfigInfo.class);
            //启动顺序
            if (!ListTool.isEmpty(vAppEntityConfigInfos)) {
                vAppConfigSpec.setEntityConfig(vAppEntityConfigInfos.toArray(new VAppEntityConfigInfo[vAppEntityConfigInfos.size()]));
            }
            //ip分配
            String ipAssignment = vappConfigObj.getString("ipAssignment");
            if (!StringUtils.isEmpty(ipAssignment)) {
                JSONObject ipAssignmentObj = JSONObject.parseObject(ipAssignment);
                List<String> supportedIpProtocols = JSONArray.parseArray(ipAssignmentObj.getString("supportedIpProtocol"), String.class);
                String[] supportedIpProtocol = new String[]{};
                if (!ListTool.isEmpty(supportedIpProtocols)) {
                    supportedIpProtocol = supportedIpProtocols.toArray(new String[supportedIpProtocols.size()]);
                }
                VAppIPAssignmentInfo vAppIPAssignmentInfo = new VAppIPAssignmentInfo();
                vAppIPAssignmentInfo.setSupportedIpProtocol(supportedIpProtocol);
                vAppIPAssignmentInfo.setIpProtocol(ipAssignmentObj.getString("ipProtocol"));
                vAppIPAssignmentInfo.setIpAllocationPolicy(ipAssignmentObj.getString("ipAllocationPolicy"));

                List<String> supportedAllocationSchemes = JSONArray.parseArray(ipAssignmentObj.getString("supportedAllocationScheme"), String.class);
                String[] supportedAllocationScheme = new String[]{};
                if (!ListTool.isEmpty(supportedAllocationSchemes)) {
                    supportedAllocationScheme = supportedAllocationSchemes.toArray(new String[supportedAllocationSchemes.size()]);
                }
                vAppIPAssignmentInfo.setSupportedAllocationScheme(supportedAllocationScheme);
                vAppConfigSpec.setIpAssignment(vAppIPAssignmentInfo);
            }
            //详细信息
            List<JSONObject> productObjs = JSONArray.parseArray(vappConfigObj.getString("product"), JSONObject.class);
            if (!ListTool.isEmpty(productObjs)) {
                logger.info("product info:{}", JSONObject.toJSON(productObjs));
                VAppProductSpec[] specs = new VAppProductSpec[productObjs.size()];
                for (int i = 0; i < productObjs.size(); i++) {
                    JSONObject vAppProductInfo = productObjs.get(i);
                    specs[i] = JSONObject.parseObject(vAppProductInfo.toJSONString(), VAppProductSpec.class);
                }
                vAppConfigSpec.setProduct(specs);
            }
            vApp.updateVAppConfig(vAppConfigSpec);
            return new BocloudResult(true, bean, "vApp修改成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "vApp修改失败：" + e.getMessage());
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult remove(String vAppName) {
        try {
            ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(VAPP, vAppName);
            Assert.notNull(entity, "vApp不存在");
            VirtualApp vapp = (VirtualApp) entity;
            Task task = vapp.destroy_Task();
            if (task.waitForTask().equals(Task.SUCCESS)) {
                return new BocloudResult(true, "vApp删除成功");
            } else {
                return new BocloudResult(false, "vApp删除失败：" + task.getTaskInfo().getError().getLocalizedMessage());
            }
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "vApp删除失败：" + e.getMessage());
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult start(String vAppName) {
        try {
            ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(VAPP, vAppName);
            Assert.notNull(entity, "vApp不存在");
            VirtualApp vapp = (VirtualApp) entity;
            Task task = vapp.powerOnVApp_Task();
            if (task.waitForTask().equals(Task.SUCCESS)) {
                return new BocloudResult(true, "vApp打开电源成功");
            } else {
                return new BocloudResult(false, "vApp打开电源失败：" + task.getTaskInfo().getError().getLocalizedMessage());
            }
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "vApp打开电源失败：" + e.getMessage());
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult stop(String vAppName) {
        try {
            ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(VAPP, vAppName);
            Assert.notNull(entity, "vApp不存在");
            VirtualApp vapp = (VirtualApp) entity;
            Task task = vapp.powerOnVApp_Task();
            if (task.waitForTask().equals(Task.SUCCESS)) {
                return new BocloudResult(true, "vApp关闭电源成功");
            } else {
                return new BocloudResult(false, "vApp关闭电源失败：" + task.getTaskInfo().getError().getLocalizedMessage());
            }
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "vApp关闭电源失败：" + e.getMessage());
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult suspend(String vAppName) {
        try {
            ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(VAPP, vAppName);
            Assert.notNull(entity, "vApp不存在");
            VirtualApp vapp = (VirtualApp) entity;
            Task task = vapp.powerOnVApp_Task();
            if (task.waitForTask().equals(Task.SUCCESS)) {
                return new BocloudResult(true, "vApp挂起成功");
            } else {
                return new BocloudResult(false, "vApp挂起失败：" + task.getTaskInfo().getError().getLocalizedMessage());
            }
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "vApp挂起失败：" + e.getMessage());
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }
}
