package com.bocloud.cmp.provider.common;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ClusterModel;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.VDCModel;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.cmp.provider.utils.MathUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.vmware.vim25.DatastoreSummary;
import com.bocloud.vmware.vim25.HostHardwareSummary;
import com.bocloud.vmware.vim25.HostListSummary;
import com.bocloud.vmware.vim25.HostSystemConnectionState;
import com.bocloud.vmware.vim25.mo.ClusterComputeResource;
import com.bocloud.vmware.vim25.mo.ComputeResource;
import com.bocloud.vmware.vim25.mo.Datacenter;
import com.bocloud.vmware.vim25.mo.Datastore;
import com.bocloud.vmware.vim25.mo.HostSystem;
import com.bocloud.vmware.vim25.mo.InventoryNavigator;
import com.bocloud.vmware.vim25.mo.ManagedEntity;

/**
 * vmware数据中心操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/14
 */
@Provider("bocloud.cmp.vmware")
public class VmwareVdcProvider extends VmwareProvider implements VdcProvider {

	private static Logger logger = LoggerFactory.getLogger(VmwareVdcProvider.class);

	public VmwareVdcProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		List<VDCModel> dcList = new ArrayList<>();
		try {
			ManagedEntity[] dcEntities = new InventoryNavigator(this.getRootFolder()).searchManagedEntities(DC);
			dcList = Arrays.stream(Optional.ofNullable(dcEntities).orElse(new ManagedEntity[0])).map(managedEntity -> {
				try {
					return assembleDc(managedEntity);
				} catch (Exception e) {
					logger.error("", e);
					return null;
				}
			}).filter(vdcModel -> Objects.nonNull(vdcModel)).collect(Collectors.toList());
		} catch (Exception e) {
			logger.error("sync info error:", e);
			return new BocloudResult(false, "同步失败");
		} finally {
			this.getServiceInstance().getServerConnection().logout();
		}
		return new BocloudResult(true, dcList, "success");
	}

	/**
	 * 组装数据中心相关元数据
	 *
	 * @param entity
	 * @return
	 * @throws Exception
	 */
	public VDCModel assembleDc(ManagedEntity entity) throws Exception {
		Datacenter datacenter = (Datacenter) entity;
		VDCModel vdc = new VDCModel(datacenter.getName(), 0L, 0D, 0D, 0L, 0D, 0D, null, null, null,
				datacenter.getMOR().getVal());
		List<ClusterModel> clusters = new ArrayList<>();
		List<HostModel> hosts = new ArrayList<>();
		ManagedEntity[] entityArray = Optional.ofNullable(datacenter.getHostFolder().getChildEntity())
				.orElse(new ManagedEntity[0]);
		for (ManagedEntity hostEntity : entityArray) {
			if (hostEntity instanceof ClusterComputeResource) {
				ClusterComputeResource cluster = (ClusterComputeResource) hostEntity;
				clusters.add(assembleCluster(cluster));
			} else if (hostEntity instanceof ComputeResource) {
				ComputeResource hostSystem = (ComputeResource) hostEntity;
				HostModel model = assembleHost(hostSystem.getHosts()[0]);
				if (null != model) {
					hosts.add(model);
				}
			}
		}
		Long cpuTotal = 0L;
		Double memTotal = 0D;
		Double diskTotal = 0D;
		Long cpuUsed = 0L;
		Double memUsed = 0D;
		Double diskUsed = 0D;
		for (ClusterModel clusterModel : clusters) {
			cpuTotal += clusterModel.getCpuTotal();
			memTotal += clusterModel.getMemTotal();
			// diskTotal += clusterModel.getDiskTotal();
			cpuUsed += clusterModel.getCpuUsed();
			memUsed += clusterModel.getMemUsed();
			// diskUsed += clusterModel.getDiskUsed();
		}
		for (HostModel hostModel : hosts) {
			cpuTotal += hostModel.getCpuTotal();
			memTotal += hostModel.getMemTotal();
			// diskTotal += hostModel.getDiskTotal();
			cpuUsed += hostModel.getCpuUsed();
			memUsed += hostModel.getMemUsed();
			// diskUsed += hostModel.getDiskUsed();
		}
		// 获取数据中心下存储列表
		Datastore[] datastores = datacenter.getDatastores();
		if (null != datastores && datastores.length > 0) {
			for (Datastore store : datastores) {
				DatastoreSummary datastoreSummary = store.getSummary();
				if (null != datastoreSummary) {
					Long disk = datastoreSummary.getCapacity();
					Long free = datastoreSummary.getFreeSpace();
					// 总容量
					BigDecimal capacity = new BigDecimal(disk);
					// 空闲容量
					BigDecimal freedisk = new BigDecimal(free);
					Double dTotal = capacity.divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP)
							.doubleValue();
					Double diskFree = freedisk.divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP)
							.doubleValue();
					diskUsed += (dTotal - diskFree);
					diskTotal += dTotal;
				}
			}
		}
		diskTotal = MathUtils.round(diskTotal, 2);
		diskUsed = MathUtils.round(diskUsed, 2);
		vdc.setCpuTotal(cpuTotal);
		vdc.setDiskTotal(diskTotal);
		vdc.setMemTotal(memTotal);
		vdc.setCpuUsed(cpuUsed);
		vdc.setMemUsed(memUsed);
		vdc.setDiskUsed(diskUsed);
		return vdc;
	}

	/**
	 * 组装主机集群相关元数据
	 *
	 * @param entity
	 * @throws Exception
	 */
	public ClusterModel assembleCluster(ClusterComputeResource entity) throws Exception {
		ClusterModel cluster = new ClusterModel();
		cluster.setName(entity.getName());
		cluster.setClusterVal(entity.getMOR().getVal());
		HostSystem[] hostSystems = entity.getHosts();
		List<HostModel> hosts = new ArrayList<>();
		if (null != hostSystems) {
			for (HostSystem hostSystem : hostSystems) {
				HostModel model = assembleHost(hostSystem);
				if (null != model) {
					hosts.add(model);
				}
			}
		}
		Long cpuTotal = 0L;
		Double memTotal = 0D;
		Double diskTotal = 0D;
		Long cpuUsed = 0L;
		Double memUsed = 0D;
		Double diskUsed = 0D;
		int cpuCore = 0;
		for (HostModel hostModel : hosts) {
			cpuTotal += hostModel.getCpuTotal();
			memTotal += hostModel.getMemTotal();
			diskTotal += hostModel.getDiskTotal();
			cpuUsed += hostModel.getCpuUsed();
			memUsed += hostModel.getMemUsed();
			diskUsed += hostModel.getDiskUsed();
			cpuCore += hostModel.getCpuCore();
		}
		cluster.setCpuTotal(cpuTotal);
		cluster.setMemTotal(memTotal);
		cluster.setDiskTotal(diskTotal);
		cluster.setCpuUsed(cpuUsed);
		cluster.setMemUsed(memUsed);
		cluster.setDiskUsed(diskUsed);
		cluster.setCpuCore(cpuCore);
		return cluster;
	}

	/**
	 * 组装主机集群相关元数据
	 *
	 * @param host
	 * @throws Exception
	 */
	public HostModel assembleHost(HostSystem host) throws Exception {
		HostListSummary summary = host.getSummary();
		if (null == summary) {
			return null;
		}
		HostHardwareSummary hardware = summary.getHardware();
		if (null == hardware) {
			return null;
		}
		HostModel hostModel = new HostModel();
		hostModel.setCpuNum((int) hardware.getNumCpuPkgs());
		hostModel.setCpuCore((int) hardware.getNumCpuCores());
		hostModel.setCpuTotal(Long.valueOf(hardware.getCpuMhz()) * hardware.getNumCpuCores());
		HostSystemConnectionState connState = summary.getRuntime().getConnectionState();// 物理机连接状态
		if (connState.equals(HostSystemConnectionState.connected)) {
			hostModel.setCpuUsed(summary.getQuickStats().getOverallCpuUsage().longValue());
			double memory = new BigDecimal(summary.getQuickStats().getOverallMemoryUsage())
					.divide(new BigDecimal(1024), 2, RoundingMode.HALF_UP).doubleValue();
			hostModel.setMemUsed(memory);
		} else {
			hostModel.setCpuUsed(0L);
			hostModel.setMemUsed(0D);
		}
		BigDecimal memory = new BigDecimal(hardware.getMemorySize());
		hostModel.setMemTotal(memory.divide(new BigDecimal(1024 * 1024 * 1024), 2, RoundingMode.HALF_UP).doubleValue());
		Datastore[] datastores = host.getDatastores();
		Long diskCapacity = 0L;
		Long freeCapacity = 0L;
		if (null != datastores) {
			for (Datastore datastore : datastores) {
				DatastoreSummary datastoreSummary = datastore.getSummary();
				diskCapacity += datastoreSummary.getCapacity();
				freeCapacity += datastoreSummary.getFreeSpace();
			}
		}
		BigDecimal capacity = new BigDecimal(diskCapacity);
		BigDecimal freedisk = new BigDecimal(freeCapacity);
		Double diskTotal = capacity.divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP).doubleValue();
		Double diskFree = freedisk.divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP).doubleValue();
		hostModel.setDiskTotal(diskTotal);
		hostModel.setDiskUsed(new BigDecimal(diskTotal).subtract(new BigDecimal(diskFree))
				.setScale(2, RoundingMode.HALF_UP).doubleValue());
		return hostModel;
	}

}
