package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.cmp.provider.utils.AssembleTools;
import com.bocloud.cmp.provider.utils.MathUtils;
import com.bocloud.cmp.provider.utils.SingleAssembleTools;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.vmware.vim25.*;
import com.bocloud.vmware.vim25.mo.HostSystem;
import com.bocloud.vmware.vim25.mo.*;
import io.jsonwebtoken.lang.Assert;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * vmware集群操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/14
 */
@Provider("bocloud.cmp.vmware")
public class VmwareClusterProvider extends VmwareProvider implements ClusterProvider {

    private static Logger logger = LoggerFactory.getLogger(VmwareClusterProvider.class);

    public VmwareClusterProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult listByVdc(String vdcName) {
        logger.info("查询数据中心{}下集群", vdcName);
        try {
            ManagedEntity managedEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, vdcName);
            Assert.notNull(managedEntity, "数据中心" + vdcName + "不存在！");
            Datacenter datacenter = (Datacenter) managedEntity;
            // 获取该数据中心下的所有的集群
            ManagedEntity[] clusterEntities = datacenter.getHostFolder().getChildEntity();
            List<ClusterModel> clusters = new ArrayList<>();
            for (ManagedEntity clusterEntity : clusterEntities) {
                if (clusterEntity instanceof ClusterComputeResource) {
                    ClusterComputeResource cluster = (ClusterComputeResource) clusterEntity;
                    ClusterModel model = SingleAssembleTools.getInstance().assembleCluster(cluster);
                    if (null != model) {
                        clusters.add(model);
                    }
                }
            }
            return new BocloudResult(true, clusters, "数据中心" + vdcName + "下集群查询成功");
        } catch (Exception e) {
            logger.error("数据中心" + vdcName + "下集群查询失败:" + e);
            return new BocloudResult(false, "数据中心" + vdcName + "下集群查询失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult create(ClusterOperateModel model) {
        logger.info("开始创建集群{}", model.getClusterName());
        try {
            ManagedEntity managedEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    model.getVdcName());
            Assert.notNull(managedEntity, "数据中心" + model.getVdcName() + "不存在！");
            Datacenter datacenter = (Datacenter) managedEntity;
            // 获取该数据中心下的所有的集群
            ManagedEntity[] childEntities = datacenter.getHostFolder().getChildEntity();
            boolean exist = Arrays.stream(childEntities).filter(entity -> entity instanceof ClusterComputeResource)
                    .anyMatch(cluster -> cluster.getName().equals(model.getClusterName()));
            Assert.isTrue(!exist, "集群已存在");
            // 创建集群
            ClusterConfigSpecEx clusterConfigSpec = new ClusterConfigSpecEx();
            VsanConfigInfo vsanConfig = model.getVsanConfig();
            if (null != vsanConfig) {
                VsanClusterConfigInfo vsanClusterConfigInfo = new VsanClusterConfigInfo();
                vsanClusterConfigInfo.setEnabled(null != vsanConfig.getEnabled() && vsanConfig.getEnabled());
                VsanClusterConfigInfoHostDefaultInfo defaultInfo = new VsanClusterConfigInfoHostDefaultInfo();
                defaultInfo.setAutoClaimStorage(null != vsanConfig.getAutoClaimStorage() && vsanConfig.getAutoClaimStorage());
                defaultInfo.setChecksumEnabled(null != vsanConfig.getChecksumEnabled() && vsanConfig.getChecksumEnabled());
                defaultInfo.setUuid(vsanConfig.getUuid());
                clusterConfigSpec.setVsanConfig(vsanClusterConfigInfo);
            }
            ClusterComputeResource clusterResult = datacenter.getHostFolder().createClusterEx(model.getClusterName(),
                    clusterConfigSpec);
            Assert.notNull(clusterResult, "创建集群失败！");
            logger.info("创建集群{}成功！", model.getClusterName());
            return new BocloudResult(true, assembleCluster(clusterResult), "创建集群成功");
        } catch (Exception e) {
            logger.info("创建集群{}异常！", model.getClusterName(), e);
            return new BocloudResult(false, "创建集群失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult remove(ClusterOperateModel model) {
        logger.info("开始删除集群{}", model.getClusterName());
        try {
            // 获取数据中心
            Datacenter datacenter = (Datacenter) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    model.getVdcName());
            Assert.notNull(datacenter, "数据中心" + model.getVdcName() + "不存在！");
            // 获取要删除的集群
            ManagedEntity cluster = new InventoryNavigator(datacenter).searchManagedEntity("ClusterComputeResource",
                    model.getClusterName());
            if (null == cluster) {
                logger.error("删除集群{}成功！该集群可能已被删除！", model.getClusterName());
                return new BocloudResult(true, "删除集群成功");
            }
            // 删除集群任务
            Task task = cluster.destroy_Task();
            logger.info("创建删除集群{}任务并执行！", model.getClusterName());
            if (task.waitForTask().equals(Task.SUCCESS)) {
                logger.info("删除集群{}任务执行成功！", model.getClusterName());
                return new BocloudResult(true, "删除集群成功");
            } else {
                logger.info("删除集群{}任务执行失败！原因：{}", model.getClusterName(),
                        task.getTaskInfo().getError().getLocalizedMessage());
                return new BocloudResult(false, "删除集群失败:" + task.getTaskInfo().getError().getLocalizedMessage());
            }
        } catch (Exception e) {
            logger.error("删除集群失败", e);
            return new BocloudResult(false, "删除集群失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    /**
     * 组装主机集群相关元数据
     *
     * @param entity
     * @return
     * @throws Exception
     */
    private ClusterModel assembleCluster(ClusterComputeResource entity) throws Exception {
        ClusterModel cluster = new ClusterModel();
        ResourcePool resourcePool = entity.getResourcePool();
        if (null != resourcePool) {
            cluster.setResourcePool(assembleVmwareResourcePool(resourcePool));
        }
        cluster.setName(entity.getName());
        cluster.setClusterVal(entity.getMOR().getVal());
        cluster.setTemplateNum(0);
        cluster.setVmNum(0);
        cluster.setHostNum(0);
        cluster.setCpuTotal(0L);
        cluster.setMemTotal(0D);
        cluster.setDiskTotal(0d);
        cluster.setCpuUsed(0L);
        cluster.setMemUsed(0D);
        cluster.setDiskUsed(0d);
        cluster.setCpuCore(0);
        return cluster;
    }

    /**
     * 组装资源池相关元数据
     *
     * @param resourcePool
     * @return
     * @throws Exception
     */
    private ResourcePoolModel assembleVmwareResourcePool(ResourcePool resourcePool) throws Exception {
        ResourcePoolModel poolModel = new ResourcePoolModel();
        ManagedEntity parent = resourcePool.getParent();
        ManagedEntity owner = resourcePool.getOwner();
        ResourceConfigSpec spec = resourcePool.getConfig();
        if (null != spec) {
            // 集群或宿主机本身资源池的父资源池名和所属对象名称相同
            if (!parent.getName().equals(owner.getName())) {
                poolModel.setParentName(parent.getName());
                poolModel.setParentPoolVal(parent.getMOR().getVal());
            }
            poolModel.setName(resourcePool.getName());
            Calendar modified = spec.getLastModified();
            if (null != modified) {
                poolModel.setGmtModify(modified.getTime());
            }
            ResourceAllocationInfo cpuInfo = spec.getCpuAllocation();
            if (null != cpuInfo) {
                poolModel.setCpuReservation(cpuInfo.getReservation());
                poolModel.setCpuExpanReservation(cpuInfo.getExpandableReservation());
                poolModel.setCpuLimit(cpuInfo.getLimit());
                SharesInfo cpuShares = cpuInfo.getShares();
                if (null != cpuShares) {
                    poolModel.setCpuShareLevel(cpuShares.getLevel().name());
                    poolModel.setCpuShares(cpuShares.getShares());
                }
            }
            ResourceAllocationInfo memInfo = spec.getMemoryAllocation();
            if (null != memInfo) {
                poolModel.setMemReservation(memInfo.getReservation());
                poolModel.setMemExpanReservation(memInfo.getExpandableReservation());
                poolModel.setMemLimit(memInfo.getLimit());
                SharesInfo memShares = memInfo.getShares();
                if (null != memShares) {
                    poolModel.setMemShareLevel(memShares.getLevel().name());
                    poolModel.setMemShares(memShares.getShares());
                }
            }
        }
        poolModel.setPoolVal(resourcePool.getMOR().getVal());
        return poolModel;
    }

    /**
     * 获取集群信息
     * <p>
     * 只查询cpu、内存、磁盘的总量和使用信息
     * </p>
     *
     * @param clusterVal 集群的标识
     * @return 集群信息
     * @author sxp
     * @since Apr 26, 2018
     */
    @Override
    public BocloudResult detail(String clusterVal) {
        Objects.requireNonNull(clusterVal, "clusterVal");
        ClusterModel clusterModel = null;
        try {
            ManagedEntity[] dcEntities = new InventoryNavigator(this.getRootFolder()).searchManagedEntities(DC);
            Assert.notNull(dcEntities, "获取vmware集群信息失败");
            Datacenter dc;
            ManagedEntity[] entities;
            ManagedEntity hostEntity;
            ClusterComputeResource clusterResource;

            for (ManagedEntity entity : dcEntities) {
                dc = (Datacenter) entity;

                entities = dc.getHostFolder().getChildEntity();
                if (entities == null || entities.length == 0) {
                    continue;
                }

                for (int i = 0; i < entities.length; i++) {
                    hostEntity = entities[i];
                    if (hostEntity instanceof ClusterComputeResource) {
                        clusterResource = (ClusterComputeResource) hostEntity;
                        ClusterDrsConfigInfo drsConfig = new ClusterDrsConfigInfo();
                        drsConfig.setDefaultVmBehavior(DrsBehavior.manual);
                        drsConfig.setEnableVmBehaviorOverrides(false);
                        drsConfig.setEnabled(true);
                        drsConfig.setVmotionRate(3);
                        ClusterConfigSpecEx config = new ClusterConfigSpecEx();
                        config.setDrsConfig(drsConfig);
                        ClusterDpmConfigInfo dpmConfig = new ClusterDpmConfigInfo();
                        config.setDpmConfig(dpmConfig);
//						ClusterConfigSpec  spec=new ComputeResourceConfigSpec();
//						clusterResource.reconfigureCluster_Task(config, true);
                        ComputeResourceConfigSpec spec = new ComputeResourceConfigSpec();
                        clusterModel = simpleAssembleCluster(clusterResource, clusterVal);
                        if (clusterModel != null) {
                            break;
                        }
                    }
                }

                if (clusterModel != null) {
                    break;
                }
            }

        } catch (Exception e) {
            logger.error("获取vmware集群信息失败", e);
            return new BocloudResult(false, "获取vmware集群信息失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }

        if (clusterModel == null) {
            return new BocloudResult(false, "获取vmware集群信息失败");
        }
        return new BocloudResult(true, clusterModel, "success");
    }

    @Override
    public BocloudResult detailCluster(String clusterVal) {
        Objects.requireNonNull(clusterVal, "clusterVal");
        ClusterModel clusterModel = null;
        try {
            ManagedEntity[] dcEntities = new InventoryNavigator(this.getRootFolder()).searchManagedEntities(DC);
            if (dcEntities == null) {
                return new BocloudResult(false, "获取vmware集群信息失败");
            }
            Datacenter dc;
            ManagedEntity[] entities;
            ManagedEntity hostEntity;
            ClusterComputeResource clusterResource;
            for (ManagedEntity entity : dcEntities) {
                dc = (Datacenter) entity;
                entities = dc.getHostFolder().getChildEntity();
                if (entities == null || entities.length == 0) {
                    continue;
                }
                for (int i = 0; i < entities.length; i++) {
                    hostEntity = entities[i];
                    if (hostEntity instanceof ClusterComputeResource) {
                        clusterResource = (ClusterComputeResource) hostEntity;
                        String morVal = entity.getMOR().getVal();
                        if (clusterVal.equalsIgnoreCase(morVal)) {
                            clusterModel = AssembleTools.getInstance().assembleCluster(clusterResource);
                            break;
                        }
                    }
                }
            }
            if (clusterModel == null) {
                return new BocloudResult(false, "获取vmware集群信息失败");
            }
            return new BocloudResult(true, clusterModel, "success");
        } catch (Exception e) {
            logger.error("获取vmware集群信息失败", e);
            return new BocloudResult(false, "获取vmware集群信息失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult create(ClusterModel clusterModel) {
        return null;
    }

    @Override
    public BocloudResult modify(ClusterModel clusterModel) {
        return null;
    }

    @Override
    public BocloudResult remove(String aggregateId) {
        return null;
    }

    @Override
    public BocloudResult addHost(String aggregateId, String hostName) {
        return null;
    }

    @Override
    public BocloudResult removeHost(String aggregateId, String hostName) {
        return null;
    }

    @Override
    public BocloudResult summary(String clusterVal) {
        return null;
    }

    @Override
    public BocloudResult queryByName(String name) {
        return null;
    }

    /**
     * 获取集群信息(ClusterModel)
     * <p>
     * 只查询cpu、内存、磁盘的总量和使用信息
     * </p>
     *
     * @author sxp
     * @since Apr 26, 2018
     */
    private ClusterModel simpleAssembleCluster(ClusterComputeResource entity, String clusterVal) throws Exception {
        String morVal = entity.getMOR().getVal();
        if (morVal == null) {
            morVal = "";
        }
        if (!morVal.equals(clusterVal)) {
            return null;
        }

        ClusterModel clusterModel = new ClusterModel();
        clusterModel.setName(entity.getName());
        clusterModel.setClusterVal(morVal);

        HostSystem[] hostSystems = entity.getHosts();
        if (hostSystems == null || hostSystems.length == 0) {
            return null;
        }

        HostModel model;
        List<HostModel> hosts = new ArrayList<>(hostSystems.length);
        for (HostSystem hostSystem : hostSystems) {
            model = simpleAssembleVmwareHost(hostSystem);
            if (model != null) {
                hosts.add(model);
            }
        }
        Long cpuTotal = 0L;
        Double memTotal = 0D;
        double diskTotal = 0D;
        Long cpuUsed = 0L;
        Double memUsed = 0D;
        double diskUsed = 0D;

        for (HostModel hostModel : hosts) {
            if (hostModel == null) {
                continue;
            }
            cpuTotal += hostModel.getCpuTotal();
            memTotal += hostModel.getMemTotal();
            // diskTotal += hostModel.getDiskTotal();
            cpuUsed += hostModel.getCpuUsed();
            memUsed += hostModel.getMemUsed();
            // diskUsed += hostModel.getDiskUsed();
        }
        // 获取数据中心下存储列表
        Datastore[] datastores = entity.getDatastores();
        if (null != datastores && datastores.length > 0) {
            for (Datastore store : datastores) {
                DatastoreSummary datastoreSummary = store.getSummary();
                if (null != datastoreSummary) {
                    long disk = datastoreSummary.getCapacity();
                    long free = datastoreSummary.getFreeSpace();
                    // 总容量
                    BigDecimal capacity = new BigDecimal(disk);
                    // 空闲容量
                    BigDecimal freedisk = new BigDecimal(free);
                    double dTotal = capacity.divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP)
                            .doubleValue();
                    double diskFree = freedisk.divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP)
                            .doubleValue();
                    diskUsed += (dTotal - diskFree);
                    diskTotal += dTotal;
                }
            }
        }
        diskTotal = MathUtils.round(diskTotal, 2);
        diskUsed = MathUtils.round(diskUsed, 2);
        clusterModel.setCpuTotal(cpuTotal);
        clusterModel.setMemTotal(memTotal);
        clusterModel.setDiskTotal(diskTotal);
        clusterModel.setCpuUsed(cpuUsed);
        clusterModel.setMemUsed(memUsed);
        clusterModel.setDiskUsed(diskUsed);
        return clusterModel;
    }

    /**
     * 获取云主机信息
     * <p>
     * 只查询cpu、内存、磁盘的总量和使用信息
     * </p>
     *
     * @author sxp
     * @since Apr 26, 2018
     */
    private HostModel simpleAssembleVmwareHost(HostSystem host) throws Exception {
        HostModel hostModel = new HostModel();

        HostListSummary summary = host.getSummary();
        HostHardwareSummary hardware = summary.getHardware();
        if (null == hardware) {
            return null;
        }

        // 单位:MHz
        hostModel.setCpuTotal((long) hardware.getCpuMhz() * hardware.getNumCpuCores());

        // 单位MB
        BigDecimal memory = new BigDecimal(host.getHardware().getMemorySize());
        hostModel.setMemTotal(memory.divide(new BigDecimal(1024 * 1024 * 1024), 2, RoundingMode.HALF_UP).doubleValue());

        HostSystemConnectionState connState = summary.getRuntime().getConnectionState();
        if (connState.equals(HostSystemConnectionState.connected)) {
            hostModel.setCpuUsed(summary.getQuickStats().getOverallCpuUsage().longValue());
            double mem = new BigDecimal(summary.getQuickStats().getOverallMemoryUsage())
                    .divide(new BigDecimal(1024), 2, RoundingMode.HALF_UP).doubleValue();
            hostModel.setMemUsed(mem);
        } else {
            hostModel.setCpuUsed(0L);
            hostModel.setMemUsed(0D);
        }

        Datastore[] datastores = host.getDatastores();
        if (null != datastores) {
            Long diskCapacity = 0L, freeCapacity = 0L;

            for (Datastore datastore : datastores) {
                DatastoreSummary datastoreSummary = datastore.getSummary();
                diskCapacity += datastoreSummary.getCapacity();
                freeCapacity += datastoreSummary.getFreeSpace();
            }

            // 单位GB
            BigDecimal capacity = new BigDecimal(diskCapacity);
            BigDecimal freedisk = new BigDecimal(freeCapacity);
            Double diskTotal = capacity.divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP)
                    .doubleValue();
            Double diskFree = freedisk.divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP)
                    .doubleValue();
            hostModel.setDiskTotal(diskTotal);
            hostModel.setDiskUsed(diskTotal - diskFree);
        }

        return hostModel;
    }

}
