package com.bocloud.cmp.provider.compute;

import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import com.bocloud.cmp.provider.utils.SingleAssembleTools;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.HostOperateModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.cmp.provider.utils.AssembleTools;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.vmware.vim25.HostConnectSpec;
import com.bocloud.vmware.vim25.HostSystemConnectionState;
import com.bocloud.vmware.vim25.HostSystemReconnectSpec;
import com.bocloud.vmware.vim25.LocalizedMethodFault;
import com.bocloud.vmware.vim25.TaskInfo;
import com.bocloud.vmware.vim25.mo.ClusterComputeResource;
import com.bocloud.vmware.vim25.mo.Datacenter;
import com.bocloud.vmware.vim25.mo.HostSystem;
import com.bocloud.vmware.vim25.mo.InventoryNavigator;
import com.bocloud.vmware.vim25.mo.ManagedEntity;
import com.bocloud.vmware.vim25.mo.Task;
import com.bocloud.vmware.vim25.mo.VirtualMachine;

import io.jsonwebtoken.lang.Assert;

/**
 * @ Description： @ Author ：丁明威 @ Date ：Created in 11:49 2018/11/13 @ Modified
 * By：
 */
@Provider("bocloud.cmp.vmware")
public class VmwareHostProvider extends VmwareProvider implements HostProvider {

    private static Logger logger = LoggerFactory.getLogger(VmwareHostProvider.class);
    private static String CLUSTER = "ClusterComputeResource";

    public VmwareHostProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult listByCluster(String clusterName) {
        logger.info("查询集群{}下宿主机", clusterName);
        try {
            ManagedEntity managedEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(CR,
                    clusterName);
            Assert.notNull(managedEntity, "集群" + clusterName + "不存在！");
            ClusterComputeResource cluster = (ClusterComputeResource) managedEntity;
            HostSystem[] hostSystems = cluster.getHosts();
            List<HostModel> hosts = new ArrayList<>();
            if (null != hostSystems) {
                for (HostSystem hostSystem : hostSystems) {
                    HostModel model = SingleAssembleTools.getInstance().assembleVmwareHost(hostSystem);
                    if (null != model) {
                        hosts.add(model);
                    }
                }
            }
            return new BocloudResult(true, hosts, "集群下" + clusterName + "宿主机查询成功");
        } catch (Exception e) {
            logger.error("集群下" + clusterName + "宿主机查询失败:" + e);
            return new BocloudResult(false, "集群下" + clusterName + "宿主机查询失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }


    public static void trustAllHosts() {
        TrustManager[] trustAllCerts = new TrustManager[]{new X509TrustManager() {
            @Override
            public X509Certificate[] getAcceptedIssuers() {
                return new X509Certificate[]{};
            }

            @Override
            public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
            }

            @Override
            public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
            }
        }};
        try {
            SSLContext sc = SSLContext.getInstance("TLS");
            sc.init(null, trustAllCerts, new SecureRandom());
            HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
        } catch (Exception e) {
            logger.error("Get error message：", e);
        }
    }

    @Override
    public BocloudResult create(HostOperateModel model) {// 该方法是添加主机：有两种情况：1）一种是添加到数据中心下面，作为一个standAloneHost，2）添加到数据中心下的集群中
        logger.info("开始添加宿主机{}", model.getHostName());
        try {
            // 获取数据中心
            Datacenter datacenter = (Datacenter) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    model.getVdcName());
            Assert.notNull(datacenter, "数据中心不存在！");
            // trustAllHosts();
            // 要添加的主机配置
            HostConnectSpec hostConnectSpec = new HostConnectSpec();
            hostConnectSpec.setForce(true);
            hostConnectSpec.setHostName(model.getHostIp());
            hostConnectSpec.setUserName(model.getUsername());
            hostConnectSpec.setPassword(model.getPassword());
            hostConnectSpec.setSslThumbprint(model.getSslThumbprint().trim().replace(" ", ":"));
            Task task = null;
            if (StringUtils.isEmpty(model.getClusterName())) {// 集群名称为null,主机只是添加到数据中心
                logger.info("未执行目标集群，添加宿主机到数据中心{}", model.getVdcName());
                task = datacenter.getHostFolder().addStandaloneHost_Task(hostConnectSpec, null, true);
            } else {// 如果集群名称不为空,证明主机要添加到数据中心下的某一集群下
                // 获取集群
                logger.info("添加宿主机{}到集群{}", model.getHostName(), model.getClusterName());
                ManagedEntity cluster = new InventoryNavigator(datacenter).searchManagedEntity(CLUSTER,
                        model.getClusterName());
                Assert.notNull(cluster, "目标集群不存在！");
                if (cluster instanceof ClusterComputeResource) {
                    task = ((ClusterComputeResource) cluster).addHost_Task(hostConnectSpec, true, null);
                } else {
                    logger.error("添加宿主机{}到集群{}失败：集群类型错误！", model.getHostName(), model.getClusterName());
                    return new BocloudResult(false, "添加宿主机到集群失败：集群类型错误！");
                }
            }
            logger.info("创建添加宿主机任务...");
            if (task.waitForTask().equals(Task.SUCCESS)) {
                logger.info("添加宿主机任务执行成功！");
                // 主机添加成功之后默认是维护模式
                HostSystem host = (HostSystem) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS,
                        model.getHostIp());
                HostModel hostModel = AssembleTools.getInstance().assembleVmwareHost(host);
                return new BocloudResult(true, hostModel, "添加主机成功");
            } else {
                // 任务执行失败：返回执行失败的信息
                String errMsg = task.getTaskInfo().getError().getLocalizedMessage();
                logger.error("添宿主机任务执行失败！原因：{}", errMsg);
                return new BocloudResult(false, "添加宿主机失败:" + errMsg);
            }
        } catch (Exception e) {
            logger.error("添加主机{}异常！", model.getHostName(), e);
            return new BocloudResult(false, "添加主机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult remove(HostOperateModel model) {
        logger.info("开始删除宿主机{}", model.getHostName());
        try {
            // 获取要删除的物理机
            HostSystem host = (HostSystem) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS,
                    model.getHostName());
            Assert.notNull(host, "宿主机不存在");
            // 判断主机是否是在集群中
            if (null == model.getClusterName()) {// 物理机作为一个standalone host存在于数据中心下,不存在任何集群中
                logger.info("宿主机处于standalone状态");
                Task task = host.getParent().destroy_Task();// 删除物理机任务
                if (task.waitForTask().equals(Task.SUCCESS)) {
                    logger.info("移除宿主机{}成功！", model.getHostName());
                    return new BocloudResult(true, "删除主机成功");
                } else {
                    logger.error("移除宿主机{}失败：", model.getHostName(),
                            task.getTaskInfo().getError().getLocalizedMessage());
                    return new BocloudResult(false, "删除主机失败");
                }
            } else {// 物理机处于集群中
                logger.info("宿主机{}处于集群中", model.getHostName());
                // 删除物理机任务(主机必须在维护模式下:如果主机处于集群中，且处于断开状态，此时不处于维护模式也是可以删除的)
                // 获取主机当前连接状态
                HostSystemConnectionState connState = host.getSummary().getRuntime().getConnectionState();
                if (connState != null && connState.equals(HostSystemConnectionState.connected)) {// 处于连接状态、
                    // 物理机位于集群中并处于连接状态：此时删除主机时，主机必须处于维护模式下
                    if (host.getSummary().getRuntime().isInMaintenanceMode()) {
                        Task task = host.destroy_Task();
                        if (task.waitForTask().equals(Task.SUCCESS)) {
                            logger.info("移除宿主机{}成功！", model.getHostName());
                            return new BocloudResult(true, "删除主机成功");
                        } else {
                            logger.error("移除宿主机{}失败！原因：{}", model.getHostName(),
                                    task.getTaskInfo().getError().getLocalizedMessage());
                            return new BocloudResult(false, "删除主机失败");
                        }
                    } else {
                        logger.info("移除宿主机{}失败！原因：{}", model.getHostName(), "宿主机未处于维护模式下！");
                        return new BocloudResult(false, "主机必须处于维护模式下才能删除");
                    }
                } else {
                    Task task = host.destroy_Task();
                    if (task.waitForTask().equals(Task.SUCCESS)) {
                        logger.info("移除宿主机{}成功！", model.getHostName());
                        return new BocloudResult(true, "删除主机成功");
                    } else {
                        logger.error("移除宿主机{}失败！原因：{}", model.getHostName(),
                                task.getTaskInfo().getError().getLocalizedMessage());
                        return new BocloudResult(false, "删除主机失败");
                    }
                }

            }
        } catch (Exception e) {
            logger.error("移除宿主机{}异常！", model.getHostName(), e);
            return new BocloudResult(false, "删除主机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult enterMode(HostOperateModel model) {
        logger.info("执行将宿主机{}进入到维护模式", model.getHostName());
        try {
            // 获取要进入维护模式的物理机
            HostSystem host = (HostSystem) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS,
                    model.getHostName());
            Assert.notNull(host, "宿主机不存在");
            boolean isInMaintenanceMode = host.getSummary().getRuntime().isInMaintenanceMode();
            if (isInMaintenanceMode) {
                logger.warn("该主机已经处于维护模式！");
                return new BocloudResult(false, "主机已处于维护模式");
            } else {
                // 主机进入维护模式任务
                Task task = host.enterMaintenanceMode(500, false, null);
                if (task.waitForTask().equals(Task.SUCCESS)) {
                    logger.info("执行将宿主机{}进入到维护模式任务成功！", model.getHostName());
                    return new BocloudResult(true, "进入维护模式成功");
                } else {
                    String errorMsg = "";
                    if (null != task.getTaskInfo().getError()) {
                        // 返回主机进入维护模式失败的信息
                        errorMsg = task.getTaskInfo().getError().getLocalizedMessage();
                    }
                    logger.error("执行将宿主机{}进入到维护模式任务失败！{}", model.getHostName(), errorMsg);
                    return new BocloudResult(false, "主机进入维护模式失败:" + errorMsg);
                }
            }
        } catch (Exception e) {
            logger.error("执行将宿主机{}进入到维护模式任务异常！", model.getHostName(), e);
            return new BocloudResult(false, "进入维护模式失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult exitMode(HostOperateModel model) {
        try {
            // 获取物理机
            HostSystem host = (HostSystem) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS,
                    model.getHostName());
            Assert.notNull(host, "宿主机不存在");
            // 获取主机模式（是否处于维护模式下）
            boolean mode = host.getSummary().getRuntime().isInMaintenanceMode();
            if (mode) {// 如果为true则证明机器处于维护模式下
                Task task = host.exitMaintenanceMode(120);// 超时时间是120秒
                // 根据任务返回结果判断主机是否退出维护模式
                if (task.waitForTask().equals(Task.SUCCESS)) {
                    return new BocloudResult(true, "退出维护模式");
                } else {
                    return new BocloudResult(false, "退出维护模式失败");
                }
            } else {
                return new BocloudResult(false, "该主机未处于维护模式");
            }
        } catch (Exception e) {
            logger.error("退出维护模式失败", e);
            return new BocloudResult(false, "退出维护模式失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult disconn(HostOperateModel model) {
        try {
            // 获取主机
            HostSystem host = (HostSystem) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS,
                    model.getHostName());
            Assert.notNull(host, "宿主机不存在");
            // 获取主机当前连接状态
            HostSystemConnectionState connState = host.getSummary().getRuntime().getConnectionState();
            if (connState.equals(HostSystemConnectionState.connected)) {
                Task task = host.disconnectHost();
                if (task.waitForTask().equals(Task.SUCCESS)) {
                    return new BocloudResult(true, "主机成功断开连接");
                } else {
                    return new BocloudResult(false, "主机断开连接失败");
                }
            } else if (connState.equals(HostSystemConnectionState.disconnected)) {
                return new BocloudResult(true, "主机已处于断开连接状态");
            } else {
                return new BocloudResult(false, "主机断开连接任务执行失败");
            }
        } catch (Exception e) {
            logger.error("主机断开连接失败", e);
            return new BocloudResult(false, "主机断开连接失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult reconn(HostOperateModel model) {
        try {
            // 获取主机
            HostSystem host = (HostSystem) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS,
                    model.getHostName());
            Assert.notNull(host, "宿主机不存在");
            boolean result = false;
            String message = null;
            // 获取主机当前连接状态
            HostSystemConnectionState connState = host.getSummary().getRuntime().getConnectionState();
            if (connState.equals(HostSystemConnectionState.disconnected)) {
                // 主机重连参数设置
                HostSystemReconnectSpec reconnSpec = new HostSystemReconnectSpec();
                reconnSpec.setSyncState(true);
                // 主机重新连接任务
                Task task = host.reconnectHost_Task(null, reconnSpec);
                if (task.waitForTask().equals(Task.SUCCESS)) {
                    result = true;
                    message = "主机重新连接成功";
                } else {
                    result = false;
                    message = "主机重新连接失败";
                }
            } else if (connState.equals(HostSystemConnectionState.connected)) {
                result = true;
                message = "主机已处于连接状态";
            } else {
                result = false;
                message = "主机重新连接失败";
            }
            List<ServerModel> serverModels = new ArrayList<>();
            if (result) {
                HostSystem hostSystem = (HostSystem) new InventoryNavigator(this.getRootFolder())
                        .searchManagedEntity(HS, model.getHostName());
                // 获取宿主机下的所有云主机
                VirtualMachine[] machines = hostSystem.getVms();// 某一个hostSystem下所有的云主机（取出的数量=云主机+模板）
                if (null != machines) {
                    for (VirtualMachine machine : machines) {
                        ServerModel serverModel = AssembleTools.getInstance().convertServer(machine);
                        if (null != serverModel) {
                            serverModels.add(serverModel);
                        }
                    }
                }
            }
            return new BocloudResult(result, serverModels, message);
        } catch (Exception e) {
            logger.error("主机重新连接失败", e);
            return new BocloudResult(false, "主机重新连接失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult moveTo(HostOperateModel model) {
        // 物理机移动：1）从数据中心下移动到集群中；2）从集群中移动到数据中心下；3）从一个集群移动到另一个集群。前提是主机与集群必须处于同一个数据中心下
        // 如果主机时处于集群中，主机若要移动，主机必须处于维护模式下
        try {
            Datacenter datacenter = (Datacenter) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    model.getVdcName());
            Assert.notNull(datacenter, "数据中心不存在！");
            // 获取物理机
            List<HostSystem> hostSystems = new ArrayList<>();
            List<String> hosts = model.getHosts();
            for (String host : hosts) {
                HostSystem hostSystem = (HostSystem) new InventoryNavigator(datacenter).searchManagedEntity(HS, host);
                hostSystems.add(hostSystem);
            }
            if (hostSystems.isEmpty()) {
                return new BocloudResult(false, "该数据中心下不包括这些主机");
            }
            if (null == model.getClusterName()) {// 如果没有集群名称证明是移动到数据中心下面
                ManagedEntity[] entities = hostSystems.toArray(new ManagedEntity[hostSystems.size()]);
                // 移动主机到数据中心下
                Task task = datacenter.getHostFolder().moveIntoFolder_Task(entities);
                if (task.waitForTask().equals(Task.SUCCESS)) {
                    return new BocloudResult(true, "移动物理机成功");
                } else {
                    return new BocloudResult(false, "移动物理机失败");
                }
            } else {
                // 获取集群
                ManagedEntity cluster = new InventoryNavigator(datacenter).searchManagedEntity(CLUSTER,
                        model.getClusterName());
                if (null != cluster) {
                    if (cluster instanceof ClusterComputeResource) {
                        // 移动主机到集群
                        HostSystem[] hostArray = hostSystems.toArray(new HostSystem[hostSystems.size()]);
                        Task task = ((ClusterComputeResource) cluster).moveInto_Task(hostArray);
                        if (task.waitForTask().equals(Task.SUCCESS)) {
                            return new BocloudResult(true, "移动物理机成功");
                        } else {
                        	String message = "移动物理机失败";
                        	LocalizedMethodFault error = task.getTaskInfo().getError();
                        	if(null != error) {
                        		message += ":" + error.getLocalizedMessage();
                        	}
                            return new BocloudResult(false, message);
                        }
                    }
                }
            }
        } catch (Exception e) {
            logger.error("移动物理机失败", e);
            return new BocloudResult(false, "移动物理机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
        return new BocloudResult(false, "移动物理机失败");
    }

    @Override
    public BocloudResult shutdown(HostOperateModel model) {
        try {
            Datacenter datacenter = (Datacenter) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    model.getVdcName());
            Assert.notNull(datacenter, "数据中心不存在！");
            HostSystem hostSystem = (HostSystem) new InventoryNavigator(datacenter).searchManagedEntity(HS,
                    model.getHostName());
            Assert.notNull(hostSystem, "宿主机不存在！");
            Task task = hostSystem.shutdownHost_Task(true);
            if (task.waitForTask().equals(Task.SUCCESS)) {
                return new BocloudResult(true, "物理机关机成功");
            } else {
                return new BocloudResult(false, "物理机关机失败");
            }
        } catch (Exception e) {
            logger.error("物理机关机失败", e);
            return new BocloudResult(false, "物理机关机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult reboot(HostOperateModel model) {
        try {
            Datacenter datacenter = (Datacenter) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    model.getVdcName());
            Assert.notNull(datacenter, "数据中心不存在！");
            HostSystem hostSystem = (HostSystem) new InventoryNavigator(datacenter).searchManagedEntity(HS,
                    model.getHostName());
            Assert.notNull(datacenter, "宿主机不存在！");
            Task task = hostSystem.rebootHost(true);
            if (task.waitForTask().equals(Task.SUCCESS)) {
                return new BocloudResult(true, "物理机重启成功");
            } else {
                return new BocloudResult(false, "物理机重启失败");
            }
        } catch (Exception e) {
            logger.error("物理机重启失败", e);
            return new BocloudResult(false, "物理机重启失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult detail(HostOperateModel model) {
        try {
            HostSystem hostSystem = (HostSystem) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS,
                    model.getHostName());
            Assert.notNull(hostSystem, "宿主机不存在");
            return new BocloudResult(true, AssembleTools.getInstance().assembleVmwareHost(hostSystem), "查询成功");
        } catch (Exception e) {
            logger.error("查询失败", e);
            return new BocloudResult(false, "查询失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult monitor() {
        return null;
    }

    @Override
    public BocloudResult listPerf() {
        return null;
    }

    @Override
    public BocloudResult summary(Long id) {
        return null;
    }

    @Override
    public BocloudResult detail(Long hostId) {
        return null;
    }

    @Override
    public BocloudResult list(String clusterId) {
        return null;
    }

}