package com.bocloud.cmp.provider.compute;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.entity.enums.NetworkType;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ResourcePoolBean;
import com.bocloud.cmp.model.ResourcePoolModel;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.cmp.provider.utils.AssembleTools;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.vmware.vim25.ClusterDrsConfigInfo;
import com.bocloud.vmware.vim25.ResourceAllocationInfo;
import com.bocloud.vmware.vim25.ResourceConfigSpec;
import com.bocloud.vmware.vim25.SharesInfo;
import com.bocloud.vmware.vim25.SharesLevel;
import com.bocloud.vmware.vim25.mo.ClusterComputeResource;
import com.bocloud.vmware.vim25.mo.ComputeResource;
import com.bocloud.vmware.vim25.mo.Datacenter;
import com.bocloud.vmware.vim25.mo.HostSystem;
import com.bocloud.vmware.vim25.mo.InventoryNavigator;
import com.bocloud.vmware.vim25.mo.ManagedEntity;
import com.bocloud.vmware.vim25.mo.ResourcePool;
import com.bocloud.vmware.vim25.mo.Task;

import io.jsonwebtoken.lang.Assert;

/**
 * vmware资源池操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/14
 */
@Provider("bocloud.cmp.vmware")
public class VmwareResourcePoolProvider extends VmwareProvider implements ResourcePoolProvider {

	private static Logger logger = LoggerFactory.getLogger(VmwareResourcePoolProvider.class);

	public VmwareResourcePoolProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult create(ResourcePoolModel model) {
		try {
			ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
					model.getVdcName());
			Assert.notNull(entity, "数据中心不存在！");
			Datacenter center = (Datacenter) entity;
			ResourcePool resourcePool = null;
			ManagedEntity[] entities = new InventoryNavigator(center).searchManagedEntities(RP);
			// 如果资源池的父集群开启了drs，才可以创建新资源池
			String ownerCategory = model.getOwnerCategory();
			if ("CLUSTER".equalsIgnoreCase(ownerCategory)) {
				ManagedEntity[] hostEntites = center.getHostFolder().getChildEntity();
				if (entities == null || entities.length == 0) {
					return new BocloudResult(false, "获取vcenter集群信息失败");
				}
				for (int i = 0; i < hostEntites.length; i++) {
					ManagedEntity hostEntity = entities[i];
					if (hostEntity instanceof ClusterComputeResource) {
						ClusterComputeResource clusterResource = (ClusterComputeResource) hostEntity;
						if (clusterResource.getName().equalsIgnoreCase(model.getOwnerName())) {
							ClusterDrsConfigInfo drsConfigInfo = clusterResource.getConfiguration().getDrsConfig();
							if (null == drsConfigInfo) {
								return new BocloudResult(false, "未查询到所在集群的DRS配置");
							}
							if (!drsConfigInfo.getEnabled()) {
								return new BocloudResult(false, "所在集群的DRS配置未启用，创建失败");
							}
						}
					}
				}
			}
			// 查询父资源池
			if (null != entities && entities.length > 0) {
				for (ManagedEntity managedEntity : entities) {
					ResourcePool pool = (ResourcePool) managedEntity;
					if ("CLUSTER".equalsIgnoreCase(ownerCategory) && !(pool.getOwner() instanceof ComputeResource)) {
						continue;
					}
					if (model.getOwnerName().equals(pool.getOwner().getName())
							&& model.getParentName().equals(pool.getName())) {
						resourcePool = pool;
						break;
					}
				}
			}
			Assert.notNull(resourcePool, "宿主机无可用资源池！");
			// CPU预留资源
			Long cpuReservation = model.getCpuReservation();
			// CPU限制资源(-1代表不受限制)
			Long cpuLimit = model.getCpuLimit();
			if (null != cpuLimit && cpuLimit != -1L && cpuReservation > cpuLimit) {
				logger.warn("CPU reserve resource is greater than the limit resource");
				return new BocloudResult(false, "CPU预留资源大于限制资源");
			}
			// 内存预留资源
			Long memReservation = model.getMemReservation();
			// 内存限制资源(-1代表不受限制)
			Long memLimit = model.getMemLimit();
			if (null != memLimit && memLimit != -1L && memReservation > memLimit) {
				logger.warn("Memory reservation resources are greater than the limit resources");
				return new BocloudResult(false, "内存预留资源大于限制资源");
			}
			ResourceConfigSpec spec = new ResourceConfigSpec();
			// CPU资源分配
			ResourceAllocationInfo cpuAllocation = new ResourceAllocationInfo();
			cpuAllocation.setReservation(cpuReservation);
			cpuAllocation.setExpandableReservation(model.getCpuExpanReservation());
			cpuAllocation.setLimit(cpuLimit);
			SharesInfo cpuShares = new SharesInfo();
			String cpuLevel = model.getCpuShareLevel();
			SharesLevel cpuShareLevel = SharesLevel.valueOf(cpuLevel);
			cpuShares.setLevel(cpuShareLevel);
			// 自定义
			if (cpuShareLevel.equals(SharesLevel.custom)) {
				cpuShares.setShares(model.getCpuShares());
			}
			cpuAllocation.setShares(cpuShares);
			spec.setCpuAllocation(cpuAllocation);
			// 内存资源分配
			ResourceAllocationInfo memoryAllocation = new ResourceAllocationInfo();
			memoryAllocation.setReservation(memReservation);
			memoryAllocation.setExpandableReservation(model.getMemExpanReservation());
			memoryAllocation.setLimit(memLimit);
			SharesInfo memShares = new SharesInfo();
			String memLevel = model.getMemShareLevel();
			SharesLevel memShareLevel = SharesLevel.valueOf(memLevel);
			memShares.setLevel(memShareLevel);
			// 自定义
			if (memShareLevel.equals(SharesLevel.custom)) {
				memShares.setShares(model.getCpuShares());
			}
			memShares.setLevel(SharesLevel.valueOf(model.getMemShareLevel()));
			memShares.setShares(model.getMemShares());
			memoryAllocation.setShares(memShares);
			spec.setMemoryAllocation(memoryAllocation);
			resourcePool = resourcePool.createResourcePool(model.getName(), spec);
			Assert.notNull(resourcePool, "资源池创建失败！");
			model.setPoolVal(resourcePool.getMOR().getVal());
			return new BocloudResult(true, model, "资源池创建成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "资源池创建失败：" + e.getMessage());
		} finally {
			this.getServiceInstance().getServerConnection().logout();
		}
	}

	@Override
	public BocloudResult modify(ResourcePoolBean bean) {
		try {
			ManagedEntity managedEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
					bean.getVdcName());
			Assert.notNull(managedEntity, "数据中心不存在！");
			Datacenter center = (Datacenter) managedEntity;
			ManagedEntity entity = new InventoryNavigator(center).searchManagedEntity(RP, bean.getOldName());
			Assert.notNull(entity, "资源池不存在！");
			ResourcePool resourcePool = (ResourcePool) entity;
			// CPU预留资源
			Long cpuReservation = bean.getCpuReservation();
			// CPU限制资源(-1代表不受限制)
			Long cpuLimit = bean.getCpuLimit() == null ? 0L : bean.getCpuLimit();
			if (cpuLimit != -1L && cpuReservation > cpuLimit) {
				logger.warn("CPU reserve resource is greater than the limit resource");
				return new BocloudResult(false, "CPU预留资源大于限制资源");
			}
			// 内存预留资源
			Long memReservation = bean.getMemReservation();
			// 内存限制资源(-1代表不受限制)
			Long memLimit = bean.getMemLimit() == null ? 0L : bean.getMemLimit();
			if (memLimit != -1L && memReservation > memLimit) {
				logger.warn("Memory reservation resources are greater than the limit resources");
				return new BocloudResult(false, "内存预留资源大于限制资源");
			}
			ResourceConfigSpec spec = new ResourceConfigSpec();
			// CPU资源分配
			ResourceAllocationInfo cpuAllocation = new ResourceAllocationInfo();
			cpuAllocation.setReservation(cpuReservation);
			cpuAllocation.setExpandableReservation(bean.getCpuExpanReservation());
			cpuAllocation.setLimit(cpuLimit);
			SharesInfo cpuShares = new SharesInfo();
			String cpuLevel = bean.getCpuShareLevel();
			SharesLevel cpuShareLevel = SharesLevel.valueOf(cpuLevel);
			cpuShares.setLevel(cpuShareLevel);
			// 自定义
			if (cpuShareLevel.equals(SharesLevel.custom)) {
				cpuShares.setShares(bean.getCpuShares());
			}
			cpuAllocation.setShares(cpuShares);
			spec.setCpuAllocation(cpuAllocation);
			// 内存资源分配
			ResourceAllocationInfo memoryAllocation = new ResourceAllocationInfo();
			memoryAllocation.setReservation(memReservation);
			memoryAllocation.setExpandableReservation(bean.getMemExpanReservation());
			memoryAllocation.setLimit(memLimit);
			SharesInfo memShares = new SharesInfo();
			String memLevel = bean.getMemShareLevel();
			SharesLevel memShareLevel = SharesLevel.valueOf(memLevel);
			memShares.setLevel(memShareLevel);
			// 自定义
			if (memShareLevel.equals(SharesLevel.custom)) {
				memShares.setShares(bean.getCpuShares());
			}
			memShares.setLevel(SharesLevel.valueOf(bean.getMemShareLevel()));
			memShares.setShares(bean.getMemShares());
			memoryAllocation.setShares(memShares);
			spec.setMemoryAllocation(memoryAllocation);
			resourcePool.updateConfig(bean.getName(), spec);
			ManagedEntity newEntity = new InventoryNavigator(center).searchManagedEntity(RP, bean.getName());
			Assert.notNull(newEntity, "资源池不存在！");
			ResourcePool newResPool = (ResourcePool) newEntity;
			bean.setPoolVal(newResPool.getMOR().getVal());
			return new BocloudResult(true, bean, "资源池修改成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "资源池修改失败：" + e.getMessage());
		} finally {
			this.getServiceInstance().getServerConnection().logout();
		}
	}

	@Override
	public BocloudResult remove(String poolName) {
		try {
			ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(RP, poolName);
			Assert.notNull(entity, "资源池不存在！");
			ResourcePool resourcePool = (ResourcePool) entity;
			Task task = resourcePool.destroy_Task();
			if (task.waitForTask().equals(Task.SUCCESS)) {
				return new BocloudResult(true, "资源池删除成功");
			}
			return new BocloudResult(false, "资源池删除失败：" + task.getTaskInfo().getError().getLocalizedMessage());
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "资源池删除失败");
		} finally {
			this.getServiceInstance().getServerConnection().logout();
		}
	}

	@Override
	public BocloudResult list(String vdcName) {
		try {
			ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, vdcName);
			Assert.notNull(entity, "数据中心不存在！");
			Datacenter datacenter = (Datacenter) entity;
			/* 组装资源池数据 */
			List<ResourcePoolModel> pools = new ArrayList<>();
			ManagedEntity[] poolEntities = new InventoryNavigator(datacenter).searchManagedEntities(RP);
			for (ManagedEntity poolEntity : poolEntities) {
				ResourcePool resourcePool = (ResourcePool) poolEntity;
				if (resourcePool.getOwner() instanceof ClusterComputeResource) {
					ResourcePoolModel clusterPool = AssembleTools.getInstance()
							.assembleVmwareResourcePool(resourcePool);
					ClusterComputeResource cluster = (ClusterComputeResource) resourcePool.getOwner();
					clusterPool.setVdcName(datacenter.getName());
					clusterPool.setClusterName(cluster.getName());
					clusterPool.setClusterVal(cluster.getMOR().getVal());
					clusterPool.setOwnerCategory(NetworkType.ResPoolOwnerType.CLUSTER.name());
					pools.add(clusterPool);
				} else if (resourcePool.getOwner() instanceof ComputeResource) {
					ResourcePoolModel hostPool = AssembleTools.getInstance().assembleVmwareResourcePool(resourcePool);
					ComputeResource host = resourcePool.getOwner();
					HostSystem hostSystem = host.getHosts()[0];
					hostPool.setVdcName(datacenter.getName());
					hostPool.setHostName(hostSystem.getName());
					hostPool.setHostVal(hostSystem.getMOR().getVal());
					hostPool.setOwnerCategory(NetworkType.ResPoolOwnerType.HOST.name());
					pools.add(hostPool);
				}
			}
			return new BocloudResult(true, pools, "资源池查询成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(true, "资源池查询失败");
		} finally {
			this.getServiceInstance().getServerConnection().logout();
		}
	}
}
