package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.entity.enums.NetworkType;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.cmp.provider.utils.AssembleTools;
import com.bocloud.cmp.provider.utils.IPAddress;
import com.bocloud.cmp.provider.utils.SingleAssembleTools;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.OSCategory;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ArrayTool;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.common.utils.ListTool;
import com.bocloud.vmware.vim25.*;
import com.bocloud.vmware.vim25.mo.HostSystem;
import com.bocloud.vmware.vim25.mo.*;
import io.jsonwebtoken.lang.Assert;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.rmi.RemoteException;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * vmware云主机操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/14
 */
@Provider("bocloud.cmp.vmware")
public class VmwareServerProvider extends VmwareProvider implements ServerProvider {

    private static Logger logger = LoggerFactory.getLogger(VmwareServerProvider.class);

    private static final Integer time = 120;

    /**
     * VMware tools镜像名称
     */
    private static List<String> VMWARE_TOOLS_ISO = new ArrayList<>();

    static {
        VMWARE_TOOLS_ISO.add("darwin.iso");
        VMWARE_TOOLS_ISO.add("freebsd.iso");
        VMWARE_TOOLS_ISO.add("linux.iso");
        VMWARE_TOOLS_ISO.add("netware.iso");
        VMWARE_TOOLS_ISO.add("solaris.iso");
        VMWARE_TOOLS_ISO.add("windows.iso");
        VMWARE_TOOLS_ISO.add("winPre2k.iso");
        VMWARE_TOOLS_ISO.add("winPreVista.iso");
    }


    public VmwareServerProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult deleteDisk(String instanceId, List<String> diskUuids) {
        try {
            VirtualMachine machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                    instanceId, true, true);
            Assert.notNull(machine, "云主机不存在");
            VirtualMachineConfigSpec configSpec = new VirtualMachineConfigSpec();
            List<VirtualDeviceConfigSpec> specs = new ArrayList<>();
            VirtualDevice[] devices = machine.getConfig().getHardware().getDevice();
            for (int i = 0; devices != null && i < devices.length; i++) {
                VirtualDevice device = devices[i];
                if (device instanceof VirtualDisk) {
                    VirtualDisk disk = (VirtualDisk) device;
                    Description deviceInfo = disk.getDeviceInfo();
                    if (null != deviceInfo && null != deviceInfo.getLabel()) {
                        if (disk.getBacking() instanceof VirtualDiskFlatVer2BackingInfo) {
                            VirtualDiskFlatVer2BackingInfo diskFlatVer2BackingInfo = (VirtualDiskFlatVer2BackingInfo) disk.getBacking();
                            if (diskUuids.contains(diskFlatVer2BackingInfo.uuid)) {
                                // 移除磁盘设备
                                specs.add(removeDiskSpec(disk));
                            }
                        }
                    }
                }
            }
            if (!ListTool.isEmpty(specs)) {
                VirtualDeviceConfigSpec[] specArray = new VirtualDeviceConfigSpec[specs.size()];
                configSpec.setDeviceChange(specs.toArray(specArray));
            }
            Task task = machine.reconfigVM_Task(configSpec);
            logger.info("云主机{}重新配置的任务成功并执行", instanceId);
            if (task.waitForTask().equals(Task.SUCCESS)) {
                return new BocloudResult(true, "硬盘删除成功");
            } else {
                String failInfo = "";
                if (null != task.getTaskInfo().getError()) {
                    // 返回主机进入维护模式失败的信息
                    failInfo = task.getTaskInfo().getError().getLocalizedMessage();
                }
                logger.info("重新配置云主机{} 任务执行失败！{}", instanceId, failInfo);
                return new BocloudResult(false, "硬盘删除失败:" + failInfo);
            }
        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult selectDiskUuid(String instanceId, List<String> diskUuids) {
        try {
            VirtualMachine machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                    instanceId, true, true);
            Assert.notNull(machine, "云主机不存在");
            VirtualMachineConfigSpec configSpec = new VirtualMachineConfigSpec();
            List<String> specsId = new ArrayList<>();
            VirtualDevice[] devices = machine.getConfig().getHardware().getDevice();
            for (int i = 0; devices != null && i < devices.length; i++) {
                VirtualDevice device = devices[i];
                if (device instanceof VirtualDisk) {
                    VirtualDisk disk = (VirtualDisk) device;
                    Description deviceInfo = disk.getDeviceInfo();
                    if (null != deviceInfo && null != deviceInfo.getLabel()) {
                        if (disk.getBacking() instanceof VirtualDiskFlatVer2BackingInfo) {
                            VirtualDiskFlatVer2BackingInfo diskFlatVer2BackingInfo = (VirtualDiskFlatVer2BackingInfo) disk.getBacking();
                            if (diskUuids.contains(diskFlatVer2BackingInfo.uuid)) {
                                // 移除磁盘设备uuid
                                specsId.add(diskFlatVer2BackingInfo.uuid);
                            }
                        }
                    }
                }
            }
            if (!ListTool.isEmpty(specsId)) {
                logger.info("云主机{}硬盘设备查询成功", instanceId);
                return new BocloudResult(true, specsId, "硬盘设备查询成功");
            } else {
                logger.info("云主机{}硬盘设备查询失败，没有当前硬盘信息", instanceId);
                return new BocloudResult(false, "硬盘设备查询失败，没有当前硬盘信息");
            }

        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult enableDiskUuid(String instanceId) {
        try {
            VirtualMachine machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                    instanceId, true, true);
            Assert.notNull(machine, "云主机不存在");
            boolean findDiskUUID = false;
            OptionValue[] values = machine.getConfig().getExtraConfig();
            if (null != values && values.length > 0) {
                for (OptionValue value : values) {
                    String key = value.getKey();
                    if (!StringUtils.isEmpty(key) && "disk.enableUUID".equals(key) && "TRUE".equals(value.getValue())) {
                        findDiskUUID = true;
                        break;
                    }
                }
            }
            if (findDiskUUID) {
                return new BocloudResult(true, "云主机diskUuid配置成功");
            }
            VirtualMachineConfigSpec configSpec = new VirtualMachineConfigSpec();
            configSpec.setExtraConfig(processExtraConfig(machine.getConfig().getExtraConfig()));
            Task task = machine.reconfigVM_Task(configSpec);
            logger.info("创建云主机{} 重新配置的任务成功并执行", instanceId);
            if (task.waitForTask().equals(Task.SUCCESS)) {
                return new BocloudResult(true, "云主机diskUuid配置成功");
            } else {
                String failInfo = "";
                if (null != task.getTaskInfo().getError()) {
                    // 返回主机进入维护模式失败的信息
                    failInfo = task.getTaskInfo().getError().getLocalizedMessage();
                }
                logger.info("重新配置云主机{} 任务执行失败！{}", instanceId, failInfo);
                return new BocloudResult(false, "云主机diskUuid配置失败:" + failInfo);
            }
        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    /**
     * 处理配置参数，开启disk.enableUUID=TRUE
     *
     * @param values
     */
    private OptionValue[] processExtraConfig(OptionValue[] values) {
        List<OptionValue> valueList = new ArrayList<>();
        boolean findDiskUUID = false;
        if (null != values && values.length > 0) {
            valueList = new ArrayList<>(Arrays.asList(values));
            for (OptionValue value : values) {
                String key = value.getKey();
                if (!StringUtils.isEmpty(key) && "disk.enableUUID".equals(key)) {
                    findDiskUUID = true;
                    value.setValue("TRUE");
                    break;
                }
            }
        }
        if (!findDiskUUID) {
            OptionValue value = new OptionValue();
            value.setKey("disk.enableUUID");
            value.setValue("TRUE");
            valueList.add(value);
        }
        return valueList.toArray(new OptionValue[valueList.size()]);
    }

    @Override
    public BocloudResult create(ServerModel server) {
        try {
            logger.info("创建虚拟机参数:{}", JSON.toJSONString(server));
            ManagedEntity datacenter = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    server.getVdcName());
            Assert.notNull(datacenter, "数据中心不存在！");
            Datacenter center = (Datacenter) datacenter;
            InventoryNavigator navigator = new InventoryNavigator(center);
            VirtualMachine template = (VirtualMachine) navigator.searchManagedEntity(VM, server.getTemplate());
            if (null == template) {
                ManagedEntity managedEntity = this.getServiceInstance().getSearchIndex().findByUuid(null,
                        server.getTempInstId(), true, true);
                if (null != managedEntity) {
                    template = (VirtualMachine) managedEntity;
                }
            }
            Assert.notNull(template, "创建云主机失败：云主机模板不存在！");
            ComputeResource computeResource = null;
            if (StringUtils.isEmpty(server.getClusterName())) {
                computeResource = (ComputeResource) navigator.searchManagedEntity(CR, server.getHostName());
            } else {
                computeResource = (ComputeResource) navigator.searchManagedEntity(CR, server.getClusterName());
            }
            HostSystem hostSystem = (HostSystem) navigator.searchManagedEntity(HS, server.getHostName());
            Datastore datastore = (Datastore) navigator.searchManagedEntity(DT, server.getDatastore());
            VirtualMachine virtualMachine = (VirtualMachine) navigator.searchManagedEntity(VM, server.getName());
            Assert.isNull(virtualMachine, "创建云主机失败：名称已经存在！");
            // 模板配置
            VirtualMachineConfigInfo tempConfig = template.getConfig();
            List<VirtualDeviceConfigSpec> specs = new ArrayList<>();
            List<VirtualMachineRelocateSpecDiskLocator> diskLocator = new ArrayList<>();
            if (null != tempConfig) {
                // 云主机关联设备
                VirtualDevice[] devices = tempConfig.getHardware().getDevice();
                for (int i = 0; devices != null && i < devices.length; i++) {
                    // 按照底层vcenter上磁盘顺序，找到的第一块磁盘设备即为系统盘
                    VirtualDevice device = devices[i];
                    if (device instanceof VirtualDisk) {
                        VirtualDisk disk = (VirtualDisk) device;
                        // 系统盘名称为
                        if ("Hard disk 1".equalsIgnoreCase(disk.getDeviceInfo().getLabel()) || "硬盘 1".equalsIgnoreCase(disk.getDeviceInfo().getLabel())) {
                            if (null != server.getDisk()) {
                                BigDecimal diskSize = new BigDecimal(disk.getCapacityInKB())
                                        .divide(new BigDecimal(1024 * 1024L), 2, RoundingMode.HALF_UP);
                                if (server.getDisk() < diskSize.intValue()) {
                                    return new BocloudResult(false, "指定的系统盘大小不能小于原模板系统盘大小");
                                }
                                // 编辑系统盘大小
                                specs.add(expandDiskSpec(server.getDiskType(), disk, server.getDisk() * 1024 * 1024L));
                            }
                        }
                        logger.info("磁盘{}磁盘格式：{}", disk.getDeviceInfo().getLabel(), server.getDiskType());
                        String diskType = server.getDiskType();
                        if (!StringUtils.isEmpty(diskType)) {
                            VirtualMachineRelocateSpecDiskLocator locator = new VirtualMachineRelocateSpecDiskLocator();
                            locator.setDiskId(disk.getKey());
                            VirtualDeviceBackingInfo backing = disk.getBacking();
                            if (null != backing && backing instanceof VirtualDiskFlatVer2BackingInfo) {
                                VirtualDiskFlatVer2BackingInfo flatVer2BackingInfo = (VirtualDiskFlatVer2BackingInfo) backing;
                                switch (diskType) {
                                    case "sameAsSource":
                                        break;
                                    case "flat":
                                        flatVer2BackingInfo.setEagerlyScrub(false);
                                        flatVer2BackingInfo.setThinProvisioned(false);
                                        break;
                                    case "thick":
                                        flatVer2BackingInfo.setEagerlyScrub(true);
                                        flatVer2BackingInfo.setThinProvisioned(false);
                                        break;
                                    case "thin":
                                        flatVer2BackingInfo.setEagerlyScrub(null);
                                        flatVer2BackingInfo.setThinProvisioned(true);
                                        break;
                                    default:
                                        break;
                                }
                                ManagedObjectReference datastoreRef = new ManagedObjectReference();
                                datastoreRef.setType("Datastore");
                                datastoreRef.setVal(datastore.getMOR().getVal());
                                locator.setDatastore(datastoreRef);
                                flatVer2BackingInfo.setDatastore(datastoreRef);
                                locator.setDiskBackingInfo(flatVer2BackingInfo);
                                String profileId = server.getProfileId();
                                if (!StringUtils.isEmpty(profileId)) {
                                    VirtualMachineDefinedProfileSpec profileSpec = new VirtualMachineDefinedProfileSpec();
                                    profileSpec.setProfileId(profileId);
                                    locator.setProfile(new VirtualMachineProfileSpec[]{profileSpec});
                                } else {
                                    VirtualMachineDefaultProfileSpec profileSpec = new VirtualMachineDefaultProfileSpec();
                                    locator.setProfile(new VirtualMachineProfileSpec[]{profileSpec});
                                }
                                diskLocator.add(locator);
                            }
                        }
                    }
                }
            }
            if (null == hostSystem) {
                logger.error("宿主机{}不存在！", server.getHostName());
                return new BocloudResult(false, "创建云主机失败：宿主机不存在！");
            }
            if (null == datastore) {
                logger.warn("共享数据存储不存在！采用本地存储");
                Datastore[] datastores = hostSystem.getDatastores();
                if (null == datastores || datastores.length <= 0) {
                    logger.error("本地存储不存在！");
                    return new BocloudResult(false, "创建云主机失败：本地存储不存在！");
                }
                for (Datastore store : datastores) {
                    if (store.getSummary().getName().equalsIgnoreCase(server.getDatastore())) {
                        datastore = store;
                        break;
                    }
                }
            }
            if (null == datastore) {
                logger.warn("数据存储【{}】不存在！", server.getDatastore());
                return new BocloudResult(false, "创建云主机失败：数据存储不存在！");
            }
            VirtualMachineRelocateSpec location = new VirtualMachineRelocateSpec();
            // 设置vapp
            String vappName = server.getVappName();
            if (!StringUtils.isEmpty(vappName)) {
                ManagedEntity vappEntity = new InventoryNavigator(center).searchManagedEntity(VAPP, vappName);
                Assert.notNull(vappEntity, "vApp不存在！");
                location.setPool(vappEntity.getMOR());
            } else if (computeResource.getResourcePool() != null) {
                location.setPool(computeResource.getResourcePool().getMOR());
            }
            location.setDatastore(datastore.getMOR());
            location.setHost(hostSystem.getMOR());
            VirtualMachineConfigSpec configSpec = new VirtualMachineConfigSpec();
            // VMware Tools相关配置
            // ToolsConfigInfo tools = new ToolsConfigInfo();
            // tools.setAfterPowerOn(true);// 打开电源后
            // tools.setAfterResume(true);// 恢复后
            // tools.setBeforeGuestStandby(true);// 挂起前
            // tools.setBeforeGuestShutdown(true);// 关闭客户机之前
            // // 电源控制
            // VirtualMachineDefaultPowerOpInfo powerOpInfo = new
            // VirtualMachineDefaultPowerOpInfo();
            // powerOpInfo.setPowerOffType("soft");
            // powerOpInfo.setSuspendType("hard");
            // powerOpInfo.setResetType("soft");
            // configSpec.setPowerOpInfo(powerOpInfo);
            // // 高级设置
            // tools.setToolsUpgradePolicy("upgradeAtPowerCycle");// 在关闭再打开过程中是否检查并升级工具
            // tools.setSyncTimeWithHost(false);// 同步客户机时间与主机时间
            // configSpec.setTools(tools);
            // 默认开启热插拔
            // 开启磁盘UUID配置
            configSpec.setExtraConfig(processExtraConfig(tempConfig.getExtraConfig()));
            configSpec.setCpuHotAddEnabled(true);
            configSpec.setMemoryHotAddEnabled(true);
            configSpec.setCpuHotRemoveEnabled(true);
            configSpec.setNpivTemporaryDisabled(false);
            // 内存不限制
            ResourceAllocationInfo memoryAllocation = new ResourceAllocationInfo();
            memoryAllocation.setLimit(-1L);
            configSpec.setMemoryAllocation(memoryAllocation);
            // 设置CPU核数
            configSpec.setNumCPUs(server.getCpu());
            // 设置内存大小
            configSpec.setMemoryMB(server.getMemory().multiply(BigDecimal.valueOf(1024)).longValue());
            // 设置云主机名称
            configSpec.setName(server.getName());
            // 设置云主机描述
            if (!StringUtils.isEmpty(server.getRemark())) {
                configSpec.setAnnotation(server.getRemark());
            }
            // 设置vapp
            // 云主机克隆配置
            VirtualMachineCloneSpec cloneSpec = new VirtualMachineCloneSpec();
            if (!ListTool.isEmpty(diskLocator)) {
                VirtualMachineRelocateSpecDiskLocator[] locators = new VirtualMachineRelocateSpecDiskLocator[diskLocator
                        .size()];
                location.setDisk(diskLocator.toArray(locators));
            }
            cloneSpec.setLocation(location);
            cloneSpec.setPowerOn(true);
            cloneSpec.setTemplate(false);
            cloneSpec.setConfig(configSpec);
            // 由于模板系统类别有可能获取不到，暂时先默认获取不到时按照Linux的处理了
            String osCategory = template.getGuest().getGuestFamily();
            if (StringUtils.isEmpty(osCategory)) {
                osCategory = template.getConfig().getGuestFullName();
                if (StringUtils.isEmpty(osCategory)) {
                    osCategory = OSCategory.LINUX.name();
                } else if (osCategory.toLowerCase().contains(OSCategory.WINDOWS.name().toLowerCase())) {
                    osCategory = OSCategory.WINDOWS.name();
                } else {
                    osCategory = OSCategory.LINUX.name();
                }
            }
            if (VirtualMachineGuestOsFamily.otherGuestFamily.name().equals(osCategory)
                    && (null != template.getGuest().getGuestFullName())) {
                osCategory = OSCategory.LINUX.name();
            }
            logger.info("模板【" + template.getName() + "】操作系统：" + template.getConfig().getGuestFullName());
            // 云主机自定义配置，包括IP地址配置
            cloneSpec.setCustomization(createCustomizationSpec(server, osCategory));
            // 配置云主机磁盘设备
            List<Integer> disks = server.getDisks();
            List<String> dataStoreNames = server.getDataStoreNames();
            if (!ListTool.isEmpty(disks)) {
                logger.info("开始配置云主机磁盘设备");
                for (int i = 0; i < disks.size(); i++) {
                    if (dataStoreNames != null && dataStoreNames.size() > 0) {
                        Datastore d = (Datastore) navigator.searchManagedEntity(DT, dataStoreNames.get(i));
                        logger.info("mutli datastores name：" + d.getName());
                        specs.add(addDiskSpec(server.getProfileId(), server.getDiskType(), d.getName(),
                                disks.get(i) * 1024 * 1024L, "persistent", template, i + 1));
                    } else {
                        logger.info("datastore name：" + datastore.getName());
                        specs.add(addDiskSpec(server.getProfileId(), server.getDiskType(), datastore.getName(),
                                disks.get(i) * 1024 * 1024L, "persistent", template, i + 1));
                    }
                }
                logger.info("配置云主机磁盘设备结束");
            }
            // 添加rdm disk
            List<JSONObject> rdmDisks = server.getRdmDisks();
            if (!ListTool.isEmpty(rdmDisks)) {
                Integer diskNum = disks.size();
                List<Integer> keys = new ArrayList<>();
                for (int i = diskNum; i < rdmDisks.size() + diskNum; i++) {
                    keys.add(i);
                }
                for (int i = 0; i < rdmDisks.size(); i++) {
                    specs.add(addRdmDiskSpec(template, server.getProfileId(), rdmDisks.get(i), keys.get(i),
                            datastore.getName()));
                }
            }
            // 控制台添加磁盘设备
            List<JSONObject> addDiskList = server.getAddDiskList();
            if (!ListTool.isEmpty(addDiskList)) {
                for (int i = 0; i < addDiskList.size(); i++) {
                    JSONObject object = addDiskList.get(i);
                    Integer disk = null == object.getInteger("disk") ? 0 : object.getInteger("disk");
                    specs.add(addDiskSpec(server.getProfileId(), object.getString("diskType"), datastore.getName(),
                            disk * 1024 * 1024L, "persistent", template, i + 1));
                }
            }
            // 配置网络设备设置
            processNetSpec(navigator, server, specs, template);
            // 新克隆云主机相关设备
            if (!ListTool.isEmpty(specs)) {
                VirtualDeviceConfigSpec[] specArray = new VirtualDeviceConfigSpec[specs.size()];
                configDeviceKeyInSpec(template,specs);
                configSpec.setDeviceChange(specs.toArray(specArray));
                //设置硬件启动顺序
//                VirtualMachineBootOptions bootOptions = new VirtualMachineBootOptions();
//                List<VirtualMachineBootOptionsBootableDevice> devices = new ArrayList<>();
//                for (int i = 0; i < specs.size(); i++) {
//                    VirtualDeviceConfigSpec spec = specs.get(i);
//                    VirtualDevice device = spec.getDevice();
//                    //网卡设备
//                    if (device instanceof VirtualEthernetCard) {
//                        VirtualMachineBootOptionsBootableEthernetDevice ethernetDevice = new VirtualMachineBootOptionsBootableEthernetDevice();
//                        ethernetDevice.setDeviceKey(device.getKey());
//                        logger.info("Network device key:" + device.getKey());
//                        devices.add(ethernetDevice);
//                        //软盘
//                    } else if (device instanceof VirtualFloppy) {
//                        VirtualMachineBootOptionsBootableFloppyDevice floppyDevice = new VirtualMachineBootOptionsBootableFloppyDevice();
//                        devices.add(floppyDevice);
//                        //磁盘设备
//                    } else if (device instanceof VirtualDisk) {
//                        VirtualMachineBootOptionsBootableDiskDevice diskDevice = new VirtualMachineBootOptionsBootableDiskDevice();
//                        diskDevice.setDeviceKey(device.getKey());
//                        logger.info("Disk device key:" + device.getKey());
//                        devices.add(diskDevice);
//                        //cd光驱
//                    } else if (device instanceof VirtualCdrom) {
//                        VirtualMachineBootOptionsBootableCdromDevice cdromDevice = new VirtualMachineBootOptionsBootableCdromDevice();
//                        devices.add(cdromDevice);
//                    }
//                }
//                if (!ListTool.isEmpty(devices)) {
//                    bootOptions.setBootOrder(devices.toArray(new VirtualMachineBootOptionsBootableDevice[devices.size()]));
//                    configSpec.setBootOptions(bootOptions);
//                }
            }
            Folder vmFolder = center.getVmFolder();
            if (!StringUtils.isEmpty(server.getVmFolder())) {
                datacenter = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, server.getVdcName());
                if (null == datacenter) {
                    logger.error("数据中心不存在！");
                    return new BocloudResult(false, "数据中心不存在！");
                }
                center = (Datacenter) datacenter;
                Map<String, Folder> vmFolderMap = new ConcurrentHashMap<>();
                // 云主机创建到指定目录下
                // 单级
                if (StringUtils.isEmpty(server.getParentVmFolder())) {
                    this.getVmFolder(vmFolderMap, center.getVmFolder(), server.getVmFolder());
                } else {
                    //key作为vmFolderMap主键在文件夹内部拼接
                    String key = "";
                    logger.info("父级文件夹：{}，子文件夹：{}", server.getParentVmFolder(), server.getVmFolder());
                    this.getVmFolder(vmFolderMap, center.getVmFolder(), server.getVmFolder(),
                            server.getParentVmFolder(), key);
                    logger.info("当前用户文件夹集合：" + JSON.toJSONString(vmFolderMap));
                    //存在父子逻辑时，重新拼接设置VmFolder值，
                    if (server.getVmFolder().contains(",") || null != server.getParentVmFolder()) {
                        server.setVmFolder(server.getParentVmFolder() + "," + server.getVmFolder());
                    }
                }
                if (!vmFolderMap.isEmpty()) {
                    vmFolder = vmFolderMap.get(server.getVmFolder());
                }
                if (null == vmFolder) {
                    vmFolder = center.getVmFolder();
                    logger.info("云主机将创建到数据中心云主机根目录下...");
                }
                logger.info("云主机创建文件夹：{},云主机所在文件夹全路径：{}", vmFolder.getName(), server.getVmFolder());
            }
            // 模板克隆云主机任务
            logger.info("云主机配置结束，创建基于模板克隆云主机的任务");
            logger.info("cloneSpec:{}",JSON.toJSONString(cloneSpec));
            Task task = template.cloneVM_Task(vmFolder, server.getName(), cloneSpec);
            // 轮询任务状态，超过半个小时自动取消
            // this.waitForTask(task);
            logger.info("开始执行创建云主机{}的任务", server.getName());
            if (task.waitForTask().equals(Task.SUCCESS)) {
                logger.info("基于模板克隆云主机的任务执行成功！");

                VirtualMachine machine = null;
                for (int i = 0; i < time; i++) {
                    logger.info("轮询第" + i + "次获取机器信息");
                    machine = (VirtualMachine) new InventoryNavigator(this.getRootFolder())
                            .searchManagedEntity(VM, server.getName());

                    if(null == machine){
                        Thread.sleep(20 * 1000);
                    }else{
                        break;
                    }
                }

                if (null == machine) {
                    return new BocloudResult(false, server, "创建失败：云主机不存在");
                }
                VirtualMachinePowerState state = machine.getSummary().getRuntime().getPowerState();
                // 少数情况会出现云主机克隆完是关机状态，此处手动开机
                if (VirtualMachinePowerState.poweredOff == state) {
                    task = machine.powerOnVM_Task(null);
                    if (task.waitForTask().equals(Task.SUCCESS)) {
                        machine = (VirtualMachine) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(VM,
                                server.getName());
                        if (null == machine) {
                            return new BocloudResult(false, server, "创建失败：云主机不存在");
                        }
                    }
                }
                ServerModel model = AssembleTools.getInstance().convertServer(machine);
                logger.info("云主机{}创建成功", model.getName());
                return new BocloudResult(true, model, "创建云主机成功！");
            } else {
                VirtualMachine machine = (VirtualMachine) new InventoryNavigator(this.getRootFolder())
                        .searchManagedEntity(VM, server.getName());
                if (null != machine && "5.5".equals(server.getVendorVersion())) {
                    Task powerTask = machine.powerOnVM_Task(null);
                    if (powerTask.waitForTask().equals(Task.SUCCESS)) {
                        ServerModel model = AssembleTools.getInstance().convertServer(machine);
                        logger.info("云主机{}创建成功", model.getName());
                        model.setHostId(server.getHostId());
                        model.setHostName(server.getHostName());
                        model.setOsVersion(server.getOsVersion());
                        return new BocloudResult(true, model, "创建云主机成功！");
                    }
                }
                logger.warn("创建云主机{}失败{}！", server.getName(), task.getTaskInfo().getError().getLocalizedMessage());
                // 删除创建失败的云主机
                if (null != machine) {
                    machine.destroy_Task();
                }
                return new BocloudResult(false, server, "创建失败：" + task.getTaskInfo().getError().getLocalizedMessage());
            }
        } catch (Exception e) {
            logger.error("Create Server By Template exception:", e);
            return new BocloudResult(false, "云主机创建失败！");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    /**
     * 解决控制器ID不能是2007的问题，解决mac地址冲突问题
     * @param machine
     * @param specs
     */
    private void configDeviceKeyInSpec(VirtualMachine machine, List<VirtualDeviceConfigSpec> specs) {
        if (specs == null || specs.isEmpty()) {
            return;
        }

        List<VirtualDisk> diskList = new ArrayList<>();
        List<VirtualEthernetCard> netCardList = new ArrayList<>();
        if (null != machine) {
            VirtualDevice[] devices = machine.getConfig().getHardware().getDevice();
            if (devices != null && devices.length > 0) {
                for (int i = 0; i < devices.length; i++) {
                    VirtualDevice device = devices[i];
                    if (device instanceof VirtualDisk) {
                        diskList.add((VirtualDisk) device);
                    } else if (device instanceof VirtualEthernetCard) {
                        netCardList.add((VirtualEthernetCard) device);
                    }
                }
            }
        }


        int lastDiskKey = diskList.stream().mapToInt(VirtualDevice::getKey).max().orElse(2000);
        int lastNetCardKey = netCardList.stream().mapToInt(VirtualDevice::getKey).max().orElse(2000);

        int lastDiskUnit = diskList.stream().mapToInt(VirtualDevice::getUnitNumber).max().orElse(0);
        int lastNetUnit = netCardList.stream().mapToInt(VirtualDevice::getUnitNumber).max().orElse(0);


        for (VirtualDeviceConfigSpec spec : specs) {
            //添加操作的话需要设置
            if (spec.getOperation().equals(VirtualDeviceConfigSpecOperation.add)) {
                VirtualDevice device = spec.getDevice();
                if (device instanceof VirtualDisk) {
                    lastDiskKey++;
                    lastDiskUnit++;
                    //跳过2007的key，key为2007无法创建
                    if (lastDiskKey == 2007) {
                        lastDiskKey++;
                    }

                    if (lastDiskUnit == 7) {
                        lastDiskUnit++;
                    }
                    device.setUnitNumber(lastDiskUnit);
                    device.setKey(lastDiskKey);
                } else if (device instanceof VirtualEthernetCard) {
                    lastNetCardKey++;
                    lastNetUnit++;
                    device.setUnitNumber(lastNetUnit);
                    device.setKey(lastNetCardKey);
                }
            }
            VirtualDevice device = spec.getDevice();
            //需要设置mac地址为null保证vc自动分配mac防止mac地址冲突
            if (device instanceof VirtualEthernetCard){
                ((VirtualEthernetCard) device).setMacAddress(null);
            }
        }
    }
    /**
     * 编辑磁盘设备
     *
     * @param diskType
     * @param vd
     * @return
     */
    @SuppressWarnings("unused")
    private VirtualDeviceConfigSpec editDiskSpec(String diskType, VirtualDisk vd) {
        VirtualDeviceConfigSpec diskSpec = new VirtualDeviceConfigSpec();
        diskSpec.setOperation(VirtualDeviceConfigSpecOperation.edit);
        if (!StringUtils.isEmpty(diskType)) {
            VirtualDeviceBackingInfo backing = vd.getBacking();
            if (null != backing && backing instanceof VirtualDiskFlatVer2BackingInfo) {
                VirtualDiskFlatVer2BackingInfo flatVer2BackingInfo = (VirtualDiskFlatVer2BackingInfo) backing;
                switch (diskType) {
                    case "sameAsSource":
                        break;
                    case "flat":
                        flatVer2BackingInfo.setEagerlyScrub(false);
                        flatVer2BackingInfo.setThinProvisioned(false);
                        break;
                    case "thick":
                        flatVer2BackingInfo.setEagerlyScrub(true);
                        flatVer2BackingInfo.setThinProvisioned(false);
                        break;
                    case "thin":
                        flatVer2BackingInfo.setEagerlyScrub(null);
                        flatVer2BackingInfo.setThinProvisioned(true);
                        break;
                    default:
                        break;
                }
                vd.setBacking(flatVer2BackingInfo);
            }
        }
        diskSpec.setDevice(vd);
        return diskSpec;
    }

    /**
     * 移除磁盘设备
     *
     * @param vd
     * @return
     */
    private VirtualDeviceConfigSpec removeDiskSpec(VirtualDisk vd) {
        VirtualDeviceConfigSpec diskSpec = new VirtualDeviceConfigSpec();
        diskSpec.setOperation(VirtualDeviceConfigSpecOperation.remove);
        diskSpec.setDevice(vd);
        return diskSpec;
    }

    /**
     * 扩容磁盘设备
     *
     * @param diskType
     * @param vd
     * @param diskSizeKB
     * @return
     */
    private VirtualDeviceConfigSpec expandDiskSpec(String diskType, VirtualDisk vd, long diskSizeKB) {
        VirtualDeviceConfigSpec diskSpec = new VirtualDeviceConfigSpec();
        diskSpec.setOperation(VirtualDeviceConfigSpecOperation.edit);
//		if (!StringUtils.isEmpty(diskType)) {
//			VirtualDeviceBackingInfo backing = vd.getBacking();
//			if (null != backing && backing instanceof VirtualDiskFlatVer2BackingInfo) {
//				VirtualDiskFlatVer2BackingInfo flatVer2BackingInfo = (VirtualDiskFlatVer2BackingInfo) backing;
//				switch (diskType) {
//				case "sameAsSource":
//					break;
//				case "flat":
//					flatVer2BackingInfo.setEagerlyScrub(false);
//					flatVer2BackingInfo.setThinProvisioned(false);
//					break;
//				case "thick":
//					flatVer2BackingInfo.setEagerlyScrub(true);
//					flatVer2BackingInfo.setThinProvisioned(false);
//					break;
//				case "thin":
//					flatVer2BackingInfo.setEagerlyScrub(null);
//					flatVer2BackingInfo.setThinProvisioned(true);
//					break;
//				default:
//					break;
//				}
//				vd.setBacking(flatVer2BackingInfo);
//			}
//		}
        vd.setCapacityInKB(diskSizeKB);
        diskSpec.setDevice(vd);
        return diskSpec;
    }

    /**
     * 添加磁盘设备
     *
     * @param profileId
     * @param diskType
     * @param dsName
     * @param diskSizeKB
     * @param diskMode
     * @param vm
     * @param key
     * @return
     */
    private VirtualDeviceConfigSpec addDiskSpec(String profileId, String diskType, String dsName, long diskSizeKB,
                                                String diskMode, VirtualMachine vm, int key) {
        if (null == vm) {
            return null;
        }
        VirtualDeviceConfigSpec diskSpec = new VirtualDeviceConfigSpec();
        diskSpec.setOperation(VirtualDeviceConfigSpecOperation.add);
        diskSpec.setFileOperation(VirtualDeviceConfigSpecFileOperation.create);
        VirtualDisk vd = new VirtualDisk();
        vd.setCapacityInKB(diskSizeKB);
        vd.setKey(getkey(vm) + key);
        vd.setUnitNumber(getUnitNumber(vm) + key);
        vd.setControllerKey(getcontrollerkey(vm));
        VirtualDiskFlatVer2BackingInfo diskfileBacking = new VirtualDiskFlatVer2BackingInfo();
        String fileName = "[" + dsName + "] ";
        diskfileBacking.setDiskMode(diskMode);
        diskfileBacking.setFileName(fileName);
        if (!StringUtils.isEmpty(diskType)) {
            switch (diskType) {
                case "sameAsSource":
                    break;
                case "flat":
                    diskfileBacking.setEagerlyScrub(false);
                    diskfileBacking.setThinProvisioned(false);
                    break;
                case "thick":
                    diskfileBacking.setEagerlyScrub(true);
                    diskfileBacking.setThinProvisioned(false);
                    break;
                case "thin":
                    diskfileBacking.setEagerlyScrub(null);
                    diskfileBacking.setThinProvisioned(true);
                    break;
                default:
                    break;
            }
        } else {
            diskfileBacking.setEagerlyScrub(false);
            diskfileBacking.setThinProvisioned(true);
        }
        vd.setBacking(diskfileBacking);
        diskSpec.setDevice(vd);
        if (!StringUtils.isEmpty(profileId)) {
            VirtualMachineDefinedProfileSpec profileSpec = new VirtualMachineDefinedProfileSpec();
            profileSpec.setProfileId(profileId);
            diskSpec.setProfile(new VirtualMachineProfileSpec[]{profileSpec});
        } else {
            VirtualMachineDefaultProfileSpec profileSpec = new VirtualMachineDefaultProfileSpec();
            diskSpec.setProfile(new VirtualMachineProfileSpec[]{profileSpec});
        }
        return diskSpec;
    }

    /**
     * 添加rdmDisk
     *
     * @param vm
     * @param profileId
     * @param rdmDisk
     * @param key
     * @param dsName
     * @return
     */
    private VirtualDeviceConfigSpec addRdmDiskSpec(VirtualMachine vm, String profileId, JSONObject rdmDisk, int key,
                                                   String dsName) {
        VirtualDeviceConfigSpec diskSpec = new VirtualDeviceConfigSpec();
        diskSpec.setOperation(VirtualDeviceConfigSpecOperation.add);
        diskSpec.setFileOperation(VirtualDeviceConfigSpecFileOperation.create);
        VirtualDisk vd = new VirtualDisk();
        VirtualDiskRawDiskMappingVer1BackingInfo backingInfo = new VirtualDiskRawDiskMappingVer1BackingInfo();
        backingInfo.setCompatibilityMode(StringUtils.isEmpty(rdmDisk.getString("compatibilityMode")) ? "physicalMode"
                : rdmDisk.getString("compatibilityMode"));
        backingInfo.setDeviceName(rdmDisk.getString("deviceName"));
        backingInfo.setDiskMode("independent_persistent");
        // backingInfo.setLunUuid(rdmDisk.getString("lunUuid"));
        backingInfo.setSharing("sharingNone");
        backingInfo.setFileName("[" + dsName + "] " + vm.getName() + "/" + IDFactory.instance().uuid() + ".vmdk");
        vd.setBacking(backingInfo);
        vd.setCapacityInKB(rdmDisk.getLong("size") * 1024 * 1024L);
        vd.setKey(getkey(vm) + key);
        vd.setUnitNumber(getUnitNumber(vm) + key);
        vd.setControllerKey(getcontrollerkey(vm));
        diskSpec.setDevice(vd);
        if (!StringUtils.isEmpty(profileId)) {
            VirtualMachineDefinedProfileSpec profileSpec = new VirtualMachineDefinedProfileSpec();
            profileSpec.setProfileId(profileId);
            diskSpec.setProfile(new VirtualMachineProfileSpec[]{profileSpec});
        } else {
            VirtualMachineDefaultProfileSpec profileSpec = new VirtualMachineDefaultProfileSpec();
            diskSpec.setProfile(new VirtualMachineProfileSpec[]{profileSpec});
        }
        return diskSpec;
    }

    @Override
    public BocloudResult customizationVm(ServerModel model) {
        try {
            ManagedEntity datacenter = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, model.getVdcName());
            Assert.notNull(datacenter, "数据中心不存在！");
            Datacenter center = (Datacenter) datacenter;
            InventoryNavigator navigator = new InventoryNavigator(center);
            VirtualMachine machine = (VirtualMachine) navigator.searchManagedEntity(VM, model.getName());
            String osCategory = machine.getGuest().getGuestFamily();
            if (StringUtils.isEmpty(osCategory)) {
                osCategory = machine.getConfig().getGuestFullName();
                if (StringUtils.isEmpty(osCategory)) {
                    osCategory = OSCategory.LINUX.name();
                } else if (osCategory.toLowerCase().contains(OSCategory.WINDOWS.name().toLowerCase())) {
                    osCategory = OSCategory.WINDOWS.name();
                } else {
                    osCategory = OSCategory.LINUX.name();
                }
            }
            if (VirtualMachineGuestOsFamily.otherGuestFamily.name().equals(osCategory)
                    && (null != machine.getGuest().getGuestFullName())) {
                osCategory = OSCategory.LINUX.name();
            }
            CustomizationSpec spec = createCustomizationSpec(model, osCategory);
            Task task = machine.customizeVM_Task(spec);
            if (task.waitForTask().equals(Task.SUCCESS)) {
                logger.info("云主机自定义配置成功！");
                machine = (VirtualMachine) navigator.searchManagedEntity(VM, model.getName());
                if (null == machine) {
                    return new BocloudResult(false, "云主机自定义配置失败");
                }
                model = AssembleTools.getInstance().convertServer(machine);
                return new BocloudResult(true, model, "云主机自定义配置成功!");
            } else {
                logger.warn("云主机{}自定义配置失败{}！", model.getName(), task.getTaskInfo().getError().getLocalizedMessage());
                return new BocloudResult(false, "云主机自定义配置失败：" + task.getTaskInfo().getError().getLocalizedMessage());
            }
        } catch (Exception e) {
            logger.error("Get error message:{}", e);
            return new BocloudResult<>(false, e.getMessage());
        }
    }

    /**
     * 创建自定义的规格信息，主要用来设置云主机网卡对应的IP地址信息
     *
     * @param serverModel
     * @param osCategory
     * @return
     */
    private CustomizationSpec createCustomizationSpec(ServerModel serverModel, String osCategory) {
        logger.info("开始构建云主机自定义配置...");
        CustomizationSpec customizationSpec = new CustomizationSpec();
        osCategory = osCategory.toLowerCase();
        if (osCategory.contains("windows")) {
            CustomizationWinOptions options = new CustomizationWinOptions();
            options.setChangeSID(true);
            options.setDeleteAccounts(false);
            customizationSpec.setOptions(options);
            // Windows Sysprep.inf应答文件对象表示。Sysprep将所有个人密钥在Sysprep.inf文件中列出
            CustomizationSysprep sysprep = new CustomizationSysprep();
            CustomizationGuiUnattended guiUnattended = new CustomizationGuiUnattended();
            // 设置administrator自动登录
            guiUnattended.setAutoLogon(false);
            guiUnattended.setAutoLogonCount(0);
            // 时区为北京时间China Standard Time (GMT+08:00) Beijing, Chongqing, Hong Kong
            // SAR,Urumqi
            guiUnattended.setTimeZone(210);
            sysprep.setGuiUnattended(guiUnattended);
            // 设置identification认证
            CustomizationIdentification identification = new CustomizationIdentification();
            // 设置administrator登录密码
            if (!StringUtils.isEmpty(serverModel.getPassword())) {
                CustomizationPassword password = new CustomizationPassword();
                password.setPlainText(true);
                password.setValue(serverModel.getPassword());
                identification.setDomainAdminPassword(password);
                guiUnattended.setPassword(password);
            }
            identification.setJoinWorkgroup("WORKGROUP");
            sysprep.setIdentification(identification);
            // 设置用户数据userData
            CustomizationUserData userData = new CustomizationUserData();
            CustomizationFixedName fixedName = new CustomizationFixedName();
            fixedName.setName(StringUtils.isEmpty(serverModel.getVmHostName()) ? "vm" : serverModel.getVmHostName());
            userData.setComputerName(fixedName);
            userData.setFullName(StringUtils.isEmpty(serverModel.getVmHostName()) ? "vm" : serverModel.getVmHostName());
            userData.setOrgName("vm.com");
            userData.setProductId("");
            sysprep.setUserData(userData);
            serverModel.setAccount("Administrator");
            customizationSpec.setIdentity(sysprep);
        } else if (osCategory.contains("linux")) {
            // 创建文件类型Linux
            CustomizationLinuxOptions options = new CustomizationLinuxOptions();
            customizationSpec.setOptions(options);
            // 创建应答文件
            CustomizationLinuxPrep sysprep = new CustomizationLinuxPrep();
            sysprep.setDomain("");
            // 如果hostname中含有特殊字符会导致spec.identity.hostName设置失败
            String hostName = serverModel.getVmHostName();
            CustomizationFixedName virtualMachineName = new CustomizationFixedName();
            if (null == hostName) {
                hostName = "vm";
            }
            String regEx = "[`~!@#$%^&*()\\-+={}':;,\\[\\].<>/?￥%…（）_+|【】‘；：”“’。，、？\\s]";
            Pattern pattern = Pattern.compile(regEx);
            Matcher matcher = pattern.matcher(hostName);
            virtualMachineName.setName(matcher.replaceAll("-").trim());
            sysprep.setHostName(virtualMachineName);
            sysprep.setHwClockUTC(true);
            customizationSpec.setIdentity(sysprep);
        }
        CustomizationGlobalIPSettings globalIPSettings = new CustomizationGlobalIPSettings();
        // 新云主机网卡配置
        List<NetcardOperation> operations = serverModel.getOperations();
        List<CustomizationAdapterMapping> adapterMappings = new ArrayList<>();
        // 设置IP地址到每个网卡
        for (int i = 0; i < operations.size(); i++) {
            NetcardOperation operation = operations.get(i);
            switch (operation.getOperation()) {
                // 要删除的网卡
                case "REMOVE":
                    continue;
                    // 新添加或编辑的网卡分配IP地址
                case "ADD":
                case "EDIT":
                    IpModel ip = operation.getIp();
                    if (null == ip) {
                        ip = new IpModel();
                        ip.setIp("127.0.0.1");
                    }
                    CustomizationAdapterMapping adapter = new CustomizationAdapterMapping();
                    CustomizationIPSettings ipSetting = new CustomizationIPSettings();
                    // 重新设置自定义规范 IP 地址
                    String[] dnss = null;
                    if (StringUtils.isEmpty(ip.getDns())) {
                        dnss = new String[]{};
                    } else {
                        dnss = ip.getDns().split(",");
                    }
                    // 这样设置的DNS才生效，可以设置多个DNS,中间用逗号隔开
                    globalIPSettings.setDnsServerList(dnss);
                    // 设置全局dns
                    customizationSpec.setGlobalIPSettings(globalIPSettings);
                    ipSetting.setDnsServerList(dnss);
                    // 设置子网掩码
                    String subnetMask = null;
                    if (StringUtils.isEmpty(ip.getMask())) {
                        subnetMask = "255.255.255.0";
                    } else {
                        subnetMask = ip.getMask();
                    }
                    ipSetting.setSubnetMask(subnetMask);
                    if (!StringUtils.isEmpty(ip.getGateway())) {
                        // 设置gateway
                        String[] gateways = new String[1];
                        gateways[0] = ip.getGateway();
                        ipSetting.setGateway(gateways);
                    }
                    // 设置IP地址
                    CustomizationFixedIp ipGenerator = new CustomizationFixedIp();
                    logger.info("IP : " + ip.getIp());
                    ipGenerator.setIpAddress(ip.getIp());
                    ipSetting.setIp(ipGenerator);
                    // 设置IPv6
                    if (operation.isIpv6Enabled()) {
                        CustomizationIPSettingsIpV6AddressSpec ipV6AddressSpec = new CustomizationIPSettingsIpV6AddressSpec();
                        CustomizationFixedIpV6 fixedIpV6 = new CustomizationFixedIpV6();
                        fixedIpV6.setIpAddress(operation.getIpv6().getIp());
                        fixedIpV6.setSubnetMask(IPAddress.get("V6").getMaskLength(operation.getIpv6().getMask()));
                        List<CustomizationFixedIpV6> ipV6s = new ArrayList<>();
                        ipV6s.add(fixedIpV6);
                        ipV6AddressSpec.setIp(ipV6s.toArray(new CustomizationFixedIpV6[ipV6s.size()]));
                        String[] gatways = {operation.getIpv6().getGateway()};
                        ipV6AddressSpec.setGateway(gatways);
                        ipSetting.setIpV6Spec(ipV6AddressSpec);
                    }
                    adapter.setAdapter(ipSetting);
                    adapterMappings.add(adapter);
                    break;
                default:
                    break;
            }

        }
        CustomizationAdapterMapping[] nicSettingMap = adapterMappings
                .toArray(new CustomizationAdapterMapping[adapterMappings.size()]);
        // 设置网卡对应的IP地址
        customizationSpec.setNicSettingMap(nicSettingMap);
        return customizationSpec;
    }

    // 获取云主机磁盘管理的controlerkey
    private int getcontrollerkey(VirtualMachine vm) {
        int controllerkey = 0;
        if (null != vm) {
            VirtualDevice[] devices = vm.getConfig().getHardware().getDevice();
            if (devices != null && devices.length > 0) {
                for (int i = 0; i < devices.length; i++) {
                    VirtualDevice device = devices[i];
                    if (device instanceof VirtualDisk) {
                        controllerkey = device.getControllerKey();
                    }
                }
            }
        }
        return controllerkey;
    }

    // 获取云主机已生成key
    private int getkey(VirtualMachine vm) {
        int key = 0;
        if (null != vm) {
            VirtualDevice[] devices = vm.getConfig().getHardware().getDevice();
            if (devices != null && devices.length > 0) {
                for (int i = 0; i < devices.length; i++) {
                    VirtualDevice device = devices[i];
                    if (device instanceof VirtualDisk) {
                        key = device.getKey();
                    }
                }
            }
        }
        key = key + 1;
        return key;
    }

    // 获取云主机已生成UnitNumber
    private int getUnitNumber(VirtualMachine vm) {
        int number = 0;
        if (null != vm) {
            VirtualDevice[] devices = vm.getConfig().getHardware().getDevice();
            if (devices != null && devices.length > 0) {
                for (int i = 0; i < devices.length; i++) {
                    VirtualDevice device = devices[i];
                    if (device instanceof VirtualDisk) {
                        number = device.getUnitNumber();
                    }
                }
            }
        }
        number = number + 1;
        return number;
    }

    @Override
    public BocloudResult start(String instanceId) {
        logger.info("启动云主机{}", instanceId);
        try {
            ManagedEntity entity = this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true, true);
            Assert.notNull(entity, "云主机不存在！");
            VirtualMachine machine = (VirtualMachine) entity;
            if (machine.getRuntime().getPowerState().equals(VirtualMachinePowerState.poweredOn)) {
                logger.info("云主机{} 已经处于开机状态", instanceId);
                return new BocloudResult(true, "该云主机已处于开机状态");
            }
            Task task = machine.powerOnVM_Task(null);
            ServerModel model = null;
            if (task.waitForTask().equals(Task.SUCCESS)) {
                VirtualMachineToolsStatus toolsStatus = machine.getGuest().getToolsStatus();
                if (null == toolsStatus || "toolsNotInstalled".equals(toolsStatus.toString())) {
                    model = AssembleTools.getInstance().convertServer(machine);
                    return new BocloudResult(true, model, "启动云主机成功");
                }
                logger.info("vmtools正在开启中...");
                boolean flag = true;
                Long start = System.currentTimeMillis();
                while (flag) {
                    // 超时和延时
                    if (System.currentTimeMillis() - start > 10 * 1000 * 60L) {
                        machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                                instanceId, true, true);
                        model = AssembleTools.getInstance().convertServer(machine);
                        flag = false;
                        break;
                    }
                    machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId,
                            true, true);
                    if ("toolsOk".equals(machine.getGuest().getToolsStatus().toString())) {
                        if (null != machine.getGuest().getNet() && machine.getGuest().getNet().length > 0) {
                            logger.info("vmtools启动成功...");
                            flag = false;
                            break;
                        }
                    }
                    if (flag) {
                        Thread.sleep(5000);
                    }
                    model = AssembleTools.getInstance().convertServer(machine);
                }
                logger.info("云主机【{}】启动成功！", instanceId);
                return new BocloudResult(true, model, "启动云主机成功");
            }
            logger.error("云主机【{}】启动失败！原因{}", instanceId, task.getTaskInfo().getError().getLocalizedMessage());
            return new BocloudResult(false, "启动云主机失败");
        } catch (Exception e) {
            logger.error("云主机[{}]启动失败！！", instanceId, e);
            return new BocloudResult(false, "启动云主机失败！");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult stop(String instanceId) {
        logger.info("执行停止云主机{}", instanceId);
        try {
            ManagedEntity entity = this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true, true);
            Assert.notNull(entity, "云主机不存在！");
            VirtualMachine machine = (VirtualMachine) entity;
            if (machine.getRuntime().getPowerState().equals(VirtualMachinePowerState.poweredOff)) {
                logger.info("云主机{}已经处于关机状态", instanceId);
                return new BocloudResult(true, "该云主机已处于关机状态");
            }
            Task task = machine.powerOffVM_Task();
            if (task.waitForTask().equals(Task.SUCCESS)) {
                logger.info("云主机{}停止成功！", instanceId);
                return new BocloudResult(true, "停止云主机成功");
            }
            logger.error("云主机{}停止失败！失败原因：{}", instanceId, task.getTaskInfo().getError().getLocalizedMessage());
            return new BocloudResult(false, "停止云主机失败");
        } catch (Exception e) {
            logger.error("云主机[{}]停止失败！！", instanceId, e);
            return new BocloudResult(false, "云主机停止失败！");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult reboot(String instanceId) {
        logger.info("执行云主机{} 重启任务", instanceId);
        try {
            ManagedEntity entity = this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true, true);
            Assert.notNull(entity, "云主机不存在！");
            VirtualMachine machine = (VirtualMachine) entity;
            machine.rebootGuest();
            logger.info("云主机{} 重启任务下发成功！", instanceId);
            return new BocloudResult(true, "success", "重启云主机成功");
        } catch (RemoteException e) {
            logger.error("云主机[{}]重启失败！！", instanceId, e);
            return new BocloudResult(false, "云主机重启失败：该云主机可能未运行vmware tools");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult remove(String instanceId, String name) {
        logger.info("执行删除云主机{} 操作", instanceId);
        try {
            ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(VM, name);
            if (null == entity) {
                entity = this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true, true);
            }
            if (null == entity) {
                return new BocloudResult(true, "删除云主机成功");
            }
            VirtualMachine machine = (VirtualMachine) entity;
            VirtualMachinePowerState state = machine.getSummary().getRuntime().getPowerState();
            Task task = null;
            // 未关机先关机
            if (VirtualMachinePowerState.poweredOff != state) {
                task = machine.powerOffVM_Task();
                if (!task.waitForTask().equals(Task.SUCCESS)) {
                    String failInfo = "";
                    if (null != task.getTaskInfo().getError()) {
                        failInfo = task.getTaskInfo().getError().getLocalizedMessage();
                    }
                    logger.info("云主机{}关机失败！{}", machine.getName(), failInfo);
                    return new BocloudResult(false, "云主机删除失败：" + failInfo);
                }
            }
            task = machine.destroy_Task();
            logger.info("云主机{}任务删除任务下发", instanceId);
            if (task.waitForTask().equals(Task.SUCCESS)) {
                logger.info("云主机{} 删除任务执行成功！", instanceId);
                return new BocloudResult(true, "删除云主机成功");
            }
            logger.error("云主机{}删除任务执行失败！失败原因：{}", instanceId, task.getTaskInfo().getError().getLocalizedMessage());
            return new BocloudResult(false, "删除云主机失败");
        } catch (Exception e) {
            logger.error("云主机[{}]删除失败！！", instanceId, e);
            return new BocloudResult(false, "删除云主机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult list() {
        logger.info("执行获取所有云主机操作");
        try {
            List<ServerModel> serverModels = new ArrayList<>();
            ManagedEntity[] entities = new InventoryNavigator(this.getRootFolder()).searchManagedEntities(VM);
            if (null == entities || entities.length == 0) {
                logger.info("No VirtualMachine!");
                return new BocloudResult(true, serverModels, "success");
            }
            for (ManagedEntity entity : entities) {
                ServerModel serverModel = AssembleTools.getInstance().convertServer((VirtualMachine) entity);
                if (null != serverModel) {
                    serverModels.add(serverModel);
                }
            }
            logger.info("获取云主机操作结束！");
            return new BocloudResult(true, serverModels, "success");
        } catch (Exception e) {
            logger.error("同步云主机异常！", e);
            return new BocloudResult(false, "同步云主机异常！");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    /**
     * 查询宿主机下的云主机列表
     *
     * @param hostId
     * @return
     */
    @Override
    public BocloudResult list(String hostId) {
        return null;
    }

    @Override
    public BocloudResult detail(String instanceId) {
        logger.info("获取云主机{} 实例详情", instanceId);
        try {
            ManagedEntity entity = this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true, true);
            Assert.notNull(entity, "云主机不存在！");
            logger.info("获取云主机{} 实例详情结束", instanceId);
            VirtualMachine machine = (VirtualMachine) entity;
            ServerModel model = null;
            VirtualMachineToolsStatus toolsStatus = machine.getGuest().getToolsStatus();
            if (null == toolsStatus || "toolsNotInstalled".equals(toolsStatus.toString())) {
                model = AssembleTools.getInstance().convertServer(machine);
            } else {
                boolean flag = true;
                Long start = System.currentTimeMillis();
                while (flag) {
                    // 超时和延时
                    if (System.currentTimeMillis() - start > 1000 * 60L * 5) {
                        flag = false;
                        model = AssembleTools.getInstance().convertServer(machine);
                    } else {
                        if ("toolsOk".equals(machine.getGuest().getToolsStatus().toString())) {
                            flag = false;
                            model = AssembleTools.getInstance().convertServer(machine);
                        } else {
                            if ("poweredOff".equals(machine.getSummary().getRuntime().getPowerState().name())
                                    || "suspended".equals(machine.getSummary().getRuntime().getPowerState().name())) {
                                break;
                            }
                            Thread.sleep(5000);
                            machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                                    instanceId, true, true);
                        }
                    }
                }
            }
            if (null == model) {
                machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true,
                        true);
                model = AssembleTools.getInstance().convertServer(machine);
            }
            return new BocloudResult(true, model, "success");
        } catch (Exception e) {
            logger.error("获取云主机{}详情异常！", instanceId, e);
            return new BocloudResult(false, "获取云主机详情异常！");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult changePassword(String instanceId, String password) {
        return null;
    }

    @Override
    public BocloudResult modify(ServerModel model) {
        String instanceId = model.getInstanceId();
        logger.info("执行修改云主机{} 配置信息", instanceId);
        try {
            VirtualMachine machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                    instanceId, true, true);
            Assert.notNull(machine, "编辑云主机失败：云主机不存在！");
            logger.info("开始修改基础配置信息");
            // 云主机运行状态
            BigDecimal memory = model.getMemory();
            Integer cpu = model.getCpu();
            Integer oldCpu = machine.getSummary().getConfig().getNumCpu();
            BigDecimal oldMemory = BigDecimal.valueOf(machine.getSummary().getConfig().getMemorySizeMB())
                    .divide(BigDecimal.valueOf(1024), 2, BigDecimal.ROUND_HALF_UP);
            // 云主机配置信息
            VirtualMachineConfigSpec configSpec = new VirtualMachineConfigSpec();
            if (null != cpu && 0 != cpu && oldCpu.intValue() != cpu.intValue()) {
                configSpec.setNumCPUs(model.getCpu());
            }
            if (null != memory && 0 != memory.intValue() && oldMemory.compareTo(memory) != 0) {
                configSpec.setMemoryMB(model.getMemory().multiply(BigDecimal.valueOf(1024)).longValue());
            }
            // 设置云主机名称
            configSpec.setName(model.getName());
            // 备注信息
            configSpec.setAnnotation(model.getRemark());
            // 设备集合
            List<VirtualDeviceConfigSpec> specs = new ArrayList<>();
            Integer maxKey = 0;
            // 添加磁盘设备
            if (null != model.getExtendDisk() && model.getExtendDisk() != 0) {
                logger.info("开始添加磁盘设备");
                VirtualDeviceConfigSpec diskSpec = null;
                // 判断云主机是否有磁盘设备
                VirtualDevice[] devices = machine.getConfig().getHardware().getDevice();
                if (null == devices || devices.length == 0) {
                    diskSpec = createDiskSpec(model.getDiskType(), model.getDatastore(),
                            model.getExtendDisk() * 1024 * 1024L, "GB", "persistent");
                } else {
                    boolean flag = false;
                    for (VirtualDevice device : devices) {
                        // 设备中是否含有磁盘设备
                        if (device instanceof VirtualDisk) {
                            flag = true;
                            break;
                        }
                    }
                    if (flag) {
                        diskSpec = addDiskSpec(model.getProfileId(), model.getDiskType(), model.getDatastore(),
                                model.getExtendDisk() * 1024 * 1024L, "persistent", machine, 0);
                    } else {
                        diskSpec = createDiskSpec(model.getDiskType(), model.getDatastore(),
                                model.getExtendDisk() * 1024 * 1024L, "GB", "persistent");
                    }
                }
                if (null == diskSpec) {
                    logger.error("云主机编辑失败：新增磁盘规格编辑失败！");
                    return new BocloudResult(false, "编辑云主机失败！");
                }
                specs.add(diskSpec);
            }
            // 要新增的磁盘
            List<JSONObject> addDiskList = model.getAddDiskList();
            if (!ListTool.isEmpty(addDiskList)) {
                for (int i = 0; i < addDiskList.size(); i++) {
                    JSONObject addDisk = addDiskList.get(i);
                    specs.add(addDiskSpec(model.getProfileId(), model.getDiskType(), addDisk.getString("storeName"),
                            addDisk.getInteger("addSize") * 1024 * 1024L, "persistent", machine, i));
                    maxKey = i;
                }
            }
            VirtualDisk systemDisk = null;
            VirtualDisk dataDisk = null;
            // 过滤出原模板的系统盘
            boolean systemDiskFind = false;
            Map<String, VirtualDisk> diskMap = new HashMap<>();
            // 云主机关联设备
            VirtualDevice[] devices = machine.getConfig().getHardware().getDevice();
            for (int i = 0; devices != null && i < devices.length; i++) {
                // 按照底层vcenter上磁盘顺序，找到的第一块磁盘设备即为系统盘
                VirtualDevice device = devices[i];
                if (device instanceof VirtualDisk) {
                    VirtualDisk disk = (VirtualDisk) device;
                    if (!systemDiskFind) {
                        systemDisk = disk;
                        systemDiskFind = true;
                    } else {
                        dataDisk = disk;
                    }
                    Description deviceInfo = disk.getDeviceInfo();
                    if (null != deviceInfo && null != deviceInfo.getLabel()) {
                        diskMap.put(deviceInfo.getLabel(), disk);
                    }
                }
            }
            if (null != model.getSystemDiskSize() && null != systemDisk) {
                specs.add(expandDiskSpec(model.getDiskType(), systemDisk, model.getSystemDiskSize() * 1024 * 1024L));
            }
            // 扩容数据盘
            if (null != model.getDataDisk() && null != dataDisk) {
                specs.add(expandDiskSpec(model.getDiskType(), dataDisk, model.getDataDisk() * 1024 * 1024L));
            }
            List<String> dataStoreNames = model.getDataStoreNames();
            if (!ListTool.isEmpty(dataStoreNames)) {
                List<Integer> disks = model.getDisks();
                for (int i = 0; i < disks.size(); i++) {
                    logger.info("为云主机{}添加磁盘，选择的数据存储：{}，磁盘大小：{}", machine.getName(), dataStoreNames.get(i),
                            disks.get(i));
                    specs.add(addDiskSpec(model.getProfileId(), model.getDiskType(), dataStoreNames.get(i),
                            disks.get(i) * 1024 * 1024L, "persistent", machine, i));
                    maxKey = i;
                }
            }
            // 要编辑的多个云主机磁盘设备
            List<JSONObject> editDiskList = model.getEditDiskList();
            if (!ListTool.isEmpty(editDiskList)) {
                for (JSONObject editDisk : editDiskList) {
                    String name = editDisk.getString("name");
                    Integer disk = editDisk.getInteger("disk");
                    VirtualDisk virtualDisk = diskMap.get(name);
                    if (null != virtualDisk) {
                        specs.add(expandDiskSpec(model.getDiskType(), virtualDisk, disk * 1024 * 1024L));
                    }
                }
            }
            // 添加rdm disk
            List<JSONObject> rdmDisks = model.getRdmDisks();
            if (!ListTool.isEmpty(rdmDisks)) {
                List<Integer> keys = new ArrayList<>();
                for (int i = maxKey; i < rdmDisks.size() + maxKey; i++) {
                    keys.add(i);
                }
                for (int i = 0; i < rdmDisks.size(); i++) {
                    specs.add(addRdmDiskSpec(machine, model.getProfileId(), rdmDisks.get(i), keys.get(i),
                            rdmDisks.get(i).getString("dsName")));
                }
            }
            // 开始设置
            logger.info("开始修改网络设备信息");
            // 配置网络设备信息
            InventoryNavigator navigator = new InventoryNavigator(this.getRootFolder());
            if (!ListTool.isEmpty(model.getOperations())) {
                logger.info("Start to process network device to new vm：" + model.getName());
                processNetSpec(navigator, model, specs, machine);
                logger.info("End to process network device to new vm：" + model.getName());
            }
            if (!ListTool.isEmpty(specs)) {
                VirtualDeviceConfigSpec[] specArray = new VirtualDeviceConfigSpec[specs.size()];
                configSpec.setDeviceChange(specs.toArray(specArray));
            }
            Task task = machine.reconfigVM_Task(configSpec);
            logger.info("创建云主机{} 重新配置的任务成功并执行", instanceId);
            if (task.waitForTask().equals(Task.SUCCESS)) {
                logger.info("重新配置云主机{} 任务执行成功！", instanceId);
                VirtualMachine entity = (VirtualMachine) new InventoryNavigator(this.getRootFolder())
                        .searchManagedEntity(VM, model.getName());
                if(null == entity) {
                    entity = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId,
                            true, true);
                }
                // 如果修改网卡信息 则异步等待网卡信息加载
                if (!ListTool.isEmpty(model.getOperations())) {
                    // 获取网卡信息
                    entity = waitNetInit(instanceId);
                }
                return new BocloudResult(true, AssembleTools.getInstance().convertServer(entity), "重新配置云主机成功");
            } else {
                String failInfo = "";
                if (null != task.getTaskInfo().getError()) {
                    // 返回主机进入维护模式失败的信息
                    failInfo = task.getTaskInfo().getError().getLocalizedMessage();
                }
                logger.info("重新配置云主机{} 任务执行失败！{}", instanceId, failInfo);
                return new BocloudResult(false, "重新配置云主机失败：" + failInfo);
            }
        } catch (Exception e) {
            logger.error("Modify server exception:", e);
            return new BocloudResult(false, "云主机编辑失败！");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    private VirtualMachine waitNetInit(String instanceId) throws RuntimeFault, RemoteException {
        VirtualMachine entity = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId,
                true, true);
        if (null == entity || null == entity.getGuest()) {
            return entity;
        }
        GuestNicInfo[] guestNicInfoArr = entity.getGuest().getNet();
        if (null == guestNicInfoArr || guestNicInfoArr.length <= 0) {
            int count = 60;
            while (count > 0) {
                sleepSomeTime(1000);
                entity = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId,
                        true, true);
                if (null == entity || null == entity.getGuest()) {
                    break;
                }
                guestNicInfoArr = entity.getGuest().getNet();
                if (null != guestNicInfoArr && guestNicInfoArr.length > 0) {
                    break;
                }
                count--;
            }
        }
        return entity;
    }

    @Override
    public BocloudResult migrate(ServerModel server) {
        try {
            ManagedEntity datacenter = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    server.getVdcName());
            Assert.notNull(datacenter, "数据中心不存在！");
            Datacenter center = (Datacenter) datacenter;
            InventoryNavigator navigator = new InventoryNavigator(center);
            VirtualMachine machine = (VirtualMachine) navigator.searchManagedEntity(VM, server.getName());
            if (null == machine) {
                machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                        server.getInstanceId(), true, true);
            }
            Assert.notNull(machine, "云主机不存在！");
            String hostName = server.getHostName();// 要迁移的宿主机
            String storeName = server.getStoreName();// 要迁移的数据存储
            // 获取宿主机
            HostSystem hostSystem = null;
            if (!StringUtils.isEmpty(hostName)) {
                hostSystem = (HostSystem) navigator.searchManagedEntity(HS, hostName);
                if (null == hostSystem) {
                    logger.error("宿主机{}不存在！", hostName);
                    return new BocloudResult(false, "迁移云主机失败：目标宿主机不存在！");
                }
            }
            // 要更改的数据存储
            Datastore datastore = null;
            if (!StringUtils.isEmpty(storeName)) {
                datastore = (Datastore) navigator.searchManagedEntity(DT, storeName);
                if (null == datastore) {
                    logger.error("数据存储{}不存在！", storeName);
                    return new BocloudResult(false, "要更改的数据存储不存在！");
                }
            }
            // 获取resourcepool
            ComputeResource resource = null;
            if (!StringUtils.isEmpty(server.getClusterName())) {
                resource = (ComputeResource) navigator.searchManagedEntity(CR, server.getClusterName());
            } else if (!StringUtils.isEmpty(hostName)) {
                resource = (ComputeResource) navigator.searchManagedEntity(CR, hostName);
            }
            VirtualMachineRelocateSpec spec = new VirtualMachineRelocateSpec();
            Task task = null;
            List<JSONObject> diskLocators = server.getDiskLocators();
            if (!ListTool.isEmpty(diskLocators)) {
                // 磁盘名称>磁盘
                Map<String, VirtualDisk> diskMap = new HashMap<>();
                // 云主机关联设备
                VirtualMachineConfigInfo configInfo = machine.getConfig();
                Assert.notNull(configInfo, "迁移失败，无法访问云主机配置");
                VirtualDevice[] devices = configInfo.getHardware().getDevice();
                for (int i = 0; devices != null && i < devices.length; i++) {
                    VirtualDevice device = devices[i];
                    if (device instanceof VirtualDisk) {
                        VirtualDisk disk = (VirtualDisk) device;
                        diskMap.put(disk.getDeviceInfo().getLabel(), disk);
                    }
                }
                List<VirtualMachineRelocateSpecDiskLocator> locators = new ArrayList<>();
                for (JSONObject diskLocator : diskLocators) {
                    String dsName = diskLocator.getString("dsName");
                    String diskName = diskLocator.getString("diskName");
                    // String diskMoveType = diskLocator.getString("diskMoveType");
                    VirtualDisk virtualDisk = diskMap.get(diskName);
                    Assert.notNull(virtualDisk, "要迁移的磁盘设备可能已被移除，请先同步数据");
                    Datastore ds = (Datastore) navigator.searchManagedEntity(DT, dsName);
                    Assert.notNull(ds, "要迁移的数据存储可能已被移除，请先同步数据");
                    VirtualMachineRelocateSpecDiskLocator relocateSpecDiskLocator = new VirtualMachineRelocateSpecDiskLocator();
                    relocateSpecDiskLocator.setDatastore(ds.getMOR());
                    relocateSpecDiskLocator.setDiskId(virtualDisk.getKey());
                    // relocateSpecDiskLocator.setDiskMoveType(
                    // VirtualMachineRelocateDiskMoveOptions.moveAllDiskBackingsAndDisallowSharing.toString());
                    locators.add(relocateSpecDiskLocator);
                }
                if (ListTool.isEmpty(locators)) {
                    return new BocloudResult(false, "要迁移的磁盘设备为空");
                }
                VirtualMachineRelocateSpecDiskLocator[] lDiskLocators = new VirtualMachineRelocateSpecDiskLocator[locators
                        .size()];
                spec.setDisk(locators.toArray(lDiskLocators));
                task = machine.relocateVM_Task(spec, VirtualMachineMovePriority.defaultPriority);
            } else if (null != datastore) {
                spec.setDatastore(datastore.getMOR());
                // 同时更改主机和数据存储
                if (null != hostSystem) {
                    spec.setHost(hostSystem.getMOR());
                    spec.setPool(resource.getResourcePool().getMOR());
                    logger.info("开始迁移云主机{}到宿主机{}", server.getName(), hostName);
                } else {
                    logger.info("开始迁移云主机{}到数据存储{}", server.getName(), storeName);
                }
                task = machine.relocateVM_Task(spec, VirtualMachineMovePriority.defaultPriority);
            } else if (null != hostSystem) {
                // 只更改主机
                logger.info("开始迁移云主机{}到宿主机{}", server.getName(), hostName);
                VirtualMachinePowerState state = null;
                if (BocloudStatus.Server.STOPPED.name().equals(server.getStatus())) {
                    state = VirtualMachinePowerState.poweredOff;
                } else if (BocloudStatus.Server.RUNNING.name().equals(server.getStatus())) {
                    state = VirtualMachinePowerState.poweredOn;
                    VirtualDevice[] devices = machine.getConfig().getHardware().getDevice();
                    for (VirtualDevice device : devices) {
                        if (device instanceof VirtualCdrom) {
                            VirtualCdrom cdrom = (VirtualCdrom) device;
                            VirtualDeviceConfigSpec cdromSpec = new VirtualDeviceConfigSpec();
                            cdromSpec.setOperation(VirtualDeviceConfigSpecOperation.edit);
                            VirtualDeviceConnectInfo connectable = new VirtualDeviceConnectInfo();
                            connectable.setAllowGuestControl(true);
                            connectable.setStartConnected(false);
                            connectable.setStatus(VirtualDeviceConnectInfoStatus.untried.name());
                            VirtualCdromAtapiBackingInfo backing = new VirtualCdromAtapiBackingInfo();
                            backing.setDeviceName(cdrom.getDeviceInfo().getLabel());
                            cdrom.setBacking(backing);
                            cdrom.setConnectable(connectable);
                            cdromSpec.setDevice(cdrom);
                            VirtualDeviceConfigSpec[] specArray = new VirtualDeviceConfigSpec[]{cdromSpec};
                            VirtualMachineConfigSpec configSpec = new VirtualMachineConfigSpec();
                            configSpec.setDeviceChange(specArray);
                            machine.reconfigVM_Task(configSpec);
                            break;
                        }
                    }
                } else if (BocloudStatus.Server.SUSPENDED.name().equals(server.getStatus())) {
                    state = VirtualMachinePowerState.suspended;
                }
                logger.info("调整云主机状态结束！");
                task = machine.migrateVM_Task(resource.getResourcePool(), hostSystem,
                        VirtualMachineMovePriority.defaultPriority, state);
                logger.info("创建云主机{}迁移任务成功并执行！", server.getName());
            }
            if (null == task) {
                return new BocloudResult(false, "参数格式错误，云主机迁移失败");
            }
            if (task.waitForTask().equals(Task.SUCCESS)) {
                logger.info("云主机{}迁移任务执行成功", server.getName());
                return new BocloudResult(true, "迁移成功!");
            }
            logger.error("云主机{} 迁移失败！ 原因：{}", server.getName(), task.getTaskInfo().getError().getLocalizedMessage());
            return new BocloudResult(false, "迁移失败！" + task.getTaskInfo().getError().getLocalizedMessage());
        } catch (Exception e) {
            logger.error("云主机{}迁移异常！", server.getName(), e);
            return new BocloudResult(false, "迁移失败！");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult snapshot(SnapshotModel model) {
        logger.info("开始执行云主机{}快照操作", model.getInstanceId());
        try {
            String instanceId = model.getInstanceId();
            ManagedEntity entity = this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true, true);
            Assert.notNull(entity, "云主机不存在！");
            VirtualMachine machine = (VirtualMachine) entity;
            // 原来的快照
            List<SnapshotModel> oldSnapshots = new ArrayList<>();
            VirtualMachineSnapshotInfo virtualMachineSnapshotInfo = machine.getSnapshot();
            String currentSnapshotVal = "";
            if (null != virtualMachineSnapshotInfo) {
                ManagedObjectReference currentSnapshot = virtualMachineSnapshotInfo.getCurrentSnapshot();
                currentSnapshotVal = currentSnapshot.getVal();

                VirtualMachineSnapshotTree[] trees = virtualMachineSnapshotInfo.getRootSnapshotList();
                AssembleTools.getInstance().traverseSnapshotTree(trees, null, oldSnapshots);
            }
            Task task = machine.createSnapshot_Task(model.getName(), model.getRemark(), false, false);
            logger.info("创建云主机{} 快照任务成功并执行！", instanceId);
            if (task.waitForTask().equals(Task.SUCCESS)) {
                logger.info("创建云主机{} 快照任务执行成功！", instanceId);
                // 返回快照
                List<SnapshotModel> newSnapshots = new ArrayList<>();
                virtualMachineSnapshotInfo = machine.getSnapshot();
                if (null != virtualMachineSnapshotInfo) {
                    VirtualMachineSnapshotTree[] trees = machine.getSnapshot().getRootSnapshotList();
                    AssembleTools.getInstance().traverseSnapshotTree(trees, null, newSnapshots);
                }
                SnapshotModel snapshot = null;
                Map<String, SnapshotModel> snapshots = new HashMap<>();
                for (SnapshotModel snapshotModel : oldSnapshots) {
                    snapshots.put(snapshotModel.getSnapshotVal(), snapshotModel);
                }
                for (SnapshotModel snapshotModel : newSnapshots) {
                    if (!snapshots.containsKey(snapshotModel.getSnapshotVal())) {
                        snapshot = snapshotModel;
                        snapshot.setParentSnapshotUuid(currentSnapshotVal);
                        break;
                    }
                }
                logger.info("云主机{} 创建快照后续操作成功结束！", instanceId);
                return new BocloudResult(true, snapshot, "创建快照成功!");
            }
            logger.error("云主机{} 创建快照操作失败：{}", instanceId, task.getTaskInfo().getError().getLocalizedMessage());
            return new BocloudResult(false, "创建快照失败！");
        } catch (Exception e) {
            logger.error("创建云主机{}快照异常！", model.getInstanceId(), e);
            return new BocloudResult(false, "创建快照失败！");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult suspend(String instanceId) {
        logger.info("执行云主机{}挂起操作", instanceId);
        try {
            ManagedEntity entity = this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true, true);
            Assert.notNull(entity, "云主机不存在！");
            VirtualMachine machine = (VirtualMachine) entity;
            if (machine.getRuntime().getPowerState().equals(VirtualMachinePowerState.suspended)) {
                logger.warn("云主机{}已经处于挂起状态！", instanceId);
                return new BocloudResult(true, "云主机已经处于挂起状态");
            }
            Task task = machine.suspendVM_Task();
            logger.info("云主机{}挂起任务创建成功并执行！", instanceId);
            if (task.waitForTask().equals(Task.SUCCESS)) {
                logger.info("云主机{} 挂起任务执行成功！", instanceId);
                ServerModel serverModel = AssembleTools.getInstance().convertServer(machine);
                return new BocloudResult(true, serverModel, "挂起云主机成功");
            }
            logger.error("云主机{}挂起操作失败：{}", instanceId, task.getTaskInfo().getError().getLocalizedMessage());
            return new BocloudResult(false, "挂起云主机失败");
        } catch (Exception e) {
            logger.error("云主机[{}]挂起异常！！", instanceId, e);
            return new BocloudResult(false, "挂起云主机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    private VirtualDeviceConfigSpec addNetSpec(String uuid, String key, VirtualMachine machine, String type,
                                               ManagedObjectReference network, String category, Map<String, OpaqueNetwork> opaqueNetworkMap, Integer count) {
        VirtualDeviceConfigSpec netSpec = new VirtualDeviceConfigSpec();
        netSpec.setOperation(VirtualDeviceConfigSpecOperation.add);
        VirtualEthernetCard vec = createVEC(type);
        if (null != opaqueNetworkMap && opaqueNetworkMap.containsKey(key)) {
            OpaqueNetwork opaqueNetwork = opaqueNetworkMap.get(key);
            VirtualEthernetCardOpaqueNetworkBackingInfo networkBackingInfo = new VirtualEthernetCardOpaqueNetworkBackingInfo();
            NetworkSummary summary = opaqueNetwork.getSummary();
            String opaqueNetworkType = null;
            String opaqueNetworkId = null;
            if (summary instanceof OpaqueNetworkSummary) {
                OpaqueNetworkSummary networkSummary = (OpaqueNetworkSummary) summary;
                opaqueNetworkType = networkSummary.getOpaqueNetworkType();
                opaqueNetworkId = networkSummary.getOpaqueNetworkId();
            }
            networkBackingInfo.setOpaqueNetworkId(opaqueNetworkId);
            networkBackingInfo.setOpaqueNetworkType(opaqueNetworkType);
            vec.setBacking(networkBackingInfo);
        } else if ("VSWITCH".equals(category)) {
            VirtualEthernetCardNetworkBackingInfo netBackInfo = new VirtualEthernetCardNetworkBackingInfo();
            netBackInfo.setDeviceName(key);
            vec.setBacking(netBackInfo);
        } else {
            DistributedVirtualSwitchPortConnection portCon = new DistributedVirtualSwitchPortConnection();
            portCon.setSwitchUuid(uuid);
            portCon.setPortgroupKey(key);
            VirtualEthernetCardDistributedVirtualPortBackingInfo portBackingInfo = new VirtualEthernetCardDistributedVirtualPortBackingInfo();
            portBackingInfo.setPort(portCon);
            vec.setBacking(portBackingInfo);
        }
        vec.setKey(getkey(machine) + count);
        vec.setAddressType("generated");
        VirtualDeviceConnectInfo connectable = new VirtualDeviceConnectInfo();
        connectable.setAllowGuestControl(true);
        connectable.setConnected(true);
        connectable.setStartConnected(true);
        connectable.setStatus("untried");
        vec.setConnectable(connectable);
        netSpec.setDevice(vec);
        return netSpec;
    }

    /**
     * 编辑网卡
     *
     * @param uuid
     * @param key
     * @param device
     * @param category
     * @return
     */
    private VirtualDeviceConfigSpec editNetSpec(String uuid, String key, VirtualDevice device, String category,
                                                Map<String, OpaqueNetwork> opaqueNetworkMap) {
        VirtualDeviceConfigSpec netSpec = new VirtualDeviceConfigSpec();
        netSpec.setOperation(VirtualDeviceConfigSpecOperation.edit);
        if (null != opaqueNetworkMap && opaqueNetworkMap.containsKey(key)) {
            OpaqueNetwork opaqueNetwork = opaqueNetworkMap.get(key);
            VirtualEthernetCardOpaqueNetworkBackingInfo networkBackingInfo = new VirtualEthernetCardOpaqueNetworkBackingInfo();
            NetworkSummary summary = opaqueNetwork.getSummary();
            String opaqueNetworkType = null;
            String opaqueNetworkId = null;
            if (summary instanceof OpaqueNetworkSummary) {
                OpaqueNetworkSummary networkSummary = (OpaqueNetworkSummary) summary;
                opaqueNetworkType = networkSummary.getOpaqueNetworkType();
                opaqueNetworkId = networkSummary.getOpaqueNetworkId();
            }
            networkBackingInfo.setOpaqueNetworkId(opaqueNetworkId);
            networkBackingInfo.setOpaqueNetworkType(opaqueNetworkType);
            device.setBacking(networkBackingInfo);
        } else if ("VSWITCH".equals(category)) {
            VirtualEthernetCardNetworkBackingInfo netBackInfo = new VirtualEthernetCardNetworkBackingInfo();
            netBackInfo.setDeviceName(key);
            device.setBacking(netBackInfo);
        } else {
            DistributedVirtualSwitchPortConnection portCon = new DistributedVirtualSwitchPortConnection();
            portCon.setSwitchUuid(uuid);
            portCon.setPortgroupKey(key);
            VirtualEthernetCardDistributedVirtualPortBackingInfo portBackingInfo = new VirtualEthernetCardDistributedVirtualPortBackingInfo();
            portBackingInfo.setPort(portCon);
            device.setBacking(portBackingInfo);
        }
        VirtualDeviceConnectInfo connectable = new VirtualDeviceConnectInfo();
        connectable.setAllowGuestControl(true);
        connectable.setConnected(true);
        connectable.setStartConnected(true);
        connectable.setStatus("untried");
        device.setConnectable(connectable);
        netSpec.setDevice(device);
        return netSpec;
    }

    private VirtualDeviceConfigSpec removeNetSpec(VirtualDevice device) {
        VirtualDeviceConfigSpec netSpec = new VirtualDeviceConfigSpec();
        netSpec.setOperation(VirtualDeviceConfigSpecOperation.remove);
        VirtualEthernetCard vd = new VirtualEthernetCard();
        vd.setKey(device.getKey());
        netSpec.setDevice(vd);
        return netSpec;
    }

    /**
     * 获取folder中的网络
     *
     * @param folder
     * @param uuidMap
     * @param portMap
     * @param networkMap
     * @throws Exception
     */
    private void getChildrenNetwork(Folder folder, Map<String, String> uuidMap, Map<String, String> portMap,
                                    Map<String, ManagedObjectReference> networkMap) throws Exception {
        ManagedEntity[] entities = folder.getChildEntity();
        if (null != entities && entities.length > 0) {
            for (ManagedEntity network : entities) {
                if (network instanceof Network && !(network instanceof DistributedVirtualPortgroup)) {
                    networkMap.put(network.getName(), network.getMOR());
                } else if (network instanceof VmwareDistributedVirtualSwitch) {
                    String uuid = ((VmwareDistributedVirtualSwitch) network).getUuid();
                    DistributedVirtualPortgroup[] portgroups = ((VmwareDistributedVirtualSwitch) network)
                            .getPortgroup();
                    for (DistributedVirtualPortgroup portgroup : portgroups) {
                        portMap.put(portgroup.getName(), portgroup.getKey());
                        uuidMap.put(portgroup.getName(), uuid);
                    }
                } else if (network instanceof Folder) {
                    Folder childFolder = (Folder) network;
                    getChildrenNetwork(childFolder, uuidMap, portMap, networkMap);
                }
            }
        }
    }

    /**
     * 处理云主机网络设备相关配置
     *
     * @param navigator
     * @param model
     * @param specs
     * @param machine
     * @throws Exception
     */
    private void processNetSpec(InventoryNavigator navigator, ServerModel model, List<VirtualDeviceConfigSpec> specs,
                                VirtualMachine machine) throws Exception {
        // 网络信息
        VirtualDevice[] devices = machine.getConfig().getHardware().getDevice();
        Map<String, VirtualDevice> deviceMap = new HashMap<>();
        Optional.ofNullable(devices).ifPresent(data -> Arrays.stream(data)
                .forEach(virtualDevice -> deviceMap.put(virtualDevice.getDeviceInfo().getLabel(), virtualDevice)));
        Datacenter datacenter = (Datacenter) navigator.searchManagedEntity(DC, model.getVdcName());
        ManagedEntity[] networks = datacenter.getNetworkFolder().getChildEntity();
        Map<String, String> uuidMap = new HashMap<>();
        Map<String, String> portMap = new HashMap<>();
        Map<String, ManagedObjectReference> networkMap = new HashMap<>();
        Map<String, OpaqueNetwork> opaqueNetworkMap = new HashMap<>();
        for (ManagedEntity network : networks) {
            if (network instanceof OpaqueNetwork) {
                opaqueNetworkMap.put(network.getName(), (OpaqueNetwork) network);
            }
            if (network instanceof Network && !(network instanceof DistributedVirtualPortgroup)) {
                networkMap.put(network.getName(), network.getMOR());
            } else if (network instanceof VmwareDistributedVirtualSwitch) {
                String uuid = ((VmwareDistributedVirtualSwitch) network).getUuid();
                DistributedVirtualPortgroup[] portgroups = ((VmwareDistributedVirtualSwitch) network).getPortgroup();
                for (DistributedVirtualPortgroup portgroup : portgroups) {
                    portMap.put(portgroup.getName(), portgroup.getKey());
                    uuidMap.put(portgroup.getName(), uuid);
                }
            } else if (network instanceof Folder) {
                Folder childFolder = (Folder) network;
                getChildrenNetwork(childFolder, uuidMap, portMap, networkMap);
            }
        }
        List<NetcardOperation> operations = model.getOperations();
        Integer key = 0;
        for (NetcardOperation operation : operations) {
            String operate = operation.getOperation();
            // 添加网络
            if ("ADD".equals(operate)) {
                String name = operation.getName();
                // 如果存在标准网络中，则添加标准交换机，否则添加分布式交换机
                if (networkMap.containsKey(name)) {
                    specs.add(addNetSpec(null, name, machine, operation.getType(), networkMap.get(name),
                            NetworkType.SwitchType.VSWITCH.name(), opaqueNetworkMap, key));
                } else {
                    specs.add(addNetSpec(uuidMap.get(name), portMap.get(name), machine, operation.getType(), null,
                            NetworkType.SwitchType.DVSWITCH.name(), opaqueNetworkMap, key));
                }
                key++;
            } else if ("REMOVE".equals(operate) && deviceMap.containsKey(operation.getLabel())) {// 删除网络
                specs.add(removeNetSpec(deviceMap.get(operation.getLabel())));
            } else if ("EDIT".equals(operate) && deviceMap.containsKey(operation.getLabel())) {// 编辑网络
                String name = operation.getName();
                // 如果存在标准网络中，则编辑标准交换机，否则编辑分布式交换机
                if (networkMap.containsKey(name)) {
                    specs.add(editNetSpec(null, name, deviceMap.get(operation.getLabel()),
                            NetworkType.SwitchType.VSWITCH.name(), opaqueNetworkMap));
                } else {
                    String pgName = name;
                    String switchUuid = uuidMap.get(pgName);
                    String portKey = portMap.get(pgName);
                    if (null == pgName) {
                        VirtualEthernetCard device = (VirtualEthernetCard) deviceMap.get(operation.getLabel());
                        // 查询分布式端口组名称
                        DistributedVirtualSwitchPortConnection portgroupConn = ((VirtualEthernetCardDistributedVirtualPortBackingInfo) (device
                                .getBacking())).getPort();
                        if (null != portgroupConn) {
                            switchUuid = portgroupConn.getSwitchUuid();
                            portKey = portgroupConn.getPortgroupKey();
                        }
                    }
                    specs.add(editNetSpec(switchUuid, portKey, deviceMap.get(operation.getLabel()),
                            NetworkType.SwitchType.DVSWITCH.name(), opaqueNetworkMap));
                }
            }
        }

    }

    @Override
    public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean memHotEnable) {
        try {
            ManagedEntity entity = this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true, true);
            Assert.notNull(entity, "云主机不存在，开启热插拔失败");
            VirtualMachine vm = (VirtualMachine) entity;
            VirtualMachinePowerState powerState = vm.getRuntime().getPowerState();// 云主机运行状态
            // 非关机状态不允许cpu或内存热插拔设置
            if (!powerState.name().equals(VirtualMachinePowerState.poweredOff.name())) {
                return new BocloudResult(false, "非关机状态不允许对cpu或内存进行热插拔设置!");
            }
            VirtualMachineConfigSpec vmConfigSpec = new VirtualMachineConfigSpec();
            vmConfigSpec.setCpuHotAddEnabled(cpuHotEnable);
            vmConfigSpec.setCpuHotRemoveEnabled(cpuHotEnable);
            vmConfigSpec.setMemoryHotAddEnabled(memHotEnable);
            // 重新配置云主机
            Task task = vm.reconfigVM_Task(vmConfigSpec);
            if (task.waitForTask().equals(Task.SUCCESS)) {
                return new BocloudResult(true, "CPU或内存热插拔设置成功");
            } else {
                return new BocloudResult(false, "CPU或内存热插拔设置失败：" + task.getTaskInfo().getError().getLocalizedMessage());
            }
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "CPU或内存热插拔设置失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult getTicket(String instanceId) {
        VirtualMachine machine = null;
        try {
            ManagedEntity entity = this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true, true);
            Assert.notNull(entity, "云主机不存在");
            machine = (VirtualMachine) entity;
            VirtualMachineTicket ticket = machine.acquireTicket(VirtualMachineTicketType.webmks.name());
            Assert.notNull(ticket, "获取ticket失败");
            Map<String, Object> resultMap = new HashMap<>();
            resultMap.put("host", ticket.getHost());
            resultMap.put("port", ticket.getPort());
            resultMap.put("ticket", ticket.getTicket());
            return new BocloudResult(true, resultMap, "获取ticket成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "web console只支持exsi 6.0以上的vcenter");
        }
    }

    @Override
    public BocloudResult createInResPool(ServerModel server) {
        try {
            ManagedEntity dcEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    server.getVdcName());
            Assert.notNull(dcEntity, "数据中心不存在");
            Datacenter dc = (Datacenter) dcEntity;
            Folder vmFolder = dc.getVmFolder();
            ManagedEntity hostEntity = new InventoryNavigator(dc).searchManagedEntity(HS, server.getHostName());
            Assert.notNull(hostEntity, "指定的主机不存在");
            HostSystem host = (HostSystem) hostEntity;
            ManagedEntity[] entities = new InventoryNavigator(dc).searchManagedEntities(RP);
            ResourcePool resPool = null;
            entities = Optional.ofNullable(entities).orElse(new ManagedEntity[0]);
            for (ManagedEntity managedEntity : entities) {
                ResourcePool pool = (ResourcePool) managedEntity;
                if (null == server.getClusterName()) {
                    if (host.getName().equals(pool.getOwner().getName())
                            && server.getResPoolName().equals(pool.getName())) {
                        resPool = pool;
                        break;
                    }
                } else {
                    if (server.getClusterName().equals(pool.getOwner().getName())
                            && server.getResPoolName().equals(pool.getName())) {
                        resPool = pool;
                        break;
                    }
                }
            }
            Assert.notNull(resPool, "资源池不存在");
            ManagedEntity vmEntity = new InventoryNavigator(dc).searchManagedEntity(VM, server.getName());
            Assert.isNull(vmEntity, "该云主机名称已存在");
            ManagedEntity dsEntity = new InventoryNavigator(dc).searchManagedEntity("Datastore", server.getDatastore());
            Assert.notNull(dsEntity, "数据存储不存在");
            Datastore ds = (Datastore) dsEntity;
            // 客户机操作系统版本
            String guestOsId = server.getOsName();
            if (StringUtils.isEmpty(guestOsId)) {
                logger.error("Please select the guest operating system and version");
                return new BocloudResult(false, "请选择客户机操作系统和版本");
            }
            // 云主机CPU核数
            Integer cpuCount = server.getCpu();
            // 云主机内存大小
            BigDecimal memorySizeMB = server.getMemory();
            // 虚拟磁盘大小
            Integer diskSize = server.getDisk();
            // 磁盘单位
            String diskUnit = server.getDiskUnit();
            VirtualMachineConfigSpec vmSpec = new VirtualMachineConfigSpec();
            // 开启磁盘UUID配置
            vmSpec.setName(server.getName().replace("_", "-"));
            vmSpec.setAnnotation(server.getRemark());
            vmSpec.setMemoryMB(memorySizeMB.multiply(BigDecimal.valueOf(1024)).longValue());
            vmSpec.setNumCPUs(cpuCount);
            vmSpec.setGuestId(guestOsId);
            vmSpec.setNpivTemporaryDisabled(false);
            vmSpec.setCpuHotAddEnabled(true);
            vmSpec.setMemoryHotAddEnabled(true);
            List<VirtualDeviceConfigSpec> specs = new ArrayList<>();
            // 创建显卡设备
            logger.info("Create a video device for the virtual machine");
            VirtualDeviceConfigSpec videoSpec = createVideoCard();
            specs.add(videoSpec);
            // 创建VMCI设备
            /*
             * logger.info("Create a VMCI device for the virtual machine");
             * VirtualDeviceConfigSpec vmConfigSpec = createVmci(); specs.add(vmConfigSpec);
             */
            // 创建SCSI控制器
            logger.info("Create a SCSI Controller for the virtual machine");
            VirtualDeviceConfigSpec scsiSpec = createScsiSpec(guestOsId);
            specs.add(scsiSpec);
            // 创建虚拟光驱
            logger.info("Create a cdrom device for the virtual machine");
            VirtualDeviceConfigSpec cdromSpec = createCdrom(201, server.getIsoPath());
            specs.add(cdromSpec);
            // 创建硬盘设备
            logger.info("Create a disk device for the virtual machine");
            VirtualDeviceConfigSpec diskSpec = createDiskSpec(server.getDiskType(), ds.getName(), diskSize, diskUnit,
                    "persistent");
            specs.add(diskSpec);
            // 创建虚拟软盘
            logger.info("Create a floppy device for the virtual machine");
            VirtualDeviceConfigSpec floppySpec = createFloppy();
            specs.add(floppySpec);
            boolean isWindows = guestOsId.contains("windows");
            if (isWindows) {
                // 如果是windows机器，则安装USB(鼠标)
                logger.info("Create a usb device for the virtual machine");
                VirtualDeviceConfigSpec usbSpec = createUsb();
                specs.add(usbSpec);
            }
            // 创建网络适配器(多网卡)
            logger.info("Create network adapter device for the virtual machine");
            List<NicAdapterModel> nicAdapters = server.getNicAdapters();
            if (!ListTool.isEmpty(nicAdapters)) {
                for (NicAdapterModel nicAdapter : nicAdapters) {
                    VirtualDeviceConfigSpec nicSpec = null;
                    if (nicAdapter.getSwitchType().equals(NetworkType.SwitchType.VSWITCH.name())) {
                        nicSpec = createNicSpec(nicAdapter.getNetName(), nicAdapter.getAdapter(),
                                nicAdapter.getSwitchType(), null, null);
                    } else if (nicAdapter.getSwitchType().equals(NetworkType.SwitchType.DVSWITCH.name())) {
                        ManagedEntity dvEntity = new InventoryNavigator(dc)
                                .searchManagedEntity("DistributedVirtualSwitch", nicAdapter.getSwitchName());
                        String switchUuid = null;
                        // 查询分布式交换机
                        if (null != dvEntity) {
                            DistributedVirtualSwitch dvswitch = (DistributedVirtualSwitch) dvEntity;
                            switchUuid = dvswitch.getUuid();
                        }
                        // 查询端口组
                        ManagedEntity dpEntity = new InventoryNavigator(dc)
                                .searchManagedEntity("DistributedVirtualPortgroup", nicAdapter.getNetName());
                        String pgroupKey = null;
                        if (null != dpEntity) {
                            DistributedVirtualPortgroup dPortGroup = (DistributedVirtualPortgroup) dpEntity;
                            pgroupKey = dPortGroup.getKey();
                        }
                        nicSpec = createNicSpec(nicAdapter.getNetName(), nicAdapter.getAdapter(),
                                nicAdapter.getSwitchType(), pgroupKey, switchUuid);
                    }
                    specs.add(nicSpec);
                }
            }
            VirtualDeviceConfigSpec[] deviceConfigSpec = specs.toArray(new VirtualDeviceConfigSpec[specs.size()]);
            vmSpec.setDeviceChange(deviceConfigSpec);
            VirtualMachineFileInfo vmfi = new VirtualMachineFileInfo();
            vmfi.setVmPathName("[" + ds.getName() + "]");
            vmSpec.setFiles(vmfi);
            ToolsConfigInfo tools = new ToolsConfigInfo();
            // 自动检查vmware-tools的更新
            // tools.setToolsUpgradePolicy(UpgradePolicy.upgradeAtPowerCycle.name());
            vmSpec.setTools(tools);
            logger.info("Create Vmware Virtual machine start...");
            Task task = vmFolder.createVM_Task(vmSpec, resPool, host);
            if (task.waitForTask() == Task.SUCCESS) {
                VirtualMachine machine = (VirtualMachine) new InventoryNavigator(dc).searchManagedEntity(VM,
                        vmSpec.getName());
                if (null == machine) {
                    logger.error("Create vmware Virtual fail");
                    return new BocloudResult(false, "创建云主机失败");
                }
                ServerModel model = AssembleTools.getInstance().convertServer(machine);
                logger.info("Create Vmware Virtual machine success...");
                return new BocloudResult(true, model, "创建云主机成功");
            } else {
                logger.warn("Create Vmware Virtual machine fail...");
                return new BocloudResult(false, "创建云主机失败：" + task.getTaskInfo().getError().getLocalizedMessage());
            }
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "创建云主机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    /**
     * 创建USB
     *
     * @return
     */
    private VirtualDeviceConfigSpec createUsb() {
        VirtualDeviceConfigSpec spec = new VirtualDeviceConfigSpec();
        spec.setOperation(VirtualDeviceConfigSpecOperation.add);
        VirtualUSBController usbCotl = new VirtualUSBController();
        usbCotl.setAutoConnectDevices(false);
        usbCotl.setEhciEnabled(true);
        usbCotl.setBusNumber(0);
        usbCotl.setControllerKey(100);
        usbCotl.setKey(7000);
        spec.setDevice(usbCotl);
        return spec;
    }

    /**
     * 创建显卡
     *
     * @return
     */
    private VirtualDeviceConfigSpec createVideoCard() {
        VirtualDeviceConfigSpec spec = new VirtualDeviceConfigSpec();
        spec.setOperation(VirtualDeviceConfigSpecOperation.add);
        VirtualMachineVideoCard card = new VirtualMachineVideoCard();
        // 是否启用自动检测
        card.setUseAutoDetect(true);
        // 是否启用3D支持
        card.setEnable3DSupport(false);
        // 3D渲染
        card.setUse3dRenderer("automatic");
        // 显示器数目
        card.setNumDisplays(1);
        spec.setDevice(card);
        return spec;
    }

    /**
     * 创建VMCI设备
     *
     * @return
     */
    @SuppressWarnings("unused")
    private VirtualDeviceConfigSpec createVmci() {
        VirtualDeviceConfigSpec spec = new VirtualDeviceConfigSpec();
        spec.setOperation(VirtualDeviceConfigSpecOperation.add);
        VirtualMachineVMCIDevice vmci = new VirtualMachineVMCIDevice();
        // 确定是否为该云主机启用对VMCI通信的过滤
        vmci.setFilterEnable(true);
        spec.setDevice(vmci);
        return spec;
    }

    /**
     * 创建SCSI控制器
     *
     * @param guestOsId
     * @return
     */
    private VirtualDeviceConfigSpec createScsiSpec(String guestOsId) {
        VirtualDeviceConfigSpec scsiSpec = new VirtualDeviceConfigSpec();
        scsiSpec.setOperation(VirtualDeviceConfigSpecOperation.add);
        VirtualSCSIController scsiCtrl = guestOsId.contains("windows9") ? new VirtualLsiLogicSASController()
                : new VirtualLsiLogicController();
        // 是否开启热插拔
        scsiCtrl.setHotAddRemove(true);
        // 虚拟磁盘不能在云主机之间共享
        scsiCtrl.setSharedBus(VirtualSCSISharing.noSharing);
        scsiCtrl.setKey(0);
        scsiCtrl.setBusNumber(0);
        scsiSpec.setDevice(scsiCtrl);
        return scsiSpec;
    }

    /**
     * 创建虚拟光驱
     *
     * @param ckey
     * @return
     */
    private VirtualDeviceConfigSpec createCdrom(int ckey, String filePath) {
        VirtualDeviceConfigSpec spec = new VirtualDeviceConfigSpec();
        spec.setOperation(VirtualDeviceConfigSpecOperation.add);
        VirtualCdrom cdrom = new VirtualCdrom();
        cdrom.setControllerKey(ckey);
        VirtualDeviceConnectInfo connectable = new VirtualDeviceConnectInfo();
        // 允许访客设置
        connectable.setAllowGuestControl(true);
        // 打开电源时是否连接
        connectable.setStartConnected(true);
        // 未校验
        connectable.setStatus(VirtualDeviceConnectInfoStatus.untried.name());
        VirtualCdromIsoBackingInfo backing = new VirtualCdromIsoBackingInfo();
        // 镜像路径
        backing.setFileName(filePath);
        cdrom.setBacking(backing);
        cdrom.setConnectable(connectable);
        spec.setDevice(cdrom);
        return spec;
    }

    /**
     * 创建硬盘
     *
     * @param diskType
     * @param dsName
     * @param diskSize
     * @param diskUnit
     * @param diskMode
     * @return
     */
    private VirtualDeviceConfigSpec createDiskSpec(String diskType, String dsName, long diskSize, String diskUnit,
                                                   String diskMode) {
        VirtualDeviceConfigSpec diskSpec = new VirtualDeviceConfigSpec();
        diskSpec.setOperation(VirtualDeviceConfigSpecOperation.add);
        diskSpec.setFileOperation(VirtualDeviceConfigSpecFileOperation.create);
        VirtualDisk vd = new VirtualDisk();
        Long diskSizeKB = 0L;
        switch (diskUnit) {
            case "MB":
                diskSizeKB = diskSize * 1024L;
                break;
            case "GB":
                diskSizeKB = diskSize * 1024L * 1024L;
                break;
            case "TB":
                diskSizeKB = diskSize * 1024L * 1024L * 1024L;
                break;
            default:
                break;
        }
        vd.setCapacityInKB(diskSizeKB);
        vd.setControllerKey(0);
        vd.setKey(0);
        diskSpec.setDevice(vd);
        VirtualDiskFlatVer2BackingInfo diskfileBacking = new VirtualDiskFlatVer2BackingInfo();
        String fileName = "[" + dsName + "]";
        diskfileBacking.setFileName(fileName);
        diskfileBacking.setDiskMode(diskMode);
        if (!StringUtils.isEmpty(diskType)) {
            switch (diskType) {
                case "sameAsSource":
                    break;
                case "flat":
                    diskfileBacking.setEagerlyScrub(false);
                    diskfileBacking.setThinProvisioned(false);
                    break;
                case "thick":
                    diskfileBacking.setEagerlyScrub(true);
                    diskfileBacking.setThinProvisioned(false);
                    break;
                case "thin":
                    diskfileBacking.setEagerlyScrub(null);
                    diskfileBacking.setThinProvisioned(true);
                    break;
                default:
                    break;
            }
        } else {
            diskfileBacking.setEagerlyScrub(false);
            diskfileBacking.setThinProvisioned(true);
        }
        vd.setBacking(diskfileBacking);
        vd.setUnitNumber(0);
        VirtualMachineDefaultProfileSpec profileSpec = new VirtualMachineDefaultProfileSpec();
        diskSpec.setProfile(new VirtualMachineProfileSpec[]{profileSpec});
        return diskSpec;
    }

    /**
     * 创建虚拟软盘
     *
     * @return
     */
    private VirtualDeviceConfigSpec createFloppy() {
        VirtualDeviceConfigSpec spec = new VirtualDeviceConfigSpec();
        spec.setOperation(VirtualDeviceConfigSpecOperation.add);
        VirtualFloppy floppy = new VirtualFloppy();
        VirtualDeviceConnectInfo connectable = new VirtualDeviceConnectInfo();
        connectable.setConnected(false);
        connectable.setAllowGuestControl(true);
        connectable.setStartConnected(true);
        connectable.setStatus(VirtualDeviceConnectInfoStatus.untried.name());
        spec.setDevice(floppy);
        VirtualFloppyRemoteDeviceBackingInfo backing = new VirtualFloppyRemoteDeviceBackingInfo();
        backing.setUseAutoDetect(false);
        backing.setDeviceName("Floppy drive 1");
        floppy.setBacking(backing);
        return spec;
    }

    private VirtualEthernetCard createVEC(String adapter) {
        if (StringUtils.isEmpty(adapter)) {
            adapter = "E1000";
        }
        VirtualEthernetCard vec = null;
        switch (adapter) {
            case "E1000":
                vec = new VirtualE1000();
                break;
            case "VMXNET2":
                vec = new VirtualVmxnet2();
                break;
            case "VMXNET3":
                vec = new VirtualVmxnet3();
                break;
            case "E1000E":
                vec = new VirtualE1000e();
                break;
            default:
                vec = new VirtualPCNet32();
                break;
        }
        return vec;
    }

    /**
     * 创建网络适配器
     *
     * @param netName
     * @param adapter
     * @param switchType
     * @param pGroupKey
     * @param switchUuid
     * @return
     * @throws Exception
     */
    private VirtualDeviceConfigSpec createNicSpec(String netName, String adapter, String switchType, String pGroupKey,
                                                  String switchUuid) throws Exception {
        VirtualDeviceConfigSpec nicSpec = new VirtualDeviceConfigSpec();
        nicSpec.setOperation(VirtualDeviceConfigSpecOperation.add);
        VirtualEthernetCard vdc = createVEC(adapter);
        if (switchType.equals(NetworkType.SwitchType.VSWITCH.name())) {
            // 标准交换机
            VirtualEthernetCardNetworkBackingInfo netBackInfo = new VirtualEthernetCardNetworkBackingInfo();
            netBackInfo.setDeviceName(netName);
            vdc.setBacking(netBackInfo);
        } else if (switchType.equals(NetworkType.SwitchType.DVSWITCH.name())) {
            // 分布式交换机
            DistributedVirtualSwitchPortConnection portCon = new DistributedVirtualSwitchPortConnection();
            portCon.setSwitchUuid(switchUuid);
            portCon.setPortgroupKey(pGroupKey);
            VirtualEthernetCardDistributedVirtualPortBackingInfo portBackingInfo = new VirtualEthernetCardDistributedVirtualPortBackingInfo();
            portBackingInfo.setPort(portCon);
            vdc.setBacking(portBackingInfo);
        }
        vdc.setAddressType("generated");
        nicSpec.setDevice(vdc);
        return nicSpec;
    }

    /**
     * 获取iso镜像名称
     *
     * @param description
     * @return
     */
    private String getIso(Description description) {
        return description.getSummary().substring(description.getSummary().lastIndexOf("/") + 1);
    }

    @Override
    public BocloudResult mountToolsInstaller(String instanceId) {
        try {
            VirtualMachine machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                    instanceId, true, true);
            Assert.notNull(machine, "云主机不存在！");
            VirtualDevice[] devices = machine.getConfig().getHardware().getDevice();
            devices = Optional.ofNullable(devices).orElse(new VirtualDevice[0]);
            boolean installed = Arrays.stream(devices).filter(device -> device instanceof VirtualCdrom)
                    .map(device -> ((VirtualCdrom) device).getDeviceInfo())
                    .filter(description -> Objects.nonNull(description))
                    .anyMatch(description -> !StringUtils.isEmpty(description.getSummary())
                            && VMWARE_TOOLS_ISO.contains(getIso(description)));
            if (installed) {
                logger.info("云主机【" + machine.getName() + "】vmware tools安装镜像已挂载，无需再次操作");
                return new BocloudResult(false, "vmware tools安装镜像已挂载，无需再次操作");
            }
            machine.mountToolsInstaller();
            return new BocloudResult(true, "挂载vmware tools安装镜像成功，请进入云主机进行手动安装配置");
        } catch (InvalidState e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "云主机未运行，或者VMware Tools CD已安装");
        } catch (RuntimeFault e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "挂载vmware tools安装镜像失败：" + e.getMessage());
        } catch (VmToolsUpgradeFault e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "挂载vmware tools安装镜像失败：" + e.getMessage());
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "挂载vmware tools安装镜像失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult unmountToolsInstaller(String instanceId) {
        try {
            VirtualMachine machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                    instanceId, true, true);
            Assert.notNull(machine, "云主机不存在！");
            machine.unmountToolsInstaller();
            return new BocloudResult(true, "卸载vmware tools安装镜像成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "卸载vmware tools安装镜像失败：" + e.getMessage());
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult upgradeToolsTask(String instanceId, String options) {
        try {
            VirtualMachine machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                    instanceId, true, true);
            Assert.notNull(machine, "云主机不存在！");
            Task task = null;
            if (StringUtils.isEmpty(options)) {
                task = machine.upgradeTools_Task("");
            } else {
                task = machine.upgradeTools_Task(options);
            }
            if (task.waitForTask().equals(Task.SUCCESS)) {
                logger.info("Upgrade vmware tools success");
                return new BocloudResult(true, "升级vmware tools成功");
            } else {
                return new BocloudResult(false,
                        "升级vmware tools失败：" + task.getTaskInfo().getError().getLocalizedMessage());
            }
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "升级vmware tools失败：" + e.getMessage());
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult pause(String instanceId) {
        logger.info("执行云主机{}休眠操作", instanceId);
        try {
            ManagedEntity entity = this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true, true);
            Assert.notNull(entity, "云主机不存在！");
            VirtualMachine machine = (VirtualMachine) entity;
            if (machine.getRuntime().getPowerState().equals(VirtualMachinePowerState.suspended)) {
                logger.warn("云主机{}已经处于休眠状态！", instanceId);
                return new BocloudResult(true, "云主机已经处于休眠状态");
            }
            machine.standbyGuest();
            ServerModel serverModel = AssembleTools.getInstance().convertServer(machine);
            return new BocloudResult(true, serverModel, "休眠云主机成功");
        } catch (Exception e) {
            logger.error("云主机[{}]休眠异常！！", instanceId, e);
            return new BocloudResult(false, "休眠云主机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult unPause(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult resume(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult getNetCard(String instanceId) {
        try {
            VirtualMachine machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                    instanceId, true, true);
            Assert.notNull(machine, "云主机或模板不存在");
            VirtualDevice[] devices = machine.getConfig().getHardware().getDevice();
            if (null == devices || devices.length == 0) {
                return new BocloudResult(false, "云主机或模板下无任何可用硬件设备");
            }
            List<NetworkCardModel> networkCardModels = new ArrayList<>();
            for (VirtualDevice device : devices) {
                if (null != device) {
                    if (device instanceof VirtualEthernetCard) {
                        NetworkCardModel networkCardModel = new NetworkCardModel();
                        networkCardModel.setLabel(device.getDeviceInfo().getLabel());
                        networkCardModel.setMac(((VirtualEthernetCard) device).getMacAddress());
                        networkCardModel.setUsed(true);
                        String name = null;
                        try {
                            name = ((VirtualDeviceDeviceBackingInfo) device.getBacking()).getDeviceName();
                            networkCardModel.setPortType(NetworkType.PortGroupType.VIRTUAL.name());
                        } catch (Exception e) {
                            name = ((VirtualEthernetCardDistributedVirtualPortBackingInfo) (device.getBacking()))
                                    .getPort().getPortgroupKey();
                            networkCardModel.setPortType(NetworkType.PortGroupType.DVPORT.name());
                        }
                        networkCardModel.setName(name);
                        networkCardModels.add(networkCardModel);
                    }
                }
            }
            return new BocloudResult(true, networkCardModels, "查询网络适配器数量成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询云主机或模板信息失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult getVmIps(String instanceId) {
        try {
            VirtualMachine machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                    instanceId, true, true);
            Assert.notNull(machine, "云主机或模板不存在");
            // vmware tools安装状态
            VirtualMachineToolsStatus toolsStatus = machine.getGuest().getToolsStatus();
            if (null == toolsStatus || VirtualMachineToolsStatus.toolsNotInstalled == toolsStatus) {
                return new BocloudResult(false, "该云主机未安装VMware tools，获取IP地址失败");
            }
            logger.info("【" + machine.getName() + "】VMware tools正在开启中...");
            boolean flag = true;
            Long start = System.currentTimeMillis();
            while (flag) {
                Thread.sleep(5 * 1000L);
                // 设置等待超时时间为20分钟
                if (System.currentTimeMillis() - start > 20 * 1000 * 60) {
                    flag = false;
                    continue;
                }
                machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true,
                        true);
                // vmware tools未运行
                if (VirtualMachineToolsStatus.toolsNotRunning == machine.getGuest().getToolsStatus()) {
                    logger.info("【" + machine.getName() + "】VMware tools还未运行成功，IP获取等待中...");
                    continue;
                }
                // 如果vmware tools启动成功，则判断IP是否已经获取到，如果未获取到继续轮询
                GuestNicInfo[] nicInfos = machine.getGuest().getNet();
                if (null == nicInfos || nicInfos.length == 0) {
                    logger.info("【" + machine.getName() + "】VMware tools启动成功，IP还未获取到...");
                    continue;
                }
                if (ipReady(nicInfos)) {
                    logger.info("【" + machine.getName() + "】VMware tools启动成功，IP获取完毕...");
                    flag = false;
                }
            }
            machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true,
                    true);
            Assert.notNull(machine, "云主机异常丢失");
            return new BocloudResult(true, AssembleTools.getInstance().convertServer(machine), "获取云主机IP地址成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "获取云主机IP地址失败");
        }
    }

    private boolean ipReady(GuestNicInfo[] nics) {
        return Arrays.stream(nics).filter(
                nic -> Objects.nonNull(nic.getIpConfig()) && !ArrayTool.isEmpty(nic.getIpConfig().getIpAddress()))
                .map(nic -> nic.getIpConfig().getIpAddress()).allMatch(addresses -> ipReady(addresses));
    }

    private boolean ipReady(NetIpConfigInfoIpAddress[] addresses) {
        return Arrays.stream(addresses).filter(address -> IPAddress.ipv4Validate(address.getIpAddress()))
                .noneMatch(address -> address.getIpAddress().startsWith("169."));
    }

    @Override
    public BocloudResult markAsTemplate(String instanceId) {
        try {
            VirtualMachine machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                    instanceId, true, true);
            Assert.notNull(machine, "云主机不存在！");
            machine.markAsTemplate();
            return new BocloudResult(true, "标记成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "标记失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
        String windowsUserName = "Administrator";
        ServerModel model = new ServerModel();
        try {
            VirtualMachine machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                    instanceId, true, true);
            Assert.notNull(machine, "云主机不存在！");
            GuestOperationsManager gom = this.getServiceInstance().getGuestOperationsManager();
            if (!"guestToolsRunning".equalsIgnoreCase(machine.getGuest().getToolsRunningStatus())
                    || !machine.getGuest().getGuestOperationsReady()) {
                logger.error("The VMware Tools is not running in the Guest OS on VM " + machine.getName());
                return new BocloudResult(false, "云主机未安装vmware tools，修改密码失败");
            }
            // 云主机操作系统
            String osFamily = machine.getGuest().getGuestFamily();
            logger.info("osFamily: " + osFamily);
            boolean isWindows = osFamily.contains("windowsGuest");
            String currentPassword = oldPasswd;
            String newPassword = newPasswd;
            model.setAccount(isWindows ? windowsUserName : "root");
            model.setPassword(newPasswd);
            NamePasswordAuthentication creds = new NamePasswordAuthentication();
            creds.setUsername(isWindows ? windowsUserName : "root");
            creds.setPassword(currentPassword);
            GuestProgramSpec spec = new GuestProgramSpec();
            if (isWindows) {
                spec.programPath = "C:\\Windows\\System32\\cmd.exe";
                spec.arguments = "/C net user "+windowsUserName+" " + newPassword;
            } else {
                spec.programPath = "/bin/bash";
                // suse操作系统特殊处理一下
                String guestFullName = machine.getConfig().getGuestFullName();
                if (!StringUtils.isEmpty(guestFullName) && guestFullName.contains("SUSE")) {
                    spec.arguments = "-c 'echo root:" + newPassword + " | chpasswd'";
                } else {
                    spec.arguments = "-c 'echo " + newPassword + " | passwd root --stdin'";
                }
            }
            logger.info("vm {} old password：{}", machine.getName(), oldPasswd);
            logger.info("vm {} new password：{}", machine.getName(), newPassword);
            GuestProcessManager gpm = gom.getProcessManager(machine);
            long pid = -1;
            pid = gpm.startProgramInGuest(creds, spec);
            logger.info("pid: " + pid);
            Thread.sleep(3000);
            creds.password = newPassword;
            for (int i = 0; i < 10; i++) {
                try {
                    GuestProcessInfo[] infoList = gpm.listProcessesInGuest(creds, new long[]{pid});
                    GuestProcessInfo info = infoList[0];
                    if (info.getExitCode() == null) {
                        logger.info("Waiting for the process to exit ... ");
                    } else {
                        logger.info("exit code: " + info.getExitCode());
                        return new BocloudResult(true, model, "密码修改成功");
                    }
                }catch (Exception e){
                    logger.warn("等待命令修改密码命令执行：{}", e.getMessage());
                }
                Thread.sleep(3000);
            }
            return new BocloudResult(true, model, "密码修改失败");
        } catch (InvalidGuestLogin e) {
            logger.error("Get error message", e);
            return new BocloudResult(false, model, e.getMessage());
        } catch (Exception e) {
            logger.error("Get error message", e);
            return new BocloudResult(false, model, "密码修改失败");
        }
    }

    @Override
    public BocloudResult initVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
        String windowsUserName = "Administrator";
        ServerModel model = new ServerModel();
        model.setPassword(newPasswd);
        try {
            VirtualMachine machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                    instanceId, true, true);
            Assert.notNull(machine, "云主机不存在！");
            GuestOperationsManager gom = this.getServiceInstance().getGuestOperationsManager();
            if (!"guestToolsRunning".equalsIgnoreCase(machine.getGuest().getToolsRunningStatus())
                    || !machine.getGuest().getGuestOperationsReady()) {
                logger.error("The VMware Tools is not running in the Guest OS on VM " + machine.getName());
                return new BocloudResult(false, "云主机未安装vmware tools，修改密码失败");
            }
            // 云主机操作系统
            String osFamily = machine.getGuest().getGuestFamily();
            logger.info("osFamily: " + osFamily);
            boolean isWindows = osFamily.contains("windowsGuest");
            String currentPassword = oldPasswd;
            String newPassword = newPasswd;
            model.setAccount(isWindows ? windowsUserName : "root");
            model.setPassword(newPasswd);
            NamePasswordAuthentication creds = new NamePasswordAuthentication();
            creds.setUsername(isWindows ? windowsUserName : "root");
            creds.setPassword(currentPassword);
            GuestProgramSpec spec = new GuestProgramSpec();
            if (isWindows) {
                spec.programPath = "C:\\Windows\\System32\\cmd.exe";
                spec.arguments = "/C net user "+windowsUserName+" " + newPassword;
            } else {
                spec.programPath = "/bin/bash";
                // suse操作系统特殊处理一下
                String guestFullName = machine.getConfig().getGuestFullName();
                if (!StringUtils.isEmpty(guestFullName) && guestFullName.contains("SUSE")) {
                    spec.arguments = "-c 'echo root:" + newPassword + " | chpasswd'";
                } else {
                    spec.arguments = "-c 'echo " + newPassword + " | passwd root --stdin'";
                }
            }
            logger.info("vm {} old password：{}", machine.getName(), oldPasswd);
            GuestProcessManager gpm = gom.getProcessManager(machine);
            long pid = -1;
            try {
                pid = gpm.startProgramInGuest(creds, spec);
            } catch (Exception e) {
                logger.warn("执行修改密码命令：", e);
            }
            logger.info("pid: " + pid);
            Thread.sleep(3000);
            creds.password = newPassword;
            for (int i = 0; i < 10; i++) {
                try {
                    GuestProcessInfo[] infoList = gpm.listProcessesInGuest(creds, new long[]{pid});
                    GuestProcessInfo info = infoList[0];
                    if (info.getExitCode() == null) {
                        logger.info("Waiting for the process to exit ... ");
                    } else {
                        logger.info("exit code: " + info.getExitCode());
                        return new BocloudResult(true, model, "密码修改成功");
                    }
                }catch (Exception e){
                    logger.warn("等待命令修改密码命令执行：{}", e.getMessage());
                }
                Thread.sleep(3000);
            }
            return new BocloudResult(true, model, "密码修改失败");
        } catch (InvalidGuestLogin e) {
            logger.error("Get error message", e);
            return new BocloudResult(false, model, e.getMessage());
        } catch (Exception e) {
            logger.error("Get error message", e);
            return new BocloudResult(false, model, "密码修改失败");
        }
    }

    @Override
    public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
        try {
            VirtualMachine machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                    instanceId, true, true);
            Assert.notNull(machine, "云主机不存在！");
            // 单位GB
            Integer extendSizeKb = extendSize * 1024 * 1024;
            // 云主机配置信息
            VirtualMachineConfigSpec configSpec = new VirtualMachineConfigSpec();
            VirtualDevice[] devices = machine.getConfig().getHardware().getDevice();
            VirtualDeviceConfigSpec diskSpec = null;
            for (VirtualDevice device : devices) {
                if (device instanceof VirtualDisk) {
                    VirtualDisk virtualDisk = (VirtualDisk) device;
                    if (virtualDisk.getDeviceInfo().getLabel().equals(diskLabel)) {
                        diskSpec = expandDiskSpec(null, virtualDisk, virtualDisk.getCapacityInKB() + extendSizeKb);
                        break;
                    }
                }
            }
            Assert.notNull(diskSpec, "该磁盘设备不支持扩容");
            VirtualDeviceConfigSpec[] deviceChange = new VirtualDeviceConfigSpec[]{diskSpec};
            configSpec.setDeviceChange(deviceChange);
            Task task = machine.reconfigVM_Task(configSpec);
            if (!task.waitForTask().equals(Task.SUCCESS)) {
                return new BocloudResult(false, "该磁盘设备不支持扩容");
            }
            return new BocloudResult(true, "云主机磁盘扩容成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "云主机磁盘扩容失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult getMetric(String type) {
        try {
            Long start = System.currentTimeMillis();
            ManagedEntity[] vmEntities = new InventoryNavigator(this.getRootFolder()).searchManagedEntities(VM);
            List<VirtualMachine> machines = new ArrayList<>();
            if (null != vmEntities) {
                for (ManagedEntity vmEntity : vmEntities) {
                    VirtualMachine machine = (VirtualMachine) vmEntity;
                    VirtualMachineSummary summary = machine.getSummary();
                    if (null == summary || summary.getConfig().isTemplate()) {
                        continue;
                    }
                    machines.add(machine);
                }
            }
            int counterId = 0;
            PerformanceManager perfManager = this.getServiceInstance().getPerformanceManager();
            Map<String, Integer> counterMap = new HashMap<>();
            PerfCounterInfo[] counterInfos = perfManager.getPerfCounter();
            for (PerfCounterInfo perfCounterInfo : counterInfos) {
                String key = perfCounterInfo.getGroupInfo().getKey() + "." + perfCounterInfo.getNameInfo().getKey()
                        + "." + perfCounterInfo.getRollupType().toString().toUpperCase();
                counterMap.put(key, perfCounterInfo.getKey());
            }
            if ("CPU".equals(type)) {
                counterId = counterMap.get("cpu.usage.AVERAGE");
            } else if ("MEM".equals(type)) {
                counterId = counterMap.get("mem.usage.AVERAGE");
            }
            List<JSONObject> valueObjs = new ArrayList<>();
            for (VirtualMachine machine : machines) {
                List<PerfMetricId> perfMetricIds = new ArrayList<>();
                PerfMetricId[] queryAvailablePerfMetric = perfManager.queryAvailablePerfMetric(machine, null, null, 20);
                if (null != queryAvailablePerfMetric) {
                    for (PerfMetricId perfMetricId : queryAvailablePerfMetric) {
                        if (counterId == perfMetricId.getCounterId()) {
                            perfMetricIds.add(perfMetricId);
                        }
                    }
                }
                PerfMetricId[] pmis = perfMetricIds.toArray(new PerfMetricId[perfMetricIds.size()]);
                PerfQuerySpec qSpec = new PerfQuerySpec();
                qSpec.setEntity(machine.getMOR());
                qSpec.setMetricId(pmis);
                qSpec.setIntervalId(20);
                PerfEntityMetricBase[] perEntityMetrics = perfManager.queryPerf(new PerfQuerySpec[]{qSpec});
                if (null == perEntityMetrics) {
                    continue;
                }
                long value = 0L;
                for (PerfEntityMetricBase perfEntityMetricBase : perEntityMetrics) {
                    PerfEntityMetric pem = (PerfEntityMetric) perfEntityMetricBase;
                    PerfMetricSeries[] mertricSeries = pem.getValue();
                    if (null != mertricSeries) {
                        PerfMetricSeries perfMetricSeries = mertricSeries[0];
                        PerfMetricIntSeries metricIntSeries = (PerfMetricIntSeries) perfMetricSeries;
                        long[] values = metricIntSeries.getValue();
                        if (null != values) {
                            value = values[values.length - 1];
                            break;
                        }
                    }
                }
                JSONObject object = new JSONObject();
                object.put("name", machine.getName());
                object.put("value",
                        BigDecimal.valueOf(value).divide(new BigDecimal(100), 2, RoundingMode.HALF_UP).doubleValue());
                valueObjs.add(object);
            }
            Long end = System.currentTimeMillis();
            logger.info("Get metric cost：{}", end - start);
            return new BocloudResult(true, valueObjs, "获取云主机性能数据成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "获取云主机性能数据失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult vmrcConsole(String instanceId) {
        VirtualMachine machine = null;
        try {
            ManagedEntity entity = this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true, true);
            if (null == entity) {
                return new BocloudResult(false, "云主机不存在");
            }
            machine = (VirtualMachine) entity;
            String vmrcUrl = "vmrc://clone:" + this.getServiceInstance().getSessionManager().acquireCloneTicket() + "@"
                    + this.getEndpoint() + "/?moid=" + machine.getMOR().getVal();
            return new BocloudResult(true, vmrcUrl.trim(), "成功获取到控制台URL");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "获取到控制台URL失败");
        }
    }

    @Override
    public BocloudResult vmToolsConfig(ServerModel model) {
        try {
            ManagedEntity entity = this.getServiceInstance().getSearchIndex().findByUuid(null, model.getInstanceId(),
                    true, true);
            if (null == entity) {
                return new BocloudResult(false, "云主机不存在，VMware tools配置失败");
            }
            VmwareToolsConfig toolsConfig = model.getVmwareToolsConfig();
            if (null == toolsConfig) {
                return new BocloudResult(false, "参数错误：VMware tools相关设置为空");
            }
            VirtualMachine vm = (VirtualMachine) entity;
            VirtualMachinePowerState powerState = vm.getRuntime().getPowerState();// 云主机运行状态
            VirtualMachineConfigSpec vmConfigSpec = new VirtualMachineConfigSpec();
            ToolsConfigInfo tools = new ToolsConfigInfo();
            if (powerState.name().equals(VirtualMachinePowerState.poweredOff.name())) {
                tools.setAfterPowerOn(toolsConfig.getAfterPowerOn());// 打开电源后
                tools.setAfterResume(toolsConfig.getAfterResume());// 恢复后
                tools.setBeforeGuestStandby(toolsConfig.getBeforeGuestStandby());// 挂起前
                tools.setBeforeGuestShutdown(toolsConfig.getBeforeGuestShutdown());// 关闭客户机之前
                // 电源控制
                VirtualMachineDefaultPowerOpInfo powerOpInfo = new VirtualMachineDefaultPowerOpInfo();
                powerOpInfo.setPowerOffType(toolsConfig.getPowerOffType());
                powerOpInfo.setSuspendType(toolsConfig.getSuspendType());
                powerOpInfo.setResetType(toolsConfig.getResetType());
                vmConfigSpec.setPowerOpInfo(powerOpInfo);
            }
            // 高级设置
            tools.setToolsUpgradePolicy(toolsConfig.getToolsUpgradePolicy());// 在关闭再打开过程中是否检查并升级工具
            tools.setSyncTimeWithHost(toolsConfig.getSyncTimeWithHost());// 同步客户机时间与主机时间
            vmConfigSpec.setTools(tools);
            // 重新配置云主机
            Task task = vm.reconfigVM_Task(vmConfigSpec);
            if (task.waitForTask().equals(Task.SUCCESS)) {
                entity = this.getServiceInstance().getSearchIndex().findByUuid(null, model.getInstanceId(), true, true);
                if (null == entity) {
                    return new BocloudResult(false, "云主机不存在，VMware tools配置失败");
                }
                vm = (VirtualMachine) entity;
                return new BocloudResult(true, AssembleTools.getInstance().convertServer(vm), "VMware tools配置成功");
            } else {
                return new BocloudResult(false,
                        "VMware tools配置失败：" + task.getTaskInfo().getError().getLocalizedMessage());
            }
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "VMware tools配置失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult getVmIps(String instanceId, String address) {
        try {
            VirtualMachine machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null,
                    instanceId, true, true);
            if (null == machine) {
                logger.error("云主机或模板{}不存在！", instanceId);
                return new BocloudResult(false, "云主机或模板不存在");
            }
            // vmware tools安装状态
            VirtualMachineToolsStatus toolsStatus = machine.getGuest().getToolsStatus();
            if (null == toolsStatus || VirtualMachineToolsStatus.toolsNotInstalled == toolsStatus) {
                logger.info("【" + machine.getName() + "】该云主机未安装VMware tools，获取IP地址失败......");
                return new BocloudResult(false, "该云主机未安装VMware tools，获取IP地址失败");
            }
            logger.info("【" + machine.getName() + "】VMware tools正在开启中...");
            boolean flag = true;
            Long start = System.currentTimeMillis();
            while (flag) {
                Thread.sleep(10 * 1000L);
                // 设置等待超时时间为20分钟
                if (System.currentTimeMillis() - start > 20 * 1000 * 60) {
                    flag = false;
                    break;
                }
                machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true,
                        true);
                // vmware tools未运行
                if (VirtualMachineToolsStatus.toolsNotRunning == machine.getGuest().getToolsStatus()) {
                    logger.info("【" + machine.getName() + "】VMware tools还未运行成功，IP获取等待中...");
                    Thread.sleep(20 * 1000L);
                    continue;
                }
                // 如果vmware tools启动成功，则判断IP是否已经获取到，如果未获取到继续轮询
                GuestNicInfo[] nicInfos = machine.getGuest().getNet();
                if (null == nicInfos || nicInfos.length == 0) {
                    // 国金证券处理欧拉操作系统
                    if (address == null) {
                        logger.info("当前虚拟机操作系统为欧拉，主机名称：" + machine.getName());
                        break;
                    }
                    logger.info("【" + machine.getName() + "】VMware tools启动成功，IP还未获取到...");
                    Thread.sleep(20 * 1000L);
                    continue;
                }
                // ip地址中是否含有169开头IP，因为此IP是windows自动获取的IP，需要等待vmware tools自定义完成
                Boolean hasDhcp169 = false;
                // 是否含有ip v4地址
                Boolean hasIpv4 = false;
                List<String> ipv4List = new ArrayList<>();
                // 云主机网卡信息
                for (GuestNicInfo nicInfo : nicInfos) {
                    NetIpConfigInfo ipConfig = nicInfo.getIpConfig();
                    if (null != ipConfig && null != ipConfig.getIpAddress() && ipConfig.getIpAddress().length > 0) {
                        for (NetIpConfigInfoIpAddress ipAddress : nicInfo.getIpConfig().getIpAddress()) {
                            if (IPAddress.ipv4Validate(ipAddress.getIpAddress())) {
                                if (ipAddress.getIpAddress().startsWith("169.")) {
                                    hasDhcp169 = true;
                                }else{
                                    hasIpv4 = true;
                                }
                                ipv4List.add(ipAddress.getIpAddress());
                            }
                        }
                    }
                }
                if (hasIpv4 && !hasDhcp169 && ipv4List.contains(address)) {
                    logger.info(
                            "【" + machine.getName() + "】VMware tools启动成功，IP获取完毕：" + JSONArray.toJSONString(ipv4List));
                    flag = false;
                    break;
                }
            }
            machine = (VirtualMachine) this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true,
                    true);
            if (null == machine) {
                logger.error("云主机或模板{}不存在！", instanceId);
                return new BocloudResult(false, "云主机或模板不存在");
            }
            logger.info("【" + machine.getName() + "】获取IP结束.....");
            return new BocloudResult(true, AssembleTools.getInstance().convertServer(machine), "获取云主机IP地址成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "获取云主机IP地址失败");
        }
    }

    @Override
    public BocloudResult getVmFolder(String vmFolder, String vdcName) {
        try {
            ManagedEntity datacenter = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, vdcName);
            Assert.notNull(datacenter, "数据中心不存在！");
            Datacenter center = (Datacenter) datacenter;
            Folder rootFolder = center.getVmFolder();
            ManagedEntity[] entities = Optional.ofNullable(rootFolder.getChildEntity()).orElse(new ManagedEntity[0]);
            boolean absent = Arrays.asList(entities).stream().filter(entity -> (entity instanceof Folder))
                    .map(folder -> (Folder) folder).noneMatch(folder -> folder.getName().equalsIgnoreCase(vmFolder));
            if (absent) {
                Folder folder = rootFolder.createFolder(vmFolder);
                Assert.notNull(folder, "创建云主机文件夹【" + vmFolder + "】失败");
            }
            return new BocloudResult(true, vmFolder, "获取云主机文件夹成功");
        } catch (Exception e) {
            logger.error("Get vm folder {} exception:{}", vmFolder, e.getMessage());
            return new BocloudResult(false, "获取云主机文件夹失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult getVmFolder(String parentFolder, String vmFolder, String vdcName) {
        try {
            ManagedEntity datacenter = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, vdcName);
            Assert.notNull(datacenter, "数据中心不存在！");
            Datacenter center = (Datacenter) datacenter;
            Folder rootFolder = center.getVmFolder();
            Map<String, Folder> vmFolderMap = new HashMap<>();
            String key = "";
            this.getVmFolder(vmFolderMap, rootFolder, vmFolder, parentFolder, key);
            if (vmFolder.contains(",")) {
                vmFolder = vmFolder.split(",")[vmFolder.split(",").length - 1];
            }
            if (vmFolderMap.isEmpty()) {
                // 查询父级文件夹
                ManagedEntity[] entities = Optional.ofNullable(rootFolder.getChildEntity())
                        .orElse(new ManagedEntity[0]);
                Folder pFolder = Arrays.asList(entities).stream().filter(entity -> (entity instanceof Folder))
                        .map(folder -> (Folder) folder)
                        .filter(folder -> folder.getName().equalsIgnoreCase(parentFolder)).findFirst().orElse(null);
                if (null == pFolder) {
                    pFolder = rootFolder.createFolder(parentFolder);
                    Assert.notNull(pFolder, "创建云主机文件夹【" + parentFolder + "】失败");
                }
                Folder folder = pFolder.createFolder(vmFolder);
                Assert.notNull(folder, "创建云主机文件夹【" + vmFolder + "】失败");
            }
            return new BocloudResult(true, vmFolder, "获取云主机文件夹成功");
        } catch (Exception e) {
            logger.error("Get vm folder {} exception:{}", vmFolder, e.getMessage());
            return new BocloudResult(false, "获取云主机文件夹失败：" + e.getMessage());
        }
    }

    /**
     * 获取vm创建文件夹
     * 存在多级目录“，”进行分割
     *
     * @param key         vmFolderMap对应最后的key
     * @param vmFolderMap 最终取到需要创建的Folder
     * @param rootFolder  当前层级的上一级Folder对象
     * @param folderName  子文件夹
     * @param parentName  当前文件夹
     * @throws Exception
     */
    private void getVmFolder(Map<String, Folder> vmFolderMap, Folder rootFolder, String folderName, String parentName, String key)
            throws Exception {
        if (rootFolder.getName().equals(folderName) && rootFolder.getParent().getName().equalsIgnoreCase(parentName)) {
            vmFolderMap.put(key, rootFolder);
            return;
        }
        ManagedEntity[] entities = rootFolder.getChildEntity();

        if (null != entities && entities.length > 0) {
            for (ManagedEntity entity : entities) {
                if (entity instanceof Folder) {
                    Folder childFolder = (Folder) entity;
                    //rootFolder中子文件夹名称有与需求parentName
                    if (childFolder.getName().equals(parentName)) {
                        logger.info("当前用户文件夹名称1：" + childFolder.getName());
                        //子文件夹中包含“，”，说明没有到最后层级
                        if (folderName.contains(",")) {
                            String parentVmFolder = folderName.substring(0, folderName.indexOf(","));
                            String vmFolder = folderName.substring(folderName.indexOf(",") + 1);
                            if (!key.isEmpty()) {
                                key = key + "," + parentName;
                            } else {
                                key = parentName;
                            }
                            this.getVmFolder(vmFolderMap, childFolder, vmFolder, parentVmFolder, key);
                        } else {//最后层级，但是层级里面有文件夹

                            if (childFolder.getName().equals(folderName)
                                    && childFolder.getParent().getName().equalsIgnoreCase(parentName)) {
                                logger.info("当前用户文件夹名称2：" + childFolder.getName());
                                key = key + "," + folderName;
                                vmFolderMap.put(key, childFolder);
                                return;
                            }
                            if (key.isEmpty()) {
                                key = parentName;
                            } else {
                                key = key + "," + parentName;
                            }
                            //childFolder在parentName的上级,需要再次进入
                            this.getVmFolder(vmFolderMap, childFolder, folderName, parentName, key);
                        }
                        //最后一级，rootFolder在parentName级
                    } else if (childFolder.getName().equals(folderName)
                            && childFolder.getParent().getName().equalsIgnoreCase(parentName)) {
                        logger.info("当前用户文件夹名称3：" + childFolder.getName());
                        key = key + "," + folderName;
                        vmFolderMap.put(key, childFolder);
                        return;
                    }
                }
            }
        }
    }

    /**
     * 获取vm创建文件夹
     *
     * @param vmFolderMap
     * @param rootFolder
     * @param folderName
     * @throws Exception
     */
    private void getVmFolder(Map<String, Folder> vmFolderMap, Folder rootFolder, String folderName) throws Exception {
        if (rootFolder.getName().equals(folderName)) {
            vmFolderMap.put(folderName, rootFolder);
            return;
        }
        ManagedEntity[] entities = rootFolder.getChildEntity();
        if (null == entities) {
            return;
        }
        for (ManagedEntity entity : entities) {
            if (entity instanceof Folder) {
                Folder childFolder = (Folder) entity;
                if (childFolder.getName().equals(folderName)) {
                    vmFolderMap.put(folderName, childFolder);
                    break;
                }
                this.getVmFolder(vmFolderMap, childFolder, folderName);
            }
        }
    }

    @Override
    public BocloudResult createWithoutPowerOff(ServerModel server) {
        try {
            ManagedEntity datacenter = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    server.getVdcName());
            Assert.notNull(datacenter, "数据中心不存在！");
            Datacenter center = (Datacenter) datacenter;
            InventoryNavigator navigator = new InventoryNavigator(center);
            VirtualMachine template = (VirtualMachine) navigator.searchManagedEntity(VM, server.getTemplate());
            if (null == template) {
                ManagedEntity managedEntity = this.getServiceInstance().getSearchIndex().findByUuid(null,
                        server.getTempInstId(), true, true);
                if (null != managedEntity) {
                    template = (VirtualMachine) managedEntity;
                }
            }
            Assert.notNull(template, "创建云主机失败：云主机模板不存在！");
            ComputeResource computeResource = null;
            if (StringUtils.isEmpty(server.getClusterName())) {
                computeResource = (ComputeResource) navigator.searchManagedEntity(CR, server.getHostName());
            } else {
                computeResource = (ComputeResource) navigator.searchManagedEntity(CR, server.getClusterName());
            }
            HostSystem hostSystem = (HostSystem) navigator.searchManagedEntity(HS, server.getHostName());
            Assert.notNull(hostSystem, "创建云主机失败：宿主机不存在！");
            VirtualMachine tempVm = (VirtualMachine) navigator.searchManagedEntity(VM, server.getName());
            Assert.notNull(tempVm, "创建云主机失败：名称已经存在！");
            Datastore datastore = (Datastore) navigator.searchManagedEntity(DT, server.getDatastore());
            if (null == datastore) {
                logger.warn("共享数据存储不存在！采用本地存储");
                Datastore[] datastores = hostSystem.getDatastores();
                Assert.notEmpty(datastores, "创建云主机失败：本地存储不存在！");
                datastore = Arrays.stream(datastores)
                        .filter(store -> store.getSummary().getName().equalsIgnoreCase(server.getDatastore()))
                        .findFirst().orElse(null);
            }
            Assert.notNull(datastore, "创建云主机失败：数据存储不存在！");
            VirtualMachineRelocateSpec relocateSpec = new VirtualMachineRelocateSpec();
            if (null != computeResource.getResourcePool()) {
                relocateSpec.setPool(computeResource.getResourcePool().getMOR());
            }
            relocateSpec.setDatastore(datastore.getMOR());
            relocateSpec.setHost(hostSystem.getMOR());
            VirtualMachineConfigSpec configSpec = new VirtualMachineConfigSpec();
            configSpec.setCpuHotAddEnabled(true);
            configSpec.setCpuHotRemoveEnabled(true);
            configSpec.setMemoryHotAddEnabled(true);
            ResourceAllocationInfo memoryAllocation = new ResourceAllocationInfo();
            memoryAllocation.setLimit(-1L);
            configSpec.setMemoryAllocation(memoryAllocation);
            configSpec.setName(server.getName());
            configSpec.setNumCPUs(server.getCpu());
            configSpec.setMemoryMB(server.getMemory().multiply(BigDecimal.valueOf(1024)).longValue());
            if (!StringUtils.isEmpty(server.getRemark())) {
                configSpec.setAnnotation(server.getRemark());
            }
            // 云主机克隆配置
            VirtualMachineCloneSpec cloneSpec = new VirtualMachineCloneSpec();
            cloneSpec.setLocation(relocateSpec);
            cloneSpec.setPowerOn(false);
            cloneSpec.setTemplate(false);
            cloneSpec.setConfig(configSpec);
            Task task = template.cloneVM_Task(center.getVmFolder(), server.getName(), cloneSpec);
            logger.info("开始执行创建云主机{}的任务", server.getName());
            if (!task.waitForTask().equals(Task.SUCCESS)) {
                return new BocloudResult(false, server, "创建失败：" + task.getTaskInfo().getError().getLocalizedMessage());
            }
            logger.info("基于模板克隆云主机的任务执行成功！");
            VirtualMachine machine = (VirtualMachine) new InventoryNavigator(this.getRootFolder())
                    .searchManagedEntity(VM, server.getName());
            if (null == machine) {
                return new BocloudResult(false, "创建失败：云主机不存在");
            }
            ServerModel model = AssembleTools.getInstance().convertServer(machine);
            logger.info("云主机{}创建成功", model.getName());
            return new BocloudResult(true, model, "创建云主机成功！");
        } catch (Exception e) {
            logger.error("Create Server By Template exception：{}", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult customize(ServerModel server) {
        try {
            InventoryNavigator navigator = new InventoryNavigator(this.getRootFolder());
            ManagedEntity datacenter = navigator.searchManagedEntity(DC, server.getVdcName());
            Assert.notNull(datacenter, "数据中心不存在！");
            Datacenter center = (Datacenter) datacenter;
            String instanceId = server.getInstanceId();
            ManagedEntity entity = this.getServiceInstance().getSearchIndex().findByUuid(null, instanceId, true, true);
            Assert.notNull(entity, "云主机不存在！");
            VirtualMachine machine = (VirtualMachine) entity;
            VirtualMachinePowerState state = machine.getSummary().getRuntime().getPowerState();
            Task task = null;
            // 云主机关机之后进行配置
            if (VirtualMachinePowerState.poweredOn == state) {
                task = machine.powerOffVM_Task();
                if (!task.waitForTask().equals(Task.SUCCESS)) {
                    String failInfo = "";
                    if (null != task.getTaskInfo().getError()) {
                        failInfo = task.getTaskInfo().getError().getLocalizedMessage();
                    }
                    logger.info("云主机{}关机失败！{}", machine.getName(), failInfo);
                    return new BocloudResult(false, "云主机关机失败：" + failInfo);
                }
            }
            // 由于模板系统类别有可能获取不到，暂时先默认获取不到时按照Linux的处理了
            String osCategory = machine.getGuest().getGuestFamily();
            if (StringUtils.isEmpty(osCategory)) {
                osCategory = machine.getConfig().getGuestFullName();
                if (StringUtils.isEmpty(osCategory)) {
                    osCategory = OSCategory.LINUX.name();
                } else if (osCategory.toLowerCase().contains(OSCategory.WINDOWS.name().toLowerCase())) {
                    osCategory = OSCategory.WINDOWS.name();
                } else {
                    osCategory = OSCategory.LINUX.name();
                }
            }
            if (VirtualMachineGuestOsFamily.otherGuestFamily.name().equals(osCategory)
                    && (null != machine.getGuest().getGuestFullName())) {
                osCategory = OSCategory.LINUX.name();
            }
            CustomizationSpec spec = createCustomizationSpec(server, osCategory);
            List<VirtualDeviceConfigSpec> specs = new ArrayList<>();
            // 处理系统盘和数据盘
            boolean systemDiskFind = false;
            VirtualMachineConfigInfo config = machine.getConfig();
            if (null != config) {
                // 云主机关联设备
                VirtualDevice[] devices = config.getHardware().getDevice();
                for (int i = 0; devices != null && i < devices.length; i++) {
                    // 按照底层vcenter上磁盘顺序，找到的第一块磁盘设备即为系统盘
                    VirtualDevice device = devices[i];
                    if (device instanceof VirtualDisk) {
                        VirtualDisk disk = (VirtualDisk) device;
                        if (systemDiskFind) {
                            // 移除掉多余的磁盘设备
                            specs.add(removeDiskSpec(disk));
                            continue;
                        }
                        if (null != server.getDisk()) {
                            BigDecimal diskSize = new BigDecimal(disk.getCapacityInKB())
                                    .divide(new BigDecimal(1024 * 1024L), 2, RoundingMode.HALF_UP);
                            if (server.getDisk() < diskSize.intValue()) {
                                return new BocloudResult(false, "指定的系统盘大小不能小于原模板系统盘大小");
                            }
                            // 编辑系统盘大小
                            specs.add(expandDiskSpec(server.getDiskType(), disk, server.getDisk() * 1024 * 1024L));
                            systemDiskFind = true;
                        }
                    }
                }
            }
            // 配置云主机磁盘设备
            List<Integer> disks = server.getDisks();
            List<String> dataStoreNames = server.getDataStoreNames();
            if (!ListTool.isEmpty(disks)) {
                logger.info("开始配置云主机磁盘设备");
                for (int i = 0; i < disks.size(); i++) {
                    if (dataStoreNames != null && dataStoreNames.size() > 0) {
                        Datastore d = (Datastore) navigator.searchManagedEntity(DT, dataStoreNames.get(i));
                        logger.info("mutli datastores name：" + d.getName());
                        specs.add(addDiskSpec(server.getProfileId(), server.getDiskType(), d.getName(),
                                disks.get(i) * 1024 * 1024L, "persistent", machine, i));
                    }
                }
                logger.info("配置云主机磁盘设备结束");
            }
            // 处理网卡相关连接网络
            processNetSpec(navigator, server, specs, machine);
            // 云主机配置信息
            VirtualMachineConfigSpec configSpec = new VirtualMachineConfigSpec();
            configSpec.setNumCPUs(server.getCpu());
            configSpec.setMemoryMB(server.getMemory().multiply(BigDecimal.valueOf(1024)).longValue());
            // 新克隆云主机相关设备
            if (!ListTool.isEmpty(specs)) {
                VirtualDeviceConfigSpec[] specArray = new VirtualDeviceConfigSpec[specs.size()];
                configSpec.setDeviceChange(specs.toArray(specArray));
            }
            Long customTime = System.currentTimeMillis();
            task = machine.customizeVM_Task(spec);
            logger.info("云主机{}自定义配置任务开始执行", machine.getName());
            if (task.waitForTask().equals(Task.SUCCESS)) {
                System.out.println("自定义配置耗时:" + (System.currentTimeMillis() - customTime));
                logger.info("云主机{}自定义配置任务执行成功！", machine.getName());
                // 重新配置云主机
                Long reconfigTime = System.currentTimeMillis();
                task = machine.reconfigVM_Task(configSpec);
                if (!task.waitForTask().equals(Task.SUCCESS)) {
                    String failInfo = "";
                    if (null != task.getTaskInfo().getError()) {
                        failInfo = task.getTaskInfo().getError().getLocalizedMessage();
                    }
                    logger.info("云主机{}重新配置任务执行失败！{}", machine.getName(), failInfo);
                    return new BocloudResult(false, "云主机重新配置失败：" + failInfo);
                }
                System.out.println("重新配置云主机耗时:" + (System.currentTimeMillis() - reconfigTime));
                logger.info("云主机{}重新配置任务执行成功！", machine.getName());
                machine = (VirtualMachine) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(VM,
                        machine.getName());
                String vmFolder = server.getVmFolder();
                if (!StringUtils.isEmpty(vmFolder)) {
                    Folder folder = center.getVmFolder();
                    Map<String, Folder> vmFolderMap = new ConcurrentHashMap<>();
                    // 云主机创建到指定目录下
                    this.getVmFolder(vmFolderMap, center.getVmFolder(), vmFolder);
                    if (!vmFolderMap.isEmpty()) {
                        folder = vmFolderMap.get(server.getVmFolder());
                    }
                    if (null != folder) {
                        ManagedEntity[] entities = new ManagedEntity[]{machine};
                        Long moveTime = System.currentTimeMillis();
                        task = folder.moveIntoFolder_Task(entities);
                        if (!task.waitForTask().equals(Task.SUCCESS)) {
                            String failInfo = "";
                            if (null != task.getTaskInfo().getError()) {
                                failInfo = task.getTaskInfo().getError().getLocalizedMessage();
                            }
                            logger.info("移动云主机{}到指定文件夹任务执行失败！{}", machine.getName(), failInfo);
                            return new BocloudResult(false, "移动云主机到指定文件夹失败：" + failInfo);
                        }
                        System.out.println("移动云主机到文件夹耗时:" + (System.currentTimeMillis() - moveTime));
                    }
                }
                // 开机
                Long powerOnTime = System.currentTimeMillis();
                task = machine.powerOnVM_Task(null);
                if (!task.waitForTask().equals(Task.SUCCESS)) {
                    String failInfo = "";
                    if (null != task.getTaskInfo().getError()) {
                        failInfo = task.getTaskInfo().getError().getLocalizedMessage();
                    }
                    logger.info("云主机{}自定义配置任务执行失败！{}", instanceId, failInfo);
                    return new BocloudResult(false, "云主机自定义配置失败：" + failInfo);
                }
                System.out.println("云主机开机耗时:" + (System.currentTimeMillis() - powerOnTime));
                Long ipTime = System.currentTimeMillis();
                BocloudResult ipResult = this.getVmIps(instanceId);
                if (ipResult.isSuccess()) {
                    System.out.println("云主机获取IP耗时:" + (System.currentTimeMillis() - ipTime));
                }
                return ipResult;
            } else {
                String failInfo = "";
                if (null != task.getTaskInfo().getError()) {
                    failInfo = task.getTaskInfo().getError().getLocalizedMessage();
                }
                logger.info("云主机{}自定义配置任务执行失败！{}", instanceId, failInfo);
                return new BocloudResult(false, "云主机自定义配置失败：" + failInfo);
            }
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new BocloudResult(false, "云主机自定义配置失败！");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult migrate(OSMigrateModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult softReboot(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult joinSGroup(String instanceId, String group) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult leaveSGroup(String instanceId, String group) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult vncConsole(String vncType, String serverId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult attachNetwork(ServerModel server) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult detachNetwork(String instanceId, String portUuid) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult confirmResize(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult revertResize(String instanceId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult migrate(H3cMigrateModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult create(H3cServerModel server) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getJnlpXml(Long vmId, String address) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult resetInstance(ServerModel serverModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult bindKeypair(ServerModel serverModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult unbindKeypair(ServerModel serverModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult cloneAsTemplate(ServerModel server) {
        try {
            ManagedEntity datacenter = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    server.getVdcName());
            Assert.notNull(datacenter, "数据中心不存在！");
            Datacenter center = (Datacenter) datacenter;
            InventoryNavigator navigator = new InventoryNavigator(center);
            VirtualMachine template = (VirtualMachine) navigator.searchManagedEntity(VM, server.getTemplate());
            if (null == template) {
                ManagedEntity managedEntity = this.getServiceInstance().getSearchIndex().findByUuid(null,
                        server.getTempInstId(), true, true);
                if (null != managedEntity) {
                    template = (VirtualMachine) managedEntity;
                }
            }
            Assert.notNull(template, "克隆云主机模板失败：所选云主机不存在！");
            ComputeResource computeResource = null;
            if (StringUtils.isEmpty(server.getClusterName())) {
                computeResource = (ComputeResource) navigator.searchManagedEntity(CR, server.getHostName());
            } else {
                computeResource = (ComputeResource) navigator.searchManagedEntity(CR, server.getClusterName());
            }
            HostSystem hostSystem = (HostSystem) navigator.searchManagedEntity(HS, server.getHostName());
            Assert.notNull(hostSystem, "克隆云主机模板失败：宿主机不存在！");
            Datastore datastore = (Datastore) navigator.searchManagedEntity(DT, server.getDatastore());
            if (null == datastore) {
                logger.warn("共享数据存储不存在！采用本地存储");
                Datastore[] datastores = hostSystem.getDatastores();
                if (null == datastores || datastores.length <= 0) {
                    logger.error("本地存储不存在！");
                    return new BocloudResult(false, "创建云主机失败：本地存储不存在！");
                }
                for (Datastore store : datastores) {
                    if (store.getSummary().getName().equalsIgnoreCase(server.getDatastore())) {
                        datastore = store;
                        break;
                    }
                }
            }
            Assert.notNull(datastore, "克隆云主机模板失败：数据存储不存在！");
            VirtualMachineRelocateSpec relocateSpec = new VirtualMachineRelocateSpec();
            if (null != computeResource.getResourcePool()) {
                relocateSpec.setPool(computeResource.getResourcePool().getMOR());
            }
            relocateSpec.setDatastore(datastore.getMOR());
            relocateSpec.setHost(hostSystem.getMOR());
            VirtualMachineConfigSpec configSpec = new VirtualMachineConfigSpec();
            // 设置CPU核数
            configSpec.setNumCPUs(template.getSummary().getConfig().getNumCpu());
            // 设置内存大小
            configSpec.setMemoryMB(Long.valueOf(template.getSummary().getConfig().getMemorySizeMB()));
            // 设置云主机名称
            configSpec.setName(server.getName());
            // 云主机克隆配置
            VirtualMachineCloneSpec cloneSpec = new VirtualMachineCloneSpec();
            cloneSpec.setLocation(relocateSpec);
            cloneSpec.setPowerOn(false);
            cloneSpec.setTemplate(true);
            cloneSpec.setConfig(configSpec);
            Folder vmFolder = center.getVmFolder();
            if (!StringUtils.isEmpty(server.getVmFolder())) {
                Map<String, Folder> vmFolderMap = new ConcurrentHashMap<>();
                datacenter = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, server.getVdcName());
                if (null == datacenter) {
                    logger.error("数据中心不存在！");
                    return new BocloudResult(false, "数据中心不存在！");
                }
                center = (Datacenter) datacenter;
                // 创建到指定目录下
                this.getVmFolder(vmFolderMap, center.getVmFolder(), server.getVmFolder());
                if (!vmFolderMap.isEmpty()) {
                    vmFolder = vmFolderMap.get(server.getVmFolder());
                }
                if (null == vmFolder) {
                    vmFolder = center.getVmFolder();
                    logger.info("云主机将创建到数据中心云主机根目录下...");
                }
            }
            logger.info("创建基于云主机克隆模板的任务");
            Task task = template.cloneVM_Task(vmFolder, server.getName(), cloneSpec);
            logger.info("开始执行创建云主机模板{}的任务", server.getName());
            if (task.waitForTask().equals(Task.SUCCESS)) {
                logger.info("基于云主机克隆模板的任务执行成功！");
                VirtualMachine machine = (VirtualMachine) new InventoryNavigator(this.getRootFolder())
                        .searchManagedEntity(VM, server.getName());
                if (null == machine) {
                    return new BocloudResult(false, server, "克隆云主机模板失败：模板不存在");
                }
                ServerModel model = AssembleTools.getInstance().convertServer(machine);
                logger.info("云主机克隆模板{}创建成功", model.getName());
                return new BocloudResult(true, model, "克隆云主机模板成功！");
            } else {
                VirtualMachine machine = (VirtualMachine) new InventoryNavigator(this.getRootFolder())
                        .searchManagedEntity(VM, server.getName());
                logger.warn("克隆云主机模板{}失败{}！", server.getName(), task.getTaskInfo().getError().getLocalizedMessage());
                // 删除创建失败的云主机
                if (null != machine) {
                    machine.destroy_Task();
                }
                return new BocloudResult(false, server,
                        "克隆云主机模板失败：" + task.getTaskInfo().getError().getLocalizedMessage());
            }
        } catch (Exception e) {
            logger.error("Clone as Template exception:{}", e.getMessage());
            return new BocloudResult(false, "云主机克隆为模板发生错误：" + e.getMessage());
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult listByHost(String hostName) {
        logger.info("查询宿主机下{}下云主机", hostName);
        try {
            ManagedEntity managedEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS,
                    hostName);
            Assert.notNull(managedEntity, "宿主机" + hostName + "不存在！");
            HostSystem host = (HostSystem) managedEntity;
            VirtualMachine[] virtualMachines = host.getVms();
            List<ServerModel> servers = new ArrayList<>();
            if (null != virtualMachines) {
                for (VirtualMachine vm : virtualMachines) {
                    ServerModel model = SingleAssembleTools.getInstance().convertServer(vm);
                    if (null != model) {
                        servers.add(model);
                    }
                }
            }
            return new BocloudResult(true, servers, "宿主机下" + hostName + "云主机查询成功");
        } catch (Exception e) {
            logger.error("宿主机下" + hostName + "云主机查询失败:" + e);
            return new BocloudResult(false, "宿主机下" + hostName + "云主机查询失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

}
