package com.bocloud.cmp.provider.compute;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.cmp.provider.utils.AssembleTools;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.vmware.vim25.VirtualDevice;
import com.bocloud.vmware.vim25.VirtualDisk;
import com.bocloud.vmware.vim25.VirtualMachineConfigInfo;
import com.bocloud.vmware.vim25.VirtualMachineSnapshotTree;
import com.bocloud.vmware.vim25.mo.HostSystem;
import com.bocloud.vmware.vim25.mo.InventoryNavigator;
import com.bocloud.vmware.vim25.mo.ManagedEntity;
import com.bocloud.vmware.vim25.mo.Task;
import com.bocloud.vmware.vim25.mo.VirtualMachine;
import com.bocloud.vmware.vim25.mo.VirtualMachineSnapshot;

import io.jsonwebtoken.lang.Assert;

/**
 * vmware快照操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/14
 */
@Provider("bocloud.cmp.vmware")
public class VmwareSnapshotProvider extends VmwareProvider implements SnapshotProvider {

	private static Logger logger = LoggerFactory.getLogger(VmwareSnapshotProvider.class);

	public VmwareSnapshotProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * 删除快照解决思路：1：首先获取云主机各个快照树的最下面一层的子节点（本身不含子节点），存入Map中；
	 * 2.取得云主机所有的快照，如果快照在Map中，即表明该快照没有子节点了，不再获取该快照的子节点； 3.快照一一与传过来的值进行比对，若对应则删除
	 */
	@Override
	public BocloudResult remove(SnapshotModel model) {
		try {
			ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(VM,
					model.getVmName());
			Assert.notNull(entity, "快照所在云主机不存在！");
			VirtualMachine machine = (VirtualMachine) entity;
			// 存储没有子节点的快照（即最下面一层的快照）
			Map<String, String> latest = new HashMap<>();
			//
			Map<String, String> snapshotMap = new HashMap<>();
			// 获取云主机快照树（可能是多个）根节点
			VirtualMachineSnapshotTree[] trees = machine.getSnapshot().getRootSnapshotList();
			VirtualMachineSnapshot[] vmSnapshotArray = machine.getRootSnapshot();
			// 获取快照树最下面一层的快照
			traverseSnapshotTree(trees, latest, snapshotMap);
			boolean result = deleteSnapshot(vmSnapshotArray, latest, snapshotMap.get(model.getSnapshotUuid()));
			return result ? new BocloudResult(true, "删除成功") : new BocloudResult(false, "删除失败");
		} catch (Exception e) {
			logger.error("sync info error:", e);
			return new BocloudResult(false, "删除失败");
		} finally {
			this.getServiceInstance().getServerConnection().logout();
		}
	}

	/**
	 * 删除快照：Map中存放的是快照树最下面的一层快照（没有子节点的快照）
	 *
	 * @param snapshots
	 * @param latest
	 * @param value
	 * @return
	 * @throws Exception
	 */
	private boolean deleteSnapshot(VirtualMachineSnapshot[] snapshots, Map<String, String> latest, String value)
			throws Exception {
		if (null != snapshots) {
			for (VirtualMachineSnapshot snapshot : snapshots) {
				if (value.equals(snapshot.getMOR().getVal())) {
					Task task = snapshot.removeSnapshot_Task(false);
					return (task.waitForTask().equals(Task.SUCCESS)) ? true : false;
				} else {
					// 若latest（Map类型）不包含此快照的getVal值，即此快照含有子节点
					if (!latest.containsKey(snapshot.getMOR().getVal())) {
						// 快照含有子节点，继续遍历比对删除
						return deleteSnapshot(snapshot.getChildSnapshot(), latest, value);
					}
				}
			}
		}
		return false;
	}

	/**
	 * 云主机快照获取出来是一个树的形式，此方法取到快照树最下面一层的快照，即没有子节点的快照，存入Map中
	 *
	 * @param trees
	 * @param latest
	 */
	private void traverseSnapshotTree(VirtualMachineSnapshotTree[] trees, Map<String, String> latest,
			Map<String, String> snapshotMap) {
		if (null != trees) {
			for (VirtualMachineSnapshotTree tree : trees) {
				snapshotMap.put(tree.getId().toString(), tree.getSnapshot().getVal());
				if (null != tree.getChildSnapshotList()) {
					traverseSnapshotTree(tree.getChildSnapshotList(), latest, snapshotMap);
				} else {
					// 如果该快照不含子节点，存入latest中
					latest.put(tree.getSnapshot().getVal(), tree.getName());
				}
			}
		}

	}

	private Map<String, String> flatSnapshots(VirtualMachineSnapshotTree[] forest) {
		return null;
	}

	private VirtualMachineSnapshot detailSnapshot(VirtualMachineSnapshot[] snapshots, Map<String, String> latest,
			String target) {
		if (null != snapshots) {
			for (VirtualMachineSnapshot snapshot : snapshots) {
				if (target.equals(snapshot.getMOR().getVal())) {
					return snapshot;
				}
				if (latest.containsKey(snapshot.getMOR().getVal())) {
					continue;
				}
				return detailSnapshot(snapshot.getChildSnapshot(), latest, target);
			}
		}
		return null;
	}

	private VirtualMachineConfigInfo detailSnapshot(VirtualMachineSnapshot[] snapshots, String value) {
		if (null != snapshots) {
			for (VirtualMachineSnapshot snapshot : snapshots) {
				if (value.equals(snapshot.getMOR().getVal())) {
					return snapshot.getConfig();
				} else {
					// 快照含有子节点，继续遍历比对查找
					return detailSnapshot(snapshot.getChildSnapshot(), value);
				}
			}
		}
		return null;
	}

	@Override
	public BocloudResult detail(SnapshotModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult modify(SnapshotModel model) {
		try {
			ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(VM,
					model.getVmName());
			VirtualMachine machine = (VirtualMachine) entity;
			// 存储没有子节点的快照（即最下面一层的快照）
			Map<String, String> latest = new HashMap<>();
			Map<String, String> snapshotMap = new HashMap<>();
			// 获取云主机快照树（可能是多个）根节点
			VirtualMachineSnapshotTree[] trees = machine.getSnapshot().getRootSnapshotList();
			VirtualMachineSnapshot[] vmSnapshotArray = machine.getRootSnapshot();
			// 获取快照树最下面一层的快照
			traverseSnapshotTree(trees, latest, snapshotMap);
			VirtualMachineSnapshot snapshot = detailSnapshot(vmSnapshotArray, latest,
					snapshotMap.get(model.getSnapshotUuid()));
			if (null == snapshot) {
				return new BocloudResult(false, "快照不存在");
			}
			snapshot.renameSnapshot(model.getName(), model.getRemark());
			return new BocloudResult(true, "修改成功");
		} catch (Exception e) {
			logger.error("modify snapshot info error:", e);
			return new BocloudResult(false, "修改失败");
		} finally {
			this.getServiceInstance().getServerConnection().logout();
		}
	}

	@Override
	public BocloudResult revert(SnapshotModel model) {
		try {
			ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(VM,
					model.getVmName());
			if (null == entity) {
				return new BocloudResult(false, "快照所在云主机不存在");
			}
			VirtualMachine machine = (VirtualMachine) entity;
			// 存储没有子节点的快照（即最下面一层的快照）
			Map<String, String> lastest = new HashMap<>();
			Map<String, String> snapshotMap = new HashMap<>();
			// 获取云主机快照树（可能是多个）根节点
			VirtualMachineSnapshotTree[] trees = machine.getSnapshot().getRootSnapshotList();
			VirtualMachineSnapshot[] snapshots = machine.getRootSnapshot();
			// 获取快照树最下面一层的快照
			traverseSnapshotTree(trees, lastest, snapshotMap);
			// 获取宿主机
			ManagedEntity hostEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS,
					model.getHostName());
			boolean result = revertSnapshot(snapshots, lastest, snapshotMap.get(model.getSnapshotUuid()),
					(HostSystem) hostEntity);
			if (result) {
				entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(VM, model.getVmName());
				return new BocloudResult(true, AssembleTools.getInstance().convertServer((VirtualMachine) entity),
						"恢复成功");
			}
			return new BocloudResult(false, "恢复失败");
		} catch (Exception e) {
			logger.error("revert snapshot error:", e);
			return new BocloudResult(false, "恢复失败");
		} finally {
			this.getServiceInstance().getServerConnection().logout();
		}
	}

	/**
	 * 恢复快照
	 *
	 * @param snapshots
	 * @param latest
	 * @param value
	 * @param host
	 * @return
	 * @throws Exception
	 */
	private boolean revertSnapshot(VirtualMachineSnapshot[] snapshots, Map<String, String> latest, String value,
			HostSystem host) throws Exception {
		Assert.notNull(snapshots, "snapshot is null");
		Assert.notNull(value, "target snapshot is null");
		for (VirtualMachineSnapshot snapshot : snapshots) {
			if (value.equals(snapshot.getMOR().getVal())) {
				Task task = snapshot.revertToSnapshot_Task(host);
				return (task.waitForTask().equals(Task.SUCCESS)) ? true : false;
			} else {
				if (!latest.containsKey(snapshot.getMOR().getVal())) {
					boolean result = revertSnapshot(snapshot.getChildSnapshot(), latest, value, host);
					if (result) {
						return result;
					}
				}
			}
		}
		return false;
	}

	@Override
	public BocloudResult removeByVolumeId(String volumeId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult getSnapshot(String snapshotId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult getAvaliableSnap(String snapshotId) {
		return null;
	}

	/**
	 * 查询云主机下的快照
	 *
	 * @param cloudServerId
	 * @return
	 */
	@Override
	public BocloudResult list(Long cloudServerId) {
		return null;
	}

	@Override
	public BocloudResult detail(String value, String serverName) {
		try {
			ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(VM, serverName);
			Assert.notNull(entity, "快照所在云主机不存在");
			VirtualMachine machine = (VirtualMachine) entity;
			VirtualMachineSnapshot[] vmSnapshotArray = machine.getRootSnapshot();
			// 获取快照树最下面一层的快照
			VirtualMachineConfigInfo config = detailSnapshot(vmSnapshotArray, value);
			Assert.notNull(config, "无法获取到当前快照云主机配置");
			ServerModel model = new ServerModel();
			model.setCpu(config.getHardware().getNumCPU());
			model.setMemory(BigDecimal.valueOf(config.getHardware().getMemoryMB()).divide(BigDecimal.valueOf(1024), 2,
					BigDecimal.ROUND_HALF_UP));
			VirtualDevice[] devices = config.getHardware().getDevice();
			long capacity = Arrays.stream(devices).filter(device -> device instanceof VirtualDisk)
					.mapToLong(disk -> ((VirtualDisk) disk).getCapacityInKB()).sum();
			model.setDisk(new BigDecimal(capacity).divide(new BigDecimal(1024 * 1024L)).intValue());
			return new BocloudResult(true, model, "查询成功");
		} catch (Exception e) {
			logger.error("sync info error:", e);
			return new BocloudResult(false, "获取到当前快照云主机配置时发生错误");
		} finally {
			this.getServiceInstance().getServerConnection().logout();
		}
	}

}
