package com.bocloud.cmp.provider.contentlibrary;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.NetworkType;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.contentlibrary.ItemModel;
import com.bocloud.cmp.model.contentlibrary.LibraryModel;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.cmp.provider.utils.AssembleTools;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.bocloud.vmware.vim25.*;
import com.bocloud.vmware.vim25.mo.*;
import com.vmware.vcenter.ovf.DiskProvisioningType;
import com.vmware.vcenter.ovf.LibraryItemTypes;
import io.jsonwebtoken.lang.Assert;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * 内容库数据访问调整
 *
 * @author wangyu
 * @version 5.0.0-SNAPSHOT
 * @since 2021/2/26
 */
@Slf4j
@Provider("bocloud.cmp.vmware")
public class VmwareLibraryProvider extends VmwareProvider implements LibraryProvider {

    private static Logger logger = LoggerFactory.getLogger(VmwareLibraryProvider.class);

    public VmwareLibraryProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult<List<LibraryModel>> list() {
        List<String> libraries = this.getClsApiClient().libraryService().list();
        List<LibraryModel> libraryModels = new ArrayList<>();
        if (!ListTool.isEmpty(libraries)) {
            for (String library : libraries) {
                com.vmware.content.LibraryModel libraryModel = this.getClsApiClient().libraryService().get(library);
                if (null != libraryModel) {
                    libraryModels.add(convertor(libraryModel));
                }
            }
        }
        return new BocloudResult<>(true, libraryModels, "查询成功");
    }

    @Override
    public BocloudResult<List<ItemModel>> listItems(String libraryUuid) {
        List<String> items = this.getClsApiClient().itemService().list(libraryUuid);
        List<ItemModel> itemModels = new ArrayList<>();
        if (!ListTool.isEmpty(items)) {
            for (String item : items) {
                com.vmware.content.library.ItemModel itemModel = this.getClsApiClient().itemService().get(item);
                if (null != itemModel
                        && null != itemModel.getType()
                        && (itemModel.getType().equalsIgnoreCase("ovf") || itemModel.getType().equalsIgnoreCase("ova") || itemModel.getType().equalsIgnoreCase("vm-template"))) {
                    itemModels.add(convertorItem(itemModel));
                }
            }
        }
        return new BocloudResult<>(true, itemModels, "查询成功");
    }

    @Override
    public BocloudResult<ServerModel> ovfDeploy(ServerModel model) {
        try {
            com.vmware.content.library.ItemModel itemModel = this.getClsApiClient().itemService().get(model.getOvfItemId());
            Assert.notNull(itemModel, "OVF模板不存在");
            ManagedEntity datacenter = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    model.getVdcName());
            Assert.notNull(datacenter, "数据中心不存在！");
            Datacenter center = (Datacenter) datacenter;
            InventoryNavigator navigator = new InventoryNavigator(center);
            //云主机创建的文件夹
            String folderId = center.getVmFolder().getMOR().getVal();
            ComputeResource computeResource = null;
            if (StringUtils.isEmpty(model.getClusterName())) {
                computeResource = (ComputeResource) navigator.searchManagedEntity(CR, model.getHostName());
            } else {
                computeResource = (ComputeResource) navigator.searchManagedEntity(CR, model.getClusterName());
            }
            Assert.notNull(computeResource, "计算资源池不存在");
            String resourcePoolId = computeResource.getResourcePool().getMOR().getVal();
            //查询宿主机
            HostSystem hostSystem = (HostSystem) navigator.searchManagedEntity(HS, model.getHostName());
            Assert.notNull(hostSystem, "宿主机不存在");
            String hostId = hostSystem.getMOR().getVal();
            LibraryItemTypes.DeploymentTarget deploymentTarget = new LibraryItemTypes.DeploymentTarget();
            deploymentTarget.setResourcePoolId(resourcePoolId);
            deploymentTarget.setFolderId(folderId);
            deploymentTarget.setHostId(hostId);
            LibraryItemTypes.ResourcePoolDeploymentSpec deploymentSpec = new LibraryItemTypes.ResourcePoolDeploymentSpec();
            deploymentSpec.setName(model.getName());
            deploymentSpec.setAcceptAllEULA(true);
            Datastore datastore = (Datastore) navigator.searchManagedEntity(DT, model.getDatastore());
            Assert.notNull(datastore, "数据存储不存在");
            String datastoreId = datastore.getMOR().getVal();
            deploymentSpec.setDefaultDatastoreId(datastoreId);
            if (!StringUtils.isEmpty(model.getProfileId())) {
                deploymentSpec.setStorageProfileId(model.getProfileId());
                deploymentSpec.setStorageProvisioning(StringUtils.isEmpty(model.getDiskType()) ? DiskProvisioningType.thin : DiskProvisioningType.valueOf(model.getDiskType()));
            }
            deploymentSpec.setAnnotation(model.getRemark());
            log.info("开始OVF部署云主机任务...");
            LibraryItemTypes.DeploymentResult deploymentResult = this.getClsApiClient().ovfLibraryItemService().deploy(UUID.randomUUID().toString(), itemModel.getId(), deploymentTarget, deploymentSpec);
            log.info("OVF部署云主机任务执行结束");
            String message = "通过OVF模板创建云主机失败";
            if (!deploymentResult.getSucceeded()) {
                LibraryItemTypes.ResultInfo error = deploymentResult.getError();
                if (null != error) {
                    if (null != error.getErrors()) {
                        message = JSONObject.toJSONString(error.getErrors());
                    } else if (null != error.getWarnings()) {
                        message = JSONObject.toJSONString(error.getWarnings());
                    } else if (null != error.getInformation()) {
                        message = JSONObject.toJSONString(error.getInformation());
                    }
                }
                return BocloudResult.FAILED(message);
            }
            log.info("OVF部署成功的云主机ID:{}", deploymentResult.getResourceId().getId());
            VirtualMachine machine = (VirtualMachine) navigator.searchManagedEntity(VM, model.getName());
            if (null == machine) {
                return BocloudResult.FAILED(message);
            }
            String nicPortGroup = model.getNicPortGroup();
            String nicPortType = model.getNicPortType();
            if (!StringUtils.isEmpty(nicPortGroup)) {
                log.info("设置云主机网卡网络：{}", nicPortGroup);
                log.info("设置云主机网卡网络类型：{}", nicPortType);
                //设置网卡
                List<VirtualDeviceConfigSpec> specs = processNetSpec(center, machine, nicPortGroup, nicPortType);
                VirtualMachineConfigSpec configSpec = new VirtualMachineConfigSpec();
                VirtualDeviceConfigSpec[] specArray = new VirtualDeviceConfigSpec[specs.size()];
                configSpec.setDeviceChange(specs.toArray(specArray));
                Task task = machine.reconfigVM_Task(configSpec);
                log.info("开始执行云主机【{}】的网卡配置任务...", machine.getName());
                if (task.waitForTask().equals(Task.SUCCESS)) {
                    log.info("执行云主机【{}】的网卡配置任务成功", machine.getName());
                } else {
                    log.info("执行云主机【{}】的网卡配置任务失败:{}", machine.getName(), task.getTaskInfo().getError().getLocalizedMessage());
                }
            }
            return new BocloudResult<>(true, AssembleTools.getInstance().convertServer(machine), "通过OVF模板创建云主机成功");
        } catch (Exception e) {
            log.error("Get error message:{}", e);
            return new BocloudResult<>(false, e.getMessage());
        }
    }

    private List<VirtualDeviceConfigSpec> processNetSpec(Datacenter datacenter, VirtualMachine machine, String nicPortGroup, String nicPortType) throws Exception {
        List<VirtualDeviceConfigSpec> specs = new ArrayList<>();
        // 网络信息
        VirtualDevice[] devices = machine.getConfig().getHardware().getDevice();
        //找出网卡设备
        List<VirtualDevice> nicDevices = new ArrayList<>();
        for (VirtualDevice virtualDevice : devices) {
            if (virtualDevice instanceof VirtualEthernetCard) {
                nicDevices.add(virtualDevice);
            }
        }
        ManagedEntity[] networks = datacenter.getNetworkFolder().getChildEntity();
        Map<String, String> uuidMap = new HashMap<>();
        Map<String, String> portMap = new HashMap<>();
        Map<String, ManagedObjectReference> networkMap = new HashMap<>();
        Map<String, OpaqueNetwork> opaqueNetworkMap = new HashMap<>();
        for (ManagedEntity network : networks) {
            if (network instanceof OpaqueNetwork) {
                opaqueNetworkMap.put(network.getName(), (OpaqueNetwork) network);
            }
            if (network instanceof Network && !(network instanceof DistributedVirtualPortgroup)) {
                networkMap.put(network.getName(), network.getMOR());
            } else if (network instanceof VmwareDistributedVirtualSwitch) {
                String uuid = ((VmwareDistributedVirtualSwitch) network).getUuid();
                DistributedVirtualPortgroup[] portgroups = ((VmwareDistributedVirtualSwitch) network).getPortgroup();
                for (DistributedVirtualPortgroup portgroup : portgroups) {
                    portMap.put(portgroup.getName(), portgroup.getKey());
                    uuidMap.put(portgroup.getName(), uuid);
                }
            } else if (network instanceof Folder) {
                Folder childFolder = (Folder) network;
                getChildrenNetwork(childFolder, uuidMap, portMap, networkMap);
            }
        }
        //给每个网卡设置网络
        for (VirtualDevice nicDevice : nicDevices) {
            // 如果存在标准网络中，则编辑标准交换机，否则编辑分布式交换机
            if (networkMap.containsKey(nicPortGroup)) {
                specs.add(editNetSpec(null, nicPortGroup, nicDevice, NetworkType.SwitchType.VSWITCH.name(), opaqueNetworkMap));
            } else {
                String pgName = nicPortGroup;
                String switchUuid = uuidMap.get(pgName);
                String portKey = portMap.get(pgName);
                if (null == pgName) {
                    VirtualEthernetCard device = (VirtualEthernetCard) nicDevice;
                    // 查询分布式端口组名称
                    DistributedVirtualSwitchPortConnection portgroupConn = ((VirtualEthernetCardDistributedVirtualPortBackingInfo) (device
                            .getBacking())).getPort();
                    if (null != portgroupConn) {
                        switchUuid = portgroupConn.getSwitchUuid();
                        portKey = portgroupConn.getPortgroupKey();
                    }
                }
                specs.add(editNetSpec(switchUuid, portKey, nicDevice, NetworkType.SwitchType.DVSWITCH.name(), opaqueNetworkMap));
            }
        }
        return specs;
    }

    /**
     * 获取folder中的网络
     *
     * @param folder
     * @param uuidMap
     * @param portMap
     * @param networkMap
     * @throws Exception
     */
    private void getChildrenNetwork(Folder folder, Map<String, String> uuidMap, Map<String, String> portMap,
                                    Map<String, ManagedObjectReference> networkMap) throws Exception {
        ManagedEntity[] entities = folder.getChildEntity();
        if (null != entities && entities.length > 0) {
            for (ManagedEntity network : entities) {
                if (network instanceof Network && !(network instanceof DistributedVirtualPortgroup)) {
                    networkMap.put(network.getName(), network.getMOR());
                } else if (network instanceof VmwareDistributedVirtualSwitch) {
                    String uuid = ((VmwareDistributedVirtualSwitch) network).getUuid();
                    DistributedVirtualPortgroup[] portgroups = ((VmwareDistributedVirtualSwitch) network)
                            .getPortgroup();
                    for (DistributedVirtualPortgroup portgroup : portgroups) {
                        portMap.put(portgroup.getName(), portgroup.getKey());
                        uuidMap.put(portgroup.getName(), uuid);
                    }
                } else if (network instanceof Folder) {
                    Folder childFolder = (Folder) network;
                    getChildrenNetwork(childFolder, uuidMap, portMap, networkMap);
                }
            }
        }
    }

    /**
     * 编辑网卡
     *
     * @param uuid
     * @param key
     * @param device
     * @param category
     * @return
     */
    private VirtualDeviceConfigSpec editNetSpec(String uuid, String key, VirtualDevice device, String category,
                                                Map<String, OpaqueNetwork> opaqueNetworkMap) {
        VirtualDeviceConfigSpec netSpec = new VirtualDeviceConfigSpec();
        netSpec.setOperation(VirtualDeviceConfigSpecOperation.edit);
        if (null != opaqueNetworkMap && opaqueNetworkMap.containsKey(key)) {
            OpaqueNetwork opaqueNetwork = opaqueNetworkMap.get(key);
            VirtualEthernetCardOpaqueNetworkBackingInfo networkBackingInfo = new VirtualEthernetCardOpaqueNetworkBackingInfo();
            NetworkSummary summary = opaqueNetwork.getSummary();
            String opaqueNetworkType = null;
            String opaqueNetworkId = null;
            if (summary instanceof OpaqueNetworkSummary) {
                OpaqueNetworkSummary networkSummary = (OpaqueNetworkSummary) summary;
                opaqueNetworkType = networkSummary.getOpaqueNetworkType();
                opaqueNetworkId = networkSummary.getOpaqueNetworkId();
            }
            networkBackingInfo.setOpaqueNetworkId(opaqueNetworkId);
            networkBackingInfo.setOpaqueNetworkType(opaqueNetworkType);
            device.setBacking(networkBackingInfo);
        } else if ("VSWITCH".equals(category)) {
            VirtualEthernetCardNetworkBackingInfo netBackInfo = new VirtualEthernetCardNetworkBackingInfo();
            netBackInfo.setDeviceName(key);
            device.setBacking(netBackInfo);
        } else {
            DistributedVirtualSwitchPortConnection portCon = new DistributedVirtualSwitchPortConnection();
            portCon.setSwitchUuid(uuid);
            portCon.setPortgroupKey(key);
            VirtualEthernetCardDistributedVirtualPortBackingInfo portBackingInfo = new VirtualEthernetCardDistributedVirtualPortBackingInfo();
            portBackingInfo.setPort(portCon);
            device.setBacking(portBackingInfo);
        }
        VirtualDeviceConnectInfo connectable = new VirtualDeviceConnectInfo();
        connectable.setAllowGuestControl(true);
        connectable.setConnected(true);
        connectable.setStartConnected(true);
        connectable.setStatus("untried");
        device.setConnectable(connectable);
        netSpec.setDevice(device);
        return netSpec;
    }

    /**
     * 内容库数据转换
     *
     * @param library
     * @return
     */
    public LibraryModel convertor(com.vmware.content.LibraryModel library) {
        LibraryModel model = new LibraryModel();
        model.setCreationTime(null == library.getCreationTime() ? null : library.getCreationTime().getTime());
        model.setLastModifiedTime(null == library.getLastModifiedTime() ? null : library.getLastModifiedTime().getTime());
        model.setLibraryUuid(library.getId());
        model.setName(library.getName());
        model.setPublishInfo(null == library.getPublishInfo() ? null : JSONObject.toJSONString(library.getPublishInfo()));
        model.setRemark(library.getDescription());
        model.setServerGuid(library.getServerGuid());
        model.setStorageBackings(null == library.getStorageBackings() ? null : JSONObject.toJSONString(library.getStorageBackings()));
        model.setType(null == library.getType() ? null : JSONObject.toJSONString(library.getType()));
        model.setVersion(library.getVersion());
        return model;
    }

    /**
     * 内容库模板数据转换
     *
     * @param item
     * @return
     */
    public ItemModel convertorItem(com.vmware.content.library.ItemModel item) {
        ItemModel model = new ItemModel();
        model.setCached(item.getCached());
        model.setContentVersion(item.getContentVersion());
        model.setCreationTime(null == item.getCreationTime() ? null : item.getCreationTime().getTime());
        model.setItemUuid(item.getId());
        model.setLastModifiedTime(null == item.getLastModifiedTime() ? null : item.getLastModifiedTime().getTime());
        model.setLibraryUuid(item.getLibraryId());
        model.setMetadataVersion(item.getMetadataVersion());
        model.setName(item.getName());
        model.setRemark(item.getDescription());
        model.setSize(item.getSize());
        model.setType(item.getType());
        model.setVersion(item.getVersion());
        return model;
    }
}
