package com.bocloud.cmp.provider.monitor;

import com.bocloud.cmp.model.AlarmModel;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.vmware.vim25.*;
import com.bocloud.vmware.vim25.mo.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * vmware告警操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/14
 */
@Provider("bocloud.cmp.vmware")
public class VmwareAlarmProvider extends VmwareProvider implements AlarmProvider {

    private static Logger logger = LoggerFactory.getLogger(VmwareAlarmProvider.class);

    public VmwareAlarmProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult sync() {
        // 存放告警信息
        List<AlarmModel> alarmList = new ArrayList<>();
        Folder rootFolder = this.getRootFolder();
        AlarmState[] alarmStates = rootFolder.getTriggeredAlarmState();
        if (null != alarmStates) {
            for (AlarmState alarmState : alarmStates) {
                // alarmModel类
                AlarmModel alarmModel = new AlarmModel();
                // 创建alarm对象
                Alarm alarm = new Alarm(rootFolder.getServerConnection(), alarmState.getAlarm());
                // 获取告警信息
                AlarmInfo alarmInfo = alarm.getAlarmInfo();
                alarmModel.setName(alarmInfo.getName());// 告警名称
                alarmModel.setRemark(alarmInfo.getDescription());// 告警描述
                alarmModel.setLevel(alarmState.getOverallStatus().toString());// 告警等级
                alarmModel.setAlarmVal(alarmInfo.getKey());// 告警key值
                alarmModel.setGmtTrigger(alarmState.getTime().getTime());// 告警触发时间
                // 告警确认时间
                if (null != alarmState.getAcknowledgedTime()) {
                    alarmModel.setGmtConfirm(alarmState.getAcknowledgedTime().getTime());
                }
                // 告警确认人
                if (null != alarmState.getAcknowledgedByUser()) {
                    alarmModel.setConfirmer(alarmState.getAcknowledgedByUser().replace("\\", "\\\\"));
                }
                // 判断不同类型的告警
                String type = alarmState.getEntity().getType();
                if (HS.equals(type)) {
                    HostSystem host = new HostSystem(rootFolder.getServerConnection(), alarmState.getEntity());
                    alarmModel.setTarget(host.getName());
                    alarmModel.setType(HS);
                } else if (VM.equals(type)) {
                    VirtualMachine vm = new VirtualMachine(rootFolder.getServerConnection(), alarmState.getEntity());
                    alarmModel.setTarget(vm.getName());
                    alarmModel.setType(VM);
                } else if ("Datastore".equals(type)) {
                    Datastore datastore = new Datastore(rootFolder.getServerConnection(), alarmState.getEntity());
                    alarmModel.setTarget(datastore.getName());
                    alarmModel.setType("Datastore");
                } else if ("Folder".equals(type)) {
                    Folder folder = new Folder(rootFolder.getServerConnection(), alarmState.getEntity());
                    alarmModel.setTarget(folder.getName());
                    alarmModel.setType("Folder");
                } else if ("Datacenter".equals(type)) {
                    Datacenter datacenter = new Datacenter(rootFolder.getServerConnection(), alarmState.getEntity());
                    alarmModel.setTarget(datacenter.getName());
                    alarmModel.setType("Datacenter");
                } else if ("Network".equals(type)) {
                    Network network = new Network(rootFolder.getServerConnection(), alarmState.getEntity());
                    alarmModel.setTarget(network.getName());
                    alarmModel.setType("Network");
                } else if ("ResourcePool".equals(type)) {
                    ResourcePool resourcePool = new ResourcePool(rootFolder.getServerConnection(),
                            alarmState.getEntity());
                    alarmModel.setTarget(resourcePool.getName());
                    alarmModel.setType("ResourcePool");
                } else if ("ClusterComputeResource".equals(type)) {
                    ClusterComputeResource cluster = new ClusterComputeResource(rootFolder.getServerConnection(),
                            alarmState.getEntity());
                    alarmModel.setTarget(cluster.getName());
                    alarmModel.setType("ClusterComputeResource");
                }
                alarmList.add(alarmModel);
            }
        }
        this.getServiceInstance().getServerConnection().logout();
        return new BocloudResult(true, alarmList, "获取告警信息成功");
    }

    @Override
    public BocloudResult create(AlarmModel alarmModel) {
        try {
            ManagedEntity datacenter = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    alarmModel.getVdcName());
            if (null == datacenter) {
                logger.error("数据中心不存在！");
                return new BocloudResult(false, "数据中心不存在！");
            }
            Datacenter center = (Datacenter) datacenter;
            InventoryNavigator navigator = new InventoryNavigator(center);
            // 绑定告警的实体类型
            String type = alarmModel.getType();
            ManagedEntity entity = navigator.searchManagedEntity(type, alarmModel.getTarget());
            if (null == entity) {
                logger.error("Alarm associated entity does not exist!");
                return new BocloudResult(false, "告警关联实体不存在!");
            }
            AlarmManager alarmManager = this.getServiceInstance().getAlarmManager();
            AlarmSpec alarmSpec = new AlarmSpec();
            alarmSpec.setName(alarmModel.getName());
            alarmSpec.setDescription(alarmModel.getRemark());
            alarmSpec.setEnabled(alarmModel.getEnable());
            Alarm alarm = null;
            switch (type) {
                case "ClusterComputeResource":
                    AlarmAction emailAction = createAlarmTriggerAction(createEmailAction());
                    GroupAlarmAction groupAlarmAction = new GroupAlarmAction();
                    groupAlarmAction.setAction(new AlarmAction[]{emailAction});
                    alarmSpec.setAction(groupAlarmAction);
                    // 创建报警表达式
                    alarmSpec.setExpression(createMetricAlarmExpression());
                    AlarmSetting alarmSetting = new AlarmSetting();
                    // 报警触发的频率，以秒为单位。 零值意味着允许警报尽可能频繁地触发。 非零值意味着在报告的触发器之后的任何后续触发器都会被抑制一段时间。
                    alarmSetting.setReportingFrequency(0);
                    // 公制触发器的公差范围以百分之一百计。零值意味着只要度量值高于或低于指定值就触发警报。非零值意味着报警仅在达到额定触发值以上或以下的特定百分比后触发。
                    alarmSetting.setToleranceRange(0);
                    alarmSpec.setSetting(alarmSetting);
                    ManagedEntity clusterEntity = new InventoryNavigator(datacenter)
                            .searchManagedEntity("ClusterComputeResource", alarmModel.getTarget());
                    if (null == clusterEntity) {
                        return new BocloudResult(false, "集群不存在，告警创建失败");
                    }
                    alarm = alarmManager.createAlarm(clusterEntity, alarmSpec);
                    break;
                case "Datastore":
                    break;
                case VM:
                    break;
                case HS:
                    break;
                default:
                    break;
            }
            if (null == alarm) {
                return new BocloudResult(false, "告警创建失败");
            }
            return new BocloudResult(true, "告警创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "告警创建失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    /**
     * 创建度量标准报警表达式
     *
     * @return
     */
    private MetricAlarmExpression createMetricAlarmExpression() {
        // 使用度量作为触发警报的条件的警报表达式
        MetricAlarmExpression expression = new MetricAlarmExpression();
        // isAbove以上/isBelow以下
        expression.setOperator(MetricAlarmOperator.isAbove);
        PerfMetricId metric = new PerfMetricId();
        // 性能计数器ID
        metric.setCounterId(165);
        metric.setInstance("Disk");
        expression.setMetric(metric);
        expression.setRed(20);
        expression.setRedInterval(300);
        expression.setYellow(10);
        expression.setYellowInterval(300);
        expression.setType("ClusterComputeResource");
        return expression;
    }

    /**
     * 告警接收和发送邮件相关配置
     *
     * @return
     */
    private SendEmailAction createEmailAction() {
        SendEmailAction action = new SendEmailAction();
        action.setToList("123@vmware.com");
        action.setCcList("admins99999@vmware.com");
        action.setSubject("Alarm - {alarmName} on {targetName}\n");
        action.setBody("Description:{eventDescription}\n" + "TriggeringSummary:{triggeringSummary}\n"
                + "newStatus:{newStatus}\n" + "oldStatus:{oldStatus}\n" + "target:{target}");
        return action;
    }

    /**
     * 创建告警触发操作
     *
     * @param action
     * @return
     */
    private AlarmTriggeringAction createAlarmTriggerAction(Action action) {
        AlarmTriggeringAction alarmAction = new AlarmTriggeringAction();
        alarmAction.setYellow2red(true);
        alarmAction.setAction(action);
        return alarmAction;
    }

    @Override
    public BocloudResult update(AlarmModel alarmModel) {
        try {
            return new BocloudResult(true, "告警更新成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "告警更新失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult confirm(AlarmModel alarmModel) {
        try {
            boolean flag = false;
            AlarmManager alarmManager = this.getServiceInstance().getAlarmManager();
            Folder rootFolder = this.getRootFolder();
            AlarmState[] alarmStates = rootFolder.getTriggeredAlarmState();
            for (AlarmState alarmState : alarmStates) {
                Alarm alarm = new Alarm(rootFolder.getServerConnection(), alarmState.getAlarm());
                if (alarm.getAlarmInfo().getKey().equals(alarmModel.getAlarmVal())) {
                    if (!StringUtils.isEmpty(alarmModel.getType()) && !StringUtils.isEmpty(alarmModel.getTarget())) {
                        ManagedEntity entity = new InventoryNavigator(this.getRootFolder())
                                .searchManagedEntity(alarmModel.getType(), alarmModel.getTarget());
                        if (null != entity) {
                            alarmManager.acknowledgeAlarm(alarm, entity);
                        }
                    }
                    flag = true;
                    break;
                }
            }
            if (flag) {
                return new BocloudResult(true, "告警已确认");
            }
            return new BocloudResult(false, "告警确认失败");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "告警确认失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

}
