package com.bocloud.cmp.provider.monitor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.MonitorCategoryModel;
import com.bocloud.cmp.model.MonitorModel;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.DateTool;
import com.bocloud.vmware.vim25.PerfCounterInfo;
import com.bocloud.vmware.vim25.PerfEntityMetric;
import com.bocloud.vmware.vim25.PerfEntityMetricBase;
import com.bocloud.vmware.vim25.PerfMetricId;
import com.bocloud.vmware.vim25.PerfMetricIntSeries;
import com.bocloud.vmware.vim25.PerfMetricSeries;
import com.bocloud.vmware.vim25.PerfQuerySpec;
import com.bocloud.vmware.vim25.PerfSampleInfo;
import com.bocloud.vmware.vim25.mo.InventoryNavigator;
import com.bocloud.vmware.vim25.mo.ManagedEntity;
import com.bocloud.vmware.vim25.mo.PerformanceManager;

/**
 * vmware性能数据操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/14
 */
@Provider("bocloud.cmp.vmware")
public class VmwareMonitorProvider extends VmwareProvider implements MonitorProvider {

	private static Logger logger = LoggerFactory.getLogger(VmwareMonitorProvider.class);

	public VmwareMonitorProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult query(String time, String name, String category, List<String> types) {
		try {
			PerformanceManager perfManager = this.getServiceInstance().getPerformanceManager();
			Map<String, Integer> counterMap = new HashMap<>();// 存放所有的counter名称以及对应的id
			PerfCounterInfo[] counterInfos = perfManager.getPerfCounter();
			for (PerfCounterInfo perfCounterInfo : counterInfos) {
				String key = perfCounterInfo.getGroupInfo().getKey() + "." + perfCounterInfo.getNameInfo().getKey()
						+ "." + perfCounterInfo.getRollupType().toString().toUpperCase();
				counterMap.put(key, perfCounterInfo.getKey());
			}
			ManagedEntity entity = null;
			if (HS.equals(category)) {
				entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS, name);
			} else if (VM.equals(category)) {
				entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(VM, name);
			} else if ("ClusterComputeResource".equals(category)) {
				entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity("ClusterComputeResource",
						name);
			} else {
				return new BocloudResult(false, "暂不支持查看该类型对象性能数据");
			}
			if (null == entity) {
				return new BocloudResult(false, "不存在该对象");
			}

			Integer interval = 20;
			time = time.trim();
			if ("REAL".equals(time)) {
				interval = 20;
			} else if ("DAY".equals(time)) {
				interval = 300;
			} else if ("WEEK".equals(time)) {
				interval = 1800;
			} else if ("MONTH".equals(time)) {
				interval = 7200;
			}
			Map<Integer, String> idsMap = new HashMap<>();
			for (String type : types) {
				int counterId = 0;
				if ("CPU".equals(type)) {
					// 获取每一个宿主机下的cpu.usagemhz.AVERAGE、mem.usage.AVERAGE、disk.usage.AVERAGE
					counterId = counterMap.get("cpu.usage.AVERAGE");// cpu.usagemhz.AVERAGE对应的counterId
				} else if ("DISK".equals(type)) {
					// 获取宿主机下的disk性能数据
					counterId = counterMap.get("disk.usage.AVERAGE");
				} else if ("POWER".equals(type)) {
					// 获取宿主机下的电源性能数据
					counterId = counterMap.get("power.power.AVERAGE");
				} else if ("MEM".equals(type)) {
					// 获取宿主机下的内存数据
					counterId = counterMap.get("mem.usage.AVERAGE");
				} else if ("DATASTORE".equals(type)) {
					// 获取宿主机下的数据存储性能数据
					counterId = counterMap.get("datastore.totalReadLatency.AVERAGE");
				} else if ("NET".equals(type)) {
					// 获取宿主机下的网络性能数据
					counterId = counterMap.get("net.usage.AVERAGE");
				} else if ("SYSTEM".equals(type)) {
					// 获取宿主机下的系统性能数据
					counterId = counterMap.get("sys.uptime.LATEST");
				} else if ("VIRTUALDISK".equals(type)) {
					// 获取宿主机下的虚拟磁盘性能数据
					counterId = counterMap.get("virtualDisk.totalReadLatency.AVERAGE");
				}
				if (counterId != 0) {
					idsMap.put(counterId, type);
				}
			}

			return getMonitorValue(idsMap, interval, perfManager, entity);
		} catch (Exception e) {
			logger.error("query performance failure:", e);
			return new BocloudResult(false, "查询失败");
		} finally {
			this.getServiceInstance().getServerConnection().logout();
		}
	}

	private BocloudResult getMonitorValue(Map<Integer, String> idsMap, Integer timeInterval,
			PerformanceManager perfManager, ManagedEntity entity) throws Exception {
		Map<String, List<PerfMetricId>> perfMetricMap = new HashMap<>();
		PerfMetricId[] queryAvailablePerfMetric = perfManager.queryAvailablePerfMetric(entity, null, null,
				timeInterval);
		if (null != queryAvailablePerfMetric) {
			for (PerfMetricId perfMetricId : queryAvailablePerfMetric) {
				if (idsMap.containsKey(perfMetricId.getCounterId())) {
					String type = idsMap.get(perfMetricId.getCounterId());
					if (perfMetricMap.containsKey(type)) {
						List<PerfMetricId> list = perfMetricMap.get(type);
						list.add(perfMetricId);
						perfMetricMap.put(type, list);
					} else {
						List<PerfMetricId> list = new ArrayList<>();
						list.add(perfMetricId);
						perfMetricMap.put(type, list);
					}
				}
			}
		}
		Map<String, Object> map = new HashMap<>();
		for (String type : perfMetricMap.keySet()) {
			List<PerfMetricId> list = perfMetricMap.get(type);
			PerfMetricId[] pmis = list.toArray(new PerfMetricId[list.size()]);
			PerfQuerySpec qSpec = new PerfQuerySpec();
			qSpec.setEntity(entity.getMOR());
			qSpec.setMetricId(pmis);
			qSpec.setIntervalId(timeInterval);
			PerfEntityMetricBase[] perEntityMetrics = perfManager.queryPerf(new PerfQuerySpec[] { qSpec });
			if (null == perEntityMetrics) {
				continue;
			}
			List<String> keys = new ArrayList<>();
			List<MonitorCategoryModel> models = new ArrayList<>();
			for (PerfEntityMetricBase perfEntityMetricBase : perEntityMetrics) {
				PerfEntityMetric pem = (PerfEntityMetric) perfEntityMetricBase;
				PerfSampleInfo[] samplerInfos = pem.getSampleInfo();// 保存有性能值对应的时间点
				if (null != samplerInfos) {
					for (PerfSampleInfo perfSampleInfo : samplerInfos) {
						keys.add(DateTool.formatTime(perfSampleInfo.getTimestamp().getTime()));
					}
				}
				PerfMetricSeries[] mertricSeries = pem.getValue();
				if (null == mertricSeries) {
					continue;
				}
				for (PerfMetricSeries perfMetricSeries : mertricSeries) {
					PerfMetricIntSeries metricIntSeries = (PerfMetricIntSeries) perfMetricSeries;
					long[] values = metricIntSeries.getValue();// 获取到的性能值
					if (null == values) {
						continue;
					}
					List<String> perfValues = new ArrayList<>();
					for (long value : values) {
						perfValues.add(String.valueOf(value));
					}
					if (StringUtils.isEmpty(metricIntSeries.getId().getInstance())) {
						models.add(new MonitorCategoryModel(type, perfValues));
					} else {
						models.add(new MonitorCategoryModel(type + "-" + metricIntSeries.getId().getInstance(),
								perfValues));
					}
				}
			}
			map.put(type, new MonitorModel(keys, models));
		}
		return new BocloudResult(true, map, "");
	}

}
