package com.bocloud.cmp.provider.storage;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.DataStoreModel;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.cmp.provider.utils.SingleAssembleTools;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.vmware.vim25.*;
import com.bocloud.vmware.vim25.mo.*;
import io.jsonwebtoken.lang.Assert;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.*;
import java.io.*;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.URL;
import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * vmware数据存储操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/14
 */
@Provider("bocloud.cmp.vmware")
public class VmwareDataStoreProvider extends VmwareProvider implements DataStoreProvider {

    private static Logger logger = LoggerFactory.getLogger(VmwareDataStoreProvider.class);

    public VmwareDataStoreProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult listByVdc(String vdcName) {
        logger.info("查询数据中心{}下网络", vdcName);
        try {
            ManagedEntity managedEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    vdcName);
            Assert.notNull(managedEntity, "数据中心" + vdcName + "不存在！");
            Datacenter datacenter = (Datacenter) managedEntity;
            Datastore[] datastores = datacenter.getDatastores();
            List<DataStoreModel> dataStoreModels = new ArrayList<>();
            if (null != datastores) {
                for (Datastore datastore : datastores) {
                    DataStoreModel model = SingleAssembleTools.getInstance().assembleStores(datastore);
                    if (null != model) {
                        dataStoreModels.add(model);
                    }
                }
            }
            return new BocloudResult(true, dataStoreModels, "数据中心" + vdcName + "下存储查询成功");
        } catch (Exception e) {
            logger.error("数据中心" + vdcName + "下存储查询失败:" + e);
            return new BocloudResult(false, "数据中心" + vdcName + "下存储查询失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult uploadFile(String dcName, String dsName, String filePath) {
        HttpsURLConnection conn = null;
        OutputStream out = null;
        InputStream in = null;
        try {
            ServiceInstance instance = this.getServiceInstance();
            ManagedEntity dcEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, dcName);
            Assert.notNull(dcEntity, "数据中心不存在");
            Datacenter dc = (Datacenter) dcEntity;
            InventoryNavigator navigator = new InventoryNavigator(dc);
            ManagedEntity dsEntity = navigator.searchManagedEntity(DT, dsName);
            Assert.notNull(dcEntity, "数据存储不存在");
            Datastore ds = (Datastore) dsEntity;
            String url = instance.getServerConnection().getUrl().toString();
            String serviceUrl = url.substring(0, url.lastIndexOf("sdk") - 1);
            Assert.isTrue(!StringUtils.isEmpty(filePath), "上传文件路径不存在！");
            int start = filePath.lastIndexOf("/") + 1;
            String fileName = filePath.substring(start);
            String httpUrl = serviceUrl + "/folder/" + fileName + "?dcPath=" + dc.getName() + "&dsName=" + ds.getName();
            httpUrl = httpUrl.replaceAll("\\ ", "%20");

            logger.info("Uploading local file to " + httpUrl);
            URL fileURL = new URL(httpUrl);
            // 信任host
            trustAllHosts();
            conn = (HttpsURLConnection) fileURL.openConnection();
            conn.setHostnameVerifier(DO_NOT_VERIFY);
            conn.setDoInput(true);
            conn.setDoOutput(true);
            conn.setAllowUserInteraction(true);
            // 从ServiceInstance连接中提取cookie
            VimPortType vimPort = instance.getServerConnection().getVimService();
            String cookieValue = vimPort.getWsc().getCookie();
            StringTokenizer tokenizer = new StringTokenizer(cookieValue, ";");
            cookieValue = tokenizer.nextToken();
            String path = "$" + tokenizer.nextToken();
            String cookie = "$Version=\"1\"; " + cookieValue + "; " + path;
            conn.setRequestProperty("Cookie", cookie);
            // 二进制流，不知道下载文件类型
            conn.setRequestProperty("Content-Type", "application/octet-stream");
            // PUT方式上传
            conn.setRequestMethod("PUT");
            conn.setRequestProperty("Content-Length", "1024");
            long fileLen = new File(filePath).length();
            logger.info("File size is: " + fileLen);
            conn.setChunkedStreamingMode((int) fileLen);
            out = conn.getOutputStream();
            in = new BufferedInputStream(new FileInputStream(filePath));
            int bufLen = 9 * 1024;
            byte[] buf = new byte[bufLen];
            byte[] tmp = null;
            int len = 0;
            while ((len = in.read(buf, 0, bufLen)) != -1) {
                tmp = new byte[len];
                System.arraycopy(buf, 0, tmp, 0, len);
                out.write(tmp, 0, len);
            }
            logger.info(conn.getResponseMessage());
            return new BocloudResult(true, "文件上传成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "上传文件至数据存储错误");
        } finally {
            try {
                if (null != in) {
                    in.close();
                }
                if (null != out) {
                    out.close();
                }
                if (null != conn) {
                    conn.disconnect();
                }
            } catch (IOException e) {
                logger.error("Get error message:", e);
            }
        }
    }

    @Override
    public BocloudResult create(DataStoreModel model) {
        try {
            ManagedEntity dcEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    model.getVdcName());
            Assert.notNull(dcEntity, "数据中心不存在");
            Datacenter dc = (Datacenter) dcEntity;
            InventoryNavigator navigator = new InventoryNavigator(dc);
            ManagedEntity hostEntity = navigator.searchManagedEntity(HS, model.getHostName());
            Assert.notNull(hostEntity, "宿主机不存在");
            HostSystem host = (HostSystem) hostEntity;
            HostDatastoreSystem dsSystem = host.getHostDatastoreSystem();
            String type = model.getType();
            BocloudResult result = null;
            DataStoreModel storeModel = new DataStoreModel();
            switch (HostFileSystemVolumeFileSystemType.valueOf(type)) {
                // 网络文件系统
                case CIFS:
                case NFS:
                case NFS41:
                    HostNasVolumeSpec nasSpec = new HostNasVolumeSpec();
                    nasSpec.setAccessMode(HostMountMode.valueOf(model.getAccessMode()).name());
                    nasSpec.setLocalPath(model.getName());
                    nasSpec.setRemoteHost(model.getRemoteHost());
                    nasSpec.setRemotePath(model.getRemotePath());
                    if (type.equals(HostFileSystemVolumeFileSystemType.CIFS.name())) {
                        nasSpec.setUserName(model.getUserName());
                        nasSpec.setPassword(model.getPassword());
                    }
                    if (type.equals(HostFileSystemVolumeFileSystemType.NFS41.name())) {
                        if (!StringUtils.isEmpty(model.getRemoteHostNames())) {
                            String[] remoteHostNames = model.getRemoteHostNames().split(",");
                            nasSpec.setRemoteHostNames(remoteHostNames);
                        }
                    }
                    Datastore nasStore = dsSystem.createNasDatastore(nasSpec);
                    if (null != nasStore) {
                        String dsName = nasStore.getName();
                        Long startTime = System.currentTimeMillis();
                        Boolean flag = true;
                        while (flag) {
                            ManagedEntity entity = navigator.searchManagedEntity(DT, dsName);
                            if (null != entity) {
                                Datastore newStore = (Datastore) entity;
                                if (0L != newStore.getSummary().getCapacity()) {
                                    storeModel = convertDataStore(newStore);
                                    flag = false;
                                }
                            } else {
                                storeModel = convertDataStore(nasStore);
                                flag = false;
                            }
                            Long endTime = System.currentTimeMillis();
                            if (endTime - startTime > 3 * 60 * 1000) {// 等待3分钟
                                storeModel = convertDataStore(nasStore);
                                flag = false;
                            }
                        }
                    }
                    break;
                // vmware文件系统
                case VMFS:
                    VmfsDatastoreCreateSpec vmfsSpec = new VmfsDatastoreCreateSpec();
                    // 分区
                    HostDiskPartitionSpec partition = new HostDiskPartitionSpec();
                    vmfsSpec.setPartition(partition);
                    // 创建规格
                    HostVmfsSpec vmfs = new HostVmfsSpec();
                    vmfsSpec.setVmfs(vmfs);
                    Datastore vmfsStore = dsSystem.createVmfsDatastore(vmfsSpec);
                    if (null != vmfsStore) {
                        storeModel = convertDataStore(vmfsStore);
                    }
                    break;
                case VVOL:
                    HostDatastoreSystemVvolDatastoreSpec vvolSpec = new HostDatastoreSystemVvolDatastoreSpec();
                    vvolSpec.setName(model.getName());
                    // 存储容器ID
                    vvolSpec.setScId(model.getScId());
                    Datastore vvolStore = dsSystem.createVvolDatastore(vvolSpec);
                    if (null != vvolStore) {
                        storeModel = convertDataStore(vvolStore);
                    }
                    break;
                case VFFS:
                case vsan:
                case OTHER:
                default:
                    result = new BocloudResult(false, "不支持此类型的数据存储");
                    break;
            }
            if (null == storeModel) {
                result = new BocloudResult(false, "数据存储添加失败");
            } else {
                result = new BocloudResult(true, storeModel, "数据存储添加成功");
            }
            return result;
        } catch (PlatformConfigFault e) {
            logger.error("Get error message:{}", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    private DataStoreModel convertDataStore(Datastore datastore) {
        DatastoreSummary datastoreSummary = datastore.getSummary();
        Long diskCapacity = datastoreSummary.getCapacity();
        Long freeCapacity = datastoreSummary.getFreeSpace();
        BigDecimal capacity = new BigDecimal(diskCapacity);
        BigDecimal freedisk = new BigDecimal(freeCapacity);
        Double diskTotal = capacity.divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP).doubleValue();
        Double diskFree = freedisk.divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP).doubleValue();
        // 获取存储下的云主机与模板
        int templateNum = 0;// 模板数量
        int vmNum = 0;// 云主机数量
        VirtualMachine[] vms = datastore.getVms();
        templateNum = Arrays.stream(vms).filter(vm -> Objects.nonNull(vm.getConfig()) && vm.getConfig().isTemplate())
                .collect(Collectors.toList()).size();
        vmNum = Arrays.stream(vms).filter(vm -> Objects.nonNull(vm.getConfig()) && !vm.getConfig().isTemplate())
                .collect(Collectors.toList()).size();
        int hostNum = Optional.ofNullable(datastore.getHost()).orElse(new DatastoreHostMount[0]).length;
        DataStoreModel dataStoreModel = new DataStoreModel(datastoreSummary.getName(), datastoreSummary.getType(),
                datastoreSummary.getUrl(), diskTotal, diskFree, hostNum, vmNum, templateNum,
                datastoreSummary.getDatastore().getVal());
        // 判断数据存储是否是活动状态
        if (!datastore.getSummary().accessible) {
            dataStoreModel.setStatus("inactive");
        } else {
            dataStoreModel.setStatus("green");
        }
        return dataStoreModel;
    }

    @Override
    public BocloudResult rename(String dcName, String oldName, String newName) {
        try {
            ManagedEntity dcEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, dcName);
            Assert.notNull(dcEntity, "数据中心不存在");
            Datacenter dc = (Datacenter) dcEntity;
            InventoryNavigator navigator = new InventoryNavigator(dc);
            ManagedEntity dsEntity = navigator.searchManagedEntity(DT, oldName);
            Assert.notNull(dsEntity, "数据存储不存在");
            Datastore ds = (Datastore) dsEntity;
            ds.renameDatastore(newName);
            return new BocloudResult(true, "重命名成功");
        } catch (DuplicateName e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "重命名失败：" + e.getMessage());
        } catch (InvalidName e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "重命名失败：" + e.getMessage());
        } catch (RuntimeFault e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "重命名失败：" + e.getMessage());
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "重命名失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult remove(String dcName, String name) {
        try {
            ManagedEntity dcEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, dcName);
            Assert.notNull(dcEntity, "数据中心不存在");
            Datacenter dc = (Datacenter) dcEntity;
            InventoryNavigator navigator = new InventoryNavigator(dc);
            ManagedEntity dsEntity = navigator.searchManagedEntity(DT, name);
            Assert.notNull(dsEntity, "数据存储不存在");
            Datastore ds = (Datastore) dsEntity;
            Task task = ds.destroy_Task();
            if (task.waitForTask().equals(Task.SUCCESS)) {
                logger.info("Success to destroy Data store");
                return new BocloudResult(true, "删除成功");
            } else {
                return new BocloudResult(false, "删除失败：" + task.getTaskInfo().getError().getLocalizedMessage());
            }
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "删除失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult unmount(String dcName, String hostName, String name) {
        try {
            ManagedEntity dcEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, dcName);
            Assert.notNull(dcEntity, "数据中心不存在");
            Datacenter dc = (Datacenter) dcEntity;
            InventoryNavigator navigator = new InventoryNavigator(dc);
            ManagedEntity dsEntity = navigator.searchManagedEntity(DT, name);
            Assert.notNull(dsEntity, "数据存储不存在");
            Datastore ds = (Datastore) dsEntity;
            ManagedEntity hostEntity = navigator.searchManagedEntity(HS, hostName);
            Assert.notNull(hostEntity, "宿主机不存在");
            HostSystem host = (HostSystem) hostEntity;
            HostDatastoreSystem dsSystem = host.getHostDatastoreSystem();
            dsSystem.removeDatastore(ds);
            return new BocloudResult(true, "数据存储卸载成功");
        } catch (HostConfigFault e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "无法卸载该数据存储");
        } catch (NotFound e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "数据存储不存在");
        } catch (ResourceInUse e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "该数据存储正在被云主机使用，卸载失败");
        } catch (RuntimeFault e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "数据存储卸载失败");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "数据存储卸载失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult detail(String dcName, String dsName) {
        try {
            ManagedEntity dcEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, dcName);
            Assert.notNull(dcEntity, "数据中心不存在");
            Datacenter dc = (Datacenter) dcEntity;
            InventoryNavigator navigator = new InventoryNavigator(dc);
            ManagedEntity dsEntity = navigator.searchManagedEntity(DT, dsName);
            Assert.notNull(dcEntity, "数据存储不存在");
            return new BocloudResult(true, convert((Datastore) dsEntity), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "查询失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult simpleDetail(String dcName, String dsName) {
        try {
            ManagedEntity dcEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, dcName);
            Assert.notNull(dcEntity, "数据中心不存在");
            Datacenter dc = (Datacenter) dcEntity;
            InventoryNavigator navigator = new InventoryNavigator(dc);
            ManagedEntity dsEntity = navigator.searchManagedEntity(DT, dsName);
            Assert.notNull(dcEntity, "数据存储不存在");
            return new BocloudResult(true, convert((Datastore) dsEntity, true), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "查询失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    private DataStoreModel convert(Datastore datastore) {
        return convert(datastore, false);
    }
    private DataStoreModel convert(Datastore datastore, boolean simple) {
        DatastoreSummary datastoreSummary = datastore.getSummary();
        Long diskCapacity = datastoreSummary.getCapacity();
        Long freeCapacity = datastoreSummary.getFreeSpace();
        BigDecimal capacity = new BigDecimal(diskCapacity);
        BigDecimal freedisk = new BigDecimal(freeCapacity);
        Double diskTotal = capacity.divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP).doubleValue();
        Double diskFree = freedisk.divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP).doubleValue();
        int hostNum = 0;
        int templateNum = 0;
        int vmNum = 0;
        if (!simple){
            // 获取存储下的云主机与模板
            VirtualMachine[] vms = datastore.getVms();
            templateNum = (int) Arrays.stream(vms)
                    .filter(vm -> Objects.nonNull(vm.getConfig()) && vm.getConfig().isTemplate()).count();
            vmNum = (int) Arrays.stream(vms).filter(vm -> Objects.nonNull(vm.getConfig()) && !vm.getConfig().isTemplate())
                    .count();
            hostNum = Optional.ofNullable(datastore.getHost()).orElse(new DatastoreHostMount[0]).length;
        }

        DataStoreModel dataStoreModel = new DataStoreModel(datastoreSummary.getName(), datastoreSummary.getType(),
                datastoreSummary.getUrl(), diskTotal, diskFree, hostNum, vmNum, templateNum,
                datastoreSummary.getDatastore().getVal());
        // 判断数据存储是否是活动状态
        if (!datastore.getSummary().accessible) {
            dataStoreModel.setStatus("inactive");
        } else {
            dataStoreModel.setStatus("green");
        }
        return dataStoreModel;
    }

    public final static HostnameVerifier DO_NOT_VERIFY = (hostname, session) -> true;

    public static void trustAllHosts() {
        TrustManager[] trustAllCerts = new TrustManager[]{new X509TrustManager() {
            @Override
            public X509Certificate[] getAcceptedIssuers() {
                return new X509Certificate[]{};
            }

            @Override
            public void checkClientTrusted(X509Certificate[] chain, String authType) {
                logger.info("check client trusted");
            }

            @Override
            public void checkServerTrusted(X509Certificate[] chain, String authType) {
                logger.info("check server trusted");
            }
        }};
        try {
            SSLContext sc = SSLContext.getInstance("TLSv1.2");
            sc.init(null, trustAllCerts, new SecureRandom());
            HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
        } catch (Exception e) {
            logger.error("Get error message：", e);
        }
    }
}
