package com.bocloud.cmp.provider;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.compute.VolcengineImageProvider;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.common.model.BocloudResult;
import com.volcengine.auth.ISignerV4;
import com.volcengine.auth.impl.SignerV4Impl;
import com.volcengine.error.SdkError;
import com.volcengine.http.ClientConfiguration;
import com.volcengine.http.HttpClientFactory;
import com.volcengine.model.Credentials;
import com.volcengine.service.SignableRequest;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.message.BasicHeader;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import java.nio.charset.StandardCharsets;
import java.util.*;

/**
 * @ClassName: VolcengineProvider
 * @Description: 火山云
 * @Author lxy
 * @Date 2021/9/9 17:25
 */
public class VolcengineProvider extends GenericProvider implements SuperProvider {

    private String ak;
    private String sk;
    private String regionId = "cn-beijing"; // 选择大区
    private HttpClient httpClient;
    private Butler butler;
    private ISignerV4 ISigner = null;


    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineProvider(Butler butler) {
        super(butler);
        if (StringUtils.isEmpty(butler.getAccessKey())) {
            throw new IllegalArgumentException("Access key cannot be null.");
        }
        if (StringUtils.isEmpty(butler.getSecretKey())) {
            throw new IllegalArgumentException("Secret key cannot be null.");
        }
        this.ak = butler.getAccessKey();
        this.sk = butler.getSecretKey();
        if (StringUtils.isNotEmpty(butler.getRegionId())) {
            this.regionId = butler.getRegionId();
        }
        this.butler = butler;
        this.ISigner = new SignerV4Impl();
        this.httpClient = HttpClientFactory.create(new ClientConfiguration(), null);
    }

    @Override
    public boolean accessable() {
        try {
            VolcengineImageProvider volcengineImageProvider = new VolcengineImageProvider(butler);
            return volcengineImageProvider.list().isSuccess();
        } catch (Exception e) {
            return false;
        }
    }

    public BocloudResult<JSONObject> operation(String api, String version, List<NameValuePair> params, VolcengineConst.service service) {
        SignableRequest request = prepareRequest(api, version, params);
        return makeRequest(api, request, service);
    }

    private SignableRequest prepareRequest(String api, String version, List<NameValuePair> params) {

        int socketTimeout = 5000;
        int connectionTimeout = 5000;
        SignableRequest request = new SignableRequest();
        request.setMethod(VolcengineConst.METHOD);

        Collection<Header> mergedH = mergeHeader(new ArrayList<Header>() {
            {
                add(new BasicHeader("Accept", "application/json"));
            }
        }, null);

        for (Header header : mergedH) {
            request.setHeader(header);
        }
        request.setHeader("User-Agent", "volc-sdk-java/v" + version);
        List<NameValuePair> mergedNV = mergeQuery(params, new ArrayList<NameValuePair>() {
            {
                add(new BasicNameValuePair("Action", api));
                if (StringUtils.isNotBlank(version)) {
                    add(new BasicNameValuePair("Version", version));
                }
            }
        });
        URIBuilder builder = request.getUriBuilder();

        builder.setScheme(VolcengineConst.SCHEME);
        builder.setHost(VolcengineConst.HOST);
        builder.setPath(VolcengineConst.PATH);
        builder.setParameters(mergedNV);

        RequestConfig requestConfig = RequestConfig.custom()
                .setSocketTimeout(socketTimeout)
                .setConnectTimeout(connectionTimeout)
                .build();
        request.setConfig(requestConfig);

        return request;
    }

    private Collection<Header> mergeHeader(List<Header> header1, List<Header> header2) {
        Set<Header> set = new HashSet<>();
        if (header1 != null) {
            set.addAll(header1);
        }
        if (header2 != null) {
            set.addAll(header2);
        }
        return set;
    }

    private List<NameValuePair> mergeQuery(List<NameValuePair> query1, List<NameValuePair> query2) {
        List<NameValuePair> res = new ArrayList<NameValuePair>();
        if (query1 != null) {
            res.addAll(query1);
        }
        if (query2 != null) {
            res.addAll(query2);
        }
        return res;
    }

    private BocloudResult<JSONObject> makeRequest(String api, SignableRequest request, VolcengineConst.service service) {
        JSONObject jsonObject = new JSONObject();
        try {
            Credentials credentials = new Credentials(regionId, service.name().toLowerCase());
            credentials.setSecretAccessKey(sk);
            credentials.setAccessKeyID(ak);
            ISigner.sign(request, credentials);
        } catch (Exception e) {
            return new BocloudResult<JSONObject>(false, "获取火山云连接异常：" + e.getMessage());
        }

        HttpClient client;
        HttpResponse response = null;
        try {
            response = this.httpClient.execute(request);
            int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode >= 300) {
                String msg = SdkError.getErrorDesc(SdkError.EHTTP);
                byte[] bytes = EntityUtils.toByteArray(response.getEntity());
                if (bytes != null && bytes.length > 0) {
                    msg = new String(bytes, StandardCharsets.UTF_8);
                }
                return new BocloudResult<JSONObject>(false, "火山云调用异常：api = " + api + "，service = " + service);
            }
            byte[] bytes = EntityUtils.toByteArray(response.getEntity());
            String s = new String(bytes);
            return new BocloudResult<JSONObject>(true, JSONObject.parseObject(new String(bytes)), api + "操作成功");
        } catch (Exception e) {
            if (response != null) {
                EntityUtils.consumeQuietly(response.getEntity());
            }
            return new BocloudResult<JSONObject>(false, "火山云调用异常：" + e);
        }
    }

    public String getRegionId() {
        return this.regionId;
    }

    public void setRegionId(final String regionId) {
        this.regionId = regionId;
    }
}
