package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.compute.VolcengineImageProvider;
import com.bocloud.cmp.provider.compute.VolcengineServerProvider;
import com.bocloud.cmp.provider.loadbalance.VolcengineLoadBalanceProvider;
import com.bocloud.cmp.provider.network.*;
import com.bocloud.cmp.provider.security.VolcengineKeypairProvider;
import com.bocloud.cmp.provider.storage.VolcengineVolumeProvider;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.google.common.collect.Lists;
import com.volcengine.helper.Utils;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: VolcengineLocationProvider
 * @Description:
 * @Author lxy
 * @Date 2021/9/15 9:54
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineLocationProvider extends VolcengineProvider implements LocationProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineLocationProvider.class);
    private Butler butler;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineLocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult sync() {
        VolcengineSyncModel syncModel = new VolcengineSyncModel();
        try {
            logger.info("开始同步火山云数据当前区域 ：" + butler.getRegionId());
            // 同步域 -- 获取可用域下的区域信息
            logger.info("地域可用域 ...");
            BocloudResult result = syncRegions(syncModel);
            if (result.isFailed()) {
                logger.warn("地域{}无法同步数据", this.getRegionId());
                return result;
            }
            logger.info("镜像 数据...");
            syncImages(syncModel);

            logger.info("秘钥 数据...");
            syncKeyPairs(syncModel);

            logger.info("规格 数据...");
            syncFlavor(syncModel);

            logger.info("VPC + subnet 数据...");
            syncVpcs(syncModel);

            logger.info("安全组 数据...");
            syncSecurityGroups(syncModel);

            logger.info("弹性IP 数据 ...");
            syncPublicIps(syncModel);

            logger.info("networkcard .... ");
            syncNetworkCard(syncModel);

            logger.info("云硬盘 数据...");
            syncVolumes(syncModel);

            logger.info("云主机 数据...");
            syncServers(syncModel);

            logger.info("路由 数据 ...");
            syncRouteTable(syncModel);

            logger.info("负载均衡 数据...");
            syncLoadBalances(syncModel);

            logger.info("监听器、健康检查、转发规则 数据...");
            syncLbListeners(syncModel);

            logger.info("服务器组及服务器 数据...");
            syncLbPools(syncModel);

            logger.info("结束同步火山云数据当前区域 ：" + butler.getRegionId());
            return new BocloudResult(true, syncModel, "同步成功");

        } catch (Exception e) {
            logger.error("同步失败 ： " + e);
            return new BocloudResult(false, "同步失败 ");
        }
    }

    private void syncLbPools(VolcengineSyncModel syncModel) {
        VolcengineLoadBalanceProvider loadBalanceProvider = new VolcengineLoadBalanceProvider(butler);
        BocloudResult lbPoolsResult = loadBalanceProvider.listPool();
        List<LbPoolModel> lbPools = new ArrayList<>();
        if (lbPoolsResult.isSuccess()) {
            lbPools = JSONArray.parseArray(JSONArray.toJSONString(lbPoolsResult.getData()), LbPoolModel.class);
            if (CollectionUtils.isNotEmpty(lbPools)) {
                List<LbMemberModel> lbMembers = new ArrayList<>();
                for (LbPoolModel pool : lbPools) {
                    List<LbMemberModel> lbMethod = pool.getMemberModels();
                    if (CollectionUtils.isNotEmpty(lbMethod)) {
                        lbMembers.addAll(lbMethod);
                    }
                }
                syncModel.setLbMemberModels(lbMembers);
            }
            syncModel.setLbPoolModels(lbPools);
        }
    }

    private void syncLbListeners(VolcengineSyncModel syncModel) {
        VolcengineLoadBalanceProvider loadBalanceProvider = new VolcengineLoadBalanceProvider(butler);
        BocloudResult lbListenerResult = loadBalanceProvider.listListener();
        List<LbListenerModel> lbListeners = new ArrayList<>();
        if (lbListenerResult.isSuccess()) {
            lbListeners = JSONArray.parseArray(JSONArray.toJSONString(lbListenerResult.getData()),
                    LbListenerModel.class);
            syncModel.setLbListenerModels(lbListeners);
        }
    }

    private void syncLoadBalances(VolcengineSyncModel syncModel) {
        VolcengineLoadBalanceProvider loadBalanceProvider = new VolcengineLoadBalanceProvider(butler);
        List<LbModel> lbModels = Lists.newArrayList();
        BocloudResult result = loadBalanceProvider.list();
        if (result.isSuccess()) {
            lbModels = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), LbModel.class);
        }
        syncModel.setLbModels(lbModels);
    }

    private void syncRouteTable(VolcengineSyncModel syncModel) {
        VolcengineRouteTableProvider routeTableProvider = new VolcengineRouteTableProvider(butler);
        List<RouteTableModel> routeTableModels = Lists.newArrayList();
        BocloudResult result = routeTableProvider.list();
        if (result.isSuccess()) {
            routeTableModels = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), RouteTableModel.class);
        }
        syncModel.setRouteTableModels(routeTableModels);
    }

    private void syncNetworkCard(VolcengineSyncModel syncModel) {
        VolcengineNetworkCardProvider networkCardProvider = new VolcengineNetworkCardProvider(butler);
        List<NetworkCardModel> networkCardModels = Lists.newArrayList();
        BocloudResult result = networkCardProvider.list();
        if (result.isSuccess()) {
            networkCardModels = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), NetworkCardModel.class);
        }
        syncModel.setNetworkCardModels(networkCardModels);
    }

    private void syncPublicIps(VolcengineSyncModel syncModel) {
        VolcengineElasticIpProvider elasticIpProvider = new VolcengineElasticIpProvider(butler);
        List<ElasticIpModel> elasticIpModels = Lists.newArrayList();
        BocloudResult result = elasticIpProvider.list();
        if (result.isSuccess()) {
            elasticIpModels = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), ElasticIpModel.class);
        }
        syncModel.setElasticIpModels(elasticIpModels);
    }

    private void syncKeyPairs(VolcengineSyncModel syncModel) {
        VolcengineKeypairProvider keypairProvider = new VolcengineKeypairProvider(butler);
        List<KeypairModel> keypairModels = Lists.newArrayList();
        BocloudResult result = keypairProvider.list();
        if (result.isSuccess()) {
            keypairModels = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), KeypairModel.class);
        }
        syncModel.setKeypairModels(keypairModels);
    }


    private void syncServers(VolcengineSyncModel syncModel) {
        VolcengineServerProvider serverProvider = new VolcengineServerProvider(butler);
        List<ServerModel> serverModels = Lists.newArrayList();
        BocloudResult serversRes = serverProvider.list();
        if (serversRes.isSuccess()) {
            serverModels = JSONArray.parseArray(JSONObject.toJSONString(serversRes.getData()), ServerModel.class);
        }
        syncModel.setServerModels(serverModels);
    }

    private void syncVolumes(VolcengineSyncModel syncModel) {
        VolcengineVolumeProvider volumeProvider = new VolcengineVolumeProvider(butler);
        List<VolumeModel> volumeModels = Lists.newArrayList();
        BocloudResult volumesRes = volumeProvider.list();
        if (volumesRes.isSuccess()) {
            volumeModels = JSONArray.parseArray(JSONArray.toJSONString(volumesRes.getData()), VolumeModel.class);
        }
        syncModel.setVolumeModels(volumeModels);
    }

    private void syncSecurityGroups(VolcengineSyncModel syncModel) {
        VolcengineSecurityGroupProvider securityGroupProvider = new VolcengineSecurityGroupProvider(butler);

        List<SecurityGroupModel> secgModels = new ArrayList<>();
        BocloudResult securityGroupRes = securityGroupProvider.list();
        if (securityGroupRes.isSuccess()) {
            secgModels = JSONArray.parseArray(JSONArray.toJSONString(securityGroupRes.getData()), SecurityGroupModel.class);
            if (CollectionUtils.isNotEmpty(secgModels)) {
                for (SecurityGroupModel scg : secgModels) {
                    BocloudResult scgRule = securityGroupProvider.detail(scg.getGroupUuid());
                    if (scgRule.isSuccess() && null != scgRule.getData()) {
                        SecurityGroupModel securityGroupModel = JSONArray.parseObject(JSONObject.toJSONString(scgRule.getData()), SecurityGroupModel.class);
                        scg.setModels(securityGroupModel.getModels());
                    }
                }
            }
        }
        syncModel.setSecgModels(secgModels);
    }

    private void syncVpcs(VolcengineSyncModel syncModel) {
        VolcengineVpcProvider vpcProvider = new VolcengineVpcProvider(butler);
        VolcengineSubnetProvider subnetProvider = new VolcengineSubnetProvider(butler);
        BocloudResult vpcResult = vpcProvider.list();
        List<VpcModel> vpcModels = new ArrayList<>();
        if (vpcResult.isSuccess()) {
            vpcModels = JSONArray.parseArray(JSONArray.toJSONString(vpcResult.getData()), VpcModel.class);
            if (CollectionUtils.isNotEmpty(vpcModels)) {
                for (VpcModel vpc : vpcModels) {
                    // 查询VPC下的子网数据
                    String vpcId = vpc.getVpcId();
                    BocloudResult subnetResult = subnetProvider.list(vpcId);
                    List<SubnetModel> subnetModels = new ArrayList<SubnetModel>();
                    if (subnetResult.isSuccess()) {
                        subnetModels = JSONArray.parseArray(JSONArray.toJSONString(subnetResult.getData()),
                                SubnetModel.class);
                    }
                    vpc.setSubnetModels(subnetModels);
                }
            }
            syncModel.setVpcModels(vpcModels);
        } else {
            syncModel.setVpcModels(null);
        }
    }

    private void syncFlavor(VolcengineSyncModel syncModel) {
        VolcengineFlavorProvider flavorProvider = new VolcengineFlavorProvider(butler);
        List<FlavorModel> flavorModels = new ArrayList<>();
        BocloudResult sync = flavorProvider.sync();
        if (sync.isSuccess()) {
            flavorModels = JSONArray.parseArray(JSONArray.toJSONString(sync.getData()), FlavorModel.class);
        }
        syncModel.setFlavorModels(flavorModels);
    }

    private void syncImages(VolcengineSyncModel model) {
        VolcengineImageProvider imageProvider = new VolcengineImageProvider(butler);
        List<ImageModel> imageModels = new ArrayList<>();
        BocloudResult imageResult = imageProvider.list();
        if (imageResult.isSuccess()) {
            imageModels = JSONArray.parseArray(JSONArray.toJSONString(imageResult.getData()), ImageModel.class);
        }

        model.setImageModels(imageModels);
    }


    /**
     * @return com.bocloud.common.model.BocloudResult
     * @Author: lxy
     * @Description: 同步区域
     * @Date 2021/9/15 10:34
     * @Param syncModel
     * @throws:
     **/
    private BocloudResult syncRegions(VolcengineSyncModel syncModel) {
        BocloudResult result = this.listZones(this.getRegionId());
        if (result.isFailed()) {
            return result;
        }
        List<RegionModel> regions = new ArrayList<>();
        RegionModel regionModel = new RegionModel();
        regionModel.setCode(this.getRegionId());
        List<ZoneModel> zones = JSON.parseArray(JSON.toJSONString(result.getData()), ZoneModel.class);
        regionModel.setZones(zones);
        regions.add(regionModel);
        syncModel.setRegions(regions);
        return new BocloudResult(true, "查询成功！");
    }

    @Override
    public BocloudResult listZones(String region) {
        try {
            Map<String, String> params = new HashMap<>();
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEZONES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject zoneResult = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<JSONObject> zones = JSONArray.parseArray(JSONObject.toJSONString(zoneResult.get("Zones")), JSONObject.class);
            List<ZoneModel> zoneModelList = Lists.newArrayList();
            if (CollectionUtils.isNotEmpty(zones)) {
                for (JSONObject zone : zones) {
                    ZoneModel zoneModel = new ZoneModel();
                    zoneModel.setZoneId(zone.getString("ZoneId"));
                    zoneModel.setRegionId(region);
                    zoneModel.setDiskCategory(zone.getString("VolumeTypes"));
                    zoneModel.setResources(zone.getString("InstanceTypes"));
                    this.getNames(zone.getString("ZoneId"), zoneModel);
                    zoneModelList.add(zoneModel);
                }
            }

            return new BocloudResult(true, zoneModelList, "可用域查询成功");
        } catch (Exception e) {
            logger.error("get zone error : " + e.getMessage());
            return new BocloudResult(false, "可用域查询失败，原因：" + e.getMessage());
        }
    }

    private void getNames(String zoneName, ZoneModel model) {
        if (zoneName.endsWith("a") || zoneName.endsWith("1")) {
            model.setName("可用区A");
        }
        if (zoneName.endsWith("b") || zoneName.endsWith("2")) {
            model.setName("可用区B");
        }
        if (zoneName.endsWith("c") || zoneName.endsWith("3")) {
            model.setName("可用区C");
        }
        if (zoneName.endsWith("d") || zoneName.endsWith("4")) {
            model.setName("可用区D");
        }
        if (zoneName.endsWith("e") || zoneName.endsWith("5")) {
            model.setName("可用区E");
        }
        if (zoneName.endsWith("f") || zoneName.endsWith("6")) {
            model.setName("可用区F");
        }
        if (zoneName.endsWith("g") || zoneName.endsWith("7")) {
            model.setName("可用区G");
        }
    }

    /**
     * @return com.bocloud.common.model.BocloudResult
     * @Author: lxy
     * @Description: 由于Api中没有region，所以暂时写死
     * @Date 2021/9/15 16:59
     * @Param
     * @throws:
     **/
    @Override
    public BocloudResult listRegions() {
        try {
            List<RegionModel> regionModels = Lists.newArrayList();
            RegionModel regionModel1 = new RegionModel();
            regionModel1.setName("华北1（青岛）");
            regionModel1.setCode("cn-qingdao");
            regionModel1.setRegionId(this.getRegionId());
            regionModels.add(regionModel1);
            RegionModel regionModel2 = new RegionModel();
            regionModel2.setName("华北2（北京）");
            regionModel2.setCode("cn-beijing");
            regionModel2.setRegionId(this.getRegionId());
            regionModels.add(regionModel2);

            return new BocloudResult(true, regionModels, "区域查询成功");
        } catch (Exception e) {
            logger.error("get Regions error : " + e.getMessage());
            return new BocloudResult(false, "区域查询失败，原因：" + e.getMessage());
        }
    }
}
