package com.bocloud.cmp.provider.loadbalance;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.AccessControlModel;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.convertor.AccessControlConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.cmp.provider.volmodel.Acl;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.google.common.collect.Lists;
import com.volcengine.helper.Utils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @ClassName: VolcengineAccessControlProvider
 * @Description: 访问控制
 * @Author lxy
 * @Date 2021/10/21 14:49
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineAccessControlProvider extends VolcengineProvider implements AccessControlProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineAccessControlProvider.class);
    private BeanConvertor<AccessControlModel, Acl> aclConvertor = new AccessControlConvertor();
    private static final int PAGE_SIZE = 100;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineAccessControlProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEACLS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<JSONObject> acls = JSONArray.parseArray(JSONObject.toJSONString(result.get("Acls")), JSONObject.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询访问控制列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBEACLS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<JSONObject> aclsList = JSONArray.parseArray(JSONObject.toJSONString(result.get("Acls")), JSONObject.class);
                    acls.addAll(aclsList);
                }
            }
            List<String> aclIds = acls.stream().map(acl -> acl.getString("AclId")).collect(Collectors.toList());
            List<AccessControlModel> accessControlModels = Lists.newArrayList();
            for (String id : aclIds) {
                BocloudResult detail = this.detail(id);
                AccessControlModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), AccessControlModel.class);
                accessControlModels.add(model);
            }
            return new BocloudResult(true, accessControlModels, "查询成功");
        } catch (Exception e) {
            logger.error("list acl error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult detail(String aclId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("AclId", aclId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEACLATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            Acl acl = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), Acl.class);
            AccessControlModel model = aclConvertor.convertModel(acl);
            model.setRegionId(this.getRegionId());
            return new BocloudResult(true, model, "查询成功");
        } catch (Exception e) {
            logger.error("detail acl error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(AccessControlModel acl) {
        try {
            Map<String, String> params = new HashMap<>();
            setParams(params, acl);
            BocloudResult<JSONObject> response = operation(VolcengineConst.CREATEACL, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            AccessControlModel accessControlModel = JSONObject.parseObject(JSONObject.toJSONString(this.detail(result.getString("AclId")).getData()), AccessControlModel.class);
            if (null != acl.getIPVersion()) {
                List<JSONObject> jsonObjects = JSONArray.parseArray(acl.getIPVersion(), JSONObject.class);
                return this.addAclEntries(accessControlModel.getAclUuId(), jsonObjects);
            }
            return new BocloudResult(true, accessControlModel, "创建成功");
        } catch (Exception e) {
            logger.error("create acl error:", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult update(AccessControlModel acl) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("AclId", acl.getAclUuId());
            setParams(params, acl);
            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYACLATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return new BocloudResult(true, this.detail(acl.getAclUuId()).getData(), "修改成功");
        } catch (Exception e) {
            logger.error("update acl error:", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    /**
     * @param acl
     * @return void
     * @Author: lxy
     * @Description: 设置非必填参数
     * @Date 2021/10/21 15:52
     * @Param params
     * @throws:
     **/
    private void setParams(Map<String, String> params, AccessControlModel acl) {
        /**
         * 访问控制策略组的名字。
         * ● 不能以http://或https://开头。
         * ● 必须以字母或中文开头，可包含数字、点号（.）、下划线（_）和短横线（-）。
         * ● 长度限制为1 ～ 128个字符。
         * ● 不填默认为访问控制策略组ID。
         */
        if (StringUtils.isNotBlank(acl.getName())) {
            params.put("AclName", acl.getName());
        }

        /**
         * 访问控制策略组的描述。
         * ● 不能以http://或https://开头。
         * ● 必须以字母或中文开头，可包含数字、英文逗号（,）、点号（.）、下划线（_）、空格（ ）、等号（=）、短横线（-）、中文逗号（，）、中文句号（。）。
         * ● 长度限制为1 ～ 255个字符。
         * ● 不填默认为空字符串。
         */
        if (StringUtils.isNotBlank(acl.getRemark())) {
            params.put("Description", acl.getRemark());
        }
    }

    @Override
    public BocloudResult remove(String aclId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("AclId", aclId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DELETEACL, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("remove acl error:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult addAclEntries(String aclId, List<JSONObject> aclEntries) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("AclId", aclId);
            for (int i = 1; i <= aclEntries.size(); i++) {
                JSONObject json = aclEntries.get(i - 1);
                params.put("AclEntries." + i + ".Entry", json.getString("Entry"));
                if (StringUtils.isNotBlank(json.getString("Description"))) {
                    params.put("AclEntries." + i + ".Description", json.getString("Description"));
                }
            }
            BocloudResult<JSONObject> response = operation(VolcengineConst.ADDACLENTRIES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return new BocloudResult(true, this.detail(result.getString("AclId")).getData(), "添加IP条目成功");
        } catch (Exception e) {
            logger.error("add AclEntries error:", e);
            return new BocloudResult(false, "添加IP条目失败");
        }
    }

    @Override
    public BocloudResult removeAclEntries(String aclId, List<String> alcIps) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("AclId", aclId);
            for (int i = 1; i <= alcIps.size(); i++) {
                params.put("Entries." + i, alcIps.get(i - 1));
            }
            BocloudResult<JSONObject> response = operation(VolcengineConst.REMOVEACLENTRIES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return new BocloudResult(true, this.detail(result.getString("AclId")).getData(), "删除IP条目成功");
        } catch (Exception e) {
            logger.error("remove AclEntries error:", e);
            return new BocloudResult(false, "删除IP条目失败");
        }
    }
}
