package com.bocloud.cmp.provider.loadbalance;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.convertor.*;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.cmp.provider.volmodel.Listeners;
import com.bocloud.cmp.provider.volmodel.LoadBalancers;
import com.bocloud.cmp.provider.volmodel.Rules;
import com.bocloud.cmp.provider.volmodel.ServerGroups;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;
import com.volcengine.helper.Utils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @ClassName: VolcengineLoadBalanceProvider
 * @Description: 负载均衡
 * @Author lxy
 * @Date 2021/10/11 11:40
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineLoadBalanceProvider extends VolcengineProvider implements LoadBalanceProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineLoadBalanceProvider.class);
    private BeanConvertor<LbModel, LoadBalancers> lbConvertor = new VolLoadBalanceConvertor();
    private BeanConvertor<LbListenerModel, Listeners> lbListenerConvertor = new VolListenerConvertor();
    private BeanConvertor<LbPoolModel, ServerGroups> lbPoolConvertor = new VolServerGroupConvertor();
    private BeanConvertor<LbListenerRuleModel, Rules> lbListenerRuleConvertor = new VolListenerRuleConvertor();
    private static final int PAGE_SIZE = 20;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineLoadBalanceProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBELOADBALANCERS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<LoadBalancers> loadBalancers = JSONArray.parseArray(JSONObject.toJSONString(result.get("LoadBalancers")), LoadBalancers.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询负载均衡列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBELOADBALANCERS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<LoadBalancers> balancersList = JSONArray.parseArray(JSONObject.toJSONString(result.get("LoadBalancers")), LoadBalancers.class);
                    loadBalancers.addAll(balancersList);
                }
            }
            List<String> lbIds = loadBalancers.stream().map(LoadBalancers::getLoadBalancerId).collect(Collectors.toList());
            List<LbModel> lbModels = Lists.newArrayList();
            for (String id : lbIds) {
                BocloudResult detail = this.detail(id);
                LbModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), LbModel.class);
                lbModels.add(model);
            }
            return new BocloudResult(true, lbModels, "查询成功");
        } catch (Exception e) {
            logger.error("list loadBalancer error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult detail(String lbId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("LoadBalancerId", lbId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBELOADBALANCERATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            LoadBalancers loadBalancers = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), LoadBalancers.class);
            LbModel model = lbConvertor.convertModel(loadBalancers);
            model.setRegionId(this.getRegionId());
            return new BocloudResult(true, model, "查询成功");
        } catch (Exception e) {
            logger.error("detail loadBalancer error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(LbModel lb) {
        try {
            Map<String, String> params = new HashMap<>();
            //请求的Region
            params.put("RegionId", this.getRegionId());
            //负载均衡实例的类型
            params.put("Type", lb.getLoadBalancerType());
            //负载均衡实例所属VPC内的子网ID。负载均衡实例的私网IP地址将从该子网网段中分配。
            params.put("SubnetId", lb.getSubnetUuid());
            //负载均衡实例的规格
            params.put("LoadBalancerSpec", lb.getSpec());
            setParams(lb, params, "create");

            BocloudResult<JSONObject> response = operation(VolcengineConst.CREATELOADBALANCER, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return waitLoadBalance(result.getString("RequestId"), result.getString("LoadBalancerId"), "Active", "创建");
        } catch (Exception e) {
            logger.error("create loadBalancer error:", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult update(LbModel lb) {
        try {
            Map<String, String> params = new HashMap<>();
            //负载均衡实例ID。
            params.put("LoadBalancerId", lb.getLbUuid());
            setParams(lb, params, "update");
            if (StringUtils.isNotBlank(lb.getSpec())) {
                params.put("LoadBalancerSpec", lb.getSpec());
            }
            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYLOADBALANCERATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return new BocloudResult(true, this.detail(lb.getLbUuid()).getData(), "修改成功");
        } catch (Exception e) {
            logger.error("update loadBalancer error:", e);
            return new BocloudResult(false, "修改失败");
        }
    }


    /**
     * @param params
     * @return void
     * @Author: lxy
     * @Description: 设置非必填参数
     * @Date 2021/10/11 16:48
     * @Param lb
     * @throws:
     **/
    private void setParams(LbModel lb, Map<String, String> params, String action) {
        /**
         *负载均衡实例的名称。
         * ● 能以http://或https://开头。
         * ● 必须以字母或中文开头，可包含数字、点号（.）、下划线（_）和短横线（-）。
         * ● 长度限制为1 ～ 128个字符。
         * ● 不填默认为实例ID。
         */
        if (StringUtils.isNotBlank(lb.getName())) {
            params.put("LoadBalancerName", lb.getName());
        }
        /**
         *	负载均衡实例的描述。
         * ● 不能以http://或https://开头。
         * ● 必须以字母或中文开头，可包含数字、英文逗号（,）、点号（.）、下划线（_）、空格（ ）、等号（=）、短横线（-）、中文逗号（，）、中文句号（。）
         * ● 长度限制为1 ～ 255个字符。
         * ● 不填默认空字符串。
         */
        if (StringUtils.isNotBlank(lb.getRemark())) {
            params.put("Description", lb.getRemark());
        }
        /**
         * 负载均衡实例在控制台上修改保护的状态。取值：
         * ● NonProtection：不开启控制台修改保护功能，表示允许通过控制台修改实例或删除实例。
         * ● ConsoleProtection：开启控制台修改保护功能，表示禁止通过控制台修改实例或删除实例。
         * ● 不填默认为空，表示不开启控制台修改保护功能。
         */
        if (StringUtils.isNotBlank(lb.getModificationProtection())) {
            params.put("ModificationProtectionStatus", lb.getModificationProtection());
        }
        /**
         * 设置修改保护状态的原因。
         * ● 必须以字母或中文开头，支持数字、半角句号（.）、下划线（_）和短划线（-）。
         * ● 长度限制为1 ~ 80个字符。
         * 说明： ：仅在参数ModificationProtectionStatus配置为ConsoleProtection时有效
         */
        if (StringUtils.isNotBlank(lb.getModificationProtectionReason())) {
            params.put("ModificationProtectionReason", lb.getModificationProtectionReason());
        }
        if ("create".equalsIgnoreCase(action)) {
            /**
             *负载均衡实例所属的VPC ID。
             * ● 不填默认为子网所属的VPC ID。
             */
            if (StringUtils.isNotBlank(lb.getVpcUuid())) {
                params.put("VpcId", lb.getVpcUuid());
            }

            /**
             * 	指定负载均衡实例的私网IP地址，该地址必须包含在所选子网网段中，且未被占用。
             * ● 不填私网IP地址将从子网网段中自动分配。
             */
            if (StringUtils.isNotBlank(lb.getVipAddress())) {
                params.put("EniAddress", lb.getVipAddress());
            }

            /**
             * 负载均衡实例计费方式，当前仅支持按量付费。
             * ● 2：按量付费（默认值）
             */
            if (StringUtils.isNotBlank(lb.getInternetChargeType())) {
                params.put("LoadBalancerBillingType", lb.getInternetChargeType());
            }
            /**
             * 公网负载均衡实例的公网IP线路类型，取值：
             * ● ChinaMobile：中国移动。
             * ● ChinaTelecom：中国电信。
             * ● ChinaUnicom：中国联通。
             * ● BGP：BGP多线（默认值）。
             */
            if (StringUtils.isNotBlank(lb.getISP())) {
                params.put("EipBillingConfig.ISP", lb.getISP());
            }
            /**
             *	公网IP的带宽峰值，取值：1 ~ 500，默认值为1，单位：Mbps
             */
            if (null != lb.getBroadbandValue()) {
                params.put("EipBillingConfig.Bandwidth", String.valueOf(lb.getBroadbandValue()));
            }
            /**
             * 公网IP的付费方式。取值：
             * ● 2：按量计费（默认值）。
             */
            if (null != lb.getFipBillingType()) {
                params.put("EipBillingConfig.EipBillingType", String.valueOf(lb.getFipBillingType()));
            }
        }
    }

    @Override
    public BocloudResult remove(String lbId) {
        try {
            Map<String, String> params = new HashMap<>();
            //负载均衡实例ID。
            params.put("LoadBalancerId", lbId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DELETELOADBALANCER, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("remove loadBalancer error:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult createListener(LbListenerModel lbListenerModel) {
        try {
            Map<String, String> params = new HashMap<>();
            //负载均衡实例的ID。
            params.put("LoadBalancerId", lbListenerModel.getLbUuid());
            //监听器的协议。取值：● TCP● UDP● HTTP● HTTPS
            params.put("Protocol", lbListenerModel.getProtocol());
            //监听器接收请求使用的端口。取值： 1-65535。
            params.put("Port", String.valueOf(lbListenerModel.getPort()));
            //监听器关联的后端服务器组 ID。
            params.put("ServerGroupId", lbListenerModel.getDefaultPoolUuid());
            setListenerParams(lbListenerModel, params);

            BocloudResult<JSONObject> response = operation(VolcengineConst.CREATELISTENER, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return waitListener(result.getString("RequestId"), result.getString("ListenerId"), "Active", "创建");
        } catch (Exception e) {
            logger.error("create Listener error:", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult updateListener(LbListenerModel lbListenerModel) {
        try {
            Map<String, String> params = new HashMap<>();
            //监听器ID
            params.put("ListenerId", lbListenerModel.getListenerUuid());
            setListenerParams(lbListenerModel, params);
            if (null != lbListenerModel.getDefaultPoolUuid()) {
                /**
                 * 监听器关联的后端服务组ID。
                 * ● 对于HTTP/HTTPS监听器，此参数代表关联的默认后端服务器组ID。
                 * ● 2021年8月30日后，七层监听器上线默认后端服务器组功能，编辑存量七层监听器时，此参数为必选，否则将无法修改监听器的其他属性。
                 */
                params.put("ServerGroupId", lbListenerModel.getDefaultPoolUuid());
            }
            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYLISTENERATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            return new BocloudResult(true, this.detailListener(lbListenerModel.getListenerUuid()).getData(), "修改成功");
        } catch (Exception e) {
            logger.error("update Listener error:", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    /**
     * @param params
     * @return void
     * @Author: lxy
     * @Description: 设置监听非必填参数
     * @Date 2021/10/12 15:14
     * @Param lbListenerModel
     * @throws:
     **/
    private void setListenerParams(LbListenerModel lbListenerModel, Map<String, String> params) throws Exception {
        /**
         *监听器的名称。
         * ● 不能以http://或https://开头。
         * ● 必须以字母或中文开头，可包含数字、点号（.）、下划线（_）和短横线（-）。
         * ● 长度限制在1-128字符之间。
         * ● 不填默认将“协议-端口”作为监听器名称。
         */
        if (null != lbListenerModel.getName()) {
            params.put("ListenerName", lbListenerModel.getName());
        }
        /**
         *监听器使用的调度算法。取值：
         * ● wrr （默认值）：加权轮询，权重值越高的后端服务器，被轮询到的次数（概率）越高。
         * ● wlc：加权最小连接数，在最少连接数的基础上，根据服务器的不同处理能力，给每个服务器分配不同的权重，使其能够接受相应权值数的服务请求。
         * ● sh：源地址哈希，基于源IP地址的一致性哈希，相同的源地址会调度到相同的后端服务器。
         */
        if (null != lbListenerModel.getLbScheduler()) {
            params.put("Scheduler", lbListenerModel.getLbScheduler());
        }
        //是否启用监听器。取值:● on：开启（默认值）。● off：不开启。-- 暂未存储
//        if (null != lbListenerModel.getName()) {
//            params.put("Enabled", lbListenerModel.getScheduler());
//        }
        /**
         * 监听器的连接超时时间，当前仅针对TCP/UDP监听器生效。取值：
         * ● TCP协议：10-900秒，默认为900秒。
         * ● UDP协议：1-300秒，默认为90秒。
         */
        if (null != lbListenerModel.getConnLimit()) {
            params.put("EstablishedTimeout", String.valueOf(lbListenerModel.getConnLimit()));
        }
        /**
         * 监听器关联的证书 ID。
         * ● 如果监听器协议为HTTPS，此参数为必选。
         * ● 如果监听器协议为TCP/UDP/HTTP，此参数不生效。
         */
        JSONArray certificates = lbListenerModel.getCertificates();
        if (!ListTool.isEmpty(certificates)) {
            JSONObject json = JSONObject.parseObject(JSONObject.toJSONString(certificates.get(0)));
            if (null != json.getString("CertificateId")) {
                params.put("CertificateId", json.getString("CertificateId"));
            }
        }
        //是否开启访问控制功能。取值：● on：开启。● off ：不开启（默认值）。
        if (null != lbListenerModel.getAclStatus()) {
            params.put("AclStatus", lbListenerModel.getAclStatus() ? "on" : "off");
        }
        /**
         * 	访问控制的方式。取值:
         * ● white ： 白名单方式。表示监听器仅转发来自所选访问控制策略组中设置的IP地址或地址段的请求。
         * ● 如果所选策略组中没有添加任何IP，则监听器不会转发任何请求。
         * ● black（默认值）：黑名单方式。表示仅拒绝来自所选访问控制策略组中设置的IP地址或地址段的请求。
         * ● 如果所选策略组中没有添加任何IP，则监听器会转发全部请求。。
         * 说明： 当AclStatus参数配置为 on 时，AclType和AclIds.N为必填。
         */
        if (null != lbListenerModel.getAclType()) {
            params.put("AclType", lbListenerModel.getAclType());
        }
        /**
         * 监听器绑定的访问控制策略组ID。
         * ● 参数 - N：表示访问控制策略组的序号，数量上限为5个。
         * ● 多个访问控制策略组ID间使用“&”分隔。
         * 说明： 当AclStatus参数配置为 on 时，AclType和AclIds.N为必填。
         */
        if (null != lbListenerModel.getAclId()) {
            params.put("AclIds.1", lbListenerModel.getAclId());
        }
        /**
         * 监听器的描述。
         * ● 不能以http://或https://开头。
         * ● 必须以字母或中文开头，可包含数字、英文逗号（,）、点号（.）、下划线（_）、空格（ ）、等号（=）、短横线（-）、中文逗号（，）、中文句号（。）* 长度限制为1 ～ 255个字符。
         * ● 不填默认为空字符串。
         */
        if (null != lbListenerModel.getRemark()) {
            params.put("Description", lbListenerModel.getRemark());
        }
        if (null != lbListenerModel.getConfiguration()) {
            JSONObject conf = JSONObject.parseObject(JSONObject.toJSONString(lbListenerModel.getConfiguration()));
            /**
             * 监听器是否开启健康检查功能。取值：
             * ● on ：开启（默认值）。
             * ● off：不开启。
             * 说明： 以下参数仅当本参数的值为 on 时有效。
             */
            String enabled = conf.getString("Enabled");
            if (null != enabled) {
                params.put("HealthCheck.Enabled", enabled);
            }
            if ("on".equalsIgnoreCase(enabled)) {

                //执行健康检查的时间间隔，取值：1 ～ 300 ，默认值为2，单位：秒。
                if (null != conf.getString("Interval")) {
                    params.put("HealthCheck.Interval", conf.getString("Interval"));
                }
                /**
                 * 健康检查的响应超时时间，表示如果后端服务器在指定的时间内没有正确响应，则判定为健康检查“异常”。
                 * 取值：1 ～ 60，默认值为2，单位：秒。
                 * 说明： 响应超时时间需小于检查时间间隔。
                 */
                if (null != conf.getString("Timeout")) {
                    params.put("HealthCheck.Timeout", conf.getString("Timeout"));
                }
                /**
                 * 健康检查的健康阈值，表示连续执行指定次数的健康检查，结果均为“正常”的后端服务器将被判定为健康检查“正常”。
                 * 取值： 2 ～ 10， 默认值为3，单位：次。
                 */
                if (null != conf.getString("HealthyThreshold")) {
                    params.put("HealthCheck.HealthyThreshold", conf.getString("HealthyThreshold"));
                }
                /**
                 * 健康检查的不健康阈值，表示连续执行指定次数的健康检查，结果均为“异常”的后端服务器将被判定为健康检查异常。
                 * 取值： 2 ～ 10， 默认值为3，单位：次。
                 */
                if (null != conf.getString("UnHealthyThreshold")) {
                    params.put("HealthCheck.UnHealthyThreshold", conf.getString("UnHealthyThreshold"));
                }

                if ("HTTP".equalsIgnoreCase(lbListenerModel.getProtocol()) || "HTTPS".equalsIgnoreCase(lbListenerModel.getProtocol())) {
                    /**
                     *监听器健康检查的方法。取值：
                     * ● GET（默认值）：服务器需支持GET方法。
                     * ● HEAD：服务器仅返回HEAD头部信息，可以降低后端开销，但要求服务器支持HEAD方法。
                     */
                    if (null != conf.getString("Method")) {
                        params.put("HealthCheck.Method", conf.getString("Method"));
                    }
                    /**
                     * 健康检查的域名，需配置为后端服务器上真实对外提供服务的地址。
                     * ● 需至少包含一个‘.’，且不允许以‘.’开头结尾。
                     * ● 单个字符串由字母、数字、‘-’、‘.’字符组成，‘-’不得出现在字符串的头部或尾部。
                     * ● 长度限制为1 ～ 128个字符。
                     * ● 不填则此参数为空，负载均衡默认使用各后端服务器的私网IP地址进行健康检查。
                     */
                    if (null != conf.getString("Domain")) {
                        params.put("HealthCheck.Domain", conf.getString("Domain"));
                    }
                    /**
                     * 健康检查的路径，需配置为后端服务器上真实对外提供服务的地址。
                     * ● 必须以字符‘/’开头。* 仅包含字母、数字、‘-’、‘_’、‘/’、‘.’、‘%’、‘?’、‘#’、‘&’、‘＝’字符。
                     * ● 长度限制为1 ～ 128个字符。
                     * ● 不填默认为“/”。
                     */
                    if (null != conf.getString("URI")) {
                        params.put("HealthCheck.URI", conf.getString("URI"));
                    }
                    /**
                     * 健康检查正常的HTTP状态码。取值：
                     * ● http_2xx （默认值）
                     * ● http_3xx
                     * ● http_4xx
                     * ● http_5xx
                     * 多个状态码间用半角逗号“,”分隔。
                     */
                    if (null != conf.getString("HttpCode")) {
                        params.put("HealthCheck.HttpCode", conf.getString("HttpCode"));
                    }
                }
            }
        }
    }

    @Override
    public BocloudResult removeListener(String listenerId) {
        try {
            Map<String, String> params = new HashMap<>();
            //监听器ID。
            params.put("ListenerId", listenerId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DELETELISTENER, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("remove Listener error:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult listListener() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBELISTENERS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<Listeners> listeners = JSONArray.parseArray(JSONObject.toJSONString(result.get("Listeners")), Listeners.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询监听器列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBELISTENERS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<Listeners> listenersList = JSONArray.parseArray(JSONObject.toJSONString(result.get("Listeners")), Listeners.class);
                    listeners.addAll(listenersList);
                }
            }
            List<String> listenerIds = listeners.stream().map(Listeners::getListenerId).collect(Collectors.toList());
            List<LbListenerModel> lbModels = Lists.newArrayList();
            for (String id : listenerIds) {
                BocloudResult detail = this.detailListener(id);
                LbListenerModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), LbListenerModel.class);
                lbModels.add(model);
            }
            return new BocloudResult(true, lbModels, "查询成功");
        } catch (Exception e) {
            logger.error("list Listener error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult detailListener(String listenerId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("ListenerId", listenerId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBELISTENERATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            Listeners listener = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), Listeners.class);
            BocloudResult serverGroups = this.listPool();
            if (serverGroups.isFailed()) {
                return new BocloudResult(false, "获取后端服务器组异常，请检查");
            }
            List<LbPoolModel> lbPoolModels = JSONArray.parseArray(JSONObject.toJSONString(serverGroups.getData()), LbPoolModel.class);
            Map<String, LbPoolModel> lbPoolModelMap = lbPoolModels.stream().collect(Collectors.toMap(LbPoolModel::getPoolUuid, Function.identity(), (key1, key2) -> key2));
            LbListenerModel model = lbListenerConvertor.convertModel(listener);
            model.setRegionId(this.getRegionId());
            BocloudResult listenerHealth = this.detailListenerHealth(model.getListenerUuid());
            List<JSONObject> object = JSONArray.parseArray(JSONObject.toJSONString(listenerHealth.getData()), JSONObject.class);
            List<JSONObject> confs = Lists.newArrayList();
            for (JSONObject json : object) {
                JSONObject conf = JSONObject.parseObject(JSONObject.toJSONString(model.getConfiguration()));
                conf.putAll(json);
                if (object.size() != 1 && !json.getString("Status").equalsIgnoreCase("Disabled")) {
                    conf.put("poolName", lbPoolModelMap.get(model.getDefaultPoolUuid()).getName());
                }
                confs.add(conf);
            }
            if (!ListTool.isEmpty(confs)) {
                model.setConfiguration(confs);
            } else {
                JSONObject conf = JSONObject.parseObject(JSONObject.toJSONString(model.getConfiguration()));
                model.setConfiguration(Lists.newArrayList(conf));
            }
            BocloudResult listenerRules = this.listListenerRule(model.getListenerUuid());
            List<LbListenerRuleModel> lbListenerRuleModels = JSONArray.parseArray(JSONObject.toJSONString(listenerRules.getData()), LbListenerRuleModel.class);
            model.setListenerRuleModels(lbListenerRuleModels);
            return new BocloudResult(true, model, "查询成功");
        } catch (Exception e) {
            logger.error("detail" + listenerId + " Listener error:", e);
            return new BocloudResult(false, listenerId, "查询失败");
        }
    }

    private BocloudResult detailListenerHealth(String ListenerId) {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            params.put("ListenerId", ListenerId);

            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBELISTENERHEALTH, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<JSONObject> listenerHealth = JSONArray.parseArray(JSONObject.toJSONString(result.get("Results")), JSONObject.class);
            JSONObject finalResult = result;
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                JSONObject json = new JSONObject();
                json.put("ListenerStatus", result.getString("Status"));
                return new BocloudResult(true, Lists.newArrayList(json), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBELISTENERHEALTH, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<JSONObject> healths = JSONArray.parseArray(JSONObject.toJSONString(result.get("Results")), JSONObject.class);
                    listenerHealth.addAll(healths);
                }
            }
            listenerHealth.stream().forEach(
                    json -> json.put("ListenerStatus", finalResult.getString("Status"))
            );
            return new BocloudResult(true, listenerHealth, "查询成功");
        } catch (Exception e) {
            logger.error("list ListenerHealth error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult listPool() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBESERVERGROUPS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<ServerGroups> serverGroups = JSONArray.parseArray(JSONObject.toJSONString(result.get("ServerGroups")), ServerGroups.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询后端服务器组列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBESERVERGROUPS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<ServerGroups> serverGroupsList = JSONArray.parseArray(JSONObject.toJSONString(result.get("ServerGroups")), ServerGroups.class);
                    serverGroups.addAll(serverGroupsList);
                }
            }
            List<String> serverGroupIds = serverGroups.stream().map(ServerGroups::getServerGroupId).collect(Collectors.toList());
            Map<String, ServerGroups> serverGroupsMap = serverGroups.stream().collect(Collectors.toMap(ServerGroups::getServerGroupId, Function.identity(), (key1, key2) -> key2));

            List<LbPoolModel> lbPoolModels = Lists.newArrayList();
            for (String id : serverGroupIds) {
                BocloudResult detail = this.detailPool(id);
                LbPoolModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), LbPoolModel.class);
                model.setRemark(serverGroupsMap.get(id).getDescription());
                lbPoolModels.add(model);
            }
            return new BocloudResult(true, lbPoolModels, "查询成功");
        } catch (Exception e) {
            logger.error("list Pool error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult detailPool(String poolId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("ServerGroupId", poolId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBESERVERGROUPATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            ServerGroups serverGroups = JSONObject.parseObject(JSON.parse(JSONObject.toJSONString(data.get("Result"))).toString(), ServerGroups.class);
            LbPoolModel model = lbPoolConvertor.convertModel(serverGroups);
            return new BocloudResult(true, model, "查询成功");
        } catch (Exception e) {
            logger.error("detail pool error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult createPool(LbPoolModel poolModel) {
        try {
            Map<String, String> params = new HashMap<>();
            //后端服务器组所属的负载均衡实例的ID
            params.put("LoadBalancerId", poolModel.getLbUuid());
            setPoolParams(poolModel, params);

            BocloudResult<JSONObject> response = operation(VolcengineConst.CREATESERVERGROUP, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return new BocloudResult(true, this.detailPool(result.getString("ServerGroupId")).getData(), "创建成功");
        } catch (Exception e) {
            logger.error("create pool error:", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult updatePool(LbPoolModel poolModel) {
        try {
            Map<String, String> params = new HashMap<>();
            //后端服务器组所属的负载均衡实例的ID
            params.put("ServerGroupId", poolModel.getPoolUuid());
            setPoolParams(poolModel, params);

            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYSERVERGROUPATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return new BocloudResult(true, this.detailPool(poolModel.getPoolUuid()).getData(), "创建成功");
        } catch (Exception e) {
            logger.error("update pool error:", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    /**
     * @param params
     * @return void
     * @Author: lxy
     * @Description: 设置后端服务器组非必填参数
     * @Date 2021/10/12 18:09
     * @Param poolModel
     * @throws:
     **/
    private void setPoolParams(LbPoolModel poolModel, Map<String, String> params) {
        /**
         * 	后端服务器组名称。
         * ● 不能以http://或https://开头。
         * ● 必须以字母或中文开头，可包含数字、点号（.）、下划线（_）和短横线（-）。
         * ● 限制为1 ～ 128个字符。
         * ● 不填默认为后端服务器组ID。
         */
        if (null != poolModel.getName()) {
            params.put("ServerGroupName", poolModel.getName());
        }
        /**
         * 后端服务器组的描述。
         * ● 不能以http://或https://开头。
         * ● 必须以字母或中文开头，可包含数字、英文逗号（,）、点号（.）、下划线（_）、空格（ ）、等号（=）、短横线（-）、中文逗号（，）、中文句号（。）。
         * ● 长度限制为1 ～ 255个字符。
         * ● 不填默认为空字符串。
         */
        if (null != poolModel.getRemark()) {
            params.put("Description", poolModel.getRemark());
        }
        //在后端服务器组中添加服务器 -- 此处废弃，参照原生先创建服务器组，再添加服务器，不采用直接添加服务器的方式
//        if (!ListTool.isEmpty(memberModels)) {
//            params.put("ServerGroupName", poolModel.getName());
//        }
    }

    @Override
    public BocloudResult removePool(String poolId) {
        try {
            Map<String, String> params = new HashMap<>();
            //负载均衡实例ID。
            params.put("ServerGroupId", poolId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DELETESERVERGROUP, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("remove pool error:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult addMembers(String vServerGroupId, List<LbMemberModel> memberModels) {
        try {
            if (ListTool.isEmpty(memberModels)) {
                return new BocloudResult(false, "后端服务器组为空，请重新选择");
            }
            Map<String, String> params = new HashMap<>();
            //后端服务器组ID。
            params.put("ServerGroupId", vServerGroupId);
            //在后端服务器组中添加服务器
            setMemberParams(memberModels, params);
            BocloudResult<JSONObject> response = operation(VolcengineConst.ADDSERVERGROUPBACKENDSERVERS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return new BocloudResult(true, this.detailPool(vServerGroupId).getData(), "添加服务器成功");
        } catch (Exception e) {
            logger.error("add members error:", e);
            return new BocloudResult(false, "添加服务器失败");
        }
    }

    /**
     * @param params
     * @return void
     * @Author: lxy
     * @Description: 设置后端服务器
     * @Date 2021/10/13 9:55
     * @Param memberModels
     * @throws:
     **/
    private void setMemberParams(List<LbMemberModel> memberModels, Map<String, String> params) {
        for (int i = 1; i <= memberModels.size(); i++) {
            LbMemberModel lbMemberModel = memberModels.get(i - 1);
            /**
             * 云服务器实例ID或已绑定云服务器实例的网卡ID。
             * ● 参数 - N：表示后端服务器的序号。
             * ● 多个后端服务器ID间使用“&”分隔。
             */
            params.put("Servers." + i + ".InstanceId", lbMemberModel.getMemberUuid());
            /**
             * 后端服务器的类型，取值：
             * ● ecs：云服务器实例。
             * ● eni：网卡。
             * ● 参数 - N：表示后端服务器的序号。
             */
            params.put("Servers." + i + ".Type", lbMemberModel.getType());
            /**
             * 后端服务器的权重，取值：0 ～ 100。
             * ● 如果设置权重为0，表示不会将请求转发给该后端服务器。
             * ● 参数 - N：表示后端服务器的序号。
             * ● 不填默认为0。
             */
            if (null != lbMemberModel.getWeight()) {
                params.put("Servers." + i + ".Weight", String.valueOf(lbMemberModel.getWeight()));
            }
            //后端服务器的私网IP地址。● 参数 - N：表示后端服务器的序号
            params.put("Servers." + i + ".Ip", lbMemberModel.getAddress());
            //后端服务器接收请求的端口号，取值： 1-65535 。● 参数 - N：表示后端服务器的序号。
            params.put("Servers." + i + ".Port", String.valueOf(lbMemberModel.getPort()));
            /**
             * 后端服务器的描述。
             * ● 参数 - N：表示后端服务器的序号。
             * ● 不能以http://或https://开头。
             * ● 必须以字母或中文开头，可包含数字、英文逗号（,）、点号（.）、下划线（_）、空格（ ）、等号（=）、短横线（-）、中文逗号（，）、中文句号（。）
             * ● 长度限制为1 ～ 255个字符。
             * ● 不填默认为空字符串。
             */
            if (StringUtils.isNotBlank(lbMemberModel.getRemark())) {
                params.put("Servers." + i + ".Description", lbMemberModel.getRemark());
            }
        }
    }

    @Override
    public BocloudResult removeMembers(String vServerGroupId, List<LbMemberModel> memberModels) {
        try {
            if (ListTool.isEmpty(memberModels)) {
                return new BocloudResult(false, "后端服务器组为空，请重新选择");
            }
            Map<String, String> params = new HashMap<>();
            //后端服务器组ID。
            params.put("ServerGroupId", vServerGroupId);
            //需要删除的后端服务器ID
            for (int i = 1; i <= memberModels.size(); i++) {
                params.put("ServerIds." + i, memberModels.get(i - 1).getMemberUuid());
            }
            BocloudResult<JSONObject> response = operation(VolcengineConst.REMOVESERVERGROUPBACKENDSERVERS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return new BocloudResult(true, "删除服务器成功");
        } catch (Exception e) {
            logger.error("remove members error:", e);
            return new BocloudResult(false, "删除服务器失败");
        }
    }

    @Override
    public BocloudResult updateMember(LbMemberModel mbModel) {
        try {
            Map<String, String> params = new HashMap<>();
            //后端服务器组所属的负载均衡实例的ID
            params.put("ServerGroupId", mbModel.getPoolUuid());
            params.put("Servers.1.ServerId", mbModel.getMemberUuid());
            params.put("Servers.1.Port", String.valueOf(mbModel.getPort()));
            params.put("Servers.1.Weight", String.valueOf(mbModel.getWeight()));
            if (StringUtils.isNotBlank(mbModel.getRemark())) {
                params.put("Servers.1.Description", mbModel.getRemark());

            }
            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYSERVERGROUPATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return new BocloudResult(true, this.detailPool(mbModel.getPoolUuid()).getData(), "修改成功");
        } catch (Exception e) {
            logger.error("update pool error:", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    public BocloudResult waitLoadBalance(String jobId, String instanceId, String status, String message) {
        BocloudResult result = new BocloudResult(false, message + "失败");
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();

        while (flag) {
            BocloudResult detail = this.detail(instanceId);
            LbModel lbModel = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), LbModel.class);
            String lbStatus = lbModel.getStatus();

            if (status.equalsIgnoreCase(lbStatus)) {
                result = new BocloudResult(true, lbModel, message + "成功");
                flag = false;
                break;
            } else {
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                }
            }
            Long endTime = System.currentTimeMillis();
            // 等待三分钟
            if (endTime - startTime > 20 * 60 * 1000) {
                flag = false;
                break;
            }
        }
        logger.info("wait job end ==  jobId :" + jobId);
        return result;
    }

    public BocloudResult waitListener(String jobId, String instanceId, String status, String message) {
        BocloudResult result = new BocloudResult(false, message + "失败");
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();

        while (flag) {
            BocloudResult detail = this.detailListener(instanceId);
            LbListenerModel lbListenerModel = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), LbListenerModel.class);
            String lbStatus = lbListenerModel.getStatus();

            if (status.equalsIgnoreCase(lbStatus)) {
                result = new BocloudResult(true, lbListenerModel, message + "成功");
                flag = false;
                break;
            } else {
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                }
            }
            Long endTime = System.currentTimeMillis();
            // 等待三分钟
            if (endTime - startTime > 20 * 60 * 1000) {
                flag = false;
                break;
            }
        }
        logger.info("wait job end ==  jobId :" + jobId);
        return result;
    }

    @Override
    public BocloudResult listListenerRule(String policyId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("ListenerId", policyId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBERULES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject object = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));

            List<Rules> rules = JSONArray.parseArray(JSONObject.toJSONString(object.get("Rules")), Rules.class);
            List<LbListenerRuleModel> model = lbListenerRuleConvertor.convertList(rules);
            model.stream().forEach(
                    ruleModel -> {
                        ruleModel.setRegionId(this.getRegionId());
                        ruleModel.setListenerUuid(policyId);
                    }
            );
            return new BocloudResult(true, model, "查询成功");
        } catch (Exception e) {
            logger.error("list listener rule error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult createListenerRule(LbListenerRuleModel lb) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("ListenerId", lb.getListenerUuid());
            params.put("Rules.1.ServerGroupId", lb.getPoolUuid());
            setListenerRuleParms(params, lb);

            BocloudResult<JSONObject> response = operation(VolcengineConst.CREATERULES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject object = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<String> ruleIds = JSONArray.parseArray(JSONObject.toJSONString(object.get("RuleIds")), String.class);
            lb.setRuleUuid(ruleIds.get(0));
            lb.setRegionId(this.getRegionId());
            return new BocloudResult(true, lb, "添加成功");
        } catch (Exception e) {
            logger.error("create listener rule error:", e);
            return new BocloudResult(false, "添加失败");
        }
    }

    @Override
    public BocloudResult updateListenerRule(LbListenerRuleModel lb) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("ListenerId", lb.getListenerUuid());
            params.put("Rules.1.RuleId", lb.getRuleUuid());
            if (StringUtils.isNotBlank(lb.getPoolUuid())) {
                params.put("Rules.1.ServerGroupId", lb.getPoolUuid());
            }

            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYRULES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            return new BocloudResult(true, lb, "修改成功");
        } catch (Exception e) {
            logger.error("update listener rule error:", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    /**
     * @param lb
     * @return void
     * @Author: lxy
     * @Description: 转发规则参数
     * @Date 2021/10/19 14:57
     * @Param params
     * @throws:
     **/
    private void setListenerRuleParms(Map<String, String> params, LbListenerRuleModel lb) {
        /**
         * 转发规则的域名，支持泛域名和精确域名。
         * ● 参数 - N：表示转发规则的序号，取值1 ～ 50。
         * ● 多个转发规则间使用“&”分隔。
         *
         * 规范：
         * ● 需至少包含一个‘.’，且不允许以‘.’开头结尾。
         * ● 仅允许包含字母、数字、‘.’、‘-‘、‘*’。* 长度限制为1 ～ 128个字符。
         * ● 泛域名：使用“*”代替1个或多个字符。“*”必须在域名开头或结尾，同一条域名中“*”不能出现两次，“*”前后不能有除了.以外的字符。合法示例：*.com、www.test.*。非法示例：*aaa.com、www.*.com
         * ● 精确域名：符合域名规范的精确域名，例如：www.test.com。
         *
         * 说明： Rules.N.Domain和 Rules.N.Url两者至少需指定一个，且Rules.N.Domain和 Rules.N.Url的组合在同一个监听器内必须唯一。
         */
        if (null != lb.getRuleValue()) {
            params.put("Rules.1.Domain", lb.getRuleValue());
        }
        /**
         * 转发规则的URL，仅支持绝对路径。
         * ● 参数 - N：表示转发规则的序号，取值1 ～ 50。
         * ● 多个转发规则间使用“&”分隔。
         *
         * 规范：
         * ● 必须以正斜线“/”开头。
         * ● 仅允许包含字母、数字、‘-’、‘_’、‘/’、‘.’、‘%’、‘?’、‘#’、‘&’、‘＝’等字符。
         * ● 长度限制为1 ～ 128个字符。
         * ● 若指定Domain，则Url不填默认为“/”。
         *
         * 说明：
         * ● Rules.N.Domain和Rules.N.Url两者至少需指定一个，且Rules.N.Domain和 Rules.N.Url的组合在同一个监听器内必须唯一。
         * ● 当Rules.N.Domain设置为空时，Rules.N.Url不允许只为正斜线“/”。
         */
        if (null != lb.getRuleValue()) {
            params.put("Rules.1.Url", lb.getUrl());
        }
    }

    @Override
    public BocloudResult removeListenerRule(String policyId, String ruleId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("ListenerId", policyId);
            params.put("RuleIds.1", ruleId);

            BocloudResult<JSONObject> response = operation(VolcengineConst.DELETERULES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.CLB);
            if (response.isFailed()) {
                return response;
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("remove listener rule error:", e);
            return new BocloudResult(false, "删除失败");
        }
    }
}
