package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ElasticIpModel;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.VolEipConvertor;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.cmp.provider.volmodel.EipAddresses;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.volcengine.helper.Utils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: VolcengineElasticIpProvider
 * @Description: 公网ip
 * @Author lxy
 * @Date 2021/9/17 17:09
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineElasticIpProvider extends VolcengineProvider implements ElasticIpProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineElasticIpProvider.class);
    private BeanConvertor<ElasticIpModel, EipAddresses> eipConvertor = new VolEipConvertor();
    private static final int PAGE_SIZE = 100;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineElasticIpProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEEIPADDRESSES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<EipAddresses> eipAddresses = JSONArray.parseArray(JSONObject.toJSONString(result.get("EipAddresses")), EipAddresses.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询公网ip列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<KeypairModel>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBEEIPADDRESSES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<EipAddresses> eipAddressesList = JSONArray.parseArray(JSONObject.toJSONString(result.get("EipAddresses")), EipAddresses.class);
                    eipAddresses.addAll(eipAddressesList);
                }
            }
            List<ElasticIpModel> convertList = eipConvertor.convertList(eipAddresses);
            for (ElasticIpModel eip : convertList) {
                eip.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("list ElasticIp error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult detail(String allocationId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("AllocationId", allocationId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEEIPADDRESSATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            EipAddresses result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), EipAddresses.class);
            return new BocloudResult(true, eipConvertor.convertModel(result), "查询成功");
        } catch (Exception e) {
            logger.error("detail ElasticIp error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult allocate(ElasticIpModel model) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("BillingType", String.valueOf(2));
            setParams(model, params);
            BocloudResult<JSONObject> response = operation(VolcengineConst.ALLOCATEEIPADDRESS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return waitForJobStatus(null, result.getString("AllocationId"), "AVAILABLE", "创建");
        } catch (Exception e) {
            logger.error("allocate ElasticIp error:", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    public BocloudResult waitForJobStatus(String jobId, String instanceId, String status, String message) {
        BocloudResult result = new BocloudResult(false, message + "失败");
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();

        while (flag) {
            BocloudResult detail = this.detail(instanceId);
            ElasticIpModel elasticIpModel = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), ElasticIpModel.class);
            String serverStatus = elasticIpModel.getStatus();

            if (status.equalsIgnoreCase(serverStatus)) {
                result = new BocloudResult(true, elasticIpModel, message + "成功");
                flag = false;
                break;
            } else {
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                }
            }
            Long endTime = System.currentTimeMillis();
            // 等待三分钟
            if (endTime - startTime > 20 * 60 * 1000) {
                flag = false;
                break;
            }
        }
        return result;
    }

    /**
     * @param params
     * @return void
     * @Author: lxy
     * @Description: 非必传参数
     * @Date 2021/9/17 18:09
     * @Param model
     * @throws:
     **/
    private void setParams(ElasticIpModel model, Map<String, String> params) {
        //EIP的带宽峰值，取值范围：1 ~ 500，默认值1，单位：Mbps。
        if (StringUtils.isNotBlank(model.getBandwidth())) {
            params.put("Bandwidth", model.getBandwidth());
        }
        //线路类型。●BGP（存量为BGPMultiLine：BGP多线（默认值）。●ChinaMobile：中国移动。●ChinaUnicom：中国联通。●ChinaTelecom：中国电信。
//        if (StringUtils.isNotBlank(model.getBandwidth())) {
//            params.put("Bandwidth", model.getBandwidth());
//        }
        /**
         * EIP的名称。
         * ●不能以http://或https://开头。
         * ●必须以字母或中文开头，可包含数字、点号（.）、下划线（_）和短划线（-）。
         * ●长度限制为1 ~ 128个字符。
         * ●不填默认是EIP的ID。
         */
        if (StringUtils.isNotBlank(model.getName())) {
            params.put("Name", model.getName());
        }
        /**
         * EIP的描述。
         * ●不能以http://或https://开头。
         * ●必须以字母或中文开头，可包含数字、英文逗号（,）、点号（.）、下划线（_）、空格（ ）、等号（=）、短横线（-）、中文逗号（，）、中文句号（。）。
         * ●长度限制为1 ~ 255个字符。
         * ●不填默认空字符串。
         */
        if (StringUtils.isNotBlank(model.getRemark())) {
            params.put("Description", model.getRemark());
        }
    }

    @Override
    public BocloudResult associate(ElasticIpModel model) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("AllocationId", model.getElasticIpUuid());
            if (StringUtils.isNotBlank(model.getPrivateIpAddress())) {
                params.put("PrivateIpAddress", model.getPrivateIpAddress());
            }
            params.put("InstanceId", model.getInstanceId());
            /**
             * 要绑定EIP的实例的类型。
             * ●Nat：NAT网关。
             * ●NetworkInterface: 弹性网卡。
             * ●ClbInstance: 负载均衡CLB。
             * ●EcsInstance：云服务器ECS。
             */
            params.put("InstanceType", model.getInstanceType());
            BocloudResult<JSONObject> response = operation(VolcengineConst.ASSOCIATEEIPADDRESS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "请求成功，关联失败，请检查原因");
            }
            return new BocloudResult(true, "关联成功");

        } catch (Exception e) {
            logger.error("associate ElasticIp error:", e);
            return new BocloudResult(false, "关联失败");
        }
    }

    @Override
    public BocloudResult disassociate(ElasticIpModel model) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("AllocationId", model.getElasticIpUuid());
            if (StringUtils.isNotBlank(model.getInstanceId())) {
                params.put("InstanceId", model.getInstanceId());
            }
            /**
             * 要绑定EIP的实例的类型。
             * ●Nat：NAT网关。
             * ●NetworkInterface: 弹性网卡。
             * ●ClbInstance: 负载均衡CLB。
             * ●EcsInstance：云服务器ECS。
             */
            params.put("InstanceType", model.getInstanceType());
            BocloudResult<JSONObject> response = operation(VolcengineConst.DISASSOCIATEEIPADDRESS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "请求成功，解绑失败，请检查原因");
            }
            return new BocloudResult(true, "解绑成功");

        } catch (Exception e) {
            logger.error("disassociate ElasticIp error:", e);
            return new BocloudResult(false, "解绑失败");
        }
    }

    @Override
    public BocloudResult release(String allocationId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("AllocationId", allocationId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.RELEASEEIPADDRESS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "请求成功，释放失败，请检查原因");
            }
            return new BocloudResult(true, "释放成功");

        } catch (Exception e) {
            logger.error("release ElasticIp error:", e);
            return new BocloudResult(false, "释放失败");
        }
    }

    @Override
    public BocloudResult update(ElasticIpModel model) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("AllocationId", model.getElasticIpUuid());
            setParams(model, params);
            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYEIPADDRESSATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "请求成功，编辑失败，请检查原因");
            }
            Thread.sleep(3000);
            return new BocloudResult(true, this.detail(model.getElasticIpUuid()).getData(), "修改成功");

        } catch (Exception e) {
            logger.error("update ElasticIp error:", e);
            return new BocloudResult(false, "修改失败");
        }
    }
}
