package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.VolNetworkConvertor;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.cmp.provider.volmodel.NetworkInterfaceSets;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.volcengine.helper.Utils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: VolcengineNetworkProvider
 * @Description: 网卡
 * @Author lxy
 * @Date 2021/9/16 17:52
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineNetworkCardProvider extends VolcengineProvider implements NetworkCardProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineNetworkCardProvider.class);
    private BeanConvertor<NetworkCardModel, NetworkInterfaceSets> networkConvertor = new VolNetworkConvertor();
    private static final int PAGE_SIZE = 100;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineNetworkCardProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult list() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBENETWORKINTERFACES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<NetworkInterfaceSets> networkInterfaceSets = JSONArray.parseArray(JSONObject.toJSONString(result.get("NetworkInterfaceSets")), NetworkInterfaceSets.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询网卡列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<KeypairModel>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBEIMAGES, VolcengineConst.DESCRIBENETWORKINTERFACES, Utils.mapToPairList(params), VolcengineConst.service.VPC);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<NetworkInterfaceSets> networkInterfaceSetsList = JSONArray.parseArray(JSONObject.toJSONString(result.get("NetworkInterfaceSets")), NetworkInterfaceSets.class);
                    networkInterfaceSets.addAll(networkInterfaceSetsList);
                }
            }

            List<NetworkCardModel> convertList = networkConvertor.convertList(networkInterfaceSets);
            for (NetworkCardModel networkModel : convertList) {
                networkModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("list NetworkCard error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult remove(String networkCardId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("NetworkInterfaceId", networkCardId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DELETENETWORKINTERFACE, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "请求成功，删除失败，请检查原因");
            }
            return new BocloudResult(true, "删除成功");

        } catch (Exception e) {
            logger.error("remove networkcard error:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult create(NetworkCardModel networkCardModel) {
        try {
            Map<String, String> params = new HashMap<>();
            //辅助网卡所在子网的ID。
            params.put("SubnetId", networkCardModel.getSubnetUuid());
            /**
             * 辅助网卡加入的安全组列表。
             * ●安全组必须与辅助网卡在同一个VPC内。
             * ●参数 - N：表示安全组ID的序号，单次调用数量上限16个ID。
             * ●多个安全组用&分隔。
             */
            List<String> securityGroupIds = networkCardModel.getSecurityGroupIds();
            for (int i = 1; i <= securityGroupIds.size(); i++) {
                params.put("SecurityGroupIds." + i, securityGroupIds.get(i - 1));
            }
            setParams(networkCardModel, params);
            params.put("PortSecurityEnabled", "true");
            BocloudResult<JSONObject> response = operation(VolcengineConst.CREATENETWORKINTERFACE, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            return new BocloudResult(true, this.detail(result.getString("NetworkInterfaceId")).getData(), "创建成功");
        } catch (Exception e) {
            logger.error("create NetworkCard error:", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    /**
     * @param params
     * @return void
     * @Author: lxy
     * @Description: 设置非必填参数
     * @Date 2021/9/18 16:38
     * @Param networkCardModel
     * @throws:
     **/
    private void setParams(NetworkCardModel networkCardModel, Map<String, String> params) {
        /**
         * 辅助网卡的主私有 IP 地址。
         * ●如果指定，必须是对应子网中的一个IP地址。
         * ●如果不指定，从对应子网中自动分配一个 IP 地址。
         */
        if (StringUtils.isNotBlank(networkCardModel.getIpAddress())) {
            params.put("PrimaryIpAddress", networkCardModel.getIpAddress());
        }
        /**
         * 辅助网卡名称。
         * ●不能以http://或https://开头。
         * ●必须以字母或中文开头，可包含数字、点号（.）、下划线（_）和短划线（-）。
         * ●长度限制为1 ~ 128个字符。
         * ●不填默认是辅助网卡的ID。
         */
        if (StringUtils.isNotBlank(networkCardModel.getName())) {
            params.put("NetworkInterfaceName", networkCardModel.getName());
        }
        /**
         * 辅助网卡的描述信息。
         * ●不能以http://或https://开头。
         * ● 必须以字母或中文开头，可包含数字、英文逗号（,）、点号（.）、下划线（_）、空格（ ）、等号（=）、短横线（-）、中文逗号（，）、中文句号（。）。
         * ●长度限制为1 ~ 255个字符。
         * ●不填默认空字符串。
         */
        if (StringUtils.isNotBlank(networkCardModel.getRemark())) {
            params.put("Description", networkCardModel.getRemark());
        }


    }

    @Override
    public BocloudResult detail(String networkId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("NetworkInterfaceId", networkId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBENETWORKINTERFACEATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            NetworkInterfaceSets networkInterfaceSet = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), NetworkInterfaceSets.class);
            NetworkCardModel networkModel = networkConvertor.convertModel(networkInterfaceSet);
            networkModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, networkModel, "查询成功");
        } catch (Exception e) {
            logger.error("detail NetworkCard error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult attachNetworkCard(NetworkCardModel networkCardModel) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("NetworkInterfaceId", networkCardModel.getNetworkcardUuid());
            params.put("InstanceId", networkCardModel.getInstanceId());
            BocloudResult<JSONObject> response = operation(VolcengineConst.ATTACHNETWORKINTERFACE, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "请求成功，挂载失败，请检查原因");
            }
            Thread.sleep(3000);
            return new BocloudResult(true, this.detail(networkCardModel.getNetworkcardUuid()).getData(), "挂载成功");
        } catch (Exception e) {
            logger.error("attach networkcard error:", e);
            return new BocloudResult(false, "挂载失败");
        }
    }

    @Override
    public BocloudResult detachNetworkCard(NetworkCardModel networkCardModel) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("NetworkInterfaceId", networkCardModel.getNetworkcardUuid());
            params.put("InstanceId", networkCardModel.getInstanceId());
            BocloudResult<JSONObject> response = operation(VolcengineConst.DETACHNETWORKINTERFACE, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "请求成功，卸载失败，请检查原因");
            }
            Thread.sleep(3000);
            return new BocloudResult(true, this.detail(networkCardModel.getNetworkcardUuid()).getData(), "卸载成功");

        } catch (Exception e) {
            logger.error("detach networkcard error:", e);
            return new BocloudResult(false, "卸载失败");
        }
    }

    @Override
    public BocloudResult modify(NetworkCardModel networkCardModel) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("NetworkInterfaceId", networkCardModel.getNetworkcardUuid());
            networkCardModel.setIpAddress(null);
            setParams(networkCardModel, params);
            List<String> securityGroupIds = networkCardModel.getSecurityGroupIds();
            if (!ListTool.isEmpty(securityGroupIds)) {
                for (int i = 1; i <= securityGroupIds.size(); i++) {
                    params.put("SecurityGroupIds." + i, securityGroupIds.get(i - 1));
                }
            }
            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYNETWORKINTERFACEATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "请求成功，编辑失败，请检查原因");
            }
            return new BocloudResult(true, this.detail(networkCardModel.getNetworkcardUuid()), "编辑成功");

        } catch (Exception e) {
            logger.error("update networkcard error:", e);
            return new BocloudResult(false, "编辑失败");
        }
    }
}
