package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.model.RouteModel;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.VolRouteConvertor;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.cmp.provider.volmodel.RouteEntries;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.volcengine.helper.Utils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: VolcengineRouteProvider
 * @Description: 路由条目
 * @Author lxy
 * @Date 2021/9/26 11:15
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineRouteProvider extends VolcengineProvider implements RouteProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineRouteProvider.class);
    private static final int PAGE_SIZE = 100;
    private BeanConvertor<RouteModel, RouteEntries> routeConvertor = new VolRouteConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineRouteProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(RouteModel model) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("RouteTableId", model.getRouteTableUuid());
            //路由条目的目标网段。
            params.put("DestinationCidrBlock", model.getDestination());
            /**
             * 自定义路由条目的下一跳的类型，取值：
             * ●Instance：ECS实例（默认值）。
             * ●NetworkInterface：辅助网卡。
             * ●NatGW：NAT网关。
             * ●VpnGW：VPN网关。
             */
            params.put("NextHopType", model.getTargetType());
            /**
             * 下一跳资源ID。
             * 注意：若NextHopType为NatGW，此处需传入NAT网关网卡的ID（非NAT网关绑定的弹性辅助网卡）。
             */
            params.put("NextHopId", model.getTargetUuid());

            setParams(model, params);
            BocloudResult<JSONObject> response = operation(VolcengineConst.CREATEROUTEENTRY, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);

            return new BocloudResult(true, this.detail(model.getRouteTableUuid(), result.getString("RouteEntryId")).getData(), "创建成功");

        } catch (Exception e) {
            logger.error("create Route error:", e);
            return new BocloudResult(false, "创建失败");
        }
    }


    /**
     * @param params
     * @return void
     * @Author: lxy
     * @Description: 设置非必填参数
     * @Date 2021/9/26 11:27
     * @Param model
     * @throws:
     **/
    private void setParams(RouteModel model, Map<String, String> params) {
        /**
         * 路由条目名称：
         * ●不能以http://或https://开头。
         * ●须以字母或中文开头，可包含数字、点号（.）、下划线（_）和短横线（-）。
         * ●长度限制为1 ~ 128个字符。
         * ●不填默认空字符串。
         */
        if (StringUtils.isNotBlank(model.getName())) {
            params.put("RouteEntryName", model.getName());
        }
        /**
         * 路由条目描述：
         * ●不能以http://或https://开头。
         * ●必须以字母或中文开头，可包含数字、英文逗号（,）、点号（.）、下划线（_）、空格（ ）、等号（=）、短横线（-）、中文逗号（，）、中文句号（。）。
         * ●长度限制为1 ~ 255个字符。
         * ●不填默认空字符串。
         */
//        if (StringUtils.isNotBlank(model.getRemark())) {
//            params.put("Description", model.getRemark());
//        }
    }

    @Override
    public BocloudResult modify(RouteModel model) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("RouteEntryId", model.getUuid());
            setParams(model, params);
            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYROUTEENTRY, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "请求成功，编辑失败，请检查原因");
            }
            Thread.sleep(3000);
            return new BocloudResult(true, this.detail(model.getRouteTableUuid(), model.getUuid()).getData(), "修改成功");

        } catch (Exception e) {
            logger.error("modify Route error:", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    @Override
    public BocloudResult delete(RouteModel model) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("RouteEntryId", model.getUuid());
            BocloudResult<JSONObject> response = operation(VolcengineConst.DELETEROUTEENTRY, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "请求成功，删除失败，请检查原因");
            }
            Thread.sleep(3000);
            return new BocloudResult(true, "删除成功");

        } catch (Exception e) {
            logger.error("delete Route error:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult list(String routeTableId) {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            params.put("routeTableId", routeTableId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEROUTEENTRYLIST, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<RouteEntries> routerList = JSONArray.parseArray(JSONObject.toJSONString(result.get("RouteEntries")), RouteEntries.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询路由列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<KeypairModel>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBEROUTEENTRYLIST, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<RouteEntries> routers = JSONArray.parseArray(JSONObject.toJSONString(result.get("RouteEntries")), RouteEntries.class);
                    routerList.addAll(routers);
                }
            }
            List<RouteModel> convertList = routeConvertor.convertList(routerList);
            for (RouteModel routeModel : convertList) {
                routeModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("list Route error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    private BocloudResult detail(String routeTableId, String routeEntryId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("routeTableId", routeTableId);
            params.put("RouteEntryId", routeEntryId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEROUTEENTRYLIST, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<RouteEntries> routerList = JSONArray.parseArray(JSONObject.toJSONString(result.get("RouteEntries")), RouteEntries.class);
            RouteModel routeModel = routeConvertor.convertModel(routerList.get(0));
            routeModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, routeModel, "查询成功");
        } catch (Exception e) {
            logger.error("detail Route error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }
}
