package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.model.RouteModel;
import com.bocloud.cmp.model.RouteTableModel;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.VolRouteTableConvertor;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.cmp.provider.volmodel.RouterTableList;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.volcengine.helper.Utils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @ClassName: VolcengineRouteTableProvider
 * @Description: 路由表
 * @Author lxy
 * @Date 2021/9/26 10:02
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineRouteTableProvider extends VolcengineProvider implements RouteTableProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineRouteTableProvider.class);
    private BeanConvertor<RouteTableModel, RouterTableList> routeTableConvertor = new VolRouteTableConvertor();
    private static final int PAGE_SIZE = 100;
    private Butler butler;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineRouteTableProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult list() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEROUTETABLELIST, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<RouterTableList> routerTableList = JSONArray.parseArray(JSONObject.toJSONString(result.get("RouterTableList")), RouterTableList.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询路由列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<KeypairModel>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBEROUTETABLELIST, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<RouterTableList> routerTableLists = JSONArray.parseArray(JSONObject.toJSONString(result.get("RouterTableList")), RouterTableList.class);
                    routerTableList.addAll(routerTableLists);
                }
            }
            List<RouteTableModel> convertList = routeTableConvertor.convertList(routerTableList);
            for (RouteTableModel routeTableModel : convertList) {
                routeTableModel.setRegionId(this.getRegionId());
                listRoute(routeTableModel);
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("list RouteTable error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    /**
     * @return void
     * @Author: lxy
     * @Description: 查询路由表下路由条目
     * @Date 2021/9/26 14:54
     * @Param routeTableModel
     * @throws:
     **/
    private void listRoute(RouteTableModel routeTableModel) {
        VolcengineRouteProvider routeProvider = new VolcengineRouteProvider(butler);
        BocloudResult result = routeProvider.list(routeTableModel.getUuid());
        List<RouteModel> routeModels = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), RouteModel.class);
        routeTableModel.setRouteModels(routeModels);
    }

    @Override
    public BocloudResult create(RouteTableModel model) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("VpcId", model.getVpcUuid());
            setParams(model, params);
            BocloudResult<JSONObject> response = operation(VolcengineConst.CREATEROUTETABLE, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);

            return new BocloudResult(true, this.detail(result.getString("RouteTableId")).getData(), "创建成功");

        } catch (Exception e) {
            logger.error("create RouteTable error:", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    /**
     * @param params
     * @return void
     * @Author: lxy
     * @Description: 设置非必填参数
     * @Date 2021/9/26 10:49
     * @Param model
     * @throws:
     **/
    private void setParams(RouteTableModel model, Map<String, String> params) {
        /**
         * 	路由表的名称。
         * ●不能以http://或https://开头
         * ●必须以字母或中文开头，可包含数字、点号（.）、下划线（_）和短横线（-）。
         * ●长度限制为1 ~ 128个字符。
         * ●不填默认是路由表的ID。
         */
        if (StringUtils.isNotBlank(model.getName())) {
            params.put("RouteTableName", model.getName());
        }
        /**
         * 路由表的描述信息。
         * ●不能以http://或https://开头。
         * ●必须以字母或中文开头，可包含数字、英文逗号（,）、点号（.）、下划线（_）、空格（ ）、等号（=）、短横线（-）、中文逗号（，）、中文句号（。）。
         * ●长度限制为1 ~ 255个字符。
         * ●不填默认空字符串。
         */
        if (StringUtils.isNotBlank(model.getRemark())) {
            params.put("Description", model.getRemark());
        }
    }

    @Override
    public BocloudResult detail(String routeTableId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("RouteTableId", routeTableId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEROUTETABLELIST, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<RouterTableList> routerTableList = JSONArray.parseArray(JSONObject.toJSONString(result.get("RouterTableList")), RouterTableList.class);
            List<RouteTableModel> convertList = routeTableConvertor.convertList(routerTableList);
            for (RouteTableModel routeTableModel : convertList) {
                routeTableModel.setRegionId(this.getRegionId());
                listRoute(routeTableModel);
            }
            return new BocloudResult(true, convertList.get(0), "查询成功");
        } catch (Exception e) {
            logger.error("detail RouteTable error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult modify(RouteTableModel model) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("RouteTableId", model.getUuid());
            setParams(model, params);
            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYROUTETABLEATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "请求成功，编辑失败，请检查原因");
            }
            Thread.sleep(3000);
            return new BocloudResult(true, this.detail(model.getUuid()).getData(), "修改成功");

        } catch (Exception e) {
            logger.error("modify RouteTable error:", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    @Override
    public BocloudResult delete(String routeTableId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("RouteTableId", routeTableId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DELETEROUTETABLE, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "请求成功，删除失败，请检查原因");
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("delete RouteTable error:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult associateSubnet(List<String> subnetUuids, String routeTableId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("RouteTableId", routeTableId);
            params.put("SubnetId", subnetUuids.get(0));
            BocloudResult<JSONObject> response = operation(VolcengineConst.ASSOCIATEROUTETABLE, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "请求成功，绑定失败，请检查原因");
            }
            return new BocloudResult(true, "绑定成功");
        } catch (Exception e) {
            logger.error("associateSubnet RouteTable error:", e);
            return new BocloudResult(false, "绑定失败");
        }
    }

    @Override
    public BocloudResult unAssociateSubnet(String subnetUuid, String routeTableId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("RouteTableId", routeTableId);
            params.put("SubnetId", subnetUuid);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DISASSOCIATEROUTETABLE, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")), JSONObject.class);
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "请求成功，解绑失败，请检查原因");
            }
            return new BocloudResult(true, "解绑成功");
        } catch (Exception e) {
            logger.error("unAssociateSubnet RouteTable error:", e);
            return new BocloudResult(false, "解绑失败");
        }
    }

    @Override
    public BocloudResult getByVpc(String vpcUuid) {
        try {
            BocloudResult list = list();
            if (list.isFailed()) {
                return new BocloudResult(false, list.getMessage());
            }
            List<RouteTableModel> models = (List<RouteTableModel>) list.getData();
            List<RouteTableModel> finalSgs = models.stream().filter(sg -> sg.getVpcUuid() != null)
                    .filter(routeTableModel -> routeTableModel.getVpcUuid().equals(vpcUuid)).collect(Collectors.toList());
            return new BocloudResult(true, finalSgs, "根据vpc查询路由表成功");
        } catch (Exception e) {
            logger.error("list route table by vpc error:", e);
            return new BocloudResult(false, "根据vpc查询路由表成功失败");
        }
    }
}
