package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.VolSecyrityRuleConvertor;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.cmp.provider.volmodel.Permissions;
import com.bocloud.cmp.provider.volmodel.SecurityGroups;
import com.bocloud.cmp.provider.volmodel.SecurityGroupsDes;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;
import com.volcengine.helper.Utils;
import com.volcengine.model.response.ResponseMetadata;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @ClassName: VolcengineSecurityGroupProvider
 * @Description: 安全组
 * @Author lxy
 * @Date 2021/9/13 18:24
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineSecurityGroupProvider extends VolcengineProvider implements SecurityGroupProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineSecurityGroupProvider.class);
    private BeanConvertor<SecurityGroupRuleModel, Permissions> secyrityRuleConvertor = new VolSecyrityRuleConvertor();
    private static final int PAGE_SIZE = 100;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineSecurityGroupProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(SecurityGroupModel group) {
        try {
            Map<String, String> params = new HashMap<>();
            //安全组所属VPC的ID。
            params.put("VpcId", group.getVpcUuid());
            setParams(group, params);
            BocloudResult<JSONObject> response = operation(VolcengineConst.CREATESECURITYGROUP, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            ResponseMetadata responseMetadata = JSONObject.parseObject(JSONObject.toJSONString(data.get("ResponseMetadata")), ResponseMetadata.class);
            if (null != responseMetadata.getError()) {
                return new BocloudResult(false, "SecurityGroup创建失败:" + responseMetadata.getError().getMessage());
            }
            JSONObject securityGroupJson = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String securityGroupId = securityGroupJson.getString("SecurityGroupId");
            Thread.sleep(2000);

            return detail(securityGroupId);
        } catch (Exception e) {
            logger.error("create SecurityGroup error:", e);
            return new BocloudResult(false, "SecurityGroup创建失败");
        }
    }

    /**
     * @param params
     * @return void
     * @Author: lxy
     * @Description: 设置安全组非必填信息--名称、描述
     * @Date 2021/9/14 9:49
     * @Param group
     * @throws:
     **/
    private void setParams(SecurityGroupModel group, Map<String, String> params) {
        /**
         * 安全组名称。
         * ● 不能以http://或https://开头。
         * ●必须以字母或中文开头，可包含数字、点号（.）、下划线（_）和短划线（-）。
         * ●长度限制为1 ~ 128个字符。
         * ●不填默认是安全组的ID。
         */
        if (StringUtils.isNoneBlank(group.getName())) {
            params.put("SecurityGroupName", group.getName());
        }
        /**
         * 	安全组描述信息。
         * ●不能以http://或https://开头。
         * ●必须以字母或中文开头，可包含数字、英文逗号（,）、点号（.）、下划线（_）、空格（ ）、等号（=）、短横线（-）、中文逗号（，）、中文句号（。）。
         * ●长度限制为1 ~ 255个字符。
         * ●不填默认空字符串。
         */
        if (StringUtils.isNoneBlank(group.getRemark())) {
            params.put("Description", group.getRemark());
        }
    }

    @Override
    public BocloudResult modify(SecurityGroupModel model) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("SecurityGroupId", model.getGroupUuid());
            setParams(model, params);
            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYSECURITYGROUPATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "修改SecurityGroup: " + model.getGroupUuid() + " 失败");
            }
            return new BocloudResult(true, this.detail(model.getGroupUuid()).getData(), "修改SecurityGroup: " + model.getGroupUuid() + " 成功");
        } catch (Exception e) {
            logger.error("update SecurityGroup error:", e);
            return new BocloudResult(false, "修改SecurityGroup失败，原因：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String sgId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("SecurityGroupId", sgId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBESECURITYGROUPATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            SecurityGroupsDes securityGroupsDes = JSONObject.parseObject(JSONObject.toJSONString(response.getData().get("Result")), SecurityGroupsDes.class);
            securityGroupsDes.setRequestId(this.getRegionId());
            return new BocloudResult(true, this.convertDes(securityGroupsDes), "SecurityGroup详情查询成功");
        } catch (Exception e) {
            logger.error("detail SecurityGroup error:" + e);
            return new BocloudResult(false, "SecurityGroup详情查询失败:" + e.getMessage());
        }

    }

    /**
     * @return com.bocloud.cmp.model.SecurityGroupModel
     * @Author: lxy
     * @Description: 单个安全组转换
     * @Date 2021/9/14 14:05
     * @Param securityGroupsDes
     * @throws:
     **/
    private SecurityGroupModel convertDes(SecurityGroupsDes securityGroupsDes) {
        if (null == securityGroupsDes) {
            return null;
        }
        SecurityGroupModel model = new SecurityGroupModel();
        model.setRemark(securityGroupsDes.getDescription());
        if (StringUtils.isBlank(securityGroupsDes.getSecurityGroupName())) {
            model.setName(securityGroupsDes.getSecurityGroupId());
        } else {
            model.setName(securityGroupsDes.getSecurityGroupName());
        }
        model.setGroupUuid(securityGroupsDes.getSecurityGroupId());
        model.setVpcUuid(securityGroupsDes.getVpcId());
        List<Permissions> permissions = securityGroupsDes.getPermissions();
        List<SecurityGroupRuleModel> securityGroupRuleModels = secyrityRuleConvertor.convertList(permissions);
        model.setModels(securityGroupRuleModels);
        return model;
    }

    @Override
    public BocloudResult list() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBESECURITYGROUPS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<SecurityGroups> securityGroupsList = JSONArray.parseArray(JSONObject.toJSONString(result.get("SecurityGroups")), SecurityGroups.class);
            if (ListTool.isEmpty(securityGroupsList)) {
                return new BocloudResult(true, "SecurityGroup查询成功,暂不存在安全组");
            }
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询安全组列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<KeypairModel>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBESECURITYGROUPS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<SecurityGroups> securityGroups = JSONArray.parseArray(JSONObject.toJSONString(result.get("SecurityGroups")), SecurityGroups.class);
                    securityGroupsList.addAll(securityGroups);
                }
            }

            List<SecurityGroupModel> models = Lists.newArrayList();
            for (SecurityGroups securityGroups : securityGroupsList) {
                SecurityGroupModel model = convert(securityGroups);
                models.add(model);
            }

            return new BocloudResult(true, models, "SecurityGroup查询成功");
        } catch (Exception e) {
            logger.error("list SecurityGroup error:" + e);
            return new BocloudResult(false, "SecurityGroup查询失败:" + e.getMessage());
        }
    }

    /**
     * @return void
     * @Author: lxy
     * @Description: 安全组转换
     * @Date 2021/9/14 14:05
     * @Param securityGroups
     * @throws:
     **/
    private SecurityGroupModel convert(SecurityGroups securityGroups) throws ParseException {
        SecurityGroupModel model = new SecurityGroupModel();
        String creationTime = securityGroups.getCreationTime();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
        if (null != creationTime && !"".equals(creationTime)) {
            model.setCreateTime(sdf.parse(creationTime));
        }
        model.setRemark(securityGroups.getDescription());
        model.setGroupUuid(securityGroups.getSecurityGroupId());
        if (StringUtils.isBlank(securityGroups.getSecurityGroupName())) {
            model.setName(securityGroups.getSecurityGroupId());
        } else {
            model.setName(securityGroups.getSecurityGroupName());
        }
        model.setVpcUuid(securityGroups.getVpcId());
        model.setStatus(securityGroups.getStatus());
        model.setRegionId(this.getRegionId());
        return model;
    }

    @Override
    public BocloudResult remove(String sgId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("SecurityGroupId", sgId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DELETESECURITYGROUP, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "删除SecurityGroup: " + sgId + " 失败");
            }
            return new BocloudResult(true, requestId, "删除SecurityGroup: " + sgId + " 成功");
        } catch (Exception e) {
            logger.error("del SecurityGroup error:", e);
            return new BocloudResult(false, "删除SecurityGroup失败，原因：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult getByVpc(String vpcId) {
        try {
            BocloudResult list = list();
            if (list.isFailed()) {
                return new BocloudResult(false, list.getMessage());
            }
            List<SecurityGroupModel> sgs = (List<SecurityGroupModel>) list.getData();
            List<SecurityGroupModel> finalSgs = sgs.stream().filter(sg -> sg.getVpcUuid() != null)
                    .filter(securityGroupModel -> securityGroupModel.getVpcUuid().equals(vpcId))
                    .collect(Collectors.toList());
            return new BocloudResult(true, finalSgs, "根据vpc查询安全组成功");
        } catch (Exception e) {
            logger.error("list SecurityGroup by vpc error:", e);
            return new BocloudResult(false, "根据vpc查询安全组成功失败");
        }
    }
}
