package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.volcengine.helper.Utils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;

/**
 * @ClassName: VolcengineSecurityRuleProvider
 * @Description: 安全组规则
 * @Author lxy
 * @Date 2021/9/14 9:55
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineSecurityRuleProvider extends VolcengineProvider implements SecurityGroupRuleProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineSecurityRuleProvider.class);

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineSecurityRuleProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(SecurityGroupRuleModel model) {
        String direction = model.getDirection();
        if ("egress".equalsIgnoreCase(direction)) {//出
            return this.securityGroupRuleOperation(model, VolcengineConst.AUTHORIZESECURITYGROUPEGRESS);
        } else if ("ingress".equalsIgnoreCase(direction)) {//入
            return this.securityGroupRuleOperation(model, VolcengineConst.AUTHORIZESECURITYGROUPINGRESS);
        } else {
            return new BocloudResult(false, "添加安全组规则失败，原因：不存在此类型");
        }
    }

    @Override
    public BocloudResult modify(SecurityGroupRuleModel model) {
        String direction = model.getDirection();
        if ("egress".equalsIgnoreCase(direction)) {//出
            return this.securityGroupRuleOperation(model, VolcengineConst.MODIFYSECURITYGROUPRULEDESCRIPTIONSEGRESS);
        } else if ("ingress".equalsIgnoreCase(direction)) {//入
            return this.securityGroupRuleOperation(model, VolcengineConst.MODIFYSECURITYGROUPRULEDESCRIPTIONSINGRESS);
        } else {
            return new BocloudResult(false, "修改安全组规则失败，原因：不存在此类型");
        }
    }

    /**
     * @return com.bocloud.common.model.BocloudResult
     * @Author: lxy
     * @Description: 安全组规则操作--增删
     * @Date 2021/9/14 10:02
     * @Param model
     * @throws:
     **/
    private BocloudResult securityGroupRuleOperation(SecurityGroupRuleModel model, String api) {
        try {
            Map<String, String> params = new HashMap<>();
            serParams(model, params);
            BocloudResult<JSONObject> response = operation(api, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "安全组规则 " + api + " 操作: " + model.getGroupUuid() + " 失败");
            }
            return new BocloudResult(true, model, "安全组规则 " + api + " 操作: " + model.getGroupUuid() + " 成功");

        } catch (Exception e) {
            logger.error(api + " SecurityGroupRuleIngress error:", e);
            return new BocloudResult(false, "安全组规则 " + api + " 操作失败");
        }
    }

    @Override
    public BocloudResult remove(SecurityGroupRuleModel ruleModel) {
        try {
            String direction = ruleModel.getDirection();
            if ("egress".equalsIgnoreCase(direction)) {
                return this.securityGroupRuleOperation(ruleModel, VolcengineConst.REVOKESECURITYGROUPEGRESS);
            } else if ("ingress".equalsIgnoreCase(direction)) {
                return this.securityGroupRuleOperation(ruleModel, VolcengineConst.REVOKESECURITYGROUPINGRESS);
            } else {
                return new BocloudResult(false, "删除安全组规则失败，原因：不存在此类型");
            }
        } catch (Exception e) {
            logger.error("删除安全组规则失败:" + e);
            return new BocloudResult(false, "删除安全组规则失败:" + e.getMessage());
        }
    }

    /**
     * @param params
     * @return void
     * @Author: lxy
     * @Description: 设置规则参数
     * @Date 2021/9/14 10:25
     * @Param model
     * @throws:
     **/
    private void serParams(SecurityGroupRuleModel model, Map<String, String> params) {
        //安全组ID。
        params.put("SecurityGroupId", model.getGroupUuid());
        /**
         * 传输层协议，此处协议指IPv4地址。
         * ●tcp：仅支持TCP协议。
         * ●udp：仅支持UDP协议。
         * ●icmp：仅支持ICMP协议。
         * ●all：支持所有协议。说明：此处协议仅支持IPv4地址。
         */
        params.put("Protocol", model.getProtocol().toUpperCase());
        /**
         * 端口范围开始值。
         * ●tcp：取值范围为1 ~ 65535
         * ●udp：取值范围为1 ~ 65535
         * ●icmp：-1
         * ●all：-1
         */
        if (null != model.getPortMin()) {
            params.put("PortStart", String.valueOf(model.getPortMin()));
        } else {
            params.put("PortStart", "-1");
            model.setPortMin(-1);
        }

        /**
         * 端口范围结束值。
         * ●tcp：取值范围为1 ~ 65535
         * ●udp：取值范围为1 ~ 65535
         * ●icmp：-1
         * ●all：-1
         */
        if (null != model.getPortMax()) {
            params.put("PortEnd", String.valueOf(model.getPortMax()));
        } else {
            params.put("PortEnd", "-1");
            model.setPortMax(-1);
        }
        /**
         * 源端IPv4 CIDR地址段。支持CIDR格式和IPv4格式的IP地址范围。默认值：无。
         */
        params.put("CidrIp", model.getRemoteIpPrefix());
        /**
         * 设置访问权限的源端安全组ID。
         * ●SourceGroupId和CidrIp至少设置一个。
         * ●如果指定了SourceGroupId没有指定参数CidrIp，则参数NicType取值只能为intranet。
         * ●如果同时指定了SourceGroupId和CidrIp，默认以CidrIp为准。
         *
         * 不存在，非必传，所以暂不设值
         */
//        if (StringUtils.isNoneBlank()) {
//            params.put("SourceGroupId", );
//        }
        /**
         * 设置访问权限。
         * ●accept（默认）：接受访问。
         * ●drop：拒绝访问。
         */
        if (StringUtils.isNoneBlank(model.getAction())) {
            params.put("Policy", model.getAction());
        }
        /**
         * 安全组规则优先级，数字越小，代表优先级越高。取值范围：1 ~ 100，默认值：1
         */
        if (null == model.getPriority()) {
            model.setPriority(1); //删除出规则，可能存在null的情况
        }
        params.put("Priority", String.valueOf(model.getPriority()));
        /**
         * 安全组规则的描述信息。
         * ●不能以http://或https://开头。
         * ●必须以字母或中文开头，可包含数字、英文逗号（,）、点号（.）、下划线（_）、空格（ ）、等号（=）、短横线（-）、中文逗号（，）、中文句号（。）。
         * ●长度限制为1 ~ 255个字符。
         * ●不填默认空字符串。
         */
        if (StringUtils.isNoneBlank(model.getRemark())) {
            params.put("Description", model.getRemark());
        }
    }

}
