package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.VolSubnetConvertor;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.cmp.provider.volmodel.Subnets;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.volcengine.helper.Utils;
import com.volcengine.model.response.ResponseMetadata;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: VolSubnetProvider
 * @Description: 子网
 * @Author lxy
 * @Date 2021/9/13 16:05
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineSubnetProvider extends VolcengineProvider implements SubnetProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineSubnetProvider.class);
    private BeanConvertor<SubnetModel, Subnets> subnetConvertor = new VolSubnetConvertor();
    private static final int PAGE_SIZE = 100;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineSubnetProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBESUBNETS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<Subnets> subnets = JSONArray.parseArray(JSONObject.toJSONString(result.get("Subnets")), Subnets.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询子网列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<KeypairModel>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBESUBNETS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<Subnets> subnetsList = JSONArray.parseArray(JSONObject.toJSONString(result.get("Subnets")), Subnets.class);
                    subnets.addAll(subnetsList);
                }
            }

            List<SubnetModel> convertList = subnetConvertor.convertList(subnets);
            for (SubnetModel subnetModel : convertList) {
                subnetModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("list subnet error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult list(String vpcId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("VpcId", vpcId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBESUBNETS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<Subnets> subnets = JSONArray.parseArray(JSONObject.toJSONString(result.get("Subnets")), Subnets.class);
            List<SubnetModel> convertList = subnetConvertor.convertList(subnets);
            for (SubnetModel subnetModel : convertList) {
                subnetModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("list subnet by vpc error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(SubnetModel subnet) {
        try {
            Map<String, String> params = new HashMap<>();
            /**
             * 子网的网段。子网网段要求如下：
             * ●子网网段的掩码长度范围为16 ~ 29位。
             * ●子网网段必须从属于所在VPC的网段。
             * ●子网网段不能与所在VPC的路由条目的目标网段相同，但可以是目标网段的子集。
             */
            params.put("CidrBlock", subnet.getCidr());
            //要创建的子网所属VPC的ID。
            params.put("VpcId", subnet.getVpcUuid());
            //要创建的子网所属的可用区ID。
            params.put("ZoneId", subnet.getZone());
            setParams(subnet, params);
            BocloudResult<JSONObject> response = operation(VolcengineConst.CREATESUBNET, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            ResponseMetadata responseMetadata = JSONObject.parseObject(JSONObject.toJSONString(data.get("ResponseMetadata")), ResponseMetadata.class);
            if (null != responseMetadata.getError()) {
                return new BocloudResult(false, "subnet创建失败:" + responseMetadata.getError().getMessage());
            }
            JSONObject subnetJson = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String subnetId = subnetJson.getString("SubnetId");

            Thread.sleep(2000);

            return getAvailable(subnetId);
        } catch (Exception e) {
            logger.error("create subnet error:", e);
            return new BocloudResult(false, "subnet创建失败");
        }
    }

    private BocloudResult getAvailable(String subnetId) {
        SubnetModel model = new SubnetModel();
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    logger.error("Get error message:", e);
                    Thread.currentThread().interrupt();
                }
                BocloudResult detail = this.detail(subnetId);
                if (detail.isFailed()) {
                    return detail;
                }
                model = (SubnetModel) detail.getData();
                if ("Available".equalsIgnoreCase(model.getStatus())) {
                    flag = false;
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 3 * 60 * 1000) {
                    flag = false;
                    break;
                }
            }
            return new BocloudResult(true, model, "Subnet创建成功");
        } catch (Exception e) {
            logger.error("getAvailable Subnet error:", e);
            return new BocloudResult(false, "创建Subnet失败：" + e.getMessage());
        }
    }

    /**
     * @param params
     * @return void
     * @Author: lxy
     * @Description: 设置Subnet非必填信息--名称、描述
     * @Date 2021/9/13 17:22
     * @Param subnet
     * @throws:
     **/
    private void setParams(SubnetModel subnet, Map<String, String> params) {
        /**
         * 创建的子网名称。
         * ●不能以http://或https://开头。
         * ●必须以字母或中文开头，可包含数字、点号（.）、下划线（_）和短划线（-）。
         * ●长度限制为1 ~ 128个字符。
         * ●不填默认是子网的ID。
         */
        if (StringUtils.isNoneBlank(subnet.getName())) {
            params.put("SubnetName", subnet.getName());
        }
        /**
         * 子网的描述信息。
         * ●不能以http://或https://开头。
         * ●必须以字母或中文开头，可包含数字、英文逗号（,）、点号（.）、下划线（_）、空格（ ）、等号（=）、短横线（-）、中文逗号（，）、中文句号（。）。
         * ●长度限制为1 ~ 255个字符。
         * ●不填默认空字符串。
         */
        if (StringUtils.isNoneBlank(subnet.getRemark())) {
            params.put("Description", subnet.getRemark());
        }
    }

    @Override
    public BocloudResult modify(SubnetModel subnet) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("SubnetId", subnet.getSubnetUuid());
            setParams(subnet, params);
            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYSUBNETATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "修改subnet: " + subnet.getSubnetUuid() + " 失败");
            }
            return new BocloudResult(true, this.detail(subnet.getSubnetUuid()).getData(), "修改subnet: " + subnet.getSubnetUuid() + " 成功");
        } catch (Exception e) {
            logger.error("update subnet error:", e);
            return new BocloudResult(false, "修改subnet失败，原因：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String subnetId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("SubnetId", subnetId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DELETESUBNET, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "删除subnet: " + subnetId + " 失败");
            }
            return new BocloudResult(true, requestId, "删除subnet: " + subnetId + " 成功");
        } catch (Exception e) {
            logger.error("del subnet error:", e);
            return new BocloudResult(false, "删除subnet失败，原因：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String subnetId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("SubnetId", subnetId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBESUBNETATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            Subnets subnet = JSONObject.parseObject(JSONObject.toJSONString(response.getData().get("Result")), Subnets.class);
            return new BocloudResult(true, subnetConvertor.convertModel(subnet), "subnet详情查询成功");
        } catch (Exception e) {
            logger.error("detail subnet error:" + e);
            return new BocloudResult(false, "subnet详情查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SubnetModel subnet) {
        return null;
    }
}
