package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.VolVpcConvertor;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.cmp.provider.volmodel.Vpc;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.volcengine.helper.Utils;
import com.volcengine.model.response.ResponseMetadata;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: VolcengineVpcProvider
 * @Description: 私有网络
 * @Author lxy
 * @Date 2021/9/13 10:32
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineVpcProvider extends VolcengineProvider implements VpcProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineVpcProvider.class);
    private BeanConvertor<VpcModel, Vpc> vpcConvertor = new VolVpcConvertor();
    private static final int PAGE_SIZE = 100;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineVpcProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(VpcModel model) {
        try {
            Map<String, String> params = new HashMap<>();
            /**
             * VPC的网段。您可以使用以下网段或其子集作为VPC的网段：
             * ● 192.168.0.0/16 ~ 24（默认）
             * ●10.0.0.0/8 ~ 24
             * ●172.16.0.0/12 ~ 24
             */
            params.put("CidrBlock", model.getCidr());
            setParams(model, params);
            BocloudResult<JSONObject> response = operation(VolcengineConst.CREATEVPC, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            ResponseMetadata responseMetadata = JSONObject.parseObject(JSONObject.toJSONString(data.get("ResponseMetadata")), ResponseMetadata.class);
            if (null != responseMetadata.getError()) {
                return new BocloudResult(false, "vpc创建失败:" + responseMetadata.getError().getMessage());
            }
            JSONObject vpcJson = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String vpcId = vpcJson.getString("VpcId");
            String routeTableId = vpcJson.getString("RouteTableId");

            Thread.sleep(2000);

            return getAvailable(vpcId, routeTableId);
        } catch (Exception e) {
            logger.error("create vpc error:", e);
            return new BocloudResult(false, "vpc创建失败");
        }
    }

    /**
     * @param params
     * @return void
     * @Author: lxy
     * @Description: 设置vpc非必填信息--名称、描述、dns
     * @Date 2021/9/13 15:20
     * @Param model
     * @throws:
     **/
    private void setParams(VpcModel model, Map<String, String> params) {
        /**
         * VPC的名称。
         * ●不能以http://或https://开头。
         * ●必须以字母或中文开头，可包含数字、点号（.）、下划线（_）和短划线（-）。
         * ●长度限制为1 ~ 128个字符。
         * ●不填默认为VPC实例的ID。
         */
        if (StringUtils.isNoneBlank(model.getName())) {
            params.put("VpcName", model.getName());
        }
        /**
         * VPC的描述信息。
         * ●不能以http://或https://开头。
         * ●必须以字母或中文开头，，可包含数字、英文逗号（,）、点号（.）、下划线（_）、空格（ ）、等号（=）、短横线（-）、中文逗号（，）、中文句号（。）。
         * ●长度限制为1 ~ 255个字符。
         * ●不填默认为空字符串。
         */
        if (StringUtils.isNoneBlank(model.getRemark())) {
            params.put("Description", model.getRemark());
        }
        /**
         * VPC的DNS服务器地址。
         * ●参数 - N：表示DNS服务器地址的序号，数量上限为5个，每个DnsServer必须以合法IP形式给出。
         * ●多个IP之间用&分隔。
         * ●如不填，则配置为默认DNS服务器地址。
         */
        if (StringUtils.isNoneBlank(model.getDnsServer())) {
            List<String> dnsServer = JSONArray.parseArray(model.getDnsServer(), String.class);
            String dns = "";
            for (int i = 1; i <= dnsServer.size(); i++) {
                dns = "DnsServers." + i + "=" + dnsServer.get(i - 1) + "&";
            }
            dns = dns.substring(0, dns.length() - 1);
            params.put("DnsServers.N", dns);
        }
    }

    private BocloudResult getAvailable(String vpcId, String routeTableId) {
        VpcModel vpc = new VpcModel();
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    logger.error("Get error message:", e);
                    Thread.currentThread().interrupt();
                }
                vpc = JSONObject.parseObject(JSONObject.toJSONString(this.detail(vpcId, this.getRegionId()).getData()), VpcModel.class);
                if (null == vpc) {
                    flag = false;
                    return new BocloudResult(false, "vpc未创建成功，请查找问题！");
                }

                if ("Available".equalsIgnoreCase(vpc.getStatus())) {
                    flag = false;
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 5 * 60 * 1000) {
                    flag = false;
                    break;
                }
            }
            return new BocloudResult(true, vpc, "vpc创建成功");
        } catch (Exception e) {
            logger.error("getAvailable vpc error:", e);
            return new BocloudResult(false, "vpc创建失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String vpcId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("VpcId", vpcId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DELETEVPC, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "删除vpc: " + vpcId + " 失败");
            }
            return new BocloudResult(true, requestId, "删除vpc: " + vpcId + " 成功");
        } catch (Exception e) {
            logger.error("del vpc error:", e);
            return new BocloudResult(false, "删除vpc失败，原因：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult list() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEVPCS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<Vpc> vpcs = JSONArray.parseArray(JSONObject.toJSONString(result.get("Vpcs")), Vpc.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询vpc列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<KeypairModel>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBEVPCS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<Vpc> vpcList = JSONArray.parseArray(JSONObject.toJSONString(result.get("Vpcs")), Vpc.class);
                    vpcs.addAll(vpcList);
                }
            }
            List<VpcModel> convertList = vpcConvertor.convertList(vpcs);
            for (VpcModel vpcModel : convertList) {
                vpcModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("list vpc error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult detail(String vpcId, String regionId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("VpcId", vpcId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEVPCATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            Vpc vpc = JSONObject.parseObject(JSONObject.toJSONString(response.getData().get("Result")), Vpc.class);
            return new BocloudResult(true, vpcConvertor.convertModel(vpc), "vpc详情查询成功");
        } catch (Exception e) {
            logger.error("detail vpc error:" + e);
            return new BocloudResult(false, "vpc详情查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult update(VpcModel model) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("VpcId", model.getVpcId());
            setParams(model, params);
            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYVPCATTRIBUTES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.VPC);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "修改vpc: " + model.getVpcId() + " 失败");
            }
            return new BocloudResult(true, this.detail(model.getVpcId(), this.getRegionId()).getData(), "修改vpc: " + model.getVpcId() + " 成功");
        } catch (Exception e) {
            logger.error("update vpc error:", e);
            return new BocloudResult(false, "修改vpc失败，原因：" + e.getMessage());
        }
    }
}
