package com.bocloud.cmp.provider.security;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.VolKeyPairConvertor;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.volcengine.helper.Utils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: VolcengineKeypairProvider
 * @Description: 秘钥
 * @Author lxy
 * @Date 2021/9/17 11:19
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineKeypairProvider extends VolcengineProvider implements KeypairProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineKeypairProvider.class);
    private BeanConvertor<KeypairModel, JSONObject> keyPairConvertor = new VolKeyPairConvertor();
    private static final int PAGE_SIZE = 100;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineKeypairProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEKEYPAIRS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<JSONObject> keyPairs = JSONArray.parseArray(JSONObject.toJSONString(result.get("KeyPairs")), JSONObject.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询秘钥列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<KeypairModel>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBEKEYPAIRS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<JSONObject> keyPairList = JSONArray.parseArray(JSONObject.toJSONString(result.get("KeyPairs")), JSONObject.class);
                    keyPairs.addAll(keyPairList);
                }
            }
            List<KeypairModel> convertList = keyPairConvertor.convertList(keyPairs);
            for (KeypairModel keypairModel : convertList) {
                keypairModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("list Keypair error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult detail(String keypairName) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("KeyPairName", keypairName);
            params.put("Region", this.getRegionId());
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEKEYPAIRS, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<JSONObject> keyPairs = JSONArray.parseArray(JSONObject.toJSONString(result.get("KeyPairs")), JSONObject.class);
            List<KeypairModel> convertList = keyPairConvertor.convertList(keyPairs);
            for (KeypairModel keypairModel : convertList) {
                keypairModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("detail Keypair error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(String keypairName, String publicKey) {
        if (StringUtils.isBlank(publicKey)) {
            return create(keypairName);
        } else {
            return importKey(keypairName, publicKey);
        }
    }

    private BocloudResult importKey(String keypairName, String publicKey) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("KeyPairName", keypairName);
            params.put("PublicKey", publicKey);
            BocloudResult<JSONObject> response = operation(VolcengineConst.IMPORTKEYPAIR, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String keyPairName = result.getString("KeyPairName");
            BocloudResult detail = this.detail(keypairName);
            if (detail.isFailed()) {
                return new BocloudResult(false, "创建成功，查询失败，请查找原因：" + detail.getMessage());
            }
            List<KeypairModel> keypairModels = JSONArray.parseArray(JSONObject.toJSONString(detail.getData()), KeypairModel.class);
            keypairModels.stream().forEach(keypairModel -> keypairModel.setPrivateKey(result.getString("PrivateKey")));
            return new BocloudResult(true, keypairModels.get(0), "创建成功");
        } catch (Exception e) {
            logger.error("importKey Keypair error:", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    private BocloudResult create(String keypairName) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("KeyPairName", keypairName);
            BocloudResult<JSONObject> response = operation(VolcengineConst.CREATEKEYPAIR, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String keyPairName = result.getString("KeyPairName");
            BocloudResult detail = this.detail(keypairName);
            if (detail.isFailed()) {
                return new BocloudResult(false, "创建成功，查询失败，请查找原因：" + detail.getMessage());
            }
            List<KeypairModel> keypairModels = JSONArray.parseArray(JSONObject.toJSONString(detail.getData()), KeypairModel.class);
            keypairModels.stream().forEach(keypairModel -> keypairModel.setPrivateKey(result.getString("PrivateKey")));
            return new BocloudResult(true, keypairModels.get(0), "创建成功");
        } catch (Exception e) {
            logger.error("create Keypair error:", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult delete(String keypairName) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("KeyPairName", keypairName);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DELETEKEYPAIR, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String keyPairName = result.getString("KeyPairName");
            if (StringUtils.isBlank(keyPairName)) {
                return new BocloudResult(false, "删除失败，返回秘钥信息为空");
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("delete Keypair error:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult modify(KeypairModel model) {
        return null;
    }


}
