package com.bocloud.cmp.provider.storage;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.VolVolumeConvertor;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.volcengine.helper.Utils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: VolcengineVolumeProvider
 * @Description: 云硬盘
 * @Author lxy
 * @Date 2021/9/15 13:56
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineVolumeProvider extends VolcengineProvider implements VolumeProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineVolumeProvider.class);
    private BeanConvertor<VolumeModel, JSONObject> volumeConvertor = new VolVolumeConvertor();
    private static final int PAGE_SIZE = 100;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineVolumeProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.LISTVOLUMES, VolcengineConst.VERSION_20180101, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<JSONObject> volumes = JSONArray.parseArray(JSONObject.toJSONString(result.get("Volumes")), JSONObject.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询云主机列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<KeypairModel>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.LISTVOLUMES, VolcengineConst.VERSION_20180101, Utils.mapToPairList(params), VolcengineConst.service.ECS);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<JSONObject> volumsList = JSONArray.parseArray(JSONObject.toJSONString(result.get("Volumes")), JSONObject.class);
                    volumes.addAll(volumsList);
                }
            }
            List<VolumeModel> convertList = volumeConvertor.convertList(volumes);
            for (VolumeModel volumeModel : convertList) {
                volumeModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("list Volume error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(VolumeModel volume) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("ZoneId", volume.getZone());
            params.put("VolumeType", volume.getType());
            params.put("Size", String.valueOf(volume.getSize()));
            params.put("VolumeName", volume.getName());
            params.put("Count", "1");
            if (StringUtils.isNotBlank(volume.getRemark())) {
                params.put("Description", volume.getRemark());
            }

            BocloudResult<JSONObject> response = operation(VolcengineConst.CREATEVOLUME, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String volumeId = result.getString("VolumeId");
            Thread.sleep(3000);
            return new BocloudResult(true, this.detail(volumeId).getData(), "创建成功");
        } catch (Exception e) {
            logger.error("create Volume error:", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult modify(VolumeModel volume) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("Id", volume.getVolumeUuid());
            if (StringUtils.isNotBlank(volume.getRemark())) {
                params.put("Description", volume.getRemark());
            }
            if (StringUtils.isNotBlank(volume.getName())) {
                params.put("Name", volume.getName());
            }

            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYVOLUME, VolcengineConst.VERSION_20180101, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            String volumeId = result.getString("VolumeId");
            Thread.sleep(3000);
            return new BocloudResult(true, this.detail(volumeId).getData(), "修改成功");
        } catch (Exception e) {
            logger.error("modify Volume error:", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    @Override
    public BocloudResult remove(String volumeId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("VolumeId", volumeId);
            params.put("Region", this.getRegionId());
            BocloudResult<JSONObject> response = operation(VolcengineConst.DELETEVOLUME, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("ResponseMetadata")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "删除磁盘: " + volumeId + " 失败");
            }
            return new BocloudResult(true, "删除磁盘成功");
        } catch (Exception e) {
            logger.error("remove Volume error:", e);
            return new BocloudResult(false, "删除磁盘失败");
        }
    }

    @Override
    public BocloudResult attach(String serverId, String volumeId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("Id", serverId);
//            params.put("Region", this.getRegionId());
            params.put("VolumeId", volumeId);
//            params.put("VolumeIDs", JSONObject.toJSONString(Lists.newArrayList(serverId)));
            BocloudResult<JSONObject> response = operation(VolcengineConst.ATTACHVOLUME, VolcengineConst.VERSION_20180101, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));

            return new BocloudResult(true, result.getString("Id"), "挂载磁盘成功");
        } catch (Exception e) {
            logger.error("attach Volume error :", e);
            return new BocloudResult(false, "挂载磁盘失败");
        }
    }

    @Override
    public BocloudResult detach(String serverId, String volumeId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("Id", serverId);
            params.put("VolumeId", volumeId);
//            params.put("VolumeIDs", JSONObject.toJSONString(Lists.newArrayList(serverId)));
            BocloudResult<JSONObject> response = operation(VolcengineConst.DETACHVOLUME, VolcengineConst.VERSION_20180101, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));

            return new BocloudResult(true, result.getString("Id"), "卸载磁盘成功");
        } catch (Exception e) {
            logger.error("detach Volume error :", e);
            return new BocloudResult(false, "卸载磁盘失败");
        }
    }

    @Override
    public BocloudResult detail(String volumeId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("Id", volumeId);
            params.put("Region", this.getRegionId());
            BocloudResult<JSONObject> response = operation(VolcengineConst.GETVOLUME, VolcengineConst.VERSION_20180101, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject volume = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            VolumeModel volumeModel = volumeConvertor.convertModel(volume);
            volumeModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, volumeModel, "查询成功");
        } catch (Exception e) {
            logger.error("detail Volume error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult extend(String volumeId, Integer newSize, String type) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("VolumeId", volumeId);
            params.put("NewSize", String.valueOf(newSize));
            BocloudResult<JSONObject> response = operation(VolcengineConst.EXTENDVOLUME, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("ResponseMetadata")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "扩容磁盘: " + volumeId + " 失败");
            }

            return new BocloudResult(true, "扩容成功");
        } catch (Exception e) {
            logger.error("extend Volume error:", e);
            return new BocloudResult(false, "扩容失败");
        }
    }
}
